#
# Copyright (c) 2001-2009 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Base for API Sets related to CMode GUI operations
## @author sindhe@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Node::GUI::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

# Module Imports
use NACL::APISet::Session::GUI::CMode;

# Import Exception class
use NATE::BaseException qw(:try);
use Params::Validate qw(:all);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

# Base class
use base qw(NACL::APISet::Node::GUI);

###############################################################################
# Method Name: new
# Objective:  This function creates a CMode GUI object
###############################################################################

sub new {
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;

    my $self = {};
    $self =
      NACL::APISet::validate_params($class,
                                    params => \%args,
                                    spec   => {},);

    # Bless the hash returned which will make it a Perl object
    bless $self, $class;

    # And give the caller the reference to the bless object.
    $Log->exit() if $may_exit;

    return $self;
} ## end sub new

##############################################################################
# Method:    set_vserver
# Objective: To set the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub set_vserver {
    $Log->enter() if $may_enter;

    my $self = shift(@_);
    my %args = NACL::APISet::validate_params(
        $self,
        params => \@_,
        spec   => { "vserver-context-name" => { type => SCALAR } }
    );
    $self->{"vserver"} = $args{"vserver-context-name"};

    $Log->exit() if $may_exit;
} ## end sub set_vserver

##############################################################################
# Method:    get_vserver
# Objective: get the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub get_vserver {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $Log->exit() if $may_exit;
    return $self->{"vserver"};
} ## end sub get_vserver

##############################################################################
# Method:    clear_vserver
# Objective: clear the current vserver context
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub clear_vserver {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    $self->{"vserver"} = "";

    $Log->exit() if $may_exit;
} ## end sub clear_vserver

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Node::GUI::CMode - Base for CMode API Sets for GUI operations

=head1 SYNOPSIS

use NACL::APISet::Node::GUI::CMode

=head1 DESCRIPTION

A C<NACL::APISet::Node::GUI::CMode> is used Initializes the API Set and make the API Set ready for work.

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (neelkant@netapp.com, dl-nacl-dev@netapp.com)

=back

 $Id$
 Last revised $Date: 2013/02/08 $ by $Author: kjp $

=cut

