# Hey emacs, this is basically -*- perl -*-
#$Id$
#
# Copyright (c) 2009 Network Appliance, Inc. as an unpublished work.
# All rights reserved.

## @summary Canned parsers for paarsing output
## @author dl-nacl-dev
## @pod here

package NACL::APISet::Response::CLI::CannedParsers;
use NACL::APISet::Response::CLI::ParserUtils;

sub key_colon_value { return key_character_value(':', @_); }

sub key_equal_value { return key_character_value('=', @_); }

sub key_character_value {
    my ($character, $args) = @_;
    my $regex      = '^\s*(.+?)\s*' . $character . '\s*(.*)';
    my $raw_output = $args->{response_object}->get_processed_output;

    my %get_lines_args = ();
    $get_lines_args{first_line} = $args->{first_line}
        if ($args->{first_line});
    my @lines = NACL::APISet::Response::CLI::ParserUtils::get_lines(
        'raw_output' => $raw_output,
        'can_throw'  => 1,
        %get_lines_args,
    );

    if ($args->{remove_pattern_from_all_lines}) {
        my $pattern = $args->{remove_pattern_from_all_lines};
        map {s/$pattern//g} @lines;
    }

    my $hash;
    for my $line (@lines) {
        my ($key, $value) = $line =~ qr/$regex/;
        if (defined $key) {
            if ($value =~ /\s/
                && !$args->{dont_split_output_by_space}) {
                $value = [split(/\s/, $value)];
            }
            $hash->{
                NACL::APISet::Response::CLI::ParserUtils::to_key_name(
                    item => $key
                )
                }
                = $value;
        }
    }
    return [$hash];
} ## end sub key_character_value

1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::CannedParsers - Canned parsers for parsing output

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::CannedParsers;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::CannedParsers> contains functions that can be
used for parsing output of commands

=head1 Methods

=head2 key_colon_value

This method parses output which is of the format key\s+:\s+value

=over

=item Synopsis

key_colon_value(
    response_object => $resp,
    remove_pattern_from_all_lines => 'str',
    first_line => 'header',
    dont_split_output_by_space => '1'
);

=item Arguments

=over

=item response_object

(mandatory)
APISet repsonse object

=item remove_pattern_from_all_lines

(optional)
Remove pattern from each line of an output

=item first_line

(optional)
first line of the command to be stripped. Generally a header
This is passed to get_lines method

=item dont_split_output_by_space

(optional)
ignore space between strings in key

=back

=item Returns

parsed output

=back

=head2 key_equal_value

This method parses output which is of the format key\s+=\s+value

=over

=item Synopsis

key_equal_value(
    response_object => $resp,
    remove_pattern_from_all_lines => 'str',
    first_line => 'header',
    dont_split_output_by_space => '1'
);

=item Arguments

=over

=item response_object

(mandatory)
APISet repsonse object

=item remove_pattern_from_all_lines

(optional)
Remove pattern from each line of an output

=item first_line

(optional)
first line of the command to be stripped. Generally a header
This is passed to get_lines method

=item dont_split_output_by_space

(optional)
ignore space between strings in key

=back

=item Returns

parsed output

=back

=head2 key_character_value

This method parses output which is of the format key\s+<character>\s+value

=over

=item Synopsis

key_character_value(
    response_object => $resp,
    remove_pattern_from_all_lines => 'str',
    first_line => 'header',
    dont_split_output_by_space => '1'
);

=item Arguments

=over

=item response_object

(mandatory)
APISet repsonse object

=item remove_pattern_from_all_lines

(optional)
Remove pattern from each line of an output

=item first_line

(optional)
first line of the command to be stripped. Generally a header
This is passed to get_lines method

=item dont_split_output_by_space

(optional)
ignore space between strings in key

=back

=item Returns

parsed output

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

