# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Base Response class for all Kerberos API Sets
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here
package NACL::APISet::Response::CLI::Kerberos;
use strict;
use warnings;
use base qw(NACL::APISet::Response::CLI);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;


sub new {
    $Log->enter() if $may_enter;

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    # a hash.
    my ( $class, %args ) = @_;

    # Fill the member vars for this instance
    my $self =
        NACL::APISet::validate_params(
                      $class,
                      params => \%args,
                      spec   => {
                          output         => { type => SCALAR, required => 1 },
                          command        => { type => SCALAR, required => 1 },
                          method_name    => { type => SCALAR, required => 1 },
                          cdef_path      => { type => SCALAR, required => 1 },
                          execution_time => { type => SCALAR, required => 1 },
                          actual_command => { type => SCALAR, required => 1 },
                      }
        );
    bless $self, $class;
    $self->_parse_output();
    $Log->debug( sub { Dumper($self) } );
    $Log->exit() if $may_exit;
    return $self;
}
1;
##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Response::CLI::Kerberos - Kerberos CLI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::CLI::Kerberos;

=head1 DESCRIPTION

A C<NACL::APISet::Response::CLI::Kerberos> object is used to recieve, parse and hold
the output from Kerberos CLI API methods.

=head1 EXCEPTIONS

None.

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::CLI::Kerberos>.

=over

=item Synopsis

 $response = NACL::APISet::Response::CLI::Kerberos->new( %args );

=item Arguments

=over

=item output

The raw output from the command.  This can be parsed into an array of key value pairs.

=item command

The command invoked on the service

=item method_name

Name of the method invoked.

=item cdef_path

Path to the directory containing the Kerberos CDEF files

=item execution_time

Time taken to execute the command

=item parser_file

The name of the parser file

=item actual command

The command as translated by the schema.

=back

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back



