# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Response class for ZAPI 7Mode APISet
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Response::ZAPI::7Mode;

use strict;
use warnings;

use base qw(NACL::APISet::Response::ZAPI);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

1;

=head1 NAME

NACL::APISet::Response::ZAPI::7Mode - 7Mode ZAPI Response class

=head1 SYNOPSIS

 use NACL::APISet::Response::ZAPI::7Mode;

=head1 DESCRIPTION

A C<NACL::APISet::Response::ZAPI::7Mode> object is used to recieve, parse and hold 
the output from 7Mode ZAPI API methods.

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Response::ZAPI::7Mode>.  

=over 

=item Synopsis

 $response = NACL::APISet::Response::ZAPI::7Mode->new( %args );

=item Arguments

=over 

=item output

The raw xml output from the command.  This can be parsed into an array of 
key value pairs.

=item command

The ZAPI command which has been sent to the filer.

=item execution_time

The time taken to execute the ZAPI command.

=item execution_status

The status returned after the execution of ZAPI command.
Possible values are passed and failed.

=item error_number

The error number returned on the execution of ZAPI

=item error_reason

The reason for error in case if any

=back

=back

=cut

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

