# $Id$

## @summary Class to hold all selenium GUI connection information.
## @author madhav.shekar@netapp.com, sindhe@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Session::GUI::SysManager::CMode;

#
# Compiler Directives
#
use strict;
use warnings;

# Module Imports
use Data::Dumper;
use NACL::APISet::Host::GUI::SysManager::GConst;
use NACL::APISet::Host::GUI::SysManager::CMode::SmVolume;
use NACL::APISet::Host::GUI::SysManager::CMode::SmVolumeMove;
use NACL::APISet::Host::GUI::SysManager::CMode::SmDisks;
use NACL::APISet::Host::GUI::SysManager::CMode::SmAggr;
use NACL::APISet::Host::GUI::SysManager::CMode::SmDedupePolicies;
use NACL::APISet::Host::GUI::SysManager::CMode::SmFcIscsi;
use NACL::APISet::Host::GUI::SysManager::CMode::SmInitiator;
use NACL::APISet::Host::GUI::SysManager::CMode::SmLun;
use NACL::APISet::Host::GUI::SysManager::CMode::SmNameMapping;
use NACL::APISet::Host::GUI::SysManager::CMode::SmNamespace;
use NACL::APISet::Host::GUI::SysManager::CMode::SmNetworking;
use NACL::APISet::Host::GUI::SysManager::CMode::SmNfsCifs;
use NACL::APISet::Host::GUI::SysManager::CMode::SmPortset;
use NACL::APISet::Host::GUI::SysManager::CMode::SmProtection;
use NACL::APISet::Host::GUI::SysManager::CMode::SmQtree;
use NACL::APISet::Host::GUI::SysManager::CMode::SmQuotas;
use NACL::APISet::Host::GUI::SysManager::CMode::SmSchedule;
use NACL::APISet::Host::GUI::SysManager::CMode::SmSecurity;
use NACL::APISet::Host::GUI::SysManager::CMode::SmServices;
use NACL::APISet::Host::GUI::SysManager::CMode::SmShares;
use NACL::APISet::Host::GUI::SysManager::CMode::SmSnapshotPolicies;
use NACL::APISet::Host::GUI::SysManager::CMode::SmSystemTools;
use NACL::APISet::Host::GUI::SysManager::CMode::SmUnixUsersGroups;
use NACL::APISet::Host::GUI::SysManager::CMode::SmExportPolicies;
use NACL::APISet::Host::GUI::SysManager::CMode::SmVserver;

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#
# Library imports
#
use base qw( NACL::APISet::Session::GUI::SysManager
  NACL::APISet::Host::GUI::SysManager::CMode::SmUtils
  NACL::APISet::Host::GUI::SysManager::NWFUtils
  );

###############################################################################
# Method:    new
# Objective: Creates a new object with selenium connection.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub new {
    my ($self);
    my $class = shift;

    $Log->enter() if $may_enter;

    my %args = validate_with(
                    params => \@_,
                    spec   => {
                        remote_server_addr => {type => SCALAR, default => "localhost"},
                        port               => {type => SCALAR, default => "4444"},
                        browser_name       => {type => SCALAR, default => "firefox"}},
                    allow_extra => 1);

    eval {
        $self = {};
        $Log->debug("Arguments Passed to "
             . "NACL::APISet::Session::GUI::SysManager::CMode->new: "
             . Dumper(\%args));

        my $browser_url = delete ($args{browser_url}) || "http://127.0.0.1:1919/";

        $Log->comment("Initializing Selenium.");
        $self = Selenium::Remote::Driver->new(
                    remote_server_addr => $args{'remote_server_addr'},
                    port               => $args{'port'},
                    browser_name       => $args{'browser_name'} );

        $Log->comment("Opening Browser");
        $self->get($browser_url);

        $Log->comment("Waiting for page to load");
        sleep 2;

        bless $self, $class
          or NATE::BaseException->throw("Can't bless $class: $!");

        $Log->comment("Updating Node Details.");
        $self->update_system_details(%{$args{SystemNodes}});

        $Log->comment("Updating Active System Details");
        $Log->debug(Dumper($args{SystemNodes}));
        my ($key, $SystemDetails);

        $key           = uc $PARAM_CLUSTERDETAIL;
        $SystemDetails = $args{SystemNodes}{$key};

        $self->update_active_system($args{vserver}, $SystemDetails);

        $self->wait_for_loading($GENERIC_WAIT);

        $Log->comment("Opening system for Managing.");
        $self->manage_system();
        $self->{timeout} = 30;
    };
    if ($@) {
        $Log->comment("Failed to create Session Object : $@");
        NATE::BaseException->throw("$@");
    }

    $Log->exit() if $may_exit;
    return $self;
} ## end sub new

1;

##############################################################################
#                                    POD here
##############################################################################

=head1 NAME

NACL::APISet::Session::GUI::SysManager::CMode - API Set Session sub class for CMode GUIs

=head1 SYNOPSIS

 use NACL::APISet::Session::GUI::SysManager::CMode;

=head1 DESCRIPTION

Creates a new GUI object with selenium connection.

=head1 EXCEPTIONS

None.

=head1 CONSTRUCTORS

=head2 new

Returns a object contaning all methods of inherited perl modules

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Session::GUI::SysManager::CMode->new( %args );

=item Arguments

=over

=item version

(Optional)
Version string

=back

=item Returns

A reference to a new GUI CMode API Set.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development madhav.shekar@netapp.com, ng-sysmgr-automation-core@netapp.com, dl-nacl-dev@netapp.com

=back

 $Id$
 Last revised $Date: 2012/04/05 $ by $Author: neelkant$

=cut
