#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for 7Mode Sessions
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::ZAPI::7Mode;

use strict;
use warnings;

use base qw(NACL::APISet::Session::ZAPI);

use NACL::APISet::Response::ZAPI::7Mode;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

sub new {
    $Log->enter();
    $Log->debug( Dumper(@_) );

	my ($self, %args) = @_;
	my $hostobj;
	if ( $args{hostobj} ) {
		$hostobj = $args{hostobj};
		unless ( defined $args{ipaddress}) {
            $Log->trace("Obtained ipaddress from hostrec object");
			$args{ipaddress} = $hostobj->default_ip_7g();
		}
		
		unless ( defined $args{username}) {
            $Log->trace("Obtained username from hostrec object");
			$args{username} = $hostobj->default_username_7g();
		}
		
		unless ( defined $args{password}) {
            $Log->trace("Obtained password from hostrec object");
			$args{password} = $hostobj->default_password_7g();
		}
	}
	
	my $ret_obj = $self->SUPER::new(%args);
	$ret_obj->set_server_type('server-type'=>"FILER");
	    
    $Log->exit();
    return $ret_obj;
}

##############################################################################
# Method:    send_command
# Objective: Send the ZAPI command to the filer.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub send_command {
    $Log->enter();
    my ($self, %args) = @_;
    my $response = $self->SUPER::send_command(%args, _response_type => '7Mode');
    $Log->exit();
    return $response;
}



=head1 NAME

NACL::APISet::Session::ZAPI::7Mode - Session class for 7Mode ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Session::ZAPI::7Mode;

=head1 DESCRIPTION

A C<NACL::APISet::Session::ZAPI::7Mode> provides the methods needs to control the connection
to a 7Mode node.  

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::ZAPI::7Mode>.

=over

=item Synopsis

 $session = NACL::APISet::Session::ZAPI::7Mode->new( %opt );

=item Arguments

For POD refer base class L<initialize|lib-NACL-APISet-Session-ZAPI-pm/initialize>

=item Returns

A reference to a new Session.

=back

=cut

=head1 METHODS

=head2 send_command

Send the command to the node.  The output of the command is returned in a 
NACL::APISet::Response::ZAPI::7Mode object.

=over 

=item Synopsis

$response = $session->send_command( command => "<volume-get-language/>" );

=item Arguments

=over 

=item command

The input raw xml to be sent

=back

=item Returns

Response object of type NACL::APISet::Response::ZAPI::7Mode

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
