#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Class to hold all connection information for CMode Sessions
## @author nsesha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

# Not all 'connections' to services will be in the form of NATE::Connectionrec
# The Session class will define all data needed for connections, whether
# NATE Connection or not.

package NACL::APISet::Session::ZAPI::CMode;

use strict;
use warnings;

use base qw(NACL::APISet::Session::ZAPI);

use NACL::APISet::Response::ZAPI::CMode;
use NACL::APISet::_Mixins::CLIZAPI qw(_handle_baseline_errors :baseline_opts);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use Data::Dumper;

sub new {
    $Log->enter();
    $Log->debug( Dumper(@_) );
	
	my ($self, %args) = @_;
	
	my $hostobj;
	if ( $args{hostobj} ) {
		$hostobj = $args{hostobj};
		unless ( defined $args{ipaddress}) {
            $Log->trace("Obtained ipaddress from hostrec object");
			$args{ipaddress} = $hostobj->default_ip_gx();
		}
		
		unless ( defined $args{username}) {
            $Log->trace("Obtained username from hostrec object");
			$args{username} = $hostobj->default_username_gx();
		}
		
		unless ( defined $args{password}) {
            $Log->trace("Obtained password from hostrec object");
			$args{password} = $hostobj->default_password_gx();
		}
	}
	
	my $ret_obj = $self->SUPER::new(%args);
	$ret_obj->set_server_type('server-type'=>"FILER");
	    
    $Log->exit();
    return $ret_obj;
}

##############################################################################
# Method:    send_command
# Objective: Send the ZAPI command to the cluster mode node.
# Details:   Refer to POD documentation at the end of the file.
##############################################################################

sub send_command {
    $Log->enter();
    my ($self, %args) = @_;
    my %baseline_args = $self->_move_baseline_args(source => \%args);
    
    my $response;
    my $execute_zapi_form_response = sub {
        $response = $self->SUPER::send_command(%args, _response_type => 'CMode');
    };
    
    $self->_handle_baseline_errors(code => $execute_zapi_form_response,
        %baseline_args);
    $Log->exit();
    return $response;
}

=head1 NAME

NACL::APISet::Session::ZAPI::CMode - Session class for Cluster mode ZAPI commands

=head1 SYNOPSIS

 use NACL::APISet::Session::ZAPI::CMode;

=head1 DESCRIPTION

A C<NACL::APISet::Session::ZAPI::CMode> provides the methods needs to control the 
connection to a Cluster Mode node.  

=cut

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Session::ZAPI::CMode>.

=over

=item Synopsis

 $session = NACL::APISet::Session::ZAPI::CMode->new( %opt );

=item Arguments

For POD refer base class L<initialize|lib-NACL-APISet-Session-ZAPI-pm/initialize>

=item Returns

A reference to a new Session.

=back

=cut

=head1 METHODS

=head2 send_command

Send the command to the node.  The output of the command is returned in a 
NACL::APISet::Response::ZAPI::CMode object.

=over 

=item Synopsis

$response = $session->send_command( command => "<system-get-version/>" );

=item Arguments

=over 

=item command

The input raw xml to be sent

=back

=item Returns

Response object of type NACL::APISet::Response::ZAPI::CMode

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

=cut

1;
