#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/Armadillo/default/sasportlist.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary Routines to parse Armadillo bridge commands
##
## @description
##    This is a library to parse the raw output of Armadillo bridge commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::Armadillo::default::sasportlist
##
##  sasportlist(%args)
##
## @author sangavai@netapp.com, dl-nacl-dev@netapp.com
##
## @status public


package NACL::APISet::Switch::CLI::Armadillo::default::sasportlist;

use NACL::APISet::Response::CLI::ParserUtils;

use NACL::APISet::Exceptions::InvalidParamValueException;

use strict;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my $error_message;
    my $error = undef;

    if ( $raw_output =~ /Wrong\/Missing Parameters/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::sasportlist
##
## @summary Parses the raw output from the "SASPortList" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from sasportlist with any of its options as described in the 
##    corresponding cdef file 'sasportlist.cdefs'
## @synopsis
##      sasportlist(%args);
## INPUTS TO METHOD CALL 'sasportlist' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = SASPortList
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp; *
##&nbsp;SASPortList
##&nbsp;
##&nbsp;10 
##&nbsp;;Connector      PHY     Link    Speed   SAS Address
##&nbsp;;=======================================================
##&nbsp;Device	A	1	Up	6Gb	5001086000602c62
##&nbsp;Device	A	2	Up	6Gb	5001086000602c62
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'link' => 'Up',
##&nbsp;            'phy' => '1',
##&nbsp;            'speed' => '6Gb',
##&nbsp;            'sas_address' => '5001086000602c62',
##&nbsp;            'connector' => 'DeviceA'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'link' => 'Up',
##&nbsp;            'phy' => '2',
##&nbsp;            'speed' => '6Gb',
##&nbsp;            'sas_address' => '5001086000602c62',
##&nbsp;            'connector' => 'DeviceA'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub sasportlist {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $resp->get_raw_output();
    my (@result_array, @head, @values, $count);
    _check_error(
        raw_output      => $raw_output,
        response_object => $resp
    );
    my @lines = get_lines( response_object => $resp, raw_output => $raw_output);
    $count = 0;
    for(my $i = 0; $i <= $#lines; $i++) {
        if($lines[$i] =~ /Connector\s+\w+/) {
            $lines[$i] =~ s/SAS Address/sas_address/;
            $lines[$i] =~ s/\;\s*//;
            @head = split(/\s+/, $lines[$i]);
        }
        if($lines[$i] =~ /Device\s+\w+/) {
            $lines[$i] =~ s/Device\t(\w)/Device$1/;
            @values = split(/\t/, $lines[$i]);
            for(my $j = 0; $j <= $#head; $j++) {
                $result_array[$count]->{to_key_name( item => $head[$j])} = $values[$j];
            }
            $count++;
        }
    }
    return \@result_array;
}

1;
