#$Id$
#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary API Set for BrocadeFC Switch CLI commands
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

package NACL::APISet::Switch::CLI::BrocadeFC;

#
# Compiler Directives
#
use strict;
use warnings;

use base qw(NACL::APISet::Switch::CLI);

#
# Module imports - APISet
#
use NACL::APISet::Session::CLI::BrocadeFC;
use NACL::APISet::_Mixins::CLI qw(_get_absolute_cdef_paths);
#
##Exceptions
##
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);

#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);

sub new
{
    $Log->enter() if $may_enter;

    my ($class, %args) = @_;
    my $self = {};
    $self = NACL::APISet::validate_params
      ($class,
       params => \%args,
       spec   =>
             {
              version   => { type => SCALAR, optional => 1, default => 99.99   },
             }
      );
    bless $self, $class;
    $Log->exit() if $may_exit;
    return $self;
}

##############################################################################
#                               Private Methods
##############################################################################

##############################################################################
# Method:    _init_API
# Objective: Initialize the BrocadeFC API Set
# Details:   Initializes the API Set.  This method drives the workflow needed
#            to make the API Set ready for work.  This is accomplished by
#            calling the parents' init_API, assigning a default CDEF path
##############################################################################
sub _init_API
{
    $Log->enter() if $may_enter;

    my ($self, %args) = @_;

    $self->_get_absolute_cdef_paths();
    unless ( defined $self->{cdef_path} ) {
        NACL::APISet::Exceptions::CDEFNotFoundException->throw(
                                          "Error in finding CDEF directories "
                                              . $self->{cdef_path}
                                              . " and the reason: \n$@" );
    }

    $self->SUPER::_init_API(%args);

    $Log->exit() if $may_exit;
}


=head1 NAME

NACL::APISet::Switch::CLI::BrocadeFC - API Set for BrocadeFC CLI commands

=head1 SYNOPSIS

use NACL::APISet::Switch::CLI::BrocadeFC;

=head1 DESCRIPTION

A C<NACL::APISet::Switch::CLI::BrocadeFC> is used to connect to and send CLI commands
to the BrocadeFC Switch.

=head1 EXCEPTIONS

=over

=item NACL::APISet::Exceptions::CDEFNotFoundException

=back

=head1 CONSTRUCTORS

=head2 new

Returns a reference to a newly created C<NACL::APISet::Switch::CLI::BrocadeFC>.

=over

=item Synopsis

 $api_set_obj = NACL::APISet::Switch::CLI::BrocadeFC->new( %args );

=item Arguments

=over

=item %args

(Optional)
Parameters passed to parent classes.  See NACL::APISet and
NACL::APISet::Switch::CLI for details.

=back

=item Returns

A reference to a new API Set.

=back

=head1 AUTHOR/MAINTAINER

=over

=item NACL Development (dl-nacl-dev@netapp.com)

=back

 $Id: //depot/prod/test/nacldev/lib/NACL/APISet/Host/CLI/BrocadeFC.pm#1 $

=cut

1;
