#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/ali.pm# $
#
#Copyright (c) 2011-2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC ali command
##
## @description
##    This is a library to parse the raw output of BrocadeFC alishow  command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::ali
##
##  alishow(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
## @chnages udaykk@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::ali;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    if($raw_output =~ /(.*)\s+does\s+not\s+exist/) {
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    }elsif($raw_output =~ /not\s+found/) {
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    } elsif($raw_output =~ /Usage/i) {
        $error = "InvalidParamValue";
        $$error_message = $1 ;
    }
    return $error;
}

## @name ::alishow
##
## @summary Parses the raw output from the "alishow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from alishow with any of its options as described in the
##    corresponding cdef file 'alishow.cdefs'
## @synopsis
##      alishow(%args);
## INPUTS TO METHOD CALL 'alishow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern' </td> </tr>
## <tr> <td> No switch for this key </td> <td> 'pattern-mode' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = alishow
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'effective_configuration' => [
##&nbsp;                                           {
##&nbsp;                                             'cfg' => [
##&nbsp;                                                        {
##&nbsp;                                                          'name' => 'camaro'
##&nbsp;                                                        }
##&nbsp;                                                      ],
##&nbsp;                                             'zone' => [
##&nbsp;                                                         {
##&nbsp;                                                           'member' => [
##&nbsp;                                                                         '50:0a:09:80:00:14:25:90',
##&nbsp;                                                                         '50:06:01:68:30:22:7a:7f'
##&nbsp;                                                                       ],
##&nbsp;                                                           'name' => 'camaro_87a_0c_cx5a_fe0'
##&nbsp;                                                         },
##&nbsp;                                                         {
##&nbsp;                                                           'member' => [
##&nbsp;                                                                         '50:0a:09:80:00:14:1b:a0',
##&nbsp;                                                                         '50:06:01:68:30:22:7a:7f',
##&nbsp;                                                                         'vgbr200es2:admin'
##&nbsp;                                                                       ],
##&nbsp;                                                           'name' => 'camaro_87b_0d_cx5b_fe0'
##&nbsp;                                                         }
##&nbsp;                                                       ]
##&nbsp;                                           }
##&nbsp;                                         ],
##&nbsp;            'defined_configuration' => [
##&nbsp;                                         {
##&nbsp;                                           'cfg' => [
##&nbsp;                                                      {
##&nbsp;                                                        'member' => [
##&nbsp;                                                                      'camaro_87a_0c_cx5a_fe0',
##&nbsp;                                                                      'camaro_87b_0d_cx5b_fe0'
##&nbsp;                                                                    ],
##&nbsp;                                                        'name' => 'camaro'
##&nbsp;                                                      }
##&nbsp;                                                    ],
##&nbsp;                                           'zone' => [
##&nbsp;                                                       {
##&nbsp;                                                         'member' => [
##&nbsp;                                                                       'vgv3240f87a_0c',
##&nbsp;                                                                       'vrem500a_3b'
##&nbsp;                                                                     ],
##&nbsp;                                                         'name' => 'camaro_87a_0c_cx5a_fe0'
##&nbsp;                                                       },
##&nbsp;                                                       {
##&nbsp;                                                         'member' => [
##&nbsp;                                                                       'vgv3240f87b_0d',
##&nbsp;                                                                       'vrem500a_3b'
##&nbsp;                                                                     ],
##&nbsp;                                                         'name' => 'camaro_87b_0d_cx5b_fe0'
##&nbsp;                                                       }
##&nbsp;                                                     ],
##&nbsp;                                           'alias' => [
##&nbsp;                                                        {
##&nbsp;                                                          'member' => [
##&nbsp;                                                                        '50:0a:09:80:00:14:25:90'
##&nbsp;                                                                      ],
##&nbsp;                                                          'name' => 'vgv3240f87a_0c'
##&nbsp;                                                        },
##&nbsp;                                                        {
##&nbsp;                                                          'member' => [
##&nbsp;                                                                        '50:0a:09:80:00:14:1b:a0'
##&nbsp;                                                                      ],
##&nbsp;                                                          'name' => 'vgv3240f87b_0d'
##&nbsp;                                                        },
##&nbsp;                                                        {
##&nbsp;                                                          'member' => [
##&nbsp;                                                                        '50:06:01:68:30:22:7a:7f'
##&nbsp;                                                                      ],
##&nbsp;                                                          'name' => 'vrem500a_3b'
##&nbsp;                                                        }
##&nbsp;                                                      ]
##&nbsp;                                         }
##&nbsp;                                       ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub alishow {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error = _check_error(raw_output=>$output,error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
    my $option=$command;
    $option =~ s/alishow\s?//;
    my ($config,$type,$flag_cfg,$flag_zone,$flag_alias);
    my $i = 0;
    my $j;
    my $k =-1;
    my $n;
    my $m = -1;

    if($option eq "" ){
        my @lines = get_lines(raw_output=>$output);
        foreach my $line(@lines){
            if($line =~ /(\w+\s+configuration):/){
                $n=0;
                $j=0;
                $k=0;
                $config = to_key_name(item => $1);
                $flag_cfg = -1;
                $flag_zone = -1;
                $flag_alias = -1;
            }elsif($line =~ /(cfg):\s+(\w+)/ || $line =~ /(zone):\s+(\w+)/ || $line =~ /(alias):\s+(\w+)/){
                my $choice =$1;
                my $name = $2;
               if($choice eq "cfg"){
                   $flag_cfg++;
                   $type = "cfg";
                   $k=0;
                   if($flag_cfg > 0){
                   $n++;
                   }
                }elsif($choice eq "zone"){
                   $flag_zone++;
                   $type = "zone";
                   $k=0;
                   if($flag_zone > 0){
                   $n++;
                   }else{
                    $n=0;
                   }
                }elsif($choice eq "alias"){
                   $flag_alias++;
                   $type = "alias";
                   $k=0;
                   if($flag_alias > 0){
                   $n++;
                   }else{
                    $n=0;
                   }
                }
                $result_array[$i]->{$config}->[$j]->{$type} ->[$n] -> {"name"} = $name;
                my @arr_mem = split (/\s+/, $line);
                if(scalar(@arr_mem) > 2){
                    splice (@arr_mem,0,2);
                    $/ = ";";
                    chomp(@arr_mem);
                    foreach my $val(@arr_mem){
                        $result_array[$i]->{$config}->[$j]->{$type}->[$n]->{"member"}->[$k] = $val;
                        $k++;
                    }
                }
            }elsif($line =~ /(.*)/){
                my @arr_mem = split(/\s+/,$line);
                $/ = ";";
                chomp(@arr_mem);
                foreach my $mem(@arr_mem){
                    $result_array[$i]->{$config}->[$j]->{$type}->[$n]->{"member"}->[$k] = $mem;
                    $k++;
                }
            }
        }
        return \@result_array;
    }else{
        my @lines = get_lines(raw_output=>$output);
        my $j =-1;
        foreach my $line(@lines){
            if($line =~ /alias:\s+(.*)/){
                $j++;
                $result_array[$i]->{"alias"}->[$j]->{"name"} = $1;
            }elsif($line =~ /(.*)/){
                $result_array[$i]->{"alias"}->[$j]->{"member"} = $1;
            }
        }return \@result_array;
    }
}

## @name ::ali_create
##
## @summary Parses the raw output from the "alicreate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from ali_create with any of its options as described in the
##    corresponding cdef file 'ali.cdefs'
## @synopsis
##      ali_create(%args);
## INPUTS TO METHOD CALL 'ali_create' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aliname-members' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = alicreate "ali3","1,2"
##&nbsp;$VAR1 = [];
##&nbsp;
sub ali_create {
    my %args = @_;
    my @result_array;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    my $error = _check_error("raw_output"=>$output,
                             "command"=>"alicreate",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    my $value;
    my %info;
    my @lines = get_lines(raw_output=>$output, response_object=>$resp);
    foreach $value (@lines) {
        if($value =~ /trans_begin:\s+(.*)/i) {
            $info{"status"} = $1;
        } elsif($value =~ /.*\s+duplicate\s+name/i) {
            $info{"status"} = $1;
        }
    }
    push(@result_array, \%info) if(%info) ;
    return \@result_array;
}

## @name ::ali_delete
##
## @summary Parses the raw output from the "alidelete" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from ali_delete with any of its options as described in the 
##    corresponding cdef file 'ali.cdefs'
## @synopsis
##      ali_delete(%args);
## INPUTS TO METHOD CALL 'ali_delete' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'aliname-members' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = alidelete "ali3"
##&nbsp;$VAR1 = [];
##&nbsp;
sub ali_delete {
    my %args = @_;
    my $error_message;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();

    my $error = _check_error("raw_output"=>$output,
                             "command"=>"alidelete",
                             "ptr_error_message"=>\$error_message);
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
}
1;
