#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/chassisshow.pm#1 $
#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC chassisshow command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch chassisshow command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::chassisshow
##
##  chassisshow(%args)
##
## @author dl-nacl-dev@netapp.com, tanil@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::chassisshow;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
use Data::Dumper;

## @name ::chassisshow
##
## @summary Parses the raw output from the "chassisshow" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from chassisshow with any of its options as described in the
##    corresponding cdef file 'chassisshow.cdefs'
## @synopsis
##      chassisshow(%args);
## INPUTS TO METHOD CALL 'chassisshow' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = chassisshow
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'chassis/wwn' => [
##&nbsp;                               {
##&nbsp;                                 'factory_part_num' => '40-0500027-06',
##&nbsp;                                 'part_num' => 'SLKWRM0000200',
##&nbsp;                                 'factory_serial_num' => 'RD060069426',
##&nbsp;                                 'time_awake' => '56 days',
##&nbsp;                                 'manufacture' => 'Day: 10  Month:  9  Year: 2006',
##&nbsp;                                 'id' => 'BRD0000CA',
##&nbsp;                                 'time_alive' => '1602 days',
##&nbsp;                                 'Unit' => '1',
##&nbsp;                                 'header_version' => '2',
##&nbsp;                                 'update' => 'Day: 21  Month:  9  Year: 2011'
##&nbsp;                               }
##&nbsp;                             ],
##&nbsp;            'fan' => [
##&nbsp;                       {
##&nbsp;                         'time_awake' => '56 days',
##&nbsp;                         'Unit' => '1'
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'time_awake' => '56 days',
##&nbsp;                         'Unit' => '2'
##&nbsp;                       },
##&nbsp;                       {
##&nbsp;                         'time_awake' => '56 days',
##&nbsp;                         'Unit' => '3'
##&nbsp;                       }
##&nbsp;                     ],
##&nbsp;            'power_supply' => [
##&nbsp;                                {
##&nbsp;                                  'time_awake' => '56 days',
##&nbsp;                                  'Unit' => '1'
##&nbsp;                                }
##&nbsp;                              ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub chassisshow {
    my %args = @_;

    my $error_message;
    my $resp    = $args{response_object};
    my $output  = $resp->get_processed_output();
    my $command = $resp->get_actual_command();
    my $error;

    $error = _parse_for_errors(
        raw_output    => $output,
        error_message => \$error_message,
        command         => $command,
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    # Now we can parse the output
    $output = _chassisshow(
        command         => $command,
        raw_output      => $output,
        response_object => $resp
    );

    return $output;
} # end sub chassisshow

sub _chassisshow {
    my %args         = @_;

    my $raw_output   = $args{"raw_output"};
    my $command_line = $args{"command"};
    my $response     = $args{"response_object"};
    my @result_array = ();

    # For 'chassisshow', some output is always expected. So we are making
    # $can_throw as 1 so that if the lines array got finally in get_lines is
    # empty, UnexpectedOutputException can be thrown there.
    my $can_throw = 1;

    my @arr = get_lines(
        "raw_output"      => $raw_output,
        "can_throw"       => $can_throw,
        "response_object" => $response
    );
    my ($data1, $data2, $data3);
    my $i = 0;
    foreach my $line (@arr) {
        if ($line =~ /(.*)\s{2,}(unit|slot)\:\s+(\d+)/i ) {
            if ($data1 eq to_key_name(item=>$1)) {
                $i++;
            } else {
                $i = 0;
            }
            $data1 = to_key_name(item=>$1);
            $data2 = $2;
            $data3 = $3;
            $result_array[0]->{$data1}->[$i]->{$data2} = $data3;
            next;
        } elsif ($line =~ /^([\w\s]+):\s+([\d\-\:A-Za-z\s]+)$/ ) {
            $result_array[0]->{$data1}->[$i]->{to_key_name(item=>$1)} = $2;
        }
    }

    return \@result_array;
}

sub _parse_for_errors {
    my %args = @_;

    my $output = $args{raw_output};
    my $error_message = $args{error_message};
    my $command = $args{command};
    my $error;

    return $error;
}

1;
