#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/date.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse BrocadeFC date command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch date command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::date
##
##  date(%args)
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::date;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;
    if ( $raw_output =~ /No\s+licenses\s+installed.*/i ) {
        $error         = "FeatureNotLicensed";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /invalid\s+date.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    elsif ( $raw_output =~ /invalid\s+option.*/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::date
##
## @summary Parses the raw output from the "date" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from date with any of its options as described in the
##    corresponding cdef file 'date.cdefs'
## @synopsis
##      date(%args);
## INPUTS TO METHOD CALL 'date' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = date
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;Wed Dec 12 16:20:41 UTC 2012
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'date' => 'Wed Dec 12 16:20:41 UTC 2012'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub date {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    $result_array[0]->{date} = $lines[1];
    return \@result_array;
} ## end sub date

1;
