#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC  fosconfig --show command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch  fosconfig --show command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::fosconfig
##
##   fosconfig(%args)
##
## @author Govardhan Yadav (yadag@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::fosconfig;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error = undef;
    return $error;
}


## @name ::fosconfig_show
##
## @summary Parses the raw output from the "fosconfig --show" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from fosconfig_show with any of its options as described in the
##    corresponding cdef file 'fosconfig.cdefs'
## @synopsis
##      fosconfig_show(%args);
## INPUTS TO METHOD CALL 'fosconfig_show' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = fosconfig --show
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'virtual_fabric' => 'Service not supported on this Platform',
##&nbsp;            'isns_client_service' => 'Service not supported on this Platform',
##&nbsp;            'fc_routing_service' => 'Service not supported on this platform',
##&nbsp;            'iscsi_service' => 'Service not supported on this Platform'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub  fosconfig_show {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    $error = _check_error(raw_output=>$output,command=>" fosconfig --show",
                             error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }
    #Now we can parse the output
    my @lines = get_lines("raw_output"=>$output,"response_object"=>$resp);
    my (@Res,$value);
    my $i = 0;
    foreach $value (@lines){
        if($value =~/^vgbr200es2:admin/){
            next;
        } elsif($value =~/^([\w\s]+):\s+(.*)$/){
            $Res[$i]->{to_key_name(item => $1)} = $2;
        }
    }
    return \@Res;
}

1;
