#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/portlogdumpport.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC  portlogdumpport command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch  portlogdumpport command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::portlogdumpport
##
##   portlogdumpport(%args)
##
## @author sangavai@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::portlogdumpport;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args       = @_;
    my $resp       = $args{response_object};
    my $raw_output = $args{raw_output};
    my ( $error_message, $error );
    my $error_level = $resp->get_execution_status();
    if ( $raw_output =~ /invalid number of arguments|invalid input/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::portlogdumpport
##
## @summary Parses the raw output from the "portlogdumpport" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from portlogdumpport with any of its options as described in the
##    corresponding cdef file 'portlogdumpport.cdefs'
## @synopsis
##      portlogdumpport(%args);
## INPUTS TO METHOD CALL 'portlogdumpport' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = portlogdumpport 0
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;time         task       event   port cmd  args
##&nbsp;-------------------------------------------------
##&nbsp;Sun Jul 29 22:50:20 2012
##&nbsp;22:50:20.385 FCPH       write     0  204  00fffc02,00fffc04,00000000,00000000,00000000
##&nbsp;22:50:20.385 FCPH       seq       0   cc  00300000,00000000,000004e1,00020000,00000000
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'cmd' => '204',
##&nbsp;            'time' => '22:50:20.385',
##&nbsp;            'args' => '00fffc02,00fffc04,00000000,00000000,00000000',
##&nbsp;            'task' => 'FCPH',
##&nbsp;            'port' => '0',
##&nbsp;            'event' => 'write'
##&nbsp;          },
##&nbsp;          {
##&nbsp;            'cmd' => 'cc',
##&nbsp;            'time' => '22:50:20.385',
##&nbsp;            'args' => '00300000,00000000,000004e1,00020000,00000000',
##&nbsp;            'task' => 'FCPH',
##&nbsp;            'port' => '0',
##&nbsp;            'event' => 'seq'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub portlogdumpport {
    my %args = @_;
    my (@result_array);
    my $resp        = $args{response_object};
    my $output      = $resp->get_raw_output();
    my $error_level = $resp->get_execution_status();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines
        = get_lines( "raw_output" => $output, "response_object" => $resp );
    my $count = -1;
    my ( @head, @values );

    for ( my $i = 0; $i <= $#lines; $i++ ) {
        if ( $lines[$i] =~ /^time/ ) {
            @head = split( /\s+/, $lines[$i] );
        }
        if ( $lines[$i] =~ /^\d+:\d+/ ) {
            @values = split( /\s+/, $lines[$i] );
            $count++;
            for ( my $j = 0; $j <= $#head; $j++ ) {
                $result_array[$count]->{ to_key_name( item => $head[$j] ) }
                    = $values[$j];
            }
        }
    }
    return \@result_array;
} ## end sub portlogdumpport

1;
