#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/portstatsclear.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC portstatsclear command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch portstatsclear command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::portstatsclear
##
##  portstatsclear(%args)
##
## @author udaykk@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::portstatsclear;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error 
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    } elsif ( $raw_output =~ /(Invalid input parameters)|(Valid Port Range 0-47)|(This is not a slot based system)/i ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::portstatsclear
##
## @summary Parses the raw output from the "portstatsclear" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from portstatsclear with any of its options as described in the 
##    corresponding cdef file 'portstatsclear.cdefs'
## @synopsis
##      portstatsclear(%args);
## INPUTS TO METHOD CALL 'portstatsclear' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> '-i' </td> <td> 'port-index' </td> <td> '0' </td> </tr> 
## <tr> <td> No switch for this key </td> <td> 'port' </td> <td> '0' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = portstatsclear 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub portstatsclear {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => "portstatsclear",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }

    return [];
} ## end sub portstatsclear
1;
