#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/timeout.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC timeout command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch timeout command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::timeout
##
##  timeout(%args)
##
## @author sangavai(sangavai@netapp.com),dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::timeout;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my $error_message;
    my $error = undef;
    if ( $raw_output =~ /Please specify a numeric timeout.*/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::timeout
##
## @summary Parses the raw output from the "timeout" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from timeout with any of its options as described in the
##    corresponding cdef file 'timeout.cdefs'
## @synopsis
##      timeout(%args);
## INPUTS TO METHOD CALL 'timeout' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = timeout
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;; echo __status__=$?
##&nbsp;Current IDLE Timeout is 0 minutes
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'current_idle_timeout' => '0 minutes'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;
sub timeout {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    my @lines = get_lines( raw_output => $output, response_object => $resp );
    my @result_array;
    foreach my $line (@lines) {
					if ( $line =~ /(\w+\s+\w+\s+\w+\s+)(\d+\s.*)/ ) {
            $result_array[0]->{current_idle_timeout} = $2;
        }
    }
    return \@result_array;
} ## end sub timeout

1;
