#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/zonecreate.pm#1 $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse BrocadeFC zonecreate command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch zonecreate command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::zonecreate
##
##  zonecreate(%args);
##
## @author dl-nacl-dev@netapp.com, skotagir@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::zonecreate;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $resp = $args{response_object};
    my $error_message ;
    if($raw_output =~
        /(duplicate name|error: Usage:.*)/) {
        $error = "InvalidParamValue";
        $error_message = $1 ;
    }elsif($raw_output =~ /(invalid parameter)/){
        $error = "InvalidParam";
        $error_message = $1 ;
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
}

## @name ::zonecreate
##
## @summary Parses the raw output from the "zonecreate" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zonecreate with any of its options as described in the
##    corresponding cdef file 'zonecreate.cdefs'
## @synopsis
##      zonecreate(%args);
## INPUTS TO METHOD CALL 'zonecreate' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'zone-name' </td> <td> '0' </td> </tr>
## <tr> <td> ',' </td> <td> 'member' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = zonecreate "zone1","1,1"
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;

sub zonecreate {
    my %args = @_;
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error(raw_output=>$output, response_object=>$resp);

    return [] ;
}
1;
