#$Id: //depot/prod/DOT/dev/test/lib/NACL/APISet/Switch/CLI/BrocadeFC/default/zoneobjectrename.pm#1 $
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
# Any other use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse BrocadeFC zoneobjectrename command
##
## @description
##    This is a library to parse the raw output of BrocadeFC switch zoneobjectrename command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::BrocadeFC::default::zoneobjectrename
##
##  zoneobjectrename(%args);
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::BrocadeFC::default::zoneobjectrename;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $resp = $args{response_object};
    if($raw_output =~
        /(duplicate name|error:\s+Usage:\s+zoneobjectrename|error:\s+Or:\s+zoneobjectrename)/) {
        $error = "InvalidParamValue";
    }elsif ( $raw_output =~ /(not\s+found|trans_begin)/ ) {
        $error         = "CommandFailed";
    }
        elsif($raw_output =~ /(invalid parameter)/){
        $error = "InvalidParam";
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$raw_output, response_object => $resp);
    }
}

## @name ::zoneobjectrename
##
## @summary Parses the raw output from the "zoneobjectrename" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from zoneobjectrename with any of its options as described in the
##    corresponding cdef file 'zoneobjectrename.cdefs'
## @synopsis
##      zoneobjectrename(%args);
## INPUTS TO METHOD CALL 'zoneobjectrename' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'objectname' </td> <td> '0' </td> </tr>
## <tr> <td> ',' </td> <td> 'newname' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples
##&nbsp;Command = zoneobjectrename vgv8080f148b_9a_VMAXe_2EP0,vgv8080f148b_9a_VMAXe_2EP0_141
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
sub zoneobjectrename {
    my %args = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    _check_error(raw_output=>$output, response_object=>$resp);

    return [];
} ## end sub zoneobjectrename
1;
