#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/port_license.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC port-license command
##
## @description
##    This is a library to parse the raw output of CiscoFC switch port-license command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::port_license
##
##  port_license(%args)
##
## @author udaykk@netapp.com dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::CiscoFC::default::port_license;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error 
    if ( $raw_output =~ /Usage:(.*)/ ) {
        $$error_message = $1;
        $error          = "InvalidParam";
    }
    return $error;
} ## end sub _check_error

## @name ::port_license
##
## @summary Parses the raw output from the "port-license" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from port_license with any of its options as described in the 
##    corresponding cdef file 'port_license.cdefs'
## @synopsis
##      port_license(%args);
## INPUTS TO METHOD CALL 'port_license' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'acquire' </td> <td> 'acquire-port-license' </td> <td> '1' </td> </tr> 
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = port-license acquire-port-license 1
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub port_license {
    my %args = @_;

    my $error_message;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error;
    my @result_array;

    # Status returned by switch for this command is
    # is ZERO even if command failed
    $error = _check_error(
        raw_output    => $output,
        command       => "port-license",
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    
    return [];
} ## end sub port_license
1;
