#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/show_flogi_database.pm# $
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC show_flogi_database command
##
## @description
##    This is a library to parse the raw output of CiscoFC show_flogi_database command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::show_flogi_database
##
##  show_flogi_database(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::show_flogi_database;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException qw(:try);

sub _check_error {
    my $error;
    my %args          = @_;
    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    if ( $raw_output =~ /(Invalid\s+command)/ ) {
        $error          = "CommandFailed";
        $$error_message = $1;
    }
    return $error;
} ## end sub _check_error

## @name ::show_flogi_database
##
## @summary Parses the raw output from the "show flogi database" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_flogi_database_detail with any of its options as described in the
##    corresponding cdef file 'show_flogi_database.cdefs'
## @synopsis
##      show_flogi_database(%args);
## INPUTS TO METHOD CALL 'show_flogi_database' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show flogi database
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;Response get_parsed_output:
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'total_number_of_flogi' => '21',
##&nbsp;            'ports_info' => [
##&nbsp;                              {
##&nbsp;                                'port_name' => '21:00:00:e0:8b:9b:7a:f1',
##&nbsp;                                'node_name' => '20:00:00:e0:8b:9b:7a:f1',
##&nbsp;                                'interface' => 'fc1/2',
##&nbsp;                                'vsan' => '1',
##&nbsp;                                'fcid' => '0x1b1600'
##&nbsp;                              },
##&nbsp;                              {
##&nbsp;                                'port_name' => '50:0a:09:80:00:02:37:1f',
##&nbsp;                                'node_name' => '50:0a:09:80:00:02:37:1f',
##&nbsp;                                'interface' => 'fc1/3',
##&nbsp;                                'vsan' => '1',
##&nbsp;                                'fcid' => '0x1b1400'
##&nbsp;                              },
##&nbsp;                              .
##&nbsp;                              .
##&nbsp;                              .
##&nbsp;                            ]
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_flogi_database {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error       = _check_error(
        raw_output    => $output,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    my $i = 0;
    my $val;
    my $j = 0;
    my $name = "ports_info";
    my @arr = get_lines( raw_output => $output, response_object => $resp );
    foreach $val (@arr) {
        if ( $val =~ /(\w+\/\d+)\s+(\d+)\s+(\w+)\s+((\w+:?)+)\s+((\w+:?)+)/ )
        {
            $result_array[$i]->{"$name"} -> [$j] ->{"interface"} = $1;
            $result_array[$i]->{"$name"} -> [$j] ->{"vsan"}      = $2;
            $result_array[$i]->{"$name"} -> [$j] ->{"fcid"}      = $3;
            $result_array[$i]->{"$name"} -> [$j] ->{"port_name"} = $4;
            $result_array[$i]->{"$name"} -> [$j] ->{"node_name"} = $6;
            $j++;
        } elsif ( $val =~ /(\w+\s+)+\=\s+(\d+)/ ) {
            $result_array[$i]->{"total_number_of_flogi"} = $2;
        }
    }
    return \@result_array;
} ## end sub show_flogi_database
1;
