#
#Copyright (c) 2011 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC showclock command
##
## @description
##    This is a library to parse the raw output of CiscoFC showclock command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::showclock
##
##  showclock(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::showclock;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException qw(:try);

sub _check_error {
    my $error;
    my %args = @_;
    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    if($raw_output =~ /Invalid\s+command/) {
        $error = "InvalidParam";
        $$error_message = $1 ;
    }
    return $error;
}

## @name ::show_clock_detail
##
## @summary Parses the raw output from the "show clock detail" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from show_clock_detail with any of its options as described in the
##    corresponding cdef file 'showclock.cdefs'
## @synopsis
##      show_clock_detail(%args);
## INPUTS TO METHOD CALL 'show_clock_detail' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = show clock detail
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'seconds' => '47.784',
##&nbsp;            'hours' => '08',
##&nbsp;            'date' => '04',
##&nbsp;            'month' => 'Oct',
##&nbsp;            'day' => 'Tue',
##&nbsp;            'time_zone' => 'UTC',
##&nbsp;            'minutes' => '52',
##&nbsp;            'year' => '2011'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub show_clock_detail {
    my %args = @_;
    my ($error_message, @result_array);
    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my $error = _check_error(raw_output=>$output,error_message=>\$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message, response_object => $resp);
    }
    my @lines = get_lines(raw_output=>$output);
    my %hash;
    if($lines[0] =~ /(\d+):(\d+):(\d+.\d+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\d+)\s+(\d+)/){
        my %hash;

        $hash{hours} = $1;
        $hash{minutes} = $2;
        $hash {seconds} = $3;
        $hash {time_zone} =$4 ;
        $hash {day} = $5;
        $hash {month} = $6;
        $hash {date} = $7;
        $hash {year} = $8;

        push (@result_array,\%hash);
    }
    return \@result_array;
}
1;
