#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoFC "shutdown" and "no shutdown" commands
##
## @description
##    This is a library to parse the raw output of CiscoFC switch "shutdown" and "no shutdown" commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::shutdown
##
##  shutdown(%args)
##
## @author jais@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::shutdown;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp = $args{"response_object"};
    my $error_message;
    my $error;

    # Check for the error
    if ($raw_output =~ /Invalid command/) {
        $error = "InvalidParam";
        $error_message = "Invalid command option";
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
}

## @name ::shutdown
##
## @summary Parses the raw output from the "shutdown" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from shutdown any of its options as described in the
##    corresponding cdef file 'shutdown.cdefs'
## @synopsis
##      shutdown(%args);
## INPUTS TO METHOD CALL 'shutdown' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = shutdown
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub shutdown {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}

1;
