#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/CiscoFC/default/terminal.pm#1 $
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
## @summary Routines to parse CiscoFC terminal command
##
## @description
##    This is a library to parse the raw output of CiscoFC terminal command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoFC::default::terminal
##
##  terminal(%args)
##
## @author skotagir@netapp.com,dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoFC::default::terminal;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::InvalidParamException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error );
    if ( $raw_output =~ /Invalid command/i ) {
        $error         = "InvalidParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Incomplete command/i ) {
        $error         = "MissingRequiredParam";
        $error_message = $raw_output;
    } elsif ( $raw_output =~ /Invalid number/i ) {
        $error         = "InvalidParamValue";
        $error_message = $raw_output;
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
                . "indicating that the command has failed",
            response_object => $resp
        );
    }

} ## end sub _check_error

## @name ::terminal_session_timeout
##
## @summary Parses the raw output from the "terminal session-timeout" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from terminal_session_timeout with any of its options as described in the
##    corresponding cdef file 'terminal.cdefs'
## @synopsis
##      terminal_session_timeout(%args);
## INPUTS TO METHOD CALL 'terminal_session_timeout' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'timeout' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub terminal_session_timeout {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
} ## end sub terminal_session_timeout
1;
