#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork cdp and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch cdp and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::IOS::cdp
##
##
## @author dl-nacl-dev@netapp.com, Aparna.S2@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::IOS::cdp;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;


sub _check_error {
    my %args = @_;
	my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $resp       = $args{response_object};
    my $error ;
    # Check for the error
    if ($raw_output =~ /Invalid command|Incomplete command/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    }
    elsif ( $raw_output =~ /Invalid input/i ) {
        $error         = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

sub cdp_run {
     my %args = @_;

    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_run

sub cdp_advertisev2 {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_aadvertise

sub cdp_holdtime {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_holdtime

sub cdp_timer {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
	return [];
} ## end sub cdp_timer
1;
