#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork cdp and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch cdp and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::cdp
##
##  cdp_enable(%args)
##  show_cdp_all(%args)
##
## @author dl-nacl-dev@netapp.com, Veerabhadraswamy.P@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::cdp;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;
#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;


sub _check_error {
    my %args = @_;
	my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $resp       = $args{response_object};
    my $error ;
    # Check for the error
    if ($raw_output =~ /Invalid command|Incomplete command/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    }
    elsif ( $raw_output =~ /Invalid number/i ) {
        $error         = "InvalidParamValue";
        $$error_message = $raw_output;
    }
    return $error;
}

## @name ::cdp_enable
##
## @summary Parses the raw output from the "cdp enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from cdp_enable with any of its options as described in the
##    corresponding cdef file 'cdp.cdefs'
## @synopsis
##      cdp_enable(%args);
## INPUTS TO METHOD CALL 'cdp_enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub cdp_enable {
     my %args = @_;

    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();

    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_enable

sub cdp_advertise {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_aadvertise

sub cdp_holdtime {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_holdtime

sub cdp_timer {
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
	return [];
} ## end sub cdp_timer

sub cdp_format{
    my %args = @_;
    my ( $error_message, @result_array );
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $command     = $resp->get_command();
    my $error = _check_error(
        raw_output    => $output,
        command       => $command,
        error_message => \$error_message
    );
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    return [];
} ## end sub cdp_format

1;
