#
#Copyright (c) 2010 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse CiscoNetwork end and its subcommands
##
## @description
##    This is a library to parse the raw output of CiscoNetwork switch end and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::CiscoNetwork::default::end
##  
##  end_mac_address_table(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::CiscoNetwork::Nexus::end;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $command = $args{command};
    my $error_message = $args{error_message};
    my $error ;

    # Check for the error
    if ($raw_output =~ /(Invalid\s+input\s+detected)/i) {
        $$error_message = $raw_output;
        $error = "InvalidParam";
    } elsif ($raw_output =~ /(Incomplete\s+command)/i) {
        $$error_message = $raw_output;
        $error = "MissingRequiredParam"
    } elsif ($raw_output =~ /(Unknown\s+command\s+or\s+computer\s+name)/i) {
        $$error_message = $raw_output;
        $error = "CommandFailed"
    }

    return $error;
}
## @name ::end
##
## @summary Parses the raw output from the "end" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from end with any of its options as described in the 
##    corresponding cdef file 'end.cdefs'
## @synopsis
##      end(%args);
## INPUTS TO METHOD CALL 'end' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = end
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub end {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my $error_level = $resp->get_execution_status();
    my ($error, $key);

    # Status returned by switch for this command is  
    # is ZERO even if command failed
    if (not defined($error)) {
        $error = _check_error(raw_output    => $output,
                              command       => "end",
                              error_message => \$error_message);
    }
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];

}

1;
