#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse DellEthernet configure command
##
## @description
##    This is a library to parse the raw output of DellEthernet switch configure command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::DellEthernet::default::configure
##
##  configure(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::DellEthernet::default::configure;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output    = $args{raw_output};
    my $command       = $args{command};
    my $error_message = $args{error_message};
    my $error;

    # Check for the error
    if ( $raw_output =~ /Usage:(.*)|Invalid\s+input\s+detected/i ) {
        $$error_message = $raw_output;
        $error          = 'InvalidParam';
    }
    return $error;
} ## end sub _check_error

## @name ::configure
##
## @summary Parses the raw output from the "configure" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from configure with any of its options as described in the 
##    corresponding cdef file 'configure.cdefs'
## @synopsis
##      configure(%args);
## INPUTS TO METHOD CALL 'configure' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'subsystem' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.

sub configure {
    my %args = @_;

    my ($error_message);

    my $resp = $args{response_object};
    my $output = $resp->get_processed_output();
    my $command = $resp->get_command();
    my ($error, $key);
    $error = _check_error(raw_output    => $output,
                              command       => "admin start",
                              error_message => \$error_message);
    if($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text=>$error_message,response_object => $resp);
    }

    return [];
} ## end sub configure

1;
