#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse DellEthernet end command
##
## @description
##    This is a library to parse the raw output of DellEthernet
##    switch end command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::DellEthernet::default::end
##
##  end(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::DellEthernet::default::end;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $error_message = $raw_output;
        $error         = 'InvalidParam';
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . 'indicating that the command has failed',
            response_object => $args{response_object}
        );
    }
    return;
}

## @name ::end
##
## @summary Parses the raw output from the "end" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from end with any of its options as described in the
##    corresponding cdef file 'end.cdefs'
## @synopsis
##      end(%args);
## INPUTS TO METHOD CALL 'end' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
## @return
## Returns an empty array reference.

sub end {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

1;
