#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse DellEthernet terminal command
##
## @description
##    This is a library to parse the raw output of DellEthernet
##    switch terminal command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::DellEthernet::default::terminal
##
##  terminal(%args)
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package  NACL::APISet::Switch::CLI::DellEthernet::default::terminal;

use strict;
use warnings;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args        = @_;
    my $raw_output  = $args{raw_output};
    my $resp        = $args{response_object};
    my $error_level = $resp->get_execution_status();
    my ( $error_message, $error ) = undef;

    # Check for the error
    if ( $raw_output =~ /Invalid\s+input.*|Please\s+specify\s+an\s+integer/i ) {
        $error_message = $raw_output;
        $error         = 'InvalidParam';
    } elsif ( $raw_output =~ /Incomplete\s+command/i ) {
        $error_message = $raw_output;
        $error         = 'MissingRequiredParam';
    }
    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
    if ($error_level) {
        NACL::APISet::Exceptions::CommandFailedException->throw(
            text => "Execution status was $error_level "
              . 'indicating that the command has failed',
            response_object => $args{response_object}
        );
    }
    return;
}

## @name ::terminal_length
##
## @summary Parses the raw output from the "terminal length" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from terminal_length with any of its options as
##    described in the corresponding cdef file 'terminal.cdefs'
## @synopsis
##      terminal(%args);
## INPUTS TO METHOD CALL 'terminal_length' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'no-of-lines' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
## @return
## Returns an empty array reference.

sub terminal_length {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

## @name ::terminal_monitor
##
## @summary Parses the raw output from the "terminal length" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from terminal_monitor with any of its options as
##    described in the corresponding cdef file 'terminal.cdefs'
## @synopsis
##      terminal(%args);
## INPUTS TO METHOD CALL 'terminal_monitor' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'no-of-lines' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
## @return
## Returns an empty array reference.

sub terminal_monitor {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    _check_error( raw_output => $output, response_object => $resp );
    return [];
}

1;
