#
#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary : configure command
##
## @description
##    This is a library to parse the raw output of Mellanox switch configure command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::Mellanox::default::configure
##
##  configure(%args)
##
## @author  dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::Mellanox::default::configure_terminal;

use strict;
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::CommandNotFoundException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp = $args{"response_object"};
    my $error_message;
    my $error;

    # Check for the errors
    if ($raw_output =~ /Unrecognized\s+command/) {
       $error_message = $raw_output;
       $error         = "CommandNotFound";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::'.$error.'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }
}

## @name ::configure_terminal
##
## @summary Parses the raw output from the "configure terminal" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from configure_terminal with any of its options as described in the 
##    corresponding cdef file 'configure_terminal.cdefs'
## @synopsis
##      configure_terminal(%args);
## INPUTS TO METHOD CALL 'configure_terminal' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = configure terminal
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub configure_terminal {
    my %args = @_;

    my $resp         = $args{response_object};
    my $raw_output   = $resp->get_processed_output();

    _check_error(raw_output=>$raw_output, response_object=>$resp);

    return [];
}
1;
