#Copyright (c) 2015 Network Appliance, Inc.
#All rights reserved.
#
## @summary : disable command
##
## @description
##    This is a library to parse the raw output of Mellanox switch disable command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::Mellanox::defalt::disable
##
##  disable(%args)
##
## @author  dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::Mellanox::default::disable;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;
use NACL::APISet::Exceptions::CommandNotFoundException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /Could not set user password.*/i ) {
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Incorrect Password.*/i ) {
        $error         = "CommandFailed";
   } elsif ($raw_output  =~ /Unrecognized\s+command/i ) {
       $error          = "CommandNotFound";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $raw_output, response_object => $resp );
    }
}

## @name ::disable
##
## @summary Parses the raw output from the "disable" command
##
## @description 
##    This method can be used for parsing the raw output obtained after calling 
##    the method from disable with any of its options as described in the 
##    corresponding cdef file 'disable.cdefs'
## @synopsis
##      disable(%args);
## INPUTS TO METHOD CALL 'disable' ON APISET OBJECT 
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = disable
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub disable {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    return [];
}

1;
