#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/CN1601/write.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary Routines to parse NetAppNetwork show and its subcommands
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch write and its subcommands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::write
##
##  write_memory(%args)
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1601::write;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}

## @name ::write_memory
##
## @summary Parses the raw output from the "write memory" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from write_memory with any of its options as described in the
##    corresponding cdef file 'write.cdefs'
## @synopsis
##      write_memory(%args);
## INPUTS TO METHOD CALL 'write_memory' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = write memory
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Config file 'startup-config' created successfully .
##&nbsp;
##&nbsp;Configuration Saved!
##&nbsp;
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [
##&nbsp;          {
##&nbsp;            'status' => 'created successfully',
##&nbsp;            'config_file' => 'startup-config'
##&nbsp;          }
##&nbsp;        ];
##&nbsp;

sub write_memory {
    my %args = @_;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    if ($error_level) {
        _check_error(
            raw_output => $output,
        );
    }
    my @result_array;
    if ( $output =~ /Config\s+file\s+\'(.*)\'\s+(created\s+successfully)/ ) {
        $result_array[0]->{'config_file'} = "$1";
        $result_array[0]->{status} = "$2";
    }

    return \@result_array;
}

1;
