#$Id: //depot/prod/test/nacldev/lib/NACL/APISet/Switch/CLI/NetAppNetwork/CN1610/clear.pm# $
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary :  clear command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch clear command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::CN1610::clear
##
##  clear_config(%args)
##
## @author skotagir@netapp.com , dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::CN1610::clear;

use strict;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /(Invalid\s+input\s+detected)/i ) {
        $error_message = $raw_output;
        $error         = "InvalidParam";
    } elsif ( $raw_output =~ /An\s+invalid\s+interface\s+has\s+been\s+used/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
} ## end sub _check_error

## @name ::clear_config
##
## @summary Parses the raw output from the "clear config" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from clear_config with any of its options as described in the
##    corresponding cdef file 'clear.cdefs'
## @synopsis
##      clear_config(%args);
## INPUTS TO METHOD CALL 'clear_config' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub clear_config {
    my %args = @_;

    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    my $error_level = $resp->get_execution_status();

    if ($error_level) {
        _check_error(
            raw_output      => $output,
            response_object => "$resp"
        );
    }

    return [];
} ## end sub clear_config
## @name ::clear_counters
##
## @summary Parses the raw output from the "clear counters" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from clear_counters with any of its options as described in the
##    corresponding cdef file 'clear.cdefs'
## @synopsis
##      clear_counters(%args);
## INPUTS TO METHOD CALL 'clear_counters' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> No switch for this key </td> <td> 'interface' </td> <td> '0' </td> </tr>
## <tr> <td> 'all' </td> <td> 'all' </td> <td> '1' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

##&nbsp;Command = clear counters 0/1
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Are you sure you want to clear the port stats? (y/n)y
##&nbsp;
##&nbsp;Port Stats Cleared.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
##&nbsp;Command = clear counters all
##&nbsp;
##&nbsp;-------- Raw Output ---------
##&nbsp;
##&nbsp;Are you sure you want to clear the switch stats? (y/n)y
##&nbsp;
##&nbsp;Switch Stats Cleared.
##&nbsp;
##&nbsp;-------- Parsed Output ------
##&nbsp;$VAR1 = [];
##&nbsp;
sub clear_counters {
    my %args = @_;
    my $resp        = $args{response_object};
    my $output      = $resp->get_processed_output();
    _check_error(
        raw_output      => $output,
        response_object => $resp,
    );

    return [];
} ## end sub clear_counters
1;
