#$Id$
#
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.
#
## @summary : enable command
##
## @description
##    This is a library to parse the raw output of NetAppNetwork switch enable command.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::NetAppNetwork::defalt::enable
##
##  enable(%args)
##
## @author veerap@netapp.com, dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::NetAppNetwork::default::enable;

use strict;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;
use NACL::APISet::Exceptions::InvalidParamValueException;
use NACL::APISet::Exceptions::CommandFailedException;

sub _check_error {
    my %args = @_;

    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ( $error_message, $error );

    # Check for the error
    if ( $raw_output =~ /Could not set user password.*/ ) {
        $error_message = $raw_output;
        $error         = "InvalidParamValue";
    } elsif ( $raw_output =~ /Incorrect Password.*/ ) {
        $error_message = $raw_output;
        $error         = "CommandFailed";
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw( text => $error_message, response_object => $resp );
    }
}
## @name ::enable
##
## @summary Parses the raw output from the "enable" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from enable with any of its options as described in the
##    corresponding cdef file 'enable.cdefs'
## @synopsis
##      enable(%args);
## INPUTS TO METHOD CALL 'enable' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> <th> BOOLEAN OPTION </th> </tr>
## <tr> <td> 'password' </td> <td> 'password' </td> <td> '0' </td> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an array reference which contains the parsed output.
## @examples

sub enable {
    my %args   = @_;
    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();

    my $error = _check_error(
        raw_output      => $output,
        response_object => $resp
    );

    return [];
}

1;
