#
#Copyright (c) 2013 NetApp, Inc.
#All rights reserved.
#
## @summary Routines to parse QlogicFC set commands.
##
## @description
##    This is a library to parse the raw output of QlogicFC switch set
##    port commands.
##
## @synopsis
##  use NACL::APISet::Switch::CLI::QlogicFC::default::set
##
## @author dl-nacl-dev@netapp.com
##
## @status public
#
package NACL::APISet::Switch::CLI::QlogicFC::default::set;

use strict;
use warnings;
use NACL::APISet::Response::CLI::ParserUtils;

#Include exceptions which will be thrown
use NACL::APISet::Exceptions::InvalidParamException;

sub _check_error {
    my %args       = @_;
    my $raw_output = $args{raw_output};
    my $resp       = $args{response_object};
    my ($error_message, $error) = undef;
    my $error_level = $resp->get_execution_status();

    # Check for the error
    if ($raw_output =~ /(Usage:)|(not a valid value)/i) {
        $error_message = $raw_output;
        $error         = 'InvalidParam';
    }

    if ($error) {
        $error = 'NACL::APISet::Exceptions::' . $error . 'Exception';
        $error->throw(text => $error_message, response_object => $resp);
    }

    return $error;
} ## end sub _check_error

## @name ::set_port
##
## @summary Parses the raw output from the "set port" command
##
## @description
##    This method can be used for parsing the raw output obtained after calling
##    the method from set port with any of its options as described in the
##    corresponding cdef file 'set.cdefs'
## @synopsis
##      set_port_state(%args);
## INPUTS TO METHOD CALL 'set port' ON APISET OBJECT
## <table border = "1" >
## <tr> <th> OPTION </th> <th> PROGRAMMATIC NAME </th> </tr>
## </table>
## @arg %args Required:
## response_object => The response object that was constructed (containing the
##                    raw output, command line constructed and execution status)
## @return
## Returns an empty array reference.
## @examples
##&nbsp;Command = set port

sub set_port {
    my %args = @_;

    my ($error_message);

    my $resp   = $args{response_object};
    my $output = $resp->get_processed_output();
    my $error;
    _check_error(raw_output => $output, response_object => $resp);

    return [];

} ## end sub set_port

1;
