#
# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Version class for finding Version attributes specific to ONTAPSelectDeploy.

## @author dl-nacl-dev@netapp.com
##
## @status Review
## @pod here

package NACL::APISet::Version::Host::ONTAPSelectDeploy;

#
# Compiler Directives
#
use strict;
use warnings;

#
# Module imports
#
use NACL::APISet::Session::CLI::ONTAPSelectDeploy;
#
# NATE imports
#
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(:all);
use NATE::Exceptions::Argument qw(:try);
use Data::Dumper;

#
# base class
#
use base qw(NACL::APISet::Version::Host);

###############################################################################
# Method:    _find_os_revision
# Objective: Will return 7MTT version
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _find_os_revision {
    my ($self) = @_;
    $Log->enter();
    my ($major, $minor, $session);
    try {
        my $get_ONTAPSelectDeploy_session = sub {
            return $self->get_session(
                interface  => "CLI",
                subtype    => "ONTAPSelectDeploy",
            );
        };
        $session = $get_ONTAPSelectDeploy_session->();

        my $hostobj = $self->{hostobj};
	my $command = 'version';
        my $output = $self->execute_CLI_command(
            connection  => $session->{connection},
            command     => $command,
        );

	if( $output =~ /App Version\s+\|\s+(.+)\_.+/ ) {
	    my @arr = split(/_/, $1);
            $major = $arr[0];
        }
    } finally {
        # Destroy the session
        $self->destroy_session(session => $session) if (defined $session);
    };

    $self->{os_revision}->{major} = $major;
    $self->{os_revision}->{minor} = $minor;

    $Log->exit();
    return $major;
}

1;

=head1 NAME

NACL::APISet::Version::Host::ONTAPSelectDeploy - Version library for ONTAPSelectDeploy 

=head1 SYNOPSIS

 use NACL::APISet::Version::Host::ONTAPSelectDeploy;

=head1 DESCRIPTION

A C<NACL::APISet::Version::Host::ONTAPSelectDeploy> is used to connect to ONTAPSelectDeploy 
Host and collect version related attributes

=cut
