#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on resource cleanup
## @author dalmond@netapp.com

package NACL::AppBuilder::EnGenX::ClientCluster::cleanup;

# Compiler directives.
use strict;
use NATE::Log qw(log_global);

use vars qw(@ISA);
@ISA = qw(Error);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

###############################################################################
# Method:    cleanup_single
# Objective: Cleanup run against a single resource
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cleanup_single {
    my ( $self, %args ) = @_;

    &ParamsMgr::process_args(%args);
    my $resource = &ParamsMgr::get_arg(
        -arg      => "resource",
        -required => 1,
        -default  => ""
    );
    if ( &ParamsMgr::error() ) {
        $self->_set_err( &ParamsMgr::get_error() );
        &SubIPC::fail( -object => $self );
        return $SanMgr::FALSE;
    }

    # Load up the resource information and get them via get_arg
    my $caller_id = $resource->{NAME} . "|" . time();
    &ParamsMgr::process_args( %{$resource}, -CALLER => $caller_id );

    #############################################
    ### General client parameters
    #############################################:q!

    my $client_cluster_name = &ParamsMgr::get_arg(
        -arg            => "NAME",
        -required       => 1,
        -default_to_env => 1,
        -CALLER         => $caller_id
    );

    # Get client object
    my ($client_cluster_obj)
        = $self->get_objects( -type => "client_cluster" , -names => [$client_cluster_name] );
    if ( $self->error() ) {
        $self->_set_err(
            "Failed to retrieve object for client '$client_cluster_name'");
        return $SanMgr::FALSE;
    }

    # kill off any running I/O program on the client systems.
    #$self->_kill_io_instances_subtest(
    #    -client            => $client_name,
    #    -resource          => $resource,
    #    -client_io_program => "all"
    #);
    #if ( $self->error() ) {
    #    $self->_set_err("Failed to kill IO instances");
    #    &SubIPC::fail( -object => $self );
    #    return $SanMgr::FALSE;
    #}
    # Gather all resource groups assigned to the cluster
    #my @resources = $client_cluster_obj->get_resource_groups();
    #if ($client_cluster_obj->error()) {
    #    $self->_set_err(
    #        "Failed to retrieve object for client '$client_cluster_name'");
    #    return $SanMgr::FALSE;        
    #}
    #
    ## Destroy each resource group on the cluster
    #foreach my $resource (@resources) {
    #    $resource->destroy();
    #    if ($resource->error()) {
    #        $self->_set_err(
    #            "Failed to destroy resource for ". $resource->{name} . "\n");
    #        return $SanMgr::FALSE;
    #    }
    #}
    #
    ## Remove each node from the cluster
    #foreach my $node_obj ($client_cluster_obj->get_nodes()) {
    #    $node_obj->remove();
    #    if ($node_obj->error()) {
    #        $self->_set_err(
    #            "Failed to remove node " .  $node_obj->{name} . "\n");
    #         return $SanMgr::FALSE;
    #    }
    #}

    # Finally destroy the cluster
    $client_cluster_obj->cluster_destroy();
    if ($client_cluster_obj->error()) {
        $self->_set_err(
            "Failed to destroy cluster " . $client_cluster_name . "\n");
        return $SanMgr::FALSE;
    }

    &SubIPC::write(
        -msg         => "Completed client cluster($client_cluster_name) cleanup",
        -device_list => {},
        -type        => "cleanup_complete",
        -name        => $client_cluster_name
    );
    if ( &SubIPC::error() ) {
        $self->_set_err( &SubIPC::get_error() );
        &SubIPC::fail(
            -msg    => "Unable to write (config_complete) message to parent",
            -object => $self
        );
        return $SanMgr::FALSE;
    }

    &SubIPC::pass( -msg => "Client Cluster cleanup complete" );

    return $SanMgr::TRUE;
} ## end sub cleanup_single

1;
