# $Id: //depot/prod/test/main/lib/EnGenX/Controller/LunAttribute.pm#2 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunAttribute Controller Layer Module
## @author dl-san-automation
## @status public
## @pod here

=head1 NAME

NACL::AppBuilder::EnGenX::Controller::LunAttribute

=head1 DESCRIPTION

NACL::AppBuilder::EnGenX::Controller::LunAttribute is a derived class of
NACL::AppBuilder::EnGenX::Controller::Task.  It is used to Set Attributes
of ONTAP Luns. All methods are static.

=cut

package NACL::AppBuilder::EnGenX::Controller::LunAttribute;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Util::DataTypes qw(TRUE FALSE);

# NACL and EnGenX Libs
use NACL::STask::LunAttribute;

#Inherit base class
use base qw( NACL::AppBuilder::EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
            %DTD_To_NACL_Ignore
            %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
    _reuse => 'if_exists'
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
);

=head1 METHODS

=head2 create

(Static method) Request creation of one or multiple lunattributes serially
or in parallel and return whether the lunattribute(s) was created successfully.
This method passes all user arguments through to NACL::AppBuilder::EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(
            command_interface   => $Node,
            'volume'            => $Volume_name,
            'lun'               => $Lun_name,
            'vserver'           => $Vserver_name,
            'qtree'             => $Qtree_name,
            'name'              => $Attribute_name,
            'path'              => $Lun_path,
            'method-timeout'    => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(
            command_interface   => $Node,
            'volume'            => $Volume_name,
            'lun'               => $Lun_name,
            'vserver'           => $Vserver_name,
            'qtree'             => $Qtree_name,
            'name'              => $Attribute_name,
            'path'              => $Lun_path,
            'value'             => $Attribute_value
            'method-timeout'    => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(
            command_interfac  => $Node,
            resource_list     => @LunAttributes,
            'method-timeout'  => 3600,
            );


  NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(
            resource_list     => @LunAttributes,
            'method-timeout'  => 3600,
            );

  NACL::AppBuilder::EnGenX::Controller::LunAttribute->create(
            resource_list      => @LunAttributes,
            );

=over

=item Options

Either the resource_list is required or the NACL LunAttribute Creation Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the lunattributes in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL Lun Creation Arguments should
NOT be passed in. (This is an array of EnGenX::Resource() objects not
an EnGenX::Resource_List() currently)

=item C<< NACL LunAttribute Creation Arguments >>

(Optional)
These are arguments that can be passed into NACL::STask::LunAttribute->set().
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $NACL_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the Lun(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "lun_attribute";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head delete
Static Method with no function real function.
=cut

sub delete {
    $Log->comment("The delete subroutine was called, but it does nothing. Lun-Attributes will be deleted upon parent LUN deletion.");
    return;
} ## end sub delete

#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for creation of a lunattributes.
# Details:    Makes the NACL::STask::LunAttribute->set() call. Also works
#             around some common non-fatal errors that can be return from
#             lun creation and then has a retry mechanism to retry lun
#             creation.
# Inputs:     NACL::STask::LunAttribute->set arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the the attribute was set.
#             of the LunAttribute  was successful or a hash with success
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $wait = exists($args{wait}) ? delete($args{wait}) : FALSE;
    my $if_exists = exists($args{if_exists}) ? delete($args{if_exists}) : 'die';
    my $retry = exists $args{retry} ? delete $args{retry} : 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Build up the lun path.
    if (!defined($args{qtree})) {
        $args{path} = "/vol/$args{volume}/$args{lun}";
    } else {
        $args{path} = "/vol/$args{volume}/$args{qtree}/$args{lun}";
    }

    # Create Lun Attribute
    NACL::STask::LunAttribute->set(command_interface => $args{command_interface},
                               'vserver' => $args{vserver},
                               'path'     => $args{path},
                               'name'    => $args{name},
                               'value'   => $args{value});

   $Log->comment( "Attribute " . $args{name} . " successfully set with value of ("  . $args{value} . ") for lun: " . $args{lun});


    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE) : TRUE;
} ## end sub _nacl_create_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Makes a NACL call to 'wait_for_creation.
# Details:    Makes the call to NACL::STask::LunAttribute->wait_for_creation();
#             that will wait for the node to finish creating the aggr.
# Inputs:     NACL::STask::LunAttribute->wait arguments
# Outputs:    Returns whether the create was successful TRUE or not FALSE
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
# TODO: Implement this method when needed
#==============================================================================
sub _nacl_wait_call {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit() if $may_exit;
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
