#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::EnGenX::HyperVisor::HyperV;

use strict;
use warnings;

use NATE::BaseException qw(:try);
use NATE::Messenger qw(messenger_global);
use ParamsMgr;

use parent qw(NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Setup
              NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Cleanup
              NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Deconfig
              NACL::AppBuilder::EnGenX::HyperVisor::HyperV::Config
              NACL::AppBuilder::EnGenX::Util);

sub new {
    my ($class, %opts) = @_;
    my $self = \%opts;
    bless ($self, $class);

    # init or reuse a rep socket and store the endpoint
    $self->{endpoint} = messenger_global()->init(socket_type => 'rep',
                                                 action      => 'bind',
                                                 reuse       => 1);

    return $self;
}

sub xml_to_list {
    my ($self) = @_;

    # hyperv
    my $hyperv_hypervisors = &ParamsMgr::search(-search_path => [],
                                                -key => "HYPERVISORS",
                                                -key_filter => qr/^HYPERV\d*$/,
                                                paramset => $self->get_paramset());
    if (&ParamsMgr::error()) {
        NATE::BaseException->throw(&ParamsMgr::errmsg());
    }

    foreach my $hyperv_elm (values %{$hyperv_hypervisors}) {
        my $index = $hyperv_elm->{_INDEX} // "";
        $self->resource_list()->add_resource(category => "HYPERVISORS",
                                             type     => "hyperv",
                                             data     => $hyperv_elm);
        # find all virtual machines of each esx host and add them as resources
        my $virtual_machines = &ParamsMgr::search(-search_path => ["HYPERVISORS"],
                                                  -key => "HYPERV" . $index,
                                                  -key_filter => qr/^VM\d*$/,
                                                  paramset => $self->get_paramset());
        if (&ParamsMgr::error()) {
            NATE::BaseException->throw(&ParamsMgr::errmsg());
        }
        foreach my $vm_elm (values %{$virtual_machines}) {
            # add the vm as a resource and store the name of the parent hypervisor resource that it is
            # linked to
            $self->resource_list()->add_resource(category => "HYPERVISORS",
                                                 type => "virtual_machines",
                                                 data => {%{$vm_elm},
                                                          HYPERVISOR_RESOURCE => $hyperv_elm->{NAME}});
        }
    }
}

1;
