#imports perl NACL objects from igniter and creates/translates to python

import json
import os
import pprint

from NATE.Log import log_global
from NACL.APISet.APISet import APISet
from NACL.APISet.Response import Response
from Hostrec import Hostrec

from pynacl.c.node import NodeComponent
from pynacl.c.cluster import ClusterComponent

#from dp.xplatform.lib.ontap import Ontap,Snapmirror,Volume
from dp.xplatform.lib.client import Btrfs,BtrfsVolume

import pynacl.task.snapmirror
from pynacl.task.solidfire import Solidfire
from pynacl.task.solidfire_volume import SolidfireVolume
from pynacl.task.volume import Volume

CLASSNAME = '__class__'

class ObjectTransfer:


    def __init__(self,params=None, log=None):
        self.params = params
        if log is None:
            self.log = log_global()
        else:
            self.log = log

        self.ci = {}

    def load(self,param = 'igniter_params'):
        #self.params = json.loads(os.environ.get[param])
        #this throws an exception w/o param...is this what we want?
        self.params = json.loads(os.environ[param])

    def convert(self,create=False):
        self.log.info("convert...")
        params = self.params
        objects = {}
        solidfire_ci = None
        for p in params.keys():
            #print(p)
            self.log.info(p)
            if type(params[p]) is list:
                objects[p] = []
                for item in params[p]:
                    if CLASSNAME in item:
                        c = item[CLASSNAME]
                        self.log.info('class:' + c)
                        objtype = self.type_from_class(class_name=c)
                        self.log.info('objtype:' + str(objtype))
                        if str(objtype) == str(pynacl.task.snapmirror.Snapmirror):
                            self.log.info('snapmirror type oboy')
                            apiset = self.make_apiset(command_interface=item)
                            o = objtype(apisetObj=apiset, params=item)
                            objects[p].append(o)
                        elif str(objtype) == str(pynacl.task.solidfire.Solidfire):
                            self.log.info('solidfire type oboy')
                            apiset = self.make_apiset(command_interface=item)
                            o = objtype(command_interface=apiset, params=item)
                            objects[p].append(o)
                        elif str(objtype) == str(pynacl.task.solidfire_volume.SolidfireVolume):
                            self.log.info('solidfire volume type oboy')
                            apiset = self.make_apiset(command_interface=item)
                            #o = objtype(command_interface=apiset, solidfire_ci = solidfire_ci, params=item)
                            o = objtype(command_interface=apiset, solidfire_ci = solidfire_ci,  **item)
                            objects[p].append(o)
                            if create:
                                self.log.info('creating solidfire volume')
                                o.create()
                        elif str(objtype) == str(pynacl.task.volume.Volume):
                            self.log.info('volume type oboy')
                            self.log.info('item...:' + str(item))
                            apiset = self.make_apiset(command_interface=item)
                            #may have NACL serialized values,remove to prevent key conflict
                            item.pop('command_interface',None)
                            o = objtype(command_interface=apiset, **item)
                            objects[p].append(o)

                        elif objtype is not None:
                            apiset = self.make_apiset(command_interface=item)
                            o = objtype(apisetObj=apiset, params=item)
                            objects[p].append(o)
                        else:
                            #TODO do we want this? not an object, but...
                            objects[p].append(item)
                    else:
                        objects[p].append(item)
            else:
                #we don't handle dicts yet.  this is for scalars
                objects[p] = params[p]

        self.objects = objects
        pprint.pprint(objects)
        return objects


    def make_apiset(self,command_interface,force_node=True):
        #if an object w/a ci passed, just pull out ci
        if 'command_interface' in command_interface:
            command_interface = command_interface['command_interface']

        if CLASSNAME in command_interface:
            ci_type = 'node'
            ci_name = None
            template_type = 'c-login'  #by default
            make_apiset = True
            do_connect = True
            do_hostrec = False
            if command_interface[CLASSNAME] == 'NACL::STask::Cluster':
                if force_node:
                    self.log.info('force_node mode, generating node connection instead of cluster!')
                    ci_name = command_interface['node']
                    self.log.info("Using Node command interface")
                else:
                    ci_type = 'cluster'
                    ci_name = command_interface['name']
                    self.log.info("Using Cluster command interface")
            elif command_interface[CLASSNAME] == 'NACL::C::Node':
                ci_name = command_interface['node']
                self.log.info("Using Node command interface")
            elif command_interface[CLASSNAME] == 'NACL::AppBuilder::Configurator::Solidfire':
                ci_name = command_interface['name']
                template_type = 'login'
                make_apiset = False #TODO make better :)
                do_connect = False
                self.log.info("Using Solidfire command interface")
            elif command_interface[CLASSNAME] == 'NACL::AppBuilder::Configurator::SolidfireVolume':
                ci_name = command_interface['solidfire']
                template_type = 'login'
                make_apiset = False #TODO make better :)
                do_connect = False
                self.log.info("Using Solidfire command interface")
            elif command_interface[CLASSNAME] == 'NACL::AppBuilder::Configurator::Btrfs':
                ci_name = command_interface['name']
                template_type = 'login'
                make_apiset = False #TODO make better :)
                do_hostrec = True #CI compatibility change

                self.log.info("Using Btrfs command interface")
            elif command_interface[CLASSNAME] == 'NACL::AppBuilder::Configurator::BtrfsVolume':
                ci_name = command_interface['node']
                template_type = 'login'
                make_apiset = False #TODO make better :)
                do_hostrec = True #CI compatibility change
                self.log.info("Using Btrfs command interface")
            else:
                if 'node' in command_interface:
                    ci_name = command_interface['node']
                    self.log.info("unknown command interface " + command_interface[CLASSNAME] + ", auto selecting 'node' key " + ci_name)
                elif 'name' in command_interface:
                    ci_name = command_interface['name']
                    self.log.info("unknown command interface " + command_interface[CLASSNAME] + ", auto selecting 'name' key " + ci_name)
                else:
                    raise Exception("unknown command interface type:" + command_interface[CLASSNAME])
            self.log.info("create hostrec with: " + ci_name)
            hostrec = Hostrec.create(id = ci_name)
            self.log.info("after hostrec create with: " + ci_name + str(hostrec))
            hostrec_connection = None
            connection = None
            ci = None
            if do_connect:
                hostrec_connection = hostrec.connect(template = template_type)
                ci = self.get_command_interface(name = ci_name,type=ci_type)
                connection = ci
            apiset = None
            if make_apiset:
                return ci
                #apiset = APISet(connection=connection,category='Node',interface='CLI',set='CMode')
                #return apiset
            elif do_connect:
                if do_hostrec:
                    return hostrec_connection
                else:
                    return connection
            else:
                return hostrec
        else:
            raise Exception("no classname key in command interface: " + CLASSNAME)




    def get_command_interface(self,name,type='node'):
        if name not in self.ci:
            if type == 'node':
                ci = NodeComponent.new(node=name)
                self.ci[name] = ci
            elif type == 'cluster':
                ci = ClusterComponent.new(cluster=name)
                self.ci[name] = ci
            else:
                raise Exception("unknown command interface type:" + type)

        return self.ci[name]


    def type_from_class(self, class_name):
        """map from NACL perl lib name to python class type"""
        mapping = {
                'NACL::AppBuilder::Configurator::Solidfire' : Solidfire,
                'NACL::AppBuilder::Configurator::SolidfireVolume' : SolidfireVolume,
                'NACL::AppBuilder::Configurator::Btrfs' : Btrfs,
                'NACL::AppBuilder::Configurator::BtrfsVolume' : BtrfsVolume,
                'NACL::STask::Volume' : Volume,
                #'NACL::STask::Snapmirror' : Snapmirror,
                'NACL::STask::Snapmirror' : pynacl.task.snapmirror.Snapmirror,
                }
        #do regex instead?

        if class_name in mapping:
            typename = mapping[class_name]
            self.log.info("Mapping " + class_name + " to " + str(typename))
            return typename
        else:
            self.log.info("unknown class, failed mapping: " + class_name)
            return None

    def __str__(self):
        return pprint.pformat(self.params)








