package NACL::AppBuilder::Xgen::Config::Asup2Config::AggrParser; 

use strict;
use warnings;
use Path::Class;
use Data::Dumper;

sub new {
    my ($class, %args) = @_;
    my $self = \%args;

    bless($self, $class);

    return $self;
}

sub parse_aggr_files {
    my ($self, %args) = @_;
    my $aggr_status_r = delete($args{'aggr_status_r'}) || die"aggr_status_r has not been provided" ;
    my $aggr_status_v = delete($args{'aggr_status_v'}) || die"aggr_status_v has not been provided" ;
    my $df_a = delete($args{'df_a'}) || die"df_a has not been provided" ;
    
    #getting root aggr name
    my $file = file("$aggr_status_v");

    # Read in the entire contents of a file
    my $content = $file->slurp();

    my @content = split(/\n/, $content);

    my $root_aggr;

    foreach my $text (@content) {
        if ($text =~ /root, /) {
            my @text = split(/ /, $text);
            foreach my $aggr (@text) {
                if(($aggr =~ /^aggr*/) && !($aggr =~ /^aggr$/)) {
                    $root_aggr = $aggr;
                    last;
                }
            }
            last;
        }
    }

    #get aggr names, raid type and disk counts.
    my $aggr_file = file("$aggr_status_r");

    my $aggr_contents = $aggr_file->slurp();

    my @aggr_contents = split(/\n/, $aggr_contents);
    my $file_size = scalar @aggr_contents;
    my %aggr;
    my @aggr_names;

    foreach my $text (@aggr_contents) {
        if (($text =~ /Aggregate/) && !($text =~ /$root_aggr/)) {
            my @text = split(/ /, $text);
            push(@aggr_names, $text[1]);
            my @raid = split(/\)/, $text[3]); 
            $aggr{$text[1]}{"raid_type"} = $raid[0];  
        }
    }

    foreach my $aggregate (@aggr_names) {
        my $start = 0;
        my $disk_count = 0;
        foreach my $text (@aggr_contents) {
            if (($text =~ /Aggregate/) && ($text =~ /$aggregate/)) {
                $start = 1;
            } elsif (($text =~ /Aggregate/) && !($text =~ /$aggregate/)) {
                $start = 0;
            }

            if ($start) {
                if ($text =~ /dparity\s+|parity\s+|data\s+/){
                    $disk_count++;
                }
            }
        }
        $aggr{$aggregate}{"disk_count"} = $disk_count;
    }

    my $df_file = file("$df_a");

    my $df_contents = $df_file->slurp();

    my @df_contents = split(/\n/, $df_contents);
    foreach my $aggregate (@aggr_names) {
        foreach my $text (@df_contents) {
            if ($text =~ /$aggregate\s/) {
                my @text = split(/ /, $text);
                $aggr{$aggregate}{"size"} = $text[1]*1024;
            }
        }
    }
    return \%aggr;
}

1;
