#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::Cifs;

use strict;
use warnings;

use Params::Validate qw(validate_with SCALAR);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;
use NATE::Log qw(log_global);

use parent qw( NACL::Service::Xgen::Resource );

use constant nacl_cs_class => 'NACL::CS::VserverCifs';

sub _validate_private_attrs_with_nacl_cs_class {
    my ($self, %args) = @_;
    log_global()->enter();

    # remove special args from the validation
    # or else the base class nacl cs class validation will complain
    delete($args{'username'});
    delete($args{'password'});
    delete($args{'group-policy-enabled'});

    $self->SUPER::_validate_private_attrs_with_nacl_cs_class(%args);

    log_global()->exit();
}

sub _validate_private_attrs {
    my ($self, $args) = @_;
    log_global()->enter();

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                'username' => {type => SCALAR, optional => 1 },
                'password' => {type => SCALAR, optional => 1 },
                'group-policy-enabled' => { type => SCALAR, optional => 1},
            },
            allow_extra => 1);
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    log_global()->exit();
    return $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::Cifs

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
