#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
package NACL::AppBuilder::Xgen::Resource::ClientCluster;

use strict;
use warnings;

use Data::Dumper;

use Hostrec;

use parent qw( NACL::Service::Xgen::Resource );

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;

sub new {
    my ($class, %args) = @_;

    my $self = $class->SUPER::new(%args);

    $self->_initialize();

    return $self;
}

sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                name                      => {type => SCALAR},
                primary_node              => {type => SCALAR, optional => 1},
                secondary_nodes           => {type => SCALAR, optional => 1},
                cluster_ip                => {type => SCALAR, optional => 1},
                cluster_subnet            => {type => SCALAR, optional => 1},
                cluster_type              => {type => SCALAR, optional => 1},
                windows_domain_name       => {type => SCALAR, optional => 1},
                windows_domain_username   => {type => SCALAR, optional => 1},
                windows_domain_password   => {type => SCALAR, optional => 1},
                windows_domain_controller => {type => SCALAR, optional => 1},
                is_hypervisor             => {type => SCALAR, optional => 1, default => 0},
            });
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    return $args;
}

sub _initialize {
    my ($self, %args) = @_;

    # cluster_name
    # cluster_ip
    # cluster_subnet
    # cluster_type
    # windows_domain_name
    # windows_domain_username
    # windows_domain_password
    # secondary_nodes
    # primary_node
    # windows_domain_controller
    # cluster_lun_regex

    my $host_obj = Hostrec->new(id => $self->get_attribute('name'));
    my %hash = $host_obj->param_fields();

    $self->set_private_attributes(
        primary_node              => $hash{PRIMARY_NODE},
        secondary_nodes           => $hash{SECONDARY_NODES},
        cluster_ip                => $hash{CLUSTER_IP},
        cluster_subnet            => $hash{CLUSTER_SUBNET},
        cluster_type              => $hash{CLUSTER_TYPE},
        windows_domain_name       => $hash{WINDOWS_DOMAIN_NAME},
        windows_domain_username   => $hash{WINDOWS_DOMAIN_USERNAME},
        windows_domain_password   => $hash{WINDOWS_DOMAIN_PASSWORD},
        windows_domain_controller => $hash{WINDOWS_DOMAIN_CONTROLLER},
    );

    return;
}


sub get_nodes {
    my ($self) = @_;

    my $host_obj = Hostrec->new(id => $self->get_attribute('name'));
    my %hash = $host_obj->param_fields();

    my $final = {};
    if ($hash{PRIMARY_NODE}) {
        $final->{primary_node} = $hash{PRIMARY_NODE};
    }
    if ($hash{SECONDARY_NODES}) {
        $final->{secondary_nodes} = [split(/\s*,\s*/, $hash{SECONDARY_NODES})];
    }

    return $final;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::ClientCluster

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (dl-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
