#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::Fabric;

use strict;
use warnings;

use parent qw( NACL::Service::Xgen::Resource );

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;
use NATE::ParamSet qw(param_global);

sub _validate_private_attrs {
    my ($self, $args) = @_;

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                name             => {type => SCALAR},
                prefix           => {type => SCALAR, default => "nate"},
                admin_domain     => {type => SCALAR, default => ""},
                virtual_fabric   => {type => SCALAR, default => ""},
                master_switch    => {type => SCALAR, default => ""},
                switches         => {type => ARRAYREF, default => []},
                zones_per_node_per_init => {type => SCALAR, default => 1},
                restrict_zoning_based_on_existing_igroups => {type => SCALAR, optional => 1}
            });
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    # if $(fabric_name).PREFIX value was set somewhere in the environment,
    # we need this to take highest precedence for the prefix value passed in
    if (my $prefix = param_global()->get($args->{name} . '.PREFIX')) {
        $args->{prefix} = $prefix;
    }

    return $args;
}

1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::Fabric

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (dl-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
