#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::AppBuilder::Xgen::Resource::Portset;

use strict;
use warnings;

use parent qw( NACL::Service::Xgen::Resource );

use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NACL::Service::Xgen::Exceptions::InvalidAttribute;
use NATE::Log qw(log_global);

use constant nacl_cs_class => 'NACL::CS::LunPortset';

sub _validate_private_attrs_with_nacl_cs_class {
    my ($self, %args) = @_;
    log_global()->enter();

    # remove special args from the validation
    # or else the base class nacl cs class validation will complain
    delete($args{'igroups'});

    $self->SUPER::_validate_private_attrs_with_nacl_cs_class(%args);

    log_global()->exit();
}

sub _validate_private_attrs {
    my ($self, $args) = @_;
    log_global()->enter();

    eval {
        $args = validate_with(
            params => [%{$args}],
            spec => {
                'igroups' => {type => SCALAR|ARRAYREF, optional => 1}
            },
            allow_extra => 1);
    };
    if (my $err = $@) {
        NACL::Service::Xgen::Exceptions::InvalidAttribute->throw($err);
    }

    log_global()->exit();
    return $args;
}



1;

__END__

=head1 NAME

NACL::AppBuilder::Xgen::Resource::Portset

=head1 AUTHOR/MAINTAINER

=over

=item PTEFIT AUTOMATION (ng-ptefit-automation-core@netapp.com)

=back

   Last revised $Date: $ by $Author: $

=cut
