
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusOnAccess Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::AntivirusOnAccess

=head1 DESCRIPTION

C<NACL::C::AntivirusOnAccess> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP AntivirusOnAccess.  A related class is
L<NACL::CS::AntivirusOnAccess|lib-NACL-CS-AntivirusOnAccess-pm>, which represents
the past state of an ONTAP AntivirusOnAccess.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=cut

package NACL::C::AntivirusOnAccess;

use strict;
use warnings;

use NACL::CS::AntivirusOnAccess;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $AntivirusOnAccess = NACL::C::AntivirusOnAccess->new(command_interface=>$ci,
                                   volume =>$volume,
                                   vserver =>$vserver,
								  );

Return a new AntivirusOnAccess component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $AntivirusOnAccess  = NACL::C::AntivirusOnAccess->find(command_interface => $ci, ...);
	my @AntivirusOnAccesss = NACL::C::AntivirusOnAccess->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $AntivirusOnAccess_state = $NACL::C::AntivirusOnAccess->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::AntivirusOnAccess would return 
a NACL::CS::AntivirusOnAccess object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::AntivirusOnAccess->modify(
                                command_interface => $ci,
	                             volume => $volume,
	                             vserver => $vserver,
                                  ...);
	                             
								 or

    $AntivirusOnAccess_object->modify(...);

(Class or instance method)
This method is used to sets the on-access policies for a Vserver or volume in the cluster.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "volume" => $volume >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "volume-policy" => $string >>

Applicable for CMode CLI.

=item C<< "vserver-policy" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus on-access modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume-policy vserver-policy
                    )
            ),
        },
        api => 'antivirus_on_access_modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::AntivirusOnAccess::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $apiset = delete $opts{apiset};

    my %cmode_args = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_on_access_modify',
        options => \%opts,
    );
    $pkg_or_obj->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $opts{command_interface},
        frontend_method   => 'modify',
    );

    $apiset->antivirus_on_access_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'volume'  => {type => SCALAR},
        'vserver' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
