# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusOnAccessPolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::AntivirusOnAccessPolicy

=head1 DESCRIPTION

C<NACL::C::AntivirusOnAccessPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP AntivirusOnAccessPolicy.  A related class is
L<NACL::CS::AntivirusOnAccessPolicy|lib-NACL-CS-AntivirusOnAccessPolicy-pm>, which represents
the past state of an ONTAP AntivirusOnAccessPolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 name

=head2 vserver

=cut

package NACL::C::AntivirusOnAccessPolicy;

use strict;
use warnings;

use NACL::CS::AntivirusOnAccessPolicy;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists;
use NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $AntivirusOnAccessPolicy = NACL::C::AntivirusOnAccessPolicy->new(command_interface=>$ci,
                                   name =>$name,
                                  );

Return a new AntivirusOnAccessPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'name',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    my $ci = $opts{command_interface};
    if (   $ci->is_cmode()
        && $ci->has_uichange(uichange => 'antivirus-vserverization'))
    {
        $self->_vserver_context_init(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub init

=head2 find

    my $AntivirusOnAccessPolicy  = NACL::C::AntivirusOnAccessPolicy->find(command_interface => $ci, ...);
    my @AntivirusOnAccessPolicys = NACL::C::AntivirusOnAccessPolicy->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

This method invokes  av-on-access-policy-get-iter for CMode ZAPI.

=head2 state

    my $AntivirusOnAccessPolicy_state = $NACL::C::AntivirusOnAccessPolicy->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::AntivirusOnAccessPolicy would return
a NACL::CS::AntivirusOnAccessPolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

This method invokes  av-on-access-policy-get-iter for CMode ZAPI.

=cut

=head2 modify

   NACL::C::AntivirusOnAccessPolicy->modify(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

                                 or

    $AntivirusOnAccessPolicy_object->modify(...);

(Class or instance method)
This method is used to modifies an on-access policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "av-on-access-policy-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-name

=item C<< "cifs-file" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: include-cifs-file

=item C<< "cifs-share-list" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-share-list-pattern

=item C<< "scan-ro-volume" => $string >>

Valid values are on/off. The component method internally translates the values
to true/false for ZAPI.

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-row-volume-scan-enabled

=item C<< "scan" => $string >>

Valid values are on/off. The component method internally translates the values
to true/false for ZAPI.

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-scan-enabled


=item C<< "cifs-file-list" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-file-list-pattern

=item C<< "protocols" => [ $protocol1, $protocol2, ... ] >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: protocols

=item C<< "cifs-share" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: include-cifs-share

=item C<< "nfs-scan-execute-permission" => $string >>

Valid values are on/off. The component method internally translates the values to true/false for ZAPI.

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-nfs-execute-permission-scan-enabled

=item C<< "scan-mandatory" => $string >>

Valid values are on/off. The component method internally translates the values to true/false for ZAPI.

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-scan-mandatory

=item C<< "id" => $string >>

Applicable for for CMode CLI

=item C<< "fileop-profile" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: fileop-profile

=item C<< "cifs-scan-execute-access" => $string >>

Valid values are on/off. The component method internally translates the values to true/false for ZAPI.

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-cifs-execute-permission-scan-enabled

=item C<< "vserver" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus on-access policy modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            protocols => {type => SCALAR | ARRAYREF, optional => 1},
            _optional_scalars(
                qw( cifs-file cifs-share-list scan-ro-volume scan cifs-file-list
                    cifs-share nfs-scan-execute-permission scan-mandatory id fileop-profile
                    cifs-scan-execute-access
                    )
            ),
        },
        api => 'antivirus_on_access_policy_modify',
    );
    $pkg_or_obj->_scalar_to_array_deprication_warning(
        params  => ['protocols'],
        options => \%opts
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::AntivirusOnAccessPolicy::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my %cmode_args = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_on_access_policy_modify',
        options => \%opts,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');

    if ($boolean) {
        $pkg_or_obj->_handle_cli_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $cmode_args{vserver},
            command_interface => $command_interface,
            frontend_method   => 'modify',
        );
    } else {
        delete $cmode_args{vserver};
    }

    $apiset->antivirus_on_access_policy_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $allowed_cmode_options = [qw( protocols fileop-profile)];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {
            'cifs-file-list'  => 'cifs-file-list-pattern',
            'cifs-share-list' => 'cifs-share-list-pattern',
            'cifs-file'       => 'include-cifs-file',
            'cifs-share'      => 'include-cifs-share',
            'cifs-scan-execute-access' =>
                'is-cifs-execute-access-scan-enabled',
            'nfs-scan-execute-permission' =>
                'is-nfs-execute-permission-scan-enabled',
            'scan-ro-volume' => 'is-row-volume-scan-enabled',
            'scan'           => 'is-scan-enabled',
            'scan-mandatory' => 'is-scan-mandatory',
            'name'           => 'policy-name',
        },
        target => \%cmode_args,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');
    if ($boolean) {
        $pkg_or_obj->_handle_zapi_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $opts{vserver},
            command_interface => $command_interface,
        );
    }

    my $conversion_values = {'on' => 'true', 'off' => 'false'};
    my $options_values_to_translate = [
        qw ( is-cifs-execute-access-scan-enabled is-nfs-execute-permission-scan-enabled
            is-row-volume-scan-enabled is-scan-mandatory is-scan-enabled
            )
    ];

    foreach my $option (@$options_values_to_translate) {
        if (defined $cmode_args{"$option"}
            && $conversion_values->{$cmode_args{$option}})
        {
            $cmode_args{"$option"}
                = $conversion_values->{$cmode_args{$option}};
        }
    }

    $apiset->av_on_access_policy_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

   NACL::C::AntivirusOnAccessPolicy->delete(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

                                 or

    $AntivirusOnAccessPolicy_object->delete(...);

(Class or instance method)
This method is used to deletes an on-access policy.

Uses a CMode CLI/ZAPI APISet.

Invokes  "av-on-access-policy-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required for class method, Not Applicable for instance method)

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-name

=item C<< "vserver" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus on-access policy delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing on-access policy.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'antivirus_on_access_policy_delete',
    );
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::AntivirusOnAccessPolicy::delete'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my %cmode_args = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_on_access_policy_delete',
        options => \%opts,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');

    if ($boolean) {
        $pkg_or_obj->_handle_cli_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $cmode_args{vserver},
            command_interface => $command_interface,
            frontend_method   => 'delete',
        );
    } else {
        delete $cmode_args{vserver};
    }

    $apiset->antivirus_on_access_policy_delete(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'name' => 'policy-name'},
        target => \%cmode_args,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');

    if ($boolean) {
        $pkg_or_obj->_handle_zapi_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $opts{vserver},
            command_interface => $command_interface
        );
    }
    $apiset->av_on_access_policy_delete(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $AntivirusOnAccessPolicy_object    = NACL::C::AntivirusOnAccessPolicy->create(
                                command_interface => $ci,
                                 name => $name,
                                  ...);

(Class method)
This method is used to adds a new on-access policy.

Uses a CMode CLI/ZAPI APISet.

Invokes "av-on-access-policy-create" for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "name" => $name >>

(Required)

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-name

=item C<< "cifs-file" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: include-cifs-file

=item C<< "cifs-share-list" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-share-list-pattern

=item C<< "scan-ro-volume" => $string >>

valid values are "on/off"

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-row-volume-scan-enabled

=item C<< "scan" => $string >>

valid values are "on/off"

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-scan-enabled

=item C<< "cifs-file-list" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-file-list-pattern

=item C<< "protocols" => [ $protocol1, $protocol2, ... ] >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: protocols

=item C<< "cifs-share" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: include-cifs-share

=item C<< "nfs-scan-execute-permission" => $string >>

valid values are "on/off"

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-nfs-execute-permission-scan-enabled

=item C<< "scan-mandatory" => $string >>

valid valuesi are "on/off"

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-scan-mandatory

=item C<< "fileop-profile" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: fileop-profile

=item C<< "cifs-scan-execute-access" => $string >>

Applicable for for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-cifs-execute-permission-scan-enabled

=item C<< "vserver" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"antivirus on-access policy create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
antivirus on access policy that already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            protocols => {type => SCALAR | ARRAYREF, optional => 1},
            _optional_scalars(
                qw( cifs-file cifs-share-list scan-ro-volume scan cifs-file-list
                    cifs-share nfs-scan-execute-permission id scan-mandatory fileop-profile
                    cifs-scan-execute-access
                    )
            ),
        },
        api => 'antivirus_on_access_policy_create',
    );
    $pkg->_scalar_to_array_deprication_warning(
        params  => ['protocols'],
        options => \%opts
    );
    my $ret;
    try {
        $ret = $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_create_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_create_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::AntivirusOnAccessPolicy::create'
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my %cmode_args = $pkg->get_api_options(
        apiset  => $apiset,
        api     => 'antivirus_on_access_policy_create',
        options => \%opts,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');

    my %args_to_new = (
        command_interface => $command_interface,
        name              => $opts{name},
    );

    if ($boolean) {
        my $vserver = $pkg->_handle_cli_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $cmode_args{vserver},
            command_interface => $command_interface,
            frontend_method   => 'create',
        );
        $args_to_new{vserver} = $vserver;
    } else {
        delete $cmode_args{vserver};
        delete $args_to_new{vserver};
    }

    $apiset->antivirus_on_access_policy_create(%cmode_args);
    $Log->exit() if $may_exit;
    return $pkg->new(%args_to_new);
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts              = $pkg->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $allowed_cmode_options = [qw( protocols fileop-profile)];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        map    => {
            'cifs-file-list'  => 'cifs-file-list-pattern',
            'cifs-share-list' => 'cifs-share-list-pattern',
            'cifs-file'       => 'include-cifs-file',
            'cifs-share'      => 'include-cifs-share',
            'cifs-scan-execute-access' =>
                'is-cifs-execute-access-scan-enabled',
            'nfs-scan-execute-permission' =>
                'is-nfs-execute-permission-scan-enabled',
            'scan-ro-volume' => 'is-row-volume-scan-enabled',
            'scan'           => 'is-scan-enabled',
            'scan-mandatory' => 'is-scan-mandatory',
            'name'           => 'policy-name',
        },
        target => \%cmode_args,
    );

    my $boolean = $command_interface->has_uichange(
        uichange => 'antivirus-vserverization');

    my $vserver;

    if ($boolean) {
        $vserver = $pkg->_handle_zapi_vserver_context(
            api_opts          => \%cmode_args,
            vserver           => $opts{vserver},
            command_interface => $command_interface
        );
    }

    my %args_to_new = (
        command_interface => $command_interface,
        name              => $opts{name},
        vserver           => $vserver,
    );

    my $conversion_values = {'on' => 'true', 'off' => 'false'};
    my $options_values_to_translate = [
        qw ( is-cifs-execute-access-scan-enabled is-nfs-execute-permission-scan-enabled
            is-row-volume-scan-enabled is-scan-mandatory is-scan-enabled
            )
    ];
    foreach my $option (@$options_values_to_translate) {
        if (defined $cmode_args{"$option"}
            && $conversion_values->{$cmode_args{$option}})
        {
            $cmode_args{"$option"}
                = $conversion_values->{$cmode_args{$option}};
        }
    }

    my %default_args = (
        'cifs-file-list-pattern'                 => '^.*$',
        'cifs-share-list-pattern'                => '^.*$',
        'fileop-profile'                         => 'multi_proto_standard',
        'include-cifs-file'                      => 'include',
        'include-cifs-share'                     => 'include',
        'is-cifs-execute-access-scan-enabled'    => 'false',
        'is-nfs-execute-permission-scan-enabled' => 'false',
        'is-row-volume-scan-enabled'             => 'false',
        'is-scan-mandatory'                      => 'true',
        'is-scan-enabled'                        => 'false',
        'protocols'                              => [qw( nfs4 cifs )],
    );

    while (my ($key, $value) = each %default_args) {
        $cmode_args{$key} ||= $value;
    }
    $apiset->av_on_access_policy_create(%cmode_args);
    my $obj = $pkg->new(%args_to_new);
    $Log->exit() if $may_exit;
    return $obj;

} ## end sub _create_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        'name'    => {type => SCALAR},
        'vserver' => {type => SCALAR, optional => 1},
    );

    if (defined $command_interface) {
        my $boolean = $command_interface->has_uichange(
            uichange => 'antivirus-vserverization');
        if ($boolean) {
            return (
                'name'    => {type => SCALAR},
                'vserver' => {type => SCALAR},
            );

        } else {
            return ('name' => {type => SCALAR});
        }
    } else {
        return %default_spec;
    }
} ## end sub _primary_keys_validate_spec

1;
