#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary DeployNetwork Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::DeployNetwork

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for DeployNetwork operations, such as deploy network info,

=for html <hr>

=head SYNOPSIS

Return network configuration info of deployed VM:
NACL::C::Client::ONTAPSelectDeploy::DeployNetwork->info(
    command_interface => $ci,
);

=cut

package NACL::C::Client::ONTAPSelectDeploy::DeployNetwork;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           DeployNetwork Operation methods
###############################################################################

=head info

Return network configuration info of deployed VM

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Return network configuration info of deployed VM:
NACL::C::Client::ONTAPSelectDeploy::DeployNetwork->info(
    command_interface => $ci,
);

=back

=cut

sub info {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_info",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub info

sub _info {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'deploy_network_info' );
    $Log->exit() if $may_exit;
} ## end sub _info

1;
