#
#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary SystemConfig Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=pod

=head NAME

NACL::C::Client::ONTAPSelectDeploy::SystemConfig

=for html <hr>

=head DESCRIPTION

This library's purpose is to aid automation of workflows around the ONTAP Select Deploy Utility
by providing methods for SystemConfig operations, such as system config modify,

=for html <hr>

=head SYNOPSIS

Modify the configuration for SystemConfig:
NACL::C::Client::ONTAPSelectDeploy::SystemConfig->modify(
    command_interface => $ci,
    proxy_url => $proxy_url,
    product_company => $product_company,
    product_site => $product_site,
);

=cut

package NACL::C::Client::ONTAPSelectDeploy::SystemConfig;

use Exporter qw(import);
@EXPORT_OK = qw();

use strict;
use warnings;

use Options;
use Params::Validate qw(:all);
use NATE::BaseException;
use base 'NACL::C::Component::Client';
use NACL::ComponentUtils qw(_optional_scalars);

use Class::MethodMaker [
];

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
###############################################################################
#           SystemConfig Operation methods
###############################################################################

=head modify

Replace the configuration for SystemConfig.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)

=item C<< proxy_url => SCALAR >>

 (Optional) URL of a proxy server which provides Internet access.

=item C<< product_company => SCALAR >>

 (Optional) Descriptive string that represents the company or organization that runs this instance of ONTAP Select Deploy.

=item C<< product_site  => SCALAR >>

 (Optional) Description of the location of this ONTAP Select Deploy instance.

=back

=item Returns

None for success, throws an exception on failure.

=item Examples

# Modify the configuration for SystemConfig:
NACL::C::Client::ONTAPSelectDeploy::SystemConfig->modify(
    command_interface => $ci,
    proxy_url => $proxy_url,
    product_company => $product_company,
    product_site => $product_site,
);

=back

=cut

sub modify {
    my ($pkg, @args) = @_;
    $Log->enter() if $may_enter;
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
	additional_spec => {
	    _optional_scalars(
		qw( proxy_url product_company product_site )
	    ),
	    help		=> { type => BOOLEAN, optional => 1 },
	},
	allow_extra => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify",
                interface => "CLI",
                set       => "ONTAPSelectDeploy"
            },
        ],
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify {
    my ( $pkg, @args ) = @_;
    $Log->enter() if $may_enter;
    $pkg->_base_backend_ontapselectdeploy_cli( @args,
	api => 'system_config_modify' );
    $Log->exit() if $may_exit;
} ## end sub _modify

1;
