
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterIdentity Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ClusterIdentity

=head1 DESCRIPTION

C<NACL::C::ClusterIdentity> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP ClusterIdentity.  A related class is
L<NACL::CS::ClusterIdentity|lib-NACL-CS-ClusterIdentity-pm>, which represents
the past state of an ONTAP ClusterIdentity.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::ClusterIdentity;

use strict;
use warnings;

use NACL::CS::ClusterIdentity;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $ClusterIdentity = NACL::C::ClusterIdentity->new(command_interface=>$ci,
                                  );

Return a new ClusterIdentity component object with the given attributes.

=cut

=head2 modify

   NACL::C::ClusterIdentity->modify(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to modify the cluster's attributes.
Uses a CMode CLI or CMode ZAPI APISet.
Invokes 'cluster-identity-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "location" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : cluster-location

=item C<< "name" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : cluster-name

=item C<< "contact" => $string >>

Applicable for CMode CLI/ZAPI
Maps to:
    CM ZAPI : cluster-contact

=item C<< 'extended_query' => $string >>

(Optional)
Applicable only for CMode CLI


=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"cluster identity modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( location name contact
                    )
            ),
        },
        cli_cmd => 'cluster identity modify',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::ClusterIdentity::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'cluster_identity_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    foreach my $key (keys(%opts)) {
       if ($opts{$key} eq '""') {
             $opts{$key} = '';
        }
    }

    $pkg->_base_backend_cmode_zapi(%opts,
        api    => 'cluster_identity_modify',
        map    => {
            "contact"  => "cluster-contact",
            "location" => "cluster-location",
            "name"     => "cluster-name",
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}

1;
