# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterPeerAddressStable Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::ClusterPeerAddressStable

=head1 DESCRIPTION

C<NACL::C::ClusterPeerAddressStable> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP ClusterPeerAddressStable.  A related class is
L<NACL::CS::ClusterPeerAddressStable|lib-NACL-CS-ClusterPeerAddressStable-pm>, which represents
the past state of an ONTAP ClusterPeerAddressStable.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 cluster

=cut

package NACL::C::ClusterPeerAddressStable;

use strict;
use warnings;

use NACL::CS::ClusterPeerAddressStable;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $ClusterPeerAddressStable = NACL::C::ClusterPeerAddressStable->new(
                                   command_interface => $ci,
                                   'cluster' => $cluster,
								  );

Return a new ClusterPeerAddressStable component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'cluster',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $ClusterPeerAddressStable_obj  = NACL::C::ClusterPeerAddressStable->find(command_interface => $ci, ...);
	my @ClusterPeerAddressStable_objs = NACL::C::ClusterPeerAddressStable->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my ClusterPeerAddressStable_state = $ClusterPeerAddressStable_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::ClusterPeerAddressStable would return 
a NACL::CS::ClusterPeerAddressStable object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 register

    NACL::C::ClusterPeerAddressStable->register(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to registers cluster addresses.
Uses CMode CLI/ZAPI .
Invokes  cluster-peer-stable-addresses-register  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "cluster peer address stable register" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub register {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd             => 'cluster peer address stable register',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_register_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_register_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::ClusterPeerAddressStable::register',
    );

    $Log->exit() if $may_exit;
}

sub _register_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'cluster_peer_address_stable_register',);
    $Log->exit() if $may_exit;
}

sub _register_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api => 'cluster-peer-stable-addresses-register',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('cluster' => {type => SCALAR},);
}

1;
