# $Id //depot/prod/test/main/lib/NACL/C/DFM/Alert.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Alert Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Alert

=head1 DESCRIPTION

C<NACL::C::DFM::Alert> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to a DFM Alert.  A related class is
L<NACL::CS::DFM::Alert|lib-NACL-CS-DFM-Alert-pm>, which represents
the State Info of a DFM Alert.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 alert_name_or_id

=cut

package NACL::C::DFM::Alert;

use strict;
use warnings;

use NACL::CS::DFM::Alert;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

# Component exceptions
use NACL::C::Exceptions::DFM::Alert::DoesNotExist();

=head1 METHODS

=head2 new

  my $Alert = NACL::C::DFM::Alert->new( command_interface => $ci,
                                   'alert-name-or-id' => $alert_name_or_id,
                            );

Return a new Alert component object with the given attributes.

=cut

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
    scalar => 'alert_name_or_id',
    scalar => [ { -type => 'NACL::C::DFM::DFMClient' }, 'command_interface', ],
];

=head2 find

    my $Alert  = NACL::C::DFM::Alert->find(command_interface => $ci, ...);
    my @Alerts = NACL::C::DFM::Alert->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my Alert_state = $NACL::C::DFM::Alert->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Alert would return
a NACL::CS::DFM::Alert object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 test

    NACL::C::DFM::Alert->test(  command_interface => $ci,
                                'alert-name-or-id' => $alert_name_or_id,
                                  ...);
                                      or

    $Alert_object->test(...);

=for html (Class or instance method)
This method is used to  test an alert by performing it's trigger actions. The test will be performed
irrespective of whether the alert is enabled or disabled. <P> Error Conditions:
<ul> <li> EALERTDOESNOTEXIST - If an alert by the specified id does not exist.
</ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'alert-name-or-id' => $alert_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub test {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_test_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _test_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( alert-name-or-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->alert_test(%args);

    $Log->exit() if $may_exit;

}

=head2 create

    my $Alert_object	=
      NACL::C::DFM::Alert->create(command_interface => $ci,
                                'alert-info' => $alert_info,
                                  ...);
                                      or

=for html (Class) 
This method is used to  create a DFM alert. The alert-info element specifies all the parameters of the new
alert. <P> Note that it is possible to specify a combination of the event-name,
event-severity and event-class such that this alert will never be triggered. It
is the user's responsibility to verify these settings are useful. <P> Error
Conditions: <ul> <li> EINVALIDEMAILADDR - If the email address has spaces,
semi-colons or unprintable characters. <li> EINVALIDEVENTSEVERITY - If the
event-severity specified is not one of 'normal', 'information', 'unknown',
'warning', 'error', 'critical', 'emergency'. <li> EINVALIDEVENTCLASSEXP - If
the regular expression specified for the event-class is not a valid POSIX.1
regular expression. <li> EINVALIDALERTTIME - If the time-specified for
time-from or time-to is greater than 86399 ((24 * 60 * 60) - 1) seconds or if only one
of time-from and time-to values are specified. <li> EINVALIDTRAPADDR - If the
trap host specified is not reachable or when the port is greater than 65535 (2^15 -
1) <li> ENOSUCHEVENT - If the event name specified is not a valid event-name of DFM.
<li> EGROUPDOESNOTEXIST - If the group specified in the alert parameters is not a
DFM resource group. <li> ENOTFOUNDUSER - If the administrator name specified in
the alert parameters is not a DFM administrator. <li> ERUNASUNSUPPORTED - If
script-runas parameter is specified on a Windows DFM platform. <li>
EINVALIDINPUT - If the event-severity value is less than the severity of the
event-name event. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>
(Required)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

(Required)

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'alert-info' => $alert_info >>

(Required for class method, Not Applicable for instance method)

=item C<< 'alert-info' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'alert-info' => { type => HASHREF, optional => 1 }, },
        ignore_primary_keys => 1,
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_create_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _create_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( alert-info
          )
    ];
    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->alert_create(%args);
    my $parsed_output = $response->get_parsed_output();
    my $alert_id      = $parsed_output->[0]->{'alert-id'};
    $Log->exit() if $may_exit;

    return $pkg->new(
        command_interface  => $command_interface,
        'alert_name_or_id' => $alert_id,
    );

}

=head2 modify

    NACL::C::DFM::Alert->modify(  command_interface => $ci,
                                'alert-info' => $alert_info,
                                  ...);
                                      or

    $Alert_object->modify(...);

=for html (Class or instance method)
This method is used to  modify settings of a DFM alert. <P> Error Conditions: <ul> <li>
EALERTDOESNOTEXIST - If an alert by the specified id does not exist. <li>
EINVALIDEMAILADDR - If the email address has spaces, semi-colons or unprintable
characters. <li> EINVALIDEVENTSEVERITY - If the event-severity specified is
not one of 'normal', 'information', 'unknown', 'warning', 'error',
'critical', 'emergency'. <li> EINVALIDEVENTCLASSEXP - If the regular
expression specified for the event-class is not a valid POSIX.1 regular
expression. <li> EINVALIDALERTTIME - If the time-specified for time-from or
time-to is greater than 86399 ((24 * 60 * 60) - 1) seconds or if only one of time-from
and time-to values are specified. <li> EINVALIDTRAPADDR - If the trap host
specified is not reachable or when the port is greater than 65535 (2^15 - 1) <li>
ENOSUCHEVENT - If the event name specified is not a valid event-name of DFM. <li>
EGROUPDOESNOTEXIST - If the group specified in the alert parameters is not a DFM
resource group. <li> ENOTFOUNDUSER - If the administrator name specified in the
alert parameters is not a DFM administrator. <li> ERUNASUNSUPPORTED - If
script-runas parameter is specified on a Windows DFM platform. <li>
EINVALIDINPUT - If the event-severity value is less than the severity of the
event-name event. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'alert-info' => $alert_info >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'alert-info' => { type => HASHREF, optional => 1 },
	    _optional_scalars( qw( alert-name-or-id)) },
	ignore_primary_keys => 1,	
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_modify_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( alert-info
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->alert_modify(%args);

    ## Check if PK alert-name-or-id field has to be changed
    ## either alert-name or alert-id can be provided
    ## to uniquely identify the Alert. If alert-name has to be changed,
    ## alert-id has to be provided.
    ## Reset the alert-name-or-id beased on what input is given.
    if( defined $args{'alert-info'}{'alert-id'} ) {

	$pkg_or_obj->alert_name_or_id( $args{'alert-info'}{'alert-id'});

    } else {

	$pkg_or_obj->alert_name_or_id( $args{'alert-info'}{'alert-name'});

    }

    $Log->exit() if $may_exit;

}

=head2 destroy

    NACL::C::DFM::Alert->destroy(  command_interface => $ci,
                                'alert-name-or-id' => $alert_name_or_id,
                                  ...);
                                      or

    $Alert_object->destroy(...);

=for html (Class or instance method)
This method is used to  delete an alert. <P> Error Conditions: <ul> <li> EALERTDOESNOTEXIST - If an alert
by the specified id does not exist. </ul>

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'alert-name-or-id' => $alert_name_or_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item C<NACL::C::Exceptions::DFM::Alert::DoesNotExist>

This type of exception is thrown when an attempt is made to destroy an Alert which does not exist

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );
    my $ret;

    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {
                    method    => '_destroy_zapi',
                    interface => 'ZAPI',
                    set       => 'DFM',
                },
            ],
        );
    }

    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        if ( $pkg_or_obj->_alert_does_not_exist_error( text => $text ) ) {
            $Log->exit() if $may_exit;
            NACL::C::Exceptions::DFM::Alert::DoesNotExist->convert_and_throw(
                exception => $exception );
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    $Log->exit() if $may_exit;
    return $ret;
}

sub _destroy_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( alert-name-or-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->alert_destroy(%args);

    $Log->exit() if $may_exit;

}

=head2 get_defaults

    $alert_defaults = NACL::C::DFM::Alert->get_defaults(  command_interface => $ci,
                                  ...);

(Class method)
This method is used to  get the default values of attributes defined by this ZAPI set.
It returns the default values of attributes.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

=item C<< 'alert-defaults' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub get_defaults {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        ignore_primary_keys => 1,
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_get_defaults_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _get_defaults_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    # Making the API Call
    my $response      = $apiset->alert_get_defaults();
    my $parsed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $parsed_output->[0]->{'alert-defaults'};

}

# Return 1 if the exception text matches those messages which would indicate
# the alert does not exist
sub _alert_does_not_exist_error {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate( @_, { text => { type => SCALAR } } );
    my $text       = $opts{text};
    if ( $text =~ /Alert with name or id .* does not exist/ ) {
        $Log->exit() if $may_exit;
        return 1;
    } else {
        $Log->exit() if $may_exit;
        return 0;
    }
}

sub _create_to_pk_mapping {

    return { 'alert-name' => 'alert-name-or-id' };

}

sub _primary_keys_validate_spec {
    return ( 'alert-name-or-id' => { type => SCALAR }, );
}

1;
