# $Id$

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Job Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @change 2011-12-20 ramyar: Modified the Component for BARD
## @change 2012-09-26 deepthis: Changing wait_for_completion backend to call job-wait-for-state apiset
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Job

=head1 DESCRIPTION

C<NACL::C::DFM::Job> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an DFM Job.  A related class is
L<NACL::CS::DFM::Job|lib-NACL-CS-DFM-Job-pm>, which represents
the State Info of an DFM Job.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 job_id

=cut

package NACL::C::DFM::Job;

use strict;
use warnings;

use NACL::CS::DFM::Job;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Job = NACL::C::DFM::Job->new( command_interface => $ci,
                                   'job-id' => $job_id,
                            );

Return a new Job component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'job_id', ];

=head2 find

    my $Job  = NACL::C::DFM::Job->find(command_interface => $ci, ...);
    my @Jobs = NACL::C::DFM::Job->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $Job_state = $Job->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DFM::Job would return
a NACL::CS::DFM::Job object)

This method is generally not implemented in individual components. The base
class L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut


=head2 wait_for_state

    NACL::C::DFM::Job->wait_for_state(  command_interface => $ci,
					'job-list' => $job_list,
					'wait-for-state-timeout' => $timeout,
					...
				    );
                                      or

    $Job_object->wait_for_state(...);

=for html (Class or instance method)
Blocks until a specified timeout period elapses or until a job state is reached by at least one job.
It returns an array of Job components for ids which have reached the job state.

Throws a NACL::APISet::Response::Exception, if no job in the job-list completes within the given time.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< 'job-list' => $job_list >>
(Array)
(Required for class method, Not Applicable for instance method)

=item C<< 'state' => $job_state >>

(Optional) State of the job to be reached.

=item C<< 'wait-for-state-timeout' => $timeout >>

(Required) Number of seconds to wait for job(s) to reach the specified state.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional, Not required if 'wait-for-state-timeout' is provided)
The NATE timeout value. If no value is specified, it is set to a value
slightly greater than the 'timeout' value (if provided) to the wait-for-state
ZAPI. If no 'timeout' value is provided, then it is set to the NATE default value
of 150.

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub wait_for_state {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
	    _optional_scalars(qw( job-id state wait-for-state-timeout )),
	    'job-list' => { type => ARRAYREF, optional => 1 },
	},
	ignore_primary_keys => 1,
    );

    # Set the method-timeout value
    if ( defined $opts{'wait-for-state-timeout'} && !defined $opts{'method-timeout'} ) {

        $opts{'method-timeout'} = $opts{'wait-for-state-timeout'} + 5;
    }

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_wait_for_state_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _wait_for_state_zapi {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_options = [
        qw( job-id job-list state wait-for-state-timeout
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Change 'job-id' input to 'job-list' array input
    # if the method is called using a job component
    if ( ref $pkg_or_obj ) {

        my @job_list = ( $args{'job-id'} );
        delete $args{'job-id'};
        $args{'job-list'} = \@job_list;

    }

    # Making the API Call
    ## Remove when BURT683645 is fixed
    $Log->comment( 'Sleeping for 2 seconds' );
    sleep 2;
    my $response = $apiset->job_wait_for_state(%args);

    my $parsed_output = $response->get_parsed_output();

    my @id_list;
    if ( ref( $parsed_output->[0]->{'job-list'}->[0] ) =~ /HASH/ ) {

        push( @id_list, $parsed_output->[0]->{'job-list'}->[0]->{'job-id'} );
    } else {

        @id_list = @{ $parsed_output->[0]->{'job-list'} };
    }

    # Array of job components to be returned
    my @job_components = ();

    foreach my $id (@id_list) {

        my $obj = NACL::C::DFM::Job->new(
            command_interface => $command_interface,
            'job_id'          => $id,
        );
        push( @job_components, $obj );
    }

    $Log->exit() if $may_exit;

    return wantarray ? @job_components : $job_components[0];

}

=head2 delete

    NACL::C::DFM::Job->delete(  command_interface => $ci,
                                'job-id' => $job_id,
                                  ...);
                                      or

    $Job_object->delete(...);

=for html (Class or instance method)
This method is used to  delete the results of a completed job.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'job-id' => $job_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_delete_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _delete_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( job-id
          )
    ];
    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->job_delete(%args);

    $Log->exit() if $may_exit;

}

=head2 abort

    NACL::C::DFM::Job->abort(  command_interface => $ci,
                                'job-id' => $job_id,
                                  ...);
                                      or

    $Job_object->abort(...);

=for html (Class or instance method)
This method is used to  abort a running management job.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'job-id' => $job_id >>

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub abort {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@_, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_abort_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _abort_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %args;
    my $allowed_options = [
        qw( job-id )

    ];
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->job_abort(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return ( 'job-id' => { type => SCALAR }, );

}

1;
