# $Id: //depot/prod/test/nacldev/lib/NACL/C/DFM/Ldap.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Ldap Component Module (auto-generated by NCAGE)
## @author dl-mpo-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::DFM::Ldap

=head1 DESCRIPTION

C<NACL::C::DFM::Ldap> is a derived class of
L<NACL::C::Component::DFM|lib-NACL-C-Component-DFM-pm>. It represents access
to an Ldap Server.  A related class is
L<NACL::CS::DFM::Ldap|lib-NACL-CS-DFM-Ldap-pm>, which represents
the State Info of an Ldap server. 

=cut

package NACL::C::DFM::Ldap;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use NACL::CS::DFM::Ldap;

use base 'NACL::C::Component::DFM';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $Ldap =  NACL::C::DFM::Ldap->new( 
		command_interface => $ci, 
		'ldap_server' => $ldap_server,
	      ); 
Returns a new ldap component with the given attributes.

=cut 

use Class::MethodMaker [
    scalar => 'ldap_server',
];

=head2 user_check

    NACL::C::DFM::Ldap->user_check(  command_interface => $ci,
                                'user-name' => $user_name,
                                  ...);
                                      or

    $Ldap_object->user_check(...);

=for html (Class or instance method)
This method is used to  check existence and authentication of a user on configured LDAP servers. The API
fails if the user does not exist or if authentication fails.

Uses a DFM ZAPI APISet.

The output is a hashref in the form:

{ 
   'full-name' => '...',

   'groups'    =>  [ 
		     { 
			'ldap-group' => '...',
		     },
		     { 
			'ldap-group' => '...'
		     },
		   ],
   'user-name' => '...',
}

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'user-name' => $user_name >>

(Required)

=item C<< 'password' => $string >>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub user_check {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( user-name password
                  )
            ),
        },
        ignore_primary_keys => 1,
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_user_check_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
}

sub _user_check_zapi {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( password user-name
          )
    ];

    my %args;

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    my $response      = $apiset->ldap_user_check(%args);
    my $parsed_output = $response->get_parsed_output();
    my $user_info     = $parsed_output->[0]->{'ldap-user-info'}->[0];

    $Log->exit() if $may_exit;

    # Return the "ldap-user-info" output
    return $user_info;

}

=head2 server_add

    NACL::C::DFM::Ldap->server_add(  command_interface => $ci,
                                'ldap-server' => $ldap_server,
                                  ...);

=for html (Class method)
This method is used to  add an LDAP server.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ldap-server' => $hashref >>

(Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub server_add {

    $Log->enter() if $may_enter;

    my $pkg = shift;

    $pkg->_frontend_log_debug_opts(@_);

    my %opts = $pkg->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'ldap-server' => { type => HASHREF, optional => 1 }, },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_server_add_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
}

sub _server_add_zapi {

    $Log->enter() if $may_enter;

    my $pkg = shift;

    my %opts = $pkg->_validate_backend_opts(@_);

    my $cmd_interface = delete $opts{command_interface};
    my $apiset        = delete $opts{apiset};

    my $allowed_options = [
        qw( ldap-server
          )
    ];

    my %args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->ldap_server_add(%args);

    $Log->exit() if $may_exit;

    # Return a component object
    return $pkg->new(
        command_interface => $cmd_interface,
        ldap_server       => $args{'ldap-server'}
    );

}

=head2 find_or_add

    $Ldap = NACL::C::DFM::Ldap->find_or_add(
        command_interface       => $client,
        find_specific_options   => { 'ldap_server_addr' => $ldap_ip, },
        create_specific_options => {
	    'ldap-server'  =>  $ldap_server  
        },
        _was_created => \$ldap_added_flag,
    );

This convenience method finds a particular ldap entry, else adds a new
ldap entry. This is a wrapper around
L<find_or_create|lib-NACL-C-Component-pm/find_or_create> (see that page
for a detailed explanation of the options accepted)

The example code shown above can be used instead of having lengthier
code such as this:

    try {
        $Ldap = NACL::C::DFM::Ldap->find(
            'command_interface' => $client,
            filter              => { 'ldap_server_addr' => $ldap_ip, }  
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $Ldap = NACL::C::DFM::Ldap->add(
            command_interface  => $client,
	    'ldap-server'      => $ldap_server  
        );
        $ldap_added_flag = 1;
    };

=cut

sub find_or_add {

    $Log->enter() if $may_enter;

    my $pkg = shift;
    my @objs =
      $pkg->find_or_create( @_, custom_create_command => 'server_add' );

    $Log->exit() if $may_exit;

    return wantarray ? @objs : $objs[0];
}

=head2 server_delete

    NACL::C::DFM::Ldap->server_delete(  command_interface => $ci,
                                'ldap-server' => $ldap_server,
                                  ...);
                                      or

    $Ldap_object->server_delete(...);

=for html (Class or instance method)
This method is used to  delete an LDAP server.

Uses a DFM ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ldap-server' => $hashref >>

(Required for class method, Not Applicable for instance method. 
Hashref, can be a reference to a hash or hashofarrays or hashofhashes)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub server_delete {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    $pkg_or_obj->_frontend_log_debug_opts(@_);

    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@_,
        additional_spec =>
          { 'ldap-server' => { type => HASHREF, optional => 1 }, },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_server_delete_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
    );

    $Log->exit() if $may_exit;

    return $ret;
}

sub _server_delete_zapi {

    $Log->enter() if $may_enter;

    my $pkg_or_obj = shift;

    my %opts = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_options = [
        qw( ldap-server
          )
    ];

    my %args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_options,
        target => \%args,
    );

    # Making the API Call
    $apiset->ldap_server_delete(%args);

    $Log->exit() if $may_exit;

}

sub _primary_keys_validate_spec {
    return ( 'ldap-server' => { type => HASHREF | UNDEF, optional => 1 } );
}

1;
;
