# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugCrsAgent Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DebugCrsAgent

=head1 DESCRIPTION

C<NACL::C::DebugCrsAgent> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. 
Unlike other components this package does not model any element/service 
but is present to provide component-level support for functionality. 
Since it does not model any element/service there is no ComponentState 
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::DebugCrsAgent->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::DebugCrsAgent;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $DebugCrsAgent = NACL::C::DebugCrsAgent->new(
                                   command_interface => $ci,
								  );

Return a new DebugCrsAgent component object with the given attributes.

=cut

use Class::MethodMaker [];

=head2 discover_peer_crs

    NACL::C::DebugCrsAgent->discover_peer_crs(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to discover peer CRS agent.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'protocol-version' => $string >>

Supported Protocol Version by CRS agent
Applicable for CMode CLI

=item C<< 'remote-cluster' => $string >>

Remote Cluster UUID
Applicable for CMode CLI

=item C<< 'crs-node' => $string >>

Node in the Cluster Running CRS Process
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs agent discover peer crs" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub discover_peer_crs {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( protocol-version remote-cluster crs-node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent discover-peer-crs',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_discover_peer_crs_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::discover_peer_crs',
    );

}

sub _discover_peer_crs_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_agent_discover_peer_crs',);
    $Log->exit() if $may_exit;
}

=head2 set_protocol_version

    NACL::C::DebugCrsAgent->set_protocol_version(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to set protocol version of CRS agent.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $string >>

Stream UUID
Applicable for CMode CLI

=item C<< 'protocol-version' => $string >>

Protocol Version
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs agent set protocol version" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_protocol_version {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( stream-uuid protocol-version
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent set-protocol-version',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_protocol_version_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::set_protocol_version',
    );

}

sub _set_protocol_version_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_agent_set_protocol_version',);
    $Log->exit() if $may_exit;
}

=head2 stream_in_error_notification

    NACL::C::DebugCrsAgent->stream_in_error_notification(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to initiate a stream-in-error notification.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $string >>

Stream UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs agent stream in error notification" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stream_in_error_notification {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( stream-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent stream-in-error-notification',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stream_in_error_notification_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::stream_in_error_notification',
    );

}

sub _stream_in_error_notification_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_agent_stream_in_error_notification',);
    $Log->exit() if $may_exit;
}

=head2 switchover

    NACL::C::DebugCrsAgent->switchover(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to perform CRS switchover.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $string >>

Stream UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs agent switchover" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub switchover {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( stream-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent switchover',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_switchover_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::switchover',
    );

}

sub _switchover_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'debug_crs_agent_switchover',
    );
    $Log->exit() if $may_exit;
}

=head2 trigger_heartbeat

    NACL::C::DebugCrsAgent->trigger_heartbeat(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to trigger a heartbeat between CRS agents.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "debug crs agent trigger heartbeat" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub trigger_heartbeat {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent trigger-heartbeat',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_trigger_heartbeat_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::trigger_heartbeat',
    );

}

sub _trigger_heartbeat_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_agent_trigger_heartbeat',);
    $Log->exit() if $may_exit;
}

=head2 drain_sender_queue

    NACL::C::DebugCrsAgent->drain_sender_queue(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to drain sender side queue during MCC switch.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $string >>

Stream UUID
Applicable for CMode CLI

=item C<< 'sender-stream-path' => $string >>

Sender Stream Path
Applicable for CMode CLI

=item C<< 'last-drained-creation-time' => $string >>

Last Drained Creation Time
Applicable for CMode CLI

=item C<< 'is-fatal' => $string >>

Is Fatal possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'sender-metainfo-path' => $string >>

Sender Meta Info Path.
Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by "debug crs agent drain sender queue" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub drain_sender_queue {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( stream-uuid sender-stream-path
                    last-drained-creation-time is-fatal sender-metainfo-path
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs agent drain-sender-queue',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_drain_sender_queue_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsAgent::drain_sender_queue',
    );

}

sub _drain_sender_queue_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_agent_drain_sender_queue',);
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
