# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugCrsQueue Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DebugCrsQueue

=head1 DESCRIPTION

C<NACL::C::DebugCrsQueue> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP DebugCrsQueue.  A related class is
L<NACL::CS::DebugCrsQueue|lib-NACL-CS-DebugCrsQueue-pm>, which represents
the past state of an ONTAP DebugCrsQueue.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 stream_uuid

=head2 message_seq_num_in_queue

=cut

package NACL::C::DebugCrsQueue;

use strict;
use warnings;

use NACL::CS::DebugCrsQueue;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $DebugCrsQueue = NACL::C::DebugCrsQueue->new(
                                   command_interface => $ci,
                                   'stream-uuid' => $stream_uuid,
                                   'message-seq-num-in-queue' => $message_seq_num_in_queue,
								  );

Return a new DebugCrsQueue component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'stream_uuid',
    scalar => 'message_seq_num_in_queue',
];

=head2 find

    my $DebugCrsQueue_obj  = NACL::C::DebugCrsQueue->find(command_interface => $ci, ...);
	my @DebugCrsQueue_objs = NACL::C::DebugCrsQueue->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my DebugCrsQueue_state = $DebugCrsQueue_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DebugCrsQueue would return 
a NACL::CS::DebugCrsQueue object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 delete_range

    NACL::C::DebugCrsQueue->delete_range(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
   
(Class method)
This method is used to delete a range of messages from queue.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required)
Stream UUID
Applicable for CMode CLI

=item C<< 'first-message-seq-num' => $string >>

First Message Sequence Number in Range
Applicable for CMode CLI

=item C<< 'last-message-seq-num' => $string >>

Last Message Sequence Number in Range
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue delete range" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_range {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( stream-uuid first-message-seq-num last-message-seq-num
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_crs_queue_delete_range',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_range_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::delete_range',
    );

}

sub _delete_range_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_delete_range',);
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::DebugCrsQueue->delete(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
	                            'message-seq-num-in-queue' => $message_seq_num_in_queue,
                                ...);
	                             
								 or

    $DebugCrsQueue_obj->delete(...);

(Class or instance method)
This method is used to delete a message from the queue.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< 'message-seq-num-in-queue' => $message_seq_num_in_queue >>

(Required for class method, Not Applicable for instance method)
Sequence Number of Message in Queue
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'debug_crs_queue_delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::delete',
    );

}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_delete',);
    $Log->exit() if $may_exit;
}

=head2 generate_and_process_baseline

    NACL::C::DebugCrsQueue->generate_and_process_baseline(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to generate and transmit a baseline to destination.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< 'objects' => [ $object1, $object2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'streams' => [ $stream1, $stream2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue generate and process baseline" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub generate_and_process_baseline {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'objects' => {type => ARRAYREF, optional => 1},
            'streams' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        api => 'debug_crs_queue_generate_and_process_baseline',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_generate_and_process_baseline_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::generate_and_process_baseline',
    );

}

sub _generate_and_process_baseline_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_generate_and_process_baseline',);
    $Log->exit() if $may_exit;
}

=head2 generate_baseline

    NACL::C::DebugCrsQueue->generate_baseline(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to generate a baseline for a domain instance.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< 'streams' => [ $stream1, $stream2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue generate baseline" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub generate_baseline {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'streams' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_crs_queue_generate_baseline',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_generate_baseline_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::generate_baseline',
    );

}

sub _generate_baseline_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_generate_baseline',);
    $Log->exit() if $may_exit;
}

=head2 generate_marker

    NACL::C::DebugCrsQueue->generate_marker(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to queue a marker message for a domain instance.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< 'streams' => [ $stream1, $stream2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue generate marker" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub generate_marker {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'streams' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_crs_queue_generate_marker',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_generate_marker_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::generate_marker',
    );

}

sub _generate_marker_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_generate_marker',);
    $Log->exit() if $may_exit;
}

=head2 is_baseline_in_progress

    NACL::C::DebugCrsQueue->is_baseline_in_progress(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to display if a baseline is in progress for a domain instance.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue is baseline in progress" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub is_baseline_in_progress {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'debug_crs_queue_is_baseline_in_progress',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_is_baseline_in_progress_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::is_baseline_in_progress',
    );

}

sub _is_baseline_in_progress_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_is_baseline_in_progress',);
    $Log->exit() if $may_exit;
}

=head2 show_baseline_per_stream

    my $show_baseline_per_stream_obj = NACL::C::DebugCrsQueue->show_baseline_per_stream(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_baseline_per_stream_objs = NACL::C::DebugCrsQueue->show_baseline_per_stream(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_baseline_per_stream_obj = $DebugCrsQueue_obj->show_baseline_per_stream(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-DebugCrsQueueShowBaselinePerStream-pm/ATTRIBUTES>
of NACL::CS::DebugCrsQueueShowBaselinePerStream as input. 
This method returns objects of the type L<NACL::CS::DebugCrsQueueShowBaselinePerStream|lib-NACL-CS-DebugCrsQueueShowBaselinePerStream-pm>.

=cut

sub show_baseline_per_stream {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::DebugCrsQueueShowBaselinePerStream;
    my @state_objs
        = NACL::CS::DebugCrsQueueShowBaselinePerStream->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_marker

    NACL::C::DebugCrsQueue->show_marker(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
	                            'message-seq-num-in-queue' => $message_seq_num_in_queue,
                                ...);
	                             
								 or

    $DebugCrsQueue_obj->show_marker(...);

(Class or instance method)
This method is used to show marker message in the queue.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< 'message-seq-num-in-queue' => $message_seq_num_in_queue >>

(Required for class method, Not Applicable for instance method)
Sequence Number of Message in Queue
Applicable for CMode CLI

=item C<< 'deleted' => $string >>

Looking for a Deleted Message possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue show marker" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_marker {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug crs queue show-marker',
        additional_spec => {
            _optional_scalars(
                qw( deleted
                    )
            )
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_marker_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::show_marker',
    );

}

sub _show_marker_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_show_marker',);
    $Log->exit() if $may_exit;
}

=head2 show_update

    NACL::C::DebugCrsQueue->show_update(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
	                            'message-seq-num-in-queue' => $message_seq_num_in_queue,
                                ...);
	                             
								 or

    $DebugCrsQueue_obj->show_update(...);

(Class or instance method)
This method is used to show update message in the queue.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< 'message-seq-num-in-queue' => $message_seq_num_in_queue >>

(Required for class method, Not Applicable for instance method)
Sequence Number of Message in Queue
Applicable for CMode CLI

=item C<< 'deleted' => $string >>

Looking for a Deleted Message possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs queue show update" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub show_update {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug crs queue show-update',
        additional_spec => {
            _optional_scalars(
                qw( deleted
                    )
            )
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_show_update_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsQueue::show_update',
    );

}

sub _show_update_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_queue_show_update',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'stream-uuid'              => {type => SCALAR},
        'message-seq-num-in-queue' => {type => SCALAR},
    );
}

1;
