# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugCrsStream Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DebugCrsStream

=head1 DESCRIPTION

C<NACL::C::DebugCrsStream> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP DebugCrsStream.  A related class is
L<NACL::CS::DebugCrsStream|lib-NACL-CS-DebugCrsStream-pm>, which represents
the past state of an ONTAP DebugCrsStream.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 stream_uuid

=cut

package NACL::C::DebugCrsStream;

use strict;
use warnings;

use NACL::CS::DebugCrsStream;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $DebugCrsStream = NACL::C::DebugCrsStream->new(
                                   command_interface => $ci,
                                   'stream-uuid' => $stream_uuid,
								  );

Return a new DebugCrsStream component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'stream_uuid',];

=head2 find

    my $DebugCrsStream_obj  = NACL::C::DebugCrsStream->find(command_interface => $ci, ...);
	my @DebugCrsStream_objs = NACL::C::DebugCrsStream->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my DebugCrsStream_state = $DebugCrsStream_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DebugCrsStream would return 
a NACL::CS::DebugCrsStream object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create_stream

    my $DebugCrsStream_obj	= NACL::C::DebugCrsStream->create_stream(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
   
(Class method)
This method is used to create a CRS stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required)
Stream UUID
Applicable for CMode CLI

=item C<< 'director-id' => $string >>

Director ID
Applicable for CMode CLI

=item C<< 'queue-storage-capacity' => $string >>

Queue Storage Capacity
Applicable for CMode CLI

=item C<< 'path-to-persistent-storage' => $string >>

Path to Persistent Storage
Applicable for CMode CLI

=item C<< 'remote-cluster-uuid' => $string >>

Remote Cluster UUID
Applicable for CMode CLI

=item C<< 'admin-state' => $string >>

Admin State possible value(s) are, up,down,paused
Applicable for CMode CLI

=item C<< 'notification-options' => $string >>

Notification Options
Applicable for CMode CLI

=item C<< 'direction' => $string >>

Direction possible value(s) are, sender,receiver
Applicable for CMode CLI

=item C<< 'domain' => $string >>

Replication Domain possible value(s) are, cluster,vserver
Applicable for CMode CLI

=item C<< 'synopsis' => $string >>

Synopsis
Applicable for CMode CLI

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< 'league-uuid' => $string >>

League UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream create stream" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create_stream {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( director-id queue-storage-capacity path-to-persistent-storage
                    remote-cluster-uuid admin-state notification-options direction domain
                    synopsis domain-instance-uuid

                    league-uuid
                    )
            ),
        },
        cli_cmd => 'debug crs stream create-stream',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_stream_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::create_stream',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        stream_uuid       => $opts{'stream-uuid'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_stream_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_create_stream',);
    $Log->exit() if $may_exit;
}

=head2 delete_stream

    NACL::C::DebugCrsStream->delete_stream(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->delete_stream(...);

(Class or instance method)
This method is used to delete a CRS stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream delete stream" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_stream {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        cli_cmd => 'debug crs stream delete-stream',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_stream_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::delete_stream',
    );

}

sub _delete_stream_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_delete_stream',);
    $Log->exit() if $may_exit;
}

=head2 modify_stream

    NACL::C::DebugCrsStream->modify_stream(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->modify_stream(...);

(Class or instance method)
This method is used to modify a CRS stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< 'queue-storage-capacity' => $string >>

Queue Storage Capacity
Applicable for CMode CLI

=item C<< 'synopsis' => $string >>

Synopsis
Applicable for CMode CLI

=item C<< 'admin-state' => $string >>

Admin State possible value(s) are, up,down,paused
Applicable for CMode CLI

=item C<< 'notification-options' => $string >>

Notification Options
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream modify stream" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify_stream {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( queue-storage-capacity synopsis admin-state notification-options
                    )
            ),
        },
        cli_cmd => 'debug crs stream modify-stream',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_stream_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::modify_stream',
    );

}

sub _modify_stream_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_modify_stream',);
    $Log->exit() if $may_exit;
}

=head2 reset_streams

    NACL::C::DebugCrsStream->reset_streams(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to reset streams.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain-instance-uuid' => $string >>

Domain Instance UUID
Applicable for CMode CLI

=item C<< 'streams' => [ $stream1, $stream2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream reset streams" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset_streams {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'streams' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( domain-instance-uuid
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs stream reset-streams',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reset_streams_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::reset_streams',
    );

}

sub _reset_streams_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_reset_streams',);
    $Log->exit() if $may_exit;
}

=head2 delete_all_streams

    NACL::C::DebugCrsStream->delete_all_streams(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to delete all CRS streams.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain' => $string >>

Replication Domain possible value(s) are, cluster,test,vserver
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream delete all streams" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_all_streams {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs stream delete-all-streams',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_all_streams_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::delete_all_streams',
    );

}

sub _delete_all_streams_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_delete_all_streams',);
    $Log->exit() if $may_exit;
}

=head2 force_delete_all_streams

    NACL::C::DebugCrsStream->force_delete_all_streams(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to delete all CRS streams even if master duties not acquired.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'domain' => $string >>

Replication Domain possible value(s) are, cluster,test,vserver
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream force delete all streams" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub force_delete_all_streams {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs stream force-delete-all-streams',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_force_delete_all_streams_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::force_delete_all_streams',
    );

}

sub _force_delete_all_streams_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_force_delete_all_streams',);
    $Log->exit() if $may_exit;
}

=head2 force_delete_stream

    NACL::C::DebugCrsStream->force_delete_stream(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->force_delete_stream(...);

(Class or instance method)
This method is used to delete a CRS stream even if master duties not acquired.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream force delete stream" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub force_delete_stream {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug crs stream force-delete-stream',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_force_delete_stream_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::force_delete_stream',
    );

}

sub _force_delete_stream_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_force_delete_stream',);
    $Log->exit() if $may_exit;
}

=head2 get_domain_of_stream

    NACL::C::DebugCrsStream->get_domain_of_stream(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->get_domain_of_stream(...);

(Class or instance method)
This method is used to get the domain of a stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream UUID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream get domain of stream" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_domain_of_stream {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug crs stream get-domain-of-stream',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_get_domain_of_stream_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::get_domain_of_stream',
    );

}

sub _get_domain_of_stream_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_get_domain_of_stream',);
    $Log->exit() if $may_exit;
}

=head2 notify_dm

    NACL::C::DebugCrsStream->notify_dm(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->notify_dm(...);

(Class or instance method)
This method is used to notify the DM about a stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream
Applicable for CMode CLI

=item C<< 'notification-type' => $string >>


Notification type possible value(s) are, data-available,over-threshold,reciprocal-not-configured,in-error
Applicable for CMode CLI

=item C<< 'is-redelivery' => $string >>

Is A Redelivery possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream notify dm" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub notify_dm {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( notification-type

                    is-redelivery
                    )
            ),
        },
        cli_cmd  => 'debug crs stream notify-dm',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_notify_dm_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::notify_dm',
    );

}

sub _notify_dm_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_notify_dm',);
    $Log->exit() if $may_exit;
}

=head2 notify_peer_cluster

    NACL::C::DebugCrsStream->notify_peer_cluster(
                                command_interface => $ci,
	                            'stream-uuid' => $stream_uuid,
                                ...);
	                             
								 or

    $DebugCrsStream_obj->notify_peer_cluster(...);

(Class or instance method)
This method is used to notify the peer cluster about a stream.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'stream-uuid' => $stream_uuid >>

(Required for class method, Not Applicable for instance method)
Stream
Applicable for CMode CLI

=item C<< 'is-redelivery' => $string >>

Is A Redelivery possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream notify peer cluster" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub notify_peer_cluster {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug crs stream notify-peer-cluster',
        ,
        additional_spec => {
            _optional_scalars(
                qw( is-redelivery
                    )
            )
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_notify_peer_cluster_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::notify_peer_cluster',
    );

}

sub _notify_peer_cluster_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_notify_peer_cluster',);
    $Log->exit() if $may_exit;
}

=head2 set_streams_in_error

    NACL::C::DebugCrsStream->set_streams_in_error(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to set streams in error.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'increment-err-count' => $string >>

Increment error count possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'err-cause' => $string >>

Reason for going into error possible value(s) are, peer,overflow
Applicable for CMode CLI

=item C<< 'streams' => [ $stream1, $stream2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug crs stream set streams in error" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_streams_in_error {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'streams' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( increment-err-count err-cause
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'debug crs stream set-streams-in-error',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_streams_in_error_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugCrsStream::set_streams_in_error',
    );

}

sub _set_streams_in_error_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_crs_stream_set_streams_in_error',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('stream-uuid' => {type => SCALAR},);
}

1;
