#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugFaultInjection Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DebugFaultInjection

=head1 DESCRIPTION

C<NACL::C::DebugFaultInjection> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP DebugFaultInjection.  A related class is
L<NACL::CS::DebugFaultInjection|lib-NACL-CS-DebugFaultInjection-pm>, which represents
the past state of an ONTAP DebugFaultInjection.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 fault_id

=head2 node

=cut

package NACL::C::DebugFaultInjection;

use strict;
use warnings;

use NACL::CS::DebugFaultInjection;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $DebugFaultInjection = NACL::C::DebugFaultInjection->new(
                                   command_interface => $ci,
                                   'fault-id' => $fault_id,
                                   'node' => $node,
								  );

Return a new DebugFaultInjection component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'fault_id',
    scalar => 'node',
];

=head2 find

    my $DebugFaultInjection  = NACL::C::DebugFaultInjection->find(command_interface => $ci, ...);
	my @DebugFaultInjections = NACL::C::DebugFaultInjection->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my DebugFaultInjection_state = $NACL::C::DebugFaultInjection->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DebugFaultInjection would return 
a NACL::CS::DebugFaultInjection object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 set_fault_mode

    NACL::C::DebugFaultInjection->set_fault_mode(
                                command_interface => $ci,
	                            'fault-id' => $fault_id,
	                            'node' => $node,
                                ...);
	                             
								 or

    $DebugFaultInjection_object->set_fault_mode(...);

(Class or instance method)
This method is used to set or modify the fault mode.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fault-id' => $fault_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'num-calls' => $string >>

Applicable for CMode CLI.

=item C<< 'period' => $string >>

Applicable for CMode CLI.

=item C<< 'probability' => $string >>

Applicable for CMode CLI.

=item C<< 'end-call' => $string >>

Applicable for CMode CLI.

=item C<< 'fault-mode' => $string >>

Applicable for CMode CLI.

=item C<< 'start-call' => $string >>

Applicable for CMode CLI.

=item C<< 'action-arg' => $string >>

Applicable for CMode CLI.

=item C<< 'action' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"debug fault-injection set-fault-mode" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_fault_mode {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( num-calls period probability end-call fault-mode start-call action-arg action
                    )
            ),
        },
        cli_cmd => 'debug fault-injection set-fault-mode',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_fault_mode_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugFaultInjection::set_fault_mode',
    );

    $Log->exit() if $may_exit;
} ## end sub set_fault_mode

sub _set_fault_mode_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;
    $pkg->_base_backend_cmode_cli(
            %opts,
            api => 'debug_fault_injection_set_fault_mode',
        );

    $Log->exit() if $may_exit;
} ## end sub _set_fault_mode_cmode_cli

=head2 disable_fault

    NACL::C::DebugFaultInjection->disable_fault(
                                command_interface => $ci,
	                            'fault-id' => $fault_id,
	                            'node' => $node,
                                ...);
	                             
								 or

    $DebugFaultInjection_object->disable_fault(...);

(Class or instance method)
This method is used to disable fault.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fault-id' => $fault_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"debug fault-injection disable-fault" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable_fault {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'debug fault-injection disable-fault',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_fault_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugFaultInjection::disable_fault',
    );

    $Log->exit() if $may_exit;
} ## end sub disable_fault

sub _disable_fault_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(
            %opts,
            api => 'debug_fault_injection_disable_fault',
        );

    $Log->exit() if $may_exit;
} ## end sub _disable_fault_cmode_cli

=head2 enable_fault

    NACL::C::DebugFaultInjection->enable_fault(
                                command_interface => $ci,
	                            'fault-id' => $fault_id,
	                            'node' => $node,
                                ...);
	                             
								 or

    $DebugFaultInjection_object->enable_fault(...);

(Class or instance method)
This method is used to enable fault.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'fault-id' => $fault_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"debug fault-injection enable-fault" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable_fault {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug fault-injection enable-fault',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_fault_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugFaultInjection::enable_fault',
    );

    $Log->exit() if $may_exit;
} ## end sub enable_fault

sub _enable_fault_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(
            %opts,
            api => 'debug_fault_injection_enable_fault',
        );

    $Log->exit() if $may_exit;
} ## end sub _enable_fault_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'fault-id' => {type => SCALAR},
        'node'     => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
