# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugNetworkSchedule Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DebugNetworkSchedule

=head1 DESCRIPTION

C<NACL::C::DebugNetworkSchedule> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP DebugNetworkSchedule.  A related class is
L<NACL::CS::DebugNetworkSchedule|lib-NACL-CS-DebugNetworkSchedule-pm>, which represents
the past state of an ONTAP DebugNetworkSchedule.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 schedule

=head2 node

=cut

package NACL::C::DebugNetworkSchedule;

use strict;
use warnings;

use NACL::CS::DebugNetworkSchedule;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $DebugNetworkSchedule = NACL::C::DebugNetworkSchedule->new(
                                   command_interface => $ci,
                                   'schedule' => $schedule,
                                   'node' => $node,
								  );

Return a new DebugNetworkSchedule component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'schedule',
    scalar => 'node',
];

=head2 find

    my $DebugNetworkSchedule_obj  = NACL::C::DebugNetworkSchedule->find(command_interface => $ci, ...);
	my @DebugNetworkSchedule_objs = NACL::C::DebugNetworkSchedule->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my DebugNetworkSchedule_state = $DebugNetworkSchedule_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::DebugNetworkSchedule would return 
a NACL::CS::DebugNetworkSchedule object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create

    my $DebugNetworkSchedule_obj	= NACL::C::DebugNetworkSchedule->create(
                                command_interface => $ci,
	                            'schedule' => $schedule,
	                            'node' => $node,
                                ...);
   
(Class method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'schedule' => $schedule >>

(Required)
Schedule
Applicable for CMode CLI

=item C<< 'node' => $node >>

(Required)
Node
Applicable for CMode CLI

=item C<< 'enabled' => $string >>

Enabled?
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug network schedule create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enabled
                    )
            ),
        },
        cli_cmd => 'debug network schedule create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugNetworkSchedule::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        schedule          => $opts{'schedule'},
        node              => $opts{'node'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'debug_network_schedule_create',);
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::DebugNetworkSchedule->delete(
                                command_interface => $ci,
	                            'schedule' => $schedule,
	                            'node' => $node,
                                ...);
	                             
								 or

    $DebugNetworkSchedule_obj->delete(...);

(Class or instance method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'schedule' => $schedule >>

(Required for class method, Not Applicable for instance method)
Schedule
Applicable for CMode CLI

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug network schedule delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'debug network schedule delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugNetworkSchedule::delete',
    );

}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_network_schedule_delete',);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::DebugNetworkSchedule->modify(
                                command_interface => $ci,
	                            'schedule' => $schedule,
	                            'node' => $node,
                                ...);
	                             
								 or

    $DebugNetworkSchedule_obj->modify(...);

(Class or instance method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'schedule' => $schedule >>

(Required for class method, Not Applicable for instance method)
Schedule
Applicable for CMode CLI

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node
Applicable for CMode CLI

=item C<< 'enabled' => $string >>

Enabled?
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "debug network schedule modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enabled
                    )
            ),
        },
        cli_cmd => 'debug network schedule modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DebugNetworkSchedule::modify',
    );

}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'debug_network_schedule_modify',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'schedule' => {type => SCALAR},
        'node'     => {type => SCALAR},
    );
}

1;
