
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DiagNbladeCifs Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DiagNbladeCifs

=head1 DESCRIPTION

C<NACL::C::DiagNbladeCifs> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::DiagNbladeCifs;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head2 shares

   NACL::C::DiagNbladeCifs->shares(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to list the nblade's CIFS share info.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

*Node

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"diag nblade cifs shares" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub shares {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver

                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'diag_nblade_cifs_shares',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_shares_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagNbladeCifs::shares',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub shares

sub _shares_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'diag_nblade_cifs_shares',);
    $Log->exit() if $may_exit;
} ## end sub _shares_cmode_cli

=head2 interfaces

   NACL::C::DiagNbladeCifs->interfaces(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to list the nblade's CIFS interface info.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

*Node

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"diag nblade cifs interfaces" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub interfaces {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver

                    node
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'diag_nblade_cifs_interfaces',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_interfaces_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::DiagNbladeCifs::interfaces',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub interfaces

sub _interfaces_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'diag_nblade_cifs_interfaces',);
    $Log->exit() if $may_exit;
} ## end sub _interfaces_cmode_cli

=head2 set_acl

    NACL::C::DiagNbladeCifs->set_acl(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to set the ACL of a specified file.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver
Applicable for CMode CLI

=item C<< 'sddl' => $string >>

SDDL ACL string
Applicable for CMode CLI

=item C<< 'path' => $string >>

Path to the file
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "diag nblade cifs set acl" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set_acl {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver sddl path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'diag_nblade_cifs_set_acl',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_acl_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::DiagNbladeCifs::set_acl',
    );

    $Log->exit() if $may_exit;
}

sub _set_acl_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'diag_nblade_cifs_set_acl',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
