# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DiagSecdTrace Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::DiagSecdTrace

=head1 DESCRIPTION

C<NACL::C::DiagSecdTrace> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP DiagSecdTrace.  A related class is
L<NACL::CS::DiagSecdTrace|lib-NACL-CS-DiagSecdTrace-pm>, which represents
the past state of an ONTAP DiagSecdTrace.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

package NACL::C::DiagSecdTrace;

use strict;
use warnings;

use NACL::CS::DiagSecdTrace;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $DiagSecdTrace = NACL::C::DiagSecdTrace->new(command_interface=>$ci,
								  );

Return a new DiagSecdTrace component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 clear

   NACL::C::DiagSecdTrace->clear(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to clear Trace Options.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "dc-names" => $string >>

Applicable for CMode CLI.

=item C<< "module-names" => $string >>

Applicable for CMode CLI.

=item C<< "ipaddrs" => $string >>

Applicable for CMode CLI.

=item C<< "domain-names" => $string >>

Applicable for CMode CLI.

=item C<< "rpc-names" => $string >>

Applicable for CMode CLI.

=item C<< "vserverids" => $string >>

Applicable for CMode CLI.

=item C<< "trace-all" => $string >>

Applicable for CMode CLI.

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'trace-connectivity' => $string >>

Clear Trace Connection Errors possible value(s) are, yes,no
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub clear {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( dc-names module-names ipaddrs domain-names rpc-names vserverids trace-all
                    node

                    trace-connectivity
                    )
            ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_clear_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub clear

sub _clear_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( dc-names module-names ipaddrs domain-names rpc-names vserverids trace-all node
            extended_query
            trace-connectivity
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'diag_secd_trace_clear' are: \n" . Dumper(\%opts));

    $apiset->diag_secd_trace_clear(%opts);
    $Log->exit() if $may_exit;
} ## end sub _clear_cmode_cli

=head2 set

   NACL::C::DiagSecdTrace->set(
                                command_interface => $ci,
                                             node => $node,
                                  ...);

(Class method)
This method is used to set Trace Options.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "trace-all" => $string >>

Applicable for CMode CLI.

=item C<< "dc-names" => [ $dc-name1, $dc-name2, ... ] >>

Applicable for CMode CLI.

=item C<< "module-names" => [ $module-name1, $module-name2, ... ] >>

Applicable for CMode CLI.

=item C<< "ipaddrs" => [ $ipaddr1, $ipaddr2, ... ] >>

Applicable for CMode CLI.

=item C<< "domain-names" => [ $domain-name1, $domain-name2, ... ] >>

Applicable for CMode CLI.

=item C<< "rpc-names" => [ $rpc-name1, $rpc-name2, ... ] >>

Applicable for CMode CLI.

=item C<< "vserverids" => [ $vserverid1, $vserverid2, ... ] >>

Applicable for CMode CLI.

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< 'trace-connectivity' => $string >>

Trace Connection Errors possible value(s) are, yes,no
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub set {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "dc-names"     => {type => ARRAYREF, optional => 1},
            "module-names" => {type => ARRAYREF, optional => 1},
            "ipaddrs"      => {type => ARRAYREF, optional => 1},
            "domain-names" => {type => ARRAYREF, optional => 1},
            "rpc-names"    => {type => ARRAYREF, optional => 1},
            "vserverids"   => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( trace-all

                    node

                    trace-connectivity
                    )
            ),
        },
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_set_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub set

sub _set_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( dc-names module-names ipaddrs domain-names rpc-names vserverids trace-all node
            extended_query
            trace-connectivity
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    foreach my $array_element (
        qw(dc-names module-names ipaddrs domain-names rpc-names vserverids))
    {
        if (defined $opts{$array_element}) {
            $opts{$array_element} = join ',', @{$opts{$array_element}};
        }
    } ## end foreach my $array_element ...

    $Log->debug(
        "Opts to API call 'diag_secd_trace_set' are: \n" . Dumper(\%opts));

    $apiset->diag_secd_trace_set(%opts);
    $Log->exit() if $may_exit;
} ## end sub _set_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR},);
}

1;
