# $Id: //depot/prod/test/main/lib/NACL/C/Dump.pm#1 $

#
# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary dump Component Module for 7Mode and NodeScope.
## @author dl-dpg-nb-automation; dl-nacl-dev
## @status shared
## @pod here

=head1 NAME

NACL::C::Dump

=head1 DESCRIPTION

C<NACL::C::Dump> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
It represents restore operations.

=cut

package NACL::C::Dump;

use strict;
use warnings;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::C::Options;

sub dump {
    my $pkg  = shift;
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw (dump-level tape-name tree ignore-acls dump-file-block-size ignore-qtree-files-directories
                  exclude-files tape-blocking-factor backup-specific-files multi-subtree-dump method-timeout
                  multi-subtree-dump-name update-dumpdates verbose restart-failed-dump
                  dumpname blocksize exclude-filelist blocking-factor-size)
            ),
        },
    );

    return $pkg->call_on_apiset(
        %opts,
        choices => [

            {
                method    => '_dump_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            }
        ],
    );
} ## end sub dump

sub _dump_7mode_cli {

    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    # enabling options at d blade for dump
    # Refer 505668 for more informaiton.
    try {
        NACL::C::Options->option(
                command_interface => $opts{command_interface},
                'option-name'     => 'nodescope.reenabledcmds',
                'option-value'    => 'dump',
            );
    } catch NACL::APISet::Exceptions::InvalidParamValueException with {};

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    $pkg->_hash_move(
        source => \%opts,
        target => \%opts,
        map => { 'method-timeout' => 'connectrec-timeout' }
    );

    $Log->debug( "Opts to API call ' dump ' are: \n" . Dumper( \%opts ) );
    $apiset->dump(%opts);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
} ## end sub _primary_keys_validate_spec

=head1 METHODS

=head2 Dump

   NACL::C::Dump->dump(
         command_interface       => $node,
        'dump-level'            => 0,
        'backup-specific-files' => 1,
        'verbose'               => 1,
        'tape-name'             => 'nrst0a',
        'tree'                  => '/vol/vol0',
        );

(Class method)
This method is used to dump tree on tape drive.
Uses a 7Mode and Nodescope CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

=item C<< 'dump-level' =>  >>

(Required) dump-level

=item C<< 'backup-specific-files' =>  >>

(Required) backup-specific-files

=item C<< 'verbose' =>  >>

(Required) verbose

=item C<< 'tape-name' =>  $MagneticTapeName >>

(Required) tape-name

=item C<< 'tree' =>  /vol/$VolumeName >>

(Required) tree

=back

=back

=cut

1;
