# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary EventNotification Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::EventNotification

=head1 DESCRIPTION

C<NACL::C::EventNotification> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP EventNotification.  A related class is
L<NACL::CS::EventNotification|lib-NACL-CS-EventNotification-pm>, which represents
the past state of an ONTAP EventNotification.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 ID

=cut

package NACL::C::EventNotification;

use strict;
use warnings;

use NACL::CS::EventNotification;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $EventNotification = NACL::C::EventNotification->new(
                                   command_interface => $ci,
                                   'ID' => $ID,
								  );

Return a new EventNotification component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'ID',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $EventNotification_obj  = NACL::C::EventNotification->find(command_interface => $ci, ...);
	my @EventNotification_objs = NACL::C::EventNotification->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my EventNotification_state = $EventNotification_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::EventNotification would return 
a NACL::CS::EventNotification object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $EventNotification_obj	= NACL::C::EventNotification->create(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to create an event notification.
Uses CMode CLI/ZAPI .
Invokes  ems-event-notification-create  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'filter-name' => $string >>

Filter Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'destinations' => [ $destination1, $destination2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "event notification create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'destinations' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( filter-name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'event notification create',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_create_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        nacl_exceptions => [qw(AlreadyExists)],
        frontend        => 'NACL::C::EventNotification::create',
    );

    ## In case if an Object is required
    ## return the last EventNotification Component Object
    ## using find method
    if(defined wantarray){
        my $obj;
        my %common_options;
        $pkg->_copy_common_component_params_with_ci(
            source => \%opts,
            target => \%common_options
        );
	my @event_obj = $pkg->find(
           %common_options,
           filter  => {
               'filter-name' => $opts{'filter-name'},
               'destinations' => $opts{'destinations'}
           }
	);
        $obj = pop @event_obj ;
        $Log->exit() if $may_exit;
        return $obj ;
    }
    $Log->exit() if $may_exit;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'event_notification_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(filter-name destinations)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'ems-event-notification-create',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(ID)],);
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::EventNotification->delete(
                                command_interface => $ci,
	                            'ID' => $ID,
                                ...);
	                             
								 or

    $EventNotification_obj->delete(...);

(Class or instance method)
This method is used to delete event notifications.
Uses CMode CLI/ZAPI .
Invokes  ems-event-notification-destroy-iter for CMode Zapi iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'ID' => $ID >>

(Required for class method, Not Applicable for instance method)
Event Notification ID
Applicable for CMode CLI/ZAPI iter

Mapping for iter-zapi: <event-notification id="$value" />

=item C<< '*' >>

This method takes all other options supported by "event notification delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'event notification delete',
    );
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        nacl_exceptions => [qw(DoesNotExist)],
        frontend        => 'NACL::C::EventNotification::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'event_notification_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    require NACL::C::Exceptions::EventNotification::DoesNotExist;
    my $copy = [qw(filter-name)];
    my $map  = {
        'ID' => 'id',
        'destinations' =>
            [make_zapi_array('destinations'), make_zapi_skip('string'),],
    };
    $pkg_or_obj->_iter_helper(
        @opts,
        api      => 'ems-event-notification-destroy-iter',
        show_cmd => 'event notification show',
        copy     => $copy,
        map      => $map,
        does_not_exist_exception =>
            'NACL::C::Exceptions::EventNotification::DoesNotExist',
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::EventNotification->modify(
                                command_interface => $ci,
	                            'ID' => $ID,
                                ...);
	                             
								 or

    $EventNotification_obj->modify(...);

(Class or instance method)
This method is used to modify event notifications.
Uses CMode CLI/ZAPI .
Invokes  ems-event-notification-modify-iter for CMode Zapi iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'ID' => $ID >>

(Required for class method, Not Applicable for instance method)
Event Notification ID
Applicable for CMode CLI/ZAPI iter

Mapping for iter-zapi: <event-notification id="$value" />

=item C<< 'filter-name' => $string >>

Event Filter Name
Applicable for CMode CLI/ZAPI iter

Mapping for iter-zapi: <event-notification filter-name="$value" />

=item C<< 'destinations' => [ $destination1, $destination2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI iter

Mapping for iter-zapi: <event-notification>
  <destinations string="@values" />
</event-notification>

=item C<< '*' >>

This method takes all other options supported by "event notification modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'destinations' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( filter-name
                    )
            ),
        },
        cli_cmd => 'event notification modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        frontend => 'NACL::C::EventNotification::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'event_notification_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    require NACL::C::Exceptions::EventNotification::DoesNotExist;
    my $copy = [qw(filter-name)];
    my $map  = {
        'ID' => 'id',
        'destinations' =>
            [make_zapi_array('destinations'), make_zapi_skip('string'),],
    };
    $pkg_or_obj->_iter_helper(
        @opts,
        api      => 'ems-event-notification-modify-iter',
        show_cmd => 'event notification show',
        copy     => $copy,
        map      => $map,
        does_not_exist_exception =>
            'NACL::C::Exceptions::EventNotification::DoesNotExist',
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('ID' => {type => SCALAR},);
}

1;
