# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary A base class for the AlreadyExists exceptions
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AlreadyExists

=head1 DESCRIPTION

This is an abstract base class for the exceptions that get thrown
when we attempt to create an element that already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::AlreadyExists qw(:try);

   # $comp_pkg could be any component package name
   try {
       $comp_pkg->create(%opts);
   } catch NACL::C::Exceptions::AlreadyExists with {
        # Element already exists
   };

=cut


package NACL::C::Exceptions::AlreadyExists;

# These are the exceptions typically thrown by the APISet
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

# Handle the typical error message
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/duplicate\s+entry/i, qr/already exists/i]
    );
    $Log->exit();
    return $detected;
}

1;
