# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/AntivirusOnAccessPolicy/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for AntivirusOnAccessPolicy
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a on-access policy
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists qw(:try);

   try {
       NACL::C::AntivirusOnAccessPolicy->create(
           command_interface  => $Node,
           vserver            => $Vserver,
	   name               => $policy,
           %other_opts);
   } catch NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::AntivirusOnAccessPolicy::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate entry/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
