# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/AntivirusOnAccessPolicy/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exist for AntivirusOnAccessPolicy
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a on-access policy
when entry doesn't exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist qw(:try);

   try {
       NACL::C::AntivirusOnAccessPolicy->delete(
           command_interface  => $Node,
           vserver            => $TestVserver,
	   name             => $Policy
           %other_opts);
   } catch NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::AntivirusOnAccessPolicy::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry\s+doesn't\s+exist/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
