#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an antivirus on-demand command operation is 
##          performed on a command that does not exist.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnDemandCommand::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when an antivirus on-demand command operation is 
performed on a command that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnDemandCommand::DoesNotExist qw(:try);

   try {
       $non_existing_avod_obj->delete();
   } catch NACL::C::Exceptions::AntivirusOnDemandCommand::DoesNotExist with {
        # AntivirusOnDemandCommand does not exist any more
   };

=cut

package NACL::C::Exceptions::AntivirusOnDemandCommand::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args,
                regexes => [qr/.*entry\s+doesn't\s+exist*/,
                        qr/There\s+are\s+no\s+entries.*/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
