#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a AntivirusOnDemandCommandScanFile of the same name already exists
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::AntivirusOnDemandCommandScanFile::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create an antivirus on-demand
file scan command which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::AntivirusOnDemandCommandScanFile::AlreadyExists qw(:try);

   try {
       NACL::C::AntivirusOnDemandCommandScanFile->create(name => $already_existing_name, %other_opts);
   } catch NACL::C::Exceptions::AntivirusOnDemandCommandScanFile::AlreadyExists with {
        # AntivirusOnDemandCommandScanFile already exists.
   };

=cut

package NACL::C::Exceptions::AntivirusOnDemandCommandScanFile::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate entry/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
