#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Client::Directory::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create an already
existing directory.

=head1 EXAMPLE

   use NACL::C::Exceptions::Client::Directory::AlreadyExists qw(:try);

   try {
       my $dir_obj = NACL::C::Client::Directory->create(dir => $Dir_name);
   } catch NACL::C::Exceptions::Client::Directory::AlreadyExists {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::Client::Directory::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NATE::BaseException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes =>
            [qr/Not able to create directory\s+\S+\s+File exists/i]);
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
