#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Client::Ifconfig::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to add a IP address
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Client::Ifconfig::AlreadyExists qw(:try);

   try {
       NACL::C::Client::Ifconfig->create(
           command_interface  =>  $Client,
           interface          =>  $Interface,
           address            =>  $ipv6_address,,
           prefixlen          => $Prefixlen,
           %other_opts);
   } catch NACL::C::Exceptions::Client::Ifconfig::AlreadyExists {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::Client::Ifconfig::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes =>
            [qr/File\s+exists/i]);
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
