#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate transition session already present
##
## @author rohit4@netapp.com, dl-nacl-dev@netapp.com
## @status Review
## @pod here

=head1 NAME

NACL::C::Exceptions::Client::Transition::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a transition session
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Client::Transition::AlreadyExists qw(:try);

   try {
       NACL::C::Client::Transition->create(
            "session-name"              => $session_name,
            command_interface           => $Client,
            "session-type"              => "standalone",
            "7-mode-host-name"          => "10.227.202.110",
            "data-copy-ipaddress"       => "10.227.202.110",
            vfiler                      => "vfiler0",
            "c-mode-host-name"          => "10.226.99.61",
            "c-mode-vserver-name"       => "vs_sec",
            %other_opts
        );
   } catch NACL::C::Exceptions::Client::Transition::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::Client::Transition::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
	    regexes => [ qr/session with name.*already exists/i ]
    );
    $Log->exit();
    return $detected;
}
1;
