#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown while performing cluster identity find,
##          for the node which is not in a cluster.
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::ClusterIdentity::PreClusterMode

=head1 DESCRIPTION

This exception is thrown while performing cluster identity find on a node,
where node is not in cluster.

=head1 EXAMPLE

   use NACL::C::Exceptions::ClusterIdentity::PreClusterMode qw(:try);

   try {
        NACL::C::ClusterIdentity->fetch(
        command_interface  => $Node,
        vserver            => $Vserver,
        %other_opts);
   } catch NACL::C::Exceptions::ClusterIdentity::PreClusterMode with {
      # Node is not in a cluster.
  };

=cut

package NACL::C::Exceptions::ClusterIdentity::PreClusterMode;

use base qw(
  NACL::C::Exceptions::Component
  NACL::APISet::Exceptions::ResponseException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/ZAPI is not enabled in pre-cluster mode/i , qr/Error: \"identity\" is not a recognized command/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

