#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to delete an
## cluster image package when syncronisation is in progress
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::ClusterImagePackage::SyncInProgress

=head1 DESCRIPTION

This exception is thrown when we attempt to elete an
cluster image package when syncronisation is in progress

=head1 EXAMPLE

   use NACL::C::Exceptions::ClusterImagePackage::SyncInProgress qw(:try);

   try {
       NACL::C::ClusterImagePackage->delete(
							  'version'=> $version);
   } catch NACL::C::Exceptions::ClusterImagePackage::SyncInProgress with {
        # Syncronisation is in progress
   };

=cut

package NACL::C::Exceptions::ClusterImagePackage::SyncInProgress;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::ResponseException
    NACL::C::Exceptions::Component
	NATE::BaseException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/A\s+package\s+synchronization\s+is\s+in\s+progress/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
