#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a cluster peer relationship already exists
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::ClusterPeer::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when a attempt is made to create a cluster peer relationship
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::ClusterPeer::AlreadyExists qw(:try);

   try {
       NACL::C::ClusterPeer->create("peer-addrs" => \@addrs, %other_opts);
   } catch NACL::C::Exceptions::ClusterPeer::AlreadyExists with {
        # ClusterPeer already exists.
   };

=head1 METHODS

=head2 detect

This method contains the regular expression match for which
C<NACL::C::Exceptions::ClusterPeer::AlreadyExists> exception
gets thrown.

=head2 get_peer_cluster_name

This method return the cluster name if an attempt is made
to peer with the same cluster. The cluster-name is found
in the error message "Cannot peer with two clusters of the name "cluster-name"".
This is pattern matched and returned.

=cut

package NACL::C::Exceptions::ClusterPeer::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

my $regex = qr/Cannot peer with two clusters of the name "(.*)"/i;

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my @regexes = ($regex);
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => \@regexes, 
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub get_peer_cluster_name {
    $Log->enter() if $may_enter;
    my $self = $_[0];
    my $text = $self->text();
    $text    =~ $regex;
    my $peer_cluster_name = $1;
    $Log->exit() if $may_exit;
    return $peer_cluster_name;
}

1;
