#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to do login
##  in Security services diagnostics with wrong credentials
##
## @author adarshm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::DiagSecd::AuthenticationRejected

=head1 DESCRIPTION

This exception is thrown when we attempt is made to do login
in Security services diagnostics with wrong credentials

=head1 EXAMPLE

   use NACL::C::Exceptions::DiagSecd::AuthenticationRejected qw(:try);

   try{
           my $obj = NACL::C::DiagSecdAuthentication->login_cifs(
               command_interface => $Node,
               node              => $Node_name,
               vserver           => $TestVserver,
               user              => $Uname,
               passwd            => $Ad_Server_Password,
          );
    }catch NACL::C::Exceptions::DiagSecd::AuthenticationRejected with {
          my $exception_object = shift;
          $Test->nlog( "Expected exception caught: " . $exception_object->text() );
    };
=cut

package NACL::C::Exceptions::DiagSecd::AuthenticationRejected;
use NACL::APISet::Exceptions::ResponseException ;
use feature 'state';
use Params::Validate qw(validate_with HASHREF);
use base qw(
    NACL::APISet::Exceptions::ResponseException
    NACL::C::Exceptions::Component
);
use NACL::ComponentUtils qw(Dumper);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Authentication rejected/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
