# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/EventRoute/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for EventRoute
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::EventRoute::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an event route
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::EventRoute::DoesNotExist qw(:try);

   try {
       NACL::C::EventRoute->delete(
           command_interface  => $Node,
           messagename        => $name,
           %other_opts);
   } catch NACL::C::Exceptions::EventRoute::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::EventRoute::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i, qr/is an invalid value for field/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
