# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Job/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a job operation is performed on a
##          id that does not exist.
##
## @author Akshay Mohit(makshay@netapp.com),dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Job::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to do a job operation
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Job::DoesNotExist qw(:try);

   try {
       $non_existing_job_obj->modify(id => $id);
   } catch NACL::C::Exceptions::Job::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::Job::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
  NACL::APISet::Exceptions::CommandFailedException
  NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/.*entry\s+doesn't\s+exist*/, qr/There\s+are\s+no\s+entries.*/] );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
