# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/JobScheduleCron/AlreadyExists.pm#1 $
#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for JobScheduleCron
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::JobScheduleCron::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a cron job which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::JobScheduleCron::AlreadyExists qw(:try);

   try {
       $sched = NACL::C::JobScheduleCron->create(
                command_interface => $Filer_Dst,
                'name' => $sched_name,
                'minute' => [0,5,10,15,20,25,30,35,40,45,50,55],
                 %other_opts);
   } catch NACL::C::Exceptions::JobScheduleCron::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::JobScheduleCron::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
