#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete a non-existing lun
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Lun::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a lun
when entry does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::Lun::DoesNotExist qw(:try);

   try {
       $non_existing_lunObj->delete();
   } catch NACL::C::Exceptions::Lun::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::Lun::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        # First regex is for CMode, second regex is for 7Mode, third is for CMode ZAPI
        regexes => [ qr/no entries/i, qr/No such LUN exists/i, qr/entry doesn't exist/i, qr/No such file or directory/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
