# 
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a Lun of very small size is created
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Lun::TooSmall

=head1 DESCRIPTION

This exception is thrown when we attempt to create a lun with small size.

=head1 EXAMPLE

   use NACL::C::Exceptions::Lun::TooSmall qw(:try);

   try {
       NACL::C::Lun->create(path => $already_existing_lunpath, %other_opts);
   } catch NACL::C::Exceptions::Lun::TooSmall with {
        # LUN size is too small
   };

=cut

package NACL::C::Exceptions::Lun::TooSmall;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        # Regex is applicable for both CLI and ZAPI
        regexes => [ qr/Requested LUN size is too small*/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
