#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete a non-existing lun
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunBind::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a lun bind
when entry does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunBind::DoesNotExist qw(:try);

   try {
       $non_existing_lunBindObj->destroy();
   } catch NACL::C::Exceptions::LunBind::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::LunBind::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/entry doesn't exist/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
