# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/LunIgroup/AlreadyBound.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to delete an igroup which is mapped
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunIgroup::AlreadyBound

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an igroup
when it is mapped.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunIgroup::AlreadyBound qw(:try);

   try {
       NACL::C::LunIgroup->bind(
           command_interface => $Node,
           volume            => $Volume,
           igroup            => $LunIgroup,
           %other_opts);
   } catch NACL::C::Exceptions::LunIgroup::AlreadyBound with {
        # map exists for this initiator group
   };

=cut

package NACL::C::Exceptions::LunIgroup::AlreadyBound;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Port\sset\sis\salready\sbound\sto\san\sigroup/i,
                     qr/The\sinitiator\sgroup\sis\salready\sbound\sto\sa\sport\sset/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
