# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/LunIgroup/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for LunIgroup
##
## @author kathar@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunIgroup::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an igroup
when entry does not exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunIgroup::DoesNotExist qw(:try);

   try {
       NACL::C::LunIgroup->delete(
           command_interface => $Node,
           volume            => $Volume,
           igroup            => $LunIgroup,
           %other_opts);
   } catch NACL::C::Exceptions::LunIgroup::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::LunIgroup::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/,
		qr/No\s+such\s+initiator\s+group\s+exists/,
                qr/does not contain initiator/,
                qr/Initiator group does not contain node/,
		]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
