# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/LunIgroup/MapExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to delete an igroup which is mapped
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunIgroup::MapExists

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an igroup
when it is mapped.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunIgroup::MapExists qw(:try);

   try {
       NACL::C::LunIgroup->delete(
           command_interface => $Node,
           volume            => $Volume,
           igroup            => $LunIgroup,
           %other_opts);
   } catch NACL::C::Exceptions::LunIgroup::MapExists with {
        # map exists for this initiator group
   };

=cut

package NACL::C::Exceptions::LunIgroup::MapExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/maps\s+for\s+this\s+initiator\s+group\s+exist/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
