# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Lun/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to unmap the lun which is
## not mapped.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::LunMapping::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to unmap the lun which is
not mapped.

=head1 EXAMPLE

   use NACL::C::Exceptions::LunMapping::DoesNotExist qw(:try);

   try {
       NACL::C::Lun->unmap(igroup => $unmapped_igroup, %other_opts);
   } catch NACL::C::Exceptions::LunMapping::DoesNotExist with {
        # Lun is not mapped.
   };

=cut

package NACL::C::Exceptions::LunMapping::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/not mapped/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
