# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkInterface/MaxInterfacesPerNodeReached.pm# $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a logical interface
##          for a node that has reached its maximum supported number of interfaces.
##
## @author jais@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached

=head1 DESCRIPTION

This exception is thrown when an attempt is made to create a logical interface
for a node that has reached its maximum supported number of interfaces.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached qw(:try);

   try {
       my $lif_obj = NACL::C::NetworkInterface->create(%opts);
   } catch NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached with {
        # Node has reached its maximum supported number of interfaces
   };

=cut

package NACL::C::Exceptions::NetworkInterface::MaxInterfacesPerNodeReached;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $regexes = [ qr/node has reached its maximum supported/, qr/cannot host any more LIFs/, qr/Unable to add the LIF to the stack because the stack currently contains the maximum number of LIFs/i ];
    my $detected
        = $pkg->_detect_helper( @args, regexes => $regexes );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
