# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkInterface/NoAddressInSubnet.pm#1 $
#
# Copyright (c) 2017 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a no address available in subnet for NetworkInterface
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet

=head1 DESCRIPTION

This exception is thrown when we attempt to create a network interface
in a subnet where no address is available.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet qw(:try);

   try {
       NACL::C::NetworkInterface->create(
           command_interface  => $Node,
	   node              => $node,
           lif               => $interface_name,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet with {
        # perform a filtered fetch, if lif exist then follow the nacltask_if_exists
	# settings otherwise fail
   };

=cut

package NACL::C::Exceptions::NetworkInterface::NoAddressInSubnet;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Subnet .* in IPspace .* does not have any addresses available/i, ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
