#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary This exception is thrown when we attempt to delete a network interface failover-group
## which is in use by the lifs.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkInterfaceFailoverGroups::FailoverGroupIsInUseByLif

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a network interface failover-group
which is in use by the lifs.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkInterfaceFailoverGroups::FailoverGroupIsInUseByLif qw(:try);

   try {
       NACL::C::NetworkInterfaceFailoverGroups->delete(
           command_interface  => $Node,
           'failover-group' => $Failover_group,
           node => $Node_name,
           port => $Port,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkInterfaceFailoverGroups::FailoverGroupIsInUseByLif with {
        # Cannot remove a failover group that is in use by a lif
   };

=cut

package NACL::C::Exceptions::NetworkInterfaceFailoverGroups::FailoverGroupIsInUseByLif;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Cannot remove a failover group that is in use by a lif/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
