#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a Ipspace of the same name already exists
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkIpspace::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a Ipspace which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkIpspace::AlreadyExists qw(:try);

   try {
       NACL::C::NetworkIpspace->create(Ipspace => $bd_name, %other_opts);
   } catch NACL::C::Exceptions::NetworkIpspace::AlreadyExists with {
        # Ipspace already exists.
   };

=cut

package NACL::C::Exceptions::NetworkIpspace::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Ipspace.*already\s+exists/i,
                     qr/duplicate\s+entry/i,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
