#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a broadcast domain exists when trying to delete a ipspace
##
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkIpspace::BroadcastDomainExists

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a ipspace which 
has a broadcast domain in it.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkIpspace::BroadcastDomainExists qw(:try);

   try {
       NACL::C::NetworkIpspace->delete(ipspace => $ipspace, %other_opts);
   } catch NACL::C::Exceptions::NetworkIpspace::BroadcastDomainExists with {
        # Broadcast domain exists on this Ipspace.
   };

=cut

package NACL::C::Exceptions::NetworkIpspace::BroadcastDomainExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/delete\s+an\s+IPspace.*broadcast\s+domain/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
