#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when trying to delete a ipspace which 
## has ports(with lifs association) assigned to it
##
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkIpspace::NetworkPortsAssigned

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a ipspace which has 
ports assigned to it.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkIpspace::NetworkPortsAssigned qw(:try);

   try {
       NACL::C::NetworkIpspace->delete(ipspace => $ipspace, %other_opts);
   } catch NACL::C::Exceptions::NetworkIpspace::NetworkPortsAssigned with {
        # Network ports are assigned to this Vserver.
   };

=cut

package NACL::C::Exceptions::NetworkIpspace::NetworkPortsAssigned;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/IPspace can't be removed because it has network ports assigned/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
