# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPort/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a NetworkPort operation is performed on a
##          NetworkPort that does not exist.
##
## @author raghaveb@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPort::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a NetworkPort operation is performed on a
NetworkPort that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPort::DoesNotExist qw(:try);

   try {
       $non_existing_network_port_obj->modify( mtu => '100');
   } catch NACL::C::Exceptions::NetworkPort::DoesNotExist with {
        # NetworkPort does not exist any more
   };

=cut

package NACL::C::Exceptions::NetworkPort::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = 10;
    $detected =
      $pkg->_detect_helper( @args, 
			# first regex is for Cmode and second is for 7mode and third is for Cmode ZAPI
			   regexes => [ qr/.*entry\s+doesn't\s+exist*/,
						qr/no\s+such\s+interface/i ,
                        qr/.*is\s+an\s+invalid\s+port\s+on\s+node.*/]);
    $Log->exit() if $may_exit;
    return $detected;
}
1;
