#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a broadcast-domain operation is performed on
##          a broadcast-domain that does not exist.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a broadcast-domain operation is performed on a
broadcast-domain that does not exist in that IPSpace.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist qw(:try);

   try {
       $domain_obj->remove_ports(...);
   } catch NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist with {
        # Broadcast-domain does not exist
   };

=cut

package NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Broadcast domain.*not found/i],
    );
    
    $Log->exit();
    return $detected;
}

1;
