#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to add a port to the
##          broadcast-domain it is already assigned to.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned

=head1 DESCRIPTION

This exception is thrown when an attempt is made to add a port to the
broadcast-domain it is already assigned to.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned qw(:try);

   try {
       $domain_obj->add_ports(...);
   } catch NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned with {
        # Port is already assigned to that broadcast-domain
   };

=cut

package NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Port.*is already assigned/i],
    );
    
    $Log->exit();
    return $detected;
}

1;
