#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when trying to delete a broadcast domain which 
## has ports(with lifs association) assigned to it
##
## @author rawat@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a broadcast domain
which has ports(with lifs association) assigned to it.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation qw(:try);

   try {
       NACL::C::NetworkPortBroadcastDomain->delete(ipspace => $ipspace, %other_opts);
   } catch NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation with {
        # Network ports are assigned to this Vserver.
   };

=cut

package NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/is currently the home port or current port of a LIF/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
