# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortIfgrp/ConfiguredUp.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to add port to an ifgrp which is up.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp

=head1 DESCRIPTION

This exception is thrown when we attempt to add port to an ifgrp which is up.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp qw(:try);

   try {
    NACL::C::NetworkPortIfgrp->create(
        command_interface => $ci,
    	node              => $Node,
        ifgrp             => $Ifgrp,
        port              => $Port
    );
   } catch NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp with {
        # Cannot add a port to ifgrp which is up
   };

=cut

package NACL::C::Exceptions::NetworkPortIfgrp::ConfiguredUp;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/while\s+configured\s+up/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
