# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortIfgrp/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to remove a port which is not a part of the ifgrp.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to remove a port which is not a part of the ifgrp.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist qw(:try);

   try {
    NACL::C::NetworkPortIfgrp->delete(
                command_interface => $ci,
		ifgrp             => $ifgrp,
		port              => $port,
    );
   } catch NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist with {
        # cannot remove port as it is not a part of ifgrp
   };

=cut

package NACL::C::Exceptions::NetworkPortIfgrp::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/is\s+currently\s+not\s+part\s+of\s+the.*ifgrp/i,
		    qr/is not a member of ifgrp/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
