# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortIfgrp/IfgrpIsUp.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete an ifgrp which is up.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp

=head1 DESCRIPTION

This exception is thrown when we attempt to delete an ifgrp which is up.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp qw(:try);

   try {
    NACL::C::NetworkPortIfgrp->create(
                command_interface => $ci,
                ifgrp             => $ifgrp
    );
   } catch NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp with {
        # cannot remove port from an ifgrp which is up
   };

=cut

package NACL::C::Exceptions::NetworkPortIfgrp::IfgrpIsUp;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Can't\s+remove\s+a\s+port\s+from\s+an\s+ifgrp\s+which\s+is\s+up/,
                    qr/cannot\s+delete\s+from\s+an\s+ifgrp\s+that\s+is\s+up/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
