# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkPortVlan/PortHasLifBound.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to delete a vlan on which logical
## interfaces got created.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound

=head1 DESCRIPTION

An exception is thrown when we attempt to delete a vlan on which logical
interfaces got created.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound qw(:try);

   try {
       NACL::C::NetworkPortVlan->delete(
           command_interface  => $Node,
	   node              => $node,
           'vlan-id'         => $vlan_id,
           'port'            => $interface_name,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound with {
        # Port has a lif bound
   };

=cut

package NACL::C::Exceptions::NetworkPortVlan::PortHasLifBound;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already has a lif bound/i, qr/member of failover rule/i ],
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
