# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkRoute/DoesNotExist.pm#1 $
#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for NetworkRoute
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status shared 
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkRoute::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a network route
when entry does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkRoute::DoesNotExist qw(:try);

   try {
       NACL::C::NetworkRoute->delete(
           command_interface  => $Node,
            gateway           => $Gateway,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkRoute::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::NetworkRoute::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
