# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/NetworkRoutingGroupsRoute/AddressNotFound.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to create a network routing-groups route
## when ipv6 is disabled.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound

=head1 DESCRIPTION

This exception is thrown when we attempt to create a network routing-groups route
when ipv6 is disabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound qw(:try);

   try {
       NACL::C::NetworkRoutingGroupsRoute->create(
           command_interface  => $Node,
	   'routing-group'    => $Routing_group,
            gateway            => $Gateway,
           %other_opts);
   } catch NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound with {
        # Cannot find the address
   };

=cut

package NACL::C::Exceptions::NetworkRoutingGroupsRoute::AddressNotFound;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Can't find address/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
