# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Options/SnapmirrorNotLicensed.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when snapmirror is not licensed
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Options::SnapmirrorNotLicensed

=head1 DESCRIPTION

This exception is thrown when we attempt to perform snapmirror operations when it
is not licensed.

This exception is a sub-class of
L<NACL::C::Exceptions::SnapmirrorNotLicensed|lib-NACL-C-Exceptions-FeatureNotLicensed-pm>,
hence offers the same functionality and has the same attributes.

=over

=item Options

=over

=item C<< "option-name" => $string >>

(Required) Name of the option.

=back

=back

=head1 EXAMPLE

   use NACL::C::Exceptions::Options::SnapmirrorNotLicensed qw(:try);

   try {
	NACL::C::Options->option(
           command_interface => $node,
           'option-name'     => $option_name,
           'option-value'    => $option_value,
        );
   } catch NACL::C::Exceptions::Options::SnapmirrorNotLicensed with {
        my $exception = shift;
        $exception->license_feature();
   };

=cut

package NACL::C::Exceptions::Options::SnapmirrorNotLicensed;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with SCALAR);
use base qw(
    NACL::APISet::Exceptions::ResponseException
    NACL::C::Exceptions::FeatureNotLicensed
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {"option-name" => {type => SCALAR, optional => 1}},
        allow_extra => 1
    );
    my $detected = 0;
    if ($opts{"option-name"} =~ /snapmirror/i) {
        delete $opts{"option-name"};
	$detected = $pkg->_detect_helper(
                           %opts,
                           regexes => [ qr/unable to set option/i,qr/snapmirror is not licensed/i,qr/please license snapmirror/i]
                        );
    }
    $Log->exit() if $may_exit;
    return $detected;
}
1;
