#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for QosPolicyGroup
##
## @author shylaja@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::QosPolicyGroup::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete/modify a qos policy group
when entry does not exist.

=head1 EXAMPLE

use NACL::C::Exceptions::QosPolicyGroup::DoesNotExist qw(:try);

try {
    NACL::C::QosPolicyGroup->delete(
           command_interface  => $Node,
           'policy-group' => $policy_group,
    );
} catch NACL::C::Exceptions::QosPolicyGroup::DoesNotExist with {
      # Entry Does not exist
};


=cut

package NACL::C::Exceptions::QosPolicyGroup::DoesNotExist;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
          @args,
          regexes => [ qr/entry doesn't exist/, qr/no entries/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

