# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/QosWorkload/DoesNotExist.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exist for QosWorkload
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::QosWorkload::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete/modify a qos workload 
when entry does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::QosWorkload::DoesNotExist qw(:try);

   try {
       NACL::C::QosWorkload->delete(
           command_interface  => $Node,
           'wokload-uuid'      => $uuid,
           %other_opts);
   } catch NACL::C::Exceptions::QosWorkload::DoesNotExist with {
        # Entry Does not exist
   };

=cut

package NACL::C::Exceptions::QosWorkload::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::DoesNotExist
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/,]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
