#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary
## This exception is thrown if the password db file is busy when a security login
## operation is performed.
##
## @author rahula@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::DbFileBusy

=head1 DESCRIPTION

This exception is thrown if the password db file is busy when a security login
operation is performed.

=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::DbFileBusy qw(:try);

   try {
       NACL::C::SecurityLogin->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
           application        => $Application,
           authmethod         => $Authmethod,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityLogin::DbFileBusy with {
        # Db file is busy
   };

=cut

package NACL::C::Exceptions::SecurityLogin::DbFileBusy;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/chpass: the password db file is busy/i ],
    );
    $Log->exit();
    return $detected;
}
1;
