# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SecurityLogin/NewPasswordSameAsOldPassword.pm#1 $

# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when New password must be different than old password for SecurityLogin
##
## @author amala@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SecurityLogin::NewPasswordSameAsOldPassword

=head1 DESCRIPTION

This exception is thrown when we attempt to change the password which is same as old password.


=head1 EXAMPLE

   use NACL::C::Exceptions::SecurityLogin::NewPasswordSameAsOldPassword qw(:try);

   try {
       NACL::C::SecurityLogin->password(
           command_interface  => $Node,
           vserver            => $Vserver,
           username           => $Username,
	   newpwd             => $password,
	   newpwd2            => $password,
           %other_opts);
   } catch NACL::C::Exceptions::SecurityLogin::NewPasswordSameAsOldPassword with {
        #New password must be different than old password 
     };

=cut

package NACL::C::Exceptions::SecurityLogin::NewPasswordSameAsOldPassword;

use base qw( 
  NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args,
        regexes => [qr/New password must be different than old password./i] );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
