#
# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to break an XDP snapmirror
##          relationship.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::BreakNotAllowedForVault

=head1 DESCRIPTION

This exception is thrown when an attempt is made to break an XDP snapmirror
relationship.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::BreakNotAllowedForVault qw(:try);

   try {
       NACL::C::Snapmirror->break(%opts);
   } catch NACL::C::Exceptions::Snapmirror::BreakNotAllowedForVault with {
        # Failed because relationship is of type XDP
   };

=cut

package NACL::C::Exceptions::Snapmirror::BreakNotAllowedForVault;

use base qw(
	NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/"snapmirror break" is not allowed.*policy of type "vault"/],
    );
    $Log->exit();
    return $detected;
}
1;
