# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/CannotConnectToSourceFiler.pm
#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown if we cannot connect to the source filer. 
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::CannotConnectToSourceFiler

=head1 DESCRIPTION

This exception is thrown if we attempt to create a snapmirror when we cannot
connect to the source filer.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::CannotConnectToSourceFiler qw(:try);

   try {
       NACL::C::Snapmirror->create(
            command_interface  =>  $command_interface,
            'source-path'      =>  $source_specification,
            'destination-path' =>  $destination_specification,
            %other_options
        );
   } catch NACL::C::Exceptions::Snapmirror::CannotConnectToSourceFiler with {
        # Cannot connect to the source filer
   };

=cut

package NACL::C::Exceptions::Snapmirror::CannotConnectToSourceFiler;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/cannot\s+connect\s+to\s+source\s+filer/]
    );
    $Log->exit();
    return $detected;
}
1;
