#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to perform snapmirror operation
##          when another operation is in progress.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::DeleteIsPending

=head1 DESCRIPTION

This exception is thrown when an attempt is made to perform snapmirror
operation when delete is in progress.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::DeleteIsPending qw(:try);

   try {
       my $sm = NACL::C::Snapmirror->release(%opts);
   } catch NACL::C::Exceptions::Snapmirror::DeleteIsPending with {
        # Failed because delete is in progress
   };

=cut

package NACL::C::Exceptions::Snapmirror::DeleteIsPending;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
    $pkg->_detect_helper( @args, regexes => [qr/delete\s+is\s+pending\s+for\s+this\s+relationship\.\s+Try\s+again\s+after/i] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
