#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Snapmirror of the same name already exists
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a Snapmirror which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::AlreadyExists qw(:try);

   try {
       NACL::C::Snapmirror->create(Snapmirror => $already_existing_sm, %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::AlreadyExists with {
        # Snapmirror already exists.
   };

=cut

package NACL::C::Exceptions::Snapmirror::DestinationAlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Relationship with destination .* already exists/i,
                    qr/can only be destination for one SnapMirror relationship/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
