# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/NoReleasableDestinationFound.pm
#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception gets thrown if an attempt is made to release snapmirror relationship
## when there are no releasable destinations found.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::NoReleasableDestinationFound

=head1 DESCRIPTION

This exception gets thrown if an attempt is made to release snapmirror relationship
when there are no releasable destinations found.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::NoReleasableDestinationFound qw(:try);

   try {
       NACL::C::Snapmirror->release(
           command_interface  => $Node,
           'destination-path'       => $Path,
           %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::NoReleasableDestinationFound with {
        # no releasable destinations found
   };

=cut

package NACL::C::Exceptions::Snapmirror::NoReleasableDestinationFound;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/no\s+release\-able\s+destination\s+found\s+that\s+matches\s+those\s+parameters/i]
    );
    $Log->exit();
    return $detected;
}
1;
