# $Id$
#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a relationship information 
## is being updated and propagated
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingUpdated

=head1 DESCRIPTION

This exception is thrown when we attempt to delete/release a snapmirror
relation when the relationship information is being updated

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingUpdated qw(:try);

    try {
        NACL::STask::Snapmirror->purge(%opts);
    } catch NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingUpdated with {
        # Snapmirror relation is being updated
    };

    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->purge(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingUpdated',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Snapmirror::RelationshipInfoBeingUpdated;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/relationship.*try\s+the\s+command\s+again/i] );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
