# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Snapmirror/SnapshotInUse.pm# $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeSnapshot is in use
##
## @author msmeena@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::SnapshotInUse

=head1 DESCRIPTION

This exception is thrown when we attempt to delete/release a snapmirror
relation when a snapshot in the volume is locked.

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::SnapshotInUse qw(:try);

    try {
        NACL::STask::Snapmirror->purge(%opts);
    } catch NACL::C::Exceptions::Snapmirror::SnapshotInUse with {
        # Snapshot is in use.
    };

    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->purge(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::SnapshotInUse',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Snapmirror::SnapshotInUse;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/Snapshot is in use/] );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
