# $Id: //depot/prod/test/lib/NACL/C/Exceptions/Snapmirror/SubsystemNotFullyOperational.pm# $
#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to perform snapmirror create
##          operation when SnapMirror subsystem is not fully operational.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::SubsystemNotFullyOperational

=head1 DESCRIPTION

This exception is thrown when an attempt is made to perform snapmirror
create operation when SnapMirror subsystem is not fully operational.

=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::SubsystemNotFullyOperational qw(:try);

   try {
       my $sm = NACL::C::Snapmirror->create(%opts);
   } catch NACL::C::Exceptions::Snapmirror::SubsystemNotFullyOperational with {
        # Failed because SnapMirror subsystem is not fully operational
   };

   # An example of how to retry the create() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->create(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::SubsystemNotFullyOperational',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Snapmirror::SubsystemNotFullyOperational;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
        $pkg->_detect_helper( @args, regexes => [qr/SnapMirror\s+subsystem\s+is\s+not\s+fully\s+operational\s+yet\.\s+Wait\s+a\s+few\s+minutes\s+and\s+try\s+the\s+command\s+again/i] );

    $Log->exit() if $may_exit;
    return $detected;
}

1;


