#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when executing some snapmirror command before
## completing the System initialization properly.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::SystemInitialization

=head1 DESCRIPTION

This exception is thrown when we try to execute snapmirror
initialize command ahead of System Initialization.

=head1 EXAMPLE

    use NACL::C::Exceptions::Snapmirror::SystemInitialization qw(:try);

    try {
        NACL::STask::Snapmirror->initialize (%opts);
    } catch NACL::C::Exceptions::Snapmirror::SystemInitialization with {
        # SystemInitialization
    };

    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::STask::Snapmirror->initialize(%opts); },
        exceptions => 'NACL::C::Exceptions::Snapmirror::SystemInitialization',
        try_count  => 8,
        sleep_time => 1
    );

=cut

package NACL::C::Exceptions::Snapmirror::SystemInitialization;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/System\s+initialization\s+is\s+not\s+complete/i] );

    $Log->exit();
    return $detected;
}

1;
