#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Volume part of a snapmirror relationship
##          is being deleted. 
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Snapmirror::VolumeIsOffline

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a snapmirror relationship
of a volume which is offline.


=head1 EXAMPLE

   use NACL::C::Exceptions::Snapmirror::VolumeIsOffline qw(:try);

   try {
       NACL::C::Snapmirror->delete('source-volume' => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::Snapmirror::VolumeIsOffline with {
        # delete volume to delete the snapmirror relationship. 
   };

=cut

package NACL::C::Exceptions::Snapmirror::VolumeIsOffline;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $error1
        = qr/Failed to get volume attributes for .*(Volume is offline)/i;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [ $error1 ] );
    $Log->exit();
    return $detected;
} ## end sub detect

1;
