# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for SnapmirrorPolicy
##
## @author ng-ptefit-automation-core@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a SnapmirrorPolicy which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists qw(:try);

   try {
       $snapmirror_policy = NACL::C::SnapmirrorPolicy->create(
                command_interface => $command_int,
                'vserver' => $vserver,
                'policy' => $policy_name,
                 %other_opts);
   } catch NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+/i, qr/policy .* already exists/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
