# Copyright (c) 2009-2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when the snaplock compliance clock is not set.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::ComplianceClockNotSet

=head1 DESCRIPTION

This exception is thrown when the snaplock compliance clock is not set.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::ComplianceClockNotSet qw(:try);

   try {
       NACL::C::Aggregate->create%opts);
   } catch NACL::C::Exceptions::StorageAggregate::ComplianceClockNotSet with {
        # SnaplockComplianceClock not set
   };

=cut

package NACL::C::Exceptions::StorageAggregate::ComplianceClockNotSet;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            # CMode
            qr/ComplianceClock must be set before creating a SnapLock.*aggregate/i,

            # 7Mode
            qr/Use "snaplock clock initialize" to set Compliance Clock/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
