#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to purge the 
##          aggregate which is associated with volumes.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::ContainsVolumes

=head1 DESCRIPTION

This exception is thrown when an attempt is made to purge the
aggregate which is associated with volumes.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::ContainsVolumes();
    try {
       NACL::STask::Aggregate->purge();
    }
    catch NACL::C::Exceptions::StorageAggregate::ContainsVolumes {
        #perform appropriate action
    }

=cut

package NACL::C::Exceptions::StorageAggregate::ContainsVolumes;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::ResponseException
    NATE::BaseException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/aggregate.* cannot be destroyed because it contains one or more/i,
            qr/Delete or move the volumes/i,
            qr/Please first remove all of the volumes on the aggregate/i,
            qr/Cannot offline aggregate.* because it contains/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;


