#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a aggregate is mirrored when the free space
##          reallocation is enabled on the aggregate.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::FreeSpaceReallocationEnabled

=head1 DESCRIPTION

This exception is thrown when a attempt to mirror a aggregate when the free space
reallocation is enabled on the aggregate.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::FreeSpaceReallocationEnabled qw(:try);

   try {
       NACL::C::Aggregate->mirror(aggregate => $aggr, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::FreeSpaceReallocationEnabled with {
        # free space realloc is enabled. Disable it and try mirror again.
   };

=cut

package NACL::C::Exceptions::StorageAggregate::FreeSpaceReallocationEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);


sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*Free space reallocation is enabled on.*/ ],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
