# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when there are not enough disks available to create the aggregate with given diskcount.
##
## @author chethanm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::NotEnoughMatchingDisksAvailable

=head1 DESCRIPTION

This exception is thrown when there are not enough disks available to create the aggregate with given diskcount.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::NotEnoughMatchingDisksAvailable qw(:try);

   try {
       NACL::C::Aggregate->create(aggregate => $already_existing_aggr, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::NotEnoughMatchingDisksAvailable with {
        # Not enough matching disks
   };

=cut

package NACL::C::Exceptions::StorageAggregate::NotEnoughMatchingDisksAvailable;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/not enough (?:matching )*disks (:?are )*available/,
            qr/no\s+disks\s+are\s+available/i
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
