#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for  the license of the aggregagte (i.e. snaplock_enterprise)
##          This is mainly observerd when the aggregate is trying to add a snaplock_enterprise license.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::SnaplockEnterpriseNotEnabled;

=head1 DESCRIPTION

This exception is thrown when the aggregate is trying to add the license for snaplock enterprise.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::SnaplockEnterpriseNotEnabled qw(:try);

   try {
       $aggr_obj->create( feature   => 'snapLock_enterprise', );
   } catch NACL::C::Exceptions::StorageAggregate::SnaplockEnterpriseNotEnabled with {
        #That -L aggregate type requires a snaplock_enterprise license.
   };

=cut

package NACL::C::Exceptions::StorageAggregate::SnaplockEnterpriseNotEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
        NACL::C::Exceptions::FeatureNotLicensed
	NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/.*That -L aggregate type requires a snaplock_enterprise license.*/i],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
