#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for  the license of the aggregagte (i.e. snaplock)
##          This is mainly observerd when the aggregate is trying to add a snaplock compliance license.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::SnaplockNotEnabled;

=head1 DESCRIPTION

This exception is thrown when the aggregate is trying to add the license for compliance snaplock.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::SnaplockNotEnabled; qw(:try);

   try {
       $aggr_obj->create( feature   => 'snapLock', );
   } catch NACL::C::Exceptions::StorageAggregate::SnaplockNotEnabled with {
        #That -L aggregate type requires a snaplock license
   };

=cut

package NACL::C::Exceptions::StorageAggregate::SnaplockNotEnabled;

use base qw(
        NACL::C::Exceptions::FeatureNotLicensed
	NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper( @args,
        regexes => [qr/That -L aggregate type requires a snaplock license*/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
