#
# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary This is an exception thrown when an attempt is
##          made to add disks to an aggregate from a storage
##          pool when another operation is being performed
##          on the same storage pool.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::StoragePoolBusy;

=head1 DESCRIPTION

This exception is thrown when an attempt is made to add disks to an aggregate from a storage pool.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::StoragePoolBusy; qw(:try);

   try {
       $aggr_obj->add_disks(...);
   } catch NACL::C::Exceptions::StorageAggregate::StoragePoolBusy with {

   };

=cut

package NACL::C::Exceptions::StorageAggregate::StoragePoolBusy;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);


sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Storage Pool is busy with another operation/],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub get_job_id {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my ($job_id) =
      ($self->text() =~
/Storage\s+Pool\s+is\s+busy\s+with\s+another\s+operation.+job\s+ID\s+(\d+)/
      );
    $Log->exit() if $may_exit;
    return $job_id;
} ## end sub get_job_id


1;
