#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Aggregate is created on a MCC environment without
##          passing the mirror option.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::UnmirroredAggregatesNotSupported

=head1 DESCRIPTION

This exception is thrown when we attempt to create a aggregate in a MCC cluster
without passing the -mirror option.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::UnmirroredAggregatesNotSupported qw(:try);

   try {
       NACL::C::Aggregate->create(aggregate => $aggr, %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::UnmirroredAggregatesNotSupported with {
        # -mirror option should be passed for aggregate creation 
   };

=cut

package NACL::C::Exceptions::StorageAggregate::UnmirroredAggregatesNotSupported;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);


sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/.*Unmirrored aggregates are not supported in Metrocluster configurations.*/],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
