#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown while performing storage aggregate fetch,
##          and the error zsm can't connect to host is obtained
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate::ZsmConnectionError

=head1 DESCRIPTION

This exception is thrown while performing a storage aggregate
show, but the error zsm can't connect to host is obtained
from the filer.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate::ZsmConnectionError qw(:try);

   try {
        NACL::CS::StorageAggregate->fetch(
        command_interface  => $Node,
        aggregate          => $Aggr,
        %other_opts);
   } catch NACL::C::Exceptions::StorageAggregate::ZsmConnectionError with {
      # ZSM - Can't connect to host
  };

=cut

package NACL::C::Exceptions::StorageAggregate::ZsmConnectionError;

use base qw(
  NACL::C::Exceptions::Component
  NACL::APISet::Exceptions::ResponseException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/ZSM.*Can\'t\s+connect\s+to\s+host/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
