#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a 32-bit aggregate support is already disabled.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled

=head1 DESCRIPTION

This exception is thrown when we attempt to disable 32-bit aggregate support which 
is already disabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled qw(:try);

   try {
       NACL::C::StorageAggregate64bitUpgrade
            ->storage_aggregate_64bit_upgrade_32bit_disable(command_interface => $ci);
   } catch NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled with {
        # 32-bit aggregate support already disabled
   };

=cut

use strict;
use warnings;

package NACL::C::Exceptions::StorageAggregate64bitUpgrade::AlreadyDisabled;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/32\-bit aggregate support is already disabled/ ],
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
