#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to stop a wafliron
##          operation which is finishing.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing

=head1 DESCRIPTION

This exception is thrown when an attempt is made to stop a wafliron
operation which is finishing.

=head1 EXAMPLE

    use NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing qw(:try);

    try {
        $wafliron->stop();
    } catch NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing with {
        # Failed because wafliron is finishing
    };

   # An example of how to retry the stop() call if it fails for this reason
   use NACL::GeneralUtils qw(nacl_method_retry);
   nacl_method_retry(
        code       => sub { $wafliron->stop() },
        exceptions => 'NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::StorageAggregateWafliron::IsFinishing;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Wafliron is finishing/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
