# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/StorageFailover/NonHAMode.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when disabling storage failover service fails 
## as the cluster is in a Non-HA mode
##
## @author bkprasha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageFailover::NonHAMode

=head1 DESCRIPTION

This exception is thrown when disabling storage failover service fails 
as the cluster is in a Non-HA mode

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageFailover::NonHAMode qw(:try);

   try {
    NACL::C::StorageFailover->modify(
                command_interface => $ci,
				node              => $Filer,
				enabled			  => 'false',
    );
   } catch NACL::C::Exceptions::StorageFailover::NonHAMode with {
        # Failed to disable storage failover service. Reason: Non-HA mode
   };

=cut

package NACL::C::Exceptions::StorageFailover::NonHAMode;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Failed\s+to\s+disable\s+storage\s+failover\s+service\.\s+Reason:\s+Non-HA\s+mode/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
