# Copyright (c) 2014,.. NetApp Inc.
# All rights reserved.
#
## @summary An exception when storage failover takeover fails due to
## LIF migration and the recommendation is to re-issue command 
## with '-skip-lif-migration-before-takeover'
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover

=head1 DESCRIPTION

This exception is thrown when storage failover takeover fails due to
LIF migration and the recommendation is to re-issue command 
with '-skip-lif-migration-before-takeover'

=head1 EXAMPLE

   use NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover qw(:try);

   try {
    NACL::C::StorageFailover->takeover(
                                command_interface => $ci,
                                ...);
   } catch NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover with {
        # Takeover failed. Reason: LIF Migration failed
   };

=cut

package NACL::C::Exceptions::StorageFailover::SkipLifMigrationBeforeTakeover;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/Could not migrate LIFs away from node.*Reissue the command with \"-skip-lif-migration-before-takeover\" to skip the migration and continue with takeover/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
