#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a attempt is made to delete a storage pool
##           which is non-existent.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StoragePool::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a attempt is made to delete a storage pool
which is non-existent.

=head1 EXAMPLE

   use NACL::C::Exceptions::StoragePool::DoesNotExist qw(:try);

   try {
       NACL::C::StoragePool->delete( 'storage-pool' => $shared_storage_pool, );
   } catch NACL::C::Exceptions::StoragePool::DoesNotExist with {
        # Storage Pool does not exist any more
   };

=cut

package NACL::C::Exceptions::StoragePool::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Could not find/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
