# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/SystemNodeCoredumpConfig/DoesNotExist.pm#1 $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a SystemNodeCoredumpConfig operation is performed on a
##          SystemNodeCoredump that does not exist.
##
## @author raghaveb@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a SystemNodeCoredumpConfig operation is performed on a
core that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist qw(:try);

   try {
       $non_existing_core_obj->modify(attempts=> 212 );
   } catch NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist with {
        # SystemNodeCoredumpConfig does not exist any more
   };

=cut

package NACL::C::Exceptions::SystemNodeCoredumpConfig::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::InvalidParamValueException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper(
		@args,
		 regexes => [qr/.*entry\s+doesn't\s+exist*/ ,
                             qr/.*is\s+an\s+invalid\s+value\s+for\s+field\s\"-node.*/] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
