#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception can be thrown during software image upgrade.
##          relationship.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::SystemNodeImage::Remove32BitData

=head1 DESCRIPTION

This exception is thrown when an attempt is made to upgrade software image.

=head1 EXAMPLE

   use NACL::C::Exceptions::SystemNodeImage::Remove32BitData;

   try {
       NACL::C::SystemNodeImage->update(%opts);
   } catch NACL::C::Exceptions::SystemNodeImage::Remove32BitData with {};

=cut

package NACL::C::Exceptions::SystemNodeImage::Remove32BitData;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/format_32bit: You must remove all 32-bit data from the cluster and disable the 32-bit aggregate support using "storage aggregate 64bit-upgrade 32bit-disable"/i
        ],
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
