#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary A base class for the StorageAlreadyAdded exceptions
##
## @author garveyj@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::StorageAlreadyAdded

=head1 DESCRIPTION

This is an abstract base class for the exceptions that get thrown
when we attempt to add a FlexVol to a Container that is already added to a container.

=head1 EXAMPLE

   use NACL::C::Exceptions::VPserver::StorageAlreadyAdded qw(:try);

   try {
       NACL::C::VPserver::Container->add_storage(command_interface => ...,
                                                 container => ...,
                                                 cluster => ...,
                                                 volume => ...,
                                                 vserver => ...);
   } catch NACL::C::Exceptions::VPserver::StorageAlreadyAdded with {
        # Storage is already added to a container
   };

=cut


package NACL::C::Exceptions::VPserver::StorageAlreadyAdded;
use base 'NACL::C::Exceptions::Component';

use Params::Validate qw(validate_with OBJECT SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already attached to another container/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
