#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Volume of the same name already exists
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a volume which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::AlreadyExists qw(:try);

   try {
       NACL::C::Volume->create(volume => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::Volume::AlreadyExists with {
        # Volume already exists.
   };

=cut

package NACL::C::Exceptions::Volume::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        #First regex is for CMode and second regex is for 7Mode
        regexes => [ qr/Duplicate volume name/i, qr/Volume .* already exists/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
