#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to modify the attributes of a
##          volume to a state the volume is already in.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::AlreadyModified

=head1 DESCRIPTION

This exception is thrown when we attempt to modify the attributes of a volume
to a state the volume is already in.

Examples:

=over

=item Attempting to offline an already offline volume

=item Attempting to force-online an already online volume

=back

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::AlreadyModified qw(:try);

   try {
       $offline_vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::Volume::AlreadyModified with {
        # Already offline
   };

=cut

package NACL::C::Exceptions::Volume::AlreadyModified;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    # If we attempt to force-online when it's already online:
    # Error: command failed: Unable to set volume attributes for volume abc
    # on Vserver vs0. Reason: The volume or plex is online.
    my $force_online_regex
            = qr/Unable to set volume attributes.*The volume.*is online/i;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ $force_online_regex, qr/is already/ ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
