#
# Copyright (c) 2017 NetApp Inc.
# All rights reserved.
#
## @summary An exception when volume delete fails because volume
## is a part of application.
##
## dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::ApplicationVolumeRemove

=head1 DESCRIPTION

This exception is thrown when volume purge fails because volume
is a part of application.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::ApplicationVolumeRemove qw(:try);

   try {
        NACL::STask::Volume->purge();
   } catch NACL::C::Exceptions::Volume::ApplicationVolumeRemove with {
        # Failed because volume is a part of application.
   };

=cut

package NACL::C::Exceptions::Volume::ApplicationVolumeRemove;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/Volume.*in Vserver.*is part of application.*/i]);

    $Log->exit() if $may_exit;
    return $detected;
}

1;
