# $Id: //depot/prod/DOT/dev/test/lib/NACL/C/Exceptions/Volume/ConstituentsInInconsistentState.pm#1 $
#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when volume offline fails with one or more
##          consitituents in inconstitent state.
##
## dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::ConstituentsInInconsistentState

=head1 DESCRIPTION

This exception is thrown when volume offline fails with one or more
          consitituents in inconstitent state.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::ConstituentsInInconsistentState qw(:try);

   try {
        NACL::C::Volume->modify(
            command_interface                     => $ci,
            volume                                => $TestVol,
            vserver                               => $TestVserver,
            'space-nearly-full-threshold-percent' => $space_nearly_full,
            'space-full-threshold-percent'        => $space_full,
        );
   } catch NACL::C::Exceptions::Volume::ConstituentsInInconsistentState with {
        # Failed because one or more consitituents in inconstitent state.
   };

=cut

package NACL::C::Exceptions::Volume::ConstituentsInInconsistentState;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/One or more constituents.*inconsistent state/i]);

    $Log->exit() if $may_exit;
    return $detected;
}

1;
