#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when the volume is busy
##
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::CurrentlyBusy;

=head1 DESCRIPTION

This exception is thrown when the volume is found to be in dynamic growth
state or busy.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::CurrentlyBusy qw(:try);

    try {
        $vol_obj->modify(state => 'offline');
    } catch NACL::C::Exceptions::Volume::CurrentlyBusy with {
        # Volume is busy, hence can't modify the state to offline
    };

    # An example of how to retry the offline() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { $vol_obj->modify(state => 'offline') },
        exceptions => 'NACL::C::Exceptions::Volume::CurrentlyBusy',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::Volume::CurrentlyBusy;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    # CMode:
    # command failed: Unable to set volume attributes for volume "test_vol"
    # on Vserver "task_vserver". Reason: Volume busy.
    #
    # 7Mode:
    # vol offline: The specified operation could not be completed as the
    # volume is currently busy (dynamic create).

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Volume.*busy|dynamic create|Volume\s+is\s+in\s+use\s+by\s+other\s+operations/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
