#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Volume part of a snapmirror relationship
##          is being deleted.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::DeleteSnapMirrorRelationship

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a volume which is in 
snapmirror relationship with other volume.


=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::DeleteSnapMirrorRelationship qw(:try);

   try {
       NACL::C::Volume->delete(volume => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::Volume::DeleteSnapMirrorRelationship with {
        # Delete the snapmirror relationship to delete volume. 
   };

=cut

package NACL::C::Exceptions::Volume::DeleteSnapMirrorRelationship;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $error1
        = qr/destination or source endpoint of one or more SnapMirror relationships. Before you can delete the volume, you must delete the relationships/i;
    my $error2
        = qr/source endpoint of one or more SnapMirror relationships. Before you delete the volume, you must release the source information of the SnapMirror relationships/i;

    my $error3
        = qr/You must delete the SnapMirror relationships before you can delete this volume/i;

    my $detected = $pkg->_detect_helper( @args,
        regexes => [ $error1, $error2, $error3 ] );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
