#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to unmount
##      a volume when root volume is offline
##
## @author sunil.hn@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::FailedToDeleteJunction

=head1 DESCRIPTION

This exception is thrown when an attempt is made to unmount
a volume when root volume is offline

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::FailedToDeleteJunction ();
    try {
       NACL::C::Volume->unmount();
    }
    catch NACL::C::Exceptions::Volume::FailedToDeleteJunction {
        #perform appropriate action
    }

=cut

package NACL::C::Exceptions::Volume::FailedToDeleteJunction;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/Failed to delete junction from volume.*/i,
            qr/The volume or plex is offline/i,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
