#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to offline a volume
##          when file operations are in progress.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::FileOperationsInProgress

=head1 DESCRIPTION

This exception is thrown when an attempt is made to offline a volume
when file operations are in progress.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::FileOperationsInProgress qw(:try);

   try {
       NACL::C::VolumeFileCopy->start(volume => $vol_obj->volume(), %other_opts);
       $vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::Volume::FileOperationsInProgress with {
        # Failed because one or more file copy/move operations are in progress
   };

=cut

package NACL::C::Exceptions::Volume::FileOperationsInProgress;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/volume is the source for one or more ongoing.*transfer/i ]
    );
    
    $Log->exit();
    return $detected;
}

1;
