# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Volume/HasClones.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when we attempt to delete a volume which contains one or more clones.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::HasClones

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a volume which contains one or more clones.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::HasClones qw(:try);

   try {
    NACL::C::Volume->delete(
            command_interface=>$ci,
            volume =>$aggr_name,
            vserver => $vserver_name);
   } catch NACL::C::Exceptions::Volume::HasClones with {
        # One or more clones present in the volume
   };

=cut

package NACL::C::Exceptions::Volume::HasClones;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/has\s+one\s+or\s+more\s+clones/i,qr/it has clone volumes/i]
    );
    $Log->exit();
    return $detected;
}
1;
