#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to delete  a volume
##          which is a origin for a flexcache
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::HasFlexcacheOrigin

=head1 DESCRIPTION

This exception is thrown when an attempt is made to delete  a volume
which is a origin for a flexcache

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::HasFlexcacheOrigin qw(:try);

   try {
       $root_vol->delete();
   } catch NACL::C::Exceptions::Volume::HasFlexcacheOrigin with {
        # Failed because it has flexcache origin
   };

=cut

package NACL::C::Exceptions::Volume::HasFlexcacheOrigin;

use base qw(
	NACL::C::Exceptions::Component
	NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Cannot delete this volume as it is a FlexCache origin/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
