# $Id: //depot/prod/DOT/dev/test/lib/NACL/C/Exceptions/Volume/InvalidDataConstituentName.pm#1 $
#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a constituent
##          volume with an invalid name.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::InvalidDataConstituentName

=head1 DESCRIPTION

This exception is thrown when an attempt is made to create a constituent
volume with an invalid name.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::InvalidDataConstituentName qw(:try);

   try {
       NACL::C::Volume->create(
           command_interface => $ci,
           volume            => $invald_const_vol_name,
           vserver           => $vs_name,
           aggregate         => $aggr_name
       );
   } catch NACL::C::Exceptions::Volume::InvalidDataConstituentName with {
        # Failed because "aggregate" is required for volume creation
   };

=cut

package NACL::C::Exceptions::Volume::InvalidDataConstituentName;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/is an invalid name for a data constituent/i]);

    $Log->exit() if $may_exit;
    return $detected;
}

1;
