# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Volume/MustBeUnmounted.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to offline or restrict a volume
##          when it is mounted.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::MustBeUnmounted

=head1 DESCRIPTION

This exception is thrown when an attempt is made to offline or restrict a volume when
it is mounted.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::MustBeUnmounted qw(:try);

   try {
       $mountedVol_obj->modify();
   } catch NACL::C::Exceptions::Volume::MustBeUnmounted with {
        # must be unmounted before being taken offline or restricted
   };

=cut

package NACL::C::Exceptions::Volume::MustBeUnmounted;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/must be unmounted before being taken offline or restricted/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
