# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Volume/NotExported.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to unmount a volume
##          when it is not exported
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::NotExported

=head1 DESCRIPTION

This exception is thrown when we attempt to unmount a volume when it is not exported.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::NotExported qw(:try);

   try {
       $root_vol->unmount();
   } catch NACL::C::Exceptions::Volume::NotExported with {
        # Failed because it is not exported
   };

=cut

package NACL::C::Exceptions::Volume::NotExported;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Not currently exported/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
