#
# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a volume operation is not permitted to
## performed on a Vserver that is configured as the destination for Vserver
## DR.
##
## @author aanusha@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::OperationNotPermittedOnDrVolume

=head1 DESCRIPTION

This exception is thrown when a volume operation is not permitted
on a Vserver that is configured as the destination for Vserver DR.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::OperationNotPermittedOnDrVolume qw(:try);

   try {
       if (   ($opts{command_interface}->mode() eq 'CMode')
                  && ($need_to_unmount)) {
                  $pkg_or_obj->unmount(%opts);
       }
   } catch NACL::C::Exceptions::Volume::OperationNotPermittedOnDrVolume with {
        # Volume operation not allowed
   };

=cut

package NACL::C::Exceptions::Volume::OperationNotPermittedOnDrVolume;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,

        regexes => [ qr/.*This\s+operation\s+is\s+not\s+permitted.*Vserver\s+DR.*/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
