#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when volume delete fails when the automated rebalancer
## is running on infinite volume.
##
## dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::RebalancerRunning

=head1 DESCRIPTION

This exception is thrown when volume purge fails when the automated rebalancer
is running on infinite volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::RebalancerRunning qw(:try);

   try {
        NACL::STask::Volume->purge();
   } catch NACL::C::Exceptions::Volume::RebalancerRunning with {
        # Failed because capacity balancer is running on infinite volumes.
   };

=cut

package NACL::C::Exceptions::Volume::RebalancerRunning;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(@args,
        regexes => [qr/The Automated Rebalancer is running on Infinite Volume .*/i]);

    $Log->exit() if $may_exit;
    return $detected;
}

1;
