# $Id: $
#
# Copyright (c) 2013-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to delete a volume which is of type DEL.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::UnsupportedDELOperation

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a volume which is of type DEL.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::UnsupportedDELOperation qw(:try);

   try {
    NACL::C::Volume->delete(
            command_interface=>$ci,
            volume =>$aggr_name,
            vserver => $vserver_name);
   } catch NACL::C::Exceptions::Volume::UnsupportedDELOperation with {
       # volume delete on Volume type 'DEL' is not supported
   };

=cut

package NACL::C::Exceptions::Volume::UnsupportedDELOperation;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/command failed.*not supported on a volume of type "DEL"/i , qr/Volume is hidden with no access pending deletion/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
