#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to unmount a volume
##          of type DP or DC.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::Volume::UnsupportedUnmount

=head1 DESCRIPTION

This exception is thrown when an attempt is made to unmount a volume of
type DP or DC.

=head1 EXAMPLE

   use NACL::C::Exceptions::Volume::UnsupportedUnmount qw(:try);

   try {
       $dp_vol_obj->unmount();
   } catch NACL::C::Exceptions::Volume::UnsupportedUnmount with {
        # Failed because type is DP or DC
   };

=cut

package NACL::C::Exceptions::Volume::UnsupportedUnmount;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/operation is not supported on a volume of type/i ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
