# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeClone/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeClone of the same name already exists
##
## @author kathar@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeClone::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a clone which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeClone::AlreadyExists qw(:try);

   try {
    NACL::C::VolumeClone->create(
        command_interface => $ci,
        vserver           => $TestVserver,
        flexclone         => $NewClone,
	%other_opts
    );
   } catch NACL::C::Exceptions::VolumeClone::AlreadyExists with {
        # VolumeClone already exists.
   };

=cut

package NACL::C::Exceptions::VolumeClone::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already\s+exists/, qr/Duplicate\s+volume\s+name.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
