# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeClone/FeatureNotLicensed.pm#1 $
#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when flexclone is not licensed
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeClone::FeatureNotLicensed

=head1 DESCRIPTION

This exception is thrown when we attempt to perform clone operations when flexclone
is not licensed.

This exception is a sub-class of
L<NACL::C::Exceptions::FeatureNotLicensed|lib-NACL-C-Exceptions-FeatureNotLicensed-pm>,
hence offers the same functionality and has the same attributes.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeClone::FeatureNotLicensed qw(:try);

   try {
       NACL::C::VolumeClone->create(vserver => $vserver,
    			    flexclone         => $NewClone,
            	            %other_opts);
   } catch NACL::C::Exceptions::VolumeClone::FeatureNotLicensed with {
        my $exception = shift;
        $exception->license_feature();
   };

=cut

package NACL::C::Exceptions::VolumeClone::FeatureNotLicensed;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::FeatureNotLicensed
    NACL::APISet::Exceptions::CommandFailedException
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/You do not have a valid.*license/i, qr/FlexClone.*not licensed/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
