# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeCloneSplit/NotAClone.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to find any ongoing split actions
## on a normal volume (it was either never cloned before or it was
## cloned but it had been split from the parent)
##
## @author isshwar@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeCloneSplit::NotAClone;

=head1 DESCRIPTION

This exception is thrown when we try to find any ongoing split actions on a 
normal volume (it was either never cloned before or it was cloned but it had
been split from the parent)

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeCloneSplit::NotAClone qw(:try);

   try {
       $object = NACL::C::VolumeCloneSplit->find(command_interface =>$ci,
            filter => {flexclone => 'clone_vol'}
       );
   } catch NACL::C::Exceptions::VolumeCloneSplit::NotAClone with {
        # volume 'clone_vol' is split from the parent
   };

=cut

package NACL::C::Exceptions::VolumeCloneSplit::NotAClone;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::ResponseException
    NACL::Exceptions::NoElementsFound
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/The\s+volume\s+is\s+not\s+a\s+clone/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
