# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeCloneSplit/SplitInProgress.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when volume clone split is in progress
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress

=head1 DESCRIPTION

This exception is thrown when volume clone split is in progress

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress qw(:try);

   try {
       NACL::C::VolumeCloneSplit->start(
           command_interface  => $Node,
           vserver            => $Vserver,
	   flexclone          => $TestClone,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress with {
	# The clone is in the process of being split
   };

=cut

package NACL::C::Exceptions::VolumeCloneSplit::SplitInProgress;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/The\sclone\sis\sin\sthe\sprocess\sof\sbeing\ssplit/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
