# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeCopy/DestinationVolumeIsTooSmall.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary This type of exception is thrown when a volume copy is performed when destination volume
##          is smaller than source volume.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall

=head1 DESCRIPTION

This type of exception is thrown when a volume copy is performed when destination volume
is smaller than source volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall qw(:try);

   try {
       NACL::C::VolumeCopy->start(
             command_interface    => $ci,
             'destination-volume' => $dest_vol_of_small_size,
             'volume'             => $src_vol,
             %other_opts,
       );
   } catch NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall with {
        # Destination volume is too small
   };

=cut

package NACL::C::Exceptions::VolumeCopy::DestinationVolumeIsTooSmall;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,

        regexes => [ qr/Destination volume.*is too small/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
