# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeCopy/RestrictBeforeCopying.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when a volume copy is performed from a source
##          volume to a unrestricted volume.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying

=head1 DESCRIPTION

An exception is thrown when a volume copy is performed from a source
volume to a unrestricted volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying qw(:try);

   try {
       NACL::C::VolumeCopy->start(
               volume  => $source_volume,
               'destination-volume' => $unrestricted_vol,
               %other_opts
       );
   } catch NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying with {
        # Destination volume needs to be restricted
   };

=cut

package NACL::C::Exceptions::VolumeCopy::RestrictBeforeCopying;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Restrict it before copying to it/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
