#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to create a directory
## that already exists.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeDirectory::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a directory which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeDirectory::AlreadyExists qw(:try);

   try {
       NACL::C::VolumeDirectory->create(path => $already_existing_path, %other_opts);
   } catch NACL::C::Exceptions::VolumeDirectory::AlreadyExists with {
        # Directory already exists.
   };

=cut

package NACL::C::Exceptions::VolumeDirectory::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::_Mixins::VolumeFileDirectory qw(_detect_builder);
use Params::Validate qw(:all);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {$pkg->_response_exception_validate_spec()},
        allow_extra => 1
    );

    my $resp = $opts{exception}->get_response_object();
    my $val;
    if ($resp->isa('NACL::APISet::Response::ZAPI::CMode')) {

        # CMode ZAPI gives erro number as 13001, which cannot be relied on,
        # hence fall back to looking at only the error text for CMode ZAPI.
        $val = $pkg->_detect_helper(%opts, regexes => [qr/File exists/i],);
    } else {
        $val = $pkg->_detect_builder(
            %opts,
            zapi_error_number => 17,
            regexes           => [qr/File exists/i],
        );
    }

    $Log->exit($val);
    return $val;
}

1;
