#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when we attempt to delete a directory
## that does not exist
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeDirectory::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a directory which does
not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeDirectory::DoesNotExist qw(:try);

   try {
       NACL::C::VolumeDirectory->delete(path => $not_existing_dir);
   } catch NACL::C::Exceptions::VolumeDirectory::DoesNotExist with {
        # Directory does not exist.
   };

=cut

package NACL::C::Exceptions::VolumeDirectory::DoesNotExist;

use base qw(
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::DoesNotExist
);

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::_Mixins::VolumeFileDirectory qw(_detect_builder);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $val = $pkg->_detect_builder(
        @args,
        zapi_error_number => 2,
        regexes           => [qr/No such file or directory/i]
    );

    $Log->exit($val);
    return $val;
}

1;
