#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeEfficiency is already disabled
##          on a volume
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled

=head1 DESCRIPTION

This exception is thrown when we attempt to disable volume efficiency
on a volume that has efficiency already disabled 

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled qw(:try);

   try {
       NACL::C::VolumeEfficiency->on(volume => $already_existing_vol, %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled with {
        # Volume efficiency already disabled.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OperationAlreadyDisabled;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Operation has already been disabled/i, qr/operation already disabled/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

