#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary
#  This exception is thrown when volume efficiency undo is performed before
#  switching off volume efficiency
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::OperationNotDisabled

=head1 DESCRIPTION

This exception is thrown when volume efficiency undo is performed before
switching off volume efficiency

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::OperationNotDisabled qw(:try);

   try {
       NACL::C::VolumeEfficiency->undo(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts
	   );
   } catch NACL::C::Exceptions::VolumeEfficiency::OperationNotDisabled with {
        # Operation is not disabled
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::OperationNotDisabled;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Operation is not disabled/i ]
    );
    $Log->exit();
    return $detected;
}

1;

