#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when an attempt is made to enable VolumeEfficiency
##          on a restricted volume
##
## @author praveent@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::RestrictedVolume

=head1 DESCRIPTION

This exception is thrown when we attempt to enable VolumeEfficiency
on a restricted volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::RestrictedVolume qw(:try);

   try {
       NACL::C::VolumeEfficiency->on(
           command_interface  => $Node,
           vserver            => $Vserver,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::RestrictedVolume with {
        # Volume is restricted.
   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::RestrictedVolume;

use base qw(
  NACL::C::Exceptions::Component
  NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Volume is restricted/i , qr/Failed\s+to.*efficiency\s+on\s+volume.*of\s+Vserver.*Volume\s+is\s+restricted/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

