# $Id: //depot/prod/DOT/dev/test/lib/NACL/C/Exceptions/VolumeEfficiency/VolumeDoesNotExist.pm#1 $
#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a volume efficiency operation is performed on a
##          volume that does not exist.
##
## @author anupamap@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::VolumeDoesNotExist

=head1 DESCRIPTION

This exception is thrown when a volume efficiency operation is performed on a
volume that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::VolumeDoesNotExist qw(:try);
    try {
        NACL::C::VolumeEfficiency->stop(
        command_interface  => $Node,
        vserver            => $Vserver,
        %other_opts);
    } catch NACL::C::Exceptions::VolumeEfficiency::VolumeDoesNotExist with {
        # Volume does not exist any more
    };

=cut

package NACL::C::Exceptions::VolumeEfficiency::VolumeDoesNotExist;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    
    my $detected = $pkg->_detect_helper(
        @args,
        
        # First regex is for CMode, second is for 7Mode
        regexes => [ qr/Volume does not exist/i , qr/entry doesn't exist/, qr/No volume named .* exists/ ]
    );
    
    $Log->exit() if $may_exit;
    return $detected;
}

1;
