#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeEfficiency undo is performed
##         while the volume is a destination for file transfer
##        operations
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeEfficiency::VolumeIsDestinationForFileTransfers

=head1 DESCRIPTION

This exception is thrown when we attempt to undo volume efficiency
on a volume while the volume is a destination for file transfer operations

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeEfficiency::VolumeIsDestinationForFileTransfers qw(:try);

   try {
       NACL::C::VolumeEfficiency->undo(volume => $vol, %other_opts);
   } catch NACL::C::Exceptions::VolumeEfficiency::VolumeIsDestinationForFileTransfers with {

   };

=cut

package NACL::C::Exceptions::VolumeEfficiency::VolumeIsDestinationForFileTransfers;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/the\s+volume\s+is\s+a\s+destination\s+for\s+file\s+transfer\s+operations/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
