#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt to write to a file with
##          overwrite set to false and the file already exists.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFile::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to write to a file with
overwrite set to false and the file already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFile::AlreadyExists qw(:try);

   try {
       $file_obj->write(content => $content, overwrite => 'false');
   } catch NACL::C::Exceptions::VolumeFile::AlreadyExists with {
        # File already exists.
   };

=cut

package NACL::C::Exceptions::VolumeFile::AlreadyExists;

use base qw(NACL::C::Exceptions::AlreadyExists);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/File.*exists/i ]
    );

    $Log->exit();
    return $detected;
}

1;
