# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeFileClone/CloneFileExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to create a file clone which already
##         exists.
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFileClone::CloneFileExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a file clone which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFileClone::CloneFileExists qw(:try);

   try {
    NACL::C::VolumeFileClone->create(
        command_interface => $ci,
        vserver           => $TestVserver,
        volume            => $Volume,
        "source-path"     => $Source_Path,
         "destination-path"=> $Destination_Path,
	%other_opts
    );
   } catch NACL::C::Exceptions::VolumeFileClone::CloneFileExists with {
        # Clone file already exists.
   };

=cut

package NACL::C::Exceptions::VolumeFileClone::CloneFileExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
         # First regex is for CMode, second is for 7Mode
        regexes => [ qr/Clone file exists/i,qr/A LUN exists at the path/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
