#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when the volume is busy
##
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy;

=head1 DESCRIPTION

This exception is thrown when the volume is found to be in busy
state and clone operation cannot be started.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy qw(:try);

    try {
		NACL::C::VolumeFileClone->create(...);
    } catch NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy with {
        # Volume is busy, hence can't create the clone
    };

    # An example of how to retry the create() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { VolumeFileClone->create(...) },
        exceptions => 'NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::VolumeFileClone::CurrentlyBusy;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    # CMode:
	# command failed: Clone start failed: Clone operation failed to start: The volume is busy
	# and the operation can be retried at a later time.
    
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/volume is busy/i, qr/Device\s+busy/i ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
