#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary This exception is thrown when we try to destory a
##          File copy operation when Job is not found.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFileCopy::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when we try to destory a file copy for a job which
does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFileCopy::DoesNotExist qw(:try);

   try {
       $vol_file_copy->destroy(%opts);
   } catch NACL::C::Exceptions::VolumeFileCopy::DoesNotExist with {
       # Job record not found
   };

=cut

package NACL::C::Exceptions::VolumeFileCopy::DoesNotExist;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/Job record not found/,
        ]
    );

    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
