#
# Copyright (c) 2009-2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when the file fingerprint
##          dumps do not match.
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeFileFingerprintDump::FingerprintMismatch

=head1 DESCRIPTION

This exception is thrown when the file fingerprint dumps do not match.
This is thrown by
L<< NACL::CS::VolumeFileFingerprintDump->compare_fingerprint_dump|lib-NACL-CS-VolumeFileFingerprintDump-pm/compare_fingerprint_dump >>.

=head1 ATTRIBUTES

=head2 mismatch_values

In addition to the attributes that exception objects generally contain,
this exception also contains an C<mismatch_values> attribute. This is
a hash-reference, with the keys denoting the fields whose values didn't match.
The value is a hash-reference with the original_value and final_value.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeFileFingerprintDump::FingerprintMismatch qw(:try);

   try {
        $orig_dump_cs->compare_fingerprint_dump(other_fingerprint_cs => $final_dump_cs);
    } catch NACL::C::Exceptions::VolumeFileFingerprintDump::FingerprintMismatch with {
        my $exception  = $_[0];
        my %mismatches = $exception->mismatch_values();
        
        # Example contents of %mismatches:
        # 
        #    {
        #    'file-size' => {
        #                    'final_value' => '234',
        #                    'original_value' => '123'
        #                    }
        #    }
        #

    };

=cut

package NACL::C::Exceptions::VolumeFileFingerprintDump::FingerprintMismatch;
use base qw(NATE::BaseException);
use Class::MethodMaker [hash => 'mismatch_values'];
1;
