# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeMove/IsAFlexClone.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when source volume is a flexclone volume for VolumeMove
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeMove::IsAFlexClone

=head1 DESCRIPTION

This exception is thrown when we attempt to perform volume move start operation
on a flexclone volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeMove::IsAFlexClone qw(:try);

   try {
       NACL::C::VolumeMove->start(
           command_interface  => $Node,
           vserver            => $Vserver,
	   volume             => $Volume,
           'destination-aggregate' => $Dst_aggregate_name,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeMove::IsAFlexClone with {
        # Specified source volume is a flexclone volume
   };

=cut

package NACL::C::Exceptions::VolumeMove::IsAFlexClone;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Specified\s+source\s+volume\s+is\s+a\s+flexclone/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
