# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeMove/VolumeMoveAlreadyStarted.pm#1 $
#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to perform volume move start operation
## on an offlined volume.
##
## @author adarshm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeMove::VolumeMoveAlreadyStarted

=head1 DESCRIPTION

This exception is thrown when we attempt to perform volume move start operation
when already one volume start operation is in progress.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeMove::VolumeMoveAlreadyStarted qw(:try);

   try {
       NACL::C::VolumeMove->start(
           command_interface  => $Node,
           vserver            => $Vserver,
           volume             => $Volume,
           'destination-aggregate' => $Dst_aggregate_name,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeMove::VolumeMoveAlreadyStarted with {
        # when already one volume start operation is in progress
   };

=cut

package NACL::C::Exceptions::VolumeMove::VolumeMoveAlreadyStarted;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/A\s+volume\s+move\s+is\s+already\s+active/,
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
