# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeMove/VolumeNotOnline.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to perform volume move start operation
## on an offlined volume.
##
## @author benjaram@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeMove::VolumeNotOnline

=head1 DESCRIPTION

This exception is thrown when we attempt to perform volume move start operation
on an offlined volume.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeMove::VolumeNotOnline qw(:try);

   try {
       NACL::C::VolumeMove->start(
           command_interface  => $Node,
           vserver            => $Vserver,
           volume             => $Volume,
           'destination-aggregate' => $Dst_aggregate_name,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeMove::VolumeNotOnline with {
        # Specified source volume is not online
   };

=cut

package NACL::C::Exceptions::VolumeMove::VolumeNotOnline;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        #first regex is for CMode and the second regex is for 7Mode
        regexes => [
            qr/cannot\s+be\s+moved\s+because\s+it\s+is\s+not\s+online/,
            qr/Specified\s+volume\s+is\s+not\s+an\s+online\s+FlexVol/
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
} ## end sub detect

1;
