# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeQtree/AlreadyExists.pm#1 $
#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Qtree of the same name already exists
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeQtree::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a qtree which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeQtree::AlreadyExists qw(:try);

   try {
       NACL::C::Qtree->create(qtree => $already_existing_qtree, %other_opts);
   } catch NACL::C::Exceptions::VolumeQtree::AlreadyExists with {
        # Qtree already exists.
   };

=cut

package NACL::C::Exceptions::VolumeQtree::AlreadyExists;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
    NACL::C::Exceptions::AlreadyExists
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/qtree exists/, qr/Qtree .* already exists/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
