 # $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VolumeQtree/QtreeContainsLuns.pm#1 $
 #
 # Copyright (c) 2015 NetApp Inc.
 # All rights reserved.
 #
 ## @summary An exception for when a delete operation is performed on a
 ##          qtree and LUNs are present inside the Qtree.
 ##
 ## @author dl-nacl-dev@netapp.com
 ## @status Shared
 ## @pod here

=head1 DESCRIPTION

This exception is thrown when a qtree operation is performed on a
qtree and LUNs are present inside the Qtree.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeQtree::QtreeContainsLuns qw(:try);

   try {
       $qtree_obj_if_LUN_present->delete(force => 'true');
   } catch NACL::C::Exceptions::VolumeQtree::QtreeContainsLuns with {
        # fetch the LUNs and delete it.
   };

=cut

package NACL::C::Exceptions::VolumeQtree::QtreeContainsLuns;

# These are the exceptions typically thrown by the APISet
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg,@args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/it\s+contains\s+one\s+or\s+more\s+LUNs/i,@$additional_regexes],
    );
    $Log->exit() if $may_exit;

    return $detected;
}

1;
