#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when snapshot is created when clone operation
##          is running in background.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::ClonesBackedBySnapshots

=head1 DESCRIPTION

This exception is thrown when snapshot is created when clone operation
is running in background.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::ClonesBackedBySnapshots qw(:try);

   try {
       NACL::C::VolumeSnapshot->create( snapshot => $snapshot );
   } catch NACL::C::Exceptions::VolumeSnapshot::ClonesBackedBySnapshots with {
        my $exception = shift;
        #retry the snapshot creation after sometime.
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::ClonesBackedBySnapshots;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);


sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/.*Snapshot operation not allowed due to clones backed by.*snapshots.*/],
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
