#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when the volume is busy
##
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::CurrentlyBusy;

=head1 DESCRIPTION

This exception is thrown when the volume is found to be in busy
state and rename operation cannot be started.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::CurrentlyBusy qw(:try);

    try {
		NACL::C::VolumeSnapshot->rename(...);
    } catch NACL::C::Exceptions::VolumeSnapshot::CurrentlyBusy with {
        # Volume is busy, hence can't rename the volume
    };

    # An example of how to retry the rename() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { VolumeSnapshot->rename(...) },
        exceptions => 'NACL::C::Exceptions::VolumeSnapshot::CurrentlyBusy',
        try_count  => 5,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::VolumeSnapshot::CurrentlyBusy;

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/volume busy/i , qr/Device\s+busy/i , qr/volume is busy/i]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
