#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VolumeSnapshot is in use
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::InUse

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a snapshot
which is in use.

=head1 EXAMPLE

    use NACL::C::Exceptions::VolumeSnapshot::InUse qw(:try);

    try {
        NACL::C::VolumeSnapshot->delete(%opts);
    } catch NACL::C::Exceptions::VolumeSnapshot::InUse with {
        # VolumeSnapshot is in use.
    };
    
    # An example of how to retry the delete() call if it fails for this reason
    use NACL::GeneralUtils qw(nacl_method_retry);
    nacl_method_retry(
        code       => sub { NACL::C::VolumeSnapshot->delete(%opts); },
        exceptions => 'NACL::C::Exceptions::VolumeSnapshot::InUse',
        try_count  => 3,
        sleep_time => 10
    );

=cut

package NACL::C::Exceptions::VolumeSnapshot::InUse;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter();
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/Snapshot is in use/] );
    $Log->exit();
    return $detected;
}

1;
