#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception  when any multi-operation like multicreate|multidelete.. get invoked on VolumeSnapshot
##
## @author adarshm@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::MultiOperationFailed

=head1 DESCRIPTION

This exception is thrown when we attempt to invoke any multi-api(create|delete|rename|validate) on  snapshot for all volumes and it got failed for one or more volumes.

=head1 ATTRIBUTES

In addition to the normal attributes of exceptions
(such as C<text> and C<response_object>), this exception also has an
array attribute called C<errors>. Each entry is a hash-reference with
details of the error. Here's an example of what this hash-reference contains:

    {
      'errno' => '13020',
      'name' => 'dstvol',
      'reason' => 'Snapshot already exists.'
    }

=cut

=head2 EXAMPLE

    use NACL::C::Exceptions::VolumeSnapshot::MultiOperationFailed qw(:try);

    try {
        NACL::C::VolumeSnapshot->multirename(%opts);
    } catch NACL::C::Exceptions::VolumeSnapshot::MultiOperationFailed with {
        my $exception = $_[0];

        # Can get the text and response_object like with other exceptions
        my $text = $exception->text();
        my $resp_obj = $exception->get_response_object();

        # Use the "errors" accessor method to get the list of errors
        my @errors = $exception->errors();

    };

=cut

package NACL::C::Exceptions::VolumeSnapshot::MultiOperationFailed;

use Class::MethodMaker [array => 'errors'];

use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
1;
