#
# Copyright (c) 2009-2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when SnapRestore is not licensed
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshot::SnapRestoreNotLicensed

=head1 DESCRIPTION

This exception is thrown when SnapRestore is not licensed.

This exception is a sub-class of
L<NACL::C::Exceptions::FeatureNotLicensed|lib-NACL-C-Exceptions-FeatureNotLicensed-pm>,
hence offers the same functionality and has the same attributes.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshot::SnapRestoreNotLicensed qw(:try);

   try {
       $snapshot->restore();
   } catch NACL::C::Exceptions::VolumeSnapshot::SnapRestoreNotLicensed with {
        my $exception = shift;
        $exception->license_feature();
   };

=cut

package NACL::C::Exceptions::VolumeSnapshot::SnapRestoreNotLicensed;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::FeatureNotLicensed
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter();
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*You\s+do\s+not\s+have\s+a\s+valid.*license.*/ ]
    );
    $Log->exit();
    return $detected;
}
1;
