# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VolumeSnapshotPolicy
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a snapshot policy which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists qw(:try);

   try {
       $policy = NACL::C::VolumeSnapshotPolicy->create(
                command_interface => $command_interface,
                'vserver'   => $vserver,
                'policy'    => $policy,
                'enabled'   => $enabled,
                'schedule1' => $schedule1,
                'count1'    => $count1,
                 %other_opts);
   } catch NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VolumeSnapshotPolicy::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/, qr/.* already exists/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
