#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when entry doesn't exists for VolumeSnapshotPolicy
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when an attempt is made to delete a 
non-existent volume snapshot policy

=head1 EXAMPLE

   use NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist qw(:try);

   try {
       NACL::C::VolumeSnapshotPolicy->delete(
           command_interface => $Node,
           volume            => $Volume,
           snapshot          => $Snapshot,
           %other_opts);
   } catch NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist with {
        # Entry Does not exists
   };

=cut

package NACL::C::Exceptions::VolumeSnapshotPolicy::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args,
	        # First regex is for CMode ZAPI, second is for CMode CLI,
		regexes => [ qr/.*entry\s+doesn't\s+exist*/i ,
                             qr/.*There\s+are\s+no\s+entries.*/i]
      );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
