# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/Vserver/LifEnabled.pm#1 $
#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary An exception when deleting a vserver
## which is associated with enabled lif(s)
##
## @author dl-nacl-dev@netapp.com
## @status Public
## @pod here

=head1 NAME

NACL::C::Exceptions::Vserver::LifEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to delete a vserver
having network interface(s) which is in enabled state.

=head1 EXAMPLE

   use NACL::C::Exceptions::Vserver::LifEnabled qw(:try);

   try {
       NACL::C::Vserver->delete(
           command_interface  => $Node,
	   node              => $node,
           lif               => $interface_name,
           %other_opts);
   } catch NACL::C::Exceptions::Vserver::LifEnabled with {
        # Lif is in enabled state
   };

=cut

package NACL::C::Exceptions::Vserver::LifEnabled;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/could not be deleted because it is enabled/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
