# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a character mapping operation is performed on
## a vserver/volume that does not exist.
##
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a character mapping operation is performed on a
vserver/volume that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist qw(:try);

   try {
       NACL::C::VserverCifsCharacterMapping->delete(
           command_interface => $ci,
           'volume'          => $volume,
           'vserver'         => $vserver,
           %other_opts,
       );

   } catch NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist with{
        # Vserver/Volume does not exist
   };

=cut

package NACL::C::Exceptions::VserverCifsCharacterMapping::DoesNotExist;

use strict;
use warnings;

use base qw(
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::InvalidParamValueException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/entry doesn't exist/, qr/Vserver \w+ does not exist/ ]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
