# Copyright (c) 2009-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when a VserverCifsDomainPreferredDc operation is performed on a
##          domain entry that does not exist.
##
## @author ng-ptefit-automation-core@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a VserverCifsDomainPreferredDc operation is performed on a
entry that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist qw(:try);

   try {
       $non_existing_domain_preferredDc_obj->remove();
   } catch NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist with {
        # VserverCifsDomainPreferredDc does not exist any more
   };

=cut

package NACL::C::Exceptions::VserverCifsDomainPreferredDc::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/.*entry\s+doesn't\s+exist*/, qr/There\s+are\s+no\s+entries.*/] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
