#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an attempt is made to perform update
##          when An update is already in progress for Vserver
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsGroupPolicy::UpdateInProgress

=head1 DESCRIPTION

This exception is thrown when an attempt is made to perform update
when An update is already in progress for Vserver

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsGroupPolicy::UpdateInProgress qw(:try);

   try {
       my $sm = NACL::C::VserverCifsGroupPolicy->update(%opts);
   } catch NACL::C::Exceptions::VserverCifsGroupPolicy->update with {
        # Failed because transfer is in progress
   };

=cut

package NACL::C::Exceptions::VserverCifsGroupPolicy::UpdateInProgress;

use base qw(
	NACL::APISet::Exceptions::CommandFailedException
	NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
          $pkg->_detect_helper( @args,
                  regexes => [qr/An update is already in progress for Vserver/i,
                              qr/Kicking GPO Main Update job for Vserver/i] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
