# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverCifsShare/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverCifsShare
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverCifsShare::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a cifs share which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverCifsShare::AlreadyExists qw(:try);

   try {
       NACL::C::VserverCifsShare->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           'share-name'       => $Share_name,
           %other_opts);
   } catch NACL::C::Exceptions::VserverCifsShare::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverCifsShare::AlreadyExists;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Duplicate\s+entry/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
