# Copyright (c) 2012-2014 NetApp Inc.
# All rights reserved.
#
## @summary An exception when the default VserverExportPolicy is tried to be deleted.
##
## @author ng-ptefit-automation-core@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverExportPolicy::CannotDeleteDefault

=head1 DESCRIPTION

This exception is thrown when we attempt to delete the default export policy

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverExportPolicy::CannotDeleteDefault qw(:try);

   try {
       NACL::C::VserverExportPolicy->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
           domain             => 'default',
           %other_opts);
   } catch NACL::C::Exceptions::VserverExportPolicy::CannotDeleteDefault with {
        # Cannot delete default ruleset.
   };

=cut

package NACL::C::Exceptions::VserverExportPolicy::CannotDeleteDefault;
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/cannot\s+delete\s+default/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
