#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a OnDemandTask has already been scheduled
##
## @author roopeshw@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverIscsiIsns::AlreadyModified

=head1 DESCRIPTION

This exception is thrown when we attempt to schedule an antivirus on-demand
task while another task has already been scheduled on a vserver.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverIscsiIsns::AlreadyModified qw(:try);

   try {
       NACL::C::VserverIscsiIsns->modify(%opts);
   } catch NACL::C::Exceptions::VserverIscsiIsns::AlreadyModified with {
   };

=cut

package NACL::C::Exceptions::VserverIscsiIsns::AlreadyModified;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException 
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Specified\starget\sname\sis\ssame\sas\scurrent\starget\sname/i ] 
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;


