#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when Iscsi service isn't found or configured.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverIscsiIsns::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when Iscsi service isn't found or configured. 
 
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverIscsiIsns::DoesNotExist qw(:try);

   try {
       NACL::C::VserverIscsiIsns->delete(
           command_interface  => $Node,
            vserver           => $TestVserver,
           %other_opts);
   } catch NACL::C::Exceptions::VserverIscsiIsns::DoesNotExist with {
        # enable the Iscsi service
   };

=cut

package NACL::C::Exceptions::VserverIscsiIsns::DoesNotExist;

use base qw(
    NACL::C::Exceptions::DoesNotExist
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
                    qr/Cannot\s+find\s+iSNS\s+service/i,
                    qr/Cannot\s+find\s+iscsi\s+service/i,
                    qr/no\sentries\smatching\syour\squery/i,
                    qr/entry\sdoesn't\sexist/i]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
