# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when a VserverLocks operation is performed on a
##          coredump that does not exist.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverLocks::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a VserverLocks operation is performed on a
core that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverLocks::DoesNotExist qw(:try);

   try {
       $non_existing_lock_obj->break(lockid => $lock_id);
   } catch NACL::C::Exceptions::VserverLocks::DoesNotExist with {
        # VserverLocks does not exist any more
   };

=cut

package NACL::C::Exceptions::VserverLocks::DoesNotExist;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::InvalidParamValueException
);

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    my $detected =
      $pkg->_detect_helper( @args, regexes => [qr/.*is\s+an\s+invalid\s+value\s+for\s+field*/] );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
