#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a vserver nfs modifyoperation is performed
##          and no values to apply are passed in.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfs::NoValuesToApply

=head1 DESCRIPTION

This exception is thrown when a vserver nfs modify operation is performed
and no values to apply are passed in.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfs::NoValuesToApply qw(:try);

   try {
       NACL::C::VserverNfs->modify(%opts);
   } catch NACL::C::Exceptions::VserverNfs::NoValuesToApply with {
        # No values to apply were passed in
   };

=cut

package NACL::C::Exceptions::VserverNfs::NoValuesToApply;


use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/.*You\s*need\s*to\s*provide\s*some\s*values\s*to\s*apply.*/i,
                     qr/.*You\s*must\s*provide\s*one\s*or\s*more\s*values\s*to\s*apply\s*your\s*changes.*/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
