#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a vserver nfs kerberos interface disable operation is
##          performed on an interface that is already disabled
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyDisabled

=head1 DESCRIPTION

This exception is thrown when a vserver nfs kerberos interface disablen is performed on
an interface that is already disabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyDisabled qw(:try);

   try {
       NACL::C::VserverNfsKerberosInterface->disable(%opts);
   } catch NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyDisabled with {
        # Interface is already disabled
   };

=cut

package NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyDisabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Kerberos is already disabled on this LIF/,
                     qr/entry doesn't exist/i        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
