#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when an vserver nfs kerberos interface is already enabled
##
## @author madhavs@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enabled an 
vserver nfs kerberos interface that is already enabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyEnabled qw(:try);

   try {
       NACL::C::VserverNfsKerberosInterface->enable(%opts);
   } catch NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyEnabled with {
        # Already Enabled.
   };

=cut

package NACL::C::Exceptions::VserverNfsKerberosInterface::AlreadyEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
);

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/Kerberos is already enabled on this LIF/i ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
