#
# Copyright (c) 2009-2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VserverNfsKerberosRealm of the same name already exists
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfsKerberosRealm::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a VserverNfsKerberosRealm which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfsKerberosRealm::AlreadyExists qw(:try);

   try {
       NACL::C::VserverNfsKerberosRealm->create(VserverNfsKerberosRealm => $already_existing_aggr, %other_opts);
   } catch NACL::C::Exceptions::VserverNfsKerberosRealm::AlreadyExists with {
        # VserverNfsKerberosRealm already exists.
   };

=cut

package NACL::C::Exceptions::VserverNfsKerberosRealm::AlreadyExists;

use base qw(
    NACL::C::Exceptions::Component
    NACL::C::Exceptions::AlreadyExists
    NACL::APISet::Exceptions::CommandFailedException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [
            qr/duplicate entry/i, # CMode CLI and ZAPI
        ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
