#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a VserverNfsKerberosRealm operation is performed on a
##          VserverNfsKerberosRealm that does not exist.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverNfsKerberosRealm::DoesNotExist

=head1 DESCRIPTION

This exception is thrown hen a VserverNfsKerberosRealm operation is performed on a
VserverNfsKerberosRealm that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverNfsKerberosRealm::DoesNotExist qw(:try);

   try {
       $non_existing_vol_obj->modify(state => 'offline');
   } catch NACL::C::Exceptions::VserverNfsKerberosRealm::DoesNotExist with {
        # VserverNfsKerberosRealm does not exist any more
   };

=cut

package NACL::C::Exceptions::VserverNfsKerberosRealm::DoesNotExist;

use base qw(
    NACL::C::Exceptions::Component
    NACL::C::Exceptions::DoesNotExist
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException
);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,

        # First regex is for CMode CLI, second is for Cmode Zapi
        regexes => [ qr/no entries/i, qr/entry doesn't exist/i]
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;
