# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverSecurityFileDirectoryPolicyTask/AlreadyExists.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverSecurityFileDirectoryPolicyTask
##
## @author Kathar.Hidayath@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create a filedirectory ids which already
exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::AlreadyExists qw(:try);

   try {
       NACL::C::VserverSecurityFileDirectoryNtfs->create(
           command_interface  => $Node,
           vserver            => $Vserver,
           'ntfs-sd'           => $Descriptor,
           %other_opts);
   } catch NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::AlreadyExists with {
        # Duplicate entry exists 
   };

=cut

package NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::AlreadyExists;
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
    NACL::C::Exceptions::VserverSecurityFileDirectoryPolicyTask::DuplicateEntry
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/duplicate\s+entry/]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
