#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary An exception when there is a duplicate entry for VserverServicesNameServiceNsSwitch
##
## @author sshaik@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverServicesNameServiceNsSwitch::AlreadyExists

=head1 DESCRIPTION

This exception is thrown when we attempt to create vserver services nameservice ns-switch
which already exists.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverServicesNameServiceNsSwitch::AlreadyExists qw(:try);

   try {
       NACL::C::VserverServicesNameServiceNsSwitch->delete(
           command_interface  => $Node,
           vserver            => $Vserver,
           sources            => [$Sources]
           database           => $database,
           %other_opts);
   } catch NACL::C::Exceptions::VserverServicesNameServiceNsSwitch::AlreadyExists with {
        # Duplicate entry exists
   };

=cut

package NACL::C::Exceptions::VserverServicesNameServiceNsSwitch::AlreadyExists;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::AlreadyExists
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/duplicate\s+entry/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}
1;
