#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception for when a Vscan has already been enabled
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverVscan::AlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable a vscan on a vserver 
on which vscan has already been enabled.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverVscan::AlreadyEnabled qw(:try);

   try {
       NACL::C::VserverVscan->enable(%opts);
   } catch NACL::C::Exceptions::VserverVscan::AlreadyEnabled with {
        # Vscan has already been enabled.
   };

=cut

package NACL::C::Exceptions::VserverVscan::AlreadyEnabled;

use base qw(
    NACL::APISet::Exceptions::CommandFailedException 
    NACL::C::Exceptions::Component
);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already enabled/i] 
    );

    $Log->exit() if $may_exit;
    return $detected;
}

1;


