#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to enable an igroup which is mapped
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyDisabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable on access policy

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyDisabled qw(:try);

   try {
       NACL::C::VserverVscanOnAccessPolicy->enable(
           command_interface => $Node,
           %other_opts);
   } catch NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyDisabled with {
        # already disabled
   };

=cut

package NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyDisabled;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already disabled/i, ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

1;
