# $Id: //depot/prod/test/nacldev/lib/NACL/C/Exceptions/VserverVscanOnAccessPolicy/AlreadyEnabled.pm#1 $
#
# Copyright (c) 2012 NetApp Inc.
# All rights reserved.
#
## @summary An exception is thrown when we attempt to enable an igroup which is mapped
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled

=head1 DESCRIPTION

This exception is thrown when we attempt to enable on access policy

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled qw(:try);

   try {
       NACL::C::VserverVscanOnAccessPolicy->enable(
           command_interface => $Node,
           %other_opts);
   } catch NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled with {
        # on access policy already enabled
   };

=cut

package NACL::C::Exceptions::VserverVscanOnAccessPolicy::AlreadyEnabled;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw(
    NACL::APISet::Exceptions::CommandFailedException
    NACL::C::Exceptions::Component
);
use Params::Validate qw(validate_with SCALAR);


use Class::MethodMaker [
    scalar => 'enabled_policy'
];

sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [ qr/already enabled/i,
                     qr/Disable the policy and try the command again/ ]
    );
    $Log->exit() if $may_exit;
    return $detected;
}

sub convert {
    $Log->enter() if $may_enter;
    my ( $pkg, %args ) = @_;
    my $obj = $pkg->SUPER::convert(%args);
    if( $obj->text() =~ /\("(\S+)"\)/ ) {
        $obj->enabled_policy($1);
    }
    $Log->exit() if $may_exit;
    return $obj;
}

1;
