# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary An exception when a VserverVscanScannerPool operation is performed on a
##          scannerpool that does not exist.
##
## @author ng-ptefit-automation-core
## @status Shared
## @pod here

=head1 NAME

NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist

=head1 DESCRIPTION

This exception is thrown when a VserverVscanScannerPool operation is performed on a
scanner pool that does not exist.

=head1 EXAMPLE

   use NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist qw(:try);

   try {
       $non_existing_symlink_obj->modify(level=>'emerg');
   } catch NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist with {
        # VserverCifsSymlink does not exist any more
   };

=cut

package NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist;

use base 'NACL::C::Exceptions::DoesNotExist';

# These are the exceptions typically thrown by the APISet
use base qw(
    NACL::C::Exceptions::Component
    NACL::APISet::Exceptions::CommandFailedException
    NACL::APISet::Exceptions::NoMatchingEntriesException 
    );

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
# Handle the typical error message
sub detect {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my $detected = $pkg->_detect_helper(
        @args,
        regexes => [qr/.*entry\s+doesn't\s+exist*/i, qr/There\s+are\s+no\s+entries.*/i, qr/.*does\s+not\s+exist*/i]
   );
    $Log->exit() if $may_exit;
    return $detected;
}

1;

