# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Fcp Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Fcp

=head1 DESCRIPTION

C<NACL::C::Fcp> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Fcp.  A related class is
L<NACL::CS::Fcp|lib-NACL-CS-Fcp-pm>, which represents
the past state of an ONTAP Fcp.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::Fcp;

use strict;
use warnings;

use NACL::CS::Fcp;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $Fcp = NACL::C::Fcp->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new Fcp component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

=head2 find

    my $Fcp_obj  = NACL::C::Fcp->find(command_interface => $ci, ...);
	my @Fcp_objs = NACL::C::Fcp->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my Fcp_state = $Fcp_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::Fcp would return 
a NACL::CS::Fcp object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 vserver_fcp_create

    NACL::C::Fcp->vserver_fcp_create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
   
(Class method)
This method is used to create FCP service configuration.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

=item C<< 'status-admin' => $string >>

Administrative Status possible value(s) are, down,up
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp create" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( status-admin target-name force
                    )
            ),
        },
        cli_cmd  => 'vserver fcp create',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_create',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_create');

    $Log->exit() if $may_exit;
}

=head2 vserver_fcp_delete

    NACL::C::Fcp->vserver_fcp_delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $Fcp_obj->vserver_fcp_delete(...);

(Class or instance method)
This method is used to delete FCP service configuration.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp delete" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        cli_cmd  => 'vserver fcp delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_delete',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_delete_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_fcp_delete'
    );

    $Log->exit() if $may_exit;
}

=head2 vserver_fcp_modify

    NACL::C::Fcp->vserver_fcp_modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $Fcp_obj->vserver_fcp_modify(...);

(Class or instance method)
This method is used to modify FCP service configuration.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'status-admin' => $string >>

Administrative Status possible value(s) are, down,up
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp modify" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( status-admin target-name force
                    )
            ),
        },
        cli_cmd  => 'vserver fcp modify',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_modify',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_fcp_modify'
    );

    $Log->exit() if $may_exit;
}

=head2 vserver_fcp_nodename

    NACL::C::Fcp->vserver_fcp_nodename(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $Fcp_obj->vserver_fcp_nodename(...);

(Class or instance method)
This method is used to display or modify a Vserver's FCP target name.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp nodename" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_nodename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( target-name force
                    )
            ),
        },
        cli_cmd  => 'vserver fcp nodename',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_nodename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_nodename',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_nodename_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_fcp_nodename'
    );

    $Log->exit() if $may_exit;
}

=head2 vserver_fcp_show

    my $vserver_fcp_show_obj = NACL::C::Fcp->vserver_fcp_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @vserver_fcp_show_objs = NACL::C::Fcp->vserver_fcp_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $vserver_fcp_show_obj = $Fcp_obj->vserver_fcp_show(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpVserverFcpShow-pm/ATTRIBUTES>
of NACL::CS::FcpVserverFcpShow as input. 
This method returns objects of the type L<NACL::CS::FcpVserverFcpShow|lib-NACL-CS-FcpVserverFcpShow-pm>.

=cut

sub vserver_fcp_show {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpVserverFcpShow;
    my @state_objs = NACL::CS::FcpVserverFcpShow->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_fcp_start

    NACL::C::Fcp->vserver_fcp_start(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $Fcp_obj->vserver_fcp_start(...);

(Class or instance method)
This method is used to starts the FCP service.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp start" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_start {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        cli_cmd  => 'vserver fcp start',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_start',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_start_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_fcp_start'
    );

    $Log->exit() if $may_exit;
}

=head2 network_fcp_adapter_stats

    my $network_fcp_adapter_stats_obj = NACL::C::Fcp->network_fcp_adapter_stats(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @network_fcp_adapter_stats_objs = NACL::C::Fcp->network_fcp_adapter_stats(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $network_fcp_adapter_stats_obj = $Fcp_obj->network_fcp_adapter_stats(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpNetworkFcpAdapterStats-pm/ATTRIBUTES>
of NACL::CS::FcpNetworkFcpAdapterStats as input. 
This method returns objects of the type L<NACL::CS::FcpNetworkFcpAdapterStats|lib-NACL-CS-FcpNetworkFcpAdapterStats-pm>.

=cut

sub network_fcp_adapter_stats {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpNetworkFcpAdapterStats;
    my @state_objs = NACL::CS::FcpNetworkFcpAdapterStats->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_fcp_status

    my $vserver_fcp_status_obj = NACL::C::Fcp->vserver_fcp_status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @vserver_fcp_status_objs = NACL::C::Fcp->vserver_fcp_status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $vserver_fcp_status_obj = $Fcp_obj->vserver_fcp_status(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpVserverFcpStatus-pm/ATTRIBUTES>
of NACL::CS::FcpVserverFcpStatus as input. 
This method returns objects of the type L<NACL::CS::FcpVserverFcpStatus|lib-NACL-CS-FcpVserverFcpStatus-pm>.

=cut

sub vserver_fcp_status {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpVserverFcpStatus;
    my @state_objs = NACL::CS::FcpVserverFcpStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_fcp_stop

    NACL::C::Fcp->vserver_fcp_stop(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $Fcp_obj->vserver_fcp_stop(...);

(Class or instance method)
This method is used to stops the FCP service.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp stop" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        cli_cmd => 'vserver fcp stop',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_fcp_stop',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_stop_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;

    $pkg_or_obj->_base_backend_cmode_cli(
        @_,
        api => 'vserver_fcp_stop'
    );

    $Log->exit() if $may_exit;
}

=head2 vserver_show

    my $vserver_show_obj = NACL::C::Fcp->vserver_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @vserver_show_objs = NACL::C::Fcp->vserver_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $vserver_show_obj = $Fcp_obj->vserver_show(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpVserverShow-pm/ATTRIBUTES>
of NACL::CS::FcpVserverShow as input. 
This method returns objects of the type L<NACL::CS::FcpVserverShow|lib-NACL-CS-FcpVserverShow-pm>.

=cut

sub vserver_show {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpVserverShow;
    my @state_objs = NACL::CS::FcpVserverShow->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_start

    NACL::C::Fcp->vserver_start(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to starts the FCP service.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_start',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_start',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_start',);
    $Log->exit() if $may_exit;
}

=head2 vserver_stop

    NACL::C::Fcp->vserver_stop(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to stops the FCP service.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp stop" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_stop',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_stop',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_stop',);
    $Log->exit() if $may_exit;
}

=head2 vserver_modify

    NACL::C::Fcp->vserver_modify(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to modify FCP service configuration.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'status-admin' => $string >>

Administrative Status possible value(s) are, down,up
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status-admin vserver target-name force
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_modify',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_modify',);
    $Log->exit() if $may_exit;
}

=head2 network_adapter_stats

    my $network_adapter_stats_obj = NACL::C::Fcp->network_adapter_stats(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @network_adapter_stats_objs = NACL::C::Fcp->network_adapter_stats(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $network_adapter_stats_obj = $Fcp_obj->network_adapter_stats(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpNetworkAdapterStats-pm/ATTRIBUTES>
of NACL::CS::FcpNetworkAdapterStats as input. 
This method returns objects of the type L<NACL::CS::FcpNetworkAdapterStats|lib-NACL-CS-FcpNetworkAdapterStats-pm>.

=cut

sub network_adapter_stats {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpNetworkAdapterStats;
    my @state_objs = NACL::CS::FcpNetworkAdapterStats->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_delete

    NACL::C::Fcp->vserver_delete(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to delete FCP service configuration.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_delete',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_delete',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_delete',);
    $Log->exit() if $may_exit;
}

=head2 vserver_create

    my $Fcp_obj	= NACL::C::Fcp->vserver_create(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to create FCP service configuration.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'status-admin' => $string >>

Administrative Status possible value(s) are, down,up
Applicable for CMode CLI

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( status-admin vserver target-name force
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_create',
    );

    my $obj = $pkg->new(command_interface => $opts{command_interface},);
    $Log->exit() if $may_exit;
    return $obj;
}

sub _vserver_create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_create',);
    $Log->exit() if $may_exit;
}

=head2 vserver_nodename

    NACL::C::Fcp->vserver_nodename(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to display or modify a Vserver's FCP target name.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $string >>

Vserver Name
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver fcp nodename" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_nodename {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver target-name force
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'vserver_fcp_nodename',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_nodename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Fcp::vserver_nodename',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_nodename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'vserver_fcp_nodename',);
    $Log->exit() if $may_exit;
}

=head2 vserver_status

    my $vserver_status_obj = NACL::C::Fcp->vserver_status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @vserver_status_objs = NACL::C::Fcp->vserver_status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $vserver_status_obj = $Fcp_obj->vserver_status(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-FcpVserverStatus-pm/ATTRIBUTES>
of NACL::CS::FcpVserverStatus as input. 
This method returns objects of the type L<NACL::CS::FcpVserverStatus|lib-NACL-CS-FcpVserverStatus-pm>.

=cut

sub vserver_status {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::FcpVserverStatus;
    my @state_objs = NACL::CS::FcpVserverStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
