# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LeagueMember Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LeagueMember

=head1 DESCRIPTION

C<NACL::C::LeagueMember> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LeagueMember.  A related class is
L<NACL::CS::LeagueMember|lib-NACL-CS-LeagueMember-pm>, which represents
the past state of an ONTAP LeagueMember.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 cluster_name

=head2 league_name

=cut

package NACL::C::LeagueMember;

use strict;
use warnings;

use NACL::CS::LeagueMember;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LeagueMember = NACL::C::LeagueMember->new(
                                   command_interface => $ci,
                                   'cluster-name' => $cluster_name,
                                   'league-name' => $league_name,
								  );

Return a new LeagueMember component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'cluster_name',
    scalar => 'league_name',
];

=head2 find

    my $LeagueMember_obj  = NACL::C::LeagueMember->find(command_interface => $ci, ...);
	my @LeagueMember_objs = NACL::C::LeagueMember->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my LeagueMember_state = $LeagueMember_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LeagueMember would return 
a NACL::CS::LeagueMember object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 delete

    NACL::C::LeagueMember->delete(
                                command_interface => $ci,
	                            'cluster-name' => $cluster_name,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueMember_obj->delete(...);

(Class or instance method)
This method is used to remove a member cluster from a league.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-name' => $cluster_name >>

(Required for class method, Not Applicable for instance method)
Name of Member Cluster
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'reason' => $string >>

Reason for Removal
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league member delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( reason
                    )
            ),
        },
        api => 'league_member_delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueMember::delete',
    );

}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_member_delete',);
    $Log->exit() if $may_exit;
}

=head2 distribute

    NACL::C::LeagueMember->distribute(
                                command_interface => $ci,
	                            'cluster-name' => $cluster_name,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueMember_obj->distribute(...);

(Class or instance method)
This method is used to push the roster for a league to a member.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-name' => $cluster_name >>

(Required for class method, Not Applicable for instance method)
Name of Member Cluster
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league member distribute" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub distribute {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'league_member_distribute',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_distribute_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueMember::distribute',
    );

}

sub _distribute_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_member_distribute',);
    $Log->exit() if $may_exit;
}

=head2 keyphrase

    NACL::C::LeagueMember->keyphrase(
                                command_interface => $ci,
	                            'cluster-name' => $cluster_name,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueMember_obj->keyphrase(...);

(Class or instance method)
This method is used to replace the shared secret for a member.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-name' => $cluster_name >>

(Required for class method, Not Applicable for instance method)
Name of Member Cluster
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'new-keyphrase' => $string >>

Keyphrase for Shared Secret
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league member keyphrase" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub keyphrase {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-keyphrase
                    )
            ),
        },
        api => 'league_member_keyphrase',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_keyphrase_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueMember::keyphrase',
    );

}

sub _keyphrase_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_member_keyphrase',);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::LeagueMember->modify(
                                command_interface => $ci,
	                            'cluster-name' => $cluster_name,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueMember_obj->modify(...);

(Class or instance method)
This method is used to adjust per-member parameters.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-name' => $cluster_name >>

(Required for class method, Not Applicable for instance method)
Name of Member Cluster
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'admin-contact' => $string >>

Cluster Administrator's Contact Info
Applicable for CMode CLI

=item C<< 'member-cluster-ips' => [ $member-cluster-ip1, $member-cluster-ip2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'cluster-uuid' => $string >>

UUID of Member Cluster
Applicable for CMode CLI

=item C<< 'shared-secret' => $string >>

Shared Secret of Member Cluster
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league member modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'member-cluster-ips' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( admin-contact

                    cluster-uuid shared-secret
                    )
            ),
        },
        api => 'league_member_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueMember::modify',
    );

}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_member_modify',);
    $Log->exit() if $may_exit;
}

=head2 rename

    NACL::C::LeagueMember->rename(
                                command_interface => $ci,
	                            'cluster-name' => $cluster_name,
	                            'league-name' => $league_name,
                                ...);
	                             
								 or

    $LeagueMember_obj->rename(...);

(Class or instance method)
This method is used to rename a league member.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cluster-name' => $cluster_name >>

(Required for class method, Not Applicable for instance method)
Name of Member Cluster
Applicable for CMode CLI

=item C<< 'league-name' => $league_name >>

(Required for class method, Not Applicable for instance method)
League Name
Applicable for CMode CLI

=item C<< 'new-name' => $string >>

New Member Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "league member rename" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-name
                    )
            ),
        },
        api => 'league_member_rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LeagueMember::rename',
    );

}

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'league_member_rename',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'cluster-name' => {type => SCALAR},
        'league-name'  => {type => SCALAR},
    );
}

1;
