#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunAttribute Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunAttribute

=head1 DESCRIPTION

C<NACL::C::LunAttribute> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunAttribute.  A related class is
L<NACL::CS::LunAttribute|lib-NACL-CS-LunAttribute-pm>, which represents
the past state of an ONTAP LunAttribute.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 name

=head2 path

=cut

package NACL::C::LunAttribute;

use strict;
use warnings;

use NACL::CS::LunAttribute;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate_with SCALAR ARRAYREF SCALARREF UNDEF OBJECT HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use NACL::C::_Mixins::Lun qw(:all);

=head1 METHODS

=head2 new

  my $LunAttribute = NACL::C::LunAttribute->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'name' => $name,
                                   'path' => $path,
                                  );

Return a new LunAttribute component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => [
        {   '-default_ctor' => sub {
                if (defined $_[0]->path()) {
                    return $_[0]
                        ->_extract_lun_from_path(path => $_[0]->path());
                }
            },
        },
        'lun'
    ],

    scalar => [
        {   '-default_ctor' => sub {
                if (defined $_[0]->path()) {
                    return $_[0]
                        ->_extract_qtree_from_path(path => $_[0]->path());
                }
            },
        },
        'qtree'
    ],
    scalar => [
        {   '-default_ctor' => sub {
                if (defined $_[0]->path()) {

                    return $_[0]
                        ->_extract_volume_from_path(path => $_[0]->path());
                }
            },
        },
        'volume'
    ],

    scalar => 'vserver',
    scalar => 'name',
    scalar => 'path',
];

#
# This init function is just a workaround for S80 release,
# the component needs cleanup and the init needs to be
# removed in upcoming sprints
#
sub init {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    $self->_construct_path(params => \%args);
    $self->path($args{path});
    try {
        $self->SUPER::init(%args);
    }
    catch NATE::Exceptions::Argument with {
        my $exception = $_[0];
        my $text      = $exception->text();
        unless ($text =~ /Passing '-' as the value to the primary key.*qtree/)
        {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
    $Log->exit() if $may_exit;
}

=head2 find

    my $LunAttribute_obj  = NACL::C::LunAttribute->find(command_interface => $ci, ...);
    my @LunAttribute_objs = NACL::C::LunAttribute->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my LunAttribute_state = $LunAttribute_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunAttribute would return
a NACL::CS::LunAttribute object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

Invokes 'lun-unset-attribute' API for CMode ZAPI.

Invokes 'lun attribute unset' command for 7Mode CLI.

=cut

=head2 unset

    NACL::C::LunAttribute->unset(
                                command_interface => $ci,
                                'lun' => $lun,
                                'qtree' => $qtree,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                'name' => $name,
                                ...);

                 or

    NACL::C::LunAttribute->unset(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'name' => $name,
                                'path' => $path,
                                ...);

                 or


   $lun_attrib->unset();


(Class or Instance method)
This method is used to unset named attribute for a given LUN.
Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required)
LUN Name
Applicable for CMode CLI

=item C<< 'qtree' => $qtree >>

(Required)
Qtree Name
Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required)
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
Applicable for CMode CLI

=item C<< 'name' => $name >>

(Required)
*Named Attribute
Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI:

<lun-unset-attribute>
<name>$value</name>
</lun-unset-attribute>

=item C<< 'path' => $path >>

(Required)
LUN Path
Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI:

<lun-unset-attribute>
<path>$value</path>
</lun-unset-attribute>

7M CLI: lun-path

=item C<< 'force' => $string >>

Applicable for CMode CLI

7M CLI : -f (Programmatic name: "force")

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub unset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( lun volume qtree name force
                    )
            ),
        },
        cli_cmd => 'lun attribute unset',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_unset_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_unset_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _unset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( lun volume qtree vserver name path force extended_query
            )
    ];
    $pkg_or_obj->_base_backend_cmode_cli(@args,
            api  => 'lun_attribute_unset',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
}

sub _unset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $allowed_cmode_options = [
        qw( name path
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
            api    => 'lun_unset_attribute',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
}    #end sub _unset_cmode_zapi

sub _unset_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %cmode_args        = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'lun_attribute_unset',
        options => \%opts,
    );
    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%cmode_args,
        boolean_options => [qw(force)],
        is_7m_cli       => 0
    );
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%cmode_args,
        copy   => [qw (name)],
        map    => {'path' => 'lun-path'},
    );
    my $response = $apiset->lun_attribute_unset(%cmode_args);
    $Log->exit() if $may_exit;
}    #end sub _unset_7mode_cli

=head2 set

    my $lun_attrib = NACL::C::LunAttribute->set(
                                command_interface => $ci,
                                'lun' => $lun,
                                'qtree' => $qtree,
                                'volume' => $volume,
                                'vserver' => $vserver,
                                'name' => $name,
                                ...);

                  or

    my $lun_attrib = NACL::C::LunAttribute->set(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'name' => $name,
                                'path' => $path,
                                ...);


(Class method)
This method is used to set named attribute for a given LUN, and  return a new
lun attribute component object that refers to it.

Uses a CMode CLI/ZAPI APISet.

Invokes 'lun-set-attribute' API for CMode ZAPI.

Invokes 'lun attribute set' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'lun' => $lun >>

(Required)
LUN Name
Applicable for CMode CLI

=item C<< 'qtree' => $qtree >>

(Required)
Qtree Name
Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required)
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
Applicable for CMode CLI

=item C<< 'name' => $name >>

(Required)
*Named Attribute
Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI:

<lun-set-attribute>
<name>$value</name>
</lun-set-attribute>

=item C<< 'path' => $path >>

(Required)
LUN Path
Applicable for CMode CLI/ZAPI, 7Mode CLI

Maps to:

CM ZAPI:

<lun-set-attribute>
<path>$value</path>
</lun-set-attribute>

7M CLI: lun-path

=item C<< 'value' => $string >>

*Named Attribute Value
Applicable for CMode CLI, 7Mode CLI

Maps to:

CM ZAPI:

<lun-set-attribute>
<value>$value</value>
</lun-set-attribute>

=item C<< 'force' => $string >>

Applicable for CMode CLI, 7Mode CLI

7M CLI : -f (Programmatic name: "force")

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub set {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( volume value name force lun qtree
                    )
            ),
        },
    );

    my $vserver;
    $opts{vserver_operated_on} ||= \$vserver;

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_set_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_set_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_set_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
    );

    my $target = $pkg->_hash_copy(
        source => \%opts,
        copy   => [$pkg->get_primary_keys()]
    );

    $pkg->_process_options(options => $target);
    $Log->exit() if $may_exit;
    return $pkg->new(%$target, command_interface => $opts{command_interface});
}

sub _set_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $pkg->_base_backend_cmode_cli(
        @args,
        api  => 'lun_attribute_set',
    );

    $Log->exit() if $may_exit;
} ## end sub _set_cmode_cli

sub _set_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(value name path)],
        api  => 'lun-set-attribute'
    );
    $Log->exit() if $may_exit;
}

sub _set_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts              = $pkg_or_obj->_validate_backend_opts(@args);
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my %cmode_args        = $pkg_or_obj->get_api_options(
        apiset  => $apiset,
        api     => 'lun_attribute_set',
        options => \%opts,
    );
    $pkg_or_obj->_handle_boolean_options(
        api_options     => \%cmode_args,
        boolean_options => [qw(force)],
        is_7m_cli       => 0
    );
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%cmode_args,
        copy   => [qw(name value)],
        map    => {'path' => 'lun-path'},
    );
    my $response = $apiset->lun_attribute_set(%cmode_args);
    $Log->exit() if $may_exit;
}    #end sub _set_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | OBJECT | UNDEF, optional => 1},
        'name'    => {type => SCALAR},
        'path'    => {type => SCALAR},
    );
}

sub _super_common_validate_with {
    my ($pkg_or_obj, @opts) = @_;
    return $pkg_or_obj->SUPER::_common_validate_with(@opts);
}

1;
