# $Id$
#
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunBind Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com , raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunBind

=head1 DESCRIPTION

C<NACL::C::LunBind> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunBind.  A related class is
L<NACL::CS::LunBind|lib-NACL-CS-LunBind-pm>, which represents
the past state of an ONTAP LunBind.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 protocol_endpoint_path

=head2 vvol_path

=cut

package NACL::C::LunBind;

use strict;
use warnings;

use NACL::CS::LunBind;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunBind = NACL::C::LunBind->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'protocol-endpoint-path' => $protocol_endpoint_path,
                                   'vvol-path' => $vvol_path,
                                  );

Return a new LunBind component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'protocol_endpoint_path',
    scalar => 'vvol_path',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $LunBind_obj  = NACL::C::LunBind->find(command_interface => $ci, ...);
    my @LunBind_objs = NACL::C::LunBind->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-bind-get-iter" API for CMode ZAPI

=head2 state

    my LunBind_state = $LunBind_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunBind would return
a NACL::CS::LunBind object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 create

    my $LunBind_obj    = NACL::C::LunBind->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'protocol-endpoint-path' => $protocol_endpoint_path,
                                'vvol-path' => $vvol_path,

                                ...);

(Class method)
This method is used to bind a VVol LUN to a protocol endpoint.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-bind" API for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
*Vserver name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'protocol-endpoint-path' => $protocol_endpoint_path >>

(Required)
*Protocol Endpoint

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: protocol-endpoint-path

=item C<< 'vvol-path' => $vvol_path >>

(Required)
*VVol Path

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: vvol-path

=item C<< 'secondary-lun-id' => $string >>

*Secondary LUN Id

Applicable for CMode CLI

=item C<< 'protocol-endpoint-identifier' => $string >>

*PE Id

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'secondary-lun' => $string >>

Applicable for CMode CLI

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"lun bind create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vvol-path secondary-lun-id vserver protocol-endpoint-path
                    protocol-endpoint-identifier

                    secondary-lun
                    )

            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'lun bind create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunBind::create',
    );

    my $obj = $pkg->new(
        command_interface      => $opts{command_interface},
        vserver                => $vserver,
        protocol_endpoint_path => $opts{'protocol-endpoint-path'},
        vvol_path              => $opts{'vvol-path'},
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_bind_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
            api => 'lun_bind',
            copy   => [qw ( vvol-path protocol-endpoint-path )],
    );
    $Log->exit() if $may_exit;
}

=head2 destroy

    NACL::C::LunBind->destroy(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'protocol-endpoint-path' => $protocol_endpoint_path,
                                'vvol-path' => $vvol_path,
                                ...);

(Class method)
This method is used to unbind a VVol LUN from a protocol endpoint.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-unbind" API for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
*Vserver name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< 'protocol-endpoint-path' => $protocol_endpoint_path >>

(Required)
*Protocol Endpoint

Applicable for CMode CLI/ZAPI

Maps to :

CM ZAPI : protocol-endpoint-path

=item C<< 'vvol-path' => $vvol_path >>

(Required)
*VVol Path

Applicable for CMode CLI/ZAPI

Maps to :

CM ZAPI : vvol-path

=item C<< 'force' => $string >>

*If true, unbind the Vvol completely even if the current reference count is greater
than 1. The default is false.

Applicable for CMode CLI/ZAPI

Maps to :

CM ZAPI : force

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"lun bind destroy" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vvol-path vserver protocol-endpoint-path force
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'lun bind destroy',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_destroy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_destroy_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunBind::destroy',
        nacl_exceptions => [ qw(DoesNotExist) ]
    );

    $Log->exit() if $may_exit;
} ## end sub destroy

sub _destroy_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_bind_destroy',);
    $Log->exit() if $may_exit;
} ## end sub _destroy_cmode_cli

sub _destroy_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $allowed_cmode_options = [
        qw( vvol-path protocol-endpoint-path force
            )
    ];
    $pkg->_base_backend_cmode_zapi(@opts,
            api    => 'lun_unbind',
            copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'                => {type => SCALAR},
        'protocol-endpoint-path' => {type => SCALAR},
        'vvol-path'              => {type => SCALAR},
    );
}

1;
