
#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunInitiatorListMap Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunInitiatorListMap

=head1 DESCRIPTION

C<NACL::C::LunInitiatorListMap> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunInitiatorListMap.  A related class is
L<NACL::CS::LunInitiatorListMap|lib-NACL-CS-LunInitiatorListMap-pm>, which represents
the past state of an ONTAP LunInitiatorListMap.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 lun

=head2 igroup

=head2 initiator

=head2 qtree

=head2 volume

=head2 vserver

=cut

package NACL::C::LunInitiatorListMap;

use strict;
use warnings;

use NACL::CS::LunInitiatorListMap;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunInitiatorListMap = NACL::C::LunInitiatorListMap->new(command_interface=>$ci,
                                   lun =>$lun,
                                   igroup =>$igroup,
                                   initiator =>$initiator,
                                   qtree =>$qtree,
                                   volume =>$volume,
                                   vserver =>$vserver,
								  );

Return a new LunInitiatorListMap component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'lun',
    scalar => 'igroup',
    scalar => 'initiator',
    scalar => 'qtree',
    scalar => 'volume',
    scalar => 'vserver',
];

=head2 find

    my $LunInitiatorListMap  = NACL::C::LunInitiatorListMap->find(command_interface => $ci, ...);
	my @LunInitiatorListMaps = NACL::C::LunInitiatorListMap->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $LunInitiatorListMap_state = $NACL::C::LunInitiatorListMap->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunInitiatorListMap would return 
a NACL::CS::LunInitiatorListMap object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'initiator' => {type => SCALAR},
        'vserver'   => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
