# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunPersistentReservation Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunPersistentReservation

=head1 DESCRIPTION

C<NACL::C::LunPersistentReservation> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunPersistentReservation.  A related class is
L<NACL::CS::LunPersistentReservation|lib-NACL-CS-LunPersistentReservation-pm>, which represents
the past state of an ONTAP LunPersistentReservation.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 lun

=head2 qtree

=head2 path

=head2 offset

=head2 volume

=head2 vserver

=back

=back

=cut

package NACL::C::LunPersistentReservation;

use strict;
use warnings;

use NACL::CS::LunPersistentReservation;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $LunPersistentReservation = NACL::C::LunPersistentReservation->new(
                                   command_interface => $ci,
                                   'lun' => $lun,
                                   'qtree' => $qtree,
                                   'path' => $path,
                                   'offset' => $offset,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
								  );

Return a new LunPersistentReservation component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'lun',
    scalar => 'qtree',
    scalar => 'path',
    scalar => 'offset',
    scalar => 'volume',
    scalar => 'vserver',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $LunPersistentReservation_obj  = NACL::C::LunPersistentReservation->find(command_interface => $ci, ...);
	my @LunPersistentReservation_objs = NACL::C::LunPersistentReservation->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my LunPersistentReservation_state = $LunPersistentReservation_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunPersistentReservation would return 
a NACL::CS::LunPersistentReservation object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 clear

    NACL::C::LunPersistentReservation->clear(
                                command_interface => $ci,
	                            'lun' => $lun,
	                            'qtree' => $qtree,
	                            'path' => $path,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
                                ...);
   
(Class method)
This method is used to clear the SCSI-3 persistent reservation information for a given LUN.
Uses CMode CLI/ZAPI .
Invokes  lun-clear-persistent-reservation-info  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'lun' => $lun >>

(Required)
LUN Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'qtree' => $qtree >>

(Required)
Qtree Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'path' => $path >>

(Required)
LUN Path
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'volume' => $volume >>

(Required)
Volume Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "lun persistent reservation clear" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub clear {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lun qtree volume vserver path
                    )
            ),
        },
        cli_cmd             => 'lun persistent-reservation clear',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_clear_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_clear_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::LunPersistentReservation::clear',
    );

    $Log->exit() if $may_exit;
}

sub _clear_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'lun_persistent_reservation_clear',);
    $Log->exit() if $may_exit;
}

sub _clear_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(volume path lun qtree)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'lun-clear-persistent-reservation-info',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(lun qtree path offset volume vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'lun'     => {type => SCALAR},
        'qtree'   => {type => SCALAR},
        'volume'  => {type => SCALAR},
        'vserver' => {type => SCALAR},
        'path'    => {type => SCALAR},
        'offset'  => {type => SCALAR, optional => 1},
    );
}

1;
