#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunTransition Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::LunTransition

=head1 DESCRIPTION

C<NACL::C::LunTransition> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP LunTransition.  A related class is
L<NACL::CS::LunTransition|lib-NACL-CS-LunTransition-pm>, which represents
the past state of an ONTAP LunTransition.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 volume

=head2 vserver

=cut

package NACL::C::LunTransition;

use strict;
use warnings;

use NACL::CS::LunTransition;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $LunTransition = NACL::C::LunTransition->new(
                                   command_interface => $ci,
                                   'volume' => $volume,
                                   'vserver' => $vserver,
								  );

Return a new LunTransition component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'vserver',
];

=head2 find

    my $LunTransition_obj  = NACL::C::LunTransition->find(command_interface => $ci, ...);
	my @LunTransition_objs = NACL::C::LunTransition->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my LunTransition_state = $LunTransition_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::LunTransition would return 
a NACL::CS::LunTransition object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 start

    my $LunTransition_obj	= NACL::C::LunTransition->start(
                                command_interface => $ci,
	                            'volume' => $volume,
	                            'vserver' => $vserver,
                                ...);
   
(Class method)
This method is used to start LUN Transition Processing.

Uses a CMode CLI/ZAPI APISet.

Invokes "lun-transition-start" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $volume >>

(Required)
Volume Name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : volume

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< '*' >>

This method takes all other options supported by "lun transition start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'lun_transition_start',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_start_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::LunTransition::start',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        volume            => $opts{'volume'},
        vserver           => $opts{'vserver'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'lun_transition_start',);
    $Log->exit() if $may_exit;
}

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'lun-transition-start',
        copy            => [qw( volume vserver )],
        frontend_method => 'start'
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'volume'  => {type => SCALAR},
        'vserver' => {type => SCALAR},
    );
}

1;
