#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MemoryUsage Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::MemoryUsage

=head1 DESCRIPTION

C<NACL::C::MemoryUsage> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
Unlike other components this package does not model any element/service
but is present to provide component-level support for functionality.
Since it does not model any element/service there is no ComponentState
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::MemoryUsage->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::MemoryUsage;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $MemoryUsage = NACL::C::MemoryUsage->new(
                                   command_interface => $ci,
                                  );

Return a new MemoryUsage component object with the given attributes.

=cut

use Class::MethodMaker [];

=head2 ut_malloc_perf

    NACL::C::MemoryUsage->ut_malloc_perf(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to simple malloc test performance test.
Uses CMode CLI/ZAPI.

Invokes "memoryperf-run" API for CMode ZAPI.

Returns hashref which contains the values of size, count and latency.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'count' => $string >>

Number of allocation and free operations to run
Applicable for CMode CLI/ZAPI

=item C<< 'node' => $string >>


The node upon which to run the test possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'size' => $string >>

Size of blocks to allocate and free
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "memory usage ut malloc perf" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ut_malloc_perf {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( count node size
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'memory usage ut-malloc-perf',
    );
    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ut_malloc_perf_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_ut_malloc_perf_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::MemoryUsage::ut_malloc_perf',
    );
    $Log->exit() if $may_exit;
}

sub _ut_malloc_perf_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = @args;
    my $response;
    $pkg->_base_backend_cmode_cli(
        @args,
        api                   => 'memory_usage_ut_malloc_perf',
        'apiset_response_obj' => \$response,
    );
    my $parsed = $response->get_processed_output();
    my ($count, $size, $latency, $hash);
    if ($parsed
        =~ /Time for (\d+)\s+calls of size\s+(\d+)\s+bytes is\s+(\d+)\s+\w+/)
    {
        $count   = $1;
        $size    = $2;
        $latency = $3;
        $hash    = {
            size    => $size,
            count   => $count,
            latency => $latency
        };
    }
    $Log->exit() if $may_exit;
    return $hash;
}

sub _ut_malloc_perf_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = @args;
    my $response;
    $pkg->_base_backend_cmode_zapi(
        @args,
        copy                  => [qw(count size latency )],
        api                   => 'memoryperf-run',
        'apiset_response_obj' => \$response,
    );
    my $parse = $response->get_parsed_output();
    my $hash  = {
        size    => $opts{size},
        count   => $opts{count},
        latency => $$parse[0]->{'latency'}
        };
        $Log->exit() if $may_exit;
    return $hash;
}

=head2 ut_reset_stats

    NACL::C::MemoryUsage->ut_reset_stats(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to reset the unit test statistics.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $string >>


The node that owns the statistics. possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "memory usage ut reset stats" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub ut_reset_stats {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'memory usage ut-reset-stats',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_ut_reset_stats_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::MemoryUsage::ut_reset_stats',
    );

    $Log->exit() if $may_exit;
}

sub _ut_reset_stats_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'memory_usage_ut_reset_stats',);
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
