# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Metrocluster Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::Metrocluster

=head1 DESCRIPTION

C<NACL::C::Metrocluster> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP Metrocluster.  A related class is
L<NACL::CS::Metrocluster|lib-NACL-CS-Metrocluster-pm>, which represents
the past state of an ONTAP Metrocluster.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::Metrocluster;

use strict;
use warnings;

use NACL::CS::Metrocluster;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $Metrocluster = NACL::C::Metrocluster->new(
                                   command_interface => $ci,
								  );

Return a new Metrocluster component object with the given attributes.

=cut

use Class::MethodMaker [];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 configure

    NACL::C::Metrocluster->configure(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to configure partners and start DR mirroring for the node and its DR group.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-configure for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'refresh' => $string >>

Refresh Configuration possible value(s) are, true,false

Applicable for CMode CLI/ZAPI

=item C<< 'node-name' => $string >>

Node to Configure possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster configure" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'override-lost-config-updates-veto' => $string >>

Override Lost Configuration Updates Veto
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub configure {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( refresh node-name
                    )
            ),
        },
        api => 'metrocluster_configure',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_configure_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_configure_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::configure',
    );

    $Log->exit() if $may_exit;
}

sub _configure_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_configure',);
    $Log->exit() if $may_exit;
}

sub _configure_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(refresh node-name)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-configure',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 disable

    NACL::C::Metrocluster->disable(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to disable mirroring between DR partners.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $string >>

Node to Disable

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster disable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node
                    )
            ),
        },
        api => 'metrocluster_disable',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::disable',
    );

    $Log->exit() if $may_exit;
}

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_disable',);
    $Log->exit() if $may_exit;
}

=head2 heal

    NACL::C::Metrocluster->heal(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to heal DR data aggregates and DR root aggregates.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-heal for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'override-vetoes' => $string >>

Override All Soft Vetoes

Applicable for CMode CLI/ZAPI

=item C<< 'phase' => $string >>


MetroCluster Healing Phase possible value(s) are, aggregates,root-aggregates

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : healing-phase

=item C<< 'force-if-partial-switchover' => $string >>

Allows Heal Operation If Node Is Partially Switched over

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster heal" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub heal {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( override-vetoes phase force-if-partial-switchover
                    )
            ),
        },
        api => 'metrocluster_heal',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_heal_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_heal_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::heal',
    );

    $Log->exit() if $may_exit;
}

sub _heal_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_heal',);
    $Log->exit() if $may_exit;
}

sub _heal_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(override-vetoes)];
    my $map = {'phase' => 'healing-phase',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-heal',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

=head2 is_configured

    NACL::C::Metrocluster->is_configured(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to check if MetroCluster is configured.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-is-configured for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'is-mcc-config' => $string >>

Is MetroCluster Configured possible value(s) are, true,false

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster is configured" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub is_configured {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( is-mcc-config
                    )
            ),
        },
        api => 'metrocluster_is_configured',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_is_configured_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_is_configured_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::is_configured',
    );

    $Log->exit() if $may_exit;
}

sub _is_configured_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_is_configured',);
    $Log->exit() if $may_exit;
}

sub _is_configured_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(is-mcc-config)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-is-configured',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::Metrocluster->modify(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to modify MetroCluster configuration.
Uses CMode CLI .


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node-object-limit' => $string >>

Node Object Limit Option possible value(s) are, on,off

Applicable for CMode CLI


=item C<< 'node-name' => $string >>

Node to Change the Option On possible value(s) are, <nodename>,local

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node-object-limit node-name
                    )
            ),
        },
        api => 'metrocluster_modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_modify',);
    $Log->exit() if $may_exit;
}


=head2 enable

    NACL::C::Metrocluster->enable(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);

(Class method)
This method is used to enable mirroring between DR partners.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'refresh' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node-name' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid refresh node

                    node-name
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid refresh node

            node-name
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_enable',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_enable(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_cli

=head2 switchover

    NACL::C::Metrocluster->switchover(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to switch over storage and client access.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-switchover for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'override-vetoes' => $string >>

Override All Soft Vetoes

Applicable for CMode CLI/ZAPI

=item C<< 'forced-on-disaster' => $string >>

Force Switchover on Disaster

Applicable for CMode CLI/ZAPI

=item C<< 'force-nvfail-all' => $string >>

Sets in-nvfailed-state on All Volumes

Applicable for CMode CLI/ZAPI

=item C<< 'retry-failed-nodes' => [ $retry-failed-node1, $retry-failed-node2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)


Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster switchover" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub switchover {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'retry-failed-nodes' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( override-vetoes forced-on-disaster force-nvfail-all
                    )
            ),
        },
        api => 'metrocluster_switchover',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_switchover_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_switchover_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::switchover',
    );

    $Log->exit() if $may_exit;
}

sub _switchover_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_switchover',);
    $Log->exit() if $may_exit;
}

sub _switchover_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy
        = [
        qw(override-vetoes forced-on-disaster retry-failed-nodes force-nvfail-all)
        ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-switchover',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 switchback

    NACL::C::Metrocluster->switchback(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to switchback storage and client access.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-switchback for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'override-vetoes' => $string >>

Override All Soft Vetoes

Applicable for CMode CLI/ZAPI

=item C<< 'override-cluster-stream-resync-errors' => $string >>

Override Cluster Stream Resync Errors

Applicable for CMode CLI/ZAPI

=item C<< 'override-vserver-stream-resync-errors' => $string >>

Override Vserver Stream Resync Errors

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster switchback" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub switchback {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( override-vetoes f override-cluster-stream-resync-errors
                    override-vserver-stream-resync-errors
                    )
            ),
        },
        api => 'metrocluster_switchback',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_switchback_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_switchback_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::switchback',
    );

    $Log->exit() if $may_exit;
}

sub _switchback_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_switchback',);
    $Log->exit() if $may_exit;
}

sub _switchback_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [
        qw(override-vetoes override-cluster-stream-resync-errors
            override-vserver-stream-resync-errors)
    ];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-switchback',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 unconfigure

    NACL::C::Metrocluster->unconfigure(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to unconfigure MetroCluster.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-unconfigure for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'dr-group-id' => $string >>

Identifier of the DR Group to Unconfigure

Applicable for CMode CLI/ZAPI

=item C<< 'force' => $string >>


Force Unconfigure to be Cluster Local Only possible value(s) are, true,false

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "metrocluster unconfigure" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unconfigure {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( dr-group-id force
                    )
            ),
        },
        api => 'metrocluster_unconfigure',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unconfigure_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_unconfigure_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::Metrocluster::unconfigure',
    );

    $Log->exit() if $may_exit;
}

sub _unconfigure_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'metrocluster_unconfigure',);
    $Log->exit() if $may_exit;
}

sub _unconfigure_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(dr-group-id force)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-unconfigure',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 wipeconfig

    NACL::C::Metrocluster->wipeconfig(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);

(Class method)
This method is used to remove the specified node and its HA, DR, and DR-auxiliary partners from the MetroCluster configuration.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub wipeconfig {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_wipeconfig_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub wipeconfig

sub _wipeconfig_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_wipeconfig',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_wipeconfig(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _wipeconfig_cmode_cli

=head2 assign

    NACL::C::Metrocluster->assign(
                                command_interface => $ci,wipeconfig
	                            'node' => $node,
                                ...);

(Class method)
This method is used to assign a node to a location.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'location' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub assign {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid location node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_assign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub assign

sub _assign_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid location node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_assign',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_assign(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _assign_cmode_cli

=head2 check

    NACL::C::Metrocluster->check(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to check the MetroCluster setup.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'verbose' => $string >>

Applicable for CMode CLI.

=item C<< 'level' => $string >>

Applicable for CMode CLI.

=item C<< 'break-on-failure' => $string >>

Applicable for CMode CLI.

=item C<< 'precondition-for' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub check {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( verbose level break-on-failure precondition-for
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_check_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub check

sub _check_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( verbose level break-on-failure precondition-for
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_check',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_check(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _check_cmode_cli

=head2 unassign

    NACL::C::Metrocluster->unassign(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);

(Class method)
This method is used to unassign a node from its assigned location.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'node-uuid' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub unassign {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( node-uuid node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_unassign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub unassign

sub _unassign_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( node-uuid node
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'metrocluster_unassign',
        api_opts => \%cmode_args
    );

    $apiset->metrocluster_unassign(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _unassign_cmode_cli

sub get_one_state_attribute {
    $Log->enter() if $may_enter;
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"get_one_state_attribute\" to get the attribute";
    $Log->exit() if $may_exit;
    NATE::BaseException->throw($error);
}

sub state {
    $Log->enter() if $may_enter;
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"state\"";
    $Log->exit() if $may_exit;
    NATE::BaseException->throw($error);
}

sub find {
    $Log->enter() if $may_enter;
    my $error
        = "Since NACL::C::Metrocluster doesn't have any primary key attributes, "
        . "use \"fetch\" instead of \"find\"";
    $Log->exit() if $may_exit;
    NATE::BaseException->throw($error);
}


__PACKAGE__->_load_event_class();

1;
