# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterInterconnect Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::MetroclusterInterconnect

=head1 DESCRIPTION

C<NACL::C::MetroclusterInterconnect> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP MetroclusterInterconnect.  A related class is
L<NACL::CS::MetroclusterInterconnect|lib-NACL-CS-MetroclusterInterconnect-pm>, which represents
the past state of an ONTAP MetroclusterInterconnect.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=head2 partner_type

=head2 adapter_index

=head2 index

=cut

package NACL::C::MetroclusterInterconnect;

use strict;
use warnings;

use NACL::CS::MetroclusterInterconnect;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use NACL::C::MetroclusterInterconnectMirror;

=head1 METHODS

=head2 new

  my $MetroclusterInterconnect = NACL::C::MetroclusterInterconnect->new(
                                   command_interface => $ci,
                                   'node' => $node,
                                   'partner-type' => $partner_type,
                                   'adapter-index' => $adapter_index,
                                   'index' => $index,
								  );

Return a new MetroclusterInterconnect component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'partner_type',
    scalar => 'adapter_index',
    scalar => 'index',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $MetroclusterInterconnect_obj  = NACL::C::MetroclusterInterconnect->find(command_interface => $ci, ...);
	my @MetroclusterInterconnect_objs = NACL::C::MetroclusterInterconnect->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my MetroclusterInterconnect_state = $MetroclusterInterconnect_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::MetroclusterInterconnect would return 
a NACL::CS::MetroclusterInterconnect object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::MetroclusterInterconnect->modify(
                                command_interface => $ci,
	                            'node' => $node,
	                            'partner-type' => $partner_type,
                                ...);
   
(Class method)
This method is used to modify MetroCluster Interconnect Options.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-interconnect-modify for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)
Node name possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : node-name

=item C<< 'partner-type' => $partner_type >>

(Required)
Partner Type possible value(s) are, HA,DR,AUX

Applicable for CMode CLI/ZAPI

=item C<< 'mirror-status' => $string >>

Mirror Status

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : mirroring-status

=item C<< '*' >>

This method takes all other options supported by "metrocluster interconnect modify" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    $pkg->_frontend_log_debug_opts(%args);
    if ($args{command_interface}->has_uichange( uichange => "mcc-interconnect-commands-review",
                                                throw_exception => 0)) {

        $Log->warn("NACL::C::MetroclusterInterconnect component is replaced by"
                   ." NACL::C::MetroclusterInterconnectMirror.\n"
                   ."Note also that the 'mirror-status' option was renamed to 'mirror-oper-status'\n"
                   ." Please update the script"
        );

        if (defined $args{'mirror-status'}) {
            $args{'mirror-oper-status'} = delete $args{'mirror-status'};
        }

        $Log->exit() if $may_exit;
        return NACL::C::MetroclusterInterconnectMirror->modify(%args);
    }

    my %opts = $pkg->_common_validate_with(
        params          => [%args],
        additional_spec => {
            _optional_scalars(
                qw( mirror-status partner-type node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'metrocluster interconnect modify',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::MetroclusterInterconnect::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_interconnect_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(partner-type)];
    my $map  = {
        'mirror-status' => 'mirroring-status',
        'node'          => 'node-name',
    };
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'metrocluster-interconnect-modify',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'node'          => {type => SCALAR},
        'partner-type'  => {type => SCALAR},
        'adapter-index' => {type => SCALAR},
        'index'         => {type => SCALAR},
    );
}

1;
