# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterVserver Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::MetroclusterVserver

=head1 DESCRIPTION

C<NACL::C::MetroclusterVserver> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP MetroclusterVserver.  A related class is
L<NACL::CS::MetroclusterVserver|lib-NACL-CS-MetroclusterVserver-pm>, which represents
the past state of an ONTAP MetroclusterVserver.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 cluster

=head2 vserver

=cut

package NACL::C::MetroclusterVserver;

use strict;
use warnings;

use NACL::CS::MetroclusterVserver;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $MetroclusterVserver = NACL::C::MetroclusterVserver->new(
                                   command_interface => $ci,
                                   'cluster' => $cluster,
                                   'vserver' => $vserver,
								  );

Return a new MetroclusterVserver component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'cluster',
    scalar => 'vserver',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $MetroclusterVserver_obj  = NACL::C::MetroclusterVserver->find(command_interface => $ci, ...);
	my @MetroclusterVserver_objs = NACL::C::MetroclusterVserver->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my MetroclusterVserver_state = $MetroclusterVserver_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::MetroclusterVserver would return 
a NACL::CS::MetroclusterVserver object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 recover_from_partial_switchback

    NACL::C::MetroclusterVserver->recover_from_partial_switchback(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to recover vservers from partial switchback.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'skip-vserver-start-stop' => $string >>

Skip Vserver start stop possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'skip-vserver-config-restore' => $string >>

Skip Vserver configuraion restore possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster vserver recover from partial switchback" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recover_from_partial_switchback {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( skip-vserver-start-stop skip-vserver-config-restore
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'metrocluster vserver recover-from-partial-switchback',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_recover_from_partial_switchback_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::MetroclusterVserver::recover_from_partial_switchback',
    );

    $Log->exit() if $may_exit;
}

sub _recover_from_partial_switchback_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_vserver_recover_from_partial_switchback',);
    $Log->exit() if $may_exit;
}

=head2 recover_from_partial_switchover

    NACL::C::MetroclusterVserver->recover_from_partial_switchover(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to recover vservers from partial switchover.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'skip-vserver-start' => $string >>

Skip Vserver start possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'skip-vserver-unlock' => $string >>

Skip Vserver unlock possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "metrocluster vserver recover from partial switchover" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recover_from_partial_switchover {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( skip-vserver-start skip-vserver-unlock
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'metrocluster vserver recover-from-partial-switchover',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_recover_from_partial_switchover_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::MetroclusterVserver::recover_from_partial_switchover',
    );

    $Log->exit() if $may_exit;
}

sub _recover_from_partial_switchover_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_vserver_recover_from_partial_switchover',);
    $Log->exit() if $may_exit;
}

=head2 resync

    NACL::C::MetroclusterVserver->resync(
                                command_interface => $ci,
	                            'cluster' => $cluster,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $MetroclusterVserver_obj->resync(...);

(Class or instance method)
This method is used to resynchronize Vserver with it's partner Vserver.
Uses CMode CLI/ZAPI .
Invokes  metrocluster-vserver-resync  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'cluster' => $cluster >>

(Required for class method, Not Applicable for instance method)
Cluster Name
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: cluster-name

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: source-vserver

=item C<< '*' >>

This method takes all other options supported by "metrocluster vserver resync" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub resync {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'metrocluster vserver resync',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_resync_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_resync_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::MetroclusterVserver::resync',
    );

    $Log->exit() if $may_exit;
}

sub _resync_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'metrocluster_vserver_resync',);
    $Log->exit() if $may_exit;
}

sub _resync_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $map = {
        'cluster' => 'cluster-name',
        'vserver' => 'source-vserver',
    };
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api => 'metrocluster-vserver-resync',
        map => $map,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts,
        _primary_keys => [qw(cluster vserver)],);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'cluster' => {type => SCALAR},
        'vserver' => {type => SCALAR},
    );
}

1;
