# $Id$
#
# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary Ndmpcopy Component Module
## @author dl-dpg-nb-automation, dl-nacl-dev;
## @status shared
## @pod here

=head1 NAME

NACL::C::Ndmpcopy

=head1 DESCRIPTION

C<NACL::C::Ndmpcopy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.
It represents ndmpcopy operations on a filer.

=cut

package NACL::C::Ndmpcopy;

use strict;
use warnings;
use NACL::C::Options;
use NACL::ComponentUtils qw (_optional_scalars);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 ndmpcopy

   NACL::C::Ndmpcopy->ndmpcopy(
                   command_interface => $ci,
                   source            => $source,
                   destination       => $destination
                  ...);

(Class method)
This method is used to transfers data between filers/vfilers using the
Network Data Management Protocol (NDMP)

Uses a 7Mode/Nodescope CLI APISet.

Invokes "ndmpcopy" command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "source" => $source >>

(Required for class method) Source path

Applicable for 7Mode CLI.

Maps to: 

7M CLI: source (programmatic name: source)

=item C<< "destination" => $destination >>

(Required for class method) Destination path

Applicable for 7Mode CLI.

Maps to: 

7M CLI: Destination path (programmatic name: destination)

=item C<< "method-timeout" => $scalar >>

(Optional)The new timeout value.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub ndmpcopy {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars qw(
            source-filer-authentication destination-filer-authentication
            source-filer-authentication-type increment-forever
            incremental-level debug-mode force help-message
            source-control-connection-mode data-connection-mode
            accept-password-interactively method-timeout
            exclude-files-dirs-from-backup-path source
            destination-filer-authentication-type destination
            destination-control-connection-mode
            ),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {
                method    => '_ndmpcopy_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _ndmpcopy_7mode_cli {
      $Log->enter() if $may_enter;
      my $pkg = shift;
      my %opts       = validate_with(
          params      => \@_,
          spec        => $pkg->_backend_validate_spec(),
          allow_extra => 1
      );

    # enabling options at d blade for Ndmpcopy
    # Refer 505668 for more informaiton.
    try {
        NACL::C::Options->option(
                command_interface => $opts{command_interface},
                'option-name'     => 'nodescope.reenabledcmds',
                'option-value'    => 'ndmpcopy',
            );
    } catch NACL::APISet::Exceptions::InvalidParamValueException with {};
      delete $opts{command_interface};
      my $apiset = delete $opts{apiset};

      $pkg->_hash_move(
          source => \%opts,
          target => \%opts,
          map => { 'method-timeout' => 'connectrec-timeout' }
      );

      my $response = $apiset->ndmpcopy(%opts);
      $Log->exit() if $may_exit;
      return $response;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
      return ();
}

1;
