# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkArpActiveEntry Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkArpActiveEntry

=head1 DESCRIPTION

C<NACL::C::NetworkArpActiveEntry> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkArpActiveEntry.  A related class is
L<NACL::CS::NetworkArpActiveEntry|lib-NACL-CS-NetworkArpActiveEntry-pm>, which represents
the past state of an ONTAP NetworkArpActiveEntry.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 subnet_group

=head2 vserver

=head2 node

=head2 remotehost

=head2 port

=cut

package NACL::C::NetworkArpActiveEntry;

use strict;
use warnings;

use NACL::CS::NetworkArpActiveEntry;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $NetworkArpActiveEntry = NACL::C::NetworkArpActiveEntry->new(
                                   command_interface => $ci,
                                   'subnet-group' => $subnet_group,
                                   'vserver' => $vserver,
                                   'node' => $node,
                                   'remotehost' => $remotehost,
                                   'port' => $port,
                                  );

Return a new NetworkArpActiveEntry component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'subnet_group',
    scalar => 'vserver',
    scalar => 'node_name',
    scalar => 'remotehost',
    scalar => 'port',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $NetworkArpActiveEntry_obj  = NACL::C::NetworkArpActiveEntry->find(command_interface => $ci, ...);
    my @NetworkArpActiveEntry_objs = NACL::C::NetworkArpActiveEntry->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my NetworkArpActiveEntry_state = $NetworkArpActiveEntry_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkArpActiveEntry would return
a NACL::CS::NetworkArpActiveEntry object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 delete

    NACL::C::NetworkArpActiveEntry->delete(
                                command_interface => $ci,
                                'subnet-group' => $subnet_group,
                                'vserver' => $vserver,
                                'remotehost' => $remotehost,
                                'port' => $port,
                                ...);

(Class method)
This method is used to delete active ARP entry.
Uses CMode CLI/ZAPI .
Invokes  net-arp-active-entry-destroy for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'subnet-group' => $subnet_group >>

(Required)
Subnet Group Name possible value(s) are, <subnet>,default

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: subnet-group-name

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: vserver-name

=item C<< 'remotehost' => $remotehost >>

(Required)
Remote IP Address

Applicable for CMode CLI/ZAPI

=item C<< 'port' => $port >>

(Required)
Port

Applicable for CMode CLI/ZAPI

=item C<< 'node-name' => $string >>

Node possible value(s) are, <nodename>,local

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network arp active entry delete" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( subnet-group node-name vserver remotehost port node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'network arp active-entry delete',
    );
    $opts{'node'} = delete $opts{'node-name'} if($opts{'node-name'});

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkArpActiveEntry::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_arp_active_entry_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $copy = [qw( port remotehost )];
    my $map  = {
        'subnet-group' => 'subnet-group-name',
        'vserver'      => 'vserver-name',
        'node'         => 'node-name'
    };
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-arp-active-entry-destroy',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'subnet-group' => {type => SCALAR},
        'vserver'      => {type => SCALAR},
        'node-name'    => {type => SCALAR},
        'remotehost'   => {type => SCALAR},
        'port'         => {type => SCALAR},
    );
}

1;
