
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkConnectionsActive Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkConnectionsActive

=head1 DESCRIPTION

C<NACL::C::NetworkConnectionsActive> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkConnectionsActive.  A related class is
L<NACL::CS::NetworkConnectionsActive|lib-NACL-CS-NetworkConnectionsActive-pm>, which represents
the past state of an ONTAP NetworkConnectionsActive.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 cid

=head2 server

=head2 node

=cut

package NACL::C::NetworkConnectionsActive;

use strict;
use warnings;

use NACL::CS::NetworkConnectionsActive;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkConnectionsActive = NACL::C::NetworkConnectionsActive->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'cid' => $cid,
                                   'server' => $server,
                                   'node' => $node,
								  );

Return a new NetworkConnectionsActive component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'cid',
    scalar => 'server',
    scalar => 'node',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $NetworkConnectionsActive_obj  = NACL::C::NetworkConnectionsActive->find(command_interface => $ci, ...);
	my @NetworkConnectionsActive_objs = NACL::C::NetworkConnectionsActive->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkConnectionsActive_state = $NetworkConnectionsActive_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkConnectionsActive would return 
a NACL::CS::NetworkConnectionsActive object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 delete

    NACL::C::NetworkConnectionsActive->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
	                            'cid' => $cid,
	                            'node' => $node,
                                ...);

(Class method)
This method is used to delete an active connection in this cluster.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< 'cid' => $cid >>

(Required)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"network connections active delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver cid node
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'network_connections_active_delete',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkConnectionsActive::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg->_base_backend_cmode_cli(%opts,
            api => 'network_connections_active_delete',
            vserver_operated_on => \$vserver,
            );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 show_lifs

    my $show_lifs_obj = NACL::C::NetworkConnectionsActive->show_lifs(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_lifs_objs = NACL::C::NetworkConnectionsActive->show_lifs(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_lifs_obj = $NetworkConnectionsActive_obj->show_lifs(); 

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkConnectionsActiveShowLifs|lib-NACL-CS-NetworkConnectionsActiveShowLifs-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub show_lifs {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::NetworkConnectionsActiveShowLifs;
    my @state_objs = NACL::CS::NetworkConnectionsActiveShowLifs->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_lifs

=head2 show_protocols

    my $show_protocols_obj = NACL::C::NetworkConnectionsActive->show_protocols(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_protocols_objs = NACL::C::NetworkConnectionsActive->show_protocols(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_protocols_obj = $NetworkConnectionsActive_obj->show_protocols(); 

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkConnectionsActiveShowProtocols|lib-NACL-CS-NetworkConnectionsActiveShowProtocols-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub show_protocols {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::NetworkConnectionsActiveShowProtocols;
    my @state_objs
        = NACL::CS::NetworkConnectionsActiveShowProtocols->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_protocols

=head2 show_services

    my $show_services_obj = NACL::C::NetworkConnectionsActive->show_services(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_services_objs = NACL::C::NetworkConnectionsActive->show_services(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_services_obj = $NetworkConnectionsActive_obj->show_services(); 

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkConnectionsActiveShowServices|lib-NACL-CS-NetworkConnectionsActiveShowServices-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub show_services {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::NetworkConnectionsActiveShowServices;
    my @state_objs
        = NACL::CS::NetworkConnectionsActiveShowServices->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_services

=head2 show_clients

    my $show_clients_obj = NACL::C::NetworkConnectionsActive->show_clients(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_clients_objs = NACL::C::NetworkConnectionsActive->show_clients(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_clients_obj = $NetworkConnectionsActive_obj->show_clients(); 

(Class or Instance method)
This method returns objects of the type L<NACL::CS::NetworkConnectionsActiveShowClients|lib-NACL-CS-NetworkConnectionsActiveShowClients-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub show_clients {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::NetworkConnectionsActiveShowClients;
    my @state_objs
        = NACL::CS::NetworkConnectionsActiveShowClients->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub show_clients


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR},
        'cid'     => {type => SCALAR},
        'server'  => {type => SCALAR},
        'node'    => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
