
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfaceCdb Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkInterfaceCdb

=head1 DESCRIPTION

C<NACL::C::NetworkInterfaceCdb> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkInterfaceCdb.  A related class is
L<NACL::CS::NetworkInterfaceCdb|lib-NACL-CS-NetworkInterfaceCdb-pm>, which represents
the past state of an ONTAP NetworkInterfaceCdb.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vif_id

=head2 lif_id

=head2 node

=cut

package NACL::C::NetworkInterfaceCdb;

use strict;
use warnings;

use NACL::CS::NetworkInterfaceCdb;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with UNDEF SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $NetworkInterfaceCdb = NACL::C::NetworkInterfaceCdb->new(command_interface=>$ci,
                                   vif_id =>$vif_id,
                                   lif_id =>$lif_id,
                                   node =>$node,
								  );

Return a new NetworkInterfaceCdb component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vif_id',
    scalar => 'lif_id',
    scalar => 'node',
];

=head2 find

    my $NetworkInterfaceCdb  = NACL::C::NetworkInterfaceCdb->find(command_interface => $ci, ...);
	my @NetworkInterfaceCdbs = NACL::C::NetworkInterfaceCdb->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $NetworkInterfaceCdb_state = $NACL::C::NetworkInterfaceCdb->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkInterfaceCdb would return 
a NACL::CS::NetworkInterfaceCdb object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 create

    my $NetworkInterfaceCdb_object	= NACL::C::NetworkInterfaceCdb->create(
                                command_interface => $ci,
	                             vif_id => $vif_id,
	                             lif_id => $lif_id,
	                             node => $node,
                                  ...);

(Class method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vif-id" => $vif_id >>

(Required)

Applicable for CMode CLI.

=item C<< "lif-id" => $lif_id >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "has-valid-id" => $string >>

Applicable for CMode CLI.

=item C<< "curr-node" => $string >>

Applicable for CMode CLI.

=item C<< "failover-group" => $string >>

Applicable for CMode CLI.

=item C<< "home-port" => $string >>

Applicable for CMode CLI.

=item C<< "failover-policy" => $string >>

Applicable for CMode CLI.

=item C<< "firewall-policy" => $string >>

Applicable for CMode CLI.

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "netmask" => $string >>

Applicable for CMode CLI.

=item C<< "lif-name" => $string >>

Applicable for CMode CLI.

=item C<< "home-node" => $string >>

Applicable for CMode CLI.

=item C<< "status-admin" => $string >>

Applicable for CMode CLI.

=item C<< "use-failover-group" => $string >>

Applicable for CMode CLI.

=item C<< "vif-name" => $string >>

Applicable for CMode CLI.

=item C<< "curr-port" => $string >>

Applicable for CMode CLI.

=item C<< "role" => $string >>

Applicable for CMode CLI.

=item C<< "routing-group" => $string >>

=item C<< 'auto-configuration' => $string >>

Use Address Auto-configuration
Applicable for CMode CLI

=item C<< 'preferred-lifetime' => $string >>

Preferred Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'valid-lifetime' => $string >>

Valid Lifetime (autoconf)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network interface cdb create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( has-valid-id curr-node failover-group home-port failover-policy
                    firewall-policy address netmask lif-name home-node status-admin
                    use-failover-group vif-name curr-port role routing-group

                    auto-configuration

                    preferred-lifetime valid-lifetime
                    )
            ),
        },
        cli_cmd => 'network interface cdb create',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::NetworkInterfaceCdb::create',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts, api => 'network_interface_cdb_create');
    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $opts{'command_interface'},
        vif_id            => $opts{'vif-id'},
        lif_id            => $opts{'lif-id'},
        node              => $opts{'node'},
    );
} ## end sub _create_cmode_cli

=head2 modify

   NACL::C::NetworkInterfaceCdb->modify(
                                command_interface => $ci,
	                             vif_id => $vif_id,
	                             lif_id => $lif_id,
	                             node => $node,
                                  ...);
	                             
								 or

    $NetworkInterfaceCdb_object->modify(...);

(Class or instance method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vif-id" => $vif_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "lif-id" => $lif_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "has-valid-id" => $string >>

Applicable for CMode CLI.

=item C<< "curr-node" => $string >>

Applicable for CMode CLI.

=item C<< "failover-group" => $string >>

Applicable for CMode CLI.

=item C<< "home-port" => $string >>

Applicable for CMode CLI.

=item C<< "failover-policy" => $string >>

Applicable for CMode CLI.

=item C<< "firewall-policy" => $string >>

Applicable for CMode CLI.

=item C<< "address" => $string >>

Applicable for CMode CLI.

=item C<< "netmask" => $string >>

Applicable for CMode CLI.

=item C<< "home-node" => $string >>

Applicable for CMode CLI.

=item C<< "status-admin" => $string >>

Applicable for CMode CLI.

=item C<< "use-failover-group" => $string >>

Applicable for CMode CLI.

=item C<< "curr-port" => $string >>

Applicable for CMode CLI.

=item C<< "routing-group" => $string >>

=item C<< 'preferred-lifetime' => $string >>

Preferred Lifetime (autoconf)
Applicable for CMode CLI

=item C<< 'valid-lifetime' => $string >>

Valid Lifetime (autoconf)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network interface cdb modify" command in ONTAP.

Applicable for CMode CLI.


=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( has-valid-id curr-node failover-group home-port failover-policy
                    firewall-policy address netmask home-node status-admin use-failover-group
                    curr-port routing-group

                    preferred-lifetime valid-lifetime
                    )
            ),
        },
        cli_cmd => 'network interface cdb modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::NetworkInterfaceCdb::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api => 'network_interface_cdb_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 delete

   NACL::C::NetworkInterfaceCdb->delete(
                                command_interface => $ci,
	                             vif_id => $vif_id,
	                             lif_id => $lif_id,
	                             node => $node,
                                  ...);

(Class method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vif-id" => $vif_id >>

(Required)

Applicable for CMode CLI.

=item C<< "lif-id" => $lif_id >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

=item C<< '*' >>

This method takes all other options supported by
"network interface cdb delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        cli_cmd    => 'network interface cdb delete',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::NetworkInterfaceCdb::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'network_interface_cdb_delete');
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 show_routing_group

    my $show_routing_group_obj = NACL::C::NetworkInterfaceCdb->show_routing_group(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_routing_group_objs = NACL::C::NetworkInterfaceCdb->show_routing_group(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_routing_group_obj = $NetworkInterfaceCdb_obj->show_routing_group(); 

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-NetworkInterfaceCdbShowRoutingGroup-pm/ATTRIBUTES>
of NACL::CS::NetworkInterfaceCdbShowRoutingGroup as input. 
This method returns objects of the type L<NACL::CS::NetworkInterfaceCdbShowRoutingGroup|lib-NACL-CS-NetworkInterfaceCdbShowRoutingGroup-pm>.

=cut

sub show_routing_group {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::NetworkInterfaceCdbShowRoutingGroup;
    my @state_objs
        = NACL::CS::NetworkInterfaceCdbShowRoutingGroup->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vif-id' => {type => SCALAR | UNDEF, optional => 1},
        'lif-id' => {type => SCALAR},
        'node'   => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
