# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkNdpNeighbor Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkNdpNeighbor

=head1 DESCRIPTION

C<NACL::C::NetworkNdpNeighbor> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkNdpNeighbor.  A related class is
L<NACL::CS::NetworkNdpNeighbor|lib-NACL-CS-NetworkNdpNeighbor-pm>, which represents
the past state of an ONTAP NetworkNdpNeighbor.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 neighbor

=head2 vserver

=cut

package NACL::C::NetworkNdpNeighbor;

use strict;
use warnings;

use NACL::CS::NetworkNdpNeighbor;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $NetworkNdpNeighbor = NACL::C::NetworkNdpNeighbor->new(
                                   command_interface => $ci,
                                   'neighbor' => $neighbor,
                                   'vserver' => $vserver,
                                  );

Return a new NetworkNdpNeighbor component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'neighbor',
    scalar => 'vserver',
];

=head2 find

    my $NetworkNdpNeighbor_obj  = NACL::C::NetworkNdpNeighbor->find(command_interface => $ci, ...);
    my @NetworkNdpNeighbor_objs = NACL::C::NetworkNdpNeighbor->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my NetworkNdpNeighbor_state = $NetworkNdpNeighbor_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkNdpNeighbor would return
a NACL::CS::NetworkNdpNeighbor object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $NetworkNdpNeighbor_obj    = NACL::C::NetworkNdpNeighbor->create(
                                command_interface => $ci,
                                'neighbor' => $neighbor,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to create a static NDP neighbor entry.
Uses CMode CLI/ZAPI .
Invokes  net-ndp-neighbor-create for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'neighbor' => $neighbor >>

(Required)
Neighbor Address

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : neighbor-address

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< 'mac-address' => $string >>

MAC Address

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network ndp neighbor create" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( mac-address
                    vserver )
            ),
        },
        cli_cmd => 'network ndp neighbor create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkNdpNeighbor::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        neighbor          => $opts{'neighbor'},
        vserver           => $vserver,
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_ndp_neighbor_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $copy = [qw(mac-address)];
    my $map = {'neighbor' => 'neighbor-address',};
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'net-ndp-neighbor-create',
        copy => $copy,
        map  => $map,
    );

    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::NetworkNdpNeighbor->delete(
                                command_interface => $ci,
                                'neighbor' => $neighbor,
                                'vserver' => $vserver,
                                ...);

                                 or

    $NetworkNdpNeighbor_obj->delete(...);

(Class or instance method)
This method is used to delete a static NDP neighbor entry.
Uses CMode CLI/ZAPI .
Invokes  net-ndp-neighbor-destroy for CMode Zapi


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'neighbor' => $neighbor >>

(Required for class method, Not Applicable for instance method)
Neighbor Address

Applicable for CMode CLI/ZAPI


Maps to:

CMode ZAPI : neighbor-address

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name

Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network ndp neighbor delete" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'network ndp neighbor delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkNdpNeighbor::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_ndp_neighbor_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@opts);
    my $map = {'neighbor' => 'neighbor-address',};

    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'net-ndp-neighbor-destroy',
        map  => $map,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'neighbor' => {type => SCALAR},
        'vserver'  => {type => SCALAR},
    );
}

1;
