# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkOptionsIpv6 Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkOptionsIpv6

=head1 DESCRIPTION

C<NACL::C::NetworkOptionsIpv6> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkOptionsIpv6.  A related class is
L<NACL::CS::NetworkOptionsIpv6|lib-NACL-CS-NetworkOptionsIpv6-pm>, which represents
the past state of an ONTAP NetworkOptionsIpv6.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::NetworkOptionsIpv6;

use strict;
use warnings;

use NACL::CS::NetworkOptionsIpv6;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkOptionsIpv6 = NACL::C::NetworkOptionsIpv6->new(
                                   command_interface => $ci,
								  );

Return a new NetworkOptionsIpv6 component object with the given attributes.

=cut

=head2 find

    my $NetworkOptionsIpv6_obj  = NACL::C::NetworkOptionsIpv6->find(command_interface => $ci, ...);
	my @NetworkOptionsIpv6_objs = NACL::C::NetworkOptionsIpv6->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkOptionsIpv6_state = $NetworkOptionsIpv6_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkOptionsIpv6 would return 
a NACL::CS::NetworkOptionsIpv6 object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 modify

    NACL::C::NetworkOptionsIpv6->modify(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to modify IPv6 options.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'enabled' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'skip-check' => $string >>


Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'is-ra-processing-enabled' => $string >>

Router Advertisement (RA) Processing Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< switchless-cluster-info => boolean_value >>

(Optional) switchless-cluster-info field of switchless cluster info to modify.^M
It is of type boolean.
where boolean_value can be true or false.

Applicable only for CMode-ZAPI

=item C<< '*' >>

This method takes all other options supported by
"network options ipv6 modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enabled

                    skip-check

                    is-ra-processing-enabled
                    )
            ),
	'switchless-cluster-info' => { type => SCALAR, optional => 1 },
        },
        cli_cmd => 'network options ipv6 modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
	    {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => "_base_check_non_iter",
            },
        ],
        frontend => 'NACL::C::NetworkOptionsIpv6::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_options_ipv6_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi_non_iter{
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    ## currently only the manual mapping is working for the net-options-modify zapi
    $opts{'net-options'}->{'switchless-cluster-info'}     = delete $opts{'switchless-cluster-info'} if($opts{'switchless-cluster-info'});
    $opts{'net-options'}->{'ipv6-options-info'}->{'enabled'} = delete $opts{'enabled'} if($opts{'enabled'});
    $opts{'net-options'}->{'ipv6-options-info'}->{'is-ipv6-ra-processing-enabled'}  = delete $opts{'is-ra-processing-enabled'} if ($opts{'is-ra-processing-enabled'});
    my $copy = [qw(net-options)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'net-options-modify',
        copy => $copy,
	frontend_method => 'modify',
    );
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
