# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkOptionsSwitchlessCluster Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkOptionsSwitchlessCluster

=head1 DESCRIPTION

C<NACL::C::NetworkOptionsSwitchlessCluster> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkOptionsSwitchlessCluster.  A related class is
L<NACL::CS::NetworkOptionsSwitchlessCluster|lib-NACL-CS-NetworkOptionsSwitchlessCluster-pm>, which represents
the past state of an ONTAP NetworkOptionsSwitchlessCluster.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::NetworkOptionsSwitchlessCluster;

use strict;
use warnings;

use NACL::CS::NetworkOptionsSwitchlessCluster;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkOptionsSwitchlessCluster = NACL::C::NetworkOptionsSwitchlessCluster->new(
                                   command_interface => $ci,
                                  );

Return a new NetworkOptionsSwitchlessCluster component object with the given attributes.

=cut

=head2 find

    my $NetworkOptionsSwitchlessCluster_obj  = NACL::C::NetworkOptionsSwitchlessCluster->find(command_interface => $ci, ...);
    my @NetworkOptionsSwitchlessCluster_objs = NACL::C::NetworkOptionsSwitchlessCluster->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
This method executes net-options-get for CMode ZAPI.

=head2 state

    my NetworkOptionsSwitchlessCluster_state = $NetworkOptionsSwitchlessCluster_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkOptionsSwitchlessCluster would return
a NACL::CS::NetworkOptionsSwitchlessCluster object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 modify

    NACL::C::NetworkOptionsSwitchlessCluster->modify(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to modify switchless cluster network options.

Uses a CMode CLI/ZAPI APISet.

Invokes "net-options-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'force' => $string >>

*Force the configuration, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'enabled' => $string >>

*Enable Switchless Cluster, possible value(s) are true,false

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : net-options of switchless-cluster-info of enabled

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"network options switchless-cluster modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( force enabled
                    )
            ),
        },
        cli_cmd => 'network options switchless-cluster modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkOptionsSwitchlessCluster::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,    api => 'network_options_switchless_cluster_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $apiset = $opts{apiset};

    my %cmode_args;
    $cmode_args{'net-options'}->{'switchless-cluster-info'}
        = $opts{'enabled'};

    $apiset->net_options_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

1;
