#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkPortBroadcastDomain Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkPortBroadcastDomain

=head1 DESCRIPTION

C<NACL::C::NetworkPortBroadcastDomain> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkPortBroadcastDomain.  A related class is
L<NACL::CS::NetworkPortBroadcastDomain|lib-NACL-CS-NetworkPortBroadcastDomain-pm>, which represents
the past state of an ONTAP NetworkPortBroadcastDomain.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 ipspace

=head2 broadcast_domain

=cut

package NACL::C::NetworkPortBroadcastDomain;

use strict;
use warnings;

use NACL::CS::NetworkPortBroadcastDomain;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation;
use NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned;
use NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyRemoved;
use NACL::C::Exceptions::NetworkPortBroadcastDomain::AlreadyExists;
use NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist;

=head1 METHODS

=head2 new

  my $NetworkPortBroadcastDomain = NACL::C::NetworkPortBroadcastDomain->new(
                                   command_interface => $ci,
                                   'ipspace' => $ipspace,
                                   'broadcast-domain' => $broadcast_domain,
                                  );

Return a new NetworkPortBroadcastDomain component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'ipspace',
    scalar => 'broadcast_domain',
];

=head2 find

    my $NetworkPortBroadcastDomain_obj  = NACL::C::NetworkPortBroadcastDomain->find(command_interface => $ci, ...);
    my @NetworkPortBroadcastDomain_objs = NACL::C::NetworkPortBroadcastDomain->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my NetworkPortBroadcastDomain_state = $NetworkPortBroadcastDomain_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkPortBroadcastDomain would return
a NACL::CS::NetworkPortBroadcastDomain object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 create

    my $NetworkPortBroadcastDomain_obj    = NACL::C::NetworkPortBroadcastDomain->create(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

(Class method)
This method is used to create a new layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-create' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required)
IPspace Name
Applicable for CMode CLI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'mtu' => $string >>

MTU
Applicable for CMode CLI/ZAPI

=item C<< 'ports' => [ $port1, $port2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain-id' => $string >>

Broadcast Domain ID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortBroadcastDomain::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a
Broadcast Domain that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ports' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( mtu

                    broadcast-domain-id
                    )
            ),
        },
        api => 'network_port_broadcast_domain_create',
    );

    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkPortBroadcastDomain::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkPortBroadcastDomain::AlreadyExists->detect_convert_and_throw(
            exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        ipspace           => $opts{'ipspace'},
        broadcast_domain  => $opts{'broadcast-domain'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-create',
        copy            => [qw( broadcast-domain ipspace mtu ports )],
        frontend_method => 'create'
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::NetworkPortBroadcastDomain->delete(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->delete(...);

(Class or instance method)
This method is used to delete a layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-destroy' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist>

This type of exception gets thrown if an attempt is made to delete a
non-existing Broadcast Domain.

=item C<NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation>

This type of exception gets thrown if an attempt is made to delete a
broadcast domain which has ports with lif association on it.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'network_port_broadcast_domain_delete',
    );
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkPortBroadcastDomain::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::NetworkPortBroadcastDomain::PortsAssignedHasLifAssociation
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-destroy',
        copy            => [qw( broadcast-domain ipspace )],
        frontend_method => 'delete',
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::NetworkPortBroadcastDomain->modify(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->modify(...);

(Class or instance method)
This method is used to modify a layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-modify' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'mtu' => $string >>

MTU
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( mtu
                    )
            ),
        },
        api => 'network_port_broadcast_domain_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-modify',
        copy            => [qw( broadcast-domain ipspace mtu )],
        frontend_method => 'modify',
    );
    $Log->exit() if $may_exit;
}

=head2 add_ports

    NACL::C::NetworkPortBroadcastDomain->add_ports(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->add_ports(...);

(Class or instance method)
This method is used to add ports to a layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-add-ports' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'ports' => [ $port1, $port2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< 'port-update-status-combined' => $string >>


Overall Port Update Status possible value(s) are, complete,in-progress,error
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain add ports" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortBroadcastDomain::PortAlreadyAssigned>

This exception is thrown when an attempt is made to add a port to the
broadcast-domain it is already assigned to.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_ports {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( port-update-status-combined
                    )
            ),
            'ports' => {type => ARRAYREF, optional => 1},
        },
        api => 'network_port_broadcast_domain_add_ports',
    );

    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_add_ports_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_ports_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::add_ports',
        nacl_exceptions => [ qw(PortAlreadyAssigned DoesNotExist) ]
    );

    $Log->exit() if $may_exit;
}

sub _add_ports_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_add_ports',);
    $Log->exit() if $may_exit;
}

sub _add_ports_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-add-ports',
        copy            => [qw( broadcast-domain ipspace ports )],
        frontend_method => 'add_ports',
    );
    $Log->exit() if $may_exit;
}

=head2 remove_ports

    NACL::C::NetworkPortBroadcastDomain->remove_ports(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->remove_ports(...);

(Class or instance method)
This method is used to remove ports from a layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-remove-ports' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'ports' => [ $port1, $port2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< 'port-update-status-combined' => $string >>


Overall Port Update Status possible value(s) are, complete,in-progress,error
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain remove ports" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::NetworkPortBroadcastDomain::DoesNotExist>

This exception is thrown when a broadcast-domain operation is performed on a
broadcast-domain that does not exist in that IPSpace.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_ports {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( port-update-status-combined
                    )
            ),
            'ports' => {type => ARRAYREF, optional => 1},
        },
        api => 'network_port_broadcast_domain_remove_ports',
    );

    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_remove_ports_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_ports_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::remove_ports',
        nacl_exceptions => [ qw(PortAlreadyRemoved) ]
    );

    $Log->exit() if $may_exit;
}

sub _remove_ports_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_remove_ports',);
    $Log->exit() if $may_exit;
}

sub _remove_ports_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-remove-ports',
        copy            => [qw( broadcast-domain ipspace ports )],
        frontend_method => 'remove_ports',
    );
    $Log->exit() if $may_exit;
}

=head2 rename

    NACL::C::NetworkPortBroadcastDomain->rename(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->rename(...);

(Class or instance method)
This method is used to rename a layer 2 broadcast domain.
Uses CMode CLI/ZAPI .

Invokes API 'net-port-broadcast-domain-rename' for Cmode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'new-name' => $string >>

New Name
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain rename" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-name
                    )
            ),
        },
        api => 'network_port_broadcast_domain_rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_rename_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::rename',
    );

    $Log->exit() if $may_exit;
}

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_rename',);
    $Log->exit() if $may_exit;
}

sub _rename_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'net-port-broadcast-domain-rename',
        copy            => [qw( broadcast-domain ipspace new-name )],
        frontend_method => 'rename',
    );
    $Log->exit() if $may_exit;
}

=head2 merge

    NACL::C::NetworkPortBroadcastDomain->merge(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->merge(...);

(Class or instance method)
This method is used to merges two layer 2 broadcast domains.
Uses CMode CLI/ZAPI .
Invokes  net-port-broadcast-domain-merge for CMode Zapi

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'into-broadcast-domain' => $string >>

Merge with This Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain merge" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub merge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( into-broadcast-domain
                    )
            ),
        },
        api => 'network_port_broadcast_domain_merge',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_merge_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_merge_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::merge',
    );

    $Log->exit() if $may_exit;
}

sub _merge_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_merge',);
    $Log->exit() if $may_exit;
}

sub _merge_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(ipspace broadcast-domain into-broadcast-domain)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'net-port-broadcast-domain-merge',
        copy => $copy,
    );
    $Log->exit() if $may_exit;

}

=head2 split

    NACL::C::NetworkPortBroadcastDomain->split(
                                command_interface => $ci,
                                'ipspace' => $ipspace,
                                'broadcast-domain' => $broadcast_domain,
                                ...);

                                 or

    $NetworkPortBroadcastDomain_obj->split(...);

(Class or instance method)
This method is used to splits a layer 2 broadcast domains into two layer 2 broadcast domains..
Uses CMode CLI/ZAPI .
Invokes  net-port-broadcast-domain-split for CMode Zapi

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ipspace' => $ipspace >>

(Required for class method, Not Applicable for instance method)
IPspace Name
Applicable for CMode CLI/ZAPI

=item C<< 'broadcast-domain' => $broadcast_domain >>

(Required for class method, Not Applicable for instance method)
Layer 2 Broadcast Domain
Applicable for CMode CLI/ZAPI

=item C<< 'new-broadcast-domain' => $string >>

New Layer 2 Broadcast Domain Name
Applicable for CMode CLI/ZAPI

=item C<< 'ports' => [ $port1, $port2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network port broadcast domain split" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub split {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'ports' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( new-broadcast-domain
                    )
            ),
        },
        api => 'network_port_broadcast_domain_split',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_split_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_split_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkPortBroadcastDomain::split',
    );

    $Log->exit() if $may_exit;
}

sub _split_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_port_broadcast_domain_split',);
    $Log->exit() if $may_exit;
}

sub _split_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [
        qw(ipspace broadcast-domain new-broadcast-domain
            ports)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'net-port-broadcast-domain-split',
        copy => $copy,
    );
    $Log->exit() if $may_exit;

}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'ipspace'          => {type => SCALAR},
        'broadcast-domain' => {type => SCALAR},
    );
}

1;
