#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoute Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkRoute

=head1 DESCRIPTION

C<NACL::C::NetworkRoute> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkRoute.  A related class is
L<NACL::CS::NetworkRoute|lib-NACL-CS-NetworkRoute-pm>, which represents
the past state of an ONTAP NetworkRoute.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 destination

=head2 gateway

=cut

package NACL::C::NetworkRoute;

use strict;
use warnings;

use NACL::CS::NetworkRoute;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::NetworkRoute::DoesNotExist;
use NACL::C::Exceptions::NetworkRoute::AlreadyExists;
use NATE::BaseException qw(:try);

=head1 METHODS

=head2 new

  my $NetworkRoute = NACL::C::NetworkRoute->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'destination' => $destination,
                                   'gateway' => $gateway,
                                  );

Return a new NetworkRoute component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'destination',
    scalar => 'gateway',
];

=head2 find

    my $NetworkRoute_obj  = NACL::C::NetworkRoute->find(command_interface => $ci, ...);
    my @NetworkRoute_objs = NACL::C::NetworkRoute->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkRoute_state = $NetworkRoute_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkRoute would return
a NACL::CS::NetworkRoute object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $NetworkRoute_obj    = NACL::C::NetworkRoute->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'destination' => $destination,
                                'gateway' => $gateway,
                                ...);

(Class method)
This method is used to create a static route.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

=item C<< 'destination' => $destination >>

(Required)
Destination/Mask
Applicable for CMode CLI/ZAPI

=item C<< 'gateway' => $gateway >>

(Required)
Gateway
Applicable for CMode CLI/ZAPI

=item C<< 'metric' => $string >>

Metric
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network route create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkRoute::AlreadyExists>

This type of exception is thrown when an attempt is made to create a route that already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( metric
                    )
            ),
        },
        api => 'network_route_create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkRoute::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkRoute::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        destination       => $opts{'destination'},
        gateway           => $opts{'gateway'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_route_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $obj = $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(destination gateway metric)],
        api  => 'net-routes-create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

    NACL::C::NetworkRoute->delete(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'destination' => $destination,
                                'gateway' => $gateway,
                                ...);

                                 or

    $NetworkRoute_obj->delete(...);

(Class or instance method)
This method is used to delete a static route.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'destination' => $destination >>

(Required for class method, Not Applicable for instance method)
Destination/Mask
Applicable for CMode CLI/ZAPI

=item C<< 'gateway' => $gateway >>

(Required for class method, Not Applicable for instance method)
Gateway
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network route delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::NetworkRoute::DoesNotExist>

This type of exception is thrown when an attempt is made to create a route that does not exists.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'network_route_delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::NetworkRoute::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::NetworkRoute::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };
    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_route_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $obj = $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(destination gateway)],
        api  => 'net-routes-destroy',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 modify

    NACL::C::NetworkRoute->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'destination' => $destination,
                                'gateway' => $gateway,
                                ...);

                                 or

    $NetworkRoute_obj->modify(...);

(Class or instance method)
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'destination' => $destination >>

(Required for class method, Not Applicable for instance method)
Destination/Mask
Applicable for CMode CLI/ZAPI

=item C<< 'gateway' => $gateway >>

(Required for class method, Not Applicable for instance method)
Gateway
Applicable for CMode CLI/ZAPI

=item C<< 'metric' => $string >>

Metric
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "network route modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( metric
                    )
            ),
        },
        api => 'network_route_modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkRoute::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_route_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw( destination gateway metric )],
        api  => 'net-routes-modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 show_lifs

    my $show_lifs_obj = NACL::C::NetworkRoute->show_lifs(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_lifs_objs = NACL::C::NetworkRoute->show_lifs(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_lifs_obj = $NetworkRoute_obj->show_lifs(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-NetworkRouteShowLifs-pm/ATTRIBUTES>
of NACL::CS::NetworkRouteShowLifs as input. 
This method returns objects of the type L<NACL::CS::NetworkRouteShowLifs|lib-NACL-CS-NetworkRouteShowLifs-pm>.

=cut

sub show_lifs {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::NetworkRouteShowLifs;
    my @state_objs = NACL::CS::NetworkRouteShowLifs->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'     => {type => SCALAR},
        'destination' => {type => SCALAR},
        'gateway'     => {type => SCALAR},
    );
}

1;
