# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRouteCdb Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::NetworkRouteCdb

=head1 DESCRIPTION

C<NACL::C::NetworkRouteCdb> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP NetworkRouteCdb.  A related class is
L<NACL::CS::NetworkRouteCdb|lib-NACL-CS-NetworkRouteCdb-pm>, which represents
the past state of an ONTAP NetworkRouteCdb.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vsid

=head2 destination

=head2 gateway

=head2 node

=cut

package NACL::C::NetworkRouteCdb;

use strict;
use warnings;

use NACL::CS::NetworkRouteCdb;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $NetworkRouteCdb = NACL::C::NetworkRouteCdb->new(
                                   command_interface => $ci,
                                   'vsid' => $vsid,
                                   'destination' => $destination,
                                   'gateway' => $gateway,
								  );

Return a new NetworkRouteCdb component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vsid',
    scalar => 'destination',
    scalar => 'gateway',
    scalar => 'node',
];

=head2 find

    my $NetworkRouteCdb_obj  = NACL::C::NetworkRouteCdb->find(command_interface => $ci, ...);
	my @NetworkRouteCdb_objs = NACL::C::NetworkRouteCdb->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my NetworkRouteCdb_state = $NetworkRouteCdb_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::NetworkRouteCdb would return 
a NACL::CS::NetworkRouteCdb object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 modify

    NACL::C::NetworkRouteCdb->modify(
                                command_interface => $ci,
	                            'vsid' => $vsid,
	                            'destination' => $destination,
	                            'gateway' => $gateway,
                                ...);
	                             
								 or

    $NetworkRouteCdb_obj->modify(...);

(Class or instance method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsid' => $vsid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'destination' => $destination >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'gateway' => $gateway >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'metric' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network route cdb modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver metric
                    )
            ),
        },
        cli_cmd => 'network route cdb modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkRouteCdb::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_route_cdb_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 delete

    NACL::C::NetworkRouteCdb->delete(
                                command_interface => $ci,
	                            'vsid' => $vsid,
	                            'destination' => $destination,
	                            'gateway' => $gateway,
                                ...);
	                             
								 or

    $NetworkRouteCdb_obj->delete(...);

(Class or instance method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsid' => $vsid >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'destination' => $destination >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'gateway' => $gateway >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"network route cdb delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'network route cdb delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkRouteCdb::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'network_route_cdb_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 create

    my $NetworkRouteCdb_obj	= NACL::C::NetworkRouteCdb->create(
                                command_interface => $ci,
	                            'vsid' => $vsid,
	                            'destination' => $destination,
	                            'gateway' => $gateway,
                                ...);

(Class method)

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsid' => $vsid >>

(Required)

Applicable for CMode CLI.

=item C<< 'node' => $node >>

(Required)

Applicable for CMode CLI.

=item C<< 'destination' => $destination >>

(Required)

Applicable for CMode CLI.

=item C<< 'gateway' => $gateway >>

(Required)

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'metric' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'ipspaceid' => $string >>

IPspace ID
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"network route cdb create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver metric

                    ipspaceid
                    )
            ),
        },
        cli_cmd => 'network route cdb create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::NetworkRouteCdb::create',
    );

    my %node;
    if ($opts{'node'}) {
        $node{'node'} = $opts{'node'};
    }

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vsid              => $opts{'vsid'},
        destination       => $opts{'destination'},
        gateway           => $opts{'gateway'},
        %node,
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'network_route_cdb_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vsid'        => {type => SCALAR},
        'destination' => {type => SCALAR},
        'gateway'     => {type => SCALAR},
        'node'        => {type => SCALAR, optional => 1},
    );
}

1;
