# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Persistent_Fence Component Module
## @author rishis@netapp.com,ng-wafl-replica-qa
## @status shared

=head1 NAME

NACL::C::PersistentFence

=head1 DESCRIPTION

C<NACL::C::Persistent_Fence> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to nodescope 'persistent_fence' command.A related class is
L<NACL::CS::Persistent_Fence|lib-NACL-CS-PersistentFence-pm>, which
represents the current state of Persistent_Fence.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::PersistentFence;

use strict;
use warnings;

use base 'NACL::C::Component::ONTAP';

use Params::Validate qw(SCALAR);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Class::MethodMaker [scalar => 'volume'];


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('volume' => {type => SCALAR},);
}

__PACKAGE__->_nodescope_builder(
    command => 'persistent_fence_set',
    method  => 'set'
);
__PACKAGE__->_nodescope_builder(
    command => 'persistent_fence_clear',
    method  => 'clear'
);

__PACKAGE__->_cs_call_method_builder(method => 'get');

1;

=head1 METHODS

=head2 get

    $pf_obj = NACL::C::PersistentFence->new (
      command_interface => $Node1,
      volume            => $Volume,
  }

    $pf_obj->get();

(Class method or Instance method)

Uses a  7Mode CLI APISet.

Invokes "persistent_fence_get"  API for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Volume on which fence is to be checked for. 

=item C<< owner => $string >>

(Required) Owner of persistence fence.

=item C<< caller_id => $string >>

(Optional) Caller id of persistence fence.

=back

=item Returns
Component State object for persistent fence.

=back

=cut

=head2 set 

    $pf_obj = NACL::C::PersistentFence->new (
      command_interface => $Node1,
      volume            => $Volume,
  }

    $pf_obj->set (
      owner => 1,
      timeout => '1000',
      ....);

(Class method or Instance method)

Uses a  7Mode CLI APISet.

Invokes "persistent_fence_set"  API for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Volume on which fence is to be set. 

=item C<< owner => $string >>

(Required) Owner of persistence fence.

=item C<< caller_id => $string >>

(Optional) Caller id of persistence fence.

=item C<< timeout => $string >>

(Optional) Timeout of persistence fence (millisecs).

=item C<< suspend_flag => $string >>

=back

=item Returns
Parsed output from APISet Call.

=back

=cut

=head2 clear 

    $pf_obj = NACL::C::PersistentFence->new (
      command_interface => $Node1,
      volume            => $Volume,
  }

    $pf_obj->clear (
      owner => 1,
      caller_id => 100
      ....);

(Class method or Instance method)

Uses a  7Mode CLI APISet.

Invokes "persistent_fence_set"  API for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< volume => $string >>

(Required for class method, Not Applicable for instance method)
Volume on which fence is to be set. 

=item C<< owner => $string >>

(Required) Owner of persistence fence.

=item C<< caller_id => $string >>

(Optional) Caller id of persistence fence.

=back

=item Returns
Parsed output from APISet Call.

=back

=cut

