# $Id$

#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosPolicyGroup Component Module (auto-generated by CGT)
## @author pbala@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosPolicyGroup

=head1 DESCRIPTION

C<NACL::C::QosPolicyGroup> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosPolicyGroup.  A related class is
L<NACL::CS::QosPolicyGroup|lib-NACL-CS-QosPolicyGroup-pm>, which represents
the past state of an ONTAP QosPolicyGroup.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 policy_group

=cut

package NACL::C::QosPolicyGroup;

use strict;
use warnings;

use NACL::CS::QosPolicyGroup;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::C::_Mixins::QosPolicyGroup qw(:all);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::C::Exceptions::QosPolicyGroup::AlreadyExists qw(:try);
use NACL::C::Exceptions::QosPolicyGroup::DoesNotExist;
use NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated;

=head1 METHODS

=head2 new

  my $QosPolicyGroup = NACL::C::QosPolicyGroup->new(
                                   command_interface => $ci,
                                   'policy-group' => $policy_group,
                                  );

Return a new QosPolicyGroup component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'policy_group',];

=head2 find

    my $QosPolicyGroup_obj  = NACL::C::QosPolicyGroup->find(command_interface => $ci, ...);
    my @QosPolicyGroup_objs = NACL::C::QosPolicyGroup->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my QosPolicyGroup_state = $QosPolicyGroup_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosPolicyGroup would return
a NACL::CS::QosPolicyGroup object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 rename

    NACL::C::QosPolicyGroup->rename(
                                command_interface => $ci,
                                'policy-group' => $policy_group,
                                ...);

                                 or

    $QosPolicyGroup_obj->rename(...);

(Class or instance method)
This method is used to rename a policy group.

Uses a CMode CLI/ZAPI APISet.

Invokes 'qos-policy-group-rename' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'policy-group' => $policy_group >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: policy-group-name

=item C<< 'newname' => $string >>

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'new-name' => $string >>

New policy group name

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: new-name

=item C<< '*' >>

This method takes all other options supported by
"qos policy-group rename" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=back

=back

=cut

sub rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( newname

                    policy-group

                    new-name
                    )
            ),
        },
        cli_cmd => 'qos policy-group rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_rename_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosPolicyGroup::rename',
    );
    $pkg_or_obj->policy_group($opts{'new-name'}) if ref($pkg_or_obj);
    $Log->exit() if $may_exit;
} ## end sub rename

sub _rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api     => 'qos_policy_group_rename',
    );
    $Log->exit() if $may_exit;
} ## end sub _rename_cmode_cli

sub _rename_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [qw( new-name )];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'qos_policy_group_rename',
        copy   => $allowed_cmode_options,
        map    => {'policy-group' => 'policy-group-name'},
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::QosPolicyGroup->modify(
                                command_interface => $ci,
                                'policy-group' => $policy_group,
                                ...);

                                 or

    $QosPolicyGroup_obj->modify(...);

(Class or instance method)
This method is used to modify a policy group.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-policy-group-modify-iter" of CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'policy-group' => $policy_group >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<policy-group>$value</policy-group>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'min-latency-value' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-latency-value

=item C<< 'max-throughput' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<max-throughput>$value</max-throughput>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'violation-detection-window' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<violation-detection-window>$value</violation-detection-window>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'target-throughput' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<target-throughput>$value</target-throughput>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'violation-threshold' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<violation-threshold>$value</violation-threshold>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'violation-fraction' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<violation-fraction>$value</violation-fraction>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'target-latency' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<target-latency>$value</target-latency>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'min-latency' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<min-latency>$value</min-latency>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'deactivation-priority' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<deactivation-priority>$value</deactivation-priority>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'target-latency-value' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-latency-value

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'target-throughput-latency-cap-value' => $string >>



=item C<< 'target-throughput-latency-cap' => $string >>

Target Throughput Latency Limit

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<target-throughput-latency-cap>$value</target-throughput-latency-cap>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< 'target-latency-enforcement-cap' => $string >>

Target Latency Enforcement Limit

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-modify-iter>
<attributes>
<qos-policy-group-info>
<target-latency-enforcement-cap>$value</target-latency-enforcement-cap>
</qos-policy-group-info>
</attributes>
</qos-policy-group-modify-iter>

=item C<< '*' >>

This method takes all other options supported by
"qos policy-group modify" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI/ZAPI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( min-latency-value max-throughput violation-detection-window
                    target-throughput violation-threshold violation-fraction target-latency
                    min-latency deactivation-priority target-latency-value
                    policy-group target-throughput-latency-cap-value
                    target-throughput-latency-cap target-latency-enforcement-cap
                    )
            ),
        },
        api => 'qos_policy_group_modify',
    );

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_zapi_check',
                },
            ],
            frontend => 'NACL::C::QosPolicyGroup::modify',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::QosPolicyGroup::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'qos_policy_group_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $common_copy = $pkg_or_obj->_common_zapi_copy();
    my $common_map  = $pkg_or_obj->_common_zapi_map();

    $pkg_or_obj->_iter_helper(
        %opts,
        api                  => 'qos-policy-group-modify-iter',
        show_cmd             => 'qos policy-group show',
        copy                 => $common_copy,
        map                  => $common_map,
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosPolicyGroup::DoesNotExist'
    );

    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::QosPolicyGroup->delete(
                                command_interface => $ci,
                                'policy-group' => $policy_group,
                                ...);

                                 or

    $QosPolicyGroup_obj->delete(...);

(Class or instance method)
This method is used to delete a policy group.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-policy-group-delete-iter" for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'policy-group' => $policy_group >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<qos-policy-group-delete-iter>
<query>
<qos-policy-group-info>
<policy-group>&value</policy-group>
</qos-policy-group-info>
</query>
</qos-policy-group-delete-iter>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

=item C<< 'force' => $string >>

Force Delete Workloads for the QoS Policy Group
Applicable for CMode CLI and ZAPI(non-iter)

=item C<< '*' >>

This method takes all other options supported by
"qos policy-group delete" command in ONTAP.

Applicable for CMode CLI.

(Optional)

Applicable for CMode CLI/ZAPI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( policy-group

                    force
                    )
            )
        },
        cli_cmd => 'qos policy-group delete',
    );

    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    zapi_type => 'iter',
                },
                {   method    => '_delete_cmode_zapi_non_iter',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    zapi_type => 'non-iter',
                },
            ],
            frontend => 'NACL::C::QosPolicyGroup::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::QosPolicyGroup::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::QosPolicyGroup::WorkloadsAssociated
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'qos_policy_group_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %args) = @_;
    $pkg_or_obj->_iter_helper(%args,
        api                      => 'qos_policy_group_delete_iter',
        show_cmd                 => 'qos policy-group show',
        copy                     => ['policy-group'],
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosPolicyGroup::DoesNotExist'
    );
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %args,
        api     => 'qos-policy-group-delete',
    );
    $Log->exit() if $may_exit;
}

=head2 create

    my $QosPolicyGroup_obj    = NACL::C::QosPolicyGroup->create(
                                command_interface => $ci,
                                'policy-group' => $policy_group,
                                ...);

(Class method)
This method is used to create a policy group.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-policy-group-create" of CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'policy-group' => $policy_group >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-group

=item C<< 'min-latency-value' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-latency-value

=item C<< 'max-throughput' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-throughput

=item C<< 'violation-detection-window' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: violation-detection-window

=item C<< 'target-throughput' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-throughput

=item C<< 'violation-threshold' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: violation-threshold

=item C<< 'violation-fraction' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: violation-fraction

=item C<< 'target-latency' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-latency

=item C<< 'min-latency' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-latency

=item C<< 'deactivation-priority' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: deactivation-priority

=item C<< 'target-latency-value' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-latency-value

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'vsid' => $string >>

Applicable for CMode CLI

=item C<< 'target-throughput-latency-cap' => $string >>

Target Throughput Latency Limit

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-throughput-latency-cap

=item C<< 'vserver' => $string >>

Vserver

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: vserver

=item C<< 'target-throughput-latency-cap-value' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-throughput-latency-cap-value

=item C<< 'target-latency-enforcement-cap' => $string >>

Target Latency Enforcement Limit

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: target-latency-enforcement-cap

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"qos policy-group create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( min-latency-value max-throughput violation-detection-window
                    target-throughput violation-threshold violation-fraction target-latency
                    min-latency deactivation-priority target-latency-value
                    policy-group vsid target-throughput-latency-cap vserver
                    target-throughput-latency-cap-value target-latency-enforcement-cap
                    )
            ),
        },
        cli_cmd => 'qos policy-group create',
    );

    try {
         $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_zapi_check',
                },
            ],
            frontend => 'NACL::C::QosPolicyGroup::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::QosPolicyGroup::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        policy_group      => $opts{'policy-group'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'qos_policy_group_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( min-latency-value max-throughput violation-detection-window
            target-throughput policy-group violation-threshold violation-fraction
            target-latency min-latency deactivation-priority target-latency-value
            vserver target-throughput-latency-cap
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api  => 'qos_policy_group_create',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('policy-group' => {type => SCALAR},);
}

sub _zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $backend_validate_spec = $pkg_or_obj->_backend_validate_spec();
    my @backend_keys          = keys %$backend_validate_spec;
    my $backend_validate_spec_zapi = $pkg_or_obj->_common_zapi_backend_validate_spec();
    push (@backend_keys,keys %$backend_validate_spec_zapi);
    my @zapi_keys             = (qw(max-throughput policy-group vserver));
    my @all_zapi_keys         = (@backend_keys, @zapi_keys);

    delete $opts{_method};
    $pkg_or_obj->_hash_move(
        source => \%opts,
        target => {},
        move   => \@all_zapi_keys
    );
    my @remaining_keys = keys %opts;

    if (@remaining_keys) {
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw('ZAPI cannot be used '
                . 'because it can only modify the fields '
                . join(', ', @zapi_keys)
                . ' but a request was made to modify the following fields: '
                . join(', ', @remaining_keys));
    }
    $Log->exit() if $may_exit;
}

1;
