# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsArchivedWorkload Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsArchivedWorkload

=head1 DESCRIPTION

C<NACL::C::QosSettingsArchivedWorkload> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsArchivedWorkload.  A related class is
L<NACL::CS::QosSettingsArchivedWorkload|lib-NACL-CS-QosSettingsArchivedWorkload-pm>, which represents
the past state of an ONTAP QosSettingsArchivedWorkload.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::QosSettingsArchivedWorkload;

use strict;
use warnings;

use NACL::CS::QosSettingsArchivedWorkload;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsArchivedWorkload = NACL::C::QosSettingsArchivedWorkload->new(
                                   command_interface => $ci,
								  );

Return a new QosSettingsArchivedWorkload component object with the given attributes.

=cut

use Class::MethodMaker [];

=head2 find

    my $QosSettingsArchivedWorkload_obj  = NACL::C::QosSettingsArchivedWorkload->find(command_interface => $ci, ...);
	my @QosSettingsArchivedWorkload_objs = NACL::C::QosSettingsArchivedWorkload->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my QosSettingsArchivedWorkload_state = $QosSettingsArchivedWorkload_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsArchivedWorkload would return 
a NACL::CS::QosSettingsArchivedWorkload object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::QosSettingsArchivedWorkload->modify(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to modify the maximum number of QoS workloads to be archived.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-settings-archived-workload-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'max-workloads' => $string >>

*Maximum Number of Workloads to be Archived

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI : max-workloads

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"qos settings archived-workload modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( max-workloads
                    )
            ),
        },
        api => 'qos_settings_archived_workload_modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsArchivedWorkload::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'qos_settings_archived_workload_modify',
    );

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $apiset                = $opts{apiset};
    my $allowed_cmode_options = [
        qw( max-workloads
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->qos_settings_archived_workload_modify(%cmode_args);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
