# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsControl Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, jais@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsControl

=head1 DESCRIPTION

C<NACL::C::QosSettingsControl> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsControl.  A related class is
L<NACL::CS::QosSettingsControl|lib-NACL-CS-QosSettingsControl-pm>, which represents
the past state of an ONTAP QosSettingsControl.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::QosSettingsControl;

use strict;
use warnings;

use NACL::CS::QosSettingsControl;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsControl = NACL::C::QosSettingsControl->new(
                                   command_interface => $ci,
                                  );

Return a new QosSettingsControl component object with the given attributes.

=cut

=head2 find

    my $QosSettingsControl_obj  = NACL::C::QosSettingsControl->find(command_interface => $ci, ...);
    my @QosSettingsControl_objs = NACL::C::QosSettingsControl->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my QosSettingsControl_state = $QosSettingsControl_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsControl would return
a NACL::CS::QosSettingsControl object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify

    NACL::C::QosSettingsControl->modify(
                                command_interface => $ci,
                                ...);

(Class method)
This method executes qos-settings-control-modify for CMode ZAPI

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-settings-control-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'enforcement' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: enforcement

=item C<< 'ratebucket-rebalance' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ratebucket-rebalance

=item C<< 'policy-deactivation' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: policy-deactivation

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"qos settings control modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enforcement ratebucket-rebalance policy-deactivation
                    )
            ),
        },
        cli_cmd => 'qos settings control modify',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsControl::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'qos_settings_control_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $allowed_cmode_options = [
        qw( enforcement ratebucket-rebalance policy-deactivation
            )
    ];

    foreach my $option (@$allowed_cmode_options) {
        if (defined($opts{$option})) {
            $opts{$option}
                = ($opts{$option} =~ /1|true|on/i) ? "true" : "false";
        }
    }

    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
        api    => 'qos_settings_control_modify',
        copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
