# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsExtCache Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsExtCache

=head1 DESCRIPTION

C<NACL::C::QosSettingsExtCache> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsExtCache.  A related class is
L<NACL::CS::QosSettingsExtCache|lib-NACL-CS-QosSettingsExtCache-pm>, which represents
the past state of an ONTAP QosSettingsExtCache.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 ext_cache_setting

=cut

package NACL::C::QosSettingsExtCache;

use strict;
use warnings;

use NACL::CS::QosSettingsExtCache;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsExtCache = NACL::C::QosSettingsExtCache->new(
                                   command_interface => $ci,
                                   'ext-cache-setting' => $ext_cache_setting,
								  );

Return a new QosSettingsExtCache component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'ext_cache_setting',];

=head2 find

    my $QosSettingsExtCache_obj  = NACL::C::QosSettingsExtCache->find(command_interface => $ci, ...);
	my @QosSettingsExtCache_objs = NACL::C::QosSettingsExtCache->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my QosSettingsExtCache_state = $QosSettingsExtCache_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsExtCache would return 
a NACL::CS::QosSettingsExtCache object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 delete

    NACL::C::QosSettingsExtCache->delete(
                                command_interface => $ci,
	                            'ext-cache-setting' => $ext_cache_setting,
                                ...);
	                             
								 or

    $QosSettingsExtCache_obj->delete(...);

(Class or instance method)
This method is used to delete ext-cache setting.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ext-cache-setting' => $ext_cache_setting >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'ext-cache-setting' => $string >>

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"qos settings ext-cache delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ext-cache-setting
                    )
            )
        },
        cli_cmd => 'qos settings ext-cache delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsExtCache::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'qos_settings_ext_cache_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 modify

    NACL::C::QosSettingsExtCache->modify(
                                command_interface => $ci,
	                            'ext-cache-setting' => $ext_cache_setting,
                                ...);
	                             
								 or

    $QosSettingsExtCache_obj->modify(...);

(Class or instance method)
This method is used to modify ext-cache setting.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ext-cache-setting' => $ext_cache_setting >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'lopri' => $string >>

Applicable for CMode CLI.

=item C<< 'normal' => $string >>

Applicable for CMode CLI.

=item C<< 'default' => $string >>

Applicable for CMode CLI.

=item C<< 'readahead' => $string >>

Applicable for CMode CLI.

=item C<< 'meta' => $string >>

Applicable for CMode CLI.

=item C<< 'randomwrite' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'ext-cache-setting' => $string >>

Applicable for CMode CLI.

=item C<< 'ext-cache-setting-id' => $string >>


Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"qos settings ext-cache modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( lopri normal default readahead meta randomwrite

                    ext-cache-setting

                    ext-cache-setting-id
                    )
            ),
        },
        cli_cmd => 'qos settings ext-cache modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsExtCache::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'qos_settings_ext_cache_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 create

    my $QosSettingsExtCache_obj	= NACL::C::QosSettingsExtCache->create(
                                command_interface => $ci,
	                            'ext-cache-setting' => $ext_cache_setting,
                                ...);

(Class method)
This method is used to create ext-cache setting.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'ext-cache-setting' => $ext_cache_setting >>

(Required)

Applicable for CMode CLI.

=item C<< 'normal' => $string >>

Applicable for CMode CLI.

=item C<< 'default' => $string >>

Applicable for CMode CLI.

=item C<< 'readahead' => $string >>

Applicable for CMode CLI.

=item C<< 'meta' => $string >>

Applicable for CMode CLI.

=item C<< 'randomwrite' => $string >>

Applicable for CMode CLI.

=item C<< 'lopri' => $string >>

Applicable for CMode CLI.

=item C<< 'class' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'ext-cache-setting' => $string >>

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"qos settings ext-cache create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( normal default readahead meta randomwrite lopri class

                    ext-cache-setting
                    )
            ),
        },
        cli_cmd => 'qos settings ext-cache create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsExtCache::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        ext_cache_setting => $opts{'ext-cache-setting'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'qos_settings_ext_cache_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('ext-cache-setting' => {type => SCALAR},);
}

1;
