#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsExternalCache Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsExternalCache

=head1 DESCRIPTION

C<NACL::C::QosSettingsExternalCache> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsExternalCache.  A related class is
L<NACL::CS::QosSettingsExternalCache|lib-NACL-CS-QosSettingsExternalCache-pm>, which represents
the past state of an ONTAP QosSettingsExternalCache.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 external_cache_setting

=cut

package NACL::C::QosSettingsExternalCache;

use strict;
use warnings;

use NACL::CS::QosSettingsExternalCache;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::C::QosSettingsCache;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsExternalCache = NACL::C::QosSettingsExternalCache->new(
                                   command_interface => $ci,
                                   'external-cache-setting' => $external_cache_setting,
								  );

Return a new QosSettingsExternalCache component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'external_cache_setting',];

=head2 find

    my $QosSettingsExternalCache_obj  = NACL::C::QosSettingsExternalCache->find(command_interface => $ci, ...);
	my @QosSettingsExternalCache_objs = NACL::C::QosSettingsExternalCache->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my QosSettingsExternalCache_state = $QosSettingsExternalCache_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsExternalCache would return 
a NACL::CS::QosSettingsExternalCache object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 modify

    NACL::C::QosSettingsExternalCache->modify(
                                command_interface => $ci,
	                            'external-cache-setting' => $external_cache_setting,
                                ...);
	                             
								 or

    $QosSettingsExternalCache_obj->modify(...);

(Class or instance method)
This method is used to modify the external cache policy.

Uses a CMode CLI APIset.

Invokes "external-cache-policy-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'external-cache-setting' => $external_cache_setting >>

(Required for class method, Not Applicable for instance method)
*External Cache Policy Name

Maps to:

 CMode ZAPI:

    <external-cache-policy-modify-iter>
       <external-cache-policy-name>$value</external-cache-policy-name>
 </external-cache-policy-modify-iter>

Applicable for CMode CLI/ZAPI.


=item C<< 'default' => $string >>

Maps to:

 CMode ZAPI:

    <external-cache-policy-modify-iter>
       <is-default>$value</is-default>
 </external-cache-policy-modify-iter>

Applicable for CMode CLI/ZAPI.

=item C<< 'ext-cache-setting-id' => $string >>


Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %args = @_;
    if ($args{command_interface}->version_manager()->is_version_ge(release => '8.3')){
        $args{'cache-setting'}=delete $args{'external-cache-setting'} if (defined $args{'external-cache-setting'});
        NACL::C::QosSettingsCache->modify(%args);
    }
    else{
        $pkg_or_obj->_frontend_log_debug_opts(@_);
        my %opts = $pkg_or_obj->_common_validate_with(
            params          => \@_,
            additional_spec => {
                _optional_scalars(
                    qw( default ext-cache-setting-id
                        )
                ),
            },
            cli_cmd => 'qos settings external-cache modify',
        );
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_modify_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_modify_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
        );
    }
    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $allowed_cmode_options = [
        qw( default ext-cache-setting-id external-cache-setting
            extended_query )
    ];

    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api  => 'qos_settings_external_cache_modify',
        copy => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api => 'external-cache-policy-modify',
        map => {
            'external-cache-setting' => 'external-cache-policy-name',
            'default'                => 'is-default',
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('external-cache-setting' => {type => SCALAR},);
}

1;
