# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsReadAhead Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsReadAhead

=head1 DESCRIPTION

C<NACL::C::QosSettingsReadAhead> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsReadAhead.  A related class is
L<NACL::CS::QosSettingsReadAhead|lib-NACL-CS-QosSettingsReadAhead-pm>, which represents
the past state of an ONTAP QosSettingsReadAhead.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 read_ahead_setting

=cut

package NACL::C::QosSettingsReadAhead;

use strict;
use warnings;

use NACL::CS::QosSettingsReadAhead;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsReadAhead = NACL::C::QosSettingsReadAhead->new(
                                   command_interface => $ci,
                                   'read-ahead-setting' => $read_ahead_setting,
                                  );

Return a new QosSettingsReadAhead component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'read_ahead_setting',];

=head2 find

    my $QosSettingsReadAhead_obj  = NACL::C::QosSettingsReadAhead->find(command_interface => $ci, ...);
    my @QosSettingsReadAhead_objs = NACL::C::QosSettingsReadAhead->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my QosSettingsReadAhead_state = $QosSettingsReadAhead_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsReadAhead would return
a NACL::CS::QosSettingsReadAhead object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 delete

    NACL::C::QosSettingsReadAhead->delete(
                                command_interface => $ci,
                                'read-ahead-setting' => $read_ahead_setting,
                                ...);

                                 or

    $QosSettingsReadAhead_obj->delete(...);

(Class or instance method)
This method is used to delete read-ahead setting.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-settings-read-ahead-destroy-iter" for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'read-ahead-setting' => $read_ahead_setting >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<qos-settings-read-ahead-destroy-iter>
<query>
<qos-settings-read-ahead-info>
<read-ahead-setting-name>$value</read-ahead-setting-name>
</qos-settings-read-ahead-info>
</query>
</qos-settings-read-ahead-destroy-iter>

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"qos settings read-ahead delete" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI/ZAPI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( read-ahead-setting
                    )
            )
        },
        cli_cmd => 'qos settings read-ahead delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsReadAhead::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'qos_settings_read_ahead_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_iter_helper(@args,
        api      => 'qos_settings_read_ahead_destroy_iter',
        show_cmd => 'qos setting read-ahead show',
        map      => {'read-ahead-setting' => 'read-ahead-setting-name'},
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosSettingsReadAhead::DoesNotExist'
    );
    $Log->exit() if $may_exit;
}

=head2 create

    my $QosSettingsReadAhead_obj    = NACL::C::QosSettingsReadAhead->create(
                                command_interface => $ci,
                                'read-ahead-setting' => $read_ahead_setting,
                                ...);

(Class method)
This method is used to create read-ahead setting.

Uses a CMode CLI/ZAPI APISet.

Invokes "qos-settings-read-ahead-create" for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'read-ahead-setting' => $read_ahead_setting >>

(Required)

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: read-ahead-setting-name

=item C<< 'disk-response-weight' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: disk-response-weight

=item C<< 'use-histogram' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: use-histogram

=item C<< 'max-blocks' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-blocks

=item C<< 'early-count' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: early-count

=item C<< 'min-disk-response-time' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: min-disk-response-time

=item C<< 'max-deadline' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-deadline

=item C<< 'interarrival-weight' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: interarrival-weight

=item C<< 'force-full' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: force-full

=item C<< 'smallfile-blocks' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: smallfile-blocks

=item C<< 'use-feedback' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: use-feedback

=item C<< 'jitter' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: jitter

=item C<< 'metadata' => $string >>

Applicable for CMode CLI

=item C<< 'align-blocks' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: align-blocks

=item C<< 'max-disk-response-time' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-disk-response-time

=item C<< 'force-none' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: force-none

=item C<< 'use-async' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: use-async

=item C<< 'default' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: default

=item C<< 'metadata-blocks' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: metadata-blocks

=item C<< 'read-ahead-setting-id' => $string >>

Applicable for CMode CLI

=item C<< 'max-range' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-range

=item C<< 'force-dump' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: force-dump

=item C<< 'max-gap' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-gap

=item C<< 'disk-response-factor' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: disk-response-factor

=item C<< 'use-timing' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: use-timing

=item C<< 'min-blocks' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: min-blocks

=item C<< 'class' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: read-ahead-class

=item C<< 'overshoot' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: overshoot

=item C<< 'min-range' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: min-range

=item C<< 'method-timeout' => $time_in_seconds >>
Applicable for CMode CLI/ZAPI

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'max-retire-time' => $string >>

*Maximum Time Before a Stream is Retired (us), possible value(s) are
10000..10000000

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: max-retire-time

=item C<< 'min-retire-time' => $string >>

*Minimum Time Before a Stream is Retired (us), possible value(s) are
10000..10000000

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: min-retire-time

=item C<< 'min-file-histogram' => $string >>


Minimum filesize for Histogram-based predictions (blocks) ranges from 8192 to 33554432
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"qos settings read-ahead create" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( disk-response-weight use-histogram max-blocks early-count
                    min-disk-response-time max-deadline interarrival-weight force-full
                    smallfile-blocks use-feedback jitter metadata align-blocks
                    max-disk-response-time force-none use-async default metadata-blocks
                    read-ahead-setting-id max-range force-dump max-gap disk-response-factor
                    use-timing min-blocks class overshoot min-range

                    read-ahead-setting

                    max-retire-time min-retire-time

                    min-file-histogram
                    )
            ),
        },
        cli_cmd => 'qos settings read-ahead create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsReadAhead::create',
    );

    my $obj = $pkg->new(
        command_interface  => $opts{command_interface},
        read_ahead_setting => $opts{'read-ahead-setting'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'qos_settings_read_ahead_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( disk-response-weight use-histogram max-blocks early-count
            min-disk-response-time max-deadline interarrival-weight force-full
            smallfile-blocks use-feedback jitter align-blocks
            max-disk-response-time force-none use-async default metadata-blocks
            max-range force-dump max-gap disk-response-factor
            use-timing min-blocks  overshoot min-range
            max-retire-time min-retire-time
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'qos_settings_read_ahead_create',
        copy   => $allowed_cmode_options,
        map    => {
            'class'              => 'read-ahead-class',
            'read-ahead-setting' => 'read-ahead-setting-name',
        },
    );

    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::QosSettingsReadAhead->modify(
                                command_interface => $ci,
                                'read-ahead-setting' => $read_ahead_setting,
                                ...);

                                 or

    $QosSettingsReadAhead_obj->modify(...);

(Class or instance method)
This method is used to modify read-ahead setting.
Uses a CMode CLI/ZAPI APISet.
This method executes qos-settings-read-ahead-modify for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'read-ahead-setting' => $read_ahead_setting >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: read-ahead-setting-name

=item C<< 'disk-response-weight' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: disk-response-weight

=item C<< 'use-histogram' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: use-histogram

=item C<< 'max-blocks' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-blocks

=item C<< 'early-count' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: early-count

=item C<< 'min-disk-response-time' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-disk-response-time

=item C<< 'max-deadline' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-deadline

=item C<< 'interarrival-weight' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: interarrival-weight

=item C<< 'force-full' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force-full

=item C<< 'smallfile-blocks' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: smallfile-blocks

=item C<< 'use-feedback' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: use-feedback

=item C<< 'jitter' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: jitter

=item C<< 'metadata' => $string >>

Applicable for CMode CLI

=item C<< 'align-blocks' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: align-blocks

=item C<< 'max-disk-response-time' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-disk-response-time

=item C<< 'force-none' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force-none

=item C<< 'use-async' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: use-async

=item C<< 'default' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: default

=item C<< 'metadata-blocks' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: metadata-blocks

=item C<< 'read-ahead-setting-id' => $string >>

Applicable for CMode CLI

=item C<< 'max-range' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-range

=item C<< 'force-dump' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force-dump

=item C<< 'max-gap' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-gap

=item C<< 'disk-response-factor' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: disk-response-factor

=item C<< 'use-timing' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: use-timing

=item C<< 'min-blocks' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-blocks

=item C<< 'overshoot' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: overshoot

=item C<< 'min-range' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-range

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'max-retire-time' => $string >>

*Maximum Time Before a Stream is Retired (us), possible value(s) are
10000..10000000

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-retire-time

=item C<< 'min-retire-time' => $string >>

*Minimum Time Before a Stream is Retired (us), possible value(s) are
10000..10000000

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: min-retire-time

=item C<< 'class' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: read-ahead-class

=item C<< 'min-file-histogram' => $string >>


Minimum filesize for Histogram-based predictions (blocks) ranges from 8192 to 33554432
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"qos settings read-ahead modify" command in ONTAP.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( disk-response-weight use-histogram max-blocks early-count
                    min-disk-response-time max-deadline interarrival-weight force-full
                    smallfile-blocks use-feedback jitter metadata align-blocks
                    max-disk-response-time force-none use-async default metadata-blocks
                    read-ahead-setting-id max-range force-dump max-gap disk-response-factor
                    use-timing min-blocks overshoot min-range

                    read-ahead-setting

                    max-retire-time min-retire-time

                    min-file-histogram
                    )
            ),
        },
        cli_cmd => 'qos settings read-ahead modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::QosSettingsReadAhead::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'qos_settings_read_ahead_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( disk-response-weight use-histogram max-blocks early-count
            min-disk-response-time max-deadline interarrival-weight force-full
            smallfile-blocks use-feedback jitter align-blocks
            max-disk-response-time force-none use-async default metadata-blocks
            max-range force-dump max-gap disk-response-factor
            use-timing min-blocks  overshoot min-range
            max-retire-time min-retire-time
            )
    ];

    $pkg_or_obj->_iter_helper(@args,
        api      => 'qos_settings_read_ahead_modify_iter',
        show_cmd => 'qos setting read-ahead show',
        copy     => $allowed_cmode_options,
        map      => {
            'class'              => 'read-ahead-class',
            'read-ahead-setting' => 'read-ahead-setting-name',
        },
        does_not_exist_error     => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosSettingsReadAhead::DoesNotExist' 
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('read-ahead-setting' => {type => SCALAR},);
}

1;
