# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityCertificate Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com,pbala@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecurityCertificate

=head1 DESCRIPTION

C<NACL::C::SecurityCertificate> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecurityCertificate.  A related class is
L<NACL::CS::SecurityCertificate|lib-NACL-CS-SecurityCertificate-pm>, which represents
the past state of an ONTAP SecurityCertificate.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 common_name

=head2 vserver

=head2 type

=head2 serial

=cut

package NACL::C::SecurityCertificate;

use strict;
use warnings;

use NACL::CS::SecurityCertificate;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF UNDEF SCALARREF HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NACL::C::Exceptions::SecurityCertificate::DoesNotExist;

use NACL::GeneralUtils qw(_move_common_component_params_with_ci
    _copy_common_component_params_with_ci);

=head1 METHODS

=head2 new

  my $SecurityCertificate = NACL::C::SecurityCertificate->new(command_interface=>$ci,
                                   common_name => $common_name,
                                   vserver => $vserver,
                                   type    => $type,
                                  );

Return a new SecurityCertificate component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'common_name',
    scalar => 'vserver',
    scalar => 'type',
    scalar => 'serial',

    # Not a primary key, but is necessary for calls to find/state.
    # Deliberately left undocumented above since it's not a primary key
    # Internal certificates are not shown by default; they require
    # 'is-system-internal-certificate => true' to be specified to show them.
    # To make calls to find/state show the certificate, the Component object
    # needs to know if it's an internal certificate, hence this new attribute
    scalar => 'is_system_internal_certificate',

    # Also not a primary key, but need this value while deleting
    scalar => 'ca',
];

=head2 find

    my $SecurityCertificate  = NACL::C::SecurityCertificate->find(command_interface => $ci, ...);
    my @SecurityCertificates = NACL::C::SecurityCertificate->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=cut

# Overridden version to handle internal certificates. See the comments
# in the MethodMaker definition for an explanation of why this is needed.
sub find {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => {filter => {type => HASHREF, default => {}}},
        allow_extra => 1,
    );
    my @objs = $pkg_or_obj->SUPER::find(%opts);
    my $is_internal_filter = $opts{filter}{'is-system-internal-certificate'};
    if ((ref $pkg_or_obj && $pkg_or_obj->is_system_internal_certificate() =~ /true/i)
        || (defined $is_internal_filter && $is_internal_filter =~ /true/i)) {
        map {$_->is_system_internal_certificate('true')} @objs;
    }

    $Log->exit() if $may_exit;
    return wantarray ? @objs : $objs[0];
}

sub _extract_from_obj_for_find {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { arr_ref => { type => ARRAYREF } },
    );
    if (ref $pkg_or_obj) {
        my %filter = $pkg_or_obj->get_primary_keys_hash();
        if ($pkg_or_obj->is_system_internal_certificate() =~ /true/i) {
            $filter{'is-system-internal-certificate'} = 'true';
        }
        my @additional_array = (
            filter            => \%filter,
            command_interface => $pkg_or_obj->command_interface()
        );
        my $arr_ref = $opts{arr_ref};
        @$arr_ref = (@$arr_ref, @additional_array);
    }

    $Log->exit() if $may_exit;
}

=head2 state

    my $SecurityCertificate_state = $NACL::C::SecurityCertificate->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecurityCertificate would return
a NACL::CS::SecurityCertificate object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

# Overridden version to handle internal certificates. See the comments
# in the MethodMaker definition for an explanation of why this is needed.
sub _extract_filter_hash_for_state {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    my $filter = $self->SUPER::_extract_filter_hash_for_state(@args);
    my $is_internal = $self->is_system_internal_certificate() // '';
    if ($is_internal =~ /true/i) {
        $filter->{'is-system-internal-certificate'} = 'true';
    }
    $Log->exit() if $may_exit;
    return $filter;
}

=head2 create

    my $SecurityCertificate_object = NACL::C::SecurityCertificate->create(
                                command_interface => $ci,
                                 common_name => $common_name,
                                 vserver => $vserver,
                         type => $type,
                                  ...);

(Class method)
This method is used to create and install a self-signed digital certificate.

Uses a CMode CLI/ZAPI APISet.
  The method invokes 'security-certificate-create' for CMode ZAPI

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "common-name" => $common_name >>

  Applicable for CMode CLI/ZAPI
    Maps to:
    CM ZAPI: common-name

(Required)

=item C<< "vserver" => $vserver >>

  Applicable only for CMode CLI.

(Required)

=item C<< "type" => $type >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: type

=item C<< "country" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: country

=item C<< "expire-days" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: country

=item C<< "email-addr" => $string >>

=item C<< "state" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: state

=item C<< "size" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: size

=item C<< "locality" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: locality

=item C<< "unit" => $string >>

  Applicable only for CMode CLI.

=item C<< "hash-function" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: hash-function

=item C<< "organization" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: organization

=item C<< 'ca' => $string >>

  Applicable only for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< 'protocol' => $string >>

Protocol
Applicable for CMode CLI

=item C<< 'is-system-internal-certificate' => true|false >>

This specifies whether a certificate is internal, which is used only by Data
ONTAP components and does not require active management by the user. An internal
certificate can be installed but is not displayed by default. The default value
is false.

Applicable for CMode-CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"security certificate create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( country expire-days email-addr state size locality unit hash-function
                    organization type
                    ca
                    protocol is-system-internal-certificate
                    )
            ),
        },
        cli_cmd => 'security certificate create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $common_args
        = $pkg->_copy_common_component_params_with_ci(source => \%opts);
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_create_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SecurityCertificate::create',
    );

    # Do all of these steps only if the user wants the object
    if (defined wantarray) {
        my $obj = $pkg->new(
            command_interface => $opts{command_interface},
            common_name       => $opts{'common-name'},
            vserver           => $opts{vserver},
            type              => $opts{'type'},
        );

        # Overridden version to handle internal certificates. See the comments
        # in the MethodMaker definition for an explanation of why this is needed.
        my %extra_fetch_args;
        my $is_internal = $opts{'is-system-internal-certificate'};
        if (defined $is_internal && $is_internal =~ /true/i) {
            $obj->is_system_internal_certificate('true');
            $extra_fetch_args{'is-system-internal-certificate'} = 'true';
        }

        my $cs = $pkg->_fetch_cs_obj(
            %$common_args,
            %extra_fetch_args,
            'common-name' => $obj->common_name(),
            vserver       => $obj->vserver(),
            type          => $obj->type(),
        );
        $obj->serial($cs->serial());
        $obj->ca($cs->ca());

        $Log->exit() if $may_exit;
        return $obj;
    } else {
        $Log->exit() if $may_exit;
    }
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api     => 'security_certificate_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( country expire-days state size locality common-name
            hash-function organization type is-system-internal-certificate
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'security_certificate_create',
        copy   => $allowed_cmode_options,
        map    => {
            "email-addr" => "email-address",
            "unit"       => "organization-unit",
        },
    );

    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 install

   NACL::C::SecurityCertificate->install(
                                command_interface => $ci,
                                 common_name => $common_name,
                                 vserver => $vserver,
                     type    => $type,
                                  ...);

(Class method)
This method is used to install a digital certificate.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-certificate-install' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "common-name" => $common_name >>

(Required)

=item C<< "vserver" => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: certificate

=item C<< "type" => $type >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: type

=item C<< "certificate" => $string >>

Applicable for CMode-CLI/ZAPI

=item C<< "private-key" => $string >>

Applicable for CMode-CLI/ZAPI

=item C<< 'intermediate-cert-list' => \@arr_of_certificates >>

(Optional, Arrayref)
Applicable for CMode-CLI/ZAPI

=item C<< "subtype" => $string >>

(Optional)
Certificate Subtype
Applicable for CMode-CLI/ZAPI

=item C<< "is-system-internal-certificate" => true|false >>

(Optional)
Applicable for CMode-CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by
"security certificate install" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub install {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "intermediate-cert-list" =>
                {type => SCALAR | ARRAYREF, optional => 1},
            _optional_scalars(
                qw( type vserver certificate private-key type
                    subtype is-system-internal-certificate
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'security certificate install',
    );

    $pkg->_scalar_to_array_deprication_warning(
        params  => ['intermediate-cert-list'],
        options => \%opts
    );

    #This code line will delete the extra '\n' at the end of certificate if any.
    $opts{certificate} =~ s/\n+$//;

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_install_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_install_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SecurityCertificate::install',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub install

sub _install_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $certificate            = delete $opts{certificate} // "";
    my $private_key            = delete $opts{'private-key'} // "";
    my $intermediate_cert_list = delete $opts{'intermediate-cert-list'};

    # Normalize to an array
    $intermediate_cert_list = [] if (!defined $intermediate_cert_list);

    my $index                  = 0;

    my $prompt_answers = [
        'Please enter Certificate:.*' => "$certificate\n",
        'Please enter Private Key:.*' => "$private_key\n",
        'Do you want to continue.*'   => sub {
            if   ($index < scalar(@$intermediate_cert_list)) { 'y'; }
            else                                             { 'n'; }
        },
        'Please enter Intermediate Certificate.*' => sub {
            $intermediate_cert_list->[$index++] . "\n";
        },
    ];

    $pkg->_base_backend_cmode_cli(%opts,
        api             => 'security_certificate_install',
        additional_args => {'connectrec-match_table' => $prompt_answers},
    );

    $Log->exit() if $may_exit;
} ## end sub _install_cmode_cli

sub _install_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( certificate vserver private-key type
            intermediate-cert-list subtype
            is-system-internal-certificate
            )
    ];

    $pkg->_base_backend_cmode_zapi(@args,
        api    => 'security_certificate_install',
        copy   => $allowed_cmode_options,
    );
    $Log->exit() if $may_exit;
} ## end sub _install_cmode_zapi

=head2 generate_csr

   NACL::C::SecurityCertificate->generate_csr(
                                command_interface => $ci,
                                 common_name => $common_name,
                                 vserver => $vserver,
                                  ...);

(Class method)
This method is used to generate a digital certificate signing request.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-certificate-generate-csr' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "common-name" => $common_name >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: common-name

=item C<< "vserver" => $vserver >>

(Required)

=item C<< "country" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: country

=item C<< "email-addr" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: email-address

=item C<< "state" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: state

=item C<< "size" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: size

=item C<< "locality" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: locality

=item C<< "unit" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: organization-unit

=item C<< "organization" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: organization

=item C<< 'hash-function' => $string >>

  Applicable only for CMode CLI

=item C<< 'get_csr' => $ref_scalar >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: get-csr

=item C<< 'get_key' => $ref_scalar >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: get-key

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"security certificate generate-csr" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub generate_csr {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( country email-addr state size locality unit organization
                    hash-function common-name
                    )
            ),
            get_csr => {type => SCALARREF},
            get_key => {type => SCALARREF},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'security certificate generate-csr',
    );

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_generate_csr_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_generate_csr_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SecurityCertificate::generate_csr',
    );

    $Log->exit() if $may_exit;
} ## end sub generate_csr

sub _generate_csr_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'security_certificate_generate_csr',
        apiset_response_obj => \$response,
    );
    my $raw_output = $response->get_raw_output();

    if (defined $opts{get_csr}) {
        ${$opts{get_csr}} = _match_csr($raw_output);
    }

    if (defined $opts{get_key}) {
        ${$opts{get_key}} = _match_key($raw_output);
    }

    $Log->exit() if $may_exit;
} ## end sub _generate_csr_cmode_cli

sub _generate_csr_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $response;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'security_certificate_generate_csr',
        copy => [qw(country state size locality common-name organization)],
        map  => {
            'email-addr' => 'email-address',
            'unit'       => 'organization-unit',
        },
        apiset_response_obj => \$response,
        inapplicable_for_help_xml => [qw(get_csr get_key)],
    );
    my $parsed_output = $response->get_parsed_output();

    if (defined $opts{get_csr}) {
        ${$opts{get_csr}} = $parsed_output->[0]{'csr'};
    }

    if (defined $opts{get_key}) {
        ${$opts{get_key}} = $parsed_output->[0]{'private-key'};
    }

    $Log->exit() if $may_exit;
} ## end sub _generate_csr_cmode_zapi

################################################
#   Helper to fetch SecurityCertificate CS object.
#   Returns a  NACL::CS::SecurityCertificate type
#   object.
################################################

sub _fetch_cs_obj {
    $Log->enter() if $may_enter;
    my ($self, %args) = @_;
    delete $args{cli_cmd};

    my $target_args
        = $self->_move_common_component_params_with_ci(source => \%args);

    $Log->exit() if $may_exit;
    return NACL::CS::SecurityCertificate->fetch(
        %$target_args,
        filter           => \%args,
        requested_fields => ['serial', 'ca'],
        allow_empty      => 1,
    );

} ## end sub _fetch_cs_obj

=head2 sign

   my $certificate = NACL::C::SecurityCertificate->sign(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

(Class method)
This method is used to sign a digital certificate using self-signed Root CA
and the certificate is returned.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'security-certificate-sign' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: vserver

=item C<< 'ca-serial' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: ca-serial-number

=item C<< 'expire-days' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: expire-days

=item C<< 'destination' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: destination

=item C<< 'ca-common-name' => $string >>

  Applicable only for CMode CLI

=item C<< 'csr' => $string >>

  Applicable only for CMode ZAPI

=item C<< 'get_signed_cert' => $ref_scalar >>

(Required)
Mention that this is the certificate and is
used to answer the prompt
"Please enter Certificate Signing Request(CSR)"

=item C<< 'file-format' => $string >>

  Applicable only for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'format' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: format

=item C<< 'hash-function' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: hash-function

=item C<< 'filepassword' => $string >>

Used to answer the prompts
"Please enter a password for pkcs12 file" and
"Please enter it again"
  Applicable only for CMode ZAPI

=item C<< 'username' => $string >>

Used to answer the prompt
"Enter User for Destination URI"
  Applicable only for CMode ZAPI

=item C<< 'password' => $string >>

Used to answer the prompt
"Enter Password"
  Applicable only for CMode ZAPI

=item C<< 'private-key' => $string >>

Used to answer the prompt
"Please enter Private Key"
  Applicable only for CMode ZAPI

item C<< 'ca' => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CM ZAPI: certificate-authority

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"security certificate sign" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub sign {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ca-serial expire-days destination ca-common-name vserver csr file-format
                    format hash-function
                    filepassword username password private-key
                    ca
                    )
            ),
            get_signed_cert => {type => SCALARREF, optional => 1},
        },
        ignore_primary_keys => 1,
        api                 => 'security_certificate_sign',
    );

    my ($vserver, $output);
    $opts{vserver_operated_on} //= \$vserver;
    $pkg->call_on_apiset(
        %opts,

        # Get raw output from the back-end
        _get_output => \$output,
        choices => [
            {   method    => '_sign_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_sign_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecurityCertificate::sign',
        cli_cmd  => 'security certificate sign',
    );

    if (defined $opts{get_signed_cert}) {
        ${$opts{get_signed_cert}} = _match_cert($output);
    }

    $Log->exit() if $may_exit;
} ## end sub sign

sub _sign_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $csr               = delete $opts{csr};
    my $private_key       = delete ($opts{'private-key'}) // '';
    my $filepassword      = delete $opts{filepassword};
    my $username          = delete $opts{username};
    my $password          = delete $opts{password};

    my $prompt_answers = [
        'Please enter Certificate Signing Request(CSR)*' => "$csr\n",
        'Please enter Private Key:*'                     => "$private_key\n",
        'Please enter a password for pkcs12 file*'       => $filepassword,
        'Please enter it again*'                         => $filepassword,
        'Enter User for Destination URI*'                => $username,
        'Enter Password*'                                => $password,
    ];

    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'security_certificate_sign',
        additional_args     => {'connectrec-match_table' => $prompt_answers},
        apiset_response_obj => \$response
    );

    ${$opts{_get_output}} = $response->get_raw_output();

    $Log->exit() if $may_exit;
} ## end sub _sign_cmode_cli

sub _sign_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $response;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'security_certificate_sign',
        copy => [qw(expire-days destination vserver csr format hash-function
            filepassword username password private-key)],
        map  => {
            'ca-serial' => 'ca-serial-number',
            'ca'        => 'certificate-authority',
        },
        apiset_response_obj => \$response,
        inapplicable_for_help_xml => [qw(_get_output get_signed_cert)],
    );
    my $parsed_output = $response->get_parsed_output();
    ${$opts{_get_output}} = $parsed_output->[0]{'signed-cert'};

    $Log->exit() if $may_exit;
} ## end sub _sign_cmode_zapi

=head2 revoke

    NACL::C::SecurityCertificate->revoke(
                                command_interface => $ci,
                                'common-name' => $common_name,
                                'vserver' => $vserver,
                                'serial' => $serial,
                                ...);

(Class method)
This method is used to revoke a digital certificate.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'common-name' => $common_name >>

(Required)
  Applicable only for CMode CLI.

=item C<< 'vserver' => $vserver >>

(Required)
  Applicable only for CMode CLI.

=item C<< 'serial' => $serial >>

(Required)
  Applicable only for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)
  Applicable only for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"security certificate revoke" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub revoke {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( common-name vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        api             => 'security_certificate_revoke',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_revoke_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SecurityCertificate::revoke',
    );

    $Log->exit() if $may_exit;
} ## end sub revoke

sub _revoke_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'security_certificate_revoke',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _revoke_cmode_cli

=head2 delete

    NACL::C::SecurityCertificate->delete(
                                command_interface => $ci,
	                            'serial' => $serial,
	                            'vserver' => $vserver,
	                            'ca' => $ca,
	                            'common-name' => $common_name,
	                            'type' => $type,
                                ...);

								 or

    $SecurityCertificate_obj->delete(...);

(Class or instance method)
This method is used to delete an Installed Digital Certificate.
Uses CMode CLI/ZAPI .
Invokes  security-certificate-delete-iter for CMode Zapi iter
Invokes  security-certificate-delete  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'serial' => $serial >>

(Required for class method, Not Applicable for instance method)
Serial Number of Certificate
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <certificate-info serial-number="$value" />

Mapping for non-iter: serial-number

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Name of Vserver
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <certificate-info vserver="$value" />

=item C<< 'ca' => $ca >>

(Required for class method, Not Applicable for instance method)
Certificate Authority
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <certificate-info certificate-authority="$value" />

Mapping for non-iter: certificate-authority

=item C<< 'common-name' => $common_name >>

(Required for class method, Not Applicable for instance method)
FQDN or Custom Common Name
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <certificate-info common-name="$value" />

=item C<< 'type' => $type >>

(Required for class method, Not Applicable for instance method)
Type of Certificate
Applicable for CMode CLI/ZAPI iter, non-iter.

Mapping for iter-zapi: <certificate-info type="$value" />

=item C<< subtype => $subtype >>

(Optional)
Certificate Subtype.
Applicable for CMode-CLI/ZAPI.

=item C<< 'is-system-internal-certificate' => true|false >>

(Optional)
Is System Internal Certificate

=item C<< '*' >>

This method takes all other options supported by "security certificate delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ca subtype is-system-internal-certificate
                    )
             )
         },
        cli_cmd => 'security certificate delete',
    );
    $opts{ca} //= $pkg_or_obj->ca();
    if (!defined $opts{ca}) {
        my $is_internal = $opts{'is-system-internal-certificate'} //
            $pkg_or_obj->is_system_internal_certificate();
        my %extra_fetch_args;
        if (defined $is_internal && $is_internal =~ /true/i) {
            $extra_fetch_args{'is-system-internal-certificate'} = 'true';
        }
        my $cs_obj = $pkg_or_obj->_fetch_cs_obj(%opts, %extra_fetch_args);
        if ($cs_obj) {
            $opts{ca} = $cs_obj->ca();
            $opts{serial} //= $cs_obj->serial();
        }
    } ## end if (!exists $opts{ca})

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                    zapi_type => 'none',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    zapi_type => 'iter',
                    check     => '_check_delete_iter',
                },
                {   method    => '_delete_cmode_zapi_non_iter',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    zapi_type => 'non-iter',
                    check     => '_check_non_iter'
                },
            ],
            frontend => 'NACL::C::SecurityCertificate::delete',
        );
    } #end try
    catch NACL::C::Exceptions::Component with {
        $_[0]->throw;
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SecurityCertificate::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'security_certificate_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $copy = [
        qw(protocol expire-days locality state size hash-function organization country
            vserver common-name type)
    ];
    my $map = {
        'email-addr'  => 'email-address',
        'expiration'  => 'expiration-date',
        'public-cert' => 'public-certificate',
        'serial'      => 'serial-number',
        'ca'          => 'certificate-authority',
        'unit'        => 'organization-unit',
        'start'       => 'start-date',
    };
    $pkg_or_obj->_iter_helper(
        %opts,
        api      => 'security-certificate-delete-iter',
        show_cmd => 'security certificate show',
        copy     => $copy,
        map      => $map,
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::SecurityCertificate::DoesNotExist',
        no_attributes => 1,
        inapplicable_for_help_xml => [qw(is-system-internal-certificate)],
    );
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(common-name type subtype is-system-internal-certificate)];
    my $map  = {
        'serial' => 'serial-number',
        'ca'     => 'certificate-authority',
    };
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'security-certificate-delete',
        copy => $copy,
        map  => $map,
    );
    $Log->exit() if $may_exit;
}

# delete-iter cannot delete certificates of non-default subtype.
sub _check_delete_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $subtype = $opts{subtype} // '-';
    if ($subtype ne '-') {
        my $msg = 'Cannot use security-ceritificate-delete-iter because '
            . "it can only delete certificates of subtype '-' but "
            . "this certificate is of subtype '$subtype'";
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(serial vserver ca common-name type)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'common-name' => {type => SCALAR},
        'vserver'     => {type => SCALAR},
        'type'        => {type => SCALAR | UNDEF, optional => 1},
        'serial'      => {type => SCALAR, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

#################################################
# This routine returns a PEM-format certificate
# Return: PEM-format string for pass, 0 for fail
#################################################
sub _match_cert {
    my $string = shift;
    my $rex
        = qr/(-----BEGIN\sCERTIFICATE-----.*?-----END\sCERTIFICATE-----)/s;

    if ($string =~ m/$rex/) {
        $Log->exit() if $may_exit;
        return $1;
    } else {
        $Log->warn('The certificate could not be determined, this'
                . 'is possibly because the regex to determine the'
                . 'certificate is no longer accurate');
        $Log->exit() if $may_exit;
        return 0;
    } ## end else [ if ( $string =~ m/$rex/)]
} ## end sub _match_cert

#################################################
# This routine returns a PEM-format CSR
# Return: PEM-format string for pass, 0 for fail
#################################################
sub _match_csr {
    my $string = shift;
    my $rex
        = qr/(-----BEGIN\sCERTIFICATE\sREQUEST-----.*?-----END\sCERTIFICATE\sREQUEST-----)/s;

    if ($string =~ m/$rex/) {
        $Log->exit() if $may_exit;
        return $1;
    } else {
        $Log->warn('CSR could not be determined, this'
                . 'is possibly because the regex to determine the'
                . 'CSR is no longer accurate');
        $Log->exit() if $may_exit;
        return 0;
    } ## end else [ if ( $string =~ m/$rex/)]
} ## end sub _match_csr

#################################################
# This routine returns a PEM-format private-key
# Return: PEM-format string for pass, 0 for fail
#################################################
sub _match_key {
    my $string = shift;
    my $rex
        = qr/(-----BEGIN\sRSA\sPRIVATE\sKEY-----.*?-----END\sRSA\sPRIVATE\sKEY-----)/s;
    #burt972576 to introduce new rex
    my $new_rex
        = qr/(-----BEGIN\sPRIVATE\sKEY-----.*?-----END\sPRIVATE\sKEY-----)/s;

    if ($string =~ m/$rex/ || $string =~ m/$new_rex/) {
        $Log->exit() if $may_exit;
        return $1;
    }
    else {
        $Log->warn('The private key could not be determined, this'
                 . 'is possibly because the regex to determine the'
                 . 'private key is no longer accurate');
        $Log->exit() if $may_exit;
        return 0;
    } ## end else [ if ( $string =~ m/$rex/ || $string =~ m/$new_rex/)]
} ## end sub _match_key

1;
