# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecuritySsl Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SecuritySsl

=head1 DESCRIPTION

C<NACL::C::SecuritySsl> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SecuritySsl.  A related class is
L<NACL::CS::SecuritySsl|lib-NACL-CS-SecuritySsl-pm>, which represents
the past state of an ONTAP SecuritySsl.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::SecuritySsl;

use strict;
use warnings;

use NACL::CS::SecuritySsl;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $SecuritySsl = NACL::C::SecuritySsl->new(command_interface=>$ci,
                                                vserver =>$vserver,
                                              );

Return a new SecuritySsl component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
} ## end sub init

=head2 find

    my $SecuritySsl  = NACL::C::SecuritySsl->find(command_interface => $ci, ...);
    my @SecuritySsls = NACL::C::SecuritySsl->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SecuritySsl_state = $NACL::C::SecuritySsl->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SecuritySsl would return
a NACL::CS::SecuritySsl object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SecuritySsl->modify(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                or

    $SecuritySsl_object->modify(...);

(Class or instance method)
This method is used to modify the SSL configuration for HTTP servers.
Uses a CMode CLI/ZAPI or 7Mode CLI APISet.

Invokes 'secureadmin addcert ssl' command for 7Mode CLI.
Invokes 'security-ssl-get-iter' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI/ZAPI.

=item C<< "certificate" => $string >>

Applicable for CMode/7Mode CLI
  Maps to:
    7M CLI : certificate

=item C<< "enabled" => $string >>

(Applicable for CMode CLI)

=item C<< 'client-enabled' => $string >>

(Applicable for CMode CLI/ZAPI)
	Maps to :
			CM ZAPI : client-authentication-enabled

=item C<< 'serial' => $string >>

(Applicable for CMode CLI/ZAPI)
	Maps to :
			CM ZAPI : certificate-serial-number

=item C<< 'server-enabled' => $string >>

(Applicable for CMode CLI/ZAPI)
	Maps to :
			CM ZAPI : server-authentication-enabled

=item C<< 'common-name' => $string >>

(Applicable for Cmode CLI/ZAPI)

=item C<< 'ca' => $string >>

(Applicable for CMode CLI/ZAPI)
	Maps to :
			CM ZAPI : certificate-authority

=item C<< "extended_query" => $string >>

(Optional)
(Applicable only for CMode CLI/ZAPI)

=item C<< '*' >>

This method takes all other options supported by
"security ssl modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( certificate enabled

                    client-enabled serial server-enabled

                    common-name ca
                    )
            ),
        },
        cli_cmd => 'security ssl modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::SecuritySsl::modify',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'security_ssl_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %ssl_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(certificate)],
        target => \%ssl_args
    );
    %opts = %ssl_args;
    $apiset->secureadmin_addcert_ssl(%opts);
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'security_ssl_modify',
        copy   => [qw( vserver common-name )],
        map    => {
            'ca'             => 'certificate-authority',
            'serial'         => 'certificate-serial-number',
            'client-enabled' => 'client-authentication-enabled',
            'server-enabled' => 'server-authentication-enabled',
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR | UNDEF, optional => 1},);
}
1;
