# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnaplockLog Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SnaplockLog

=head1 DESCRIPTION

C<NACL::C::SnaplockLog> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SnaplockLog.  A related class is
L<NACL::CS::SnaplockLog|lib-NACL-CS-SnaplockLog-pm>, which represents
the past state of an ONTAP SnaplockLog.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::SnaplockLog;

use strict;
use warnings;

use NACL::CS::SnaplockLog;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $SnaplockLog = NACL::C::SnaplockLog->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new SnaplockLog component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SnaplockLog_obj  = NACL::C::SnaplockLog->find(command_interface => $ci, ...);
	my @SnaplockLog_objs = NACL::C::SnaplockLog->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SnaplockLog_state = $SnaplockLog_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SnaplockLog would return
a NACL::CS::SnaplockLog object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $SnaplockLog_obj	= NACL::C::SnaplockLog->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

(Class method)
This method is used to create audit log configuration for a Vserver..
Uses CMode CLI/ZAPI .
Invokes  snaplock-create-vserver-log  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'volume' => $string >>

Log Volume Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'max-log-size' => $string >>

Maximum Size of Log File
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'default-retention-period' => $string >>


Default Log Record Retention Period possible value(s) are, {<integer> seconds,minutes,hours,days,months,years} , infinite
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "snaplock log create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume max-log-size default-retention-period
                    )
            ),
        },
        cli_cmd => 'snaplock log create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_create_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        nacl_exceptions => [qw(AlreadyExists)],
        frontend        => 'NACL::C::SnaplockLog::create',
    );

    my %obj_opts = (
        command_interface => $opts{command_interface},
        vserver           => $vserver,
    );
    my $obj = $pkg->new(%obj_opts);
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'snaplock_log_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(max-log-size volume default-retention-period)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api                     => 'snaplock-create-vserver-log',
        copy                    => $copy,
        zapi_field_translations => {convert_to_bytes => [qw(max-log-size)],},
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::SnaplockLog->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $SnaplockLog_obj->delete(...);

(Class or instance method)
This method is used to delete audit log configuration for a Vserver..
Uses CMode CLI/ZAPI .
Invokes  snaplock-destroy-vserver-log  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "snaplock log delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'snaplock log delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_delete_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        nacl_exceptions => [qw(DoesNotExist)],
        frontend        => 'NACL::C::SnaplockLog::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'snaplock_log_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        api => 'snaplock-destroy-vserver-log',);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::SnaplockLog->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);

								 or

    $SnaplockLog_obj->modify(...);

(Class or instance method)
This method is used to modify audit log configuration for a Vserver..
Uses CMode CLI/ZAPI .
Invokes  snaplock-modify-vserver-log  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'volume' => $string >>

Log Volume Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'max-log-size' => $string >>

Maximum Size of Log File
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'default-retention-period' => $string >>


Default Log Record Retention Period possible value(s) are, {<integer> seconds,minutes,hours,days,months,years} , infinite
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "snaplock log modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume max-log-size default-retention-period
                    )
            ),
        },
        cli_cmd => 'snaplock log modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::SnaplockLog::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'snaplock_log_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(max-log-size volume default-retention-period)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api                     => 'snaplock-modify-vserver-log',
        copy                    => $copy,
        zapi_field_translations => {convert_to_bytes => [qw(max-log-size)],},
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
