#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnapmirrorPolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SnapmirrorPolicy

=head1 DESCRIPTION

C<NACL::C::SnapmirrorPolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SnapmirrorPolicy.  A related class is
L<NACL::CS::SnapmirrorPolicy|lib-NACL-CS-SnapmirrorPolicy-pm>, which represents
the past state of an ONTAP SnapmirrorPolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 policy

=cut

package NACL::C::SnapmirrorPolicy;

use strict;
use warnings;

use NACL::CS::SnapmirrorPolicy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);
use NACL::Exceptions::InvalidChoice;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::C::Exceptions::SnapmirrorPolicy::DoesNotExist ();
use NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists;
use NACL::C::Exceptions::SnapmirrorPolicy::RuleAlreadyExists;
use NACL::C::Exceptions::SnapmirrorPolicy::RuleDoesNotExist ();
use NACL::APISet::Exceptions::ResponseException qw(:try);

=head1 METHODS

=head2 new

  my $SnapmirrorPolicy = NACL::C::SnapmirrorPolicy->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                   'policy' => $policy,
                                  );

Return a new SnapmirrorPolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $SnapmirrorPolicy_obj  = NACL::C::SnapmirrorPolicy->find(command_interface => $ci, ...);
    my @SnapmirrorPolicy_objs = NACL::C::SnapmirrorPolicy->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
Uses snapmirror-policy-get-iter method for CMode ZAPI.

=head2 state

    my SnapmirrorPolicy_state = $SnapmirrorPolicy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SnapmirrorPolicy would return
a NACL::CS::SnapmirrorPolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 add_rule

    NACL::C::SnapmirrorPolicy->add_rule(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

                                 or

    $SnapmirrorPolicy_obj->add_rule(...);

(Class or instance method)
This method is used to add a new rule to snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-add-rule' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'policy' => $policy >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

=item C<< 'warn' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: warn

=item C<< 'keep' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: keep


=item C<< 'preserve' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: preserve

=item C<< 'snapmirror-label' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: snapmirror-label

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy add-rule" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::SnapmirrorPolicy::RuleAlreadyExists>

This type of exception is thrown if an attempt is made to add a rule that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_rule {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( warn keep preserve snapmirror-label

                    vserver policy
                    )
            ),
        },
        cli_cmd => 'snapmirror policy add-rule',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_add_rule_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_add_rule_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::SnapmirrorPolicy::add_rule',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SnapmirrorPolicy::RuleAlreadyExists
            ->detect_convert_and_throw(exception => $exception);

        NACL::C::Exceptions::SnapmirrorPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub add_rule

sub _add_rule_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'snapmirror_policy_add_rule',
    );
    $Log->exit() if $may_exit;
} ## end sub _add_rule_cmode_cli

sub _add_rule_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( warn keep preserve snapmirror-label
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'snapmirror_policy_add_rule',
        copy   => $allowed_cmode_options,
        map    => {'policy' => 'policy-name'},
    );
    $Log->exit() if $may_exit;
} ## end sub _add_rule_cmode_zapi

=head2 delete

    NACL::C::SnapmirrorPolicy->delete(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

                                 or

    $SnapmirrorPolicy_obj->delete(...);

(Class or instance method)
This method is used to delete a snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

(Required for class method, Not Applicable for instance method)

=item C<< 'policy' => $policy >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

(Required for class method, Not Applicable for instance method)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver policy
                    )
            )
        },
        cli_cmd => 'snapmirror policy delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::SnapmirrorPolicy::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SnapmirrorPolicy::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'snapmirror_policy_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'snapmirror_policy_delete',
        map    => {'policy' => 'policy-name'},
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $SnapmirrorPolicy_obj    = NACL::C::SnapmirrorPolicy->create(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

(Class method)
This method is used to create a new snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

(Required)

=item C<< 'policy' => $policy >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

(Required)

=item C<< 'restart' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: restart

=item C<< 'tries' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: tries

=item C<< 'comment' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: comment

=item C<< 'transfer-priority' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: transfer-priority

=item C<< 'ignore-atime' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ignore-atime

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< 'network-compression-enabled' => $string >>

Network Compression Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'create-snapshot' => $string >>

Create a New Snapshot Copy possible value(s) are, true,false
   Applicable for CMode CLI/ZAPI.
   Maps to:
      CM ZAPI: create-snapshot

=item C<< 'is-network-compression-enabled' => $string >>

Is Network Compression Enabled possible value(s) are, true,false
   Applicable for CMode CLI/ZAPI.
   Maps to:
      CM ZAPI: is-network-compression-enabled

=item C<< 'type' => $string >>


Snapmirror Policy Type possible value(s) are, vault,async-mirror,mirror-vault,sync-mirror
Applicable for CMode CLI/ZAPI.

=item C<< 'replication-mode' => $string >>

Replication Mode possible value(s) are, sync,semi-sync
Applicable for CMode CLI

=item C<< 'rpo' => $string >>

Recovery Point Objective (minutes)
Applicable for CMode CLI

=item C<< 'max-client-io-delay' => $string >>

Maximum Allowed Client I/O Delay (msecs)
Applicable for CMode CLI

=item C<< 'is-domino-mode-enabled' => $string >>

Is Domino Mode Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'max-io-latency-per-op' => $string >>

Maximum Allowed I/O Latency Per Operation (msecs)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy create" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'window-size-for-tdp-mirror' => $integer >>

Window Size for Transfer Operations for type 'tdp-mirror'
Applicable for CMode CLI/ZAPI.


=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists>

This type of exception gets thrown if an attempt is made to create a snapmirror policy
that already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( restart tries comment transfer-priority ignore-atime
                    vserver policy
                    network-compression-enabled
                    create-snapshot is-network-compression-enabled
                    type
                    replication-mode rpo max-client-io-delay is-domino-mode-enabled
                    max-io-latency-per-op window-size-for-tdp-mirror
                    )
            ),
        },
        cli_cmd => 'snapmirror policy create',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_create_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_create_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                    check     => '_create_check_zapi',
                },
            ],
            frontend => 'NACL::C::SnapmirrorPolicy::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SnapmirrorPolicy::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        policy            => $opts{'policy'},
    );
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args, api => 'snapmirror_policy_create');
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'snapmirror-policy-create',
        copy => [
            qw( restart tries comment transfer-priority ignore-atime
                create-snapshot is-network-compression-enabled type window-size-for-tdp-mirror)
        ],
        map             => {'policy' => 'policy-name'},
        zapi_field_translations => { convert_to_bytes => [qw( window-size-for-tdp-mirror)] },
        frontend_method => 'create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _create_check_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $type = $opts{type};

    # Should look for sync-mirror or sync_mirror. Note that there's also
    # an "async-mirror" value, so the anchor characters (^, $) are needed.
    if (defined $type && $type =~ /^sync[-_]mirror$/i) {
        my $msg = "The type was provided as '$type', but this is not "
            . 'supported in ZAPI, so the ZAPI implementation cannot be used.';
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::SnapmirrorPolicy->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

                                 or

    $SnapmirrorPolicy_obj->modify(...);

(Class or instance method)
This method is used to modify a snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

(Required for class method, Not Applicable for instance method)

=item C<< 'policy' => $policy >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

(Required for class method, Not Applicable for instance method)

=item C<< 'restart' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: restart

=item C<< 'tries' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: tries

=item C<< 'comment' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: comment

=item C<< 'transfer-priority' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: transfer-priority

=item C<< 'ignore-atime' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ignore-atime

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'policy' => $string >>

  Applicable only for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< 'network-compression-enabled' => $string >>

Network Compression Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'create-snapshot' => $string >>

Create a New Snapshot Copy possible value(s) are, true,false
  Applicable for CMode CLI/ZAPI.
  Maps to:
     CM ZAPI: create-snapshot

=item C<< 'is-network-compression-enabled' => $string >>

Is Network Compression Enabled possible value(s) are, true,false
  Applicable for CMode CLI/ZAPI.
  Maps to:
     CM ZAPI: is-network-compression-enabled

=item C<< 'type' => $string >>


Snapmirror Policy Type possible value(s) are, vault,async-mirror,mirror-vault,sync-mirror
Applicable for CMode CLI

=item C<< 'replication-mode' => $string >>

Replication Mode possible value(s) are, sync,semi-sync
Applicable for CMode CLI

=item C<< 'rpo' => $string >>

Recovery Point Objective (minutes)
Applicable for CMode CLI

=item C<< 'max-client-io-delay' => $string >>

Maximum Allowed Client I/O Delay (msecs)
Applicable for CMode CLI

=item C<< 'is-domino-mode-enabled' => $string >>

Is Domino Mode Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'max-io-latency-per-op' => $string >>

Maximum Allowed I/O Latency Per Operation (msecs)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy modify" command in ONTAP.

Applicable for CMode CLI.

=item C<< 'window-size-for-tdp-mirror' => $integer >>

Window Size for Transfer Operations for type 'tdp-mirror'.
Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( restart tries comment transfer-priority ignore-atime

                    vserver policy

                    network-compression-enabled

                    create-snapshot is-network-compression-enabled

                    type

                    replication-mode rpo max-client-io-delay is-domino-mode-enabled

                    max-io-latency-per-op window-size-for-tdp-mirror
                    )
            ),
        },
        cli_cmd => 'snapmirror policy modify',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SnapmirrorPolicy::modify',
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'snapmirror_policy_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'snapmirror-policy-modify',
        copy => [
            qw( restart tries comment transfer-priority ignore-atime
                is-network-compression-enabled create-snapshot window-size-for-tdp-mirror)
        ],
        map             => {'policy' => 'policy-name'},
        zapi_field_translations => { convert_to_bytes => [qw( window-size-for-tdp-mirror)] },
        frontend_method => 'modify'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 modify_rule

    NACL::C::SnapmirrorPolicy->modify_rule(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

                                 or

    $SnapmirrorPolicy_obj->modify_rule(...);

(Class or instance method)
This method is used to modify an existing rule in snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-modify-rule' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'policy' => $policy >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

=item C<< 'warn' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: warn

=item C<< 'keep' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: keep

=item C<< 'preserve' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: preserve

=item C<< 'snapmirror-label' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: snapmirror-label

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy modify-rule" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify_rule {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( warn keep preserve snapmirror-label

                    vserver policy
                    )
            ),
        },
        cli_cmd => 'snapmirror policy modify-rule',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_rule_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_rule_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SnapmirrorPolicy::modify_rule',
    );

    $Log->exit() if $may_exit;
} ## end sub modify_rule

sub _modify_rule_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'snapmirror_policy_modify_rule',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_rule_cmode_cli

sub _modify_rule_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( snapmirror-label warn keep preserve
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'snapmirror_policy_modify_rule',
        copy   => $allowed_cmode_options,
        map    => {'policy' => 'policy-name'},
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_rule_cmode_zapi

=head2 remove_rule

    NACL::C::SnapmirrorPolicy->remove_rule(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                'policy' => $policy,
                                ...);

                                 or

    $SnapmirrorPolicy_obj->remove_rule(...);

(Class or instance method)
This method is used to remove a rule from snapmirror policy.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'snapmirror-policy-remove-rule' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< 'policy' => $policy >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: policy-name

=item C<< 'snapmirror-label' => $string >>
  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: snapmirror-label

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)
  Applicable only for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"snapmirror policy remove-rule" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::SnapmirrorPolicy::RuleDoesNotExist>

This type of exception is thrown if an attempt is made to remove a rule that does not exist.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove_rule {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( snapmirror-label

                    vserver policy
                    )
            ),
        },
        cli_cmd => 'snapmirror policy remove-rule',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_remove_rule_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_remove_rule_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
            ],
            frontend => 'NACL::C::SnapmirrorPolicy::remove_rule',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SnapmirrorPolicy::RuleDoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
} ## end sub remove_rule

sub _remove_rule_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api     => 'snapmirror_policy_remove_rule',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_rule_cmode_cli

sub _remove_rule_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my $allowed_cmode_options = [
        qw( snapmirror-label
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api    => 'snapmirror_policy_remove_rule',
        copy   => $allowed_cmode_options,
        map    => {'policy' => 'policy-name'},
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_rule_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR},
        'policy'  => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
