#$Id: //depot/prod/test/nacldev/lib/NACL/C/Switch/Vsan.pm#1 $
#
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary Vsan Component Module
## @author veerap@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::C::Switch::Vsan

=head1 DESCRIPTION

C<NACL::C::Switch::Vsan> Is a derived class of
L<NACL::C::Component::Switch|lib-NACL-C-Component-Switch-pm>. It represents access to
a Switch.

=head1 ATTRIBUTES

=head2 vsan_id

vsan id on switch

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=cut

###################
# Package
package NACL::C::Switch::Vsan;

###################
# Everytime
use strict;
use warnings;

###################
use base 'NACL::C::Component::Switch';

# NATE Module includes
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);

=head1 METHODS

=cut

use Class::MethodMaker [
    scalar => 'vsan_id',
];

=head2 create

    my $vsan = NACL::C::Switch::Vsan->create(
        command_interface   => $ci,
        'vsan-id'           => 128
    );

OR

    $vsan->create();

(Class or instance method) This method create a vsan.
List of supported sets are: CiscoFC.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsan-id'=>$string >>

(Required) Id of the vsan on which action will be performed.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@opts);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@opts,
        additional_spec => {
            deconfigure => { type => SCALAR, optional => 1 },
        }
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_create_ciscofc_cli",
                interface => "CLI",
                set       => "CiscoFC",
            },

        ],
    );

    $Log->exit() if $may_exit;

    return $pkg_or_obj->new(
        vsan_id             => $opts{'vsan-id'},
        command_interface   => $opts{'command_interface'},
    );
}

sub _create_ciscofc_cli {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, @opts) = @_;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@opts);

    my $apiset = delete $opts{apiset};
    delete $opts{command_interface};

    $apiset->configure( subsystem => "terminal" );

    if (exists $opts{deconfigure} and $opts{deconfigure} == 1) {
        my $prompts_answers = [ '.*Do you want to continue.*' => 'y' ];
        $opts{'connectrec-match_table'} = $prompts_answers;
    }
    
    try {
        $apiset->vsan_database();
        $apiset->vsan(%opts);
    } finally {
        $apiset->end();
    };

    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::Switch::Vsan->delete(
        command_interface   => $ci,
        'vsan-id'           => 128
    );

OR

    $vsan->delete();

(Class or instance method) This method deletes a vsan.
List of supported sets are: CiscoFC.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsan-id'=>$string >>

(Required) Id of the vsan on which action will be performed.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;

    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->create(deconfigure => 1, %opts);

    $Log->exit() if $may_exit;
}

=head2 disable

    NACL::C::Switch::Vsan->disable(
        command_interface   => $ci,
        'vsan-id'           => 128
    );

OR

    $vsan->disable();

(Class or instance method) This method disables the vsan.
List of supported sets are ciscofc and brocadefc.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsan-id'=>$string >>

(Required) Id of the vsan on which action will be performed.

=item C<< 'suspend'=>$string >>

(Required) Suspends the vsan.

=back

=back

=cut

sub disable {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        allow_extra => 1,
        params      => \@_,
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_disable_ciscofc_cli",
                interface => "CLI",
                set       => "CiscoFC",
            },
            {
                method    => "_disable_brocadefc_cli",
                interface => "CLI",
                set       => "BrocadeFC",
            },
        ],
    );
    $Log->exit() if $may_exit;
}

sub _disable_ciscofc_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    $opts{'suspend'} = "1";

    $apiset->configure( subsystem => "terminal" );
    try {
        $apiset->vsan_database();
        $apiset->vsan(%opts);
    } finally {
        $apiset->end();
    };

    $Log->exit() if $may_exit;
}

sub _disable_brocadefc_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );

    my $apiset = delete $opts{apiset};
    my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => { "vsan-id" => "fid-number" },
    );

    $api_opts{'command'} = "switchdisable";
    $apiset->fosexec(%api_opts);

    $Log->exit() if $may_exit;
}

=head2 enable

    NACL::C::Switch::Vsan->enable(
        command_interface   => $ci,
        vsan_id             => 128
    );

OR

    $vsan->enable();

(Class or instance method) This method enables the vsan.
List of supported sets are ciscofc and brocadefc.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsan-id'=>$string >>

(Required) Id of the vsan on which action will be performed.

=item C<< 'suspend'=>$string >>

(Required) Suspends the vsan.

=back

=back

=cut

sub enable {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        allow_extra => 1,
        params      => \@_,
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_enable_ciscofc_cli",
                interface => "CLI",
                set       => "CiscoFC",
            },
            {
                method    => "_enable_brocadefc_cli",
                interface => "CLI",
                set       => "BrocadeFC",
            },

        ],
    );
    $Log->exit() if $may_exit;
}

sub _enable_ciscofc_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );
    my $apiset = delete $opts{apiset};
    delete $opts{command_interface};
    $opts{'suspend'} = "1";
    $apiset->configure( subsystem => "terminal" );
    try {
        $apiset->vsan_database();
        $apiset->vsan( %opts, deconfigure => '1' );
    } finally {
        $apiset->end();
    };
    
    $Log->exit() if $may_exit;
}

sub _enable_brocadefc_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );
    my $apiset = delete $opts{apiset};
    my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => { "vsan-id" => "fid-number" },
    );
    $api_opts{'command'} = "switchenable";
    $apiset->fosexec(%api_opts);

    $Log->exit() if $may_exit;
}

=head2 setcontext

    NACL::C::Switch::Vsan->setcontext(
        command_interface   => $ci,
        vsan_id             => 128
    );

OR

    $vsan->setcontext();

(Class or instance method)
This method sets the logical switch context to a specified id of brocadefc switch.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required)See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vsan-id'=>$string >>

(Required) Id of the vsan on which action will be performed.

=back

=back

=cut

sub setcontext {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        allow_extra => 1,
        params      => \@_,
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {
                method    => "_setcontext_brocadefc_cli",
                interface => "CLI",
                set       => "BrocadeFC",
            },

        ],
    );
    $Log->exit() if $may_exit;
}

sub _setcontext_brocadefc_cli {

    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1,
    );
    my $apiset = delete $opts{apiset};
    my %api_opts;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => { "vsan-id" => "fid-number" },
    );
    $apiset->setcontext(%api_opts);
    $Log->exit() if $may_exit;
}

sub _primary_keys_validate_spec {
    return ( 'vsan-id' => { type => SCALAR }, );
}

1;
