# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#  Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary Vserver Component Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SysManager::Vserver

=head1 DESCRIPTION

C<NACL::C::SysManager::Vserver> is a derived class of
L<NACL::C::Component::SysManager|lib-NACL-C-Component-SysManager-pm>.  It represents access
to an ONTAP Vserver present on the System Manger GUI Screen.  A related class is
L<NACL::CS::SysManager::Vserver|lib-NACL-CS-SysManager-Vserver-pm>, which represents
the State Info of an ONTAP Vserver present on the System Manger GUI Screen.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

The name of the vserver, used to identify it uniquely. (Vserver
names are assumed to be unique)

=cut

package NACL::C::SysManager::Vserver;

use strict;
use warnings;
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

use NACL::CS::SysManager::Vserver;
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use base 'NACL::C::Component::SysManager';
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

	my $vserver = NACL::C::SysManager::Vserver->new(command_interface => $ci,
													vserver           => $vserver_name);

Return a new Vserver component object with the given attributes.

=cut

####################### Method Maker ###############################

use Class::MethodMaker [ scalar => 'vserver', ];

=head2 create

	my $vserver_object = 
		NACL::C::SysManager::Vserver->create(command_interface => $ci,
											 vserver           => $vserver_name,...);

(Class method) Request creation of a new vserver element, and return a new
vserver component object that refers to it.
Uses CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required) Name of vserver.

=item C<< aggregate => $string >>

(Required) Name of the aggregate

=item C<< language => $string >>

(Required) Language type

=item C<< protocol => $string >>

(Required) Protocols to be configured.

=item C<< "delegate-name" => $string >>

(Optional) Delegate Name

=item C<< "delegate-password" => $string >>

(Optional) Delegate Password

=item C<< "delegate-confirm-password" => $string >>

(Optional) Delegate Confirmation Password

=item C<< "delegate-volume" => $string >>

(Optional) Delegate Volume

=item C<< "delegate-aggregate" => $string >>

(Optional) Delegate Aggregate

=item C<< "interface-type" => $string >>

(Optional) Interface Type

=item C<< "interface-name" => $string >>

(Optional) Interface Name

=item C<< "homeport" => $string >>

(Optional) Homeport

=item C<< "ip-address" => $string >>

(Optional) IP Address

=item C<< "netmask" => $string >>

(Optional) Netmask

=item C<< "gateway" => $string >>

(Optional) Gateway

=item C<< "primary-domain-name" => $string >>

(Optional) Primary Domain Name

=item C<< "add-search-domain-name" => $string >>

(Optional) Add Search Domain Name list

=item C<< "delete-search-domain-name" => $string >>

(Optional) Delete Search Domain Name list

=item C<< "add-name-servers" => $string >>

(Optional) Add Name servers

=item C<< "delete-name-servers" => $string >>

(Optional) Delete Name servers

=item C<< "add-nfs-interface-name" => $string >>

(Optional) NFS interface name Add

=item C<< "nfs-homeport" => $string >>

(Optional) NFS Homeport

=item C<< "nfs-ip-address" => $string >>

(Optional) NFS IP Address

=item C<< "nfs-netmask" => $string >>

(Optional) NFS Netmask

=item C<< "nfs-gateway" => $string >>

(Optional) NFS Gateway

=item C<< "nfs-allow-management" => $string >>

(Optional) NFS Allow Management

=item C<< "delete-nfs-interface-name" => $string >>

(Optional) NFS Interface Name delte list

=item C<< "nfs-version" => $string >>

(Optional) NFS Version

=item C<< "name-servers" => $string >>

(Optional) Name Servers

=item C<< "nis-primary-domain-name" => $string >>

(Optional) NIS Primary Domain Name

=item C<< "nis-server" => $string >>

(Optional) NIS Servers

=item C<< "add-secondary-domains" => $string >>

(Optional) Secondary Domains Add

=item C<< "add-secondary-nis-server" => $string >>

(Optional) Secondary NIS server Add

=item C<< "delete-secondary-domains" => $string >>

(Optional) Secondary Domains Delete

=item C<< "local-users-group-name" => $string >>

(Optional) Local Users Group Name

=item C<< "local-users-username" => $string >>

(Optional) Local Users Username

=item C<< "local-users-user-id" => $string >>

(Optional) Local Users User ID

=item C<< "local-users-full-name" => $string >>

(Optional) Local Users FullName

=item C<< "add-mapping" => $string >>

(Optional) Mapping Add

=item C<< "mapping-direction" => $string >>

(Optional) Mapping Direction

=item C<< "mapping-position" => $string >>

(Optional) Mapping Position

=item C<< "mapping-pattern" => $string >>

(Optional) Mapping Pattern

=item C<< "mapping-replacement" => $string >>

(Optional) Mapping Replacement

=item C<< "edit-mapping" => $string >>

(Optional) Edit Mapping

=item C<< "edit-mapping-pattern" => $string >>

(Optional) Edit Mapping Pattern

=item C<< "edit-mapping-replacement" => $string >>

(Optional) Edit Mapping Replacement

=item C<< "delete-mapping" => $string >>

(Optional) Delete Mapping

=item C<< "dns-configuration" => $string >>

(Optional) DNS Configuration

=item C<< "iscsi-target-node-name" => $string >>

(Optional)iSCSI Target NodeName

=item C<< "iscsi-target-alias" => $string >>

(Optional)iSCSI Target Alias

=item C<< "iscsi-controller-name" => $string >>

(Optional)iSCSI Controller Name

=item C<< "iscsi-interface-per-control" => $string >>

(Optional)iSCSI Interface Per Control

=item C<< "iscsi-starting-ip" => $string >>

(Optional)iSCSI Starting IP

=item C<< "iscsi-netmask" => $string >>

(Optional)iSCSI Netmask

=item C<< "iscsi-gateway" => $string >>

(Optional)iSCSI Gateway

=item C<< "iscsi-portset" => $string >>

(Optional)iSCSI Portset

=item C<< "interface-protocol" => $string >>

(Optional) Interface Protocol

=item C<< "add-cifs-interfacename" => $string >>

(Optional) CIFS Interface Name Add

=item C<< "cifs-homeport" => $string >>

(Optional) CIFS Homeport

=item C<< "cifs-netmask" => $string >>

(Optional) CIFS Netmask

=item C<< "cifs-gateway" => $string >>

(Optional) CIFS Gateway

=item C<< "delete-cifs-interfacename" => $string >>

(Optional) CIFS InterfaceName Delete

=item C<< "cifs-server-name" => $string >>

(Optional) CIFS Server Name

=item C<< "cifs-domain-name" => $string >>

(Optional) CIFS Domain Name

=item C<< "cifs-admin-name" => $string >>

(Optional) CIFS Admin Name

=item C<< "ldap-client" => $string >>

(Optional) LDAP Client

=item C<< "ldap-server" => $string >>

(Optional) LDAP Server

=item C<< "ldap-active-directory-domain" => $string >>

(Optional) LDAP Active Directory Domain

=item C<< "ldap-active-directory-domain-server-add" => $string >>

(Optional) LDAP Active Directory Domain Server Add

=item C<< "ldap-active-directory-domain-server-delete" => $string >>

(Optional) LDAP Active Directory Domain Server Delete

=item C<< "ldap-active-domain-server-add" => $string >>

(Optional) LDAP Active Domain Server Add

=item C<< "ldap-active-domain-server-delete" => $string >>

(Optional) LDAP Active Domain Server Delete

=item C<< "ldap-authentication-level" => $string >>

(Optional) LDAP Authentication Level

=item C<< "ldap-bind-dn" => $string >>

(Optional) LDAP Bind DN

=item C<< "ldap-bind-user-password" => $string >>

(Optional) LDAP Bind User Password

=item C<< "ldap-base-dn" => $string >>

(Optional) LDAP Base DN

=item C<< "ldap-tcp-port" => $string >>

(Optional) LDAP TCP Port

=item C<< "fc-node-name" => $string >>

(Optional) FC Node Name

=item C<< "fc-controller-select" => $string >>

(Optional) FC Controller Select

=item C<< "fc-datainterface-per-controller" => $string >>

(Optional) FC Data Interface Per Controller

=item C<< "fc-portset-number" => $string >>

(Optional) FC Portset Number


=back

=back

=cut

####################### GUI Create ###############################
sub create {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;

    $pkg->_frontend_log_debug_opts(@args);

    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars
                qw(aggregate language protocol delegate-name delegate-password
                delegate-confirm-password delegate-volume delegate-aggregate
                interface-type interface-name homeport ip-address netmask
                gateway primary-domain-name add-search-domain-name
                delete-search-domain-name add-name-servers delete-name-servers
                add-nfs-interface-name nfs-homeport nfs-ip-address nfs-netmask
                nfs-gateway nfs-allow-management delete-nfs-interface-name
                nfs-version name-servers nis-primary-domain-name nis-server
                add-secondary-domains add-secondary-nis-server
                delete-secondary-domains local-users-group-name
                local-users-username local-users-user-id local-users-full-name
                add-mapping mapping-direction mapping-position mapping-pattern
                mapping-replacement edit-mapping edit-mapping-pattern
                edit-mapping-replacement delete-mapping dns-configuration
                iscsi-target-node-name iscsi-target-alias iscsi-controller-name
                iscsi-interface-per-control iscsi-starting-ip iscsi-netmask
                iscsi-gateway iscsi-portset interface-protocol add-cifs-interfacename
                cifs-homeport cifs-ipaddress cifs-netmask cifs-gateway delete-cifs-interfacename
                cifs-server-name cifs-domain-name cifs-admin-name cifs-admin-password
                ldap-client ldap-server ldap-active-directory-domain
                ldap-active-directory-domain-server-add
                ldap-active-directory-domain-server-delete
                ldap-active-domain-server-add ldap-active-domain-server-delete
                ldap-authentication-level ldap-bind-dn ldap-bind-user-password
                ldap-base-dn ldap-tcp-port fc-node-name fc-controller-select
                fc-datainterface-per-controller fc-portset-number ldap-client-select
                ldap-client-add local-users-group-id iscsi-rename interface-disable
                fc-port-rename),
        },
    );

    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method => '_create_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;

} ## end sub create

####################### GUI CMode Create ###############################
sub _create_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = $pkg->_validate_backend_opts(@args);

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my $vserver_args = $pkg->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                      => "-vservername",
            "aggregate"                    => "-aggrname",
            "language"                     => "-lang",
            "protocol"                     => "-protocol",
            "delegate-name"                => "-delegatename",
            "delegate-password"            => "-delegatepwd",
            "delegate-confirm-password"    => "-delegateconfirmpwd",
            "delegate-volume"              => "-delegatevol",
            "delegate-aggregate"           => "-delegateaggr",
            "interface-type"               => "-interface",
            "interface-name"               => "-interfacename",
            "homeport"                     => "-homeport",
            "ip-address"                   => "-ipaddress",
            "netmask"                      => "-netmask",
            "gateway"                      => "-gateway",
            "primary-domain-name"          => "-pridomainname",
            "add-search-domain-name"       => "-addsearchdomainname",
            "delete-search-domain-name"    => "-delsearchdomainname",
            "add-name-servers"             => "-addnameserver",
            "delete-name-servers"          => "-delnameserver",
            "add-nfs-interface-name"       => "-addnfsinterfacename",
            "interface-protocol"           => "-createinterfacefor",
            "add-cifs-interfacename"       => "-addcifsinterfacename",
            "cifs-homeport"                => "-cifshomeport",
            "cifs-ipaddress"               => "-cifsipaddress",
            "cifs-netmask"                 => "-cifsnetmask",
            "cifs-gateway"                 => "-cifsgateway",
            "delete-cifs-interfacename"    => "-delcifsinterfacename",
            "cifs-server-name"             => "-cifsservername",
            "cifs-domain-name"             => "-cifsdomainname",
            "cifs-admin-name"              => "-cifsadminname",
            "cifs-admin-password"          => "-cifsadminpass",
            "name-servers"                 => "-nameservers",
            "ldap-client-select"           => "-ldapclientselect",
            "ldap-client-add"              => "-ldapclientadd",
            "ldap-server"                  => "-ldapserver",
            "ldap-client"                  => "-ldapclient",
            "ldap-active-directory-domain" => "-ldapaddomain",
            "ldap-active-directory-domain-server-add" => "-ldapaddserveradd",
            "ldap-active-directory-domain-server-delete" =>
                "-ldapaddserverdel",
            "ldap-active-domain-server-add"    => "-ldapadsserveradd",
            "ldap-active-domain-server-delete" => "-ldapadsserverdel",
            "ldap-authentication-level"        => "-ldapauthlevel",
            "ldap-bind-dn"                     => "-ldapbinddn",
            "ldap-bind-user-password"          => "-ldapbindupass",
            "ldap-base-dn"                     => "-ldapbasedn",
            "ldap-tcp-port"                    => "-ldaptcpport",
            "local-users-group-name"           => "-localusersgroupname",
            "local-users-group-id"             => "-localusersgroupid",
            "local-users-username"             => "-localusersusername",
            "local-users-user-id"              => "-localusersuserid",
            "local-users-full-name"            => "-localusersfullname",
            "nfs-homeport"                     => "-nfshomeport",
            "nfs-ip-address"                   => "-nfsipaddress",
            "nfs-netmask"                      => "-nfsnetmask",
            "nfs-gateway"                      => "-nfsgateway",
            "nfs-allow-management"             => "-nfsallowmgmt",
            "delete-nfs-interface-name"        => "-delnfsinterfacename",
            "nfs-version"                      => "-nfsversion",
            "nis-primary-domain-name"          => "-nisprimarydomain",
            "nis-server"                       => "-nisserver",
            "add-secondary-domains"            => "-addsecondarydomains",
            "add-secondary-nis-server"         => "-addsecondarynisserver",
            "delete-secondary-domains"         => "-delsecondarydomains",
            "add-mapping"                      => "-addmapping",
            "mapping-direction"                => "-mappingdirection",
            "mapping-position"                 => "-mappingposition",
            "mapping-pattern"                  => "-mappingpattern",
            "mapping-replacement"              => "-mappingreplacement",
            "edit-mapping"                     => "-editmapping",
            "edit-mapping-pattern"             => "-editmappingpattern",
            "edit-mapping-replacement"         => "-editmappingreplacement",
            "delete-mapping"                   => "-delmapping",
            "dns-configuration"                => "-dnsconfig",
            "iscsi-target-node-name"           => "-iscsitargetnodename",
            "iscsi-target-alias"               => "-iscsitargetalias",
            "iscsi-controller-name"            => "-iscsicontrollername",
            "iscsi-interface-per-control"      => "-iscsiinterfacepercntrl",
            "iscsi-starting-ip"                => "-iscsistartingip",
            "iscsi-netmask"                    => "-iscsinetmask",
            "iscsi-gateway"                    => "-iscsigateway",
            "iscsi-portset"                    => "-iscsiportset",
            "iscsi-rename"                     => "-iscsiportrename",
            "interface-disable"                => "-interf_disable",
            "fc-node-name"                     => "-fcnodename",
            "fc-controller-select"             => "-fccontrollersel",
            "fc-datainterface-per-controller" =>
                "-fcdatainterfacepercontroller",
            "fc-portset-number" => "-fcportsetnumber",
            "fc-port-rename"    => "-fcportrename",
        }
    );

    $apiset->vserver_create(%$vserver_args);
    $Log->exit() if $may_exit;

    return $pkg->new(
        vserver           => $vserver_args->{-vservername},
        command_interface => $command_interface
    );

} ## end sub _create_cmode_gui

=head2 delete

	NACL::C::SysManager::Vserver->delete(command_interface => $ci,
										 vserver           => $vserver_name,...);

                            or

	$vserver_object->delete(...);

(Class or instance method)This method deletes a vserver element.
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver=> $string >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
The name of the vserver


=back

=back

=cut

####################### GUI Delete ###############################
sub delete {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_delete_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

####################### GUI CMode Delete ###############################
sub _delete_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->vserver_delete(%$vserver_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_gui

=head2 modify

	NACL::C::SysManager::Vserver->modify(command_interface => $ci,
									     vserver           => $vserver_name,...);

                            or

	$vserver_object->modify(...);

(Class or instance method) Modify a vserver element.  
Uses a CMode GUI APISet.

=over

=item Options

=over

=item C<< command_interface=>$command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

=item C<< "rename" => $string >>

(Optional) Rename the Vserver

=item C<< "name-server-switch-nis" => $string >>

(Optional) Enable Name Server Switch NIS

=item C<< "name-server-switch-file" => $string >>

(Optional) Enable Name Server Switch File

=item C<< "name-server-switch-ldap" => $string >>

(Optional) Enable Name Server Switch LDAP

=item C<< "name-mapping-switch-file" => $string >>

(Optional) Enable Name Mapping Switch File

=item C<< "name-mapping-switch-ldap" => $string >>

(Optional) Enable Name Mapping Switch LDAP

=item C<< "data-protocol-cifs" => $string >>

(Optional) Enable Data Protocol CIFS

=item C<< "data-protocol-nfs" => $string >>

(Optional) Enable Data Protocol NFS

=item C<< "data-protocol-iscsi" => $string >>

(Optional) Enable Data Protocol iSCSI

=item C<< "data-protocol-fc" => $string >>

(Optional) Enable Data Protocol FC

=item C<< "volume-creation-type" => $string >>

(Optional) Volume Creation Type

=item C<< "delegate-aggregate-name" => $string >>

(Optional) Delegate Aggregate Name


=back

=back

=cut

####################### GUI Modify ###############################
sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw (rename name-server-switch-nis name-server-switch-file
                    name-server-switch-ldap name-mapping-switch-file
                    name-mapping-switch-ldap data-protocol-cifs
                    data-protocol-nfs data-protocol-iscsi data-protocol-fc
                    volume-creation-type delegate-aggregate-name)
            ),
        },
    );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_modify_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

####################### GUI CMode Modify ###############################
sub _modify_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "vserver"                  => "-vservername",
            "rename"                   => "-changename",
            "name-server-switch-nis"   => "-nssnis",
            "name-server-switch-file"  => "-nssfile",
            "name-server-switch-ldap"  => "-nssldap",
            "name-mapping-switch-file" => "-nmsfile",
            "name-mapping-switch-ldap" => "-nmsldap",
            "data-protocol-cifs"       => "-cifs",
            "data-protocol-nfs"        => "-nfs",
            "data-protocol-iscsi"      => "-iscsi",
            "data-protocol-fc"         => "-fc",
            "volume-creation-type"     => "-volcreation",
            "delegate-aggregate-name"  => "-delegateaggrname",
        }
    );

    $apiset->vserver_edit(%$vserver_args);

    if ( ( defined $vserver_args->{'-changename'} ) && ( ref $pkg_or_obj ) ) {
        $pkg_or_obj->vserver( $opts{'rename'} );
    }
    
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_gui

=head2 stop

	NACL::C::SysManager::Vserver->stop(command_interface => $ci,
									   vserver           => $vserver_name,...);

                            or

	$vserver_object->stop(...);

(Class or instance method) This method stops a vserver element.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
Name of vserver.

=back

=back

=cut

####################### GUI Stop ###############################
sub stop {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_stop_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

####################### GUI CMode Stop ###############################
sub _stop_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->vserver_stop(%$vserver_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_gui

=head2 start

	NACL::C::SysManager::Vserver->start(command_interface => $ci,
									    vserver           => $vserver_name,...);

                            or

	$vserver_object->start(...);

(Class or instance method) This method starts a vserver element.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $string >>

(Required for class method, Not Applicable for instance method)
Name of vserver.


=back

=back

=cut

####################### GUI Start ###############################
sub start {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_start_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

####################### GUI CMode Start ###############################
sub _start_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername", }
    );

    $apiset->vserver_start(%$vserver_args);
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_gui

=head2 refresh

	NACL::C::SysManager::Vserver->refresh(command_interface => $ci,...);

                            or

	$vserver_object->refresh();

(Class or instance method) This method refreshs a vserver page.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

####################### GUI Refresh ###############################
sub refresh {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_refresh_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub refresh

####################### GUI CMode Refresh ###############################
sub _refresh_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->vserver_refresh(%$vserver_args);
    $Log->exit() if $may_exit;
} ## end sub _refresh_cmode_gui

=head2 clear

	NACL::C::SysManager::Vserver->clear(command_interface => $ci,...);

                            or

	$vserver_object->clear();

(Class or instance method) This method clears a vserver page.
Uses a CMode GUI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=back

=back

=cut

sub clear {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);

    my %opts = $pkg_or_obj->_common_validate_with( params => \@args, );

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method => '_clear_cmode_gui',
                set    => 'SysManager_CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub clear

####################### GUI CMode Clear ###############################
sub _clear_cmode_gui {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    my $vserver_args = $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => { "vserver" => "-vservername" }
    );

    $apiset->clear_vserver_window(%$vserver_args);
    $Log->exit() if $may_exit;
} ## end sub _clear_cmode_gui

####################### Primary Keys Validate ###############################
sub _primary_keys_validate_spec {
    return ( vserver => { type => SCALAR }, );
}

1;
