# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary System Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::System

=head1 DESCRIPTION

C<NACL::C::System> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.

=head1 ATTRIBUTES

None

=cut

package NACL::C::System;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR SCALARREF ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 setup

   NACL::C::System->setup(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to setup wizard.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub setup {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(params => \@args,);
    $Log->exit() if $may_exit;

    return $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_setup_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );
}

sub _setup_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw(
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug("Opts to API call 'system_setup' are: \n" . Dumper(\%opts));

    $apiset->system_setup(%opts);
    $Log->exit() if $may_exit;
}

=head2 hostname

    NACL::C::System->hostname(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to show or set the hostname.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'newname' => $string >>

New Name possible value(s) are, text

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub hostname {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( newname
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_hostname_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::hostname',
    );

    $Log->exit() if $may_exit;
}

sub _hostname_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( newname extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->system_hostname(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 timezone

    NACL::C::System->timezone(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to system Timezone.

Uses a CMode CLI/ZAPI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'timezone' => $string >>

Timezone Name possible value(s) are, Area/Location Timezone

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<clock-get-timezone>
<timezone>$value</timezone>
</clock-get-timezone>

=item C<< 'version' => $string >>

Timezone Version possible value(s) are, true

Applicable for CMode CLI.

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub timezone {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( timezone version
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_timezone_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_timezone_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::System::timezone',
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _timezone_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( timezone version extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response         = $apiset->system_timezone(%cmode_args);
    my $processed_output = $response->get_parsed_output();
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }
    $Log->exit() if $may_exit;
    return $processed_output->[0];
}

sub _timezone_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $allowed_cmode_options = [
        qw( timezone
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response         = $apiset->clock_get_timezone(%cmode_args);
    my $processed_output = $response->get_parsed_output();

    $Log->exit() if $may_exit;
    return $processed_output->[0];
}

=head2 uichanges

    my $uichanges_obj = NACL::C::System->uichanges(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

    my @uichanges_objs = NACL::C::System->uichanges(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or

   my $uichanges_obj = $System_obj->uichanges();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemUichanges-pm/ATTRIBUTES>
of NACL::CS::SystemUichanges as input.
This method returns objects of the type L<NACL::CS::SystemUichanges|lib-NACL-CS-SystemUichanges-pm>.

=cut

sub uichanges {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::SystemUichanges;
    my @state_objs = NACL::CS::SystemUichanges->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 node_halt

    NACL::C::System->node_halt(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $System_obj->node_halt(...);

(Class or instance method)
This method is used to shut down a node.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'dump' => $string >>

Create a Core Dump 
Applicable for CMode CLI

=item C<< 'inhibit-takeover' => $string >>

Disallow Storage Takeover by Partner
Applicable for CMode CLI

=item C<< 'skip-lif-migration' => $string >>

Skip Migrating LIFs Away from Node
Applicable for CMode CLI

=item C<< 'reason' => $string >>

Reason for Shutdown
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node halt" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub node_halt {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( dump inhibit-takeover skip-lif-migration reason

                    node
                    )
            ),
        },
         cli_cmd => 'system node halt',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_node_halt_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::node_halt',
    );

    $Log->exit() if $may_exit;
}

sub _node_halt_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_halt',
    );
    $Log->exit() if $may_exit;
}

=head2 node_modify

    NACL::C::System->node_modify(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $System_obj->node_modify(...);

(Class or instance method)
This method is used to modify node attributes.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'owner' => $string >>

Owner
Applicable for CMode CLI

=item C<< 'eligibility' => $string >>

Eligibility possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'location' => $string >>

Location
Applicable for CMode CLI

=item C<< 'epsilon' => $string >>

Epsilon possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'assettag' => $string >>

Asset Tag
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node modify" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub node_modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( owner eligibility location epsilon assettag

                    node
                    )
            ),
        },
         cli_cmd => 'system node modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_node_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::node_modify',
    );

    $Log->exit() if $may_exit;
}

sub _node_modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_modify',
    );
    $Log->exit() if $may_exit;
}

=head2 node_reboot

    NACL::C::System->node_reboot(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $System_obj->node_reboot(...);

(Class or instance method)
This method is used to reboot a node.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'dump' => $string >>

Create a Core Dump
Applicable for CMode CLI

=item C<< 'inhibit-takeover' => $string >>

Disallow Storage Takeover by Partner
Applicable for CMode CLI

=item C<< 'skip-lif-migration' => $string >>

Skip Migrating LIFs Away from Node
Applicable for CMode CLI

=item C<< 'reason' => $string >>

Reason for Reboot
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node reboot" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub node_reboot {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( dump inhibit-takeover skip-lif-migration reason

                    node
                    )
            ),
        },
           cli_cmd => 'system node reboot',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_node_reboot_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::node_reboot',
    );

    $Log->exit() if $may_exit;
}

sub _node_reboot_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_reboot',
    );
    $Log->exit() if $may_exit;
}

=head2 node_rename

    NACL::C::System->node_rename(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $System_obj->node_rename(...);

(Class or instance method)
This method is used to rename a node.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'newname' => $string >>

New Name
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node rename" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub node_rename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( newname

                    node
                    )
            ),
        },
         cli_cmd => 'system node rename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_node_rename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::node_rename',
    );

    $Log->exit() if $may_exit;
}

sub _node_rename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_node_rename',
    );
    $Log->exit() if $may_exit;
}

=head2 cluster_setup

    NACL::C::System->cluster_setup(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to setup wizard.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "cluster setup" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub cluster_setup {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {'cmd_execution_time' => {type => SCALARREF, optional => 1},},
        ignore_primary_keys => 1,
         cli_cmd                 => 'cluster setup',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_cluster_setup_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::System::cluster_setup',
    );

    $Log->exit() if $may_exit;
}

sub _cluster_setup_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api     => 'cluster_setup',
    );
    $Log->exit() if $may_exit;
}

=head2 node_show

    my $node_show_obj = NACL::C::System->node_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @node_show_objs = NACL::C::System->node_show(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $node_show_obj = $System_obj->node_show(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-SystemNodeShow-pm/ATTRIBUTES>
of NACL::CS::SystemNodeShow as input. 
This method returns objects of the type L<NACL::CS::SystemNodeShow|lib-NACL-CS-SystemNodeShow-pm>.

=cut

sub node_show {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::SystemNodeShow;
    my @state_objs = NACL::CS::SystemNodeShow->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
