#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemCapability Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemCapability

=head1 DESCRIPTION

C<NACL::C::SystemCapability> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemCapability.  A related class is
L<NACL::CS::SystemCapability|lib-NACL-CS-SystemCapability-pm>, which represents
the past state of an ONTAP SystemCapability.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=head2 capability

=cut

package NACL::C::SystemCapability;

use strict;
use warnings;

use NACL::CS::SystemCapability;
use NACL::ComponentUtils qw (_optional_scalars);
use Data::Dumper;

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemCapability = NACL::C::SystemCapability->new(command_interface=>$ci,
                                   node =>$node,
                                   capability =>$capability,
                                  );

Return a new SystemCapability component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'capability',
];

=head2 find

    my $SystemCapability  = NACL::C::SystemCapability->find(command_interface => $ci, ...);
    my @SystemCapabilitys = NACL::C::SystemCapability->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemCapability_state = $NACL::C::SystemCapability->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemCapability would return
a NACL::CS::SystemCapability object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 software_update_check_shim

   NACL::C::SystemCapability->software_update_check_shim(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Software update image check.

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-software-update-check-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "image-name" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: image-name

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability software-update-check-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub software_update_check_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( image-name
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability software-update-check-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_software_update_check_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_software_update_check_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::software_update_check_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub software_update_check_shim

sub _software_update_check_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_software_update_check_shim',);
    $Log->exit() if $may_exit;
} ## end sub _software_update_check_shim_cmode_cli

sub _software_update_check_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(image-name)],
        api  => 'capability-software-update-check-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _software_update_check_shim_cmode_zapi

=head2 replicate_manifest_for_join_shim

   NACL::C::SystemCapability->replicate_manifest_for_join_shim(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to uNIT TEST: Replicate manifest for join..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-replicate-manifest-for-join-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability replicate-manifest-for-join-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub replicate_manifest_for_join_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'system capability replicate-manifest-for-join-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_replicate_manifest_for_join_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_replicate_manifest_for_join_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend =>
            'NACL::C::SystemCapability::replicate_manifest_for_join_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub replicate_manifest_for_join_shim

sub _replicate_manifest_for_join_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_replicate_manifest_for_join_shim',);
    $Log->exit() if $may_exit;
} ## end sub _replicate_manifest_for_join_shim_cmode_cli

sub _replicate_manifest_for_join_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-replicate-manifest-for-join-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _replicate_manifest_for_join_shim_cmode_zapi

=head2 is_enabled_during_downgrade_shim

   NACL::C::SystemCapability->is_enabled_during_downgrade_shim(
                                command_interface => $ci,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to UNIT TEST: Peek through hole in capability fence..

Uses a CMode CLI/ZAPI APISet.

Invokes capability-is-enabled-during-downgrade-shim api for CMode ZAPI.

This method return's '1' or '0' for CMode CLI.
It will return 1 if the capability is enabled during downgrade shim,
otherwise it will return 0.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "capability" => $capability >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: capability

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

(Required)

=item C<< '*' >>

This method takes all other options supported by
"system capability is-enabled-during-downgrade-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub is_enabled_during_downgrade_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( capability
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd => 'system capability is-enabled-during-downgrade-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_is_enabled_during_downgrade_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_is_enabled_during_downgrade_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend =>
            'NACL::C::SystemCapability::is_enabled_during_downgrade_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub is_enabled_during_downgrade_shim

sub _is_enabled_during_downgrade_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(
        @opts,
        apiset_response_obj => \$response,
        api => 'system_capability_is_enabled_during_downgrade_shim',);
    my $output = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $output->[0]->{'is_enabled'};
} ## end sub _is_enabled_during_downgrade_shim_cmode_cli

sub _is_enabled_during_downgrade_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(capability)],
        api  => 'capability-is-enabled-during-downgrade-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _is_enabled_during_downgrade_shim_cmode_zapi

=head2 canbe_enabled

   NACL::C::SystemCapability->canbe_enabled(
                                command_interface => $ci,
                                 cap-in => $cap_in,
                                 cap-out => $cap_out,
                                  ...);

(Class method)
This method is used to uNIT TEST: Given a list of capabilities return which ones can be enabled..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-can-be-enabled' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "cap-in" => [ $cap-in1, $cap-in2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cap-in

=item C<< "cap-out" => [ $cap-out1, $cap-out2, ... ] >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability canbe-enabled" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub canbe_enabled {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "cap-in"  => {type => ARRAYREF, optional => 1},
            "cap-out" => {type => ARRAYREF, optional => 1},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability canbe-enabled',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_canbe_enabled_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_canbe_enabled_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::canbe_enabled',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub canbe_enabled

sub _canbe_enabled_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_canbe_enabled',);
    $Log->exit() if $may_exit;
} ## end sub _canbe_enabled_cmode_cli

sub _canbe_enabled_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(cap-in)],
        api  => 'capability-can-be-enabled',
    );
    $Log->exit() if $may_exit;
} ## end sub _canbe_enabled_cmode_zapi

=head2 enable

   NACL::C::SystemCapability->enable(
                                command_interface => $ci,
                                 cap-in => $cap_in,
                                 all-or-none => $all_or_none,
                                     cap-out => $cap_out,
                                  ...);

(Class method)
This method is used to uNIT TEST: Enable for every node in the cluster if and only if every node has the capability..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-enable' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "all-or-none" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: all-or-none

=item C<< "cap-in" => [ $cap-in1, $cap-in2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cap-in

=item C<< "cap-out" => [ $cap-out1, $cap-out2, ... ] >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)
Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability enable" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "cap-in"  => {type => ARRAYREF, optional => 1},
            "cap-out" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( all-or-none
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability enable',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_enable_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_enable_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::enable',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub enable

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_enable',);
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_cli

sub _enable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(all-or-none cap-in)],
        api  => 'capability-enable',
    );
    $Log->exit() if $may_exit;
} ## end sub _enable_cmode_zapi

=head2 create

    my $SystemCapability_object    = NACL::C::SystemCapability->create(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "require-for-boot" => $string >>

Applicable for CMode CLI.

=item C<< "description" => $string >>

Applicable for CMode CLI.

=item C<< "auto-disable" => $string >>

Applicable for CMode CLI.

=item C<< "obsolete" => $string >>

Applicable for CMode CLI.

=item C<< "marked-downgrade" => $string >>

Applicable for CMode CLI.

=item C<< "howto-downgrade" => $string >>

Applicable for CMode CLI.

=item C<< "required-on-all-nodes" => $string >>

Applicable for CMode CLI.

=item C<< "release-debut" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< 'auto-enable' => $string >>

Applicable for CMode CLI.

=item C<< 'enable-ready' => $string >>

Applicable for CMode CLI.

=item C<< 'uses-enable-ready' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability create" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( require-for-boot description auto-disable obsolete marked-downgrade
                    howto-downgrade required-on-all-nodes release-debut enabled

                    auto-enable enable-ready uses-enable-ready
                    )
            ),
        },
        cli_cmd => 'system capability create',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::create',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'system_capability_create',);
    $Log->exit() if $may_exit;
    return $pkg_or_obj->new(
        command_interface => $opts{command_interface},
        node              => $opts{'node'},
        capability        => $opts{'capability'},
    );
} ## end sub _create_cmode_cli

=head2 delete

   NACL::C::SystemCapability->delete(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

                                 or

    $SystemCapability_object->delete(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'system capability delete',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 recommend_release_shim

   NACL::C::SystemCapability->recommend_release_shim(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Recommend a release having specified capabilities..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-recommend-release-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "cap-in" => [ $cap-in1, $cap-in2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cap-in

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability recommend-release-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub recommend_release_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {"cap-in" => {type => ARRAYREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability recommend-release-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_recommend_release_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_recommend_release_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::recommend_release_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub recommend_release_shim

sub _recommend_release_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_capability_recommend_release_shim',);
    $Log->exit() if $may_exit;
} ## end sub _recommend_release_shim_cmode_cli

sub _recommend_release_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(cap-in)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-recommend-release-shim',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _recommend_release_shim_cmode_zapi

=head2 modify

   NACL::C::SystemCapability->modify(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

                                 or

    $SystemCapability_object->modify(...);

(Class or instance method)

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "require-for-boot" => $string >>

Applicable for CMode CLI.

=item C<< "description" => $string >>

Applicable for CMode CLI.

=item C<< "auto-disable" => $string >>

Applicable for CMode CLI.

=item C<< "obsolete" => $string >>

Applicable for CMode CLI.

=item C<< "marked-downgrade" => $string >>

Applicable for CMode CLI.

=item C<< "howto-downgrade" => $string >>

Applicable for CMode CLI.

=item C<< "required-on-all-nodes" => $string >>

Applicable for CMode CLI.

=item C<< "release-debut" => $string >>

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< 'auto-enable' => $string >>

Applicable for CMode CLI.

=item C<< 'enable-ready' => $string >>

Applicable for CMode CLI.

=item C<< 'uses-enable-ready' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)
Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( require-for-boot description auto-disable obsolete marked-downgrade
                    howto-downgrade required-on-all-nodes release-debut enabled

                    auto-enable enable-ready uses-enable-ready
                    )
            ),
        },
        cli_cmd => 'system capability modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

=head2 is_enabled

   NACL::C::SystemCapability->is_enabled(
                                command_interface => $ci,
                                 cap-in => $cap_in,
                                 cap-out => $cap_out,
                                  ...);

(Class method)
This method is used to uNIT TEST: Given a list of capabilities return which ones are enabled..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-is-enabled' api for CMode ZAPI.

This method returns array of is-enabled capabilities for CMode CLI/ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "cap-in" => [ $cap-in1, $cap-in2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cap-in

=item C<< "cap-out" => [ $cap-out1, $cap-out2, ... ] >>

Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item C<< '*' >>

This method takes all other options supported by
"system capability is-enabled" command in ONTAP.

Applicable for CMode CLI.

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub is_enabled {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "cap-in"  => {type => ARRAYREF, optional => 1},
            "cap-out" => {type => ARRAYREF, optional => 1},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability is-enabled',
    );
    my @ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_is_enabled_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_is_enabled_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::is_enabled',
    );

    $Log->exit() if $may_exit;
    return @ret;
} ## end sub is_enabled

sub _is_enabled_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(
        @opts,
        apiset_response_obj => \$response,
        api                 => 'system_capability_is_enabled',
    );
    my $output   = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return @{$output->[0]->{'is_enabled'}};
} ## end sub _is_enabled_cmode_cli

sub _is_enabled_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $response;
    my $copy = [qw(cap-in cap-out)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-is-enabled',
        copy => $copy,
        apiset_response_obj => \$response,
    );

    my $processed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    if (defined $processed_output->[0]{'cap-out'}->[0]{'string'}) {
        return @{$processed_output->[0]{'cap-out'}->[0]{'string'}};
    } else {
        return ['-'];
    }
} ## end sub _is_enabled_cmode_zapi

=head2 can_join_shim

   NACL::C::SystemCapability->can_join_shim(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: simulate what join code will do..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "local-ip" => $string >>

Applicable for CMode CLI.

=item C<< "member-ip" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability can-join-shim" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub can_join_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( local-ip member-ip
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability can-join-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_can_join_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => '_can_join_shim_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },

        ],
        frontend => 'NACL::C::SystemCapability::can_join_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub can_join_shim

sub _can_join_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_capability_can_join_shim',);
    $Log->exit() if $may_exit;
} ## end sub _can_join_shim_cmode_cli

sub _can_join_shim_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts, api => 'capability-can-join-shim',);
    $Log->exit() if $may_exit;
}

=head2 downgrade_commit_shim

   NACL::C::SystemCapability->downgrade_commit_shim(
                                command_interface => $ci,
                                 missing_capabilities => $missing_capabilities,
                                  ...);

(Class method)
This method is used to uNIT TEST: Replicate the default image's manifest..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-downgrade-commit-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "cluster-wide" => $string >>

Applicable for CMode CLI.

=item C<< "missing_capabilities" => [ $missing_capabilitie1, $missing_capabilitie2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: missing_capabilities

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability downgrade-commit-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub downgrade_commit_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "missing_capabilities" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( cluster-wide
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability downgrade-commit-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_downgrade_commit_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_downgrade_commit_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::downgrade_commit_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub downgrade_commit_shim

sub _downgrade_commit_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_downgrade_commit_shim',);
    $Log->exit() if $may_exit;
} ## end sub _downgrade_commit_shim_cmode_cli

sub _downgrade_commit_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(missing_capabilities)],
        api  => 'capability-downgrade-commit-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _downgrade_commit_shim_cmode_zapi

=head2 downgrade_prepare_shim

   NACL::C::SystemCapability->downgrade_prepare_shim(
                                command_interface => $ci,
                                 missing_capabilities => $missing_capabilities,
                                  ...);

(Class method)
This method is used to uNIT TEST: Prevent capabilities from be used during revert/downgrade..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-downgrade-prepare-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "missing_capabilities" => [ $missing_capabilitie1, $missing_capabilitie2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: missing_capabilities

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability downgrade-prepare-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub downgrade_prepare_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {"missing_capabilities" => {type => ARRAYREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability downgrade-prepare-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_downgrade_prepare_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_downgrade_prepare_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::downgrade_prepare_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub downgrade_prepare_shim

sub _downgrade_prepare_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_downgrade_prepare_shim',);
    $Log->exit() if $may_exit;
} ## end sub _downgrade_prepare_shim_cmode_cli

sub _downgrade_prepare_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(missing_capabilities)],
        api  => 'capability-downgrade-prepare-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _downgrade_prepare_shim_cmode_zapi

=head2 load_manifest_shim

   NACL::C::SystemCapability->load_manifest_shim(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Replicate local capability manifest..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-load-manifest-shim' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "image" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: image

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability load-manifest-shim" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub load_manifest_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( image
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability load-manifest-shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_load_manifest_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_load_manifest_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::load_manifest_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub load_manifest_shim

sub _load_manifest_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_load_manifest_shim',);
    $Log->exit() if $may_exit;
} ## end sub _load_manifest_shim_cmode_cli

sub _load_manifest_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(image)],
        api  => 'capability-load-manifest-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _load_manifest_shim_cmode_zapi

=head2 can_boot_shim

   NACL::C::SystemCapability->can_boot_shim(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Check if local node can continue booting..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-can-boot-shim' api for CMode ZAPI.

This method return 'yes' for CMode CLI if local node can continue booting.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability can-boot-shim" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub can_boot_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'system_capability_can_boot_shim',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_can_boot_shim_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_can_boot_shim_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::can_boot_shim',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub can_boot_shim

sub _can_boot_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $response;
    $pkg_or_obj->_base_backend_cmode_cli(
        @opts,
        apiset_response_obj => \$response,
        api => 'system_capability_can_boot_shim',);

    my $output   = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $output->[0]->{'can_boot'};
} ## end sub _can_boot_shim_cmode_cli

sub _can_boot_shim_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-can-boot-shim',
    );
    $Log->exit() if $may_exit;
} ## end sub _can_boot_shim_cmode_zapi

=head2 disable

   NACL::C::SystemCapability->disable(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Disable capabilities for every node..

Uses a CMode CLI/ZAPI APISet.

Invokes 'capability-disable' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "capabilities" => [ $capabilitie1, $capabilitie2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: capabilities

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability disable" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {"capabilities" => {type => ARRAYREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability disable',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_disable_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_disable_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::disable',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub disable

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_disable',);
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_cli

sub _disable_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(capabilities)],
        api  => 'capability-disable',
    );
    $Log->exit() if $may_exit;
} ## end sub _disable_cmode_zapi

=head2 get_clusterset

   NACL::C::SystemCapability->get_clusterset(
                                command_interface => $ci,
                                 node => $node,
                                 capability => $capability,
                                  ...);

(Class method)
This method is used to uNIT TEST: Get list of capabilities which are installed on all nodes..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "capability" => $capability >>

(Required)

Applicable for CMode CLI.

=item C<< "capabilities" => [ $capabilitie1, $capabilitie2, ... ] >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability get-clusterset" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub get_clusterset {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        additional_spec =>
            {"capabilities" => {type => ARRAYREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability get-clusterset',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_get_clusterset_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_get_clusterset_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemCapability::get_clusterset',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub get_clusterset

sub _get_clusterset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_capability_get_clusterset',);
    $Log->exit() if $may_exit;
} ## end sub _get_clusterset_cmode_cli

sub _get_clusterset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-get-cluster-set',
    );
    $Log->exit() if $may_exit;
} ## end sub _get_clusterset_cmode_zapi

=head2 is_local_node_enable_ready

    NACL::C::SystemCapability->is_local_node_enable_ready(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to uNIT TEST: Given a list of capabilities return which ones are enable-ready..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cap-in' => [ $cap-in1, $cap-in2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI/ZAPI.

=item C<< 'cap-out' => [ $cap-out1, $cap-out2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI/ZAPI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability is-local-node-enable-ready" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub is_local_node_enable_ready {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cap-in'  => {type => ARRAYREF, optional => 1},
            'cap-out' => {type => ARRAYREF, optional => 1},
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability is-local-node-enable-ready',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_is_local_node_enable_ready_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method => '_is_local_node_enable_ready_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },

        ],
        frontend => 'NACL::C::SystemCapability::is_local_node_enable_ready',
    );

    $Log->exit() if $may_exit;
} ## end sub is_local_node_enable_ready

sub _is_local_node_enable_ready_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_capability_is_local_node_enable_ready',);
    $Log->exit() if $may_exit;
} ## end sub _is_local_node_enable_ready_cmode_cli

sub _is_local_node_enable_ready_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(cap-in cap-out)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-is-local-node-enable-ready',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


=head2 mark_local_node_enable_ready

    NACL::C::SystemCapability->mark_local_node_enable_ready(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to uNIT TEST: Mark the given list of capabilities as enable-ready..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cap-in' => [ $cap-in1, $cap-in2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI/ZAPI.

=item C<< 'cap-out' => [ $cap-out1, $cap-out2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)

Applicable for CMode CLI/ZAPI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)
 
Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system capability mark-local-node-enable-ready" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub mark_local_node_enable_ready {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cap-in'  => {type => ARRAYREF, optional => 1},
            'cap-out' => {type => ARRAYREF, optional => 1},
        },
        ignore_primary_keys => 1,
        cli_cmd => 'system capability mark-local-node-enable-ready',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_mark_local_node_enable_ready_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method => '_mark_local_node_enable_ready_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },

        ],
        frontend => 'NACL::C::SystemCapability::mark_local_node_enable_ready',
    );

    $Log->exit() if $may_exit;
} ## end sub mark_local_node_enable_ready

sub _mark_local_node_enable_ready_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_capability_mark_local_node_enable_ready',);
    $Log->exit() if $may_exit;
} ## end sub _mark_local_node_enable_ready_cmode_cli

sub _mark_local_node_enable_ready_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(cap-in cap-out)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'capability-mark-local-node-enable-ready',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}


=head2 can_be_enabled_shim

    NACL::C::SystemCapability->can_be_enabled_shim(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to uNIT TEST: Peek to check that a capability can be enabled..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'cap_in' => [ $cap_in1, $cap_in2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system capability can be enabled shim" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub can_be_enabled_shim {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {'cap_in' => {type => ARRAYREF, optional => 1},},
        ignore_primary_keys => 1,
        cli_cmd             => 'system capability can-be-enabled-shim',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_can_be_enabled_shim_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_can_be_enabled_shim_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },

        ],
        frontend => 'NACL::C::SystemCapability::can_be_enabled_shim',
    );

    $Log->exit() if $may_exit;
}

sub _can_be_enabled_shim_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_capability_can_be_enabled_shim',);
    $Log->exit() if $may_exit;
}

sub _can_be_enabled_shim_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(@opts,
        api => 'capability-can-be-enabled-shim',);
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts,
        _primary_keys => [qw(node capability)],);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'node'       => {type => SCALAR},
        'capability' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
