# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthPolicyDefinition Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemHealthPolicyDefinition

=head1 DESCRIPTION

C<NACL::C::SystemHealthPolicyDefinition> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemHealthPolicyDefinition.  A related class is
L<NACL::CS::SystemHealthPolicyDefinition|lib-NACL-CS-SystemHealthPolicyDefinition-pm>, which represents
the past state of an ONTAP SystemHealthPolicyDefinition.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 policy_id

=head2 monitor

=head2 node

=cut

package NACL::C::SystemHealthPolicyDefinition;

use strict;
use warnings;

use NACL::CS::SystemHealthPolicyDefinition;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemHealthPolicyDefinition = NACL::C::SystemHealthPolicyDefinition->new(command_interface=>$ci,
                                   policy_id =>$policy_id,
                                   monitor =>$monitor,
                                   node =>$node,
                                  );

Return a new SystemHealthPolicyDefinition component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'policy_id',
    scalar => 'monitor',
    scalar => 'node',
];

=head2 find

    my $SystemHealthPolicyDefinition  = NACL::C::SystemHealthPolicyDefinition->find(command_interface => $ci, ...);
    my @SystemHealthPolicyDefinitions = NACL::C::SystemHealthPolicyDefinition->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemHealthPolicyDefinition_state = $NACL::C::SystemHealthPolicyDefinition->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemHealthPolicyDefinition would return
a NACL::CS::SystemHealthPolicyDefinition object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SystemHealthPolicyDefinition->modify(
                                command_interface => $ci,
                                 policy_id => $policy_id,
                                 monitor => $monitor,
                                 node => $node,
                                  ...);

                                 or

    $SystemHealthPolicyDefinition_object->modify(...);

(Class or instance method)

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes 'diagnosis-policy-modify' API for CMode ZAPI.

Invokes 'system health policy definition modify' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "policy-id" => $policy_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: policy-id

7M CLI : -policy id <value> (Programmatic name: policy id)

=item C<< "monitor" => $monitor >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: monitor

7M CLI : -monitor <value> (Programmatic name: monitor)

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: node

7M CLI : node

=item C<< "enable" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: enable

7M CLI : -enable <value> (Programmatic name: enable)

=item C<< 'process' => $string >>

Applicable for CMode CLI.

=item C<< 'asup-enable' => $string >>

Enable ASUP for this alert, possible value(s) are true,false

Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system health policy definition modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enable

                    process

                    asup-enable
                    )
            ),
        },
        cli_cmd => 'system health policy definition modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_modify_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            }
        ],
        frontend => 'NACL::C::SystemHealthPolicyDefinition::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'system_health_policy_definition_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( enable policy-id monitor node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'diagnosis_policy_modify',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( enable policy-id monitor node
            )
    ];

    my %mode7_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode7_args,
    );
    %opts = %mode7_args;

    $apiset->system_health_policy_definition_modify(%opts);
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'policy-id' => {type => SCALAR},
        'monitor'   => {type => SCALAR},
        'node'      => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
