#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthSubscriptions Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemHealthSubscriptions

=head1 DESCRIPTION

C<NACL::C::SystemHealthSubscriptions> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemHealthSubscriptions.  A related class is
L<NACL::CS::SystemHealthSubscriptions|lib-NACL-CS-SystemHealthSubscriptions-pm>, which represents
the past state of an ONTAP SystemHealthSubscriptions.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 event_type

=head2 subscription_id

=head2 monitor

=head2 node

=head2 notify_dest_node

=head2 notify_dest_hm

=head2 class_name

=cut

package NACL::C::SystemHealthSubscriptions;

use strict;
use warnings;
use NACL::CS::SystemHealthSubscriptions;
use NACL::ComponentUtils qw (_optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $SystemHealthSubscriptions = NACL::C::SystemHealthSubscriptions->new(command_interface=>$ci,
                                   event_type =>$event_type,
                                   subscription_id =>$subscription_id,
                                   monitor =>$monitor,
                                   node =>$node,
                                   notify_dest_node =>$notify_dest_node,
                                   notify_dest_hm =>$notify_dest_hm,
                                   class_name => $class_name,
                                  );

Return a new SystemHealthSubscriptions component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'event_type',
    scalar => 'subscription_id',
    scalar => 'monitor',
    scalar => 'node',
    scalar => 'notify_dest_node',
    scalar => 'notify_dest_hm',
    scalar => 'class_name',
];

=head2 find

    my $SystemHealthSubscriptions  = NACL::C::SystemHealthSubscriptions->find(command_interface => $ci, ...);
    my @SystemHealthSubscriptionss = NACL::C::SystemHealthSubscriptions->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemHealthSubscriptions_state = $NACL::C::SystemHealthSubscriptions->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemHealthSubscriptions would return
a NACL::CS::SystemHealthSubscriptions object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::SystemHealthSubscriptions->modify(
                                command_interface => $ci,
                                 event_type => $event_type,
                                 subscription_id => $subscription_id,
                                 monitor => $monitor,
                                 node => $node,
                                 notify_dest_node => $notify_dest_node,
                                 notify_dest_hm => $notify_dest_hm,
                                     class_name => $class_name,
                                  ...);

                                 or

    $SystemHealthSubscriptions_object->modify(...);

(Class or instance method)
This method is used to modify a System Health Subscription.

Uses a CMode CLI APISet or CMode ZAPI APISet.

Invokes 'diagnosis-subscriptions-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "event-type" => $event_type >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: event-type

=item C<< "subscription-id" => $subscription_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: subscription-id

=item C<< "monitor" => $monitor >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: monitor

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< "notify-dest-node" => $notify_dest_node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-dest-node

=item C<< "notify-dest-hm" => $notify_dest_hm >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-dest-hm

=item C<< "class-name" => $class_name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: class-name

=item C<< "creation-time" => $string >>

Applicable for CMode CLI.

=item C<< "max-notify-period" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-notify-period

=item C<< "psc-option" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: psc-option

=item C<< "notify-table" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-table

=item C<< "fail-thresh" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: fail-thresh

=item C<< "instance-name" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: instance-name

=item C<< "notify-type" => $string >>

Applicable for CMode CLI.

=item C<< "time-gap-notify" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: time-gap-notify

=item C<< 'process' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system health subscriptions modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( creation-time max-notify-period class-name psc-option notify-table
                    fail-thresh instance-name notify-type time-gap-notify

                    process
                    )
            ),
        },
        cli_cmd => 'system health subscriptions modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemHealthSubscriptions::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'system_health_subscriptions_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( max-notify-period subscription-id class-name psc-option
            monitor notify-table node notify-dest-hm fail-thresh event-type
            instance-name time-gap-notify notify-dest-node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'diagnosis_subscriptions_modify',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 create

    my $SystemHealthSubscriptions_object    = NACL::C::SystemHealthSubscriptions->create(
                                command_interface => $ci,
                                 event_type => $event_type,
                                 subscription_id => $subscription_id,
                                 monitor => $monitor,
                                 node => $node,
                                 notify_dest_node => $notify_dest_node,
                                 notify_dest_hm => $notify_dest_hm,
                                     class_name => $class_name,
                                  ...);

(Class method)
This method is used to create a System Health Subscription.

Uses a CMode CLI APISet or CMode ZAPI APISet.

Invokes 'diagnosis-subscriptions-create' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "event-type" => $event_type >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: event-type

=item C<< "subscription-id" => $subscription_id >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: subscription-id

=item C<< "monitor" => $monitor >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: monitor

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node

=item C<< "notify-dest-node" => $notify_dest_node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-dest-node

=item C<< "notify-dest-hm" => $notify_dest_hm >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-dest-hm

=item C<< "class-name" => $class_name >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: class-name

=item C<< "creation-time" => $string >>

Applicable for CMode CLI.

=item C<< "max-notify-period" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-notify-period

=item C<< "psc-option" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: psc-option

=item C<< "notify-table" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: notify-table

=item C<< "fail-thresh" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: fail-thresh

=item C<< "instance-name" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: instance-name

=item C<< "notify-type" => $string >>

Applicable for CMode CLI.

=item C<< "time-gap-notify" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: time-gap-notify

=item C<< 'process' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system health subscriptions create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( creation-time max-notify-period class-name psc-option notify-table
                    fail-thresh instance-name notify-type time-gap-notify

                    process
                    )
            ),
        },
        cli_cmd => 'system health subscriptions create',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemHealthSubscriptions::create',
    );
    my $obj = return $pkg->new(
        command_interface => $opts{command_interface},
        event_type        => $opts{'event-type'},
        subscription_id   => $opts{'subscription-id'},
        monitor           => $opts{'monitor'},
        node              => $opts{'node'},
        notify_dest_node  => $opts{'notify-dest-node'},
        notify_dest_hm    => $opts{'notify-dest-hm'},
        class_name        => $opts{'class-name'},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,    api  => 'system_health_subscriptions_create');
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( max-notify-period subscription-id class-name psc-option
            monitor notify-table node notify-dest-hm fail-thresh event-type
            instance-name time-gap-notify notify-dest-node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'diagnosis_subscriptions_create',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 delete

   NACL::C::SystemHealthSubscriptions->delete(
                                command_interface => $ci,
                                 event_type => $event_type,
                                 subscription_id => $subscription_id,
                                 monitor => $monitor,
                                 node => $node,
                                 notify_dest_node => $notify_dest_node,
                                 notify_dest_hm => $notify_dest_hm,
                                     class_name => $class_name,
                                  ...);

                                 or

    $SystemHealthSubscriptions_object->delete(...);

(Class or instance method)
This method is used to delete a System Health Subscription.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "event-type" => $event_type >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "subscription-id" => $subscription_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "monitor" => $monitor >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "notify-dest-node" => $notify_dest_node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "notify-dest-hm" => $notify_dest_hm >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "class-name" => $class_name >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'process' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system health subscriptions delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( class-name

                    process
                    )
            ),
        },
         cli_cmd => 'system health subscriptions delete',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemHealthSubscriptions::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) =@_;
    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api     => 'system_health_subscriptions_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'event-type'       => {type => SCALAR},
        'subscription-id'  => {type => SCALAR},
        'monitor'          => {type => SCALAR},
        'node'             => {type => SCALAR},
        'notify-dest-node' => {type => SCALAR},
        'notify-dest-hm'   => {type => SCALAR},
        'class-name'       => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
