#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeAutosupport Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeAutosupport

=head1 DESCRIPTION

C<NACL::C::SystemNodeAutosupport> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeAutosupport.  A related class is
L<NACL::CS::SystemNodeAutosupport|lib-NACL-CS-SystemNodeAutosupport-pm>, which represents
the past state of an ONTAP SystemNodeAutosupport.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::SystemNodeAutosupport;

use strict;
use warnings;

use NACL::CS::SystemNodeAutosupport;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NATE::Exceptions::Argument;

# Import methods common to C and CS
use NACL::C::_Mixins::SystemNodeAutosupport qw(:all);

=head1 METHODS

=head2 new

  my $SystemNodeAutosupport = NACL::C::SystemNodeAutosupport->new(command_interface=>$ci,
                                   node =>$node,
                                  );

Return a new SystemNodeAutosupport component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $SystemNodeAutosupport  = NACL::C::SystemNodeAutosupport->find(command_interface => $ci, ...);
    my @SystemNodeAutosupports = NACL::C::SystemNodeAutosupport->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemNodeAutosupport_state = $NACL::C::SystemNodeAutosupport->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeAutosupport would return
a NACL::CS::SystemNodeAutosupport object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 invoke

   NACL::C::SystemNodeAutosupport->invoke(
                                command_interface => $ci,
                                 node => $node,
                                  ...);

                                 or

    $SystemNodeAutosupport_object->invoke(...);

(Class or instance method)
This method is used to generate and send an AutoSupport.


Uses a CMode CLI/ZAPI, 7Mode APISet.

Invokes 'autosupport-invoke' API for CMode ZAPI.

Invokes 'options' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node-name

=item C<< "type" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: type

=item C<< "force" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: force

=item C<< "message" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: message

=item C<< "uri" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: uri

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system node autosupport invoke" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub invoke {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( type force message uri
                    )
            ),
        },
        cli_cmd => 'system node autosupport invoke',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_invoke_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_invoke_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_invoke_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        frontend => 'NACL::C::SystemNodeAutosupport::invoke',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub invoke

sub _invoke_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'system_node_autosupport_invoke');
    $Log->exit() if $may_exit;
} ## end sub _invoke_cmode_cli

sub _invoke_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    if (exists($opts{message})) {
        $apiset->options('autosupport.doit' => $opts{message});
    }
    if (exists($opts{type})) {
        $apiset->options('autosupport.content' => 'complete');
    }
    $Log->exit() if $may_exit;

} ## end sub _invoke_7mode_cli

sub _invoke_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( type force message uri
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'autosupport_invoke',
            copy   => $allowed_cmode_options,
            map    => {"node" => "node-name"},
	);
    $Log->exit() if $may_exit;
} ## end sub _invoke_cmode_zapi

=head2 modify

   NACL::C::SystemNodeAutosupport->modify(
                                command_interface => $ci,
                                 node => $node,
                                  ...);

                                 or

    $SystemNodeAutosupport_object->modify(...);

(Class or instance method)
This method is used to modify AutoSupport configuration.

Uses a CMode CLI/ZAPI, 7Mode APISet.

Invokes 'autosupport-config-modify' API for CMode ZAPI.

Invokes 'options' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node-name

=item C<< "log-file-size" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: log-file-size

=item C<< "support" => $string >>

Send AutoSupport Messages to Vendor Support. Valid values are "enable"
and "disable".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-support-enabled

=item C<< "state-for-zapi" => $string >>

Applicable for CMode CLI.

=item C<< "transport" => $string >>

Protocol to Contact Support. Valid values are "smtp", "http" and "https".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: transport

7M CLI : autosupport.support.transport

=item C<< "local-perf" => $string >>

Applicable for CMode CLI.

=item C<< "nht" => $string >>

NHT Enable. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: is-nht-data-enabled

7M CLI : autosupport.nht data.enable

=item C<< "state" => $string >>

Valid values are "enable" and "disable".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: is-enabled

7M CLI : autosupport.enable

=item C<< "perf" => $string >>

Performance Data Enable. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: is-perf-data-enabled

7M CLI : autosupport.performance data.enable

=item C<< "local-nht" => $string >>

Applicable for CMode CLI.

=item C<< "hostname-subj" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-node-in-subject

=item C<< "retry-interval" => $string >>

Retry Interval. If the value is provided as an integer, then it is assumed
to be in seconds. The value can also be provided in the string format
accepted by CMode CLI:

<int>h<int>m<int>s

For example, these values would mean the following:

"1h1m" = 1 hour 1 minute.

"1m2s" = 1 minute 2 seconds.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: retry-interval

7M CLI : autosupport.retry.interval

=item C<< "from" => $string >>

From Address.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: from

7M CLI : autosupport.from

=item C<< "periodic-tx-window" => $string >>

The Transmission Window. See C<retry-interval> for an explanation
of the formats in which this value can be provided.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: periodic-tx-window

7M CLI : autosupport.periodic.tx window

=item C<< "url" => $string >>

Support URL for HTTP/HTTPS

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: url

7M CLI : autosupport.support.url

=item C<< "max-smtp-size" => $string >>

Maximum HTTP Size. If the value is provided without any unit, then it is
assumed to be in terms of bytes. The value provided can also have a size
suffix. The following are examples of valid values:

3600 (3600 bytes)
2MB
3GB

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: max-smtp-size

7M CLI : autosupport.max smtp size

=item C<< "payload-format" => $string >>

Payload Format. Valid values are "7z" and "tgz".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: payload-format

7M CLI : autosupport.payload format

=item C<< "remove-private-data" => $string >>

Remove Sensitive Data. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-private-data-removed

=item C<< "reminder" => $string >>

Reminder Enable. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: is-reminder-enabled

7Mode CLI: autosupport.support.reminder

=item C<< "put-url" => $string >>

Support URL for HTTP/S PUT.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: put-url

7M CLI : autosupport.support.put url

=item C<< "support-for-zapi" => $string >>

Applicable for CMode CLI.

=item C<< "retry-count" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: retry-count

7M CLI : autosupport.retry.count

=item C<< "validate-digital-certificate" => $string >>

Validate Digital Certificate Received. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: validate-digital-certificate

7M CLI : autosupport.validate digital certificate

=item C<< "max-http-size" => $string >>

Maximum HTTP Size. See the description of C<max-smtp-size> to see the
formats in which this value can be provided.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: max-http-size

7M CLI : autosupport.max http size

=item C<< "throttle" => $string >>

Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: is-throttle-enabled

7M CLI : autosupport.throttle

=item C<< "proxy-url" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: proxy-url

7M CLI : autosupport.support.proxy

=item C<< "partner-address" => [ $partner-addres1, $partner-addres2, ... ] >>

List of Partner Addresses.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: partner-address

7M CLI : autosupport.partner.to

=item C<< "to" => [ $to1, $to2, ... ] >>

List of To Addresses

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: to

7M CLI : autosupport.to

=item C<< "support-address" => [ $support-addres1, $support-addres2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: support-address

=item C<< "noteto" => [ $noteto1, $noteto2, ... ] >>

List of Noteto Addresses

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: noteto

7M CLI : autosupport.noteto

=item C<< "mail-hosts" => [ $mail-host1, $mail-host2, ... ] >>

SMTP Mail Hosts.

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: mail-hosts

7M CLI : autosupport.mailhost

=item C<< 'local-collection' => $string >>

Continue Local Collection while Disabled. Valid values are "true" and "false".

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI: is-local-collection-enabled

7Mode CLI: autosupport.local_collection

=item C<< 'minimum-private-data-length' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CMode ZAPI: minimum-private-data-length

=item C<< 'ondemand-state' => $string >>

AutoSupport OnDemand State. Valid values are "enable" and "disable".

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI: autosupport.ondemand.state

=item C<< 'ondemand-server-url' => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI: ondemand-server-url

7Mode CLI: autosupport.ondemand.server_url

=item C<< 'ondemand-polling-interval' => $string >>

AutoSupport OnDemand Polling Interval (in minutes).

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI: ondemand-polling-interval

7Mode CLI: autosupport.ondemand.polling_interval

=item C<< 'ondemand-remotediag-state' => $string >>

Applicable for CMode CLI, 7Mode CLI.

*AutoSupport OnDemand Remote Diagnostics State, possible value(s) are
enable,disable

Maps to:

7Mode CLI: autosupport.ondemand.remotediag.state

Applicable for CMode CLI

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'max-upload-rate' => $string >>

AutoSupport OnDemand Upload Throttle Setting
Applicable for CMode CLI

=item C<< 'ondemand-download-state' => $string >>


AutoSupport OnDemand Download State possible value(s) are, enable,disable
Applicable for CMode CLI

=item C<< 'max-download-rate' => $string >>

Maximum Download Rate
Applicable for CMode CLI

=item C<< 'diagnostic-content-state' => $string >>

Diagnostic Content State possible value(s) are, enable,disable
Applicable for CMode CLI

=item C<< 'chunk-size' => $string >>

AutoSupport chunk size
Applicable for CMode CLI

=item C<< 'chunk-threads' => $string >>

AutoSupport chunk size
Applicable for CMode CLI

=item C<< 'diagnostic-content-delivery-state' => $string >>


Diagnostic Content Default Delivery State possible value(s) are, enable,disable
Applicable for CMode CLI

=item C<< 'http-delivery-timeout' => $string >>

AutoSupport HTTP delivery timeout
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"system node autosupport modify" command in ONTAP.

=item C<< '*' >>

This method takes all other options supported by
"system node autosupport modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "partner-address" => {type => ARRAYREF, optional => 1},
            "to"              => {type => ARRAYREF, optional => 1},
            "support-address" => {type => ARRAYREF, optional => 1},
            "noteto"          => {type => ARRAYREF, optional => 1},
            "mail-hosts"      => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( log-file-size support state-for-zapi transport local-perf nht state perf
                    local-nht hostname-subj retry-interval from periodic-tx-window url
                    max-smtp-size payload-format remove-private-data reminder put-url
                    support-for-zapi retry-count validate-digital-certificate max-http-size
                    throttle proxy-url local-collection
                    minimum-private-data-length ondemand-state ondemand-server-url
                    ondemand-polling-interval ondemand-remotediag-state

                    max-upload-rate

                    ondemand-download-state max-download-rate

                    diagnostic-content-state

                    chunk-size chunk-threads diagnostic-content-delivery-state
                    http-delivery-timeout
                    )
            ),
        },
        cli_cmd => 'system node autosupport modify',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => "_modify_zapi_check",
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupport::modify'
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args, api  => 'system_node_autosupport_modify');
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts   = $pkg_or_obj->_validate_backend_opts(@args);
    my $apiset = $opts{apiset};

    # In 7Mode, we should supply multiple values delimited by a comma
    my @array_fields = $pkg_or_obj->_array_fields_cmode();
    foreach my $array_field (@array_fields) {
        if (defined $opts{$array_field}) {
            $opts{$array_field} = join ',', @{$opts{$array_field}};
        }
    }

    # CMode accepts the values of certain fields as enable/disable. 7Mode
    # expects the values to be on/off.
    $pkg_or_obj->_translate_enable_disable(
        options         => \%opts,
        mode_of_options => 'CMode'
    );

    # CMode accepts the values of certain fields as true/false. 7Mode
    # expects the values to be on/off.
    $pkg_or_obj->_translate_true_false(
        options         => \%opts,
        mode_of_options => 'CMode'
    );

    # CMode accepts unit suffixes with certain fields. 7Mode understands
    # these values only in bytes.
    my @unit_normalize_fields = $pkg_or_obj->_unit_normalize_fields();
    foreach my $unit_field (@unit_normalize_fields) {
        if (defined $opts{$unit_field}) {
            $opts{$unit_field} = NACL::C::UnitNormalization::convert_to_bytes(
                $opts{$unit_field});
        }
    }

    # 7Mode is fussy about the way the times are passed in. It only allows
    # one unit to be specified at a time. For example, all of the following
    # are considering invalid in 7Mode: "1h1m", "1m1s". (The first is accepted
    # as "61m" and the second as "61s").
    # For simplicity, let's convert the value passed in to seconds and then
    # pass that down to the API.
    my $time_fields = $pkg_or_obj->_time_fields();
    foreach my $time_field (@$time_fields) {
        if (defined $opts{$time_field}) {
            $opts{$time_field}
                = $pkg_or_obj->_convert_time(time => $opts{$time_field});
        }
    }

    my %map = $pkg_or_obj->_c_7_fields_map();
    my %api_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_args,
        map    => \%map
    );

    while (my ($option, $value) = each %api_args) {
        $apiset->options($option => $value);
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $enable_disable_fields = $pkg_or_obj->_enable_disable_fields();
    my %enable_true = (enable => 'true', disable => 'false');
    foreach my $field (@$enable_disable_fields) {
        if (defined $opts{$field}) {
            my $value = $opts{$field};
            if (exists $enable_true{$value}) {
                $opts{$field} = $enable_true{$value};
            }
        }
    }

    # CLI accepts unit suffixes with certain fields. ZAPI understands
    # these values only in bytes.
    my @unit_normalize_fields = $pkg_or_obj->_unit_normalize_fields();
    foreach my $unit_field (@unit_normalize_fields) {
        if (defined $opts{$unit_field}) {
            $opts{$unit_field} = NACL::C::UnitNormalization::convert_to_bytes(
                $opts{$unit_field});
        }
    }

    # CMode accepts time values as strings, ZAPI expects it to be in seconds
    my $time_fields = $pkg_or_obj->_time_fields();
    foreach my $time_field (@$time_fields) {
        if (defined $opts{$time_field}) {
            $opts{$time_field}
                = $pkg_or_obj->_convert_time(time => $opts{$time_field});
        }
    }

    my $copy = $pkg_or_obj->_cli_zapi_copy();
    my $common_map  = $pkg_or_obj->_cli_zapi_map();
    $common_map->{'support-address'} = 'support-address';
    $common_map->{'mail-hosts'} = 'mail-hosts';
    $common_map->{'partner-address'} = 'partner-address';
    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'autosupport-config-modify',
        copy => $copy,
        map  => $common_map,
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _modify_zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my @fields_to_check = ('node');
    $pkg_or_obj->_fallback_for_relational_regex(%opts,
        fields_to_check => \@fields_to_check);

    $Log->exit() if $may_exit;
} ## end sub _modify_zapi_check

=head2 invoke_core_upload

    NACL::C::SystemNodeAutosupport->invoke_core_upload(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $SystemNodeAutosupport_obj->invoke_core_upload(...);

(Class or instance method)
This method is used to generate and send a coredump AutoSupport message..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'core-filename' => $string >>

The Core Filename to Upload
Applicable for CMode CLI

=item C<< 'case-number' => $string >>

Case Number for This Core Upload
Applicable for CMode CLI

=item C<< 'uri' => $string >>

Alternate Destination for this AutoSupport
Applicable for CMode CLI

=item C<< 'message' => $string >>

Message Included in the AutoSupport Subject
Applicable for CMode CLI

=item C<< 'force' => $string >>

Generate and Send Even if Disabled
Applicable for CMode CLI

=item C<< 'aod-initiated' => $string >>

Initiated by AutoSupport OnDemand possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node autosupport invoke coredump" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub invoke_core_upload {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( core-filename case-number uri message force

                    aod-initiated
                    )
            ),
        },
        api => 'system_node_autosupport_invoke_core_upload',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_invoke_core_upload_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupport::invoke_core_upload',
    );

    $Log->exit() if $may_exit;
}

sub _invoke_core_upload_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_autosupport_invoke_core_upload',);
    $Log->exit() if $may_exit;
}

=head2 invoke_diagnostic

    NACL::C::SystemNodeAutosupport->invoke_diagnostic(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $SystemNodeAutosupport_obj->invoke_diagnostic(...);

(Class or instance method)
This method is used to generate and send an AutoSupport message with diagnostic content from specified subsystems..
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'message-subject' => $string >>

Message Included in the AutoSupport Subject
Applicable for CMode CLI

=item C<< 'uri' => $string >>

Alternate Destination for This AutoSupport
Applicable for CMode CLI

=item C<< 'force' => $string >>

Generate and Send Even if Disabled
Applicable for CMode CLI

=item C<< 'subsystems' => [ $subsystem1, $subsystem2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node autosupport invoke diagnostic" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub invoke_diagnostic {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'subsystems' => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( message-subject uri force
                    )
            ),
        },
        api => 'system_node_autosupport_invoke_diagnostic',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_invoke_diagnostic_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupport::invoke_diagnostic',
    );

    $Log->exit() if $may_exit;
}

sub _invoke_diagnostic_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_autosupport_invoke_diagnostic',);
    $Log->exit() if $may_exit;
}

=head2 invoke_splog

    NACL::C::SystemNodeAutosupport->invoke_splog(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to trigger an upload of Service Processor logs and invoke the AutoSupport.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'log-sequence' => $string >>

Sequence Number
Applicable for CMode CLI

=item C<< 'remote-node' => $string >>

Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'uri' => $string >>

Alternate Destination for This AutoSupport
Applicable for CMode CLI

=item C<< 'force' => $string >>

Generate and Send Even if Disabled
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node autosupport invoke splog" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub invoke_splog {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( log-sequence remote-node uri force
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_node_autosupport_invoke_splog',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_invoke_splog_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupport::invoke_splog',
    );

    $Log->exit() if $may_exit;
}

sub _invoke_splog_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_node_autosupport_invoke_splog',);
    $Log->exit() if $may_exit;
}

=head2 invoke_performance_archive

    NACL::C::SystemNodeAutosupport->invoke_performance_archive(
                                command_interface => $ci,
	                            'node' => $node,
                                ...);
	                             
								 or

    $SystemNodeAutosupport_obj->invoke_performance_archive(...);

(Class or instance method)
This method is used to generate and send an AutoSupport message with performance archives..
Uses CMode CLI/ZAPI .
Invokes  autosupport-invoke-performance-archive  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI, non-iter.

Mapping for non-iter: node-name

=item C<< 'case-number' => $string >>

Case Number for This Performance Archive Upload
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'duration' => $string >>

Duration of Performance Archive Dataset
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'start-date' => $string >>

Start Date for Performance Archive Dataset
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'aod-initiated' => $string >>

Initiated by AutoSupport OnDemand possible value(s) are, true,false
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'message' => $string >>

Message Included in the AutoSupport Subject
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'uri' => $string >>

Alternate Destination for This AutoSupport
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'force' => $string >>

Generate and Send Even if Disabled
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'end-date' => $string >>

End Date for Performance Archive Dataset
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "system node autosupport invoke performance archive" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub invoke_performance_archive {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( case-number duration start-date aod-initiated message uri force end-date
                    )
            ),
        },
        api => 'system_node_autosupport_invoke_performance_archive',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_invoke_performance_archive_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method => '_invoke_performance_archive_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend =>
            'NACL::C::SystemNodeAutosupport::invoke_performance_archive',
    );

    $Log->exit() if $may_exit;
}

sub _invoke_performance_archive_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_autosupport_invoke_performance_archive',);
    $Log->exit() if $may_exit;
}

sub _invoke_performance_archive_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy
        = [qw(case-number duration start-date force uri message end-date)];
    my $map = {'node' => 'node-name',};
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'autosupport-invoke-performance-archive',
        copy => $copy,
        map  => $map,
        zapi_field_translations =>
            {mmddyyyy_to_timestamp => [qw(start-date end-date)],},
    );
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR | UNDEF, optional => 1},);
}

1;
