# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeAutosupportHistory Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeAutosupportHistory

=head1 DESCRIPTION

C<NACL::C::SystemNodeAutosupportHistory> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeAutosupportHistory.  A related class is
L<NACL::CS::SystemNodeAutosupportHistory|lib-NACL-CS-SystemNodeAutosupportHistory-pm>, which represents
the past state of an ONTAP SystemNodeAutosupportHistory.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 seq_num

=head2 destination

=head2 node

=cut

package NACL::C::SystemNodeAutosupportHistory;

use strict;
use warnings;

use NACL::CS::SystemNodeAutosupportHistory;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NACL::C::UnitNormalization;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemNodeAutosupportHistory = NACL::C::SystemNodeAutosupportHistory->new(command_interface=>$ci,
                                   seq_num =>$seq_num,
                                   destination =>$destination,
                                   node =>$node,
                                  );

Return a new SystemNodeAutosupportHistory component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'seq_num',
    scalar => 'destination',
    scalar => 'node',
];

=head2 find

    my $SystemNodeAutosupportHistory  = NACL::C::SystemNodeAutosupportHistory->find(command_interface => $ci, ...);
    my @SystemNodeAutosupportHistorys = NACL::C::SystemNodeAutosupportHistory->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemNodeAutosupportHistory_state = $NACL::C::SystemNodeAutosupportHistory->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeAutosupportHistory would return
a NACL::CS::SystemNodeAutosupportHistory object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 retransmit

   NACL::C::SystemNodeAutosupportHistory->retransmit(
                                command_interface => $ci,
                                 seq_num => $seq_num,
                                 destination => $destination,
                                 node => $node,
                                  ...);

(Class method)
This method is used to selectively retransmit a previously collected AutoSupport..

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "seq-num" => $seq_num >>

(Required)

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: seq-num

7M CLI : -seq num <value> (Programmatic name: seq num)

=item C<< "destination" => $destination >>

(Required)

Applicable for CMode CLI.

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: "node-name" of "autosupport-history-retransmit"

=item C<< "size-limit" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: size-limit

7M CLI : -size limit <value> (Programmatic name: size limit)

=item C<< "uri" => $string >>

Applicable for CMode CLI/ZAPI,7Mode CLI.

Maps to:

CM ZAPI: uri

7M CLI : -uri <value> (Programmatic name: uri)

=item C<< 'aod-initiated' => $string >>

Applicable for CMode CLI.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"system node autosupport history retransmit" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub retransmit {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( size-limit uri

                    aod-initiated
                    )
            ),
        },
        cli_cmd => 'system node autosupport history retransmit',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_retransmit_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_retransmit_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_retransmit_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupportHistory::retransmit',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub retransmit

sub _retransmit_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,    api  => 'system_node_autosupport_history_retransmit');
    $Log->exit() if $may_exit;
} ## end sub _retransmit_cmode_cli

sub _retransmit_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw(seq-num size-limit uri
            )
    ];
    my %mode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%mode_args,
    );

    $apiset->autosupport_history_retransmit(%mode_args);
    $Log->exit() if $may_exit;
} ## end sub _retransmit_7mode_cli

sub _retransmit_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $allowed_cmode_options = [
        qw( seq-num size-limit uri
            )
    ];
    if (defined $opts{'size-limit'}) {
        $opts{'size-limit'}
            = NACL::C::UnitNormalization::to_bytes($opts{'size-limit'});
        $opts{'size-limit'} = NACL::C::UnitNormalization::strip_unit_spec(
            $opts{'size-limit'});
    }
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'autosupport_history_retransmit',
        copy => $allowed_cmode_options,
        map =>  {"node" => "node-name"},
    );
    $Log->exit() if $may_exit;
} ## end sub _retransmit_cmode_zapi

=head2 cancel

    NACL::C::SystemNodeAutosupportHistory->cancel(
                                command_interface => $ci,
	                            'node' => $node,
	                            'seq-num' => $seq_num,
	                            'destination' => $destination,
                                ...);
	                             
								 or

    $SystemNodeAutosupportHistory_obj->cancel(...);

(Class or instance method)
This method is used to cancel an AutoSupport message.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node possible value(s) are, <nodename>,local
Applicable for CMode CLI

=item C<< 'seq-num' => $seq_num >>

(Required for class method, Not Applicable for instance method)
AutoSupport Sequence Number
Applicable for CMode CLI

=item C<< 'destination' => $destination >>

(Required for class method, Not Applicable for instance method)

Destination for this AutoSupport possible value(s) are, smtp,http,noteto,retransmit
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system node autosupport history cancel" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub cancel {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'system_node_autosupport_history_cancel',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_cancel_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemNodeAutosupportHistory::cancel',
    );

    $Log->exit() if $may_exit;
}

sub _cancel_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_node_autosupport_history_cancel',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'seq-num'     => {type => SCALAR},
        'node'        => {type => SCALAR, optional => 1},
        'destination' => {type => SCALAR, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
