# $Id$
#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeCoredumpSegment Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeCoredumpSegment

=head1 DESCRIPTION

C<NACL::C::SystemNodeCoredumpSegment> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemNodeCoredumpSegment.  A related class is
L<NACL::CS::SystemNodeCoredumpSegment|lib-NACL-CS-SystemNodeCoredumpSegment-pm>, which represents
the past state of an ONTAP SystemNodeCoredumpSegment.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 segment

=head2 owner_node

=head2 node

=cut

package NACL::C::SystemNodeCoredumpSegment;

use strict;
use warnings;

use NACL::CS::SystemNodeCoredumpSegment;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::BaseException;
use
    NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform
    qw(:try);

=head1 METHODS

=head2 new

  my $SystemNodeCoredumpSegment = NACL::C::SystemNodeCoredumpSegment->new(
                                   command_interface => $ci,
                                   'segment' => $segment,
                                   'owner-node' => $owner_node,
                                   'node' => $node,
  );

Return a new SystemNodeCoredumpSegment component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'segment',
    scalar => 'owner_node',
    scalar => 'node',
];

=head2 find

    my $SystemNodeCoredumpSegment_obj  = NACL::C::SystemNodeCoredumpSegment->find(command_interface => $ci, ...);
    my @SystemNodeCoredumpSegment_objs = NACL::C::SystemNodeCoredumpSegment->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet or 7Mode CLI/ZAPI APISet.
This method executes core-segment-config-get for 7Mode ZAPI.
This method executes core-segment-get-iter for CMode ZAPI.

=head2 state

    my SystemNodeCoredumpSegment_state = $SystemNodeCoredumpSegment_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemNodeCoredumpSegment would return
a NACL::CS::SystemNodeCoredumpSegment object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet or 7Mode CLI APISet.

=cut

=head2 delete_all

    NACL::C::SystemNodeCoredumpSegment->delete_all(
                                command_interface => $ci,
                                'owner-node' => $owner_node,
                                'node' => $node,
                                ...);

(Class method)
This method is used to delete All Core Segments on a Node.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APISet.

Invokes 'core-segment-delete-all' API for CMode ZAPI, 7Mode CLI/ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'owner-node' => $owner_node >>


Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: owner-node

7M ZAPI: owner-node

(Required)

=item C<< 'node' => $node >>


Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: node

7M ZAPI: node

(Required)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node coredump segment delete-all" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_all {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( node owner-node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system node coredump segment delete-all',
    );
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => '_delete_all_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode',
                },
                {   method    => '_delete_all_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode',
                },
                {   method    => '_delete_all_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
                {   method    => '_delete_all_7mode_zapi',
                    interface => 'ZAPI',
                    set       => '7Mode',
                },
            ],
            frontend => 'NACL::C::SystemNodeCoredumpSegment::delete_all',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform
            ->detect_convert_and_throw(exception => $exception);
    };
    $Log->exit() if $may_exit;
} ## end sub delete_all

sub _delete_all_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_node_coredump_segment_delete_all',);
    $Log->exit() if $may_exit;
} ## end sub _delete_all_cmode_cli

sub _delete_all_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( node owner-node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'core_segment_delete_all',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _delete_all_cmode_zapi

sub _delete_all_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %allowed_7mode_args;
    $pkg->_hash_copy(
        source => \%opts,
        target => \%allowed_7mode_args,
    );
    $apiset->coredump_segment_delete_all(%allowed_7mode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_all_7mode_cli

sub _delete_all_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( node owner-node
            )
    ];
    my %_7mode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%_7mode_args,
    );
    $apiset->core_segment_delete_all(%_7mode_args);
    $Log->exit() if $may_exit;
} ## end sub _delete_all_7mode_zapi

=head2 stop

    NACL::C::SystemNodeCoredumpSegment->stop(
                                command_interface => $ci,
                            'node' => $node,
                                ...);

(Class method)
This method is used to cancel Core Segmentation Job..

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APISet.

Invokes 'core-segment-stop' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>


Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: node

7M ZAPI: node

(Required)

=item C<< 'job-id' => $string >>


Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI

Maps to:

CM ZAPI: job-id

7M CLI : -job id <value> (Programmatic name: job id)

7M ZAPI: job-id

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node coredump segment stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( job-id node
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system node coredump segment stop',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_stop_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_stop_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_stop_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::SystemNodeCoredumpSegment::stop',
    );

    $Log->exit() if $may_exit;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_node_coredump_segment_stop',);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( job-id node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'core_segment_stop',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_zapi

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %allowed_7mode_args;
    my $out = $pkg->_hash_copy(
        source => \%opts,
        copy   => [qw( job-id )],
        target => \%allowed_7mode_args,
    );
    $apiset->coredump_segment_stop(%allowed_7mode_args);
    $Log->exit() if $may_exit;

} ## end sub _stop_7mode_cli

sub _stop_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( job-id node
            )
    ];
    my %_7mode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%_7mode_args,
    );

    $apiset->core_segment_stop(%_7mode_args);
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_zapi

=head2 status

    my $status_obj = NACL::C::SystemNodeCoredumpSegment->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

or

    my @status_objs = NACL::C::SystemNodeCoredumpSegment->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $status_obj = $SystemNodeCoredumpSegment_obj->status();

(Class or Instance method)
This method returns objects of the type L<NACL::CS::SystemNodeCoredumpSegmentStatus|lib-NACL-CS-SystemNodeCoredumpSegmentStatus-pm>.

This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch>
(requested_fields, filter, allow_empty) and has all of the other properties of
fetch methods (callable in scalar or list context, throws a NoElementsFound
exception if no matching elements are found)

=cut

sub status {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::SystemNodeCoredumpSegmentStatus;
    my @state_objs = NACL::CS::SystemNodeCoredumpSegmentStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub status

=head2 start

    my $SystemNodeCoredumpSegment_obj= NACL::C::SystemNodeCoredumpSegment->start(
                                command_interface => $ci,
                                'owner-node'      => $owner_node,
                                'node'            => $node,
                                ...);

(Class method)
This method is used to start Core Segmentation Job.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APISet.

Invokes 'core-segment-start' API for CMode ZAPI, 7Mode ZAPI.

Invokes 'coredump segment start' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'owner-node' => $owner_node >>


Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: owner-node

7M ZAPI: owner-node

(Required)

=item C<< 'node' => $node >>


Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: node

7M ZAPI: node

(Required)

=item C<< 'core-name' => $string >>


Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI

Maps to:

CM ZAPI: core-name

7M CLI : -core name <value> (Programmatic name: core name)

7M ZAPI: core-name

=item C<< 'delete-core' => $string >>


Applicable for CMode CLI, 7Mode CLI

Maps to:

CM ZAPI: is-delete-core

7M CLI : -delete core <value> (Programmatic name: delete core)

7M ZAPI: is-delete-core


=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node coredump segment start" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( core-name node owner-node delete-core segment
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd             => 'system node coredump segment start',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_start_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_start_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_start_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::SystemNodeCoredumpSegment::start',
    );

    my %new_args;
    $pkg->_hash_copy_defined(
        source => \%opts,
        target => \%new_args,
        copy   => [qw(segment owner-node node)]
    );
    $pkg->_process_options(options => \%new_args);
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        %new_args
    );

    $Log->exit() if $may_exit;
    return $obj;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_node_coredump_segment_start',);
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'core_segment_start',
            copy   => [qw( core-name node owner-node )],
            map    => {'delete-core' => 'is-delete-core',},
	);
    $Log->exit() if $may_exit;
}    ##end _start_cmode_zapi

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %allowed_7mode_args;
    $pkg->_hash_copy(
        copy   => [qw(core-name delete-core )],
        source => \%opts,
        target => \%allowed_7mode_args,
    );
    $apiset->coredump_segment_start(%allowed_7mode_args);
    $Log->exit() if $may_exit;
}    ##end _start_7mode_cli

sub _start_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %_7mode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => [qw( core-name node owner-node )],
        map    => {'delete-core' => 'is-delete-core',},
        target => \%_7mode_args,
    );
    $apiset->core_segment_start(%_7mode_args);
    $Log->exit() if $may_exit;

}    ##end _start_7mode_zapi

=head2 delete

    NACL::C::SystemNodeCoredumpSegment->delete(
                                command_interface => $ci,
                            'segment' => $segment,
                            'owner-node' => $owner_node,
                            'node' => $node,
                                ...);

 or

    $SystemNodeCoredumpSegment_obj->delete(...);

(Class or instance method)
This method is used to delete a Core Segment.

Uses a CMode CLI/ZAPI, 7Mode CLI/ZAPI APISet.

Invokes 'core-segment-destroy' API for CMode ZAPI, 7Mode ZAPI.

Invokes 'coredump segment delete' command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'segment' => $segment >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI

Maps to:

CM ZAPI: segment

7M CLI : -segment <value> (Programmatic name: segment)

7M ZAPI: segment

=item C<< 'owner-node' => $owner_node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode CLI/ZAPI

Maps to:

CM ZAPI: owner-node

7M CLI : -owner node <value> (Programmatic name: owner node)

7M ZAPI: owner-node

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI, 7Mode ZAPI

Maps to:

CM ZAPI: node

7M ZAPI: node

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system node coredump segment delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( segment owner-node node
                    )
            )
        },
        cli_cmd => 'system node coredump segment delete',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_delete_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_delete_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode',
            },
        ],
        frontend => 'NACL::C::SystemNodeCoredumpSegment::delete',
    );

    $Log->exit() if $may_exit;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_node_coredump_segment_delete',);

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $allowed_cmode_options = [
        qw( segment node owner-node
            )
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(%opts,
            api    => 'core_segment_destroy',
            copy   => $allowed_cmode_options,
	);
    $Log->exit() if $may_exit;

} ## end sub _delete_cmode_zapi

sub _delete_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};
    my %allowed_7mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => [qw(segment owner-node)],
        target => \%allowed_7mode_args,
    );

    $apiset->coredump_segment_delete(%allowed_7mode_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_7mode_cli

sub _delete_7mode_zapi {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_7mode_options = [
        qw( segment node owner-node
            )
    ];
    my %_7mode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_7mode_options,
        target => \%_7mode_args,
    );

    $apiset->core_segment_destroy(%_7mode_args);
    $Log->exit() if $may_exit;

} ## end sub _delete_7mode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'segment'    => {type => SCALAR | UNDEF, optional => 1},
        'owner-node' => {type => SCALAR | UNDEF, optional => 1},
        'node'       => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec
1;
