#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeFirmware Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemNodeFirmware

=head1 DESCRIPTION

C<NACL::C::SystemNodeFirmware> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::SystemNodeFirmware;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head2 download

   NACL::C::SystemNodeFirmware->download(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to download motherboard firmware and system diagnostics.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "isZipFile" => $string >>

Applicable for CMode CLI.

=item C<< "sequence" => $string >>

Applicable for CMode CLI.

=item C<< "temppath" => $string >>

Applicable for CMode CLI.

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< "package" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub download {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'download' front end are:\n" . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( isZipFile sequence temppath node package
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_download_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub download

sub _download_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( isZipFile sequence temppath node package
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug("Opts to API call 'system_node_firmware_download' are: \n"
            . Dumper(\%opts));

    $apiset->system_node_firmware_download(%opts);
    $Log->exit() if $may_exit;
} ## end sub _download_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
