# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemScript Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemScript

=head1 DESCRIPTION

C<NACL::C::SystemScript> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemScript.  A related class is
L<NACL::CS::SystemScript|lib-NACL-CS-SystemScript-pm>, which represents
the past state of an ONTAP SystemScript.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 filename

=head2 username

=cut

package NACL::C::SystemScript;

use strict;
use warnings;

use NACL::CS::SystemScript;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::SystemScript;

=head1 METHODS

=head2 new

  my $SystemScript = NACL::C::SystemScript->new(
                                   command_interface => $ci,
                                   'filename' => $filename,
                                   'username' => $username,
								  );

Return a new SystemScript component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'filename',
    scalar => 'username',
];

=head2 find

    my $SystemScript_obj  = NACL::C::SystemScript->find(command_interface => $ci, ...);
	my @SystemScript_objs = NACL::C::SystemScript->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my SystemScript_state = $SystemScript_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemScript would return 
a NACL::CS::SystemScript object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 start

    my $SystemScript_obj	= NACL::C::SystemScript->start(
                                command_interface => $ci,
	                            'filename' => $filename,
                                ...);
   
(Class method)
This method is used to start logging all CLI I/O to session log.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filename' => $filename >>

(Required)
Filename to Log To
Applicable for CMode CLI

=item C<< 'size-limit' => $string >>

Logfile Size Limit Max:2GB
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system script start" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        additional_spec     => {
            _optional_scalars(
                qw( size-limit filename
                    )
            ),
        },
        api => 'system_script_start',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemScript::start',
    );

    my $username = NACL::CS::SystemScript->fetch(
        command_interface => $opts{command_interface},
        requested_fields  => ['username'],
        filter            => {filename => $opts{'filename'}}
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        filename          => $opts{'filename'},
        username          => $username->username(),
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_script_start',);
    $Log->exit() if $may_exit;
}

=head2 stop

    NACL::C::SystemScript->stop(
                                command_interface => $ci,
                                   ...);

(Class method)
This method is used to stops logging CLI I/O.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "system script stop" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'system_script_stop',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemScript::stop',
    );

    $Log->exit() if $may_exit;
}

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_script_stop',);
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::SystemScript->delete(
                                command_interface => $ci,
	                            'filename' => $filename,
	                            'username' => $username,
                                ...);
	                             
								 or

    $SystemScript_obj->delete(...);

(Class or instance method)
This method is used to delete saved CLI session logs.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filename' => $filename >>

(Required for class method, Not Applicable for instance method)
Log Filename
Applicable for CMode CLI

=item C<< 'username' => $username >>

(Optional)
Log Owner Username
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system script delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'system_script_delete',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemScript::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'system_script_delete',);
    $Log->exit() if $may_exit;
}

=head2 upload

    NACL::C::SystemScript->upload(
                                command_interface => $ci,
	                            'filename' => $filename,
	                            'username' => $username,
                                ...);
	                             
								 or

    $SystemScript_obj->upload(...);

(Class or instance method)
This method is used to upload the selected CLI session log.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filename' => $filename >>

(Required for class method, Not Applicable for instance method)
Filename to Log To
Applicable for CMode CLI

=item C<< 'username' => $username >>

(Optional)
Username If Not Your Own. Default: admin
Applicable for CMode CLI

=item C<< "ftp_username" => $username >>

(Required)

Applicable for CMode CLI.

=item C<< "ftp_password" => $password >> 

(Required)

=item C<< 'destination' => $string >>

URI to Send File To possible value(s) are, (ftp,http)://(hostname,IPv4 Address,'['IPv6 Address']')...
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "system script upload" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub upload {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( destination ftp_password ftp_username
                    )
            ),
        },
        api => 'system_script_upload',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_upload_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemScript::upload',
    );

    $Log->exit() if $may_exit;
}

sub _upload_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $user = $opts{ftp_username};
    my $pwd  = $opts{ftp_password};

    my @prompts_answers = (
        'Enter User:'     => $user,
        'Enter Password:' => $pwd,
    );

    my %additional_args;
    $additional_args{'connectrec-match_table'} = \@prompts_answers;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api             => 'system_script_upload',
        additional_args => \%additional_args
    );
    $Log->exit() if $may_exit;
} ## end sub _upload_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'filename' => {type => SCALAR},
        'username' => {type => SCALAR},
    );
}

1;
