# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorNetworkAutoConfiguration Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemServiceProcessorNetworkAutoConfiguration

=head1 DESCRIPTION

C<NACL::C::SystemServiceProcessorNetworkAutoConfiguration> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemServiceProcessorNetworkAutoConfiguration.  A related class is
L<NACL::CS::SystemServiceProcessorNetworkAutoConfiguration|lib-NACL-CS-SystemServiceProcessorNetworkAutoConfiguration-pm>, which represents
the past state of an ONTAP SystemServiceProcessorNetworkAutoConfiguration.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::SystemServiceProcessorNetworkAutoConfiguration;

use strict;
use warnings;

use NACL::CS::SystemServiceProcessorNetworkAutoConfiguration;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $SystemServiceProcessorNetworkAutoConfiguration = NACL::C::SystemServiceProcessorNetworkAutoConfiguration->new(
                                   command_interface => $ci,
								  );

Return a new SystemServiceProcessorNetworkAutoConfiguration component object with the given attributes.

=cut

use Class::MethodMaker [];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $SystemServiceProcessorNetworkAutoConfiguration_obj  = NACL::C::SystemServiceProcessorNetworkAutoConfiguration->find(command_interface => $ci, ...);
	my @SystemServiceProcessorNetworkAutoConfiguration_objs = NACL::C::SystemServiceProcessorNetworkAutoConfiguration->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my SystemServiceProcessorNetworkAutoConfiguration_state = $SystemServiceProcessorNetworkAutoConfiguration_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemServiceProcessorNetworkAutoConfiguration would return 
a NACL::CS::SystemServiceProcessorNetworkAutoConfiguration object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 disable

    NACL::C::SystemServiceProcessorNetworkAutoConfiguration->disable(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to disable Service Processor Auto-Configuration.
Uses CMode CLI/ZAPI .
Invokes  service-processor-auto-configuration-disable  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'address-family' => $string >>

Subnet Address Family possible value(s) are, IPv4,IPv6
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "system service processor network auto configuration disable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub disable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( address-family
                    )
            ),
        },
        api => 'system_service_processor_network_auto_configuration_disable',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_disable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_disable_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend =>
            'NACL::C::SystemServiceProcessorNetworkAutoConfiguration::disable',
    );

    $Log->exit() if $may_exit;
}

sub _disable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_service_processor_network_auto_configuration_disable',
    );
    $Log->exit() if $may_exit;
}

sub _disable_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(address-family)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'service-processor-auto-configuration-disable',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

=head2 enable

    NACL::C::SystemServiceProcessorNetworkAutoConfiguration->enable(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to enable Service Processor Auto-Configuration.
Uses CMode CLI/ZAPI .
Invokes  service-processor-auto-configuration-enable  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'subnet-name' => $string >>

Subnet Name
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'address-family' => $string >>

Subnet Address Family possible value(s) are, IPv4,IPv6
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "system service processor network auto configuration enable" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub enable {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( subnet-name address-family
                    )
            ),
        },
        api => 'system_service_processor_network_auto_configuration_enable',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_enable_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_enable_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend =>
            'NACL::C::SystemServiceProcessorNetworkAutoConfiguration::enable',
    );

    $Log->exit() if $may_exit;
}

sub _enable_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'system_service_processor_network_auto_configuration_enable',);
    $Log->exit() if $may_exit;
}

sub _enable_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(address-family subnet-name)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'service-processor-auto-configuration-enable',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

__PACKAGE__->_load_event_class();

1;
