# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServicesDns Component Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemServicesDns

=head1 DESCRIPTION

C<NACL::C::SystemServicesDns> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>.  It represents access
to an ONTAP System Services Dns.  A related class is
L<NACL::CS::SystemServicesDns|lib-NACL-CS-SystemServicesDns-pm>,
which represents the past state of an ONTAP System Services Dns.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

The Node name.

=cut

package NACL::C::SystemServicesDns;

use strict;
use warnings;
use NACL::CS::SystemServicesDns ();
use NACL::ComponentUtils qw (_optional_scalars);
use Params::Validate qw(validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base 'NACL::C::Component::ONTAP';

=head1 METHODS

=head2 new

  my $dns_object = NACL::C::SystemServicesDns->new(
                                                   command_interface=>$ci,
                                                   node =>$node_name,
											       ....);

Return a new System Services Dns component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
];

=head2 modify

NACL::C::SystemServicesDns->modify(command_interface=>,
                               node=>$node_name,...);

                            or

  $dns_object->modify(...);

(Class or instance method)This method modifies DNS element.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface=> $ci >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should=>$ruleset >>

(Optional)See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< node=>$string >>

(Required for class method, Not Applicable for instance method)
(Required) Node
This specifies the node whose DNS configuration is to be modified.

Applicable for CMode CLI.

=item C<< domains=>$string >>

(Optional) Domains
  This optionally specifies a DNS domain or domains. Specify multi-
  ple domains in a comma-separated list.

Applicable for CMode CLI.

=item C<< ns1=>$string >>

(Optional) First Nameserver
  This optionally specifies the IP address of the primary name
  server.

Applicable for CMode CLI.

=item C<< ns2=>$string >>

(Optional) Second Nameserver
  This optionally specifies the IP address of the secondary name
  server.

Applicable for CMode CLI.

=item C<< ns3=>$string >>

(Optional) Third Nameserver
  This optionally specifies the IP address of the tertiary name
  server.

Applicable for CMode CLI.

=item C<< state=>$boolean >>

(Optional) Enable/Disable DNS

Applicable for CMode CLI.

=item C<< timeout=>$number >>

(Optional) Timeout (secs)
  This optionally specifies, in seconds, the timeout value for DNS
  requests. Possible values range from 1 to 5 seconds; the default set-
  ting is 2 seconds.

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system services dns modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut


sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            domains => { type => ARRAYREF, optional => 1 },
            _optional_scalars(qw(ns1 ns2 ns3 state timeout attempts))
        },
        api => 'system_services_dns_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode"
            }
        ],
        frontend => 'NACL::C::SystemServicesDns::modify',
    );
    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @opts ) = @_;
    $pkg_or_obj->_base_backend_cmode_cli( @opts,
        api => 'system_services_dns_modify', );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ( node => { type => SCALAR }, );
}

1;
