# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServicesNdmpLog Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemServicesNdmpLog

=head1 DESCRIPTION

C<NACL::C::SystemServicesNdmpLog> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. 
Unlike other components this package does not model any element/service 
but is present to provide component-level support for functionality. 
Since it does not model any element/service there is no ComponentState 
module for this component. For the same reason, 'find' and 'state' will
not work for this component.
Instead NACL::CS::SystemServicesNdmpLog->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::SystemServicesNdmpLog;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemServicesNdmpLog = NACL::C::SystemServicesNdmpLog->new(
                                   command_interface => $ci,
								  );

Return a new SystemServicesNdmpLog component object with the given attributes.

=cut

use Class::MethodMaker [];

=head2 stop

    NACL::C::SystemServicesNdmpLog->stop(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to stop logging for the specified NDMP session.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'session-id' => $string >>

*Session Identifier, possible value(s) are <integer>,server
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( session-id node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_stop_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( session-id node extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->system_services_ndmp_log_stop(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
}

=head2 start

    my $SystemServicesNdmpLog_obj	= NACL::C::SystemServicesNdmpLog->start(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to start logging for the specified NDMP session.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'filter' => $string >>

*Level Filter
Applicable for CMode CLI

=item C<< 'session-id' => $string >>

*Session Identifier, possible value(s) are <integer>,server
Applicable for CMode CLI

=item C<< 'node' => $string >>

Node
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( filter session-id node
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_start_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $obj;
}

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my %opts = $pkg->_validate_backend_opts(@opts);

    my $command_interface     = $opts{command_interface};
    my $apiset                = $opts{apiset};
    my $execution_time        = $opts{cmd_execution_time};
    my $allowed_cmode_options = [
        qw( filter session-id node extended_query
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    my $response = $apiset->system_services_ndmp_log_start(%cmode_args);
    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }

    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $command_interface,);
}


__PACKAGE__->_load_event_class();

1;
