# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServicesWebDiagnostic Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemServicesWebDiagnostic

=head1 DESCRIPTION

C<NACL::C::SystemServicesWebDiagnostic> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP SystemServicesWebDiagnostic.  A related class is
L<NACL::CS::SystemServicesWebDiagnostic|lib-NACL-CS-SystemServicesWebDiagnostic-pm>, which represents
the past state of an ONTAP SystemServicesWebDiagnostic.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=cut

package NACL::C::SystemServicesWebDiagnostic;

use strict;
use warnings;

use NACL::CS::SystemServicesWebDiagnostic;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemServicesWebDiagnostic = NACL::C::SystemServicesWebDiagnostic->new(command_interface=>$ci,
                                   node =>$node,
								  );

Return a new SystemServicesWebDiagnostic component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'node',];

=head2 find

    my $SystemServicesWebDiagnostic  = NACL::C::SystemServicesWebDiagnostic->find(command_interface => $ci, ...);
	my @SystemServicesWebDiagnostics = NACL::C::SystemServicesWebDiagnostic->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $SystemServicesWebDiagnostic_state = $NACL::C::SystemServicesWebDiagnostic->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemServicesWebDiagnostic would return
a NACL::CS::SystemServicesWebDiagnostic object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 start

    my $SystemServicesWebDiagnostic_object	= NACL::C::SystemServicesWebDiagnostic->start(
                                command_interface => $ci,
	                             node => $node,
                                  ...);
(Class method)
This method is used to start the web service diagnostic testing.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required)

Applicable for CMode CLI.

=item C<< "foreground" => $string >>

Applicable for CMode CLI.

=item C<< "method-timeout"=> $scalar >>

(Optional) The new timeout value.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"system services web diagnostic start" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( foreground
                    )
            ),
        },
        cli_cmd => 'system services web diagnostic start',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_start_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemServicesWebDiagnostic::start',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my %returnobj;

    $pkg->_base_backend_cmode_cli (
        %opts,
        api => 'system_services_web_diagnostic_start',
    );
    $Log->exit() if $may_exit;
    $returnobj{"command_interface"} = $opts{command_interface};
    if ($opts{'node'}) {
        $returnobj{node} = $opts{'node'};
    }
    return $pkg->new(%returnobj);
} ##end sub _start_cmode_cli

=head2 stop

   NACL::C::SystemServicesWebDiagnostic->stop(
                                command_interface => $ci,
	                             node => $node,
                                  ...);

								 or

    $SystemServicesWebDiagnostic_object->stop(...);

(Class or instance method)
This method is used to stop the web service diagnostic testing.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "node" => $node >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"system services web diagnostic stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params    => \@args,
        cli_cmd   => 'system services web diagnostic stop',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_stop_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::SystemServicesWebDiagnostic::stop',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @args,
        api => 'system_services_web_diagnostic_stop',
    );

    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('node' => {type => SCALAR, optional => 1},);
}

1;
