# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSmtape Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemSmtape

=head1 DESCRIPTION

C<NACL::C::SystemSmtape> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. Unlike other components this
package does not model any element/service but is present to provide
component-level support for functionality. Since it does not model any
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.
Instead NACL::CS::SystemSmtape->fetch() should be used.
(See L<fetch|lib-NACL-CS-ComponentState-pm/fetch> for a full description)

=cut

=head1 ATTRIBUTES

=head2 session

=cut

package NACL::C::SystemSmtape;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemSmtape = NACL::C::SystemSmtape->new(
                                   command_interface => $ci,
                                  );

Return a new SystemSmtape component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'session',];

=head2 break

    NACL::C::SystemSmtape->break(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to make a restored volume read-write.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system smtape break" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub break {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_break',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_break_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::break',
    );

    $Log->exit() if $may_exit;
} ## end sub break

sub _break_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_smtape_break',);
    $Log->exit() if $may_exit;
} ## end sub _break_cmode_cli

=head2 continue

    NACL::C::SystemSmtape->continue(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to continue SMTape session waiting at the end of tape.

Uses a CMode CLI, 7Mode APISet.

Invokes 'smtape continue' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'session' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : job id

=item C<< 'tape' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'volume' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : path

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system smtape continue" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub continue {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( session tape volume
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_continue',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_continue_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_continue_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::continue',
    );

    $Log->exit() if $may_exit;
} ## end sub continue

sub _continue_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_smtape_continue',);
    $Log->exit() if $may_exit;
} ## end sub _continue_cmode_cli

sub _continue_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my %api_opts;
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => {
            'session' => 'job-id',
            'volume'  => 'path',
        },
    );

    $apiset->smtape_continue(%api_opts);
    $Log->exit() if $may_exit;
} ## end sub _continue_7mode_cli

=head2 showheader

    NACL::C::SystemSmtape->showheader(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to display SMTape header.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'tape-block-size' => $string >>

Applicable for CMode CLI.

=item C<< 'tape' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system smtape showheader" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub showheader {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( tape-block-size tape
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_showheader',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_showheader_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::showheader',
    );

    $Log->exit() if $may_exit;
} ## end sub showheader

sub _showheader_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_smtape_showheader',);
    $Log->exit() if $may_exit;
} ## end sub _showheader_cmode_cli

=head2 abort

    NACL::C::SystemSmtape->abort(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to abort an active SMTape session.

Uses a CMode CLI, 7Mode APISet.

Invokes 'smtape abort' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'session' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : job id

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"system smtape abort" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub abort {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( session
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_abort',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_abort_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_abort_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::abort',
    );

    $Log->exit() if $may_exit;
} ## end sub abort

sub _abort_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'system_smtape_abort',);
    $Log->exit() if $may_exit;
} ## end sub _abort_cmode_cli

sub _abort_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my %api_opts;
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => {'session' => 'job-id'},
    );

    $apiset->smtape_abort(%api_opts);
    $Log->exit() if $may_exit;
} ## end sub _abort_7mode_cli

=head2 restore

    NACL::C::SystemSmtape->restore(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to restore a volume from tape devices.

Uses a CMode CLI, 7Mode APISet.

Invokes 'smtape restore' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : path

=item C<< 'tape-block-size' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : -b <value> (Programmatic name: block-size)

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'tape' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : tape-device

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'file-path' => $string >>

File name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"system smtape restore" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restore {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume tape-block-size vserver tape

                    file-path
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_restore',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restore_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_restore_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::restore',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub restore

sub _restore_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'system_smtape_restore',
        apiset_response_obj => \$response,
    );

    my $output = $response->get_raw_output();
    my $session;
    if ($output =~ m/(\d+)/g) {
        $session = $1;
    }
    $Log->exit() if $may_exit;

    return $pkg->new(
        session           => $session,
        command_interface => $opts{command_interface},
    );

} ## end sub _restore_cmode_cli

sub _restore_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my %api_opts;
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => {
            'tape'            => 'tape-device',
            'volume'          => 'path',
            'tape-block-size' => 'block-size',
        },
    );

    my $response      = $apiset->smtape_restore(%api_opts);
    my $parsed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $pkg->new(
        session           => $parsed_output->[0]{'job_id'},
        command_interface => $command_interface
    );
} ## end sub _restore_7mode_cli

=head2 backup

    NACL::C::SystemSmtape->backup(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to backup a volume to tape devices.

Uses a CMode CLI or 7Mode APISet.

Invokes 'smtape backup' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'volume' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : path

=item C<< 'tape-block-size' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : -b <value> (Programmatic name: block-size)

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'tape' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : tape-device

=item C<< 'backup-snapshot' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : -s <value> (Programmatic name: snapshot-name)

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'session' => $string >>

Applicable for CMode CLI, 7Mode/Nodescope CLI.

Maps to:

7M CLI : job-id

=item C<< 'extended_query' => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< 'ref-snapshot' => $string >>

Snapshot Name
Applicable for CMode CLI

=item C<< 'backup-set-id' => $string >>

Backup set ID
Applicable for CMode CLI

=item C<< 'file-path' => $string >>

File name
Applicable for CMode CLI

=item C<< 'metadata-only' => $string >>

Metadata possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"system smtape backup" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub backup {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( volume tape-block-size vserver tape backup-snapshot

                    ref-snapshot backup-set-id

                    file-path metadata-only
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'system_smtape_backup',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_backup_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_backup_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        frontend => 'NACL::C::SystemSmtape::backup',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub backup

sub _backup_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'system_smtape_backup',
        apiset_response_obj => \$response
    );
    my $output = $response->get_raw_output();
    my $session;
    if ($output =~ m/(\d+)/g) {
        $session = $1;
    }
    $Log->exit() if $may_exit;

    return $pkg->new(
        session           => $session,
        command_interface => $opts{command_interface},
    );

} ## end sub _backup_cmode_cli

sub _backup_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );
    my %api_opts;
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    $pkg->_hash_copy(
        source => \%opts,
        target => \%api_opts,
        map    => {
            'backup-snapshot' => 'snapshot-name',
            'volume'          => 'path',
            'tape-block-size' => 'block-size',
            'tape'            => 'tape-device',
            'session'         => 'job-id',
        },
    );

    #$apiset->smtape_backup(%api_opts);
    #$Log->exit() if $may_exit;

    my $response      = $apiset->smtape_backup(%api_opts);
    my $parsed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;
    return $pkg->new(
        session           => $parsed_output->[0]{'job_id'},
        command_interface => $command_interface
    );
} ## end sub _backup_7mode_cli

=head2 restore_get_volume_geometry

   my $smtape_restore_get_volume_geometry_object =
            NACL::C::Smtape->restore_get_volume_geometry(
                   command_interface => $ci,
                   'volume-path'     => $volume_path
            ...);

(Class method)
This method is used to list the volume geometry of the specified path.
For this purpose, use this method instead of NACL::C::Smtape->restore because
the latter is used only for performing an Smtape restore operation.

This method returns an object of type NACL::CS::SmtapeRestoreGetVolumeGeometry.

Uses a 7Mode, Nodescope and CLI APISet.

Invokes 'smtape restore' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "volume-path" => $volume_path >>

(Required) Path for which volume geometry will be retrieved

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restore_get_volume_geometry {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'restore_get_volume_geometry' "
            . "front end are:\n"
            . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => {'volume-path' => {type => SCALAR},},
        ignore_primary_keys => 1,
    );
    my $volume_geometry_obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restore_get_volume_geometry_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $volume_geometry_obj;
} ## end sub restore_get_volume_geometry

sub _restore_get_volume_geometry_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $response      = $apiset->smtape_restore(%opts);
    my $parsed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;

    return NACL::CS::SmtapeRestoreGetVolumeGeometry->new(
        %{$parsed_output->[0]});
} ## end sub _restore_get_volume_geometry_7mode_cli

=head2 restore_get_image_header

   my $smtape_restore_get_image_header_object =
            NACL::C::Smtape->restore_get_image_header(
                   command_interface => $ci,
                   'backup-image'    => $tape_name
            ...);

(Class method)
This method is used to list the contents of a backup image header.
For this purpose, use this method instead of NACL::C::Smtape->restore because
the latter is used only for performing an Smtape restore operation.

This method returns an object of type NACL::CS::SmtapeRestoreGetImageHeader.

Uses a 7Mode, Nodescope and CLI APISet.

Invokes 'smtape restore' command for 7Mode/Nodescope CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "backup-image" => $tape_name >>

(Required) Tape device for which image header will be retrieved

Applicable for 7Mode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub restore_get_image_header {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->debug("Opts to 'restore_get_image_header' "
            . "front end are:\n"
            . Dumper(\@_));
    my %opts = $pkg->_common_validate_with(
        params              => \@_,
        additional_spec     => {'backup-image' => {type => SCALAR},},
        ignore_primary_keys => 1,
    );
    my $image_header_obj = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_restore_get_image_header_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $image_header_obj;
} ## end sub restore_get_image_header

sub _restore_get_image_header_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $response      = $apiset->smtape_restore(%opts);
    my $parsed_output = $response->get_parsed_output();
    $Log->exit() if $may_exit;

    my $backup_sequence_no
        = delete $parsed_output->[0]{'backup_sequence_no.'};
    $parsed_output->[0]{'backup_sequence_no'} = $backup_sequence_no;
    my $image_header_obj
        = NACL::CS::SmtapeRestoreGetImageHeader->new(%{$parsed_output->[0]});
} ## end sub _restore_get_image_header_7mode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('session' => {type => SCALAR},);
}
1;
