# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSmtapeStatus Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::SystemSmtapeStatus

=head1 DESCRIPTION

C<NACL::C::SystemSmtapeStatus> is a derived class of
L<NACL::C::Component|lib-NACL-C-Component-pm>. It represents access
to an ONTAP SystemSmtapeStatus.  A related class is
L<NACL::CS::SystemSmtapeStatus|lib-NACL-CS-SystemSmtapeStatus-pm>, which represents
the past state of an ONTAP SystemSmtapeStatus.
However, since it does not have any primary keys, 'find' and 'state' do not
work for this component. (This component is considered a 'singleton',
i.e. there is only one row shown by the relevant 'show' command)

=cut

=head1 ATTRIBUTES

None.

=cut

package NACL::C::SystemSmtapeStatus;

use strict;
use warnings;

use NACL::CS::SystemSmtapeStatus;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $SystemSmtapeStatus = NACL::C::SystemSmtapeStatus->new(
                                   command_interface => $ci,
								  );

Return a new SystemSmtapeStatus component object with the given attributes.

=cut

=head2 find

    my $SystemSmtapeStatus_obj  = NACL::C::SystemSmtapeStatus->find(command_interface => $ci, ...);
	my @SystemSmtapeStatus_objs = NACL::C::SystemSmtapeStatus->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my SystemSmtapeStatus_state = $SystemSmtapeStatus_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::SystemSmtapeStatus would return 
a NACL::CS::SystemSmtapeStatus object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 clear

    NACL::C::SystemSmtapeStatus->clear(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to clear SMTape sessions.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'end-time' => $string >>

Applicable for CMode CLI.

=item C<< 'status' => $string >>

Applicable for CMode CLI.

=item C<< 'progress' => $string >>

Applicable for CMode CLI.

=item C<< 'path' => $string >>

Applicable for CMode CLI.

=item C<< 'backup-snapshot' => $string >>

Applicable for CMode CLI.

=item C<< 'device' => $string >>

Applicable for CMode CLI.

=item C<< 'node' => $string >>

Applicable for CMode CLI.

=item C<< 'update-time' => $string >>

Applicable for CMode CLI.

=item C<< 'start-time' => $string >>

Applicable for CMode CLI.

=item C<< 'tape-block-size' => $string >>

Applicable for CMode CLI.

=item C<< 'session' => $string >>

Applicable for CMode CLI.

=item C<< 'type' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub clear {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( end-time status progress path backup-snapshot device node update-time
                    start-time tape-block-size session type
                    )
            ),
        },
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_clear_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _clear_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( end-time status progress path backup-snapshot device node update-time
            start-time tape-block-size session type
            )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->system_smtape_status_clear(%cmode_args);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

1;
