# $Id$
#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Container Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VPserver::Container

=head1 DESCRIPTION

C<NACL::C::VPserver::Container> is a derived class of
L<NACL::C::Component::VPserver|lib-NACL-C-Component-VPserver-pm>. It represents access
to VPserver Container Objects.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::VPserver|lib-NACL-C-Component-VPserver-pm>

=head2 container

The name of the Container.

=head2 container

The protocol of the Container.

=cut

package NACL::C::VPserver::Container;

use strict;
use warnings;

use base 'NACL::C::Component::VPserver';

use NACL::C::VPserver;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);

use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::C::Exceptions::VPserver::StorageAlreadyAdded;

=head1 METHODS

=head2 new

  my $Container = NACL::C::VPserver::Container->new(command_interface => $vp_ci);

Return a new VPserver Container component object with the given attributes.

=cut

use Class::MethodMaker [
        scalar => [
            {-type => 'NACL::C::VPserver'},
            'command_interface'
        ],
        scalar => 'container',
        scalar => 'protocol',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub init {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (!$opts{command_interface}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::new.");
    }

    $Log->exit() if $may_exit;
    return $pkg;
}

=head2 list

    my $container_objs  = NACL::C::VPserver::Container->list(command_interface => $vp_ci);

(Class method)
A method which discovers all containers that are registered with the VPserver (by
interacting with the given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::list.");
    }

    my $response = $pkg->command_interface->call(command => "containerList",
                                                 params  => [{fullDetail => 'false'}]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 list_storage

    my $container_storage_lists  = NACL::C::VPserver::Container->list_storage(command_interface => $vp_ci);

(Class method)
A method which discovers all storage backing containers that are registered with the VPserver (by
interacting with the given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub list_storage {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::list.");
    }

    my $response = $pkg->command_interface->call(command => "containerListStorage");

    $Log->exit() if $may_exit;
    return $response;
}


=head2 create

    my $container_create_result = NACL::C::VPserver::Container->create(command_interface => $vp_ci,
                                                                       container => $container_ip,
                                                                       protocol => $protocol);

(Class method)
A method which creates a Container with the VPserver (by interacting with the
given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< container => $container_ip >>

(Required) The ip address of the container to remove.

=item C<< protocol => $protocol >>

(Required) The protocol for the container to add.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface and !$opts{command_interface}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::create.");
    }
    if (!$opts{container}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'container' missing in call to NACL::C::VPserver::Container::create.");
    }
    if (!$opts{protocol}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'protocol' missing in call to NACL::C::VPserver::Container::create.");
    }

    my $response = $pkg->command_interface->call(command => "containerCreate",
                                                 params => [
                                                     {
                                                          container => {
                                                              name => $opts{container},
                                                              protocol => $opts{protocol}
                                                          }
                                                     }
                                                 ]);
    $pkg->container($opts{container});
    $pkg->protocol($opts{protocol});

    $Log->exit() if $may_exit;
    return $response;
}

=head2 delete

    my $container_create_result = NACL::C::VPserver::Container->delete(command_interface => $vp_ci,
                                                                       container => $container_name);

A method which deletes a Container with the VPserver (by interacting with the
given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< container => $container_name >>

(Required) The name of the container to remove.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::delete.");
    }

    my $container = $opts{container};
    if ($pkg->container) {
        $container = $pkg->container;
    }
    if (!$container) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'container' missing in call to NACL::C::VPserver::Container::delete.");
    }

    my $response = $pkg->command_interface->call(command => "containerDelete",
                                                 params => [
                                                     {containerName => $container},
                                                 ]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 delete_storage

    my $delete_storage_result = NACL::C::VPserver::Container->delete_storage(command_interface => $vp_ci,
                                                                             container => $container_name,
                                                                             cluster => $cluster,
                                                                             volume => $volume,
                                                                             vserver => $vserver);

(Class method)
A method which deletes storage on a Container with the VPserver (by interacting with the
given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< container => $container_name >>

(Required) The name of the container to remove.

=item C<< cluster => $cluster >>

(Required) The ip for the cluster that the storage resides on.

=item C<< volume => $volume >>

(Required) The name of the volume to add the containter to.

=item C<< vserver => $vserver >>

(Required) The vserver of the volume to add the container to.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_storage {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::delete_storage.");
    }
    if (!$opts{container}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'container' missing in call to NACL::C::VPserver::Container::delete_storage.");
    }
    if (!$opts{cluster}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'cluster' missing in call to NACL::C::VPserver::Container::delete_storage.");
    }
    if (!$opts{volume}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'volume' missing in call to NACL::C::VPserver::Container::delete_storage.");
    }
    if (!$opts{vserver}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'vserver' missing in call to NACL::C::VPserver::Container::delete_storage.");
    }

    my $response = $pkg->command_interface->call(command => "containerDeleteStorage",
                                                 params => [{containerName => $opts{container}},
                                                            {storageLocation => {
                                                                 flexVolName => $opts{volume},
                                                                 vserverName => $opts{vserver},
                                                                 clusterIp => $opts{cluster}}}
                                                           ]);
    $Log->exit() if $may_exit;
    return $response;
}

=head2 add_storage

    my $credential = NATE::SOAP::Data->new(credential => {VPUsername => "vpserver",
                                                          filerIP => $ip,
                                                          filerUsername => "admin",
                                                          filerPassword => "netapp1!",
                                                          VPPassword => "netapp1!",
                                                          vCenterIP => "10.228.180.203",
                                                          vCenterPassword => 'Admin!23',
                                                          vCenterUsername => 'Administrator@vsphere.local',
                                                          applicationName => "appname_vp.thpl"});

    my $command_interface = NACL::C::VPserver->new(vpserver => $vpserver_name,
                                                   credential => $credential);


    my $container_add_result = NACL::C::VPserver::Container->add_storage(command_interface => $vp_ci,
                                                                         cluster => $cluster,
                                                                         container => $container,
                                                                         volume => $volume,
                                                                         vserver => $vserver);

(Class method)
A method which adds storage to a container with the VPserver (by interacting with the
given vpserver command interface).

Uses a SOAP VPserver APISet.

=over

=item Options

=over

=item C<< command_interface => $vp_ci >>

(Optional) See L<NACL::C::VPserver|lib-NACL-C-VPserver-pm>

=item C<< cluster => $cluster >>

(Required) The ip for the cluster that the storage resides on.

=item C<< container => $container >>

(Required) The name of the container to add storage to.

=item C<< volume => $volume >>

(Required) The name of the volume for the storage, which will be added to the container.

=item C<< vserver => $vserver >>

(Required) The vserver of the volume for the storage, which will be added to the container.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add_storage {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->command_interface($opts{command_interface}) if (!$pkg->command_interface);
    if (!$pkg->command_interface) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'command_interface' missing in call to NACL::C::VPserver::Container::add_storage.");
    }
    if (!$opts{container}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'container' missing in call to NACL::C::VPserver::Container::add_storage.");
    }
    if (!$opts{cluster}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'cluster' missing in call to NACL::C::VPserver::Container::add_storage.");
    }
    if (!$opts{volume}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'volume' missing in call to NACL::C::VPserver::Container::add_storage.");
    }
    if (!$opts{vserver}) {
        NACL::APISet::Exceptions::MissingRequiredParamException->throw("Mandatory parameter 'vserver' missing in call to NACL::C::VPserver::Container::add_storage.");
    }

    my $response;
    try {
        $response = $pkg->command_interface->call(command => "containerAddStorage",
                                                  params => [{containerName => $opts{container}},
                                                             {storageLocation => {
                                                                  flexVolName => $opts{volume},
                                                                  vserverName => $opts{vserver},
                                                                  clusterIp => $opts{cluster}}}
                                                            ]);
    } catch NATE::BaseException with {
        my $exception = shift;
        NACL::C::Exceptions::VPserver::StorageAlreadyAdded->detect_convert_and_throw(exception => $exception);

        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $response;
}

1;
