#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileCloneToken Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeFileCloneToken

=head1 DESCRIPTION

C<NACL::C::VolumeFileCloneToken> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VolumeFileCloneToken.  A related class is
L<NACL::CS::VolumeFileCloneToken|lib-NACL-CS-VolumeFileCloneToken-pm>, which represents
the past state of an ONTAP VolumeFileCloneToken.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 node

=head2 token-uuid

=cut

package NACL::C::VolumeFileCloneToken;

use strict;
use warnings;

use NACL::CS::VolumeFileCloneToken;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VolumeFileCloneToken = NACL::C::VolumeFileCloneToken->new(
                                   command_interface => $ci,
                                   'node' => $node,
                                   'token-uuid' => $token_uuid,
                                  );

Return a new VolumeFileCloneToken component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'token_uuid',
];

=head2 find

    my $VolumeFileCloneToken_obj  = NACL::C::VolumeFileCloneToken->find(command_interface => $ci, ...);
    my @VolumeFileCloneToken_objs = NACL::C::VolumeFileCloneToken->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VolumeFileCloneToken_state = $VolumeFileCloneToken_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VolumeFileCloneToken would return 
a NACL::CS::VolumeFileCloneToken object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 destroy

    NACL::C::VolumeFileCloneToken->destroy(
                                command_interface => $ci,
                                'node' => $node,
                                ...);
                                 
                                 or

    $VolumeFileCloneToken_obj->destroy(...);

(Class or instance method)
Uses CMode CLI/ZAPI .

Invokes "clone-token-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required for class method, Not Applicable for instance method)
Node Name possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI

=item C<< 'token-uuid' => $string >>
(Required for class method, Not Applicable for instance method)

Token UUID
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "volume file clone token destroy" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub destroy {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( token-uuid
                    )
            ),
        },
        api => 'volume_file_clone_token_destroy',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_destroy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_destroy_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileCloneToken::destroy',
    );

    $Log->exit() if $may_exit;
}

sub _destroy_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'volume_file_clone_token_delete',);
    $Log->exit() if $may_exit;
}

sub _destroy_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'clone-token-delete',
        copy => [qw( node token-uuid )],
    );
    $Log->exit() if $may_exit;
} ## end sub _destroy_cmode_zapi

=head2 create

    my $VolumeFileCloneToken_obj    = NACL::C::VolumeFileCloneToken->create(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

(Class method)
This method is used to create new token to reserve split load.
Uses CMode CLI/ZAPI .

Invokes "clone-token-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)
Node Name possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI

=item C<< 'reserve-size' => $string >>

Available Reserve in Split Load
Applicable for CMode CLI/ZAPI

=item C<< '*' >>

This method takes all other options supported by "volume file clone token create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $token_uuid;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            node => {type => SCALAR},
            _optional_scalars(
                qw( reserve-size
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'volume_file_clone_token_create',
    );

    $pkg->call_on_apiset(
        %opts,
        _token_uuid => \$token_uuid,
        choices     => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileCloneToken::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        node              => $opts{'node'},
        'token_uuid'      => $token_uuid
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my $response;
    $pkg->_base_backend_cmode_cli(
        %opts,
        api                 => 'volume_file_clone_token_create',
        apiset_response_obj => \$response,
    );

    my $token_uuid = $response->{'parsed_output'}->[0]->{'token_uuid'};
    ${$opts{_token_uuid}} = $token_uuid;
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    if (defined $opts{'reserve-size'}) {
        $opts{'reserve-size'}
            = $pkg_or_obj->convert_to_bytes($opts{'reserve-size'});
    }

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api                 => 'clone-token-create',
        copy                => [qw( node reserve-size )],
        apiset_response_obj => \$response,
    );

    my $token_uuid = $response->{'parsed_output'}->[0]->{'token-uuid'};
    ${$opts{_token_uuid}} = $token_uuid;
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VolumeFileCloneToken->modify(
                                command_interface => $ci,
                                'node' => $node,
                                ...);

(Class method)
This method is used to modify the allowed expiry limit for tokens.
Uses CMode CLI/ZAPI .

Invokes "clone-token-modify-expiry-limit" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'node' => $node >>

(Required)
Node Name possible value(s) are, <nodename>,local
Applicable for CMode CLI/ZAPI

=item C<< 'expiry-limit' => $string >>

Token Expiry Limit (in seconds)
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: reservation-duration-limit

=item C<< '*' >>

This method takes all other options supported by "volume file clone token modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            node => {type => SCALAR},
            _optional_scalars(
                qw( expiry-limit
                    )
            ),
        },
        ignore_primary_keys => 1,
        api                 => 'volume_file_clone_token_modify',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeFileCloneToken::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'volume_file_clone_token_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_zapi(
        %opts,
        api  => 'clone-token-modify-expiry-limit',
        copy => [qw( node )],
        map  => {'expiry-limit' => 'reservation-duration-limit',},
    );
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'node'       => {type => SCALAR},
        'token-uuid' => {type => SCALAR},
    );
}

1;
