# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeQtreePolicy Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VolumeQtreePolicy

=head1 DESCRIPTION

C<NACL::C::VolumeQtreePolicy> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VolumeQtreePolicy.  A related class is
L<NACL::CS::VolumeQtreePolicy|lib-NACL-CS-VolumeQtreePolicy-pm>, which represents
the past state of an ONTAP VolumeQtreePolicy.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 qtree

=head2 vserver

=head2 volume

=cut

package NACL::C::VolumeQtreePolicy;

use strict;
use warnings;

use NACL::CS::VolumeQtreePolicy;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VolumeQtreePolicy = NACL::C::VolumeQtreePolicy->new(
                                   command_interface => $ci,
                                   'qtree' => $qtree,
                                   'vserver' => $vserver,
                                   'volume' => $volume,
								  );

Return a new VolumeQtreePolicy component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'qtree',
    scalar => 'vserver',
    scalar => 'volume',
];

=head2 find

    my $VolumeQtreePolicy_obj  = NACL::C::VolumeQtreePolicy->find(command_interface => $ci, ...);
	my @VolumeQtreePolicy_objs = NACL::C::VolumeQtreePolicy->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VolumeQtreePolicy_state = $VolumeQtreePolicy_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VolumeQtreePolicy would return 
a NACL::CS::VolumeQtreePolicy object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $VolumeQtreePolicy_obj	= NACL::C::VolumeQtreePolicy->create(
                                command_interface => $ci,
	                            'qtree' => $qtree,
	                            'vserver' => $vserver,
	                            'volume' => $volume,
                                ...);
   
(Class method)
This method is used to create qtree policy.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'qtree' => $qtree >>

(Required)
Qtree Name
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required)
Vserver Name
Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required)
Volume Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume qtree policy create" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        api    => 'volume_qtree_policy_create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQtreePolicy::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        qtree             => $opts{'qtree'},
        vserver           => $vserver,
        volume            => $opts{'volume'},
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts, api => 'volume_qtree_policy_create',
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VolumeQtreePolicy->delete(
                                command_interface => $ci,
	                            'qtree' => $qtree,
	                            'vserver' => $vserver,
	                            'volume' => $volume,
                                ...);
	                             
								 or

    $VolumeQtreePolicy_obj->delete(...);

(Class or instance method)
This method is used to delete qtree policy.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume qtree policy delete" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        api    => 'volume_qtree_policy_delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQtreePolicy::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'volume_qtree_policy_delete',);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VolumeQtreePolicy->modify(
                                command_interface => $ci,
	                            'qtree' => $qtree,
	                            'vserver' => $vserver,
	                            'volume' => $volume,
                                ...);
	                             
								 or

    $VolumeQtreePolicy_obj->modify(...);

(Class or instance method)
This method is used to modify qtree policy.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'qtree' => $qtree >>

(Required for class method, Not Applicable for instance method)
Qtree Name
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'volume' => $volume >>

(Required for class method, Not Applicable for instance method)
Volume Name
Applicable for CMode CLI

=item C<< 'policyname' => $string >>

Policy Name
Applicable for CMode CLI

=item C<< 'policy' => $string >>

Policy Name
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "volume qtree policy modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( policyname

                    policy
                    )
            ),
        },
        api => 'volume_qtree_policy_modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VolumeQtreePolicy::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'volume_qtree_policy_modify',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'qtree'   => {type => SCALAR},
        'vserver' => {type => SCALAR},
        'volume'  => {type => SCALAR},
    );
}

1;
