#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverActiveDirectory Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverActiveDirectory

=head1 DESCRIPTION

C<NACL::C::VserverActiveDirectory> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverActiveDirectory.  A related class is
L<NACL::CS::VserverActiveDirectory|lib-NACL-CS-VserverActiveDirectory-pm>, which represents
the past state of an ONTAP VserverActiveDirectory.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverActiveDirectory;

use strict;
use warnings;

use NACL::CS::VserverActiveDirectory;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverActiveDirectory = NACL::C::VserverActiveDirectory->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new VserverActiveDirectory component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

=head2 find

    my $VserverActiveDirectory_obj  = NACL::C::VserverActiveDirectory->find(command_interface => $ci, ...);
	my @VserverActiveDirectory_objs = NACL::C::VserverActiveDirectory->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VserverActiveDirectory_state = $VserverActiveDirectory_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverActiveDirectory would return 
a NACL::CS::VserverActiveDirectory object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 create

    my $VserverActiveDirectory_obj	= NACL::C::VserverActiveDirectory->create(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
   
(Class method)
This method is used to create an Active Directory account.

Uses a CMode CLI/ZAPI APISet.

Invokes "active-directory-account-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required)
Vserver

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'account-name' => $string >>

Active Directory NetBIOS Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : account-name

=item C<< 'domain' => $string >>

Fully Qualified Domain Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : domain

=item C<< "admin-username" => $admin_username>>

(optional)For handling user Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-username

=item C<< "admin-password" => $admin_password >>

(optional)For handling password  Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-password

=item C<< 'ou' => $string >>

Organizational Unit
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : organizational-unit

=item C<< 'force-account-overwrite' => $string >>

Force Overwrite possible value(s) are, true,false
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver active directory create" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( account-name domain ou admin-username admin-password
                    force-account-overwrite
                    )
            ),
        },
        cli_cmd  => 'vserver active-directory create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_create_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_create_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverActiveDirectory::create',
    );

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
    );
    $Log->exit() if $may_exit;
    return $obj;
}

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_cli_with_prompts(%opts,
        api => 'vserver_active_directory_create',);
    $Log->exit() if $may_exit;
}

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $opts{'force-account-overwrite'} ||= "true";
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api  => 'active-directory-account-create',
        copy => [qw( account-name domain admin-password admin-username force-account-overwrite)],
        map  => {"ou" => "organizational-unit",},
        frontend_method => 'create'
    );
    $Log->exit() if $may_exit;
}

=head2 delete

    NACL::C::VserverActiveDirectory->delete(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverActiveDirectory_obj->delete(...);

(Class or instance method)
This method is used to delete an Active Directory account.

Uses a CMode CLI/ZAPI APISet.

Invokes "active-directory-account-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< "admin-username" => $admin_username>>

(optional)For handling user Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-username

=item C<< "admin-password" => $admin_password >>

(optional)For handling password  Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-password

=item C<< '*' >>

This method takes all other options supported by "vserver active directory delete" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( admin-username admin-password
                    )
            ),
        },
        cli_cmd => 'vserver active-directory delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverActiveDirectory::delete',
    );

    $Log->exit() if $may_exit;
}

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli_with_prompts(%opts,
        api => 'vserver_active_directory_delete',);
    $Log->exit() if $may_exit;
}

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'active-directory-account-delete',
        copy            => [qw( admin-password admin-username )],
        frontend_method => 'delete'
    );
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VserverActiveDirectory->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverActiveDirectory_obj->modify(...);

(Class or instance method)
This method is used to modify the domain of an Active Directory account.

Uses a CMode CLI/ZAPI APISet.

Invokes "active-directory-account-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< 'domain' => $string >>

Fully Qualified Domain Name
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : domain

=item C<< "admin-username" => $admin_username>>

(optional)For handling user Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-username

=item C<< "admin-password" => $admin_password >>

(optional)For handling password  Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-password

=item C<< '*' >>

This method takes all other options supported by "vserver active directory modify" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( domain admin-username admin-password force-account-overwrite)
            ),
        },
        cli_cmd => 'vserver active-directory modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverActiveDirectory::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli_with_prompts(%opts,
        api => 'vserver_active_directory_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $opts{'force-account-overwrite'} ||= "true";
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'active-directory-account-modify',
        copy            => [qw( domain admin-password admin-username force-account-overwrite)],
        frontend_method => 'modify'
    );
    $Log->exit() if $may_exit;
}

=head2 password_change

    NACL::C::VserverActiveDirectory->password_change(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverActiveDirectory_obj->password_change(...);

(Class or instance method)
This method is used to change the domain account password for an Active Directory account.

Uses a CMode CLI/ZAPI APISet.

Invokes "active-directory-account-password-change" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< '*' >>

This method takes all other options supported by "vserver active directory password change" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub password_change {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver active-directory password-change',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_password_change_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_password_change_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverActiveDirectory::password_change',
    );

    $Log->exit() if $may_exit;
}

sub _password_change_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api => 'vserver_active_directory_password_change',);
    $Log->exit() if $may_exit;
}

sub _password_change_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'active-directory-account-password-change',
        frontend_method => 'password_change'
    );
    $Log->exit() if $may_exit;
}

=head2 password_reset

    NACL::C::VserverActiveDirectory->password_reset(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverActiveDirectory_obj->password_reset(...);

(Class or instance method)
This method is used to reset the domain account password for an Active Directory account.

Uses a CMode CLI/ZAPI APISet.

Invokes "active-directory-account-password-reset" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : vserver

=item C<< "admin-username" => $admin_username>>

(optional)For handling user Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-username

=item C<< "admin-password" => $admin_password >>

(optional)For handling password  Prompts.

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI : admin-password

=item C<< '*' >>

This method takes all other options supported by "vserver active directory password reset" command in ONTAP
Applicable for CMode CLI/ZAPI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub password_reset {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( admin-username admin-password
                    )
            ),
        },
        cli_cmd => 'vserver active-directory password-reset',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_password_reset_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_password_reset_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverActiveDirectory::password_reset',
    );

    $Log->exit() if $may_exit;
}

sub _password_reset_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;

    my $prompts_answers = [
        qr/Enter\s+\w+\s+user\s+ID:\s*/i => sub { $opts{'admin-username'} },
        qr/Enter\s+\w+\s+password:\s*/i  => sub { $opts{'admin-password'} },
    ];
    my %additional_args;
    $additional_args{'connectrec-match_table'} = $prompts_answers;

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api             => 'vserver_active_directory_password_reset',
        additional_args => \%additional_args,
    );
    $Log->exit() if $may_exit;
}

sub _password_reset_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    $pkg->_base_backend_cmode_zapi(
        %opts,
        api             => 'active-directory-account-password-reset',
        copy            => [qw( admin-password admin-username )],
        frontend_method => 'password_reset'
    );
    $Log->exit() if $may_exit;
}

sub _base_backend_cmode_cli_with_prompts {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $prompts_answers = [
        qr/Enter\s+\w+\s+user\s+name:\s*/ => sub { $opts{'admin-username'} },
        qr/Enter\s+\w+\s+password:\s*/    => sub { $opts{'admin-password'} }
    ];
    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        additional_args => {'connectrec-match_table' => $prompts_answers});

    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
