#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsHomeDirectory Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsHomeDirectory

=head1 DESCRIPTION

C<NACL::C::VserverCifsHomeDirectory> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsHomeDirectory.  A related class is
L<NACL::CS::VserverCifsHomeDirectory|lib-NACL-CS-VserverCifsHomeDirectory-pm>, which represents
the past state of an ONTAP VserverCifsHomeDirectory.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverCifsHomeDirectory;

use strict;
use warnings;

use NACL::CS::VserverCifsHomeDirectory;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverCifsHomeDirectory = NACL::C::VserverCifsHomeDirectory->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                  );

Return a new VserverCifsHomeDirectory component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

=head2 find

    my $VserverCifsHomeDirectory  = NACL::C::VserverCifsHomeDirectory->find(command_interface => $ci, ...);
    my @VserverCifsHomeDirectorys = NACL::C::VserverCifsHomeDirectory->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsHomeDirectory_state = $NACL::C::VserverCifsHomeDirectory->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsHomeDirectory would return
a NACL::CS::VserverCifsHomeDirectory object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VserverCifsHomeDirectory->modify(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverCifsHomeDirectory_object->modify(...);

(Class or instance method)
This method is used to modify home directory configuration.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "enabled" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'is-home-dirs-access-for-public-enabled' => $string >>


Is Home Directory Access for Public Enabled possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'is-home-dirs-access-for-admin-enabled' => $string >>


Is Home Directory Access for Admin Enabled possible value(s) are, true,false
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( enabled

                    is-home-dirs-access-for-public-enabled
                    is-home-dirs-access-for-admin-enabled
                    )
            ),
        },
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( vserver enabled
            extended_query
            is-home-dirs-access-for-public-enabled
            is-home-dirs-access-for-admin-enabled
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'vserver_cifs_home_directory_modify' are: \n"
            . Dumper(\%opts));

    $apiset->vserver_cifs_home_directory_modify(%opts);
    $Log->exit() if $may_exit;
}

=head2 show_user

    my $show_user_obj = NACL::C::VserverCifsHomeDirectory->show_user(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_user_objs = NACL::C::VserverCifsHomeDirectory->show_user(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_user_obj = $VserverCifsHomeDirectory_obj->show_user(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverCifsHomeDirectoryShowUser-pm/ATTRIBUTES>
of NACL::CS::VserverCifsHomeDirectoryShowUser as input. 
This method returns objects of the type L<NACL::CS::VserverCifsHomeDirectoryShowUser|lib-NACL-CS-VserverCifsHomeDirectoryShowUser-pm>.

=cut

sub show_user {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverCifsHomeDirectoryShowUser;
    my @state_objs = NACL::CS::VserverCifsHomeDirectoryShowUser->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
