#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsHomeDirectorySearchPath Component Module
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::VserverCifsHomeDirectorySearchPath

=head1 DESCRIPTION

C<NACL::C::VserverCifsHomeDirectorySearchPath> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsHomeDirectorySearchPath.  A related class is
L<NACL::CS::VserverCifsHomeDirectorySearchPath|lib-NACL-CS-VserverCifsHomeDirectorySearchPath-pm>, which represents
the past state of an ONTAP VserverCifsHomeDirectorySearchPath.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 path

=cut

package NACL::C::VserverCifsHomeDirectorySearchPath;

use strict;
use warnings;

use NACL::CS::VserverCifsHomeDirectorySearchPath;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverCifsHomeDirectorySearchPath = NACL::C::VserverCifsHomeDirectorySearchPath->new(command_interface=>$ci,
                                   vserver => $vserver,
                                   path => $path,
                                  );

Return a new VserverCifsHomeDirectorySearchPath component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'path',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsHomeDirectorySearchPath  = NACL::C::VserverCifsHomeDirectorySearchPath->find(command_interface => $ci, ...);
    my @VserverCifsHomeDirectorySearchPaths = NACL::C::VserverCifsHomeDirectorySearchPath->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsHomeDirectorySearchPath_state = $NACL::C::VserverCifsHomeDirectorySearchPath->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsHomeDirectorySearchPath would return
a NACL::CS::VserverCifsHomeDirectorySearchPath object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 remove

   NACL::C::VserverCifsHomeDirectorySearchPath->remove(
                                command_interface => $ci,
                                vserver => $vserver,
                                path => $path,
                                ...);

                                 or

    $VserverCifsHomeDirectorySearchPath_object->remove(...);

(Class or instance method)
This method is used to remove a home directory search path.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-home-directory-search-path-remove" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< path => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs home-directory search-path remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver cifs home-directory search-path remove',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_remove_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsHomeDirectorySearchPath::remove',
        nacl_exceptions => [qw(DoesNotExist)]
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_home_directory_search_path_remove',);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

sub _remove_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(path)],
        api  => 'cifs-home-directory-search-path-remove',
    );
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_zapi

=head2 add

   my $VserverCifsHomeDirectorySearchPath_object =
       NACL::C::VserverCifsHomeDirectorySearchPath->add(
                                command_interface => $ci,
                                vserver => $vserver,
                                path => $path,
                                  ...);

(Class method)
This method is used to add a home directory search path. It returns a new
component object modeling the newly created home directory search path.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-home-directory-search-path-add" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< path => $path >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs home-directory search-path add" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver cifs home-directory search-path add',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_add_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsHomeDirectorySearchPath::add',
        nacl_exceptions => [qw(AlreadyExists)]
    );

    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        path              => $opts{path},
    );
} ## end sub add

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_home_directory_search_path_add',);
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_cli

sub _add_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(path)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-home-directory-search-path-add',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_zapi

=head2 reorder

   NACL::C::VserverCifsHomeDirectorySearchPath->reorder(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 path => $path,
                                  ...);

                                 or

    $VserverCifsHomeDirectorySearchPath_object->reorder(...);

(Class or instance method)
This method is used to change the search path order used to find a match.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-home-directory-search-path-reorder" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< vserver => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
         option.

=item C<< path => $path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: path

=item C<< "to-position" => $string >>

Target Position

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: to-position

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs home-directory search-path reorder" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reorder {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {_optional_scalars(qw(to-position))},
        cli_cmd         => 'vserver cifs home-directory search-path reorder',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_reorder_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_reorder_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsHomeDirectorySearchPath::reorder',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub reorder

sub _reorder_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_home_directory_search_path_reorder',);
    $Log->exit() if $may_exit;
} ## end sub _reorder_cmode_cli

sub _reorder_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = [qw(to-position path)];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-home-directory-search-path-reorder',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
} ## end sub _reorder_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver' => {type => SCALAR | UNDEF, 'optional' => '1'}
        ,    #vserver made optional because of the burt : 581933
        'path' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
