#
# Copyright (c) 2001-2017 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsOptions Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsOptions

=head1 DESCRIPTION

C<NACL::C::VserverCifsOptions> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsOptions.  A related class is
L<NACL::CS::VserverCifsOptions|lib-NACL-CS-VserverCifsOptions-pm>, which represents
the past state of an ONTAP VserverCifsOptions.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverCifsOptions;

use strict;
use warnings;

use NACL::CS::VserverCifsOptions;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);

=head1 METHODS

=head2 new

  my $VserverCifsOptions = NACL::C::VserverCifsOptions->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                  );

Return a new VserverCifsOptions component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsOptions  = NACL::C::VserverCifsOptions->find(command_interface => $ci, ...);
    my @VserverCifsOptionss = NACL::C::VserverCifsOptions->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsOptions_state = $NACL::C::VserverCifsOptions->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsOptions would return
a NACL::CS::VserverCifsOptions object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VserverCifsOptions->modify(
                                command_interface => $ci,
                                 vserver => $vserver,
                                  ...);

                                 or

    $VserverCifsOptions_object->modify(...);

(Class or instance method)
This method is used to modify CIFS options.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "cifs-options-modify" API for CMode ZAPI.

Invokes "options" command for 7Mode CLI if option-name option passed in
is "on"/"off".

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "smb2-enabled" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-smb2-enabled

7M CLI : SMB2 version enabled (Programmatic name: "cifs.smb2.enable")

=item C<< "smb1-enabled" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-smb1-enabled

=item C<< "default-unix-user" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: default-unix-user

=item C<< "read-grants-exec" => $string >>

Applicable for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CM ZAPI: read-grants-execute

7M CLI : execute permissions granted
(Programmatic name: "cifs.grant_implicit_exe_perms")

=item C<< "wins-servers" => [ $wins-server1, $wins-server2, ... ] >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: wins-servers

=item C<< 'max-mpx' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-mpx

=item C<< 'smb2-2-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-smb2-2-enabled

=item C<< 'shadowcopy-dir-depth' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: shadowcopy-dir-depth

=item C<< 'copy-offload-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-copy-offload-enabled

=item C<< 'default-unix-group' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: default-unix-group

=item C<< 'shadowcopy-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-shadowcopy-enabled

=item C<< 'node-referrals-enabled' => $string >>

Applicable for CMode CLI.

=item C<< 'local-auth-enabled' => $string >>

Enable/Disable Local User Authentication, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'local-users-and-groups-enabled' => $string >>

Applicable for CMode CLI.

=item C<< 'is-referral-enabled' => $string >>

*Refer Clients to More Optimal LIFs, possible value(s) are true,false

Applicable for CMode CLI.

=item C<< 'is-local-auth-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-local-auth-enabled

=item C<< 'is-local-users-and-groups-enabled' => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-local-users-and-groups-enabled

=item C<< 'smb3-enabled' => $string >>

*Enable/Disable the SMB3 Protocol, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-smb3-enabled

=item C<< 'is-exportpolicy-enabled' => $string >>

*Enable/Disable Export Policies for CIFS, possible value(s) are true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-exportpolicy-enabled

=item C<< 'is-use-junctions-as-reparse-points-enabled' => $string >>

Enable/Disable Reparse Point Support possible value(s) are, true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-use-junctions-as-reparse-points-enabled

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< 'is-unix-nt-acl-enabled' => $string >>

Enable/Disable NT ACLs on UNIX security-style volumes possible value(s) are, true,false

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: is-unix-nt-acl-enabled

=item C<< 'is-trusted-domain-enum-search-enabled' => $string >>

Enable/Disable enumeration of trusted domain and search capability possible value(s) are, true,false

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: is-trusted-domain-enum-search-enabled

=item C<< 'client-session-timeout' => $string >>

Idle Timeout Before CIFS Session Disconnect (secs)
Applicable for CMode CLI

=item C<< 'dac-enabled' => $string >>


Enable/Disable Dynamic Access Control (DAC) possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'restrict-anonymous' => $string >>


Restrictions for Anonymous User possible value(s) are, none,no-enumeration,no-access
Applicable for CMode CLI

=item C<< 'is-read-only-delete-enabled' => $string >>


Enable/Disable Deletion of Read-Only Files possible value(s) are, enabled,disabled
Applicable for CMode CLI

=item C<< 'is-copy-offload-direct-copy-enabled' => $string >>


Enable/Disable the Direct-copy Copy Offload Mechanism possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'file-system-sector-size' => $string >>


Size of File System Sector Reported to SMB Clients (bytes) possible value(s) are, 512,4096 (in bytes)
Applicable for CMode CLI

=item C<< 'is-dac-enabled' => $string >>


Enable/Disable Dynamic Access Control (DAC) possible value(s) are, true,false
Applicable for CMode CLI

=item C<< 'is-advanced-sparse-file-support-enabled' => $string >>


Enable/Disable Advance Sparse File Support, possible value(s) are, true,false
Applicable for CMode CLI/ZAPI

=item C<< 'max-file-write-zero-length' => $integer >>


Maximum Length of Data in a File Zeroed by One Operation, possible value(s) are 4K to 2GB.
Applicable for CMode CLI/ZAPI

=item C<< 'guest-unix-user' => $string >>

Map the Guest User to Valid UNIX User
Applicable for CMode CLI/ZAPI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs options modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            "wins-servers" => {type => ARRAYREF, optional => 1},
            _optional_scalars(
                qw( smb2-enabled default-unix-user read-grants-exec

                    max-mpx

                    smb2-2-enabled shadowcopy-dir-depth copy-offload-enabled

                    default-unix-group

                    shadowcopy-enabled node-referrals-enabled

                    local-auth-enabled local-users-and-groups-enabled

                    is-referral-enabled

                    is-local-auth-enabled is-local-users-and-groups-enabled

                    smb3-enabled is-exportpolicy-enabled

                    is-use-junctions-as-reparse-points-enabled

                    is-unix-nt-acl-enabled

                    is-trusted-domain-enum-search-enabled

                    client-session-timeout

                    dac-enabled

                    restrict-anonymous

                    is-read-only-delete-enabled

                    is-copy-offload-direct-copy-enabled

                    file-system-sector-size

                    is-dac-enabled is-advanced-sparse-file-support-enabled

                    max-file-write-zero-length
                    is-fake-open-enabled guest-unix-user
		    smb1-enabled
                    )
            ),
        },
        cli_cmd => 'vserver cifs options modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
        ],
        frontend => 'NACL::C::VserverCifsOptions::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_options_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $map = {
        'copy-offload-enabled' => 'is-copy-offload-enabled',
        'smb2-enabled'         => 'is-smb2-enabled',
        'smb2-2-enabled'       => 'is-smb2-2-enabled',
        'read-grants-exec'     => 'read-grants-execute',
        'smb3-enabled'         => 'is-smb3-enabled',
        'shadowcopy-enabled'   => 'is-shadowcopy-enabled',
	'smb1-enabled'         => 'is-smb1-enabled',
    };
    my $copy = [
        qw(  default-unix-user max-mpx wins-servers shadowcopy-dir-depth is-use-junctions-as-reparse-points-enabled
            is-unix-nt-acl-enabled  is-trusted-domain-enum-search-enabled is-local-users-and-groups-enabled is-local-auth-enabled
            is-exportpolicy-enabled  default-unix-group is-referral-enabled is-advanced-sparse-file-support-enabled
            max-file-write-zero-length guest-unix-user)
    ];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'cifs-options-modify',
        map  => $map,
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my %hash = (
        'read-grants-exec' => "cifs.grant_implicit_exe_perms",
        'smb2-enabled'     => "cifs.smb2.enable"
    );

    foreach my $val (keys(%hash)) {
        if (defined $opts{$val}) {
            if ($opts{$val} =~ /on|1|ON|yes|YES|true|TRUE|enabled/) {
                $opts{'option-name'} = $hash{$val};
                $apiset->options($opts{'option-name'} => "on");
            } elsif ($opts{$val} =~ /(off|0|OFF|no|NO|false|FALSE|disabled)/)
            {
                $opts{'option-name'} = $hash{$val};
                $apiset->options($opts{'option-name'} => "off");
            } else {
                NATE::BaseException->throw(
                    "Cifs options modify command did not execute");
            }
        } ## end if ( defined $opts{$val...
    } ## end foreach my $val ( keys(%hash...

    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

=head2 prepare_to_downgrade

    NACL::C::VserverCifsOptions->prepare_to_downgrade(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to restore the CIFS restrict_anonymous settings for releases earlier than Data ONTAP 8.2.2.
Uses CMode CLI .

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< '*' >>

This method takes all other options supported by "vserver cifs options prepare to downgrade" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub prepare_to_downgrade {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        api                 => 'vserver_cifs_options_prepare_to_downgrade',
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_prepare_to_downgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsOptions::prepare_to_downgrade',
    );

    $Log->exit() if $may_exit;
}

sub _prepare_to_downgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_options_prepare_to_downgrade',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR | UNDEF, optional => 1},)
        ;    #vserver been commented because of the product burt 583929
}

1;
