# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsShareProperties Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsShareProperties

=head1 DESCRIPTION

C<NACL::C::VserverCifsShareProperties> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsShareProperties.  A related class is
L<NACL::CS::VserverCifsShareProperties|lib-NACL-CS-VserverCifsShareProperties-pm>, which represents
the past state of an ONTAP VserverCifsShareProperties.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 share_name

=head2 vserver

=cut

package NACL::C::VserverCifsShareProperties;

use strict;
use warnings;

use NACL::CS::VserverCifsShareProperties;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverCifsShareProperties = NACL::C::VserverCifsShareProperties->new(
                                   command_interface => $ci,
                                   'share-name' => $share_name,
                                   'vserver' => $vserver,
								  );

Return a new VserverCifsShareProperties component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'share_name',
    scalar => 'vserver',
];

=head2 find

    my $VserverCifsShareProperties_obj  = NACL::C::VserverCifsShareProperties->find(command_interface => $ci, ...);
	my @VserverCifsShareProperties_objs = NACL::C::VserverCifsShareProperties->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VserverCifsShareProperties_state = $VserverCifsShareProperties_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsShareProperties would return 
a NACL::CS::VserverCifsShareProperties object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 remove

    NACL::C::VserverCifsShareProperties->remove(
                                command_interface => $ci,
	                            'share-name' => $share_name,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifsShareProperties_obj->remove(...);

(Class or instance method)
This method is used to remove from the list of share properties.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'share-name' => $share_name >>

(Required for class method, Not Applicable for instance method)
Share
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI

=item C<< 'share-properties' => [ $share-propertie1, $share-propertie2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs share properties remove" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub remove {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'share-properties'   => {type => ARRAYREF,  optional => 1},
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        api => 'vserver_cifs_share_properties_remove',
    );

    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_remove_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsShareProperties::remove',
    );

    $Log->exit() if $may_exit;
} ## end sub remove

sub _remove_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_share_properties_remove',);
    $Log->exit() if $may_exit;
} ## end sub _remove_cmode_cli

=head2 add

    NACL::C::VserverCifsShareProperties->add(
                                command_interface => $ci,
	                            'share-name' => $share_name,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverCifsShareProperties_obj->add(...);

(Class or instance method)
This method is used to add to the list of share properties.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'share-name' => $share_name >>

(Required for class method, Not Applicable for instance method)
Share
Applicable for CMode CLI

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI

=item C<< 'share-properties' => [ $share-propertie1, $share-propertie2, ... ] >>

(Arrayref, even if only a single value is to be sent it should specified
as an arrayref containing that single value)
Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs share properties add" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub add {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'share-properties'   => {type => ARRAYREF,  optional => 1},
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
        },
        api => 'vserver_cifs_share_properties_add',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_add_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverCifsShareProperties::add',
    );

    $Log->exit() if $may_exit;
} ## end sub add

sub _add_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_share_properties_add',);
    $Log->exit() if $may_exit;
} ## end sub _add_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'share-name' => {type => SCALAR},
        'vserver'    => {type => SCALAR},
    );
}

1;
