#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsSuperuser Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCifsSuperuser

=head1 DESCRIPTION

C<NACL::C::VserverCifsSuperuser> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsSuperuser.  A related class is
L<NACL::CS::VserverCifsSuperuser|lib-NACL-CS-VserverCifsSuperuser-pm>, which represents
the past state of an ONTAP VserverCifsSuperuser.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 domain

=head2 vserver

=head2 accountname

=cut

package NACL::C::VserverCifsSuperuser;

use strict;
use warnings;

use NACL::CS::VserverCifsSuperuser;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverCifsSuperuser = NACL::C::VserverCifsSuperuser->new(command_interface=>$ci,
                                   domain =>$domain,
                                   vserver =>$vserver,
                                   accountname =>$accountname,
                                  );

Return a new VserverCifsSuperuser component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'domain',
    scalar => 'vserver',
    scalar => 'accountname',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsSuperuser  = NACL::C::VserverCifsSuperuser->find(command_interface => $ci, ...);
    my @VserverCifsSuperusers = NACL::C::VserverCifsSuperuser->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsSuperuser_state = $NACL::C::VserverCifsSuperuser->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsSuperuser would return
a NACL::CS::VserverCifsSuperuser object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::VserverCifsSuperuser->delete(
                                command_interface => $ci,
                                 domain => $domain,
                                 vserver => $vserver,
                                 accountname => $accountname,
                                  ...);

                                 or

    $VserverCifsSuperuser_object->delete(...);

(Class or instance method)
This method is used to deletes superuser permissions from a CIFS account.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "domain" => $domain >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "accountname" => $accountname >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs superuser delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd    => 'vserver cifs superuser delete',
    );
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_delete_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverCifsSuperuser::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg_or_obj->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_cifs_superuser_delete',
        vserver_operated_on => \$vserver,
    );

    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

=head2 create

    my $VserverCifsSuperuser_object    = NACL::C::VserverCifsSuperuser->create(
                                command_interface => $ci,
                                 domain => $domain,
                                 vserver => $vserver,
                                 accountname => $accountname,
                                  ...);

(Class method)
This method is used to adds superuser permissions to a CIFS account.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "domain" => $domain >>

(Required)

Applicable for CMode CLI.

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI.

=item C<< "accountname" => $accountname >>

(Required)

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs superuser create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params => \@args,
        cli_cmd    => 'vserver cifs superuser create',
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverCifsSuperuser::create',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my($pkg, %opts) = @_;
    my $vserver = $opts{vserver};

    $pkg->_base_backend_cmode_cli(
        %opts,
        api => 'vserver_cifs_superuser_create',
        vserver_opearetd_on => \$vserver,
    );

    $Log->exit() if $may_exit;
    return $pkg->new(
        command_interface => $opts{'command_interface'},
        domain            => $opts{'domain'},
        vserver           => $opts{'vserver'},
        accountname       => $opts{'accountname'},
    );
} ## end sub _create_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'domain'      => {type => SCALAR},
        'vserver'     => {type => SCALAR},
        'accountname' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
