# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsSymlink Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::VserverCifsSymlink

=head1 DESCRIPTION

C<NACL::C::VserverCifsSymlink> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCifsSymlink.  A related class is
L<NACL::CS::VserverCifsSymlink|lib-NACL-CS-VserverCifsSymlink-pm>, which represents
the past state of an ONTAP VserverCifsSymlink.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=head2 unix_path

=cut

package NACL::C::VserverCifsSymlink;

use strict;
use warnings;

use NACL::CS::VserverCifsSymlink;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::C::Exceptions::Component qw(:try);
use NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists;
use NACL::C::Exceptions::VserverCifsSymlink::DoesNotExist ();

=head1 METHODS

=head2 new

  my $VserverCifsSymlink = NACL::C::VserverCifsSymlink->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   unix_path =>$unix_path,
                                  );

Return a new VserverCifsSymlink component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'unix_path',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCifsSymlink  = NACL::C::VserverCifsSymlink->find(command_interface => $ci, ...);
    my @VserverCifsSymlinks = NACL::C::VserverCifsSymlink->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverCifsSymlink_state = $NACL::C::VserverCifsSymlink->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCifsSymlink would return
a NACL::CS::VserverCifsSymlink object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 modify

   NACL::C::VserverCifsSymlink->modify(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 unix_path => $unix_path,
                                  ...);

                                 or

    $VserverCifsSymlink_object->modify(...);

(Class or instance method)
This method is used to modify a symlink path mapping.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-symlink-modify" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "unix-path" => $unix_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: unix-path

=item C<< "home-directory" => $string >>

Applicable for CMode CLI.

=item C<< "share-name" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: share-name

=item C<< "cifs-server" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-server

=item C<< "locality" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: locality

=item C<< "cifs-path" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs symlink modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( home-directory share-name cifs-server locality cifs-path
                    )
            ),
        },
        cli_cmd => 'vserver cifs symlink modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },

        ],
        frontend => 'NACL::C::VserverCifsSymlink::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_symlink_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy
        = [qw( share-name cifs-server unix-path locality cifs-path )];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => $copy,
        api  => 'cifs-symlink-modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

   NACL::C::VserverCifsSymlink->delete(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 unix_path => $unix_path,
                                  ...);

                                 or

    $VserverCifsSymlink_object->delete(...);

(Class or instance method)
This method is used to delete a symlink path mapping.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-symlink-delete" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "unix-path" => $unix_path >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: unix-path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs symlink delete" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsSymlink::DoesNotExist>

This type of exception gets thrown when an operation is performed on symlink that doesn't exist.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params   => \@args,
        cli_cmd  => 'vserver cifs symlink delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::VserverCifsSymlink::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverCifsSymlink::DoesNotExist
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_symlink_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(unix-path)],
        api  => 'cifs_symlink_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 create

    my $VserverCifsSymlink_object    = NACL::C::VserverCifsSymlink->create(
                                command_interface => $ci,
                                 vserver => $vserver,
                                 unix_path => $unix_path,
                                  ...);

(Class method)
This method is used to create a symlink path mapping.

Uses a CMode CLI/ZAPI APISet.

Invokes "cifs-symlink-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "unix-path" => $unix_path >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: unix-path

=item C<< "home-directory" => $string >>

Applicable for CMode CLI.

=item C<< "share-name" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: share-name

=item C<< "cifs-server" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-server

=item C<< "locality" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: locality

=item C<< "cifs-path" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: cifs-path

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver cifs symlink create" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists>

This type of exception gets thrown if an attempt is amde to create a symlink which already exists.

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( home-directory share-name cifs-server locality cifs-path
                    )
            ),
        },
        cli_cmd => 'vserver cifs symlink create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_create_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_create_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverCifsSymlink::create',
    );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverCifsSymlink::AlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return  $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $vserver,
        unix_path         => $opts{'unix-path'},
    );

} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_cifs_symlink_create',);
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy
        = [qw( share-name cifs-server unix-path locality cifs-path )];
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => $copy,
        api  => 'cifs_symlink_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'   => {type => SCALAR | UNDEF, 'optional' => '1'},
        'unix-path' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
