# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCopyOffload Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverCopyOffload

=head1 DESCRIPTION

C<NACL::C::VserverCopyOffload> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverCopyOffload.  A related class is
L<NACL::CS::VserverCopyOffload|lib-NACL-CS-VserverCopyOffload-pm>, which represents
the past state of an ONTAP VserverCopyOffload.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverCopyOffload;

use strict;
use warnings;

use NACL::CS::VserverCopyOffload;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverCopyOffload = NACL::C::VserverCopyOffload->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
                                  );

Return a new VserverCopyOffload component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverCopyOffload_obj  = NACL::C::VserverCopyOffload->find(command_interface => $ci, ...);
    my @VserverCopyOffload_objs = NACL::C::VserverCopyOffload->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=head2 state

    my VserverCopyOffload_state = $VserverCopyOffload_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverCopyOffload would return
a NACL::CS::VserverCopyOffload object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI APISet.

=cut

=head2 delete_tokens

    NACL::C::VserverCopyOffload->delete_tokens(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverCopyOffload_obj->delete_tokens(...);

(Class or instance method)
This method is used to delete token scratch space used by copy-offload..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'volume' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete_tokens {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( volume
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_tokens_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _delete_tokens_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( volume vserver
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    $pkg_or_obj->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $command_interface,
        frontend_method   => 'delete_tokens',
    );

    $apiset->vserver_copy_offload_delete_tokens(%cmode_args);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VserverCopyOffload->modify(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverCopyOffload_obj->modify(...);

(Class or instance method)
This method is used to modify the copy-offload configuration of a Vserver..

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< 'nfs' => $string >>

Applicable for CMode CLI.

=item C<< 'throttle' => $string >>

Applicable for CMode CLI.

=item C<< 'sce' => $string >>

Applicable for CMode CLI.

=item C<< 'scsi' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< 'subfile-sisclone' => $string >>

*Copy-Offload Via Sub-File Cloning, possible value(s) are enabled,disabled
Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    $pkg_or_obj->_frontend_log_debug_opts(@_);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( nfs throttle sce scsi

                    subfile-sisclone
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( vserver nfs throttle sce scsi
            extended_query
            subfile-sisclone
            )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    $pkg_or_obj->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $command_interface,
        frontend_method   => 'modify',
    );

    $apiset->vserver_copy_offload_modify(%cmode_args);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
