# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverExportPolicyAccessCacheConfig Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverExportPolicyAccessCacheConfig

=head1 DESCRIPTION

C<NACL::C::VserverExportPolicyAccessCacheConfig> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverExportPolicyAccessCacheConfig.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverExportPolicyAccessCacheConfig;

use strict;
use warnings;

use NACL::CS::VserverExportPolicyAccessCacheConfig;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use NATE::Log qw(log_global);
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

=head1 METHODS

=head2 new

  my $VserverExportPolicyAccessCacheConfig = NACL::C::VserverExportPolicyAccessCacheConfig->new(
                                   command_interface => $ci,
                                   'vserver' => $vserver,
								  );

Return a new VserverExportPolicyAccessCacheConfig component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head2 find

    my $VserverExportPolicyAccessCacheConfig_obj  = NACL::C::VserverExportPolicyAccessCacheConfig->find(command_interface => $ci, ...);
	my @VserverExportPolicyAccessCacheConfig_objs = NACL::C::VserverExportPolicyAccessCacheConfig->find(command_interface => $ci, ...); 

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=head2 state

    my VserverExportPolicyAccessCacheConfig_state = $VserverExportPolicyAccessCacheConfig_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverExportPolicyAccessCacheConfig would return 
a NACL::CS::VserverExportPolicyAccessCacheConfig object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.

=cut

=head2 modify_all_vservers

    NACL::C::VserverExportPolicyAccessCacheConfig->modify_all_vservers(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to modify exports access cache configuration for all Vservers.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'ttl-negative' => $string >>

TTL For Negative Entries (Secs) ranges from 60 to 86400
Applicable for CMode CLI

=item C<< 'harvest-timeout' => $string >>

Harvest Timeout (Secs) ranges from 60 to 2592000
Applicable for CMode CLI

=item C<< 'ttl-positive' => $string >>

TTL For Positive Entries (Secs) ranges from 300 to 86400
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config modify all vservers" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify_all_vservers {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ttl-negative harvest-timeout ttl-positive
                    )
            ),
        },
        ignore_primary_keys => 1,
        cli_cmd =>
            'vserver export-policy access-cache config modify-all-vservers',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_all_vservers_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::VserverExportPolicyAccessCacheConfig::modify_all_vservers',
    );

    $Log->exit() if $may_exit;
}

sub _modify_all_vservers_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api =>
            'vserver_export_policy_access_cache_config_modify_all_vservers',);
    $Log->exit() if $may_exit;
}

=head2 modify

    NACL::C::VserverExportPolicyAccessCacheConfig->modify(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverExportPolicyAccessCacheConfig_obj->modify(...);

(Class or instance method)
This method is used to modify exports access cache configuration.
Uses CMode CLI/ZAPI .
Invokes  exports-access-cache-config-modify  for CMode Zapi non-iter

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'ttl-negative' => $string >>

TTL For Negative Entries (Secs) ranges from 60 to 86400
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'harvest-timeout' => $string >>

Harvest Timeout (Secs) ranges from 60 to 2592000
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'ttl-failure' => $string >>

TTL For Entries with Failure (Secs) ranges from 1 to 60
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< 'ttl-positive' => $string >>

TTL For Positive Entries (Secs) ranges from 300 to 86400
Applicable for CMode CLI/ZAPI, non-iter.

=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config modify" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( ttl-negative harvest-timeout ttl-failure ttl-positive
                    )
            ),
        },
        cli_cmd => 'vserver export-policy access-cache config modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_modify_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        frontend => 'NACL::C::VserverExportPolicyAccessCacheConfig::modify',
    );

    $Log->exit() if $may_exit;
}

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_export_policy_access_cache_config_modify',);
    $Log->exit() if $may_exit;
}

sub _modify_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my $copy = [qw(harvest-timeout ttl-negative ttl-failure ttl-positive)];
    $pkg_or_obj->_base_backend_cmode_zapi(
        @opts,
        api  => 'exports-access-cache-config-modify',
        copy => $copy,
    );
    $Log->exit() if $may_exit;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head2 reset_to_defaults

    NACL::C::VserverExportPolicyAccessCacheConfig->reset_to_defaults(
                                command_interface => $ci,
	                            'vserver' => $vserver,
                                ...);
	                             
								 or

    $VserverExportPolicyAccessCacheConfig_obj->reset_to_defaults(...);

(Class or instance method)
This method is used to reset to default values.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver
Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config reset to defaults" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub reset_to_defaults {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params => \@args,
        cli_cmd =>
            'vserver export-policy access-cache config reset-to-defaults',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_reset_to_defaults_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::VserverExportPolicyAccessCacheConfig::reset_to_defaults',
    );

    $Log->exit() if $may_exit;
}

sub _reset_to_defaults_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_export_policy_access_cache_config_reset_to_defaults',
    );
    $Log->exit() if $may_exit;
}

=head2 show_all_vservers

    my $show_all_vservers_obj = NACL::C::VserverExportPolicyAccessCacheConfig->show_all_vservers(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_all_vservers_objs = NACL::C::VserverExportPolicyAccessCacheConfig->show_all_vservers(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_all_vservers_obj = $VserverExportPolicyAccessCacheConfig_obj->show_all_vservers(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverExportPolicyAccessCacheConfigShowAllVservers-pm/ATTRIBUTES>
of NACL::CS::VserverExportPolicyAccessCacheConfigShowAllVservers as input. 
This method returns objects of the type L<NACL::CS::VserverExportPolicyAccessCacheConfigShowAllVservers|lib-NACL-CS-VserverExportPolicyAccessCacheConfigShowAllVservers-pm>.

=cut

sub show_all_vservers {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverExportPolicyAccessCacheConfigShowAllVservers;
    my @state_objs
        = NACL::CS::VserverExportPolicyAccessCacheConfigShowAllVservers
        ->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 show_nblade

    my $show_nblade_obj = NACL::C::VserverExportPolicyAccessCacheConfig->show_nblade(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @show_nblade_objs = NACL::C::VserverExportPolicyAccessCacheConfig->show_nblade(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $show_nblade_obj = $VserverExportPolicyAccessCacheConfig_obj->show_nblade(); 
						  
(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of 
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty) 
and has all of the other properties of fetch methods (callable in scalar or list 
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverExportPolicyAccessCacheConfigShowNblade-pm/ATTRIBUTES>
of NACL::CS::VserverExportPolicyAccessCacheConfigShowNblade as input. 
This method returns objects of the type L<NACL::CS::VserverExportPolicyAccessCacheConfigShowNblade|lib-NACL-CS-VserverExportPolicyAccessCacheConfigShowNblade-pm>.

=cut

sub show_nblade {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverExportPolicyAccessCacheConfigShowNblade;
    my @state_objs
        = NACL::CS::VserverExportPolicyAccessCacheConfigShowNblade->fetch(
        %opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 test_online_callback

    NACL::C::VserverExportPolicyAccessCacheConfig->test_online_callback(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to test online callback.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config test online callback" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test_online_callback {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd =>
            'vserver export-policy access-cache config test-online-callback',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_test_online_callback_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::VserverExportPolicyAccessCacheConfig::test_online_callback',
    );

    $Log->exit() if $may_exit;
}

sub _test_online_callback_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api =>
            'vserver_export_policy_access_cache_config_test_online_callback',
    );
    $Log->exit() if $may_exit;
}

=head2 test_revert

    NACL::C::VserverExportPolicyAccessCacheConfig->test_revert(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to test revert to 8.3.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config test revert" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test_revert {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd => 'vserver export-policy access-cache config test-revert',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_test_revert_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::VserverExportPolicyAccessCacheConfig::test_revert',
    );

    $Log->exit() if $may_exit;
}

sub _test_revert_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_export_policy_access_cache_config_test_revert',);
    $Log->exit() if $may_exit;
}

=head2 test_upgrade

    NACL::C::VserverExportPolicyAccessCacheConfig->test_upgrade(
                                command_interface => $ci,
                                ...);
   
(Class method)
This method is used to test upgrade to 8.4.
Uses CMode CLI.


=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>


=item C<< '*' >>

This method takes all other options supported by "vserver export policy access cache config test upgrade" command in ONTAP

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub test_upgrade {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params              => \@args,
        ignore_primary_keys => 1,
        cli_cmd => 'vserver export-policy access-cache config test-upgrade',
    );
    $pkg->call_on_apiset_with_exceptions(
        %opts,
        choices => [
            {   method    => '_test_upgrade_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        frontend =>
            'NACL::C::VserverExportPolicyAccessCacheConfig::test_upgrade',
    );

    $Log->exit() if $may_exit;
}

sub _test_upgrade_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_export_policy_access_cache_config_test_upgrade',);
    $Log->exit() if $may_exit;
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR},);
}

1;
