# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFcp Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status public
## @pod here
##

=head1 NAME

NACL::C::VserverFcp

=head1 DESCRIPTION

C<NACL::C::VserverFcp> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverFcp.  A related class is
L<NACL::CS::VserverFcp|lib-NACL-CS-VserverFcp-pm>, which represents
the past state of an ONTAP VserverFcp.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverFcp;

use strict;
use warnings;
use NACL::CS::VserverFcp;
use NACL::ComponentUtils qw (Dumper _optional_scalars);
use base 'NACL::C::Component::ONTAP';
use Params::Validate
    qw(validate validate_with SCALAR SCALARREF ARRAYREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::BaseException qw(:try);
use NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted           ();
use NACL::C::Exceptions::VserverFcp::ServiceAlreadyStopped           ();
use NACL::C::Exceptions::VserverFcp::ServiceAlreadyExists            ();
use NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled ();
use NACL::C::Exceptions::VserverFcp::CannotFindService               ();

=head1 METHODS

=head2 new

  my $VserverFcp = NACL::C::VserverFcp->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                   );

Return a new VserverFcp component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'nodename',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverFcp  = NACL::C::VserverFcp->find(command_interface => $ci, ...);
    my @VserverFcps = NACL::C::VserverFcp->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverFcp_state = $NACL::C::VserverFcp->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverFcp would return
a NACL::CS::VserverFcp object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 start

   NACL::C::VserverFcp->start(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                or

    $VserverFcp_object->start(...);

(Class or instance method)
This method is used to starts the FCP service.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "fcp-service-start" API for CMode ZAPI.

Invokes "fcp start" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp start" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item

C<NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted>

This type of exception gets thrown if an attempt is made to start when the service is already started

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver fcp start',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_start_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_start_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_start_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverFcp::start',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = $_[0];
            NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api => 'vserver_fcp_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_zapi(@args,
        api => 'fcp_service_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_zapi

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};

    $apiset->fcp_start();
    $Log->exit() if $may_exit;
} ## end sub _start_7mode_cli

=head2 stop

   NACL::C::VserverFcp->stop(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                or

    $VserverFcp_object->stop(...);

(Class or instance method)
This method is used to stops the FCP service.

Uses a CMode CLI/ZAPI,7Mode CLI APISet.

Invokes "fcp-service-stop" API for CMode ZAPI.

Invokes "fcp stop" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item

C<NACL::C::Exceptions::VserverFcp::ServiceAlreadyStopped>

This type of exception gets thrown if an attempt is made to stop when the service is already stopped

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver fcp stop',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_stop_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_stop_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_stop_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverFcp::stop',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = $_[0];
            NACL::C::Exceptions::VserverFcp::ServiceAlreadyStopped
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_fcp_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'fcp_service_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_zapi

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};

    $apiset->fcp_stop();
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_cli

=head2 create

    my $VserverFcp_object   = NACL::C::VserverFcp->create(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

(Class method)
This method is used to create FCP Service.

Uses a CMode CLI/ZAPI APISet.

Invokes "fcp-service-create" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "nodename" => $string >>

Applicable for CMode CLI.

=item C<< "status-admin" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: start

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "force" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: force-node-name

=item C<< "start" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: start

=item C<< 'target-name' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: node-name

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverFcp::ServiceAlreadyExists>

This type of exception gets thrown if an attempt is made to create a
fcp service for a vserver that is already configured.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( nodename status-admin status force start

                    target-name
                    )
            ),
        },
        cli_cmd => 'vserver fcp create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_create_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_create_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::VserverFcp::create',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverFcp::ServiceAlreadyExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{vserver},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api => 'vserver_fcp_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    if (exists $opts{'status-admin'}) {
        my %map = ('up', 'true', 'down', 'false');
        $opts{'status-admin'} = $map{$opts{'status-admin'}};
    }

    $pkg->_base_backend_cmode_zapi(%opts,
        api => 'fcp_service_create',
        copy   => [qw( start )],
        map    => {
            'force'        => 'force-node-name',
            'target-name'  => 'node-name',
            'status-admin' => 'start'
        },
    );

    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

=head2 modify

   NACL::C::VserverFcp->modify(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                or

    $VserverFcp_object->modify(...);

(Class or instance method)
This method is used to modify FCP Service Configuration.

Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI.

=item C<< "nodename" => $string >>

Applicable for CMode CLI.

=item C<< "status-admin" => $string >>

Applicable for CMode CLI.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "force" => $string >>

Applicable for CMode CLI.

=item C<< 'target-name' => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver fcp modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( nodename status-admin status force

                    target-name
                    )
            ),
        },
        cli_cmd => 'vserver fcp modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
        frontend => 'NACL::C::VserverFcp::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_fcp_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    my $command_interface   = $opts{command_interface};
    my $vserver             = $opts{vserver};
    my $apiset              = delete $opts{apiset};
    $opts{'target-name'} = delete $opts{nodename} if(exists $opts{nodename});
    my $status_admin_diff = 1;
    my $target_name_diff  = 1;
    if(exists $opts{'status-admin'} and exists $opts{'target-name'}) {
        # Fetch the two fields to see if both are being updated at the same time to mimic cli
        my $fcp_service = NACL::CS::VserverFcp->fetch(
                                    command_interface => $command_interface,
                                    filter            =>
                                        {vserver           => $vserver,},
                                    apiset_must       => {interface => 'ZAPI'},
                                    );

        # Check to see if the current status-admin and the passed in status-admin match.
        if ($opts{'status-admin'} eq $fcp_service->status_admin()) {
            $status_admin_diff = 0;
        }

        # Check to see if the current target-name and the passed in target-name/nodename match.
        if ($opts{'target-name'} eq $fcp_service->target_name()) {
            $target_name_diff = 0;
        }

        # tb3170_05::*> fcp modify -status-admin up -target-name 20:00:00:a0:98:26:3d:86
        # Error: command failed: Cannot modify FCP target-name and status-admin together.
        # Throw a response exception.
        if ($status_admin_diff and $target_name_diff) {
            # Create a dummy_response object so that we can throw a
            # CommandFailedException so users have a single exception
            # type to catch for zapi and cli.
            my $dummy_response = NACL::APISet::Response::ZAPI::CMode->new(
                        output              => 'Error: command failed: Cannot modify FCP target-name and status-admin together.',
                        command             => '<vserver_fcp_modify>',
                        execution_time      => '0',
                        execution_status    => 'failed',
                        error_number        => '1342',
                        error_reason        => 'Cannot modify FCP target-name and status-admin together',
                        complete_raw_output => 'Error: command failed: Cannot modify FCP target-name and status-admin together.',
                    );
            $Log->exit() if $may_exit;
            NACL::APISet::Exceptions::CommandFailedException->throw(
                text            =>
                    "Error: command failed: Cannot modify FCP target-name and status-admin together.",
                response_object => $dummy_response);
        }
    }

    if ($status_admin_diff and exists ($opts{'status-admin'})) {
        my $status_admin = delete $opts{'status-admin'};
        if ($status_admin eq 'down') {
            try {
                NACL::C::VserverFcp->stop(
                                        command_interface => $command_interface,
                                        vserver           => $vserver,
                                        apiset_must       => {interface => 'ZAPI'});
            } catch NACL::C::Exceptions::VserverFcp::ServiceAlreadyStopped with {
                # Ignoring this because it is ignored in modify cli
            };
        }
        if ($status_admin eq 'up') {
            try {
                NACL::C::VserverFcp->start(
                                        command_interface => $command_interface,
                                        vserver           => $vserver,
                                        apiset_must       => {interface => 'ZAPI'});
            } catch NACL::C::Exceptions::VserverFcp::ServiceAlreadyStarted with {
                # Ignoring this because it is ignored in modify cli
            };
        }
    }

    if ($target_name_diff and (exists ($opts{'target-name'}) or exists($opts{'nodename'}))) {
        my %set_name_opts;
        $set_name_opts{'node-name'} = $opts{'target-name'};
        $set_name_opts{force} = $opts{force} if (defined $opts{force});
        $pkg_or_obj->_handle_zapi_vserver_context(
            api_opts          => \%set_name_opts,
            vserver           => $vserver,
            command_interface => $command_interface
        );

        $apiset->fcp_node_set_name(%set_name_opts);
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

=head2 delete

   NACL::C::VserverFcp->delete(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                or

    $VserverFcp_object->delete(...);

(Class or instance method)
This method is used to delete FCP Service Configuration.

Uses a CMode CLI/ZAPI APISet.

Invokes "fcp-service-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.


=item C<< '*' >>

This method takes all other options supported by
"vserver fcp delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled>

This type of exception gets thrown if an attempt is made to delete a
fcp service for a vserver that is already in running state.

=item C<NACL::C::Exceptions::VserverFcp::CannotFindService>

This type of exception gets thrown if an attempt is made to delete a
fcp service for a vserver that is already deleted.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver fcp delete',
    );
    my $ret;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::VserverFcp::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverFcp::CannotRemoveServiceWhileEnabled
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VserverFcp::CannotFindService
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_fcp_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'fcp_service_destroy',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 status

    my $status_obj = NACL::C::VserverFcp->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @status_objs = NACL::C::VserverFcp->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $status_obj = $VserverFcp_obj->status();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverFcpStatus-pm/ATTRIBUTES>
of NACL::CS::VserverFcpStatus as input.
This method returns objects of the type L<NACL::CS::VserverFcpStatus|lib-NACL-CS-VserverFcpStatus-pm>.

=cut

sub status {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverFcpStatus;
    my @state_objs = NACL::CS::VserverFcpStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

=head2 vserver_fcp_nodename

    NACL::C::VserverFcp->vserver_fcp_nodename(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverFcp_obj->vserver_fcp_nodename(...);

(Class or instance method)
This method is used to display or modify a Vserver's FCP target name.
Uses a CMode CLI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver Name
Applicable for CMode CLI

=item C<< 'target-name' => $string >>

Target Name
Applicable for CMode CLI

=item C<< 'force' => $string >>

Force
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by "vserver fcp nodename" command in ONTAP

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub vserver_fcp_nodename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( target-name force
                    )
            ),
        },
        cli_cmd => 'vserver fcp nodename',
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_vserver_fcp_nodename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverFcp::vserver_fcp_nodename',
    );

    $Log->exit() if $may_exit;
}

sub _vserver_fcp_nodename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;

    $pkg_or_obj->_base_backend_cmode_cli(
        @opts,
        api => 'vserver_fcp_nodename',
    );

    $Log->exit() if $may_exit;
} ## end sub _vserver_fcp_nodename_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'vserver'  => {type => SCALAR | UNDEF, optional => 1},
        'nodename' => {type => SCALAR | UNDEF, optional => 1},
    );
} ## end sub _primary_keys_validate_spec

1;
