#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsi Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status public
## @pod here

=head1 NAME

NACL::C::VserverIscsi

=head1 DESCRIPTION

C<NACL::C::VserverIscsi> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverIscsi.  A related class is
L<NACL::CS::VserverIscsi|lib-NACL-CS-VserverIscsi-pm>, which represents
the past state of an ONTAP VserverIscsi.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 vserver

=cut

package NACL::C::VserverIscsi;

use strict;
use warnings;

use NACL::CS::VserverIscsi;
use NACL::ComponentUtils qw (Dumper _optional_scalars);
use NACL::C::Exceptions::VserverIscsi::IscsiServiceExists qw(:try);
use NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStarted           ();
use NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStopped           ();
use NACL::C::Exceptions::VserverIscsi::CannotRemoveServiceWhileEnabled ();
use NACL::C::Exceptions::VserverIscsi::CannotFindService               ();

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR SCALARREF UNDEF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $VserverIscsi = NACL::C::VserverIscsi->new(command_interface=>$ci,
                                   vserver =>$vserver,
                                );

Return a new VserverIscsi component object with the given attributes.

=cut

use Class::MethodMaker [scalar => 'vserver',];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverIscsi  = NACL::C::VserverIscsi->find(command_interface => $ci, ...);
    my @VserverIscsis = NACL::C::VserverIscsi->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverIscsi_state = $NACL::C::VserverIscsi->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverIscsi would return
a NACL::CS::VserverIscsi object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 stop

   NACL::C::VserverIscsi->stop(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsi_object->stop(...);

(Class or instance method)
This method is used to stops the iSCSI service.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "iscsi-service-stop" API for CMode ZAPI.

Invokes "iscsi stop" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi stop" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStopped>

This type of exception gets thrown if an attempt is made to stop a
iscsi service which is already stopped.

=back

=back

=cut

sub stop {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver iscsi stop',
    );
    my $ret;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_stop_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_stop_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_stop_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverIscsi::stop',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = $_[0];
            NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStopped
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub stop

sub _stop_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_iscsi_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_cli

sub _stop_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'iscsi_service_stop',
    );
    $Log->exit() if $may_exit;
} ## end sub _stop_cmode_zapi

sub _stop_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};

    $apiset->iscsi_stop();
    $Log->exit() if $may_exit;
} ## end sub _stop_7mode_cli

=head2 start

   NACL::C::VserverIscsi->start(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsi_object->start(...);

(Class or instance method)
This method is used to starts the iSCSI service.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "iscsi-service-start' API for CMode ZAPI.

Invokes "iscsi start" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi start" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStarted>

This type of exception gets thrown if an attempt is made to start a
iscsi service which is already started.

=back

=back

=cut

sub start {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver iscsi start',
    );
    my $ret;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_start_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_start_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => "_start_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverIscsi::start',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = $_[0];
            NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStarted
                ->detect_convert_and_throw(exception => $exception);
            $Log->exit() if $may_exit;
            $exception->throw();
    };
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub start

sub _start_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_iscsi_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_cli

sub _start_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'iscsi_service_start',
    );
    $Log->exit() if $may_exit;
} ## end sub _start_cmode_zapi

sub _start_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $apiset = delete $opts{apiset};

    $apiset->iscsi_start();
    $Log->exit() if $may_exit;
} ## end sub _start_7mode_cli

=head2 modify

   NACL::C::VserverIscsi->modify(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsi_object->modify(...);

(Class or instance method)
This method is used to modify a Vserver's iSCSI service.

Uses a CMode CLI/ZAPI, 7Mode CLI APISet.

Invokes "iscsi-target-alias-clear-alias, iscsi-service-start,
iscsi-service-stop, iscsi-service-modify" API's for CMode ZAPI.

Invokes "iscsi nodename, iscsi alias" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "target-alias" => $target_alias >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

alias-name of iscsi-target-alias-set-alias CM ZAPI


=item C<< "max-conn-per-session" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "nodename" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : new node name (Programmatic name: "new-node-name")

=item C<< "status-admin" => $string >>

Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI:

If the status-admin option passed in is 'up'
then it executes the iscsi-service-start API.

If the status-admin option passed in is 'down'
then it executes the iscsi-service-stop API.

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "clear" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: Invokes iscsi-target-alias-clear-alias API for CMode ZAPI

=item C<< "login-timeout" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "max-ios-per-session" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-cmds-per-session

=item C<< "max-error-recovery-level" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "tcp-window-size" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< "alias" => $string >>

Applicable for CMode CLI, 7Mode CLI.

Maps to:

7M CLI : new alias name (Programmatic name: "new-alias")

=item C<< "retain-timeout" => $string >>

Applicable for CMode CLI/ZAPI.

=item C<< 'force' => $string >>

*Force overwrite for nodename

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi modify" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( nodename alias clear c status-admin max-error-recovery-level
                    retain-timeout login-timeout max-conn-per-session
                    max-ios-per-session tcp-window-size status target-name target-alias

                    force
                    )
            ),
        },
        cli_cmd => 'vserver iscsi modify',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_modify_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_modify_7mode_cli",
                interface => "CLI",
                set       => "7Mode",
            },
            {   method    => "_modify_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
                check     => '_check_cmode_zapi_modify',
            },

        ],
        frontend => 'NACL::C::VserverIscsi::modify',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_iscsi_modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = validate_with(
        params      => \@args,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};

    my %cmode_args;
    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $opts{vserver},
        command_interface => $command_interface
    );

    if (defined $opts{'target-name'}) {
        $apiset->iscsi_node_set_name(%cmode_args,
            'node-name' => $opts{'target-name'});
    }

    if (defined $opts{'target-alias'}) {
        $apiset->iscsi_target_alias_set_alias(%cmode_args,
            'alias-name' => $opts{'target-alias'});
    }

    if (defined $opts{'clear'}) {
        $apiset->iscsi_target_alias_clear_alias(%cmode_args);
    }
    if (defined $opts{'status-admin'}) {
        if ($opts{'status-admin'} eq "up") {
            try {
                $apiset->iscsi_service_start(%cmode_args);
            } otherwise {
                my $exception = $_[0];
                # Do nothing if Service Already Started as the CLI modify
                # does not do anything in this case.
                unless (NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStarted
                        ->detect(exception => $exception)) {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            };
        } elsif ($opts{'status-admin'} eq "down") {
            try {
                $apiset->iscsi_service_stop(%cmode_args);
            } otherwise {
                my $exception = $_[0];
                # Do nothing if Service Already Stopped as the CLI modify
                # does not do anything in this case.
                unless (NACL::C::Exceptions::VserverIscsi::ServiceAlreadyStopped
                        ->detect(exception => $exception)) {
                    $Log->exit() if $may_exit;
                    $exception->throw();
                }
            };
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw('Invalid value '
                    . $opts{'status-admin'}
                    . ' passed for the status-admin attribute of modify method'
            );
        }
    }

    my $copy = [
        qw( login-timeout max-conn-per-session
            max-error-recovery-level retain-timeout
            tcp-window-size)
    ];
    my $map = {'max-ios-per-session' => 'max-cmds-per-session'};

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $copy,
        map    => $map,
        target => \%cmode_args,
    );
    my @options = (@$copy, keys($map));
    foreach my $option (@options) {
        if (defined $opts{$option}) {
            $apiset->iscsi_service_modify(%cmode_args);
            last;
        }
    }

    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _check_cmode_zapi_modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (defined $opts{force} && $opts{force} =~ /true/i) {
         my $msg = "force was provided in the call to NACL::C::VserverIscsi->modify"
             . ' so the ZAPI implementation cannot be used';
         $Log->comment($msg);
         $Log->exit() if $may_exit;
         NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

sub _modify_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params      => \@_,
        spec        => $pkg_or_obj->_backend_validate_spec(),
        allow_extra => 1
    );
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %quota_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {
            "nodename" => "new-node-name",
            "alias"    => "new-alias"
        },
        target => \%quota_args
    );
    my $target_name  = delete $opts{'nodename'};
    my $target_alias = delete $opts{'alias'};
    %opts = %quota_args;

    if (($target_name ne "") && ($target_alias eq "")) {
        delete $quota_args{'new-alias'};
        $apiset->iscsi_nodename(%quota_args);

    } elsif (($target_name eq "") && ($target_alias ne "")) {
        delete $opts{'new-node-name'};
        $apiset->iscsi_alias(%opts);
    } else {
        delete $quota_args{'new-alias'};
        $apiset->iscsi_nodename(%quota_args);
        delete $opts{'new-node-name'};
        $apiset->iscsi_alias(%opts);
    }
    $Log->exit() if $may_exit;
} ## end sub _modify_7mode_cli

=head2 state

    my Iscsi_state = $NACL::C::VserverIscsi->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverIscsi would return
a NACL::CS::VserverIscsiStats object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 stats

    my $stats_obj = NACL::C::VserverIscsi->stats(command_interface => $ci,
                                                  [ filter => { %filter } ,]
                                                  [ requested_fields => [ @req_fields ]);

                    or

    my @stats_objs = NACL::C::VserverIscsi->stats(command_interface => $ci,
                                                  [ filter => { %filter } ,]
                                                  [ requested_fields => [ @req_fields ]);

                    or

   my $stats_obj = $VserverIscsi_obj->stats();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverIscsiStats-pm/ATTRIBUTES>
of NACL::CS::VserverIscsiStats as input.
This method returns objects of the type L<NACL::CS::VserverIscsiStats|lib-NACL-CS-VserverIscsiStats-pm>.

=cut

sub stats {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::VserverIscsiStats;
    my @state_objs = NACL::CS::VserverIscsiStats->fetch(@_);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];

} ## end sub stats

=head2 create

    my $VserverIscsi_object    = NACL::C::VserverIscsi->create(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

(Class method)
This method is used to create a Vserver's iSCSI service.

Uses a CMode CLI/ZAPI, 7Mode CLI  APISet.

Invokes "iscsi-service-create" API for CMode ZAPI.

Invokes "options" command for 7Mode CLI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "max-conn-per-session" => $string >>

Applicable for CMode CLI.

=item C<< "nodename" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node-name

=item C<< "status-admin" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: start

=item C<< "status" => $string >>

Applicable for CMode CLI.

=item C<< "login-timeout" => $string >>

Applicable for CMode CLI.

=item C<< "max-ios-per-session" => $string >>

Applicable for CMode CLI.

=item C<< "max-error-recovery-level" => $string >>

Applicable for CMode CLI.

=item C<< "tcp-window-size" => $string >>

Applicable for CMode CLI.

=item C<< "alias" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: alias-name

=item C<< "retain-timeout" => $string >>

Applicable for CMode CLI.

=item C<< "start" => $string >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: start

=item C<< 'target-name' => $string >>

Applicable for CMode CLI.

=item C<< 'target-alias' => $string >>

Applicable for CMode CLI.

=item C<< 'force' => $string >>

*Force overwrite for nodename

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi create" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverIscsi::IscsiServiceExists>

This type of exception gets thrown if an attempt is made to create a
iscsi service which already exists.

=back

=back

=cut

sub create {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( max-conn-per-session nodename status-admin status login-timeout
                    max-ios-per-session max-error-recovery-level tcp-window-size alias
                    retain-timeout start target-name target-alias
                    force
                    )
            ),
        },
        cli_cmd => 'vserver iscsi create',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $pkg->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_create_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_create_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                    check     => '_check_cmode_zapi_create',
                },
                {   method    => "_create_7mode_cli",
                    interface => "CLI",
                    set       => "7Mode",
                },
            ],
            frontend => 'NACL::C::VserverIscsi::create',
        );
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverIscsi::IscsiServiceExists
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        vserver           => $opts{vserver},
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub create

sub _create_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_base_backend_cmode_cli(@args,
        api => 'vserver_iscsi_create',
    );
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_cli

sub _create_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my %mod_opts;
    if (exists $opts{'status-admin'}) {
        my %map = ('up', 'true', 'down', 'false');
        $opts{'status-admin'} = $map{$opts{'status-admin'}};
    }
    foreach my $parameter (
        qw(max-error-recovery-level retain-timeout login-timeout max-conn-per-session max-ios-per-session tcp-window-size)
        )
    {
        if (defined $opts{$parameter}) {
            $mod_opts{$parameter} = $opts{$parameter};
            delete $opts{$parameter};
        }
    }
    my $exception;
    try{
        $pkg->_base_backend_cmode_zapi(%opts,
            api    => 'iscsi_service_create',
            copy   => [qw( start )],
            map    => {
                'nodename'     => 'node-name',
                'alias'        => 'alias-name',
                'status-admin' => 'start'
            },
        );
    } otherwise {
        #Burt952313 Don't fail immediately, but execute all ZAPIs first
        $exception = $_[0];
    };
    if ( %mod_opts ) {
        $pkg->modify(
            command_interface => $opts{command_interface},
            vserver           => $opts{vserver},
            %mod_opts,
        );
    }
    if ($exception){
        $Log->exit() if $may_exit;
        $exception->throw();
    }
    $Log->exit() if $may_exit;
} ## end sub _create_cmode_zapi

sub _check_cmode_zapi_create {
    $Log->enter() if $may_enter;
    my @array;
    my ($pkg_or_obj, %opts) = @_;
    if (defined $opts{force} && $opts{force} =~ /true/i) {
          my $msg = "force was provided in the call to NACL::C::VserverIscsi->create"
             . ' so the ZAPI implementation cannot be used';
         $Log->comment($msg);
         $Log->exit() if $may_exit;
         NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

sub _create_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    $opts{'option-name'}  = "iscsi.enable";
    $opts{'option-value'} = "on";
    $apiset->options($opts{'option-name'} => $opts{'option-value'});

    $opts{'option-name'}  = "licensed_feature.iscsi.enable";
    $opts{'option-value'} = "on";
    $apiset->options($opts{'option-name'} => $opts{'option-value'});
    $Log->exit() if $may_exit;
    return $pkg->new(command_interface => $command_interface,);
} ## end sub _create_7mode_cli



=head2 delete

   NACL::C::VserverIscsi->delete(
                                command_interface => $ci,
                                vserver => $vserver,
                                  ...);

                                 or

    $VserverIscsi_object->delete(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-service-destroy" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< extended_query => $string >>

(Optional)

Applicable for CMode CLI

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=item C<NACL::C::Exceptions::VserverIscsi::CannotRemoveServiceWhileEnabled>

This type of exception gets thrown if an attempt is made to delete a
iscsi service which is enabled.

=item C<NACL::C::Exceptions::VserverIscsi::CannotFindService>

This type of exception gets thrown if an attempt is made to delete a
iscsi service which does not exists.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver iscsi delete',
    );
    my $ret;
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    try {
        $ret = $pkg_or_obj->call_on_apiset(
            %opts,
            choices => [
                {   method    => "_delete_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_delete_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
            ],
            frontend => 'NACL::C::VserverIscsi::delete',
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        NACL::C::Exceptions::VserverIscsi::CannotRemoveServiceWhileEnabled
            ->detect_convert_and_throw(exception => $exception);
        NACL::C::Exceptions::VserverIscsi::CannotFindService
            ->detect_convert_and_throw(exception => $exception);
        $Log->exit() if $may_exit;
        $exception->throw();
    };

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@args,
        api => 'vserver_iscsi_delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@args,
        api => 'iscsi_service_destroy',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 alias

    NACL::C::VserverIscsi->alias(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverIscsi_obj->alias(...);

(Class or instance method)
This method is used to manage a Vserver's iSCSI target alias.
Uses a CMode CLI/ZAPI APISet.

Invokes 'iscsi-target-alias-set-alias' ZAPI for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver possible value(s) are, vserver name
Applicable for CMode CLI

=item C<< 'clear' => $string >>

Clear the Target Alias possible value(s) are, true
Applicable for CMode CLI

=item C<< 'target-alias' => $string >>

Target Alias possible value(s) are, text
Applicable for CMode CLI/ZAPI

Maps to:

CM ZAPI: alias-name

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi alias" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub alias {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;

    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( clear target-alias
                    )
            ),
        },
        cli_cmd => 'vserver iscsi alias',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_alias_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_alias_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverIscsi::alias',
    );

    $Log->exit() if $may_exit;
} ## end sub alias

sub _alias_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(%opts, api => 'vserver_iscsi_alias');
    $Log->exit() if $may_exit;
} ## end sub _alias_cmode_cli

sub _alias_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;

    my ($map, $api);
    if (defined $opts{'clear'} && ($opts{'clear'} eq "true")) {
      $api = 'iscsi-target-alias-clear-alias' ;
      ## the api iscsi_target_alias_clear_alias don't take any inputs
      delete $opts{clear};
      delete $opts{'target-alias'} if (defined $opts{'target-alias'}) ;
    }else{
      $api = 'iscsi-target-alias-set-alias';
      $map = {map => {'target-alias' => 'alias-name'}};
    }

    $pkg->_base_backend_cmode_zapi(
	 %opts,
	 %{$map},
	 api => $api,
         inapplicable_for_help_xml => [qw(clear)]
    );

    $Log->exit() if $may_exit;
} ## end sub _alias_cmode_zapi

=head2 nodename

    NACL::C::VserverIscsi->nodename(
                                command_interface => $ci,
                                'vserver' => $vserver,
                                ...);

                                 or

    $VserverIscsi_obj->nodename(...);

(Class or instance method)
This method is used to manage a Vserver's iSCSI target name.

Uses a CMode CLI/ZAPI APISet.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'vserver' => $vserver >>

(Required for class method, Not Applicable for instance method)
Vserver possible value(s) are, vserver name

Applicable for CMode CLI/ZAPI.

=item C<< 'node' => $node >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: node-name

=item C<< 'target-name' => $string >>

Target Name possible value(s) are, text
Applicable for CMode CLI

=item C<< 'force' => $string >>

Allow Non-Vendor Target Name possible value(s) are, true
Applicable for CMode CLI

=item C<< 'cmd_execution_time' => \$scalar >>

(Optional) Scalar reference , it will be updated with the command execution
time returned by the apiset response object

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< '*' >>

This method takes all other options supported by
"vserver iscsi nodename" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub nodename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            'cmd_execution_time' => {type => SCALARREF, optional => 1},
            _optional_scalars(
                qw( target-name force
                    )
            ),
        },
        cli_cmd => 'vserver iscsi nodename',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $return = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_nodename_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_nodename_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                check     => '_check_cmode_zapi_nodename',
            }
        ],
        frontend => 'NACL::C::VserverIscsi::nodename',
    );

    $Log->exit() if $may_exit;
    return $return;
} ## end sub nodename

sub _nodename_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $response;
    my $execution_time = $opts{cmd_execution_time};

    $pkg_or_obj->_base_backend_cmode_cli(%opts,
        api                 => 'vserver_iscsi_nodename',
        apiset_response_obj => \$response
    );

    if (defined $execution_time) {
        $$execution_time = $response->get_execution_time();
    }
    if (!$opts{node}) {
        my $output = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{'nodename'};
    }
    $Log->exit() if $may_exit;
} ## end sub _nodename_cmode_cli

sub _nodename_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@opts);

    my $apiset            = $opts{apiset};
    my $vserver           = $opts{vserver};
    my $command_interface = $opts{command_interface};
    my %cmode_args;

    $pkg_or_obj->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    $pkg_or_obj->_hash_copy(
        source => \%opts,
        map    => {'target-name' => 'node-name'},
        target => \%cmode_args,
    );

    if ($opts{'target-name'}) {
        $apiset->iscsi_node_set_name(%cmode_args);
    } else {
        my $response = $apiset->iscsi_node_get_name(%cmode_args);
        my $output   = $response->get_parsed_output();
        $Log->exit() if $may_exit;
        return $output->[0]{'node-name'};
    }

    $Log->exit() if $may_exit;
} ## end sub _nodename_cmode_zapi

sub _check_cmode_zapi_nodename {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (defined $opts{force} && $opts{force} =~ /true/i) {
        my $msg = "force was provided in the call to NACL::C::VserverIscsi->nodename"
                                              . ' so the ZAPI implementation cannot be used';
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}

=head2 status

    my $status_obj = NACL::C::VserverIscsi->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

    my @status_objs = NACL::C::VserverIscsi->status(command_interface => $ci,
                          [ filter => { %filter } ,]
              [ requested_fields => [ @req_fields ]);

                    or

   my $status_obj = $VserverIscsi_obj->status();

(Class or Instance method)

This method is a CS-like fetch method which accepts all the parameters of
L<fetch|lib-NACL-CS-ComponentState-pm/fetch> (requested_fields, filter, allow_empty)
and has all of the other properties of fetch methods (callable in scalar or list
context, throws a NoElementsFound exception if no matching elements are found).
The requested fields and filter takes L<ATTRIBUTES|lib-NACL-CS-VserverIscsiStatus-pm/ATTRIBUTES>
of NACL::CS::VserverIscsiStatus as input.
This method returns objects of the type L<NACL::CS::VserverIscsiStatus|lib-NACL-CS-VserverIscsiStatus-pm>.

=cut

sub status {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->_handle_obj_cs_like_calls(@args);
    require NACL::CS::VserverIscsiStatus;
    my @state_objs = NACL::CS::VserverIscsiStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ('vserver' => {type => SCALAR | UNDEF, optional => 1},);
}
1;
