# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiConnection Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverIscsiConnection

=head1 DESCRIPTION

C<NACL::C::VserverIscsiConnection> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverIscsiConnection.  A related class is
L<NACL::CS::VserverIscsiConnection|lib-NACL-CS-VserverIscsiConnection-pm>, which represents
the past state of an ONTAP VserverIscsiConnection.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 tsih

=head2 vserver

=head2 connection-id

=head2 tpgroup

=cut

package NACL::C::VserverIscsiConnection;

use strict;
use warnings;

use NACL::CS::VserverIscsiConnection;
use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head1 METHODS

=head2 new

  my $VserverIscsiConnection = NACL::C::VserverIscsiConnection->new(command_interface=>$ci,
                                   tsih =>$tsih,
                                   vserver =>$vserver,
                                   'connection-id' =>$connection_id,
                                  );

Return a new VserverIscsiConnection component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'tsih',
    scalar => 'vserver',
    scalar => 'connection_id',
    scalar => 'tpgroup',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, @args) = @_;
    $self->SUPER::init(@args);
    $self->_vserver_context_init(@args);
    $Log->exit() if $may_exit;
}

=head2 find

    my $VserverIscsiConnection  = NACL::C::VserverIscsiConnection->find(command_interface => $ci, ...);
    my @VserverIscsiConnections = NACL::C::VserverIscsiConnection->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverIscsiConnection_state = $NACL::C::VserverIscsiConnection->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverIscsiConnection would return
a NACL::CS::VserverIscsiConnection object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 shutdown

   NACL::C::VserverIscsiConnection->shutdown(
                                command_interface => $ci,
                                 tsih => $tsih,
                                 vserver => $vserver,
                                     tpgroup => $tpgroup,
                                     'connection-id' => $connection_id
                                  ...);

                                 or

    $VserverIscsiConnection_object->shutdown(...);

(Class or instance method)

Uses a CMode CLI/ZAPI APISet.

Invokes "iscsi-connection-shutdown" API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "tsih" => $tsih >>

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: session-id

(Required for class method, Not Applicable for instance method)

=item C<< "vserver" => $vserver >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
option.

=item C<< "connection_id" => $connection_id >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: connection-id

=item C<< "tpgroup" => $tpgroup >>

(Required for class method, Not Applicable for instance method)

Applicable for CMode CLI/ZAPI.

Maps to:

CM ZAPI: tpgroup-name

=item C<< "connId" => $string >>

Applicable for CMode CLI.

=item C<< "node" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub shutdown {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( connId node

                    connection-id tpgroup
                    )
            ),
        },
        cli_cmd => 'vserver iscsi connection shutdown',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_shutdown_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => "_shutdown_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub shutdown

sub _shutdown_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    $pkg->_base_backend_cmode_cli(%opts,
        api     => 'vserver_iscsi_connection_shutdown',
    );
    $Log->debug("Opts to API call 'vserver_iscsi_connection_shutdown' are: \n"
            . Dumper(\%opts));
    $Log->exit() if $may_exit;
} ## end sub _shutdown_cmode_cli

sub _shutdown_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $allowed_cmode_options = [qw( connection-id )];
    $pkg->_base_backend_cmode_zapi(@opts,
        api    => 'iscsi_connection_shutdown',
        copy   => $allowed_cmode_options,
        map    => {
            'tpgroup' => 'tpgroup-name',
            'tsih'    => 'session-id'
        },
    );
    $Log->exit() if $may_exit;
} ## end sub _shutdown_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return (
        'tsih'          => {type => SCALAR},
        'vserver'       => {type => SCALAR},
        'tpgroup'       => {type => SCALAR},
        'connection-id' => {type => SCALAR},
    );
} ## end sub _primary_keys_validate_spec

1;
