# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesLdapClientSchema Component Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesLdapClientSchema

=head1 DESCRIPTION

C<NACL::C::VserverServicesLdapClientSchema> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP VserverServicesLdapClientSchema.  A related class is
L<NACL::CS::VserverServicesLdapClientSchema|lib-NACL-CS-VserverServicesLdapClientSchema-pm>, which represents
the past state of an ONTAP VserverServicesLdapClientSchema.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 schema

=head2 vserver

=cut

package NACL::C::VserverServicesLdapClientSchema;

use strict;
use warnings;

use NACL::CS::VserverServicesLdapClientSchema;
use NACL::ComponentUtils qw (_optional_scalars);
use NACL::C::_Mixins::VserverServicesLdapClientSchema qw(:all);
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF OBJECT);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::C::_Version;

=head1 METHODS

=head2 new

  my $VserverServicesLdapClientSchema = NACL::C::VserverServicesLdapClientSchema->new(command_interface=>$ci,
                                   schema =>$schema,
                                  );

Return a new VserverServicesLdapClientSchema component object with the given attributes.

=cut

use Class::MethodMaker [
    scalar => 'schema',
    scalar => 'vserver',
];

sub init {
    $Log->enter() if $may_enter;
    my ($self, %opts) = @_;
    $self->SUPER::init(%opts);
    $self->_vserver_context_init(%opts);
    $Log->exit() if $may_exit;
} ## end sub init

=head2 find

    my $VserverServicesLdapClientSchema  = NACL::C::VserverServicesLdapClientSchema->find(command_interface => $ci, ...);
    my @VserverServicesLdapClientSchemas = NACL::C::VserverServicesLdapClientSchema->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

=head2 state

    my $VserverServicesLdapClientSchema_state = $NACL::C::VserverServicesLdapClientSchema->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::VserverServicesLdapClientSchema would return
a NACL::CS::VserverServicesLdapClientSchema object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

=cut

=head2 delete

   NACL::C::VserverServicesLdapClientSchema->delete(
                                command_interface => $ci,
                                 schema => $schema,
                                  ...);

                                 or

    $VserverServicesLdapClientSchema_object->delete(...);

(Class or instance method)
This method is used to delete an LDAP schema template.

Uses a CMode CLI or ZAPI APISet.
  Invokes 'ldap-client-schema-delete' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "schema" => $schema >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: schema

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap client schema delete" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub delete {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params  => \@args,
        cli_cmd => 'vserver services ldap client schema delete',
    );
    my $vserver;
    $opts{vserver_operated_on} = \$vserver;
    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_delete_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_delete_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesLdapClientSchema::delete',
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub delete

sub _delete_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_services_ldap_client_schema_delete',);
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_cli

sub _delete_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(schema)],
        api  => 'ldap-client-schema-delete',
    );
    $Log->exit() if $may_exit;
} ## end sub _delete_cmode_zapi

=head2 copy

    my $VserverServicesLdapClientSchema_object    = NACL::C::VserverServicesLdapClientSchema->copy(
                                command_interface => $ci,
                                 schema => $schema,
                                  ...);

                                 or

    $VserverServicesLdapClientSchema_object->copy(...);

(Class or instance method)
This method is used to copy an existing LDAP schema template.

Uses a CMode CLI or ZAPI APISet.
  Invokes 'ldap-client-schema-copy' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "schema" => $schema >>

(Required for class method, Not Applicable for instance method)
  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: schema

=item C<< "new-schema-name" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: new-schema-name

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap client schema copy" command in ONTAP.

Applicable for CMode CLI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub copy {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( new-schema-name
                    )
            ),
        },
        cli_cmd  => 'vserver services ldap client schema copy',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_copy_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_copy_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesLdapClientSchema::copy',
    );

    my %new_args;
    $new_args{vserver} = $vserver if ($vserver);
    my $obj = $pkg->new(
        command_interface => $opts{command_interface},
        schema            => $opts{schema},
        %new_args,
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub copy

sub _copy_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_cli(@opts,
        api => 'vserver_services_ldap_client_schema_copy',);

    $Log->exit() if $may_exit;
} ## end sub _copy_cmode_cli

sub _copy_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => [qw(new-schema-name schema)],
        api  => 'ldap-client-schema-copy',
    );
    $Log->exit() if $may_exit;

} ## end sub _copy_cmode_zapi

=head2 modify

   NACL::C::VserverServicesLdapClientSchema->modify(
                                command_interface => $ci,
                                 schema => $schema,
                                  ...);

                                 or

    $VserverServicesLdapClientSchema_object->modify(...);

(Class or instance method)
This method is used to modify an LDAP schema template.

Uses a CMode CLI or ZAPI APISet.
  Invokes 'ldap-client-schema-modify' API for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "schema" => $schema >>

(Required for class method, Not Applicable for instance method)
Applicable for CMode CLI/ZAPI

=item C<< "member-uid-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: member-uid-attribute

=item C<< "posix-account-object-class" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: posix-account-object-class

=item C<< "posix-group-object-class" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: posix-group-object-class

=item C<< "unix-account-attribute" => $string >>

Applicable only for CMode CLI

=item C<< "uid-number-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: uid-number-attribute

=item C<< "cn-netgroup-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: cn-netgroup-attribute

=item C<< "windows-to-unix-attribute" => $string >>

Applicable only for CMode CLI

=item C<< "unique-member-attribute" => $string >>

Applicable only for CMode CLI

=item C<< "nis-netgroup-triple-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-netgroup-triple-attribute

=item C<< "user-password-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: user-password-attribute

=item C<< "home-directory-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: home-directory-attribute

=item C<< "windows-to-unix-object-class" => $string >>

Applicable only for CMode CLI

=item C<< "cn-group-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: cn-group-attribute

=item C<< "login-shell-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: login-shell-attribute

=item C<< "gecos-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: gecos-attribute

=item C<< "member-nis-netgroup-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: member-nis-netgroup-attribute

=item C<< "gid-number-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: gid-number-attribute

=item C<< "nis-netgroup-object-class" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: nis-netgroup-object-class

=item C<< "group-of-unique-names-object-class" => $string >>

Applicable only for CMode CLI

=item C<< "uid-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: uid-attribute

=item C<< "comment" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: comment

=item C<< "windows-account-attribute" => $string >>

  Applicable for CMode CLI/ZAPI
  Maps to:
    CMode ZAPI: windows-account-attribute

=item C<< "enable-rfc2307bis" => $string >>

Applicable only for CMode CLI

=item C<< "vserver" => $string >>

  Applicable for CMode CLI/ZAPI.
  Maps to:
    CM ZAPI: ZAPI will be invoked in the context of the vserver specified to this
             option.

=item C<< "extended_query" => $string >>

(Optional)
Applicable only for CMode CLI

=item C<< other_options >>

This method takes other common options described in L<Common method options|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#Other_options_valid_on_component_methods>

=item C<< '*' >>

This method takes all other options supported by
"vserver services ldap client schema modify" command in ONTAP.

Applicable for CMode CLI/ZAPI.

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( member-uid-attribute posix-account-object-class posix-group-object-class
                    unix-account-attribute uid-number-attribute cn-netgroup-attribute
                    windows-to-unix-attribute unique-member-attribute
                    nis-netgroup-triple-attribute user-password-attribute
                    home-directory-attribute windows-to-unix-object-class cn-group-attribute
                    login-shell-attribute gecos-attribute member-nis-netgroup-attribute
                    gid-number-attribute nis-netgroup-object-class
                    group-of-unique-names-object-class uid-attribute comment
                    windows-account-attribute enable-rfc2307bis
                    )
            ),
        },
        cli_cmd => 'vserver services ldap client schema modify',
    );

    my $vserver;
    $opts{vserver_operated_on} = \$vserver;

    my $ret = $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        frontend => 'NACL::C::VserverServicesLdapClientSchema::modify',
    );
    $Log->exit() if $may_exit;
    return $ret;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_cli(@opts,
        api => 'vserver_services_ldap_client_schema_modify',);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my $copy = $pkg->_common_zapi_copy();
    $pkg->_base_backend_cmode_zapi(
        @opts,
        copy => $copy,
        api  => 'ldap-client-schema-modify',
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    my $pkg_or_obj = shift;
    my %opts       = validate(
        @_,
        {   command_interface => {
                type     => OBJECT,
                isa      => 'NACL::C::CommandInterface',
                optional => 1
            }
        }
    );
    my $command_interface = $opts{command_interface};

    my %default_spec = (
        'schema'  => {type => SCALAR},
        'vserver' => {type => SCALAR, optional => 1},
    );

    return (%default_spec);
} ## end sub _primary_keys_validate_spec
1;
