#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesNetgroup Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::VserverServicesNetgroup

=head1 DESCRIPTION

C<NACL::C::VserverServicesNetgroup> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. Unlike other components this
package does not model any element/service but is present to provide 
component-level support for functionality. Since it does not model any 
element/service there is no ComponentState module for this component.
For the same reason, 'find' and 'state' will not work for this component.

=cut

package NACL::C::VserverServicesNetgroup;

use strict;
use warnings;

use NACL::ComponentUtils qw (_optional_scalars);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

=head2 load_from_uri

   NACL::C::VserverServicesNetgroup->load_from_uri(
                                command_interface => $ci,
                                  ...);

(Class method)
This method is used to load netgroup definitions from a URI.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< "vserver" => $string >>

Applicable for CMode CLI.

=item C<< "uri" => $string >>

Applicable for CMode CLI.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub load_from_uri {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->_frontend_log_debug_opts(@args);
    my %opts = $pkg->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( vserver uri
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    my $ret = $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => "_load_from_uri_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
    );

    $Log->exit() if $may_exit;
    return $ret;
} ## end sub load_from_uri

sub _load_from_uri_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => $pkg->_backend_validate_spec(),
        allow_extra => 1
    );

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( vserver uri
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    $pkg->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $command_interface,
        frontend_method   => 'load_from_uri',
    );

    %opts = %cmode_args;

    $Log->debug(
        "Opts to API call 'vserver_services_netgroup_load_from_uri' are: \n"
            . Dumper(\%opts));

    $apiset->vserver_services_netgroup_load_from_uri(%opts);
    $Log->exit() if $may_exit;
} ## end sub _load_from_uri_cmode_cli

=head2 status

    my $status_obj = NACL::C::VserverServicesNetgroup->status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);
						  
                    or
					
    my @status_objs = NACL::C::VserverServicesNetgroup->status(command_interface => $ci,
                          [ filter => { %filter } ,]
			  [ requested_fields => [ @req_fields ]);

                    or
  
   my $status_obj = $VserverServicesNetgroup_obj->status(); 

(Class or Instance method)
This method accepts all the parameters accepted by L<fetch|lib-NACL-CS-ComponentState-pm/fetch> 
(requested_fields, filter, allow_empty) and has all of the other properties of 
fetch methods (callable in scalar or list context, throws a NoElementsFound 
exception if no matching elements are found)

=cut

sub status {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_handle_obj_cs_like_calls(@_);
    require NACL::CS::VserverServicesNetgroupStatus;
    my @state_objs = NACL::CS::VserverServicesNetgroupStatus->fetch(%opts);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub status

=head2 load

    NACL::C::VserverServicesNetgroup->load(
                                command_interface => $ci,
                                ...);

(Class method)
This method is used to load netgroup definitions from a URI.

Uses a CMode CLI APIset.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'source' => $string >>

Applicable for CMode CLI.

=item C<< 'vserver' => $string >>

Applicable for CMode CLI.

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=item C<< "extended_query" => $string >>

(Optional)

Applicable for CMode CLI

=back

=back

=over

=item Exceptions

=over

=item C<APISet exceptions>

In general, Component methods propagate the exceptions thrown from 
underlying API layer and the details of those exceptions are described in the
L<Exceptions section|http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/TestAutomation/NACL/UsersGuide/ComponentLayerUsersGuide/index.html#How_do_I_know_which_exceptions_are_thrown_by_a_component_method>
of user guide.

=back

=back

=cut

sub load {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $pkg->_frontend_log_debug_opts(@_);
    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        additional_spec => {
            _optional_scalars(
                qw( source vserver
                    )
            ),
        },
        ignore_primary_keys => 1,
    );
    $pkg->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_load_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub load

sub _load_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg  = shift;
    my %opts = $pkg->_validate_backend_opts(@_);

    my $command_interface = $opts{command_interface};
    my $apiset            = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( source vserver
            extended_query )
    ];
    my %cmode_args;
    $pkg->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );
    $pkg->_handle_cli_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $cmode_args{vserver},
        command_interface => $command_interface,
        frontend_method   => 'load',
    );

    $pkg->_api_call_log_debug_opts(
        api      => 'vserver_services_netgroup_load',
        api_opts => \%cmode_args
    );

    $apiset->vserver_services_netgroup_load(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _load_cmode_cli


__PACKAGE__->_load_event_class();

sub _primary_keys_validate_spec {
    return ();
}
1;
