# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Cluster and Cserver modules
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::ClusterCserver;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        _ci_to_test_param_mapping
        _hosttype
        _check_if_valid_hostrec
        resource_record_method_name
        default_apiset_must
    );
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

# For this command interface, what is the common prefix for the test parameters?
sub _ci_to_test_param_mapping {
    $Log->enter();
    $Log->exit();
    return 'CLUSTER';
}

# When we dynamically create hostrecs, this is the hosttype which we should
# assign
sub _hosttype {
    $Log->enter();
    $Log->exit();
    return 'filer-ng-cserver';
}

sub _check_if_valid_hostrec {
    $Log->enter();
    my ($self, @opts) = @_;
    my $base = $self->base_class();
    my $base_method = "${base}::_check_if_valid_hostrec";
    $self->$base_method(@opts);
    $self->_check_if_hostrec_cmode(@opts);
    $Log->exit();
} ## end sub _check_if_valid_hostrec

sub resource_record_method_name {
    $Log->enter();
    $Log->exit();
    return 'clusters';
}

sub default_apiset_must {
    $Log->enter();
    my ($self) = @_;
    my $pkg  = $self->get_package_name();
    my $base = $self->base_class();
    my $base_apiset_must = "${base}::default_apiset_must";
    $Log->exit();
    return {
        %{$self->$base_apiset_must()},
        set        => 'CMode|Nodescope|Systemshell',
        set_source => "command interface is a $pkg",
    };
} ## end sub default_apiset_must

1;
