#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the ClusterPeer Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::ClusterPeer;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_convert_to_unix_time);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}
use NACL::ComponentUtils qw(convert_to_unix_time);

sub _convert_to_unix_time {
    my ($pkg, %opts) = @_;
    my $offer_expiration;
    if ($opts{'offer-expiration'} =~ /(\d+)\/(\d+)\/(\d+) (\d+):(\d+):(\d+)/) {
        my ($mm, $dd, $hh, $min, $sec, $year);
        $mm  = $1;
        $dd  = $2;
        $year = $3;
        $hh  = $4;
        $min = $5;
        $sec = $6;
        $offer_expiration = $pkg->convert_to_unix_time(
            command_interface => $opts{command_interface},
            interface         => 'ZAPI',
            year              => $year,
            month             => $mm,
            day               => $dd,
            hour              => $hh,
            minute            => $min,
            second            => $sec,
        );
    } else {
        $offer_expiration = $opts{'offer-expiration'};
    }
    return $offer_expiration;
} ## end sub _convert_to_unix_time
1;
