#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the StorageAggregate.pm and Volume.pm Components
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::ComponentCommandInterfaceONTAP;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALARREF ARRAYREF HASHREF);
use NACL::EventService qw(load_branch_events load_leaf_events);
use NACL::ChooseAPISet;
use NATE::ParamSet qw(param_global);
my $service = param_global()->get('NACL_PUBLISH_EVENTS');

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_get_additional_common_validate_spec call_on_apiset);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

# These fields cannot be added to _common_validate_spec since they end up
# causing validation failures on the CS side, hence this extra spec for
# options that are common to all methods on this Component on the C side.
sub _get_additional_common_validate_spec {
    my ($pkg_or_obj) = @_;
    my $spec = {};
    $spec->{job_component}  = {type => SCALARREF, optional => 1};
    $spec->{job_components} = {type => ARRAYREF,  optional => 1};
    $spec->{extended_query} = {type => HASHREF,   optional => 1};
    return $spec;
}

#
# Overridden call_on_apiset which calls the call_on_apiset in
# NACL::ChooseAPISet and in the end it publishes events
#
sub call_on_apiset {
    $Log->enter() if $may_enter;
    my( $pkg_or_obj, %opts ) = @_;
    my @val = $pkg_or_obj->NACL::ChooseAPISet::call_on_apiset(%opts);
    my $pkg_name = $pkg_or_obj->get_package_name();
    if ( $service
         && $service == 1
         && $pkg_name =~ /::C::/ ) {
         my $front             = delete $opts{frontend};
         my $cli_cmd           = $opts{cli_cmd};
         my $command_interface = $opts{command_interface};
         if ( ($front || $cli_cmd)
              && $command_interface->is_cmode() ) {
              my $action_event_name;
              if ( $front && $front =~ /NACL::C::(.*$)/  ) {
                  $action_event_name = "NACL::Events::C_STask::".$1;
              } elsif ( $cli_cmd ) {
                  my @sub_cmds = split( /\s/, $cli_cmd );
                  my $leaf_cmd = pop @sub_cmds;
                  map { $action_event_name .= ucfirst($_) } @sub_cmds;
                  $action_event_name = "NACL::Events::C_STask::".$action_event_name."::".$leaf_cmd;
              }
              load_leaf_events(events => [$action_event_name]);
              my $event_obj   = $action_event_name->new();
	      my %copy_opts = %opts;
	      #payload should never include scalar references, hence just drop them before publishing
	      while (my ($key, $value) = each %opts) {
		delete $copy_opts{$key} if (ref $value eq 'SCALAR');
	      }
              $event_obj->publish( input_options        => \%copy_opts,
                                   original_pkg_or_obj  => $pkg_or_obj);
        }
    }
    $Log->exit() if $may_exit;
    return wantarray ? @val : $val[0];
}

1;
