#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Job and JobPrivate Components
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Job;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(wait_on_job);
    our %EXPORT_TAGS = ('all' => \@EXPORT_OK);
}

# Make it possible to do $job->wait_on_job() and not need to pass in
# job_component as argument, but instead use $self as the job_component
# to be waited on.
sub wait_on_job {
    $Log->enter();
    my ($pkg_or_obj, %opts) = @_;
    if (ref $pkg_or_obj) {
        $opts{job_component} ||= $pkg_or_obj;
    }
    
    # Can't use SUPER since we're in a mix-in. Use the base class method
    # to determine where to get this method from.
    my $base  = $pkg_or_obj->base_class();
    my $super = "${base}::wait_on_job";
    $pkg_or_obj->$super(%opts);
    $Log->exit();
}

1;
