#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the various Logger Components
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Logger;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(SCALAR);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK   = qw(_common_validate_with _primary_keys_validate_spec init
        _node_warning);
    our %EXPORT_TAGS = ('all' => \@EXPORT_OK);
}

# ONTAP change 821888 made "node" mandatory. We default node to "local" if not
# provided.
sub _common_validate_with {
    $Log->enter();
    my ($pkg_or_obj, %orig_opts) = @_;
    my $base_class = $pkg_or_obj->base_class();
    my $super_common_validate_with = "${base_class}::_common_validate_with";
    
    my %opts = $pkg_or_obj->$super_common_validate_with(%orig_opts);
    
    # Once the ONTAP change is available in all branches, we can remove the
    # uichange check.
    if (!defined $opts{node} && _has_uichange($opts{command_interface})) {
        my $api = $orig_opts{api};
        my $method = (split '_', $api)[-1];
        $opts{node} = 'local';
        $pkg_or_obj->_node_warning($method);
    }

    $Log->exit();
    return %opts;
}

sub _primary_keys_validate_spec {
    return (
        node   => {type => SCALAR, optional => 1},
        module => {type => SCALAR},
    );
}

sub init {
    $Log->enter();
    my ($self, %opts) = @_;
    my $base_class = $self->base_class();
    my $super_init = "${base_class}::init";
    $self->$super_init(%opts);
    if (!defined $opts{node} && _has_uichange($opts{command_interface})) {
        $self->node('local');
        $self->_node_warning('new');
    }
    $Log->exit();
}

sub _node_warning {
    my ($pkg_or_obj, $method) = @_;
    $Log->warn("With the fix for burt 821888, the 'node' option is "
        . 'mandatory for all logger * commands, however this was not '
        . 'provided in the call to '
        . $pkg_or_obj->get_package_name()
        . "->$method(). The Component has defaulted the value of 'node' "
        . "to 'local'. but the call should be updated to pass in the "
        . 'appropriate node on which the logging is required'
    );
}

sub _has_uichange {
    my $ci = $_[0];
    return $ci->has_uichange(uichange => 'logger-supporting-node-parameter');
}

1;
