# $Id: //depot/prod/test/nacldev/lib/NACL/C/_Mixins/Lun.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Lun Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::Lun;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with HASHREF SCALAR ARRAYREF);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_construct_path _extract_volume_qtree_lun_from_path
        _extract_volume_from_path _extract_qtree_from_path
        _extract_lun_from_path _common_validate_with);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}
use NATE::Exceptions::Argument qw(:try);

sub _construct_path {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    %opts = validate_with(
        params => [%opts],
        spec   => {params => {type => HASHREF},}
    );

    my $params = $opts{params};
    my %opts_for_processing;
    $pkg_or_obj->_hash_move(
        source => $params,
        target => \%opts_for_processing,
        move   => [qw(volume lun qtree)]
    );
    $pkg_or_obj->_process_options(options => \%opts_for_processing);
    if ($params->{path}
        && (   $opts_for_processing{volume}
            || $opts_for_processing{lun}
            || $opts_for_processing{qtree})
        )
    {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "Cannot provide any of 'volume', 'lun' or 'qtree' along with 'path'"
        );
    } ## end if ( $params->{path} && ...

    if (   !$params->{path}
        && !($opts_for_processing{volume} && $opts_for_processing{lun}))
    {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw(
            "When 'path' is not provided, then both 'volume' and 'lun' must be provided "
                . 'but at least one of them is missing');
    }

    if (!$params->{path}) {
        $params->{path} = '/vol';
        $params->{path} .= '/' . delete $opts_for_processing{volume};
        $params->{path} .= '/' . delete $opts_for_processing{qtree}
            if (exists $opts_for_processing{qtree});
        $params->{path} .= '/' . delete $opts_for_processing{lun};
    } ## end if ( !$params->{path} )
    $Log->exit() if $may_exit;
}

sub _extract_volume_qtree_lun_from_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { path => { type => SCALAR }, }
    );

    my ( $volume, $qtree, $lun )
        = ( $opts{path} =~ /\/vol\/([\w\-]+)(?:\/([\w\-]+))?\/([\S]+)$/  );
    if ( !defined $volume or !defined $lun ) {
        NATE::Exceptions::Argument->throw( "Path needs to be provided in the "
                . "format '/vol/volume/lun' or /vol/volume/qtree/lun'." );
    }
    $Log->exit() if $may_exit;
    return ( $volume, $qtree, $lun );
} ## end sub _extract_volume_qtree_lun_from_path

sub _extract_volume_from_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    $Log->exit() if $may_exit;
    return ( $pkg_or_obj->_extract_volume_qtree_lun_from_path(%opts) )[0];
} ## end sub _extract_volume_from_path

sub _extract_qtree_from_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    $Log->exit() if $may_exit;
    return ( $pkg_or_obj->_extract_volume_qtree_lun_from_path(%opts) )[1];
} ## end sub _extract_qtree_from_path

sub _extract_lun_from_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, %opts ) = @_;
    $Log->exit() if $may_exit;
    return ( $pkg_or_obj->_extract_volume_qtree_lun_from_path(%opts) )[2];
} ## end sub _extract_lun_from_path

sub _common_validate_with {
    my $pkg_or_obj = shift;
    my $flag       = 0;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            params              => {type => ARRAYREF},
            additional_spec     => {type => HASHREF, default => {}},
            ignore_primary_keys => {type => SCALAR, default => 0},
        },
        allow_extra => 1
    );
    my %params = @{$opts{params}};

    my %spec = %{$opts{additional_spec}};

 #
 # This is a hack, since path is the primary key ,if path has not been passed
 # we insert a dummy path ,so that SUPER::_common_validate_with  wont throw an
 # error
 #
    if (!ref $pkg_or_obj && !$params{path} && !$opts{ignore_primary_keys}) {
        $params{path} = "dummy";
        $flag = 1;
    }
    my %allow_extra;

        if ((exists $params{'allow_extra'}) or exists($opts{'allow_extra'})) {
        $allow_extra{'allow_extra'} = 1;
    }
    if (!$opts{ignore_primary_keys}) {
        foreach my $pk (qw(volume qtree lun)) {
            $spec{$pk} ||= {type => SCALAR, optional => 1};
        }
    }
    my @array = %params;
    %params = $pkg_or_obj->_super_common_validate_with(
        params              => \@array,
        additional_spec     => \%spec,
        ignore_primary_keys => $opts{ignore_primary_keys},
        %allow_extra
    );
    if ($flag) {
        delete $params{path};
    }

    if (!$opts{ignore_primary_keys}) {
        $pkg_or_obj->_construct_path(params => \%params);
    }
    return %params;
} ## end sub _common_validate_with

1;
