# $Id: //depot/prod/test/nacldev/lib/NACL/C/_Mixins/LunCopyRelocation.pm#1 $

#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the Lun Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::LunCopyRelocation;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with HASHREF SCALAR);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw( _get_uuid_lun_index _modify_cmode_zapi_helper
                         _construct_lun_path_pairs _start_common_copy
                         _common_fetch_zapi_copy _check_zapi_option _get_destination_lun_obj);
    our %EXPORT_TAGS = ('all' => \@EXPORT_OK);
}
use NATE::Exceptions::Argument qw(:try);

sub _get_uuid_lun_index {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $opts) = @_;
    if (!defined $opts->{'job-uuid'} || !defined $opts->{'lun-index'}) {
        my @fields = (qw(job-uuid lun-index));
        my $cs_pkg = $pkg_or_obj->get_CS_package_name();
        ##Using CLI interface to work around the ZAPI bug
        my $cs = $cs_pkg->fetch(
            command_interface => $opts->{command_interface},
            filter            => {
                vserver            => $opts->{vserver},
                'destination-path' => $opts->{'destination-path'}
            },
            requested_fields => \@fields
        );
        map { $opts->{$_} ||= $cs->$_() } @fields;

        # Update object if called with one
        if (ref $pkg_or_obj) {
            foreach my $field (@fields) {
                my $isset = "${field}_isset";
                if (!$pkg_or_obj->$isset()) {
                    $pkg_or_obj->$field($cs->$field());
                }
            }
        }
    }
    $Log->exit() if $may_exit;
} ## end sub _get_uuid_lun_index

sub _modify_cmode_zapi_helper {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @opts) = @_;
    $pkg_or_obj->_base_backend_cmode_zapi(@opts,
        copy => [qw(max-throughput)]);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi_helper

sub _construct_lun_path_pairs {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) =@_;
    my $src_paths = $opts{'source-paths'} || [$opts{'source-path'}];
    my $dst_paths = $opts{'destination-paths'} || [$opts{'destination-path'}];
    my $num_src_paths = scalar @$src_paths;
    my $num_dst_paths = scalar @$dst_paths;
    if ($num_src_paths != $num_dst_paths) {
        $Log->exit() if $may_exit;
        NATE::Exceptions::Argument->throw("Number of source paths"
             . " '$num_src_paths' doesnot  match number of destination"
             . " paths '$num_dst_paths'")
    }
    my @lun_path_pairs;
    for (my $i = 0; $i < $num_src_paths; $i++) {
        push @lun_path_pairs, { 'source-path' => $src_paths->[$i],
        'destination-path' => $dst_paths->[$i] };
    }
    $Log->exit() if $may_exit;
    return \@lun_path_pairs;
} ## end sub  _construct_lun_path_pairs

sub _start_common_copy {
    $Log->enter() if $may_enter;
    my $copy_options = [
         qw( max-throughput promote-late promote-early)
    ];
    $Log->exit() if $may_exit;
    return $copy_options;
} ## end sub start_common_copy

sub _common_fetch_zapi_copy {
    $Log->enter() if $may_enter;
    my @copy = (
         qw(cutover-time destination-path elapsed-time is-destination-ready
            is-snapshot-fenced job-status job-uuid last-failure-reason lun-index
            max-throughput progress-percent scanner-progress scanner-total source-path)
         );
    $Log->exit() if $may_exit;
    return wantarray ? @copy : [@copy];
}

sub _check_zapi_option {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    if (exists $opts{'pause-transfer'}) {
        my $msg = 'Could not use ZAPI implementation because field '
            . "'pause-transfer' is not accepted by the ZAPI command ";
        $Log->comment($msg);
        $Log->exit() if $may_exit;
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    $Log->exit() if $may_exit;
}
sub _get_destination_lun_obj {
    $Log->enter() if $may_enter;
      my ($self,%opts) = @_;
       my $lun = NACL::STask::Lun->new(
          command_interface => $self->command_interface(),
          vserver           => $self->vserver(),
          path              => $self->destination_path()
        );
    $Log->exit() if $may_exit;
    return $lun;

}
1;
