#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the VolumeQtree Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::VolumeQtree;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with HASHREF);
use NATE::Exceptions::Argument;
use File::Spec;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK
        = qw(_convert_path_to_volume_qtree _construct_new_qtree_path _construct_qtree_path);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

sub _construct_new_qtree_path {
    $Log->enter() if $may_enter;
    my $qtree;
    my ( $qtree_path, $newname ) = @_;
    ( undef, undef, $qtree ) = File::Spec->splitpath($qtree_path);
    $qtree_path =~ s/\Q$qtree\E$/$newname/;
    $Log->exit() if $may_exit;
    return $qtree_path;

} ## end sub _construct_new_qtree_path

sub _construct_qtree_path {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = validate_with(
        params => \@args,
        spec   => { args_ptr => { type => HASHREF }, },
    );
    my $args = $opts{args_ptr};
    if ( exists $args->{volume} and exists $args->{qtree} ) {
        $args->{'qtree-path'}
            = "/vol/" . $args->{volume} . "/" . $args->{qtree};
    }
} ## end sub _construct_qtree_path

# Updates in-place the hash-reference provided.
sub _convert_path_to_volume_qtree {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = validate_with(
        params => \@_,
        spec   => {
            args_ptr          => { type => HASHREF },
            command_interface => 1
        },
    );

    my $args_ptr = $opts{args_ptr};
    if (   exists $args_ptr->{'qtree-path'}
        or exists $args_ptr->{'qtree_path'} ) {
        $args_ptr->{'qtree-path'} = $args_ptr->{'qtree-path'}
            || $args_ptr->{'qtree_path'};
        my $path = delete $args_ptr->{'qtree-path'};
        if ( $path =~ m!/vol/(.*)/(.*)! ) {
            $args_ptr->{volume} = $1;
            $args_ptr->{qtree}  = $2;
        } else {
            $Log->exit() if $may_exit;
            NATE::Exceptions::Argument->throw(
                      "The 'qtree-path' ($path) argument "
                    . 'was not in the expected format' );
        } ## end else [ if ( $path =~ m!/vol/(.*)/(.*)!)]
    } ## end if ( exists $args_ptr->...)

    $Log->exit() if $may_exit;
} ## end sub _convert_path_to_volume_qtree

1;
