# $Id: //depot/prod/test/nacldev/lib/NACL/C/_Mixins/VserverIscsi.pm#1 $

#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Common methods for the VserverIscsi Component and ComponentState
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::C::_Mixins::VserverIscsi;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate_with HASHREF SCALAR ARRAYREF);
use NACL::CS::NetworkInterface;

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_handle_lif_all_options _get_all_iscsi_lifs _check_all_lif_both_exist);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}
sub _check_all_lif_both_exist {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;
    if (defined $opts{lif}  && defined $opts{all}) {
        my $resp_obj = bless {}, 'NACL::APISet::Response::ZAPI::CMode';
        my $err_msg = 'Response exception: Field "-lif" cannot be used with field "-all".';
        NACL::APISet::Exceptions::ResponseException->throw(text => $err_msg , response_object => $resp_obj );
    }
    $Log->exit() if $may_exit;
} ## end sub _check_all_lif_both_exist

sub _get_all_iscsi_lifs {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;
    my $command_interface = $opts{command_interface};
    my $vserver           = $opts{vserver};
    my (@lifs,@lifname);
    @lifs = NACL::CS::NetworkInterface->fetch(
            command_interface => $command_interface,
            requested_fields  => [qw(lif)],
            filter            => {vserver => $vserver, 'data-protocol' => ['iscsi'] },
            allow_empty       => 1,
            );
    foreach my $lif (@lifs){
        push @lifname,$lif->lif();
    }
    $Log->exit() if $may_exit;
    return @lifname;
    
}## end sub _get_all_iscsi_lifs

sub _handle_lif_all_options {
    $Log->enter() if $may_enter;
    my ($pkg,%opts) = @_;
    my $lif;
    if (defined $opts{all} && $opts{all} eq 'true') {
        $pkg->_check_all_lif_both_exist(%opts);
        my @lifname = $pkg->_get_all_iscsi_lifs(%opts);
        $lif = \@lifname;
    } else {
        $lif = $opts{lif};
    }
    $Log->exit() if $may_exit;
    return $lif;
} ## end _handle_lif_all_options

1;
