#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary EcacheStats Component Module
## @author Paul Chen chenp@netapp.com,dl-metawafl-qa@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::AggregateEcacheStats

=head1 DESCRIPTION

C<NACL::CS::AggregateEcacheStats> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the state of metawafl Eicach. A related class is
L<NACL::C::AggregateEcache|lib-NACL-C-AggregateEcache-pm>, which represents
access to metawafl nodescope 'mw tec'.

=head1 ATTRIBUTES

=over

=item C<< equeue_free_count >>

Count of free queue

Filled in for 7Mode Nodescope.

=item C<< equeue_unused_count >>

Count of unused queue

Filled in for 7Mode Nodescope.

=item C<< equeue_recycle_count >>

Count of recycle queue

Filled in for 7Mode Nodescope.

=item C<< equeue_dirty_count >>

Count of dirty queue

Filled in for 7Mode Nodescope.

=cut

package NACL::CS::AggregateEcacheStats;

use strict;
use warnings;

use Tharn qw($Log);
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'equeue_free_count',
    scalar => 'equeue_unused_count',
    scalar => 'equeue_recycle_count',
    scalar => 'equeue_dirty_count',
    scalar => 'aggregate',
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::AggregateEcacheStats->fetch(
        command_interface => $ci,
        filter => { 'aggregate' => aggr_name,},
    );

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
    apiset_method => 'mw_tec_stats',
    filter_keys_to_be_passed_to_apiset => ['aggregate']
);

1;

