# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusOnAccessPolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::AntivirusOnAccessPolicy

=head1 DESCRIPTION

C<NACL::CS::AntivirusOnAccessPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP AntivirusOnAccessPolicy.  A related class is
L<NACL::C::AntivirusOnAccessPolicy|lib-NACL-C-AntivirusOnAccessPolicy-pm>, which represents access
to an ONTAP AntivirusOnAccessPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the AntivirusOnAccessPolicy
element are the attributes of the AntivirusOnAccessPolicy ComponentState.

=over

=item C<< "cifs_file" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <include-cifs-file>$value</include-cifs-file>
 </av-on-access-policy-info>

=item C<< "cifs_share_list" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <cifs-share-list-pattern>$value</cifs-share-list-pattern>
 </av-on-access-policy-info>

=item C<< "name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <policy-name>$value</policy-name>
 </av-on-access-policy-info>

=item C<< "scan_ro_volume" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <is-row-volume-scan-enabled>$value</is-row-volume-scan-enabled>
 </av-on-access-policy-info>

=item C<< "scan" >>

Filled in for CMode CLI/ZAPI.

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <is-scan-enabled>$value</is-scan-enabled>
 </av-on-access-policy-info>

=item C<< "cifs_file_list" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <cifs-file-list-pattern>$value</cifs-file-list-pattern>
 </av-on-access-policy-info>

=item C<< "protocols" >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $protocols = $obj->protocols();
# $protocols contains a reference to the array of values
my @protocols = $obj->protocols();
# @protocols contains the array of values

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <protocols>
        <protocol>@values</protocol>
    <protocols>
 </av-on-access-policy-info>

=item C<< "cifs_share" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <include-cifs-share>$value</include-cifs-share>
 </av-on-access-policy-info>

=item C<< "nfs_scan_execute_permission" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <is-nfs-execute-permission-scan-enabled>$value</is-nfs-execute-permission-scan-enabled>
 </av-on-access-policy-info>

=item C<< "scan_mandatory" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <is-scan-mandatory>$value</is-scan-mandatory>
 </av-on-access-policy-info>

=item C<< "id" >>

Filled in for CMode CLI.

=item C<< "fileop_profile" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <fileop-profile>$value</fileop-profile>
 </av-on-access-policy-info>

=item C<< "cifs_scan_execute_access" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <is-cifs-execute-access-scan-enabled>$value</is-cifs-execute-access-scan-enabled>
 </av-on-access-policy-info>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <vserver>$value</vserver>
 </av-on-access-policy-info>

=item C<< policy_owner >>

Creator of the Policy

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <av-on-access-policy-info>
    <policy-owner>$value</policy-owner>
 </av-on-access-policy-info>

=back

=cut

package NACL::CS::AntivirusOnAccessPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cifs_file',
    scalar => 'cifs_share_list',
    scalar => 'name',
    scalar => 'scan_ro_volume',
    scalar => 'scan',
    scalar => 'cifs_file_list',
    array  => 'protocols',
    scalar => 'cifs_share',
    scalar => 'nfs_scan_execute_permission',
    scalar => 'scan_mandatory',
    scalar => 'id',
    scalar => 'fileop_profile',
    scalar => 'cifs_scan_execute_access',
    scalar => 'vserver',
    scalar => 'policy_owner',
];

=head1 METHODS

=head2 fetch

    my $AntivirusOnAccessPolicy_state   = NACL::CS::AntivirusOnAccessPolicy->fetch(command_interface => $ci, ...);
    my @AntivirusOnAccessPolicy_states = NACL::CS::AntivirusOnAccessPolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "av-on-access-policy-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching antivirus on access policy(s) found',
        show_cmd       => 'antivirus on-access policy show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'antivirus_on_access_policy_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options
        = [qw( protocols fileop-profile vserver policy-owner)];
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'av_on_access_policy_get_iter',
        copy => $allowed_cmode_options,
        map  => {
            'cifs-file-list'  => 'cifs-file-list-pattern',
            'cifs-share-list' => 'cifs-share-list-pattern',
            'cifs-file'       => 'include-cifs-file',
            'cifs-share'      => 'include-cifs-share',
            'cifs-scan-execute-access' =>
                'is-cifs-execute-access-scan-enabled',
            'nfs-scan-execute-permission' =>
                'is-nfs-execute-permission-scan-enabled',
            'scan-ro-volume' => 'is-row-volume-scan-enabled',
            'scan'           => 'is-scan-enabled',
            'scan-mandatory' => 'is-scan-mandatory',
            'name'           => 'policy-name',
        },
    );

    my $conversion_values = {'true' => 'on', 'false' => 'off'};

    my $options_values_to_translate = [
        qw ( cifs-scan-execute-access nfs-scan-execute-permission
            scan-ro-volume scan-mandatory scan
            )
    ];
    foreach my $state_obj (@state_objs) {
        foreach my $field (@$options_values_to_translate) {
            if (defined $state_obj->$field()) {
                $state_obj->$field(
                    $conversion_values->{$state_obj->$field()});
            }
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
