#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterContactInfo ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterContactInfo

=head1 DESCRIPTION

C<NACL::CS::ClusterContactInfo> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterContactInfo.  A related class is
L<NACL::C::ClusterContactInfo|lib-NACL-C-ClusterContactInfo-pm>, which represents access
to an ONTAP ClusterContactInfo.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ClusterContactInfo
element are the attributes of the ClusterContactInfo ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< country >>

Filled in for CMode CLI/ZAPI.

=item C<< business_name >>

Filled in for CMode CLI/ZAPI.

=item C<< zip_code >>

Filled in for CMode CLI/ZAPI.

=item C<< primary_email >>

Filled in for CMode CLI/ZAPI.

=item C<< secondary_name >>

Filled in for CMode CLI/ZAPI.

=item C<< primary_name >>

Filled in for CMode CLI/ZAPI.

=item C<< state >>

Filled in for CMode CLI/ZAPI.

=item C<< primary_phone >>

Filled in for CMode CLI/ZAPI.

=item C<< city >>

Filled in for CMode CLI/ZAPI.

=item C<< secondary_phone >>

Filled in for CMode CLI/ZAPI.

=item C<< primary_alt_phone >>

Filled in for CMode CLI/ZAPI.

=item C<< address >>

Filled in for CMode CLI/ZAPI.

=item C<< secondary_alt_phone >>

Filled in for CMode CLI/ZAPI.

=item C<< secondary_email >>

Filled in for CMode CLI/ZAPI.

=back

=cut

package NACL::CS::ClusterContactInfo;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'country',
    scalar => 'business_name',
    scalar => 'zip_code',
    scalar => 'primary_email',
    scalar => 'secondary_name',
    scalar => 'primary_name',
    scalar => 'state',
    scalar => 'primary_phone',
    scalar => 'city',
    scalar => 'secondary_phone',
    scalar => 'primary_alt_phone',
    scalar => 'address',
    scalar => 'secondary_alt_phone',
    scalar => 'secondary_email',
];

=head1 METHODS

=head2 fetch

    my $ClusterContactInfo_state  = NACL::CS::ClusterContactInfo->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI .

Invokes "cluster-contact-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching cluster contact-info(s) found',
        is_singleton   => 1,
        show_cmd       => 'cluster contact-info show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'cluster_contact_info_show',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        copy => [
            qw(address business-name city country  primary-alt-phone
               primary-email primary-name primary-phone secondary-alt-phone
               secondary-email secondary-name secondary-phone state zip-code)
        ],
        api => "cluster-contact-get",
       primary_keys => [qw()],
        );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
