#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterHa ComponentState Module (auto-generated by CGT)
## @author Akshay Mohit(makshay@netapp.com), dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterHa

=head1 DESCRIPTION

C<NACL::CS::ClusterHa> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterHa.  A related class is
L<NACL::C::ClusterHa|lib-NACL-C-ClusterHa-pm>, which represents access
to an ONTAP ClusterHa.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the ClusterHa
element are the attributes of the ClusterHa ComponentState.

=over

=item C<< "configured" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cluster-ha-info>
     <configured>$value</configured>
 </cluster-ha-info>

=item C<< "sfo_configured" >>

Filled in for CMode CLI/ZAPI.

 <cluster-ha-info>
     <sfo-configured>$value</sfo-configured>
 </cluster-ha-info>

=back

=cut

package NACL::CS::ClusterHa;

use strict;
use warnings;
use Params::Validate qw(validate SCALAR);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'configured',
    scalar => 'sfo_configured',
];

=head1 METHODS

=head2 fetch

    my $ClusterHa_state   = NACL::CS::ClusterHa->fetch(command_interface => $ci, ...);
	my @ClusterHa_states = NACL::CS::ClusterHa->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet..

nvokes "cluster-ha-get" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        exception_text => 'No matching cluster ha(s) found',
        is_singleton   => 1,
        show_cmd       => 'cluster ha show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'cluster_ha_show',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi{
    $Log->enter() if $may_enter;
    my($pkg,@args) =@_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec(allow_extra => 1);
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        api  =>'cluster-ha-get',
        copy =>[qw(sfo-configured)],
        map  =>{'configured' => 'ha-configured'},
        primary_keys =>[qw()],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
