#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::ClusterImagePackage->show_repository() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterImagePackageShowRepository

=head1 DESCRIPTION

C<NACL::CS::ClusterImagePackageShowRepository> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::ClusterImagePackage->show_repository() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::ClusterImagePackage->show_repository() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< download_ver >>

Downloaded Version

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI: package-version

=item C<< component_name >>

Component Name
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $component_name = $obj->component_name();
 
# $component_name contains a reference to the array of values
 
my @component_name = $obj->component_name();
 
# @component_name contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { component_name = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<cluster-image-package-local-info>
<package-details>
<component-name>$value</component-name>
</package-details>
</cluster-image-package-local-info>

=item C<< component_version >>

Component Version
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $component_version = $obj->component_version();
 
# $component_version contains a reference to the array of values
 
my @component_version = $obj->component_version();
 
# @component_version contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { component_version = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<cluster-image-package-local-info>
<package-details>
<component-version>$value</component-version>
</package-details>
</cluster-image-package-local-info>

=back

=cut

package NACL::CS::ClusterImagePackageShowRepository;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'download_ver',
    array  => 'component_name',
    array  => 'component_version',
];

=head1 METHODS

=head2 fetch

    my $ClusterImagePackageShowRepository_state  = NACL::CS::ClusterImagePackageShowRepository->fetch(command_interface => $ci, ...);
	my @ClusterImagePackageShowRepository_states = NACL::CS::ClusterImagePackageShowRepository->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd => 'cluster image package show-repository',
	exception_text => 'No matching cluster image package(s) found',
        frontend => 'NACL::CS::ClusterImagePackageShowRepository::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'cluster_image_package_show_repository',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api => 'cluster-image-package-local-get-iter',
        map => {
            'component-name' => [
                make_zapi_array('package-details'),
                make_zapi_skip('package-details-info'),
                'component-name',
            ],
            'component-version' => [
                make_zapi_array('package-details'),
                make_zapi_skip('package-details-info'),
                'component-version',
            ],
            'download-ver' => 'package-version',
        },
    );

    $Log->exit() if $may_exit;
    return @states;
}

1;
