# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::ClusterImage->show_update_log() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterImageShowUpdateLog

=head1 DESCRIPTION

C<NACL::CS::ClusterImageShowUpdateLog> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::ClusterImage->show_update_log() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::ClusterImage->show_update_log() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< timestamp >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-update-transaction-log-info>
    <start-date-time>$value</start-date-time>
 </cluster-update-transaction-log-info>

=item C<< trans_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-update-transaction-log-info>
    <transaction-name>$value</transaction-name>
 </cluster-update-transaction-log-info>

=item C<< status >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-update-transaction-log-info>
    <transaction-status>$value</transaction-status>
 </cluster-update-transaction-log-info>

=item C<< detailed_status >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-update-transaction-log-info>
    <detailed-transaction-status>$value</detailed-transaction-status>
 </cluster-update-transaction-log-info>

=item C<< componentid >>

Filled in for CMode CLI.

=item C<< trans_id >>

Filled in for CMode CLI.

=item C<< component_id >>

*Component Id

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <cluster-update-transaction-log-info>
    <component-id>$value</component-id>
 </cluster-update-transaction-log-info>

=item C<< phase >>

*Transaction phase

Filled in for CMode CLI.

=item C<< error >>

*Error Description

Filled in for CMode CLI.

=back

=cut

package NACL::CS::ClusterImageShowUpdateLog;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'timestamp',
    scalar => 'trans_name',
    scalar => 'status',
    scalar => 'detailed_status',
    scalar => 'componentid',
    scalar => 'trans_id',
    scalar => 'component_id',
    scalar => 'phase',
    scalar => 'error',
];

=head1 METHODS

=head2 fetch

    my $ClusterImageShowUpdateLog_state  = NACL::CS::ClusterImageShowUpdateLog->fetch(command_interface => $ci, ...);
    my @ClusterImageShowUpdateLog_states = NACL::CS::ClusterImageShowUpdateLog->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "cluster-image-update-log-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
       show_cmd       => 'cluster image show-update-log',
       exception_text => 'No matching cluster image(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api => 'cluster_image_show_update_log',

    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => 'cluster_image_update_log_get_iter',
        map => {
            'detailed_status' => 'detailed-transaction-status',
            'status'          => 'transaction-status',
            'trans_id'        => 'transaction-id',
            'trans_name'      => 'transaction-name',
            'timestamp'       => 'start-date-time',
        },
        copy => [qw ( component-id )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
