# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterPeerAddressStable ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterPeerAddressStable

=head1 DESCRIPTION

C<NACL::CS::ClusterPeerAddressStable> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterPeerAddressStable.  A related class is
L<NACL::C::ClusterPeerAddressStable|lib-NACL-C-ClusterPeerAddressStable-pm>, which represents access
to an ONTAP ClusterPeerAddressStable.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ClusterPeerAddressStable
element are the attributes of the ClusterPeerAddressStable ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< cluster >>

Cluster Name
Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: cluster-name


=item C<< addrs >>

Stable Peer IP Addresses
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $addrs = $obj->addrs();
 
# $addrs contains a reference to the array of values
 
my @addrs = $obj->addrs();
 
# @addrs contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { addrs = [ value1, value2...] }

Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: <peer-addresses remote-inet-address="@values" />

=back

=cut

package NACL::CS::ClusterPeerAddressStable;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cluster',
    array  => 'addrs',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $ClusterPeerAddressStable_state  = NACL::CS::ClusterPeerAddressStable->fetch(command_interface => $ci, ...);
	my @ClusterPeerAddressStable_states = NACL::CS::ClusterPeerAddressStable->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes cluster-peer-stable-addresses-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_zapi_check'
            },
        ],
        exception_text => 'No matching cluster peer address stable(s) found',
        frontend       => 'NACL::CS::ClusterPeerAddressStable::fetch',
        show_cmd       => 'cluster peer address stable show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'cluster_peer_address_stable_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        map => {
            'cluster' => 'cluster-name',
            'addrs'   => [
                make_zapi_array('peer-addresses'),
                make_zapi_skip('remote-inet-address'),
            ],
        },
        api          => "cluster-peer-stable-addresses-get",
        primary_keys => [qw( cluster )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => ['cluster']);
    $Log->exit() if $may_exit;
}

1;
