#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ClusterTimeServiceNtpServer ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::ClusterTimeServiceNtpServer

=head1 DESCRIPTION

C<NACL::CS::ClusterTimeServiceNtpServer> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP ClusterTimeServiceNtpServer.  A related class is
L<NACL::C::ClusterTimeServiceNtpServer|lib-NACL-C-ClusterTimeServiceNtpServer-pm>, which represents access
to an ONTAP ClusterTimeServiceNtpServer.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ClusterTimeServiceNtpServer
element are the attributes of the ClusterTimeServiceNtpServer ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< is_preferred >>


Is Preferred NTP Server (default: false) possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

=item C<< version >>

NTP Version For Server (default: auto) possible value(s) are, 3,4,auto

Filled in for CMode CLI/ZAPI

=item C<< server >>

NTP Server Host Name, IPv4 or IPv6 Address

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI : server-name

=item C<< is_public >>

Is Public NTP Server Default possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI : is-public-default

=back

=cut

package NACL::CS::ClusterTimeServiceNtpServer;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::C::ClusterTimeServiceNtpServer;

use Class::MethodMaker [
    scalar => 'is_preferred',
    scalar => 'version',
    scalar => 'server',
    scalar => 'is_public',
];

=head1 METHODS

=head2 fetch

    my $ClusterTimeServiceNtpServer_state  = NACL::CS::ClusterTimeServiceNtpServer->fetch(command_interface => $ci, ...);
	my @ClusterTimeServiceNtpServer_states = NACL::CS::ClusterTimeServiceNtpServer->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes API 'ntp-server-get-iter' for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        show_cmd => 'cluster time-service ntp server show',
	exception_text =>
            'No matching cluster time-service ntp server(s) found',
        frontend => 'NACL::CS::ClusterTimeServiceNtpServer::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'cluster_time_service_ntp_server_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'ntp-server-get-iter',
        copy => [qw( is-preferred version )],
        map  => {server => 'server-name', 'is-public' => 'is-public-default'},
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

=head2 recreate

 $clusterTimeServiceNtpServerObj->recreate();

(Instance method) Request for recreation of ClusterTimeServiceNtpServer object from recently deleted NACL::CS::ClusterTimeServiceNtpServer object

Redirects the call to NACL::C::ClusterTimeServiceNtpServer->create with all required params 

=over

=item options

=over

=item C<< ntpserv_obj >>

Previously deleted NACL::CS::ClusterTimeServiceNtpServer object

=back

=back

=cut

sub recreate {
    $Log->enter() if $may_enter;
    my $self  = shift;
    my $obj = NACL::C::ClusterTimeServiceNtpServer->create(
         'command_interface' => $self->command_interface(),
         'server'            => $self->server(),
         'version'           => $self->version(),
         'is-preferred'      => $self->is_preferred(),
    );
    $Log->exit() if $may_exit;
    return $obj;
} ## end sub recreate

1;
