# $Id: //depot/prod/test/main/lib/NACL/CS/Aggregate.pm $
#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Aggregate ComponentState Module (auto-generated by OCUM CGT)
## @author dl-mpo-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DFM::Aggregate

=head1 DESCRIPTION

C<NACL::CS::DFM::Aggregate> is a derived class of
L<NACL::CS::ComponentState::DFM|lib-NACL-CS-ComponentState-DFM-pm>. It represents the
state of a Aggregate.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Aggregate
element are the attributes of the Aggregate ComponentState.

=over


=item C<< is_hybrid_enabled >>

=item C<< is_hybrid >>

=item C<< cluster_node_resource_key >>

=item C<< aggregate_resource_key >>

=item C<< hybrid_cache_size_total >>

=item C<< cluster_resource_key >>

=item C<< has_partner_root >>

=item C<< compression_space_savings >>

=item C<< cluster_node_name >>

=item C<< dedupe_space_savings >>

=item C<< aggregate_state >>

=item C<< aggr_volume_move_info >>
( Type L<NACL::CS::DFM::AggrVolumeMoveInfo|lib-NACL-CS-DFM-AggrVolumeMoveInfo-pm> )

=item C<< has_local_root >>

=item C<< aggregate_name >>

=item C<< block_type >>

=item C<< raid_type >>

=item C<< cluster_name >>

=item C<< aggregate_space_status >>

=item C<< aggregate_status >>

=item C<< aggregate_size >>
( Type L<NACL::CS::DFM::AggregateSize|lib-NACL-CS-DFM-AggregateSize-pm> )

=item C<< time_to_full >>

=back

=head2 ADDITIONAL FILTER FIELDS

The fetch method also allows the following options to be passed in the
filter even though they are not MethodMaker fields.

=over

=item C<< resource-filter >>

=item C<< max-records >>
=back

=cut 

package NACL::CS::DFM::Aggregate;

use strict;
use warnings;
use Params::Validate qw(validate validate_with SCALAR ARRAYREF HASHREF);
use NACL::ComponentUtils qw(Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base 'NACL::CS::ComponentState::DFM';

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => 'is_hybrid_enabled',
    scalar => 'is_hybrid',
    scalar => 'cluster_node_resource_key',
    scalar => 'aggregate_resource_key',
    scalar => 'hybrid_cache_size_total',
    scalar => 'cluster_resource_key',
    scalar => 'has_partner_root',
    scalar => 'compression_space_savings',
    scalar => 'cluster_node_name',
    scalar => 'dedupe_space_savings',
    scalar => 'aggregate_state',
    scalar => [ { -type => 'NACL::CS::DFM::AggrVolumeMoveInfo' }, 'aggr_volume_move_info' ],
    scalar => 'has_local_root',
    scalar => 'aggregate_name',
    scalar => 'block_type',
    scalar => 'raid_type',
    scalar => 'cluster_name',
    scalar => 'aggregate_space_status',
    scalar => 'aggregate_status',
    scalar => [ { -type => 'NACL::CS::DFM::AggregateSize' }, 'aggregate_size' ],
    scalar => 'time_to_full',
];

=head1 METHODS


=head2 fetch

    my $state = NACL::CS::DFM::Aggregate->fetch(command_interface => $ci, ...);
    my @states = NACL::CS::DFM::Aggregate->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=cut

sub fetch {

    $Log->enter() if $may_enter;
    my $pkg = shift;

    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {
                method    => '_fetch_zapi',
                interface => 'ZAPI',
                set       => 'DFM',
            },
        ],
        exception_text => 'No matching Aggregate(s) found.',
    );

    $Log->exit() if $may_exit;

    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_zapi {

    $Log->enter() if $may_enter;
    my $pkg = shift;

    my @state_objs = $pkg->SUPER::_fetch_dfm_zapi(
        @_,
        api => 'aggregate',
    );

    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _extra_filter_fields {

    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return [
      qw( resource-filter max-records 
      ) ]; 
}

sub _nested_objects {
    return {
        'aggr_volume_move_info' => 'NACL::CS::DFM::AggrVolumeMoveInfo',
        'aggregate_size' => 'NACL::CS::DFM::AggregateSize',
    };
}

1;
