#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DashboardAlarm ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DashboardAlarm

=head1 DESCRIPTION

C<NACL::CS::DashboardAlarm> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP DashboardAlarm.  A related class is
L<NACL::C::DashboardAlarm|lib-NACL-C-DashboardAlarm-pm>, which represents access
to an ONTAP DashboardAlarm.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the DashboardAlarm
element are the attributes of the DashboardAlarm ComponentState.

=over

=item C<< "high_value" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <high-value>$value</high-value>
 </dashboard-alarm-info>

=item C<< "value" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <last-value>$value</last-value>
 </dashboard-alarm-info>

=item C<< "name" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <object-name>$value</object-name>
 </dashboard-alarm-info>

=item C<< "type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <dashboard-metric-type>$value</dashboard-metric-type>
 </dashboard-alarm-info>

=item C<< "ems_state" >>

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <node>$value</node>
 </dashboard-alarm-info>

=item C<< "state" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'requested_fields', 'filter' and Output mapping:

 <dashboard-alarm-info>
    <state>$value</state>
 </dashboard-alarm-info>

=back

=cut

package NACL::CS::DashboardAlarm;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'high_value',
    scalar => 'value',
    scalar => 'name',
    scalar => 'type',
    scalar => 'ems_state',
    scalar => 'node',
    scalar => 'state',
];

=head1 METHODS

=head2 fetch

    my $DashboardAlarm_state   = NACL::CS::DashboardAlarm->fetch(command_interface => $ci, ...);
	my @DashboardAlarm_states = NACL::CS::DashboardAlarm->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "dashboard-alarm-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'dashboard alarm show',
        exception_text => 'No matching dashboard alarm(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'dashboard_alarm_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'dashboard_alarm_get_iter',
        copy => [qw(node state high-value)],
        map  => {
            "name"  => ['object-name'],
            "type"  => ['dashboard-metric-type'],
            "value" => ['last-value'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
