# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::DebugCpsServer->show_history() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugCpsServerShowHistory

=head1 DESCRIPTION

C<NACL::CS::DebugCpsServerShowHistory> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::DebugCpsServer->show_history() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::DebugCpsServer->show_history() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< epoch >>

CPS Storage Area Epoch

Filled in for CMode CLI

=item C<< timestamp >>

Sample Time Stamp

Filled in for CMode CLI

=item C<< volume >>

CPS Volume Name

Filled in for CMode CLI

=item C<< event >>

CPS Storage Event

Filled in for CMode CLI

=item C<< description >>

CPS Storage Event Description

Filled in for CMode CLI

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< history_timestamp >>

Event History Time Stamp

Filled in for CMode CLI.


=item C<< seqnum >>

Sequence Number

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugCpsServerShowHistory;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'epoch',
    scalar => 'timestamp',
    scalar => 'volume',
    scalar => 'event',
    scalar => 'description',
    scalar => 'node',
    scalar => 'history_timestamp',
    scalar => 'seqnum',
];

=head1 METHODS

=head2 fetch

    my $DebugCpsServerShowHistory_state  = NACL::CS::DebugCpsServerShowHistory->fetch(command_interface => $ci, ...);
	my @DebugCpsServerShowHistory_states = NACL::CS::DebugCpsServerShowHistory->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug cps server show-history',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug cps server(s) found',
        frontend       => 'NACL::CS::DebugCpsServerShowHistory::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_cps_server_show_history',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
