# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DebugCrsStream ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugCrsStream

=head1 DESCRIPTION

C<NACL::CS::DebugCrsStream> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP DebugCrsStream.  A related class is
L<NACL::C::DebugCrsStream|lib-NACL-C-DebugCrsStream-pm>, which represents access
to an ONTAP DebugCrsStream.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the DebugCrsStream
element are the attributes of the DebugCrsStream ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< stream_uuid >>

Stream UUID

Filled in for CMode CLI

=item C<< queue_storage_capacity >>

Queue Storage Capacity

Filled in for CMode CLI

=item C<< league_uuid >>

League UUID

Filled in for CMode CLI

=item C<< path_to_persistent_storage >>

Path to Persistent Storage

Filled in for CMode CLI

=item C<< remote_cluster_uuid >>

Remote Cluster UUID

Filled in for CMode CLI

=item C<< admin_state >>

Admin State possible value(s) are, up,down,paused

Filled in for CMode CLI

=item C<< notification_options >>

Notification Options

Filled in for CMode CLI

=item C<< direction >>

Direction possible value(s) are, sender,receiver

Filled in for CMode CLI

=item C<< synopsis >>

Synopsis

Filled in for CMode CLI

=item C<< domain_instance_uuid >>

Domain Instance UUID

Filled in for CMode CLI

=item C<< director_id >>

Director ID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugCrsStream;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'stream_uuid',
    scalar => 'queue_storage_capacity',
    scalar => 'league_uuid',
    scalar => 'path_to_persistent_storage',
    scalar => 'remote_cluster_uuid',
    scalar => 'admin_state',
    scalar => 'notification_options',
    scalar => 'direction',
    scalar => 'synopsis',
    scalar => 'domain_instance_uuid',
    scalar => 'director_id',
];

=head1 METHODS

=head2 fetch

    my $DebugCrsStream_state  = NACL::CS::DebugCrsStream->fetch(command_interface => $ci, ...);
	my @DebugCrsStream_states = NACL::CS::DebugCrsStream->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug crs stream show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug crs stream(s) found',
        frontend       => 'NACL::CS::DebugCrsStream::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_crs_stream_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
