# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::DebugCsmCache->session_table() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::DebugCsmCacheSessionTable

=head1 DESCRIPTION

C<NACL::CS::DebugCsmCacheSessionTable> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::DebugCsmCache->session_table() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::DebugCsmCache->session_table() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< hard_error >>

Hard error possible value(s) are, Hex Integer

Filled in for CMode CLI.

=item C<< uniquifier >>

Session uniquifier possible value(s) are, unsorted HexUInt 64bit

Filled in for CMode CLI.

=item C<< hard_error_time >>

Hard error time (ms) possible value(s) are, integer

Filled in for CMode CLI.

=item C<< conn_start_failures >>

Connection start failures possible value(s) are, integer

Filled in for CMode CLI.

=item C<< node >>

Node name possible value(s) are, <nodename>,local

Filled in for CMode CLI.

=item C<< active_conn_close >>

Active connection closes possible value(s) are, integer

Filled in for CMode CLI.

=item C<< state >>

State of the record possible value(s) are, text

Filled in for CMode CLI.

=item C<< position >>

Uniquifier Position

Filled in for CMode CLI.


=back

=cut

package NACL::CS::DebugCsmCacheSessionTable;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'hard_error',
    scalar => 'uniquifier',
    scalar => 'hard_error_time',
    scalar => 'conn_start_failures',
    scalar => 'node',
    scalar => 'active_conn_close',
    scalar => 'state',
    scalar => 'position',
];

=head1 METHODS

=head2 fetch

    my $DebugCsmCacheSessionTable_state  = NACL::CS::DebugCsmCacheSessionTable->fetch(command_interface => $ci, ...);
	my @DebugCsmCacheSessionTable_states = NACL::CS::DebugCsmCacheSessionTable->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'debug csm cache session-table',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching debug csm cache(s) found',
        frontend       => 'NACL::CS::DebugCsmCacheSessionTable::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'debug_csm_cache_session_table',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
