#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary EhashStats Component Module
## @author jabhijit@netapp.com,dl-metawafl-qa@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::EhashStats

=head1 DESCRIPTION

C<NACL::CS::EhashStats> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the state of metawafl Ehash. A related class is
L<NACL::C::Ehash|lib-NACL-C-Ehash-pm>, which represents
access to metawafl nodescope 'mw teh stats'.

=head1 ATTRIBUTES

=over

=item C<< total >>

Total number of elements present in Ehash.

Filled in for 7Mode Nodescope.

=item C<< inserted >>

Total number of elements inserted to the Ehash

Filled in for 7Mode Nodescope.

=item C<< deleted >>

Total number of elements deleted from the Ehash

Filled in for 7Mode Nodescope.

=item C<< lookup_success >>

Total number of success lookup for keys in Ehash

Filled in for 7Mode Nodescope.

=item C<< lookup_failed >>

Total number of lookup failure in the Ehash

Filled in for 7Mode Nodescope.

=item C<< lock >>

Total number of lock performed on the Ehash

Filled in for 7Mode Nodescope.

=cut

package NACL::CS::EhashStats;

use strict;
use warnings;

use Tharn qw($Log);
use base 'NACL::CS::ComponentState::ONTAP';
use Params::Validate qw(validate);

use Class::MethodMaker [
    scalar => 'total',
    scalar => 'inserted',
    scalar => 'deleted',
    scalar => 'lookup_success',
    scalar => 'lookup_failed',
    scalar => 'lock',
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::EhashStats->fetch(command_interface => $ci,);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
    apiset_method => 'mw_teh_stats',
    filter_keys_to_be_passed_to_apiset => ['aggregate'],
);

1;
