#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobInitstate ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::JobInitstate

=head1 DESCRIPTION

C<NACL::CS::JobInitstate> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP JobInitstate.  A related class is
L<NACL::C::JobInitstate|lib-NACL-C-JobInitstate-pm>, which represents access
to an ONTAP JobInitstate.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the JobInitstate
element are the attributes of the JobInitstate ComponentState.

=over

=item C<< "recovery_enabled" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-is-recovery-enabled>$value</job-mgr-is-recovery-enabled>
 </job-init-state-info>

=item C<< "lp_threads" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-low-priority-threads>$value</job-mgr-low-priority-threads>
 </job-init-state-info>

=item C<< "initmsg" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-init-msg>$value</job-mgr-init-msg>
 </job-init-state-info>

=item C<< "ex_threads" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-excl-priority-threads>$value</job-mgr-excl-priority-threads>
 </job-init-state-info>

=item C<< "siteid" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-site-id>$value</job-mgr-site-id>
 </job-init-state-info>

=item C<< "threadsmsg" >>

Filled in for CMode CLI.

CMode CLI: Accessible through the field "threadsmsg" in the output of command "job_initstate_show"

=item C<< "node" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-node>$value</job-mgr-node>
 </job-init-state-info>

=item C<< "mp_threads" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-med-priority-threads>$value</job-mgr-med-priority-threads>
 </job-init-state-info>

=item C<< "process" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-process>$value</job-mgr-process>
 </job-init-state-info>

=item C<< "initialized" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-is-initialized>$value</job-mgr-is-initialized>
 </job-init-state-info>

=item C<< "cache_root" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-cache-root>$value</job-mgr-cache-root>
 </job-init-state-info>

=item C<< "hp_threads" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-high-priority-threads>$value</job-mgr-high-priority-threads>
 </job-init-state-info>

=item C<< "tx_interval" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-init-state-info>
     <job-mgr-rdb-commit-interval>$value</job-mgr-rdb-commit-interval>
 </job-init-state-info>

=item C<< thread_initmsg >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<job-init-state-info>
     <job-mgr-thread-init-msg>$value</job-mgr-thread-init-msg>
</job-init-state-info>

=back

=cut

package NACL::CS::JobInitstate;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_optional_scalars Dumper);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'recovery_enabled',
    scalar => 'lp_threads',
    scalar => 'initmsg',
    scalar => 'ex_threads',
    scalar => 'siteid',
    scalar => 'threadsmsg',
    scalar => 'node',
    scalar => 'mp_threads',
    scalar => 'process',
    scalar => 'initialized',
    scalar => 'cache_root',
    scalar => 'hp_threads',
    scalar => 'tx_interval',
    scalar => 'thread_initmsg',
];

=head1 METHODS

=head2 fetch

    my $JobInitstate_state   = NACL::CS::JobInitstate->fetch(command_interface => $ci, ...);
	my @JobInitstate_states = NACL::CS::JobInitstate->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "job-init-state-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        %opts,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
         show_cmd       => 'job initstate show',
	 exception_text => 'No matching job initstate(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(%opts, api => 'job_initstate_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api => "job-init-state-get-iter",
        map => {
            "node"             => 'job-mgr-node',
            "process"          => 'job-mgr-process',
            "initialized"      => 'job-mgr-is-initialized',
            "cache-root"       => 'job-mgr-cache-root',
            "siteid"           => 'job-mgr-site-id',
            "hp-threads"       => 'job-mgr-high-priority-threads',
            "mp-threads"       => 'job-mgr-med-priority-threads',
            "lp-threads"       => 'job-mgr-low-priority-threads',
            "tx-interval"      => 'job-mgr-rdb-commit-interval',
            "initmsg"          => 'job-mgr-init-msg',
            "recovery-enabled" => 'job-mgr-is-recovery-enabled',
            "ex-threads"       => 'job-mgr-excl-priority-threads',
            "thread-initmsg"   => 'job-mgr-thread-init-msg',
        },
    );
    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi

1;
