#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary JobScheduleInterval ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::JobScheduleInterval

=head1 DESCRIPTION

C<NACL::CS::JobScheduleInterval> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP JobScheduleInterval.  A related class is
L<NACL::C::JobScheduleInterval|lib-NACL-C-JobScheduleInterval-pm>, which represents access
to an ONTAP JobScheduleInterval.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the JobScheduleInterval
element are the attributes of the JobScheduleInterval ComponentState.

=over

=item C<< "seconds" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-interval-seconds>$value</job-schedule-interval-seconds>
 </job-schedule-interval-info>

=item C<< "hours" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-interval-hours>$value</job-schedule-interval-hours>
 </job-schedule-interval-info>

=item C<< "minutes" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-interval-minutes>$value</job-schedule-interval-minutes>
 </job-schedule-interval-info>

=item C<< "name" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-name>$value</job-schedule-name>
 </job-schedule-interval-info>

=item C<< "description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-description>$value</job-schedule-description>
 </job-schedule-interval-info>

=item C<< "days" >>

lled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <job-schedule-interval-info>
     <job-schedule-interval-days>$value</job-schedule-interval-days>
 </job-schedule-interval-info>

=back

=cut

package NACL::CS::JobScheduleInterval;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(_optional_scalars Dumper);
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'seconds',
    scalar => 'hours',
    scalar => 'minutes',
    scalar => 'name',
    scalar => 'description',
    scalar => 'days',
];

=head1 METHODS

=head2 fetch

    my $JobScheduleInterval_state   = NACL::CS::JobScheduleInterval->fetch(command_interface => $ci, ...);
	my @JobScheduleInterval_states = NACL::CS::JobScheduleInterval->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "job-schedule-interval-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    show_cmd       => 'job schedule interval show',
    exception_text => 'No matching job schedule interval(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'job_schedule_interval_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => "job_schedule_interval_get_iter",
        map => {
            "name"        => ['job-schedule-name'],
            "days"        => ['job-schedule-interval-days'],
            "hours"       => ['job-schedule-interval-hours'],
            "minutes"     => ['job-schedule-interval-minutes'],
            "seconds"     => ['job-schedule-interval-seconds'],
            "description" => ['job-schedule-description'],
        },
    );
    $Log->exit() if $may_exit;
    return @states;
} ## end sub _fetch_cmode_zapi

1;
