
#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary LunImport ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::LunImport

=head1 DESCRIPTION

C<NACL::CS::LunImport> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP LunImport.  A related class is
L<NACL::C::LunImport|lib-NACL-C-LunImport-pm>, which represents access
to an ONTAP LunImport.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the LunImport
element are the attributes of the LunImport ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< import_current_node_id >>

Import Current Node Id

Filled in for CMode CLI.

=item C<< total_blocks >>

Total Blocks

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <total-blocks>$value</total-blocks>
</lun-import-info>

=item C<< compared_blocks >>

Blocks Compared

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <compared-blocks>$value</compared-blocks>
</lun-import-info>

=item C<< foreign_disk >>

Foreign Disk Serial Number

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <foreign-disk>$value</foreign-disk>
</lun-import-info>

=item C<< operation_in_progress >>

Operation in Progress possible value(s) are, import,verify

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <operation-in-progress>$value</operation-in-progress>
</lun-import-info>

=item C<< foreign_disk_uid >>

Foreign Disk UID

Filled in for CMode CLI

=item C<< imported_blocks >>

Blocks Imported

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <imported-blocks>$value</imported-blocks>
</lun-import-info>

=item C<< import_home_node >>

Import Home Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< import_current_node >>

Import Current Node possible value(s) are, <nodename>,local

Filled in for CMode CLI

=item C<< percent_complete >>

Percent Complete

Filled in for CMode CLI

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <vserver>$value</vserver>
</lun-import-info>

=item C<< vs_uuid >>

Vserver UUID

Filled in for CMode CLI

=item C<< admin_state >>

Admin State possible value(s) are, stopped,started,paused

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <administrative-status>$value</administrative-status>
</lun-import-info>

=item C<< path >>

LUN Path

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <path>$value</path>
</lun-import-info>

=item C<< lun_uuid >>

LUN UUID

Filled in for CMode CLI

=item C<< lun_vdisk_id >>

LUN vdisk ID

Filled in for CMode CLI

=item C<< operational_state >>

Operational State possible value(s) are, in_progress,failed,completed,stopped,paused

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:
<lun-import-info>
    <operational-status>$value</operational-status>
</lun-import-info>

=back

=cut

package NACL::CS::LunImport;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'import_current_node_id',
    scalar => 'total_blocks',
    scalar => 'compared_blocks',
    scalar => 'foreign_disk',
    scalar => 'operation_in_progress',
    scalar => 'foreign_disk_uid',
    scalar => 'imported_blocks',
    scalar => 'import_home_node',
    scalar => 'import_current_node',
    scalar => 'percent_complete',
    scalar => 'vserver',
    scalar => 'vs_uuid',
    scalar => 'admin_state',
    scalar => 'path',
    scalar => 'lun_uuid',
    scalar => 'lun_vdisk_id',
    scalar => 'operational_state',
];

=head1 METHODS

=head2 fetch

    my $LunImport_state  = NACL::CS::LunImport->fetch(command_interface => $ci, ...);
	my @LunImport_states = NACL::CS::LunImport->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI APISet.

Invokes "lun-import-get-iter" api for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching lun import(s) found',
        frontend       => 'NACL::CS::LunImport::fetch',
        show_cmd       => 'lun import show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'lun_import_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'lun_import_get_iter',
        copy => [
            qw(percent-complete imported-blocks compared-blocks
                total-blocks vserver path foreign-disk import-home-node
                import-current-node operation-in-progress percent-complete)
        ],
        map => {
            'operational-state' => "operational-status",
            'admin-state'       => "administrative-status",
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
