#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterCheck ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterCheck

=head1 DESCRIPTION

C<NACL::CS::MetroclusterCheck> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterCheck.  A related class is
L<NACL::C::MetroclusterCheck|lib-NACL-C-MetroclusterCheck-pm>, which represents access
to an ONTAP MetroclusterCheck.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterCheck
element are the attributes of the MetroclusterCheck ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< component >>

Name of the component

Filled in for CMode CLI/ZAPI

=item C<< additional_info >>

Additional information/Recovery steps

Filled in for CMode CLI/ZAPI

=item C<< result >>

Result of the check

Filled in for CMode CLI/ZAPI

=item C<< clusterID >>

Cluster the Result Was Added By

Filled in for CMode CLI.

=item C<< timestamp >>

Time of Check

Filled in for CMode CLI.


=back

=cut

package NACL::CS::MetroclusterCheck;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'component',
    scalar => 'additional_info',
    scalar => 'result',
    scalar => 'clusterID',
    scalar => 'timestamp',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterCheck_state  = NACL::CS::MetroclusterCheck->fetch(command_interface => $ci, ...);
	my @MetroclusterCheck_states = NACL::CS::MetroclusterCheck->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "metrocluster-check-get-iter" for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching metrocluster check(s) found',
        frontend       => 'NACL::CS::MetroclusterCheck::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_check_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @opts,
        copy => [qw(component result additional-info)],
        api  => 'metrocluster-check-get-iter',
    );
    $Log->exit() if $may_exit;
    return @states;
}

1;
