# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterCheckLif ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterCheckLif

=head1 DESCRIPTION

C<NACL::CS::MetroclusterCheckLif> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterCheckLif.  A related class is
L<NACL::C::MetroclusterCheckLif|lib-NACL-C-MetroclusterCheckLif-pm>, which represents access
to an ONTAP MetroclusterCheckLif.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterCheckLif
element are the attributes of the MetroclusterCheckLif ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< cluster >>

Destination Cluster

Filled in for CMode CLI.

=item C<< clusterID >>

Cluster Uuid

Filled in for CMode CLI.

=item C<< vserver >>

sync-source vserver in MetroCluster configuration

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-show-lif-placement-failures sync-source-vserver="$value" />



=item C<< data_protocol >>

Data protocol possible value(s) are, nfs,cifs,iscsi,fcp,fcache,none
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $data_protocol = $obj->data_protocol();
 
# $data_protocol contains a reference to the array of values
 
my @data_protocol = $obj->data_protocol();
 
# @data_protocol contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { data_protocol = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-show-lif-placement-failures>
  <data-protocol string="@values" />
</metrocluster-show-lif-placement-failures>



=item C<< vsname >>

Vserver name

Filled in for CMode CLI.

=item C<< lif >>

Logical Interface Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-show-lif-placement-failures sync-source-vserver-lif="$value" />



=item C<< lif_placement_failure >>

LIF placement Failure Type

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<metrocluster-show-lif-placement-failures lif-placement-failure="$value" />



=back

=cut

package NACL::CS::MetroclusterCheckLif;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cluster',
    scalar => 'clusterID',
    scalar => 'vserver',
    array  => 'data_protocol',
    scalar => 'vsname',
    scalar => 'lif',
    scalar => 'lif_placement_failure',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $MetroclusterCheckLif_state  = NACL::CS::MetroclusterCheckLif->fetch(command_interface => $ci, ...);
	my @MetroclusterCheckLif_states = NACL::CS::MetroclusterCheckLif->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes metrocluster-show-lif-placement-failures-get-iter API for CMode ZAPI.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd       => 'metrocluster check lif show',
        exception_text => 'No matching metrocluster check lif(s) found',
        frontend       => 'NACL::CS::MetroclusterCheckLif::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_check_lif_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(lif-placement-failure
                )
        ],
        map => {
            'vserver' => 'sync-source-vserver',
            'data-protocol' =>
                [make_zapi_array('data-protocol'), make_zapi_skip('string'),],
            'lif' => 'sync-source-vserver-lif',
        },
        api => "metrocluster-show-lif-placement-failures-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
