#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterCheckNode ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterCheckNode

=head1 DESCRIPTION

C<NACL::CS::MetroclusterCheckNode> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterCheckNode.  A related class is
L<NACL::C::MetroclusterCheckNode|lib-NACL-C-MetroclusterCheckNode-pm>, which represents access
to an ONTAP MetroclusterCheckNode.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterCheckNode
element are the attributes of the MetroclusterCheckNode ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< cluster >>

Cluster Name

Filled in for CMode CLI/ZAPI

Maps to :

CM ZAPI:

    <metrocluster-check-node-info>
        <cluster-name>$value</cluster-name>
    </metrocluster-check-node-info>

=item C<< check >>

Description

Filled in for CMode CLI/ZAPI

Maps to :

CM ZAPI:

    <metrocluster-check-node-info>
        <check>$value</check>
    </metrocluster-check-node-info>

=item C<< additional_info >>

Additional information/Recovery steps

Filled in for CMode CLI/ZAPI

Maps to :

CM ZAPI:

    <metrocluster-check-node-info>
        <additional-info>$value</additional-info>
    </metrocluster-check-node-info>

=item C<< node >>

Name of the node

Filled in for CMode CLI/ZAPI

Maps to :

CM ZAPI:

    <metrocluster-check-node-info>
        <node-name>$value</node-name>
    </metrocluster-check-node-info>

=item C<< result >>

Result of the check

Filled in for CMode CLI/ZAPI

Maps to :

CM ZAPI:

    <metrocluster-check-node-info>
        <result>$value</result>
    </metrocluster-check-node-info>

=item C<< clusterID >>

Cluster the Result Was Added By

Filled in for CMode CLI.

=item C<< cluster_uuid >>

Cluster UUID

Filled in for CMode CLI.

=item C<< timestamp >>

Time of Check

Filled in for CMode CLI.


=back

=cut

package NACL::CS::MetroclusterCheckNode;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cluster',
    scalar => 'check',
    scalar => 'additional_info',
    scalar => 'node',
    scalar => 'result',
    scalar => 'clusterID',
    scalar => 'cluster_uuid',
    scalar => 'timestamp',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterCheckNode_state  = NACL::CS::MetroclusterCheckNode->fetch(command_interface => $ci, ...);
	my @MetroclusterCheckNode_states = NACL::CS::MetroclusterCheckNode->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'metrocluster check node show',
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching metrocluster check node(s) found',
        frontend       => 'NACL::CS::MetroclusterCheckNode::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_check_node_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'metrocluster-check-node-get-iter',
        copy => [qw (additional-info check result)],
        map  => {
            node    => 'node-name',
            cluster => 'cluster-name',
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
