#
# Copyright (c) 2011-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterConfigReplication ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterConfigReplication

=head1 DESCRIPTION

C<NACL::CS::MetroclusterConfigReplication> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterConfigReplication.  A related class is
L<NACL::C::MetroclusterConfigReplication|lib-NACL-C-MetroclusterConfigReplication-pm>, which represents access
to an ONTAP MetroclusterConfigReplication.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterConfigReplication
element are the attributes of the MetroclusterConfigReplication ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< abandoned_volumes >>

Abandoned Volumes
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $abandoned_volumes = $obj->abandoned_volumes();
 
# $abandoned_volumes contains a reference to the array of values
 
my @abandoned_volumes = $obj->abandoned_volumes();
 
# @abandoned_volumes contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { abandoned_volumes = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<metrocluster-cr-storage-info>
<abandoned-volumes>
<volume-name>@array</volume-name>
</abandoned-volumes>
</metrocluster-cr-storage-info>

=item C<< current_volume >>

Current Volume

Filled in for CMode CLI/ZAPI

=item C<< auto_recreate >>

Auto-Recreate possible value(s) are, true,false

Filled in for CMode CLI

=item C<< use_mirrored_aggregate >>

Use Mirrored Aggregate possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

=item C<< auto_repair >>

Auto-Repair possible value(s) are, true,false

Filled in for CMode CLI

=item C<< disallowed_aggregates >>

Disallowed Aggregates
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $disallowed_aggregates = $obj->disallowed_aggregates();
 
# $disallowed_aggregates contains a reference to the array of values
 
my @disallowed_aggregates = $obj->disallowed_aggregates();
 
# @disallowed_aggregates contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { disallowed_aggregates = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

<metrocluster-cr-storage-info>
<disallowed-aggregates>
<aggr-name>@array</aggr-name>
</disallowed-aggregates>
</metrocluster-cr-storage-info>

=back

=cut

package NACL::CS::MetroclusterConfigReplication;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'abandoned_volumes',
    scalar => 'current_volume',
    scalar => 'auto_recreate',
    scalar => 'use_mirrored_aggregate',
    scalar => 'auto_repair',
    array  => 'disallowed_aggregates',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterConfigReplication_state  = NACL::CS::MetroclusterConfigReplication->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "metrocluster-config-replication-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text =>
            'No matching metrocluster config-replication(s) found',
        frontend     => 'NACL::CS::MetroclusterConfigReplication::fetch',
        is_singleton => 1,
        show_cmd     => 'metrocluster config-replication show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @args,
        api          => 'metrocluster_config_replication_show',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args,
        $pkg->_fetch_backend_validate_spec(allow_extra => 1);
    my @state_objs;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $response          = $apiset->metrocluster_config_replication_get();
    my $output            = $response->get_parsed_output();
    my $row
        = $output->[0]->{'attributes'}->[0]->{'metrocluster-cr-storage-info'}
        ->[0];
    my $targ = {};

    $pkg->_hash_copy(
        source => $row,
        copy   => [
            qw(use-mirrored-aggregate current-volume abandoned-volumes disallowed-aggregates)
        ],
        target => $targ,
    );
    my $obj = $pkg->new(command_interface => $command_interface,);
    $obj->_set_fields(row => $targ);
    push @state_objs, $obj;
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
