#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroclusterOperationHistory ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::MetroclusterOperationHistory

=head1 DESCRIPTION

C<NACL::CS::MetroclusterOperationHistory> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP MetroclusterOperationHistory.  A related class is
L<NACL::C::MetroclusterOperationHistory|lib-NACL-C-MetroclusterOperationHistory-pm>, which represents access
to an ONTAP MetroclusterOperationHistory.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the MetroclusterOperationHistory
element are the attributes of the MetroclusterOperationHistory ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< end_time >>

End time

Filled in for CMode CLI/ZAPI.

=item C<< operation_uuid >>

Identifier for the operation

Filled in for CMode CLI/ZAPI.

=item C<< error_list >>

Error list for the operation
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $error_list = $obj->error_list();

# $error_list contains a reference to the array of values

my @error_list = $obj->error_list();

# @error_list contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { error_list = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

=item C<< state >>

State of the operation

Filled in for CMode CLI/ZAPI.

=item C<< cluster_name >>

Cluster where the command was issued

Filled in for CMode CLI/ZAPI.

=item C<< start_time >>

Start time

Filled in for CMode CLI/ZAPI.

=item C<< node_name >>

Node where the command was run possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI.

=item C<< operation >>

Name of the operation

Filled in for CMode CLI/ZAPI.

=item C<< cluster_uuid >>

UUID Of The Cluster Where The Command Was Run

Filled in for CMode CLI/ZAPI.


=item C<< cluster >>

Cluster Where the Command Was Run

Filled in for CMode CLI/ZAPI.


=item C<< job_id >>

Identifier for the Job

Filled in for CMode CLI/ZAPI.


=item C<< operation_id >>

ID To Sort Operations in the UI

Filled in for CMode CLI.


=back

=cut

package NACL::CS::MetroclusterOperationHistory;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::ComponentUtils qw (enumerate_month);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'end_time',
    scalar => 'operation_uuid',
    array  => 'error_list',
    scalar => 'state',
    scalar => 'cluster_name',
    scalar => 'start_time',
    scalar => 'node_name',
    scalar => 'operation',
    scalar => 'cluster_uuid',
    scalar => 'cluster',
    scalar => 'job_id',
    scalar => 'operation_id',
];

=head1 METHODS

=head2 fetch

    my $MetroclusterOperationHistory_state  = NACL::CS::MetroclusterOperationHistory->fetch(command_interface => $ci, ...);
    my @MetroclusterOperationHistory_states = NACL::CS::MetroclusterOperationHistory->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI

Invokes "metrocluster-operation-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text =>
            'No matching metrocluster operation history(s) found',
        frontend => 'NACL::CS::MetroclusterOperationHistory::fetch',
        show_cmd => 'metrocluster operation history show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'metrocluster_operation_history_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi{
    $Log->enter() if $may_enter;
    my ($pkg,@args) =@_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec(allow_extra => 1);
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  =>'metrocluster-operation-get-iter',
        copy =>[qw(operation-uuid node-name operation  end-time
                    state  error-list job-id cluster-name start-time)],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _update_state_objs_cmode_zapi {
     $Log->enter() if $may_enter;
     my ($pkg, @args) = @_;
     $pkg->SUPER::_update_state_objs_cmode_zapi(
          @args,
          zapi_field_translations => {timestamp_to_string => [qw(start-time end-time)]},
     );
     $Log->exit() if $may_exit;
}

1;
