# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfaceCheckFailover ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkInterfaceCheckFailover

=head1 DESCRIPTION

C<NACL::CS::NetworkInterfaceCheckFailover> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkInterfaceCheckFailover.  A related class is
L<NACL::C::NetworkInterfaceCheckFailover|lib-NACL-C-NetworkInterfaceCheckFailover-pm>, which represents access
to an ONTAP NetworkInterfaceCheckFailover.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkInterfaceCheckFailover
element are the attributes of the NetworkInterfaceCheckFailover ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< destination_nodes >>

Set Of Nodes Over Capacity
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $destination_nodes = $obj->destination_nodes();
 
# $destination_nodes contains a reference to the array of values
 
my @destination_nodes = $obj->destination_nodes();
 
# @destination_nodes contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { destination_nodes = [ value1, value2...] }

Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: <destination-nodes filer-id="@values" />


=item C<< vserver_name >>

Vserver Name
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< over_amount >>

Amount Capacity Exceeded
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< failover_policy >>


Failover Policy possible value(s) are, system-defined,local-only,sfo-partner-only,ipspace-wide,disabled,broadcast-domain-wide
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< failover_group >>

Failover Group Name
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< lif_name >>

LIF Name
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::NetworkInterfaceCheckFailover;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'destination_nodes',
    scalar => 'vserver_name',
    scalar => 'over_amount',
    scalar => 'failover_policy',
    scalar => 'failover_group',
    scalar => 'lif_name',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts,
        _primary_keys => [qw(destination-nodes vserver-name lif-name)],);
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $NetworkInterfaceCheckFailover_state  = NACL::CS::NetworkInterfaceCheckFailover->fetch(command_interface => $ci, ...);
	my @NetworkInterfaceCheckFailover_states = NACL::CS::NetworkInterfaceCheckFailover->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes net-check-failover API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'network interface check failover show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text =>
            'No matching NetworkInterfaceCheckFailover(s) found',
        frontend => 'NACL::CS::NetworkInterfaceCheckFailover::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'network_interface_check_failover_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(vserver-name over-amount failover-policy failover-group lif-name
                )
        ],
        map => {
            'destination-nodes' => [
                make_zapi_array('destination-nodes'),
                make_zapi_skip('filer-id'),
            ],
        },
        api          => "net-check-failover",
        primary_keys => [qw( vserver-name destination-nodes lif-name )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _extra_filter_fields {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return [qw(source-nodes)];
}
1;
