#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkInterfaceFailoverGroups ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkInterfaceFailoverGroups

=head1 DESCRIPTION

C<NACL::CS::NetworkInterfaceFailoverGroups> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkInterfaceFailoverGroups.  A related class is
L<NACL::C::NetworkInterfaceFailoverGroups|lib-NACL-C-NetworkInterfaceFailoverGroups-pm>, which represents access
to an ONTAP NetworkInterfaceFailoverGroups.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the NetworkInterfaceFailoverGroups
element are the attributes of the NetworkInterfaceFailoverGroups ComponentState.

=over

=item C<< "failover_group" >>

Filled in for CMode CLI/ZAPI.

=item C<< "port" >>

Filled in for CMode CLI.

=item C<< "node" >>

Filled in for CMode CLI.

=item C<< targets >>

Failover Targets
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $targets = $obj->targets();
 
# $targets contains a reference to the array of values
 
my @targets = $obj->targets();
 
# @targets contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { targets = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI/ZAPI.


=item C<< broadcast_domain >>

Broadcast Domain

Filled in for CMode CLI.


=back

=cut

package NACL::CS::NetworkInterfaceFailoverGroups;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'failover_group',
    scalar => [
        {   '-read_cb' => sub {
                my ($self, $val) = @_;
                return () if (!ref $self);
                if ($self->command_interface()->has_uichange(
                        uichange        => 'ipspaces-ms4',
                        throw_exception => 0
                    )
                    )
                {
                    $Log->warn(
                        "The script should be updated to use target instead of port,"
                            . "ports are now a 2 tuple (node:port) and referred to as targets."
                    );
                }
                return $val;
                }
        },
        'port'
    ],
    scalar => 'node',
    array  => 'targets',
    scalar => 'vserver',
    scalar => 'broadcast_domain',
];

=head1 METHODS

=head2 fetch

    my $NetworkInterfaceFailoverGroups_state   = NACL::CS::NetworkInterfaceFailoverGroups->fetch(command_interface => $ci, ...);
	my @NetworkInterfaceFailoverGroups_states = NACL::CS::NetworkInterfaceFailoverGroups->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "net-failover-group-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_validate_spec;

    if ($opts{command_interface}->has_uichange(
            uichange        => 'ipspaces-ms4',
            throw_exception => 0
        )
        )
    {

        # Modify the requested field array if 'port' is passed
        if (grep {/^port$/} @{$opts{requested_fields}}) {
            $Log->warn(
                "The script should be updated to use target instead of port,"
                    . "ports are now a 2 tuple (node:port) and referred to as targets."
            );
        }

        # Modify the filter option if 'port' is part of filter
        if (exists $opts{filter}->{port}) {
            $Log->warn(
                "The script should be updated to use target instead of port,"
                    . "ports are now a 2 tuple (node:port) and referred to as targets."
            );
        }
    }

    my @state_objs = $pkg->SUPER::fetch(
        %opts,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text =>
            'No matching network interface failover groups(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'network_interface_failover_groups_show');

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'net-failover-group-get-iter',
        copy => [qw( failover-group vserver )],
        map  => {
            'targets' => [
                make_zapi_array('targets'),
                make_zapi_skip('net-qualified-port-name'),
            ]
        },
    );

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
