#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NetworkRoute ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::NetworkRoute

=head1 DESCRIPTION

C<NACL::CS::NetworkRoute> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP NetworkRoute.  A related class is
L<NACL::C::NetworkRoute|lib-NACL-C-NetworkRoute-pm>, which represents access
to an ONTAP NetworkRoute.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the NetworkRoute
element are the attributes of the NetworkRoute ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vsid >>

Filled in for CMode CLI.

=item C<< destination >>

Filled in for CMode CLI/ZAPI.

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

=item C<< gateway >>

Filled in for CMode CLI/ZAPI.

=item C<< ipspace >>

Filled in for CMode CLI.

=item C<< metric >>

Filled in for CMode CLI/ZAPI.

=item C<< address_family >>

Filled in for CMode CLI/ZAPI.

=back

=cut

package NACL::CS::NetworkRoute;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vsid',
    scalar => 'destination',
    scalar => 'vserver',
    scalar => 'gateway',
    scalar => 'ipspace',
    scalar => 'metric',
    scalar => 'address_family',
];

=head1 METHODS

=head2 fetch

    my $NetworkRoute_state  = NACL::CS::NetworkRoute->fetch(command_interface => $ci, ...);
	my @NetworkRoute_states = NACL::CS::NetworkRoute->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI
Invokes "net-routes-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @opts,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching network route(s) found',
        show_cmd       => 'network route show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'network_route_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'net-routes-get-iter',
        copy => [qw (address-family destination gateway metric vserver)],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
