# $Id$

#
# Copyright (c) 2001-2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsCache ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::QosSettingsCache

=head1 DESCRIPTION

C<NACL::CS::QosSettingsCache> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP QosSettingsCache.  A related class is
L<NACL::C::QosSettingsCache|lib-NACL-C-QosSettingsCache-pm>, which represents access
to an ONTAP QosSettingsCache.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the QosSettingsCache
element are the attributes of the QosSettingsCache ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< normal >>

Cache Normal Data Blocks possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-normal-data="$value" />

Mapping for CMode ZAPI non-iter: is-normal-data


=item C<< read_cache >>

FP Read Cache Option
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< num_workloads >>

Number Of Workloads With This Policy
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <qos-policy-group-info num-workloads="$value" />


=item C<< write_cache >>

FP Write Cache Option
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< is_hidden >>

Is this policy Hidden? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< default >>

Is Default? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-default="$value" />

Mapping for CMode ZAPI non-iter: is-default


=item C<< is_advanced >>

Available in advanced level? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< meta >>

Cache Metadata possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-meta-data="$value" />

Mapping for CMode ZAPI non-iter: is-meta-data


=item C<< readahead >>

Cache Readahead Blocks possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-readahead-data="$value" />

Mapping for CMode ZAPI non-iter: is-readahead-data


=item C<< randomwrite >>

Cache Random Write Blocks possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-randomwrite-data="$value" />

Mapping for CMode ZAPI non-iter: is-randomwrite-data


=item C<< cache_setting >>

Cache Policy Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info cache-policy-name="$value" />

Mapping for CMode ZAPI non-iter: cache-policy-name


=item C<< lopri >>

Cache Lopri possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info is-lopri-data="$value" />

Mapping for CMode ZAPI non-iter: is-lopri-data


=item C<< class >>


Cache Policy Class possible value(s) are, preset,user-defined,system-defined,autovolume
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <cache-info cache-class="$value" />

Mapping for CMode ZAPI non-iter: cache-class

=back

=cut

package NACL::CS::QosSettingsCache;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'normal',
    scalar => 'read_cache',
    scalar => 'num_workloads',
    scalar => 'write_cache',
    scalar => 'is_hidden',
    scalar => 'default',
    scalar => 'is_advanced',
    scalar => 'meta',
    scalar => 'readahead',
    scalar => 'randomwrite',
    scalar => 'cache_setting',
    scalar => 'lopri',
    scalar => 'class',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(cache-setting)],);
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $QosSettingsCache_state  = NACL::CS::QosSettingsCache->fetch(command_interface => $ci, ...);
	my @QosSettingsCache_states = NACL::CS::QosSettingsCache->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes cache-policy-get-iter API for CMode ZAPI iter.

Invokes cache-policy-get API for CMode ZAPI non-iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'qos settings cache show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text => 'No matching QosSettingsCache(s) found',
        frontend       => 'NACL::CS::QosSettingsCache::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'qos_settings_cache_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(num-workloads
                )
        ],
        map => {
            'normal'        => 'is-normal-data',
            'default'       => 'is-default',
            'randomwrite'   => 'is-randomwrite-data',
            'readahead'     => 'is-readahead-data',
            'meta'          => 'is-meta-data',
            'lopri'         => 'is-lopri-data',
            'cache-setting' => 'cache-policy-name',
            'class'         => 'cache-class',
        },
        api => "cache-policy-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(num-workloads
                )
        ],
        map => {
            'normal'        => 'is-normal-data',
            'default'       => 'is-default',
            'randomwrite'   => 'is-randomwrite-data',
            'readahead'     => 'is-readahead-data',
            'meta'          => 'is-meta-data',
            'lopri'         => 'is-lopri-data',
            'cache-setting' => 'cache-policy-name',
            'class'         => 'cache-class',
        },
        api          => "cache-policy-get",
        primary_keys => [qw( cache-setting )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
