# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsControl ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::QosSettingsControl

=head1 DESCRIPTION

C<NACL::CS::QosSettingsControl> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP QosSettingsControl.  A related class is
L<NACL::C::QosSettingsControl|lib-NACL-C-QosSettingsControl-pm>, which represents access
to an ONTAP QosSettingsControl.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the QosSettingsControl
element are the attributes of the QosSettingsControl ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< enforcement >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <qos-settings-control-info>
    <enforcement>$value</enforcement>
 </qos-settings-control-info>

=item C<< ratebucket_rebalance >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <qos-settings-control-info>
    <ratebucket-rebalance>$value</ratebucket-rebalance>
 </qos-settings-control-info>

=item C<< policy_deactivation >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <qos-settings-control-info>
    <policy-deactivation>$value</policy-deactivation>
 </qos-settings-control-info>

=back

=cut

package NACL::CS::QosSettingsControl;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'enforcement',
    scalar => 'ratebucket_rebalance',
    scalar => 'policy_deactivation',
];

=head1 METHODS

=head2 fetch

    my $QosSettingsControl_state  = NACL::CS::QosSettingsControl->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "qos-settings-control-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching qos settings control(s) found',
        is_singleton   => 1,
        show_cmd       => 'qos settings control show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'qos_settings_control_show',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $qos_response;
    my @state_objs;
    my $cmode_options
        = [qw( enforcement ratebucket-rebalance policy-deactivation )],

        $qos_response = $apiset->qos_settings_control_get();
    my $qos_response_output = $qos_response->get_parsed_output();
    foreach my $row (
        @{  $qos_response_output->[0]{'attributes'}[0]
                {'qos-settings-control-info'}
        }
        )
    {
        my $state_fields = $pkg->_zapi_hash_copy(
            source => $row,
            copy   => $cmode_options,
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface},);
        $obj->_set_fields(row => $state_fields);

        foreach my $option (@$cmode_options) {
            if ($obj->$option() =~ 'true') {
                $obj->$option("on");
            } else {
                $obj->$option("off");
            }
        }

        push @state_objs, $obj;
    } ## end foreach my $row ( @{ $qos_response_output-...

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
