#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsExternalCache ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::QosSettingsExternalCache

=head1 DESCRIPTION

C<NACL::CS::QosSettingsExternalCache> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP QosSettingsExternalCache.  A related class is
L<NACL::C::QosSettingsExternalCache|lib-NACL-C-QosSettingsExternalCache-pm>, which represents access
to an ONTAP QosSettingsExternalCache.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the QosSettingsExternalCache
element are the attributes of the QosSettingsExternalCache ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< normal >>

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <is-normal-data>$value</is-normal-data>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< num_workloads >>

Filled in for CMode CLI

=item C<< default >>

*Is Default?, possible value(s) are true,false

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <is-default>$value</is-default>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< ext_cache_setting_id >>

Filled in for CMode CLI

=item C<< readahead >>

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <is-readahead-data>$value</is-readahead-data>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< external_cache_setting >>

*External Cache Policy Name

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <external-cache-policy-name>$value</external-cache-policy-name>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< meta >>

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <is-meta-data>$value</is-meta-data>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< randomwrite >>

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <is-randomwrite-data>$value</is-randomwrite-data>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< lopri >>

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <lopri>$value</lopri>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=item C<< class >>

*External Cache Policy Class

Maps to:

 CMode ZAPI:

  <external-cache-info>
    <external-cache-class>$value</external-cache-class>
  </external-cache-info>

Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::QosSettingsExternalCache;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::CS::QosSettingsCache;

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'normal',
    scalar => 'num_workloads',
    scalar => 'default',
    scalar => 'ext_cache_setting_id',
    scalar => 'readahead',
    scalar => 'external_cache_setting',
    scalar => 'meta',
    scalar => 'randomwrite',
    scalar => 'lopri',
    scalar => 'class',
];

=head1 METHODS

=head2 fetch

    my $QosSettingsExternalCache_state  = NACL::CS::QosSettingsExternalCache->fetch(command_interface => $ci, ...);
	my @QosSettingsExternalCache_states = NACL::CS::QosSettingsExternalCache->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "external-cache-policy-get-iter" API for CMode ZAPI.
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my %opts = @_;
    if ($opts{command_interface}->version_manager()->is_version_ge(release => '8.3')){
        $opts{filter}{'cache-setting'}=delete $opts{filter}{'external-cache-setting'} if (defined $opts{filter}{'external-cache-setting'});
        if (defined @{$opts{'requested_fields'}}){
            my $array_size = @{$opts{'requested_fields'}};
            my $i;
            for ($i=0;$i<$array_size;$i++){
               if(${$opts{'requested_fields'}}[$i] eq 'external-cache-setting'){
                   ${$opts{'requested_fields'}}[$i] = 'cache-setting';
                   last;
               }
            }
        }


        my @state_objs = NACL::CS::QosSettingsCache->fetch(%opts);
        return wantarray ? @state_objs : $state_objs[0];
    }
    else{

        my @state_objs = $pkg->SUPER::fetch(
            @_,
            show_cmd => 'qos settings external-cache show',
            choices  => [
                {   method    => '_fetch_cmode_cli',
                    interface => 'CLI',
                    set       => 'CMode'
                },
                {   method    => '_fetch_cmode_zapi',
                    interface => 'ZAPI',
                    set       => 'CMode'
                }

            ],
            exception_text => 'No matching qos settings external-cache(s) found',
        );
        $Log->exit() if $may_exit;
        return wantarray ? @state_objs : $state_objs[0];
    }
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'qos_settings_external_cache_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api => 'external-cache-policy-get-iter',
        map => {
            'class'                  => 'external-cache-class',
            'external-cache-setting' => 'external-cache-policy-name',
            'default'                => 'is-default',
            'lopri'                  => 'is-lopri-data',
            'meta'                   => 'is-meta-data',
            'normal'                 => 'is-normal-data',
            'randomwrite'            => 'is-randomwrite-data',
            'readahead'              => 'is-readahead-data'
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
