# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityConfig ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityConfig

=head1 DESCRIPTION

C<NACL::CS::SecurityConfig> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecurityConfig.  A related class is
L<NACL::C::SecurityConfig|lib-NACL-C-SecurityConfig-pm>, which represents access
to an ONTAP SecurityConfig.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SecurityConfig
element are the attributes of the SecurityConfig ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< change_seqnum >>

Change Sequence Number
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< supported_ciphers >>

Supported Ciphers
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< is_fips_enabled >>

FIPS Mode possible value(s) are, true,false
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< interface >>

FIPS Compliant Interface
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< cluster_security_config_ready >>

Cluster Security Configuration Ready possible value(s) are, yes,no
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< supported_protocols >>

Supported Protocols possible value(s) are, TLSv1.2,TLSv1.1,TLSv1,SSLv3
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $supported_protocols = $obj->supported_protocols();
 
# $supported_protocols contains a reference to the array of values
 
my @supported_protocols = $obj->supported_protocols();
 
# @supported_protocols contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { supported_protocols = [ value1, value2...] }

Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: <supported-protocols supported-protocols="@values" />

=back

=cut

package NACL::CS::SecurityConfig;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'change_seqnum',
    scalar => 'supported_ciphers',
    scalar => 'is_fips_enabled',
    scalar => 'interface',
    scalar => 'cluster_security_config_ready',
    array  => 'supported_protocols',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(interface)],);
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $SecurityConfig_state  = NACL::CS::SecurityConfig->fetch(command_interface => $ci, ...);
	my @SecurityConfig_states = NACL::CS::SecurityConfig->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes security-config-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'security config show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text => 'No matching SecurityConfig(s) found',
        frontend       => 'NACL::CS::SecurityConfig::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(
        @args,
        zapi_field_translations => {
            uc             => [qw(interface)],
            yes_no_boolean => [qw(cluster-security-config-ready)],
        },
    );
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'security_config_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(supported-ciphers is-fips-enabled interface cluster-security-config-ready
                )
        ],
        map => {
            'supported-protocols' => [
                make_zapi_array('supported-protocols'),
                make_zapi_skip('supported-protocols'),
            ],
        },
        api          => "security-config-get",
        primary_keys => [qw( interface )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
