# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecurityLoginBanner ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityLoginBanner

=head1 DESCRIPTION

C<NACL::CS::SecurityLoginBanner> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecurityLoginBanner.  A related class is
L<NACL::C::SecurityLoginBanner|lib-NACL-C-SecurityLoginBanner-pm>, which represents access
to an ONTAP SecurityLoginBanner.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SecurityLoginBanner
element are the attributes of the SecurityLoginBanner ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< has_default_values >>

Row has Default Values? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter, non-iter.

=item C<< vserver >>

Vserver Name
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <vserver-login-banner-info vserver="$value" />


=item C<< message >>

Login Banner Message
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <vserver-login-banner-info message="$value" />

=back

=cut

package NACL::CS::SecurityLoginBanner;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'has_default_values',
    scalar => 'vserver',
    scalar => 'message',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $SecurityLoginBanner_state  = NACL::CS::SecurityLoginBanner->fetch(command_interface => $ci, ...);
	my @SecurityLoginBanner_states = NACL::CS::SecurityLoginBanner->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vserver-login-banner-get-iter API for CMode ZAPI iter.

Invokes vserver-login-banner-get API for CMode ZAPI non-iter.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'security login banner show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text => 'No matching SecurityLoginBanner(s) found',
        frontend       => 'NACL::CS::SecurityLoginBanner::fetch',
        is_singleton   => 1
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'security_login_banner_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(vserver message
                )
        ],
        api => "vserver-login-banner-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(vserver message
                )
        ],
        api                 => "vserver-login-banner-get",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw(  )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
