# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SecurityLogin->whoami() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecurityLoginWhoami

=head1 DESCRIPTION

C<NACL::CS::SecurityLoginWhoami> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SecurityLogin->whoami() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SecurityLogin->whoami() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< userid >>

User

Filled in for CMode CLI

=back

=cut

package NACL::CS::SecurityLoginWhoami;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [scalar => 'userid',];

=head1 METHODS

=head2 fetch

    my $SecurityLoginWhoami_state  = NACL::CS::SecurityLoginWhoami->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'security login whoami',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching security login(s) found',
        frontend       => 'NACL::CS::SecurityLoginWhoami::fetch',
        is_singleton   => 1
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @args,
        api          => 'security_login_whoami',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
