# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SecuritySsl ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SecuritySsl

=head1 DESCRIPTION

C<NACL::CS::SecuritySsl> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SecuritySsl.  A related class is
L<NACL::C::SecuritySsl|lib-NACL-C-SecuritySsl-pm>, which represents access
to an ONTAP SecuritySsl.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SecuritySsl
element are the attributes of the SecuritySsl ComponentState.

=over

=item C<< "certificate" >>

Filled in for CMode CLI.

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<vserver>$value</vserver>
 </vserver-ssl-info>

=item C<< "enabled" >>

Filled in for CMode CLI, 7Mode CLI.

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< client_enabled >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<client-authentication-enabled>$value</client-authentication-enabled>
 </vserver-ssl-info>

=item C<< serial >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<certificate-serial-number>$value</certificate-serial-number>
 </vserver-ssl-info>

=item C<< server_enabled >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<server-authentication-enabled>$value</server-authentication-enabled>
 </vserver-ssl-info>

=item C<< common_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<common-name>$value</common-name>
 </vserver-ssl-info>

=item C<< ca >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <vserver-ssl-info>
	<certificate-authority>$value</certificate-authority>
 </vserver-ssl-info>

=item C<< "extended_query" => $string >>

(Optional)
(Applicable only for CMode CLI/ZAPI)

=back

=cut

package NACL::CS::SecuritySsl;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'certificate',
    scalar => 'vserver',
    scalar => 'enabled',
    scalar => 'client_enabled',
    scalar => 'serial',
    scalar => 'server_enabled',
    scalar => 'common_name',
    scalar => 'ca',
    scalar => 'ocsp_max_response_age',
    scalar => 'ocsp_responder_timeout',
    scalar => 'ocsp_use_request_nonce',
    scalar => 'ocsp_max_response_time_skew',
    scalar => 'ocsp_enabled',
    scalar => 'ocsp_override_responder',
    scalar => 'ocsp_default_responder',
];

=head1 METHODS

=head2 fetch

    my $SecuritySsl_state   = NACL::CS::SecuritySsl->fetch(command_interface => $ci, ...);
    my @SecuritySsl_states = NACL::CS::SecuritySsl->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "security-ssl-get-iter" API for CMode ZAPI.

Invokes "secureadmin status" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'security ssl show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => "_fetch_cmode_zapi",
                interface => "ZAPI",
                set       => "CMode",
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching security ssl(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'security_ssl_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $response          = $apiset->secureadmin_status();
    my $output            = $response->get_parsed_output();
    foreach my $list_output (@$output) {

        if ($$list_output{service} =~ /ssl/) {
            my $target = $pkg->_hash_copy(
                source => $list_output,
                map    => {'state' => 'enabled',},
            );
            my $obj = $pkg->new(command_interface => $command_interface);
            $obj->_set_fields(row => $target);
            push @state_objs, $obj;
        } ## end if ( $$list_output{service...
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    $Log->exit() if $may_exit;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api => 'security_ssl_get_iter',
        map => {
            'ca'             => 'certificate-authority',
            'serial'         => 'certificate-serial-number',
            'client-enabled' => 'client-authentication-enabled',
            'server-enabled' => 'server-authentication-enabled',
        },
        copy => [qw (vserver common-name ocsp-max-response-age ocsp-responder-timeout ocsp-use-request-nonce ocsp-max-response-time-skew ocsp-enabled ocsp-override-responder ocsp-default-responder)],
    );
} ## end sub _fetch_cmode_zapi

1;
