# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnaplockClockNode ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SnaplockClockNode

=head1 DESCRIPTION

C<NACL::CS::SnaplockClockNode> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SnaplockClockNode.  A related class is
L<NACL::C::SnaplockClockNode|lib-NACL-C-SnaplockClockNode-pm>, which represents access
to an ONTAP SnaplockClockNode.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SnaplockClockNode
element are the attributes of the SnaplockClockNode ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< system_compliance_clock >>

System Compliance Clock
Filled in for CMode/7Mode CLI

=item C<< filer_id >>

filer ID
Filled in for CMode CLI

=item C<< node >>

Node name possible value(s) are, <nodename>,local
Filled in for CMode/7Mode CLI
=back

=cut

package NACL::CS::SnaplockClockNode;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'system_compliance_clock',
    scalar => 'filer_id',
    scalar => 'node',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SnaplockClockNode_state  = NACL::CS::SnaplockClockNode->fetch(command_interface => $ci, ...);
    my @SnaplockClockNode_states = NACL::CS::SnaplockClockNode->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode/7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
                zapi_type => 'none',
            },

        ],
        exception_text => 'No matching SnaplockClockNode(s) found',
        frontend       => 'NACL::CS::SnaplockClockNode::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'snaplock_clock_node_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts   = validate @args, $pkg->_fetch_backend_validate_spec();
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %api_args;
    if ($pkg->_want_any_field_of(
        requested_fields => $opts{requested_fields},
        filter => $opts{filter},
        fields_filled_by_api => [qw(filer-id)]
        )) {
        $api_args{verbose} = 1;
    }
    my $response = $apiset->snaplock_clock_status(%api_args);
    my $parsed_output = $response->get_parsed_output();
    my $only_row = $parsed_output->[0];
    my $node = $command_interface->node();
    my @state_objs;
    my $obj = $pkg->new(command_interface => $command_interface);
    $obj->node($node);
    $obj->system_compliance_clock($only_row->{system_compliance_clock});
    my $filer_id = $only_row->{node_id};
    $obj->filer_id($filer_id) if (defined $filer_id);
    push @state_objs, $obj;
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
