# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnaplockLog ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SnaplockLog

=head1 DESCRIPTION

C<NACL::CS::SnaplockLog> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SnaplockLog.  A related class is
L<NACL::C::SnaplockLog|lib-NACL-C-SnaplockLog-pm>, which represents access
to an ONTAP SnaplockLog.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SnaplockLog
element are the attributes of the SnaplockLog ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< volume >>

Log Volume Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-config volume="$value" />


=item C<< max_log_size >>

Maximum Size of Log File
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-config max-log-size="$value" />


=item C<< vserver >>

Vserver Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-config vserver="$value" />


=item C<< default_retention_period >>


Default Log Record Retention Period possible value(s) are, {<integer> seconds,minutes,hours,days,months,years} , infinite
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-config default-retention-period="$value" />

=back

=cut

package NACL::CS::SnaplockLog;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'volume',
    scalar => 'max_log_size',
    scalar => 'vserver',
    scalar => 'default_retention_period',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SnaplockLog_state  = NACL::CS::SnaplockLog->fetch(command_interface => $ci, ...);
	my @SnaplockLog_states = NACL::CS::SnaplockLog->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes snaplock-get-vserver-log-iter API for CMode ZAPI iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'snaplock log show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        exception_text => 'No matching SnaplockLog(s) found',
        frontend       => 'NACL::CS::SnaplockLog::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'snaplock_log_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(max-log-size volume vserver default-retention-period
                )
        ],
        api => "snaplock-get-vserver-log-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
