# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnaplockLogFile ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SnaplockLogFile

=head1 DESCRIPTION

C<NACL::CS::SnaplockLogFile> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SnaplockLogFile.  A related class is
L<NACL::C::SnaplockLogFile|lib-NACL-C-SnaplockLogFile-pm>, which represents access
to an ONTAP SnaplockLogFile.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SnaplockLogFile
element are the attributes of the SnaplockLogFile ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< file_path >>

Log File Path
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file file-path="$value" />


=item C<< volume >>

Log Volume Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file volume="$value" />


=item C<< file_size >>

File Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file file-size="$value" />


=item C<< vserver >>

Vserver Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file vserver="$value" />


=item C<< expiry_time >>

Log File Expiry Time
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file expiry-time="$value" />


=item C<< base_name >>


Base Name of Log File possible value(s) are, privileged-delete , system
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <snaplock-vserver-log-file base-name="$value" />

=back

=cut

package NACL::CS::SnaplockLogFile;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'file_path',
    scalar => 'volume',
    scalar => 'file_size',
    scalar => 'vserver',
    scalar => 'expiry_time',
    scalar => 'base_name',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SnaplockLogFile_state  = NACL::CS::SnaplockLogFile->fetch(command_interface => $ci, ...);
	my @SnaplockLogFile_states = NACL::CS::SnaplockLogFile->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes snaplock-get-vserver-active-log-files-iter API for CMode ZAPI iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'snaplock log file show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        exception_text => 'No matching SnaplockLogFile(s) found',
        frontend       => 'NACL::CS::SnaplockLogFile::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations => {hyphenate_value => [qw(base-name)],},);
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'snaplock_log_file_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(file-path file-size volume vserver expiry-time base-name
                )
        ],
        api => "snaplock-get-vserver-active-log-files-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
