# $Id$

#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnapmirrorSnapshotOwner ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @changes : raghaveb@netapp.com Added CMode ZAPI Support.
## @status shared
## @pod here

=head1 NAME

NACL::CS::SnapmirrorSnapshotOwner

=head1 DESCRIPTION

C<NACL::CS::SnapmirrorSnapshotOwner> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SnapmirrorSnapshotOwner.  A related class is
L<NACL::C::SnapmirrorSnapshotOwner|lib-NACL-C-SnapmirrorSnapshotOwner-pm>, which represents access
to an ONTAP SnapmirrorSnapshotOwner.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SnapmirrorSnapshotOwner
element are the attributes of the SnapmirrorSnapshotOwner ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< snaplist >>

Snapshot Copy List
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $snaplist = $obj->snaplist();
# $snaplist contains a reference to the array of values
my @snaplist = $obj->snaplist();
# @snaplist contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { snaplist = [ value1, value2...] }

Filled in for CMode CLI

=item C<< owner_list >>

Snapshot Copy Owner Name List
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $owner_list = $obj->owner_list();
# $owner_list contains a reference to the array of values
my @owner_list = $obj->owner_list();
# @owner_list contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { owner_list = [ value1, value2...] }

Filled in for CMode CLI

=item C<< volume >>

Volume Name

Filled in for CMode CLI

=item C<< vserver >>

Vserver Name

Filled in for CMode CLI

=item C<< snapshot >>

Snapshot Copy Name

Filled in for CMode CLI

=back

=cut

package NACL::CS::SnapmirrorSnapshotOwner;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'snaplist',
    array  => 'owner_list',
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'snapshot',
];

=head1 METHODS

=head2 fetch

    my $SnapmirrorSnapshotOwner_state  = NACL::CS::SnapmirrorSnapshotOwner->fetch(command_interface => $ci, ...);
	my @SnapmirrorSnapshotOwner_states = NACL::CS::SnapmirrorSnapshotOwner->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching snapmirror snapshot-owner(s) found',
        frontend       => 'NACL::CS::SnapmirrorSnapshotOwner::fetch',
        show_cmd       => 'snapmirror snapshot-owner show',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'snapmirror_snapshot_owner_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $command_interface = $opts{command_interface};
    my %copy_filter       = %{$opts{filter}};
    my $response = $apiset->snapmirror_snapshot_owner_get(%copy_filter);
    my $output   = $response->get_parsed_output();
    my $target;
    my $obj;
    my @state_objs;

    foreach my $row (@$output) {
        $target = $pkg->_hash_copy(
            source => $row->{'snapshot-owner-names'}->[0],
            map    => {'snapshot-owner-name' => 'owner-list'},
        );
        $obj = $pkg->new(command_interface => $command_interface,);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    }
    ## executing snapmirror-snapshot-owner-get-snapshots command
    my $snapshot = delete $copy_filter{snapshot};
    $response
        = $apiset->snapmirror_snapshot_owner_get_snapshots(%copy_filter);
    $output = $response->get_parsed_output();
    foreach my $row (@$output) {
        $target = $pkg->_hash_copy(
            source => $row->{'snapshots'}->[0],
            map    => {'snapshot-id' => 'snapshot'},
        );
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    }
    foreach my $state_obj (@state_objs) {
        $state_obj->volume($copy_filter{'volume'});
        $state_obj->vserver($copy_filter{'vserver'});
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
