# $Id: //depot/prod/test/main/lib/NACL/CS/SnapvaultSnap.pm#1 $

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Snapvault ComponentState Module
## @author anupamap@netapp.com
## @status shared
## @pod here
## Supports 7Mode CLI

=head1 NAME

NACL::CS::SnapvaultSnap

=head1 DESCRIPTION

C<NACL::CS::SnapvaultSnap> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the
state of ONTAP Snapvault.  A related class is
L<NACL::C::SnapvaultSnap|lib-NACL-C-Snapvault-pm>, which represents access
to ONTAP snapvault.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SnapvaultSnap
element are the attributes of the SnapvaultSnap ComponentState.

=over

=item C<< primary_location >>

=item C<< secondary_path >>

=item C<< vol_name >>

=item C<< id >>

=item C<< snapname >>

=back

=cut

package NACL::CS::SnapvaultSnap;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'primary_location',
    scalar => 'secondary_path',
    scalar => 'vol_name',
    scalar => 'id',
    scalar => 'snapname',
    scalar => 'destination',

];


=head1 METHODS

=head2 fetch

    my $SnapvaultSnap_state   = NACL::CS::SnapvaultSnap->fetch(command_interface => $ci, ...);
    my @SnapvaultSnap_states = NACL::CS::SnapvaultSnap->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut


sub fetch {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode|Nodescope',
            },
        ],
        exception_text => 'No matching snapvault(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_7mode_cli {
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my %cmd_opts;
    my %copy_filter = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter = $pkg->_remove_relational_regex_filters(filter => \%copy_filter,
                                requested_fields => \@copy_requested_fields);
    my $filter = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    if ($filter->{'vol-name'}) {
        $cmd_opts{'vol-name'} = $filter->{'vol-name'};
    }
    if ($filter->{'snap-name'}) {
        $cmd_opts{'snap-name'} = $filter->{'snap-name'};
    }
    my $response = $apiset->snapvault_snap_preservations(%cmd_opts);
    my $output   = $response->get_parsed_output();
    foreach my $list_output (@$output) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            map    =>  {
                        'snap_id'        => 'id',
                        'snap_name'  => 'snapname',
                        },);
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    }
    return @state_objs;
}

1;
