# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsPreset ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, pkj@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StatisticsPreset

=head1 DESCRIPTION

C<NACL::CS::StatisticsPreset> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StatisticsPreset.  A related class is
L<NACL::C::StatisticsPreset|lib-NACL-C-StatisticsPreset-pm>, which represents access
to an ONTAP StatisticsPreset.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StatisticsPreset
element are the attributes of the StatisticsPreset ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< newname >>

Filled in for CMode CLI.

=item C<< is_read_only >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <is-read-only>$value</is-read-only>
 </preset-info>

=item C<< generation_id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <generation-id>$value</generation-id>
 </preset-info>

=item C<< uuid >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <uuid>$value</uuid>
 </preset-info>

=item C<< is_vserver_visible >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <is-vserver-visible>$value</is-vserver-visible>
 </preset-info>

=item C<< comment >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <comment>$value</comment>
 </preset-info>

=item C<< is_archiver_enabled >>

Filled in for CMode CLI.

=item C<< privilege >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <privilege>$value</privilege>
 </preset-info>

=item C<< archiver_expiration >>

Filled in for CMode CLI.

=item C<< preset >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <preset>$value</preset>
 </preset-info>

=item C<< expiration_end_date >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <expiration-end-date>$value</expiration-end-date>
 </preset-info>

=item C<< is_archive_enabled >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <is-archive-enabled>$value</is-archive-enabled>
 </preset-info>

=item C<< new_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <new-name>$value</new-name>
 </preset-info>

=item C<< expiration_length >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

 <preset-info>
    <expiration-length>$value</expiration-length>
 </preset-info>

=item C<< store >>

Name of Store Where Data is Saved

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StatisticsPreset;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'newname',
    scalar => 'vserver',
    scalar => 'is_read_only',
    scalar => 'generation_id',
    scalar => 'uuid',
    scalar => 'is_vserver_visible',
    scalar => 'comment',
    scalar => 'is_archiver_enabled',
    scalar => 'privilege',
    scalar => 'archiver_expiration',
    scalar => 'preset',
    scalar => 'expiration_end_date',
    scalar => 'is_archive_enabled',
    scalar => 'new_name',
    scalar => 'expiration_length',
    scalar => 'store',
];

=head1 METHODS

=head2 fetch

    my $StatisticsPreset_state  = NACL::CS::StatisticsPreset->fetch(command_interface => $ci, ...);
    my @StatisticsPreset_states = NACL::CS::StatisticsPreset->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "perf-preset-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd       => 'statistics preset show',
        exception_text => 'No matching statistics preset(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'statistics_preset_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'perf_preset_get_iter',
        copy => [
            qw ( expiration-length new-name is-archive-enabled expiration-end-date preset
                uuid generation-id is-read-only vserver expiration-length
                privilege comment is-vserver-visible )
        ],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
