# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsSamples ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StatisticsSamples

=head1 DESCRIPTION

C<NACL::CS::StatisticsSamples> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StatisticsSamples.  A related class is
L<NACL::C::StatisticsSamples|lib-NACL-C-StatisticsSamples-pm>, which represents access
to an ONTAP StatisticsSamples.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StatisticsSamples
element are the attributes of the StatisticsSamples ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< stop_time >>

Stop Time

Filled in for CMode CLI

=item C<< start_time >>

Start Time

Filled in for CMode CLI

=item C<< object >>

List of Objects
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $object = $obj->object();
# $object contains a reference to the array of values
my @object = $obj->object();
# @object contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { object = [ value1, value2...] }
Filled in for CMode CLI

=item C<< sample_id >>

Sample ID

Filled in for CMode CLI

=item C<< counter >>

List of Counters
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $counter = $obj->counter();
# $counter contains a reference to the array of values
my @counter = $obj->counter();
# @counter contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { counter = [ value1, value2...] }

Filled in for CMode CLI

=item C<< instance >>

List of Instances
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $instance = $obj->instance();
# $instance contains a reference to the array of values
my @instance = $obj->instance();
# @instance contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { instance = [ value1, value2...] }

Filled in for CMode CLI

=item C<< status >>

Sample Status

Filled in for CMode CLI.

=item C<< sample_status >>

Filled in for CMode CLI.

=item C<< vserver >>

Vserver

Filled in for CMode CLI.


=item C<< privilege >>

*Privilege Level, possible value(s) are admin,advanced,diagnostic

Filled in for CMode CLI.


=item C<< error_description >>

*Error In Sample Collection

Filled in for CMode CLI.


=item C<< sample_type >>

Sample Type possible value(s) are, User,System

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StatisticsSamples;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'stop_time',
    scalar => 'start_time',
    array  => 'object',
    scalar => 'sample_id',
    array  => 'counter',
    array  => 'instance',
    scalar => 'status',
    scalar => 'sample_status',
    scalar => 'vserver',
    scalar => 'privilege',
    scalar => 'error_description',
    scalar => 'sample_type',
];

=head1 METHODS

=head2 fetch

    my $StatisticsSamples_state  = NACL::CS::StatisticsSamples->fetch(command_interface => $ci, ...);
	my @StatisticsSamples_states = NACL::CS::StatisticsSamples->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI .

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'statistics samples show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching statistics samples(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'statistics_samples_show',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
