# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StatisticsV1 ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StatisticsV1

=head1 DESCRIPTION

C<NACL::CS::StatisticsV1> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StatisticsV1.  A related class is
L<NACL::C::StatisticsV1|lib-NACL-C-StatisticsV1-pm>, which represents access
to an ONTAP StatisticsV1.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StatisticsV1
element are the attributes of the StatisticsV1 ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< aggr_display >>

Filled in for CMode CLI

=item C<< object >>

Object

Filled in for CMode CLI

=item C<< base_value >>

*Base Value

Filled in for CMode CLI

=item C<< denominator >>

*Denominator

Filled in for CMode CLI

=item C<< base_units >>

*Base Units

Filled in for CMode CLI

=item C<< scope_display_desc >>

Filled in for CMode CLI

=item C<< scope_display >>

Filled in for CMode CLI

=item C<< numerators >>

*Numerators
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $numerators = $obj->numerators();
# $numerators contains a reference to the array of values
my @numerators = $obj->numerators();
# @numerators contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { numerators = [ value1, value2...] }

Filled in for CMode CLI

=item C<< node >>

Node

Filled in for CMode CLI

=item C<< countername >>

Filled in for CMode CLI

=item C<< base_delta >>

*Base Delta

Filled in for CMode CLI

=item C<< base_rate >>

*Base Rate (per sec)

Filled in for CMode CLI

=item C<< labels >>

*List of Labels
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $labels = $obj->labels();
# $labels contains a reference to the array of values
my @labels = $obj->labels();
# @labels contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { labels = [ value1, value2...] }

Filled in for CMode CLI

=item C<< valueorlabel >>

*Text Value

Filled in for CMode CLI

=item C<< value >>

Value

Filled in for CMode CLI

=item C<< vserver >>

Vserver

Filled in for CMode CLI

=item C<< numerator >>

*Numerator

Filled in for CMode CLI

=item C<< values >>

*List of Values
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $values = $obj->values();
# $values contains a reference to the array of values
my @values = $obj->values();
# @values contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { values = [ value1, value2...] }

Filled in for CMode CLI

=item C<< description >>

Description

Filled in for CMode CLI

=item C<< instance >>

Instance

Filled in for CMode CLI

=item C<< prop >>

Properties

Filled in for CMode CLI

=item C<< delta >>

Delta

Filled in for CMode CLI

=item C<< objectname >>

Filled in for CMode CLI

=item C<< denominators >>

*Denominators
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $denominators = $obj->denominators();
# $denominators contains a reference to the array of values
my @denominators = $obj->denominators();
# @denominators contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { denominators = [ value1, value2...] }

Filled in for CMode CLI

=item C<< counter >>

Counter

Filled in for CMode CLI

=item C<< instance_uuid >>

Instance UUID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StatisticsV1;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'aggr_display',
    scalar => 'object',
    scalar => 'base_value',
    scalar => 'denominator',
    scalar => 'base_units',
    scalar => 'scope_display_desc',
    scalar => 'scope_display',
    array  => 'numerators',
    scalar => 'node',
    scalar => 'countername',
    scalar => 'base_delta',
    scalar => 'base_rate',
    array  => 'labels',
    scalar => 'valueorlabel',
    scalar => 'value',
    scalar => 'vserver',
    scalar => 'numerator',
    array  => 'values',
    scalar => 'description',
    scalar => 'instance',
    scalar => 'prop',
    scalar => 'delta',
    scalar => 'objectname',
    array  => 'denominators',
    scalar => 'counter',
    scalar => 'instance_uuid',
];

=head1 METHODS

=head2 fetch

    my $StatisticsV1_state  = NACL::CS::StatisticsV1->fetch(command_interface => $ci, ...);
	my @StatisticsV1_states = NACL::CS::StatisticsV1->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI .

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'statistics-v1 show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching statistics-v1(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'statistics_v1_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
