# $Id: $
# This is a prototype and serves as an example.	 It still needs
# documentation.

# Most of the structure is present, but the only data represented
# about an aggregate so far is its name.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateMember ComponentState Module
## @author Chethan.M@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME
 
NACL::CS::StorageAggregateMember
 
=head1 DESCRIPTION
 
C<NACL::CS::> is a derived class of
 
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the state of an Storage Aggregate Member.
A related class is
 
L<NACL::C::StorageAggregateMember|lib-NACL-C-StorageAggregateMember-pm>, which represents access
to an ONTAP Storage Aggregate Member.

 
=head1 ATTRIBUTES
 
The individual pieces of data that are part of the  state of the
 
element are the attributes of the Storage Aggregate Member ComponentState.

=over
 
=item C<< aggregate  >>

Filled in for CMode CLI.

=item C<< uuid >>

Filled in for CMode CLI.

=item C<< size >>

Filled in for CMode CLI.

=item C<< usedsize >>

Filled in for CMode CLI.

=item C<< percent_used >>

Filled in for CMode CLI.

=item C<< availsize >>

Filled in for CMode CLI.

=item C<< state >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< diskcount >>

Filled in for CMode CLI.

=item C<< disklist >>

Filled in for CMode CLI.

=item C<< rawvolcount >>

Filled in for CMode CLI.

=item C<< plexes >>

Filled in for CMode CLI.

=item C<< raidgroups >>

Filled in for CMode CLI.

=item C<< raidtype >>

Filled in for CMode CLI.

=item C<< maxraidsize >>

Filled in for CMode CLI.

=item C<< raidstatus >>

Filled in for CMode CLI.

=item C<< chksumenabled >>

Filled in for CMode CLI.

=item C<< chksumstatus >>

Filled in for CMode CLI.

=item C<< chksumstyle >>

Filled in for CMode CLI.

=item C<< inconsistent >>

Filled in for CMode CLI.

=item C<< ignore_inconsistent >>

Filled in for CMode CLI.

=item C<< raid_block_checksum >>

Filled in for CMode CLI.

=item C<< raid_zoned_checksum >>

Filled in for CMode CLI.

=item C<< snapshot_autodelete >>

Filled in for CMode CLI.

=item C<< thourough_scrub >>

Filled in for CMode CLI.

=item C<< iron_status >>

Filled in for CMode CLI.

=item C<< iron_scan_percentage >>

Filled in for CMode CLI.

=item C<< iron_last_start_errno >>

Filled in for CMode CLI.

=item C<< iron_last_start_error_info >>

Filled in for CMode CLI.

=item C<< type >>

Filled in for CMode CLI.

=item C<< volcount_quiesced >>

Filled in for CMode CLI.

=item C<< volcount_not_online >>

Filled in for CMode CLI.

=item C<< mirror_count_ls_destinations >>

Filled in for CMode CLI.

=item C<< mirror_count_ds_destinations >>

Filled in for CMode CLI.

=item C<< mirror_count_move_destinations >>

Filled in for CMode CLI.

=item C<< mirror_count_dp_qtree_destinations >>

Filled in for CMode CLI.

=item C<< ha_policy >>

Filled in for CMode CLI.

=item C<< block_type >>

Filled in for CMode CLI.

=item C<< disksize >>

Filled in for CMode CLI.

=item C<< resync_snap_time >>

Filled in for CMode CLI.

=item C<< raid_lost_write >>

Filled in for CMode CLI.

=item C<< striped_volcount >>

Filled in for CMode CLI.

=item C<< home_name >>

Filled in for CMode CLI.

=item C<< home_id >>

Filled in for CMode CLI.

=item C<< free_space_realloc >>

Filled in for CMode CLI.

=item C<< auto_snapshots >>

Filled in for CMode CLI.

=item C<< files_used >>

Filled in for CMode CLI.

=item C<< disksize_with_unit >>

Filled in for CMode CLI.

=item C<< owner_id >>

Filled in for CMode CLI.

=item C<< files_total >>

Filled in for CMode CLI.

=item C<< snap_mirrored >>

Filled in for CMode CLI.

=item C<< plex_count >>

Filled in for CMode CLI.

=item C<< striping >>

Filled in for CMode CLI.

=item C<< disksize_in_blks >>

Filled in for CMode CLI.

=item C<< filesys_size_fixed >>

Filled in for CMode CLI.

=item C<< iron_summary_scan_percentage >>

Filled in for CMode CLI.

=item C<< owner_name >>

Filled in for CMode CLI.

=item C<< volcount >>

Filled in for CMode CLI.

=item C<< root >>

Filled in for CMode CLI.

=item C<< fsid >>

Filled in for CMode CLI.

=item C<< raid_lost_write_state >>

Filled in for CMode CLI.

=item C<< collective_volcount >>

Filled in for CMode CLI.

=item C<< mirror_disklist >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $mirror_disklist = $obj->mirror_disklist();
# $mirror_disklist contains a reference to the array of values
my @mirror_disklist = $obj->mirror_disklist();
# @mirror_disklist contains the array of values

Filled in for CMode CLI.

=item C<< ignore_pool >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::StorageAggregateMember;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);

use NACL::Exceptions::NoElementsFound qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => "aggregate",
    scalar => "uuid",
    scalar => "size",
    scalar => "usedsize",
    scalar => "percent_used",
    scalar => "availsize",
    scalar => "state",
    scalar => "node",
    scalar => "diskcount",
    array  => "disklist",
    scalar => "rawvolcount",
    array  => "plexes",
    array  => "raidgroups",
    scalar => "raidtype",
    scalar => "maxraidsize",
    scalar => "raidstatus",
    scalar => "chksumenabled",
    scalar => "chksumstatus",
    scalar => "chksumstyle",
    scalar => "inconsistent",
    scalar => "ignore_inconsistent",
    scalar => "raid_block_checksum",
    scalar => "raid_zoned_checksum",
    scalar => "snapshot_autodelete",
    scalar => "thorough_scrub",
    scalar => "iron_status",
    scalar => "iron_scan_percentage",
    scalar => "iron_last_start_errno",
    scalar => "iron_last_start_error_info",
    scalar => "type",
    scalar => "volcount_quiesced",
    scalar => "volcount_not_online",
    scalar => "mirror_count_ls_destinations",
    scalar => "mirror_count_dp_destinations",
    scalar => "mirror_count_move_destinations",
    scalar => "mirror_count_dp_qtree_destinations",
    scalar => "ha_policy",
    scalar => "block_type",
    scalar => 'disksize',
    scalar => 'resync_snap_time',
    scalar => 'raid_lost_write',
    scalar => 'striped_volcount',
    scalar => 'home_name',
    scalar => 'home_id',
    scalar => 'free_space_realloc',
    scalar => 'auto_snapshots',
    scalar => 'files_used',
    scalar => 'disksize_with_unit',
    scalar => 'owner_id',
    scalar => 'files_total',
    scalar => 'snap_mirrored',
    scalar => 'plex_count',
    scalar => 'striping',
    scalar => 'disksize_in_blks',
    scalar => 'filesys_size_fixed',
    scalar => 'iron_summary_scan_percentage',
    scalar => 'owner_name',
    scalar => 'volcount',
    scalar => 'root',
    scalar => 'fsid',
    scalar => 'raid_lost_write_state',
    scalar => 'collective_volcount',
    array  => 'mirror_disklist',
    scalar => 'ignore_pool',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::AggregateMember'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

    my $node_state  = NACL::CS::StorageAggregateMember->fetch(command_interface=>$ci,...);
    my @node_states = NACL::CS::StorageAggregateMember->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => "_fetch_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        exception_text => 'No matching storage aggregate member(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "storage_aggregate_member_show");
}

1;
