# $Id: //depot/prod/test/nacldev/lib/NACL/CS/StorageAggregateShowScrubStatus.pm#7 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StorageAggregate->show_scrub_status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StorageAggregateShowScrubStatus

=head1 DESCRIPTION

C<NACL::CS::StorageAggregateShowScrubStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StorageAggregate->show_scrub_status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StorageAggregate->show_scrub_status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< raidgroup >>

Filled in for CMode CLI/ZAPI.
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <raid-group>$value<\raid-group>
               <\aggr-scrub-list-info>

=item C<< suspended >>

Filled in for CMode CLI/ZAPI.
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <is-suspended>$value<\is-suspended>
               <\aggr-scrub-list-info>

=item C<< aggregate >>

Filled in for CMode CLI/ZAPI or 7Mode CLI.
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <aggregate>$value<\aggregate>
               <\aggr-scrub-list-info>
      7M CLI : aggregate

=item C<< last_scrub_time >>

Filled in for CMode CLI/ZAPI
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <last-scrub-time>$value<\last-scrub-time>
               <\aggr-scrub-list-info>

=item C<< complete_percentage >>

Filled in for CMode CLI/ZAPI or 7Mode CLI.
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <percentage-complete>$value<\percentage-complete>
               <\aggr-scrub-list-info>
      7M CLI : status

=item C<< plex >>

Filled in for CMode CLI.


=item C<< verbose >>

Filled in for CMode CLI/ZAPI.
  Maps to:
      CM ZAPI: <aggr-scrub-list-info>
               <verbose>$value<\verbose>
               <\aggr-scrub-list-info>


=back

=cut

package NACL::CS::StorageAggregateShowScrubStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::Exceptions::InvalidChoice ();

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'raidgroup',
    scalar => 'suspended',
    scalar => 'aggregate',
    scalar => 'last_scrub_time',
    scalar => 'complete_percentage',
    scalar => 'plex',
    scalar => 'verbose',
];

=head1 METHODS

=head2 fetch

    my $StorageAggregateShowScrubStatus_state  = NACL::CS::StorageAggregateShowScrubStatus->fetch(command_interface => $ci, ...);
	my @StorageAggregateShowScrubStatus_states = NACL::CS::StorageAggregateShowScrubStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet.
  Invokes aggr-scrub-list-info API for CMode ZAPI.
  Invokes "aggr scrub status" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            },
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            }
        ],
        exception_text =>
            'No matching storage aggregate show scrub status(s) found',
        show_cmd       =>
            'storage aggregate show-scrub-status',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'storage_aggregate_show_scrub_status',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();

    my @state_objs;
    my $apiset            = $opts{apiset};
    my $command_interface = $opts{command_interface};

    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my %cmd_opts;

    my $deleted_filter = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $response;
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;

    if ($filter->{aggregate}) {
        $cmd_opts{'aggregate'} = $filter->{'aggregate'};
    }

    $response = $apiset->aggr_scrub_list_info(%cmd_opts);

    my $output = $response->get_parsed_output();

    foreach my $list_output (
        @{$output->[0]{'scrub-details'}[0]{'scrub-detail-info'}})
    {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            map    => {
                'raid-group'           => 'raidgroup',
                'percentage-complete'  => 'complete-percentage',
                'last-scrub-timestamp' => 'last-scrub-time',
                'is-suspended'         => 'suspended',
            }
        );
        my $obj = $pkg->new(command_interface => $command_interface,);
        if (exists $target->{raidgroup}) {
            $target->{raidgroup} =~ /\/(\w+)\//;
            $target->{aggregate} = $1;
        }
        if ($target->{'complete-percentage'}) {
            $target->{'complete-percentage'}
                = $target->{'complete-percentage'} . '%';
        } else {
            $target->{'complete-percentage'} = "-";
        }
        if (!$target->{'last-scrub-time'}) {
            $target->{'last-scrub-time'} = "-";
        }
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my %cmd_opts;
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;

    if ($filter->{aggregate}) {
        $cmd_opts{'aggregate'} = $filter->{'aggregate'};
    }
    my $response = $apiset->aggr_scrub_status(%cmd_opts);
    my $output   = $response->get_parsed_output();
    foreach my $list_output (@$output) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            map    => {
                'name'   => 'aggregate',
                'status' => 'complete_percentage',
            },
        );
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
