# $Id: //depot/prod/test/nacldev/lib/NACL/CS/StorageAggregateWaflironMember.pm#18 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StorageAggregateWaflironMember ComponentState Module
## @author Sumit.Mittal@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME
 
NACL::CS::StorageAggregateWaflironMember
 
=head1 DESCRIPTION
 
C<NACL::CS::StorageAggregateWaflironMember> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the 
state  of a Wafliron Member.  A related class is
L<NACL::C::StorageAggregateWaflironMember|lib-NACL-C-StorageAggregateWaflironMember-pm>, which represents 
access to a ONTAP Wafliron Member.
 
=head1 ATTRIBUTES
 
The individual pieces of data that are part of the  state of the 
element are the attributes of the  ComponentState.
 
=over
 
=item C<< aggregate >>

Filled in for CMode CLI.

=item C<< state >>

Filled in for CMode CLI.

=item C<< iron_status >>

Filled in for CMode CLI.

=item C<< iron_scan_precentage >>

Filled in for CMode CLI.

=item C<< iron_last_start_errno >>
 
Filled in for CMode CLI.

=item C<< iron_last_start_error_info >>

Filled in for CMode CLI.

=item C<< iron_summary_scan_percentage >>

Filled in for CMode CLI.

=back
 
 
=cut

package NACL::CS::StorageAggregateWaflironMember;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);

use NACL::Exceptions::NoElementsFound qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'aggregate',
    scalar => 'state',
    scalar => 'iron_status',
    scalar => 'iron_scan_percentage',
    scalar => 'iron_last_start_errno',
    scalar => 'iron_last_start_error_info',

    scalar => 'iron_summary_scan_percentage',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::AggregateWaflironMember'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

    my $StorageAggregateWaflironMember_state   = NACL::CS::StorageAggregateWaflironMember->fetch(command_interface => $ci, ...);
    my @StorageAggregateWaflironMember_states = NACL::CS::StorageAggregateWaflironMember->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => "_fetch_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
        ],
        exception_text =>
            'No matching storage aggregate wafliron member(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "storage_aggregate_wafliron_member_show");
}

1;
