# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary StoragePool ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StoragePool

=head1 DESCRIPTION

C<NACL::CS::StoragePool> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP StoragePool.  A related class is
L<NACL::C::StoragePool|lib-NACL-C-StoragePool-pm>, which represents access
to an ONTAP StoragePool.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the StoragePool
element are the attributes of the StoragePool ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< allocation_unit_size >>

Allocation Unit Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info allocation-unit-size="$value" />


=item C<< pool_unassigned_size >>

Storage Pool Unassigned Size
Filled in for CMode CLI/ZAPI iter

=item C<< current_operation_job_id >>

Job Id of the currently running operation
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info current-operation-job-id="$value" />


=item C<< pool_total_size >>

Storage Pool Total Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info pool-total-size="$value" />


=item C<< unhealthy_reason >>

Reason for storage pool being unhealthy
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info unhealthy-reason="$value" />


=item C<< selected_allocation_unit_size >>

Allocation Unit Size that Would be Selected for the Specified Input
Filled in for CMode CLI/ZAPI iter

=item C<< allocation_unit_depth >>

Allocation Unit Depth
Filled in for CMode CLI/ZAPI iter

=item C<< is_healthy >>

Is Pool Healthy? possible value(s) are, true,false
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info is-healthy="$value" />


=item C<< disk_count >>

Number of Disks in Storage Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info disk-count="$value" />


=item C<< nodes >>

Nodes Sharing the Storage Pool possible value(s) are, <nodename>,local
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $nodes = $obj->nodes();
 
# $nodes contains a reference to the array of values
 
my @nodes = $obj->nodes();
 
# @nodes contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { nodes = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info>
  <nodes node-name="@values" />
</storage-pool-info>


=item C<< pool_state >>

State of the Storage Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info pool-state="$value" />


=item C<< pool_usable_size >>

Storage Pool Usable Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info pool-usable-size="$value" />


=item C<< storage_type >>


Storage Type possible value(s) are, ATA , BSAS , FCAL , FSAS , LUN , MSATA , SAS  , SATA , SSD , VMDISK
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info storage-type="$value" />


=item C<< storage_pool_uuid >>

Uuid of Storage Pool
Filled in for CMode CLI/ZAPI iter

=item C<< storage_pool >>

Storage Pool Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-info storage-pool="$value" />

=back

=cut

package NACL::CS::StoragePool;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'allocation_unit_size',
    scalar => 'pool_unassigned_size',
    scalar => 'current_operation_job_id',
    scalar => 'pool_total_size',
    scalar => 'unhealthy_reason',
    scalar => 'selected_allocation_unit_size',
    scalar => 'allocation_unit_depth',
    scalar => 'is_healthy',
    scalar => 'disk_count',
    array  => 'nodes',
    scalar => 'pool_state',
    scalar => 'pool_usable_size',
    scalar => 'storage_type',
    scalar => 'storage_pool_uuid',
    scalar => 'storage_pool',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $StoragePool_state  = NACL::CS::StoragePool->fetch(command_interface => $ci, ...);
	my @StoragePool_states = NACL::CS::StoragePool->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-pool-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'storage pool show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        exception_text => 'No matching storage pool(s) found',
        frontend       => 'NACL::CS::StoragePool::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'storage_pool_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(allocation-unit-size current-operation-job-id pool-total-size
                unhealthy-reason is-healthy disk-count storage-type pool-usable-size
                pool-state storage-pool
                )
        ],
        map => {
            'nodes' => [
                make_zapi_array('nodes'), 
                make_zapi_skip('node-name'),
            ],
        },
        api => "storage-pool-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
