# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::StoragePool->show_spare_capacity() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::StoragePoolShowAvailableCapacity

=head1 DESCRIPTION

C<NACL::CS::StoragePoolShowAvailableCapacity> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::StoragePool->show_spare_capacity() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::StoragePool->show_spare_capacity() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< allocation_unit_size >>

Allocation Unit Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info allocation-unit-size="$value" />

=item C<< allocation_unit_count >>

Number of Spare MUA Assigned
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info allocation-unit-count="$value" />

=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info node="$value" />

=item C<< syncmirror_pool_num >>

SyncMirror Pool
Filled in for CMode CLI/ZAPI iter

=item C<< available_size >>

Total Available Spare Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-available-capacity-info available-size="$value" />

=item C<< storage_type >>

Disk Type of Storage Pool possible value(s) are, ATA , BSAS , FCAL , FSAS , LUN , MSATA , SAS  , SATA , SSD , VMDISK
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info storage-type="$value" />

=item C<< syncmirror_pool >>

Syncmirror Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info syncmirror-pool="$value" />

=item C<< storage_pool >>

Name of Storage Pool
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-spare-capacity-info storage-pool="$value" />

=item C<< spare_usable_size >>

Total Available Spare Size
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <storage-pool-available-capacity-info available-size="$value" />

=back

=cut

package NACL::CS::StoragePoolShowAvailableCapacity;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'allocation_unit_size',
    scalar => 'allocation_unit_count',
    scalar => 'node',
    scalar => 'syncmirror_pool_num',
    scalar => 'available_size',
    scalar => 'storage_type',
    scalar => 'syncmirror_pool',
    scalar => 'storage_pool',
    scalar => 'spare_usable_size',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $StoragePoolShowAvailableCapacity_state  = NACL::CS::StoragePoolShowAvailableCapacity->fetch(command_interface => $ci, ...);
	my @StoragePoolShowAvailableCapacity_states = NACL::CS::StoragePoolShowAvailableCapacity->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes storage-pool-available-capacity-get-iter API for CMode ZAPI iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        show_cmd       => 'storage pool show-available-capacity',
        exception_text => 'No matching storage pool(s) found',
        frontend       => 'NACL::CS::StoragePoolShowAvailableCapacity::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'storage_pool_show_available_capacity',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(allocation-unit-size allocation-unit-count node storage-type
               available-size storage-pool syncmirror-pool
                )
        ],
        map => {'spare-usable-size' => 'available-size'},
        api => "storage-pool-available-capacity-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
