#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
#
## @summary PortErrorShow ComponentState Module
## @author gupendra@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here
################################################################################

=head1 NAME

NACL::CS::Switch::PortErrorShow

=head1 DESCRIPTION

C<NACL::CS::Switch::PortErrorShow> is a derived class of
L<NACL::CS::ComponentState::Switch|lib-NACL-CS-ComponentState-Switch-pm>.  It represents the
state of PortErrorShow element.  A related class is
L<NACL::C::Switch::PortErrorShow|lib-NACL-C-Switch-PortErrorShow-pm>, which represents access
to a PortErrorShow element.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the Port element
are the individual attributes of the Port ComponentState

=over

=item C<< 'port_number'=>$string >>

=item C<< 'frames_transmitted'=>$string >>

=item C<< 'signal_loss'=>$string >>

=item C<< 'frames_received'=>$string >>

=item C<< 'fbsy_frames'=>$string >>

=item C<< 'sync_loss'=>$string >>

=item C<< 'c3_frames_disc'=>$string >>

=item C<< 'bad_eof'=>$string >>

=item C<< 'crc_err_eof'=>$string >>

=item C<< 'frjt_frames'=>$string >>

=item C<< 'encoded_errors_out'=>$string >>

=item C<< 'too_short'=>$string >>

=item C<< 'crc_errors'=>$string >>

=item C<< 'link_failures'=>$string >>

=item C<< 'encoded_errors_in'=>$string >>

=back

=cut

###################
# Package
package NACL::CS::Switch::PortErrorShow;

###################
# Everytime
use strict;
use warnings;

###################
# Module includes
use Params::Validate qw (validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw (_dump_one);

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);

use base 'NACL::CS::ComponentState::Switch';

use Data::Dumper;

use Class::MethodMaker [

    scalar => "port_number",
    scalar => "frames_transmitted",
    scalar => "signal_loss",
    scalar => "frames_received",
    scalar => "fbsy_frames",
    scalar => "sync_loss",
    scalar => "c3_frames_disc",
    scalar => "bad_eof",
    scalar => "crc_err_eof",
    scalar => "frjt_frames",
    scalar => "encoded_errors_out",
    scalar => "too_short",
    scalar => "crc_errors",
    scalar => "too_long",
    scalar => "link_failures",
    scalar => "encoded_errors_in",
];

=head1 METHODS

=head2 fetch

    my $port = NACL::CS::Switch::PortErrorShow->fetch(command_interface=>$ci,...);
    my @port = NACL::CS::Switch::PortErrorShow->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>

=cut

sub fetch {
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {
                method    => "_fetch_brocadefc_cli",
                interface => "CLI",
                set       => "BrocadeFC"
            },
        ],
        exception_text => 'No matching elements found'
    );
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_brocadefc_cli {
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $response          = $apiset->porterrshow();
    my $output            = $response->get_parsed_output();
    foreach my $row (@$output) {
        my $obj = $pkg->new( command_interface => $command_interface );
        $obj->_set_fields( row => $row );
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}    #end sub _fetch_brocadefc_cli

1;
