# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
#  without prior written consent from NetApp, Inc.
#

## @summary ExportPolicy ComponentState Module
## @author vikas.p@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SysManager::ExportPolicy

=head1 DESCRIPTION

C<NACL::CS::SysManager::ExportPolicy> is a derived class of
L<NACL::CS::ComponentState::SysManager|lib-NACL-CS-ComponentState-SysManager-pm>.  It represents the  
state of an ONTAP ExportPolicy.  A related class is
L<NACL::C::SysManager::ExportPolicy|lib-NACL-C-SysManager-ExportPolicy-pm>, which represents access
to an ONTAP ExportPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the ExportPolicy 
element are the attributes of the ExportPolicy ComponentState.

=over

=item C<< policy >>

The name of the Export Policy element whose state is being represented.

=item C<< vserver >>

=back

=cut

package NACL::CS::SysManager::ExportPolicy;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::SysManager';

use NACL::C::SysManager::ExportPolicy;
use NACL::Exceptions::NoElementsFound qw(:try);

# Import Exception class
use NACL::APISet::Exceptions::ResponseException qw(:try);

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policy',
];

=head1 METHODS 

=head2 fetch

    my $export_policy_state  = 
		NACL::CS::SysManager::ExportPolicy->fetch(command_interface => $ci,...);
    my @export_policy_states = 
		NACL::CS::SysManager::ExportPolicy->fetch(command_interface => $ci,...);

see L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm>.

Uses a CMode GUI APISet.
(Works only if policy and vserver parameters are passed under filter option).

=cut

####################### GUI Fetch ###############################
sub fetch {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method => "_fetch_cmode_gui",
                set    => "SysManager_CMode",
            },
        ],
        exception_text => 'No matching Export Policy found',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

####################### GUI CMode Fetch ###############################
sub _fetch_cmode_gui {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();

    my $apiset            = $opts{apiset};
    my $requested_fields  = $opts{requested_fields};
    my $filter            = $opts{filter};
    my $command_interface = $opts{command_interface};

    my $export_policy_args = $pkg->_hash_copy(
        source => $filter,
        map    => {
            "policy"  => "-policyName",
            "vserver" => "-vservername",
        }
    );

    my ( $response, $caught_exception, $parsed_output );
    try {

        # caling the gui api to get export policy status
        $response = $apiset->get_export_policy_info(%$export_policy_args);
        $parsed_output = $response->get_parsed_output();
    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {

        # A caught exception indicates that the export policy being looked for
        # does not exist. We catch the exception and return immediately.
        # The 'fetch' frontend decides whether to throw a NoElementsFound
        # exception based on the value of 'allow_empty'
        $Log->debug( 'ResponseException caught. Requested export policy does '
                . 'not exist' );
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my %state_base_field_settings;

    $state_base_field_settings{'policy'}
        = $export_policy_args->{'-policyName'};
    $state_base_field_settings{'vserver'}
        = $export_policy_args->{'-vservername'};

    my %final_attributes = %state_base_field_settings;

    my $obj = $pkg->new( command_interface => $command_interface );
    $obj->_set_fields( row => \%final_attributes );
    $Log->exit() if $may_exit;

    return $obj;
} ## end sub _fetch_cmode_gui

1;
