#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemConfigurationBackupSettings ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemConfigurationBackupSettings

=head1 DESCRIPTION

C<NACL::CS::SystemConfigurationBackupSettings> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemConfigurationBackupSettings.  A related class is
L<NACL::C::SystemConfigurationBackupSettings|lib-NACL-C-SystemConfigurationBackupSettings-pm>, which represents access
to an ONTAP SystemConfigurationBackupSettings.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemConfigurationBackupSettings
element are the attributes of the SystemConfigurationBackupSettings ComponentState.

=over

=item C<< "schedule3" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <job-schedule3>$value</job-schedule3>
 </av-engine-option>

=item C<< "destination" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <destination-url>$value</destination-url>
 </av-engine-option>

=item C<< "numbackups2" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <backup-count2>$value</backup-count2>
 </av-engine-option>

=item C<< "username" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <username-for-destination-url>$value</username-for-destination-url>
 </av-engine-option>

=item C<< "numbackups3" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <backup-count3>$value</backup-count3>
 </av-engine-option>

=item C<< "password" >>

Filled in for CMode CLI.

CMode CLI: Accessible through the field "password" in the output of command "system_configuration_backup_settings_show"

=item C<< "numbackups1" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <backup-count1>$value</backup-count1>
 </av-engine-option>

=item C<< "schedule2" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <job-schedule2>$value</job-schedule2>
 </av-engine-option>

=item C<< "schedule1" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For 'filter':
Applicable, Filtering will be done by Components.

For 'requested_fields',
Not applicable, but the field will be populated in the CS object.

Output mapping:

 <av-engine-option>
    <job-schedule1$value</job-schedule1>
 </av-engine-option>

=item C<< usernameforasup >>

Filled in for CMode CLI.

=item C<< passwordforasup >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemConfigurationBackupSettings;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'schedule3',
    scalar => 'destination',
    scalar => 'numbackups2',
    scalar => 'username',
    scalar => 'numbackups3',
    scalar => 'password',
    scalar => 'numbackups1',
    scalar => 'schedule2',
    scalar => 'schedule1',
    scalar => 'enabled1',
    scalar => 'enabled2',
    scalar => 'enabled3',
    scalar => 'usernameforasup',
    scalar => 'passwordforasup',
];

=head1 METHODS

=head2 fetch

    my $SystemConfigurationBackupSettings_state   = NACL::CS::SystemConfigurationBackupSettings->fetch(command_interface => $ci, ...);
	my @SystemConfigurationBackupSettings_states = NACL::CS::SystemConfigurationBackupSettings->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "config-backup-settings-get" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text =>
            'No matching system configuration backup settings(s) found',
        show_cmd       =>
            'system configuration backup settings show',
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_configuration_backup_settings_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $zapi_map = {
         "destination"   =>  "destination-url",
         "username"      =>  "username-for-destination-url",
         "schedule1"     =>  "job-schedule1",
         "schedule2"     =>  "job-schedule2",
         "numbackups1"   =>  "backup-count1",
         "numbackups2"   =>  "backup-count2",
         "schedule3"     =>  "job-schedule3",
         "numbackups3"   =>  "backup-count3"
    };
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        api          => 'config-backup-settings-get',
        map          => $zapi_map,
        primary_keys => [qw()]
    );
    $Log->exit() if $may_exit;

    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
