# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SystemFeatureUsage->show_history() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemFeatureUsageShowHistory

=head1 DESCRIPTION

C<NACL::CS::SystemFeatureUsageShowHistory> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SystemFeatureUsage->show_history() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SystemFeatureUsage->show_history() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< feature_message >>

Feature Message
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info feature-message="$value" />


=item C<< owner >>

Owner
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info owner="$value" />


=item C<< serial_number >>

Node Serial Number
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info serial-number="$value" />


=item C<< date_collected >>

Collection Date
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info date-collected="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info node="$value" />


=item C<< feature_name >>

Feature Name
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info feature-name="$value" />


=item C<< week_number >>

Week Number
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info week-number="$value" />


=item C<< usage_status >>


Usage Status possible value(s) are, not-used,configured,in-use,not-available
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <usage-state-info usage-status="$value" />

=back

=cut

package NACL::CS::SystemFeatureUsageShowHistory;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'feature_message',
    scalar => 'owner',
    scalar => 'serial_number',
    scalar => 'date_collected',
    scalar => 'node',
    scalar => 'feature_name',
    scalar => 'week_number',
    scalar => 'usage_status',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SystemFeatureUsageShowHistory_state  = NACL::CS::SystemFeatureUsageShowHistory->fetch(command_interface => $ci, ...);
	my @SystemFeatureUsageShowHistory_states = NACL::CS::SystemFeatureUsageShowHistory->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes feature-usage-get-iter API for CMode ZAPI iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        show_cmd => 'system feature-usage show-history',
	exception_text => 'No matching system feature-usage(s) found',
        frontend       => 'NACL::CS::SystemFeatureUsageShowHistory::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_feature_usage_show_history',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(owner feature-message date-collected serial-number week-number feature-name
                node usage-status
                )
        ],
        api => "feature-usage-get-iter",
    );
    foreach my $cs (@state_objs) {
       my $date_collected = $cs->date_collected();
       if (defined $date_collected ) {
           my $time_stamp = scalar gmtime($date_collected);
           if ( $time_stamp =~ /(\w+)\s(\w+)\s+(\d+)\s(\d+\:\d+\:\d+)\s(\d+)/) {
               my $month     = $pkg->enumerate_month(month => $2);
               my $date      = $3;
               my $time      = $4;
               my $year      = $5;
               my $date_time = "$month/$date/$year $time";
               $cs->date_collected($date_time);
           }
       }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
