#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary SystemHealthAlertDefinition ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthAlertDefinition

=head1 DESCRIPTION

C<NACL::CS::SystemHealthAlertDefinition> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthAlertDefinition.  A related class is
L<NACL::C::SystemHealthAlertDefinition|lib-NACL-C-SystemHealthAlertDefinition-pm>, which represents access
to an ONTAP SystemHealthAlertDefinition.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemHealthAlertDefinition
element are the attributes of the SystemHealthAlertDefinition ComponentState.

=over

=item C<< "alert_id" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <alert-id>$value</alert-id>
</diagnosis-alert-definition-info>

=item C<< "perceived_severity" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <perceived-severity>$value</perceived-severity>
</diagnosis-alert-definition-info>

=item C<< "monitor" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <monitor>$value</monitor>
</diagnosis-alert-definition-info>

=item C<< "additional_information" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <additional-information>$value</additional-information>
</diagnosis-alert-definition-info>

=item C<< "subsystem" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <subsystem>$value</subsystem>
</diagnosis-alert-definition-info>

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <node>$value</node>
</diagnosis-alert-definition-info>

=item C<< "alert_type" >>

=item C<< "probable_cause_description" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <probable-cause-description>$value</probable-cause-description>
</diagnosis-alert-definition-info>

=item C<< "probable_cause" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <probable-cause>$value</probable-cause>
</diagnosis-alert-definition-info>

=item C<< "recommended_actions" >>

=item C<< possible_effect >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <possible-effect>$value</possible-effect>
</diagnosis-alert-definition-info>

=item C<< corrective_actions >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-alert-definition-info>
    <corrective-actions>$value</corrective-actions>
</diagnosis-alert-definition-info>

=item C<< process >>

Filled in for CMode CLI.

=item C<< tags >>

Filled in for CMode CLI.

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $tags = $obj->tags();
# $tags contains a reference to the array of values
my @tags = $obj->tags();
# @tags contains the array of values

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemHealthAlertDefinition;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'alert_id',
    scalar => 'perceived_severity',
    scalar => 'monitor',
    scalar => 'additional_information',
    scalar => 'subsystem',
    scalar => 'node',
    scalar => 'alert_type',
    scalar => 'probable_cause_description',
    scalar => 'probable_cause',
    scalar => 'recommended_actions',
    scalar => 'possible_effect',
    scalar => 'corrective_actions',
    scalar => 'process',
    array  => 'tags',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthAlertDefinition_state   = NACL::CS::SystemHealthAlertDefinition->fetch(command_interface => $ci, ...);
	my @SystemHealthAlertDefinition_states = NACL::CS::SystemHealthAlertDefinition->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
	show_cmd       => 'system health alert definition show',
        exception_text =>
            'No matching system health alert definition(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(%opts,
        api => 'system_health_alert_definition_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "diagnosis-alert-definition-get-iter",
        copy => [
            qw(alert-id perceived-severity monitor
                additional-information subsystem node alert-type
                probable-cause-description probable-cause
                recommended-actions corrective-actions possible-effect)
        ],
    );

    foreach my $state_obj (@state_objs) {
        my $monitor = $state_obj->monitor();
        if (defined $monitor) {
            $monitor =~ s/_/-/g;
            $state_obj->monitor($monitor);
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $filter            = delete $opts{filter};
    my $response          = $apiset->system_health_alert_definition_show();
    my $output            = $response->get_parsed_output();
    pop(@$output);

    foreach my $list_output (@{$output}) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw( monitor node subsystem tags)],
            map    => {
                'corrective_actions' => 'corrective-actions',
                'additional_info'    => 'additional-information',
                'severity'           => 'perceived-severity',
                'probable_cause'     => 'probable-cause',
                'alertid'            => 'alert-id',
            },
        );
        my $obj = $pkg->new(command_interface => $command_interface,);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
