# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemHealthClusterSwitchPower ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthClusterSwitchPower

=head1 DESCRIPTION

C<NACL::CS::SystemHealthClusterSwitchPower> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthClusterSwitchPower.  A related class is
L<NACL::C::SystemHealthClusterSwitchPower|lib-NACL-C-SystemHealthClusterSwitchPower-pm>, which represents access
to an ONTAP SystemHealthClusterSwitchPower.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemHealthClusterSwitchPower
element are the attributes of the SystemHealthClusterSwitchPower ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< oper_status >>

Operational Status, possible value(s) are
operational,failed,notoperational,notpresent,unknown

Filled in for CMode CLI

=item C<< status >>

Power Supply Resource Health, possible value(s) are
Ok,Ok_with_suppressed,degraded,unreachable

Filled in for CMode CLI

=item C<< err_descr >>

Error Description

Filled in for CMode CLI

=item C<< monitor >>

Health Monitor, possible value(s) are
node-connect,system-connect,system,controller,chassis,cluster-switch,example

Filled in for CMode CLI

=item C<< unique_name >>

Power Supply Unique Name

Filled in for CMode CLI

=item C<< psu_name >>

Power Supply Name

Filled in for CMode CLI

=item C<< device >>

Switch Name

Filled in for CMode CLI

=item C<< display_name >>

Power Supply Display Name

Filled in for CMode CLI

=item C<< node >>

Node

Filled in for CMode CLI.


=item C<< admin_status >>

Adminstrative Status, possible value(s) are on,off,notdefined,unknown

Filled in for CMode CLI.


=item C<< error_description >>

Error Description

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemHealthClusterSwitchPower;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'oper_status',
    scalar => 'status',
    scalar => 'err_descr',
    scalar => 'monitor',
    scalar => 'unique_name',
    scalar => 'psu_name',
    scalar => 'device',
    scalar => 'display_name',
    scalar => 'node',
    scalar => 'admin_status',
    scalar => 'error_description',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthClusterSwitchPower_state  = NACL::CS::SystemHealthClusterSwitchPower->fetch(command_interface => $ci, ...);
	my @SystemHealthClusterSwitchPower_states = NACL::CS::SystemHealthClusterSwitchPower->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system health cluster-switch power show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text =>
            'No matching system health cluster-switch power(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_health_cluster_switch_power_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
