#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
## @summary SystemHealthConfig ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemHealthConfig

=head1 DESCRIPTION

C<NACL::CS::SystemHealthConfig> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemHealthConfig.  A related class is
L<NACL::C::SystemHealthConfig|lib-NACL-C-SystemHealthConfig-pm>, which represents access
to an ONTAP SystemHealthConfig.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemHealthConfig
element are the attributes of the SystemHealthConfig ComponentState.

=over

=item C<< "aggregator" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <aggregator>$value</aggregator>
</diagnosis-config-info>

=item C<< "resources" >>
(Array)

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <resources>
        <string>$value</string>
    </resources>
</diagnosis-config-info>

=item C<< "pol_version" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <pol-version>$value<pol-version>
</diagnosis-config-info>

=item C<< "monitor" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <monitor>$value<monitor>
</diagnosis-config-info>

=item C<< "health" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <health>$value<health>
</diagnosis-config-info>

=item C<< "subsystem" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <subsystem>
        <hm-subsystem>$value<hm-subsystem>
    </subsystem>
</diagnosis-config-info>

=item C<< "node" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <node>$value<node>
</diagnosis-config-info>

=item C<< "context" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <context>$value<context>
</diagnosis-config-info>

=item C<< "mon_version" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <mon-version>$value<mon-version>
</diagnosis-config-info>

=item C<< "discovery_state" >>

=item C<< "sub_pol_versions" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <sub-pol-versions>$value<sub-pol-versions>
</diagnosis-config-info>

=item C<< process >>

Filled in for CMode CLI.


=item C<< init_state >>

Subsystem Initialization status, possible value(s) are
Invalid,Initailizing,Initialized,Starting_Discovery,Starting_Re-Discovery,Discovery_Done_Partially,Discovery_Done

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "requested_fields", "filter" and Output mapping:

<diagnosis-config-info>
    <init-state>$value<init-state>
</diagnosis-config-info>


=back

=cut

package NACL::CS::SystemHealthConfig;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use Class::MethodMaker [
    scalar => 'aggregator',
    array  => 'resources',
    scalar => 'pol_version',
    scalar => 'monitor',
    scalar => 'health',
    array  => 'subsystem',
    scalar => 'node',
    scalar => 'context',
    scalar => 'mon_version',
    scalar => 'discovery_state',
    scalar => 'sub_pol_versions',
    scalar => 'process',
    scalar => 'init_state',
];

=head1 METHODS

=head2 fetch

    my $SystemHealthConfig_state   = NACL::CS::SystemHealthConfig->fetch(command_interface => $ci, ...);
	my @SystemHealthConfig_states = NACL::CS::SystemHealthConfig->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
	show_cmd       => 'system health config show',
        exception_text => 'No matching system health config(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(%opts,
        api => 'system_health_config_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "diagnosis-config-get-iter",
        copy => [
            qw(aggregator init-state pol-version
                monitor health node context mon-version
                sub-pol-versions)
        ],
        map => {
            'subsystem' => [
                make_zapi_array('subsystem'),
                make_zapi_skip('hm-subsystem'),
            ],
            'resources' =>
                [make_zapi_array('resources'), make_zapi_skip('string'),],
        },
    );

    foreach my $state_obj (@state_objs) {
        for my $state (qw( aggregator monitor health )) {
            my $state_value = $state_obj->$state();
            if (defined $state_value) {
                $state_value =~ s/_/-/g;
                $state_obj->$state($state_value);
            }
        }
    }

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset                = delete $opts{apiset};
    my $command_interface     = delete $opts{command_interface};
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );

    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = $requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;

    my %health_config;
    my @fields
        = qw (  monitor node subsystem health mon-version pol-version context
        aggregator sub-pol-versions resources );
    if (!@$requested_fields) {

        # requested_fields is [], i.e. get all fields
        # Join the array into a comma-separated string
        my $fields = join ',', @fields;
        $health_config{fields} = $fields;
    } else {
        @fields = ();
        foreach my $val (@$requested_fields) {
            if ($val ne "node") {
                push(@fields, $val);
            }
        }

        # requested_fields explicitly provided to be certain fields
        my $fields = join ',', @fields;
        $health_config{fields} = $fields;
    } ## end else [ if ( !@$requested_fields)
    my $response = $apiset->system_health_config_show(%health_config);
    my $output   = $response->get_parsed_output();

    foreach my $list_output (@{$output}) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [
                qw( monitor node subsystem health mon-version pol-version
                    context aggregator sub-pol-versions resources )
            ],
        );
        my $obj = $pkg->new(command_interface => $command_interface,);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
