#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeAutosupportManifest ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeAutosupportManifest

=head1 DESCRIPTION

C<NACL::CS::SystemNodeAutosupportManifest> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemNodeAutosupportManifest.  A related class is
L<NACL::C::SystemNodeAutosupportManifest|lib-NACL-C-SystemNodeAutosupportManifest-pm>, which represents access
to an ONTAP SystemNodeAutosupportManifest.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemNodeAutosupportManifest
element are the attributes of the SystemNodeAutosupportManifest ComponentState.

=over

=item C<< "cmd" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<cmd>$value</cmd>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -cmd (Programmatic name: cmd)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "query" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<query>$value</query>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -query (Programmatic name: query)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "status" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<status>$value</status>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -status  (Programmatic name: status)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "subsys" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<subsystem>$value</subsystem>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -subsys (Programmatic name: subsys)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "body_file" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<body-file>$value</body-file>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -body-file (Programmatic name: body-file)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "node" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<node-name>$value</node-name>
 </autosupport-manifest-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "seq_num" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<seq-num>$value</seq-num>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -seq-num (Programmatic name: seq-num)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "prio_num" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<prio-num>$value</prio-num>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -prio-num (Programmatic name: prio-num)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "content_type" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<content-type>$value</content-type>
 </autosupport-manifest-info>

=item C<< "time_collected" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<time-collected>$value</time-collected>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -time-collected (Programmatic name: time-collected)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "cmd_tgt" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<cmd-tgt>$value</cmd-tgt>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -cmd-tgt (Programmatic name: cmd-tgt)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "error" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<error>$value</error>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -error (Programmatic name: error)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "size_collected" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <autosupport-manifest-info>
	<size-collected>$value</size-collected>
 </autosupport-manifest-info>

7Mode CLI:

For "filter": -size-collected (Programmatic name: size-collected)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=back

=cut

package NACL::CS::SystemNodeAutosupportManifest;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ComponentUtils qw(Dumper);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cmd',
    scalar => 'query',
    scalar => 'status',
    scalar => 'subsys',
    scalar => 'body_file',
    scalar => 'node',
    scalar => 'seq_num',
    scalar => 'prio_num',
    scalar => 'content_type',
    scalar => 'time_collected',
    scalar => 'cmd_tgt',
    scalar => 'error',
    scalar => 'size_collected',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeAutosupportManifest_state   = NACL::CS::SystemNodeAutosupportManifest->fetch(command_interface => $ci, ...);
	my @SystemNodeAutosupportManifest_states = NACL::CS::SystemNodeAutosupportManifest->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "autosupport-manifest-get-iter" API for CMode ZAPI.

Invokes "autosupport manifest show" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
	show_cmd       => 'system node autosupport manifest show',
        exception_text =>
            'No matching system node autosupport manifest(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(%opts,
        api => 'system_node_autosupport_manifest_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg               = shift;
    my %opts              = validate @_, $pkg->_fetch_backend_validate_spec();
    my $command_interface = delete $opts{command_interface};
    my $apiset            = delete $opts{apiset};
    my %copy_filter       = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $filter                = \%copy_filter;
    my $requested_fields      = \@copy_requested_fields;
    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = $requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;

    if ($pkg->_want_any_field_of(
            %{$req_field_filter}, fields_filled_by_api => ['node']
        )
        )
    {
        delete $req_field_filter->{'filter'}->{'node'};
    } ## end if ( $pkg->_want_any_field_of...

    my %manifest_status;
    foreach my $key (keys(%{$opts{filter}})) {
        $manifest_status{$key} = $req_field_filter->{'filter'}->{$key};
    }
    my $response = $apiset->autosupport_manifest_show(%manifest_status);
    my $output   = $response->get_parsed_output();
    my @state_objs;
    my $final_attributes;

    foreach my $row (@$output) {
        $final_attributes = $pkg->_hash_copy(
            source => $row,
            copy   => [
                qw(cmd query status subsys error body_file seq_num prio_num cmd_tgt time_collected size_collected)
            ],
        );
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $final_attributes);

        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    return @state_objs;
} ## end sub _fetch_7mode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "autosupport_manifest_get_iter",
        copy => [
            qw(error status seq-num prio-num cmd-tgt body-file cmd query
                size-collected time-collected content-type )
        ],
        map => {
            'subsys' => 'subsystem',
            'node'   => 'node-name',
        },
    );

    foreach my $state_obj (@state_objs) {
        my $status = $state_obj->status();
        if (defined $status) {
            $status =~ s/_/-/g;
            $state_obj->status($status);
        }
    }

    $Log->exit() if $may_exit;
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
