# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SystemNodeCoredumpSegment->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com , gupendra@netapp.com, raghaveb@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeCoredumpSegmentStatus

=head1 DESCRIPTION

C<NACL::CS::SystemNodeCoredumpSegmentStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SystemNodeCoredumpSegment->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SystemNodeCoredumpSegment->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< percent_completed >>

Filled in for CMode CLI/ZAPI and 7Mode CLI/ZAPI.

=item C<< status >>

Filled in for CMode CLI/ZAPI and 7Mode CLI/ZAPI.

=item C<< core_name >>

Filled in for CMode CLI/ZAPI and 7Mode CLI/ZAPI.

=item C<< job_id >>

Filled in for CMode CLI/ZAPI and 7Mode CLI/ZAPI.

=item C<< node >>

Filled in for CMode CLI/ZAPI and 7Mode CLI/ZAPI.

=item C<< owner_node >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::SystemNodeCoredumpSegmentStatus;

use strict;
use warnings;
use Params::Validate qw(validate validate_pos HASHREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'percent_completed',
    scalar => 'status',
    scalar => 'core_name',
    scalar => 'job_id',
    scalar => 'node',
    scalar => 'owner_node',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeCoredumpSegmentStatus_state  = NACL::CS::SystemNodeCoredumpSegmentStatus->fetch(command_interface => $ci, ...);
    my @SystemNodeCoredumpSegmentStatus_states = NACL::CS::SystemNodeCoredumpSegmentStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet or 7Mode CLI/ZAPI APISet.
Uses core-segment-status-get-iter for CMode ZAPI and 7Mode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut


sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {
                method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {
                method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_zapi',
                interface => 'ZAPI',
                set       => '7Mode'
            },
        ],
	show_cmd       => 'system node coredump segment status',
        exception_text => 'No matching system node coredump segment(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli( @_,
        api => 'system_node_coredump_segment_status', );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $apiset                = $opts{apiset};
    my $command_interface     = $opts{command_interface};
    my %filter = %{$opts{filter}};

    my $delete_cmode_only_options = sub {

        # Not applicable for 7Mode
        my ($hashref) = validate_pos(@_, {type => HASHREF});
        delete $hashref->{node};
        delete $hashref->{'owner-node'};
    };

    $delete_cmode_only_options->(\%filter);

    my %api_args;
    while (my ($key, $value) = each %filter) {
        $api_args{$key} = $value;
    }

    my %requested_fields;
    map { $requested_fields{$_} = 1 } @{$opts{requested_fields}};

    $delete_cmode_only_options->(\%requested_fields);

    my @final_req_field = keys %requested_fields;
    if (@final_req_field) {
        $api_args{fields} = join ',', @final_req_field;
    }

    my ($response, $caught_exception);
    try {
        $response = $apiset->coredump_segment_status(%api_args);
    } catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
        $caught_exception = 1;
    };
    
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my $output = $response->get_parsed_output();

    my @state_objs;
    foreach my $list_output ( @{$output} ) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw(core_name status node owner_node)],
            map    => {
                id      => "job-id",
                percent => "percent-completed",
            },
        );
        my $obj = $pkg->new( command_interface => $command_interface, );
        $obj->_set_fields( row => $target );
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

sub _fetch_7mode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $apiset                = $opts{apiset};
    my $command_interface     = $opts{command_interface};
    my @copy_requested_fields = @{$opts{requested_fields}};

    my (%desired_attributes_hash, %core_options);
    foreach (@copy_requested_fields) {
        $desired_attributes_hash{$_} = 1;
    }
    if (keys %desired_attributes_hash) {
        $core_options{'desired-attributes'} = {%desired_attributes_hash};
    }

    my @state_objs;

    my $response         = $apiset->core_segment_status_get_iter(%core_options);
    my $output           = $response->get_parsed_output();

    foreach my $list_output (
        @{ $output->[0]{'attributes-list'}[0]{'core-segment-status-info'} } )
    {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw(core-name job-id percent-completed status node)],
        );
        my $obj = $pkg->new( command_interface => $command_interface, );
        $obj->_set_fields( row => $target );
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_zapi

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'core-segment-status-get-iter',
        copy => [
            qw( core-name job-id node percent-completed status )
        ]
    );
} ## end sub _fetch_cmode_zapi
1;
