#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemNodeImageShowUpdateProgressNoFollow ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeImageShowUpdateProgressNoFollow

=head1 DESCRIPTION

C<NACL::CS::SystemNodeImageShowUpdateProgressNoFollow> is an object that contains the details
of the image update on a particular node (either in progress or the last one
to have completed). It gets returned when
L<NACL::C::SystemNodeImage->show_update_progress_no_follow|lib-NACL-C-SystemNodeImage-pm/show_update_progress_no_follow>
is invoked. This CS object does not correspond to any ONTAP element, but is
merely an object representation of output.

Uses a CMode CLI or a CMode ZAPI APISet.

=head1 ATTRIBUTES

The fields of output are attributes of the object.

=over

=item C< command_interface >

The command interface with which the component method was
invoked.

=item C< last_message >

The last message related to the update. This should be
"There is no update/install in progress" in case the operation has already
completed.

Filled in for CMode CLI, CMode ZAPI.

CMode CLI: Corresponds to the field "last-message" of the parsed output
of the command "system node image-show-update-progress"
CMode ZAPI: Corresponds to the field "last-message" in the parsed output 
of ZAPI "system-image-update-progress-get".

=item C< exit_message >

The message that the update exited with (generally mentions whether
the operation passed or why it failied if it failed)

This field only gets filled in when the operation has completed.

Filled in for CMode CLI, CMode ZAPI.

CMode CLI: Corresponds to the field "exit-message" of the parsed output
of the command "system node image-show-update-progress"
CMode ZAPI: Corresponds to the field "exit-message" in the parsed output 
of ZAPI "system-image-update-progress-get".

=item C< exit_status >

Indicates whether the update operation passed or failed.

This field only gets filled in when the operation has completed.

Filled in for CMode CLI, CMode ZAPI.

CMode CLI: Corresponds to the field "exit-status" of the parsed output
of the command "system node image-show-update-progress"
CMode ZAPI: Corresponds to the field "exit-status" in the parsed output 
of ZAPI "system-image-update-progress-get".

=item C< phase >

Indicates the current phase of the operation. This will be "Download"
when the operation is still in progress.

This field only gets filled in when the operation has completed.

Filled in for CMode CLI, CMode ZAPI.

CMode CLI: Corresponds to the field "phase" of the parsed output
of the command "system node image-show-update-progress"
CMode ZAPI: Corresponds to the field "phase" in the parsed output 
of ZAPI "system-image-update-progress-get".

=item C< run_status >

Indicates the running status of the operation. It will be "Working"
if the operation is still in progress and will be "Exited" if the
operation has completed.

Filled in for CMode CLI, CMode ZAPI.

CMode CLI: Corresponds to the field "run-status" of the parsed output
of the command "system node image-show-update-progress"
CMode ZAPI: Corresponds to the field "run-status" in the parsed output 
of ZAPI "system-image-update-progress-get".

=back

=cut

package NACL::CS::SystemNodeImageShowUpdateProgressNoFollow;

use strict;
use warnings;
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'exit_message',
    scalar => 'exit_status',
    scalar => 'last_message',
    scalar => 'phase',
    scalar => 'run_status',
];

1;
