#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SystemNode->show_delete_backlog() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeShowDeleteBacklog

=head1 DESCRIPTION

C<NACL::CS::SystemNodeShowDeleteBacklog> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SystemNode->show_delete_backlog() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SystemNode->show_delete_backlog() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< node >>

Node Name possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI

=item C<< available_delete_workers >>

Available Delete Workers

Filled in for CMode CLI/ZAPI

=item C<< pending_delete_work >>

Pending Delete Work

Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::SystemNodeShowDeleteBacklog;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'node',
    scalar => 'available_delete_workers',
    scalar => 'pending_delete_work',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeShowDeleteBacklog_state  = NACL::CS::SystemNodeShowDeleteBacklog->fetch(command_interface => $ci, ...);
	my @SystemNodeShowDeleteBacklog_states = NACL::CS::SystemNodeShowDeleteBacklog->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "system-node-delete-backlog-get" API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching system node(s) found',
        frontend       => 'NACL::CS::SystemNodeShowDeleteBacklog::fetch',
        show_cmd       => 'system node show-delete-backlog',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_node_show_delete_backlog',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $zapi_copy = [qw(available-delete-workers pending-delete-work node)];
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        %opts,
        api          => 'system-node-delete-backlog-get',
        copy         => $zapi_copy,
        primary_keys => [qw(node)]
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
