#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::SystemNode->show_discovered() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemNodeShowDiscovered

=head1 DESCRIPTION

C<NACL::CS::SystemNodeShowDiscovered> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::SystemNode->show_discovered() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::SystemNode->show_discovered() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< nvramid >>

Node NVRAM ID

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<system-node-discovery-get-iter>
 <node-nvram-id>$value</node-nvram-id>
</system-node-discovery-get-iter>

=item C<< serial_number >>

Node Serial Number

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<system-node-discovery-get-iter>
 <node-serial-number>$value</node-serial-number>
</system-node-discovery-get-iter>

=item C<< version >>

Software Version

Filled in for CMode CLI/ZAPI

=item C<< model >>

Model

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<system-node-discovery-get-iter>
 <node-model>$value</node-model>
</system-node-discovery-get-iter>

=item C<< partner_nvramid >>

Partner NVRAM ID

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<system-node-discovery-get-iter>
 <partner-nvram-id>$value</partner-nvram-id>
</system-node-discovery-get-iter>

=item C<< node >>

Node Name

Filled in for CMode CLI/ZAPI

=item C<< is_in_cluster >>

Is in a cluster possible value(s) are, true,false

Filled in for CMode CLI/ZAPI

Maps to:

CMode ZAPI:

<system-node-discovery-get-iter>
 <is-node-in-cluster>$value</is-node-in-cluster>
</system-node-discovery-get-iter>

=item C<< cluster_name >>

Cluster Name

Filled in for CMode CLI/ZAPI

=item C<< cluster_uuid >>

Cluster UUID

Filled in for CMode CLI/ZAPI

=item C<< addresses >>

Cluster IP Addresses
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $addresses = $obj->addresses();

# $addresses contains a reference to the array of values

my @addresses = $obj->addresses();

# @addresses contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { addresses = [ value1, value2...] }

Filled in for CMode CLI

=item C<< netmask >>

Cluster Address Mask

Filled in for CMode CLI/ZAPI

=item C<< addr >>

First address

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemNodeShowDiscovered;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'nvramid',
    scalar => 'serial_number',
    scalar => 'version',
    scalar => 'model',
    scalar => 'partner_nvramid',
    scalar => 'node',
    scalar => 'is_in_cluster',
    scalar => 'cluster_name',
    scalar => 'cluster_uuid',
    array  => 'addresses',
    scalar => 'netmask',
    scalar => 'addr',
];

=head1 METHODS

=head2 fetch

    my $SystemNodeShowDiscovered_state  = NACL::CS::SystemNodeShowDiscovered->fetch(command_interface => $ci, ...);
    my @SystemNodeShowDiscovered_states = NACL::CS::SystemNodeShowDiscovered->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "system-node-discovery-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'system node show-discovered',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching system node(s) found',
        frontend       => 'NACL::CS::SystemNodeShowDiscovered::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_node_show_discovered',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @opts,
        api  => 'system-node-discovery-get-iter',
        copy => [qw(node cluster-uuid cluster-name netmask version)],
        map  => {
            'is-in-cluster'   => 'is-node-in-cluster',
            'serial-number'   => 'node-serial-number',
            'nvramid'         => 'node-nvram-id',
            'partner-nvramid' => 'partner-nvram-id',
            'model'           => 'node-model',
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
