# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorImage ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemServiceProcessorImage

=head1 DESCRIPTION

C<NACL::CS::SystemServiceProcessorImage> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemServiceProcessorImage.  A related class is
L<NACL::C::SystemServiceProcessorImage|lib-NACL-C-SystemServiceProcessorImage-pm>, which represents access
to an ONTAP SystemServiceProcessorImage.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemServiceProcessorImage
element are the attributes of the SystemServiceProcessorImage ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< status >>


Image Status possible value(s) are, installed,corrupt,updating,auto-updating,none
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< version >>

Firmware Version
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< image >>

Image possible value(s) are, primary,backup
Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: firmware-image


=item C<< autoupdate >>

Firmware Autoupdate possible value(s) are, true,false
Filled in for CMode CLI/ZAPI, non-iter.
Mapping for CMode ZAPI non-iter: is-autoupdate-enabled


=item C<< last_update_status >>

Last Update Status possible value(s) are, failed,passed
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< type >>

Type possible value(s) are, RLM,SP,NONE
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< is_current >>

Is Image Current possible value(s) are, true,false
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::SystemServiceProcessorImage;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'status',
    scalar => 'version',
    scalar => 'node',
    scalar => 'image',
    scalar => 'autoupdate',
    scalar => 'last_update_status',
    scalar => 'type',
    scalar => 'is_current',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SystemServiceProcessorImage_state  = NACL::CS::SystemServiceProcessorImage->fetch(command_interface => $ci, ...);
	my @SystemServiceProcessorImage_states = NACL::CS::SystemServiceProcessorImage->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes service-processor-image-get API for CMode ZAPI non-iter.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_zapi_check',
            },
        ],
        show_cmd => 'system service-processor image show',
	exception_text =>
            'No matching system service-processor image(s) found',
        frontend => 'NACL::CS::SystemServiceProcessorImage::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _zapi_check {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(@opts, _primary_keys => [qw(node image)]);
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'system_service_processor_image_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(status node last-update-status is-current version type
                )
        ],
        map => {
            'autoupdate' => 'is-autoupdate-enabled',
            'image'      => 'firmware-image',
        },
        api          => "service-processor-image-get",
        primary_keys => [qw( node image )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
