# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemServiceProcessorNetworkAutoConfiguration ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemServiceProcessorNetworkAutoConfiguration

=head1 DESCRIPTION

C<NACL::CS::SystemServiceProcessorNetworkAutoConfiguration> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemServiceProcessorNetworkAutoConfiguration.  A related class is
L<NACL::C::SystemServiceProcessorNetworkAutoConfiguration|lib-NACL-C-SystemServiceProcessorNetworkAutoConfiguration-pm>, which represents access
to an ONTAP SystemServiceProcessorNetworkAutoConfiguration.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemServiceProcessorNetworkAutoConfiguration
element are the attributes of the SystemServiceProcessorNetworkAutoConfiguration ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< subnet_name_v6 >>

Service Processor IPv6 Subnet Name
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< ipv6_subnet_id >>

Service Processor IPv6 Subnet ID
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< ipv4_subnet_id >>

Service Processor IPv4 Subnet ID
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< subnet_name_v4 >>

Service Processor IPv4 Subnet Name
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< clustername >>

Cluster Name
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::SystemServiceProcessorNetworkAutoConfiguration;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'subnet_name_v6',
    scalar => 'ipv6_subnet_id',
    scalar => 'ipv4_subnet_id',
    scalar => 'subnet_name_v4',
    scalar => 'clustername',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $SystemServiceProcessorNetworkAutoConfiguration_state  = NACL::CS::SystemServiceProcessorNetworkAutoConfiguration->fetch(command_interface => $ci, ...);
(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. This being a "singleton" table (i.e. one which can only have one
row) is callable only in scalar context.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes service-processor-auto-configuration-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        show_cmd => 'system service-processor network auto-configuration show',
	exception_text =>
            'No matching system service-processor network auto-configuration(s) found',
        frontend =>
            'NACL::CS::SystemServiceProcessorNetworkAutoConfiguration::fetch',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @args,
        api => 'system_service_processor_network_auto_configuration_show',
        is_singleton => 1,
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(subnet-name-v6 subnet-name-v4
                )
        ],
        api          => "service-processor-auto-configuration-get",
        primary_keys => [qw(  )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
