# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSmtapeStatus ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemSmtapeStatus

=head1 DESCRIPTION

C<NACL::CS::SystemSmtapeStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemSmtapeStatus.  A related class is
L<NACL::C::SystemSmtapeStatus|lib-NACL-C-SystemSmtapeStatus-pm>, which represents access
to an ONTAP SystemSmtapeStatus.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the SystemSmtapeStatus
element are the attributes of the SystemSmtapeStatus ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< node_id >>

Filled in for CMode CLI.

=item C<< end_time >>

Filled in for CMode CLI.

=item C<< status >>

Filled in for CMode CLI.

=item C<< progress >>

Filled in for CMode CLI.

=item C<< path >>

Filled in for CMode CLI.

=item C<< backup_snapshot >>

Filled in for CMode CLI.

=item C<< device >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< vserver_id >>

Filled in for CMode CLI.

=item C<< start_time >>

Filled in for CMode CLI.

=item C<< tape_block_size >>

Filled in for CMode CLI.

=item C<< session >>

Filled in for CMode CLI.

=item C<< error >>

Filled in for CMode CLI.

=item C<< type >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::SystemSmtapeStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'node_id',
    scalar => 'end_time',
    scalar => 'status',
    scalar => 'progress',
    scalar => 'path',
    scalar => 'backup_snapshot',
    scalar => 'device',
    scalar => 'node',
    scalar => 'vserver_id',
    scalar => 'start_time',
    scalar => 'tape_block_size',
    scalar => 'session',
    scalar => 'error',
    scalar => 'type',
];

=head1 METHODS

=head2 fetch

    my $SystemSmtapeStatus_state  = NACL::CS::SystemSmtapeStatus->fetch(command_interface => $ci, ...);
	my @SystemSmtapeStatus_states = NACL::CS::SystemSmtapeStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI or 7Mode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system smtape status show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching system smtape status(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'system_smtape_status_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my ($caught_exception, %api_args);
    my ($status_response, $jobid_status, $status_response_output);
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};

    my $filter           = $opts{filter};
    my $requested_fields = $opts{requested_fields};

    if (exists $filter->{'job-id'}) {
        $api_args{'job-id'} = $filter->{'job-id'};
    } elsif (exists $filter->{path}) {
        $api_args{path} = $filter->{path};
    }

    if ($pkg->_want_any_field_of(
            requested_fields     => $requested_fields,
            filter               => $filter,
            fields_filled_by_api => [qw(job-begin job-end last-update-time)]
        )
        )
    {
        $api_args{'long-status'} = 1;
    } ## end if ( $pkg->_want_any_field_of...

    try {

        #invoking appropiate API
        $status_response = $apiset->smtape_status(%api_args);
    }
    catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
        $caught_exception = 1;
    };
    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }

    my @state_objs;

    #retrieve the parsed output
    $status_response_output = $status_response->get_parsed_output();

    foreach $jobid_status (@{$status_response_output}) {
        my $final_attributes = $pkg->_hash_copy(
            source => $jobid_status,
            copy   => [qw( node type status path device progress )],
            map    => {
                'session'         => 'job-id',
                'start-time'      => 'job-begin',
                'end-time'        => 'job-end',
                'backup-snapshot' => 'snapshot-name',
            },
        );
        my $obj = $pkg->new(command_interface => $opts{command_interface});

        #creating a object
        $obj->_set_fields(row => $final_attributes);

        # push the above created object to an array
        push @state_objs, $obj;
    } ## end foreach $jobid_status ( @{$status_response_output...
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
