# $Id: //depot/prod/test/nacldev/lib/NACL/C/SystemSnmpCommunity.pm#6 $
#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SystemSnmpCommunity ComponentState Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::SystemSnmpCommunity

=head1 DESCRIPTION

C<NACL::CS::SystemSnmpCommunity> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP SystemSnmpCommunity.  A related class is
L<NACL::C::SystemSnmpCommunity|lib-NACL-C-SystemSnmpCommunity-pm>, which represents access
to an ONTAP SystemSnmpCommunity.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the SystemSnmpCommunity
element are the attributes of the SystemSnmpCommunity ComponentState.

=over

=item C<< "community" >>

(Array)

=item C<< "access" >>

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< community_name >>

Filled in for CMode CLI.


=back

=cut

package NACL::CS::SystemSnmpCommunity;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'community',
    scalar => 'access',
    scalar => 'vserver',
    scalar => 'community_name',
];

=head1 METHODS

=head2 fetch

    my $SystemSnmpCommunity_state   = NACL::CS::SystemSnmpCommunity->fetch(command_interface => $ci, ...);
	my @SystemSnmpCommunity_states = NACL::CS::SystemSnmpCommunity->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet and 7Mode CLI APIset.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'system snmp community show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
        ],
        exception_text => 'No matching system snmp community(s) found',
        is_singleton   => 1
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
        @_,
        api          => 'system_snmp_community_show',
        is_singleton => 1
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $snmp_response;

    $snmp_response = $apiset->snmp();
    my $snmp_output = $snmp_response->get_parsed_output();
    my @state_objs;
    my $state_fields = $pkg->_hash_copy(
        source => $snmp_output->[0],
        copy   => [qw( community )],
    );

    my $obj = $pkg->new(command_interface => $opts{command_interface});
    $obj->_set_fields(row => $state_fields);

    push @state_objs, $obj;

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
