# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileCloneExtension ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileCloneExtension

=head1 DESCRIPTION

C<NACL::CS::VolumeFileCloneExtension> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeFileCloneExtension.  A related class is
L<NACL::C::VolumeFileCloneExtension|lib-NACL-C-VolumeFileCloneExtension-pm>, which represents access
to an ONTAP VolumeFileCloneExtension.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeFileCloneExtension
element are the attributes of the VolumeFileCloneExtension ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< extensions >>

Supported Extensions For Clone Delete
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $extensions = $obj->extensions();

# $extensions contains a reference to the array of values

my @extensions = $obj->extensions();

# @extensions contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { extensions = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<clone-extension-info></clone-extension-info>



=item C<< volume >>

Volume Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<clone-extension-info volume="$value" />



=item C<< vserver >>

Vserver Name

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VolumeFileCloneExtension;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'extensions',
    scalar => 'volume',
    scalar => 'vserver',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VolumeFileCloneExtension_state  = NACL::CS::VolumeFileCloneExtension->fetch(command_interface => $ci, ...);
    my @VolumeFileCloneExtension_states = NACL::CS::VolumeFileCloneExtension->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes clone-extension-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching volume file clone extension(s) found',
        frontend       => 'NACL::CS::VolumeFileCloneExtension::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_clone_extension_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $vserver           = $opts{filter}->{vserver};
    my $volume            = $opts{filter}->{volume};
    my $requested_fields  = $opts{requested_fields};
    my %cmode_args;

    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );

    my $response = $apiset->clone_extension_get_iter(%cmode_args,
        query => {volume => $volume});
    my $output = $response->get_parsed_output();

    foreach my $clone_extension (
        @{$output->[0]->{'attributes-list'}->[0]->{'clone-extension-info'}})
    {
        $pkg->_hash_copy(
            source => $clone_extension,
            copy   => [qw (volume)],
            target => \%cmode_args,
        );
        my $extension = $clone_extension->{extensions}->[0]->{string};
        if (defined $extension) {
            my @extensions = @{$extension};
            $cmode_args{extensions} = $extension;
        }
        my $obj = $pkg->new(command_interface => $command_interface,);
        $obj->_set_fields(row => \%cmode_args);
        push @state_objs, $obj;
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
