#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileCloneSplitLoad ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileCloneSplitLoad

=head1 DESCRIPTION

C<NACL::CS::VolumeFileCloneSplitLoad> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeFileCloneSplitLoad.  A related class is
L<NACL::C::VolumeFileCloneSplitLoad|lib-NACL-C-VolumeFileCloneSplitLoad-pm>, which represents access
to an ONTAP VolumeFileCloneSplitLoad.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeFileCloneSplitLoad
element are the attributes of the VolumeFileCloneSplitLoad ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< max_split_load >>

Maximum Clone Split Load

Filled in for CMode CLI/ZAPI

Maps to

Cmode ZAPI:

<clone-split-load-info>
  <max-split-load>$value</max-split-load>
</clone-split-load-info>

=item C<< allowable_split_load >>

Allowable Clone Split Load

Filled in for CMode CLI/ZAPI

Maps to

Cmode ZAPI:

<clone-split-load-info>
  <available-split-load>$value</available-split-load>
</clone-split-load-info>

=item C<< token_reserved_load >>

Load Reserved for Clone Creation

Filled in for CMode CLI/ZAPI

Maps to

Cmode ZAPI:

<clone-split-load-info>
  <token-reserved-load>$value</token-reserved-load>
</clone-split-load-info>

=item C<< node >>

Node Name possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI

=item C<< current_split_load >>

Current Clone Split Load

Filled in for CMode CLI/ZAPI

Maps to

Cmode ZAPI:

<clone-split-load-info>
  <current-split-load>$value</current-split-load>
</clone-split-load-info>

=back

=cut

package NACL::CS::VolumeFileCloneSplitLoad;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'max_split_load',
    scalar => 'allowable_split_load',
    scalar => 'token_reserved_load',
    scalar => 'node',
    scalar => 'current_split_load',
];

=head1 METHODS

=head2 fetch

    my $VolumeFileCloneSplitLoad_state  = NACL::CS::VolumeFileCloneSplitLoad->fetch(command_interface => $ci, ...);
	my @VolumeFileCloneSplitLoad_states = NACL::CS::VolumeFileCloneSplitLoad->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "clone-split-load-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd       => 'volume file clone split load show',
        exception_text => 'No matching volume file clone split load(s) found',
        frontend       => 'NACL::CS::VolumeFileCloneSplitLoad::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_clone_split_load_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'clone-split-load-get-iter',
        copy => [
            qw( node max-split-load token-reserved-load
                current-split-load )
        ],
        map => {'allowable-split-load' => 'available-split-load',},
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
