# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileRetention ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileRetention

=head1 DESCRIPTION

C<NACL::CS::VolumeFileRetention> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeFileRetention.  A related class is
L<NACL::C::VolumeFileRetention|lib-NACL-C-VolumeFileRetention-pm>, which represents access
to an ONTAP VolumeFileRetention.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeFileRetention
element are the attributes of the VolumeFileRetention ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< formatted_retention_time >>

Formatted Retention Period
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< vserver >>

Vserver Name
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< file >>

File path
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< is_wraparound >>

Is Retention Time Wraparound possible value(s) are, true,false
Filled in for CMode CLI/ZAPI, non-iter.

=item C<< retention_time >>

Retention Time of the File
Filled in for CMode CLI/ZAPI, non-iter.
=back

=cut

package NACL::CS::VolumeFileRetention;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'formatted_retention_time',
    scalar => 'vserver',
    scalar => 'file',
    scalar => 'is_wraparound',
    scalar => 'retention_time',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(file vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VolumeFileRetention_state  = NACL::CS::VolumeFileRetention->fetch(command_interface => $ci, ...);
	my @VolumeFileRetention_states = NACL::CS::VolumeFileRetention->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes snaplock-get-file-retention API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'volume file retention show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text => 'No matching VolumeFileRetention(s) found',
        frontend       => 'NACL::CS::VolumeFileRetention::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_retention_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(formatted-retention-time vserver is-wraparound file retention-time
                )
        ],
        api                 => "snaplock-get-file-retention",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw( file )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
