#
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState module for the method NACL::C::VolumeFile->show_fhmap()
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileShowFhmap

=head1 DESCRIPTION

C<NACL::CS::VolumeFileShowFhmap> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this
type are returned when NACL::C::VolumeFile->show_fhmap() is invoked.

This module does not represent the state of any element, but is an object
repesentation of the output of show-fhmap command.

=head1 ATTRIBUTES

The attribute names are selected to be the same as what ONTAP returns
in the output of 'volume file show-fhmap'  command.

=over

=item C<< vserver >>

Filled in for CMode CLI

=item C<< path >>

Filled in for CMode CLI

=item C<< file_handle >>

Filled in for CMode CLI

=item C<< namespace_path >>

Filled in for CMode CLI

=item C<< namespace_filehandle >>

Filled in for CMode CLI

=item C<< namespace_constituent >>

Filled in for CMode CLI

=item C<< nc_junction_path >>

Filled in for CMode CLI

=item C<< nc_junction_active >>

Filled in for CMode CLI

=item C<< data_path >>

Filled in for CMode CLI

=item C<< data_filehandle >>

Filled in for CMode CLI

=item C<< data_constituent >>

Filled in for CMode CLI

=item C<< dc_junction_path >>

Filled in for CMode CLI

=item C<< dc_junction_active >>

Filled in for CMode CLI

=back

=cut

package NACL::CS::VolumeFileShowFhmap;

use strict;
use warnings;

use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use base 'NACL::CS::ComponentState::ONTAP';
use NACL::Exceptions::NoElementsFound qw(:try);

use Class::MethodMaker [
    new    => ['-hash', 'new'],
    scalar => 'vserver',
    scalar => 'path',
    scalar => 'filehandle',
    scalar => 'namespace_path',
    scalar => 'namespace_filehandle',
    scalar => 'namespace_constituent',
    scalar => 'nc_junction_path',
    scalar => 'nc_junction_active',
    scalar => 'data_path',
    scalar => 'data_filehandle',
    scalar => 'data_constituent',
    scalar => 'dc_junction_path',
    scalar => 'dc_junction_active',
];

=head1 METHODS

=head2 fetch

    my $VolumeFileShowFhmap  = NACL::CS::VolumeFileShowFhmap->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'volume file show-fhmap',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
        ],
        exception_text => 'No matching volume file found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'volume_file_show_fhmap',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

1;
