# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeFileSource ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileSource

=head1 DESCRIPTION

C<NACL::CS::VolumeFileSource> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeFileSource.  A related class is
L<NACL::C::VolumeFileSource|lib-NACL-C-VolumeFileSource-pm>, which represents access
to an ONTAP VolumeFileSource.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeFileSource
element are the attributes of the VolumeFileSource ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< cursor >>

Filled in for CMode CLI.

=item C<< source_file_path >>

Filled in for CMode CLI.

=item C<< last_contact >>

Filled in for CMode CLI.

=item C<< file_index >>

Filled in for CMode CLI.

=item C<< blocks_used >>

Filled in for CMode CLI.

=item C<< node >>

Filled in for CMode CLI.

=item C<< source_msid >>

Filled in for CMode CLI.

=item C<< source_dsid >>

Filled in for CMode CLI.

=item C<< job_uuid >>

Filled in for CMode CLI.

=item C<< source_vserver_id >>

Filled in for CMode CLI.

=item C<< recover >>

Filled in for CMode CLI.


=item C<< source_snap_id >>

Source Snapshot copy ID

Filled in for CMode CLI.


=item C<< recover_path >>

Recovery Path Name

Filled in for CMode CLI.

=item C<< disallow_lost_found >>

Disallow Lost+Found Recover possible value(s) are, true,false

Filled in for CMode CLI.


=item C<< source_vserver_uuid >>

Source Vserver UUID

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VolumeFileSource;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'cursor',
    scalar => 'source_file_path',
    scalar => 'last_contact',
    scalar => 'file_index',
    scalar => 'blocks_used',
    scalar => 'node',
    scalar => 'source_msid',
    scalar => 'source_dsid',
    scalar => 'job_uuid',
    scalar => 'source_vserver_id',
    scalar => 'recover',
    scalar => 'source_snap_id',
    scalar => 'recover_path',
    scalar => 'disallow_lost_found',
    scalar => 'source_vserver_uuid',
];

=head1 METHODS

=head2 fetch

    my $VolumeFileSource_state  = NACL::CS::VolumeFileSource->fetch(command_interface => $ci, ...);
	my @VolumeFileSource_states = NACL::CS::VolumeFileSource->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
	show_cmd => 'volume file source show',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching volume file source(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'volume_file_source_show',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
