#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#
#
## @summary VolumeFileWaflGetSubtreeLevel Component Module
## @author onkarb@netapp.com,asmitak@netapp.com,kvishal@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileWaflGetSubtreeLevel

=head1 DESCRIPTION

C<NACL::CS::VolumeFileWaflGetSubtreeLevel> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
It represents the level for a given fbn of a buftree.

=head1 ATTRIBUTES

=over

=item C<< level >>

This is the level of the lowest level subtree for which the specified fbn
is not the left-most child(base fbn).
It returns the level of the inode itself for fbn=0.

Filled in for 7Mode Nodescope.

=cut

package NACL::CS::VolumeFileWaflGetSubtreeLevel;

use strict;
use warnings;

use Tharn qw($Log);
use Params::Validate qw(validate);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'path',
    scalar => 'fbn',
    scalar => 'level',
];

=head1 METHODS

=head2 fetch

    my $state = NACL::CS::VolumeFileWaflGetSubtreeLevel->fetch(
        command_interface => $ci,
        filter => { 'path' => file_path,
                    'fbn'       => fbn,
        },
    );

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
    apiset_method => 'wafl_get_subtree_level',
    filter_keys_to_be_passed_to_apiset => [qw/path fbn/],
);
1;

