#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState module for the method NACL::C::VolumeFile->WaflShowvbns()
## @author jabhijit@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeFileWaflShowvbns

=head1 DESCRIPTION

C<NACL::CS::VolumeFileWaflShowvbns> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.
Object(s) of this type are returned when NACL::C::VolumeFile->wafl_showvbns()
is invoked.

This module does not represent the state of any element, but is an object
repesentation of the output of 'wafl showvbns' DBlade command.

=head1 ATTRIBUTES

The attribute names are selected to be the same as what ONTAP returns
in the output of 'wafl showvbns' DBlade command.

Example:
myvsim1*> wafl showvbns /vol/testvol/file1
fbn 0 is vbn 8471, vvbn 46
fbn 1 is vbn 8472, vvbn 47
fbn 2 is vbn 8473, vvbn 48
fbn 3 is vbn 8474, vvbn 49
fbn 4 is vbn 8475, vvbn 50

=over

=item C<< fbn >>

=item C<< vbn >>

=item C<< vvbn >>

=back

=cut

package NACL::CS::VolumeFileWaflShowvbns;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar   => 'fbn',
    scalar   => 'vbn',
    scalar   => 'vvbn',
    scalar   => 'path'
];


=head1 METHODS

=head2 fetch

    my $state = NACL::CS::VolumeFileWaflShowvbns->fetch(command_interface => $ci,);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Supports 7Mode CLI

=over

=cut

__PACKAGE__->_fetch_7mode_cli_method_builder(
        apiset_method => 'wafl_showvbns',
        filter_keys_to_be_passed_to_apiset => ['path','fbn','end-fbn'],
);

sub _extra_filter_fields { return ['end-fbn'] }
1;
