# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VolumeQtree->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeQtreeStatus

=head1 DESCRIPTION

C<NACL::CS::VolumeQtreeStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VolumeQtree->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VolumeQtree->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< oplock_mode >>

Oplock Mode possible value(s) are, enable,disable

Filled in for CMode CLI.

=item C<< volume >>

Volume Name possible value(s) are, volume name

Filled in for CMode CLI.

=item C<< qtree_path >>

Qtree Path possible value(s) are, qtree path

Filled in for CMode CLI.

=item C<< status >>

Qtree Status possible value(s) are, normal,readonly,snapvaulted,snapmirrored

Filled in for CMode CLI.

=item C<< vserver >>

Vserver Name possible value(s) are, vserver name

Filled in for CMode CLI.

=item C<< qtree_id >>

Qtree Id possible value(s) are, integer

Filled in for CMode CLI.

=item C<< security_style >>

Security Style possible value(s) are, unix,ntfs,mixed,unified

Filled in for CMode CLI.

=item C<< qtree >>

Qtree Name possible value(s) are, qtree name

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VolumeQtreeStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'oplock_mode',
    scalar => 'volume',
    scalar => 'qtree_path',
    scalar => 'status',
    scalar => 'vserver',
    scalar => 'qtree_id',
    scalar => 'security_style',
    scalar => 'qtree',
];

=head1 METHODS

=head2 fetch

    my $VolumeQtreeStatus_state  = NACL::CS::VolumeQtreeStatus->fetch(command_interface => $ci, ...);
	my @VolumeQtreeStatus_states = NACL::CS::VolumeQtreeStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'volume qtree status',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching volume qtree(s) found',
        frontend       => 'NACL::CS::VolumeQtreeStatus::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli( @args, api => 'volume_qtree_status',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
