# $Id: $

# This is a prototype.	It needs documentation.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeQuotaPolicy ComponentState Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME
 
NACL::CS::VolumeQuotaPolicy
 
=head1 DESCRIPTION
 
C<NACL::CS::VolumeQuotaPolicy> is a derived class of
 
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the 
 
state of a Volume Quota Policy.  A related class is
 
L<NACL::C::VolumeQuotaPolicy|lib-NACL-C-VolumeQuotaPolicy-pm>, which represents 
 
access to a Volume Quota Policy
 
=head1 ATTRIBUTES
 
The individual pieces of data that are part of the  state of the 
 
element are the attributes of the  ComponentState.
 
=over
 
=item C<< vserver >>

Filled in for CMode CLI.

=item C<< policy_name >>

Filled in for CMode CLI.

=item C<< destination >>

Filled in for CMode CLI.

=item C<< last_modified >>

Filled in for CMode CLI.

=item C<< policyid >>

Filled in for CMode CLI.

=back
 
=cut

package NACL::CS::VolumeQuotaPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);

use NACL::Exceptions::NoElementsFound qw(:try);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [

    # These fields are from the C Mode "volume quota policy show"
    scalar => 'vserver',
    scalar => 'policy_name',
    scalar => 'destination',
    scalar => 'last_modified',
    scalar => 'policyid',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::QuotaPolicy'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch 
  
    my $node_state  = NACL::CS::VolumeQuotaPolicy->fetch(command_interface=>$ci,...); 
    my @node_states = NACL::CS::VolumeQuotaPolicy->fetch(command_interface=>$ci,...); 
  
see L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm> 

Supports CMode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'volume quota policy show',
        choices  => [
            {   method    => "_fetch_cmode_cli",
                interface => "CLI",
                set       => "CMode",
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching volume quota policy(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter();
    my $pkg = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(
	@_,
        api => "volume_quota_policy_show"
    );
    $Log->exit();
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter();
    my ($pkg, %opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => "quota-policy-get-iter",
        map  => {
	},
        copy => [qw(
	    vserver
            policy-name
            last-modified
            policyid
	)],
    );
    $Log->exit();
    return @state_objs;

} ## end sub _fetch_cmode_zapi

1;
