# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VolumeStorageService ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VolumeStorageService

=head1 DESCRIPTION

C<NACL::CS::VolumeStorageService> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VolumeStorageService.  A related class is
L<NACL::C::VolumeStorageService|lib-NACL-C-VolumeStorageService-pm>, which represents access
to an ONTAP VolumeStorageService.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VolumeStorageService
element are the attributes of the VolumeStorageService ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vserver_id >>

Filled in for CMode CLI.

=item C<< storage_service >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <storage-service>$storage_service</storage-service>
 </volume-storage-service-get-iter>

=item C<< storage_service_id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <storage-service-id>$storage_service_id</storage-service-id>
 </volume-storage-service-get-iter>

=item C<< rcg_id >>

Filled in for CMode CLI.

=item C<< storage_service_type >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <storage-service-type>$storage_service_type</storage-service-type>
 </volume-storage-service-get-iter>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <vserver>$vserver</vserver>
 </volume-storage-service-get-iter>

=item C<< volume >>

*Volume Name

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <volume>$volume</volume>
 </volume-storage-service-get-iter>

=item C<< constituent_group_id >>

*Constituent Group ID
 
Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

For "filter", "requested_fields" and Output mapping:

 <volume-storage-service-get-iter>
    <constituent-group-id>$constituent_group_id</constituent-group-id>
 </volume-storage-service-get-iter>

=back

=cut

package NACL::CS::VolumeStorageService;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver_id',
    scalar => 'storage_service',
    scalar => 'storage_service_id',
    scalar => 'rcg_id',
    scalar => 'storage_service_type',
    scalar => 'vserver',
    scalar => 'volume',
    scalar => 'constituent_group_id',
];

=head1 METHODS

=head2 fetch

    my $VolumeStorageService_state  = NACL::CS::VolumeStorageService->fetch(command_interface => $ci, ...);
	my @VolumeStorageService_states = NACL::CS::VolumeStorageService->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "volume-storage-service-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'volume storage-service show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching volume storage-service(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'volume_storage_service_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'volume_storage_service_get_iter',
        copy => [
            qw ( vserver volume storage-service-type
                storage-service-id storage-service
                constituent-group-id
                )
        ],
    );

    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
