# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsBranchcache ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsBranchcache

=head1 DESCRIPTION

C<NACL::CS::VserverCifsBranchcache> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsBranchcache.  A related class is
L<NACL::C::VserverCifsBranchcache|lib-NACL-C-VserverCifsBranchcache-pm>, which represents access
to an ONTAP VserverCifsBranchcache.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverCifsBranchcache
element are the attributes of the VserverCifsBranchcache ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< versions >>

Supported Versions of BranchCache, possible value(s) are
v1-enable,v2-enable,enable-all
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $versions = $obj->versions();
# $versions contains a reference to the array of values
my @versions = $obj->versions();
# @versions contains the array of values
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref
# filter => { versions = [ value1, value2...] }

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-branchcache>
    <versions>
        <strings>@values</strings>
    </versions>
 </cifs-branchcache>

=item C<< hash_store_max_size >>

Maximum Size of the Hash Store, possible value(s) are <integer>[KB,MB,GB,TB,PB]

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-branchcache>
    <hash-store-max-size>$value</hash-store-max-size>
 </cifs-branchcache>

=item C<< server_key >>

Encryption Key Used to Secure the Hashes

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-branchcache>
    <server-key>$value</server-key>
 </cifs-branchcache>

=item C<< hash_store_path >>

Path to Hash Store

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-branchcache>
    <hash-store-path>$value</hash-store-path>
 </cifs-branchcache>

=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI.

=item C<< operating_mode >>

CIFS BranchCache Operating Modes, possible value(s) are
disable,all-shares,per-share

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <cifs-branchcache>
    <operating-mode>$value</operating-mode>
 </cifs-branchcache>

=back

=cut

package NACL::CS::VserverCifsBranchcache;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'versions',
    scalar => 'hash_store_max_size',
    scalar => 'server_key',
    scalar => 'hash_store_path',
    scalar => 'vserver',
    scalar => 'operating_mode',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsBranchcache_state  = NACL::CS::VserverCifsBranchcache->fetch(command_interface => $ci, ...);
	my @VserverCifsBranchcache_states = NACL::CS::VserverCifsBranchcache->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "cifs-branchcache-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver cifs branchcache show',
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }

        ],
        exception_text => 'No matching vserver cifs branchcache(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_cifs_branchcache_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $allowed_cmode_options = [
        qw( hash-store-max-size hash-store-path versions operating-mode server-key
            )
    ];

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => $allowed_cmode_options,
        api  => 'cifs_branchcache_get_iter'
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
