#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsCharacterMapping ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsCharacterMapping

=head1 DESCRIPTION

C<NACL::CS::VserverCifsCharacterMapping> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsCharacterMapping.  A related class is
L<NACL::C::VserverCifsCharacterMapping|lib-NACL-C-VserverCifsCharacterMapping-pm>, which represents access
to an ONTAP VserverCifsCharacterMapping.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverCifsCharacterMapping
element are the attributes of the VserverCifsCharacterMapping ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< mapping >>

Character Mapping
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $mapping = $obj->mapping();

# $mapping contains a reference to the array of values

my @mapping = $obj->mapping();

# @mapping contains the array of values

If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { mapping = [ value1, value2...] }


Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<cifs-character-mapping></cifs-character-mapping>



=item C<< volume >>

Volume Name

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<cifs-character-mapping volume="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<cifs-character-mapping vserver="$value" />



=back

=cut

package NACL::CS::VserverCifsCharacterMapping;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    array  => 'mapping',
    scalar => 'volume',
    scalar => 'vserver',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverCifsCharacterMapping_state  = NACL::CS::VserverCifsCharacterMapping->fetch(command_interface => $ci, ...);
    my @VserverCifsCharacterMapping_states = NACL::CS::VserverCifsCharacterMapping->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes cifs-character-mapping-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
        show_cmd => 'vserver cifs character-mapping show',
	exception_text =>
            'No matching vserver cifs character-mapping(s) found',
        frontend => 'NACL::CS::VserverCifsCharacterMapping::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_cifs_character_mapping_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(volume vserver
                )
        ],
        map => {
            'mapping' =>
                [make_zapi_array('mapping'), make_zapi_skip('string'),],
        },
        api => "cifs-character-mapping-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
