#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverCifsDomainTrusts ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverCifsDomainTrusts

=head1 DESCRIPTION

C<NACL::CS::VserverCifsDomainTrusts> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverCifsDomainTrusts.  A related class is
L<NACL::C::VserverCifsDomainTrusts|lib-NACL-C-VserverCifsDomainTrusts-pm>, which represents access
to an ONTAP VserverCifsDomainTrusts.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverCifsDomainTrusts
element are the attributes of the VserverCifsDomainTrusts ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< home_domain >>

Home Domain Name

Filled in for CMode CLI/ZAPI

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI

=item C<< trusted_domain >>

Trusted Domain Name
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $trusted_domain = $obj->trusted_domain();
 
# $trusted_domain contains a reference to the array of values
 
my @trusted_domain = $obj->trusted_domain();
 
# @trusted_domain contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { trusted_domain = [ value1, value2...] }

Filled in for CMode CLI/ZAPI

Maps to:

CM ZAPI:

<trusted-domain>
<realm-name>$value</realm-name>
</trusted-domain>

=item C<< node >>

Node possible value(s) are, <nodename>,local

Filled in for CMode CLI/ZAPI

=back

=cut

package NACL::CS::VserverCifsDomainTrusts;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'home_domain',
    scalar => 'vserver',
    array  => 'trusted_domain',
    scalar => 'node',
];

=head1 METHODS

=head2 fetch

    my $VserverCifsDomainTrusts_state  = NACL::CS::VserverCifsDomainTrusts->fetch(command_interface => $ci, ...);
	my @VserverCifsDomainTrusts_states = NACL::CS::VserverCifsDomainTrusts->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes "cifs-domain-trusts-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd       => 'vserver cifs domain trusts show',
        exception_text => 'No matching vserver cifs domain trusts(s) found',
        frontend       => 'NACL::CS::VserverCifsDomainTrusts::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_cifs_domain_trusts_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my @states = $pkg->SUPER::_fetch_cmode_zapi(
        @opts,
        copy => [qw(node home-domain vserver)],
        map  => {
            "trusted-domain" => [
                make_zapi_array('trusted-domain'),
                make_zapi_skip('realm-name'),
            ],
        },
        api => 'cifs-domain-trusts-get-iter',
    );
    $Log->exit() if $may_exit;
    return @states;
}

1;
