# $Id: $
# This is a prototype and serves as an example.	 It still needs
# documentation.

# Most of the structure is present, but the only data represented
# about an VserverExportPolicy so far is its name.

#
# Copyright (c) 2001-2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverExportPolicy ComponentState Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverExportPolicy

=head1 DESCRIPTION

C<NACL::CS::VserverExportPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the 
state of an Vserver Export Policy.  A related class is
L<NACL::C::VserverExportPolicy|lib-NACL-C-VserverExportPolicy-pm>, which represents 
access to an ONTAP Vserver Export Policy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the 
element are the attributes of the Vserver Export Policy ComponentState.

=over
 
=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <export-policy-info>
    <vserevr>$array</vserevr>
  </export-policy-info>

=item C<< policyname >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <export-policy-info>
    <policy-name>$array</policy-name>
 </export-policy-info>

=item C<< policyid >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <export-policy-info>
    <policy-id>$array</policy-id>
 </export-policy-info>

=back
 
=cut

package NACL::CS::VserverExportPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'policyname',
    scalar => 'policyid',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::ExportPolicy'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

    my $node_state  = NACL::CS::VserverExportPolicy->fetch(command_interface=>$ci,...);
    my @node_states = NACL::CS::VserverExportPolicy->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>

Supports CMode CLI/ZAPI.

Invokes "export-policy-get-iter" API for CMode ZAPI

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver export-policy show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching vserver export policy(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_export_policy_show");
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => "export_policy_get_iter",
        map => {
            "vserver"    => ['vserver'],
            "policyid"   => ['policy-id'],
            "policyname" => ['policy-name'],
        },

    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
