# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverExportPolicy->check_access() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverExportPolicyCheckAccess

=head1 DESCRIPTION

C<NACL::CS::VserverExportPolicyCheckAccess> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverExportPolicy->check_access() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverExportPolicy->check_access() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< protocol >>

Protocol
Filled in for CMode CLI

=item C<< volume >>

Volume Name
Filled in for CMode CLI

=item C<< vserver >>

Vserver
Filled in for CMode CLI

=item C<< rule_index >>

Export Policy Rule Index
Filled in for CMode CLI

=item C<< access >>

Access Rights possible value(s) are, read,read-write
Filled in for CMode CLI

=item C<< authentication_method >>

Authentication Method
Filled in for CMode CLI

=item C<< path >>

Path
Filled in for CMode CLI

=item C<< iter_index >>

Iterator Index
Filled in for CMode CLI

=item C<< access_type >>

Access Rights to Check for possible value(s) are, read,read-write
Filled in for CMode CLI

=item C<< client_ip >>

Client Address
Filled in for CMode CLI

=item C<< policy_owner >>

Export Policy Owner
Filled in for CMode CLI

=item C<< policy >>

Export Policy
Filled in for CMode CLI

=item C<< policy_owner_type >>

Type of Export Policy Owner possible value(s) are, volume,qtree
Filled in for CMode CLI

=item C<< qtree >>

Name of the qtree
Filled in for CMode CLI

=back

=cut

package NACL::CS::VserverExportPolicyCheckAccess;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'protocol',
    scalar => 'volume',
    scalar => 'vserver',
    scalar => 'rule_index',
    scalar => 'access',
    scalar => 'authentication_method',
    scalar => 'path',
    scalar => 'iter_index',
    scalar => 'access_type',
    scalar => 'client_ip',
    scalar => 'policy_owner',
    scalar => 'policy',
    scalar => 'policy_owner_type',
    scalar => 'qtree',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverExportPolicyCheckAccess_state  = NACL::CS::VserverExportPolicyCheckAccess->fetch(command_interface => $ci, ...);
	my @VserverExportPolicyCheckAccess_states = NACL::CS::VserverExportPolicyCheckAccess->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },

        ],
        exception_text =>
            'No matching VserverExportPolicyCheckAccess(s) found',
        frontend => 'NACL::CS::VserverExportPolicyCheckAccess::fetch',
        show_cmd => 'vserver export-policy check-access',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_export_policy_check_access',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _extra_filter_fields {
    $Log->enter() if $may_enter;
    $Log->exit() if $may_exit;
    return [qw( qtree )];
}
1;
