#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFcpPortname ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFcpPortname

=head1 DESCRIPTION

C<NACL::CS::VserverFcpPortname> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFcpPortname.  A related class is
L<NACL::C::VserverFcpPortname|lib-NACL-C-VserverFcpPortname-pm>, which represents access
to an ONTAP VserverFcpPortname.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverFcpPortname
element are the attributes of the VserverFcpPortname ComponentState.

=over

=item C<< "adapter" >>

=item C<< "vserver" >>

=item C<< "used" >>

=item C<< "wwpn" >>

=item C<< "lif" >>

=item C<< "force" >>

=back

=cut

package NACL::CS::VserverFcpPortname;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::Exceptions::NoElementsFound qw(:try);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'adapter',
    scalar => 'vserver',
    scalar => 'used',
    scalar => 'lif',
    scalar => 'force',
    scalar => 'wwpn',

];

=head1 METHODS

=head2 fetch

    my $VserverFcpPortname_state   = NACL::CS::VserverFcpPortname->fetch(command_interface => $ci, ...);
    my @VserverFcpPortname_states = NACL::CS::VserverFcpPortname->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        choices => [
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        show_cmd       => 'vserver fcp portname show',
        exception_text => 'No matching vserver fcp portname(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_fcp_portname_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'fcp_port_name_get_iter',
        copy => [qw ( vserver )],
        map  => {
            'lif'  => 'fcp-adapter',
            'used' => 'is-used',
            'wwpn' => 'port-name',
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my %copy_filter           = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter            = \%copy_filter;
    my $requested_fields  = \@copy_requested_fields;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $response          = $apiset->fcp_portname_show();
    my $output            = $response->get_parsed_output();

    foreach my $list_output (@$output) {
        if (!defined $filter->{'adapter'} or $$list_output{'adapter'} =~ /$filter->{'adapter'}/) {
            my $target = $pkg->_hash_copy(
                source => $list_output,
                map    => {
                    'portname' => 'wwpn',
                },
                copy => [qw(adapter)],
            );
            my $obj = $pkg->new(command_interface => $command_interface);
            $obj->_set_fields(row => $target);
            push @state_objs, $obj;
        } ## end if ( $$list_output{'adapter'...
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli
1;
