# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFpolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFpolicy

=head1 DESCRIPTION

C<NACL::CS::VserverFpolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFpolicy.  A related class is
L<NACL::C::VserverFpolicy|lib-NACL-C-VserverFpolicy-pm>, which represents access
to an ONTAP VserverFpolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverFpolicy
element are the attributes of the VserverFpolicy ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< sequence_number >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-status-info>
    <sequence-number>$value</sequence-number>
 <fpolicy-policy-status-info>

=item C<< vserver >>

Filled in for CMode CLI.

=item C<< status >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-status-info>
    <status>$value</status>
 <fpolicy-policy-status-info>

=item C<< policy_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-status-info>
    <policy-name>$value</policy-name>
 <fpolicy-policy-status-info>

=item C<< engine >>

FPolicy Engine
Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverFpolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'sequence_number',
    scalar => 'vserver',
    scalar => 'status',
    scalar => 'policy_name',
    scalar => 'engine',
];

=head1 METHODS

=head2 fetch

    my $VserverFpolicy_state  = NACL::CS::VserverFpolicy->fetch(command_interface => $ci, ...);
    my @VserverFpolicy_states = NACL::CS::VserverFpolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "fpolicy-policy-status-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver fpolicy show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }    ## end

        ],
        exception_text => 'No matching vserver fpolicy(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'vserver_fpolicy_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => 'fpolicy_policy_status_get_iter',
        copy => [qw ( policy-name sequence-number status )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi
1;
