# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverFpolicyPolicy ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFpolicyPolicy

=head1 DESCRIPTION

C<NACL::CS::VserverFpolicyPolicy> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverFpolicyPolicy.  A related class is
L<NACL::C::VserverFpolicyPolicy|lib-NACL-C-VserverFpolicyPolicy-pm>, which represents access
to an ONTAP VserverFpolicyPolicy.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverFpolicyPolicy
element are the attributes of the VserverFpolicyPolicy ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< is_mandatory >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <is-mandatory>$value</is-mandatory>
 </fpolicy-policy-info>

=item C<< engine >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <engine-name>$value</engine-name>
 </fpolicy-policy-info>

=item C<< allow_privileged_access >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <allow-privileged-access>$value</allow-privileged-access>
 </fpolicy-policy-info>

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

=item C<< events >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <events>
       <event-name>@value</event-name>
    </events>
 </fpolicy-policy-info>

=item C<< privileged_user_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <privileged-user-name>$value</privileged-user-name>
 </fpolicy-policy-info>

=item C<< policy_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI:

 <fpolicy-policy-info>
    <policy-name>$value</policy-name>
 </fpolicy-policy-info>

=item C<< is_passthrough_read_enabled >>

Is Passthrough Read Enabled possible value(s) are, true,false

Filled in for CMode CLI/ZAPI.


=back

=cut

package NACL::CS::VserverFpolicyPolicy;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'is_mandatory',
    scalar => 'engine',
    scalar => 'allow_privileged_access',
    scalar => 'vserver',
    array  => 'events',
    scalar => 'privileged_user_name',
    scalar => 'policy_name',
    scalar => 'is_passthrough_read_enabled',
];

=head1 METHODS

=head2 fetch

    my $VserverFpolicyPolicy_state  = NACL::CS::VserverFpolicyPolicy->fetch(command_interface => $ci, ...);
	my @VserverFpolicyPolicy_states = NACL::CS::VserverFpolicyPolicy->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "fpolicy-policy-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver fpolicy policy show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }    ## end

        ],
        exception_text => 'No matching vserver fpolicy policy(s) found'
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_fpolicy_policy_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, %opts)        = @_;
    if( exists $opts{filter}->{'allow-privileged-access'} ) {
        my %filter = %{$opts{filter}};
        if ( $filter{'allow-privileged-access'} eq 'yes') {
             $filter{'allow-privileged-access'} = 'true';
        } elsif ($filter{'allow-privileged-access'} eq 'no') {
             $filter{'allow-privileged-access'} = 'false';
          }
       $opts{filter} = \%filter;
    }
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        %opts,
        api  => 'fpolicy_policy_get_iter',
        copy => [
            qw( is-mandatory policy-name privileged-user-name events
                allow-privileged-access is-passthrough-read-enabled)
        ],
        map => {'engine' => 'engine-name',},
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

sub _update_state_objs_cmode_zapi {
           $Log->enter() if $may_enter;
           my ($pkg, @args) = @_;
           $pkg->SUPER::_update_state_objs_cmode_zapi(
               @args,
               zapi_field_translations => {yes_no_boolean => [qw(allow-privileged-access)]},
           );
           $Log->exit() if $may_exit;
       }

1;
