# $Id$

#
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverFpolicy->show_enabled() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverFpolicyShowEnabled

=head1 DESCRIPTION

C<NACL::CS::VserverFpolicyShowEnabled> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverFpolicy->show_enabled() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverFpolicy->show_enabled() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< priority >>

Policy Priority
Filled in for CMode CLI

=item C<< vserver >>

Vserver
Filled in for CMode CLI

=item C<< policy_name >>

Policy Name
Filled in for CMode CLI

=item C<< sequence_number >>

Sequence Number ranges from 1 to 20

Filled in for CMode CLI.


=back

=cut

package NACL::CS::VserverFpolicyShowEnabled;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'priority',
    scalar => 'vserver',
    scalar => 'policy_name',
    scalar => 'sequence_number',
];

=head1 METHODS

=head2 fetch

    my $VserverFpolicyShowEnabled_state  = NACL::CS::VserverFpolicyShowEnabled->fetch(command_interface => $ci, ...);
	my @VserverFpolicyShowEnabled_states = NACL::CS::VserverFpolicyShowEnabled->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
	show_cmd => 'vserver fpolicy show-enabled',
	choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching vserver fpolicy(s) found',
        frontend       => 'NACL::CS::VserverFpolicyShowEnabled::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_fpolicy_show_enabled',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
