#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsi ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, raghaveb@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsi

=head1 DESCRIPTION

C<NACL::CS::VserverIscsi> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverIscsi.  A related class is
L<NACL::C::VserverIscsi|lib-NACL-C-VserverIscsi-pm>, which represents access
to an ONTAP VserverIscsi.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverIscsi
element are the attributes of the VserverIscsi ComponentState.

=over

=item C<< "status" >>

Filled in for CMode CLI.

=item C<< "clear" >>

Filled in for CMode CLI.

=item C<< "max-ios-per-session" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI: max-cmds-per-session

=item C<< "uuid" >>

Filled in for CMode CLI.

=item C<< "is_alias_assigned" >>

Filled in for CMode CLI.

=item C<< "retain-timeout" >>

Filled in for CMode CLI/ZAPI.

=item C<< "is_available" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-service-info>
     <is-available>$value</is-available>
 </iscsi-service-info>

=item C<< "max-conn-per-session" >>

Filled in for CMode CLI/ZAPI.

=item C<< "nodename" >>

Filled in for CMode CLI/ZAPI.

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-service-info>
     <vserver>$value</vserver>
 </iscsi-service-info>

=item C<< "status_admin" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-service-info>
     <is-available>$value</is-available>
 </iscsi-service-info>

=item C<< "login-timeout" >>

Filled in for CMode CLI/ZAPI.

=item C<< "status_oper" >>

Filled in for CMode CLI.

=item C<< "tcp-window-size" >>

Filled in for CMode CLI/ZAPI.

=item C<< "max-error-recovery-level" >>

Filled in for CMode CLI/ZAPI.

=item C<< "alias" >>

Filled in for CMode CLI.

=item C<< "start" >>

Filled in for CMode CLI.

=item C<< target_name >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-service-info>
     <node-name>$value</node-name>
 </iscsi-service-info>

=item C<< target_alias >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-service-info>
     <alias-name>$value</alias-name>
 </iscsi-service-info>

=item C<< vif_owner >>

Filled in for CMode CLI.

=item C<< vserver_uuid >>

Filled in for CMode CLI.

=item C<< force >>

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VserverIscsi;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Data::Dumper;

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'status',
    scalar => 'clear',
    scalar => 'max_ios_per_session',
    scalar => 'uuid',
    scalar => 'is_alias_assigned',
    scalar => 'retain_timeout',
    scalar => 'is_available',
    scalar => 'max_conn_per_session',
    scalar => 'nodename',
    scalar => 'vserver',
    scalar => 'status_admin',
    scalar => 'login_timeout',
    scalar => 'status_oper',
    scalar => 'tcp_window_size',
    scalar => 'max_error_recovery_level',
    scalar => 'alias',
    scalar => 'start',
    scalar => 'target_name',
    scalar => 'target_alias',
    scalar => 'vif_owner',
    scalar => 'vserver_uuid',
    scalar => 'force',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsi_state   = NACL::CS::VserverIscsi->fetch(command_interface => $ci, ...);
    my @VserverIscsi_states = NACL::CS::VserverIscsi->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI.

Invokes "iscsi-service-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver iscsi show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching vserver iscsi(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@_, api => 'vserver_iscsi_show');
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg    = shift;
    my %opts   = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset = $opts{apiset};
    my $filter = $opts{filter};

    # If status-admin is in the filter then need to update the value to true/false
    # status-admin will be updated to is-available by map in the fetch call.
    if(exists $opts{filter}{'status-admin'}) {
        $filter->{'status-admin'} = 'true'  if ($filter->{'status-admin'} eq 'up');
        $filter->{'status-admin'} = 'false' if ($filter->{'status-admin'} eq 'down');
    }

    my $requested_fields = $opts{requested_fields};
    my @state_objs       = $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api => 'iscsi_service_get_iter',
        map => {
            'target-alias'        => 'alias-name',
            'target-name'         => 'node-name',
            'status-admin'        => 'is-available',
            'max-ios-per-session' => 'max-cmds-per-session',
            'nodename'            => 'node-name',
        },
        copy => [
            qw ( vserver is-available login-timeout max-conn-per-session
                max-error-recovery-level retain-timeout tcp-window-size)
        ],
    );

    foreach my $obj (@state_objs) {
        my $status = $obj->status_admin();
        if ($status) {
            if ($status =~ /true/) {
                $obj->status_admin("up");
            } else {
                $obj->status_admin("down");
            }
        }
    }
    if ($pkg->_want_any_field_of(
            requested_fields     => $requested_fields,
            filter               => $filter,
            fields_filled_by_api => [qw ( is-alias-assigned )],
        )
        )
    {
        foreach my $state_obj (@state_objs) {
            my $response = $apiset->iscsi_target_alias_get_alias(
                'vserver-override-name' => $state_obj->vserver());
            my $parsed_output = $response->get_parsed_output();
            $state_obj->is_alias_assigned(
                $parsed_output->[0]{'is-alias-assigned'});
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
