#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverIscsiIsns ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, benjaram@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsiIsns

=head1 DESCRIPTION

C<NACL::CS::VserverIscsiIsns> is a derived class of
L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm|NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverIscsiIsns.  A related class is
L<NACL::C::VserverIscsiIsns|lib-NACL-C-VserverIscsiIsns-pm>, which represents access
to an ONTAP VserverIscsiIsns.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the VserverIscsiIsns
element are the attributes of the VserverIscsiIsns ComponentState.

=over

=item C<< "vserver" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <vserver>$value</vserver>
 </iscsi-isns-info>

=item C<< "status" >>

Filled in for CMode/7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< "hostname" >>

Filled in for CMode CLI.

=item C<< "address" >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <isns-ip-addr>$value</isns-ip-addr>
 </iscsi-isns-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.
 
=item C<< "enabled" >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <is-isns-enabled>$value</is-isns-enabled>
 </iscsi-isns-info>

=item C<< status_admin >>

Filled in for CMode CLI.

=item C<< last_processed >>

Filled in for CMode CLI.

=item C<< last_update_attempt >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <last-update-attempt>$value</last-update-attempt>
 </iscsi-isns-info>

=item C<< force >>

Filled in for CMode CLI.

=item C<< last_successful_update >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <last-successful-update>$value</last-successful-update>
 </iscsi-isns-info>

=item C<< last_update_result >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <last-update-result>$value</last-update-result>
 </iscsi-isns-info>

=item C<< start >>

Filled in for CMode CLI.

=item C<< target_name >>

Filled in for CMode CLI.

=item C<< entity_id >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <iscsi-isns-info>
     <isns-entity-id>$value</isns-entity-id>
 </iscsi-isns-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=back

=cut

package NACL::CS::VserverIscsiIsns;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one Dumper);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::C::Exceptions::VserverIscsiIsns::IscsiServiceNotFound qw();

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => "vserver",
    scalar => "status",
    scalar => "hostname",
    scalar => "address",
    scalar => "enabled",
    scalar => "entity_id",
    scalar => 'status_admin',
    scalar => 'last_processed',
    scalar => 'last_update_attempt',
    scalar => 'force',
    scalar => 'last_successful_update',
    scalar => 'last_update_result',
    scalar => 'start',
    scalar => 'target_name',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsiIsns_state   = NACL::CS::VserverIscsiIsns->fetch(command_interface => $ci, ...);
    my @VserverIscsiIsns_states = NACL::CS::VserverIscsiIsns->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "iscsi-isns-get-iter" API for CMode ZAPI.

Invokes "iscsi isns show" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=item C<NACL::C::Exceptions::VserverIscsiIsns::IscsiServiceNotFound>

When iscsi service is not found or configured, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs;

    try {
        @state_objs = $pkg->SUPER::fetch(
            @args,
            show_cmd => 'vserver iscsi isns show',
            choices  => [
                {   method    => "_fetch_cmode_cli",
                    interface => "CLI",
                    set       => "CMode",
                },
                {   method    => "_fetch_cmode_zapi",
                    interface => "ZAPI",
                    set       => "CMode",
                },
                {   method    => '_fetch_7mode_cli',
                    interface => 'CLI',
                    set       => '7Mode',
                },
            ],
            exception_text => 'No matching vserver iscsi isns(s) found'
        );

    } ## end try
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        my %args      = @args;
        if (NACL::C::Exceptions::VserverIscsiIsns::IscsiServiceNotFound
            ->detect(
                exception => $exception
            )
            )
        {
            if (!$args{allow_empty}) {
                $Log->exit() if $may_exit;
                NACL::C::Exceptions::VserverIscsiIsns::IscsiServiceNotFound
                    ->detect_convert_and_throw(exception => $exception);
            }
        } else {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->exit() if $may_exit;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_iscsi_isns_show");
} ## end sub _fetch_cmode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my $pkg = shift;
    $Log->exit() if $may_exit;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "iscsi_isns_get_iter",
        copy => [
            qw ( last-successful-update last-update-attempt last-update-result vserver )
        ],
        map => {
            'enabled'   => 'is-isns-enabled',
            'entity-id' => 'isns-entity-id',
            'address'   => 'isns-ip-addr',
        },
    );

} ## end sub _fetch_cmode_zapi

sub _fetch_7mode_cli {
    $Log->enter() if $may_enter;
    my $pkg         = shift;
    my %opts        = validate @_, $pkg->_fetch_backend_validate_spec();
    my $apiset      = $opts{apiset};
    my %copy_filter = %{$opts{filter}};
    my @copy_requested_fields = @{$opts{requested_fields}};
    my $deleted_filter        = $pkg->_remove_relational_regex_filters(
        filter           => \%copy_filter,
        requested_fields => \@copy_requested_fields
    );
    my $filter           = \%copy_filter;
    my $requested_fields = \@copy_requested_fields;
    my $req_field_filter;
    $req_field_filter->{'requested_fields'} = $requested_fields;
    $req_field_filter->{'filter'}           = \%copy_filter;
    my %cmd_opts;
    my @state_objs;

    my $response = $apiset->iscsi_isns_show();
    my $output   = $response->get_parsed_output();

    foreach my $row (@$output) {
        my $obj;
        $obj = $pkg->new(command_interface => $opts{command_interface});
        my $row_modified = $pkg->_hash_copy(
            source => $row,
            map    => {
                'isns_entity_id'      => 'entity_id',
                'isns_server_ip-addr' => 'address',
                'isns_status'         => 'status',
            },
        );
        $obj->_set_fields(row => $row_modified);
        push @state_objs, $obj;
    } ## end foreach my $row (@$output)
    $Log->exit() if $may_exit;
    $Log->debug("value of the parsed output is :" . Dumper(\@state_objs));

    return @state_objs;
} ## end sub _fetch_7mode_cli

1;
