# $Id //depot/prod/test/nacldev/lib/NACL/CS/VserverIscsiStatus.pm#1 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverIscsi->status() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverIscsiStatus

=head1 DESCRIPTION

C<NACL::CS::VserverIscsiStatus> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverIscsi->status() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverIscsi->status() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< vserver >>

Vserver possible value(s) are, vserver name

Filled in for CMode CLI.

=item C<< status_admin >>

Administrative Status possible value(s) are, down,up

Filled in for CMode CLI/ZAPI.

=item C<< is_available >>

Is Service Available possible value(s) are, true,false

Filled in for CMode CLI/ZAPI.

Maps to:

CM ZAPI: ststus-admin

=back

=cut

package NACL::CS::VserverIscsiStatus;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::NoMatchingEntriesException qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vserver',
    scalar => 'status_admin',
    scalar => 'is_available',
];

=head1 METHODS

=head2 fetch

    my $VserverIscsiStatus_state  = NACL::CS::VserverIscsiStatus->fetch(command_interface => $ci, ...);
	my @VserverIscsiStatus_states = NACL::CS::VserverIscsiStatus->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI APISet.

Invokes "iscsi-service-status" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver iscsi status',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            }
        ],
        exception_text => 'No matching vserver iscsi(s) found',
        frontend       => 'NACL::CS::VserverIscsiStatus::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli( @args, api => 'vserver_iscsi_status',
        );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ( $pkg, @args ) = @_;
    my %opts = validate @args,
      $pkg->_fetch_backend_validate_spec();
    my @state_objs;
    my $command_interface = $opts{command_interface};
    my $apiset            = $opts{apiset};
    my $vserver           = $opts{filter}->{vserver};
    my %cmode_args;

    $pkg->_handle_zapi_vserver_context(
        api_opts          => \%cmode_args,
        vserver           => $vserver,
        command_interface => $command_interface
    );
    my ($response, $caught_exception);
    try {
         $response = $apiset->iscsi_service_status(%cmode_args);
    } catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
         $caught_exception = 1;
    };

    if ($caught_exception) {
        $Log->exit() if $may_exit;
        return;
    }
    my $output = $response->get_parsed_output();

    foreach my $iscsi_status (@$output) {
        my $target = $pkg->_hash_copy(
            source => $iscsi_status,
            map    => { 'is-available' => 'status-admin' },
        );
        my $obj = $pkg->new( command_interface => $command_interface, );
        $obj->_set_fields( row => $target );

        if ( defined( $obj->status_admin() ) ) {
            if ( $obj->status_admin() =~ /true/i ) {
                $obj->status_admin('up');
            }
            elsif ( $obj->status_admin() =~ /false/i ) {
                $obj->status_admin('down');
            }
            else {
                $obj->status_admin('-');
            }
        }
        push @state_objs, $obj;
    } ## end foreach my $iscsi_status (@$output)

    $Log->exit() if $may_exit;
    return @state_objs;
} # end of _fetch_cmode_zapi

1;
