# This is a prototype.	It needs documentation.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNameMapping ComponentState Module
## @author Rupamjyoti.Baruah@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverNameMapping

=head1 DESCRIPTION

C<NACL::CS::VserverNameMapping> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the
state  of Vserver Name Mapping.  A related class is
L<NACL::C::VserverNameMapping|lib-NACL-C->, which represents
to a VserverNameMapping.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the

element are the attributes of the  ComponentState.

=over

=item C<< vserver  >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <name-mapping-info>
     <vserver>$value</vserver>
 </name-mapping-info>

=item C<< direction >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <name-mapping-info>
     <direction>$value</direction>
 </name-mapping-info>

=item C<< position >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <name-mapping-info>
     <position>$value</position>
 </name-mapping-info>

=item C<< pattern >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <name-mapping-info>
     <pattern>$value</pattern>
 </name-mapping-info>

=item C<< replacement >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <name-mapping-info>
     <replacement>$value</replacement>
 </name-mapping-info>

=back

=cut

package NACL::CS::VserverNameMapping;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use NACL::Exceptions::NoElementsFound qw(:try);

use Class::MethodMaker [

    # These fields are from the C Mode "vserver name mapping show"
    scalar => 'vserver',
    scalar => 'direction',
    scalar => 'position',
    scalar => 'pattern',
    scalar => 'replacement'
];

=head1 METHODS


=head2 fetch

    my $node_state  = NACL::CS::->fetch(command_interface=>$ci,...);
    my @node_states = NACL::CS::->fetch(command_interface=>$ci,...);

see L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>

Supports CMode CLI/ZAPI APIset.

Invokes "name-mapping-get-iter" API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver name-mapping show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching vserver name mapping(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_name_mapping_show");
}

sub _fetch_cmode_zapi {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_zapi(
        @_,
        api  => "name_mapping_get_iter",
        copy => [qw( direction position pattern replacement vserver )]
    );

} ## end sub _fetch_cmode_zapi

1;
