#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverPeer->show_all()
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverPeerShowAll

=head1 DESCRIPTION

C<NACL::CS::VserverPeerShowAll> is a derived class of
L<NACL::CS::VserverPeer|lib-NACL-CS-VserverPeer-pm>. Object(s) of this type
are returned when NACL::C::VserverPeer->show_all() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverPeer->show_all() is invoked.)

=head1 ATTRIBUTES

In addition to the fields listed here, the attributes of NACL::CS::VserverPeer are also applicable here

=over

=item C<< peer_cluster >>

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).

=item C<< applications >>

(Array)

Note that for array fields, the accessor method can be invoked in either
scalar or list context.
my $applications = $obj->applications();
# $applications contains a reference to the array of values
my @applications = $obj->applications();
# @applications contains the array of values

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).


=item C<< peer_state >>

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).

=item C<< vserver >>

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).

=item C<< peer_vserver >>

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).


=item C<< peer_vserver_uuid >>

Peer Vserver UUID

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).

=item C<< vserver_uuid >>

Local Vserver UUID

Filled in for CMode CLI; CMode ZAPI (both iter and non-iter).


=back

=cut

package NACL::CS::VserverPeerShowAll;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use NACL::CS::_Mixins::VserverPeer qw(:all);

use Class::MethodMaker [
    scalar => 'notification',
    scalar => 'peer_type',
    scalar => 'peer_cluster',
    array  => 'applications',
    scalar => 'peer_state',
    scalar => 'vserver',
    scalar => 'peer_vserver',
    scalar => 'peer_vserver_uuid',
    scalar => 'vserver_uuid',
];

=head1 METHODS

=head2 fetch

    my $VserverPeerShowAll_state  = NACL::CS::VserverPeerShowAll->fetch(command_interface => $ci, ...);
	my @VserverPeerShowAll_states = NACL::CS::VserverPeerShowAll->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI APISet or CMode ZAPI APISet (supports both iter and non-iter ZAPI).

Invokes vserver-peer-get-iter for iter CMode ZAPI.

Invokes vserver-peer-get for non-iter CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(@args, $pkg->_fetch_opts(), show_cmd => 'vserver peer show-all',);
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_peer_show_all',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(@args,
        $pkg->_fetch_cmode_zapi_opts(),);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _super_fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(@opts);
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
