# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverServicesNameServiceNisDomain->show_bound() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesNameServiceNisDomainShowBound

=head1 DESCRIPTION

C<NACL::CS::VserverServicesNameServiceNisDomainShowBound> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverServicesNameServiceNisDomain->show_bound() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverServicesNameServiceNisDomain->show_bound() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< domain >>

NIS Domain
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nis-domain-show-bound nis-domain="$value" />

Mapping for CMode ZAPI non-iter: nis-domain


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nis-domain-show-bound vserver="$value" />


=item C<< bound_servers >>

Bound NIS Servers
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $bound_servers = $obj->bound_servers();
 
# $bound_servers contains a reference to the array of values
 
my @bound_servers = $obj->bound_servers();
 
# @bound_servers contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { bound_servers = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <nis-domain-show-bound>
  <bound-servers ip-address="@values" />
</nis-domain-show-bound>

Mapping for CMode ZAPI non-iter: <bound-servers ip-address="@values" />

=back

=cut

package NACL::CS::VserverServicesNameServiceNisDomainShowBound;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'domain',
    scalar => 'vserver',
    array  => 'bound_servers',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver domain)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesNameServiceNisDomainShowBound_state  = NACL::CS::VserverServicesNameServiceNisDomainShowBound->fetch(command_interface => $ci, ...);
	my @VserverServicesNameServiceNisDomainShowBound_states = NACL::CS::VserverServicesNameServiceNisDomainShowBound->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes name-service-nis-show-bound-iter API for CMode ZAPI iter.

Invokes name-service-nis-show-bound API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver services name-service nis-domain show-bound',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text =>
            'No matching VserverServicesNameServiceNisDomainShowBound(s) found',
        frontend =>
            'NACL::CS::VserverServicesNameServiceNisDomainShowBound::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_services_name_service_nis_domain_show_bound',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(vserver
                )
        ],
        map => {
            'domain'        => 'nis-domain',
            'bound-servers' => [
                make_zapi_array('bound-servers'),
                make_zapi_skip('ip-address'),
            ],
        },
        api => "name-service-nis-show-bound-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(vserver
                )
        ],
        map => {
            'domain'        => 'nis-domain',
            'bound-servers' => [
                make_zapi_array('bound-servers'),
                make_zapi_skip('ip-address'),
            ],
        },
        api                 => "name-service-nis-show-bound",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw( domain )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
