# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesNameServiceNsSwitch ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesNameServiceNsSwitch

=head1 DESCRIPTION

C<NACL::CS::VserverServicesNameServiceNsSwitch> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverServicesNameServiceNsSwitch.  A related class is
L<NACL::C::VserverServicesNameServiceNsSwitch|lib-NACL-C-VserverServicesNameServiceNsSwitch-pm>, which represents access
to an ONTAP VserverServicesNameServiceNsSwitch.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverServicesNameServiceNsSwitch
element are the attributes of the VserverServicesNameServiceNsSwitch ComponentState. 

Additionally, the command_interface used to obtain the ComponentState object is 
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using 
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)


=item C<< database >>


Name Service Switch Database possible value(s) are, hosts,group,passwd,netgroup,namemap
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <namservice-nsswitch-config-info nameservice-database="$value" />

Mapping for CMode ZAPI non-iter: nameservice-database


=item C<< sources >>

Name Service Source Order possible value(s) are, files,dns,ldap,nis
(Array)
Note that for array fields, the accessor method can be invoked in either
scalar or list context.

my $sources = $obj->sources();
 
# $sources contains a reference to the array of values
 
my @sources = $obj->sources();
 
# @sources contains the array of values
 
If this field needs to be passed to the filter hash,
the value for this field should be passed in as an arrayref

# filter => { sources = [ value1, value2...] }

Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <namservice-nsswitch-config-info>
  <nameservice-sources nss-source-type="@values" />
</namservice-nsswitch-config-info>

Mapping for CMode ZAPI non-iter: <nameservice-sources nss-source-type="@values" />


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter, non-iter.

Mapping for CMode ZAPI iter: <namservice-nsswitch-config-info vserver-name="$value" />

Mapping for CMode ZAPI non-iter: vserver-name

=back

=cut

package NACL::CS::VserverServicesNameServiceNsSwitch;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Class::MethodMaker [
    scalar => 'database',
    array  => 'sources',
    scalar => 'vserver',
    scalar => [
        {   '-read_cb' => sub {
                    $Log->warn("The state field enabled is removed from ONTAP, update your scripts,".
                               "so that it doesnt the access this field");
                    return;
                }
        },
        'enabled'
    ],

];

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        _primary_keys => [qw(vserver database)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesNameServiceNsSwitch_state  = NACL::CS::VserverServicesNameServiceNsSwitch->fetch(command_interface => $ci, ...);
	my @VserverServicesNameServiceNsSwitch_states = NACL::CS::VserverServicesNameServiceNsSwitch->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes nameservice-nsswitch-get-iter API for CMode ZAPI iter.

Invokes nameservice-nsswitch-get API for CMode ZAPI non-iter.

            
=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, %args) = @_;

    my $requested_fields = delete $args{'requested-fields'};
    if ( $requested_fields ) {
        my %hash;
        map { $hash{$_} = 1 } @{ $requested_fields };    
        if ( $hash{enabled} ) {
            $Log->warn("\n The enabled field is not valid anymore, removing this field " .
                   " from requested_fields in the call to NACL::CS::VserverServicesNameServiceNsSwitch->fetch." .
                   " Please update your scripts");
            delete $hash{enabled};
            my @keys = keys %hash;
            $args{'requested-fields'} = \@keys; 
        }
    }
    if ( $args{filter}
         && $args{filter}->{enabled} ) {
        $Log->warn("\n The enabled field is not valid anymore, removing this field " .
                   " from filter hash in the call to NACL::CS::VserverServicesNameServiceNsSwitch->fetch" .
                   " Please update your scripts");
        delete $args{filter}->{enabled};
    }
    my @state_objs = $pkg->SUPER::fetch(
        %args,
        show_cmd => 'vserver services name-service ns-switch show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter'
            },
        ],
        exception_text =>
            'No matching vserver services name-service ns-switch(s) found',
        frontend => 'NACL::CS::VserverServicesNameServiceNsSwitch::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_services_name_service_ns_switch_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        map => {
            'vserver'  => 'vserver-name',
            'database' => 'nameservice-database',
            'sources'  => [
                make_zapi_array('nameservice-sources'),
                make_zapi_skip('nss-source-type'),
            ],
        },
        api => "nameservice-nsswitch-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my $vserver;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        vserver_operated_on => \$vserver,
        @args,
        map => {
            'vserver'  => 'vserver-name',
            'database' => 'nameservice-database',
            'sources'  => [
                make_zapi_array('nameservice-sources'),
                make_zapi_skip('nss-source-type'),
            ],
        },
        api          => "nameservice-nsswitch-get",
        primary_keys => [qw( database )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
