# $Id: //depot/prod/test/nacldev/lib/NACL/CS/VserverServicesNdmpGeneratePassword.pm#6 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverServicesNdmp->generate_password() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesNdmpGeneratePassword

=head1 DESCRIPTION

C<NACL::CS::VserverServicesNdmpGeneratePassword> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverServicesNdmp->generate_password() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverServicesNdmp->generate_password() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over

=item C<< password >>

Password

Filled in for CMode CLI/ZAPI
  Maps to:
  CM ZAPI:
    Input:
    If this option is specified in,
       * For 'filter', ndmp-password-generate doesn't support this in the input options,
       so filtering for this option would be done by Component layer itself
       * 'requested_fields', it  would translate to
            <ndmp-password-info>
            <password>$value</password>
            </ndmp-password-info>
    Output:
            <ndmp-password-info>
                <password>$value</password>
            </ndmp-password-info>

=item C<< vserver >>

Vserver

Filled in for CMode CLI/ZAPI
  Maps to:
  CM ZAPI:
    Input:
    If this option is specified in,
       * 'filter',Invoked in the context of the specified vserver
       * 'requested_fields', it  would translate to
            <ndmp-password-info>
            <vserver-name>$value</vserver-name>
            </ndmp-password-info>
    Output:
            <ndmp-password-info>
            <vserver-name>$value</vserver-name>
            </ndmp-password-info>

=item C<< user >>

User

Filled in for CMode CLI/ZAPI
  Maps to:
  CM ZAPI:
    Input:
    If this option is specified in,
       * 'filter', it would translate to 'user-name'
       * 'requested_fields', it  would translate to
            <ndmp-password-info>
            <user-name>$value</user-name>
            </ndmp-password-info>
    Output:
            <ndmp-password-info>
            <user-name>$value</user-name>
            </ndmp-password-info>

=back

=cut

package NACL::CS::VserverServicesNdmpGeneratePassword;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NACL::Exceptions::NoElementsFound qw(:try);

use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'password',
    scalar => 'vserver',
    scalar => 'user',
];

=head1 METHODS

=head2 fetch

    my $VserverServicesNdmpGeneratePassword_state  = NACL::CS::VserverServicesNdmpGeneratePassword->fetch(command_interface => $ci, ...);
    my @VserverServicesNdmpGeneratePassword_states = NACL::CS::VserverServicesNdmpGeneratePassword->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses a CMode CLI/ZAPI APISet.
  Invokes 'ndmp-password-generate' API for CMode ZAPI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type

doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver services ndmp generate-password',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
            },
            {   method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
            },
        ],
        exception_text => 'No matching vserver services ndmp(s) found',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@_,
        api => 'vserver_services_ndmp_generate_password',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args, $pkg->_fetch_backend_validate_spec();
    my $vserver;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter (
        %opts,
        vserver_operated_on => \$vserver,
        api  => 'ndmp-password-generate',
        copy => [qw(password)],
        map => {
            'user' => 'user-name',
            'vserver' => 'vserver-name',
        },
        primary_keys => [qw( user )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end _fetch_cmode_zapi

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi_non_iter(
        @args,
        copy => [
            qw(password
                )
        ],
        map => {'user' => 'user-name', 'vserver' => 'vserver-name',},
        api                 => "ndmp-password-generate",
        vserver_operated_on => \my $vserver,
        primary_keys        => [qw( user )],
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}

1;
