# $Id$

# This is a prototype.	It needs documentation.

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverServicesUnixUser ComponentState Module
## @author Dheeraj.K@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverServicesUnixUser

=head1 DESCRIPTION

C<NACL::CS::VserverServicesUnixUser> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>.  It represents the
state  of an ONTAP VserverServicesUnixUser.  A related class is
L<NACL::C::VserverServicesUnixUser|lib-NACL-C-VserverServicesUnixUser-pm>, which represents access
to an ONTAP VserverServicesUnixUser.

=head1 ATTRIBUTES

The individual pieces of data that are part of the  state of the
element are the attributes of the  ComponentState.

=over

=item C<< vserver >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-user-info>
     <vserver>$value</vserver>
 </unix-user-info>

=item C<< user >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-user-info>
     <user-name>$value</user-name>
 </unix-user-info>

7Mode CLI:

For "filter": login name (Programmatic name: login-name)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< id >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-user-info>
     <user-id>$value</user-id>
 </unix-user-info>

=item C<< primary_gid >>

Filled in for CMode CLI/ZAPI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-user-info>
     <group-id>$value</group-id>
 </unix-user-info>

=item C<< full_name >>

Filled in for CMode CLI/ZAPI, 7Mode CLI.

Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

 <unix-user-info>
     <full-name>$value</full-name>
 </unix-user-info>

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< group >>

Filled in for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":-g <value> (Programmatic name:group)

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< Rid >>

Filled in for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< Info >>

Filled in for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=item C<< status >>

Filled in for CMode CLI, 7Mode CLI.

Maps to:

7Mode CLI:

For "filter":
Applicable, Filtering will be done by Components.

For "requested_fields",
Not applicable, but the field will be populated in the CS object.

=back

=cut

package NACL::CS::VserverServicesUnixUser;

use strict;
use warnings;
use Params::Validate qw(validate);
use NACL::ComponentUtils qw(_dump_one);
use Data::Dumper;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [

    # These fields are from the C Mode "vserver services unix user show"
    scalar => 'vserver',
    scalar => 'user',
    scalar => 'id',
    scalar => 'primary_gid',
    scalar => 'full_name',
    scalar => 'group',

    # These fields are from the 7 Mode
    scalar => 'Rid',
    scalar => 'Info',
    scalar => 'status',
];

sub isa {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, $kind) = @_;
    my $isa = $pkg_or_obj->_build_isa(
        kind  => $kind,
        alias => 'NACL::CS::UnixUser'
    );
    $Log->exit() if $may_exit;
    return $isa;
}

=head1 METHODS

=head2 fetch

    my $VserverServicesUnixUser_state   = NACL::CS::VserverServicesUnixUser->fetch(command_interface => $ci, ...);
    my @VserverServicesUnixUser_states = NACL::CS::VserverServicesUnixUser->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Supports CMode CLI/ZAPI, 7Mode CLI.

Invokes "name-mapping-unix-user-get-iter" API for CMode ZAPI.

Invokes "useradmin user list" command for 7Mode CLI.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my $pkg        = shift;
    my @state_objs = $pkg->SUPER::fetch(
        @_,
        show_cmd => 'vserver services unix-user show',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_fetch_7mode_cli',
                interface => 'CLI',
                set       => '7Mode',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
        exception_text => 'No matching unix user(s) found'
    );

    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
} ## end sub fetch

sub _fetch_cmode_cli {
    my $pkg = shift;
    return $pkg->SUPER::_fetch_cmode_cli(@_,
        api => "vserver_services_unix_user_show");
}

sub _fetch_7mode_cli {
    my $pkg = shift;
    my %opts = validate @_, $pkg->_fetch_backend_validate_spec();
    my %api_opts;
    my @state_objs;
    my $apiset            = delete $opts{apiset};
    my $command_interface = delete $opts{command_interface};
    my $filter            = delete $opts{filter};

    my $response = $apiset->useradmin_user_list(
        'group'      => $filter->{group},
        'login-name' => $filter->{user}
    );
    my $output = $response->get_parsed_output();

    foreach my $list_output (@$output) {
        my $target = $pkg->_hash_copy(
            source => $list_output,
            copy   => [qw(Rid Info)],
            map    => {
                'Groups'    => 'group',
                'Name'      => 'user',
                'Full Name' => 'full_name',
                'Status'    => 'status'
            },
        );
        my $obj = $pkg->new(command_interface => $command_interface);
        $obj->_set_fields(row => $target);
        push @state_objs, $obj;
    } ## end foreach my $list_output (@$output)
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_7mode_cli

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        api  => 'name_mapping_unix_user_get_iter',
        copy => [qw(vserver full-name)],
        map  => {
            "id"          => ['user-id'],
            "user"        => ['user-name'],
            "primary-gid" => ['group-id'],
        },
    );
    $Log->exit() if $may_exit;
    return @state_objs;
} ## end sub _fetch_cmode_zapi

1;
