# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverVscan ComponentState Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscan

=head1 DESCRIPTION

C<NACL::CS::VserverVscan> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. It represents the
state of an ONTAP VserverVscan.  A related class is
L<NACL::C::VserverVscan|lib-NACL-C-VserverVscan-pm>, which represents access
to an ONTAP VserverVscan.

=head1 ATTRIBUTES

The individual pieces of data that are part of the state of the VserverVscan
element are the attributes of the VserverVscan ComponentState.

Additionally, the command_interface used to obtain the ComponentState object is
also an attribute of the object. This makes it easier to obtain the component
object corresponding to the CS object, using
L<get_component_instance|lib-NACL-CS-ComponentState-pm/get_component_instance>.

=over

=item command_interface

The command_interface with which the CS object was obtained. (Available in
all CS objects regardless of requested_fields and the mode/interface)

=item C<< vscan_status >>

Vscan Status possible value(s) are, on,off

Filled in for CMode CLI/ZAPI.
Maps to:

CMode ZAPI:

For "requested_fields", "filter" and Output mapping:

<vscan-status-info is-vscan-enabled="$value" />



=item C<< vserver >>

Vserver

Filled in for CMode CLI.

=back

=cut

package NACL::CS::VserverVscan;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'vscan_status',
    scalar => 'vserver',
];

my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscan_state  = NACL::CS::VserverVscan->fetch(command_interface => $ci, ...);
    my @VserverVscan_states = NACL::CS::VserverVscan->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-status-get-iter API for CMode ZAPI.


=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        choices => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode'
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode'
            },
        ],
	show_cmd => 'vserver vscan show',
	exception_text => 'No matching vserver vscan(s) found',
        frontend       => 'NACL::CS::VserverVscan::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs
        = $pkg->SUPER::_fetch_cmode_cli(@args, api => 'vserver_vscan_show',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my %opts = validate @args,
        $pkg->_fetch_backend_validate_spec(allow_extra => 1);

    my $conversion_values = {'true' => 'on', 'false' => 'off'};
    my %reverse_map       = reverse %{$conversion_values};
    my $val               = $opts{filter}->{'vscan-status'};
    $opts{filter}->{'vscan-status'} = $reverse_map{$val};

    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [qw(vserver)],
        map  => {'vscan-status' => 'is-vscan-enabled',},
        api  => "vscan-status-get-iter",
    );

    my $vscan_status = 'vscan-status';

    foreach my $obj (@state_objs) {
        if (defined $obj->$vscan_status()) {
            $obj->$vscan_status($conversion_values->{$obj->$vscan_status()});
        }
    }
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
