# $Id$

#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary ComponentState Module for the method NACL::C::VserverVscan->show_events() (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::CS::VserverVscanShowEvents

=head1 DESCRIPTION

C<NACL::CS::VserverVscanShowEvents> is a derived class of
L<NACL::CS::ComponentState::ONTAP|lib-NACL-CS-ComponentState-ONTAP-pm>. Object(s) of this type
are returned when NACL::C::VserverVscan->show_events() is invoked.

((This module does not represent the state of any element, but is an object
repesentation of the output obtained when NACL::C::VserverVscan->show_events() is invoked.)

=head1 ATTRIBUTES

The fields of the output are fields of the ComponentState object.

=over


=item C<< file_path >>

File Path
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info file-path="$value" />


=item C<< vserver >>

Vserver
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info vserver="$value" />


=item C<< version >>

Vscanner Version
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info version="$value" />


=item C<< disconnect_reason >>

Server Disconnect Reason
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info disconnect-reason="$value" />


=item C<< scan_engine_status >>

Scan Engine Status Code
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info scan-engine-status="$value" />


=item C<< node >>

Node possible value(s) are, <nodename>,local
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info node="$value" />


=item C<< event_id >>

Event Number
Filled in for CMode CLI/ZAPI iter

=item C<< event_type >>

Event Type
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info event-type="$value" />


=item C<< consecutive_occurrence_count >>

Consecutive Occurrence Count
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info consecutive-occurrence-count="$value" />


=item C<< event_time >>

Event Log Time
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info event-time="$value" />


=item C<< server >>

Server
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info server="$value" />


=item C<< lif >>

Vserver LIF Used for Connection
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info lif="$value" />


=item C<< vendor >>

Vscanner Vendor
Filled in for CMode CLI/ZAPI iter

Mapping for CMode ZAPI iter: <vscan-events-info vendor="$value" />

=back

=cut

package NACL::CS::VserverVscanShowEvents;

use strict;
use warnings;
use Params::Validate qw(validate);
use NATE::Log qw(log_global);

use NACL::Exceptions::NoElementsFound qw(:try);

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);
use base 'NACL::CS::ComponentState::ONTAP';

use Class::MethodMaker [
    scalar => 'file_path',
    scalar => 'vserver',
    scalar => 'version',
    scalar => 'disconnect_reason',
    scalar => 'scan_engine_status',
    scalar => 'node',
    scalar => 'event_id',
    scalar => 'event_type',
    scalar => 'consecutive_occurrence_count',
    scalar => 'event_time',
    scalar => 'server',
    scalar => 'lif',
    scalar => 'vendor',
];
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 fetch

    my $VserverVscanShowEvents_state  = NACL::CS::VserverVscanShowEvents->fetch(command_interface => $ci, ...);
	my @VserverVscanShowEvents_states = NACL::CS::VserverVscanShowEvents->fetch(command_interface => $ci, ...);

(Class method)
Discovers which elements are present and returns their state in ComponentState
objects. Called in scalar context it returns only one state object, in list
context it returns all state objects.
See L<NACL::CS::ComponentState|lib-NACL-CS-ComponentState-pm/fetch> for a
more detailed description along with a complete explanation of the options
it accepts.

Uses CMode CLI/ZAPI.

Invokes vscan-events-get-iter API for CMode ZAPI iter.

=over

=item Exceptions

=over

=item C<NACL::Exceptions::NoElementsFound>

When there are no elements matching the query specified or elements of that type
doesn't exist, then this exception will be thrown.

=back

=back

=cut

sub fetch {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::fetch(
        @args,
        show_cmd => 'vserver vscan show-events',
        choices  => [
            {   method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {   method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter'
            },

        ],
        exception_text => 'No matching VserverVscanShowEvents(s) found',
        frontend       => 'NACL::CS::VserverVscanShowEvents::fetch',
    );
    $Log->exit() if $may_exit;
    return wantarray ? @state_objs : $state_objs[0];
}

sub _update_state_objs_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    $pkg->SUPER::_update_state_objs_cmode_zapi(@args,
        zapi_field_translations =>
            {mmddyyyy_to_timestamp => [qw(event-time)],},);
    $Log->exit() if $may_exit;
}

sub _fetch_cmode_cli {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_cli(@args,
        api => 'vserver_vscan_show_events',);
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _fetch_cmode_zapi {
    $Log->enter() if $may_enter;
    my ($pkg, @args) = @_;
    my @state_objs = $pkg->SUPER::_fetch_cmode_zapi(
        @args,
        copy => [
            qw(file-path disconnect-reason version vserver scan-engine-status node
                event-type consecutive-occurrence-count event-time server lif vendor
                )
        ],
        api => "vscan-events-get-iter",
    );
    $Log->exit() if $may_exit;
    return @state_objs;
}
1;
