#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mix-in for common functionality for VserverPeer and VserverPeerShowAll CS
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::CS::_Mixins::VserverPeer;

use strict;
use warnings;

use NACL::CS::ComponentState::ZapiSkip qw(make_zapi_skip);
use NACL::CS::ComponentState::ZapiArray qw(make_zapi_array);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use NATE::BaseException qw(:try);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(_fetch_opts _fetch_cmode_zapi_opts
        _fetch_cmode_zapi_non_iter _check_non_iter);
    our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );
}

sub _fetch_opts {
    return (
        choices => [
            {
                method    => '_fetch_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
                zapi_type => 'none',
            },
            {
                method    => '_fetch_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'iter',
            },
            {
                method    => '_fetch_cmode_zapi_non_iter',
                interface => 'ZAPI',
                set       => 'CMode',
                zapi_type => 'non-iter',
                check     => '_check_non_iter',
            },
        ],
        exception_text => 'No matching vserver peer(s) found'
    );
}

sub _fetch_cmode_zapi_opts {
    return (
        api  => 'vserver-peer-get-iter',
        copy => [qw(vserver peer-vserver peer-state peer-cluster
            peer-vserver-uuid vserver-uuid)],
        map  => {
            'applications' => [
                make_zapi_array('applications'),
                make_zapi_skip('vserver-peer-application')
            ],
        }
    );
}

sub _fetch_cmode_zapi_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    
    my ($vserver, @state_objs);
    try {
        @state_objs = $pkg->_super_fetch_cmode_zapi_non_iter(
            @opts,
            api  => 'vserver-peer-get',
            copy => [qw(peer-cluster peer-state peer-vserver
                peer-vserver-uuid vserver vserver-uuid)],
            map  => {
                'applications' => [
                    make_zapi_array('applications'),
                    make_zapi_skip('vserver-peer-application')
                ],
            },
            primary_keys        => ['peer-vserver'],
            vserver_operated_on => \$vserver,
        );
    } catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
        # This exception gets thrown if a peer relationship does not exist
        # between the two vservers. Suppress it.
    } catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = $_[0];
        
        # Even if an invalid vserver is provided on the CLI, it turns it
        # into "No matching entries found", so we should provide this
        # equivalence and suppress the exception here.
        if ($exception->text() !~ /Specified vserver not found/i) {
            $Log->exit() if $may_exit;
            $exception->throw();
        }
    };
        
    $Log->exit() if $may_exit;
    return @state_objs;
}

sub _check_non_iter {
    $Log->enter() if $may_enter;
    my ($pkg, @opts) = @_;
    $pkg->_base_check_non_iter(
        @opts,
        
        # Have to specify primary_keys because base class won't be able to
        # determine them for NACL::CS::VserverPeerShowAll
        _primary_keys => [qw(vserver peer-vserver)],
        _vs_context   => 1
    );
    $Log->exit() if $may_exit;
}

1;
