#
# Copyright (c) 2001-2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary NACL::Cleanup::Database::SQLite
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::Cleanup::Database::SQLite

=cut

package NACL::Cleanup::Database::SQLite;

use strict;
use warnings;

use Params::Validate qw(validate validate_with :types);
use NATE::BaseException qw(:try);
use Tharn qw( param);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::ResourceList::Schema;
use SQL::Abstract;
use Storable qw(nfreeze thaw);
use NACL::GeneralUtils qw(random_name_generator);
use NATE::ParamSet;
use File::Spec;
use base 'NACL::Cleanup::Database';

use Class::MethodMaker [
    new    => [ '-hash', '-init', 'new' ],
];

sub init {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $database = $self->validate_dbname(%opts);
    $self->db_name($database);
    $self->deploy_schema( $self->should_schema_be_deployed(%opts) );
    my $dsn = "dbi:SQLite:" . $database;
    $self->dsn($dsn);
    $Log->exit() if $may_exit;
}

sub should_schema_be_deployed {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $deploy_schema = $opts{'deploy_schema'} || $self->deploy_schema();
    my $database      = $opts{'db_name'} || $self->db_name();
    my $database_exists;
    if ( -e $database ) {
        $database_exists = 1;
    } else {
        $database_exists = 0;
    }

    return (! $database_exists && $deploy_schema ) ? 1 : 0; 
}

sub validate_dbname {
    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my $nate_log_dir = $self->nate_log_dir();

    my $file;
    if ( $opts{'random_db'} ) {
        while (1) {
            my $rand_name = random_name_generator(
                prefix => "resourceList",
                size   => '1'
            ) . '.db';
            $rand_name = $self->_db_name_with_unique_id('name' => $rand_name);
            $file = $nate_log_dir . '/' . $rand_name;
            unless ( -e $file ) {
                last;
            }
        }
    } else {
        # If file name is not absolute then assume its a path under the current log dir
        if (File::Spec->file_name_is_absolute($opts{'db_name'})) {
            $file=$opts{'db_name'};
        } else {
            $file = $nate_log_dir . '/' . $opts{'db_name'};
        }
    }

    $Log->exit() if $may_exit;
    return $file;
}

sub get_connection_params {

    $Log->enter() if $may_enter;
    my ( $self, %opts ) = @_;
    my %connect_params = ( 'db_name' => $self->db_name() );

    $Log->exit() if $may_exit;
    return \%connect_params;
}

sub _drop_database {
    my $self = shift;
    $Log->enter() if $may_enter;
    $self->_truncate_database();
    $Log->exit() if $may_exit;
}

sub _does_database_exists {
    my ( $self, %opts ) = @_;
    my $db_file = $opts{'db_name'} || $self->db_name();
    return ( -e $db_file) ? 1 : 0;
}


#sub DESTROY {
#    my $self = shift;
#    $Log->debug("Purging the DB...");
#    my $db_file = $self->db_name();
#    unlink $db_file;
#}

1;
