#$Id:$
#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
##
## @author sushantg@netapp.com, dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::Exceptions::ConfigValidateFailure

=head1 DESCRIPTION
This exception is used in NACL::MTask::ConfigValidation task to report
validation errors with mismatching objects

In addition to the attributes that exception objects generally contain,
this exception also contains return_param attribute.

=head1 ATTRIBUTES

=head2 return_param

This attribute is a hash-reference, which will hold all the details of 
matching/unmatching/partial_matched objest.

=head1 EXAMPLE

   use NACL::Exceptions::ConfigValidateFailure qw(:try);
   use NACL::MTask::ConfigValidate;

   try {
       NACL::MTask::ConfigValidate->compare(
	    primary_objects => \@pri_objs,
	    secondary_objects => \@sec_objs,
	);
   } catch NACL::Exceptions::ConfigValidateFailure with {
        my $exception = shift;
        $Log->comment("ConfigValidation failed " 
           . $exception->text());
        my $return_param = $exception->return_param();
        %unmached_objs = %$return_param->{unmached};
        %partial_matched_objs = %$return_param->{partial_matched};
   };

=cut

package NACL::Exceptions::ConfigValidateFailure;
use base qw(NATE::BaseException);

use Class::MethodMaker [ scalar => 'return_param' ];

1;
