#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://wikid.netapp.com/w/AERO/ONTAP_Select_Projects/init.thpl#NACL::Init::Config::ONTAPSelect::Server::BootLun
###

package NACL::Init::Config::ONTAPSelect::Server::BootLun;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);

use Moo;
extends qw(NATE::Init::Config);

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('NODES') // NATE::BaseException->throw('Missing Mandatory NODES');
    my @resources = split(',', $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_off',
        'task_args' => {
            'POWER_ACTION' => 'off',
        }
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Delete',
        'alias'     => 'delete',
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Create',
        'alias'     => 'create',
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_on',
        'task_args' => {
            'POWER_ACTION' => 'on',
        }
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_off',
        'task_args' => {
            'POWER_ACTION' => 'off',
        }
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Delete',
        'alias'     => 'delete',
    );
}

1;
