#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

###
# Documentation:
# https://wikid.netapp.com/w/AERO/ONTAP_Select_Projects/init.thpl#NACL::Init::Config::ONTAPSelect::Server::Lun
###

package NACL::Init::Config::ONTAPSelect::Server::Lun;

use strict;
use warnings;
use List::Util qw(first);

use NATE::Log qw(log_global);
use NATE::ParamSet qw(param_global);

use NACL::Init::Utils qw(process_init_args);

use Moo;
extends qw(NATE::Init::Config);

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('HOSTS') // NATE::BaseException->throw('Missing Mandatory HOSTS');
    my @resources = split(m/,|:/, $resources_param);
    #
    # use Data::Dumper qw(Dumper);
    # log_global->log('Resources: ' . Dumper(\@resources));
    # log_global->log("CHECKING RESOURCES!!!!");
    # if (! first {$_ =~ m/-b200-/ || $_=~ m/-rx2540-/} @resources)
    # {
    #     die("You can only run LUN swap automation on the b200s and rx2540s for now.");
    # }

    $self->resources(\@resources);
}

sub global_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'HOST_OS',
            'description' => 'Host OS to swap to',
            'required'    => 1,
        },
        {
            'name'        => 'DATASTORE_SIZE',
            'description' => 'Size of the datastore',
            'default'     => '12T',
        },
        {
            'name'        => 'ANSIBLE_HOST',
            'description' => 'Host with ansible to apply post setup to hosts',
            'default'     => 'sti-ansible-master',
        },
    );

    return process_init_args(\@definitions);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_off',
        'task_args' => {
            %{$global_args},
            'POWER_ACTION' => 'off',
        }
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Delete',
        'alias'     => 'delete_bootlun',
        'task_args' => $global_args,
    );
    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Create',
        'alias'     => 'create_bootlun',
        'task_args' => $global_args,
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::DatastoreLun::Delete',
        'alias'     => 'delete_datastorelun',
        'task_args' => $global_args,
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::DatastoreLun::Create',
        'alias'     => 'create_datastorelun',
        'task_args' => $global_args,
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_on',
        'timeout'   => 1200,
        'task_args' => {
            %{$global_args},
            'POWER_ACTION' => 'on',
        }
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    my $global_args = $self->global_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_off',
        'task_args' => {
            %{$global_args},
            'POWER_ACTION' => 'off',
        }
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Delete',
        'alias'     => 'delete_bootlun',
        'task_args' => $global_args,
    );

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::DatastoreLun::Delete',
        'alias'     => 'delete_datastorelun',
        'task_args' => $global_args,
    );
}

1;
