#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

package NACL::Init::Config::ONTAPSelect::Server::Power;

use strict;
use warnings;

use NATE::ParamSet qw(param_global);

use Moo;
extends qw(NATE::Init::Config);

sub setup_args
{
    my $self = shift;

    my @definitions = (
        {
            'name'        => 'HOST_OS',
            'description' => 'Host OS to swap to',
            'required'    => 1,
        },
    );

    return process_init_args(\@definitions);
}

sub generate_resources {
    my $self = shift;

    # Get the value of the Resources.
    my $resources_param = param_global->get('HOSTS') // NATE::BaseException->throw('Missing Mandatory HOSTS');
    my @resources = split(m/,|:/, $resources_param);
    $self->resources(\@resources);
}

sub generate_setup_tasks
{
    my $self = shift;

    my $setup_args = $self->setup_args;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_on',
        'timeout'   => 1200,
        'task_args' => {
            %{$setup_args},
            'POWER_ACTION' => 'on',
        }
    );
}

sub generate_cleanup_tasks
{
    my $self = shift;

    $self->add_task(
        'task_name' => 'NACL::Init::Tasks::ONTAPSelect::Server::Power',
        'alias'     => 'power_off',
        'task_args' => {
            'POWER_ACTION' => 'off',
        }
    );
}

1;
