#
#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::InstallONTAP;

use strict;
use warnings;

use NACL::Transit;
use Hostrec;

use Carp qw(confess);

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use constant defaults => {
    ONTAP_BUILDROOT        => undef,
    ONTAP_VARIANT          => undef,
    ONTAP_INSTALL_PROTOCOL => "http",
    ONTAP_INSTALL_SERVER   => undef,
    ONTAP_INSTALL_PORT     => undef,
    ONTAP_INSTALL_IMAGE    => undef,
    ONTAP_INSTALL_TIMEOUT  => 600,
};

sub execute {
    my ($self, %opts) = @_;

    my $node = $opts{resource};

    my $task_args        = $self->task_args;
    my $ontap_image      = $task_args->{ONTAP_INSTALL_IMAGE};
    my $install_protocol = $task_args->{ONTAP_INSTALL_PROTOCOL};
    my $install_server   = $task_args->{ONTAP_INSTALL_SERVER};
    my $install_port     = $task_args->{ONTAP_INSTALL_PORT};
    my $install_timeout  = $task_args->{ONTAP_INSTALL_TIMEOUT};

    # if an ontap image is not provided, build it dynamically using the buildroot/variant
    if (!$ontap_image) {
        my $ontap_buildroot  = $task_args->{ONTAP_BUILDROOT} // confess "Missing required task argument 'ONTAP_BUILDROOT'";
        my $ontap_variant    = $task_args->{ONTAP_VARIANT}   // confess "Missing required task argument 'ONTAP_VARIANT'";
        $ontap_image = join("",
                            $ontap_buildroot,
                            "/final/bedrock/export/",
                            $ontap_variant,
                            "/tarball/image.tgz");
    }

    # construct the http URL for the ontap image using the install server details provided
    # using the ${protocol}://${install_server}[:${install_port}]${ontap_image}
    if ($install_port) {
        $install_port = ":" . $install_port;
    } else {
        $install_port = "";
    }

    my $install_url =
        $install_protocol . "://" .
        $install_server .
        $install_port .
        $ontap_image;

    my $hostrec = Hostrec->new(id => $node);

    # transition to the boot menu
    my $t = NACL::Transit->new(name => $hostrec);
    $t->change_state(to => "BOOT_MENU");

    my $c = $t->get_connection();
    $c->execute(command => "fake_install",
                match_table => [
                    'Do you want to continue\? \{y\|n\}' => "y",
                    'What is the URL for the package\?' => $install_url,
                    'What is the user name on ".+?", if any\?' => "",
                    'Do you want to restore the backup configuration' => "n",
                    'reboot now\? \{y\|n\}' => "y",
                    'Starting AUTOBOOT' => "\cc" # abort autoboot, should leave us at firmware
                ],
                timeout => $install_timeout);

    return;
}

1;
