#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAP::VolumeGranularEncryption;

use strict;
use warnings;

use NATE::Process;

use Moo;
extends qw(NATE::Init::Task);

sub default_run_scheme {
    my ($self, %args) = @_;

    my $resources = $self->resources;
    my ( @res, @resources_run_scheme, @groups );
    my @nodes;

    # loop through the resources and make [[[$resources]]]
    foreach my $resource (@{$resources}) {

        # create a node object for each node
        my $node_obj = NACL::C::Node->new( node => $resource );

        # push all the node objects into an array
        push (@nodes, $node_obj);
    }
    my $nodes_by_unique_clusters = NACL::STask::Cluster->group_nodes_by_unique_clusters( nodes => \@nodes);

    foreach my $cluster_uuid(keys %$nodes_by_unique_clusters) {
        my $cluster_nodes = $nodes_by_unique_clusters->{$cluster_uuid};
        my @nodes = map {$_->node()} @$cluster_nodes;
        push(@resources_run_scheme, \@nodes);
    }

    # This will group all the resources together into something like:
    # [[[res1,res2,res3,...]]] or [[[res1,res2,..],[res3,res4,..]]]
    push (@groups, \@resources_run_scheme);

    return \@groups;
}

sub execute {

    my ($self, %args) = @_;

    my $node = $args{resource};
    my @nodes = @{$node};
    my $nodes = join(",",@nodes);

    my $proc = NATE::Process->new(
            codespec => "os/boot/vge_setup.thpl",
            runid    => "volume_granular_encryption_setup",
            inherit  => 1,
            onexit   => \&NATE::Process::on_exit_propagate_worst_result,
            params   => {
                           DISABLE_ROOT_PASSWORD=>1,
                           DISABLE_RAID_SCRUB=>1,
                           IMAGE_CHECKER=>0,
                           SET_ASUP_TAG=>1,
                           CLUSTERED_ONTAP=>1,
                           SAY_SWALLOW_OVERRIDE=>0,
                           SWALLOW_TIMEOUT => 0,
                           SWALLOW_TIMEOUT_CONSOLE => 0.2,
                           SAY_SWALLOW => 1,
                           NODES => $nodes
                        },
        );
    $proc->start();
    $proc->wait();
    $proc->destroy();
}

1;
