#
#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::Init::Tasks::ONTAP::WipeConfig;

use strict;
use warnings;

use NACL::Transit;
use Hostrec;

use Carp qw(confess);

use Moo;
extends qw(NATE::Init::Task);

# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use constant defaults => {
    WIPE_CONFIG_TIMEOUT => 600,
};

sub execute {
    my ($self, %opts) = @_;

    my $node = $opts{resource};

    my $task_args = $self->task_args;
    my $wipe_config_timeout = $task_args->{WIPE_CONFIG_TIMEOUT};

    my $hostrec = Hostrec->new(id => $node);

    # transition to the boot menu
    my $t = NACL::Transit->new(name => $hostrec);
    $t->change_state(to => "BOOT_MENU");

    my $c = $t->get_connection();

    # wipe the config
    $c->execute(command => "wipeconfig",
                match_table => [
                    'want to continue\?' => "y",
                    'Starting AUTOBOOT' => "\cc" # abort autoboot, should leave us at firmware
                ],
                timeout => $wipe_config_timeout);

    # make sure we get to the FIRMWARE
    $t->wait_for_state(wait_for => "FIRMWARE");

    # boot ontap. Since we performed a wipeconfig operation, this should send us back to the firmware
    #$c->execute(command => "boot_ontap", timeout => 600);

    # make sure we got back to the firmware
    #$t->wait_for_state(to => "FIRMWARE");

    # transition back to the boot menu one last time
    $t->change_state(to => "BOOT_MENU");

    return;
}

1;
