#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Appliance::Upgrade;

use strict;
use warnings;
use Carp qw(croak);
use Cwd  qw(abs_path);
use Data::Dumper qw(Dumper);
use Net::Ping;

use Net::SFTP::Foreign;

use Hostrec;
use NATE::Events   qw(eventloop);
use NATE::Log      qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my $deploy_appliance = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $tgz = abs_path("$task_args{BUILDROOT}/offtap/lib/deploy/bedrock/export/"
        . "linux64.debug/upgrade_pkg/ONTAPdeploy_upgrade.tar.gz");

    # For credentials
    my $hostrec_da = Hostrec->new('id' => $deploy_appliance);

    $log->comment("IP: " . $hostrec_da->default_ip);
    $log->comment("Username: " . $hostrec_da->default_username);
    $log->comment("Password: " . $hostrec_da->default_password);

    my $sftp = Net::SFTP::Foreign->new(
        'host'     => $hostrec_da->default_ip,
        'user'     => $hostrec_da->default_username,
        'password' => $hostrec_da->default_password,
        'more'     => '-v',
    );
    $log->comment("Uploading $tgz to $deploy_appliance");
    $sftp->put($tgz) or croak("put failed: " . $sftp->error);
    $log->comment("Upload complete.");

    my $client = get_deploy_apiclient($deploy_appliance);

    my $response = $client->POST('path' => 'upgrade', 'data' => {'path' => '/home/'.$hostrec_da->default_username.'/ONTAPdeploy_upgrade.tar.gz'});
    $log->comment(Dumper($response));

    # $conn->send("upgrade /home/".$conn->username."/ONTAPdeploy_upgrade.tar.gz\r");   
    # my $out = $conn->expect('timeout' => 240, 'match' => 'Finalizing upgrade');

    _wait_for_off($hostrec_da);
    _wait_for_on($hostrec_da);
}

sub _wait_for_off
{
    my $hostrec = shift;
    my $log = log_global;
    my $p   = Net::Ping->new;

    while (1)
    {
        my $exit_status = system('ping', '-c', 1, '-t', 5, $hostrec->default_ip);
        if ($exit_status == 0)
        {
            $log->info("Still responsive, waiting for it to "
                . 'be unresponsive...');
            eventloop('seconds' => 10);
        }
        else
        {
            $log->info("Not responsive, assuming it is powered "
                . 'completely off.');
            return 1;
        }
    }
}

sub _wait_for_on
{
    my $hostrec = shift;

    my $log = log_global;

    while (1)
    {
        my $exit_status = system('ping', '-c', 1, '-t', 5, $hostrec->default_ip);
        if ($exit_status != 0)
        {
            $log->info("Still unresponsive, waiting for it to "
                . 'be responsive...');
            eventloop('seconds' => 5);
        }
        else
        {
            $log->info("Reponsive, assuming it is powered on.");
            last;
        }
    }

    # $log->info("Waiting until we can login to the host...");
    # while (1)
    # {
    #     while (1)
    #     {
    #         Connectrec->new(
    #             'id'       => $hostrec->id,
    #             'host'     => $hostrec->id,
    #             'conntype' => 'ssh-persist_service',
    #             'connect'  => 1,
    #         );
    #         if (my $err = $@)
    #         {
    #             $log->info("Unable to login still...");
    #             eventloop('seconds' => 30);
    #         }
    #         else
    #         {
    #             $log->info("Successfully logged in.");
    #             return 1;
    #         }
    #     }
    # }
}


1;
