#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Delete;

use strict;
use warnings;
use Carp qw(croak);

use Hostrec;
use NATE::Events qw(eventloop);
use NATE::Log    qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my $log           = log_global;
    my %task_args     = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});

    my $cluster_data = $client->GET('path' => 'clusters');

    my @cluster_names = $task_args{'CLUSTER_NAME'}   ?
                        ($task_args{'CLUSTER_NAME'}) :
                        map {$_->{'name'}} @{$cluster_data->{'clusters'}};

    for my $cluster_name (@cluster_names)
    {
        $log->comment("Deleting $cluster_name cluster...");
        eval {
            $client->DELETE('path' => "clusters/$cluster_name", 'data' => {'force' => \1});
        };
        if ($@ && $@ =~ m/404/)
        {
            $log->comment("$cluster_name does not exist, returning.");
            return 1;
        }
        elsif ($@)
        {
            croak($@);
        }

        $log->comment("Polling for the $cluster_name cluster to be gone...");

        my $exists = 1;
        while($exists)
        {
            $log->comment('Waiting 30 more seconds for a check...');
            eventloop('seconds' => 30);

            eval {
                my $cluster = $client->GET('path' => "clusters/$cluster_name");
                $log->comment("$cluster->{name} cluster state is: $cluster->{state}.");
            };
            if (my $err = $@)
            {
                if ($err =~ m/ClusterNotFound/m)
                {
                    $log->comment('Cluster was successfully deleted.');
                    return 1;
                }
                else
                {
                    $log->error("Error encountered while getting information for the $cluster_name cluster: $err");
                }
            }
        }
    }
}

1;
