#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Deploy::Cluster::Offline;

use strict;
use warnings;
use Carp qw(croak);

use Hostrec;
use NATE::Events qw(eventloop);
use NATE::Log    qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::ONTAPSelect::Deploy qw(get_deploy_apiclient);

sub execute
{
    my ($self, %args) = @_;

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $client = get_deploy_apiclient($task_args{'DEPLOY_APPLIANCE'});
    my $cluster_data = $client->GET('path' => 'clusters');

    my @cluster_names = $task_args{'CLUSTER_NAME'}   ?
                        ($task_args{'CLUSTER_NAME'}) :
                        map {$_->{'name'}} @{$cluster_data->{'clusters'}};
    for my $cluster_name (@cluster_names)
    {
        eval {
            $client->POST('path' => "clusters/$cluster_name/offline", 'data' => {});
        };
        if ($@ && $@ =~ m/is in the \\"offline\\" state/m)
        {
            $log->comment("$cluster_name is already offline, returning.");
            return 1;
        }
        elsif ($@ && $@ =~ m/404/)
        {
            $log->comment("$cluster_name does not exist, returning.");
            return 1;
        }
        elsif ($@)
        {
            croak($@);
        }

        $log->comment("Polling for the state of the $cluster_name cluster until it is offline");

        my $offline = 0;
        while(! $offline)
        {
            $log->comment('Waiting 30 more seconds for a state check...');
            eventloop('seconds' => 30);

            eval {
                my $cluster = $client->GET('path' => "clusters/$cluster_name");
                $log->comment("$cluster_name cluster state is: $cluster->{state}.");
                if ($cluster->{'state'} eq 'offline')
                {
                    $offline = 1;
                }
            };
            if (my $err = $@)
            {
                $log->error("Error encountered while getting information for the $cluster_name cluster: $err");
            }
        }
    }
}

1;
