#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::DatastoreCreate;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Events        qw(eventloop);
use NATE::Log           qw(log_global);

use NACL::Init::Utils::VMware qw(wait_for_task rot13);

use constant is_retryable => 1;

sub execute
{
    my ($self, %args) = @_;

    my $resource = $args{'resource'};
    my $log = log_global;

    my $hostrec_resource = Hostrec->new('id' => $resource);
    my $hostrec;

    if ($hostrec_resource->simhost)
    {
         $hostrec = Hostrec->new('id' => $hostrec_resource->simhost);
    }
    else
    {
        $hostrec = $hostrec_resource;
    }

    my $host = $hostrec->id . '.' . $hostrec->{'DOMAIN'};

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    for (my $i = 0; $i < 10; $i++)
    {
        eval {
            my $hostSystem = $vc->find_entity_view(
                'view_type' => 'HostSystem',
                'filter'    => {'name' => $host},
            );

            # Biggest is for the datastore, the other is for boot
            my $scsiDisk;
            for my $lun (@{$hostSystem->config->storageDevice->scsiLun})
            {
                if ($lun->can('capacity'))
                {
                    $log->info("lun can capacity");
                    if (! defined($scsiDisk) ||
                        $lun->capacity->block > $scsiDisk->capacity->block)
                    {
                        $scsiDisk = $lun;
                    }
                }
                else
                {
                    $log->warn("lun cannot capacity");
                }
            }

            $log->info("scsDisk devicePath: " . $scsiDisk->devicePath);

            my $datastoreSystem = $vc->get_view('mo_ref' => $hostSystem->configManager->datastoreSystem);

            my $vmfs_datastore_options = $datastoreSystem->QueryVmfsDatastoreCreateOptions(
                'devicePath' => $scsiDisk->devicePath,
            );

            my $vmfs_spec = @$vmfs_datastore_options[0]->spec;
            $vmfs_spec->{'vmfs'}->{'volumeName'} = "sDOT-$resource";

            $datastoreSystem->CreateVmfsDatastore('spec' => $vmfs_spec);
        };
        if ($@)
        {
            $log->info('Error countered: ' . $@);
            $log->info('Rescanning All HBA for next attempt in 60 seconds...');
            eventloop('seconds' => 60);
            my $hostSystem = $vc->find_entity_view(
                'view_type' => 'HostSystem',
                'filter'    => {'name' => $host},
            );
            my $hostStorageSystem = $vc->get_view('mo_ref' => $hostSystem->configManager->storageSystem);
            $hostStorageSystem->RescanAllHba();
        }
        else
        {
            $log->info('Successfully created the datastore!');
            return 1;
        }
    }
}

1;
