#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::DatastoresDestroy;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::Init::Utils::VMware qw(rot13);

sub execute
{
    my ($self, %args) = @_;

    my $resource = $args{'resource'};
    my $log = log_global;

    my $hostrec_resource = Hostrec->new('id' => $resource);
    my $hostrec;

    if ($hostrec_resource->simhost)
    {
         $hostrec = Hostrec->new('id' => $hostrec_resource->simhost);
    }
    else
    {
        $hostrec = $hostrec_resource;
    }

    my $host = $hostrec->id . '.' . $hostrec->{'DOMAIN'};

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    my $datastoreSystem = $vc->get_view('mo_ref' => $hostSystem->configManager->datastoreSystem);

    if (!$hostSystem->datastore)
    {
        $log->info("No datastores to cleanup, exiting.");
        return 1;
    }

    for my $dataStore (@{$hostSystem->datastore})
    {
        $datastoreSystem->RemoveDatastore('datastore' => $dataStore);
    }
}

1;
