#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::Destroy;

use strict;
use warnings;
use Carp         qw(croak);
use Digest::SHA  qw(sha1_hex);
use File::Temp   qw(tempfile);
use MIME::Base64 qw(decode_base64);

use Params::Validate qw(validate :types);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Connectrec;
use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::Init::Utils::VMware qw(wait_for_task);

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log  = log_global;

    my $hostrec = Hostrec->new('id' => $host);

    $host .= '.' . $hostrec->{'DOMAIN'};

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => $hostrec_vc->default_password,
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    wait_for_task(
        'message' => "Successfully disconnected the host.",
        'task'    => $hostSystem->DisconnectHost_Task,
    );

    wait_for_task(
        'message' => "Successfully destroyed the host.",
        'task'    => $hostSystem->Destroy_Task,
    );
}

1;
