#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::DestroyVMs;

use strict;
use warnings;
use Carp qw(croak);
use Data::Dumper qw(Dumper);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use NATE::Events qw(eventloop);
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);

use NACL::Init::Utils::VMware qw(rot13 wait_for_task);

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};

    my $log = log_global;

    my $hostrec = Hostrec->new('id' => $host);

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host . '.' . $hostrec->{'DOMAIN'}},
    );

    if (! $hostSystem->vm)
    {
        $log->info("No VMs found, nothing to do.");
        return 1;
    }

    for my $managedObjectRef (@{$hostSystem->vm})
    {
        my $vm = $vc->get_view('mo_ref' => $managedObjectRef);
        $log->info('Powering off VM...');
        if ($vm->runtime->powerState->val eq 'poweredOff')
        {
            $log->info("It is already powered off.");
        }
        else
        {
            $vm->PowerOffVM;

            $log->info("Waiting for VM to be powered off...");
            while ($vm->runtime->powerState->val ne 'poweredOff')
            {
                eventloop('seconds' => 5);
                $log->info("Still waiting for VM to be powered off...");
                $vm->update_view_data; # get any state changes, etc.
            }
            $log->info("It is powered off.");
        }

        $log->info("Destroying....");
        wait_for_task(
            'message' => "Succcessfully removed VM.",
            'task'    => $vm->Destroy_Task,
        );
        $log->info("Destroyed!");
    }
}

1;
