#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::ESX::Host::LicenseAssign;

use strict;
use warnings;
use Carp qw(croak);

# vmware
use VMware::VICommon;
use VMware::VIMRuntime;
use VMware::VIM25Stub;
use VMware::VILib;
use VMware::VIExt;

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::Init::Utils::VMware qw(rot13);

use constant 'defaults' => {
    'HOST_OS'  => '',
    'LICENSES' => {
        'ESXi55' => 'VMware vSphere 5 Enterprise Plus',
        'ESXi60' => 'VMware vSphere 6 Enterprise Plus',
        'ESXi65' => 'VMware vSphere 6.5 Enterprise Plus',
    },
};

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log = log_global;
    my %task_args = %{$self->task_args};

    my $hostrec = Hostrec->new('id' => $host); # for data within host file
    $host .= '.' . $hostrec->{'HOST_DOMAIN'};

    $log->info("Logging in to $hostrec->{'VCENTER'} vcenter...");
    my $hostrec_vc = Hostrec->new('id' => $hostrec->{'VCENTER'});
    my $vc = Vim::login(
        'service_url' => 'https://' . $hostrec->{'VCENTER'} . '/sdk/vimService',
        'user_name'   => $hostrec_vc->default_username,
        'password'    => rot13('string' => $hostrec_vc->default_password),
    );
    $log->info("Logged in to $hostrec->{'VCENTER'}.");

    my $hostSystem = $vc->find_entity_view(
        'view_type' => 'HostSystem',
        'filter'    => {'name' => $host},
    );

    my $content = $vc->get_service_content;
    my $licMgr  = $vc->get_view('mo_ref' => $content->licenseManager);

    my $licenseInfo;
    for my $license (@{$licMgr->licenses})
    {
        if ($license->name eq $task_args{'LICENSES'}->{$task_args{'HOST_OS'}})
        {
            if ((defined($license->used) && defined($license->total)) &&
                $license->used < $license->total)
            {
                my $available = $license->total - $license->used;
                $log->info("Found $available available licenses: " . $license->name);
                $licenseInfo = $license;
                last;
            }
        }
    }

    if (! $licenseInfo)
    {
        croak("No available licenses found.  Looking for: " . $task_args{'LICENSES'}->{$task_args{'HOST_OS'}});
    }

    my $licAsgnMgr = $vc->get_view('mo_ref' => $licMgr->licenseAssignmentManager);

    $licAsgnMgr->UpdateAssignedLicense(
        'licenseKey' => $licenseInfo->licenseKey,
        'entity'     => $hostSystem->{'mo_ref'}->value,
    );
}

1;
