#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Host::DomainsUndefine;

use strict;
use warnings;
use Carp qw(croak);

use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my ($self, %args) = @_;

    my $resource = $args{'resource'};
    my $log  = log_global;

    my $hostrec = Hostrec->new('id' => $resource);

    # It may be a node resource
    my $host = $hostrec->simhost // $resource;

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $output = $conn->execute(
        'command'      => ['virsh', 'list', '--state-shutoff'],
        'check_status' => 1,
    );

    my @stopped_domains;
    for my $line (split(/\n/, $output))
    {
        if ($line =~ m/^\s+\-+\s+([A-Za-z0-9\-_]+)/)
        {
            push(@stopped_domains, $1);
        }
    }

    # Domain exists at this point, undefine it.
    for my $domain (@stopped_domains)
    {
        $log->info("Found shut down domain '$domain', undefining it.");
        $conn->execute(
            'command'      => ['virsh', 'undefine', $domain],
            'must_match'   => "Domain $domain has been undefined",
            'check_status' => 1,
        );
    }
}

1;
