#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::BrRemove;

use strict;
use warnings;
use Carp qw(croak);

use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    my $output;
    try {
        $output = $conn->execute(
            'command'      => ['/usr/bin/ovs-vsctl', 'list-br'],
            'must_match'   => '[a-zA-Z]',
            'check_status' => 1,
        );
    } catch NATE::BaseException with {
        my $e = shift;
        if ($e->text =~ m/Command output did not match/)
        {
            $log->info('Did not find a bridge, nothing to do.');
            exit(0);
        }
        else
        {
            croak($e->text);
        }
    };

    my @bridges = split(/\n/, $output);
    for my $bridge (@bridges)
    {
        $log->info("Removing $bridge bridge.");
        $conn->execute(
            'command' => ['/usr/bin/ovs-vsctl', 'del-br', $bridge],
            'check_status' => 1,
        );
    }
}

1;
