#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::KVM::Node::DomainDestroy;

use strict;
use warnings;
use Carp qw(croak);

use Params::Validate qw(validate :types);

use Connectrec;
use NATE::Log qw(log_global);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}
use NATE::BaseException qw(:try);

sub execute
{
    my ($self, %args) = @_;

    my $node = $args{'resource'};

    my $log   = log_global;

    my %task_args = %{$self->task_args};

    my $host = $node;
    $host =~ s/[a-z]$//;

    my $conn = Connectrec->new(
        'id'       => $host,
        'host'     => $host,
        'template' => 'kvm_ssh',
    );

    try {
        $conn->execute(
            'command'    => ['virsh', 'schedinfo', $node, '--set', 'vcpu_quota=20000'],
            'must_match' => 'vcpu_quota\s+: 20000',
        );
    } catch NATE::BaseException with {
        my $e = shift;
        if ($e->text =~ m/Domain not found/)
        {
            $log->info('No domain found to destroy.');
            exit(0);
        }
        else
        {
            croak($e);
        }
    };

    my $output = $conn->execute(
        'command'      => ['virsh', 'list'],
        'check_status' => 1,
    );
    my @running_domains;
    for my $line (split(/\n/, $output))
    {
        if ($line =~ m/^\s+[0-9]+\s+(\w+)\s+running/)
        {
            push(@running_domains, $1);
        }
    }

    for my $domain (@running_domains)
    {
        # Domain exists at this point, undefine it.
        $log->info("Found running domain '$domain', destroying it.");
        $conn->execute(
            'command'      => ['virsh', 'destroy', $node],
            'must_match'   => "Domain $domain destroyed",
            'check_status' => 1,
        );
    }
}

1;
