#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Ansible;

# core
use strict;
use warnings;
use Carp       qw(croak);
use File::Temp qw(tempfile);

# cpan
use Params::Validate qw(validate :types);
use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

# nate
use Connectrec;
use Hostrec;
use NATE::Log qw(log_global);
use NATE::Process;

sub execute
{
    my ($self, %args) = @_;

    my @hosts = @{$args{'resource'}};
    my $log = log_global;
    my %task_args = %{$self->task_args};

    my $playbook;
    my $conn_template;
    if ($task_args{'HOST_OS'} =~ m/esx/i)
    {
        $conn_template = 'esx_ssh';
    }
    elsif ($task_args{'HOST_OS'} =~ m/rhel/i)
    {
        $conn_template = 'kvm_ssh';
        $playbook = '/u/sti/p4/clientutils/ansible/sdot/rhel/rhel_post/fix_usrsoft.yml';
    }

    if ($playbook && $task_args{'ANSIBLE_HOST'})
    {
        my $conn = Connectrec->new(
            'id'       => $hosts[0],
            'host'     => $hosts[0],
            'template' => $conn_template,
            'connect'  => 0,
        );

        my $ansible_proc = NATE::Process->new(
            'client'   => $task_args{'ANSIBLE_HOST'},
            'codespec' => 'tools/ctl/ansible.thpl',
            'onexit'   => \&NATE::Process::on_exit_die_worst_result,
            'params'   => {
                'ANSIBLE_TARGETS'          => join(',', @hosts),
                'ANSIBLE_TARGETS_USERNAME' => $conn->username,
                'ANSIBLE_TARGETS_PASSWORD' => $conn->password,
                'ANSIBLE_CONFIGURATIONS'   => 'retry_files_enabled=False',
                'ANSIBLE_PLAYBOOK'         => $playbook,
            },
        );
        $ansible_proc->start;
        $ansible_proc->wait;
        $ansible_proc->destroy;
    }
}

1;
