#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Server::BootLun::Create;

use strict;
use warnings;
use Carp       qw(croak);
use File::Temp qw(tempfile);

use Params::Validate qw(validate :types);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::C::Vserver;
use NACL::C::LunMapping;
use NACL::C::VolumeFileClone;
use NACL::CS::Lun;

sub execute
{
    my ($self, %args) = @_;

    my $host      = $args{'resource'};
    my $log       = log_global;
    my %task_args = %{$self->task_args};

    my $hostrec = Hostrec->new('id' => $host);

    my $vserver = NACL::C::Vserver->new('name' => $hostrec->{'BOOTLUN_VSERVER'});

    #########################################################################
    # Create the appropriate LUN
    #########################################################################
    my $host_os = uc($task_args{'HOST_OS'});
    my $template = $hostrec->{"${host_os}_TEMPLATE"}
        // croak("You must specify template in host file: ${host_os}_TEMPLATE=<template>");
    my $volume = $hostrec->{"${host_os}_VOLUME"}
        // croak("You must specify volume in host file: ${host_os}_VOLUME=<volume>");

    NACL::C::VolumeFileClone->create(
        'command_interface' => $vserver,
        'vserver'           => $hostrec->{'BOOTLUN_VSERVER'},
        'volume'            => $volume,
        'source-path'       => $template,
        'destination-path'  => $host,
        'no-reserve'        => 'true',
    );

    #########################################################################
    # Map the LUN
    #########################################################################
    # c6220s have to iscsi boot Linux
    my $igroup = ($template !~ m/esx/i && $hostrec->model =~ m/c6220/i) ? "$host-iscsi" : "$host-boot";
    NACL::C::LunMapping->create(
        'command_interface' => $vserver,
        'vserver'           => $hostrec->{'BOOTLUN_VSERVER'},
        'volume'            => $volume,
        'lun'               => $host,
        'igroup'            => $igroup,
    );
}

1;
