#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Server::DatastoreLun::Create;

use strict;
use warnings;
use Carp       qw(croak);
use File::Temp qw(tempfile);

use Params::Validate qw(validate :types);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Events        qw(eventloop);
use NATE::Log           qw(log_global);

use NACL::C::Vserver;
use NACL::C::Lun;
use NACL::C::LunIgroup;
use NACL::C::LunMapping;

sub execute
{
    my ($self, %args) = @_;

    my $host      = $args{'resource'};
    my $log       = log_global;
    my %task_args = %{$self->task_args};

    my $hostrec = Hostrec->new('id' => $host);

    if (! $hostrec->{'DATALUN_VSERVER'})
    {
        $log->info("No DATALUN_VSERVER within host file, exiting...");
        return 0;
    }

    my $vserver = NACL::C::Vserver->new('name' => $hostrec->{'DATALUN_VSERVER'});

    my $host_underbar = $host;
    $host_underbar =~ tr/-/_/;

    my $igroup = "$host-iscsi";
    my $ostype = $task_args{'HOST_OS'} =~ m/esx/i ? 'vmware' : 'linux';
    my $path   = "/vol/$host_underbar/$host_underbar";

    #########################################################################
    # Create the appropriate LUN
    #########################################################################
    # sometimes this can fail due to API limits -- adding retry
    for my $i (1 .. 3)
    {
        eval {
            NACL::C::Lun->create(
                'command_interface' => $vserver,
                'vserver'           => $hostrec->{'DATALUN_VSERVER'},
                'path'              => $path,
                'size'              => $task_args{'DATASTORE_SIZE'},
                'space-reserve'     => 'disabled',
                'ostype'            => $ostype,
            );
        };
        if (my $err = $@)
        {
            $log->info("Attempt $i failed with error: $err");
            $log->info("Waiting 60 seconds for the next attempt...");
            eventloop('seconds' => 60);
        }
        else
        {
            last;
        }
    }

    #########################################################################
    # Modify the igroup
    #########################################################################
    NACL::C::LunIgroup->modify(
        'command_interface' => $vserver,
        'igroup'            => $igroup,
        'ostype'            => $ostype,
    );

    #########################################################################
    # Map the LUN
    #########################################################################
    NACL::C::LunMapping->create(
        'command_interface' => $vserver,
        'vserver'           => $hostrec->{'DATALUN_VSERVER'},
        'path'              => $path,
        'igroup'            => $igroup,
    );
}

1;
