#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::ONTAPSelect::Server::DatastoreLun::Delete;

use strict;
use warnings;
use Carp       qw(croak);
use File::Temp qw(tempfile);

use Params::Validate qw(validate :types);

use Moo;
extends qw(NATE::Init::Task);
# need to do this to prevent Moo from clashing with NATE::BaseException
BEGIN {undef *with;}

use Hostrec;
use NATE::BaseException qw(:try);
use NATE::Log           qw(log_global);

use NACL::C::Vserver;
use NACL::C::LunMapping;
use NACL::C::VolumeFileClone;
use NACL::CS::Lun;

sub execute
{
    my ($self, %args) = @_;

    my $host = $args{'resource'};
    my $log = log_global;

    my $hostrec = Hostrec->new('id' => $host);

    if (! $hostrec->{'DATALUN_VSERVER'})
    {
        $log->info("No DATALUN_VSERVER within host file, exiting...");
        return 0;
    }

    my $vserver = NACL::C::Vserver->new('name' => $hostrec->{'DATALUN_VSERVER'});

    my $host_underbar = $host;
    $host_underbar =~ tr/-/_/;

    ########################################################################
    # Get all LUNs created for the host
    ########################################################################
    my @luns = eval { NACL::CS::Lun->fetch(
        'command_interface' => $vserver,
        'filter' => {
            'lun' => $host_underbar,
        },
        'requested_fields' => ['volume','lun', 'qtree', 'path', 'ostype','mapped'],
    )};
    if (! scalar(@luns))
    {
        $log->info("Unable to find a lun for $host, nothing to cleanup first.");
    }
    else
    {
        for my $lun (@luns)
        {
            #################################################################
            # Delete LUN Mapping
            #################################################################
            if ($lun->{'mapped'} eq 'mapped')
            {
                $log->info("$lun->{lun} LUN is mapped, deleting the mapping.");

                # vmware is -boot and linux is -iscsi in CTL
                my $igroup = "$host-iscsi";

                NACL::C::LunMapping->delete(
                    'command_interface' => $vserver,
                    'vserver'           => $hostrec->{'DATALUN_VSERVER'},
                    'igroup'            => $igroup,
                    'path'              => $lun->{'path'},
                );
            }
            else
            {
                $log->info("$lun->{lun} LUN is not mapped, no need to delete the mapping.");
            }

            #################################################################
            # Delete LUN
            #################################################################
            $log->info("Deleting $lun->{lun} LUN.");

            NACL::C::Lun->delete(
                'command_interface' => $vserver,
                'vserver'           => $hostrec->{'DATALUN_VSERVER'},
                'path'              => $lun->{'path'},
            );
        }
    }
}

1;
