#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::Vsim::Start;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name prepare_client_apiset);
use NATE::BaseException qw(try);
use Carp qw(confess);
use NACL::Transit;
use Connectrec;
use NACL::C::Node;

use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    VSIM_SCRIPT   => "/usr/software/test/bin/vsim",
};

sub execute {

    my ($self, %args) = @_;

    my @nodes = $args{resource};

    my ($task, %task_args);
    $task = $self->task_args;
    %task_args = %$task;

    my $Vsim_tool = $task->{VSIM_SCRIPT};

    log_global()->comment("Start all vsims");

    foreach my $node (@nodes) {
        # identify the vsim user to operate on for this
        my $vsim_details = get_vsim_details_from_node_name($node);
        my $vsim_user = $vsim_details->{user};
        my $vsim_dir = "/u/".$vsim_user."/vsims";
        my $sudo_user = "/usr/software/bin/sudo -i -u ".$vsim_details->{user};

        log_global()->comment("Preparing client APISet");
        my $c_api = prepare_client_apiset();

        my $status = 0;
        my ($output) = $c_api->execute_raw_command(command => "$sudo_user $Vsim_tool start -smoke -simdir $vsim_dir $vsim_details->{id}",
                                                    get_status => \$status);
        if ($status ne 0) {
            NATE::BaseException->throw("bad happened while trying to start vsims");
        }

        log_global()->comment("Waiting for loader prompt");
        my $t = NACL::Transit->new(name => $node);
        my $state = $t->get_state(transient => 1);
        if ($state ne "FIRMWARE") {
            NATE::BaseException->throw("VSim $node never got to the loader prompt");
        }
    }
}

1;
