#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

## @pod here
## @author ng-crampton-rtp-tools@netapp.com
## @summary Utility library for init.thpl Tasks

=head1 NAME

NACL::Init::Tasks::VsimUtils

=head1 SYNOPSIS

use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name
                                    prepare_client_apiset
                                    );
=head1 EXPORTED FUNCTIONS

get_vsim_details_from_node_name()
prepare_client_apiset()

=head1 DESCRIPTION

C<NACL::Init::Tasks::VsimUtils> is a utility library with convenience routines
for developers of init.thpl Tasks that are derived from NATE::Init::Task.

=cut

package NACL::Init::Tasks::VsimUtils;

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(get_vsim_details_from_node_name
                        prepare_client_apiset);
}

use strict;
use warnings;
use Data::Dumper;
use NATE::ParamSet qw(param_global);;
use NATE::Log qw(log_global);
use Carp qw(confess);
use NACL::C::Client;
use NATE::Util::Sockets qw(get_localhost_address get_hostname);

=head1 FUNCTIONS

=head2 get_vsim_details_from_node_name

=over

=item Synopsis

use NACL::Init::Tasks::VsimUtils qw(get_vsim_details_from_node_name);

my $vsim_details = get_vsim_details_from_node_name(node1);

=item Arguments

=over

=item C<< $scalar  >>

(Required) The name of the node 

=back

=item Return Value

This will return a HASHREF that contains the vsim user,id, blade and instance

=back

=cut

sub get_vsim_details_from_node_name {
    my $node_name = shift;
    # parse these two forms
    # sti1-vsim-ucs049a
    # hrivera-vsim1

    my $hash = {};
    if ($node_name =~ /^(.+?)[_-]vsim[_-]*(.+)$/) {
        $hash->{user} = $1;
        $hash->{id}   = $2;
        if ($hash->{id} =~ /^(.+)([a-z])$/) {
            $hash->{blade}    = $1;
            $hash->{instance} = $2;
        } else {
            $hash->{blade}    = undef;
            $hash->{instance} = undef;
        }
    }
    return $hash;
}

=head2 prepare_client_apiset

=over

=item Synopsis

use NACL::Init::Tasks::VsimUtils qw(prepare_client_apiset);

my $client_api = prepare_client_apiset();

=item Arguments

=over

=item Return Value

This will return a client api object to execute commands.

=back

=cut

sub prepare_client_apiset {
    my (%args) = @_;
    my $host = get_hostname(get_localhost_address());
    $host =~ s/\..+$//;
    my ($c) = NACL::C::Client->new(name => $host, apiset_must => {connid => "ssh"});
    my $c_api = $c->apiset();
    $c_api->set_timeout("connectrec-timeout" => 1200);
    return $c_api;
}

1;

