#   Copyright (c) 2016 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

package NACL::Init::Tasks::deux_init;

use strict;
use warnings;
use NATE::Log qw(log_global);
use Data::Dumper;
use Carp qw(confess);
use NATE::Process;
use NATE::ParamSet qw(param_global);
use NACL::C::Node;
use NACL::STask::Cluster;

use Moo;
extends qw(NATE::Init::Task);

use constant defaults => {
    SFO  => 0,
    CIFS => undef,
};

sub default_run_scheme {
    my ($self, %args) = @_;

    my $resources = $self->resources;
    my ( @res, @resources_run_scheme, @groups );
    my @nodes;

    # loop through the resources and make [[[$resources]]]
    foreach my $resource (@{$resources}) {

        # create a node object for each node
        my $node_obj = NACL::C::Node->new( node => $resource );

        # push all the node objects into an array
        push (@nodes, $node_obj);
    }
    my $nodes_by_unique_clusters = NACL::STask::Cluster->group_nodes_by_unique_clusters( nodes => \@nodes);

    foreach my $cluster_uuid(keys %$nodes_by_unique_clusters) {
        my $cluster_nodes = $nodes_by_unique_clusters->{$cluster_uuid};
        my @nodes = map {$_->node()} @$cluster_nodes;
        push(@resources_run_scheme, \@nodes);

    }

    # This will group all the resources together into something like:
    # [[[res1,res2,res3,...]]] or [[[res1,res2,..],[res3,res4,..]]]
    push (@groups, \@resources_run_scheme);

    return \@groups;
}

sub execute {

    my ($self, %args) = @_;

    my $node = $args{resource};
    my @nodes = @{$node};
    my $nodes = join(",",@nodes);

    my ($task, %task_args);
    $task = $self->task_args;

    # In the existing SmokeTest.pm, deux_init default parameters skips CIFS
    # server creation for vs0. Some of the CIT's using default deux_init parameters 
    # create their own CIFS server in their tests. To support this, we have replicated
    # similar scenario here as well.
    # TODO: Verify if this condition is being actually used by any CIT's at all.
    # if CIFS is explicitly passed in, do not modify based on the existence
    # of Windows clients.

    if(!defined $task->{CIFS}){
        my $all_win = param_global->get('SMOKE_ALL_WINDOWS_CLIENTS');
        if(!defined $all_win || $all_win eq ''){
            # There are no Windows clients in the job.
            $task->{CIFS} = 0;
        } else {
            # There is at least a single Windows client. Do not pass a CIFS
            # value and use the default provided by deux_init.
            delete $task->{CIFS};
        }
    }

    my $proc = NATE::Process->new(
                    codespec => "os/boot/deux_init.thpl",
                    runid    => "deux_init",
                    inherit  => 1,
                    params   => {
                                    CIFS => $task->{CIFS},
                                    STRIPING_INIT => 0,
                                    SWALLOW_TIMEOUT => 0,
                                    SWALLOW_TIMEOUT_CONSOLE => 0.2,
                                    SAY_SWALLOW => 1,
                                    SFO => $task->{SFO},
                                    NO_MGMT_LIF_WAIT => 1,
                                    DISABLE_RAID_SCRUB => 1,
                                    NODES => $nodes
                                },
                    onexit   => \&NATE::Process::on_exit_propagate_worst_result,
               );

    $proc->start();
    $proc->wait();
    $proc->destroy();
}

1;
