#   Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#    without prior written consent from NetApp, Inc.

package NACL::Init::Utils::JSON;

# core
use strict;
use warnings;
use Carp qw(croak);
use IO::File;
use Scalar::Util qw(looks_like_number);

# cpan
use JSON::XS qw(decode_json);

BEGIN {
    require Exporter;
    our @ISA = qw(Exporter);
    our @EXPORT_OK = qw(
        json_true_false
        decode_json_from_file
    );
}

sub decode_json_from_file
{
    my $file = shift;

    if (! -f $file)
    {
        croak("Unable to find file: $file");
    }

    my $fh = IO::File->new($file, 'r') or croak("Unable to open file: $!");
    my $json = <$fh>;
    $fh->close;
    chomp($json);
    return decode_json($json);
}

sub json_true_false
{
    my $value = shift;

    if ((looks_like_number($value) && $value == 0) ||
        lc($value) eq 'false')
    {
        return \0;
    }
    elsif ((looks_like_number($value) && $value == 1) ||
           lc($value) eq 'true')
    {
        return \1;
    }
    else
    {
        croak('Unknown value passed, should be either true, false, 0, or 1');
    }
}

1;
