#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::DataSet::VBNBADBasic;

use Moose;
use MTF::MooseCompatibleException qw(:all);

extends 'NACL::MTask::DataSet';

use Data::Dumper;

use NACL::APISet::Exceptions::ResponseException;
use NACL::C::VolumeFile;
use NACL::Exceptions::UnexpectedState;

sub create {
    my ($self_or_pkg, %args) = @_;
    my $self = $self_or_pkg->_new_if_required(%args);

    my $dataset_directory = $self->_create_dataset_directory();

    my $file = $self->vbn_bad_file();

    $file->write(
        content => '8BADF00D'x1024,
    );
    
    $file->punch_hole(
        'hole-range-info' => [
            { 'hole-start' => 0, 'hole-size' => 4096 },
        ],
        'punch-bad' => 1,
        determine_appropriate_nodescope_node => 1,
    );

    $self->_write_manifest_file();

    $self->_fixup_permissions();

    return $self;
}

sub vbn_bad_file {
    my ($self) = @_;
    my $dataset_directory = $self->_get_dataset_directories();

    return NACL::C::VolumeFile->new(
        command_interface => $dataset_directory->command_interface(),
        vserver => $dataset_directory->vserver(),
        path => $dataset_directory->path()."/".$self->_file_name(),
    );
}

sub _file_name {
    return "file_with_vbn_bad";
}

sub _validate_file_has_vbn_bad {
    my ($self, $file) = @_;

    try {
        # Forcing ZAPI to make sure we get a predictable error.
        $file->read(apiset_must => { interface => 'ZAPI' });
        NACL::Exceptions::UnexpectedState->throw(
            "expected file read to return an Input/output error because ".
            "of VBN_BAD",
            monitoring_fields => [],
        );
    } catch NACL::APISet::Exceptions::ResponseException with_handler {
        my $ex = shift;

        if ($ex->text() !~ /Input.*output.*error/i) {
            $ex->throw();
        }
    };
}

sub validate {
    my ($self) = @_;

    $self->_validate_file_has_vbn_bad($self->vbn_bad_file());

    return $self;
}

1;
