# $Id$
# Copyright (c) 2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary common routines for any NACL::MTask::Group class
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Group::_Group;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR validate_with SCALARREF HASHREF
    OBJECT UNDEF);

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = (
        all => [
            qw(
                node_objs
                nodes
                )
        ]
    );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
} ## end BEGIN

=head1 NAME

NACL::MTask::Group::_Group

=head1 DESCRIPTION

Common code for any NACL::MTask::Group class

=head1 METHODS

=head2 nodes

    my @nodes = $cgroup->nodes();
    my $node  = $cgroup->nodes();

The intent of this method is to provide a "names" only view of the
node_set().  The node_set() is a Component Set object, while nodes()
returns an array of the node names in array context or the first node
name in scalar context.

=head2 node_objs

    my @nodes = $cgroup->node_objs();
    my $node  = $cgroup->node_objs();

The intent of this method is to provide an array of the node objects in
the node_set().  The node_set() is a Component Set object, while node_objs()
returns an array of the node in array context or the first node
in scalar context.

=cut

sub node_objs {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );
    my @node_objs = $self->node_set()->get_all_members();
    $Log->exit() if $may_exit;
    return wantarray ? @node_objs : $node_objs[0];
} ## end sub node_objs

sub nodes {
    $Log->enter() if $may_enter;
    my $self = shift;
    $self->_verify_invocation( style => 'instance_only' );
    my @node_objs = $self->node_objs();
    my @node_names = map $_->node(), @node_objs;
    $Log->exit() if $may_exit;
    return wantarray ? @node_names : $node_names[0];
} ## end sub nodes


1;
