# $Id: //depot/prod/test/main/lib/NACL/MTask/MBUSTest.pm#1 $
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MBUSTest Task Module
## @author clausner@netapp.com
## @status shared
## @pod here

package NACL::MTask::MBUSTest;

use strict;
use warnings;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate validate_with BOOLEAN SCALAR HASHREF OBJECT SCALARREF);
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::APISet;

use NACL::ComponentUtils qw(Dumper);

#use Data::Dumper;
use Time::ParseDate qw(parsedate);

=head1 NAME

NACL::MTask::MBUSTest

=head1 DESCRIPTION

C<NACL::MTask::MBUSTest> provides a few well-defined operations
that either send or receive messages through the MRT process.
This is the action of running the Freebsd command "mbustest".

mbustest command startups up a client that can either send or
receive messages that are sent through mrt. mbus is an acroynm
for Message BUS. It's a form of communicaiton between nodes
within a cluster.

MBUSTest is the test client to transmit messages and receive
messages through the MRT process.

mrt task is a command that will be running on the filer by default.
This will serve as a replacement for RPCs. MRT stands for
Message Router. This process controls where messages can go. 

This task class is not intended to be initialized directly.

=head1 METHODS

=cut

use base qw (NACL::MTask::MTask);

use Class::MethodMaker [
    hash => 'output',
];

=head2 get_routing_hash

    (Instance method)
      my %output = $mbus_object->get_routing_hash();

    Method that will return the parsed output hash of the current
    routing network that this test client can see.

=over

=back

=cut

sub get_routing_hash {
    my $self = shift;

    my %routing_hash = ();
    my $output = $self->output();
    my %output = %$output;

    if(defined($output{"route"})) {
        my $routing_ref = $output{"route"};
        %routing_hash = %$routing_ref;
    }

    return \%routing_hash;

} ## end sub get_routing_hash

1;

