
#
# Copyright (c) 2011-2021 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MCC Disk Stub MTask Module
## @author dl-mcc-dpg-qa@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Disk;

use strict;
use warnings;
use Carp;

use base qw (NACL::MTask::MTask);

use NACL::C::Disk ();

use NACL::Exceptions::NoElementsFound qw(:try);

use NATE::BaseException qw(:try);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR);

use Data::Dumper;

=head1 NAME

NACL::MTask::MCC::Disk

=head1 DESCRIPTION

This task is to be used for C-Mode MetroCluster testing.

    # Start test
    use NACL::MTask::MCC::Disk;
    ...
    my @disks = XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
    ...
    # Perform test here
    ...

    # End test

=cut

=head1 Public METHODS

There is no public constructor for this class.  A
NACL::MTask::MCC::Node object is obtained via calls such as

=over

    my @disks = @{ XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX };

=back

=cut

# automatically generate the  Non-public constructor (includes a call to 'init')
# and member accessor methods
use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
                { -type => 'NACL::C::Disk' },
                '_disk_component',
              ],
                        ];

# validate the parameters passed in and what else?
sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
#TBD: several parameters will be passed in here to uniquely identify a disk, which will al be used to find the particular NACL::C::Disk component currently associated with them.
                  },
                            );

#    $self->_disk_component(XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx);
    $Log->exit() if $may_exit;
}

=head2 get_Disk_component()

    my $disk_comp = $disk->get_Disk_component();

Returns the component object  (NACL::C::Disk) of the disk

=cut

sub get_Disk_component {
    my $self = shift;

    return $self->_disk_component();
}


=head2 get_owner()

    my $owner = $disk->get_owner();

Returns a MCC::Node object  This represents a node in Metrocluster which own's
the disk at the moment.

=cut

sub get_owner{
    my $self = shift;

    my $disk_comp = $self->_disk_component();

    # and here we use $disk_comp to find out the current owner of the disk
    return 1;
}

=head2 get_original_home_owner()

    my $orig_owner = $disk->get_original_home_owner();

Returns a MCC::Node object  This represents a node in Metrocluster which is
the origional owner of the disk

=cut

sub get_original_home_owner{
    my $self = shift;

    my $disk_comp = $self->_disk_component();

    # and here we use $disk_comp to find out the origional owner of the disk
    return 1;
}

=head2 get_DR_home_owner()

    my $dr_home_owner = $disk->get_DR_home_owner();

Returns a MCC::Node object  This represents a node in Metrocluster which is
the DR home owner of the disk

=cut

sub get_DR_home_owner{
    my $self = shift;

    my $disk_comp = $self->_disk_component();

    # and here we use $disk_comp to find out the DR home owner of the disk
    return 1;
}

1;
