#
#
# Copyright (c) 2009-2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when a method use is
##          deemed to be invalid.  E.g., a temporary work-around method
##          that is able to determine that it's usage is no longer valid
##          due to changes in ONTAP.
##
## @author dl-mcc-dpg-qa@netapp.com
## @status Shared
## @pod here

package NACL::MTask::MCC::Exceptions::InvalidMethodUsage;

use strict;
use warnings;
use Carp;

use base qw(NACL::MTask::MCC::Exceptions::BaseException);

=pod

=head1 Package Name

NACL::MTask::MCC::Exceptions::InvalidMethodUsage

=head1 Author/Maintainer

dl-mcc-dpg@netapp.com

=head1 Description

NACL::MTask::MCC::Exceptions::InvalidMethodUsage is a subclass of
NATE::BaseException that will be thrown when a method use in the MCC library
is deemed to be invalid.  E.g., a temporary work-around method
that is able to determine that it's usage is no longer valid
due to changes in ONTAP.

    NACL::MTask::MCC::Exceptions::InvalidMethodUsage->
                                                     throw('my error message');

See NACL::MTask::MCC::Metrocluster::get_singleton_DR_group() for an example
usage.

=head1 See Also

Package inherits from the BaseException module in NACL::MTask::MCC::Exceptions.

=head2 Base Class

NACL::MTask::MCC::Exceptions::BaseException

=cut

# No functionality changes compared to NATE::BaseException, just more specific
# object identity wrt MCC systems.

1;

