#
#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NACL::MTask::MCC::Exceptions::State
##          that will be thrown when MCC is in state
##          complete-with-manual-recovery
##
## @author dl-mcc-dpg-qa@netapp.com
## @status Shared
## @pod here

package NACL::MTask::MCC::Exceptions::State::CompletedWithManualRecovery;

use strict;
use warnings;
use Carp;

use parent qw(NACL::MTask::MCC::Exceptions::State);

=pod

=head1 Package Name

NACL::MTask::MCC::Exceptions::State::CompletedWithManualRecovery

=head1 Author/Maintainer

dl-mcc-dpg-qa@netapp.com

=head1 Description

NACL::MTask::MCC::Exceptions::State::New is a subclass of
NATE::Mtask::MCC:Exceptions::State that will be thrown when MCC is
in state completed-with-manual-recovery.

It is designed to be used by a script to deal with the expected
recovery steps outlined in metrocluster operation show in the errors field
when mcc is in this state.  If a script can't deal with the recovery
it should not attempt to catch the error.

    NACL::MTask::MCC::Exceptions::State::CompletedWithManualRecovery->
                                                     throw('my error message');


=head1 See Also

Package inherits from the State module in NACL::MTask::MCC::Exceptions.

=head2 Base Class

NACL::MTask::MCC::Exceptions::State

=cut

# No functionality changes compared to NATE::BaseException, just more specific
# object identity wrt MCC systems.

1;

