package NACL::MTask::MCC::Menu::Metrocluster;

use strict;
use warnings;
use Carp;


use base qw (NACL::MTask::MTask);

use NATE::BaseException qw(:try);
use NACL::MTask::MCC::Exceptions::BaseException;

use NACL::MTask::MCC::Metrocluster;
use NACL::MTask::MCC::Network::Topology;
#use NACL::MTask::MCC::Network::Switch;
use Tharn qw( subtest);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

#double check these qw are used TODO
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use Data::Dumper;

=head1 NAME

NACL::MTask::MCC::MetroCluster::Menu
??::Menu

=head1 DESCRIPTION

Displays an interactive cli utility menu to call common library functionality.

=cut

=head2 menu()

B<EXPERIMENTAL!>

This will be removed at some point to be replaced with a different structure
(put the same/similar functionality).

Displays an interactive cli utility menu to call common library functionality.


=cut

use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [
        { -type => 'NACL::MTask::MCC::Metrocluster' },
        'metrocluster',
              ],
    scalar => [
        qw/ nodes
          /,
              ],
                       ];

# validate the parameters passed in and what else?
sub init {
    $Log->enter() if $may_enter;
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            metrocluster => { isa   => 'NACL::MTask::MCC::Metrocluster', optional => 1 },
            nodes        => { type => ARRAYREF | SCALAR, optional => 1 }, #TODO only optional if MC above...
        },
                            );

    #nodes should be an array ref
    my $nodes = ref $opts{'nodes'} ? $opts{'nodes'} : [ $opts{'nodes'} ];

    NACL::MTask::MCC::Menu::mprint("Loading MCC menu (may take several minutes)...\n");

    my $metrocluster = $opts{'metrocluster'} ||
            NACL::MTask::MCC::Metrocluster->new(nodes => $nodes);
if (!$metrocluster) { print "COULD NOT INITIALIZE MCC OBJECT\n"; } #throw err TODO

    # set members per the passed constructor arguments.
    $self->metrocluster($metrocluster);
    #$self->metrocluster($opts{'metrocluster'}) ||
    #NACL::MTask::MCC::Metrocluster->new(nodes => $nodes);
    #$self->_site(        $opts{'site'});
    $Log->exit() if $may_exit;
    return;
}




sub main_menu {
    my $self = shift;

    my $metrocluster = $self->metrocluster;

    my $dr_state;  #TODO fix this, take out and look up dynamically

    my $command = {

        'dr_normal'     => { name => 'cycle to dr state: normal',
                              code => sub {
                                  my $menu = shift;
                                  my $site = $menu->
                                      get_variable(name=>'survivor-site');
                                  my $verify_disks = $menu->
                                      get_variable(name=>'verify-disks');
                                      #my $metrocluster = $self->metrocluster;
                                  my $forced_on_disaster = $menu->
                                      get_variable(name=>'forced_on_disaster');
                                  my $do_peer_metrocluster_site_nodes_disaster = $menu->
                                      get_variable(name=>'do_peer_metrocluster_site_nodes_disaster');
                                  my $disaster_type = $menu->
                                      get_variable(name=>'disaster_type');
                                  my $panic_core_dump = $menu->
                                      get_variable(name=>'panic_core_dump');
                                  my $disable_core_file = $menu->
                                      get_variable(name=>'disable_core_file');
                                  my $inhibit_takeover = $menu->
                                      get_variable(name=>'inhibit_takeover');
                                  my $skip_lif_migration = $menu->
                                      get_variable(name=>'skip_lif_migration');
                                  my $do_isl_link_down_after_disaster = $menu->
                                      get_variable(name=>'do_isl_link_down_after_disaster');
                                  my $do_isl_link_down = $menu->
                                      get_variable(name=>'do_isl_link_down');
                                  my $do_intercluster_network_interface_disable = $menu->
                                      get_variable(name=>'do_intercluster_network_interface_disable');
                                  my $do_isl_link_up = $menu->
                                      get_variable(name=>'do_isl_link_up');
                                  my $override_vetoes = $menu->
                                      get_variable(name=>'override-vetoes');
                                  $metrocluster->get_cluster( site=>$site)->
                                      cycle_to_DR_state(
                                          state  => 'normal',
                                          do_peer_metrocluster_site_nodes_disaster => $do_peer_metrocluster_site_nodes_disaster,
                                          'forced-on-disaster' => $forced_on_disaster,
                                          do_intercluster_network_interface_disable  => $do_intercluster_network_interface_disable,
                                          do_isl_link_down => $do_isl_link_down,
                                          do_isl_link_down_after_disaster => $do_isl_link_down_after_disaster,
                                          do_isl_link_up => $do_isl_link_up,
                                          'override-vetoes' => $override_vetoes,
                                          #these are only for type 'custom'
                                          disaster_type => $disaster_type,
                                          panic_core_dump => $panic_core_dump,
                                          disable_core_file => $disable_core_file,
                                          inhibit_takeover => $inhibit_takeover,
                                          skip_lif_migration => $skip_lif_migration,
                                          verify => { disks => $verify_disks });
                               },
                            },

        'dr_switchover' => { name => 'cycle to dr state: switchover_completed',
                              code => sub {
                                  my $menu = shift;
                                  my $site = $menu->
                                      get_variable(name=>'survivor-site');
                                  my $verify_disks = $menu->
                                      get_variable(name=>'verify-disks');
                                  my $forced_on_disaster = $menu->
                                      get_variable(name=>'forced_on_disaster');
                                  my $do_peer_metrocluster_site_nodes_disaster = $menu->
                                      get_variable(name=>'do_peer_metrocluster_site_nodes_disaster');
                                  my $disaster_type = $menu->
                                      get_variable(name=>'disaster_type');
                                  my $panic_core_dump = $menu->
                                      get_variable(name=>'panic_core_dump');
                                  my $disable_core_file = $menu->
                                      get_variable(name=>'disable_core_file');
                                  my $inhibit_takeover = $menu->
                                      get_variable(name=>'inhibit_takeover');
                                  my $skip_lif_migration = $menu->
                                      get_variable(name=>'skip_lif_migration');
                                  my $do_isl_link_down_after_disaster = $menu->
                                      get_variable(name=>'do_isl_link_down_after_disaster');
                                  my $do_ic_lif_disable_after_disaster = $menu->
                                      get_variable(name=>'do_ic_lif_disable_after_disaster');
                                  my $do_isl_link_down = $menu->
                                      get_variable(name=>'do_isl_link_down');
                                  my $do_intercluster_network_interface_disable = $menu->
                                      get_variable(name=>'do_intercluster_network_interface_disable');
                                  my $do_isl_link_up = $menu->
                                      get_variable(name=>'do_isl_link_up');
                                  my $override_vetoes = $menu->
                                      get_variable(name=>'override-vetoes');
                                  $metrocluster->get_cluster( site=>$site)->
                                      cycle_to_DR_state(
                                          state  => 'switchover_completed',
                                          do_peer_metrocluster_site_nodes_disaster => $do_peer_metrocluster_site_nodes_disaster,
                                          'forced-on-disaster' => $forced_on_disaster,
                                          do_isl_link_down => $do_isl_link_down,
                                          do_intercluster_network_interface_disable => $do_intercluster_network_interface_disable,
                                          do_isl_link_down_after_disaster => $do_isl_link_down_after_disaster,
                                          do_ic_lif_disable_after_disaster => $do_ic_lif_disable_after_disaster,
                                          do_isl_link_up => $do_isl_link_up,
                                          'override-vetoes' => $override_vetoes,
                                          #these are only for type 'custom'
                                          disaster_type => $disaster_type,
                                          panic_core_dump => $panic_core_dump,
                                          disable_core_file => $disable_core_file,
                                          inhibit_takeover => $inhibit_takeover,
                                          skip_lif_migration => $skip_lif_migration,
                                          verify => { disks => $verify_disks });
                               },
                            },

        'dr_heal_aggrs' => { name => 'cycle to dr state: heal_aggrs_completed',
                              code => sub {
                                  my $menu = shift;
                                  my $site = $menu->
                                      get_variable(name=>'survivor-site');
                                  my $verify_disks = $menu->
                                      get_variable(name=>'verify-disks');
                                  my $forced_on_disaster = $menu->
                                      get_variable(name=>'forced_on_disaster');
                                  my $do_peer_metrocluster_site_nodes_disaster = $menu->
                                      get_variable(name=>'do_peer_metrocluster_site_nodes_disaster');
                                  my $disaster_type = $menu->
                                      get_variable(name=>'disaster_type');
                                  my $panic_core_dump = $menu->
                                      get_variable(name=>'panic_core_dump');
                                  my $disable_core_file = $menu->
                                      get_variable(name=>'disable_core_file');
                                  my $inhibit_takeover = $menu->
                                      get_variable(name=>'inhibit_takeover');
                                  my $skip_lif_migration = $menu->
                                      get_variable(name=>'skip_lif_migration');
                                  my $do_isl_link_down_after_disaster = $menu->
                                      get_variable(name=>'do_isl_link_down_after_disaster');
                                  my $do_isl_link_down = $menu->
                                      get_variable(name=>'do_isl_link_down');
                                  my $do_isl_link_up = $menu->
                                      get_variable(name=>'do_isl_link_up');
                                  $metrocluster->get_cluster( site=>$site)->
                                      cycle_to_DR_state(
                                          state  => 'heal_aggrs_completed',
                                          do_peer_metrocluster_site_nodes_disaster => $do_peer_metrocluster_site_nodes_disaster,
                                          'forced-on-disaster' => $forced_on_disaster,
                                          do_isl_link_down => $do_isl_link_down,
                                          do_isl_link_down_after_disaster => $do_isl_link_down_after_disaster,
                                          do_isl_link_up => $do_isl_link_up,
                                          #these are only for type 'custom'
                                          disaster_type => $disaster_type,
                                          panic_core_dump => $panic_core_dump,
                                          disable_core_file => $disable_core_file,
                                          inhibit_takeover => $inhibit_takeover,
                                          skip_lif_migration => $skip_lif_migration,
                                          verify => { disks => $verify_disks });
                               },
                            },

        'dr_heal_roots' => { name => 'cycle to dr state: heal_roots_completed',
                              code => sub {
                                  my $menu = shift;
                                  my $site = $menu->
                                      get_variable(name=>'survivor-site');
                                  my $verify_disks = $menu->
                                      get_variable(name=>'verify-disks');
                                  my $forced_on_disaster = $menu->
                                      get_variable(name=>'forced_on_disaster');
                                  my $do_peer_metrocluster_site_nodes_disaster = $menu->
                                      get_variable(name=>'do_peer_metrocluster_site_nodes_disaster');
                                  my $disaster_type = $menu->
                                      get_variable(name=>'disaster_type');
                                  my $panic_core_dump = $menu->
                                      get_variable(name=>'panic_core_dump');
                                  my $disable_core_file = $menu->
                                      get_variable(name=>'disable_core_file');
                                  my $inhibit_takeover = $menu->
                                      get_variable(name=>'inhibit_takeover');
                                  my $skip_lif_migration = $menu->
                                      get_variable(name=>'skip_lif_migration');
                                  $metrocluster->get_cluster( site=>$site)->
                                      cycle_to_DR_state(
                                          state  => 'heal_roots_completed',
                                          do_peer_metrocluster_site_nodes_disaster => $do_peer_metrocluster_site_nodes_disaster,
                                          'forced-on-disaster' => $forced_on_disaster,
                                          #these are only for type 'custom'
                                          disaster_type => $disaster_type,
                                          panic_core_dump => $panic_core_dump,
                                          disable_core_file => $disable_core_file,
                                          inhibit_takeover => $inhibit_takeover,
                                          skip_lif_migration => $skip_lif_migration,
                                          verify => { disks => $verify_disks });
                               },
                            },

        'dump_mcc'       => { name => 'dump metrocluster object',
                              code => sub {
my $local_setting = shift;
$Log->comment(NACL::MTask::MCC::Utils->dump(
                     target              => $metrocluster,
                     #show_shared_storage => $local_setting->{'verify-disks'}
                     )
                     );
                                          }
                            },

        'disk_verify'    => { name => 'do disk verification',
                              code => sub {
foreach my $dr (sort keys %{$metrocluster->get_DR_groups()}) {
    my $message = "validate shared storage in dr group $dr";
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";

    $metrocluster->get_DR_groups()->{$dr}->get_shared_storage()->validate_disk_states();
}
                                          },
                            },
        'interconnect_verify'    => { name => 'do interconnect verification',
                              code => sub {
    my $message = 'validate interconnect for all mcc';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";

try {
    my $disaster_status = $metrocluster->get_disaster_status(include_HA => 1);
    $metrocluster->{_interconnect}->validate_metrocluster_interconnect_state(
        disaster_status => $disaster_status);
}
                                          },
                            },
        'node_verify'    => { name => 'do node show verification',
                              code => sub {
    my $message = 'validate metrocluster node show fields';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validate_node();
                                          },
                            },
        'operation_verify'    => { name => 'do operation error verification',
                              code => sub {
    my $message = 'validate metrocluster operation show errors';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validate_operation();
                                          },
                            },
        'vserver_verify'    => { name => 'do vserver verification',
                              code => sub {
    my $message = 'validate vservers for all aggregates in mcc';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validate_vservers();
                                          },
                            },
        'run_all_validations'    => { name => 'run all validations',
                              code => sub {
    my $message = 'run all validations';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->run_all_validations();
                                          },
                            },
        'check_config_cluster_diff'    => { name => 'do validation check config cluster diff',
                              code => sub {
    my $message = 'validate metrocluster check config cluster diff - should be empty';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validate_check_config_cluster_diff();
                                          },
                            },
        'check_config_vserver_diff'    => { name => 'do validation check config vserver diff',
                              code => sub {
    my $message = 'validate metrocluster check config vserver diff - should be empty';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validate_check_config_vserver_diff();
                                          },
                            },
        'validation_report'    => { name => 'dump validation report',
                              code => sub {
    my $message = 'dump validate report for all validation performed';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";
    $metrocluster->validation_report();
                                          },
                            },
        'fabric_version_verify'    => { name => 'do switch version verification',
                              code => sub {
    my $message = 'validate versions for bridges and switches';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";

try {
    $metrocluster->version_is_valid();
}
                                          },
                            },
        'wafl_iron_validate'    => { name => 'validate data integrity - wafliron and aggr verify',
                              code => sub {
    my $message = 'run wafliron and aggr verify for all aggregates';
    $Log->comment($message);
    print ntest::NTEST_STDOUT "$message\n";

try {
    $metrocluster->verify_data_integrity();
}
                                          },
                            },

        'mcc_check' => { name => 'metrocluster check',
                              code => sub {
                                  my $menu = shift;
                                  my $site = $menu->
                                      get_variable(name=>'survivor-site');
                                  $metrocluster->check( site=>$site)
                               },
                            },
        'show_disasters' => { name => 'show DR and HA states',
                              code => sub {
try {
    my $disaster_status = $metrocluster->get_disaster_status(include_HA => 1);
    my $status_message = "overall disaster status:\n" .
                         NACL::MTask::MCC::Utils->dump($disaster_status);
    $Log->comment($status_message);
}
catch NACL::MTask::MCC::Exceptions::BaseException with {
    my $exception = shift;
    my $text = $exception->text();

    my $message = sprintf("%s", <<"END_PRINT");

Exception occurred during disaster status queries:

$text

This may be due to the DR or HA state currently transitioning, or it may be due
to an invalid DR or HA state.  You may safely repeat this query.  If this
query continues to fail in this manner for several minutes, it is likely that
the latter case is the reason for this failure, and you will need to triage
the MCC system manually.

END_PRINT

    $Log->comment($message);
};
                                          },
                            },
        'mcc_info'    => { name => 'show mcc info',
                           code => sub {
                                   my $disaster_status =
                                   $metrocluster->L_get_disaster_status(known_live_sites => [ 'A', 'B' ],
                                                                include_HA       => 1);
                                   my $constructor_log = Tharn::param('LOGDIR') .
                                                         '/MCC_Constructor.log';

                                   $metrocluster->_log_some_mcc_system_info(
                                       disaster_status => $disaster_status,
                                       constructor_log => $constructor_log);

                               },
                       },
        'refresh_mcc_ci'    => { name => 'refresh command interfaces',
                           code => sub {
                                   $metrocluster->refresh_command_interfaces();
                               },
                       },

                  };

  my $main_menu = { 'menu' => {
                        'items' => $command,
                         order => [ qw(dr_normal dr_switchover dr_heal_aggrs dr_heal_roots)],
                        'variables' => {
                            'verify-disks' => { list => sub { qw( 0 1 ) },
                                                default => 1,
                                            },
                            'survivor-site' => { list => sub { qw( A B ) },
                                                default => 'A',
                                            },
                            'forced_on_disaster' => { list => sub { qw( 0 1 ) },
                                                default => '1',
                                            },
                            'panic_core_dump' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'disable_core_file' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'inhibit_takeover' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'skip_lif_migration' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'do_isl_link_down' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'do_intercluster_network_interface_disable' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'do_isl_link_down_after_disaster' => { list => sub { qw( 0 1 ) },
                                                default => '1',
                                            },
                            'do_ic_lif_disable_after_disaster' => { list => sub { qw( 0 1 ) },
                                                default => '1',
                                            },
                            'do_isl_link_up' => { list => sub { qw( 0 1 ) },
                                                default => '1',
                                            },
                            'override-vetoes' => { list => sub { qw( 0 1 ) },
                                                default => '0',
                                            },
                            'do_peer_metrocluster_site_nodes_disaster' =>
                                { list => sub { qw( halt halt_inhibit_takeover none panic panic_nocore power reboot reboot_inhibit_takeover custom ) },
                                                default => 'halt_inhibit_takeover',
                                            },
                            'disaster_type' =>
                                { list => sub { qw( halt none power reboot ) },
                                                default => undef,
                                            },

                        },
                        'info' => {code => sub {
                                     my $menu = shift;

                                     #TODO don't like this, there should be a default for vars maybe...

                                     unless (defined $menu->get_variable(name=>'verify-disks')) {
                                             $menu->set_variable(name=>'verify-disks',
                                         value => 1 );
                                     }
                                     unless (defined $menu->get_variable(name=>'survivor-site')) {
                                             $menu->set_variable(name=>'survivor-site',
                                         value => 'A' );
                                     }
                                     unless (defined $menu->get_variable(name=>'forced_on_disaster')) {
                                             $menu->set_variable(name=>'forced_on_disaster',
                                         value => 1 );
                                     }
                                     unless (defined $menu->get_variable(name=>'do_peer_metrocluster_site_nodes_disaster')) {
                                             $menu->set_variable(name=>'do_peer_metrocluster_site_nodes_disaster',
                                         value => 'halt_inhibit_takeover' );
                                     }
									 #this won't work on a vsim
									 #unless (defined $menu->get_variable(name=>'do_isl_link_up')) {
									 #$menu->set_variable(name=>'do_isl_link_up',
									 ##value => 1 );
									 #}
                                     my $nodes =
                                         $metrocluster->get_all_mcc_nodes();
                                         $nodes = [map {$_->get_name()} @{$nodes}];
                                     $menu->set_variable(name=>'log_nodes',
                                         value => $nodes );
                                     #print "NODES:" . Dumper $nodes;
                                     print "MCC NODES:" . Dumper $menu->get_variable(name=>'log_nodes');
                                     return;

                            }
                        },
                    } ,
                    'id'    => 'mcc',
                    'name'    => 'mcc',
  };

  NACL::MTask::MCC::Menu->add_menu(menu => $self->topology_menu(), add_to_menu => $main_menu);

   return $main_menu;
}

###########topology menu code ###########
#
#
sub topology_menu {
    my $self = shift;

    my $metrocluster = $self->metrocluster;

    my $switch = {
        'switch_refresh'     => { name => 'refresh switch connection',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $switchimpl =
                                         $switchobj->get_switch_impl();
                                     my $apiset =
                                         $switchimpl->get_switch_APISet();
                                     $apiset->api_reconnect_session();
                                 },
                         },
        'switch_enable'     => { name => 'enable switch',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     $switchobj->enable();
                                 },
                         },
        'switch_disable'    => { name => 'disable switch',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     $switchobj->disable();
                                 },
                         },
        'fibrebridge_enable'    => { name => 'enable fibrebridge ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports =
                                         $switchobj->get_fibreBridge_ports();
                                     map { $_->enable() } @{ $ports };
                                 },
                         },
        'fibrebridge_disable'    => { name => 'disable fibrebridge ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports =
                                         $switchobj->get_fibreBridge_ports();
                                     map { $_->disable() } @{ $ports };
                                 },
                         },
        'fcvi_enable'    => { name => 'enable fcvi ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_fcvi_ports();
                                     map { $_->enable() } @{ $ports };
                                 },
                         },
        'fcvi_disable'    => { name => 'disable fcvi ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_fcvi_ports();
                                     map { $_->disable() } @{ $ports };
                                 },
                         },
        'storage_enable'    => { name => 'enable storage ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_storage_ports();
                                     map { $_->enable() } @{ $ports };
                                 },
                         },
        'storage_disable'    => { name => 'disable storage ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_storage_ports();
                                     map { $_->disable() } @{ $ports };
                                 },
                         },
        'isl_enable'    => { name => 'enable isl ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_isl_ports();
                                     print Dumper $ports;
                                     map { $_->enable() } @{ $ports };
                                     print Dumper $ports;
                                 },
                         },
        'isl_disable'    => { name => 'disable isl ports',
                                 code => sub {
                                     my $menu = shift;
                                     my $switchobj = $self->get_switch($menu);
                                     my $ports = $switchobj->get_isl_ports();
                                     print Dumper $ports;
                                     map { $_->disable() } @{ $ports };
                                     print Dumper $ports;
                                 },
                         },
        'switch_description'    => { name => 'switch description',
            code => sub {
                my $menuobj = shift;
                my $fabricnum = $menuobj->get_variable(name => 'fabric_var');
                my $drs = $metrocluster->get_DR_groups();
                my $dr_group = $drs->{$menuobj->get_variable(name => 'dr_group_var' )};
                my $topology = $dr_group->get_network_topology();
                my $fabric = @{$topology->get_intersite_fabrics()}[$fabricnum];
                my $switchsite = $menuobj->get_variable(name => 'switch_var');
                my $desc = $fabric->get_switch(site => $switchsite)->get_description();
                print "SWITCH:$desc\n";
                NACL::MTask::MCC::Menu::mprint("---\n");
                NACL::MTask::MCC::Menu::mprint("SWITCH:$desc\n");
                my $switchobj = $self->get_switch($menuobj);
                #TODO this does not work with the current switch implementation
                #we should wait for the library to support is_active calls for
                #ports...
                #my $switchimpl = $switchobj->get_switch_impl();
                #$switchimpl->_port_info_cache($switchimpl->_gather_port_info());

                #print Dumper $switchobj;


            },

                         },

    };

    my $switch_info = {
            name => 'switch description',
            hierarchy => 0,
            code => sub {
                my $menuobj = shift;
                my $fabricnum = $menuobj->get_variable(name => 'fabric_var');
                my $drs = $metrocluster->get_DR_groups();
                my $dr_group = $drs->{$menuobj->get_variable(name => 'dr_group_var' )};
                my $topology = $dr_group->get_network_topology();
                my $fabric = @{$topology->get_intersite_fabrics()}[$fabricnum];
                my $switchsite = $menuobj->get_variable(name => 'switch_var');
                my $switchobj = $fabric->get_switch(site => $switchsite);
                my $desc = $fabric->get_switch(site => $switchsite)->get_description();
                print "SWITCH:$desc\n";
                #NACL::MTask::MCC::Menu::mprint("---\n");
                #NACL::MTask::MCC::Menu::mprint("SWITCH:$desc\n");
                my $out = "switch: $desc ";
                #apparently not implemented yet...?
                #$out .= $switchobj->is_enabled() ? "[enabled] " : "[disabled] ";
                $out .= $switchobj->get_switch_name() . " ";
                $out .= $switchobj->get_vendor();

                return ($self->switch_info($menuobj));
                #return($out);



            },
    };

    my $switch_site = {
        'switch' => { name => 'switch',
                        menu => {
                            items => $switch,
                            prerequisites => {
                                variables => [ qw(switch_var) ] ,
                            },
                            info  => $switch_info,
                        },
                    },
    };

    my $fab_switch = {

        'switch_site' => { name => 'switch site',
                             menu => {
                                 info => {
                                        hierarchy => 1,
                                        code => sub { return 'switch site:' .
                                            shift->get_variable(
                                                name=>'switch_var');
                                        },
                                 },
                                 items => $switch_site,
                                 prerequisites => {
                                     variables => [ qw(fabric_var) ] ,
                                 },
                                 variables => {
                                     switch_var => {
                                         name => 'choose switch site',

                                      list => sub { return qw(A B) },
                                  },
                                  #testvar => {
                                  #list => sub { return qw(this is a test) },
                                  #},
                                 },
                             },
                         },
        'enable_fab'    => { name => '*enable fabric',
                             code => sub {
                               NACL::MTask::MCC::Menu::mprint(
                                   "library not yet implemented\n");
                             },
                         },
        'disable_fab'    => { name => '*disable fabric',
                             code => sub {
                               NACL::MTask::MCC::Menu::mprint(
                                   "library not yet implemented\n");
                             },
                         },

    };

    my $fabric = {
        'fabric' => { name => 'fabric',
                        menu => {
                            info => {
                                        hierarchy => 1,
                                        code => sub { return 'fabric:' .
                                            shift->get_variable(
                                                name=>'fabric_var');
                                        },
                                    },
                            items => $fab_switch,
                            prerequisites => {
                                variables => [ qw(dr_group_var) ] ,
                            },
                            variables => {
                                fabric_var => {
                                 list => sub {
                                     my $menuobj = shift;
                                     my $drs = $metrocluster->get_DR_groups();
                                     my $dr_group = $drs->{$menuobj->get_variable(name => 'dr_group_var' )};
                my $topology = $dr_group->get_network_topology();
                my $fabrics = $topology->get_intersite_fabrics();
                #print Dumper $fabrics;
                #return (@{$fabrics});
                return qw(0 1); #TODO change back when fabrics == keys
                                 },
                                },
                            },
                        },
                    },
                                 'show_dr_groups' => {
                                     name => 'show dr groups',
                                     code => sub {
                                         NACL::MTask::MCC::Menu::mprint( (join ',', keys %{$metrocluster->get_DR_groups()}) . "\n" ) },
                                         #return ( join ',', keys %{$metrocluster->get_DR_groups();}) },
                                 },
                                     #code => sub { print Dumper $metrocluster->get_DR_groups(); }, },
    };

    my $menu = {
        'dr_group' => { name => 'dr group',
                        menu => {
                            info => {
                                        hierarchy => 1,
                                        code => sub { return 'dr group:' .
                                            ( shift->get_variable(
                                                name=>'dr_group_var') || '');
                                        },
                                    },
                            items => $fabric,

                            variables => {
                                dr_group_var => {
                                 list => sub { return keys %{$metrocluster->get_DR_groups();} },
                                },
                            },
                        },
                        info => {hierarchy => 1,
                                 code => sub {
                                     return keys %{$metrocluster->get_DR_groups();} },
                             },

                        },

    };

    my $structure = {
        menu  => { items => $menu },
        name  => 'network topology',
        id    => 'topology',
        #prompt => '',
        #order => [],
    };



    #return $menu;
    return $structure;
}


sub get_switch {
    my $self = shift;
    my $menuobj = shift;
    my $metrocluster = $self->metrocluster;

    my $fabricnum = $menuobj->get_variable(name => 'fabric_var');
    my $drs = $metrocluster->get_DR_groups();
    my $dr_group = $drs->{$menuobj->get_variable(name => 'dr_group_var' )};
    my $topology = $dr_group->get_network_topology();
    my $fabric = @{$topology->get_intersite_fabrics()}[$fabricnum];
    my $switchsite = $menuobj->get_variable(name => 'switch_var');
    my $switch = $fabric->get_switch(site => $switchsite);

    my $switchimpl = $switch->get_switch_impl();
    my $apiset = $switchimpl->get_switch_APISet();
    $apiset->api_reconnect_session() if !$apiset->api_is_session_alive();
    return $switch;

}

sub switch_info {
    my $self = shift;
    my $menuobj = shift;
    my $metrocluster = $self->metrocluster;

    my $switchobj = $self->get_switch($menuobj);
    my $name          = $switchobj->get_description();
print "switch name:$name\n";
print NACL::MTask::MCC::Utils->dump($switchobj) if $menuobj->debug();
    my $fb_ports      = $switchobj->get_fibreBridge_ports();
    my $fcvi_ports    = $switchobj->get_fcvi_ports();
    my $storage_ports = $switchobj->get_storage_ports();
    my $isl_ports     = $switchobj->get_isl_ports();

    $switchobj->update_switch_info();

    my $ports;

    my ($fb,$fcvi,$storage,$isl);
    foreach my $port (sort @{ $fb_ports }) {
        $ports->{fibrebridge}->{$port->{_port_descriptor}}->
            {enabled} = $port->is_enabled;
    }

    foreach my $port (sort @{ $fcvi_ports }) {
        $ports->{fcvi}->{$port->{_port_descriptor}}->
            {enabled} = $port->is_enabled;
    }

    foreach my $port (sort @{ $storage_ports }) {
        $ports->{storage}->{$port->{_port_descriptor}}->
            {enabled} = $port->is_enabled;
    }

    foreach my $port (sort @{ $isl_ports }) {
        $ports->{isl}->{$port->{_port_descriptor}}->
            {enabled} = $port->is_enabled;
    }
#                7[+] fiberbridge
#      0--[-]##############################
# FCVI 1--[-]#                            #
# FCVI 2--[+]# dpg-mcc-brcd6510-01-A-fabY #[+]--STORAGE 0
#      5--[-]#                            #
#      9--[-]#                            #
#     12--[-]##############################
#              8[+] ISL
#
    my $ascii = $self->generate_filer_art(
        switchobj=>$switchobj,
        port=>$ports
    );
    my $out = "\n$ascii\n";
    return $out;
}

sub generate_filer_art {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            switchobj  => { type => OBJECT },
            port       => { type => HASHREF },
            width      => { type => SCALAR, default=>70 },
            height     => { type => SCALAR, default=>20 },
        },
    );

    my $switchobj = $opts{switchobj};
    my $port      = $opts{port};
    my $width     = $opts{width};
    my $height    = $opts{height};

    my $auto      = 1; #auto adjust sizes as needed

    my $box;

    foreach my $type (keys %{$port}) {
        my $side;
        if ($type eq 'isl') {
            $side = 'bottom';
        } elsif ($type eq 'fibrebridge' ) {
            $side = 'top';
        } elsif ($type eq 'fcvi' ) {
            $side = 'left';
        } elsif ($type eq 'storage' ) {
            $side = 'right';
        }
        foreach my $item (keys %{$port->{$type}}) {
            $box->{$side}->{$type}->{$item} = $port->{$type}->{$item};
            $box->{$side}->{_total}++;
        }
    }

    #print Dumper  $box;
    my $strings = $self->generate_port_strings(box=>$box);
    my $minh = $box->{left}->{_total} > $box->{right}->{_total} ?
               $box->{left}->{_total} : $box->{right}->{_total};

    my $minv = $box->{top}->{_total} > $box->{bottom}->{_total} ?
               $box->{top}->{_total} : $box->{bottom}->{_total};

    $minh = @{$strings->{left}} > @{$strings->{right}} ?
            @{$strings->{left}} : @{$strings->{right}};

    $minv = @{$strings->{top}} > @{$strings->{bottom}} ?
            @{$strings->{top}} : @{$strings->{bottom}};

    my $boxv = $minv
               + 2  #border
               + 2; #padding

    my $portsizet =   length join q{},(keys %{$box->{top}})
                    + $box->{top}->{_total} #padding
                    - 6; #  _total
    my $portsizeb =  length join q{},(keys %{$box->{bottom}})
                    + $box->{bottom}->{_total} #padding
                    - 6; #  _total

    my $portsizeh = $portsizet > $portsizeb ? $portsizet : $portsizeb;

    my $name      = $switchobj->get_description();

    my $boxh = ($portsizeh > length $name ? $portsizeh : length $name )
               + 2  #border
               + 2; #padding


    my $boxleft = int(($width-$boxh)/2);
    my $boxtop  = int(($height-$boxv)/2);
    my $boxchar = '#';


    my $border = 0;

    $boxleft = $border + length $strings->{left}->[0];
    $boxtop  = $border + 1; #1=port line

    if ($auto) {
        $height = $boxv+3;
    }

    my $art = $self->home(width=>$width, height=>$height);

    my $indent = {
        left   => $border,
        right  => $boxleft + $boxh + 1,
        top    => $boxleft,
        bottom => $boxleft,
    };
    foreach my $side (keys %{$strings}) {
        my $itemcount = 0;
        foreach my $item (@{$strings->{$side}}) {
          my $indentx = $indent->{$side};
          my $indenty = 0;
          if ($side eq 'left' || $side eq 'right') {
              $indenty = $itemcount + $boxtop + 1;
          } else {
              $indentx += 1 + $itemcount * (($boxh -  length $item)/
                          @{$strings->{$side}});
              $indenty = $boxtop - 1;
              $indenty += $boxv + 2 if $side eq 'bottom';
          }

          $self->plot(x=>$indentx,y=>$indenty,char=>$item,art=>$art);
          $itemcount++;
        }

    }


    #draw box (TODO a method)
    $self->hlin(x1=>$boxleft, x2=>$boxleft+$boxh, y=>$boxtop,
                art=>$art, char=>$boxchar);
    $self->hlin(x1=>$boxleft, x2=>$boxleft+$boxh, y=>$boxtop+$boxv,
                art=>$art, char=>$boxchar);
    $self->vlin(y1=>$boxtop, y2=>$boxtop+$boxv, x=>$boxleft,
                art=>$art, char=>$boxchar);
    $self->vlin(y1=>$boxtop, y2=>$boxtop+$boxv, x=>$boxleft+$boxh,
                art=>$art, char=>$boxchar);


    $self->plot(x=>$boxleft+2,y=>$boxtop+2,char=>$name,art=>$art);


    return join "\n",@{$art};


}


sub generate_port_strings {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            box        => { type => HASHREF },
        },
    );

    my $box = $opts{box};
    my $strings;

    my $format = {
        'left' => '%s %4s%s%s',
        'right' => '%4$s%3$s%2$-4s %1$s',
        'top' => '%s %4s%4$s',
        'bottom' => '%s %4s%4$s',
    };

    foreach my $side (keys %{$format}) {
        $strings->{$side} = [] unless defined $strings->{$side};
    }

    foreach my $side (keys %{$box}) {
        #$strings->{$side} = [] unless defined $strings->{$side};
        foreach my $type (sort keys %{$box->{$side}}) {
            next if $type eq '_total';
            { ## no critic (TestingAndDebugging::ProhibitNoWarnings)
                no warnings 'numeric';
                foreach my $item
                    (sort { $a <=> $b || $a cmp $b }
                        keys %{$box->{$side}->{$type}}) {
                my $enable = $box->{$side}->{$type}->{$item}->{enabled} ?
                             '[+]' : '[-]';
                my $line   = '--'; #in future, can change w/state (no_light)
                my $string = sprintf $format->{$side},$type,$item,$line,$enable;
                push @{$strings->{$side}},$string;

            }
            }

        }
    }

    #print Dumper $strings;

    return $strings;

}

sub home {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            width      => { type => SCALAR, default=>60 },
            height     => { type => SCALAR, default=>20 },
            char       => { type => SCALAR, default=>' ' },
            art        => { type => ARRAYREF | UNDEF, optional=>1    },
        },
    );

    my $art      = $opts{art} ;
    my $width    = $opts{width};
    my $height   = $opts{height};
    my $char     = $opts{char};

    my $line = $char x $width;

    if (!defined $art) {
        $art = [];
        foreach my $h (1..$height) {
            push @{$art},$line;
        }
    }

    foreach my $h (0..$height-1) {
        foreach my $w (0..$width-1) {
            $self->plot(x=>$w,y=>$h,char=>$char,art=>$art);

        }
    }
    return $art;
}

sub plot {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            x     => { regex => qr/^[\d\.]+$/ },
            y     => { regex => qr/^[\d\.]+$/ },
            char       => { type => SCALAR, default=>' ' },
            art        => { type => ARRAYREF },
        },
    );

    my $art  = $opts{art};
    my $x    = int $opts{x};
    my $y    = int $opts{y};
    my $char = $opts{char};

    substr($art->[$y],$x,length $char) = $char;
    return $art;
}

sub hlin {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            x1     => { regex => qr/^\d+$/ },
            x2     => { regex => qr/^\d+$/ },
            y      => { regex => qr/^\d+$/ },
            char       => { type => SCALAR, default=>' ' },
            art        => { type => ARRAYREF },
        },
    );

    my $art  = $opts{art};
    my $x1    = $opts{x1};
    my $x2    = $opts{x2};
    my $y    = $opts{y};
    my $char = $opts{char};

    foreach my $x ($x1..$x2) {
            $self->plot(x=>$x,y=>$y,char=>$char,art=>$art);
    }

    return $art;
}

sub vlin {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            x     => { regex => qr/^\d+$/ },
            y1     => { regex => qr/^\d+$/ },
            y2      => { regex => qr/^\d+$/ },
            char       => { type => SCALAR, default=>' ' },
            art        => { type => ARRAYREF },
        },
    );

    my $art  = $opts{art};
    my $y1    = $opts{y1};
    my $y2    = $opts{y2};
    my $x    = $opts{x};
    my $char = $opts{char};

    foreach my $y ($y1..$y2) {
            $self->plot(x=>$x,y=>$y,char=>$char,art=>$art);
    }

    return $art;
}




sub save_mcc_logs {
    my $self = shift;
    my $metrocluster = $self->metrocluster;

            mprint
                ("enter burt # <or leave blank to write logs to logdir>");

            my ($burt) = $self->get_input() =~ /(\d*)/;
            my $path = q{};

            my $nodes = $self->get_variable(name=>'log_nodes');
            if (ref $nodes ne 'ARRAY') {
                $nodes = [ split /,/,$nodes ];
            }

            if ($burt) {
                print ntest::NTEST_STDOUT "BURT: $burt\n";
                my $ci;
                if ($self->{'survivor-site'} eq 'A') {
                    $ci = $metrocluster->_site_A_metrocluster_component()->
                                 command_interface();
                } else {
                    $ci = $metrocluster->_site_B_metrocluster_component()->
                                 command_interface();
                }

                #figure out the logs path
                my $show = NACL::CS::SystemNode->fetch(
                    command_interface => $ci,
                    requested_fields  => [ 'location' ] );
                my $location = $show->{location} || 'rtp';
                mprint( "ci filer location:$show->{location}\n");
                mprint( "filer location:$location\n");

                $path = "/x/eng/$location-cores/burt.$burt";
            } else {
                $path = Tharn::param('LOGDIR') . '/logs';
            }
            mprint( "log destination path:$path\n");

            mprint( "p: paloma info\n");
            mprint(
                's: controller scraper (adds san logs, core files, more -' .
                " can be SLOW -- 30+ min)\n");
            mprint( "<hit return to abort log download>");

            my ($script) =  $self->get_input =~ /([p|s])/i;

            $nodes = $metrocluster->get_all_mcc_nodes();

            if ($script) {
                my $eval_return = eval {
                    #$self->_get_logs( nodes     => $self->get_all_mcc_nodes(),
                    $self->_get_logs( nodes     => $nodes,
                                      log_path  => $path,
                                      use_tool  => lc $script  );
                    1;
                };
                unless ($eval_return) { mprint($@); };
            }
}

1;

