# $Id: //depot/prod/test/main/lib/NACL/MTask/MCC/Metrocluster.pm#32 $

# this is here due to our standard usage of @_ in Params::Validate methods.
##no critic (RequireArgUnpacking ProhibitFixedStringMatches)
##no critic (ControlStructures::ProhibitUnlessBlocks)
##no critic (ValuesAndExpressions::ProhibitInterpolationOfLiterals)
##no critic (Modules::ProhibitMultiplePackages)
##no critic (RequireBracedFileHandleWithPrint)

#
# Copyright (c) 2011-2021 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary MetroCluster Stub MTask Module
## @author dl-mcc-dpg@netapp.com
## @status shared
## @pod here

package NACL::MTask::MCC::Network::FibreBridge;

use strict;
use warnings;
use Carp;

use base qw (NACL::MTask::MTask);

# Default implementation
use NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl;
use Params::Validate qw( validate_with );

use Class::MethodMaker [
    new    => [ '-init', 'new' ],
    scalar => [ qw/ _fibrebridge_impl / ]
];

=head1 NAME

NACL::MTask::MCC::Network::FibreBridge

=head1 DESCRIPTION

This task is to be used for C-Mode MetroCluster testing.

=cut

sub init {
    my $trace = NACL::MTask::MCC::Log::TRACELOG->start_timer();    #metrics

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            name => { regex => qr/^[\w\-]+$/ },
            impl => 1
        }
    );

    my $impl;

    if ( $opts{'impl'} ) {
        $impl = $opts{'impl'};
    } else {
        $impl =
          NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl->new(
            armadillo_name => $opts{'name'} );
    }

    $self->_fibrebridge_impl($impl);

    return;

}

sub get_name {
    my $self = shift;

    return $self->_fibrebridge_impl()->L_get_name();
}

sub get_info {
    my $self = shift;

    return $self->_fibrebridge_impl()->L_get_info();
}

sub get_apiset {
    my $self = shift;

    return $self->_fibrebridge_impl()->L_get_apiset();
}

sub get_vendor_name {
    my $self = shift;

    return $self->_fibrebridge_impl()->L_get_vendor_name();
}

1;
