#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Set of component objects
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Set::Component;

use strict;
use warnings;

use base 'NACL::MTask::Set';

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 NAME

NACL::MTask::Set::Component

=head1 DESCRIPTION

This module implements a set containing component objects (and subclasses of).
Each set contains only unique components, where the uniqueness criteria is
determined by the primary key string.

See L<NACL::MTask::Set|lib-NACL-MTask-Set-pm> for a description of a set and
the operations that can be performed on a set.

=cut

sub _calculate_key_of_member {
    $Log->enter() if $may_enter;
    my ( $self, $member ) = @_;
    my $str = $member->get_primary_key_string();
    $Log->exit() if $may_exit;
    return $str;
}

sub _base_type { return 'NACL::C::Component'; }

1;
