#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Set of ComponentState objects
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::Set::ComponentState;

use strict;
use warnings;

use base 'NACL::MTask::Set';

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 NAME

NACL::MTask::Set::ComponentState

=head1 DESCRIPTION

This module implements a set containing CS objects.
Each set contains only CS object for unique elements, i.e. there cannot
be more than one CS object for a particular element in a set.

See L<NACL::MTask::Set|lib-NACL-MTask-Set-pm> for a description of a set and
the operations that can be performed on a set.

=head1 USE CASE

A CS_Set is most often useful when you want to compare the ComponentStates
saved from one moment in time, with ComponentStates saved at another.

If you were to compare 2 C_Sets, you would only be able to tell if Components
which had the same primary keys existed at time T1 as existed at time T2.

However with 2 CS_Sets, you can verify existence, along with comparing any or
all of the non-key attributes, like (for example) volume online state,
junction path, or any other field.

You might want to do such a comparison after an upgrade or revert, or after any
other action where you want to assure that the ComponentStates either remain
the same, or change in some predictable and expected way.

=cut

# Cheers to Frank Radish for sending me that use case!

sub _calculate_key_of_member {
    $Log->enter() if $may_enter;
    my ( $self, $member ) = @_;
    my $str = $member->get_component_instance()->get_primary_key_string();
    $Log->exit() if $may_exit;
    return $str;
}

sub _base_type { return 'NACL::CS::ComponentState'; }

1;
