#
# Copyright (c) 2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::MTask::VolumeFileIterator::UnitTestFile;

use Moose;
use MTF::MooseCompatibleException qw(:all);

has 'path' => (
    is => 'ro',
    isa => 'Str',
);

has 'volume' => (
    is => 'ro',
    isa => 'Object',
);

has '_type' => (
    is => 'rw',
    isa => 'Str',
    required => 0,
);

has '_files' => (
    is => 'rw',
    isa => 'ArrayRef',
    default => sub { return []; },
);

# Make a fake directory tree of files for testing purposes
my @files = (
    { path => '/', _type => 'dir', _files => [
        { path => '/dir_empty_1', _type => 'dir', },
        { path => '/dir_one_file', _type => 'dir', _files => [
            { path => '/dir_one_file/file', _type => 'file', },
        ]},
        { path => '/dir_b', _type => 'dir', _files => [
            { path => '/dir_b/file_match', _type => 'file', },
            { path => '/dir_b/file_other', _type => 'file', },
            { path => '/dir_b/dir_bc', _type => 'dir', _files => [
                { path => '/dir_b/dir_bc/canary_file', _type => 'file', },
            ]},
        ]},
        { path => '/a_file', _type => 'file', },
        { path => '/symlink', _type => 'symlink', },
    ]},
);

sub BUILD {
    my $self = shift;

    # Need to special case the creation of 'pa
    if ($self->path() eq '/' && !$self->_type()) {
        $self->_type($files[0]->{_type});
        $self->_files($files[0]->{_files});
    }
}

sub raw_name {
    my $self = $_[0];
    return $self->path();
}

sub standard_type {
    my $self = $_[0];
    return $self->_type();
}

sub is_directory {
    my $self = $_[0];
    if ($self->_type() eq 'dir') {
        return 1;
    }
    return 0;
}

sub enumerate_directory {
    my $self = $_[0];
    if (scalar(@{$self->_files()})) {
        my $type = ref $self;
        $Log->comment("Making objects for ".
            join ',', map { $_->{path} } @{$self->_files()}
        );
        return map {
            $type->new(
                %{$_},
                volume => $self->volume(),
            )
        } @{$self->_files()};
    }
    return;
}

sub make_volume_file {
    my $self = $_[0];
    return NACL::C::VolumeFile->new(
        command_interface => $self->volume()->command_interface(),
        vserver => $self->volume()->vserver(),
        path => '/vol/'.$self->volume()->volume().$self->path(),
    );
}


1;

=head1 NAME

This is private and only used for unit tests on the VolumeFileIterator.

Do not use!

=cut

