#$Id: //depot/prod/test/nacldev/lib/NACL/MTask/_DumpCore.pm#1 $
#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary common routines for NACL::MTask::SpareCore and NACL::MTask::SprayCore
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::MTask::_DumpCore;

use strict;
use warnings;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use base qw (NACL::MTask::MTask);
use Params::Validate qw(validate SCALAR validate_with SCALARREF HASHREF UNDEF);
use NACL::CS::SystemNodeCoredumpStatus;
use NACL::C::SystemNodeCoredumpSegment;
use NACL::C::SystemNodeCoredump;
use NACL::Transit;
use NACL::STask::Node;
use NACL::STask::StorageFailover;
use TestUtils qw(get_site_info);
use NACL::Exceptions::NoElementsFound qw(:try);
use NATE::BaseException;
use NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform;

use constant CORE_TOOL_LOC  => "/x/eng/localtest/noarch/share/coretool/freebsd8";
use constant CORE_TOOL_PATH => "/mroot/etc/crash/";
use constant FILE_PATH      => "/mroot/etc/crash/";

BEGIN {
    use Exporter qw(import);
    our %EXPORT_TAGS = (
        all => [
            qw(_cleanup_cores
              _get_node
              _panic
              _upload
              _is_coretool_installed
              _install_coretool
              run_coretool
              cleanup_core_segment
              )
        ]
    );
    our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
}

###############################################################################
# Method:    run_coretool
# Objective: return output after running coretool on core file
# Details:   See POD documentation.
###############################################################################
sub run_coretool {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'core-id'       => { type => SCALAR },
            'core-tool-loc' => { type => SCALAR|UNDEF, optional => 1 },
            'file-path'     => { type => SCALAR|UNDEF, optional => 1 },
            'core-tool-path'=> { type => SCALAR|UNDEF, optional => 1 },
        },
        allow_extra => 1,
    );
    my $command_interface = $opts{command_interface};
    if (! defined $opts{'core-tool-loc'} ) {
        if ($command_interface->version_manager()->is_version_ge(release => '8.2')) {
            $opts{'core-tool-loc'}  =  CORE_TOOL_LOC;
        }else {
            $opts{'core-tool-loc'}  = "/usr/software/test/noarch/share/coretool";
        }
    } 
    $opts{'core-tool-path'} =  CORE_TOOL_PATH if (! defined $opts{'core-tool-path'} );
    $opts{'file-path'}      =  FILE_PATH      if (! defined $opts{'file-path'} );
    my $apiset            = $command_interface->apiset(set => 'Systemshell' );
    if (
        $pkg_or_obj->_is_coretool_installed(
            command_interface => $opts{command_interface},
            'core-tool-path'  => $opts{'core-tool-path'},
        )
      )
    {
        $Log->comment("Coretool is already installed in node $command_interface->node() at location $opts{'core-tool-path'}");
    }
    else {
        $pkg_or_obj->_install_coretool(
            command_interface => $opts{command_interface},
            'core-tool-loc'   => $opts{'core-tool-loc'},
            'core-tool-path'  => $opts{'core-tool-path'},
        );
    }
    my $core_file = $opts{'file-path'} . $opts{'core-id'};
    $opts{'method-timeout'} ||= 600;
    my $response  = $apiset->coretool( 'core-name' => $core_file, 'connectrec-timeout' => $opts{'method-timeout'} );
    my $output    = $response->get_parsed_output();
    if ( $output->[0]->{missing_blocks} ) {
        NATE::BaseException->throw("Verifying Integrity on corefile $output->[0]->{filename} got failed!! \n"
                                                . " Found $output->[0]->{missing_blocks} missing blocks while running coretool \n");
    }
    $Log->exit() if $may_exit;
    return $output;
}
################################################################################
# Method   : _install_coretool
# Objective: Install coretool on filer
################################################################################
sub _install_coretool {
    $Log->enter() if $may_enter;
    my $pkg_or_obj      = shift;
    my %opts      = @_;
    my $cmd_if    = $opts{command_interface};
    my $path      = $opts{'core-tool-path'};
    my $ctool_loc = $opts{'core-tool-loc'};
    my ( $options, %site_of, $site, );

    ### BUG: get_site_info() doesn't know where site file lives unless an
    ### Option option is instantiated.  The Option instance doesn't see site
    ### variable unless a Server object is created (I'll get site info from
    ### the env myself).
    $options = new Options();
    $options->process();
    $site    = $ENV{"SITE"};
    %site_of = get_site_info();
    ### Copy over coretool file to the host directory under test
    ### using wget.
    $site = $cmd_if->hostrec()->SITE();
    unless ( $site_of{$site}->{install_server} ) {
        NATE::BaseException->throw( "ERROR: No site_info file found using SITE parameter '$site'\n"
          . "Verify that a SITE parameter is defined in ~/.ntest file\n");
    }
    my $apiset = $cmd_if->apiset(set       => 'Systemshell');
    my $arch = $apiset->execute_raw_command( command => "uname -p" );
    chomp($arch);
    $apiset->execute_raw_command( command => "cd " . $path );
    my $cmd = "wget "
      . $site_of{$site}->{install_proto} . '://'
      . $site_of{$site}->{install_user} . ':'
      . $site_of{$site}->{install_pass} . '@'
      . $site_of{$site}->{install_server}
      . $ctool_loc . "/"
      . $arch
      . "/coretool";
    $apiset->execute_raw_command( command => $cmd );
    $apiset->execute_raw_command(
        command => "chmod 777 " . $path . "coretool" );
    $Log->exit() if $may_exit;
    return 1;

}
################################################################################
# Method    : _is_coretool_installed
# Objective : Verify if coretool is installed on node
################################################################################
sub _is_coretool_installed {
    $Log->enter() if $may_enter;
    my $pkg_or_obj    = shift;
    my %opts   = @_;
    my $cmd_if = $opts{command_interface};
    my $path   = $opts{'core-tool-path'};
    my ( $exist, $output, $file_name );
    $file_name = $path . "coretool";
    my $apiset = $cmd_if->apiset(
        set       => 'Systemshell'
    );
    try {
        $output = $apiset->ls( paths => $file_name );
        $exist = 1;
    } catch NACL::APISet::Exceptions::InvalidParamValueException with {
        $exist = 0;
    };
    $Log->exit() if $may_exit;
    return $exist;
}

###########################################################
# Routine : _cleanup_cores()                              #
# Description : Local subroutine to delete pre-existing   #
# cores.                                                  #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _cleanup_cores {
    $Log->enter() if $may_enter;
    ## Grab the object handle from
    ## the input array @_
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            delete_cores_timeout  => { type => SCALAR , optional => 1 }
        },
        allow_extra => 1,
    );
    my $timeout = delete $opts{delete_cores_timeout} || 1800 ;
    my $ci      = delete $opts{command_interface} ;
    my $coredump_type = "kernel";
    $Log->comment(
        "Deleting all the existing core files of type $coredump_type");
    ## Delete all saved kernel core files
    NACL::C::SystemNodeCoredump->delete_all(
        command_interface => $ci,
        node              => $ci->node(),
        type              => $coredump_type,
	'method-timeout'  => $timeout
    );

    ## Delete all unsaved kernel core files
    $coredump_type = "unsaved-kernel";
    $Log->comment(
        "Deleting all the existing core files of type $coredump_type");
    NACL::C::SystemNodeCoredump->delete_all(
        command_interface => $ci,
        node              => $ci->node(),
        type              => $coredump_type,
	'method-timeout'  => $timeout
    );
    $Log->exit() if $may_exit;
}    # end _cleanup_cores()

###########################################################
# Routine      : cleanup_core_segment()                   #
# Description  : Subroutine to delete segment which got   #
#                created during panic.                    #
# Returns      : Nothing                                  #
# Dependencies : CMode only                               #
###########################################################
sub cleanup_core_segment {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@_,
        additional_spec => {
            'node'              => { type => SCALAR, optional => 1 },
            delete_cores_timeout  => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1,
    );
    my $timeout = delete $opts{delete_cores_timeout} || 1800 ;
    my %common_opts;
    $pkg_or_obj->_move_common_component_params(
        source => \%opts,
        target => \%common_opts
    );
    my $command_interface = delete $opts{command_interface};
    $opts{'node'} = $command_interface->node()
        if ( !( defined $opts{'node'} ) );
    my $command_supports = 0;


    try {
        NACL::C::SystemNodeCoredumpSegment->delete_all(
            command_interface => $command_interface,
            node              => $command_interface->node(),
            'method-timeout'  => $timeout
        );
    } catch NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform with {
		$command_supports = 1;
    };
    if (! $command_supports) {
		my $coredump_type = "kernel";
		NACL::C::SystemNodeCoredump->delete_all(
			command_interface => $command_interface,
			node              => $opts{'node'},
			type              => $coredump_type,
			%common_opts
		);
                my $find_passed;
                try {
                    NACL::C::SystemNodeCoredump->find(
                        command_interface => $command_interface,
                        filter            => {
                            node => $command_interface->node(),
                            type => $coredump_type,
                        },
                    );
                    $find_passed = 1;
                }
                catch NACL::Exceptions::NoElementsFound with { };
                if ($find_passed) {
                    require NACL::Exceptions::VerifyFailure;
                    $Log->exit() if $may_exit;
                    NACL::Exceptions::VerifyFailure->throw( "Corefiles are not deleted successfully !! ");
                }
		try {
			NACL::C::SystemNodeCoredumpSegment->find(
				command_interface => $command_interface,
				filter            => { node => $command_interface->node(), },
                               allow_empty       => 1
			);

		}
		catch NACL::C::Exceptions::SystemNodeCoredumpSegment::NotSupportedForThisPlatform with { };
    }
    $Log->exit() if $may_exit;
}    # end cleanup_core_segment()

###########################################################
# Routine : _get_node()                                   #
# Description : Local subroutine to get the node  that is #
# being used                                              #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _get_node {
    $Log->enter() if $may_enter;
    ## Grab the object handle from
    ## the input array @_
    my $self = shift;

    ## Local variables
    my $ci   = $self->command_interface();
    my $node = $ci->hostrec()->hostname();
    $Log->comment("Hostname of the node - $node ");
    $Log->exit() if $may_exit;
    return $node;

}    # end _get_node()

###########################################################
# Routine : _panic()                                      #
# Description : Local subroutine to cause a sysctl        #
# panic                                                   #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _panic {
    $Log->enter() if $may_enter;
    ## Grab the object handle from
    ## the input array @_
    my $self = shift;
    my $ci   = $self->command_interface();
    ## Local variables
    my $node = $self->_get_node();
    $Log->comment("Issuing debug sysctl panic on the node $node");
    NACL::STask::Node->panic(
        command_interface => $ci,
        nacltask_reboot   => 0
    );
    $Log->exit() if $may_exit;
}    # end _panic()

###########################################################
# Routine : _upload()                                     #
# Description : Local subroutine to upload a core file    #
# to a server.                                            #
# Returns : Nothing                                       #
# Dependencies : CMode only                               #
###########################################################
sub _upload {
    $Log->enter() if $may_enter;
    ## Grab the object handle from
    ## the input array @_
    my $self = shift;

    ## Local varibales
    #my $api_set_Obj = $self->{cli_apiset_Obj};
    my $api_set_Obj           = undef;
    my $corefile              = $self->corefile();
    my $core_upload_location  = undef;
    my $node_name             = $self->_get_node();
    my $default_upload_server = $self->core_upload_server();

    ## Get the APISet ref from the sparecore
    ## object's command interface
    my $ci = $self->command_interface();
    $api_set_Obj = $ci->apiset(
        category  => 'Node',
        interface => 'CLI',
        set       => 'CMode'
    );
    $Log->comment("Determining the core upload location ");
    if ( defined $self->core_upload_location() ) {
        $core_upload_location = $self->core_upload_location();
    }
    else {
        $core_upload_location = 'http://' . $default_upload_server;
    }

    ## Start the upload operation
    $Log->comment(
        "The upload server that will be used is $core_upload_location");
    $Log->comment("The core file that will be uploaded is $corefile");
    $Log->comment("Starting the core file upload ..");
    ## Need to use the apiset here as it is interactive
    $api_set_Obj->system_node_coredump_upload(
        node                     => $node_name,
        location                 => $core_upload_location,
        corename                 => $corefile,
        'connectrec-timeout'     => '3600',
        'connectrec-match_table' => [
            ("Enter User:")     => "",
            ("Enter Password:") => ""
        ]
    );
    $Log->comment("core file upload is successful for file $corefile");
    $Log->exit() if $may_exit;
}    # end _upload()

1;
