#
# Copyright (c) 2001-2013 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary AntivirusOnAccess Task Module
## @author swati.bajaj@netapp.com, dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::AntivirusOnAccess;

use strict;
use warnings;

use base qw(NACL::C::AntivirusOnAccess NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR BOOLEAN SCALARREF);
use NATE::Exceptions::Argument qw(:try);

use NACL::APISet::Exceptions::ResponseException ();

=head1 NAME

NACL::STask::AntivirusOnAccess

=head1 DESCRIPTION

C<NACL::STask::AntivirusOnAccess> provides a number of well-defined but
potentially complex or multi-step methods related to AntivirusOnAccess in
ONTAP.

It builds on top of, and is a derived class of C<NACL::C::AntivirusOnAccess>,
and so it also provides methods that are more in the scope of
individual AntivirusOnAccess-related commands.  See C<NACL::C::AntivirusOnAccess> for
details.  This also means that a C<NACL::STask::AntivirusOnAccess> object may
generally be used in place of a component object.

=head1 ATTRIBUTES

=head2 command_interface

(Required) As NACL::C::Node. A component object that represents the host to 
which to send commands.

=head1 METHODS

=head2 modify

  NACL::STask::AntivirusOnAccess->modify(
      command_interface => $command_interface,
      vserver           => $vserver_name,
      volume            => $volume_name,
      'vserver-policy'  => $policy_name,
      nacltask_verify   => 0|1 # defaults to 0,
      %other_options
  );


(Class method) Modifies AntivirusOnAccess policy for vserver and/or volume.
Also verifies that all attributes got set to the specified values if C<nacltask_verify>
is set to 1.

=over 

=item C<< nacltask_verify => 0|1  >>

(Optional, defaults to 0)
When set to 1, it verifies that the on-access policy has been modifed
on the vserver/volume.
When set to 0, verification is not performed.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item other options

See L<NACL::C::AntivirusOnAccess|lib-NACL-C-AntivirusOnAccess-pm/modify> for
all the other options accepted by this method.

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification for on-access policy modify has failed.

=back

=cut

sub modify {
    $Log->enter();
    my $pkg = shift;
    $pkg->_call_method_then_verify(@_);
    $Log->exit();
}

1;
