package NACL::STask::Client::RPC::Server;

use NATE::STAF::Peer qw($Parent);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use MIME::Base64 qw(encode_base64 decode_base64);
use Storable qw(thaw nfreeze);
use NACL::STask::Client::RPC::Wrapper ();
use Data::Dumper;

#use NACL::ComponentUtils qw(Dumper);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw( serve );
}

sub serve {
    $Log->enter() if $may_enter;
    while (1) {
        # decode base64 and thaw the input STAF message
        my $rpc_response;
        my $rpc_request = thaw(
            decode_base64(
                $Parent->message_get(
                    type => "NACL/STask/Client/RPC/Request"
                )
            )
        );

        my $package_or_obj = $rpc_request->{package}
            || $rpc_request->{object};
        my $method = $rpc_request->{method};
        if ( $package_or_obj && $method ) {

            # $package->$method(%args)
            $rpc_response->{_result}
                = eval { $package_or_obj->$method( @{ $rpc_request->{args} } ) };
        } elsif ( defined $rpc_request->{eval} ) {

            # eval {code}.
            # wrapper objects will be handled in AUTOLOAD.
            @_ = @{ $rpc_request->{args} };
            $rpc_response->{_result} = eval( $rpc_request->{eval} );
        } ## end elsif ( defined $rpc_request...)

        # catch error's and update $future->exception()
        if ($@) {
            if ( defined($@) ) {
                $rpc_response->{_exception} = $@;
            }
        }

        if ( $rpc_request->{wrap_return} ) {
            $rpc_response->{_result}
                = NACL::STask::Client::RPC::Wrapper->new(
                value => $rpc_response->{_result}, )
                if ( $rpc_response->{_result} );
        } ## end if ( $rpc_request->{wrap_return...})

        $NATE::STAF::Peer::Parent->message_put(
            message => encode_base64( nfreeze($rpc_response) ),
            type    => $rpc_request->{message_id}
        );
    } ## end while (1)
    $Log->exit() if $may_exit;
} ## end sub serve

1;
