#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseExceptions that will be thrown whenever
##          Aggregate Relocation fails for various reasons
##
## @author hemamali@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::Exceptions::AggregateRelocationFailure

=head1 DESCRIPTION

This exception is thrown when the Aggregate Relocation Fails for one or more aggregates
from the aggregate-list in the start command .

It holds an array of unexpected results as its attribute .

Each array member is a hashref which contains 'aggregate' and 'text'
as the keys of the hash. The 'aggregate' field is used to store the name of the aggregate
which failed to relocate and the field 'text' is used to store the failure message .

=head1 ATTRIBUTES

=head2 unexpected_results

=over

This attribute is an array . It holds information about the failures which occured
in the various aggregates of the aggregate-list in the start command.
This array, contains a number of hashref as its elements,
The hashref contains the following keys :: 'aggregate' and 'text'

=back

=head3 Example format of "unexpected_results"

=over

$VAR1 = [
          {
            'aggregate' => 'aggr1',
            'text' => "'relocation_status is not 'Done'"
          },
          {
            'aggregate' => 'aggr3',
            'text' => " aggr state is not online after relocate "
           }
        ];

=back

=head1 EXAMPLE


    Example of throwing the exception

    NACL::STask::Exceptions::AggregateRelocationFailure->throw(
                 ## Reference to an array which hold the information
                 ## as hashref with aggregate and text as its keys.
                 unexpected_results => \@list_of_arl_failures  );

    Example of catching the exception

   use NACL::STask::Exceptions::AggregateRelocationFailure qw(:try);

   try { $arl_obj = NACL::STask::StorageAggregateRelocation->start(nacltask_wait=>1,...);

   } catch NACL::Exceptions::AggregateRelocationFailure with {
      my $exception = shift;

        ## Getting the list of unexpected failures
        my $unexpected_results = $exception->unexpected_results();

        or

        my @unexpected_results = $exception->unexpected_results();

   };

=cut

package NACL::STask::Exceptions::AggregateRelocationFailure;
use base qw(NATE::BaseException);

use Class::MethodMaker [
    array => 'unexpected_results'
    ];

1;
