#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when all the nodes in a
##          cluster are in a down state. The state can be in any of the state
##          such as firmware,loader prompt or boot menu.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::AllNodesDown

=head1 DESCRIPTION

This exception is thrown when all the nodes in a cluster are in a down state.
The state can be in any of the state such as firmware,loader prompt or boot menu.

=head3 EXAMPLE

    use NACL::STask::Exceptions::AllNodesDown qw(:try);
    use NACL::STask::StorageDisk;

    try {
        NACL::STask::StorageDisk->unfail(
            command_interface  => $Node,
            disk               => $disk,
            $fdr_param         => $check_fdr
        );

    }
    catch NACL::STask::Exceptions::AllNodesDown with {
        my $exception = shift;
        $Log->comment("All nodes are in down state hence cannot check the
        . "FDR table");
    };

=back

=cut

package NACL::STask::Exceptions::AllNodesDown;
use base qw( NATE::BaseException );

1;
