#
# Copyright (c) 2016 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when the
##          call to NACL::STask::Client::FileLock->release fails because
##          existing locks could not be unlocked .
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::Client::FileLock::NotReleased

=head1 EXAMPLES

=head2 Throwing exception

  my $file_lock = NACL::STask::Client::Filelock->lock(
        command_interface  => $Client,
		node               => $Node->node(),
		file_lock_opts     => $file_lock_opts,
		file_open_opts     => $file_open_opts,
		nacltask_verify    => 1,
		path               => '/test_vol/test_file2',
		vserver            => $Testvserver,
		volume             => $Volume
    );
	try{
       $file_lock->released(nacltask_verify    => 1);
	}catch  NACL::STask::Exceptions::Client::FileLock::NotReleased with{;
    }
=cut

package NACL::STask::Exceptions::Client::FileLock::NotReleased;

use base 'NATE::BaseException';

1;
