#
# Copyright (c) 2014 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when the
##          call to NACL::STask::Client::Ifconfig->fetch fails because 
##          Ifconfig with given atrribute was found.
##
## @author  dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::Client::Ifconfig::NotDeleted

=head1 EXAMPLES

=head2 Throwing exception

 my $found = NACL::CS::Client::Ifconfig->fetch(
        command_interface => $opts{command_interface},
        interface         => $interface,
        allow_empty       => 1
    );
    my $ipv6      = $found->inet6_info();
    foreach my $val (@{$ipv6}) {
        if ($val->{addr} eq $inet_addr) {
            $flag = 1;
            last
        }
    }
    if ($flag) {
        $Log->exit();
        NACL::STask::Exceptions::Client::Ifconfig::NotDeleted->throw();
    }
=cut

package NACL::STask::Exceptions::Client::Ifconfig::NotDeleted;

use base 'NATE::BaseException';

1;
