# $Id: $
#
# Copyright (c) 2001-2012 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseExceptions that will be thrown whenever
##          job wait for state fails with unexpected status 
##
## @author poornima@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::DFM::JobWaitForStateFailure

=head1 DESCRIPTION

This exception is thrown when the job wait for state fails to complete
with the expected status

It holds an array of job tasks info as one of its attribute .

Each array member is a hashref which contains various fields defined in the
NACL::CS::DFM::JobTask module

=head1 ATTRIBUTES

=head2 job_state

=over

This attribute is a scalar. It holds information about the current state of the job

=back

=head2 job_type

=over

This attribute is a scalar. It holds information about the current type of the job

=back

=head2 job_tasks_info 

=over

This attribute is an array . It holds information about the various fields for 
each task returned by the NACL::CS::DFM::JobTask module

=back

=head1 EXAMPLE


    Example of throwing the exception

    NACL::STask::Exceptions::DFM::JobWaitForStateFailure->throw(
                 ## Reference to an array which hold the information
                 ## as arrayref with various job task related info.
                 job_tasks_info => \@state_objs  );

    Example of catching the exception

   use NACL::STask::Exceptions::DFM::JobWaitForStateFailure qw(:try);

   try { NACL::STask::DFM::Job->wait_for_state(  command_interface => $ci,
                                        'job-list' => $job_list,
					....);

   } catch NACL::Exceptions::DFM::JobWaitForStateFailure with {
      my $exception = shift;

        ## Getting the list of unexpected failures
        my $job_tasks_info = $exception->job_tasks_info();

        or

        my @job_tasks_info = $exception->job_tasks_info();

   };

=cut

package NACL::STask::Exceptions::DFM::JobWaitForStateFailure;
use base qw(NATE::BaseException);

use Class::MethodMaker [
    scalar => 'job_id',
    scalar => 'job_state',
    scalar => 'job_type',
    array => [ { -type => 'NACL::CS::DFM::JobTask' }, 'job_tasks_info' ],
    ];

1;
