#
# Copyright (c) 2013 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when
##          cluster create or join is attempted on a node
##          that is already a part of the cluster.
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::NodeAlreadyInCluster

=head1 DESCRIPTION

This exception is thrown when a cluster create or join is attempted
on a node that is already a part of the cluster.

=head3 EXAMPLE

    use NACL::STask::Exceptions::NodeAlreadyInCluster qw(:try);
    use NACL::STask::Cluster;

    try {
        NACL::STask::Cluster->create(
            command_interface  => $Node,
        );

    }
    catch NACL::STask::Exceptions::NodeAlreadyInCluster with {
        my $exception = shift;
        $Log->comment($Node->node(). " is already a member of a cluster");
    };

=back

=cut

package NACL::STask::Exceptions::NodeAlreadyInCluster;
use base qw( NATE::BaseException );

1;
