#
# Copyright (c) 2015 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when there are MCC config
##          diffs
##
## @author dl-nacl-dev@netapp.com
## @status Shared
## @pod here

=head1 NAME

NACL::STask::Exceptions::VserverConfigDiffsPresent

=head1 DESCRIPTION

This exception is thrown when there MCC config diffs.

=head3 EXAMPLE

    use NACL::STask::Exceptions::VserverConfigDiffsPresent qw(:try);

    try {
        NACL::STask::StorageDisk->config_diff(
            command_interface  => $Node,
            vserver            => $vs_name,,
        );

    }
    catch NACL::STask::Exceptions::VserverConfigDiffsPresent with {
        my $exception = shift;
        $Log->comment("Config diffs: " . $exception->text());

        # The exception also stores the NACL::CS::VserverConfigDiff objects
        # that contain full details of the diffs.
        my @state_objs = $exception->state_objs();
    };

=back

=cut

package NACL::STask::Exceptions::VserverConfigDiffsPresent;
use base qw( NATE::BaseException );
use Class::MethodMaker [array => 'state_objs'];
1;
