#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mkfile Task Module
## @author yifeng@netapp.com
## @status shared
## @pod here

package NACL::STask::Mkfile;

use strict;
use warnings;

use base qw(NACL::C::Mkfile NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use Params::Validate qw(validate SCALAR UNDEF);
use NACL::C::VolumeFile;
use NACL::ComponentUtils qw(_optional_scalars);

=head1 NAME

NACL::STask::Mkfile

=head1 DESCRIPTION

B<DEPRECATED>.

This module is deprecated and has been superceded by
L<NACL::STask::VolumeFile|lib-NACL-STask-VolumeFile-pm>. This module is no
longer supported or enhanced - all bug fixes and enhancements will be
provided in NACL::STask::VolumeFile.

C<NACL::STask::Mkfile> supports all functionalities provided by internal tool
mkfile.

It also enables an option to allow user to create a file and use volume
space without filling file holes. This option can be completed by a combination
of using mkfile, testsetattr and enable file reservation.

=head1 METHODS

=head2 mkfile

B<DEPRECATED>. Please use
L<< NACL::STask::VolumeFile->create|lib-NACL-STask-VolumeFile-pm/create >>
instead.

  NACL::STask::Mkfile->mkfile(
      command_interface => $command_interface,
      nacltask_use_space_reserve => $boolean,            # default '1'
      %other_options
  );

(Class method) Create files by using internal mkfile tool. This method enables
an option to create a file and use volume space without filling file holes. This
is extremely useful when creating a file with very large size. It also adds an
option that allows user to provide expected write error message when volume is
full, so the call will succeed. Without this option, user will typically get an
error message when volume is full, though the file is created.

=over

=item Options

=over

=item C<< nacltask_use_space_reserve => 0|1 >>

(Optional) If 1 (the default), create file with file reservation enabled, skip
filling file holes. If 0, use normal mkfile write to fill all file holes without
file reservation enabled.

=item C<< vserver => $vserver >>

(Optional) This is required for CMode only and only if user wants to create a
file by enabling file reservation.

=back

All of the other various options to
L<< NACL::C::Mkfile->mkfile | lib-NACL-C-Mkfile-pm/mkfile >>

=back

=over

=item Enhancement

Add an option to create multiple files with characteristics that allow the user
to specify the number of files, the size of the files, or a range of file sizes.

=back

=over

=item Note

This method can not be used by scripts which are designed for testing promoted
builds.

=back

=cut

sub mkfile {
    $Log->enter() if $may_enter;
    my $pkg = shift;

    my %opts = $pkg->_common_validate_with(
        params          => \@_,
        allow_extra     => 1,
        additional_spec => {
            nacltask_use_space_reserve => { type => SCALAR, default => 1 },
            vserver                    => { type => SCALAR|UNDEF, optional => 1 },
            determine_appropriate_nodescope_node => { type => SCALAR, default => 1 },
        },
    );

    my $command_interface = $opts{command_interface};
    my $nacltask_use_space_reserve = delete $opts{nacltask_use_space_reserve};
    my $vserver = $opts{vserver};
    
    my %nodescope_override;
    if ($nacltask_use_space_reserve && $opts{determine_appropriate_nodescope_node}
        && $command_interface->mode() eq 'CMode') {
        # If this is set then we need to make another nodescope call,
        # so let's figure out the appropriate node on which to make both
        # the mkfile as well as the testsetattr call.
        $nodescope_override{'nodescope-node-name'} = $pkg->get_local_node_name(
            command_interface => $command_interface,
            vserver           => $vserver,
            filename          => $opts{filename}
        );
    }

    my $desired_size;
    if ( $nacltask_use_space_reserve ) {
        $desired_size = $opts{size};
        $opts{size} = 0;
    }

    $pkg->SUPER::mkfile(%opts, %nodescope_override);

    # Get common component params: apiset_must, apiset_should, etc
    my %common_params = ();
    $pkg->_copy_common_component_params(
        source => \%opts,
        target => \%common_params
    );

    $common_params{'vserver'} = $vserver if (defined $vserver);

    if ( $nacltask_use_space_reserve ) {
        $Log->trace("Set file size and enable file reservation.\n"
            . $opts{filename} . " : " . $desired_size);

        my $apiset = $command_interface->get_7m_or_nodescope_apiset();
        $apiset->testsetattr(
            'path'              => $opts{filename},
            "file-size"         => $desired_size,
            %nodescope_override
        );

        NACL::C::VolumeFile->reservation(
            command_interface   => $command_interface,
            'path'              => $opts{filename},
            'is-enabled'        => "enable",
            %common_params
        );
    }

    $Log->exit() if $may_exit;

} # end sub create

1;
