#
# Copyright (c) 2011-2013 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.

## @summary VolumeFlexcache Task Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

package NACL::STask::VolumeFlexcache;

use strict;
use warnings;

use base qw(NACL::C::VolumeFlexcache NACL::STask::STask);

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NACL::STask::Node;

=head1 NAME

NACL::STask::VolumeFlexcache

=head1 DESCRIPTION

C<NACL::STask::VolumeFlexcache> builds on top of, and is a derived class of
L<NACL::C::VolumeFlexcache|lib-NACL-C-VolumeFlexcache-pm>. It provides the
same methods as the component. The only additional facility it provides in
addition to the component is that it automatically determines an appropriate
timeout to use if one is not provided (the time taken by the "volume
flexcache" commands depends on the number of nodes in the cluster - the
library sets the timeout proportional to the number of nodes present in
the cluster)

=head1 METHODS

=head2 purge

Provides the same functionality and accepts the same options as the
L<delete|lib-NACL-C-VolumeFlexcache-pm/delete> method. This has been
implemented so that the standard STask interface of providing a C<purge>
method is adhered to. 

=cut

sub purge {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    $pkg_or_obj->delete(@args);
    $Log->exit() if $may_exit;
}

# All methods go through this, so this is the quickest way of calculating
# the timeout value for all methods
sub _common_validate_with {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, @args) = @_;
    my %opts = $pkg_or_obj->SUPER::_common_validate_with(@args);
    $opts{'method-timeout'} = $pkg_or_obj->_determine_timeout(%opts);
    $Log->exit() if $may_exit;
    return %opts;
}

# Sets the timeout to be 3 minutes * num of nodes.
# This /should/ be reasonably safe while not being too large
sub _determine_timeout {
    $Log->enter() if $may_enter;
    my ($pkg_or_obj, %opts) = @_;
    my $timeout = $opts{'method-timeout'};
    unless (defined $timeout) {
        my @nodes = NACL::STask::Node->find(
            command_interface => $opts{command_interface});
        $timeout = 180 * scalar(@nodes);
    }

    $Log->exit($timeout);
    return $timeout;
}

1;
