
=head1 NAME

NACL::STask::VserverVscanScannerPool

=head1 DESCRIPTION

C<NACL::STask::VserverVscanScannerPool> provides a number of well-defined but
potentially complex or multi-step methods related to Vserver Vserver Vscan Scanner Pool
in ONTAP.

It builds on top of, and is a derived class of C<NACL::C::VserverVscanScannerPool>,
and so it also provides methods that are more in the scope of
individual vserver-related commands.  See C<NACL::C::VserverVscanScannerPool> for
details.  This also means that a C<NACL::STask::VserverVscanScannerPool> object may
generally be used in place of a component object.

=head1 METHODS

=head2 create

This accepts the same options and provides the same facilities as
L<NACL::C::VserverVscanScannerPool::create|lib-NACL-C-VserverVscanScannerPool-pm/create>.
Additionally, the STask provides the following facilities: C<Cleanup>,
C<nacltask_verify>. The description of these features are
present at:
L<Common Create options|lib-NACL-STask-STask-pm/Common_Create_Options>.

=head2 purge

This accepts the same options and provides the same facilities as
L<NACL::C::VserverVscanScannerPool::delete|lib-NACL-C-VserverVscanScannerPool-pm/delete>.
Additionally, the STask provides the following facilities: C<nacltask_verify>.
The description of these features are present at:
L<Common Purge options|lib-NACL-STask-STask-pm/Common_Purge_Options>.

=cut

=head2 modify
  
(Class or instance method)
This method is used to modify the scanner pool. "nacltask_verify"
parameter can be set to ensure all the attributes got a specified value.
And also provides an mechanism of cleanup for the objects that are modified
by this method.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Verifies whether all of the attributes got set to the specified value.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item other options

See L<NACL::C::VserverVscanScannerPool::modify
|lib-NACL-C-VserverVscanScannerPool-pm/modify> for all the other options
accepted by this method.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for the modified
scanner pool.

=back

=back

=cut

=head3 apply_policy
  
(Class or instance method)
This method is used to apply-policy to the scanner pool. "nacltask_verify"
parameter can be set to ensure all the attributes got a specified value.
And also provides an mechanism of cleanup for the objects that are created
by this method.

=over

=item Options

=over

=item C<< command_interface => $ci >>

(Required for class method, Not Applicable for instance method)
A component object that represents the host which to send commands.
See L<NACL::C::Component::command_interface
  |lib-NACL-C-Component-pm/command_interface>.

=item C<< nacltask_verify => 0|1 >>

(Optional, defaults to 0)
Verifies whether all of the attributes got set to the specified value.

Note that it is also possible to perform the verification later by calling
L<taskverify_modify|lib-NACL-STask-STask-pm/taskverify_modify>. This can
be used for cases where ONTAP chooses to not set all attributes to the exact
values specified. In the call we can specify those attributes we want
verified.

=item C<< nacltask_to_cleanup => 0|1 >>

(Optional, default to 0(no to cleanup)
Flag indicating if this operation needs to be registered for clean up or not.

=item C<< nacltask_cleanup_manager >>

Cleanup manager to be used for registering.
Default : Will use the default cleanup manager.

=item other options

See L<NACL::C::VserverVscanScannerPool::apply-policy
|lib-NACL-C-VserverVscanScannerPool-pm/apply-policy> for all the other options
accepted by this method.

=back

=back

=over

=item Exceptions

=over

=item C<NACL::Exceptions::VerifyFailure>

This type of exception is thrown when verification fails for apply-policy of
scanner pool.

=back

=back

=cut

package NACL::STask::VserverVscanScannerPool;

use strict;
use warnings;
use base qw(NACL::C::VserverVscanScannerPool NACL::STask::STask);
use Params::Validate
  qw(validate validate_with SCALAR SCALARREF ARRAYREF BOOLEAN );
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use NATE::Exceptions::Argument qw(:try);
use NATE::BaseException;
use NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist qw(:try);
use NACL::C::Exceptions::VserverVscanScannerPool::DoesNotExist;

__PACKAGE__->create_method_builder();
__PACKAGE__->modify_method_builder();
__PACKAGE__->purge_method_builder();
__PACKAGE__->create_method_builder(
                 stask_method_name => 'apply_policy',
                 c_method_name => 'apply_policy',
				 c_exception_name => 'DoesNotExist',
			 );
		
1;
