#
# Copyright (c) 2017 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::ServiceAPI::ASUPTrigger;

use strict;
use warnings;

use NATE::Log qw/log_global/;

use Exporter;
use parent qw(NATE::ServiceAPI Exporter);

use constant SERVICE_CLASS => 'NACL::Service::ASUPTrigger';

our @EXPORT_OK = qw/asuptriggerapi_global/;

my $Global;

sub asuptriggerapi_global {
    $Global = __PACKAGE__->new( global => 1 ) if (!$Global);
    return $Global;
}

# Override NATE::ServiceAPI::new to add the record_errors functionality
# By default, errors will be logged as warnings. If record_errors is set 1
# however, errors will be logged as NATE errors.
sub new {
    my ($class, %opts) = @_;

    my $global        = delete $opts{global} // 1;
    return $Global if ($Global && $global);

    my $record_errors = delete $opts{record_errors} // 0;

    my $self = $class->SUPER::new(
        type          => 'ASUPTrigger',
        service_class => SERVICE_CLASS,
        language      => 'perl',
        %opts
    );
    # Generate the serviceapi subroutines that aren't explicitely defined in this class
    $self->_generate_capabilities(namespace => SERVICE_CLASS);

    $self->{record_errors} = $record_errors;

    $Global = $self if ($global);

    return $self;
}

sub _api_call {
    my ($self,$api,@args) = @_;
    my $response = $self->_core_client_api(@args,
        {
            _api            => $api,
            _namespace      => SERVICE_CLASS,
            request_timeout => -1,
        }
    );
    my $errors = $response->{errors};
    if ( $errors && @$errors ){
        for my $e ( @$errors ){
            if ($self->{record_errors} || $response->{force_error_logging}){
                log_global()->error($e);
            }
            else{
                log_global()->warn($e);
            }
        }
    }
    return $response;
}

sub before_normal_test_start {
    shift->_api_call('before_normal_test_start', @_);
}

sub normal_test_started {
    shift->_api_call('normal_test_started', @_);
}

sub normal_test_ended {
    shift->_api_call('normal_test_ended', @_);
}

sub testcase_started {
    shift->_api_call('testcase_started', @_);
}

sub testcase_ended {
    shift->_api_call('testcase_ended', @_);
}

sub post_job_started {
    shift->_api_call('post_job_started', @_);
}

sub all_tests_ended {
    shift->_api_call('all_tests_ended', @_);
}

sub get_errors {
    shift->_api_call('get_errors', @_);
}

sub wait_for_asups_collected {
    shift->_api_call('wait_for_asups_collected', @_);
}

sub reset_and_quiesce {
    my $self = shift;
    $self->reset;
    while ($self->has_timer_running){
        log_global()->comment("Running timer detected. Sleeping 5s");
        sleep 5;
    }
    # Only run this if we have connected to nodes already
    # Saves on memory
    if ($self->has_node_objs){
        $self->wait_for_asups_collected;
    }
    return;
}

1;
