#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Mergevalidate Tools Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here


=head1 NAME

NACL::Tools::cdef_merge::Mergevalidate - validates the merge process

=head1 REQUIRES

Tharn, File::Copy::Recursive,NATE::BaseException.

=head1 DESCRIPTION

NACL::Tools::cdef_merge::Mergevalidate is a class for which hold the routines
that validates the results of cdef_merging. Based on the results
obtained the call for the other process like testing and checkin
will be decided.

=head1 METHODS

=head2 validate_result

It will validate the results of the merge process based on the results obtained
and returns "1".

=cut

package NACL::Tools::cdef_merge::Mergevalidate;

use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();
use File::Copy::Recursive qw(dirmove);
use NATE::BaseException qw(:try);

sub validate_result {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $nacl = "$args{workspace}" . '/' . "$args{'nacl-cdefs-path'}->{$args{buildtype}}";
    my $present_proc = $args{process_id};
    my $merge        = $args{'merge-location'};

    my $worst_result = ref( $present_proc->worst_result() );
    if ( $worst_result !~ /Script|Param|Fail|Fatal|Config/i ) {

        opendir( DIR, $nacl )
            or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "couldnt open directory $nacl ,Reason: $! \n");
            };
        my @cdef_files = grep( /^\S+\.(cdefs|xml)$/, readdir(DIR) );
        foreach $file (@cdef_files) {
            $file = "$nacl" . "/" . "$file";
            chmod 0777, $file
                or do {
                $Log->exit() if $may_exit;
                NATE::BaseException->throw(
                    "couldnt change permission for the file $file ,Reason: $! \n"
                );
                };
        }
        @mov = dirmove( "$merge", "$nacl" )
            or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "couldnt move the files from $merge directory to $nacl directory ,Reason: $! \n"
            );
            };
    } else {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw(
            "the cdef merging process failed please check the log metioned below"
        );
    }

    $Log->exit() if $may_exit;

    return 1;
} ## end sub validate_result

1;

