#
# Copyright (c) 2014 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Testmerge Module
## @author dl-nacl-dev@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::Tools::cdef_merge::Testmerge

=head1 REQUIRES

Tharn, XML::XPath,NATE::BaseException.

=head1 DESCRIPTION

NACL::Tools::cdef_merge::Testmerge is a class for which hold the routines
that checks for syntactical errors and validates the data present in cdefs

=head1 METHODS

=head2 test_loading

It will test for the syntactical error of cdefs validate the data in cdefs
and returns "PASS" if no errors in cdefs else returns "FAIL".

=cut

package NACL::Tools::cdef_merge::Testmerge;
use NATE::BaseException;
use XML::XPath;
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

sub test_loading {
    $Log->enter() if $may_enter;
    my ( $self, %args ) = @_;
    my $path = "$args{workspace}" . '/' . "$args{'nacl-cdefs-path'}->{$args{buildtype}}";
    my $interface = $args{interface};
    my $load_result;
    if ( $interface =~ /cli/i ) {
        $load_result = load_cli( cdef_path => $path );
    } elsif ( $interface =~ /zapi/i ) {
        $load_result = load_zapi( cdef_path => $path );
    }
    $Log->exit() if $may_exit;
    return $load_result;
} ## end sub test_loading

sub load_cli {
    my %args      = @_;
    my $cdef_path = $args{cdef_path};
    my $result    = "PASS";
    my @files     = ();
    opendir( DIR, $cdef_path );
    my $sep = ( $cdef_path =~ /^\// ) ? "/" : "\\";

    foreach my $file ( readdir(DIR) ) {
        next if ( $file !~ /\.cdefs$/ );
        push @files, $cdef_path . $sep . $file;
    }
    closedir(DIR);

    foreach my $file (@files) {
        my $content = NATE::Inc::get_file($file)
            or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Error reading file '$file': $@");
            };
        eval $content;
        if ($@) {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw(
                "Could not load CDEF file: $file\nReason: $@ \n");
        }
    }
    return $result;
} ## end sub load_cli

sub load_zapi {
    my %args   = @_;
    my $path   = $args{cdef_path};
    my $result = "PASS";
    my @files  = ();

    # check for valid cdef path
    if ( -d $path ) {

        # retrieve cdef files. non HTTP path
        opendir( DIR, $path ) or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw( "Cannot open dir: $path", );
        };
        my $sep = ( $path =~ /^\// ) ? "/" : "\\";
        foreach my $file ( readdir(DIR) ) {

            # skip files that does not have cdefs extension
            next if ( $file !~ /\.xml$/ );
            push @files, ( $path . $sep . $file );
        }
        closedir(DIR);
    } else {
        $Log->exit() if $may_exit;

        # Throw an exception if it is not a directory
        NATE::BaseException->throw(
            "$path is not a directory ,the error is : $@",
        );

    }

    # if files list is empty, cdef files have not been consolidated
    if ( scalar(@files) == 0 ) {
        $Log->exit() if $may_exit;
        NATE::BaseException->throw("No cdef data files found in: $path ");
    }

    # Load each of the XML file and get an XPATH object for each of the
    # ZAPI and store them directly in the name of ZAPI in schema object
    foreach my $cdef_file ( sort @files ) {
        $cdef_file =~ /([\w-]+)\.xml$/;
        my $file = $1;
        my $content = NATE::Inc::get_file($cdef_file) or do {
            $Log->exit() if $may_exit;
            NATE::BaseException->throw("Error reading file '$cdef_file': $@");
        };
        my $xpath_obj = XML::XPath->new($content);
        my @nodes     = $xpath_obj->findnodes("//command");
        foreach my $node (@nodes) {

            # If there any options whose name is "command", even that will be
            # picked up. So ignore processing in that case
            my $xp = XML::XPath->new( $node->toString() );

            # The 'command' tag which we are interested in will have no
            # attributes and there are no child nodes under it.
            # Its always of the form
            #           <command>zapi_name</command>
            # This is the basis for our check
            my @attr_nodes = $xp->findnodes("//@*");
            next if ( scalar @attr_nodes > 0 );
            my $value    = $node->string_value;
            my @commands = $xpath_obj->findnodes("/$file/$value");
            if ( scalar(@commands) != 1 ) {
                $Log->exit() if $may_exit;
                $text
                    = "The ZAPI '$value' should be defined only once in the "
                    . "XML CDEFs but its defined "
                    . scalar(@commands)
                    . " times";
                NATE::BaseException->throw(
                    "$text",

                );
            }
        }
    }
    return $result;
} ## end sub load_zapi
1;

