# Copyright (c) 2015 NetApp, Inc.
# All rights reserved.

## @summary [Sortof]UnitTest supporting class for Bubblegum.
##
## @description
##   The main use for this is when changes are being made to C<Bubblegum> and
##   its high complexity methods and this would provide a baseline reference
##   for I<data> and I<lookup>.
##   Since the tests for lookup and data are identical (and possibly other
##   future data structures generated by Bubblegum), C<Sub::Exporter> is used
##   for generating both generate and create tests.
##   The default C<IKDumper> method options plus C<Purity =E<gt> 1> are used
##
## @author ng-dpg-smgr-auto
##

package NACL::UnitTest::AppBuilder::IceKing::Bubblegum;

use 5.014;
use warnings;

use Carp;
use Readonly;
use Test::Builder ();
use Test::Deep    ();
use Text::Diff;
use NACL::AppBuilder::IceKing::Util qw(IKDumper);
use Sub::Exporter -setup => {
    exports => [
        data_ok       => \&_generate_ok,
        lookup_ok     => \&_generate_ok,
        data_create   => \&_generate_create,
        lookup_create => \&_generate_create,
    ],
    groups => {default => [qw(data_ok lookup_ok data_create lookup_create)]},
};

Readonly my $DEFAULT_TEMPLATE_FILE => 'BG-%s.reference';
Readonly my $DEFAULT_TEMPLATE_MESSAGE_DIFF =>
    'Current %s matches the reference.';
Readonly my $DEFAULT_TEMPLATE_MESSAGE_CREATE =>
    'Reference %s file "%s" was saved.';

my $Test = Test::Builder->new;

sub _generate_ok {
    my ($class, $name, $arg, $col) = @_;
    return sub {
        my ($bg, $file, $message) = @_;
        my $selector = substr($name, 0, -3);
        $file    //= sprintf($DEFAULT_TEMPLATE_FILE,         $selector);
        $message //= sprintf($DEFAULT_TEMPLATE_MESSAGE_DIFF, $selector);
        my $diff = diff(
            $file,
            \&IKDumper($bg->$selector, Purity => 1),
            {STYLE => 'Table'}
        );
        if ($diff) {
            $Test->diag("Found a mismatch! ($diff)");
            return;
        }
        $Test->ok(1, $message);
        return 0;
        }
}

sub _generate_create {
    my ($class, $name, $arg, $col) = @_;
    return sub {
        my ($bg, $file, $message) = @_;
        my $selector = substr($name, 0, -7);
        $file //= sprintf($DEFAULT_TEMPLATE_FILE, $selector);
        $message
            //= sprintf($DEFAULT_TEMPLATE_MESSAGE_CREATE, $selector, $file);
        open my $F, '>', $file or croak("Couldn\'t open file $file");
        print {$F} IKDumper($bg->$selector, Purity => 1);
        close $F or croak("Couldn\'t close file $file");
        $Test->ok(1, $message);
        return 0;
        }
}
1;

=head1 NAME

IceKing::Test::Bubblegum - [Sortof]UnitTest supporting class for Bubblegum.

=head1 SYNOPSIS

use IceKing::Test::Bubblegum;

=head1 DESCRIPTION

This class will export the C<data_ok lookup_ok data_create lookup_create>
methods used for unit tests of the C<Bubblegum> data processing class.

The C<*_create> methods are creating files in the current directory by default
(a I<file> parameter can be specified to overwrite that) for establishing
a baseline using Dumper. These would normally be used at the beginning of the
dev process.

The C<*_ok> methods are the ones that would normally be used in the unit test
script to verify whether there are differences and those will be report in a
table format.

=cut

=head1 AUTHOR/MAINTAINER

=over

=item SMGR Automation Team (ng-dpg-smgr-auto@netapp.com)

=back

$Id:$
Last revised $Date:$ by $Author:$

=cut

