#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::CIFS;

use strict;
use warnings;

use parent qw(NACL::UnitTest::AppBuilder::Xgen::Config::Lif);

use NACL::AppBuilder::Xgen::Resource::Cifs;
use NACL::AppBuilder::Xgen::Resource::Dns;

use constant DOMAIN_NAME => 'ctl.gdl.englab.netapp.com';
use constant AD_SERVER_USER_NAME => 'adadmin';
use constant AD_SERVER_PASSWORD => 'adadmin';
use constant NAME_SERVER_IPS => [ '10.224.223.130', '10.224.223.131' ];

sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

   
    #Configure CIFS
    $self->generate_cifs_dns_resources(%args);
    $self->generate_cifs_resources(%args);
}

sub generate_cifs_dns_resources {
    my ($self, %args) = @_;
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    my $vserver = shift @vservers;
    my $vserver_name = $vserver->get_attribute("vserver");

    my $dns_resource = NACL::AppBuilder::Xgen::Resource::Dns->new(
           'vserver'      => $vserver_name,
           'domains'       => $self->DOMAIN_NAME,
           'name-servers' => $self->NAME_SERVER_IPS,
           'state'        => 'enabled',
       );
   $dns_resource->add_parent_resources(resources => [$vserver]);
   $self->add_resources(resources => [ $dns_resource ]);
}


sub generate_cifs_resources {
    my ($self, %args) = @_;
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    my $vserver = shift @vservers;
    my $vserver_name = $vserver->get_attribute("vserver");

    my $cifs_resource = NACL::AppBuilder::Xgen::Resource::Cifs->new(
           'vserver'  => $vserver_name,
           'domain'   => $self->DOMAIN_NAME,
           'username' => $self->AD_SERVER_USER_NAME,
           'password' => $self->AD_SERVER_PASSWORD,
           'netbios-aliases' => [ 'test_alias' ],
           'group-policy-enabled' => 'enabled' 
       );
   $cifs_resource->add_parent_resources(resources => [$vserver]);
   $self->add_resources(resources => [ $cifs_resource ]);
}
1;
