#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::FailoverGroup;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Vserver );
use NACL::AppBuilder::Xgen::Resource::FailoverGroup;

# This demo is a very simple example that creates vservers and volumes based on constant values.
# This demo makes 1 vserver and 2 volumes.  It uses aggregates that are found from it's parent's (ONTAP.pm)
# build function.  This demo assumes that there are data aggregates that already exist.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate broadcast domain resources
    $self->generate_failover_group_resources(%args);
}

sub generate_failover_group_resources {
    my ($self, %args) = @_;

    # Get my broadcast domains.
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    # Add the port to the first BD found
    my $vserver = $vservers[0];

    # Get my ethernet ports.
    my $Port_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Port::Eth");

    my $bd_port = undef;

    foreach my $port (@{$Port_resources}) {
        next if($port->get_attribute('cluster_name') ne $vserver->get_attribute('cluster_name'));
        my $failover_group_res = NACL::AppBuilder::Xgen::Resource::FailoverGroup->new('vserver' => $vserver->get_attribute('vserver'),
                                                                                      'failover-group'          => "FG_" . $port->get_attribute('ipspace') . "_" . $port->get_attribute('broadcast_domain'),
                                                                                      node => $port->get_attribute('node_name'),
                                                                                      port => $port->get_attribute('port_name'),
                                                                                      );

        # connect the vs resource to the aggr's cluster parent resource
        $failover_group_res->add_parent_resources(resources => [$port,$vserver]);

        $self->add_resources(resources => [$failover_group_res]);
    }


}

1;

