#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::IPspace::BroadcastDomain::Port;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::IPspace::BroadcastDomain );
use NACL::AppBuilder::Xgen::Resource::IPspace::BroadcastDomain::Port;

# This demo is a very simple example that creates vservers and volumes based on constant values.
# This demo makes 1 vserver and 2 volumes.  It uses aggregates that are found from it's parent's (ONTAP.pm)
# build function.  This demo assumes that there are data aggregates that already exist.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate broadcast domain resources
    $self->generate_broadcast_domain_port_resources(%args);
}

sub generate_broadcast_domain_port_resources {
    my ($self, %args) = @_;

    # Get my broadcast domains.
    my $BD_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::IPspace::BroadcastDomain");
    my @BDs = @{$BD_resources};
    # Add the port to the first BD found
    my $BD = $BDs[0];

    # Get my ethernet ports.
    my $Port_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Port::Eth");

    my $bd_port = undef;
    # Just going to use the first port that is in the default ipspace and default bd
    foreach my $port (@{$Port_resources}) {
        if($BD->get_attribute('cluster_name') eq $port->get_attribute('cluster_name') and
           $port->get_attribute("ipspace") eq 'Default' and
           $port->get_attribute("broadcast_domain") eq 'Default') {
            $bd_port = $port;
        }
        last if defined $bd_port;
    }

    NATE::BaseException->throw("Could not find a port in the Default IPspace and Default Broadcast Domain") if (not defined $bd_port);

    my $bd_port_res = NACL::AppBuilder::Xgen::Resource::IPspace::BroadcastDomain::Port->new('broadcast-domain' => $BD->get_attribute('broadcast-domain'),
                                                                                            'ipspace'          => $BD->get_attribute('ipspace'),
                                                                                            node => $bd_port->get_attribute('node_name'),
                                                                                            port => $bd_port->get_attribute('port_name'),
                                                                                            );

    # connect the vs resource to the aggr's cluster parent resource
    $bd_port_res->add_parent_resources(resources => [$bd_port,$BD]);

    $self->add_resources(resources => [$bd_port_res]);
}

1;

