#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::IscsiIsns;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Lif);
use NACL::AppBuilder::Xgen::Resource::VserverIscsiIsns;
use NACL::AppBuilder::Xgen::Resource::Iscsi;
use constant DATA_PROTOCOLS => [ 'none' ];
use constant ISNS_SERVER_IP => '10.235.33.253';


sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate iscsi resources.
    $self->generate_iscsi_resources(%args);

    # Call to generate iscsi isns resourses
    $self->generate_iscsi_isns_resources(%args);
}


sub generate_iscsi_resources {
    my ($self, %args) = @_;

    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    my @iscsi;

    foreach my $vs (@vservers) {
        my $iscsi_res = NACL::AppBuilder::Xgen::Resource::Iscsi->new(vserver   => $vs->get_attribute("vserver"),
                                                                 'status-admin' => "up",
                                                                 'target-alias' => $vs->get_attribute("vserver"));

        $iscsi_res->add_parent_resources(resources => [$vs]);

        push(@iscsi,$iscsi_res);
    }
    $self->add_resources(resources => \@iscsi);
}


sub generate_iscsi_isns_resources {
    my ($self, %args) = @_;
    my ($vserver_resources, @vservers, $vs);
    $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    @vservers = @{$vserver_resources};
    foreach $vs ( @vservers ) {
        my $iscsi_isns_res = NACL::AppBuilder::Xgen::Resource::VserverIscsiIsns->new(
            vserver => $vs->get_attribute("vserver"),
            'status-admin' => "up",
            'address' => $self->ISNS_SERVER_IP);
        $iscsi_isns_res->add_parent_resources(resources => [$vs]);
        $self->add_resources(resources => [ $iscsi_isns_res ]);
    }
}

1;

