#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::LunAttribute;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Lun );

use NACL::AppBuilder::Xgen::Resource::LunAttribute;

# This demo is a very simple example that creates vservers and volumes based on constant values.
# This demo makes 1 vserver and 2 volumes.  It uses aggregates that are found from it's parent's (ONTAP.pm)
# build function.  This demo assumes that there are data aggregates that already exist.
sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate lun resources.
    $self->generate_lun_attribute_resources(%args);
}

# Generates lun resources based on the TOTAL_lun_COUNT constant.
# If there is more than one lun then lun will alternate volumes
# based on the volumes created in NACL::UnitTest::AppBuilder::Xgen::Config::Volume
sub generate_lun_attribute_resources {
    my ($self, %args) = @_;

    my $lun_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Lun");
    my @luns = @{$lun_resources};
    my @lun_attributes;

    foreach my $lun (@luns) {
        my $lun_attribute_res = NACL::AppBuilder::Xgen::Resource::LunAttribute->new('vserver'       => $lun->get_attribute("vserver"),
                                                                                    'volume'        => $lun->get_attribute("volume"),
                                                                                    'lun'           => $lun->get_attribute("lun"),
                                                                                    'name'          => 'foo',
                                                                                    'value'         => 'bar');

        # connect the vs resource to the lun's cluster parent resource
        $lun_attribute_res->add_parent_resources(resources => [$lun]);

        push(@lun_attributes, $lun_attribute_res);
    }
    $self->add_resources(resources => \@lun_attributes);
}

1;

