#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::LunMap;

use strict;
use warnings;

use base qw( NACL::UnitTest::AppBuilder::Xgen::Config::Lun );
use base qw( NACL::UnitTest::AppBuilder::Xgen::Config::Igroup );
use base qw( NACL::UnitTest::AppBuilder::Xgen::Config::Lif );

use NACL::AppBuilder::Xgen::Resource::Vserver;
use NACL::AppBuilder::Xgen::Resource::Volume;
use NACL::AppBuilder::Xgen::Resource::Lun;
use NACL::AppBuilder::Xgen::Resource::LunMap;
use NACL::UnitTest::AppBuilder::Xgen::Config::Igroup;

# These are the constant values that determine what happens in the config.
use constant LUN_OS_TYPE => 'linux';
use constant IGROUP_OS_TYPE => 'linux';

sub build {
    my ($self, %args) = @_;

    # We only want to do this if something did not pass in any Ethernet fabric definitions.
    # This fabric will include all the ips from all nodes and not sort specific to clusters.
    # This means that IPs will not be owned by a specific cluster and can be used by any cluster.
    unless (scalar keys %{$args{eth_fabric_data}}) {
        $args{eth_fabric_data} = $self->generate_eth_fabric_data_from_natehost_files(%args);
    }

    # generate luns
    $self->SUPER::build(%args);

    $self->generate_igroup_resources(%args);

    $self->generate_lif_resources(%args);

    $self->generate_lunmap_resources(%args);
}

sub generate_lunmap_resources {
    my ($self, %args) = @_;

    my @lunmap_resources;
    my $vserver_resources = $self->get_resources(
                                type =>"NACL::AppBuilder::Xgen::Resource::Vserver");

    foreach my $vs_res (@{$vserver_resources}) {
        my @lun_resources = $vs_res->get_descendants(
                                type => 'NACL::AppBuilder::Xgen::Resource::Lun');
        my @igroup_resources = $vs_res->get_children_resources(
                                type => 'NACL::AppBuilder::Xgen::Resource::Igroup');

        foreach my $lun_res (@lun_resources) {
            my $igroup_res = shift(@igroup_resources);
            push(@igroup_resources, $igroup_res);

            my $lunmap_res = NACL::AppBuilder::Xgen::Resource::LunMap->new(
                                                vserver => $lun_res->get_attribute('vserver'),
                                                volume => $lun_res->get_attribute('volume'),
                                                lun => $lun_res->get_attribute('lun'),
                                                igroup => $igroup_res->get_attribute('igroup'),
                                               );

            $lunmap_res->add_parent_resources(resources => [$lun_res, $igroup_res]);

            push(@lunmap_resources, $lunmap_res);
        }
    }
    $self->add_resources(resources => \@lunmap_resources);
}

1;

