#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Snapmirror;

use strict;
use warnings;
use Data::Dumper qw(Dumper);

use parent qw(NACL::UnitTest::AppBuilder::Xgen::Config::Volume);

use NACL::AppBuilder::Xgen::Resource::Snapmirror;
use NATE::BaseException qw(:try);

use constant TOTAL_VSERVER_COUNT => 2;
use constant TOTAL_VOLUME_COUNT => 2;
use constant TOTAL_SNAPMIRROR_COUNT => 1;


sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);
    #Call to create snapmirror
    $self->generate_snapmirror_resources(%args);
}



sub generate_snapmirror_resources {
    my ($self, %args) = @_;

    my (@snapmirrors, $src_vserver, $dst_vserver, 
        $src_vserver_name, $dst_vserver_name, $src_volume,
        $dst_volume, @src_volumes, @dst_volumes);
    my $vserver_resources = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Vserver");
    my @vservers = @{$vserver_resources};
    for( my $count = 0; $count < TOTAL_SNAPMIRROR_COUNT; $count++ ) {
       $src_vserver = shift @vservers;
       $dst_vserver = shift @vservers;
       $src_vserver_name = $src_vserver->get_attribute("vserver");
       $dst_vserver_name = $dst_vserver->get_attribute("vserver");
       if ( $src_vserver_name eq $dst_vserver_name ) {
           NATE::BaseException->throw("\n source and destination vserver should not be the same..");
       }
       @src_volumes = $src_vserver->get_children_resources(type => 'NACL::AppBuilder::Xgen::Resource::Volume');
       $src_volume = shift @src_volumes;

       @dst_volumes = $dst_vserver->get_children_resources(type => 'NACL::AppBuilder::Xgen::Resource::Volume');
       $dst_volume = shift @dst_volumes;
       $dst_volume->set_private_attributes(type => 'DP');

       my $snapmirror_resource = NACL::AppBuilder::Xgen::Resource::Snapmirror->new(
           'source-volume'   => $src_volume->get_attribute("volume"),
           'source-vserver'  => $src_vserver_name,
           'destination-volume'   => $dst_volume->get_attribute("volume"),
           'destination-vserver'  => $dst_vserver_name,
       );
       $snapmirror_resource->add_parent_resources(resources => [$src_volume, $dst_volume]);
       push(@snapmirrors,$snapmirror_resource);
       $self->add_resources(resources => \@snapmirrors);
   }
}
1;
