#
# Copyright (c) 2015 NetApp, Inc., All Rights Reserved.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package NACL::UnitTest::AppBuilder::Xgen::Config::Volume::Mount;

use strict;
use warnings;

use parent qw( NACL::UnitTest::AppBuilder::Xgen::Config::Volume );

use NACL::AppBuilder::Xgen::Resource::Volume::Mount;
use constant TOTAL_VOLUME_COUNT => 4;

sub build {
    my ($self, %args) = @_;

    $self->SUPER::build(%args);

    # Call to generate volume resourses
    $self->generate_volume_mount_resources(%args);
}

sub generate_volume_mount_resources {
    my ($self, %args) = @_;

    my $volumes  = $self->get_resources(type => "NACL::AppBuilder::Xgen::Resource::Volume");

    my @mounts;
    my $first_volume = shift (@{$volumes});
    my $mount_res = NACL::AppBuilder::Xgen::Resource::Volume::Mount->new('vserver'        => $first_volume->get_attribute("vserver"),
                                                                         'volume'         => $first_volume->get_attribute("volume"),
                                                                         'junction-path'  => "/" . $first_volume->get_attribute("volume"),);

    # connect the vs resource to the aggr's cluster parent resource
    $mount_res->add_parent_resources(resources => [$first_volume]);

    push(@mounts, $mount_res);

    my $junction_path = '';
    my @volume_mount_info = ();
    foreach my $volume (@{$volumes}) {
        $junction_path .= "/" . $volume->get_attribute("volume");
        my $volume_mount = {'vserver'        => $volume->get_attribute("vserver"),
                            'volume'         => $volume->get_attribute("volume"),
                            'junction-path'  => $junction_path};
        push (@volume_mount_info,$volume_mount);
    }
    $mount_res = NACL::AppBuilder::Xgen::Resource::Volume::Mount->new(volumes => \@volume_mount_info);
    foreach my $volume (@{$volumes}) {
    # connect the vs resource to the aggr's cluster parent resource
    $mount_res->add_parent_resources(resources => [$volume]); 
    }
  
    push(@mounts, $mount_res);
    $self->add_resources(resources => \@mounts);
    push (@{$volumes},$first_volume);

}

1;

