# Copyright (c) 2016 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.

## @summary Vault workflow for metawafl for cad testing
## @author vinitaa@netapp.com
## @status shared
## @pod here
##

package dp::snapmirror::vault::invoke_validation;

use strict;
use warnings;


use Params::Validate qw(validate validate_with :types);

use Data::Dumper;

use NACL::STask::Volume;
use NACL::STask::VolumeFile;
use NACL::C::VolumeFileClone;
use NACL::AppBuilder::DpgWorkflows::Workload qw(_execute_iofence_cmd);
use NACL::AppBuilder::DpgWorkflows::Validation qw(get_composite_aggr_stats);
use dp::snapmirror::paloma::vault::lib::VaultCommonUtils qw(
    dataset
    SFR_test
    SFR_verify
    create_snapshot
);
use NACL::AppBuilder::DpgWorkflows::workflow qw(
    Wf_sm_checker
    Wf_snapmirror_cleanup
);                                               

use NATE::Log qw(log_global);

sub no_op {
    return 1;
}

=head1 NAME

FUNC_RESYNC_DeletedRelationship

=head2 DESCRIPTION
   Update transfer a relationship where the source volume contains sis-cloned
   files from snapshot

Steps
   1) Create a snapmirror relationship
   1a) Populate the source volume with a lot of qtrees and text files
   2) Issue a baseline transfer
   3) Create some new dataset or modify existing data on the source
   4) Create multiple sis-clones from snapshot by 'file clone create -volume
       -source-path -destination-path -snapshot'
   5) Create multiple sis-clones from active filesystem by 'file clone create
      -source-path -destination-path'
   6) Issue snapmirror update transfer; let it complete
   7) Run 'snapmirror check'

=head3 OPTIONS
  nfsclients: ARRAYREF. (Required) nfsclients Object
  sm_objs: ARRAYREF. (Required) Snapmirror Objects

=cut

sub FUNC_RESYNC_DeletedRelationship {
    shift;
    my %args = validate_with(
        params => \@_,
        spec   => {
            nfsclients => {type => ARRAYREF | SCALAR, optional => 0},
            sm_objs    => {type => ARRAYREF,          optional => 0},
        },
    );

    my $log = log_global();
    my $may_enter = $log->may_enter();
    my $may_exit  = $log->may_exit();
    $log->enter() if $may_enter;

    my $nfsclients  = $args{nfsclients};
    my @nfsclients  = split ',', $nfsclients;
    my $snapmirror  = @{$args{sm_objs}}[0];
    my $snapmirrors = $args{sm_objs};

    my $source_path      = $snapmirror->{source_path};
    my $destination_path = $snapmirror->{destination_path};
    my ($vserver_src, $vol_src_name) = ($source_path      =~ /^(.*):(.*)$/);
    my ($vserver_dst, $vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);

    # Get vol objs
    my $vol_src = NACL::C::Volume->find(
        command_interface => $snapmirror->{source_ci},
        filter            => {
            volume  => $vol_src_name,
            vserver => $vserver_src,
        }
    );

    my $vol_dst = NACL::C::Volume->find(
        command_interface => $snapmirror->{command_interface},
        filter            => {
            volume  => $vol_dst_name,
            vserver => $vserver_dst,
        }
    );

    # get src/dst nodes based on vol residency
    my $node_src = NACL::C::Node->new(name => $vol_src->get_local_node_name());
    my $node_dst = NACL::C::Node->new(name => $vol_dst->get_local_node_name());

    my $src_path = '/vol/'.$vol_src->volume().'/f1';
    my $dst_path = '/vol/'.$vol_src->volume().'/dst_clone';

    my $client_name = $nfsclients[0];
    my $Client = NACL::C::Client->new( name => $client_name );
    my $Populate_flag = 'false';

    if ($vol_src->get_one_state_attribute('junction-path') eq "-") {
        $vol_src->mount('junction-path' => "/" . $vol_src->volume());
    }
    
    my $Dst_Aggr_name = $vol_dst->get_one_state_attribute('aggregate');
    my $dst_aggr = NACL::STask::Aggregate->find(
               command_interface => $node_dst,
                filter => { aggregate => $Dst_Aggr_name },
            ); 
    my($comp_t1,$perf_t1) = get_composite_aggr_stats(aggr=>$dst_aggr);
    $log->comment('value of comp_t1 ' . $comp_t1);
    $log->comment('value of perf_t1 ' . $perf_t1);

    $log->step('Initialize snapmirror');
    $snapmirror->initialize(nacltask_wait => 1);
    
    sleep 120;

    my($comp_t2,$perf_t2) = get_composite_aggr_stats(aggr=>$dst_aggr);

    $log->comment('value of comp_t2 ' . $comp_t2);
    $log->comment('value of perf_t2 ' . $perf_t2);
   
    if($perf_t2 < $perf_t1){
       $log->error("After initialize performance tier value must have been greater than previous value");  
    }   

    $log->log(NATE::Result::Pass->new('se_bre_int_48'));

    $log->exit();
    return;

}
1;
