# Hey emacs, this is basically -*- perl -*-
#
#  Copyright (c) 2003-2010 Network Appliance, Inc.
#  All rights reserved.
#
## @summary VserverDiscovery
##
##
## @description
##   VserverDiscovery doe's an auto construction of vserver hostrec.
##   This is need because in smokes environment the tharnhost for
##   Vserver doesn't exist.
##
##
## @usage
##
##   build_vserver_tharnhost() constructs a vserver thanrnhost. This has
##   to be called before component call NACL::C::Vserver->find() so that
##   find() can get a vserver tharnhost. It also takes node name and
##   vserver name (in this order) as its inputs if you would want this
##   method from avoid reading test parameters to find vserver and node.
##
##
##   use NACL::UnitTest::C::VserverDiscovery qw(build_vserver_tharnhost);
##
##   build_vserver_tharnhost();
##   NACL::C::Vserver->find();
##
##       or
##   build_vserver_tharnhost($nodename, $vservername);
##
## @status shared
##
## @author dheeraj.k@netapp.com

package NACL::UnitTest::C::VserverDiscovery;
use NACL::C::Node;
use NACL::C::Aggregate;
use NACL::C::Vserver;
use NACL::CS::Interface;
use NACL::CS::Vserver;
use Tharn;
use NACL::Exceptions::NoElementsFound qw(:try);
use Params::Validate qw(validate_with ARRAYREF SCALAR);

BEGIN {
    use Exporter qw(import);
    our @EXPORT_OK = qw(
        build_vserver_tharnhost
        update_vserver_aggr_list
    );
} ## end BEGIN

sub build_vserver_tharnhost {
    my ($node_name, $vserver) = @_;
    my ($node, $filter);
    if (defined $node_name) {
        $node = NACL::C::Node->new(name => $node_name);
    } else {
        $node = NACL::C::Node->find();
    }
    unless (defined $vserver) {
        $vserver = param( 'VSERVER', -default, "vs0" );
    }
    if ( $node->mode() eq 'CMode' ) {
        try {
            $filter = { vserver => $vserver, 'home-node' => $node->node(), 'address-family' => 'ipv4' };
            my $address = NACL::CS::Interface->fetch(
                command_interface => $node,
                filter => $filter,
                apiset_must => { interface => "CLI" }
            )->address();
            Tharn::host( $vserver
                    . ";hosttype=filer-ng-vserver;default_ip=$address;"
                    . "default_username=vsadmin;default_password=netapp1!" );
        } ## end try
        catch NACL::Exceptions::NoElementsFound with {
            Tharn::host( $vserver . ";hosttype=filer-ng-vserver" );
        };
    } ## end if ( $node->mode() eq ...)
} ## end sub build_vserver_tharnhost

sub update_vserver_aggr_list {
    my %opts = validate_with(
        params => \@_,
        spec   => {
            command_interface  => { isa  => 'NACL::C::CommandInterface' },
            vserver            => { type => SCALAR },
            aggregate          => { type => ARRAYREF, default => [] },
            add_all_aggregates => { type => SCALAR, default => 1 },
        },
    );

    my @aggrs;
    my @aggr_names;
    my @aggr_list_final;

    # get existing vserver aggregate list.
    my $vserver = NACL::CS::Vserver->fetch(
        command_interface => $opts{command_interface},
        filter            => { vserver => $opts{vserver} },
        requested_fields  => ['aggr-list']
    );
    my @aggr_list = $vserver->aggr_list();
    foreach $aggr (@aggr_list){
        push @aggr_list_final , $aggr if ( $aggr ne '-' );
    }

    # add all aggregates to vserver aggr list.
    if ( $opts{add_all_aggregates} ) {
        @aggrs = NACL::C::Aggregate->find(
            command_interface => $opts{command_interface} );
        foreach my $aggr (@aggrs) {
            push @aggr_names, $aggr->aggregate();
        }
    } ## end if ( $opts{add_all_aggregates...})

    # remove duplicates.
    my %hash
        = map { $_, 1 } ( @aggr_names, @aggr_list_final , @{ $opts{aggregate} } );

    @aggr_names = keys(%hash);

    NACL::C::Vserver->modify(
            command_interface => $opts{command_interface},
            vserver           => $opts{vserver},
            'aggr-list'       => \@aggr_names
    );
} ## end sub update_vserver_aggr_list

1;

