package NACL::UnitTest::MTask::MCC::MockObject;

use strict;
use warnings;

use parent 'Test::MockObject';

# Hash for storing the initialization filename and line number of each
# object
my %inits = ();

sub new {
    my $type = shift;

    my (undef, $file, $line) = caller();

    my $self = $type->SUPER::new(@_);

    $inits{ $self->_get_key } = [ $file, $line ];

    return $self;
}

sub dispatch_mocked_method
{
    my $self = $_[0];
    my $sub  = splice( @_, 1, 1 );

    my $subs = Test::MockObject::_subs( $self );
    if (exists $subs->{$sub})
    {
        $self->log_call( $sub, @_ );
        goto &{ $subs->{$sub} };
    }
    else
    {
        require Carp;
        Carp::carp("Un-mocked method '$sub()' called on object" . $self->initialized_whence());
    }

    return;
}


sub initialized_whence {
    my $self = shift;

    if ( wantarray ) {
        return @{ $inits{ $self->_get_key } };
    }
    else {
        return join(q{ }, @{ $inits{ $self->_get_key } });
    }
}

1;

=head1 NAME

NACL::UnitTest::MTask::MCC::MockObject - Mock object with additions

=head1 SYNOPSIS

   use NACL::UnitTest::MTask::MCC::MockObject;

   # From Test::MockObject
   my $mock = NACL::UnitTest::MTask::MCC::MockObject->new();
   $mock->set_true( 'somemethod' );
   ok( $mock->somemethod() );

   $mock->set_true( 'veritas')
        ->set_false( 'ficta' )
        ->set_series( 'amicae', 'Sunny', 'Kylie', 'Bella' );

   # Get the file name and line number that this was initialized
   my ($filename, $line) = $mock->initialized_whence();


=head1 DESCRIPTION

Mock Objects are very useful for creating unit tests. In complex
systems like metrocluster, though, it can get very confusing to debug
your unit tests. This is a subclass of MockObject that adds a method
for keeping track of where any given object was initialized.

=head1 METHODS

=head2 initialized_whence()

   my ($filename, $line) = $mock->initialized_whence()

Returns the filename and line number on which the mock object was
initialized.

=head1 SEE ALSO

=over 4

=item * L<Test::MockObject|"Test::MockObject">

=back

=cut
