#!/usr/software/bin/perl

use strict;
use warnings;

use Carp;

use Test::More tests => 8, import => ['!subtest'];
use FindBin '$Bin';
use Cwd qw/abs_path/;

use Test::MockObject;
use Test::MockObject::Extends;

use NACL::UnitTest::MTask::MCC::TestUtils qw( :all );

MAIN: {
    test_use_mcc();
    test_init();
    test_perform_data_integration_checks();
}

############# Tests ###################

sub test_init {

    my $mcc = make_mock_mcc();

    isa_ok( $mcc, 'NACL::MTask::MCC::Metrocluster' );

    return;
}

sub test_perform_data_integration_checks {

    my $mcc = make_mock_mcc();

    # Set up MCC to return mock aggregates
    my $state_ref     = 'OK';
    my $verify_called = 0;
    my $module = mock_aggregates(
        'mcc'        => $mcc,
        'state_ref'  => \$state_ref,
        'verify_ref' => \$verify_called
    );

    # Mock wafliron
    my ($wafliron_mock_module, $wafliron) = mock_wafliron();

    # Serialize calls to 'subtest'.
    my ( $mcc_mod, $mock_subtest ) = mock_subtest();

    # Skip waiting for resync
    my $mock_mod = Test::MockModule->new('NACL::STask::Aggregate');
    $mock_mod->mock('wait_for_resync', sub { return; });

    # Test the method
    $mcc->verify_data_integrity();

    # Wafliron should be called
    ok( $wafliron->called('start'), 'wafliron called' );

    # We should have called 'verify' on the aggr
    is( $verify_called, 2, "verify called" );

    # Reset and try again passing aggregate names
    $verify_called   = 0;
    $wafliron->clear();

    $mcc->verify_data_integrity( 'aggregates' => [qw/aggr1 aggr2 aggr3/] );

    is( $verify_called,   3, 'named aggr verify' );
    $wafliron->called_ok('start', 'named aggr wafliron' );

    # Test with passed-in Aggregate
    $wafliron->clear();
    my $aggr = make_mock_aggregate( state_ref => \ 'ok' );

    $mcc->verify_data_integrity( 'aggregates' => [$aggr] );

    $aggr->called_ok( 'verify', 'passed aggr verify' );
    $wafliron->called_ok('start', 'passed wafliron verify');

    return;
}
