#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use NACL::MTask::MCC::Network::FibreBridge;

use Test::More tests => 4;
use Test::Warn;
use UNIVERSAL::isa;

use Test::MockObject;
use Test::MockObject::Extends;
use Test::MockModule;
use Hostrec;

MAIN: {
    test_interface();
    test_armadillo();

    exit(0);
}


#############################################################
#
# Tests
#
# These subroutines test the NACL::MTask::MCC::FibreBridge
# object.
##############################################################

sub test_interface {

    my $mock_interface = Test::MockObject->new();
    $mock_interface->set_always( 'L_get_name', 'testfibre' );
    $mock_interface->set_always( 'L_get_info', { 'item' => 'data' } );
    $mock_interface->set_always( 'L_get_apiset', {} );

    my $fibre = NACL::MTask::MCC::Network::FibreBridge->new(
        name => 'testfibre',
        impl => $mock_interface
    );

    # Name
    is( $fibre->get_name(), 'testfibre' );

    # Info
    is_deeply( $fibre->get_info(), { 'item' => 'data' } );

    # Interface
    is_deeply( $fibre->get_apiset(), {} );

    return;

}

sub test_armadillo {
    my ( $ext_arm, $mock_result ) = _make_test_armadillo();

    $mock_result->set_always( 'get_parsed_output', { test => 'data' } );

    is_deeply( $ext_arm->L_get_info(), { test => 'data' } );

    return;
}

#############################################################
#
# Subroutines
#
# These are utility subroutines used by the test routines.
#
#############################################################

sub _make_test_armadillo {

    # Create the armadillo class with a mock apiset

    my $mock_init = Test::MockModule->new(
        'NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl');

    $mock_init->mock( 'init', sub { } );

    my $arm =
      NACL::MTask::MCC::Network::FibreBridge_Impl::Armadillo_Impl->new(
        armadillo_name => 'test_armadillo' );

    my $ext_arm = Test::MockObject::Extends->new($arm);

    my $mock_apiset = Test::MockObject->new();
    $mock_apiset->set_isa('NACL::APISet::Switch');

    my $mock_result = Test::MockObject->new();

    $mock_apiset->set_always( 'info', $mock_result );

    $ext_arm->set_always( '_create_apiset', $mock_apiset );

    $mock_init->unmock_all();

    $ext_arm->init( { armadillo_name => 'test_armadillo' } );

    return ( $ext_arm, $mock_result );

}

__END__

=head1 NAME

03_fibrebridge.t - Unit tests for FibreBridge

=head1 SYNOPSIS

    perl 02_fibrebridge.t

=head1 DESCRIPTION

Runs a series of independent unit tests on
NACL::MTask::MCC::Network::FibreBridge.  For each test, prints out
'OK' if the test passes, or 'not ok' if it fails.

=cut

