#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use Test::MockModule;
use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use NACL::MTask::MCC::Asup;

MAIN: {
    test_use_mcc();
    test_asup_invoke_all();
    done_testing();
}

sub test_asup_invoke_all {

    # Keep logger from spamming screen
    $NACL::MTask::MCC::Asup::Log = make_mock_log();

    # Create some mock nodes
    my @nodes;

    foreach ( 1 .. 4 ) {
        push(@nodes, mock_system_node());
    }

    my $asup = NACL::MTask::MCC::Asup->new(
        nodes => \@nodes
    );

    # Keep my test code from actually calling Autosupport
    my $mock_autosupport = Test::MockModule->new(
        'NACL::C::SystemNodeAutosupport'
    );

    my $invoke_was_called = 0;

    $mock_autosupport->mock('invoke', sub { $invoke_was_called = 1; } );

    $asup->invoke_all();

    ok($invoke_was_called, 'Autosupport invoked');

    # Mock AutoSupportHistory

    my ($as_mod, $as_cs) = mock_component_state_fetch(
        cs_class => 'NACL::CS::SystemNodeAutosupportHistory',
        cs_states => {
            status => 'sent-successful',
            subject => "Gather_diagnostic_invoked_asup_" . time() . "_"
        }
    );

    # Test validate_sent
    my $all_ok = $asup->validate_sent();
    ok($all_ok, 'All validations successful');


} ## end sub test_MY_TEST



##
## Creates a mock system node
##
sub mock_system_node {

    my $mock_node = NACL::UnitTest::MTask::MCC::MockObject->new();

    $mock_node->set_isa( 'NACL::C::SystemNode' );

    $mock_node->set_always('node', 'mock node');

    return $mock_node;

}


