#!/usr/software/bin/perl5.14.0

use
strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

use File::Slurp;
use FindBin '$Bin';

MAIN: {
    test_use_mcc();
    test_sas_adapters();
    done_testing();
}

sub test_sas_adapters {

    my $network_node = make_mock_network_node();

    my $mock_sysconfig = mock_sysconfig();

    $network_node->set_always('_init_sysconfig_va_response', $mock_sysconfig);

    my $sas_adapters = $network_node->get_sas_host_adapters();

    is_deeply(
        $sas_adapters,
           [{
             'length' => '5',
             'shelf_ids' => [
                              '21',
                              '22'
                            ],
             'name' => '0b'
           },
           {
             'length' => '2',
             'shelf_ids' => [
                              '11',
                              '12'
                            ],
             'name' => '0a'
           },
           {
             'length' => '5',
             'shelf_ids' => [
                              '21',
                              '22'
                            ],
             'name' => '0d'
           },
           {
             'length' => '2',
             'shelf_ids' => [
                              '11',
                              '12'
                            ],
             'name' => '0c'
           }
         ]
    );


} ## end sub test_sas_adapters

sub mock_sysconfig {

    my $mock_sysconfig_file = "$Bin/sysconfig.out";
    my $sysconfig_raw = read_file( $mock_sysconfig_file );

    my $mock_sysconfig = Test::MockObject->new();
    $mock_sysconfig->set_always('get_raw_output', $sysconfig_raw);

    return $mock_sysconfig;

}


