#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

MAIN: {
    test_use_mcc();
    test_plex_verify_normal();
    test_plex_verify_nso();
    test_plex_heal_aggregates();
    test_plex_heal_roots();
    done_testing();
}

sub test_plex_verify_normal {

    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    set_mcc_state_normal($mcc);

    my $mock = mock_many_plexes(
        states => [
            (   {   'status'            => 'normal,active',
                    'is-online'         => 'true',
                }
            ) x 8,
            {
                'status'            => 'normal,resyncing',
                'is-online'         => 'true',
                'resyncing-percent' => '20%'
            }
        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->validate_plex( die_on_failure => 1 );
    } 'normal state';

    return;

} ## end sub test_plex_verify_normal

sub test_plex_verify_nso {

    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    set_mcc_state_switchover($mcc);

    my $mock = mock_many_plexes(
        states => [
            (   {   'status'            => 'normal,active',
                    'is-online'         => 'true',
                }
            ) x 8,
        ( { 'status' => 'failed,inactive',
          'is-online' => 'false'
      } ) x 2,
        {
            'status'            => 'normal,resyncing',
            'is-online'         => 'true',
            'resyncing-percent' => '20%'
        }
        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->validate_plex( die_on_failure => 1 );
    } 'switchover';

    return;

}

sub test_plex_heal_aggregates {
    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    set_mcc_state_heal_aggregates($mcc);

    my $mock = mock_many_plexes(
        states => [
            (   {   'status'            => 'normal,active',
                    'is-online'         => 'true',
                }
            ) x 8,

            {
                'status'            => 'normal,resyncing',
                'is-online'         => 'true',
                'resyncing-percent' => '20%'
            },
        {
            'status'                => 'normal,inactive',
            'is-online'             => 'true'
        }
        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->validate_plex( die_on_failure => 1 );
    } 'heal aggregates';

    return;


}

sub test_plex_heal_roots {
    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    set_mcc_state_heal_roots($mcc);

    my $mock = mock_many_plexes(
        states => [
            (   {   'status'            => 'normal,active',
                    'is-online'         => 'true',
                }
            ) x 8,
            {
                'status'            => 'normal,resyncing',
                'is-online'         => 'true',
                'resyncing-percent' => '20%'
            },

        ]
    );

    my $rc;

    lives_ok {
        $rc = $mcc->validate_plex( die_on_failure => 1 );
    } 'heal roots';

    return;


}

sub mock_many_plexes {

    my %opts = validate_with(
        params => \@_,
        spec => {
            states => {
                'type' => ARRAYREF,
            }
        }
    );

    my $mock = Test::MockModule->new('NACL::CS::StorageAggregatePlexShow');

    my @plexes = map { make_mock_plex($_) } @{ $opts{'states'} };

    $mock->mock('fetch', sub {
                    my ($type, %opts) = @_;

                    defined $opts{'command_interface'}
                        or croak "Command interface not defined";

                    return @plexes;
                });

    return $mock;

}

sub make_mock_plex {

    my %opts = validate_with(
        params => \@_,
        spec => {
            status => {
                'type' => SCALAR,
            },
            'is-online' => {
                'type' => SCALAR,
            },
            'resyncing-percent' => {
                'type' => SCALAR,
                'default' => q{-},
            },
            'in-progress' => {
                'type' => SCALAR,
                'default' => 'false'
            },

        }
    );

    my $mock_plex = Test::MockObject->new();

    $mock_plex->set_always('status', $opts{'status'});

    $mock_plex->set_always('is_online', $opts{'is-online'});

    $mock_plex->set_always('resyncing_percent', $opts{'resyncing-percent'});

    $mock_plex->set_always('in_progress', $opts{'in-progress'});

    $mock_plex->set_always('plex', "Mock plex");

    return $mock_plex;

}


