#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

# Class::MethodMaker tests object type with UNIVERSAL::isa, so we
# use the UNIVERSAL::isa module to be able to use Test::MockObject
use UNIVERSAL::isa;

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;

my @errors;

MAIN: {
    test_validator_base();
    done_testing();
}

sub test_validator_base {

    my $mcc = make_mock_mcc();

    my $vtor = Test::MyValidator->new( metrocluster => $mcc );

    is($vtor->validation_type(), 'my_validator', 'validation_type' );

    is($vtor->metrocluster(), $mcc, 'metrocluster');

    @errors = ();
    lives_ok {
        $vtor->run();
    } 'Did not die when validate() returned no errors';

    @errors = ('Got an error');
    throws_ok {
        $vtor->run();
    } 'NACL::MTask::MCC::Exceptions::Validation', 'Threw validation exception';


}

{
    package Test::MyValidator;

    use base 'NACL::MTask::MCC::Validator::Base';

    sub validate {
        my $self = shift;

        $self->errors_push(@errors);
    }

}


