#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use Test::MockModule;
use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

MAIN: {
    test_use_mcc();
    test_verify_disk_paths_success();
    test_verify_disk_paths_failure();
    test_verify_disk_paths_failure_multi();
    done_testing();
}

sub test_verify_disk_paths_success {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    # Serialize calls to 'subtest'.
    my ($mcc_mod, $mock_subtest) = mock_subtest();

    $mcc->set_always('get_live_nodes', {
        'foo' => [ make_mock_network_node(
                sysconfig_output => [{'192_disks' => []}, {'192_disks' => []}]
        ) ],
        'bar' => [ make_mock_network_node(
                sysconfig_output => [{'192_disks' => []}, {'192_disks' => []}]
        ) ]

    });

    set_mcc_state_normal($mcc);

    lives_ok {
        $mcc->validate_disks( die_on_failure => 1 );
    } 'Same disks';


} ## end sub test_verify_disk_paths_success

sub test_verify_disk_paths_failure {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_live_nodes',
        {   'foo' => [ make_mock_network_node(
                sysconfig_output => [{'192_disks' => []}, {'193_disks' => []}]
            ) ]
        }
    );

    set_mcc_state_normal($mcc);

    dies_ok {
        $mcc->validate_disks( die_on_failure => 1 );
    } 'Same node, different disks';


} ## end sub test_verify_disk_paths_failure

sub test_verify_disk_paths_failure_multi {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    $mcc->set_always(
        'get_live_nodes',
        {   'foo' => [ make_mock_network_node(
                sysconfig_output => [{'192_disks' => []}, {'192_disks' => []}]
            ) ],
            'bar' => [ make_mock_network_node(
                sysconfig_output => [{'193_disks' => []}, {'193_disks' => []}]
            ) ]
        }
    );

    set_mcc_state_normal($mcc);

    dies_ok {
        $mcc->validate_disks( die_on_failure => 1 );
    } 'Different nodes, different disks';


} ## end sub test_verify_disk_paths_failure



