#!/usr/software/bin/perl5.14.0

use strict;
use warnings;

use Test::Most import => ['!subtest'];

use NACL::UnitTest::MTask::MCC::MockObject;
use NACL::UnitTest::MTask::MCC::TestUtils ':all';
use Carp;
use Params::Validate
    qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);

MAIN: {
    test_use_mcc();
    use_ok('NACL::MTask::MCC::Validator::StorageFailoverShow');
    test_storage_failover_show_vtor();
    done_testing();
}

sub test_storage_failover_show_vtor {

    # Create mock MCC
    my $mcc = make_mock_mcc();

    # Mock the Storage Failover CS fetch method
    my ($sf_mod, $mock_sf) = mock_component_state_fetch(
        cs_class  => 'NACL::CS::StorageFailover',
        cs_states => {'enabled' => 'true', 'node' => 'mock node'}
    );

    # Set the metrocluster to normal
    set_mcc_state_normal($mcc);

    # Test the validator in expected state
    lives_ok {
        my $vtor = NACL::MTask::MCC::Validator::StorageFailoverShow->new(
            metrocluster => $mcc);

        $vtor->run(die_on_failure => 1);
    }
    'nominal state';

    # Test the validator in an unexpected state
    $mock_sf->set_always('enabled', 'false');

    throws_ok {
        my $vtor = NACL::MTask::MCC::Validator::StorageFailoverShow->new(
            metrocluster => $mcc);

        $vtor->run(die_on_failure => 1);
    } 'NACL::MTask::MCC::Exceptions::Validation', 'error state';


} ## end sub test_storage_failover_show_vtor

