#!/usr/software/bin/perl

## Script to check the dependecy of a particular library
## run perl dynamic.pl -help to get the available options
use strict;
use Getopt::Long;
use warnings;
use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request::Common;
use JSON;

my $Change_Number  = undef;
my $Lib            = undef;
my $help           = undef;
my $build_path     = undef;
my $submit_to_ctl  = undef;

&help(1) if (! GetOptions('c|C=n'  => \$Change_Number,
                          'p|P=s'  => \$Lib,
                          'b|B=s'  => \$build_path,
                          'ns|NS'  => \$submit_to_ctl,
                          'n|n'    => \$submit_to_ctl,
                          'h|help' => \$help )
                       );
&help(1) if( $help || ( !$Change_Number || !$Lib  || ! -e $Lib."/lib/NACL/" ) );

$build_path = "/x/eng/rlse/DOT/devN" if ( !$build_path );

$Lib =~ m/\/u\/(\w+)\//;
my $user = $1;

## array of stest files to parse for testcase numbers
my @cli_stests = qw( ./C/c_cmode_cli1.stest ./C/c_cmode_cli2.stest ./C/c_cmode_cli3.stest ./C/c_cmode_cli4.stest ./C/c_cmode_cli5.stest
./MTask/mtask_cmode.stest ./STask/stask_cmode_cli1.stest ./STask/stask_cmode_cli2.stest ./STask/stask_cmode_repository.stest ./C/c_linux.stest );

my @zapi_stests = qw (./C/c_cmode_zapi_node1.stest ./C/c_cmode_zapi_node2.stest ./C/c_cmode_zapi_node3.stest ./STask/stask_cmode_zapi1.stest );

my @zapi_vserver = qw ( ./C/c_cmode_zapi_vserver1.stest );

my @filer_stests = qw ( ./C/c_cmode_filer.stest ./MTask/mtask_cmode_hardware.stest ./MTask/mtask_cmode_one_node_hardware.stest ./STask/stask_cmode_hardware.stest );

my ( %stest_contents, @thpl_array, %hash_thpl, %hash_library, %hash_library_2 );
my $testsuite = "NACL_CMode_Dependency_SN0";
my $stest_file = "dynamic.stest";
my $file_path = "lib/NACL/";

print "Please wait extracting the dependent scripts using nacre and generating stest file\n";

my @libraries = `p4 describe $Change_Number | egrep -r "\.\.\. .*NACL\/.*.pm#"`;
die("There are no NACL libraries in your change number\n")if ( ! @libraries );

sub match_library {
    my $array = $_[0];
    foreach ( @{$array} ) {
        $_ = $& if ( m/(lib\/NACL\/.*.pm)/ );
        $hash_library{$_} = $_ if ( ! (exists ($hash_library{$_})) );
    }
}

match_library(\@libraries);
foreach ( keys %hash_library ) {
    _get_dependency($_);
}

## Recursive Method to check the dependency of a particular script
sub _get_dependency {
    my ( $lib) = @_;
    # Get JSON from API
    my $ua = LWP::UserAgent->new;

    my $final_regex;
    $lib =~ s/^.*lib\///g;
    $lib =~ s/\.pm$//;
    $lib =~ s/\//::/g;
    $final_regex = $lib."[ ;]";
    return if ( $final_regex !~ /NACL::(C|STask|MTask|CS)::\w+.*\]$/ );
    print " . ";
    my $response = $ua->request(
        POST 'http://nacre.rtp.netapp.com/nacre/webstore/File::RegexSearch',
        [   login       => getlogin(),
            start       => 0,
            limit       => 500,
            query       => $final_regex,
            file_path   => $file_path,
        ]
    );
    my $hash = from_json($response->content);
    my $items = $hash->{items};
    my @paths = map { $_->{path} } @$items;
    foreach ( @paths ) {
        next if(grep(/nacl_setup|resources_snapshot|find_or_create/,$_));
        if ( grep(/thpl/,$_ ) ) {
            $hash_thpl{$_}=1  if ( !(exists $hash_thpl{$_}) );
        } elsif ( grep(/\.pm$/,$_ ) ) {
            if( grep(/lib\/NACL\/(C|STask|MTask|CS)\//,$_) ){
	        if ( ! (exists ($hash_library{$_})) ) {
                    $hash_library{$_}=1;
		    $file_path = "lib/NACL/UnitTest/";
		    _get_dependency($_);
                }
            }
        }
    }
}

# subroutine to get the details of Testcase numbers from stest file
# If any script is inside the RR Block it will be ignored
sub sed_command {
    my ( $lindex, $stest) = @_ ;
    my $rindex = $lindex -1;

    my $cmd = "sed -n '$rindex,$lindex p' /u/sti/p4/golden_client/dot-dev/hot/lib/NACL/UnitTest/$stest | \
        tee | sed -n ' \
        /^[      | ]*TESTCASES/ ! {\
            n\
            p\
        }\
        /^[      | ]*TESTCASES/ {\
            H\
            p\
            n\
            p\
    }';";

    my @array = `sed -n '1,$lindex p' /u/sti/p4/golden_client/dot-dev/hot/lib/NACL/UnitTest/$stest`;
    ## If its inside the RR block then ignore
    my $count =0;
    foreach ( @array ) {
        if ( $_ =~ /run_block_value=("RR_0&RR_1"|"RR_1"|"RR_0")/ ) {
            $count++;
        } elsif ( $_ =~ /\}/ && $count ) {
            $count--;
        }
    }
    ( $count ) ? return : return `$cmd`;
}

#subroutine to fill the contents of the generated stest file
sub populate_stest {
    my ( $cli_or_zapi, $stests ) = @_;
    $stest_contents{$cli_or_zapi}{'header1'}="####  $cli_or_zapi Block ####";
    if ( $cli_or_zapi =~ /VSERVER/ ) {
         $stest_contents{$cli_or_zapi}{'header2'} = "COMMAND_INTERFACE=\"vserver\"";
         $stest_contents{$cli_or_zapi}{'header3'} = "VSERVER_APISET_SHOULD_INTERFACE=ZAPI";
    } else {
         $stest_contents{$cli_or_zapi}{'header2'} = "FILER_APISET_SHOULD_INTERFACE=$cli_or_zapi";
    }
    my @temp = keys %hash_thpl;
    for ( my $i = 0 ; $i < @temp ; $i++ ) {
        for ( my $j = 0; $j < @$stests ; $j++ ) {
            my @lines = `cat -n /u/sti/p4/golden_client/dot-dev/hot/lib/NACL/UnitTest/$$stests[$j] | grep -r $temp[$i]`;
            foreach ( @lines ) {
                my $number = $1 if ( m/\s*(\d*)\s*\S*/ ) ;
                my @stest_lines = sed_command($number,$$stests[$j]);
		map {s/^\s*|\s*$//g;$_}@stest_lines;
		if ( scalar(@stest_lines) == 1 ) {
		    push (@{$stest_contents{$cli_or_zapi}{'TESTCASES'}{'TESTCASES=all'}}, $stest_lines[0]);
		} elsif ( scalar(@stest_lines) == 2 && $stest_lines[0] =~ /TESTCASES=all/i ) {
		    push (@{$stest_contents{$cli_or_zapi}{'TESTCASES'}{'TESTCASES=all'}}, $stest_lines[1]);
		} elsif ( scalar(@stest_lines) == 2 ) {
		    $stest_contents{$cli_or_zapi}{'TESTCASES'}{$stest_lines[0]} = $stest_lines[1];
		}
            }
        }
    }
}

my @interface = qw(CLI ZAPI VSERVER);
populate_stest('CLI', \@cli_stests );
populate_stest('ZAPI', \@zapi_stests);
populate_stest('VSERVER', \@zapi_vserver);
if ( !(exists $stest_contents{'CLI'}{'TESTCASES'} || $stest_contents{'ZAPI'}{'TESTCASES'}
	|| $stest_contents{'VSERVER'}{'TESTCASES'} )) {
    print "Check for filer related thpl";
    $testsuite = "NACL_CMode_Dependency_FILER";
    $stest_file = "dynamic_filer.stest";
    populate_stest('CLI', \@filer_stests );
    die("The NACL unit test script is not present in any of the stest files\n")
	if ( !(exists $stest_contents{'CLI'}{'TESTCASES'} ));
}

`chmod 644 $Lib/lib/NACL/UnitTest/$stest_file`;
`cp $Lib/lib/NACL/UnitTest/template.stest $Lib/lib/NACL/UnitTest/$stest_file`;
open(OUTLIST,">>","$Lib/lib/NACL/UnitTest/$stest_file") || die "Couldn't open stest file , $!\n";

foreach my $interface ( @interface ) {
print OUTLIST "\n";
next if ( ! (exists $stest_contents{$interface}{'TESTCASES'}) );
    print OUTLIST $stest_contents{$interface}{'header1'}."\n" if ( exists $stest_contents{$interface}{'header1'} );
    print OUTLIST $stest_contents{$interface}{'header2'}."\n" if ( exists $stest_contents{$interface}{'header2'} );
    print OUTLIST $stest_contents{$interface}{'header3'}."\n" if ( exists $stest_contents{$interface}{'header3'} );
    my ( $key,$value );
    while ( ($key,$value ) = each %{$stest_contents{$interface}{'TESTCASES'}} ) {
	if ( ref ($value) eq "ARRAY" ) {
            print OUTLIST $key."\n";
	    foreach (@{$value}) {
		print OUTLIST $_."\n";
	    }
	} else {
            print OUTLIST $key."\n";
	    print OUTLIST $value."\n";
	}
    }
}

print OUTLIST "}";

print "\nGenerated stest file $Lib/lib/NACL/UnitTest/$stest_file\n";

if( !defined $submit_to_ctl){
    print "\nTrigering ctl. You will receive the CTL results mail soon\n";
    my $cmd = "create execution
        testsuite    = \"$testsuite\"
        submitted_by = \"$user\"
        build_root   = \"$build_path\"
        variant      = \"x86_64.debug\"
            temp       = \"hot\"
        parameters set (NATE_LIB=\"$Lib;$Lib/ontap/main\",MAX_RETRIES=\"2\",MAIL_TO=\"$user\",NATE_VERSION=\"/usr/software/test/share/nate/5/current/bin/ntest\")
    ;";
    my $execution = `/x/eng/ctl/bin/ctl -c '$cmd'`;
    print "\nID = $execution";
}

sub help() {
    print <<HELP;
Description:
    This script generates the stest file which contains the dependent scripts for a particular library
    present in the change number

Usage :
    dynamic.pl -c <changelist#> -a <lib path> [-b <buiuld root>] [-ns]
    If the user didnt pass the buildroot it will be defaulted to DOT:dev

Options :
    -c         valid changelist number. it is mandatory.
    -p         P4_CLIENT_ROOT path.
    -b         Buildroot
    -n | ns    run with no submit to CTL
    -h | -help to display the help message

Examples:
     dynamic.pl -c 2098569 -p /u/kjp/p4/dotdev/test -b /x/eng/rlse/DOT/RsierranevadaN
     dynamic.pl -c 1048706 -p /u/kjp/p4/dotdev/test -n

HELP
    exit;
}
