#!/bin/ksh

# _check_rc : Checks the return code which is passed and exits if the return
# code is non-zero else returns zero(0) 
function _check_rc {
    rc=$1
    command_name=$2
    if [[ $rc -ne 0 ]]
    then
        printf "'$command_name' failed!! Please verify the client setup\n";
        exit 1;
    fi
    return 0
}

# MAIN
while getopts d:s:p: option
do case "$option" in
    p   )   WORKSPACE_PATH="$OPTARG" ;; 
    s   )   SRC_RELEASE="$OPTARG" ;;
    d   )   DST_RELEASE="$OPTARG" ;;
    [?] )   print >&2 "Usage: $0 -s <source release> -d <destination release> \
                        [-p <workspace path>]"
            exit 1 ;;
    esac
done
shift $OPTIND-1

# check for the mandatory source and destination release 
# names 
if [[ ! -n $SRC_RELEASE || ! -n $DST_RELEASE ]] 
then
    print >&2 "Please specify the valid release name(s)";
    print >&2 "Usage: $0 -s <source release> -d <destination release> \
                [-p <workspace path>]";
    exit 1;
fi

if [[ ! -d $WORKSPACE_PATH ]]
then
    print >&2 "$WORKSPACE_PATH is not valid directory";
    print >&2 "Please pass the valid workspace directory path";
    exit 1;
fi  

if [[ -n $WORKSPACE_PATH ]]
then
    cd $WORKSPACE_PATH
else
    WORKSPACE_PATH=`pwd`;
    printf "Using the '$WORKSPACE_PATH' as the client workspace";
fi

p4 client -o >/dev/null;
if [[ $? -ne 0 ]]
then
    printf "'$WORKSPACE_PATH' is not a valid client workspace.\n"; 
    printf "Please pass the valid client workspace path\n";
    exit 1;
fi

printf "Syncing workspace ...";
p4 sync >/dev/null;
_check_rc $? "p4 sync";

printf "\nDoing integrate...\n";
p4 integ -i -d //depot/prod/test/$SRC_RELEASE/lib/NACL/... \
    //depot/prod/test/$DST_RELEASE/lib/NACL/... >/dev/null;
_check_rc $? "p4 integ";

printf "\nDone with integrate, doing resolve...\n";
p4 resolve -as >/dev/null;
_check_rc $? "p4 resolve -as";

p4 resolve -am >/dev/null;
_check_rc $? "p4 resolve -am";

printf "\nLooking for the conflicts:\n"
p4 resolve -n > ~/conflicts.txt ;
if [[ -s ~/conflicts.txt ]]
then
    printf "Following are the files which are having conflicts.\n";
    printf "Please resolve these conflicts manually and continue..\n";
    cat ~/conflicts.txt;
    printf "The files are also listed in ~/conflicts.txt file for your refernce\n"; 
    exit 1;
fi

printf "No file conflicts. Updating the burt fixes_db by running p4 \
    anno_burts...\n";

p4 anno_burts -src test:$SRC_RELEASE -tgt test:$DST_RELEASE >/dev/null;
_check_rc $? "p4 anno_burts";
printf "Done with 'p4 anno_burts'. Please continue with testing\n";

exit 0
