# $Id: //depot/prod/test/nacldev/lib/NACL/C/QosSettingsExternalCache.pm#6 $

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary QosSettingsExternalCache Component Module (auto-generated by CGT)
## @author dl-nacl-dev@netapp.com, skotagir@netapp.com
## @status shared
## @pod here

=head1 NAME

NACL::C::QosSettingsExternalCache

=head1 DESCRIPTION

C<NACL::C::QosSettingsExternalCache> is a derived class of
L<NACL::C::Component::ONTAP|lib-NACL-C-Component-ONTAP-pm>. It represents access
to an ONTAP QosSettingsExternalCache.  A related class is
L<NACL::CS::QosSettingsExternalCache|lib-NACL-CS-QosSettingsExternalCache-pm>, which represents
the past state of an ONTAP QosSettingsExternalCache.

=head1 ATTRIBUTES

=head2 command_interface

See L<NACL::C::Component::command_interface|lib-NACL-C-Component-pm/command_interface>

=head2 external_cache_setting

=cut

package NACL::C::QosSettingsExternalCache;

use strict;
use warnings;

use NACL::CS::QosSettingsExternalCache;
use NACL::ComponentUtils qw (_optional_scalars Dumper);

use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate validate_with SCALAR ARRAYREF);
use NATE::Log qw(log_global);
my $Log = log_global();
my $may_enter = $Log->may_enter();
my $may_exit  = $Log->may_exit();

=head1 METHODS

=head2 new

  my $QosSettingsExternalCache = NACL::C::QosSettingsExternalCache->new(
                                   command_interface => $ci,
                                   'external-cache-setting' => $external_cache_setting,
                                  );

Return a new QosSettingsExternalCache component object with the given attributes.

=cut

use Class::MethodMaker [ scalar => 'external_cache_setting', ];

=head2 find

    my $QosSettingsExternalCache_obj  = NACL::C::QosSettingsExternalCache->find(command_interface => $ci, ...);
    my @QosSettingsExternalCache_objs = NACL::C::QosSettingsExternalCache->find(command_interface => $ci, ...);

(Class method)
A constructor of sorts, which discovers which elements are present (by
interacting with the given command interface).  In an array context it
creates a component object for each present element and returns them.
In a scalar context it creates a component object for just one
element.

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::find|lib-NACL-C-Component-pm/find> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
Invokes 'external-cache-policy-get-iter' api for CMode ZAPI.

=head2 state

    my QosSettingsExternalCache_state = $QosSettingsExternalCache_obj->state();

(Instance method)
Fetch the current state of the element associated
with this component, and return a Component State object corresponding
to this component (calling this on NACL::C::QosSettingsExternalCache would return
a NACL::CS::QosSettingsExternalCache object)

This method is generally not implemented in individual components. The base
class (L<NACL::C::Component|lib-NACL-C-Component-pm> provides an implementation.
See <L<NACL::C::Component::state|lib-NACL-C-Component-pm/state> for a more
detailed description along with explanation of all the options accepted.

Uses a CMode CLI/ZAPI APISet.
Invokes 'external-cache-policy-get-iter' api for CMode ZAPI.

=cut

=head2 modify

    NACL::C::QosSettingsExternalCache->modify(
                                command_interface => $ci,
                                'external-cache-setting' => $external_cache_setting,
                                ...);

                                 or

    $QosSettingsExternalCache_obj->modify(...);

(Class or instance method)
This method is used to modify the external cache policy.
Uses a CMode CLI/ZAPI APISet.
  Invokes 'external-cache-policy-modify-iter' api for CMode ZAPI.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required for class method, Not Applicable for instance method)
See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< apiset_must => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_must>

=item C<< apiset_should => $ruleset >>

(Optional) See L<NACL::C::Component|lib-NACL-C-Component-pm/apiset_should>

=item C<< 'external-cache-setting' => $external_cache_setting >>

(Required for class method, Not Applicable for instance method)
*External Cache Policy Name
Applicable for CMode CLI/ZAPI

Maps to:
  CMode ZAPI : 'external-cache-policy-name'

=item C<< 'default' => $string >>

Applicable for CMode CLI/ZAPI

Maps to:
  CMode ZAPI : 'is-default'

=item C<< 'ext-cache-setting-id' => $string >>

Applicable for CMode CLI

=item C<< 'method-timeout' => $time_in_seconds >>

(Optional)
The default NATE timeout is 60 seconds. If the command might take more
time to complete, this option can be used to specify a larger timeout value.
The value should be provided in seconds.

=back

=back

=cut

sub modify {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    $pkg_or_obj->_frontend_log_debug_opts(@args);
    my %opts = $pkg_or_obj->_common_validate_with(
        params          => \@args,
        additional_spec => {
            _optional_scalars(
                qw( default ext-cache-setting-id
                    )
            ),
        },
    );
    $pkg_or_obj->call_on_apiset(
        %opts,
        choices => [
            {   method    => '_modify_cmode_cli',
                interface => 'CLI',
                set       => 'CMode',
            },
            {   method    => '_modify_cmode_zapi',
                interface => 'ZAPI',
                set       => 'CMode',
            },
        ],
    );

    $Log->exit() if $may_exit;
} ## end sub modify

sub _modify_cmode_cli {
    $Log->enter() if $may_enter;
    my $pkg_or_obj = shift;
    my %opts       = $pkg_or_obj->_validate_backend_opts(@_);

    delete $opts{command_interface};
    my $apiset = delete $opts{apiset};

    my $allowed_cmode_options = [
        qw( default ext-cache-setting-id external-cache-setting
            extended_query )
    ];
    my %cmode_args;
    $pkg_or_obj->_hash_copy(
        source => \%opts,
        copy   => $allowed_cmode_options,
        target => \%cmode_args,
    );

    $apiset->qos_settings_external_cache_modify(%cmode_args);
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_cli

sub _modify_cmode_zapi {
    $Log->enter() if $may_enter;
    my ( $pkg_or_obj, @args ) = @_;
    my %opts = $pkg_or_obj->_validate_backend_opts(@args);

    $pkg_or_obj->_iter_helper(
        @args,
        api => 'external-cache-policy-modify-iter',
        map => {
            'external-cache-setting' => 'external-cache-policy-name',
            'default'                => 'is-default',
        },
        does_not_exist_error => "entry doesn't exist",
        does_not_exist_exception =>
            'NACL::C::Exceptions::QosSettingsExternalCache::DoesNotExist'
    );
    $Log->exit() if $may_exit;
} ## end sub _modify_cmode_zapi

sub _primary_keys_validate_spec {
    return ( 'external-cache-setting' => { type => SCALAR }, );
}

1;
