# -*-perl-*-
#
# Copyright (c) 2015,.. NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary DPG Workflows
## @author ng-smgr-auto@netapp.com
## @status shared
## @pod here

package NACL::AppBuilder::DpgWorkflows::workflow;

use strict;
use warnings;

use NATE::Process;
use NATE::ProcManager;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument;
use List::MoreUtils qw(any);
use NACL::Expectation;
use NACL::MTask::Data;
use NACL::AppBuilder::WFC::IO;
use FG::DP::SnapshotHelper;
use NACL::STask::Aggregate;
use NACL::STask::Snapmirror;
use NACL::CS::Snapshot;
use NACL::CS::VolumeSnapshot;
use NACL::C::Node;
use NACL::ComponentUtils qw(Dumper);
use NACL::C::Client;
use NACL::STask::Volume;
use NACL::STask::SnapmirrorPolicy;
use NACL::CS::SnapmirrorPolicy;
use NACL::C::SnapmirrorPolicy;
use NACL::C::Volume;
use NACL::CS::Volume;
use NACL::C::Snapmirror;
use NACL::CS::VolumeSnapshot;
use NACL::MTask::LunSetup;
use NACL::APISet::Exceptions::TimeoutException;
use NACL::GeneralUtils qw(
    random_name_generator
    nacl_method_retry
);
use NACL::C::Lun;
use NACL::APISet;
use TestEnv::MinWin qw(minwin_compatible);
use NACL::ServiceAPI::Events;
use NACL::MTask::Traffic;
use NACL::STask::Lun;
use NACL::C::Node;
use NACL::MTask::Wafliron;
use NACL::STask::VolumeFile;
use NACL::MTask::SystemLogDetector;
use dp::snapmirror::paloma::vault::lib::VaultCommonUtils
    qw( dataset SFR_restore SFR_verify
    create_snapshot delete_snapshot );
use NACL::Exceptions::UnexpectedOutputException;
use NACL::Exceptions::InvalidParam;

use NACL::AppBuilder::DpgWorkflows::Fiji qw(
    fiji_clear_all
);

use NACL::AppBuilder::DpgWorkflows::Validation qw(Validate_PCS_Tags
    Validate_add_cg_item
    Validate_del_cg_item
    Validate_Initialize
    Validate_Update
    Validate_flip_resync_cg_items
    Validate_resync
    Validate_snapmirror_logs
    Validate_appcg_forward_transfer
    Validate_appcg_restore
    Validate_appcg_sfr
    verify_transferring
);
use NACL::AppBuilder::DpgWorkflows::Workload
    qw(Wf_create_multiplesize_filesnclones
    Wf_create_fileclones_filestodirectories
    start_read_on_dst_cg_files
    populate_hecr_dataset
    find_files_dir_links
    fg_set_placement
    appcg_dataset_secondary
    appcg_dataset_adaptive
    appcg_populate_overwrite
);
use NACL::AppBuilder::DpgWorkflows::StringConstants;

use base qw(Exporter);
use Set::Scalar;
use Tharn;

use Params::Validate qw(validate_with validation_options :types);
use NATE::Log qw(log_global);
my $log = log_global();

our @EXPORT_OK = qw(
    snapmirror_create
    snapmirror_initialize
    snapmirror_initialize_many_rels
    snapmirror_update
    snapmirror_abort
    snapmirror_abort_hard
    snapmirror_quiesce
    snapmirror_resume
    snapmirror_restore
    snapmirror_break
    snapmirror_resync
    snapmirror_delete
    snapmirror_release
    snapmirror_check
    snapmirror_modify
    sm_checker_fg
    Wf_sm_checker
    run_sm_checker
    Wf_basic_snapmirror_setup
    Wf_sync_app_cons_creation
    Wf_in_sync
    Wf_out_of_sync
    Wf_sfr
    Wf_lretolrse
    Wf_snapmirror_cleanup
    _verify_list_destination_status
    Wf_flip_resync
    sm_flip_resync
    lun_setup
    validate_ACS
    Wf_CG_Failover
    Wf_CG_Failback
    get_audit_log
    create_pcs
    enable_disable_scheduled_pcs
    enable_disable_autoresync
    get_latest_smgr_snapshot
    wait_for_newer_smgr_snapshot
    create_max_snapshots
    common_snapshots
    check_for_ift_log
    wf_add_cg_item
    wf_del_cg_item
    Wf_add_cg_item
    Wf_delete_cg_item
    ds_create_tree
    sm_resume
    in_sync
    Wf_sfr_restore
    Wf_vault_lifecycle
    Wf_DR_lifecycle
    Wf_Cascade_Vault_DR_lifecycle
    Wf_Cascade_DR_Vault_lifecycle
    clear_fijis_toggle_auto_resync_scheduled_pcs
    fg_get_constituents
    fg_wait_on_attributes
    fg_wait_on_attributes_all_constituents
    fg_snapshot_checkpoint
    fg_checkpoint_verify
    is_rel_flexgroup
    Wf_vault_lifecycle
    Wf_DR_lifecycle
    Wf_hecr_lifecycle
    set_wafl_flags
    match_vol_attributes
    get_fh
    runcmd_get_vol_config_attrs
    get_volume_show
    enable_dedup_compression
    is_shiner
    get_destination_volume
    check_io_fence_client
    check_meta_files
    enable_nfs_v4
    is_vol_on_CA
    throttle_transfer_on_node
    compare_afs
    compare_ri2p_metafile
    SM_initialize_update_quiesce
    fg_expand
    fg_create_qtrees
    fg_populate_qtrees
    volume_is_snapmirror_source
    Wf_protect_api
    _run_wafliron
    start_monitoring_snapmirror_logs
    Wf_Auto_L2L_Cascade_Resync
    Wf_Auto_L2L_Fanout
    Wf_Appcg_forward_transfer
    Wf_Appcg_restore
    Wf_Appcg_sfr
    Wf_restore
    update_with_retry
    fg_snapmirror_restore_baseline
    fg_snapmirror_restore_incremental
);

our %EXPORT_TAGS = (
    'all' => \@EXPORT_OK,
);

use vars qw (
        $Test
        $Node
        $Volume
        $Aggregate
        $Vserver
        $is_repository
        $volume_created
        $aggregate_created
        $counter
        );

# Set the on_fail behavior for Params::Validate to throw an exception (of the
# NATE::Exceptions::Argument class) on errors instead of a plain croak().
validation_options(
    on_fail => sub {
        NATE::Exceptions::Argument->throw(shift);
    }
);

# Constant strings for TE verifications
my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;



=head1 NAME

NACL::AppBuilder::DpgWorkflows::workflow

=head1 DESCRIPTION

This library contains the DPG workflows.

=head1 METHODS

=cut

# private method
sub sm_create_cg{

    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            src_vol       => { type => SCALAR,          optional => 1 },
            dst_vol       => { type => SCALAR,          optional => 1 },
            src_vserver   => { type => SCALAR,          optional => 1 },
            dst_vserver   => { type => SCALAR,          optional => 1 },
            src_cg_name   => { type => SCALAR,          optional => 1 },
            dst_cg_name   => { type => SCALAR,          optional => 1 },
            cg_item_list  => { type => ARRAYREF,        optional => 1 },
            node_dst      => { type => OBJECT | SCALAR, optional => 1 },
            node_src      => { type => OBJECT | SCALAR, optional => 1 },
            stagger_delay => { type => SCALAR,          optional => 1 },
            tries => { type => SCALAR, optional => 1, default => 8 },
            add_label => { type => SCALAR, optional => 1, default => 'SyncProtectMe' },
        },
    );

    my $src_vol_name     = $args{'src_vol'};
    my $dst_vol_name     = $args{'dst_vol'};
    my $Src_Vserver_Name = $args{'src_vserver'};
    my $Dst_Vserver_Name = $args{'dst_vserver'};
    my $Src_Cg_Name      = $args{'src_cg_name'};
    my $Dst_Cg_Name      = $args{'dst_cg_name'};
    my $Dst_Node_Obj     = $args{'node_dst'};
    my $Src_Node_Obj     = $args{'node_src'};
    my $cg_item_list     = $args{'cg_item_list'};
    my $tries            = $args{'tries'};
    my $add_label        = $args{'add_label'};
    _stagger($args{'stagger_delay'});

     my @cg_item_lists       = _list_of_vals($cg_item_list);
     $log->comment("And I got this as an array of cg-items:\n" .Dumper(@cg_item_lists));
    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }
    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    #right now create policy since there is no default policy
    my $sm_policy;
    my $policy_name = 'sync_policy_'.$$;
    nacl_method_retry(
        code => sub {
            $log->log('Creating policy '.$policy_name);
            $sm_policy = NACL::STask::SnapmirrorPolicy->create(
                command_interface       => $Dst_Node_Obj,
                vserver                 => $Dst_Vserver_Name,
                policy                  => $policy_name,
                type                    => 'sync-mirror',
                nacltask_if_exists      => 'purge',
                'user-snapshot-timeout' => 45, # lower to 20 if SAN IO throttling is in
                tries                   => $tries,
            );
        },
        exceptions  => 'NACL::APISet::Exceptions::CommandFailedException',
        tries_count => 6,
        sleep_time  => 30,
    );

    #add label/rule for ACS replication if provided
    if ($add_label eq 'none') {
        $log->log('Default label SyncProtectMe to be added to the policy '.$policy_name);
	$add_label = 'SyncProtectMe';
    }
    $log->log('Adding rule '.$add_label.' to policy '.$policy_name);
    NACL::STask::SnapmirrorPolicy->add_rule(command_interface  => $Dst_Node_Obj,
                                            policy             => $policy_name,
                                            vserver            => $Dst_Vserver_Name,
                                            'snapmirror-label' => $add_label,
                                            keep               => 1,
                                            preserve           => 'false',
                                            nacltask_if_exists => 'reuse'
                                            );

    $log->comment('Run snapmirror create to create cgs and setup sm relationship between them');

    my %cg_item_mappings;
    my @keys;
    my @values;

    foreach my $item(@cg_item_lists){
        $cg_item_mappings{"/vol/$src_vol_name/$item"} = "/vol/$dst_vol_name/$item";
    }

    NACL::C::Snapmirror->create(
         command_interface     => $Dst_Node_Obj,
         'source-path'         => $Src_Vserver_Name.":".$Src_Cg_Name,
         'destination-path'    => $Dst_Vserver_Name.":".$Dst_Cg_Name,
         'cg-item-mappings'    => \%cg_item_mappings,
         type                  => 'XDP',
         policy                => $sm_policy->policy
    );

}

# private method
sub sm_create_adr{

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol           => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            node_dst          => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            stagger_delay     => {type => SCALAR, optional => 1},
            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    _stagger($args{'stagger_delay'});

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    NACL::STask::Snapmirror->create(
            command_interface  => $Dst_Node_Obj,
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            type               => 'DP',
            do_initialize      => 0,
            nacltask_wait      => 1,
            );
}

# private method
sub sm_create_vault{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol           => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            node_dst          => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            stagger_delay     => {type => SCALAR, optional => 1},
            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    _stagger($args{'stagger_delay'});

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }
    NACL::STask::Snapmirror->create(
            command_interface  => $Dst_Node_Obj,
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            type               => 'XDP',
            do_initialize      => 0,
            nacltask_wait      => 1,
            );
}

# private method
sub sm_initialize_adr{

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol           => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            node_dst          => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            validate          => {type => SCALAR, optional=> 1},
            timeout           => {type => SCALAR, optional=> 1, default => 1200},
            stagger_delay     => {type => SCALAR, optional => 1},
            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $validate            = $args{'validate'};
    my $timeout             = $args{'timeout'};
    _stagger($args{'stagger_delay'});



    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    $log->comment('Initialize that relationship');
    my $o = NACL::STask::Snapmirror->initialize(
            command_interface  => $Dst_Node_Obj,
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            type               => 'DP',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
            );

    if($validate eq 'true'){
        Validate_Initialize(   node_src  => $Src_Node_Obj,
                               node_dst  => $Dst_Node_Obj,
                               sm_obj    => $o);
    }

}

# private method
sub sm_initialize_vault{

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol           => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional =>1 },
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            node_dst          => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            snapshot          => {type => SCALAR, optional =>1},
            validate          => {type => SCALAR, optional=> 1},
            timeout           => {type => SCALAR, optional=> 1, default => 1200},
            stagger_delay     => {type => SCALAR, optional => 1},
            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $validate            = $args{'validate'};
    my $timeout             = $args{'timeout'};
    my $snapshot            = $args{'snapshot'};
    _stagger($args{'stagger_delay'});
    my $o;


    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    $log->comment('Initialize that relationship');

    my %init_args;

    if (exists($args{'snapshot'})){
        $init_args{'source-snapshot'} = $snapshot;
    }

    $o  = NACL::STask::Snapmirror->initialize(command_interface  => $Dst_Node_Obj,
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            type               => 'XDP',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
            %init_args
            );


    if($validate eq 'true'){
        my %more_args;
        if (exists($args{'snapshot'})){
            $more_args{snapshot} = $snapshot;
        }
        Validate_Initialize(   node_src  => $Src_Node_Obj,
                               node_dst  => $Dst_Node_Obj,
                               sm_obj    => $o,
                               %more_args
                            );
    }

}


# private method
sub sm_initialize_cg {

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol           => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            src_cg_name       => {type => OBJECT| SCALAR, optional=>1},
            dst_cg_name       => {type => OBJECT|SCALAR, optional =>1},
            cg_item_list      => {type => ARRAYREF, optional=>1},
            node_dst          => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            validate          => {type => OBJECT|SCALAR, optional =>1},
            timeout           => {type => SCALAR, optional =>1, default => 1200},
            stagger_delay     => {type => SCALAR, optional => 1},
            retry_list_dest_count  => {type => SCALAR, default=>30},
            },
            );


    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Src_Cg_Name         = $args{'src_cg_name'};
    my $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $validate            = $args{'validate'};
    my $timeout             = $args{'timeout'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $retry_list_dest_count    = $args{'retry_list_dest_count'};
    _stagger($args{'stagger_delay'});

    my @cg_item_lists       = _list_of_vals($cg_item_list);


    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }
    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    my %cg_item_mappings;
    my $o;
    if (exists($args{'cg_item_list'})) {
        foreach my $item(@cg_item_lists){
            $cg_item_mappings{"/vol/$src_vol_name/$item"} = "/vol/$dst_vol_name/$item";
        }

       # nacltask_wait => 0 otherwise it looks from snapmirror show output
        $log->comment('Initialize sync relationship');
        $o =  NACL::STask::Snapmirror->initialize(
            command_interface     => $Dst_Node_Obj,
            'source-path'         => $Src_Vserver_Name.":".$Src_Cg_Name,
            'destination-path'    => $Dst_Vserver_Name.":".$Dst_Cg_Name,
            'cg-item-mappings'    => \%cg_item_mappings,
            nacltask_wait         => 0,
            );
    } else{
        $log->comment('Initialize the sync relationship');
        $o =  NACL::STask::Snapmirror->initialize(
            command_interface  => $Dst_Node_Obj,
            'source-path'      => $Src_Vserver_Name.":".$Src_Cg_Name,
            'destination-path' => $Dst_Vserver_Name.":".$Dst_Cg_Name,
            nacltask_wait      => 0,
            );
    }

    $log->comment('Verify on the source if the relationship is in sync');
    _verify_list_destination_status($Src_Node_Obj,$Dst_Cg_Name,$Src_Cg_Name,$Src_Vserver_Name,$Dst_Vserver_Name,'InSync',$retry_list_dest_count);

    if($validate eq 'true'){
        Validate_Initialize (  node_src       => $Src_Node_Obj,
                               node_dst       => $Dst_Node_Obj,
                               sm_obj         => $o,
                               source_cg_name => $Src_Cg_Name,
                               dst_cg_name    => $Dst_Cg_Name
                               );
    }

    return;

}

# private method
sub sm_resync{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );

    $o->resync(nacltask_wait => 0 );
    if($validate eq 'true'){
        $o->monitor('method-timeout' => 3600);
        Validate_resync(node_src=>$Src_Node_Obj,node_dst=>$Dst_Node_Obj,sm_obj=>$o);
    }
}

# private method
sub sm_quiesce{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );

    # uncomment later
    #$log->warn('Uncomment quiesce later');
    $o->quiesce(nacltask_verify=>1);
    if($validate eq 'true'){
        quiesce_validations($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path);
    }
}

# private method
sub quiesce_validations{
    #Validation1: Relationship status should be quiesced afterwards.
    #Validation2(TODO): In case of Sync mirror - Pseudo snapshot creation should be triggered.
    #Validation3: Validate the OutOfSync(OOS) status.

    my ($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path)=@_;
    # This code is to fetch vserver and volume for cg relationship

    my $state = NACL::CS::Snapmirror->fetch( command_interface => $Dst_Node_Obj,
            filter => {'source-path'      => $src_path,
            'destination-path' => $dst_path},
            );


    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
            'relationship-id' => $state->relationship_id,
            },
            requested_fields => [qw(source-path destination-path)]);

    my($Src_Vserver_Name,$src_vol_name) = split(/:/,$list_dst->source_path);
    my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$list_dst->destination_path);

    if($state->status ne 'Quiesced'){
        $log->error('Relationship should have been quiesced after quiesce operation');
    }

    if($state->policy_type eq 'sync-mirror'){
        #TODO : Check for Pseudo Snapshot creation when code is there
        # Validate the OutOfSync status
        _verify_list_destination_status($Src_Node_Obj,$dst_vol_name,$src_vol_name,
                            $Src_Vserver_Name,$Dst_Vserver_Name,'OutOfSync');
    }

}


=head2 Wf_flip_resync

This method will do flip resync on multiple snapmirror relationships.

=over

=item Example

    Wf_flip_resync(sm_objs => $arrayref_sm_obj,
                    src_node => $src_node,
                    dst_node => $dst_node,
                    cg_items => $arrayref_luns,
                    cg_type => 'lun',
                    validate => 'true',
                    );

=item Options

=over

=item sm_objs

(Required): ArrayRef of snapmirror objects of type NACL::STask::Snapmirror.

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item cg_items

(Required): ArrayRef of cg item objects. In case of lun, arrayref of lun objects.

=item cg_type

(Required): Type of cg item. Possible values : 'lun' / 'file'. Default 'lun'.

=item validate

(Optional) If 'true', will validate if the dst luns are RW and src luns are Read only.
If validations fails an exception will be thrown.
Default: 'false'.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=back

=cut

sub Wf_flip_resync{

    my %args = validate_with(
            params => \@_,
            spec   => {
                        sm_objs  => {type => ARRAYREF,},
                        node_src => {type => OBJECT|SCALAR,},
                        node_dst => {type => OBJECT|SCALAR,},
                        validate => {type => SCALAR, default => 'false', regex => qr/^(true|false)$/,},
                        },
                    );

    my @sm_objs        = _list_of_vals($args{sm_objs});
    my $validate       = $args{validate};
    my $node_src_obj   = $args{node_src};
    my $node_dst_obj   = $args{node_dst};

    # convert to object if nodenames are passed
    if ( ($node_src_obj) && (ref($node_src_obj) !~ /Node/i) ) {
        $node_src_obj = NACL::STask::Node->new( name => $node_src_obj );
    }
    if ( ($node_dst_obj) && (ref($node_dst_obj) !~ /Node/i) ) {
        $node_dst_obj = NACL::STask::Node->new( name => $node_dst_obj );
    }

    my @subtests;
    foreach my $sm (@sm_objs) {
        my $src_path = $sm->{source_path};
        my $dst_path = $sm->{destination_path};
        my @cg_item_names;
        if ($src_path =~ /\/cg\//) {
            # update path's in case of SYNC mirrors to look like old style
            ($src_path, $dst_path) = $sm->volume_paths_from_cg;
            $log->log("SMGR so updated paths to source-path=$src_path destination-path=$dst_path");
            my $cg_data = $sm->get_consistency_group_data;
            for my $key (sort keys %{$cg_data}) {
                my ($item) = ($key =~ /^.*:\/vol\/(?:.*)\/(.*)$/);
                if ($item) {
                    push @cg_item_names, $item;
                } else {
                    $log->warn("Could not find item in $key");
                }
            }
            $log->log("CG item names are @cg_item_names");
        }
        $log->log("Got source-path=$src_path destination-path=$dst_path");
        my ($src_vserver_name,$src_vol_name) = ($src_path =~ /^(.*):(.*)$/);
        my ($dst_vserver_name,$dst_vol_name) = ($dst_path =~ /^(.*):(.*)$/);
        $log->log('Got source vserver='.$src_vserver_name.' src vol='.$src_vol_name.
                    ' dest vserver='.$dst_vserver_name.' dest vol='.$dst_vol_name);

        my %args_func = (node_src => $node_src_obj,
                        node_dst => $node_dst_obj,
                        snapmirror_object => $sm,
                        src_vs_name => $src_vserver_name,
                        src_vol_name => $src_vol_name,
                        dst_vol_name => $dst_vol_name,
                        validate => $validate,
                        policy_opts => 'existing',
                        stagger_delay => 10,
                        );
        if (scalar (@cg_item_names)) {
            $args_func{cg_item_list} = \@cg_item_names;
        }
        push @subtests, create_parallel_processes('sm_flip_resync',\%args_func);
    }
    run_processes_bg('sm_flip_resync',\@subtests);
}


=head2 sm_flip_resync

Will run the flip resync workflow. The steps include:

    - quiesce the CG
    - break the CG
    - del CG on the dst
    - release CG on the src with relationship-info-only => true
    - create a new relationship with the src and dst swapped
    - Issue a resync - unless do_a_resync => 0
    - Wait for resync job to complate - unless wait_for_resync => 0
    - run Validations ONLY if do_a_resync => 1 && wait_for_resync => 1

=over

=item example

    my $newsm = sm_flip_resync (
                        snapmirror_object  => $sm,
                        node_src           => $node_src,
                        node_dst           => $node_dst,
                        src_vs_name        => $src_vs_name,
                        src_vol_name       => $vol_src_name,
                        dst_vol_name       => $vol_dst_name,
                        cg_item_list       => \@sitemnames,
                        do_a_resync        => 0,
                        wait_for_resync    => 0,);

=item options

=over

=item snapmirror_object

(Required) The existing sm object which you want to flip.

=item node_src

(Required) The current src node object which will act as the dst post the flip.

=item node_dst

(Required) The current dst node object which will act as the src post the flip.

=item src_vs_name

(Required) The current src vs name. Post the flip this vs will act as the dst and will
be required to create a sm policy.

=item src_vol_name

(Required) The current src vol name. This vol will act as the dst vol post the flip and is
required to create the cg item mapping.

=item dst_vol_name

(Required) The current dst vol name. This vol will act as the src vol post the flip and is
 required to create the cg item mapping.

=item cg_item_list

(Required) The list of cg items (luns) that form the current cg relationship. The same luns
will be used to create the flip relationship also.

=item do_a_resync

(Optional) Option to control if the user wants to do a resync once the relationship has been
flipped.
(Default) 1.

=item wait_for_resync

(Optional) Option to control if the user wants to wait for the resync to complete.
(Default) 1.

=item validate

(Optional) If 'true', will validate if the dst luns are RW and src luns are Read only.
(Default) 'true'.
Note:
    a. Validation is possible only if do_a_resync and wait_for_resync is set.
    b. Validation is done only for SYNC mirrors (i.e. if cg_item_list is specified.)

=item policy_opts

(Optional) If set to 'existing', will create a new policy on the SRC(new DEST) using same values from old policy.
Otherwise will use new values.
(Default) 'new'.

=item stagger_delay

(Optional) If set, will stagger the start of the process. This is useful when processes are run in parallel.
(Default) 1.

=back

=item Returns

The newly created SM object where the src and dst are swapped.

=back

=cut


sub sm_flip_resync {
    my %args = validate_with (
                params   => \@_,
                spec     => {
                    snapmirror_object  => {type => HASHREF,},
                    node_src           => {type => OBJECT| SCALAR,}, #the current/exiting src node of the CG
                    node_dst           => {type => OBJECT| SCALAR,}, #the current/exiting dst node of the CG
                    src_vs_name        => {type => SCALAR}, #the current/existing src vserver
                    src_vol_name       => {type => SCALAR}, #the current src vol name
                    dst_vol_name       => {type => SCALAR}, #the current dst vol name
                    cg_item_list       => {type => ARRAYREF, optional => 1},
                    do_a_resync        => {type => SCALAR, optional => 1, default => 1},
                    wait_for_resync    => {type => SCALAR, optional => 1, default => 1},
                    validate           => {type => SCALAR, default => 'true', regex=>qr/^(true|false)$/,},
                    tries              => {type => SCALAR, optional => 1, default => 8 },
                    stagger_delay      => {type => SCALAR, optional => 1},
                    policy_opts        => {type => SCALAR, default => 'new', regex=>qr/^(new|existing)$/,},
                },
            );

    my $sm = $args{snapmirror_object};
    my $node_dst        = $args{node_dst};
    my $node_src        = $args{node_src};
    my $dst_vs          = $args{src_vs_name};
    my $src_vol         = $args{src_vol_name};
    my $dst_vol         = $args{dst_vol_name};
    my $wait_for_resync = $args{wait_for_resync};
    my $validate        = $args{validate};
    my $doresync        = $args{do_a_resync};
    my $cg_item_list    = $args{cg_item_list}; #flip resync will be created using the same items
    my $src_path        = $sm->{'source_path'};
    my $dst_path        = $sm->{'destination_path'};
    _stagger($args{stagger_delay});

    $log->enter();

    # would have been better if we took all the possible fields for the new policy
    # but this satisifies the current need as of now
    my %policy_args;
    if ($args{policy_opts} eq 'new') {
        $policy_args{policy} = 'sync_policy_'.$$;
        $policy_args{type} = 'sync-mirror';
        $policy_args{'user-snapshot-timeout'} = 45; # lower to 20 if SAN IO throttling is in
        $policy_args{tries} = $args{tries};
    } else {
        # Extract policy info - use same values for creating policy on other node
        my $policy_obj = NACL::CS::Snapmirror->fetch(command_interface => $node_dst,
                                                filter => { 'destination-path' => $dst_path, },
                                                requested_fields => [qw(policy)],
                                                );
        my $policy = $policy_obj->policy;
        $policy_args{policy} = $policy;
        $log->comment("Existing policy is $policy");
        my $existing_policy_obj = NACL::CS::SnapmirrorPolicy->fetch(command_interface => $node_dst,
                                                            filter => { policy => $policy, },
                                                            requested_fields => [qw(type tries)],
                                                        );
        $policy_args{type} = $existing_policy_obj->type;
        $policy_args{tries} = $existing_policy_obj->tries;
    }

    #1. do a quiesce
    $log->comment('quiescing the relationship');
    $sm->quiesce(nacltask_verify => 1);

    #2. do a break
    $log->comment('breaking the relationship');
    $sm->break(nacltask_verify => 0,
               nacltask_wait   => 0);

    #wait for it reach broken off
    my $nacltask_wait_for_status;
    if ($policy_args{type} eq 'sync-mirror') {
         $nacltask_wait_for_status = 'OutOfSync';
    } else {
         $nacltask_wait_for_status = 'Idle';
    }
    $log->comment('Wait for mirror state broken off and status='.$nacltask_wait_for_status);
    $sm->monitor (
         nacltask_wait_for_status => $nacltask_wait_for_status,
         nacltask_wait_for_state  => 'Broken-off');

    #run validations
    my $voltocheck = NACL::CS::Volume->fetch (
                    command_interface => $node_dst,
                    filter            => {
                        volume        => $dst_vol,
                    },
                    requested_fields  => [qw(type)]);

    if($voltocheck->type ne 'RW') {
        NATE::BaseException->throw('The dst volume should have been RW post successful break');
    }

    #3. do a sm delete on the dst
    $log->comment('Delete the relationship on the dst');
    $sm->delete();

    #4. do a sm release on the src with -relationship-info-only to true so that it retains the snapshot
    $log->comment('Release the relationship on the src with relationship-info-only set to true');
    NACL::STask::Snapmirror->release(
        command_interface        => $node_src,
        'source-path'            => $src_path,
        'destination-path'       => $dst_path,
        nacltask_verify          => 1,
        'relationship-info-only' => 'true');

    #5. create the relationship that was previously A -> B from B->A
    $log->comment('Now swap the src and dst and recreate the relationship');
    #swapping the src and dst nodes
    my $tmp          = $node_src;
    $node_src        = $node_dst; #swapping the src node to be the dst node now.
    $node_dst        = $tmp; #swapping the dst node to be the src node now.

    #swapping the src and dst vols
    $tmp          = $src_vol;
    $src_vol         = $dst_vol; #swapping the src vol to be the dst vol now.
    $dst_vol         = $tmp; #swapping the dst vol to be the src vol now.

    #swapping the src and dst paths
    $tmp             = $src_path;
    $src_path        = $dst_path; #swapping the src with dst path
    $dst_path        = $tmp; #swapping the dst with the src path

    $log->comment('Create(Reuse) a policy '.$policy_args{policy}.' on the dst node: ' . $node_dst->node() .
                  ' on vesrver: ' . $dst_vs.
                  ' Policy args are '.Dumper %policy_args);
    my $sm_policy;
    nacl_method_retry(
        code => sub {
            $sm_policy = NACL::STask::SnapmirrorPolicy->create(
                command_interface       => $node_dst,
                vserver                 => $dst_vs,
                nacltask_if_exists      => 'reuse',
                %policy_args,
            );
        },
        exceptions  => 'NACL::APISet::Exceptions::CommandFailedException',
        tries_count => 6,
        sleep_time  => 30,
    );

    $log->comment(
        'Issue a sm create using - ' . "\n" .
        'dst node as : ' . $node_dst->node() . "\n" .
        'sm policy as: ' . $sm_policy->policy . "\n" .
        'src path as : ' . $src_path . "\n" .
        'dst path as : ' . $dst_path
    );

    my %create_args = (
        command_interface  => $node_dst,
        source_ci          => $node_src,
        'source-path'      => $src_path,
        'destination-path' => $dst_path,
        type               => 'XDP',
        policy             => $sm_policy->policy,
        nacltask_wait      => 0,
        'do_initialize'    => 0,
    );

    my %mapping;
    if ($cg_item_list) {
        $log->comment('Get the cg mapping from the relationship');
        foreach my $item (@{ $cg_item_list }) {
            $mapping{"/vol/$src_vol/$item"} = "/vol/$dst_vol/$item";
        }
        $log->comment('cg mapping as: ' . Dumper(\%mapping));
        $create_args{'cg-item-mappings'} = \%mapping;
    }

    my $newsm;
    nacl_method_retry(
        code => sub {
	    $newsm = NACL::STask::Snapmirror->create(%create_args);
        },
        exceptions  => 'NACL::APISet::Exceptions::CommandFailedException',
        tries_count => 3,
        sleep_time  => 60,
    );

    #7. run sm resync
    if ($doresync) {
        $log->comment('Issue a resync. Will '.(($wait_for_resync) ? 'wait' : 'not wait').' for resync to complete');
        $newsm->resync(nacltask_wait => $wait_for_resync);

        if ($validate eq 'true') {
            if ($wait_for_resync) {
                if ($cg_item_list) {
                    #8. Validate if the dst luns (that are now src) are RW
                    #9. Validate if the src luns that are now dst) are Read only
                    Validate_flip_resync_cg_items (
                        node_src        => $node_dst, #the new src post flip-resync
                        node_dst        => $node_src, #the new dst post flip-resync
                        src_path        => $src_path,
                        dst_path        => $dst_path,
                        cg_item_list    => $cg_item_list,
                    );
                } else {
                    $log->warn('Skipping validation. Only cg item validation is present');
                }
            } else {
                $log->warn('Skipping validation. wait_for_resync is not set. Cannot perform validation till we wait for resync to complete.');
            }
        } else {
                $log->log('Skipping validation as requested.');
        }

	# run checker
	$log->log('Run checker.');
	my $sm_is_flexgroup = is_rel_flexgroup(sm_obj => $newsm);
	if($sm_is_flexgroup) {
	    sm_checker_fg(sm_obj => [$newsm]);
	} else {
	    $newsm->check(nacltask_wait => 1);
	}
    } else {
        $log->log('Skipping resync as requested');
    }

    return $newsm;

}

# private method
sub sm_resume{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            retry_list_dest_count    => {type => SCALAR, default=>30},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    my $retry_list_dest_count    = $args{'retry_list_dest_count'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );
    ##nacltask_wait => 0 due to Burt 946982
    $o->resume(nacltask_wait=>0);
    if($validate eq 'true'){
        resume_validations($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path,$retry_list_dest_count);
    }

}

# private method
sub _sm_init{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            snapshot           => {type => SCALAR|UNDEF, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            timeout            => {type => SCALAR, optional => 1, default=>1200},
            nacltask_wait      => {type => SCALAR, optional => 1, default=>1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    my $snapshot            = $args{'snapshot'};
    my $timeout             = $args{'timeout'};
    my $nacltask_wait       = $args{'nacltask_wait'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );
    my %init_options = ( 'nacltask_wait'  => $nacltask_wait,
                         'method-timeout' => $timeout,);
    if ($snapshot) {
        $init_options{'source-snapshot'} = $snapshot;
    }
    $o->initialize(%init_options);

    if($validate eq 'true'){
        my %more_args;
        if (exists($args{'snapshot'})){
            $more_args{snapshot} = $snapshot;
        }
        Validate_Initialize(   node_src  => $Src_Node_Obj,
                               node_dst  => $Dst_Node_Obj,
                               sm_obj    => $o,
                               %more_args
                            );
    }

}

# private method
sub resume_validations{
    #Validation1: Relationship status should not be quiesced afterwards.
    #Validation2: In case of Sync mirror - Validate the Insync status.

    my ($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path,$retry_count)=@_;
    # This code is to fetch vserver and volume for cg relationship

    if (!defined $retry_count) {
        $retry_count = 30;
    }

    my $state = NACL::CS::Snapmirror->fetch( command_interface => $Dst_Node_Obj,
            filter => {'source-path'      => $src_path,
            'destination-path' => $dst_path},
            );


    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
            'relationship-id' => $state->relationship_id,
            },
            requested_fields => [qw(source-path destination-path)]);

    my($Src_Vserver_Name,$src_vol_name) = split(/:/,$list_dst->source_path);
    my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$list_dst->destination_path);

    if($state->status eq 'Quiesced'){
        $log->error('Relationship should not have been quiesced after resume operation');
    }

    if($state->policy_type eq 'sync-mirror'){
        # Validate the InSync status
        _verify_list_destination_status($Src_Node_Obj,$dst_vol_name,$src_vol_name,$Src_Vserver_Name,$Dst_Vserver_Name,'InSync',$retry_count);
    }
}

# private method
sub _sm_modify{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            modify_options     => {type => HASHREF, default => {}},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );

    $o->modify(%{$args{'modify_options'}});
}

# private method
sub sm_break{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR|UNDEF, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR|UNDEF, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );

    $o->break();

    if($validate eq 'true'){
        break_validations($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path);
    }

}

# private method
sub break_validations{
    #Validation1: Destination Volume became RW.
    #Validation2: relationship state "broken off".
    #Validation3(TODO): In case of sync mirror - cg-items become RW(waiting on Randy;s code for this).
    #Validation4:Validate the OOS status of relationship.

    my ($Src_Node_Obj,$Dst_Node_Obj,$dst_path,$src_path)=@_;
    # This code is to fetch vserver and volume for cg relationship

    my $state = NACL::CS::Snapmirror->fetch( command_interface => $Dst_Node_Obj,
            filter => {'source-path'      => $src_path,
            'destination-path' => $dst_path},
            );


    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
            'relationship-id' => $state->relationship_id,
            },
            requested_fields => [qw(source-path destination-path)]);

    my($Src_Vserver_Name,$src_vol_name) = split(/:/,$list_dst->source_path);
    my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$list_dst->destination_path);
    $log->comment('verify that dst vol is RW');

    my $Dst_Vol_Obj = NACL::C::Volume->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            volume  => $dst_vol_name,
            vserver => $Dst_Vserver_Name,
            },
            );

    if ( $Dst_Vol_Obj->get_one_state_attribute('type') !~ /RW/ ) {
        $log->error('Destination volume is expected to be RW, but it is not.');
    }
    else{
        $log->comment('Destination volume is RW');
    }

    $log->comment('verify that relationship is in broken-off state');

    if($state->state ne 'Broken-off'){
        $log->error('Relationship should have been broken after break operation');
    }

    if($state->policy_type eq 'sync-mirror'){
        #TODO : get the list of cg-items(when Randy's code is here) and see if they are RW.
        # Validate the OutOfSync status
        _verify_list_destination_status($Src_Node_Obj,$dst_vol_name,$src_vol_name,
                            $Src_Vserver_Name,$Dst_Vserver_Name,'OutOfSync');
    }

}

# private method
sub sm_delete{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }
    my $sm_state = NACL::CS::Snapmirror->fetch(
            command_interface => $Dst_Node_Obj,
            filter=> {
            'destination-path' => $dst_path,
            },
            requested_fields => [qw(policy-type)]);

    my $o = NACL::STask::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
            },
            allow_empty => 1
            );

    $log->comment('delete snapmirror relationship');
    $o->delete();

    if($validate eq 'true'){
        delete_validations($dst_path,$src_path,$Dst_Node_Obj,$sm_state->policy_type);
    }

}

# private method
sub delete_validations{
    #Validation1: No relatinship left on destination side.
    #Validation2(TODO): In case of sync mirror - cg-items become RW(implicit break-waiting on Randy;s code for this).

    my($dst_path,$src_path,$Dst_Node_Obj,$policy_type)=@_;
    $log->comment('verify that there is no relationship left');
    my $relationship =  NACL::CS::Snapmirror->fetch(
            command_interface => $Dst_Node_Obj,
            filter => {
            'destination-path' => $dst_path,
            'source-path'      => $src_path, },
            allow_empty => 1,
            );
    if ($relationship) {
        $log->error("This is still a relationship after Delete");
    }

    $log->comment('verify that cg-tems become R/W');


    if($policy_type eq 'sync-mirror'){
        #TODO : get the list of cg-items(when Randy's code is here) and see if they are RW.
    }

}

# private method
sub sm_release{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT|SCALAR, optional=>1},
            node_dst           => {type => OBJECT|SCALAR, optional=>1},
            src_path           => {type => OBJECT|SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional=>1},
            validate           => {type => SCALAR, optional =>1},
            stagger_delay      => {type => SCALAR, optional => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $src_path            = $args{'src_path'};
    my $dst_path            = $args{'dst_path'};
    my $validate            = $args{'validate'};
    _stagger($args{'stagger_delay'});


    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    # This code is to get vserver and volume of a cg relationship
    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
                    'source-path'      => $src_path,
                    'destination-path' => $dst_path,
                    },
	        allow_empty => 1,
            );

    if($list_dst){
	my($Src_Vserver_Name,$src_vol_name) = split(/:/,$list_dst->source_path);
	my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$list_dst->destination_path);

	$src_path = $list_dst->source_path;
	my $dst_path = $list_dst->destination_path;

	try {
	    NACL::STask::Snapmirror->release(
		    command_interface  => $Src_Node_Obj,
		    'destination-path' => $dst_path,
		    'source-path'      => $src_path,
		    );
	}
	catch NATE::BaseException with {
	    $log->comment("Trying to release ,did not work");
	};

	if($validate eq 'true'){
	    release_validations($Src_Node_Obj,$dst_path,$src_path);
	}
    }
}

# private method
sub release_validations{
    #Validation1: No relatinship left on source side.
    my($Src_Node_Obj,$dst_path,$src_path)=@_;
    $log->comment('verify that there is no relationship left');
    my $relationship =  NACL::CS::SnapmirrorListDestinations->fetch(
            command_interface => $Src_Node_Obj,
            filter => {
            'destination-path' => $dst_path,
            'source-path'      => $src_path, },
            allow_empty => 1,
            );
    if ($relationship) {
        $log->error("This is still a relationship after release");
    }
}

# private method
sub sm_check{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT|SCALAR, optional=>1},
            node_dst           => {type => OBJECT|SCALAR, optional=>1},
            dst_path           => {type => SCALAR, optional =>1},
            src_path           => {type => SCALAR, optional =>1},
            source_snapshot    => {type => SCALAR, optional =>1},
            target_snapshot    => {type => SCALAR, optional =>1},
            file_list          => {type => ARRAYREF, optional=>1},
            stagger_delay      => {type => SCALAR, optional => 1, default=>10},
            'ift-check'       => { type => SCALAR,  optional => 1 },
            timeout           => {type => SCALAR,  optional => 1 , default => 1800},
            quiesce_resume_validation => {type => SCALAR,  optional => 1 , default => 'true'},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $dst_path            = $args{'dst_path'};
    my $src_path            = $args{'src_path'};
    my $target_ss           = $args{'target_snapshot'};
    my $source_ss           = $args{'source_snapshot'};
    my $file_list           = $args{'file_list'};
    my @files               = _list_of_vals($file_list);
    my $iftcheck            = $args{'ift-check'};
    my $timeout             = $args{'timeout'};
    my $quiesce_resume_validation = $args{'quiesce_resume_validation'};
    my $we_did_quiesce = 0;
    _stagger($args{'stagger_delay'});

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my %extra_args;
    if (defined $iftcheck) {
       $extra_args{'ift-check'} = $iftcheck;
    }

    my $o = NACL::STask::Snapmirror->find(
        command_interface => $Dst_Node_Obj,
        filter            => {
            'source-path'      => $src_path,
            'destination-path' => $dst_path,
        },
        allow_empty => 1
    );

    ##fix for the burt 1050739
    ##The solution for the issue "Check failed. (Could not read Snapshot copy for transfer () (Replication engine error))"
    ##1) Quiesce the relatioship
    ##2) snapmirror check
    ##3) resume the relationship
    sm_quiesce(node_src => $Src_Node_Obj,
		       node_dst => $Dst_Node_Obj,
		       src_path => $src_path,
		       dst_path => $dst_path,
		       validate => $quiesce_resume_validation,
		       'method-timeout' => '3600',

    );
    $we_did_quiesce = 1;
    if($target_ss && $source_ss){
	    my $st = time;
        my $op_done = 0;
        while (time < $st + $timeout) {
            try {
                $o->check('target-snapshot' => $target_ss,
                    'source-snapshot' => $source_ss,
                    'file-list'       => \@files,
                    %extra_args);
                $op_done = 1;
                last;
            } catch NATE::BaseException with {
                my $ex = shift;
                if ($ex->text =~ /Could not read Snapshot copy for transfer/i || $ex->text =~ /Volume access error/i ) {
                    $log->warn('Check failed. Could not read Snapshot copy for transfer or volume acess error . Expected failure . Retry Again');
                    Tharn::snooze(60);
                } else {
                    $ex->throw();
                }
            };
        }
        NATE::BaseException->throw('check failed after 30 minutes retry') if !$op_done;
    }
    elsif($target_ss){
        my $st = time;
        my $op_done = 0;
        while (time < $st + $timeout) {
            try {
                $o->check('target-snapshot' => $target_ss,
                 %extra_args);
                $op_done = 1;
                last;
            } catch NATE::BaseException with {
                my $ex = shift;
                if ($ex->text =~ /Could not read Snapshot copy for transfer/i || $ex->text =~ /Volume access error/i) {
                    $log->warn('Check failed. Could not read Snapshot copy for transfer or volume acess error . Expected failure . Retry Again');
                    Tharn::snooze(60);
                } else {
                    $ex->throw();
                }
            };
        }
        NATE::BaseException->throw('check failed after 30 minutes retry') if !$op_done;
    }
    else {
        my $st = time;
        my $op_done = 0;
        while (time < $st + $timeout) {
            try {
                $o->check(%extra_args,
                  'method-timeout' => 4500);
                $op_done = 1;
                last;
            } catch NATE::BaseException with {
                my $ex = shift;
                if ($ex->text =~ /Could not read Snapshot copy for transfer/i || $ex->text =~ /Volume access error/i) {
                    $log->warn('Check failed. Could not read Snapshot copy for transfer or volume acess error. Expected failure . Retry Again ');
                    Tharn::snooze(60);
                } else {
                    $ex->throw();
                }
            };
        }
        NATE::BaseException->throw('check failed after 30 minutes retry') if !$op_done;
    }

    if($we_did_quiesce) {
	$log->log('Issue resume since we did quiesce.');
	sm_resume( node_src => $Src_Node_Obj,
	   node_dst => $Dst_Node_Obj,
	   dst_path => $dst_path,
	   src_path => $src_path,
	   validate => $quiesce_resume_validation,);
    }
}

# private method
sub sm_cleanup{
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src           => {type => OBJECT|SCALAR, optional=>1},
            node_dst           => {type => OBJECT|SCALAR, optional=>1},
            dst_path           => {type => OBJECT|SCALAR, optional =>1},
            src_path           => {type => OBJECT|SCALAR, optional =>1},
            validate           => {type => OBJECT|SCALAR, optional =>1,
                                   default=>'false'},
            stagger_delay      => {type => SCALAR,  optional => 1},
            delete_policy      => {type => BOOLEAN, optional => 1,
                                   default=>'1'}
        },
    );

    my $node_src      = $args{'node_src'};
    my $node_dst      = $args{'node_dst'};
    my $dst_path      = $args{'dst_path'};
    my $src_path      = $args{'src_path'};
    my $validate      = $args{'validate'};
    my $delete_policy = $args{'delete_policy'};
    _stagger($args{'stagger_delay'});

    my $Dst_Node_Obj = $node_dst;
    if ( ($node_dst) && (ref($node_dst) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $node_dst );
    }

     $log->comment("####destination path here we got  is" . Dumper($dst_path));
    my $sm_state = NACL::CS::Snapmirror->fetch(
        command_interface => $Dst_Node_Obj,
        filter            => {
            'destination-path' => $dst_path,
            'source-path'      => $src_path,
        },
        requested_fields  => [qw(policy-type policy)]
    );
    my $policy_type = $sm_state->policy_type;
    my $policy_name = $sm_state->policy;

    my @path_split = split /:/, $dst_path;
    my $dst_vserver= $path_split[0];

    sm_quiesce(
        'node_src'      => $node_src,
        'node_dst'      => $node_dst,
        'dst_path'      => $dst_path,
        'src_path'      => $src_path,
        'validate'      => $validate,
        'stagger_delay' => 10,
        'method-timeout'   => '3600',

    );
    #If policy type is sync then dont call break because in In sync delete calls the implicit break

    if($policy_type ne 'sync-mirror'){
        sm_break(
            'node_src'      => $node_src,
            'node_dst'      => $node_dst,
            'dst_path'      => $dst_path,
            'src_path'      => $src_path,
            'validate'      => $validate,
            'stagger_delay' => 10,
        );
    }
    sm_delete(
        'node_src'      => $node_src,
        'node_dst'      => $node_dst,
        'dst_path'      => $dst_path,
        'src_path'      => $src_path,
        'validate'      => $validate,
        'stagger_delay' => 10,
    );
    sm_release(
        'node_src'      => $node_src,
        'node_dst'      => $node_dst,
        'src_path'      => $src_path,
        'dst_path'      => $dst_path,
        'validate'      => $validate,
        'stagger_delay' => 10,
    );

    NACL::C::SnapmirrorPolicy->delete(
        command_interface => $Dst_Node_Obj,
        vserver           => $dst_vserver,
        policy            => $policy_name,
    ) if $delete_policy;
}

# private method
sub app_ss{

    my %args = validate_with(
            params => \@_,
            spec   => {
                node_src       => {type => OBJECT| SCALAR, optional=>1},
                node_dst       => {type => OBJECT| SCALAR, optional=>1},
                dst_path       => {type => OBJECT|SCALAR, default => 1,},
                snapmirror_lbl => {type => SCALAR, default=>'SyncProtectMe'},
                validate       => {type => OBJECT|SCALAR, optional =>1,default=>'false'},
                snapshot_name  => {type => SCALAR, optional=>1}, # default 'acs'.pid.count
                snapshot_count => {type => SCALAR, default=>1, optional=>1},
                stagger_delay  => {type => SCALAR,optional =>1, default => 10},
                delete_ss      => {type => SCALAR,optional =>1, default => 0},
                delete_delay   => {type => SCALAR,optional =>1, default => 10},
            },
        );

    my $Src_Node       = $args{'node_src'};
    my $Dst_Node       = $args{'node_dst'};
    my $snapshot_count = $args{'snapshot_count'};
    my $dst_path       = $args{'dst_path'};
    my $validate       = $args{'validate'};
    my $del            = $args{'delete_ss'};
    my $del_delay      = $args{'delete_delay'};
    _stagger($args{'stagger_delay'});

    $Src_Node = NACL::STask::Node->new( name => $Src_Node )
        if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) );
    $Dst_Node = NACL::STask::Node->new( name => $Dst_Node )
        if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) );

    # get the volume and vserver names of src and dest
    my $cg_data = NACL::C::Snapmirror->get_consistency_group_data(
        command_interface => $Dst_Node,
        'destination-path' => $dst_path,
    );
    my ($src_vserver_name,$src_vol_name,$dst_vol_name,$dst_vserver_name);
    for my $key (sort keys %{$cg_data}) {
        ($src_vserver_name,$src_vol_name) = ($cg_data->{$key}->{source_path} =~ /(.*):\/vol\/(.*)\//);
        ($dst_vserver_name,$dst_vol_name) = ($cg_data->{$key}->{destination_path} =~ /(.*):\/vol\/(.*)\//);
        $log->comment('Got source vserver:'.$src_vserver_name.' src vol:'.$src_vol_name.
                     ' dest vserver:'.$dst_vserver_name.' dest vol:'.$dst_vol_name);
    }

    # TODO: timeout in the policy for App Consistent Snapshot in later chunk
    # fetch the policy of snapmirror object
    my $Policy_Obj = NACL::CS::Snapmirror->fetch(
            command_interface => $Dst_Node,
            filter=> { 'destination-path' =>$dst_path, },
            requested_fields => [qw(policy)]);

    my $sm_lbl = NACL::CS::SnapmirrorPolicy->fetch(command_interface => $Dst_Node,
            filter=> { policy => $Policy_Obj->policy, },
            requested_fields => [qw(snapmirror-label)]);

    my @sm_lbls  = $sm_lbl->snapmirror_label;
    my $sm_label = 'SyncProtectMe';
    if ( grep( /^$sm_label$/, @sm_lbls ) ) {
        $log->comment('Found rule '. $sm_label);
    } else {
        # TODO:Once sync_Protectme lbl is there in the ontap code then remove the code below.
        $log->warn('Remove this later. Adding rule '.$sm_label);
        NACL::STask::SnapmirrorPolicy->add_rule(command_interface => $Dst_Node,
                                            policy             => $Policy_Obj->policy,
                                            vserver            => $dst_vserver_name,
                                            'snapmirror-label' => $sm_label,
                                            'keep'             => 1,
                                            preserve           => 'false',
                                            nacltask_if_exists => 'reuse'
                                            );
	Tharn::snooze(30); #allow policy to propagate
    }

    # Create ACS
    my @app_cons_snaps;
    for(my $i=0;$i<$snapshot_count;$i++){
        my $snapshot_name = ($args{snapshot_name}?$args{snapshot_name}.$i:'acs'.$$.$i);
        $log->comment("Creating $i ACS $snapshot_name on Source Volume $src_vol_name using snapmirror label $sm_label.");
        my $app_cons_snap = NACL::STask::VolumeSnapshot->create(
                command_interface  => $Src_Node,
                volume             => $src_vol_name,
                vserver            => $src_vserver_name,
                snapshot           => $snapshot_name,
                'snapmirror-label' => $sm_label,
                #apiset_must        => {interface => 'zapi'}, not working now
                );
        push (@app_cons_snaps, $app_cons_snap);
    }

    # Validate the ACS
    if ($validate eq 'true') {
        Tharn::snooze(30); #allow SS to settle on dst site
        foreach my $ss(@app_cons_snaps) {
            nacl_method_retry(
                code => sub {
                            validate_ACS(
                                    snapshot => $ss,
                                    node_src => $Src_Node,
                                    node_dst => $Dst_Node,
                                    vol_src_name => $src_vol_name,
                                    vol_dst_name => $dst_vol_name,
                                    snapmirror_label => $sm_label,
                                    skip_validation => 'none', );
                            },
                tries_count => 5,
                sleep_time => 60,
                exceptions => 'NATE::BaseException',
            );
        }
    }

    #delete the ss after waiting for $del_delay secs
    if ($del) {
        $log->comment('Sleeping for ' . $del_delay . ' secs before attempting to delete the ss');
        Tharn::snooze($del_delay);

        foreach my $acs (@app_cons_snaps) {
            $log->comment('Attempting to delete ss ' .  $acs->snapshot());
            $acs->purge();
        }
    }
}

=head2 validate_ACS

This method will validate the ACS on the primary and secondary.

    Validation 1: name tags should be present on destination as well as source snapshot.
    Validation 2: Snapmirror label should be applied on both source and destination snapshot.
    Validation 3: version uuids of both source and destination application consistent snapshot should match.

Will throw exceptions on failure.

=over

=item Example

    The following example will perform validation of the ACS snapshot on the source and destination.
    validate_ACS(
        snapshot => $snapshot_obj,
        node_src => $node_src,
        node_dst => $node_dst,
        vol_src_name => 'srcvol',
        vol_dst_name => 'dstvol', );

    The following example will perform validation of the ACS snapshot on the destination.
    validate_ACS(
        snapshot => $snapshot_obj,
        node_dst => $node_dst,
        vol_dst_name => $vol_dst->volume,
        skip_validation => 'source',);

=item Options

=over

=item snapshot

(Required): Snapshot object of type NACL::STask::VolumeSnapshot.

=item skip_validation

(Optional): Defaults to 'none'. Possible values 'source/src/dest/dst/destination/none'.
If src/source will skip the validation for the Source Snapshot.
If dest/dst/destination will skip the validation for the Source Snapshot.
If none will NOT skip any validation.

=item node_src

(Optional): Source Node Object of type NACL::C::Node. Required if validation is to be performed on source.

=item node_dst

(Optional): Destination Node Object of type NACL::C::Node. Required if validation is to be performed on destination.

=item vol_src_name

(Optional): Name of the source volume. Required if validation is to be performed on source.

=item vol_dst_name

(Optional): Name of the destination volume. Required if validation is to be performed on destination.

=item snapmirror_label

(Optional): Defaults to 'SyncProtectMe'

=back

=item Exceptions

    NACL::Exceptions::InvalidParam
    NACL::Exceptions::UnexpectedOutputException
    NATE::BaseException

=back

=cut

sub validate_ACS {
    my %args = validate_with(
            params => \@_,
            spec   => {
                snapshot  => {type => OBJECT, optional=>0, isa => 'NACL::STask::VolumeSnapshot'},
                node_src  => {type => OBJECT, optional=>1, isa => 'NACL::C::Node'},
                node_dst  => {type => OBJECT, optional=>1, isa => 'NACL::C::Node'},
                vol_src_name => {type => SCALAR, optional=>1},
                vol_dst_name => {type => SCALAR, optional=>1},
                snapmirror_label => {type => SCALAR, optional=>1, default=>'SyncProtectMe'},
                skip_validation => {type => SCALAR, optional=>1, default=>'none',
                            regex => qr/^(none|src|source|dest|dst|destination)$/i},
                },
            );
    my $skip_validation=$args{skip_validation};
    if ($skip_validation =~ /^none$/i) {
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need vol_src_name argument') if (!$args{vol_src_name});
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need vol_dst_name argument') if (!$args{vol_dst_name});
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need node_src argument') if (!$args{node_src});
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need node_dst argument') if (!$args{node_dst});
    }

    my $snapshot_name = $args{snapshot}->snapshot();
    my $sm_label=$args{snapmirror_label};
    $log->log('Validate ACS:'.$snapshot_name.' using snapmirror label '.$sm_label);

    # sample output from snaptags
    #c-login_11|SUCCESS\r\n
    #c-login_11|SM_Label: succeeded val=SyncProtectMe attrs=0x1\r\n
    #c-login_11|smCG_name.srccgss18468.gran: succeeded val=snapmirror attrs=0x1\r\n
    #c-login_11|smCG_version_uuid.f3307b2d-10c6-4100-9066-6e35d58dd220.gran: succeeded val=snapmirror attrs=0x1\r\n
    #c-login_11|result=255\r\n
    #c-login_11|\r\n

    my $exception_text;
    my $dest_uuid;
    if ($skip_validation =~ /^(dest|dst|destination)$/i) {
        $log->comment('Argument skip_validation='.$skip_validation.' Skipping validation for destination ACS.');
    } else {
        # Validate destination ACS
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need vol_dst_name argument') if (!$args{vol_dst_name});
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need node_dst argument') if (!$args{node_dst});

        # fetch dst volume local node
        my $vol_dst_obj = NACL::CS::Volume->fetch(
                command_interface => $args{node_dst},
                filter => {
                    volume => $args{vol_dst_name},
                },
                requested_fields  => ['node'],
        );

        my $apiset = NACL::APISet->new(
            hostobj   => host($vol_dst_obj->node),
            category  => "Node",
            interface => "CLI",
            set       => "Nodescope",
        );

	# make sure volume is online after volmove: BURT 948446
	$log->log('Verify volume ' . $args{vol_dst_name} . ' is online');
        my $vol_obj = NACL::C::Volume->find(
                command_interface => $args{node_dst},
                filter		  => {
                    volume => $args{vol_dst_name},
                },
        );

	$vol_obj->wait_on_attribute(
	    attribute_to_check => 'state',
	    till_value         => qr/online/i,
	    'method-timeout'   => '900',
	);

        my $raw_output_dst = $apiset->execute_command(command => "snaptags_test -g -v $args{vol_dst_name} -S $snapshot_name");

        #NACL::Exceptions::UnexpectedOutputException->throw(text=>"snaptags_test command failed.\nGot $raw_output_dst")
        NATE::BaseException->throw("snaptags_test command failed.\nGot $raw_output_dst")
            if ($raw_output_dst =~ /Can\'t get id of snapshot/i);

        if ($raw_output_dst =~ /SUCCESS/ ) {
            # the output and error stream gets mixed sometimes so make the regex flexible
            if ($raw_output_dst !~ /SM_Label:.*succeeded.*val=$sm_label.*attrs=0x1/s) {
                my $txt = 'Destination ACS snapmirror-label mismatch. Expected '.$sm_label."\nOutput got:".$raw_output_dst;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }

            if ($raw_output_dst !~ /smCG_name.\S+.gran:.*succeeded.*val=snapmirror.*attrs=0x1/s) {
                my $txt = 'Destination ACS name tag 1 missing. Expected cg name.'."\nOutput got: ".$raw_output_dst;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }
            ($dest_uuid) = ($raw_output_dst =~ /smCG_version_uuid.(\S+).gran:.*succeeded.*val=snapmirror.*attrs=0x1/s);
            if (!$dest_uuid) {
                my $txt = 'Destination ACS name tag 2 missing. Expected version UUID.'."\nOutput got: ".$raw_output_dst;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }
        } else {
            my $txt = 'Validation of destination ACS failed';
            $exception_text .= $txt;
            $log->log($txt);
            #NACL::Exceptions::UnexpectedOutputException->throw(text=>'Validation of destination ACS failed');
        }
    }

    my $src_uuid;
    if ($skip_validation =~ /^(source|src)$/i) {
        $log->comment('Argument skip_validation='.$skip_validation.' Skipping validation for source ACS.');
    } else {
        # Validate source ACS
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need vol_src_name argument') if (!$args{vol_src_name});
        NACL::Exceptions::InvalidParam->throw('Argument skip_validation='.$skip_validation.' Need node_src argument') if (!$args{node_src});

        # fetch src volume local node
        my $vol_src_obj = NACL::CS::Volume->fetch(
                command_interface => $args{node_src},
                filter => {
                    volume => $args{vol_src_name},
                },
                requested_fields  => ['node'],
        );

        my $apiset = NACL::APISet->new(
            hostobj   => host($vol_src_obj->node),
            category  => "Node",
            interface => "CLI",
            set       => "Nodescope",
        );

	# make sure volume is online after volmove: BURT 948446
	$log->log('Verify volume ' . $args{vol_src_name} . ' is online');
        my $vol_obj = NACL::C::Volume->find(
                command_interface => $args{node_src},
                filter		  => {
                    volume => $args{vol_src_name},
                },
        );

	$vol_obj->wait_on_attribute(
	    attribute_to_check => 'state',
	    till_value         => qr/online/i,
	    'method-timeout'   => '900',
	);

        my $raw_output_src = $apiset->execute_command(command => "snaptags_test -g -v $args{vol_src_name} -S $snapshot_name");

        NACL::Exceptions::UnexpectedOutputException->throw(text=>'snaptags_test command failed. Got '.$raw_output_src)
            if ($raw_output_src =~ /Can\'t get id of snapshot/i);

        if ( $raw_output_src =~ /SUCCESS/ ) {
            # the output and error stream gets mixed sometimes so make the regex flexible
            if ($raw_output_src !~ /SM_Label:.*succeeded.*val=$sm_label.*attrs=0x1/s) {
                my $txt = 'Source ACS snapmirror-label mismatch. Expected '.$sm_label."\nOutput got:".$raw_output_src;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }

            if ($raw_output_src !~ /smCG_name.\S+.gran:.*succeeded.*val=snapmirror.*attrs=0x1/s) {
                my $txt = 'Source ACS name tag 1 missing. Expected cg name.'."\nOutput got: ".$raw_output_src;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }
            ($src_uuid) = ($raw_output_src =~ /smCG_version_uuid.(\S+).gran:.*succeeded.*val=snapmirror.*attrs=0x1/s);
            if (!$src_uuid) {
                my $txt = 'Source ACS name tag 2 missing. Expected version UUID.'."\nOutput got: ".$raw_output_src;
                $exception_text .= $txt;
                $log->log($txt);
                #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
            }
        } else {
            my $txt = 'Validation of source ACS failed';
            #NACL::Exceptions::UnexpectedOutputException->throw(text=>'Validation of source ACS failed');
            $log->log($txt);
            $exception_text .= $txt;
        }
    }

    # Compare UUID of source and destination ACS
    if ( ($args{skip_validation} =~ /^both$/i) && ($src_uuid !~ $dest_uuid) ) {
        my $txt = 'Version UUIDs of ACS does not match on source and destination. '.
                   'Got Source UUID:'.$src_uuid.'. Destination UUID:'.$dest_uuid;
        $exception_text .= $txt;
        $log->log($txt);
        #NACL::Exceptions::UnexpectedOutputException->throw(text=>$txt);
    }

    if ($exception_text) {
        NATE::BaseException->throw($exception_text);
    } else {
        $log->log('Validation of ACS succeeded');
    }

    return;
}


#Actual Workflows start here:
#-----------------------------------

#This will create any no of relationships with any relationship type(adr/vault/sync) given

=head2 snapmirror_create

Provides the ability to create any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are created.

=over

=item Example

This example will create 2, 1-1 sync relationship,
first one vol1->vol_dst1 contain luns and
second one vol2->vol_dst2 contains files.

    %cg_item_list_hash = ('vol1' => ['lun1','lun2'],
                          'vol2' => ['file1','file2']);,

    %src_cg_name  =     ('vol1'     => '/cg/Src_Cg_Name1',
                         'vol2'     => '/cg/Src_Cg_Name2');
    %dst_cg_name  =     ('vol_dst1' => '/cg/Dst_Cg_Name1',
                         'vol_dst2' => '/cg/Dst_Cg_Name2');

    snapmirror_create('node_src'                => $src_node_name,
                      'node_dst'                =>  $dst_node_name,
                      'src_volumes'             => [$vol1,$vol2],
                      'dst_volumes'             => [$vol_dst1,$vol_dst2],
                      'cg_item_list'            => \%cg_item_list_hash,
                      'src_cg_name'             => \%src_cg_name,
                      'dst_cg_name'             => \%dst_cg_name,
                      'relationship_type'       => 'SYNC');

=item Options

=over

=item node_src

(Optional) src node object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) dst node object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref of destination volume objects

=item src_cg_name

(Optional) hashref of src cg name with key as src vol name

=item dst_cg_name

(Optional) hashref of dst cg name with key as dst vol name

=item cg_item_list

(Optional) Arrayref of list of cg-items(luns or files). NOTE:Its a required parameter for sync but optional for ADR/Vault

=item relationship_type

(Required) type(ADR/VAULT/SYNC) of relationship(s) to be created.Default Value is 'SYNC'

=item add_label

(Optional) Name of label to add to policy. Default 'none', meaning no label will be added.

=back

=back

=cut

sub snapmirror_create{

        my %args = validate_with(
            params      => \@_,
            allow_extra => 1,
            spec        => {
                node_src =>
                    { type => OBJECT, optional => 1, isa => 'NACL::C::Node' },
                node_dst =>
                    { type => OBJECT, optional => 0, isa => 'NACL::C::Node' },
                src_volumes  => { type => ARRAYREF | OBJECT, optional => 0 },
                dst_volumes  => { type => ARRAYREF | OBJECT, optional => 0 },
                src_cg_name  => { type => HASHREF,           optional => 1 },
                dst_cg_name  => { type => HASHREF,           optional => 1 },
                cg_item_list => { type => HASHREF,           optional => 1 },
                relationship_type => {
                    type     => SCALAR,
                    optional => 1,
                    default  => 'SYNC',
                    regex    => qr/ADR|VAULT|SYNC/,
                },
                tries => { type => SCALAR, optional => 1, default => 8, },
                add_label => { type => SCALAR, optional => 1, default => 'none' },
            },
        );


        my ($Src_Node,    $Dst_Node,     $src_vol,
            $dst_vol,     $cg_item_list, $Src_Cg_Name,
            $Dst_Cg_Name, $src_cg_name,  $dst_cg_name,
            $tries,       $relationship_type,
        );

        my @src_volumes;
        my @dst_volumes;

        # Hash ref of NACL objects for caller of this function.
        my $all_objects;

        $Src_Node          = $args{'node_src'};
        $Dst_Node          = $args{'node_dst'};
        $src_vol           = $args{'src_volumes'};
        $dst_vol           = $args{'dst_volumes'};
        $cg_item_list      = $args{'cg_item_list'};
        $Src_Cg_Name       = $args{'src_cg_name'};
        $Dst_Cg_Name       = $args{'dst_cg_name'};
        $relationship_type = $args{'relationship_type'};
        $tries             = $args{'tries'};
       my $add_label        = $args{'add_label'};

       @src_volumes = _list_of_vals($src_vol);
       @dst_volumes = _list_of_vals($dst_vol);

        my @subtests;
        my @Sm_Objs;
        my @src_cg_names;
        my @dst_cg_names;
        my @src_vol_names;
        my @dst_vol_names;

        my $indx=0;
        my  $Src_Vserver_Name;
        my  $Dst_Vserver_Name;
          for(my $i=0;$i<=$#src_volumes;$i++){
                $Src_Vserver_Name = $src_volumes[$i]->vserver;
                $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
            if ( $relationship_type eq 'SYNC' ) {
                $src_cg_name = ${$Src_Cg_Name}{ $src_volumes[$i]->volume };
                $dst_cg_name = ${$Dst_Cg_Name}{ $dst_volumes[$i]->volume };
                my $cg_item_arr_ref
                    = ${$cg_item_list}{ $src_volumes[$i]->volume };
                my @cg_items_array = _list_of_vals($cg_item_arr_ref);
                push @subtests,
                    NATE::Process->new(
                    codespec => \&sm_create_cg,
                    runid    => 'CREATE_SYNC_RELATIONSHIP' . $indx,
                    args     => [
                        'src_vol'       => $src_volumes[$i]->volume,
                        'dst_vol'       => $dst_volumes[$i]->volume,
                        'src_vserver'   => $Src_Vserver_Name,
                        'dst_vserver'   => $Dst_Vserver_Name,
                        'src_cg_name'   => $src_cg_name,
                        'dst_cg_name'   => $dst_cg_name,
                        'cg_item_list'  => \@cg_items_array,
                        'node_dst'      => $Dst_Node,
                        'node_src'      => $Src_Node,
                        'tries'         => $tries,
                        'add_label'     => $add_label,
                        'stagger_delay' => 10,
                    ],
                    inherit_params => 1,
                    onexit => \&NATE::Process::on_exit_propagate_worst_result,
                    );
                push @src_cg_names, $src_cg_name;
                push @dst_cg_names, $dst_cg_name;
            }
            elsif ($relationship_type eq 'ADR'){
                push @subtests,NATE::Process->new(
                        codespec => \&sm_create_adr,
                        runid => 'CREATE_ADR_RELATIONSHIP' .$indx,
                        args => [
                    'src_vol'     => $src_volumes[$i]->volume,
                    'dst_vol'     => $dst_volumes[$i]->volume,
                    'src_vserver' => $Src_Vserver_Name,
                    'dst_vserver' => $Dst_Vserver_Name,
                    'node_dst'    => $Dst_Node,
                    'node_src'    => $Src_Node,
                    'stagger_delay' => 10,
                    ],
                        inherit_params => 1,
                        onexit=> \&NATE::Process::on_exit_propagate_worst_result,
                        );
                push @src_vol_names , $src_volumes[$i]->volume ;
                push @dst_vol_names , $dst_volumes[$i]->volume ;
            }
            elsif ($relationship_type eq 'VAULT'){
                push @subtests,NATE::Process->new(
                        codespec => \&sm_create_vault,
                        runid => 'CREATE_VAULT_RELATIONSHIP' .$indx,
                        args => [
                    'src_vol'     => $src_volumes[$i]->volume,
                    'dst_vol'     => $dst_volumes[$i]->volume,
                    'src_vserver' => $Src_Vserver_Name,
                    'dst_vserver' => $Dst_Vserver_Name,
                    'node_dst'    => $Dst_Node,
                    'node_src'    => $Src_Node,
                    'stagger_delay' => 10,
                    ],
                        inherit_params => 1,
                        onexit=> \&NATE::Process::on_exit_propagate_worst_result,
                        );
                push @src_vol_names , $src_volumes[$i]->volume ;
                push @dst_vol_names , $dst_volumes[$i]->volume ;
            }

            $indx++;

          }


    my $pm = NATE::ProcManager->new(
            runid               => 'PM_Create_' . $relationship_type,
            max_active_children => 16,
            proc_info           => \@subtests,
            );
    $pm->start();
    $pm->wait();
    $pm->destroy();
    $Src_Node->refresh_command_interface(); # workaround for burt863951
    $Dst_Node->refresh_command_interface(); # workaround for burt863951

    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }

    if ($relationship_type eq 'SYNC'){
        foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }
    else{
        foreach(my $i=0;$i<=$#src_vol_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_vol_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_vol_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

=head2 snapmirror_initialize

Provides the ability to initialize any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are initialized.If Initialize is done without
create then cg-item-list should be passed.

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref of destination volume objects

=item src_cg_name

(Optional) hashref of src cg name with key as src vol name

=item dst_cg_name

(Optional) hashref of dst cg name with key as dst vol name

=item cg_item_list

(Optional) hashref of list of cg-items(luns or files names) with key as src vol name

=item relationship_type

(Required) type(ADR/VAULT/SYNC) of relationship(s) to be created.Default Value is SYNC.

=item validate

(Optional) boolean(true/false) to do validations after relationship(s) is/are intialized.

=item timeout

(Optional) method-timeout for the initialize operation. Default is 1200 seconds.

=item snapshot

(Optional) snapshot name for initialize-s command.

=item max_active_children

(Optional) Number of parallel processes to start at a time.  Default is 16.

If Validate is true.Its going to Validate these:
Validation1: status =InSync, state=snapmirrored, healthy=true, last-tranfer-type =initialize via snapmirror-show
Validation2: Snapmirror-snapshot gets cut on the destination after initialize.
Validation3 : In case of sync validate the "Insync" status via snapmirror-list-destionations

=back

=back

=cut

sub snapmirror_initialize {
    my %args = validate_with(
	params => \@_,
	allow_extra => 1,
	spec   => {
	node_src            => {type => OBJECT|SCALAR, optional=>1, },
	node_dst            => {type => OBJECT|SCALAR, optional=>0, },
	src_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
	dst_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
	src_cg_name         => {type => HASHREF, optional => 1},
	dst_cg_name         => {type => HASHREF, optional => 1},
	cg_item_list        => {type => HASHREF, optional=>1},
	relationship_type   => {type => SCALAR, optional => 1, default => 'SYNC', regex => qr/ADR|VAULT|SYNC/,},
	snapshot            => {type => SCALAR, optional =>1},
	validate            => {type => SCALAR, optional =>1,default=>'false'},
	timeout             => {type => SCALAR, optional =>1,default=>1200},
	max_active_children => {type => SCALAR, optional =>1,default=>16},
	retry_list_dest_count    => {type => SCALAR, default=>30},
	},
    );

    # Hash ref of NACL objects for caller of this function.
    my $all_objects;

    my $Src_Node            = $args{'node_src'};
    my $Dst_Node            = $args{'node_dst'};
    my $src_vol             = $args{'src_volumes'};
    my $dst_vol             = $args{'dst_volumes'};
    my $Src_Cg_Name         = $args{'src_cg_name'};
    my $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $relationship_type   = $args{'relationship_type'};
    my $validate            = $args{'validate'};
    my $timeout             = $args{'timeout'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $retry_list_dest_count    = $args{'retry_list_dest_count'};
    my $snapshot             = $args{'snapshot'};
    my $max_active_children  = $args{'max_active_children'};

    my @src_volumes = _list_of_vals($src_vol);
    my @dst_volumes = _list_of_vals($dst_vol);

    my @sorted_src = sort { $a->{volume} cmp $b->{volume}} @src_volumes;
    my @sorted_dst = sort { $a->{volume} cmp $b->{volume} } @dst_volumes;

    @src_volumes = @sorted_src;
    @dst_volumes = @sorted_dst;

    my @subtests;
    my @Sm_Objs;
    my @src_cg_names;
    my @dst_cg_names;
    my @src_vol_names;
    my @dst_vol_names;
    my $Src_Vserver_Name;
    my $Dst_Vserver_Name;

    if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) ) {
	$Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
	$Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }

    my $indx=0;
    my ($src_cg_name, $dst_cg_name);
    for(my $i=0;$i<=$#src_volumes;$i++){
	if ($relationship_type eq 'SYNC'){
	    $Src_Vserver_Name = $src_volumes[$i]->vserver;
	    $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
	    if(exists($args{'src_cg_name'}) && exists($args{'dst_cg_name'})){
	       $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
	       $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};
	    }
	    if (exists($args{'cg_item_list'})) {
		 my $cg_item_arr_ref = ${$cg_item_list}{$src_volumes[$i]->volume};
		 my @cg_items_array =  _list_of_vals($cg_item_arr_ref);

		push @subtests,NATE::Process->new(
		    codespec => \&sm_initialize_cg,
		    runid => 'INIT_SYNC_RELATIONSHIP' .$indx,
		    args => [
		    'src_vol'       => $src_volumes[$i]->volume,
		    'dst_vol'       => $dst_volumes[$i]->volume,
		    'src_vserver'   => $Src_Vserver_Name,
		    'dst_vserver'   => $Dst_Vserver_Name,
		    'src_cg_name'   => $src_cg_name,
		    'dst_cg_name'   => $dst_cg_name,
		    'cg_item_list'  => \@cg_items_array,
		    'node_dst'      => $Dst_Node,
		    'node_src'      => $Src_Node,
		    'stagger_delay' => 10,
		    'retry_list_dest_count' => $retry_list_dest_count,
		    'validate'      => $validate,
		    'timeout'       => $timeout,
		    ],
		    inherit_params => 1,
		    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
		    );
	    }
	    else{
		push @subtests,NATE::Process->new(
		    codespec => \&sm_initialize_cg,
		    runid => 'INIT_SYNC_RELATIONSHIP' .$indx,
		    args => [
		    'src_vol'       => $src_volumes[$i]->volume,
		    'dst_vol'       => $dst_volumes[$i]->volume,
		    'src_vserver'   => $Src_Vserver_Name,
		    'dst_vserver'   => $Dst_Vserver_Name,
		    'src_cg_name'   => $src_cg_name,
		    'dst_cg_name'   => $dst_cg_name,
		    'node_dst'      => $Dst_Node,
		    'node_src'      => $Src_Node,
		    'stagger_delay' => 10,
		    'validate'      => $validate,
		    'timeout'       => $timeout,
		    'retry_list_dest_count' => $retry_list_dest_count,
		    ],
		    inherit_params => 1,
		    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
		    );
	    }
	    push @src_cg_names , $src_cg_name ;
	    push @dst_cg_names , $dst_cg_name ;
	}
	elsif ($relationship_type eq 'ADR'){
	    $Src_Vserver_Name = $src_volumes[$i]->vserver;
	    $Dst_Vserver_Name = $dst_volumes[$i]->vserver;

	    push @subtests,NATE::Process->new(
		    codespec => \&sm_initialize_adr,
		    runid => 'INIT_ADR_RELATIONSHIP' .$indx,
		    args => [
		    'src_vol'     => $src_volumes[$i]->volume,
		    'dst_vol'     => $dst_volumes[$i]->volume,
		    'src_vserver' => $Src_Vserver_Name,
		    'dst_vserver' => $Dst_Vserver_Name,
		    'node_dst'    => $Dst_Node,
		    'node_src'    => $Src_Node,
		    'stagger_delay' => 10,
		    'validate'      => $validate,
		    'timeout'       => $timeout,
		    ],
		    inherit_params => 1,
		    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
		    );
	    push @src_vol_names , $src_volumes[$i]->volume ;
	    push @dst_vol_names , $dst_volumes[$i]->volume ;
	}
	elsif ($relationship_type eq 'VAULT'){
	    if (exists($args{'snapshot'})) {
	       $Src_Vserver_Name = $src_volumes[$i]->vserver;
	       $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
		push @subtests,NATE::Process->new(
		    codespec => \&sm_initialize_vault,
		    runid => 'INIT_VAULT_RELATIONSHIP' .$indx,
		    args => [
		    'src_vol'     => $src_volumes[$i]->volume,
		    'dst_vol'     => $dst_volumes[$i]->volume,
		    'src_vserver' => $Src_Vserver_Name,
		    'dst_vserver' => $Dst_Vserver_Name,
		    'node_dst'    => $Dst_Node,
		    'node_src'    => $Src_Node,
		    'snapshot'    => $snapshot,
		    'stagger_delay' => 10,
		    'validate'      => $validate,
		    'timeout'       => $timeout,
		    ],
		    inherit_params => 1,
		    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
		    );
		push @src_vol_names , $src_volumes[$i]->volume ;
		push @dst_vol_names , $dst_volumes[$i]->volume ;
	   }
	   else{
	       $Src_Vserver_Name = $src_volumes[$i]->vserver;
	       $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
		push @subtests,NATE::Process->new(
		    codespec => \&sm_initialize_vault,
		    runid => 'INIT_VAULT_RELATIONSHIP' .$indx,
		    args => [
		    'src_vol'     => $src_volumes[$i]->volume,
		    'dst_vol'     => $dst_volumes[$i]->volume,
		    'src_vserver' => $Src_Vserver_Name,
		    'dst_vserver' => $Dst_Vserver_Name,
		    'node_dst'    => $Dst_Node,
		    'node_src'    => $Src_Node,
		    'stagger_delay' => 10,
		    'validate'      => $validate,
		    'timeout'       => $timeout,
		    ],
		    inherit_params => 1,
		    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
		    );
		push @src_vol_names , $src_volumes[$i]->volume ;
		push @dst_vol_names , $dst_volumes[$i]->volume ;

	   }
	}
	$indx++;

    }

    my $pm = NATE::ProcManager->new(
	runid               => 'PM_Initialize_' . $relationship_type,
	max_active_children => $max_active_children,
	proc_info           => \@subtests,
	);
    $pm->start();
    $pm->wait();
    $pm->destroy();
    $Src_Node->refresh_command_interface(); # workaround for burt863951
    $Dst_Node->refresh_command_interface(); # workaround for burt863951

    if ($relationship_type eq 'SYNC'){
        foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }
    else{
        foreach(my $i=0;$i<=$#src_vol_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_vol_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_vol_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }
    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

=head2 snapmirror_update

to perform update operation on a snapmirror relatioship

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item interval_time

(Optional)  The time interval betweeen 2 successive runs.

=item cycles_per_day

(Optional)  The number of times the operation is executed per day at equal intervals

=item snapshot

(Optional)  snapshot name for update-s command.

=item validate

(Optional) Do additional validation. Default 'false'. If set to 'true' will validate the snapshot used.
If nacltask_wait is 0 then validate will be set to 'false'.

=item nacltask_wait

(Optional) If set, will wait for update to finish. Default: 1.

=back

=back

=cut

sub snapmirror_update {
    my %args = validate_with(
                     params      => \@_,
                     allow_extra => 1,
                     spec        => {
                         node_src => { type => OBJECT | SCALAR, optional => 1 },
                         node_dst => { type => OBJECT | SCALAR, optional => 0 },
                         sm_objs  => { type => ARRAYREF,        optional => 0 },
                         snapshot => { type => SCALAR , optional => 1},
                         throttle => { type => SCALAR , optional => 1},
                         iterations  => { type => SCALAR, optional => 1 },
                         validate    => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
                         interval_time => { type=> SCALAR, optional =>1},
                         cycles_per_day => {type=>SCALAR, optional=>1},
                         nacltask_wait  => {type => SCALAR, default => 1},
                     },
    );

    my $interval_time  = $args{ 'interval_time' };
    my $cycles_per_day = $args{ 'cycles_per_day' };
    my $validate     = $args{ 'validate' };
    my $node_src_Obj = $args{ 'node_src' };
    my $node_dst_Obj = $args{ 'node_dst' };
    my $snapshot = $args{'snapshot'};
    my $throttle = $args{'throttle'};
    my $updates      = $args{ 'sm_objs' };
    my $iterations   = $args{ 'iterations' };
    my @Updates      = _list_of_vals($updates);
    my @subtests_update;

    $counter = 0;
    foreach my $o (@Updates) {
        my %args_func = (

            'node_src'    => $node_src_Obj,
            'dst_path'    => $o->destination_path,
            'src_path'    => $o->source_path,
            'dst_volume'  => $o->destination_volume(),
            'node_dst'    => $node_dst_Obj,
            'dst_vserver' => $o->destination_vserver(),
            'validate'    => $validate,
            'nacltask_wait' => $args{nacltask_wait},
        );
        if(exists($args{'snapshot'})){
           $args_func{'snapshot'} = $snapshot;
       }
        if(exists($args{'throttle'})){
           $args_func{'throttle'} = $throttle;
       }

        push @subtests_update,
          create_parallel_processes( 'sm_update', \%args_func );

    }
    #run_processes_bg( 'sm_update', \@subtests_update );
    my $interval_delay = 0;

    if ( $iterations && !$interval_time ) {
        # There seems to be a ProcManager bug: the first proc starts a second iteration
        #  even if 'iterations' = 1 and the PM never seems to figure out it finished
        while ($iterations-- > 0) {
            run_processes_bg( 'sm_update', \@subtests_update);
        }
    }
    elsif ( $iterations ) {
        run_processes_bg_iteration( 'sm_update', \@subtests_update, $iterations, $interval_time );
    }
    elsif ( !$cycles_per_day && !$interval_time ) {
        run_processes_bg_iteration( 'sm_update', \@subtests_update, 1, 0 );
    }
    elsif ( $cycles_per_day && !$interval_time ) {
        my $iterations     = $cycles_per_day;
        my $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_update',  \@subtests_update,
                          $iterations, $interval_delay );
    }
    elsif ( $interval_time && !$cycles_per_day ) {
        $iterations     = 100000000;
        $interval_delay = $interval_time;
        run_processes_bg_iteration( 'sm_update',  \@subtests_update,
                                $iterations, $interval_delay );
    }
    else {
        $log->warn(
            "Please provide either interval time or cycles per day not both. ");
        $log->comment("Using cycles_per_day a2s default parameter");
        $iterations     = $cycles_per_day;
        $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_update',  \@subtests_update,
                          $iterations, $interval_delay );

    }

}

=head2 snapmirror_abort

to perform abort operation on a tranfer

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item validate

(Optional) Do additional validation. Default 'false'.

=item interval_time

(Optional)  The time interval betweeen 2 successive runs.

=item cycles_per_day

(Optional)  The number of times the operation is executed per day at equal intervals

=item validate_transferring

(Optional)  Check to make sure the relationship is in Transferring state before issuing abort.  Value of 1 or 0.

=item sleep_before_abort

(Optional)  Number of seconds to sleep for before issuing the abort.  Default is 0 seconds (don't sleep).

=item ignore_no_transfer_to_abort

(Optional)  If set to true, catch the exception if snapmirror abort fails with "No transfer to abort" and continue (do not FAIL).  Default is false (FAIL when this failure is hit).

=item ignore_verify_transferring_err

(Optional)  If set to true, catch the exception if the verify_transferring method hits the exception where the relationship does not go to Transferring status and continue (do not FAIL).  Default is false (FAIL when this failure is hit).  Note that if this is set and validate_transferring=1, ignore_no_trasfer_to_abort should also be set as true since the verify_transferring method will not fail if it does not hit the transferring state, and the subsequent abort will fail because there is no active transfer.

=back

=back

=cut

sub snapmirror_abort {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src       => { type => OBJECT | SCALAR, optional => 1},
            node_dst       => { type => OBJECT | SCALAR, optional => 0 },
            sm_objs        => { type => ARRAYREF,        optional => 0 },
            validate       => { type => SCALAR,          default => 'false', regex => qr/^(true|false)$/ },
            iterations     => { type => SCALAR,          optional => 1 },
            interval_time  => { type => SCALAR,          optional => 1, default => 0 },
            cycles_per_day => { type => SCALAR,          optional => 1 },
            stagger_delay  => { type => SCALAR,          optional => 1 },
            validate_transfering => { type => SCALAR , optional => 1 },
            sleep_before_abort   => { type => SCALAR , optional => 1 },
            ignore_no_transfer_to_abort   => { type => SCALAR , optional => 1, default => 'false', regex => qr/^(true|false)$/ },
            ignore_verify_transferring_err   => { type => SCALAR , default => 'false', regex => qr/^(true|false)$/ },

        },
    );

    my $validate                       = $args{ 'validate' };
    my $node_src_Obj                   = $args{ 'node_src' };
    my $node_dst_Obj                   = $args{ 'node_dst' };
    my $aborts                         = $args{ 'sm_objs' };
    my @Aborts                         = _list_of_vals($aborts);
    my @subtests_abort;
    my $interval_time                  = $args{ 'interval_time' };
    my $cycles_per_day                 = $args{ 'cycles_per_day' };
    my $iterations                     = $args{ 'iterations' };
    my $validate_transfering           = $args{ 'validate_transfering' };
    my $sleep_before_abort             = $args{ 'sleep_before_abort' };
    my $ignore_no_transfer_to_abort    = $args{ 'ignore_no_transfer_to_abort' };
    my $ignore_verify_transferring_err = $args{ 'ignore_verify_transferring_err' };

    $counter = 0;
    foreach my $o (@Aborts) {
        my %args_func = (
            'node_src'                       => $node_src_Obj,
            'dst_path'                       => $o->destination_path,
            'src_path'                       => $o->source_path,
            'dst_volume'                     => $o->destination_volume(),
            'node_dst'                       => $node_dst_Obj,
            'dst_vserver'                    => $o->destination_vserver(),
            'validate'                       => $validate,
            'ignore_no_transfer_to_abort'    => $ignore_no_transfer_to_abort,
            'ignore_verify_transferring_err' => $ignore_verify_transferring_err,

        );

        if(exists($args{'validate_transfering'})){
			$args_func{'validate_transfering'} = $validate_transfering ;
		}
        if(exists($args{'sleep_before_abort'})){
			$args_func{'sleep_before_abort'} = $sleep_before_abort ;
		}

        push @subtests_abort,
          create_parallel_processes( 'sm_abort', \%args_func );

    }
    my $interval_delay = 0;

    if ( $iterations && !$interval_time ) {
# There seems to be a ProcManager bug: the first proc starts a second iteration
#  even if 'iterations' = 1 and the PM never seems to figure out it finished
        while ($iterations-- > 0) {
            run_processes_bg( 'sm_abort', \@subtests_abort);
        }
    }
    elsif ( $iterations ) {
        run_processes_bg_iteration( 'sm_abort', \@subtests_abort, $iterations, $interval_time );
    }
    elsif ( !$cycles_per_day && !$interval_time ) {
        run_processes_bg_iteration( 'sm_abort', \@subtests_abort, 1, 0 );
    }
    elsif ( $cycles_per_day && !$interval_time ) {
        $iterations     = $cycles_per_day;
        $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_abort',  \@subtests_abort,
                          $iterations, $interval_delay );
    }
    elsif ( $interval_time && !$cycles_per_day ) {
        $iterations     = 100000000;
        $interval_delay = $interval_time;
        run_processes_bg_iteration( 'sm_abort',  \@subtests_abort,
                                $iterations, $interval_delay );
    }
    else {
        $log->warn(
            "Please provide either interval time or cycles per day not both. ");
        $log->comment("Using cycles_per_day as default parameter");
        $iterations     = $cycles_per_day;
        $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_abort',  \@subtests_abort,
                          $iterations, $interval_delay );

    }

}

=head2 snapmirror_abort_hard

to perform hard abort on a tranfer

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item validate

(Optional) Do additional validation. Default 'false'.

=item interval_time

(Optional)  The time interval betweeen 2 successive runs.

=item cycles_per_day

(Optional)  The number of times the operation is executed per day at equal intervals


=back

=back

=cut

sub snapmirror_abort_hard {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src       => { type => OBJECT | SCALAR, optional => 1 },
            node_dst       => { type => OBJECT | SCALAR, optional => 0 },
            sm_objs        => { type => ARRAYREF,        optional => 0 },
            validate       => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
            interval_time  => { type => SCALAR,          optional => 1 },
            cycles_per_day => { type => SCALAR,          optional => 1 },
            stagger_delay  => { type => SCALAR,          optional => 1 },

        }
    );

    my $validate     = $args{ 'validate' };
    my $node_src_Obj = $args{ 'node_src' };
    my $node_dst_Obj = $args{ 'node_dst' };
    my $abort_hards  = $args{ 'sm_objs' };
    my @abort_hards  = _list_of_vals($abort_hards);
    my @subtests_abort_hard;
    my $interval_time  = $args{ 'interval_time' };
    my $cycles_per_day = $args{ 'cycles_per_day' };

    $counter = 0;
    foreach my $o (@abort_hards) {
        my %args_func = (
            'node_src'    => $node_src_Obj,
            'dst_path'    => $o->destination_path,
            'src_path'    => $o->source_path,
            'dst_volume'  => $o->destination_volume(),
            'node_dst'    => $node_dst_Obj,
            'dst_vserver' => $o->destination_vserver(),
            'validate'    => $validate,

        );

        push @subtests_abort_hard,
          create_parallel_processes( 'sm_abort_hard', \%args_func );

    }

    my $iterations     = 0;
    my $interval_delay = 0;

    if ( !$cycles_per_day && !$interval_time ) {
        run_processes_bg_iteration( 'sm_abort_hard', \@subtests_abort_hard, 1, 0 );
    }
    elsif ( $cycles_per_day && !$interval_time ) {
        $iterations     = $cycles_per_day;
        $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_abort_hard', \@subtests_abort_hard,
                                $iterations,     $interval_delay );
    }
    elsif ( $interval_time && !$cycles_per_day ) {
        $iterations     = 100000000;
        $interval_delay = $interval_time;
        run_processes_bg_iteration( 'sm_abort_hard', \@subtests_abort_hard,
                                $iterations,     $interval_delay );
    }
    else {
        $log->warn(
            "Please provide either interval time or cycles per day not both. ");
        $log->comment("Using cycles_per_day a2s default parameter");
        $iterations     = $cycles_per_day;
        $interval_delay = 86400 / $cycles_per_day;
        run_processes_bg_iteration( 'sm_abort_hard', \@subtests_abort_hard,
                                $iterations,     $interval_delay );

    }

}

=head2 snapmirror_quiesce

Provides the ability to quiesce any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are quiesced.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are quiesced

If Validate is true.Its going to Validate these:
Validation1: Relationship status should be quiesced afterwards.
Validation2(TODO): In case of Sync mirror - Pseudo snapshot creation should be triggered.
Validation3: Validate the OutOfSync(OOS) status.

=back

=back

=cut

sub snapmirror_quiesce{
    my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src           => {type => OBJECT| SCALAR, optional=>1},
                node_dst           => {type => OBJECT| SCALAR, optional=>0},
                sm_objs            => {type => ARRAYREF, optional => 0},
                validate           => {type => SCALAR , optional => 1, default=>'false'},
                },
                );

        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $sm_obj              = $args{'sm_objs'};
        my @Quiesce_objs        = _list_of_vals($sm_obj);
        my $validate            = $args{'validate'};

        my @subtests_quiesce;
        $counter = 0;
        foreach my $o(@Quiesce_objs) {
            my %args_func = ('node_src'    => $Src_Node_Obj,
                    'node_dst'    => $Dst_Node_Obj,
                    'dst_path'    => $o->destination_path,
                    'src_path'         => $o->source_path,
                    'validate'    => $validate,
                    'stagger_delay'    => 10,
                    );
            push @subtests_quiesce, create_parallel_processes('sm_quiesce',\%args_func);
        }

        run_processes_bg('sm_quiesce',\@subtests_quiesce);
}

=head2 snapmirror_resume

Provides the ability to resume any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are quiesced.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are resumed

If Validate is true.Its going to Validate these:
Validation1: Relationship status should not be quiesced afterwards.
Validation2: In case of Sync mirror - Validate the Insync status.

=back

=back

=cut

sub snapmirror_resume{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>0},
            sm_objs            => {type => ARRAYREF, optional => 0},
            validate           => {type => SCALAR , optional => 1, default=>'false'},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $sm_obj              = $args{'sm_objs'};
    my @resume_objs         = _list_of_vals($sm_obj);
    my $validate            =  $args{'validate'};

    my @subtests_resume;
    $counter = 0;
    foreach my $o(@resume_objs) {
        my %args_func = (
                'node_src'         => $Src_Node_Obj,
                'node_dst'         => $Dst_Node_Obj,
                'dst_path'         => $o->destination_path,
                'src_path'         => $o->source_path,
                'validate'         => $validate,
                'stagger_delay'    => 10,
                );
        push @subtests_resume, create_parallel_processes('sm_resume',\%args_func);
    }

    run_processes_bg('sm_resume',\@subtests_resume);
}

=head2 snapmirror_initialize_many_rels

Provides the ability to initialize any no. of relationships.

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are resumed

If Validate is true.  Its going to Validate these:
Validation1: state=snapmirrored, healthy=true, last-tranfer-type =initialize via snapmirror-show
Validation2: Snapmirror-snapshot gets cut on the destination after initialize.

=item snapshot

(Optional) string - name of snapshot for which initialize will be sent with 
snapmirror initialize -s.  All initializes will use the same snapshot, so the
same snapshot needs to be created on all of the source volumes prior to callng
this method.

=item timeout

(Optional):  method-timeout for snapmirror initailize.  If not set will default to 1200 sec.

=item max_active_children

(Optional):  To control how many sub processes will start in parallel.  If not set will default to 16 simulatenous snapmirror initializes.

=item nacltask_wait

(Optional) If set, will wait for initialize to finish. Default: 1.

=back

=back

=cut

sub snapmirror_initialize_many_rels{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT| SCALAR, optional => 1},
            node_dst            => {type => OBJECT| SCALAR, optional => 0},
            sm_objs             => {type => ARRAYREF, optional => 0},
            validate            => {type => SCALAR , optional => 1, default => 'false'},
            snapshot            => {type => SCALAR , optional => 1},
            timeout             => {type => SCALAR , optional => 1, default => 1200},
	    max_active_children => {type => SCALAR, optional =>1, default => 16},
            nacltask_wait       => {type => SCALAR, optional => 1, default => 1},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $sm_obj              = $args{'sm_objs'};
    my @init_objs           = _list_of_vals($sm_obj);
    my $validate            =  $args{'validate'};
    my $snapshot            =  $args{'snapshot'};
    my $timeout             =  $args{'timeout'};
    my $max_active_children =  $args{'max_active_children'};
    my $nacltask_wait       =  $args{'nacltask_wait'};

    my @subtests_init;
    $counter = 0;
    foreach my $o(@init_objs) {
        my %args_func = (
                'node_src'         => $Src_Node_Obj,
                'node_dst'         => $Dst_Node_Obj,
                'dst_path'         => $o->destination_path,
                'src_path'         => $o->source_path,
                'validate'         => $validate,
                'timeout'          => $timeout,
                'nacltask_wait'     => $nacltask_wait,
                'stagger_delay'    => 10,
                );
        if($snapshot) {
            $args_func{'snapshot'} = $snapshot;
        }
        push @subtests_init, create_parallel_processes('_sm_init',\%args_func);
    }

    run_processes_bg('_sm_init',\@subtests_init, $max_active_children);
}

=head2 snapmirror_modify

Provides the ability to modify any no. of relationships.

=over

=item Options

=over

=item node_src

(Optional) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item modify_options

(Required) Hashref of snapmirror relationship options to modify (like schedule or throttle).  The same options will be modified for all relationships.

=back

=cut

sub snapmirror_modify{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>0},
            sm_objs            => {type => ARRAYREF, optional => 0},
            modify_options     => {type => HASHREF, optional => 0},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $sm_obj              = $args{'sm_objs'};
    my $modify_options      = $args{'modify_options'};
    my @modify_objs         = _list_of_vals($sm_obj);

    my @subtests_modify;
    $counter = 0;
    foreach my $o(@modify_objs) {
        my %args_func = (
                'node_src'         => $Src_Node_Obj,
                'node_dst'         => $Dst_Node_Obj,
                'dst_path'         => $o->destination_path,
                'src_path'         => $o->source_path,
                'modify_options'   => $modify_options,
                'stagger_delay'    => 10,
                );
        push @subtests_modify, create_parallel_processes('_sm_modify',\%args_func);
    }

    run_processes_bg('_sm_modify',\@subtests_modify);
}

=head2 snapmirror_restore

to perform hard abort on a tranfer

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item snapshot

(Optional)  snapshot name for restore with a particular snapshot command.

=item validate

(Optional)  Value 0 or 1 . 1 to validate . By default 0.

=back

=back

=cut


sub snapmirror_restore {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src      => { type => OBJECT | SCALAR, optional => 0 },
            node_dst      => { type => OBJECT | SCALAR, optional => 0 },
            sm_objs       => { type => ARRAYREF,        optional => 0 },
            snapshot      => { type => SCALAR, optional => 1},
            validate      => { type => SCALAR,          optional => 1, default => 0 },
            stagger_delay => { type => SCALAR,          optional => 1 },

        },
    );

    my $validate     = $args{ 'validate' };
    my $node_src_Obj = $args{ 'node_src' };
    my $node_dst_Obj = $args{ 'node_dst' };
    my $restores     = $args{ 'sm_objs' };
    my $snapshot = $args{'snapshot'};
    my @Restores     = _list_of_vals($restores);
    my @subtests_restore;

    $counter = 0;
    foreach my $o (@Restores) {
        my %args_func = (
            'node_src'    => $node_src_Obj,
            'dst_path'    => $o->destination_path,
            'src_path'    => $o->source_path,
            'dst_volume'  => $o->destination_volume(),
            'node_dst'    => $node_dst_Obj,
            'dst_vserver' => $o->destination_vserver(),
            'validate'    => $validate

        );

        if(exists($args{'snapshot'})){
           $args_func{'snapshot'} = $snapshot;
       }
        push @subtests_restore,
          create_parallel_processes( 'sm_restore', \%args_func );

    }
    run_processes_bg( 'sm_restore', \@subtests_restore );

}



=head2 snapmirror_break

Provides the ability to break any no of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are broken-off.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are broken.

If Validate is true.Its going to Validate these:

Validation1: Destination Volume became RW.
Validation2: relationship state "broken off".
Validation3(TODO): In case of sync mirror - cg-items become RW(waiting on Randy;s code for this).
Validation4:Validate the OOS status of relationship.

=back

=back

=cut

sub snapmirror_break {
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT| SCALAR, optional=>1},
            node_dst           => {type => OBJECT| SCALAR, optional=>0},
            sm_objs            => {type => ARRAYREF, optional => 0},
            validate          => {type =>  SCALAR, optional =>1,default=>'false'},
            },
            );

    my $Src_Node_Obj        = $args{'node_src'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $sm_obj              = $args{'sm_objs'};
    my $validate            = $args{'validate'};
    my @Break_objs          = _list_of_vals($sm_obj);
    my @subtests_break;
    $counter = 0;
    foreach my $o(@Break_objs) {
        my %args_func = (
                'node_src'         => $Src_Node_Obj,
                'node_dst'         => $Dst_Node_Obj,
                'dst_path'         => $o->destination_path,
                'src_path'         => $o->source_path,
                'validate'         => $validate,
                'stagger_delay'    => 10,
                );
        push @subtests_break, create_parallel_processes('sm_break',\%args_func);
    }

    run_processes_bg('sm_break',\@subtests_break);
}

=head2 snapmirror_resync

Provides the ability to resync any no. of relationships (ADR/Vault/Sync).

Verifies that the n no of relationships are resynced.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are resynced

If Validate is true.Its going to Validate these:
Validation1: status =Insync, state=snapmirroed, healthy=true, last-tranfer-type =resync
Validation2 : In case of sync validate the "Insync" status via snapmirror-list-destinations

=back

=back

=cut

sub snapmirror_resync{
        my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src           => {type => OBJECT| SCALAR, optional=>1},
                node_dst           => {type => OBJECT| SCALAR, optional=>0},
                sm_objs            => {type => ARRAYREF, optional => 0},
                validate           => {type => SCALAR, optional =>1,default=>'false'},
                },
                );

        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $sm_obj              = $args{'sm_objs'};
        my $validate            = $args{'validate'};
        my @Resync_objs          = _list_of_vals($sm_obj);

        my @subtests_resync;
        $counter = 0;
        foreach my $o(@Resync_objs) {
            my %args_func = (
                    'node_src'         => $Src_Node_Obj,
                    'node_dst'         => $Dst_Node_Obj,
                    'dst_path'         => $o->destination_path,
                    'src_path'         => $o->source_path,
                    'validate'         => $validate,
                    'stagger_delay'    => 10,
                    );
            push @subtests_resync, create_parallel_processes('sm_resync',\%args_func);
        }

        run_processes_bg('sm_resync',\@subtests_resync);
}

=head2 snapmirror_delete

Provides the ability to delete any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are deleted.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are deleted

If Validate is true.Its going to Validate these:
Validation1: No relatinship left on destination side.
Validation2(TODO): In case of sync mirror - cg-items become RW(implicit break-waiting on Randy;s code for this).

=back

=back

=cut

sub snapmirror_delete{
        my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src           => {type => OBJECT| SCALAR, optional=>0},
                node_dst           => {type => OBJECT| SCALAR, optional=>0},
                sm_objs            => {type => ARRAYREF, optional => 0},
                validate           => {type => SCALAR , optional => 1, default=>'false'},
                },
                );

        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $sm_obj              = $args{'sm_objs'};
        my $validate            = $args{'validate'};
        my @Delete_objs          = _list_of_vals($sm_obj);

        my @subtests_delete;
        foreach my $o(@Delete_objs) {
            my %args_func = (
                    'node_src'         => $Src_Node_Obj,
                    'node_dst'         => $Dst_Node_Obj,
                    'dst_path'         => $o->destination_path,
                    'src_path'         => $o->source_path,
                    'validate'         => $validate,
                    'stagger_delay'    => 10,
                    );
            push @subtests_delete, create_parallel_processes('sm_delete',\%args_func);
        }

        run_processes_bg('sm_delete',\@subtests_delete);

    }

=head2 snapmirror_release

Provides the ability to release any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are released.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) string(true/false) to do validations after relationship(s) is/are released

If Validate is true.Its going to Validate these:
Validation1: No relatinship left on source side.

=back

=back

=cut

sub snapmirror_release {
        my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src           => {type => OBJECT| SCALAR, optional=>0},
                node_dst           => {type => OBJECT| SCALAR, optional=>0},
                sm_objs            => {type => ARRAYREF, optional => 0},
                validate           => {type => SCALAR , optional => 1, default=>'false'},
                },
                );

        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $sm_obj              = $args{'sm_objs'};
        my $validate            = $args{'validate'};
        my @Release_objs          = _list_of_vals($sm_obj);

        my @subtests_release;
        $counter = 0;
        foreach my $o(@Release_objs) {
            my %args_func = (
                    'node_src'         => $Src_Node_Obj,
                    'node_dst'         => $Dst_Node_Obj,
                    'src_path'         => $o->source_path,
                    'validate'         => $validate,
                    'stagger_delay'    => 10,
                    );
            push @subtests_release, create_parallel_processes('sm_release',\%args_func);
        }

        run_processes_bg('sm_release',\@subtests_release);
}


=head2 Wf_sm_checker

A wrapper to run snapmirror checker on a vault/ADR/LRSEDR/sync relationship with ACS or PCS option

=over

=item Options

=over

=item src_node

(Required) NACL Node Object - Source side NACL Node Object

=item dst_node

(Required) NACL Node Object - Destination side NACL Node Object

=item ss_type

(Optional) To specify if you want to run the checker using an ACS or a PCS.
 Note: This is a required param for smgr relationship.Optional for ADR/Vault/LRSEDR
 Note: The method will not cut a pcs for you if you chose ss_type => 'pcs' It
 will expect that pcs already exists. If not it will throw an exception.values can be
 'acs' or 'pcs'.

=item ss_name

 (Optional) The SS name to be created on which checker will  be run.Default value is 'checker01'.

=item sm_objs

(Required) The Snapmirror objects.

=item resourceobjs

(Optional) The list of file objects or lun objects to use.
 Note: This is a required param for sync reltaionship

=item quiesce_resume_validations

(Optional) boolean value (true/false) to allow for skipping validations for quiesce and resume operations done as part of sm_check.  Default is true.

=item Example

    For smgr: Wf_sm_checker(
                 src_node          => $src_node,
                 dst_node          => $dst_node,
                 ss_type           => 'acs',
                 ss_name           => 'checker01',
                 sm_objs           => $snapmirror,
                 resourceobjs      => ['lun_obj1', 'lun_obj2'] or ['file_obj1', 'file_obj2'],
                 cg_type           => 'lun'
               );

    For Vault/ADR/LRSEDR:  Wf_sm_checker(
                              src_node          => $src_node,
                              dst_node          => $dst_node,
                              sm_objs           => $snapmirror,
                           );


=back

=back

=item exceptions

    NATE::BaseException

=cut

sub  Wf_sm_checker {

    my %args = validate_with (
        params      => \@_,
        allow_extra => 1,
        spec        => {
            src_node      => { type => OBJECT|SCALAR,  },
            dst_node      => { type => OBJECT|SCALAR,  },
            ss_type       => { type => SCALAR, regex => qr/acs|pcs/, optional=>1, default=>'acs' },
            sm_objs        => { type => ARRAYREF },
            resourceobjs => { type => ARRAYREF, optional =>1 }, #can be lun objs or file objs
            cg_type       => { type => SCALAR, optional=>1, default=>'lun'},
            ss_name       => { type => SCALAR, optional => 1, default => 'undef' },
            timeout       =>  { type => SCALAR, optional => 1, default => 1800 },
            quiesce_resume_validation => { type => SCALAR, optional =>1, defauult => 'true' }, #can be lun objs or file objs
        },);

    my $Src_Node_Obj = $args{src_node};
    my $Dst_Node_Obj = $args{dst_node};
    my $ss_type = $args{ss_type};
    my $resourcesnames = $args{resourceobjs};
    my $sm_obj         = delete $args{'sm_objs'};
    my @check_objs     = _list_of_vals($sm_obj);
    my @resourcesnames     = _list_of_vals($resourcesnames);
    my $cg_type           = $args{cg_type};
    my $ssname            = $args{ss_name};
    my $timeout           = $args{timeout};



    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    my $name;
    my $path;
    my @src_item_names;
    my $src_path;
    my $dst_path;

    if(defined $args{resourcenames}){
        foreach my $item(@resourcesnames){
            if ($cg_type eq 'lun') {
                $name = $item->lun();
            }else {
                $path = $item->path();
                ($name = $path) =~ s{.*/}{};
            }
            push(@src_item_names,$name);
         }
   }
   my @subtests_check;
   foreach my $o(@check_objs) {
       my $source_path = $o->source_path;
       my $destination_path = $o->destination_path;
       my ($src_vserver_name,$src_vol_name,$dst_vol_name,$dst_vserver_name);

       if(defined $args{resourcenames}){
          ($src_path, $dst_path) = $o->volume_paths_from_cg;
          ($src_vserver_name,$src_vol_name) = ($src_path =~ /(.*):\/vol\/(.*)\//);
          ($dst_vserver_name,$dst_vol_name) = ($dst_path =~ /(.*):\/vol\/(.*)\//);
          $log->comment('Got source vserver:'.$src_vserver_name.' src vol:'.$src_vol_name.
                     ' dest vserver:'.$dst_vserver_name.' dest vol:'.$dst_vol_name);

           my $Src_obj = NACL::C::Volume->find(
               command_interface => $Src_Node_Obj,
               filter            => {
                  volume  => $src_vol_name,
                  vserver => $src_vserver_name,
               },
            );

           my $Dst_obj = NACL::C::Volume->find(
                command_interface => $Dst_Node_Obj,
                filter            => {
                   volume  => $dst_vol_name,
                   vserver => $dst_vserver_name,
                },
            );

          my %args_func = ('src_node'    => $Src_Node_Obj,
                           'dst_node'    => $Dst_Node_Obj,
                           'src_vol'     => $Src_obj,
                           'dst_vol'     => $Dst_obj,
                           'ss_type'     => $ss_type,
                           'sm_obj'      => $o,
                           'resourcenames' => \@src_item_names,
                           'stagger_delay'    => 10,
                           );
           push @subtests_check, create_parallel_processes('run_sm_checker',\%args_func);
       }else{
          my %args_func = (
                           'node_src'        => $Src_Node_Obj,
                            'node_dst'        => $Dst_Node_Obj,
                            'dst_path'        => $destination_path,
                            'src_path'        => $source_path,
                            'stagger_delay'   => 10,
                             'timeout'         => $timeout,
                            %args, #to pass the extra params
                           );
           push @subtests_check, create_parallel_processes('sm_check',\%args_func);

       }

   }
   if($args{resourcenames}){
       run_processes_bg('run_sm_checker',\@subtests_check);
   }else{
       run_processes_bg('sm_checker',\@subtests_check);
   }


}

=head2 run_sm_checker

A wrapper to run snapmirror checker on a PCS SS or an ACS SS

=over

=item Options

=over

=item src_node

(Required) NACL Node Object - Source side NACL Node Object

=item dst_vol

(Required) NACL Node Object - Destination side NACL Node Object

=item src_vol

(Required) NACL Volume Object - Source side NACL Volume Object

=item dst_vol

(Required) NACL Volume Object - Destination side NACL Volume Object

=item ss_type

(Required) To specify if you want to run the checker using an ACS or a PCS.
 Note: The method will not cut a pcs for you if you chose ss_type => 'pcs' It
 will expect that pcs already exists. If not it will throw an exception.

=item ss_name

 (Optional) The SS name to be created on which checker will  be run.

=item sm_objs

(Required) The Snapmirror objects.

=item resourcenames

(Required) The list of file names or lun names to use

=item Example

    run_sm_checker(
        src_node          => $src_node,
        dst_node          => $dst_node,
        src_vol           => $src_vol,
        node_dst          => $dst_vol,
        ss_type           => 'acs',
        ss_name           => 'checker01',
        sm_objs           => $snapmirror,
        resourcenames     => ['lun_name1', 'lun_name2'] or ['file_name1', 'file_name2'],

    );

=back

=item Exceptions

    NATE::BaseException

=back

=cut

sub run_sm_checker {

    my %args = validate_with (
        params      => \@_,
        allow_extra => 1,
        spec        => {
            src_node      => { type => OBJECT,  },
            dst_node      => { type => OBJECT,  },
            src_vol       => { type => OBJECT,  },
            dst_vol       => { type => OBJECT,  },
            ss_type       => { type => SCALAR, regex => qr/acs|pcs/ },
            ss_name       => { type => SCALAR, optional => 1 },
            sm_obj        => { type => OBJECT }, #TODO: Taking a single object. Probably add a wrapper which runs checker on multiple
            resourcenames => { type => ARRAYREF }, #can be lun names or file names
            stagger_delay      => {type => SCALAR, optional => 1},
        },);

        my $src_vol = $args{src_vol};
        my $dst_vol = $args{dst_vol};
        my $src_cmd_interface = $args{src_node};
        my $dst_cmd_interface = $args{dst_node};
        my $ss_type = $args{ss_type};
        my $pcs_ss  = $args{pcs_ss};
        my $sm =  $args{sm_obj};
        my $resourcesnames = $args{resourcenames};
         _stagger($args{'stagger_delay'});

        #create the checker items
        my @checkeritems;
        foreach my $item (@{ $resourcesnames }){
           push (@checkeritems,  "/$item");
           push (@checkeritems, "\@/$item");
        }
        $log->comment('The checker items are: ' . Dumper(\@checkeritems));

        my $ss_name = $args{ss_name};
        my %ss_check_args;

        if (defined $ss_type && $ss_type =~ /acs/) {
            $log->comment('Checker will be run on an ACS');

            if( !defined $ss_name) {
                $ss_name = 'checker' . $$;
            }

            $log->comment('Take ACS SS prefixed: ' . $ss_name .
                          ' to be used in checker');
	    my $n = 0;
	    while ($n < 3) {
		$n ++;
		try {
		    Wf_sync_app_cons_creation(
				    node_src => $src_cmd_interface,
				    node_dst => $dst_cmd_interface,
				    sm_objs  => [$sm],
				    snapshot_count => 1,
				    snapshot_name => $ss_name,
				    validate => 'true',);
		    $n = 0;
		    last;
		} catch NATE::BaseException with {
		    $log->log('...attempt to delete partially created ACS...');
		    my $acs_snap = NACL::STask::VolumeSnapshot->find(
			command_interface  => $src_cmd_interface,
			volume             => $src_vol->volume(),
			vserver            => $src_vol->vserver(),
			snapshot           => $ss_name . '0',
			allow_empty	   => 1,
		    );
		    $acs_snap->purge() if $acs_snap;

		    $log->log('...retry ACS create in 180s...');
		    Tharn::snooze(180) if ($n < 3);
		};
	    }
	    die "ACS create failed." if ($n);

            $src_cmd_interface->refresh_command_interface();
            $dst_cmd_interface->refresh_command_interface();

            $ss_name = $ss_name . '0';  # snapshot name supplied to creation call appends a 0
            $log->comment('ACS SS that will be used by checker is: ' . $ss_name);
         }

         if (defined $ss_type && $ss_type =~ /pcs/) {

            $log->comment('Get latest PCS SS on the src');
            my $src_ss = get_latest_smgr_snapshot (
                            node    => $src_cmd_interface,
                            volume  => $src_vol->volume() );

            $log->comment('Validate PCS tags for the SS obtained on the src');

            my $tag_check = Validate_PCS_Tags(volname => $src_vol->volume(),
                                                 node => $src_cmd_interface,
                                             snapshot => $src_ss);
            if ($tag_check == 0) {
                my $error_text = 'PCS tags check failed on : ' . $src_ss .
                                 '. Failed to retrieve a PCS SS on the src to run checker.';
                $log->error($error_text); #just log an error here we will throw an exception further down.
                $src_ss = undef;

            } else {
                $log->comment('Found PCS SS on the src: ' . $src_ss);
            }

            $log->comment('Get latest PCS SS on the dst');
            my $dst_ss = get_latest_smgr_snapshot (
                             node    => $dst_cmd_interface,
                             volume  => $dst_vol->volume());

            $log->comment('Validate PCS tags for the SS obtained on the dst');

            $tag_check = Validate_PCS_Tags(
                volname  => $dst_vol->volume(),
                node     => $dst_cmd_interface,
                snapshot => $dst_ss
            );

            if ($tag_check == 0) {
                my $error_text = 'PCS tags check failed on : ' . $dst_ss .
                                 '. Failed to retrieve a PCS SS on the dst to run checker.';
                $log->error($error_text); #just log an error here we will throw an exception further down.
                $dst_ss = undef;
            } else {
                $log->comment('Found PCS SS on the dst: ' . $dst_ss);
            }

            #make sure that the SS on the src and dst match.
            #if they do pick one (either on src or dst)

            if (!defined $src_ss ||
                !defined $dst_ss) {
                NATE::BaseException->throw('No PCS SS found to run checker on');
            }
            if ($src_ss eq $dst_ss) {
                $ss_name = $src_ss;
            }

            $log->comment('PCS SS that will be used by checker is: ' . $ss_name);
            $ss_check_args{'ift-check'} = 'true';
         }

         $log->comment('Finally doing a SS check');
         snapmirror_check (
                    node_src => $src_cmd_interface,
                    node_dst => $dst_cmd_interface,
                    ss_name_to_sm_obj => { $ss_name => $sm },
                    file_list         => { $ss_name => \@checkeritems },
                    %ss_check_args);

        $log->comment('.... success');
        return;
}

=head2 check_for_ift_log

Method to check for the presence or absence of IFT log on the source volume

=over

=item Options

=over

=item node_src
 (Required) NACL Node Object - Source side NACL Node Object

=item tries_count
 (Optional) The number of times to loop for while waiting ofr ht IFT log

=item sleep_time
 (Optional) The number of seconds to sleep between loops

=item vol_src
 (Required) NACL Volume Object - Source side NACL Volume Object

=item is_present
 (Optional) Set it to 1 if you want to  check the presence of the IFT well.
 Or set it to 0 to check the absence. By default it is 0.

=item fail_on_error
 (Optional) Fail when the result is not as expected.
 (Default)  1

=back

=back

=cut

sub check_for_ift_log {

    my %args = validate_with(
            params => \@_,
            spec   => {
                node_src    => {type => OBJECT, },
                tries_count => {type => SCALAR, default => 12},
                sleep_time  => {type => SCALAR, default => 10},
                vol_src     => {type => OBJECT, },
                is_present  => {type => SCALAR, default => 0, regex => qr/0|1/},
              fail_on_error => { type => SCALAR, default => 1 , regex => qr/0|1/ },
            }
        );

    my $src_vol        = $args{vol_src};
    my $is_present     = $args{is_present};
    my $tries          = $args{tries_count};
    my $sleep          = $args{sleep_time};
    my $node_src       = $args{node_src};
    my $fail_on_error  = $args{fail_on_error};

    my $ift_file_path = $src_vol->volume() . './metadir/granular_sync/ift_logs';
    my $cmd = "vol explore -format dir $ift_file_path";
    my $api = $node_src->apiset();

    my $IFT_found;
    my $expected_IFT_found_value;
    my $res;

    if ($is_present) {
        $expected_IFT_found_value = 1; #should be 1 if user expects the IFT
        $IFT_found = 0; #lets start the counter with 0 which means we did not find the IFT yet
        $log->comment('The user wants us to ensure that we can catch the IFT');
        $res = ' Expect to find the IFT ';
    }  else {
        $expected_IFT_found_value = 0; #should be 0 if user does not expect the IFT
        $IFT_found = 1; #lets start the counter with 1 which means we have the IFT
        $log->comment('The user wants us to ensure that we do not see the IFT');
        $res = ' Dont expect the IFT ';
    }
    my $cur_count = 0;
    my $ret_val;
    my $IFT_inum_val;

    while (($IFT_found !=  $expected_IFT_found_value) && ($cur_count <= $tries)) {
        $log->comment ('Running command: ' . $cmd .
                       ' on node: ' . $node_src->name() .
                       ' for volume: ' . $src_vol->volume() .
                       '. Count #: ' . $cur_count);
        $ret_val = $api->execute_command (command => $cmd);
        $log->comment('Value returned from vol explore is: ' . Dumper($ret_val));

        ## The IF block ##
        # If we are expecting the IFT and we find it via the pattern match. Then we
        # the IFT_found = 1 and exit the while loop.

        # If we are NOT expecting the IFT but we will find it then we sleep for a few
        # seconds and try again.


        ## Else block ##
        # If we don't find the IFT when we are expecting to, then we will sleep for a few
        # seconds and try again.

        # If we dont find the IFT and that is what we are expecting then we will set the
        # IFT_fount = 0 and exit the while loop

        if ($ret_val =~ /.*\sinum (\d+P).*\"ift\"\s\(8.3 \"IFT\"\)/) {
            if ($expected_IFT_found_value)  {
                $IFT_inum_val = $1;
                $log->comment('Found the IFT logs in AFS with inum: ' . $IFT_inum_val);
                $IFT_found = 1;
            } elsif (!$expected_IFT_found_value) {
                $log->comment('IFT log still exists even though the user expects' .
                              'it shouldnt. Lets try again after ' . $sleep . ' seconds');
                Tharn::snooze($sleep);
            }
        } else {
            if ($expected_IFT_found_value) {
                #if user expects the IFT lets sleep for a bit and try again
                $log->comment("IFT logs not created yet retry after $sleep seconds");
                Tharn::snooze($sleep);
            } elsif (!$expected_IFT_found_value) {
                $log->comment('IFT logs not found as expected.');
                $IFT_found = 0;
            }
        }
        $cur_count++;
    }

    my $err_msg = "Failed to get the expected result: $res" .
                  " after trying for $tries times with " .
                  "a sleep of $sleep seconds";

    if (($IFT_found != $expected_IFT_found_value) && $fail_on_error) {
        NATE::BaseException->throw ($err_msg);
    } elsif ( ($IFT_found != $expected_IFT_found_value) && !$fail_on_error) {
        $log->error($err_msg);
    } else  {
        $log->comment('Found the result we were looking for');
    }

    return ($IFT_found, $IFT_inum_val);
}

=head2 snapmirror_check

Provides the ability to run snapmirror check on any no. of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are consistent.

=over

=item Options

=over

=item node_src

(Required) NACL Node Object - Source side NACL Node Object

=item node_dst

(Required) NACL Node Object - Destination side NACL Node Object

=item ss_name_to_sm_obj

(Required) Hash reference - Keys are snapshot names as scalars. Values are
corresponding NACL STask Snapmirror objects. The snapshot must exist in the
snapmirror relationship.

=item file_list

(Optional) Hash reference - Keys are snapshot names as scalars. Values are
corresponding array references of the file names in the special format needed
by snapmirror check. The special format is one or more pairs of filenames that
are the names of the files on the source and destination sides.

Example: ['/filename01','@/filename01']

The first item in the list is the name of the file on the source side
preceded by a '/'. The destination filename is the same name, but preceded
by a '@/' instead.

=item Example

    snapmirror_check(
        node_src          => $node_src_a,
        node_dst          => $node_dst_b,
        ss_name_to_sm_obj => { $snapshot_name => $snapmirror },
        file_list         => { $snapshot_name => [ '/file01', '@/file01' ] },
    );

=back

=back

=cut

sub snapmirror_check {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src          => { type => OBJECT,  optional => 0 },
            node_dst          => { type => OBJECT,  optional => 0 },
            ss_name_to_sm_obj => { type => HASHREF, optional => 0 },
            file_list         => { type => HASHREF, optional => 0 },
            'ift-check'       => { type => SCALAR,  optional => 1 },
            timeout           => { type => SCALAR,  optional => 1 , default => 1800 },

        },
    );

    my $node_src  = $args{'node_src'};
    my $node_dst  = $args{'node_dst'};
    my $map       = $args{'ss_name_to_sm_obj'};
    my $file_list = $args{'file_list'};
    my $iftcheck  = $args{'ift-check'};
    my $timeout   = $args{'timeout'};

    my @subtests_check;

    while ( my ( $snapshot_name, $sm_obj ) = each %{$map} ) {

        my $file_list_arr = $file_list->{$snapshot_name};

        my %args_func = (
            'node_src'        => $node_src,
            'node_dst'        => $node_dst,
            'dst_path'        => $sm_obj->destination_path,
            'src_path'        => $sm_obj->source_path,
            'file_list'       => $file_list_arr,
            'source_snapshot' => $snapshot_name,
            'target_snapshot' => $snapshot_name,
             'timeout'         => $timeout,
        );

        if (defined $iftcheck) {
            $args_func{'ift-check'} = $iftcheck;
        }

        $log->comment( 'snapmirror_check args: ' . Dumper( %args_func ) );
        push @subtests_check,
            create_parallel_processes( 'sm_check', \%args_func );
    }

    run_processes_bg( 'sm_check', \@subtests_check );

    return 1;
}


# CANNED Workflows


=head2 Wf_basic_snapmirror_setup

This is a 'CANNED' workflow.
This will create any no of 1-1 relationships(intialized) with any relationship type(adr/vault/sync) given.
Provides the ability to initialize any no. 1to1 of relationships (ADR/Vault/Sync).
Verifies that the n no of relationships are initialized.

=over

=item Example

This example will create and initialize 2, 1-1 sync relationship,
first one vol1->vol_dst1 contain luns and
second one vol2->vol_dst2 contains files.


    %cg_item_list_hash = ('vol1' => ['lun1','lun2'],
                          'vol2' => ['file1','file2']);,

    %src_cg_name  =     ('vol1'     => '/cg/Src_Cg_Name1',
                         'vol2'     => '/cg/Src_Cg_Name2');
    %dst_cg_name  =     ('vol_dst1' => '/cg/Dst_Cg_Name1',
                         'vol_dst2' => '/cg/Dst_Cg_Name2');

    Wf_basic_snapmirror_setup('node_src'                => $src_node_name,
                      'node_dst'                =>  $dst_node_name,
                      'src_volumes'             => [$vol1,$vol2],
                      'dst_volumes'             => [$vol_dst1,$vol_dst2],
                      'cg_item_list'            => \%cg_item_list_hash,
                      'src_cg_name'             => \%src_cg_name,
                      'dst_cg_name'             => \%dst_cg_name,
                      'relationship_type'       => 'SYNC');


=item Options

=over

=item node_src

(Optional) src node object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) dst node object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref of destination volume objects

=item src_cg_name

(Required) hashref of src cg name with key as src vol name

=item dst_cg_name

(Required) hashref of dst cg name with key as dst vol name

=item cg_item_list

(Optional) Arrayref of list of cg-items(luns or files). NOTE:Its a required parameter for sync but optional for ADR/Vault

=item relationship_type

(Required) type(ADR/VAULT/SYNC) of relationship(s) to be created.Default Value is 'SYNC'

=item retry_list_dest_count

(Optional) Number of times to retry the snapmirror show output. Default: 30.

=item add_label

(Optional) Name of label to added to policy. Default 'none', meaning no label will be added.

=back

=back

=cut

sub Wf_basic_snapmirror_setup{

     my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src            => {type => OBJECT, optional=>1, isa => 'NACL::C::Node'},
                node_dst            => {type => OBJECT, optional=>0, isa => 'NACL::C::Node'},
                src_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
                dst_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
                src_cg_name         => {type => HASHREF, optional => 1},
                dst_cg_name         => {type => HASHREF, optional => 1},
                cg_item_list        => {type => HASHREF, optional => 1},
                relationship_type   => {type => SCALAR, optional => 1, default => 'SYNC', regex => qr/ADR|VAULT|SYNC/,},
                validate            => {type => SCALAR, optional =>1,default=>'false'},
                retry_list_dest_count    => {type => SCALAR, default=>30},
                tries               => { type => SCALAR, optional => 1, default => 8 },
                add_label => { type => SCALAR, optional => 1, default => 'none' },
                },
                );
        my @src_volumes;
        my @dst_volumes;

        # Hash ref of NACL objects for caller of this function.
        my $all_objects;

        my $Src_Node            = $args{'node_src'};
        my $Dst_Node            = $args{'node_dst'};
        my $src_vol             = $args{'src_volumes'};
        my $dst_vol             = $args{'dst_volumes'};
        my $cg_item_list        = $args{'cg_item_list'};
        my $Src_Cg_Name         = $args{'src_cg_name'};
        my $Dst_Cg_Name         = $args{'dst_cg_name'};
        my $tries               = $args{'tries'};
        my $retry_list_dest_count    = $args{'retry_list_dest_count'};
        my $relationship_type   = $args{'relationship_type'};
        my $validate            = $args{'validate'};
        my $add_label        = $args{'add_label'};

       @src_volumes = _list_of_vals($src_vol);
       @dst_volumes = _list_of_vals($dst_vol);

        my @subtests;
        my @Sm_Objs;
        my @src_cg_names;
        my @dst_cg_names;
        my @src_vol_names;
        my @dst_vol_names;
        my $Src_Vserver_Name;
        my $Dst_Vserver_Name;

        my $indx=0;

        my ($src_cg_name, $dst_cg_name);
          for(my $i=0;$i<=$#src_volumes;$i++){
            if ($relationship_type eq 'SYNC'){
                $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
                $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};
                my $cg_item_arr_ref = ${$cg_item_list}{$src_volumes[$i]->volume};
                my @cg_items_array =  _list_of_vals($cg_item_arr_ref);
                $Src_Vserver_Name = $src_volumes[$i]->vserver;
                $Dst_Vserver_Name = $dst_volumes[$i]->vserver;

            push @subtests,NATE::Process->new(
                    codespec => \&one_to_one_cg_relationship,
                    runid => 'SYNC_RELATIONSHIP' .$indx,
                    args => [
                        'src_vol'       => $src_volumes[$i]->volume,
                        'dst_vol'       => $dst_volumes[$i]->volume,
                        'src_vserver'   => $Src_Vserver_Name,
                        'dst_vserver'   => $Dst_Vserver_Name,
                        'src_cg_name'   => $src_cg_name,
                        'dst_cg_name'   => $dst_cg_name,
                        'cg_item_list'  => \@cg_items_array,
                        'node_dst'      => $Dst_Node,
                        'node_src'      => $Src_Node,
                        'stagger_delay' => 10,
                        'validate'      => $validate,
                        'tries'         => $tries,
                        'retry_list_dest_count' => $retry_list_dest_count,
                        'add_label'     => $add_label,
                        ],
                    inherit_params => 1,
                    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
                    );
            push @src_cg_names , $src_cg_name ;
            push @dst_cg_names , $dst_cg_name ;
        }
        elsif ($relationship_type eq 'ADR'){
            push @subtests,NATE::Process->new(
                    codespec => \&one_to_one_adr_relationship,
                    runid => 'ADR_RELATIONSHIP' .$indx,
                    args => [
                    'src_vol'     => $src_volumes[$i]->volume,
                    'dst_vol'     => $dst_volumes[$i]->volume,
                    'src_vserver' => $Src_Vserver_Name,
                    'dst_vserver' => $Dst_Vserver_Name,
                    'node_dst'    => $Dst_Node,
                    'node_src'    => $Src_Node,
                    'stagger_delay' => 10,
                    'validate' => $validate,
                    ],
                    inherit_params => 1,
                    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
                    );
            push @src_vol_names , $src_volumes[$i]->volume ;
            push @dst_vol_names , $dst_volumes[$i]->volume ;
        }
        elsif ($relationship_type eq 'VAULT'){
		    $log->comment('src vol object inside workflow' . Dumper( $src_volumes[$i] ) );
			$Src_Vserver_Name = $src_volumes[$i]->vserver;
            $Dst_Vserver_Name = $dst_volumes[$i]->vserver;

            push @subtests,NATE::Process->new(
                    codespec => \&one_to_one_vault_relationship,
                    runid => 'VAULT_RELATIONSHIP' .$indx,
                    args => [
                    'src_vol'     => $src_volumes[$i]->volume,
                    'dst_vol'     => $dst_volumes[$i]->volume,
                    'src_vserver' => $src_volumes[$i]->vserver,
                    'dst_vserver' => $dst_volumes[$i]->vserver,
                    'node_dst'    => $Dst_Node,
                    'node_src'    => $Src_Node,
                    'stagger_delay' => 10,
                    'validate' => $validate,
                    ],
                    inherit_params => 1,
                    onexit=> \&NATE::Process::on_exit_propagate_worst_result,
                    );
            push @src_vol_names , $src_volumes[$i]->volume ;
            push @dst_vol_names , $dst_volumes[$i]->volume ;
        }
        $indx++;

    }

    my $pm = NATE::ProcManager->new(
            runid               => 'PM_' . $relationship_type,
            max_active_children => 16,
            proc_info           => \@subtests,
            );
    $pm->start();
    $pm->wait();
    $pm->destroy();
    $Src_Node->refresh_command_interface(); # workaround for burt863951
    $Dst_Node->refresh_command_interface(); # workaround for burt863951

    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }

    if ($relationship_type eq 'SYNC'){
        foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }
    else{
        foreach(my $i=0;$i<=$#src_vol_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_vol_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_vol_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }
    }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

=head2 Wf_sync_app_consistent_create

This is a 'CANNED' workflow.
Provides the ability to create multiple app consistent snapshots on multiple sync relationships.
Verifies that the given no .(snapshot_count) of app consistent snapshots are created on sm_objs relationships

=over

=item Option

=over

=item node_src

(Required) String value - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) String value - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item snapshot_count

(Optional) no of snapshots to be generated, default value is 1. values allowed are 1..255(max limit is 255).

=item snapshot_name

(Optional) Defaults to 'acs'.pid.count. E.g. 'acs98081' where 9808 is the PID and 1 is the snapshot count.
If specified will look like $snapshotname.count. E.g. if snapshot_name=>foo and snapshot_count=>2 this will
result in 2 snapshots 'foo.0' and 'foo.1'.

=item validate

(Optional) Performs validation (invokes validate_ACS method) of the ACS taken on the primary and secondary.
Possible values: 'true'/'false'. Default: false.

=item delete_ss

(Optional) If set to 1 the workflow will delete the ACS snapshot(s) that it created.
(default) 0

=item delete_delay

(Optional) The number of seconds to wait before attempting to delete the acs ss that were created
(Default) 10

=back

=back

=cut

sub Wf_sync_app_cons_creation {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node_src       => {type => OBJECT|SCALAR, optional=>0},
            node_dst       => {type => OBJECT|SCALAR, optional=>0},
            snapshot_name  => {type => SCALAR, optional => 1}, # default 'acs'.pid.count
            snapshot_count => {type => SCALAR, optional => 1, default => 1}, # values allowed - any number ( 1..255)
            validate       => {type => SCALAR, optional => 1, default =>'false'}, # values allowed - true/false
            stagger_delay  => {type => SCALAR, optional => 1,},
            sm_objs        => {type => ARRAYREF, optional => 0},
            delete_ss      => {type => SCALAR, optional => 1},
            delete_delay   => {type => SCALAR, optional => 1},
            }
        );

    my $smobjs = delete $args{'sm_objs'};
    my @SmObjs = _list_of_vals($smobjs);
    my @subtests_app_cons;
    foreach my $obj (@SmObjs) {
        $args{dst_path} = $obj->destination_path,
        push @subtests_app_cons, create_parallel_processes('app_ss',\%args);
    }
    run_processes_bg('app_ss',\@subtests_app_cons);
}

=head2 Wf_in_sync

This is a 'CANNED' workflow.
Provides the ability to run nas/san workloads and volume/inode attribute changes on an in-sync relationship.
Verifies that the relationships remain in insync state afterwards.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref of destination volume object

=item src_cg_name

(Required) hashref of src cg name with key as src vol name

=item dst_cg_name

(Required) hashref of dst cg name with key as dst vol name

=item client_list

(Required) Arrayref of list of client ojects

=item cg_item_list

(Required) hashref of list of cg-items(luns or files)

=item cg_item_type

(Required) 'lun' or 'file' - type of cg item (string)

=item perform_inode_ops

 (Optional) param to decide whether user wants to run file/lun metadata ops once
            the relaionship is in sync.

=item lun
    (Required) An arrayref of lun objects that will create the CG relationship

=back


=back

=cut

sub Wf_in_sync {

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            vol_src             => {type => ARRAYREF|OBJECT, optional => 0},
            vol_dst             => {type => ARRAYREF|OBJECT, optional => 0},
            src_cg_name         => {type => HASHREF, optional => 0},
            dst_cg_name         => {type => HASHREF, optional => 0},
            cg_item_list        => {type => HASHREF, optional => 1},
            cg_item_type        => {type => SCALAR,  regex => qr/lun|file/, default => 'lun'},
            lun                 => {type => ARRAYREF, optional => 0},
            validate            => {type => SCALAR, optional =>1,default=>'false'},
            perform_inode_ops   => {type => SCALAR, optional =>1, default => 0},
            retry_list_dest_count    => {type => SCALAR, default=>30},
            add_label => { type => SCALAR, optional => 1, default => 'none' },
            },
            );


    my ( $Src_Node, $Dst_Node, $src_vol, $dst_vol,
            $Src_Vserver_Name, $Dst_Vserver_Name,
            $relationship_type, $cg_item_list, $cg_item_type,$Src_Cg_Name,$Dst_Cg_Name);
    my @src_volumes;
    my @dst_volumes;

    # Hash ref of NACL objects for caller of this function.
    my $all_objects;

    $Src_Node            = $args{'node_src'};
    $Dst_Node            = $args{'node_dst'};
    $src_vol             = $args{'vol_src'};
    $dst_vol             = $args{'vol_dst'};
    $cg_item_list        = $args{'cg_item_list'};
    $cg_item_type        = $args{'cg_item_type'};
    $Src_Cg_Name         = $args{'src_cg_name'};
    $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $inode_ops        = $args{'perform_inode_ops'};
    my $lun              = $args{'lun'};
    my $validate         = $args{'validate'};
    my $retry_list_dest_count = $args{'retry_list_dest_count'};
    my $add_label        = $args{'add_label'};


    @src_volumes = _list_of_vals($src_vol);
    @dst_volumes = _list_of_vals($dst_vol);

    if(scalar(@src_volumes) != scalar(@dst_volumes)){
       $log->error("please provide equal number of source and destination volumes");
    }

    my @Sm_Objs;
    my @src_cg_names;
    my @src_vs;
    my @dst_cg_names;
    my @dst_vs;
    my @src_vol_names;
    my @dst_vol_names;
    my @subtests;

    my $indx=0;
    $counter = 0;
    for(my $i=0;$i<=$#src_volumes;$i++){
        my $Src_Vserver_Name = $src_volumes[$i]->vserver;
        my $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
        my $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
        my $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};


            # TODO: make scalar capable
#            my $cg_item_arr_ref = [ keys %{ map { ($_->volume() eq $src_volumes[$i]->volume()) ?  ($_ => 1) : () } @{ $lun } } ];
            my $cg_item_arr_ref = [];
            foreach my $l (@{ $lun }) {
                if ($l->volume() eq $src_volumes[$i]->volume()) {
                    push @{ $cg_item_arr_ref }, $l;
                }
            }
            next if (scalar @{ $cg_item_arr_ref } == 0);
            #my $cg_type         = ${$cg_item_type}{$src_volumes[$i]->volume};
            my $cg_type         = $cg_item_type;
            my @cg_items_array   =  _list_of_vals($cg_item_arr_ref);
            my @cg_names_array;
            foreach my $item(@cg_items_array){
                $log->comment("lun object" . Dumper($item->{path}));
                my(undef,$Src_pqth,$src_vol_name,$lun_name) = split(/\//,$item->{path});
                push @cg_names_array, $lun_name;
            }

            my %args_func = ('node_src'     => $Src_Node,
                             'node_dst'     => $Dst_Node,
                             'src_volume'   => $src_volumes[$i]->volume,
                             'dst_volume'   => $dst_volumes[$i]->volume,
                             'src_vserver'  => $Src_Vserver_Name,
                             'dst_vserver'  => $Dst_Vserver_Name,
                             'cg_item_list'  => \@cg_names_array,
                             'cg_item_type'  => $cg_type,
                             'src_cg_name'   =>  $src_cg_name,
                             'dst_cg_name'   =>  $dst_cg_name,
                             'stagger_delay' => 10,
                             'validate'      => $validate,
                             'retry_list_dest_count' => $retry_list_dest_count,
                             'add_label'     => $add_label,
                             'inode_ops'     => $inode_ops);

            push @subtests , create_parallel_processes('in_sync',\%args_func);
            push @src_cg_names , $src_cg_name ;
            push @src_vs, $Src_Vserver_Name;
            push @dst_cg_names , $dst_cg_name ;
            push @dst_vs, $Dst_Vserver_Name;

    }
    run_processes_bg('in_sync',\@subtests);

    $Src_Node->refresh_command_interface();
    $Dst_Node->refresh_command_interface();

    foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(
                    command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $src_vs[$i].":".$src_cg_names[$i],
                    'destination-path' => $dst_vs[$i].":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;

}

# private method
sub in_sync{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>1},
            node_dst            => {type => OBJECT, optional=>1},
            src_volume          => {type => SCALAR, optional => 1},
            dst_volume          => {type => SCALAR, optional => 1},
            src_vserver         => {type => SCALAR, optional => 1},
            dst_vserver         => {type => SCALAR, optional => 1},
            cg_item_list        => {type => ARRAYREF|SCALAR|UNDEF, optional => 1},
            cg_item_type        => {type => SCALAR, optional =>1},
            src_cg_name         => {type => SCALAR, optional =>1,},
            dst_cg_name         => {type => SCALAR, optional =>1,},
            validate            => {type => SCALAR, optional =>1,default=>'false'},
            retry_list_dest_count    => {type => SCALAR, default=>30},
            add_label           => { type => SCALAR, optional => 1, default => 'none' },
            inode_ops           => {type => SCALAR, optional =>1,default=> 0},
            },
            );

    my @src_volumes;
    my @dst_volumes;

    my $Src_Node            = $args{'node_src'};
    my $Dst_Node            = $args{'node_dst'};
    my $src_vol             = $args{'src_volume'};
    my $dst_vol             = $args{'dst_volume'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $cg_item_type        = $args{'cg_item_type'};
    my $src_cg_name         = $args{'src_cg_name'};
    my $dst_cg_name         = $args{'dst_cg_name'};
    my $validate            = $args{'validate'};
    my $inode_ops           = $args{'inode_ops'};
    my $retry_list_dest_count    = $args{'retry_list_dest_count'};
    my $add_label           = $args{'add_label'};


    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }


    if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) ) {
        $Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

    $log->comment('Create and initialize a sm relatioship');
    # This will bring one relationship in sync
    one_to_one_cg_relationship(
            src_vol    => $src_vol,
            dst_vol           => $dst_vol,
            src_vserver       => $Src_Vserver_Name,
            dst_vserver       => $Dst_Vserver_Name,
            src_cg_name       => $src_cg_name,
            dst_cg_name       => $dst_cg_name,
            cg_item_list      => $cg_item_list,
            node_dst          => $Dst_Node,
            node_src          => $Src_Node,
            stagger_delay     => 10,
            retry_list_dest_count  => $retry_list_dest_count,
            add_label         => $add_label,
            validate          => $validate);

    if ($inode_ops) {
        $log->warn('Pending implementation for LUN Inode ops');
    }

    return;
}

=head2 Wf_out_of_sync

Provides the ability to bring the relationship to OOS if an event has occured on it.
Verifies that the relationships remain in outofsync state afterwards.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref of destination volume objects

=item src_cg_name

(Required) hashref of src cg name with key as src vol name

=item dst_cg_name

(Required) hashref of dst cg name with key as dst vol name

=item client_list_string

(Required) Arrayref of list of client objects

=item cg_item_list

(Required) Arrayref of list of cg-items(luns or files)

=item cg_item_type

(Required) 'lun' or 'file' - type of cg item (string)

=item event_type

(Optional) event(values allowed : togb/volOfflineOnine/ARL/interclusterFailure) to be run on a sync relationship

=item aggr_names

(Optional) Required for ARL -aggregate names to be relocated.

=back

=back

=cut

sub Wf_out_of_sync {
   my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            dst_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            src_cg_name         => {type => HASHREF, optional => 0},
            dst_cg_name         => {type => HASHREF, optional => 0},
            client_list_string  => {type => ARRAYREF|SCALAR, optional => 0},
            cg_item_list        => {type => HASHREF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            event_type          => {type => SCALAR, optional=>0 , default=> 'togb'},
            aggr_names          => {type => ARRAYREF, optional=>1,},
            },
            );


    my ( $Src_Node, $Dst_Node, $src_vol, $dst_vol,
            $Src_Vserver_Name, $Dst_Vserver_Name, $client,
            $relationship_type, $cg_item_list, $cg_item_type,$Src_Cg_Name,$Dst_Cg_Name);
    my @src_volumes;
    my @dst_volumes;
    my @client_list ;

    # Hash ref of NACL objects for caller of this function.
    my $all_objects;

    $Src_Node            = $args{'node_src'};
    $Dst_Node            = $args{'node_dst'};
    $src_vol             = $args{'src_volumes'};
    $dst_vol             = $args{'dst_volumes'};
    $Src_Vserver_Name    = $args{'src_vserver'};
    $Dst_Vserver_Name    = $args{'dst_vserver'};
    $client              = $args{'client_list_string'};
    $relationship_type   = $args{'relationship_type'};
    $cg_item_list        = $args{'cg_item_list'};
    $cg_item_type        = $args{'cg_item_type'};
    $Src_Cg_Name         = $args{'src_cg_name'};
    $Dst_Cg_Name         = $args{'dst_cg_name'};

    @src_volumes = _list_of_vals($src_vol);
    @dst_volumes = _list_of_vals($dst_vol);
    @client_list = _list_of_vals($client);

    my @subtests;
    my @Sm_Objs;
    my @src_cg_names;
    my @dst_cg_names;
    my @src_vol_names;
    my @dst_vol_names;

    my $indx=0;
    $counter = 0;
    for(my $i=0;$i<=$#src_volumes;$i++){
        my $client=$client_list[$i];
        $Src_Vserver_Name = $src_volumes[$i]->vserver;
        $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
        my $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
        my $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};
        my $cg_item_arr_ref = ${$cg_item_list}{$src_volumes[$i]->volume};
        my @cg_items_array =  _list_of_vals($cg_item_arr_ref);

        my %args_func = ('node_src'    => $Src_Node,
                        'node_dst'    => $Dst_Node,
                        'src_volume'  => $src_volumes[$i]->volume,
                        'dst_volume'  => $dst_volumes[$i]->volume,
                        'src_vserver' => $Src_Vserver_Name,
                        'dst_vserver' => $Dst_Vserver_Name,
                        'client'      => $client,
                        'cg_item_list' => \@cg_items_array,
                        'cg_item_type' => $cg_item_type,
                        'src_cg_name' => $src_cg_name,
                        'dst_cg_name' => $dst_cg_name,
                        'stagger_delay' => 10,);

        push @subtests , create_parallel_processes('out_of_sync',\%args_func);
        push @src_cg_names , $src_cg_name ;
        push @dst_cg_names , $dst_cg_name ;
    }
    run_processes_bg('out_of_sync',\@subtests);
    foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_names[$i],
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

# private method
sub out_of_sync{
    # Events that can bring sync relationship to OOS are:
    # a) SFO (takeover/giveback)
    # b) ARL
    # c) volume offline/online
    # d) Ic down
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volume          => {type => SCALAR, optional => 0},
            dst_volume          => {type => SCALAR, optional => 0},
            src_vserver         => {type => SCALAR, optional => 0},
            dst_vserver         => {type => SCALAR, optional => 0},
            client              => {type => SCALAR, optional => 0},
            cg_item_list        => {type => ARRAYREF|SCALAR|UNDEF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            src_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Src_Cg_Name'},
            dst_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Dst_Cg_Name'},
            event_type          => {type => SCALAR, optional=>0, default=> 'volOfflineOnline'},
            aggr_names          => {type => ARRAYREF, optional=>1,},
            },
            );

    my $Src_Node            = $args{'node_src'};
    my $Dst_Node            = $args{'node_dst'};
    my $src_vol             = $args{'src_volume'};
    my $dst_vol             = $args{'dst_volume'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $client              = $args{'client'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $cg_item_type        = $args{'cg_item_type'};
    my $relationship_type   = $args{'relationship_type'};
    my $src_cg_name         = $args{'src_cg_name'};
    my $dst_cg_name         = $args{'dst_cg_name'};
    my $agg_names           = $args{'aggr_names'};
    my $event_type          = $args{'event_type'};
    my $aggr_names           = $args{'aggr_names'};

    #First bring the workflow in sync

    in_sync(node_src            => $Src_Node,
               node_dst            => $Dst_Node,
               src_volume          => $src_vol,
               dst_volume          => $dst_vol,
               src_vserver         => $Src_Vserver_Name,
               dst_vserver         => $Dst_Vserver_Name,
               client              => $client,
               cg_item_list        => $cg_item_list,
               cg_item_type        => $cg_item_type,
               src_cg_name         => $src_cg_name,
               dst_cg_name         => $dst_cg_name,);

    ##TODO:Removve
    my $Dst_vol;
    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }


    if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) ) {
        $Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

    my $srv_client = NACL::ServiceAPI::Events->new();

    if($event_type eq 'ARL'){
     if(!$aggr_names){
            $log->error("aggr_names is a mandatory parameter for ARL event");
        }
     $srv_client->ARL(
                request_timeout => -1,
                node            => $Src_Node->name,
                dest            => $Dst_Node->name,
                aggr_list       => $aggr_names,
                );
    }
    if($event_type eq 'togb'){
        $srv_client->togb(
                request_timeout => -1,
                node            => $Src_Node->name,
                partner         => $Dst_Node->name,
                );
    }

    if($event_type eq 'interclusterFailure'){
        #
        # iterations defaults to 1 - takes ic lifs down
        # for ten minutes and then brings ic lifs up
        # and then returns.
        #
        $srv_client->interclusterFailure(
            request_timeout => -1,
            nodes           => $Src_Node->name,
            'event-delay'   => 600,
        );
    }

    #
    # iterations defaults to 1 - vol offline
    # for ten minutes and then vol online and
    # then returns.
    #
    if($event_type eq 'volOfflineOnline'){
        #$srv_client->volOfflineOnline(
        #    nodes => $Dst_Node->name,
        #    vol_list => [ $dst_vol ],
        #    iterations => 1,
        #    'event-delay' => 60,
        #    'iteration-delay' => 10,
        #)
        if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
        }



    }


    if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }


    if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) ) {
        $Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

    # Verify status remains in "OutOfSync" afterwards event got triggered
    _verify_list_destination_status($Src_Node,$dst_cg_name,$src_cg_name,$Src_Vserver_Name,$Dst_Vserver_Name,'OutOfSync');

    $Dst_vol->online();
}


=head2 Wf_CG_Failover

Provides the ability to failover a CG relationship
1)Break the relationship
2)Validate Destination items are RW
  Redirection - coming soon
3)Delete the relationship
4)Relase the relationship with relationship-info-only flag
5)Create the relationship with source and destination interchanged

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref/Scalar of destination volume objects

=item src_cg_name

(Required) hashref of src cg name with key as src vol name

=item dst_cg_name

(Required) hashref of dst cg name with key as dst vol name

=item cg_item_list

(Required) Arrayref of list of cg-items(luns or files)

=item cg_item_type

(Required) 'lun' or 'file' - type of cg item (string)

=back

=back

=cut

sub Wf_CG_Failover {
   my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            dst_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            src_cg_name         => {type => HASHREF, optional => 0},
            dst_cg_name         => {type => HASHREF, optional => 0},
            cg_item_list        => {type => HASHREF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            },
            );


    my ( $Src_Node, $Dst_Node, $src_vol, $dst_vol,
            $Src_Vserver_Name, $Dst_Vserver_Name, $client,
            $relationship_type, $cg_item_list, $cg_item_type,$Src_Cg_Name,$Dst_Cg_Name);
    my @src_volumes;
    my @dst_volumes;

    # Hash ref of NACL objects for caller of this function.
    my $all_objects;

    $Src_Node            = $args{'node_src'};
    $Dst_Node            = $args{'node_dst'};
    $src_vol             = $args{'src_volumes'};
    $dst_vol             = $args{'dst_volumes'};
    $Src_Vserver_Name    = $args{'src_vserver'};
    $Dst_Vserver_Name    = $args{'dst_vserver'};
    $cg_item_list        = $args{'cg_item_list'};
    $cg_item_type        = $args{'cg_item_type'};
    $Src_Cg_Name         = $args{'src_cg_name'};
    $Dst_Cg_Name         = $args{'dst_cg_name'};

    @src_volumes = _list_of_vals($src_vol);
    @dst_volumes = _list_of_vals($dst_vol);

    my @subtests;
    my @Sm_Objs;
    my @src_cg_names;
    my @dst_cg_names;
    my @src_vol_names;
    my @dst_vol_names;

    my $indx=0;
    $counter = 0;
    for(my $i=0;$i<=$#src_volumes;$i++){
        $Src_Vserver_Name = $src_volumes[$i]->vserver;
        $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
        my $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
        my $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};
        my $cg_item_arr_ref = ${$cg_item_list}{$src_volumes[$i]->volume};
        my @cg_items_array =  _list_of_vals($cg_item_arr_ref);

        my %args_func = ('node_src'    => $Src_Node,
                        'node_dst'    => $Dst_Node,
                        'src_volume'  => $src_volumes[$i]->volume,
                        'dst_volume'  => $dst_volumes[$i]->volume,
                        'src_vserver' => $Src_Vserver_Name,
                        'dst_vserver' => $Dst_Vserver_Name,
                        'cg_item_list' => \@cg_items_array,
                        'cg_item_type' => $cg_item_type,
                        'src_cg_name' => $src_cg_name,
                        'dst_cg_name' => $dst_cg_name,);

        push @subtests , create_parallel_processes('cg_failover',\%args_func);
        push @src_cg_names , $src_cg_name ;
        push @dst_cg_names , $dst_cg_name ;
    }
    run_processes_bg('cg_failover',\@subtests);

    foreach(my $i=0;$i<=$#src_cg_names;$i++){
            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'destination-path'      => $Src_Vserver_Name.":".$src_cg_names[$i], #send with source and destination interchanged and failover does that
                    'source-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

# private method
sub cg_failover{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volume          => {type => SCALAR, optional => 0},
            dst_volume          => {type => SCALAR, optional => 0},
            src_vserver         => {type => SCALAR, optional => 0},
            dst_vserver         => {type => SCALAR, optional => 0},
            cg_item_list        => {type => ARRAYREF|SCALAR|UNDEF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            src_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Src_Cg_Name'},
            dst_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Dst_Cg_Name'},
            add_label           => { type => SCALAR, optional => 1, default => 'none' },
            },
            );

    my $Src_Node            = $args{'node_src'};
    my $Dst_Node            = $args{'node_dst'};
    my $src_vol             = $args{'src_volume'};
    my $dst_vol             = $args{'dst_volume'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $cg_item_type        = $args{'cg_item_type'};
    my $src_cg_name         = $args{'src_cg_name'};
    my $dst_cg_name         = $args{'dst_cg_name'};
    my $add_label           = $args{'add_label'};

    $Src_Node->refresh_command_interface(); # workaround for burt863951
    $Dst_Node->refresh_command_interface(); # workaround for burt863951
    my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_name,
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_name,
                    },
                    allow_empty => 1
                    );

    $log->step(" Breaking the relationship ");
    $sm_obj->break(nacltask_wait=>0);

#validate break
#will move to Validations.pm once approved
    $log->step("Validating Break");
     my $sm_state = NACL::CS::Snapmirror->fetch(
            command_interface => $Dst_Node,
            filter            => {
                'source-path'      => $sm_obj->{'source_path'},
                'destination-path' => $sm_obj->{'destination_path'},
            },
            allow_empty => 0,
        );

        if ( $sm_state->{'status'}  !~ /outofsync/i
            || $sm_state->{'state'} !~ /Broken-off/ )
        {
            $log->error(" Relationship state expected to be Broken-off and status expected to be OutofSync");
        }
        else
        {
            $log->comment(" Relationship is Broken-off and OutOfSync");
        }

    $log->step("Validating the items are RW on the destination ");
    my @items_dst= _list_of_vals($cg_item_list);

    if ($cg_item_type eq 'lun') {
       $log->comment("cgitems are luns");
        my $lun_state;
        my $perm;
        my $Lun;
        my $check = 0;
        my $k     = 0;
        foreach my $lun (@items_dst) {
            $lun_state = NACL::CS::Lun->fetch(
                command_interface => $Dst_Node,
                filter            => {
                    vserver => $Dst_Vserver_Name,
                    volume  => $dst_vol,
                    lun     => $lun,
                },
                requested_fields => ['read-only'],
            );
            $perm = $lun_state->read_only();
            $log->comment(" Lun $lun is read only : $perm ");
            if ($perm eq 'True') {
                $log->log(
                 NATE::Result::Fail->new(
" Lun cg item $lun is Read Only after break : FAIL",
                            abortive => 0
                     )
                    );
            last;
        }
}
}


    else {  #cgitem is file
    #there is no easy way to verify this.
    $log->comment("cgitems are files");
    $log->warn("Coming soon - granset validation for files");
   }

#redirection???

    $log->step("Perform flip-resync part 1");
#Failover to the destination
=begin
STEPS
2)Issue delete on destination
3)issue release with relationship-info-only flag
4)Issue snapmirror create with source and destination interchanged
=cut

    $log->step("Delete the relationship");
    $sm_obj->delete("nacltask_wait=>1");

    $log->step("Release the relationship with -relationship-info-only flag");
    $sm_obj->release(
        'source_ci'              => $Src_Node,
        'relationship-info-only' => 'true',
         nacltask_wait            => 1,
    );

    Tharn::snooze(10); #just to settle

#now create the relationship with source and destination interchanged.
    $log->step("create the relationship with source and destination interchanged.");
    my %args_func_flip = ('node_dst'    => $Src_Node,
                        'node_src'    => $Dst_Node,
                        'dst_vol'  => $src_vol,
                        'src_vol'  => $dst_vol,
                        'dst_vserver' => $Src_Vserver_Name,
                        'src_vserver' => $Dst_Vserver_Name,
                        'cg_item_list' => $cg_item_list,
                        'dst_cg_name' => $src_cg_name,
                        'src_cg_name' => $dst_cg_name,
                        'add_label' => $add_label,
                        'stagger_delay' => 10,);
    sm_create_cg(%args_func_flip);

    my $snapmirror = NACL::STask::Snapmirror->find(command_interface => $Src_Node,
                    filter            => {
                    'destination-path'      => $Src_Vserver_Name.":".$src_cg_name,
                    'source-path' => $Dst_Vserver_Name.":".$dst_cg_name,
                    },
                    allow_empty => 1
                    );

    try {
        $snapmirror->wait_on_attributes(
            'method-timeout' => 90,
            attributes       => [
                {
                    attribute_to_check => 'state',
                    till_value         => [qr/Broken-off/]
                },

                {
                    attribute_to_check => 'status',
                    till_value         => qr/OutOfSync/i
                },

            ]
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        $log->comment( "Caught exception $exception" . "TXT $text" );
        $log->log(
            NATE::Result::Fail->new(
                "Relationship state is expected to be Broken-off and status OutOfSync"
            ),
            abortive => 0
        );
    };

    my $dst_node_name=$Dst_Node->node();
    $log->warn(" The failover workflow till create is now complete . Traffic could be started on site $dst_node_name . Issue WF_CG_Failback workflow when ready to switch to original Relationship.");
}

=head2 Wf_CG_Failback

Provides the ability to failback a CG relationship
We will assume that failover is called before this call is made.
1)Validate that the relationship is in broken-off state
2)Perform Resync on the Snapmirror relationship.
2)Do Failback to the original source
    - Quiesce the relationship.
    - Delete the snapmirror relationship
    - Release the snapmirror relationship with -relationship-info-only flag
    - Issue snapmirror create with source and destination CGs interchanged.
    - Issue Resync. The relationship is now as it was before failover.

=over

=item Options

=over

=item node_src

(Required) Src Node Object - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) Dst Node Object - the natehost record of the node_dst the
destination volume resides on

=item src_volumes

(Required) Arrayref of source volume objects

=item dst_volumes

(Required) Arrayref/Scalar of destination volume objects

=item src_cg_name

(Required) hashref of src cg name with key as src vol name

=item dst_cg_name

(Required) hashref of dst cg name with key as dst vol name

=item cg_item_list

(Required) Arrayref of list of cg-items(luns or files)

=item cg_item_type

(Required) 'lun' or 'file' - type of cg item (string)

=back

=back

=cut



sub Wf_CG_Failback {
   my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            dst_volumes         => {type => ARRAYREF|OBJECT, optional => 0},
            src_cg_name         => {type => HASHREF, optional => 0},
            dst_cg_name         => {type => HASHREF, optional => 0},
            cg_item_list        => {type => HASHREF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            },
            );


    my ( $Src_Node, $Dst_Node, $src_vol, $dst_vol,
            $Src_Vserver_Name, $Dst_Vserver_Name, $client,
            $relationship_type, $cg_item_list, $cg_item_type,$Src_Cg_Name,$Dst_Cg_Name);
    my @src_volumes;
    my @dst_volumes;

    # Hash ref of NACL objects for caller of this function.
    my $all_objects;

    $Src_Node            = $args{'node_src'};
    $Dst_Node            = $args{'node_dst'};
    $src_vol             = $args{'src_volumes'};
    $dst_vol             = $args{'dst_volumes'};
    $Src_Vserver_Name    = $args{'src_vserver'};
    $Dst_Vserver_Name    = $args{'dst_vserver'};
    $cg_item_list        = $args{'cg_item_list'};
    $cg_item_type        = $args{'cg_item_type'};
    $Src_Cg_Name         = $args{'src_cg_name'};
    $Dst_Cg_Name         = $args{'dst_cg_name'};

    @src_volumes = _list_of_vals($src_vol);
    @dst_volumes = _list_of_vals($dst_vol);

    my @subtests;
    my @Sm_Objs;
    my @src_cg_names;
    my @dst_cg_names;
    my @src_vol_names;
    my @dst_vol_names;

    my $indx=0;
    $counter = 0;
    for(my $i=0;$i<=$#src_volumes;$i++){
        $Src_Vserver_Name = $src_volumes[$i]->vserver;
        $Dst_Vserver_Name = $dst_volumes[$i]->vserver;
        my $src_cg_name = ${$Src_Cg_Name}{$src_volumes[$i]->volume};
        my $dst_cg_name = ${$Dst_Cg_Name}{$dst_volumes[$i]->volume};
        my $cg_item_arr_ref = ${$cg_item_list}{$src_volumes[$i]->volume};
        my @cg_items_array =  _list_of_vals($cg_item_arr_ref);

        my %args_func = ('node_src'    => $Src_Node,
                        'node_dst'    => $Dst_Node,
                        'src_volume'  => $src_volumes[$i]->volume,
                        'dst_volume'  => $dst_volumes[$i]->volume,
                        'src_vserver' => $Src_Vserver_Name,
                        'dst_vserver' => $Dst_Vserver_Name,
                        'cg_item_list' => \@cg_items_array,
                        'cg_item_type' => $cg_item_type,
                        'src_cg_name' => $src_cg_name,
                        'dst_cg_name' => $dst_cg_name,
);

        push @subtests , create_parallel_processes('cg_failback',\%args_func);
        push @src_cg_names , $src_cg_name ;
        push @dst_cg_names , $dst_cg_name ;
    }
    run_processes_bg('cg_failback',\@subtests);

    foreach(my $i=0;$i<=$#src_cg_names;$i++){

            my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'destination-path'      => $Src_Vserver_Name.":".$src_cg_names[$i],
                    'source-path' => $Dst_Vserver_Name.":".$dst_cg_names[$i],
                    },
                    allow_empty => 1
                    );
            push @Sm_Objs,$sm_obj;
        }

    $all_objects->{'sm_obj'} = \@Sm_Objs;
    return $all_objects;
}

       # private method
sub cg_failback{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src            => {type => OBJECT, optional=>0},
            node_dst            => {type => OBJECT, optional=>0},
            src_volume          => {type => SCALAR, optional => 0},
            dst_volume          => {type => SCALAR, optional => 0},
            src_vserver         => {type => SCALAR, optional => 0},
            dst_vserver         => {type => SCALAR, optional => 0},
            cg_item_list        => {type => ARRAYREF|SCALAR|UNDEF, optional => 0},
            cg_item_type        => {type => SCALAR, optional =>0},
            src_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Src_Cg_Name'},
            dst_cg_name         => {type => SCALAR, optional =>1, default=> '/cg/Dst_Cg_Name'},
            add_label           => { type => SCALAR, optional => 1, default => 'none' },
            },
            );

    my $Src_Node            = $args{'node_src'};
    my $Dst_Node            = $args{'node_dst'};
    my $src_vol             = $args{'src_volume'};
    my $dst_vol             = $args{'dst_volume'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $cg_item_type        = $args{'cg_item_type'};
    my $src_cg_name         = $args{'src_cg_name'};
    my $dst_cg_name         = $args{'dst_cg_name'};
    my $add_label           = $args{'add_label'};
#    my $src_vol_obj         = $args{'src_vol_obj'};
#    my $dst_vol_obj         = $args{'dst_vol_obj'};

    $Src_Node->refresh_command_interface(); # workaround for burt863951
    $Dst_Node->refresh_command_interface(); # workaround for burt863951

    my $sm_obj = NACL::STask::Snapmirror->find(command_interface => $Dst_Node,
                    filter            => {
                    'source-path'      => $Src_Vserver_Name.":".$src_cg_name,
                    'destination-path' => $Dst_Vserver_Name.":".$dst_cg_name,
                    },
                    allow_empty => 1
                    );
    sleep(60);
    #validate failover
    $log->step("Validating Failover");
    #essentially validating break
    #will move to Validations.pm once approved
    my $dst_path= $sm_obj->{'destination_path'};
    my $sm_state = NACL::CS::Snapmirror->fetch(
            command_interface => $Dst_Node,
            filter            => {
                'source-path'      => $sm_obj->{'source_path'},
                'destination-path' => $sm_obj->{'destination_path'},
            },
            allow_empty => 0,
        );

    if ( $sm_state->{'status'}  !~ /outofsync/i
	|| $sm_state->{'state'} !~ /Broken-off/ )
    {
	$log->warn(" Relationship state expected to be Broken-off and status expected to be OutofSync. Issue Failover before Failback");
    }
    else
    {
	$log->comment(" Relationship is Broken-off and OutOfSync- In Failover state");
	#get back from broken off state
	$log->step(" Issue snapmirror resync - Flip Resync");
	$sm_obj->resync(nacltask_wait=>1,
                    'method-timeout' =>7200);
    }
    #create a PCS for the next Resync to work.
    #Give some delay of 5 minutes before creating PCS
    $log->step("Sleep 300s.");
    sleep(300);

    $log->step("Create a PCS on the Destination");

    my $src_vol_obj = NACL::C::Volume->find(
            command_interface => $Src_Node,
            filter            => {
            volume  => $src_vol,
            vserver => $Src_Vserver_Name,
            },
            );

    my $dst_vol_obj = NACL::C::Volume->find(
            command_interface => $Dst_Node,
            filter            => {
            volume  => $dst_vol,
            vserver => $Dst_Vserver_Name,
            },
            );

    my $dst;
    my $dst_node_name = $dst_vol_obj->get_local_node_name();
    if ( $dst_node_name eq $Dst_Node->node() ) {
        $dst = $Dst_Node;
    }
    else {
        $dst = NACL::C::Node->new( name => $dst_node_name );
    }

    my $src;
    my $src_node_name = $src_vol_obj->get_local_node_name();
    if ( $src_node_name eq $Src_Node->node() ) {
        $src = $Src_Node;
    }
    else {
        $src = NACL::C::Node->new( name => $src_node_name );
    }

    my $retries_cnt = 10;
    while (my $n ++ < $retries_cnt){
	try{
	    create_pcs (    node_dst    => $dst,
			    vol_dst     => $dst_vol_obj,
			    dst_path    => $dst_path,
			    node_src    => $src,
			    vol_src     => $src_vol_obj,
			    validate    => 'true');
	    last;
	}
	catch NATE::BaseException with {
	    $log->log("retry in 60s...");
	    sleep (60);
	};
    }

    $log->step("Perform flip-resync part 2");
#failback to the original source
=begin
STEPS
1)Issue quiesce
2)Issue delete on destination
3)issue release with relationship-info-only flag
4)Issue snapmirror create with source and destination interchanged
5)Issue Snapmirror resync to complete failback
Now the relationship is as it was before failover.
=cut

    $log->step("Quiesce the relationship");
    $sm_obj->quiesce("nacltask_wait=>1");
# Validate quiesce
    try {
        $sm_obj->wait_on_attributes(
            'method-timeout' => 90,
            attributes       => [
                {
                    attribute_to_check => 'state',
                    till_value         => [qr/Snapmirrored/]
                },

                {
                    attribute_to_check => 'status',
                    till_value         => qr/Quiesced/,
                },

            ]
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        $log->comment( "Caught exception $exception" . "TXT $text" );
        $log->log(
            NATE::Result::Fail->new(
                "Relationship state is expected to be Quiesced and status OutOfSync"
            ),
            abortive => 0
        );
    };
#delete
    $log->step("Delete the relationship");
    $sm_obj->delete("nacltask_wait=>1");

#release
    $log->step("Release the relationship with -relationship-info-only flag");
    $sm_obj->release(
        'source_ci'              => $Src_Node,
        'relationship-info-only' => 'true',
         nacltask_wait            => 1,
    );

    Tharn::snooze(10); #just to settle
#now create the relationship with source and destination interchanged.

   $log->step("create the relationship with source and destination interchanged.");
   my %args_func_flip = ('node_dst'    => $Src_Node,
                        'node_src'    => $Dst_Node,
                        'dst_vol'  => $src_vol,
                        'src_vol'  => $dst_vol,
                        'dst_vserver' => $Src_Vserver_Name,
                        'src_vserver' => $Dst_Vserver_Name,
                        'cg_item_list' => $cg_item_list,
                        'dst_cg_name' => $src_cg_name,
                        'src_cg_name' => $dst_cg_name,
                        'add_label' => $add_label,
                        'stagger_delay' => 10,);
    sm_create_cg(%args_func_flip);

#validate the relationship is created in Broken off state with Source and Destination interchanged.
    $log->step("Validate the relationship is created in Broken off state with Source and Destination interchanged");
    my $snapmirror = NACL::STask::Snapmirror->find(command_interface => $Src_Node,
                    filter            => {
                    'destination-path'      => $Src_Vserver_Name.":".$src_cg_name,
                    'source-path' => $Dst_Vserver_Name.":".$dst_cg_name,
                    },
                    allow_empty => 1
                    );

    try {
        $snapmirror->wait_on_attributes(
            'method-timeout' => 90,
            attributes       => [
                {
                    attribute_to_check => 'state',
                    till_value         => [qr/Broken-off/]
                },

                {
                    attribute_to_check => 'status',
                    till_value         => qr/OutOfSync/i
                },

            ]
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        my $text      = $exception->text();
        $log->comment( "Caught exception $exception" . "TXT $text" );
        $log->log(
            NATE::Result::Fail->new(
                "Relationship state is expected to be Broken-off and status OutOfSync"
            ),
            abortive => 0
        );
    };

    $log->step("Issue Snapmirror Resync to complete the Failback");
    $snapmirror->resync(nacltask_wait=>1);
#checker ??
    $log->warn(" The failback workflow is now complete");
}

=head2 Wf_sfr

TO perform single file restore operation

=over

=item Options

=over

=item node_src

(Required) String value - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) String value - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item client

(Required)  The name of the client which isused to populate the data of files/luns.

=item validate

(Optional)  Value true or false , true to validate . By default false.

=item timeout

(Optional):  timeout value used for snapshot create.  If not set will default to 60 sec.

=item restore_timeout

(Optional):  method-timeout value for snapmirror restore operation.  If not set will default to 1200 sec.

=back

=back

=cut


sub Wf_sfr{
     my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                node_src           =>  {type => OBJECT|SCALAR, optional => 0},
                node_dst           =>  {type => OBJECT|SCALAR, optional => 0},
                sm_objs            => {type => ARRAYREF, optional => 0},
                client             =>  {type => SCALAR, optional => 0},
                validate           => {type => SCALAR , optional => 1, default=>'false'},
                timeout            => {type => SCALAR , optional => 1, default=>60},
                restore_timeout    => {type => SCALAR , optional => 1, default=>1200},
                },
                );
        my $sm_obj          = $args{'sm_objs'};
        my $node_src        = $args{'node_src'};
        my $node_dst        = $args{'node_dst'};
        my $client          = $args{'client'};
        my $validate        = $args{'validate'};
        my $timeout         = $args{'timeout'};
        my $restore_timeout = $args{'restore_timeout'};
        my @Sfr_objs        = _list_of_vals($sm_obj);

        my @subtests_sfrs;
        $counter = 0;
        foreach my $o(@Sfr_objs) {
            my %args_func = (
                    'node_src'         => $node_src,
                    'node_dst'         => $node_dst,
                    'sm_obj'           => $o,
                    'client'           => $client,
                    'stagger_delay'    => 10,
                    'validate'         => $validate,
                    'timeout'          => $timeout,
                    'restore_timeout'  => $restore_timeout,
                    );
            push @subtests_sfrs, create_parallel_processes('sm_sfr',\%args_func);
        }

        run_processes_bg('sm_sfr',\@subtests_sfrs);
}

# private method
sub sm_sfr{

    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src          =>  {type => OBJECT|SCALAR, optional => 0},
            node_dst          =>  {type => OBJECT|SCALAR, optional => 0},
            sm_obj            => {type => OBJECT, optional=>0},
            client            => {type => SCALAR, optional => 0},
            stagger_delay     => {type => SCALAR, optional => 1},
            validate          => {type => SCALAR , optional => 1},
            timeout           => {type => SCALAR , optional => 1},
            restore_timeout   => {type => SCALAR , optional => 1, default=>1200},
            },
            );

     my $Sm_Obj          = $args{'sm_obj'};
     my $Src_Node        = $args{'node_src'};
     my $Dst_Node        = $args{'node_dst'};
     my $validate        = $args{'validate'};
     my $timeout         = $args{'timeout'};
     my $restore_timeout = $args{'restore_timeout'};
     my $is_shiner       = is_shiner( snapmirror => $Sm_Obj );


    _stagger($args{'stagger_delay'});

    my($Src_Vserver_Name,$src_vol_name) = split(/:/,$Sm_Obj->source_path);
    my($Dst_Vserver_Name,$dst_vol_name) = split(/:/,$Sm_Obj->destination_path);

    my $Client = NACL::C::Client->new(client => $args{'client'});

    if ( ($Src_Node) && (ref($Src_Node) !~ /Node/i) ) {
        $Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

     if ( ($Dst_Node) && (ref($Dst_Node) !~ /Node/i) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }

    my $Src_obj = NACL::C::Volume->find(
            command_interface => $Src_Node,
            filter            => {
            volume  => $src_vol_name,
            vserver => $Src_Vserver_Name,
            },
            );

    my $Dst_obj;
    if($is_shiner == 0) {
        $Dst_obj = NACL::C::Volume->find(
                command_interface => $Dst_Node,
                filter            => {
                volume  => $dst_vol_name,
                vserver => $Dst_Vserver_Name,},
            );
    }

    # Mount the volume
    my $Mount = NACL::STask::Client::Mount->create(
            command_interface   => $Client,
            server_command_interface => $Src_obj->command_interface,
            vserver             => $Src_obj->vserver,
            volume              => $Src_obj,
            exclusive           => 1,
            mountopts          => 'nfsvers=3',
    );

    # Populate data and modify
    my $dataset_info = {};
    dataset (
        operation       => 'populate',
        volume          => $Src_obj,
        client          => $Client,
        mount           => $Mount,
        file_list       => [ qw(/f1 /f2 /f3 /f4) ],
        file_type       => [ qw(file file file file) ],
        file_size       => [ qw(20m 10m 50m 70m) ],
        dataset_info    => $dataset_info,
    );

    my @file_list = @{$dataset_info->{file_list}};

    # find the snapmirror label from the policy and use that for the snapshot
    my $Policy_Obj;
    my $sm_lbl;
    if($is_shiner == 0) {
        $Policy_Obj = NACL::CS::Snapmirror->fetch( command_interface => $Dst_Node,
                                                filter=> { 'destination-path' => $Sm_Obj->destination_path, },
                                                requested_fields => [qw(policy)]
                                                );
        $sm_lbl = NACL::CS::SnapmirrorPolicy->fetch(command_interface => $Dst_Node,
                                                filter=> { policy  => $Policy_Obj->policy, },
                                                requested_fields => [qw(snapmirror-label)]
                                                );
    } else {
       $Policy_Obj = NACL::CS::Snapmirror->fetch( command_interface => $Src_Node,
                                                filter=> { 'destination-path' => $Sm_Obj->destination_path, },
                                                requested_fields => [qw(policy)]
                                                );
        $sm_lbl = NACL::CS::SnapmirrorPolicy->fetch(command_interface => $Src_Node,
                                                filter=> { policy  => $Policy_Obj->policy, },
                                                requested_fields => [qw(snapmirror-label)]
                                                );
    }
    my @sm_lbls = $sm_lbl->snapmirror_label;
    my $sm_label = $sm_lbls[0];
    #  2. create snap1 and replicate to vault dst B
    my %opts = ( 'snapmirror-label' => $sm_label);
    if($timeout) {
        $log->comment('Setting method-timeout for snapshot create command to: ' . $timeout);
        $opts{'method-timeout'} = $timeout;
    }
    my $Snapshot1 = $Src_obj->snapshot_create(snapshot => "snap1_sfr_$$", %opts);

    # Create and Modify Streams
    #  3. modify 20%-50% of data in each file (f1-f4)
    #  4. for f1,f2, delete all streams, create 5 streams; for f3, modify streams; for f4 change names of streams
    $dataset_info->{options} = [
            { delete_streams => { file_list => [ qw(/f1 /f2) ], }, },
            { modify_streams => { file_list => [ qw(/f3) ], }, },
            { rename_streams => { file_list => [ qw(/f4) ], }, },
            { create_streams => { file_list => [ qw(/f1 /f2) ], number=>5, }, },
    ];
    dataset (
        operation       => 'modify',
        volume          => $Src_obj,
        client          => $Client,
        mount           => $Mount,
        file_list       => [ qw(/f1 /f2 /f3 /f4) ],
        file_type       => [ qw(file file file file) ],
        file_size       => [ qw(30m 20m 60m 90m) ],
        dataset_info    => $dataset_info,
    );

    $dataset_info->{options} = undef;

    #  5. create another 4 files f5,f6,f7,f8, attach 5-10 streams to each files
    $dataset_info->{options} = [
            { create_streams => { file_list => [ qw(/f5 /f6 /f7 /f8) ], number => 10 }, },
    ];

    dataset (
        operation       => 'populate',
        volume          => $Src_obj,
        client          => $Client,
        mount           => $Mount,
        file_list       => [ qw(/f5 /f6 /f7 /f8) ],
        file_size       => [ qw(20m 10m 30m 40m) ],
        dataset_info    => $dataset_info,
    );
    $dataset_info->{options} = undef;

    #  6. create snap2 and replicate to vault dst B

    my $Snapshot2 = $Src_obj->snapshot_create(snapshot => "snap2_sfr_$$", %opts);

    $Sm_Obj->update ( nacltask_verify=>1 );

    #  7. modify 20% data for f5; delete f6; delete f7 and create another file f7; rename f8 to f88
    #  8. modify 20% data for f1; delete f2; delete f3 and create another file f3; rename f4 to f44

    $dataset_info->{options} = [
            { modify_files => { file_list => [ qw(/f5 /f1) ], percent=> 20}, },
            { delete_files => { file_list => [ qw(/f6 /f7 /f2 /f3) ], }, },
            { rename_files => { file_list => [ qw(/f8=>/f88 /f4=>/f44) ],}, },
            { create_files => { file_list => [ qw(/f7 /f3) ],}, },
    ];
    dataset (
        operation       => 'modify',
        volume          => $Src_obj,
        client          => $Client,
        mount           => $Mount,
        file_list       => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
        file_type       => [ qw(file file file file file file file file) ],
        dataset_info    => $dataset_info,
    );

    # Create extra snapshot to be common snapshot for Vault relation.
    my $Snapshot3 = $Src_obj->snapshot_create(snapshot => "snap3_sfr_$$", %opts);
    $Sm_Obj->update ( nacltask_verify=>1 );

    # Add more data on src volume
    dataset (
            operation       => 'populate',
            volume          => $Src_obj,
            client          => $Client,
            mount           => $Mount,
            file_count      => 1,
            file_size       => '20m',
            dataset_info    => $dataset_info,
    );

    delete_snapshot ( $Src_obj, $Snapshot2 );

    #  9. SFR f1-f8 in one session
    if ($is_shiner == 0) {
        my $restore_sm = SFR_restore (
            src_volume => $Dst_obj,
            dst_volume => $Src_obj,
            file_list => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
            source_snapshot => $Snapshot2,
            nacltask_wait => 1,
            extra_options => { 'method-timeout' => $restore_timeout },
    );
        if($validate eq 'true'){
            SFR_verify (
                src_volume      => $Src_obj,
                dst_volume      => $Dst_obj,
                mount           => $Mount,
                client          => $Client,
                source_snapshot => $Snapshot2,
                file_list => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
            );
        }
    } else {
        my $dest_path = $Sm_Obj->destination_path;
        my $restore_sm = SFR_restore (
            src_volume=> $dest_path,
            dst_volume => $Src_obj,
            file_list => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
            source_snapshot => $Snapshot2,
            nacltask_wait => 1,);
    }

    $dataset_info->{options} = [
            { delete_files => { file_list => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ], }, },
    ];

     dataset (
        operation       => 'delete',
        volume          => $Src_obj,
        client          => $Client,
        mount           => $Mount,
        file_list       => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
        file_type       => [ qw(file file file file file file file file) ],
        dataset_info    => $dataset_info,
    );

#    my @luns = ('f2','f4');

#    foreach my $lun(@luns){
#        NACL::C::Lun->delete(command_interface => $Src_Node,
#                         vserver => $Src_obj->vserver,
#                         volume => $Src_obj,
#                         lun => $lun);
#    }
}

=head2 Wf_sfr_restore

This method does SFR(single file restore) from source volume to destination volume with given number of files/luns.
SFR Max file limit per SFR  is 8

=over

=item Example

    Wf_sfr_restore (
        src_volume=> $Dst_obj,
        dst_volume => $Src_obj,
        file_list => [ qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8) ],
        source_snapshot => $Snapshot2,
        nacltask_wait => 0,
        extra_options => {
            throttle => '20',
        },
    );

=item Options

=over

=item src_volume

(Required): Source volume Object.

=item dst_volume

(Required): Destination volume Object.

=item file_list

(Required): list of files to be restored.

=item source_snapshot

(Required): source snapshot object from which the files/luns needs to be restored.

=item sm_check

(Optional): boolean flag to create checker snapshot.Default vaule is 1.

=item extra_options

(Optional): Hashref to pass extra parameters like throttle etc.

=item nacltask_wait

(Optional): option to run in background/foreground.

=back

=back

=cut

sub Wf_sfr_restore {
    my %args = validate_with(
        params => \@_,
        spec   => {
            src_volume      => {type => OBJECT | SCALAR, optional => 0},
            dst_volume      => {type => OBJECT | SCALAR, optional => 0},
            source_snapshot => {type => OBJECT | SCALAR, optional => 0},
            file_list       => {type => ARRAYREF | SCALAR, optional => 0},
            sm_check        => {type => BOOLEAN, optional => 1, default => 1},
            extra_options   => {type => HASHREF, optional => 1},
            nacltask_wait   => {type => SCALAR, optional => 1, default => 1},
        },
    );



 $Log->comment( "start SFR in background " ) unless ($args{nacltask_wait});
    $Log->comment( "SFR restore: " . Dumper ( \%args ) ) ;

    my $ss_name = ( ref($args{source_snapshot}) ) ?
                    $args{source_snapshot}->snapshot() : $args{source_snapshot};

    # Add flag needed to rn snapmirror check
    $args{extra_options}->{'create-check-snapshot'} = 'true'
        if ( $args{sm_check} );

    my $restore_obj = NACL::STask::Snapmirror->restore(
        command_interface   => $args{dst_volume}->command_interface(),
        'source-path'       => $args{src_volume},
        'destination-path'  => $args{dst_volume},
        'source-snapshot'   => $ss_name,
        'file-list'         => $args{file_list},
        nacltask_wait       => $args{nacltask_wait},
        %{$args{extra_options}},
    );

    return $restore_obj;
}


# private method
sub _stagger {
    my $delay = shift;
    $delay = 1 if (!defined $delay);
    $delay = int(rand($delay));
    $log->comment('Staggered start delay is: ' . $delay);
    Tharn::snooze($delay);
}

#private method
sub _start_dt_traffic {

    my %args = validate_with(
        params => \@_,
        spec   => {
            lun         => { type => OBJECT, optional => 0 },
            traffic_obj => { type => OBJECT, optional => 0 },
        },
    );

    my $traffic_obj    = $args{traffic_obj};
    my $lun            = $args{lun};
    my $lun_size_bytes = $lun->state->size;

    #
    # Use 1% of the total lun size for each dt pass. There will be
    # multiple dt passes
    #
    my $dt_limit_bytes = int( $lun_size_bytes * 0.01 );
    my $modulo         = $dt_limit_bytes % 512;
    $dt_limit_bytes   -= $modulo;

    if ( $dt_limit_bytes < 512 ) {
        $dt_limit_bytes = 512;
    }

    return $traffic_obj->start(
        tool   => 'dt',
        iotype => 'random',
        limit  => $dt_limit_bytes,

        # This puts a 5 second delay at the end of each pass
        cdelay            => '5',
        oflags            => 'trunc',
        bs                => '4k',
        incr              => 'var',
        min               => '2k',
        max               => '256k',
        pattern           => 'iot',
        runtime           => '-1',
        dispose           => 'keeponerror',
        prefix            => '"%d@%h"',
        disable           => 'pstats',
        enable            => 'compare,lbdata,debug,syslog',
        'privilege-level' => 'root',
    );
}

# private method
sub san_nas_workload{
    my ($node_src_name,$operation,$src_vol,$client,$data_protocols)=@_;
    my $script = 'DPSAS/IO/run_create_tree.thpl';
    my @clients = _list_of_vals($client);
    push my @subtests,
            NATE::Process->new(
                codespec => $script,
                runid => "run_create_tree_$_",
                params => {
                    'CLUSTERS' => $node_src_name,
                    'OPERATION_TYPE' => $operation,
                    'VOL_PATTERN_MATCH' => $src_vol,
                    'TEST_VOLS' => $src_vol,
                    'CLIENTS' => @clients,
                    'BYTES_PER_SEC' => 'nolimit',
                    'DATA_PROTOCOLS' => $data_protocols,
                },
                inherit_params => 1,
                onexit=> \&NATE::Process::on_exit_propagate_worst_result,
            );
    my $pm = NATE::ProcManager->new(proc_info=> \@subtests);
    $pm->start();
    $pm->destroy();
}

# private method
sub one_to_one_cg_relationship{
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            src_vol      => { type => OBJECT | SCALAR, optional => 1 },
            dst_vol      => { type => OBJECT | SCALAR, optional => 1 },
            src_vserver  => { type => OBJECT | SCALAR, optional => 1 },
            dst_vserver  => { type => OBJECT | SCALAR, optional => 1 },
            client       => { type => OBJECT | SCALAR, optional => 1 },
            src_cg_name  => { type => OBJECT | SCALAR, optional => 1 },
            dst_cg_name  => { type => OBJECT | SCALAR, optional => 1 },
            cg_item_list => { type => ARRAYREF,        optional => 1 },
            node_dst     => { type => OBJECT | SCALAR, optional => 1 },
            node_src     => { type => OBJECT | SCALAR, optional => 1 },
            tries        => { type => SCALAR, optional => 1, default => 8 },
            stagger_delay => { type => OBJECT | SCALAR, optional => 1 },
            validate => { type => SCALAR, optional => 1, default => 'false' },
            retry_list_dest_count => {type => SCALAR, default=>30},
            add_label => { type => SCALAR, optional => 1, default => 'none' },
        },
    );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Src_Cg_Name         = $args{'src_cg_name'};
    my $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $cg_item_list        = $args{'cg_item_list'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $client              = $args{'client'};
    my $validate            = $args{'validate'};
    my $retry_list_dest_count    = $args{'retry_list_dest_count'};
    my $tries               = $args{'tries'};
    my $add_label        = $args{'add_label'};
    _stagger($args{'stagger_delay'});

    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

    #find if relationship already exists
    my $dst_snapmirror_obj = NACL::C::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $Src_Vserver_Name.":".$Src_Cg_Name,
            'destination-path' => $Dst_Vserver_Name.":".$Dst_Cg_Name,
            },
            allow_empty => 1
            );

    # If Cg doesn't exist then create that cg and intialize that cg else just initialize that cg
    if ( !$dst_snapmirror_obj ) {
        sm_create_cg(
            'src_vol'      => $src_vol_name,
            'dst_vol'      => $dst_vol_name,
            'src_vserver'  => $Src_Vserver_Name,
            'dst_vserver'  => $Dst_Vserver_Name,
            'client'       => $client,
            'src_cg_name'  => $Src_Cg_Name,
            'dst_cg_name'  => $Dst_Cg_Name,
            'cg_item_list' => $cg_item_list,
            'tries'        => $tries,
            'node_dst'     => $Dst_Node_Obj,
            'node_src'     => $Src_Node_Obj,
            'add_label'    => $add_label,
        );

    }

    sm_initialize_cg(
            'src_vol'     => $src_vol_name,
            'dst_vol'     => $dst_vol_name,
            'src_vserver' => $Src_Vserver_Name,
            'dst_vserver' => $Dst_Vserver_Name,
            'client'      => $client,
            'src_cg_name' => $Src_Cg_Name,
            'dst_cg_name' => $Dst_Cg_Name,
            'node_dst'    => $Dst_Node_Obj,
            'node_src'    => $Src_Node_Obj,
            'validate'    => $validate,
            'retry_list_dest_count' => $retry_list_dest_count,
            );

    return;
}

# private method
sub one_to_one_adr_relationship{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol          => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            client            => {type => OBJECT|SCALAR, optional=>1},
            src_cg_name       => {type => OBJECT| SCALAR, optional=>1},
            dst_cg_name       => {type => OBJECT|SCALAR, optional =>1},
            node_dst        => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            stagger_delay     => {type => OBJECT|SCALAR ,optional =>1},

            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Src_Cg_Name         = $args{'src_cg_name'};
    my $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $client              = $args{'client'};
    _stagger($args{'stagger_delay'});

# Right now apiset call - to be replaced with NACL call later
    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }

#find if relationship already exists
    my $dst_snapmirror_obj = NACL::C::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            },
            allow_empty => 1
            );

# If ADR relationship doesnt exist then create and initialize
    if(!$dst_snapmirror_obj){

        $log->comment('Run snapmirror create to setup adr relationship between them');
        sm_create_adr('src_vol'     => $src_vol_name,
                'dst_vol'     => $dst_vol_name,
                'src_vserver' => $Src_Vserver_Name,
                'dst_vserver' => $Dst_Vserver_Name,
                'client'      => $client,
                'node_dst'    => $Dst_Node_Obj,
                'node_src'    => $Src_Node_Obj);
    }

    sm_initialize_adr('src_vol'     => $src_vol_name,
            'dst_vol'     => $dst_vol_name,
            'src_vserver' => $Src_Vserver_Name,
            'dst_vserver' => $Dst_Vserver_Name,
            'client'      => $client,
            'node_dst'    => $Dst_Node_Obj,
            'node_src'    => $Src_Node_Obj);
}

# private method
sub one_to_one_vault_relationship{
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            src_vol          => {type => OBJECT| SCALAR, optional=>1},
            dst_vol           => {type => OBJECT| SCALAR, optional=>1},
            src_vserver       => {type => OBJECT|SCALAR, optional => 1},
            dst_vserver       => {type => OBJECT| SCALAR, optional=>1},
            client            => {type => OBJECT|SCALAR, optional=>1},
            src_cg_name       => {type => OBJECT| SCALAR, optional=>1},
            dst_cg_name       => {type => OBJECT|SCALAR, optional =>1},
            node_dst        => {type => OBJECT|SCALAR, optional => 1},
            node_src          => {type => OBJECT|SCALAR, optional => 1},
            stagger_delay     => {type => OBJECT|SCALAR ,optional =>1},

            },
            );

    my $src_vol_name        = $args{'src_vol'};
    my $dst_vol_name        = $args{'dst_vol'};
    my $Src_Vserver_Name    = $args{'src_vserver'};
    my $Dst_Vserver_Name    = $args{'dst_vserver'};
    my $Src_Cg_Name         = $args{'src_cg_name'};
    my $Dst_Cg_Name         = $args{'dst_cg_name'};
    my $Dst_Node_Obj        = $args{'node_dst'};
    my $Src_Node_Obj        = $args{'node_src'};
    my $client              = $args{'client'};
    _stagger($args{'stagger_delay'});


    if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
        $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
    }

    if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
        $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
    }
    my $node_name = $Dst_Node_Obj->node();
    my $host = host($node_name);

#find if relationship already exists
    my $dst_snapmirror_obj = NACL::C::Snapmirror->find(
            command_interface => $Dst_Node_Obj,
            filter            => {
            'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
            'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
            },
            allow_empty => 1
            );


# If Vault relationship doesnt exist then create (initialize will do implicit create if its not created ) else will just initialize
    if(!$dst_snapmirror_obj){
        $log->comment('Run snapmirror create to create vault relationship between them');
        sm_create_vault('src_vol'     => $src_vol_name,
                'dst_vol'     => $dst_vol_name,
                'src_vserver' => $Src_Vserver_Name,
                'dst_vserver' => $Dst_Vserver_Name,
                'client'      => $client,
                'node_dst'    => $Dst_Node_Obj,
                'node_src'    => $Src_Node_Obj);

    }
    sm_initialize_vault('src_vol'     => $src_vol_name,
            'dst_vol'     => $dst_vol_name,
            'src_vserver' => $Src_Vserver_Name,
            'dst_vserver' => $Dst_Vserver_Name,
            'client'      => $client,
            'node_dst'    => $Dst_Node_Obj,
            'node_src'    => $Src_Node_Obj);

}

# private method
sub create_parallel_processes{
    my($func_name,$params)=@_;
    my %params_hash = %$params;
    my @subtests;
    my $subtest = NATE::Process->new(
            codespec => \&$func_name,
            runid => sprintf("%03d_$func_name$$", ++$counter),
            args => [%params_hash],
            inherit_params => 1,
            onexit => \&NATE::Process::on_exit_die_worst_result,
            );
    return $subtest;
}

# private method
sub run_processes_bg{
    my($func_name,$subtest,$max_active_children)=@_;
    my @subtests  = _list_of_vals($subtest);

    if( scalar @subtests == 0 ) {
        NATE::BaseException->throw('No processes in @subtests.');
    }

    if ( ! $max_active_children ) {
        $max_active_children = 16;
    }

    my $pm = NATE::ProcManager->new(
            runid               => 'PM_' . $func_name,
            max_active_children => $max_active_children,
            proc_info           => \@subtests,
            );
    $pm->start();
    $pm->wait();
    $pm->destroy();

}

# private method
sub _verify_list_destination_status {

    my($Src_Node_Obj,$dst_vol_name,$src_vol_name,$Src_Vserver_Name,$Dst_Vserver_Name,$status,$retry_count)= @_;

# this code can be used when destination node object is provided - will eliminate snoozing.
=begin
    my $dst_node_name = $dst_vol->get_local_node_name();
    my $Dst_Node_Obj = NACL::C::Node->new( name => $dst_node_name );

    nacl_method_retry(
        code => sub {
      my $state = NACL::CS::Snapmirror->fetch(
        command_interface => $Dst_Node_Obj,
        filter            => {
         'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
         'source-path'      => $Src_Vserver_Name.":".$src_vol_name,
    }
    );

      if ( ( $state->state ) != /Snapmirrored/ ) {
         my $err = "Relationship is not $status";
            $log->comment($err);
            NACL::Exceptions::UnexpectedState->throw(
                $err,
                monitoring_fields => ['state'],
                status => $state->state(),
                );
            }
            },
            tries_count => 30,
            sleep_time => 60, #wait for a litle while to get into Snapmirrored State
                exceptions => 'NACL::Exceptions::UnexpectedState',
            );
=cut
    my $retries=0;
    my $list_dst;

    if (!defined $retry_count) {
	$retry_count = 30;
    }

    while ($retries < 10)
    {
	my $check=0;
	try{
	    $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
		    command_interface => $Src_Node_Obj,
		    filter => {
		    'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
		    'source-path'      => $Src_Vserver_Name.":".$src_vol_name, },
		    allow_empty => 0,
		    );
	}catch NATE::BaseException with {
	    my $exception = shift;
	    $check=1;
	    my $text      = $exception->text();
	    $log->comment( "Caught exception $exception" . "TXT $text" );
	};
	if ($check==1){
	    my $num=$retries+1;
	    $log->warn("Retry number $num failed to find snapmirror relationship with list-destinations");
	}
	else { last };
	$retries++;
	if ($retries < 10){
	    $log->comment("Snoozing for 60 seconds and trying again");
	    sleep(60);
	}
    }

    $log->log("Verify that list-destinations show relationship status as $status");
    nacl_method_retry(
	code => sub {
	    my $list_dst = NACL::CS::SnapmirrorListDestinations->fetch(
		command_interface => $Src_Node_Obj,
		filter => {
		    'destination-path' => $Dst_Vserver_Name.":".$dst_vol_name,
		    'source-path'      => $Src_Vserver_Name.":".$src_vol_name, },
		    allow_empty        => 0,);

	    if ($list_dst->status() !~ /$status/){
		my $err = "Relationship is not $status";
		$log->comment($err);
		NACL::Exceptions::UnexpectedState->throw(
		    $err,
		    monitoring_fields => ['status'],
		    status => $list_dst->status(),
		);
	    }
	},
	tries_count => $retry_count,
	sleep_time => 60, #wait for a litle while to get InSync
        exceptions => 'NACL::Exceptions::UnexpectedState',
    );
}

# Input: Client name, Volume obj (or Mount obj)
# Returns: Client, Volume, Mount objects
# Mount volume on the client, if mount obj not passed
# private method
sub _mount_vol {
    my ( $volume, $client, $mount ) = @_;

    my ( $Mount, $Client, $Volume );

    if (ref($client) =~ /Client/i) {
        $Client = $client;
    } elsif ($client) {
        $Client = NACL::C::Client->new( name => $client );
    } else {
        $Client = NACL::C::Client->find();
    }

    if ($mount && ref($mount) =~ /Mount/i ) {
        $Log->debug( "Using given mount object" );
        $Mount = $mount;
        $Volume = $volume;
    } else {
        if (ref($volume) =~ /Volume/i ) {
            $Volume = $volume;
        } else {
            die "Only Volume object param expected, scalar passed.\n"
        }

        if ($Volume->get_one_state_attribute('junction-path') eq "-") {
            $Volume->mount( 'junction-path' => "/".$Volume->volume() );
        }

        $Mount = NACL::STask::Client::Mount->create(
                command_interface   => $Client,
                server_command_interface => $Volume->command_interface,
                vserver             => $Volume->vserver,
                volume              => $Volume,
                exclusive           => 1,
                mountopts          => 'nfsvers=3',
        );
    }

    $Log->debug("Mount obj: " . Dumper $Mount );
    $Log->debug("Client obj: " . Dumper $Client );
    $Log->debug("Volume obj: " . Dumper $Volume );
    return ($Mount, $Client, $Volume);
}

# Setup every thing needed for iscsi lun
# * creates igroup
# * lun
# * map lun,
# * On Linux client map and mkfs and mount the lun
# private method
sub lun_setup {
    my ($Node, $vserver, $volume,
        $lun_names, $igroup,
        $Client,) = @_;

    my $initiator = _get_iscsi_client_initiator($Client);
    my @Lun_sizes = ('1g');
     $log->comment("And I got this lun-names for main workflow:\n" . Dumper($lun_names));

    my $obj = NACL::MTask::LunSetup->setup(
                command_interface                => $Node,
                vserver                          => $vserver,
                volume                           => $volume,
                nacltask_lun                     => $lun_names,
                nacltask_size                    => \@Lun_sizes,
                igroup                           => $igroup,
                initiator                        => $initiator,
                data_protocol                    => 'iscsi',
                ostype                           => 'linux',
                protocol                         => 'mixed',
                #nacltask_datalif_opts            => \%DataLif_Opts,
                nacltask_if_lif_exists           => 'reuse', #default die
                nacltask_if_lun_exists           => 'reuse', #default die
                nacltask_if_igroup_exists        => 'reuse', #default die
                nacltask_if_lun_mapped_igroup    => 'remap', #default die
                #nacltask_ignore_lif_checks       => 0,       #default 0
                #nacltask_ignore_dataproto_checks => 0,       #default 0
              );

    $Log->comment( "All objects : " );
    $Log->comment( Dumper($obj->all_objects()) );

    # Create file system on lun and mount the lun
    my $luns = $obj->all_objects()->{lun};
    $Log->comment( Dumper($luns) );


    my $VserverIscsi =
        NACL::C::VserverIscsi->new(
            command_interface=>$Node,
            vserver =>$vserver,
        );

    my $iqn_name = $VserverIscsi->state()->target_name();
    $log->comment( "Filer IQN name : $iqn_name");
    my ($ip_address) = _get_iscsi_lif($Node, $vserver);

    $log->comment(Dumper $luns);

    my $interface_state = NACL::CS::NetworkInterface->fetch(
        command_interface => $Node,
        filter            => { vserver => $vserver, address => $ip_address },
        requested_fields  => ['lif']
    );

    $Client = NACL::C::Client->new( name => $Client );
    my @mount_paths;
    my %Luns_hash;
    foreach my $lun (@$luns) {
        $log->debug( "Lun path : " . $lun->{path} );
         my $lun_path =  $lun->{path};
        my(undef,$Src_pqth,$src_vol_name,$lun_name) = split(/\//,$lun_path);
        $Log->comment( "lun name" . Dumper($lun_name) );

        my $mount_obj = NACL::MTask::Client::LunMount->setup(
                command_interface => $Client,
                'lun-path'   => $lun->{path},
                'filer_command_interface' => $Node,
                 vserver                   => $vserver,
                 'filer_lif'               => $interface_state->{'lif'},
                #'ip-address' => $ip_address,
                #'filer-iqn'  => $iqn_name,
            );
        my $src_device = NACL::MTask::Client::LunMount->lun_discovery(
            command_interface       => $Client,
            filer_command_interface => $Node,
            vserver                 => $vserver,
            filer_lif               => $interface_state->{'lif'},
            'lun-path'              => $lun->{path}
        );
        $log->debug( Dumper($src_device) );
        #push ( @mount_paths, $src_device);
        $Luns_hash{$lun_name} = $src_device;
    }
    $log->comment("And I got this lun hash for main workflow:\n" . Dumper(%Luns_hash));

    return %Luns_hash;
}

=head2 get_audit_log

Retrieves contents of the audit log

=over

=item Example

        This produces a diff between the audit log across a time interval.

        my @audit_log_init_state = get_audit_log( $node_dst->node() );

        #
        # diff the two arrays of the audit_log entries to verify
        #

        Tharn::snooze( <n seconds> );

        my @audit_log_next_state = get_audit_log( $node_dst->node() );

        my $lc = List::Compare->new(
            '--unsorted',
            \@audit_log_next_state,
            \@audit_log_init_state,
        );

        #
        # generates a diff between the state and next_state list compare.
        #
        my @audit_log_diff = $lc->get_unique();

        foreach my $line ( @audit_log_diff ) {
            $log->comment( 'AUDIT LOG ENTRY FOUND: ' . $line );
        }

=item Options

=over

=item node

(Required) node name - the natehost record of the node_src the source volume resides on.

=item command

(optional) command - cat to get whole logs or tail to get partial logs.

=item lines

(optional) lines - number of lines to get using tail.

=back

=item Return

    Returns an array of raw lines from the audit log.

=back

=cut

sub get_audit_log {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node    => { type => SCALAR, optional => 0 },
            command => { type => SCALAR, optional => 1, regex => qr/cat|tail/, default => 'cat' },
            lines   => { type => SCALAR, optional => 1, default => 30 },},
    );

    my $node = NACL::C::Node->new( name => $args{'node'} );

    my $cmd = ($args{'command'} eq 'cat') ? "cat /mroot/etc/log/snapmirror_audit"
        : "tail -n $args{'lines'} /mroot/etc/log/snapmirror_audit";

    my $api = $node->apiset( interface => 'CLI', set => 'Systemshell' );

    my $raw = $api->execute_raw_command( command => $cmd, 'connectrec-timeout' => '300' );

    return split( /\n/, $raw );
}


=head2 enable_disable_scheduled_pcs

enable disable scheduled pcs creation

=over

=item Options

(Required) node - node object to set the flag.
(optional) enable - flag to set
if '0' will disable scheduled PCS
if '1' will enable scheduled PCS

=item Return

    none.

=back

=cut

sub enable_disable_scheduled_pcs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT,  optional => 0 },
            enable => { type => BOOLEAN, optional => 0 },
                       }
    );

    my $node = $args{'node'};
    my $flag = $args{'enable'} ? '0' : '1';

    my $Node_Obj = NACL::STask::Node->new( name => $node->name() );

    my $api = $Node_Obj->get_7m_or_nodescope_apiset();
    $Node_Obj->privilege_level('test');

#run local setflag sm_disable_pcs_sched 1
    my $cmd = "setflag sm_disable_pcs_sched $flag";
    $api->execute_command( command => $cmd, );

    return;
}

=head2 enable_disable_autoresync

enable disable smgr auto-resync

=over

=item Options

(Required) node - node object to set the flag.
(optional) enable - flag to set
if '0' will disable smgr auto-resync
if '1' will enable ssmgr auto-resync

=item Return

    none.

=back

=cut

sub enable_disable_autoresync {

    my %args = validate_with(
        params => \@_,
        spec   => {
            node   => { type => OBJECT,  optional => 0 },
            enable => { type => BOOLEAN, optional => 0 },
        },
    );

    my $node = $args{node};
    my $flag = $args{enable} ? 'false' : 'true';

    my $Node_Obj = NACL::STask::Node->new( name => $node->name() );

    my $api = $Node_Obj->get_7m_or_nodescope_apiset();
    $Node_Obj->privilege_level('test');
    my $ba  = 'bootarg.sm_cg_auto_resync_disabled';
    my $cmd = "bootargs get $ba";
    my $result = $api->execute_command( command => $cmd, );
    $result =~ s/[\r\n]//gm;
    $result =~ s/${ba} == //;

    if($result ne $flag) {
        my $cmd = "sudo sysctl kern.bootargs=$ba=$flag";
        my $api = $Node_Obj->apiset( interface => 'CLI', set => 'Systemshell' );
        $api->execute_raw_command( command => $cmd );
    } else {
        $log->comment($ba.' already set to '.$flag);
    }

    return;
}

=head2 get_latest_smgr_snapshot

retrieve the latest smgr snapmirror snapshot

=over

=item Options

=over

=item node

(Required) node - the natehost record of the node_src the source volume resides on.
(Required) volume name - the source volume of the smgr relationship.
(Optional) ss_regex - Default to 'snapmirrorCG.*.*'.
(Optional) Returns the last X latest SS that matches the ss_regex passed in. Default 0

=back

=item Return

    none.

=back

=cut

sub get_latest_smgr_snapshot {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node   => { type => OBJECT, optional => 0 },
            volume => { type => SCALAR, optional => 0 },
            ss_regex => { type => SCALAR, optional => 1, default =>'snapmirrorCG.*.*' },
            last_x_ss => { type => SCALAR, optional => 1, default => 0 },
        },
    );

    my $vol_name = $args{'volume'};
    my $node = $args{'node'};
    my $last_x_ss = $args{'last_x_ss'};
    my $log = log_global();

    $log->comment('Get all snapshots');
    my @snapshot_pcs = NACL::CS::Snapshot->fetch(
        command_interface => $node,
        filter            => {
            volume   => $vol_name,
            snapshot => $args{ss_regex},
            #snapshot => 'snapmirrorCG.*.*',
        },
    );

    my  $return_val;

    if (! $last_x_ss) {
        $log->comment('Returning the latest SS ');
        my $latest_snapshot =
          ( sort { $a->logical_snap_id <=> $b->logical_snap_id }
              @snapshot_pcs )[-1];
        $return_val = $latest_snapshot->snapshot();
    } else {
       $log->comment('Returning the last ' . $last_x_ss . ' latest ss');
       my @latest_ss = ( sort { $a->logical_snap_id <=> $b->logical_snap_id }@snapshot_pcs );
       my $arr_cnt = scalar(@latest_ss) - 1;
       my $cnt = 0;
       my @temp;

       while ($cnt < $last_x_ss) {
            push (@temp, $latest_ss[$arr_cnt]);
            $cnt ++;
            $arr_cnt--;
       }

       foreach my $t (@temp) {
            push (@{ $return_val }, $t->snapshot());
       }
    }

    $log->comment('Return val is: ' . Dumper($return_val));
    return $return_val;
}

=head2 create_max_snapshots

create max snapshots on a given volume

=over

=item Options

=over

=item node

(Required) object - the natehost record of the node.

=item volume

(Required) object - volume to create SS on.

=back

=item Return

    Returns the number of snapshots that were created.
    exception thrown in failure.

=back

=cut

sub create_max_snapshots {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node          => { type => OBJECT, optional => 0 },
            volume        => { type => OBJECT, optional => 0 },
            snapshot_name => { type => SCALAR, optional => 1, default => 'MySS' },
        },
    );

    my $node = $args{'node'};
    my $volume = $args{'volume'};
    my $snapshot_name = $args{'snapshot_name'};

    my $ss_autodel = NACL::CS::VolumeSnapshotAutodelete->fetch(
        command_interface => $node,
        filter            => {
            volume  => $volume->volume(),
            vserver => $volume->vserver(),
        },
        requested_fields => [qw(enabled)],
    );
    if ( $ss_autodel->enabled() =~ /true/ ) {
        $ss_autodel->modify( enabled => 'false' );
    }

    #create snapshots till it fails
    my $reached_last;
    my $total_ss;
    for (my $i = 0; $i < 255; $i ++){
        try {
            my $snapshot = $snapshot_name . $i;
            $volume->snapshot_create(snapshot => $snapshot,);
            $total_ss = $i;
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = $_[0];
            # TBD make this into an exception in NACL
            if ($exception->text() =~
                /Failed to create snapshot.*(No more snapshots available|Cannot exceed maximum number of Snapshot copies)/i) {
                $reached_last = 1; # exit loop
            } elsif($exception->text() =~  /Snapshot operation failed: No space left on device/i)  {
                #smaller size volumes get full sooner and may not require 255 SS. Hence handling this case here.
                $log->warn('Volume is full. Probably you are trying to run heave IO or ' .
                           'the vol is full. Resize the vol to accept max vols');
                $reached_last = 1;
            } else {
                $exception->throw();
            }
        };
        last if ($reached_last);
    }

    return $total_ss;
}


# private method
sub _get_iscsi_client_initiator {
    my ($Client) = @_;

    if (ref($Client) =~ /Client/i) {
        $Client = $Client;
    } elsif ($Client) {
        $Client = NACL::C::Client->new( name => $Client );
    } else {
        $Client = NACL::C::Client->find();
    }

    my $Initiator;
    my $output = $Client->apiset(interface => "CLI")->execute_raw_command(
            command=> '/bin/cat /etc/iscsi/initiatorname.iscsi');
    if ( $output =~ /InitiatorName=(\S+)$/ ) {
        $Initiator = [ $1 ];
    }

    return $Initiator;
}

# private method
sub _get_iscsi_lif {
    my ($Node, $vserver) = @_;
    my $Data_Lif_Iscsi;
    eval {
        ($Data_Lif_Iscsi) =
            NACL::CS::NetworkInterface->fetch(
                command_interface => $Node,
                filter => {
                    'role'          => 'data',
                    'data-protocol' => [qw(iscsi)],
                    vserver         => $vserver,
                },
                requested_fields => [qw(address netmask)],
                allow_empty => 1,
            );
    };

    return ($Data_Lif_Iscsi->address(), $Data_Lif_Iscsi->netmask())
        if ($Data_Lif_Iscsi);
}

# Use create_tree.pl tool and populate dataset
# private method
sub ds_create_tree {
    my ($client,$operation,$size,$path) = @_;

    # pass populate for operation create for create_tree.pl
    $operation = 'populate' if ($operation eq "create");
    my $CrTree_Bin = "/usr/bin/sudo /usr/software/test/noarch/agnostic/DPSAS/IO/create_tree.pl";

    my $opts = " DEDUPE_PCT=50 COMPRESSION_PCT=30 DIR_DEPTH=2 DIR_CNT=10 FILE_CNT=1 FILE_SIZE=1k";
    $opts .=   " FILE_SIZE=$size OPERATION_TYPE=$operation";
    $opts .=   " NUM_RUNS=1 " if ($operation eq 'modify');
    $opts .=   " $path";

    my $cmd = "$CrTree_Bin $opts";
    my $client_obj;
    if (ref($client) =~ /Client/i) {
        $client_obj = $client;
    } elsif ($client) {
        $client_obj = NACL::C::Client->new( name => $client );
    }

    # touch the file before passing to create_tree.pl (other wise it will fail)
    #$client_obj->apiset(interface => "CLI")->touch(path => "$path");

    my $create_tree_raw =
        $client_obj->apiset(interface => "CLI")->execute_raw_command(
            command              => $cmd,
            'connectrec-timeout' => 36000,
        );
    if (($operation eq "modify") &&
            ($create_tree_raw !~ /Modify iteration \d+ finished in \d+ secs/)){
        NATE::BaseException->throw(
            "Failed to modify $path using create_tree.pl"
            . " Output from command: $create_tree_raw"
        );
    }
    if (($operation eq "populate") &&
            ($create_tree_raw !~ /Created a total of \d+ directories and \d+ files/)){
        NATE::BaseException->throw(
            "Failed to populate $path using create_tree.pl"
            . " Output from command: $create_tree_raw"
        );
    }

    return 1;
}

# private method
sub start_dt_traffic{
    my ($lun,$dt_traffic_obj) =@_;
    my $lun_size_bytes = $lun->state->size();
    my $dt_limit_bytes = int ($lun_size_bytes * .01); #use 1% of the total lun size for each dt pass. There will be multiple dt passes
    my $modulo = $dt_limit_bytes % 512;
    $dt_limit_bytes -= $modulo;
    if ($dt_limit_bytes < 512){
        $dt_limit_bytes = 512;
    }
    $dt_traffic_obj->start(
            tool              => 'dt',
            iotype            => 'random',
            limit             => $dt_limit_bytes,
            cdelay            => '5', #this puts a 5 second delay at the end of each pass
            oflags            => 'trunc',
            bs                => '4k',
            incr              => 'var',
            min               => '2k',
            max               => '256k',
            pattern           => 'iot',
            runtime           => '-1',
            dispose           => 'keeponerror',
            prefix            => '"%d@%h"',
            disable           => 'pstats',
            enable            => 'compare,lbdata,debug,syslog',
            'privilege-level' => 'root',
        );
}

# private method
sub _stop_dt_traffic {
    $Test->step('Stop dt traffic on every source lun');
    my $dt_traffic_obj = @_;
    $dt_traffic_obj ->stop();

}

# private method
sub _list_of_vals{
    my $obj = shift;
    my @objs;
    if( ref($obj) eq 'ARRAY'){
        push @objs , @$obj;
    }
    else{
        push @objs, $obj;
    }
    return @objs;

}

# private method
sub run_processes_bg_iteration {  #used for abort and update - interval capability
    my ( $func_name, $subtest, $iterations, $iteration_delay ) = @_;
    my @subtests = _list_of_vals($subtest);
    my $pm = NATE::ProcManager->new(
        runid               => 'PM_' . $func_name,
        max_active_children => 16,
        proc_info           => \@subtests,
#        proc_definition     => [
#              { iterations => $iterations, iteration_delay => $iteration_delay }
#        ],
#        iterate_individually => 1,
        iterate_individually => 0,
        parallel            => 1,
        background          => 1,
        iterations          => $iterations,
        on_iter_end         => sub { sleep($iteration_delay) },

    );
    $pm->start();
    $pm->wait();
    $pm->destroy();

}


=head2 Wf_lretolrse

To perform lre to lrse update.

=over

=item Options

=over

=item node_src

(Optional) String value - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) String value - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref value - The array consisting of snapmirror objects
on which abort operation would be performed

=item validate

(Optional)  Value 0 or 1 . 1 to validate . By default 0.


=back

=back

=cut

sub Wf_lretolrse {
    my %args = validate_with(
                     params      => \@_,
                     allow_extra => 1,
                     spec        => {
                         node_src => { type => OBJECT | SCALAR, optional => 1 },
                         node_dst => { type => OBJECT | SCALAR, optional => 0 },
                         sm_objs  => { type => ARRAYREF,        optional => 0 },
                         validate => { type => SCALAR,          optional => 1, default => 0 },
                         stagger_delay => { type => SCALAR, optional => 1 },
                     },
    );

    my $validate     = $args{ 'validate' };
    my $node_src_Obj = $args{ 'node_src' };
    my $node_dst_Obj = $args{ 'node_dst' };
    my $updates      = $args{ 'sm_objs' };
    my @Updates      = _list_of_vals($updates);
    my @subtests_update;

    $counter = 0;
    foreach my $o (@Updates) {
        my %args_func = (

            'node_src'    => $node_src_Obj,
            'dst_path'    => $o->destination_path,
            'src_path'    => $o->source_path,
            'dst_volume'  => $o->destination_volume(),
            'node_dst'    => $node_dst_Obj,
            'dst_vserver' => $o->destination_vserver(),
            'validate'    => $validate

        );

        push @subtests_update,
          create_parallel_processes( 'sm_lretolrse', \%args_func );

    }
    run_processes_bg( 'sm_lretolrse', \@subtests_update );

}

=head2 Wf_snapmirror_cleanup

This is a 'CANNED' workflow.
Provides the ability to quiesce,break,delete and release a number of relationships.
Verifies that the given snapmirror objects are removed from source and destination side.

=over

=item Options

=over

=item node_src

(Required) String value - the natehost record of the node_src the
source volume resides on.

=item node_dst

(Required) String value - the natehost record of the node_dst the
destination volume resides on

=item sm_objs

(Required) Arrayref of snapmirror objs(stask)

=item validate

(Optional) Calls the validations method (validations are listed inside the
method), default value is false, i.e. skip validation.

=item delete_policy

(Optional)SCALAR. Deletes the policy. Default value is 1

=back

=back

=cut

sub Wf_snapmirror_cleanup {
    my %args = validate_with(
            params => \@_,
            allow_extra => 1,
            spec   => {
            node_src           => {type => OBJECT|SCALAR, optional=>1},
            node_dst           => {type => OBJECT|SCALAR, optional=>1},
            sm_objs            => {type => ARRAYREF, optional => 1},
            validate           => {type => SCALAR, optional => 1, default =>'false'},
            delete_policy      => {type => SCALAR, optional => 1, default=>1},
            },
            );

    my $Src_Node_Obj        = $args{node_src};
    my $Dst_Node_Obj        = $args{node_dst};
    my $cleanup             = $args{'sm_objs'};
    my $validate            = $args{'validate'};
    my @Cleanup            = _list_of_vals($cleanup);
    my $delete_policy      = $args{'delete_policy'};

    my @subtests_cleanup;
    $counter = 0;
    foreach my $o(@Cleanup) {
        my $dst_path = $o->destination_path;
        $log->comment("####destination path is" . Dumper($dst_path));

        my %args_func = (
                'node_src'         => $Src_Node_Obj,
                'node_dst'         => $Dst_Node_Obj,
                'dst_path'         => $dst_path,
                'src_path'         => $o->source_path,
                'validate'         => $validate,
                'stagger_delay'    => 10,
                'delete_policy'    => $delete_policy
                );
        push @subtests_cleanup , create_parallel_processes('sm_cleanup',\%args_func);

    }
    run_processes_bg('sm_cleanup',\@subtests_cleanup);

}

# private method
sub sm_abort {
    my %args = validate_with(
                  params      => \@_,
                  allow_extra => 1,
                  spec        => {
                      node_src    => { type => OBJECT | SCALAR | UNDEF, optional => 1 },
                      dst_path    => { type => OBJECT | SCALAR, optional => 0 },
                      src_path    => { type => OBJECT | SCALAR | UNDEF, optional => 1 },
                      dst_volume  => { type => OBJECT | SCALAR, optional => 0 },
                      node_dst    => { type => OBJECT | SCALAR, optional => 0 },
                      dst_vserver => { type => OBJECT | SCALAR, optional => 0 },
                      validate    => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
                      validate_transfering => { type => SCALAR, optional => 1 },
                      sleep_before_abort   => { type => SCALAR, optional => 1 },
                      ignore_no_transfer_to_abort   => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
                      ignore_verify_transferring_err   => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
                  },
    );
    my $validate                       = $args{ 'validate' };
    my $node_dst_Obj                   = $args{ 'node_dst' };
    my $node_src_Obj                   = $args{ 'node_src' };
    my $dst_vserver_Name               = $args{ 'dst_vserver' };
    my $dst_volume                     = $args{ 'dst_volume' };
    my $src_path                       = $args{ 'src_path' };
    my $dst_path                       = $args{ 'dst_path' };
    my $sleep_before_abort             = $args{ 'sleep_before_abort' };
    my $validate_transfering           = $args{ 'validate_transfering' };
    my $ignore_no_transfer_to_abort    = $args{ 'ignore_no_transfer_to_abort' };
    my $ignore_verify_transferring_err = $args{ 'ignore_verify_transferring_err' };

    if ( ($node_dst_Obj) && ( ref($node_dst_Obj) !~ /Node/i ) ) {
        $node_dst_Obj = NACL::STask::Node->new( name => $node_dst_Obj );
    }
    my $o =
      NACL::STask::Snapmirror->find(
                                     command_interface => $node_dst_Obj,
                                     filter            => {
                                                'source-path'      => $src_path,
                                                'destination-path' => $dst_path,
                                     },
                                     allow_empty => 1
      );
    $log->comment( ' SnapMirror state is : ' . Dumper($o->state()) );
    $log->comment('Abort the transfer');

    if(exists($args{'validate_transfering'})){
		verify_transferring( sm_objs => $o, ignore_verify_transferring_err => $ignore_verify_transferring_err );
	}

    if(exists($args{'sleep_before_abort'})){
    		$log->comment('Sleeping for '.$sleep_before_abort.' seconds before aborting');
		Tharn::snooze($sleep_before_abort);
	}

    try {
        $o->abort( nacltask_verify => 1, 'method-timeout' => 300 );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        my $text      = $exception->text();
        $log->comment( "Caught exception $exception" . "TXT $text" );
        if($text =~ /No transfer to abort/ && $ignore_no_transfer_to_abort eq 'true') {
            $log->warn( "Continuing as exception is: $text" );
        } else {
	    $exception->throw();
        }
    };
    if ( $validate eq 'true' ) {
         my $checkpoint = $o->snapshot_checkpoint;
         if ( $checkpoint <= 0 ) {
            $log->error("Checkpoint has not been created. Got:$checkpoint. Check if abort has been issued atleast 30 seconds after the transfer has started.");
        } else {
            $log->log("Got expected non-zero snapshot-checkpoint. Got:$checkpoint");
        }
    } else {
        $log->warn('Skipping validation of snapmirror abort as requested');
    }
}

# private method
sub sm_abort_hard {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src    => { type => OBJECT | SCALAR, optional => 1 },
            dst_path    => { type => OBJECT | SCALAR, optional => 0 },
            src_path    => { type => OBJECT | SCALAR, optional => 0 },
            dst_volume  => { type => OBJECT | SCALAR, optional => 0 },
            node_dst    => { type => OBJECT | SCALAR, optional => 0 },
            dst_vserver => { type => OBJECT | SCALAR, optional => 0 },
            validate    => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },

        },
    );
    my $validate         = $args{ 'validate' };
    my $node_dst_Obj     = $args{ 'node_dst' };
    my $node_src_Obj     = $args{ 'node_src' };
    my $dst_vserver_Name = $args{ 'dst_vserver' };
    my $dst_volume       = $args{ 'dst_volume' };
    my $src_path         = $args{ 'src_path' };
    my $dst_path         = $args{ 'dst_path' };

    if ( ($node_dst_Obj) && ( ref($node_dst_Obj) !~ /Node/i ) ) {
        $node_dst_Obj = NACL::STask::Node->new( name => $node_dst_Obj );
    }
    my $o =
      NACL::STask::Snapmirror->find(
                                     command_interface => $node_dst_Obj,
                                     filter            => {
                                                'source-path'      => $src_path,
                                                'destination-path' => $dst_path,
                                     },
                                     allow_empty => 1
      );
    $log->comment( ' SnapMirror state is : ' . Dumper($o->state()) );
    $log->comment('Hard Abort the transfer');

    #$o->abort(hard =>1, nacktask_verify =>1);
    try {
        $o->abort( 'hard' => 'true', nacltask_verify => 1, 'method-timeout' => 300 );
    }
    catch NATE::BaseException with {
        my $exception = shift;
        my $text      = $exception->text();
        $log->comment( "Caught exception $exception" . "TXT $text" );
	$exception->throw();
    };

    if ( $validate eq 'true' ) {
         my $checkpoint = $o->snapshot_checkpoint;
         if ( $checkpoint != '-' ) {
            $log->error("Checkpoint not reverted to -. Expected:-. Got:$checkpoint");
         } else {
            $log->log("Got expected snapshot-checkpoint. Expected:'-'. Got:$checkpoint");
         }
    } else {
        $log->warn('Skipping validation of snapmirror abort hard as requested');
    }
}

# private method
sub sm_restore {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src    => { type => OBJECT | SCALAR, optional => 0 },
            dst_path    => { type => OBJECT | SCALAR, optional => 0 },
            src_path    => { type => OBJECT | SCALAR, optional => 0 },
            dst_volume  => { type => OBJECT | SCALAR, optional => 0 },
            node_dst    => { type => OBJECT | SCALAR, optional => 0 },
            dst_vserver => { type => OBJECT | SCALAR, optional => 0 },
            snapshot    => { type => SCALAR, optional => 1},
            validate    => { type => SCALAR,          optional => 1 },

        },
    );
    my $validate         = $args{ 'validate' };
    my $node_dst_Obj     = $args{ 'node_dst' };
    my $node_src_Obj     = $args{ 'node_src' };
    my $dst_vserver_Name = $args{ 'dst_vserver' };
    my $dst_volume       = $args{ 'dst_volume' };
    my $src_path         = $args{ 'src_path' };
    my $dst_path         = $args{ 'dst_path' };
    my $snapshot         = $args{'snapshot'};

    if ( ($node_dst_Obj) && ( ref($node_dst_Obj) !~ /Node/i ) ) {
        $node_dst_Obj = NACL::STask::Node->new( name => $node_dst_Obj );
    }
    my $o =
      NACL::STask::Snapmirror->find(
                                     command_interface => $node_dst_Obj,
                                     filter            => {
                                                'source-path'      => $src_path,
                                                'destination-path' => $dst_path,
                                     },
                                     allow_empty => 1
      );
    $log->comment( ' SnapMirror state is : ' . Dumper($o->state()) );
    $log->comment('restore operation on snapmirror relationship');

    #$o->restore( nacltask_verify => 1 );
    my %restore_args;
    $restore_args{'source-snapshot'} = $snapshot if (defined($snapshot));
    NACL::STask::Snapmirror->restore(
                command_interface  => $node_src_Obj,
                'source-path'      => $dst_path,
                'destination-path' => $src_path,
                 %restore_args);


}

# private method
sub sm_update {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src    => { type => OBJECT | SCALAR | UNDEF, optional => 1 },
            dst_path    => { type => OBJECT | SCALAR, optional => 0 },
            src_path    => { type => OBJECT | SCALAR | UNDEF, optional => 1 },
            dst_volume  => { type => OBJECT | SCALAR, optional => 0 },
            node_dst    => { type => OBJECT | SCALAR, optional => 0 },
            dst_vserver => { type => OBJECT | SCALAR, optional => 0 },
            snapshot    => { type => SCALAR, optional => 1},
            throttle    => { type => SCALAR, optional => 1},
            validate    => { type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
            nacltask_wait => {type => SCALAR, optional => 1, default => 1},
        },
    );
    my $validate         = $args{ 'validate' };
    my $node_dst_Obj     = $args{ 'node_dst' };
    my $node_src_Obj     = $args{ 'node_src' };
    my $dst_vserver_Name = $args{ 'dst_vserver' };
    my $dst_volume       = $args{ 'dst_volume' };
    my $src_path         = $args{ 'src_path' };
    my $dst_path         = $args{ 'dst_path' };
    my $snapshot         = $args{ 'snapshot' };
    my $throttle         = $args{ 'throttle' };

    if ( ($node_dst_Obj) && ( ref($node_dst_Obj) !~ /Node/i ) ) {
        $node_dst_Obj = NACL::STask::Node->new( name => $node_dst_Obj );
    }
    my $o =
      NACL::STask::Snapmirror->find(
                                     command_interface => $node_dst_Obj,
                                     filter            => {
                                                'source-path'      => $src_path,
                                                'destination-path' => $dst_path,
                                     },
                                     allow_empty => 1
      );
    $log->comment( ' SnapMirror state is : ' . Dumper($o->state()) );
    $log->comment('update operation on snapmirror relationship');

    my %update_args;
    $update_args{'source-snapshot'} = $snapshot if (defined($snapshot));
    $update_args{'throttle'} = $throttle if (defined($throttle));
    $update_args{nacltask_wait} = $args{nacltask_wait};
    $update_args{'method-timeout'} = 3600;

    $log->comment('snapmirror relationship status check');
    $o->wait_on_attributes( # avoid another sm operation
	'method-timeout' => 600,
	attributes       => [
	    {
		attribute_to_check => 'status',
		till_value         => qr/idle/i,
	    },
	]
    );

    my $st = time;
    my $op_done = 0;
    while (time < $st + 600) { # workaround for conflict ops
	try {
	    $o->update(nacltask_verify => 1, %update_args);
	    $op_done = 1;
	    last;
	} catch NACL::APISet::Exceptions::CommandFailedException with {
	    my $ex = shift;
	    if ($ex->text =~ /Another SnapMirror operation is in progress|Try again/i) {
		$log->warn('Another SnapMirror operation is in progress. Retry update in 60s.');
		Tharn::snooze(60);
	    } else {
		$ex->throw();
	    }
	}
    }
    NATE::BaseException->throw('update failed after 10 mins retry') if !$op_done;

    if ($args{nacltask_wait}) {
        # do extra validations if validate is 'true'
        if ($validate eq 'true') {
            Validate_Update( sm_obj   => $o,
                             snapshot => $snapshot,
                            );
        } else {
            $log->warn('Skipping validation of snapmirror update as requested');
        }
    } else {
        $log->warn('Skipping validation of snapmirror update since nacltask_wait is 0');
    }
}

# private method
sub sm_lretolrse {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src    => { type => OBJECT | SCALAR, optional => 1 },
            dst_path    => { type => OBJECT | SCALAR, optional => 0 },
            src_path    => { type => OBJECT | SCALAR, optional => 0 },
            dst_volume  => { type => OBJECT | SCALAR, optional => 0 },
            node_dst    => { type => OBJECT | SCALAR, optional => 0 },
            dst_vserver => { type => OBJECT | SCALAR, optional => 0 },
            validate    => { type => SCALAR,          optional => 1 }

        },
    );
    my $validate         = $args{ 'validate' };
    my $node_dst_Obj     = $args{ 'node_dst' };
    my $node_src_Obj     = $args{ 'node_src' };
    my $dst_vserver_Name = $args{ 'dst_vserver' };
    my $dst_volume       = $args{ 'dst_volume' };
    my $src_path         = $args{ 'src_path' };
    my $dst_path         = $args{ 'dst_path' };

    if ( ($node_dst_Obj) && ( ref($node_dst_Obj) !~ /Node/i ) ) {
        $node_dst_Obj = NACL::STask::Node->new( name => $node_dst_Obj );
    }


   if ( ($node_src_Obj) && ( ref($node_src_Obj) !~ /Node/i ) ) {
        $node_src_Obj = NACL::STask::Node->new( name => $node_src_Obj );
    }


    my $o =  NACL::STask::Snapmirror->find(
                                     command_interface => $node_dst_Obj,
                                     filter            => {
                                                'source-path'      => $src_path,
                                                'destination-path' => $dst_path,
                                     },
                                     allow_empty => 1
      );
    $log->comment( ' SnapMirror state is : ' . Dumper($o->state()) );
    my $inter_detector;

    if ( $validate == 1 ) {

        my $sm_state = $o->state();
        my $rst_id;
        if ($sm_state) {
            $rst_id = $sm_state->relationship_id;
            $Log->comment( "Relationship-ID: " . $rst_id );
        }
        $inter_detector =
          NACL::MTask::SystemLogDetector->new(
                      command_interface =>
                        NACL::C::Node->new(
                          name => $node_dst_Obj->get_one_state_attribute('node')
                        ),
                      set_log_file           => ['snapmirror_audit'],
                      check_for_any_presence => [qr/$rst_id/],
                      set_log_dir            => '/mroot/etc/log'
          );
        $inter_detector->start();

    }

   my $src_vserver;
   my $src_volume;

   ($src_vserver,$src_volume)=split(/:/,$src_path);


   my $Src_obj = NACL::C::Volume->find(
            command_interface => $node_src_Obj,
            filter            => {
            volume  => $src_volume,
            vserver => $src_vserver,
            },
            );


   my $Policy_Obj = NACL::CS::Snapmirror->fetch(
            command_interface => $node_dst_Obj,
            filter=> {
            'destination-path' =>$dst_path,
            },
            requested_fields => [qw(policy)]);

    my $sm_lbl = NACL::CS::SnapmirrorPolicy->fetch(command_interface    => $node_dst_Obj,
            filter=> {
            policy  => $Policy_Obj->policy,
            },
            requested_fields => [qw(snapmirror-label)]);

    my @sm_lbls  = $sm_lbl->snapmirror_label;
    my $sm_label=  $sm_lbls[0];
    my $sm_type;
    my $s1= create_snapshot($Src_obj,"snap1_sfr_$$",'XDP' , $sm_label);

    sleep(30);



    #check compression - do off

    # my $dst_volume_name = $dst_volume->volume();
    NACL::STask::VolumeEfficiency->modify(
                                           command_interface => $node_dst_Obj,
                                           vserver => $dst_vserver_Name,
                                           volume  => $dst_volume,
                                           'inline-compression' => 'false',
                                           compression          => 'false'
    );

    $log->comment('lretolrse operation on snapmirror relationship');
    $o->update( 'enable-storage-efficiency' => 'true', nacltask_verify => 1 );
    sleep(30);

    if ( $validate == 1 ) {

        my $val_to_assert =
          'Logical Transfer with Storage Efficiency - .* Directory Mode';
        my $rst_id;
        my $sm_state = $o->state();
        if ($sm_state) {
            $rst_id = $sm_state->relationship_id;
            $Log->comment( "Relationship-ID: " . $rst_id );
        }
        sleep(30);

        try {
            $inter_detector->check( check_for_any_presence => [qr/$rst_id/], );
        }
        catch NACL::Exceptions::EventCheckFailure with {
            $Log->error(
                "Unable to get grep $rst_id from " . "snapmirror_audit files" );
        };

        my @logs_inter = $inter_detector->get_matched_logs();
        $Log->comment( 'get_matched_logs output' . Dumper( \@logs_inter ) );

        my $str           = \@logs_inter;
        my @logs_internal = @$str;

        for ( my $i = 0 ; $i < $#logs_internal ; $i++ ) {
            if ( $logs_internal[$i] =~ /transfer_desc=(.*)/ ) {
                my $xfer_desc = $1;
                $Log->comment("Transfer description value is: $xfer_desc");
                $Log->comment("The value to be matched is: $val_to_assert");
                if ( $xfer_desc =~ /$val_to_assert/ ) {
                    $Log->comment(
"Transfer occured in $xfer_desc which is the expected value" );
                }
                else {
                    my $error_text =
"The transfer description value should be $val_to_assert but Value found in snapmirror audit logs is $xfer_desc ";
                    NACL::Exceptions::UnexpectedState->throw($error_text);
                }

            }

        }

    }

}


=head2 create_pcs

This method will create a PCS, find the PCS name and validate the snapshot tags.
It will also validate that the old PCS SS has been sucessfully rotated.
It will return the PCS name on success.
Validations are not done by default. If validations fails an exception will be thrown.

=over

=item Example

    my $pcs_name = create_pcs( node_dst => $node_dst,
                                vol_dst  => $vol_dst,
                                dst_path => $sm_dest_path,
                                validate => 'true',);

=item Options

=over

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item vol_dst

(Required): Destination volume Object.

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item vol_src

(Required): Source volume Object.

=item dst_path

(Required): snapmirror destination path.

=item validate

(Optional): If set to 'true', will validate the snapshot tags to confirm if its a PCS snapshot or not.
Default: 'false'.
Validations are not done by default. If validations fails an exception will be thrown.

=item tries_count

(Optional): This is used when we wait for the PCS to be created. Number of iterations. Default 12.

=item sleep_time

(Optional): This is used when we wait for the PCS to be created. Number of sec to snooze between iterations. Default 10.

=back

=item Exceptions

    NATE::BaseException

=back

=cut

sub create_pcs {

    my %args = validate_with(
            params => \@_,
            spec   => {
                node_dst => {type => OBJECT, },
                vol_dst  => {type => OBJECT, },
                node_src => {type => OBJECT, },
                vol_src  => {type => OBJECT, },
                dst_path => {type => SCALAR, },
                validate => {type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
                tries => {type => SCALAR, default => 12 },
                sleep_time => {type => SCALAR, default => 10 },
		promote_fiji_used  => {type => SCALAR, default => 'false', regex => qr/^(true|false)$/ },
            }
        );

    my $node_dst = $args{node_dst};
    my $node_src = $args{node_src};
    my $vol_dst = $args{vol_dst};
    my $vol_src = $args{vol_src};

    # extract latest SS name on DST and SRC
    my $prepcs_snapshot_name_dst = get_latest_smgr_snapshot(node=>$node_dst,volume=>$vol_dst->volume);
    my $prepcs_snapshot_name_src = get_latest_smgr_snapshot(node=>$node_src,volume=>$vol_src->volume);
    $log->comment('Latest snapshot on DST before PCS creation '.$prepcs_snapshot_name_dst);
    $log->comment('Latest snapshot on SRC before PCS creation '.$prepcs_snapshot_name_src);

    # create pcs
    $log->comment('Create PCS on destination:'.$args{dst_path});
    my @tempcg = split( '/', $args{dst_path}); # split to extact cg name from dest path
    my $command = 'dctrl pcs_create '.$tempcg[-1];
    # make sure pcs_create is run on local node that owns the dst volume
    my $local_dst_node_name = $vol_dst->get_one_state_attribute('node');
    if ($local_dst_node_name ne $node_dst->node()) {
	$log->comment('...get true local node...');
	$node_dst = NACL::C::SystemNode->new (
			    name => $local_dst_node_name );
    }
    my $apiset_dst = $node_dst->get_7m_or_nodescope_apiset();
    my $raw_output_dst = $apiset_dst->execute_command( command => $command, );

    $log->comment('Sleep for 5 minutes for the pcs workflow to finish retrying');
    Tharn::snooze(300);

    # make sure we have the right PCS - the one we created
    my $pcs_snapshot_name_src = wait_for_newer_smgr_snapshot(site => 'src',
                                                            node=>$node_src, vol=>$vol_src,
                                                            last_snapshot_name=>$prepcs_snapshot_name_src,
                                                            tries_count=>$args{tries}, sleep_time=>$args{sleep_time});
    my $pcs_snapshot_name_dst = wait_for_newer_smgr_snapshot(site => 'dst',
                                                            node=>$node_dst, vol=>$vol_dst,
                                                            last_snapshot_name=>$prepcs_snapshot_name_dst,
                                                            tries_count=>$args{tries}, sleep_time=>$args{sleep_time});
    $log->comment('PCS on DST '.$pcs_snapshot_name_dst);
    $log->comment('PCS on SRC '.$pcs_snapshot_name_src);




    if ($args{validate} eq 'true') {
        my $found_old_ss = 0;
        my $exception_text;
	my $retries = 5;
	my $n = 0;



        $log->comment('Ensure the old ss ' . $prepcs_snapshot_name_dst . ' has been deleted from the dst');


	while ($n < $retries) {
	    $found_old_ss = 0;
	    try {
		NACL::STask::VolumeSnapshot->find(
		    command_interface => $node_dst,
		    filter            => {
			vserver       => $vol_dst->vserver(),
			volume        => $vol_dst->volume(),
			snapshot      => $prepcs_snapshot_name_dst
		    });
		$found_old_ss = 1;
		$n ++;
		Tharn::snooze(30);
	    } catch NATE::BaseException with {
		$log->comment('Exception expected because old SS should not have been present on dst.');
	    };
	    last if (!$found_old_ss);
	}

        if ($found_old_ss && $args{promote_fiji_used} eq 'false') {
            my $err_txt = ' Old PCS SS ' . $prepcs_snapshot_name_dst . ' still present on the dst';
            $log->error($err_txt);
            $exception_text .= $err_txt;
        }

        $n = 0;
        $log->comment('Ensure the old ss ' . $prepcs_snapshot_name_src . ' has been deleted from the src');
	while ($n < $retries) {
	    $found_old_ss = 0;
	    try {
		NACL::STask::VolumeSnapshot->find(
		    command_interface => $node_src,
		    filter            => {
			vserver       => $vol_src->vserver(),
			volume        => $vol_src->volume(),
			snapshot      => $prepcs_snapshot_name_src,
		    });
		$found_old_ss = 1;
		$n ++;
		Tharn::snooze(30);
	    } catch NATE::BaseException with {
		$log->comment('Exception expected because old SS should not have been present on src.');
	    };
	    last if (!$found_old_ss);
	}



        if ($found_old_ss && $args{promote_fiji_used} eq 'false') {
            my $err_txt = ' Old PCS SS ' . $prepcs_snapshot_name_src . ' still present on the src';
            $log->error($err_txt);
            $exception_text .= $err_txt;
        }

        $log->comment('Snoozing for 60 sec for snaptags to be applied');
        Tharn::snooze(60);

        # validate names of PCS on SRC and DST - should be the same
        if ($pcs_snapshot_name_dst ne $pcs_snapshot_name_src) {
            my $error_text = 'PCS snapshot name mismatch. DST PCS name:'.$pcs_snapshot_name_dst.' SRC PCS name:'. $pcs_snapshot_name_src;
            $log->error($error_text);
            $exception_text .= $error_text;
        } else {
            $log->comment('PCS names on SRC and DST match');
        }

        $log->comment('Do a fresh find on the dst vol to get the latest info');
        my $n_vol_dst = NACL::C::Volume->find (
                    command_interface => $node_dst,
                    filter            => {
                        volume        => $vol_dst->volume(),
                    });

        $vol_dst = $n_vol_dst;
        my $local_dst_node_name = $vol_dst->get_one_state_attribute('node');
        if ($local_dst_node_name ne $node_dst->node()) {
            $node_dst = NACL::C::SystemNode->new (
                                name => $local_dst_node_name );
        }

        # validate tags on PCS on DST
        my $tag_check = Validate_PCS_Tags(volname => $vol_dst->volume(),
                                        node => $node_dst,
                                        snapshot => $pcs_snapshot_name_dst);
        if ($tag_check==0) {
            my $error_text = 'Did not find PCS tags on DST snapshot '.$pcs_snapshot_name_dst;
            $log->error($error_text);
            $exception_text .= $error_text;
        } else {
            $log->comment('Found PCS tags on DST snapshot '.$pcs_snapshot_name_dst);
        }

        $log->comment('Do a fresh find on the src vol to get the latest info');
        my $n_vol_src = NACL::C::Volume->find (
                    command_interface => $node_src,
                    filter            => {
                        volume        => $vol_src->volume(),
                    });

        my $vol_src = $n_vol_src;
        #get_one_state_attribute only works on NACL::C::Volume and not on
        #NACL::CS::Volume
        my $local_src_node_name = $vol_src->get_one_state_attribute('node');
        if ($local_src_node_name ne $node_src->node()) {
            $node_src = NACL::C::SystemNode->new (
                                name => $local_src_node_name );
        }
        # validate tags on PCS on SRC
        $tag_check = Validate_PCS_Tags(volname => $vol_src->volume(),
                                        node => $node_src,
                                        snapshot => $pcs_snapshot_name_src);
        if ($tag_check==0) {
            my $error_text = 'Did not find PCS tags on SRC snapshot '.$pcs_snapshot_name_src;
            $log->error($error_text);
            $exception_text .= $error_text;
        } else {
            $log->comment('Found PCS tags on SRC snapshot '.$pcs_snapshot_name_src);
        }

        # collect all the failures and throw execption if any
        if ($exception_text) {
            NATE::BaseException->throw($exception_text); # for now use BaseExcption TBD find another exception
        } else {
            $log->log('Validate of PCS '.$pcs_snapshot_name_dst.' succeeded');
        }
    } else {
        $log->comment('Skipping validation of PCS tags as requested');
    }

    return $pcs_snapshot_name_dst;
}

=head2 wait_for_newer_smgr_snapshot

Given a snapshot name, wait for a newer smgr snapshot.
On success, will return the new smgr snapshot name.
On failure, will throw an exception

=over

=item Example

    Typical usage could be,

    my $smgr_snapshot_name = get_latest_smgr_snapshot(
                                        node=>$node_dst,
                                        volume=>$vol_dst->volume);

    # do something here.... that will generate a new SMGR snapshot

    my $new_smgr_snapshot = wait_for_newer_smgr_snapshot(
                                site => 'dst',
                                node => $node_dst,
                                vol  => $vol_dst,
                                last_snapshot_name => $smgr_snapshot_name,);

=item Options

=over

=item node

(Required): Node Object of type NACL::C::Node.

=item vol

(Required): volume Object.

=item last_snapshot_name

(Required): Last smgr snapshot name that is existing on the node.

=item tries_count

(Optional): Number of iterations. Default 12.

=item sleep_time

(Optional): Number of sec to snooze between iterations. Default 10.

=item site
(Optional): To differentiate if the new SS create failed on the SRC or DST
            Expected values: src or dst

=item ss_regex

(Optional) Default to 'snapmirrorCG.*.*'.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=back

=cut

sub wait_for_newer_smgr_snapshot {
    my %args = validate_with(
            params => \@_,
            spec   => {
                site => {type => SCALAR },
                node => {type => OBJECT, },
                vol  => {type => OBJECT, },
                last_snapshot_name => {type => SCALAR, },
                tries_count => {type => SCALAR, default=>12 },
                sleep_time  => {type => SCALAR, default=>10 },
                ss_regex => { type => SCALAR, optional => 1, default=>'snapmirrorCG.*.*' },
            }
        );
    my $site = $args{site};
    my $last_snapshot_name = $args{last_snapshot_name};

    my $tries_count = $args{tries_count};
    my $retry_counter = $tries_count;
    my $found=0;
    my $sleep_time = $args{sleep_time};
    my $new_snapshot_name = $last_snapshot_name;
    while ($retry_counter--) {

        $new_snapshot_name = get_latest_smgr_snapshot(node=>$args{node},volume=>$args{vol}->volume,ss_regex=>$args{ss_regex});

        # exit loop if new SS is cut
        if ($last_snapshot_name ne $new_snapshot_name) {
            my $attempts=$tries_count-$retry_counter;
            $log->comment('Found new snapshot '.$new_snapshot_name.
                        ' in '.$attempts.' iterations with sleep of '.
                        $sleep_time.' sec between iterations.');
            $found=1;
            last;
        }
        $log->comment('Snoozing for '.$sleep_time.' sec for new SS');
        Tharn::snooze($sleep_time);
    }
    if (!$found) {
        my $error_text = 'New snapshot was not created on the '. $site .
                        ' on node: ' . $args{node}->name() . ' even after '.
                        $tries_count.' iterations with sleep of '.
                        $sleep_time.' sec between iterations';
        NATE::BaseException->throw($error_text); # for now use BaseExcption TBD find another exception
    }


    return $new_snapshot_name;
}


=head2 Wf_add_cg_item

This method will quiesce the relationship if it is already initialized and add a cg-item(s), and resume afterwards, validate that relationship comes into Insync state after addition is done.
In case of uninitialized relationship no quiesce/Resume is required.
Also,It validates that items are successfully added to the cg relationship and got replicated to the destination as well.
Validations are not done by default. If validations fails an exception will be thrown.

=over

=item Example

    Wf_add_cg_item(sm_objs => $sm_obj ,
                   src_node => $src_node,
                   dst_node => $dst_node,
                   cg_item_list => ['lun1_obj','lun2_obj'],
                   initialize => 0,
                   validate => 'true',);

=item Options

=over

=item sm_objs

(Required): ARRARREF of snapmirror objects of type NACL::STask::Snapmirror.

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item cg_item_list

(Required): list of cg items to be added to the cg relationship.

=item initialize

(Optional) If set, will initialize the relationship after composition change. Default:0

=item validate

(Optional): If set to 'true', will validate that relationship comes to Insync after adding items and items are visible on destination.
Default: 'false'.
Validations are not done by default. If validations fails an exception will be thrown.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=back

=cut

sub Wf_add_cg_item{

    my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                sm_objs      => {type => ARRAYREF, optional => 0},
                node_src     => {type => OBJECT| SCALAR, optional=>0},
                node_dst     => {type => OBJECT| SCALAR, optional=>0},
                cg_item_list => {type => ARRAYREF},
                initialize   => {type => SCALAR, default => 0,},
                validate     => {type => SCALAR, default => 'false'},
                cg_type      => {type => SCALAR, default => 'lun'},
                },
                );

        my $sm_obj              = delete $args{'sm_objs'};
        my @add_objs            = _list_of_vals($sm_obj);
        my $validate            = $args{'validate'};
        my $cg_item_arr_ref     = $args{cg_item_list};
        my @cg_items_array      = _list_of_vals($cg_item_arr_ref);
        my $initialize          = $args{initialize};
        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $cg_type             = $args{'cg_type'};

        if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
             $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
        }

        if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
             $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
        }

       my $name;
       my $path;
       my @src_item_names;

       foreach my $item(@cg_items_array){
           if ($cg_type eq 'lun') {
               $name = $item->lun();
           }else {
              $path = $item->path();
             ($name = $path) =~ s{.*/}{};
           }
           push(@src_item_names,$name);
       }

        my @subtests_add;
        foreach my $o(@add_objs) {
            my $source_path = $o->source_path;
            my $destination_path = $o->destination_path;
            # get the volume and vserver names of src and dest
            my $cg_data = NACL::C::Snapmirror->get_consistency_group_data(
                command_interface => $Dst_Node_Obj,
                'destination-path' => $destination_path,
            );
            my ($src_vserver_name,$src_vol_name,$dst_vol_name,$dst_vserver_name);
            for my $key (sort keys %{$cg_data}) {
               ($src_vserver_name,$src_vol_name) = ($cg_data->{$key}->{source_path} =~ /(.*):\/vol\/(.*)\//);
               ($dst_vserver_name,$dst_vol_name) = ($cg_data->{$key}->{destination_path} =~ /(.*):\/vol\/(.*)\//);
               $log->comment('Got source vserver:'.$src_vserver_name.' src vol:'.$src_vol_name.
               ' dest vserver:'.$dst_vserver_name.' dest vol:'.$dst_vol_name);
            }
            my $Src_obj = NACL::C::Volume->find(
               command_interface => $Src_Node_Obj,
               filter            => {
                  volume  => $src_vol_name,
                  vserver => $src_vserver_name,
               },
            );

            my $Dst_obj = NACL::C::Volume->find(
                command_interface => $Dst_Node_Obj,
                filter            => {
                   volume  => $dst_vol_name,
                   vserver => $dst_vserver_name,
                },
            );
            my %args_func = ('node_src'    => $Src_Node_Obj,
                             'node_dst'    => $Dst_Node_Obj,
                             'src_path'    => $source_path,
                             'dst_path'    => $destination_path,
                             'vol_src'     => $Src_obj,
                             'vol_dst'     => $Dst_obj,
                             'cg_item_list' => \@src_item_names,
                             'initialize'   => $initialize,
                             'validate'    => $validate,
                             'stagger_delay'    => 10,
                            );
            push @subtests_add, create_parallel_processes('wf_add_cg_item',\%args_func);
        }

        run_processes_bg('wf_add_cg_item',\@subtests_add);

}


=head2 Wf_delete_cg_item

This method will delete cg items from a granular sync snapmirror relationship.

If the relationship is initialized, it will quiesce it before deleting and resume afterwards.

Validation is performed if validate is 'true'. Default is 'false'.

Validation will be to check if the relationship comes InSync after the composition change.
Validation also includes checking if the deleted cg items are read/write'able.

Exceptions will be thrown on failure

=over

=item Example

    Wf_delete_cg_item(sm_objs => $sm_obj },
                      src_node => $src_node,
                      dst_node => $dst_node,
                      cg_item_list => ['lun1_obj','lun2_obj'],
                      initialize => 0,
                      validate => 'true',);

=item Options

=over

=item sm_objs

(Required): ARRARREF of snapmirror objects of type NACL::STask::Snapmirror.

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item cg_item_list

(Required): list of cg items to be added to the cg relationship.

=item initialize

(Optional) If set, will initialize the relationship after composition change. Default:0

=item validate

(Optional): If set to 'true', will validate that relationship comes to Insync after adding items and items are visible on destination.
Default: 'false'.
Validations are not done by default. If validations fails an exception will be thrown.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=back

=cut

sub Wf_delete_cg_item{

    my %args = validate_with(
                params => \@_,
                allow_extra => 1,
                spec   => {
                sm_objs      => {type => ARRAYREF, optional => 0},
                node_src     => {type => OBJECT| SCALAR, optional=>1},
                node_dst     => {type => OBJECT| SCALAR, optional=>1},
                cg_item_list => {type => ARRAYREF},
                initialize   => {type => SCALAR, default => 0,},
                validate     => {type => SCALAR, default => 'false',},
                cg_type      => {type => SCALAR, default => 'lun'},
                },
                );

        my $sm_obj              = $args{'sm_objs'};
        my @add_objs            = _list_of_vals($sm_obj);
        my $validate            = $args{'validate'};
        my $cg_item_arr_ref     = $args{cg_item_list};
        my @cg_items_array      = _list_of_vals($cg_item_arr_ref);
        my $initialize          = $args{initialize};
        my $Src_Node_Obj        = $args{'node_src'};
        my $Dst_Node_Obj        = $args{'node_dst'};
        my $cg_type             = $args{'cg_type'};

        if ( ($Src_Node_Obj) && (ref($Src_Node_Obj) !~ /Node/i) ) {
             $Src_Node_Obj = NACL::STask::Node->new( name => $Src_Node_Obj );
        }

        if ( ($Dst_Node_Obj) && (ref($Dst_Node_Obj) !~ /Node/i) ) {
             $Dst_Node_Obj = NACL::STask::Node->new( name => $Dst_Node_Obj );
        }

        my $name;
        my $path;
        my @src_item_names;

        foreach my $item(@cg_items_array){
           if ($cg_type eq 'lun') {
               $name = $item->lun();
           }else {
              $path = $item->path();
             ($name = $path) =~ s{.*/}{};
           }
           push(@src_item_names,$name);
       }


        my @subtests_del;
        foreach my $o(@add_objs) {
            my $source_path = $o->{source_path};
            my $destination_path = $o->{destination_path};
             # get the volume and vserver names of src and dest
            my $cg_data = NACL::C::Snapmirror->get_consistency_group_data(
                command_interface => $Dst_Node_Obj,
                'destination-path' => $destination_path,
            );
            my ($src_vserver_name,$src_vol_name,$dst_vol_name,$dst_vserver_name);
            for my $key (sort keys %{$cg_data}) {
               ($src_vserver_name,$src_vol_name) = ($cg_data->{$key}->{source_path} =~ /(.*):\/vol\/(.*)\//);
               ($dst_vserver_name,$dst_vol_name) = ($cg_data->{$key}->{destination_path} =~ /(.*):\/vol\/(.*)\//);
               $log->comment('Got source vserver:'.$src_vserver_name.' src vol:'.$src_vol_name.
               ' dest vserver:'.$dst_vserver_name.' dest vol:'.$dst_vol_name);
           }
            my $Src_obj = NACL::C::Volume->find(
                command_interface => $Src_Node_Obj,
                filter            => {
                    volume  => $src_vol_name,
                    vserver => $src_vserver_name,
                },
            );

            my $Dst_obj = NACL::C::Volume->find(
                command_interface => $Dst_Node_Obj,
                filter            => {
                    volume  => $dst_vol_name,
                    vserver => $dst_vserver_name,
                },
            );

            my %args_func = ('node_src'    => $Src_Node_Obj,
                             'node_dst'    => $Dst_Node_Obj,
                             'src_path'    => $source_path,
                             'dst_path'    => $destination_path,
                             'vol_src'     => $Src_obj,
                             'vol_dst'     => $Dst_obj,
                             'cg_item_list' => \@src_item_names,
                             'initialize'   => $initialize,
                             'validate'    => $validate,
                             'stagger_delay'    => 10,
                            );
            push @subtests_del, create_parallel_processes('wf_del_cg_item',\%args_func);
        }

        run_processes_bg('wf_del_cg_item',\@subtests_del);

}



=head2 wf_add_cg_item

This method will quiesce the relationship if it is already initialized and add a cg-item(s), and resume afterwards, validate that relationship comes into Insync state after addition is done.
In case of uninitialized relationship no quiesce/Resume is required.
Also,It validates that items are successfully added to the cg relationship and got replicated to the destination as well.
Validations are not done by default. If validations fails an exception will be thrown.

=over

=item Example

    wf_add_cg_item( node_src => $node_src,
	                node_dst => $node_dst,
					src_path => $src_path,
					dst_path => $dst_path,
					vol_src  => $vol_src
					vol_dst  => $vol_dst,
                                        cg_item_list => ['lun1_name','lun2_name'],
                                        initialize => 0,
					validate => 'true',);

=item Options

=over

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item src_path

(Required): snapmirror source path.

=item dst_path

(Required): snapmirror destination path.

=item vol_src

(Required): Source volume Object.

=item vol_dst

(Required): Destination volume Object.

=item cg_item_list

(Required): list of cg items to be added to the cg relationship.

=item initialize

(Optional) If set, will initialize the relationship after composition change. Default:0

=item validate

(Optional): If set to 'true', will validate that relationship comes to Insync after adding items and items are visible on destination.
Default: 'false'.
Validations are not done by default. If validations fails an exception will be thrown.

=back

=item Exceptions

    NACL::Exceptions::BaseException

=back

=cut

sub wf_add_cg_item{
    _add_del_cg_item(@_,action=>'add');
}

=head2 wf_del_cg_item

This method will delete cg items from a granular sync snapmirror relationship.

If the relationship is initialized, it will quiesce it before deleting and resume afterwards.

Validation is performed if validate is 'true'. Default is 'false'.

Validation will be to check if the relationship comes InSync after the composition change.
Validation also includes checking if the deleted cg items are read/write'able.

Exceptions will be thrown on failure.

=over

=item Example

    wf_del_cg_item( node_src => $node_src,
                    node_dst => $node_dst,
                    src_path => $snapmirror->{source_path},
                    dst_path => $snapmirror->{destination_path},
                    vol_src  => $vol_src
                    vol_dst  => $vol_dst,
                    cg_item_list => ['lun1_name','lun2_name'],
                    initialize => 0,
                    validate => 'true',
                    );

=item Options

=over

=item node_src

(Required): Source Node Object of type NACL::C::Node.

=item node_dst

(Required): Destination Node Object of type NACL::C::Node.

=item src_path

(Required): snapmirror source path.

=item dst_path

(Required): snapmirror destination path.

=item vol_src

(Required): Source volume Object.

=item vol_dst

(Required): Destination volume Object.

=item cg_item_list

(Required): List of cg items to be deleted from the cg relationship.
Note: The method will not do a pre-check to see if list of items provided for deletion are really part of the relationship or not.
Note: OnTap does not allow to establish a CG relationship with 0 cg items.
So if all the items in the relationship are provided for deletion, the entire CG relationship will be deleted.
The method does a quiesce if relationship needs to be quiesced, and does a resume later. In the above case, the resume will fail.

=item validate

(Optional): If set to 'true', will validate that relationship comes to InSync after deleting the cg items,
and if the show -expand output shows the change in cg composition,
and that the cg items are now read/writable on the destination.
Default: 'false'.
If the relationship was not initialized the validation will be limited to checking show -expand output only.

=item initialize

(Optional) If set, will initialize the relationship after composition change. Default:0

=item retry_list_dest_count

(Optional) Number of times to retry snapmirror list-destinations to verify status. Default: 30

=back

=item Exceptions

NACL::Exceptions::BaseException

=back

=cut

sub wf_del_cg_item {
    _add_del_cg_item(@_,action=>'del');
}

# private method
sub _add_del_cg_item {
    my %args = validate_with(
        params => \@_,
        spec   => {
            node_src     => {type => OBJECT},
            node_dst     => {type => OBJECT},
            src_path     => {type => SCALAR},
            dst_path     => {type => SCALAR},
	        vol_src      => {type => OBJECT},
	        vol_dst      => {type => OBJECT},
            cg_item_list => {type => ARRAYREF},
            initialize   => {type => SCALAR, default => 0,},
            validate     => {type => SCALAR, default => 'false',},
            retry_list_dest_count => {type => SCALAR, default => 30,}, # used with initialize
            action       => {type => SCALAR, regex=>qr/add|del/,}, # private arg
            stagger_delay      => {type => SCALAR, optional => 1},
        },
        allow_extra => 0,
    );

    my $node_src        = $args{node_src};
    my $node_dst        = $args{node_dst};
    my $src_path        = $args{src_path};
    my $dst_path        = $args{dst_path};
    my $src_vol         = $args{vol_src};
    my $dst_vol         = $args{vol_dst};
    my $cg_item_arr_ref = $args{cg_item_list};
    my $validate        = $args{validate};
    my @cg_items_array  = _list_of_vals($cg_item_arr_ref);
    my $src_vol_name    = $src_vol->volume;
    my $dst_vol_name    = $dst_vol->volume;
    my $action          = $args{action};
    my $initialize      = $args{initialize};
    my $retry_list_dest_count= $args{retry_list_dest_count};

    _stagger($args{'stagger_delay'});

    my $policy;
    if ($action eq 'add') {
        #Get the policy associated with the relationship.
        my $policy_obj = NACL::CS::Snapmirror->fetch( command_interface => $node_dst,
                                                filter=> {
                                                        'destination-path' =>$dst_path,
                                                },
                                                requested_fields => [qw(policy)]);
        $policy = $policy_obj->policy;
        $log->log('Policy is '.$policy);
    }

    # Use snapmirror delete to delete existing cg items from the cg relationship
    # Use snapmirror create to add new cg items to the cg relationship
    # quiesce if needed. Quiesce wil be needed if the relationship has been initalized.
    my %cg_item_mappings;
    foreach my $item(@cg_items_array){
        $cg_item_mappings{"/vol/$src_vol_name/$item"} = "/vol/$dst_vol_name/$item";
    }
    my $we_did_quiesce = 0;
    my $try_count = 2;
    my $total_wait_time = 0;
    my $max_wait_time = 360;
    while ($try_count) {
        $log->log('Issue '.$action.' of CG items '.Dumper(\%cg_item_mappings));
        try {
            if ($action eq 'del') {
                NACL::C::Snapmirror->delete(command_interface => $node_dst,
                                    'source-path'      => $src_path,
                                    'destination-path' => $dst_path,
                                    'cg-item-mappings' => \%cg_item_mappings,
                                    );
            } else {
                NACL::C::Snapmirror->create( command_interface => $node_dst,
                                    'source-path'      => $src_path,
                                    'destination-path' => $dst_path,
                                    'cg-item-mappings' => \%cg_item_mappings,
                                    type               => 'XDP',
                                    policy             => $policy,
                                    );
            }
            $try_count = 0;
        } catch NATE::BaseException with {
            my $exception = shift;
            my $text      = $exception->text();
            if ($text =~ /$NACL::AppBuilder::DpgWorkflows::StringConstants::CG_REL_NOT_QUIESED/) {
                # Quiesce the relationship and retry
                $log->log('Quiesce the relationship and validate quiesce.');
                sm_quiesce( node_src => $node_src,
                            node_dst => $node_dst,
                            dst_path => $dst_path,
                            src_path => $src_path,
                            validate => 'true',);
                $we_did_quiesce = 1;
                $try_count--;
            } elsif ($text =~ /$NACL::AppBuilder::DpgWorkflows::StringConstants::CG_MODIFY_IN_PROGRESS/) {
                # CG modify in progress - give it some time to finish and retry command - exit if we waited for too long
                if ($total_wait_time >= $max_wait_time) {
                    $log->error("Waited for $total_wait_time for CG modify to finish and retried. Modification did not finish. So aborting.");
                    $exception->throw($text);
                } else {
                    my $wait_time = 30;
                    $log->log("CG modify in progress. Give it some time to finish and retry. Snoozing for $wait_time sec.");
                    Tharn::snooze($wait_time);
                    $total_wait_time += $wait_time;
                }
            } else {
                $exception->throw($text);
            }
        };
    }

    # Resume relationship if we quiesced it
    if ($we_did_quiesce) {
        $log->log('Issue resume since we did quiesce before deleting cg items.');
        sm_resume( node_src => $node_src,
                   node_dst => $node_dst,
                   dst_path => $dst_path,
                   src_path => $src_path,
                   validate => 'true',);
    }

    if ($initialize) {
        # get src and dst cg name from src and dst path
        my ($src_vserver_name,$src_cg_path) = split(/:/,$src_path);
        my ($dst_vserver_name,$dst_cg_path) = split(/:/,$dst_path);
        sm_initialize_cg(node_dst   => $node_dst,
                        node_src    => $node_src,
                        src_vserver => $src_vserver_name,
                        dst_vserver => $dst_vserver_name,
                        src_cg_name => $src_cg_path,
                        dst_cg_name => $dst_cg_path,
                        validate    => 'true',
                        retry_list_dest_count => $retry_list_dest_count,
                        );
    }

    if ($validate eq 'true'){
        # do full validation if relationship was initialize or quiesced
        # otherwise skip since relationship is not ready for extensive checking
        my $validate_expand_only = ($we_did_quiesce|$initialize) ? 0 : 1;
        if ($action eq 'del') {
            Validate_del_cg_item(node_src    => $node_src,
                                node_dst     => $node_dst,
                                dst_path     => $dst_path,
                                src_path     => $src_path,
                                dst_vol_name => $dst_vol_name,
                                cg_item_list => $cg_item_arr_ref,
                                validate_expand_only => $validate_expand_only); # will skip validation of InSync and rw items checking
        } else {
            Validate_add_cg_item(node_src    => $node_src,
                                node_dst     => $node_dst,
                                dst_path     => $dst_path,
                                src_path     => $src_path,
                                dst_vol_name => $dst_vol_name,
                                cg_item_list => $cg_item_arr_ref,
                                validate_expand_only => $validate_expand_only); # will skip validation of InSync and rw items checking
        }
    } else {
        $log->log('Skipping validation after deletion of cg items');
    }

    return;
}



=head2 Wf_vault_lifecycle

This method will cover vault lifecycle.

=over

=item Example

    Wf_vault_lifecycle(sm_objs => $sm_obj,
                       nfsclients => $NFSCLIENTS,
                       sm_for_initialize_map => \@sm_init_map,
                       dst_files_map         => \@dst_files_map,
                       ss_for_update_map     => \@ss_for_update_map);


=item Options

=over

=item sm_objs

(Required): ARRARREF of snapmirror objects of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item ss_for_update_map
 (Required) ARRAYREF. An arrayref containing one Hashref element.
                     The hashref holds the map between the sm relationship and the corresponding ss
                     to use for intialization. Note the key naming convention
                     should be sm_obj_<0/1/2/3>. Key => SCALAR
            Example:
            $VAR = {
                    sm_obj_1 => 's1',
                    sm_obj_2 => 's2' };

=item ss_for_initialize_map
 (Required): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list
                      of ss that will be used for sm update. Note the key naming convention
                      shoudl be sm_obj_<0/1/2/3>. Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['s2','s3'],
                        sm_obj_2 => ['s4', 's5']};

=item dst_files_map
 (Optional): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list of
                      files to use to start traffic on.  Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['file1', 'file2'],
                        sm_obj_2 => ['file3', 'file4']};
 Default   : undef

=item validate
 (Optional): Whether to run validation at the end of the workflow.
 (Default): false

=item stagger_delay_map
 (Optional): An arrayref containing one Hashref element.
             The hashref holds the map of how many seconds to delay before invoking a process.
 (Default): 1

=back

=cut

sub Wf_vault_lifecycle {

    my %args = validate_with(
                params => \@_,
                spec   => {
                    sm_objs                  => { type => ARRAYREF, optional => 0 },
                    nfsclients               => { type => ARRAYREF|SCALAR, optional => 0 },
                    validate                 => { type => SCALAR, optional => 1, default => 'false' },
                    ss_for_initialize_map    => { type => ARRAYREF, optional => 0 },
                    ss_for_update_map        => { type => ARRAYREF, optional => 0 },
                    dst_files_map            => { type => ARRAYREF|UNDEF, optional => 1 },
                    stagger_delay_map        => { type => ARRAYREF, optional => 1 },
                    validate_holes           => { type => SCALAR, optional =>1 , default=>'false' },
		    extent_read              => { type => SCALAR, default => 0 },
                    rst_flag                 => { type => SCALAR, default => 'false'},
                },
                );


    my $sm_obj              = $args{'sm_objs'};
    my @vault_objs          = _list_of_vals($sm_obj);
    my $nfclients           = $args{'nfsclients'};
    my $ss_for_init_map     = $args{ss_for_initialize_map}->[0];
    my $ss_for_update_map   = $args{ss_for_update_map}->[0];
    my $dst_files_map       = $args{dst_files_map}->[0];
    my $count               = scalar(@vault_objs);
    my $validate            = $args{validate};
    my $stagger_map         = $args{stagger_delay_map}->[0];
    my $validate_holes      = $args{validate_holes};
    my $extent_read         = $args{extent_read} || 0;
    my $ss_del_map             = $args{ss_for_update_map}->[0];
    my $rst_flag             = $args{rst_flag};
    my @subtests_vault;

    for (my $i=0; $i < $count; $i++) {
        my $o = $vault_objs[$i];
        my $key = 'sm_obj_' . $i;
        my $ss_for_initialize = $ss_for_init_map->{$key};
        my $ss_for_update     = $ss_for_update_map->{$key};
        my $ss_del            = $ss_del_map->{$key};
        my $delay;
        if (!defined $stagger_map->{key}) {
            $delay = 1;
        } else {
            $delay = $stagger_map->{key};
        }
        _stagger($delay);

        my $dst_files;
        if (defined $dst_files_map) {
            $dst_files         = $dst_files_map->{$key};
        }
        $log->comment('For sm relationship with dst path: ' . $o->{destination_path} .
                      "\n" .
                      ' ss that will be used for initialize is: ' . $ss_for_initialize .
                      "\n" .
                      ' ss for updates are: ' . Dumper($ss_for_update) .
                      "\n" .
                      ' dst files on which traffic will be started are: ' . Dumper($dst_files                      ) .
                      "\n");

        my %args_func = (sm_obj             => $o,
                         ss_for_initialize  => $ss_for_initialize,
                         ss_for_update      => $ss_for_update,
                         ss_del             => $ss_del,
                         dst_files          => $dst_files,
                         validate           => $validate,
                         'nfsclients'       => $nfclients,
                         'validate_holes'   => $validate_holes,
			 extent_read        => $extent_read,
                         rst_flag           => $rst_flag);
         push @subtests_vault, create_parallel_processes('vault_workflow',\%args_func);
    }

    run_processes_bg('vault_workflow',\@subtests_vault);
}

=head2 vault_workflow

 backup lifecycle test - use s1 to initialize the backup, run updates with s2(TODO: with other snapshots in future) ,
 run abort during updates, run checkpoint restart, run hard-abort during updates. Restore one of the snapshots to a new
 volume (vol1), pick any snapshot to restore to vol1 (incremental restore), pick a few more snapshots to restore to vol1.
 Do read IO on destination files. Snapvault policy and schedule should use common settings. Use "common validation" at the end of test.

=over

=item Example

    vault_workflow( sm_obj => $sm_obj,
                    nfsclients  => $NFSCLIENTS,);

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::C::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item validate

(optional): SCALAR.Default vaule is false.

=item ss_for_update

(Required): ARRAYREF. List of snapshots that will be used for sm update
=item ss_for_initialize
 (Required): SCALAR. Snapshot that will be used to initialize the sm relationship

=item dst_files
 (Optional): ARRAYREF. The list of files to start traffic on.
 Default   : undef

=item validate_holes
 (Optional): SCALAR. The default value is false. It is to Validate that push_holes_cntr is getting incremented when holes are replicated when last transfer snapshot is s7.

=back

=cut

sub vault_workflow{
     my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients           => { type => ARRAYREF|SCALAR, optional => 0 },
            sm_obj               => { type => HASHREF, optional => 0 },
            validate             => { type => SCALAR, optional => 1, default => 0 },
            ss_for_initialize    => { type => SCALAR, optional => 0 }, #should I default to s1
            ss_for_update        => { type => ARRAYREF, optional => 0 },
            ss_del               => { type => ARRAYREF, optional =>1 },
            dst_files            => { type => ARRAYREF|UNDEF, optional => 1},
            validate_holes       => { type => SCALAR, optional =>1 , default=> 'false' },
            extent_read          => { type => SCALAR, default => 0 },
            rst_flag             => { type => SCALAR, default => 'false'},
        },
    );

    my $log = log_global();
    $log->enter();

    my $nfsclients  = $args{nfsclients};

    my $validate         = $args{validate} ;
    my $iterations       = $args{no_of_iterations};
    my $snapmirror_1     =  $args{sm_obj} ;
    my $ss_for_initialize= $args{ss_for_initialize};
    my $ss_for_update    = $args{ss_for_update};
    my $source_path      = $snapmirror_1->{source_path};
    my $destination_path = $snapmirror_1->{destination_path};
    my $dst_files        = $args{dst_files};
    my $validate_holes   = $args{validate_holes};
    my $ss_del           = $args{ss_del};
    my $rst_flag         = $args{rst_flag};
    my $before_update_ctr;
    my @before_update_ctr_arr;

    # get the volume and vserver names of src and dest
    my ($vserver_src,$vol_src_name) = ($source_path =~ /^(.*):(.*)$/);
    my ($vserver_dst,$vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);
    my $Dst_Node_Obj = $snapmirror_1->{command_interface};
    my $Src_Node_Obj = $snapmirror_1->{source_ci};
    my $vol_dstrst_name = 'vol_dstrst' . $$;

    $log->comment('Got source vserver:'.$vserver_src.' src vol:'.$vol_src_name.
                  ' dest vserver:'.$vserver_dst.' dest vol:'.$vol_dst_name);
    my $vol_src = NACL::C::Volume->find(
                     command_interface => $Src_Node_Obj,
                     filter            => {
                        volume  => $vol_src_name,
                        vserver => $vserver_src,
                     },
                  );

    my $vol_dst = NACL::C::Volume->find(
                    command_interface => $Dst_Node_Obj,
                    filter            => {
                        volume  => $vol_dst_name,
                        vserver => $vserver_dst,
                    },
                );


    my $node_src = NACL::C::Node->new(name => $vol_src->get_local_node_name());
    my $node_dst = NACL::C::Node->new(name => $vol_dst->get_local_node_name());

    my @traffic_clients;

    if (ref($nfsclients) eq "ARRAY") {
        @traffic_clients = map( NACL::C::Client->new( name => $_ ), @{$nfsclients} );
    }else{
        @traffic_clients = map( NACL::C::Client->new( name => $_ ), $nfsclients );
    }

    my $client_name = $traffic_clients[0]->client();

    $log->step('Initialize the vault relationship with snapshot ' . $ss_for_initialize);
    my $retval = snapmirror_initialize(node_src => $node_src,
                                       node_dst => $node_dst,
                                   src_volumes  => $vol_src,
                                   dst_volumes  => $vol_dst,
                             relationship_type  => 'VAULT',
                                       snapshot => $ss_for_initialize,
                                       validate => 'true',);
    my $snapmirrors = $retval->{'sm_obj'};

    my @traffic_objs;
    if (defined $dst_files) {
        $log->step('Do read IO on destination files');
	my $extent_size = $vol_dst->get_one_state_attribute('default-extent-size') if ($args{'extent_read'});
	$extent_size = '4k' if !$extent_size;
        @traffic_objs = start_read_on_dst_cg_files(
				    client_name => $client_name,
				    volume      => $vol_dst,
				    cg_items    => [@{$dst_files}[0]], # limit to 1 file per vol
				    io_size     => $extent_size );
    }

    my $ss_cnt = scalar @{ $ss_for_update };
    my $snapshot_name;
    if ($ss_cnt == 1) {
        $log->comment('There is only one snapshot to update. Retaining the array');
    } else {
        $snapshot_name= pop @{ $ss_for_update };
        $log->comment('The ss that will be used to do the final update is: ' . $snapshot_name);
    }

    foreach my $ss (@{ $ss_for_update }) {
	$log->step('Modify snapmirror throttle with value 2 to throttle the transfer');
	$snapmirror_1->modify('throttle' => '2');

	$log->step("Update the snapmirror relationship with snapshot $ss");
	snapmirror_update(node_src => $node_src,
			node_dst => $node_dst,
			sm_objs =>  $snapmirrors,
			snapshot => $ss,
			validate => $validate,
			nacltask_wait => 0,
		    );

	# wait for 5 mins only.
	$log->step('Wait 3 minutes for checkpoint to be generated');
	my $ckpt = 0;
	try {
	    $snapmirror_1->wait_on_attributes(
		'method-timeout' => '180',
		attributes => [
		    {
			attribute_to_check => 'snapshot-checkpoint',
			till_value         => qr/[1-9]\d*/
		    },
		]
           );
	    $ckpt = 1;
	} catch NATE::BaseException with {
	    $log->comment("exit waiting for checkpoint.");
        };

	# XXX Add checking for transfer-snapshot here - it should be the $snapshot_name

	if($snapmirror_1->state->status !~ /Idle/){
	    $log->step('Abort the transfer and validate the checkpoint');
	    snapmirror_abort( node_src => $node_src,
			      node_dst => $node_dst,
			      sm_objs  => $snapmirrors,
			      validate => ($ckpt) ? 'true' : 'false',
	    );
	}

	$log->step('Modify snapmirror throttle back to unlimited');
	$snapmirror_1->modify('throttle' => 'unlimited');

	if($snapmirror_1->state->last_transfer_error =~ /Transfer aborted/){
	    $log->step('Do an update again and let it finish using snapshot ' . $ss);
	    snapmirror_update(node_src => $node_src,
			    node_dst => $node_dst,
			    sm_objs =>  $snapmirrors,
			    snapshot => $ss,
			    validate => $validate,
			    nacltask_wait => 1,
	    );
	}
    }

    if ($ss_cnt > 1) {
	$log->step('Modify snapmirror throttle with value 2 to throttle the transfer');
	$snapmirror_1->modify('throttle' => '2');

        $log->step('Update the snapmirror relationship again using snapshot ' . $snapshot_name);
        snapmirror_update(node_src => $node_src,
                          node_dst => $node_dst,
                          sm_objs =>  $snapmirrors,
                          snapshot => $snapshot_name,
                          validate => $validate,
                          nacltask_wait => 0,
                          );

	# wait for 3 mins only in case of missing checkpoint.
	$log->step('Wait 3 minutes for checkpoint to be generated');
	my $ckpt = 0;
	try {
	    $snapmirror_1->wait_on_attributes(
		'method-timeout' => '180',
		attributes => [
		    {
			attribute_to_check => 'snapshot-checkpoint',
			till_value         => qr/[1-9]\d*/
		    },
		]
	    );
	    $ckpt = 1;
	} catch NATE::BaseException with {
	    $log->comment("exit waiting for checkpoint.");
        };

	if($snapmirror_1->state->status !~ /Idle/){
	    $log->step('Hard Abort the transfer and validate the checkpoint starts from zero');
	    snapmirror_abort_hard( node_src => $node_src,
				   node_dst => $node_dst,
				   sm_objs  => $snapmirrors,
				   validate => ($ckpt) ? 'true' : 'false',
				 );
	}

	$log->step('Modify snapmirror throttle back to unlimited');
	$snapmirror_1->modify('throttle' => 'unlimited');

	if($snapmirror_1->state->last_transfer_error =~ /Transfer aborted/){
	    if($snapshot_name =~ /s3|s7/){ # ss with holes
		$log->comment('Get statistic counters for holes before update') ;
		@before_update_ctr_arr = NACL::CS::Statistics->fetch(
					    command_interface => $node_dst,
					    filter=> {
						object  => 'repl_rcvr_lrse',
						counter => 'push_holes_ctr',
						node    => $node_dst->node(),
						raw     => 'true',
					    },
				    );
		$log->comment(Dumper(\@before_update_ctr_arr)) ;
	    }

	    $log->step('Update the transfer and let it finish successfuly');
	    snapmirror_update(node_src => $node_src,
			      node_dst => $node_dst,
			      sm_objs =>  $snapmirrors,
			      snapshot => $snapshot_name,
			      validate => $validate,
			      nacltask_wait => 1,
			      );
	}
    }

    if($validate_holes eq 'true' && scalar(@before_update_ctr_arr)){
        my $before_update_ctr;
        # Get the last instance and its value
        for my $t (@before_update_ctr_arr) {
            if (!defined($before_update_ctr) or
                $t->instance() > $before_update_ctr->instance()) {
                $before_update_ctr = $t;
            }
        }
        my $before_update_val = $before_update_ctr->value;
        $Log->comment("Value of counter before update is : " .
                $before_update_ctr->value() . " for instance " . $before_update_ctr->instance());
        my $after_update_val;
        my @after_update_ctr_arr = NACL::CS::Statistics->fetch(
                             command_interface => $node_dst,
                             filter=> {
                                object => 'repl_rcvr_lrse',
                                counter => 'push_holes_ctr',
                                node     => $node_dst->node(),
                                raw  => 'true',
			    },
			);
	my $after_update_ctr;
          # Get the last instance and its value
	for my $t (@after_update_ctr_arr) {
            if (!defined($after_update_ctr) or
                $t->instance() > $after_update_ctr->instance()) {
                $after_update_ctr = $t;
            }
        }

	$after_update_val = $after_update_ctr->value();
	$log->comment("value of counter after update is : " .  $after_update_ctr->value(). " for instance " .  $after_update_ctr->instance() );

	if ( ($before_update_val == $after_update_val )){
	    NATE::BaseException->throw('After update the counters should have been incremented');
	}
    }

    if($rst_flag eq 'true'){
	if (defined $ss_del){
	    $log->step('Delete passed snapshots from the src vol.');
	    my $src_vol_name = $vol_src->volume();
	    for my $j (@{$ss_del}) {
		NACL::STask::VolumeSnapshot->purge(
		   command_interface  => $node_src,
		   volume             => $vol_src,
		   vserver            => $vol_src->vserver,
		   snapshot           =>  $j,
		);
                $log->comment("Delete snapshot "  . $j . " from " . $src_vol_name);


		my $new_ss = $vol_src->snapshot_create(
		   snapshot           => 'sshot_del' . $j,
		   'snapmirror-label' => 'weekly'
		);

		snapmirror_update(node_src => $node_src,
				node_dst => $node_dst,
				sm_objs =>  $snapmirrors,
				snapshot => $new_ss->snapshot(),
				validate => 'true',
				nacltask_wait => 1,
			    );

		$log->step('Run incremental restore with deleted snapshots');
		$log->step("Restore back snapshot " . $j);

		snapmirror_restore(node_src => $node_src,
				node_dst => $node_dst,
				sm_objs =>  $snapmirrors,
				snapshot => $j,
				validate => $validate,
				nacltask_wait =>1,
			    );
	    }
	} else {
	    my $extent_size  = $vol_dst->get_one_state_attribute('default-extent-size');
	    my $rst_size = $vol_dst->get_one_state_attribute('size');
	    my $vol_dstrst = NACL::STask::Volume->create(
                                command_interface => $Dst_Node_Obj,
                                size => $rst_size,
                                volume => $vol_dstrst_name,
                                'default-extent-size' => $extent_size,
                                nacltask_if_exists => 'purge',
                                type => 'DP'
                            );

            NACL::STask::Snapmirror->restore(
                command_interface   => $node_dst,
                'source-path'       => $vol_dst,
                'destination-path'  => $vol_dstrst,
                'method-timeout'    => 3600,
                'source-snapshot'   => $ss_for_initialize,
                nacltask_wait       => 1,
            );

            foreach my $ss (@{ $ss_for_update }) {
                NACL::STask::Snapmirror->restore(
                   command_interface   => $node_dst,
                   'source-path'       => $vol_dst,
                   'destination-path'  => $vol_dstrst,
                   'method-timeout'    => 3600,
                   'source-snapshot'   => $ss,
                   nacltask_wait       => 1,
                );
            }
	}
    }

    if (@traffic_objs) {
        $log->step('Stop read traffic on dst');
        foreach my $o (@traffic_objs) {
            $o->stop();
        }
    }
    $log->exit();
}


=head2 Wf_DR_lifecycle

This method will cover DR lifecycle.

=over

=item Example

    Wf_DR_lifecycle(sm_objs => $sm_obj,
                    nfsclients => $NFSCLIENTS,
                    validate   => 'true');


=item Options

=over

=item sm_objs

(Required): ARRAYREF of snapmirror objects of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item validate

(Optional) boolean(true/false) to do validations

=item ss_for_update_map
 (Required) ARRAYREF. An arrayref containing one Hashref element.
                     The hashref holds the map between the sm relationship and the corresponding ss
                     to use for intialization. Note the key naming convention
                     should be sm_obj_<0/1/2/3>. Key => SCALAR
            Example:
            $VAR = {
                    sm_obj_1 => 's1',
                    sm_obj_2 => 's2' };

=item ss_for_initialize_map
 (Required): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list
                      of ss that will be used for sm update. Note the key naming convention
                      shoudl be sm_obj_<0/1/2/3>. Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['s2','s3'],
                        sm_obj_2 => ['s4', 's5']};

=item dst_files_map
 (Optional): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list of
                      files to use to start traffic on.  Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['file1', 'file2'],
                        sm_obj_2 => ['file3', 'file4']};
 Default   : undef

=item validate
 (Optional): Whether to run validation at the end of the workflow.
 (Default): false

=item stagger_delay_map
 (Optional): An arrayref containing one Hashref element.
             The hashref holds the map of how many seconds to delay before invoking a process.
 (Default): 1

=back

=cut

sub Wf_DR_lifecycle{

    my %args = validate_with(
                params => \@_,
                spec   => {
                    sm_objs                  => { type => ARRAYREF, optional => 0 },
                    nfsclients               => { type => ARRAYREF|SCALAR, optional =>0 },
                    validate                 => { type => SCALAR, optional => 1, default => 'false' },
                    ss_for_initialize_map    => { type => ARRAYREF, optional => 0 },
                    ss_for_update_map        => { type => ARRAYREF, optional => 0 },
                    dst_files_map            => { type => ARRAYREF|UNDEF, optional => 1 },
                    stagger_delay_map        => { type => ARRAYREF, optional => 1 },
		    extent_read              => { type => SCALAR, default => 0 },
                },
    );

    my $sm_obj              = $args{'sm_objs'};
    my @vault_objs          = _list_of_vals($sm_obj);
    my $nfclients           = $args{'nfsclients'};
    my $ss_for_init_map     = $args{ss_for_initialize_map}->[0];
    my $ss_for_update_map   = $args{ss_for_update_map}->[0];
    my $dst_files_map       = $args{dst_files_map}->[0];
    my $count               = scalar(@vault_objs);
    my $validate            = $args{validate};
    my $stagger_map         = $args{stagger_delay_map}->[0];
    my $extent_read         = $args{extent_read} || 0;
    my @subtests_vault;

    for (my $i=0; $i < $count; $i++) {
        my $o = $vault_objs[$i];
        my $key = 'sm_obj_' . $i;
        my $ss_for_initialize = $ss_for_init_map->{$key};
        my $ss_for_update     = $ss_for_update_map->{$key};
        my $delay;
        if (!defined $stagger_map->{key}) {
            $delay = 1;
        } else {
            $delay = $stagger_map->{key};
        }
        _stagger($delay);

        my $dst_files;
        if (defined $dst_files_map) {
            $dst_files         = $dst_files_map->{$key};
        }
        $log->comment('For sm relationship with dst path: ' . $o->{destination_path} .
                      "\n" .
                      ' ss that will be used for initialize is: ' . $ss_for_initialize .
                      "\n" .
                      ' ss for updates are: ' . Dumper($ss_for_update) .
                      "\n" .
                      ' dst files on which traffic will be started are: ' . Dumper($dst_files) .
                      "\n");

        my %args_func = (sm_obj             =>  $o,
                         ss_for_initialize  => $ss_for_initialize,
                         ss_for_update      => $ss_for_update,
                         dst_files          => $dst_files,
                         validate           => $validate,
                         nfsclients         => $nfclients,
			 extent_read        => $extent_read,);
        push @subtests_vault, create_parallel_processes('DR_workflow',\%args_func);
    }

    run_processes_bg('DR_workflow',\@subtests_vault);
}

=head2 DR_workflow

   replication lifecycle test - use s1 to initalize the replication, run udpates with s2 - s13 and at random orders,
   run abort during updates, run hard-abort during updates, run checkpoint reststart during updates; run break;
   create a few more snapshot (like s1 - s13) on both source and destination volumes; do read/write IO on destination clones,
   and let it run for some time; run resync between those 2 volumes; use "common validation" at the end of test

Steps
  1) create and initialize a lrsedr relationship using snapshot s1.
  2) run updates with s2((TODO: with other snapshots in future)).
  3) run abort during updates.
  4) run hard-abort during updates
  5) run checkpoint restart(covered as part of validation of snapmirror abort).
  6) Break the snapmirror relationship.
  7) Create few more snapshots.
  8) Resync the snapmirror relationship.

=over

=item Example

    DR_workflow( sm_obj => $sm_obj,
                 nfsclients  => $NFSCLIENTS,);

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item validate

(Optional) boolean(true/false) to do validations

=item ss_for_update
(Required): ARRAYREF. List of snapshots that will be used for sm update

= item ss_for_initialize
 (Required): SCALAR. Snapshot that will be used to initialize the sm relationship

=item dst_files
 (Optional): ARRAYREF. The list of files to start traffic on.
 Default   : undef

=back

=cut

sub DR_workflow{
     my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients           => { type => ARRAYREF|SCALAR, optional => 0 },
            sm_obj               => { type =>  HASHREF, optional => 0 },
            validate             => { type => SCALAR, optional => 1, default => 'false' },
            ss_for_initialize    => { type => SCALAR, optional => 0 }, #should I default to s1
            ss_for_update        => { type => ARRAYREF, optional => 0 },
            dst_files            => { type => ARRAYREF|UNDEF, optional => 1 },
	    extent_read          => { type => SCALAR, default => 0 },
        },
    );

    my $log = log_global();
    $log->enter();

    my $nfsclients  = $args{nfsclients};

    my $validate           = $args{validate} ;
    my $iterations         = $args{no_of_iterations};
    my $snapmirror_1       = $args{sm_obj} ;
    my $ss_for_initialize  = $args{ss_for_initialize};
    my $ss_for_update      = $args{ss_for_update};
    my $dst_files          = $args{dst_files};
    my $source_path = $snapmirror_1->{source_path};
    my $destination_path = $snapmirror_1 ->{destination_path};
    # get the volume and vserver names of src and dest
    my ($vserver_src,$vol_src_name) = ($source_path =~ /^(.*):(.*)$/);
    my ($vserver_dst,$vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);
    my $Dst_Node_Obj = $snapmirror_1->{command_interface};
    my $Src_Node_Obj = $snapmirror_1->{source_ci};

    $log->comment('Got source vserver:'.$vserver_src.' src vol:'.$vol_src_name.
                  ' dest vserver:'.$vserver_dst.' dest vol:'.$vol_dst_name);
    my $vol_src = NACL::C::Volume->find(
                     command_interface => $Src_Node_Obj,
                     filter            => {
                        volume  => $vol_src_name,
                        vserver => $vserver_src,
                     },
                  );

    my $vol_dst = NACL::C::Volume->find(
                    command_interface => $Dst_Node_Obj,
                    filter            => {
                        volume  => $vol_dst_name,
                        vserver => $vserver_dst,
                    },
                );

    my $node_src = NACL::C::Node->new(name => $vol_src->get_local_node_name());
    my $node_dst = NACL::C::Node->new(name => $vol_dst->get_local_node_name());

    my @traffic_clients;

    if (ref($nfsclients) eq "ARRAY") {
        @traffic_clients
        = map( NACL::C::Client->new( name => $_ ), @{$nfsclients} );
    }else{
        @traffic_clients = map( NACL::C::Client->new( name => $_ ), $nfsclients );
    }

    my $client_name = $traffic_clients[0]->client();

    $log->step('Initialize the vault relationship with snapshot ' . $ss_for_initialize);
    my $retval = snapmirror_initialize(node_src => $node_src,
                                       node_dst => $node_dst,
                                   src_volumes  => $vol_src,
                                   dst_volumes  => $vol_dst,
                             relationship_type  => 'VAULT',
                                       snapshot => $ss_for_initialize,
                                       validate => $validate,);

    my $snapmirrors = $retval->{'sm_obj'};
    #my $snapmirror_1  = $snapmirrors->[0];

    my @traffic_objs;
    if (defined $dst_files) {
        $log->step('Do read IO on destination files');
	my $extent_size = $vol_dst->get_one_state_attribute('default-extent-size') if ($args{'extent_read'});
	$extent_size = '4k' if !$extent_size;
        @traffic_objs = start_read_on_dst_cg_files(
				    client_name => $client_name,
				    volume      => $vol_dst,
				    cg_items    => [@{$dst_files}[0]], # limit to 1 file per vol
				    io_size     => $extent_size );
    }

    my $ss_cnt = scalar @{ $ss_for_update };
    my $snapshot_name;
    if ($ss_cnt == 1) {
        $log->comment('There is only one snapshot to update. Retaining the array');
    } else {
        $snapshot_name= pop @{ $ss_for_update };
        $log->comment('The ss that will be used to do the final update is: ' . $snapshot_name);
    }

    #for(my $i=0;$i<$iterations;$i++)
    foreach my $ss (@{ $ss_for_update }) {
	$log->step('Modify snapmirror throttle with value 2 to throttle the transfer');
	$snapmirror_1->modify('throttle' => '2');

        $log->step('Update the snapmirror relationship with snapshot ' . $ss);
	snapmirror_update(node_src => $node_src,
			  node_dst => $node_dst,
			  sm_objs =>  $snapmirrors,
			  snapshot => $ss,
			  validate => $validate, # will be skipped since wait is 0
			  nacltask_wait => 0,
			  );

	# wait for 5 mins only.
	$log->step('Wait 3 minutes for checkpoint to be generated');
	my $ckpt = 0;
	try {
	    $snapmirror_1->wait_on_attributes(
		'method-timeout' => '180',
		attributes => [
		    {
			attribute_to_check => 'snapshot-checkpoint',
			till_value         => qr/[1-9]\d*/
		    },
		]
	    );
	    $ckpt = 1;
	} catch NATE::BaseException with {
	    $log->comment("exit waiting for checkpoint.");
        };

	if($snapmirror_1->state->status !~ /Idle/){
	    $log->step('Abort the transfer and validate the checkpoint');
	    snapmirror_abort( node_src => $node_src,
			     node_dst => $node_dst,
			     sm_objs  => $snapmirrors,
			     validate => ($ckpt) ? 'true' : 'false',
	    );
	}

	$log->step('Modify snapmirror throttle back to unlimited');
	$snapmirror_1->modify('throttle' => 'unlimited');

	if($snapmirror_1->state->last_transfer_error =~ /Transfer aborted/){
	    $log->step('Do an update again and let it finish using snapshot ' . $ss);
	    snapmirror_update(node_src => $node_src,
			    node_dst => $node_dst,
			    sm_objs =>  $snapmirrors,
			    snapshot => $ss,
			    validate => $validate,
			    nacltask_wait => 1,
			    );
	}
    }

    if ($ss_cnt > 1) {
	$log->step('Modify snapmirror throttle with value 2 to throttle the transfer');
	$snapmirror_1->modify('throttle' => '2');

        $log->step('Update the snapmirror relationship again using snapshot ' . $snapshot_name);
        snapmirror_update(node_src => $node_src,
                          node_dst => $node_dst,
                          sm_objs =>  $snapmirrors,
                          snapshot => $snapshot_name,
                          validate => $validate, # will be skipped since wait is 0
                          nacltask_wait => 0,
                          );

	$log->step('Wait 3 minutes for checkpoint to be generated');
	my $ckpt = 0;
	try {
	    $snapmirror_1->wait_on_attributes(
		'method-timeout' => '180',
		attributes => [
		    {
			attribute_to_check => 'snapshot-checkpoint',
			till_value         => qr/[1-9]\d*/
		    },
		]
	    );
	    $ckpt = 1;
	} catch NATE::BaseException with {
	    $log->comment("exit waiting for checkpoint.");
        };

	if($snapmirror_1->state->status !~ /Idle/){
	    $log->step('Hard Abort the transfer and validate the checkpoint starts from zero');
	    snapmirror_abort_hard( node_src => $node_src,
				   node_dst => $node_dst,
				   sm_objs  => $snapmirrors,
				   validate => ($ckpt) ? 'true' : 'false',
				 );
	}

	$log->step('Modify snapmirror throttle back to unlimited');
	$snapmirror_1->modify('throttle' => 'unlimited');

	if($snapmirror_1->state->last_transfer_error =~ /Transfer aborted/){
	    $log->step('Update the transfer and let it finish successfuly');
	    $log->comment('Modify the throttle back to unlimited');
	    $snapmirror_1->modify( throttle => 'unlimited' );
	    snapmirror_update(node_src => $node_src,
			      node_dst => $node_dst,
			      sm_objs =>  $snapmirrors,
			      snapshot => $snapshot_name,
			      validate => $validate,
			      nacltask_wait => 1,
			      );
	}
    }

    #Script should have some kind of IO running at this point. Else all the new ss
    #that will be cut will have no new data and the resync will basically be a null
    #transfer.
    $log->step('Break the snapmirror relationship');
    snapmirror_break( node_src => $node_src,
                      node_dst => $node_dst,
                      sm_objs =>  $snapmirrors,
                      validate => $validate,
                      );

    $log->step('Create some more new snapshots');
    for (my $i = 0; $i < 10; $i ++){
        my $snapshot_name = 'dr_wf_' . $$ . '_';
        my $snapshot = $snapshot_name . $i;
        try {
            $vol_src->snapshot_create(snapshot => $snapshot,);
        } catch NATE::BaseException with {
            my $ex = shift;
            my $txt = $ex->text();
            $log->comment("Caught exception $ex" . "; TXT $txt" );

            if ($txt =~ /Reason: Snapshot already exists/i) {
                $log->comment('Multiple instances of this method must be running ' .
                              'ignoring this error');
            } else {
                $ex->throw();
            }
        };
    }

    $log->step('Resync the snapmirror relationship');
    snapmirror_resync(node_src => $node_src,
                      node_dst => $node_dst,
                      sm_objs =>  $snapmirrors,
                      validate => $validate,
                      );

    if (@traffic_objs) {
        $log->step('Stop read traffic on dst');
        foreach my $o (@traffic_objs) {
            $o->stop();
        }
    }

    $log->exit();
}

=head2 Wf_hecr_lifecycle

This method will parallely run vault and DR workflows.
This method will internally invoke Wf_vault_lifecycle() and Wf_dr_lifecycle().

=over

=item Example

    Wf_vault_lifecycle(sm_objs => $sm_obj,
                       nfsclients => $NFSCLIENTS,
                       ss_for_initialize_map => {sm_obj_0 => 's1'},
                       ss_for_update_map     => {sm_obj_0 => ['s2', 's3', 's4]},
                       dst_files_map         => {sm_obj_0 => \@files},
                       );


=item Options

=over

=item sm_objs

(Required): ARRARREF of snapmirror objects of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item ss_for_update_map
 (Required) ARRAYREF. An arrayref containing one Hashref element.
                     The hashref holds the map between the sm relationship and the corresponding ss
                     to use for intialization. Note the key naming convention
                     should be sm_obj_<0/1/2/3>. Key => SCALAR
            Example:
            $VAR = {
                    sm_obj_1 => 's1',
                    sm_obj_2 => 's2' };

=item ss_for_initialize_map
 (Required): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list
                      of ss that will be used for sm update. Note the key naming convention
                      shoudl be sm_obj_<0/1/2/3>. Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['s2','s3'],
                        sm_obj_2 => ['s4', 's5']};

=item dst_files_map
 (Optional): ARRAYREF. An arrayref containing one Hashref element.
                      The hashref holds the map between the sm relationship and the corresponding list of
                      files to use to start traffic on.  Key => ARRAYREF.
             Example:
             $VAR = {
                        sm_obj_1 => ['file1', 'file2'],
                        sm_obj_2 => ['file3', 'file4']};
 Default   : undef

=item validate
 (Optional): Whether to run validation at the end of the workflow.
 (Default): false

=item stagger_delay_map
 (Optional): An arrayref containing one Hashref element.
             The hashref holds the map of how many seconds to delay before invoking a process.
 (Default): 1


=back

=cut

sub Wf_hecr_lifecycle {
    my %args = validate_with(
                params => \@_,
                spec   => {
                    node_dst                 => {type => OBJECT, optional => 0},
                    sm_objs                  => {type => ARRAYREF, optional => 0},
                    nfsclients               => {type => ARRAYREF|SCALAR, optional =>0},
                    validate                 => { type => SCALAR, optional => 1, default => 'false'},
                    ss_for_initialize_map    => { type => ARRAYREF, optional => 0 },
                    ss_for_update_map        => { type => ARRAYREF, optional => 0 },
                    dst_files_map            => { type => ARRAYREF|UNDEF, optional => 1,},
                    stagger_delay_map        => { type => ARRAYREF, optional => 1 },
                },
                );

    my $sm_obj              = $args{'sm_objs'};
    my @vault_objs          = _list_of_vals($sm_obj);
    my $nfclients           = $args{'nfsclients'};
    my $ss_for_init_map     = $args{ss_for_initialize_map}->[0];
    my $ss_for_update_map   = $args{ss_for_update_map}->[0];
    my $dst_files_map       = $args{dst_files_map}->[0];
    my $count               = scalar(@vault_objs);
    my $validate            = $args{validate};
    my $stagger_map         = $args{stagger_delay_map}->[0];
    my $cmd_interface       = $args{node_dst};
    my @subtests_vault;

    for (my $i=0; $i < $count; $i++) {
        my $o = $vault_objs[$i];
        my $key = 'sm_obj_' . $i;
        my $ss_for_initialize = $ss_for_init_map->{$key};
        my $ss_for_update     = $ss_for_update_map->{$key};
        my $delay;
        if (!defined $stagger_map->{key}) {
            $delay = 1;
        } else {
            $delay = $stagger_map->{key};
        }
        _stagger($delay);

        my $dst_files;
        if (defined $dst_files_map) {
            $dst_files         = $dst_files_map->{$key};
        }

        my $dst_path = $o->{destination_path};
        $log->step('Get the sm policy');
        my $policy_obj = NACL::CS::Snapmirror->fetch(
                            command_interface => $cmd_interface,
                            filter=> {
                                'destination-path' => $dst_path,
                            },
                            requested_fields => [qw(policy)]);

        my $policy  =  $policy_obj->policy;
        $log->comment('Policy obtained is: ' . $policy);

        my %args_func = ('sm_obj'           =>  $o,
                         ss_for_initialize  => $ss_for_initialize,
                         ss_for_update      => $ss_for_update,
                         dst_files          => $dst_files,
                         validate           => $validate,
                         'nfsclients'       => $nfclients);


        if ($policy  =~ /xdpdefault/i) {
            $log->comment('For sm relationship with dst path: ' . $o->{destination_path} .
                          "\n" .
                          ' ss that will be used for initialize is: ' . $ss_for_initialize .
                          "\n" .
                          ' ss for updates are: ' . Dumper($ss_for_update) .
                          "\n" .
                          ' dst files on which traffic will be started are: ' . Dumper($dst_files) .
                          "n" .
                          '. Wf_vault_lifecycle will be invoked');
            push @subtests_vault, create_parallel_processes('vault_workflow',\%args_func);

        } elsif ($policy =~ /Mirror*/i) {
            $log->comment('For sm relationship with dst path: ' . $o->{destination_path} .
                          "\n" .
                          ' ss that will be used for initialize is: ' . $ss_for_initialize .
                          "\n" .
                          ' ss for updates are: ' . Dumper($ss_for_update) .
                          "\n" .
                          ' dst files on which traffic will be started are: ' . Dumper($dst_files) .
                          "n" .
                          '. Wf_dr_lifecycle will be invoked');
            push @subtests_vault, create_parallel_processes('DR_workflow',\%args_func);

        }
    }

    run_processes_bg('HECR workflow', \@subtests_vault);
}
=head2 Wf_Cascade_Vault_DR_lifecycle

    The purpose of this workflow is to excercise snapmirror lifecycle on Vault->DR cascade.

Steps
  1) Populate some HECR data and Initialize with snapshot both Vault and DR relationships
  2) update concurrently both dr and vault relationships.
  3) Restore from B to A.
  4) Resync from B->C
  3) Delete B to C
  4) Bring down middle node
  5) Do resync from A to C, wait till complete

=over

=item Example

  Wf_Cascade_Vault_DR_lifecycle( sm_obj => $sm_obj,
                                 nfsclients  => $NFSCLIENTS,
                                );

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item ss_for_intiialze

(Optional): snapshot to be used for initialize.

=item validate

(Optional) boolean(true/false) to do validations

=back

=cut

sub Wf_Cascade_Vault_DR_lifecycle{
     my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients  => { type => ARRAYREF|SCALAR, optional => 0 },
            sm_objs    => { type =>  ARRAYREF, optional => 0},
            validate  => { type => SCALAR, optional => 1, default => 'false'},
            ss_for_initialize    => { type => SCALAR, optional => 1, default => 's1' },
        },
    );

    my $log = log_global();
    $log->enter();

    my $nfsclients  = $args{nfsclients};
    my @traffic_clients;
    if (ref($nfsclients) eq "ARRAY") {
        @traffic_clients
        = map( NACL::C::Client->new( name => $_ ), @{$nfsclients} );
    }else{
        @traffic_clients = map( NACL::C::Client->new( name => $_ ), $nfsclients );
    }

    my $client_name = $traffic_clients[0]->client();
    my $validate = $args{validate};
    my $sm_objs = $args{sm_objs};
    my $sm_obj_dr = @{ $args{sm_objs} }[1] ;
    my $sm_obj_vault = @{ $args{sm_objs} }[0] ;
    my $snapshot_name;
    my @vol_srcs;
    my @vol_dsts;
    my @sm_objs;

    ## array to return back to caller
    my @snapmirrors;

    foreach my $sm_obj(@{$args{sm_objs}}){
        my $source_path = $sm_obj->{source_path};
        my $destination_path = $sm_obj ->{destination_path};
        # get the volume and vserver names of src and dest
        my ($vserver_src,$vol_src_name) = ($source_path =~ /^(.*):(.*)$/);
        my ($vserver_dst,$vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);
        my $Dst_Node_Obj = $sm_obj->{command_interface};
        my $Src_Node_Obj = $sm_obj->{source_ci};

        $log->comment('Got source vserver:'.$vserver_src.' src vol:'.$vol_src_name.
                  ' dest vserver:'.$vserver_dst.' dest vol:'.$vol_dst_name);
        my $vol_src = NACL::C::Volume->find(
                     command_interface => $Src_Node_Obj,
                     filter            => {
                        volume  => $vol_src_name,
                        vserver => $vserver_src,
		    },
		);

        my $vol_dst = NACL::C::Volume->find(
                    command_interface => $Dst_Node_Obj,
                    filter            => {
                        volume  => $vol_dst_name,
                        vserver => $vserver_dst,
                    },
                );

        push @vol_srcs, $vol_src;
        push @vol_dsts, $vol_dst;
    }

    $log->step('Initialize DR and Vault relationships.');
    $snapshot_name = $args{'ss_for_initialize'};
    my $retval_vault = snapmirror_initialize(node_src => $sm_obj_vault->{source_ci},
                          node_dst => $sm_obj_vault->{command_interface},
                      src_volumes  => $vol_srcs[0],
                      dst_volumes  => $vol_dsts[0],
                relationship_type  => 'VAULT',
                          snapshot => $snapshot_name,
                          validate => $validate,);

    my $retval_dr = snapmirror_initialize(node_src => $sm_obj_dr->{source_ci},
                          node_dst => $sm_obj_dr->{command_interface},
                      src_volumes  => $vol_dsts[0],
                      dst_volumes  => $vol_dsts[1],
                relationship_type  => 'VAULT',
                          #validate => $validate,
                    );

    $log->step('Update DR and Vault relationships concurrently.');
    my $snapmirrors_v = $retval_vault->{'sm_obj'};
    my $snapmirrors_dr = $retval_dr->{'sm_obj'};
    snapmirror_update(node_src => $sm_obj_vault->{source_ci},
                      node_dst => $sm_obj_vault->{command_interface},
                      sm_objs  => $snapmirrors_v,
                      validate => $validate,
                      nacltask_wait =>0,
		    );

    snapmirror_update(node_src => $sm_obj_dr->{source_ci},
                      node_dst => $sm_obj_dr->{command_interface},
                      sm_objs  => $snapmirrors_dr,
                      validate => $validate,
                      nacltask_wait =>0, # Bug in the sm_update if snapshot is not passed and nacltask_wait=>1
                    );

    $sm_obj_vault->wait_for_transfer_to_complete();
    $sm_obj_dr->wait_for_transfer_to_complete();

    $log->step('Do Restore B->A.');
    snapmirror_restore(node_src => $sm_obj_vault->{source_ci},
		       node_dst => $sm_obj_vault->{command_interface},
		       sm_objs  => $snapmirrors_v,
		       validate => $validate,
		       nacltask_wait =>1,
		    );

    $log->step('Resync the DR B->C relationship');
    snapmirror_resync( node_src => $sm_obj_dr->{source_ci},
		       node_dst => $sm_obj_dr->{command_interface},
		       sm_objs  => $snapmirrors_dr,
		       validate => $validate,
		    );

    my $mid_node = $sm_obj_vault->{command_interface} ;
    my $dst_node = $sm_obj_dr->{command_interface};
    my $src_node = $sm_obj_vault->{source_ci};

    my @nodes;
    push @nodes,$mid_node;
    push @nodes,$dst_node;
    push @nodes,$src_node;

    $log->step('Break the DR relationship');
    snapmirror_break( node_src => $sm_obj_dr->{source_ci},
		      node_dst => $sm_obj_dr->{command_interface},
		      sm_objs  => $snapmirrors_dr,
		      validate => $validate,
		    );

    $log->step('Delete DR relationship.');
    snapmirror_delete( node_src => $sm_obj_dr->{source_ci},
		       node_dst => $sm_obj_dr->{command_interface},
		       sm_objs  => $snapmirrors_dr,
		       validate => $validate,
		    );

    $log->step('Turn off HA and SFO configuration.');
    change_sfo_setting(nodes => \@nodes);
    $log->step('Reboot Node B.');
    my $transit_obj = NACL::Transit->new( name => $mid_node->node );
    $transit_obj->reboot(timeout => 1800);
    $log->step('Sleep 180s after reboot.');
    sleep(180);
    $dst_node->refresh_command_interface(
	no_version_change => 1,
	max_reconnect => 100,
	max_reconnect_timewait => 10,
    );
    # reset HA
    change_sfo_setting(nodes => \@nodes, option => 'true');

    $log->step('Create and resync Vault relationship A->C');
    my $retval_ac = snapmirror_create(
	node_src          => $src_node,
	node_dst          => $dst_node,
	src_volumes       => $vol_srcs[0],
	dst_volumes       => $vol_dsts[1],
	relationship_type => 'VAULT',
    );

    my $snapmirrors_ac = $retval_ac->{'sm_obj'};
    my $src_vol_name = $vol_srcs[0]->volume;
    my $vol_src1 = NACL::CS::Volume->fetch(
                     command_interface => $src_node,
                     filter            => {
                        volume  => $src_vol_name,
                        vserver => $vol_srcs[0]->vserver,
            },
        );

    my $node = $vol_src1->node;
    my $node_obj = NACL::C::Node->new( name => $node );
    NACL::C::VolumeFile->create(
    command_interface => $node_obj,
    vserver           => $vol_srcs[0]->vserver,
    size              => '20m',
    path              => "/vol/$src_vol_name/file_cc" . $$,
    filecount         => '20',
    );

    snapmirror_resync( node_src => $src_node,
		       node_dst => $dst_node,
		       sm_objs  => $snapmirrors_ac,
		       );

    my $snapmirror_ac = $snapmirrors_ac->[0];
    $snapmirror_ac->wait_for_transfer_to_complete();
    push @snapmirrors,$sm_obj_vault;
    push @snapmirrors,$snapmirror_ac;

    $log->exit;
    return \@snapmirrors;
}


=head2 Wf_Cascade_DR_Vault_lifecycle

    The purpose of this workflow is to excercise snapmirror lifecycle on DR->Vault cascade.

Steps
  1) Initialize both DR and vault relationship with some snapshot.
  2) Do update from A to B, B to C .
  3) Delete B to C
  4) Bring down middle node
  5) Create and resync from A to C.
  6) Delete some snapshots from A
  7) Create some snapshots and update.
  8) Do incremental restore from C to A, verify the restored snapshots.

=over

=item Example

  Wf_Cascade_Vault_DR_lifecycle( sm_obj => $sm_obj,
                                   nfsclients  => $NFSCLIENTS,
                                );

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required): Arrayref of Nfs clients.

=item ss_for_intiialze

(Optional): snapshot to be used for initialize.

=item ss_to_delete

(Optional): snapshots to be deleted.

=item validate

(Optional) boolean(true/false) to do validations

=back

=cut

sub Wf_Cascade_DR_Vault_lifecycle{
     my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients  => { type => ARRAYREF|SCALAR, optional => 0 },
            sm_objs    => { type =>  ARRAYREF, optional => 0},
            validate  => { type => SCALAR, optional => 1, default => 'false'},
            ss_for_initialize    => { type => SCALAR, optional => 1, default => 's1' },
            ss_to_delete         => { type => ARRAYREF, optional => 0,  }
  },
    );

    my $log = log_global();
    $log->enter();

    my $nfsclients  = $args{nfsclients};
    my @traffic_clients;
    if (ref($nfsclients) eq "ARRAY") {
        @traffic_clients
        = map( NACL::C::Client->new( name => $_ ), @{$nfsclients} );
    }else{
        @traffic_clients = map( NACL::C::Client->new( name => $_ ), $nfsclients );
    }

    my $client_name = $traffic_clients[0]->client();
    my $snapshots = $args{'ss_to_delete'};
    my @ss_del = _list_of_vals($snapshots);
    my $validate = $args{validate};
    my $sm_objs = $args{sm_objs};
    my $sm_obj_dr = @{ $args{sm_objs} }[0] ;
    my $sm_obj_vault = @{ $args{sm_objs} }[1] ;
    my $snapshot_name;
    my @vol_srcs;
    my @vol_dsts;
    my @sm_objs;
    my @snapmirrors;

    foreach my $sm_obj(@{$args{sm_objs}}){
        my $source_path = $sm_obj->{source_path};
        my $destination_path = $sm_obj ->{destination_path};
        # get the volume and vserver names of src and dest
        my ($vserver_src,$vol_src_name) = ($source_path =~ /^(.*):(.*)$/);
        my ($vserver_dst,$vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);
        my $Dst_Node_Obj = $sm_obj->{command_interface};
        my $Src_Node_Obj = $sm_obj->{source_ci};

        $log->comment('Got source vserver:'.$vserver_src.' src vol:'.$vol_src_name.
                  ' dest vserver:'.$vserver_dst.' dest vol:'.$vol_dst_name);
        my $vol_src = NACL::C::Volume->find(
                     command_interface => $Src_Node_Obj,
                     filter            => {
                        volume  => $vol_src_name,
                        vserver => $vserver_src,
                     },
                  );

        my $vol_dst = NACL::C::Volume->find(
                    command_interface => $Dst_Node_Obj,
                    filter            => {
        volume  => $vol_dst_name,
                        vserver => $vserver_dst,
                    },
                );

        push @vol_srcs, $vol_src;
        push @vol_dsts, $vol_dst;
    }

    $log->step('Initialize relationships.');
    $snapshot_name = $args{'ss_for_initialize'};
    my $retval_dr = snapmirror_initialize(node_src => $sm_obj_dr->{source_ci},
                          node_dst => $sm_obj_dr->{command_interface},
                      src_volumes  => $vol_srcs[0],
                      dst_volumes  => $vol_dsts[0],
                relationship_type  => 'VAULT',
                          snapshot => $snapshot_name,
                          #validate => $validate,
			);

    my $retval_vault = snapmirror_initialize(node_src => $sm_obj_vault->{source_ci},
                          node_dst => $sm_obj_vault->{command_interface},
                      src_volumes  => $vol_dsts[0],
                      dst_volumes  => $vol_dsts[1],
                relationship_type  => 'VAULT',
                      #snapshot => $snapshot_name,
                          #validate => $validate,
                    );

    $log->step('Do update from A to B, B to C');
    my $snapmirrors_v = $retval_vault->{'sm_obj'};
    my $snapmirrors_dr = $retval_dr->{'sm_obj'};
    my $src_vol_name = $vol_srcs[0]->volume();
    snapmirror_update(node_src => $sm_obj_dr->{source_ci},
                      node_dst => $sm_obj_dr->{command_interface},
                      sm_objs  => $snapmirrors_dr,
                      validate => $validate,
                      nacltask_wait =>0, # Bug in the sm_update if snapshot is not passed and nacltask_wait=>1
                      );
    $sm_obj_dr->wait_for_transfer_to_complete();

    snapmirror_update(node_src => $sm_obj_vault->{source_ci},
                      node_dst => $sm_obj_vault->{command_interface},
                      sm_objs  => $snapmirrors_v,
                      validate => $validate,
                      nacltask_wait =>0,
                      );
    $sm_obj_vault->wait_for_transfer_to_complete();

    my $mid_node =  $sm_obj_dr->{command_interface} ;
    my $dst_node =  $sm_obj_vault->{command_interface};
    my $src_node =  $sm_obj_dr->{source_ci};

    my @nodes;
    push @nodes,$mid_node;
    push @nodes,$dst_node;
    push @nodes,$src_node;

    $log->step('Delete Vault relationship.');
    snapmirror_delete( node_src => $sm_obj_vault->{source_ci},
		      node_dst  => $sm_obj_vault->{command_interface},
		      sm_objs   => $snapmirrors_v,
		      validate  => $validate,
		    );

    $log->step('Turn off HA and SFO configuration.');
    change_sfo_setting(nodes => \@nodes);
    $log->step('Reboot Node B.');
    my $transit_obj = NACL::Transit->new( name => $mid_node->node );
    $transit_obj->reboot(timeout => 1800);
    $log->step('Sleep 60s after reboot.');
    sleep(60);
    $dst_node->refresh_command_interface(
	no_version_change => 1,
	max_reconnect => 100,
	max_reconnect_timewait => 10,
     );
    # reset HA
    change_sfo_setting(nodes => \@nodes, option => 'true');

    $log->step('Create and resync Vault relationship A->C');
    my $retval_ac = snapmirror_create(
	node_src           =>  $src_node,
	node_dst           =>  $dst_node,
	src_volumes        =>  $vol_srcs[0],
	dst_volumes        =>  $vol_dsts[1],
	relationship_type  =>  'VAULT',
     );

    my $snapmirrors_ac = $retval_ac->{'sm_obj'};
    my $snapmirror_ac = $snapmirrors_ac->[0];
    snapmirror_resync(node_src =>  $src_node,
		       node_dst =>  $dst_node,
		       sm_objs  =>  $snapmirrors_ac,
		      );
    $snapmirror_ac->wait_for_transfer_to_complete();

    $log->step('Delete some of the snapshots from the src vol.');
    for my $j (@ss_del) {
	 NACL::STask::VolumeSnapshot->purge(
		command_interface  => $src_node,
		volume             => $vol_srcs[0],
		vserver            => $vol_srcs[0]->vserver,
		snapshot           => $j,
	 );
	 $log->comment("Delete snapshot "  . $j . " from " . $src_vol_name);
    }

    $vol_srcs[0]->snapshot_create(
        snapshot           => "sshot_del",
        'snapmirror-label' => 'weekly'
    );


    $snapmirror_ac->update('source-snapshot' => "sshot_del",
                            nacltask_wait => 1,
                          );

    # do restore from C to A  while update is going on from A to B

    $sm_obj_dr->modify('throttle' => '5');

    $sm_obj_dr->update('source-snapshot' => "sshot_del",
                        nacltask_wait => 0);

    $log->step('Run incremental restore with deleted snapshots from C->A');
    for my $k (@ss_del) {
        $log->step("Restore back snapshot " . $k);

        snapmirror_restore(node_src => $src_node,
                      node_dst => $dst_node,
                      sm_objs  => $snapmirrors_ac,
                      snapshot => $k,
                      validate => $validate,
                      nacltask_wait =>1,
                      );
    }

    push @snapmirrors,$sm_obj_dr;
    push @snapmirrors,$snapmirror_ac;

    $log->exit;
    return \@snapmirrors;
}



#private method
sub change_sfo_setting {
    my %args = validate_with(
        params => \@_,
        spec => {
            nodes  => { type => ARRAYREF|SCALAR, optional => 0 },
	    option => { type => SCALAR, default => 'false' },
        },
    );

    foreach my $node_obj (@{$args{nodes}}) {
        my $ha_cs_obj = NACL::CS::ClusterHa->fetch(
	    command_interface => $node_obj);
	if ($ha_cs_obj->configured() ne $args{option} ||
	    $ha_cs_obj->sfo_configured() ne $args{option}) {
	    try {
		NACL::C::Cluster->modify_ha(
		    command_interface => $node_obj,
		    configured        => $args{option},
		);
	    } catch NATE::BaseException with {
		$log->warn('Failed to modify HA setting');
	    };
	    try {
		NACL::C::StorageFailover->modify(
		    command_interface => $node_obj,
		    node              => $node_obj->state()->node(),
		    enabled           => $args{option},
		);
	    } catch NATE::BaseException with {
		$log->warn('Failed to modify SFO setting');
	    };
	}
    }
} # end of sub change_sfo_setting



# private method
sub validate_abort {
    my $o          = @_;
    my $checkpoint = $o->snapshot_checkpoint;

    if ( $checkpoint <= 0 ) {
        $log->error(
' Checkpoint has not been created. Check if abort has been issued atleast 30 seconds after the transfer has started. '
        );
    }

}

# private method
sub validate_abort_hard {
    my $o          = @_;
    my $checkpoint = $o->snapshot_checkpoint;

    if ( $checkpoint != '-' ) {
        $log->error('Checkpoint not reverted to -');
        return;
    }
}


=head2 clear_fijis_toggle_auto_resync_scheduled_pcs

This method will,
    clear all fijis,
    set auto-resync flag, and
    set scheduled-pcs flag

=over

=item Options

(Required) node_src - src node object to set the flags.
(Required) node_dst - dst node object to set the flags.
(optional) clear_fiji - clear all fijis or not : default is 1 to clear
(optional) toggle_auto_resync - Set the value of auto-resync based on 'enable_auto_resync'. Skip otherwise.
(optional) enable_auto_resync - 0 (default): disable auto-resync; 1: enable it
(optional) toggle_scheduled_pcs - Set the value of scheduled-pcs based on 'enable_schedued_pcs'. Skip otherwise.
(optional) enable_scheduled_pcs - 0 (default): disable scheduled-pcs; 1: enable it

=item Return

    none.

=back

=cut

sub clear_fijis_toggle_auto_resync_scheduled_pcs {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            node_src      => { type => OBJECT, optional => 0 },
            node_dst      => { type => OBJECT, optional => 0 },
            clear_fiji    => { type => SCALAR, optional => 1, default => '1' },
            toggle_auto_resync => { type => SCALAR, optional => 1, default => 1 },
            enable_auto_resync => { type => SCALAR, optional => 1, default => '0' },
            toggle_scheduled_pcs => { type => SCALAR, optional => 1, default => 1 },
            enable_scheduled_pcs => { type => SCALAR, optional => 1, default => '0' },
        },
    );

    # get HA nodes to make sure flags are consistant on both nodes
    my $node_src_ha_name = NACL::STask::Node->get_partner(command_interface => $args{node_src});
    my $node_src_ha      = NACL::STask::Node->new(name => $node_src_ha_name);
    my $node_dst_ha_name = NACL::STask::Node->get_partner(command_interface => $args{node_dst});
    my $node_dst_ha      = NACL::STask::Node->new(name => $node_dst_ha_name);

    # clear all fijis
    if ($args{clear_fiji}) {
        fiji_clear_all( node => $args{node_src} );
        fiji_clear_all( node => $node_src_ha );
        fiji_clear_all( node => $args{node_dst} );
        fiji_clear_all( node => $node_dst_ha );
    }

    # set flag for auto-resync
    if ($args{toggle_auto_resync}) {
        enable_disable_autoresync(node => $args{node_src}, enable => $args{enable_auto_resync});
        enable_disable_autoresync(node => $args{node_dst}, enable => $args{enable_auto_resync});
        enable_disable_autoresync(node => $node_src_ha, enable => $args{enable_auto_resync});
        enable_disable_autoresync(node => $node_dst_ha, enable => $args{enable_auto_resync});
    }

    # set flag for scheduled-pcs
    if ($args{toggle_scheduled_pcs}) {
        enable_disable_scheduled_pcs(node => $args{node_src}, enable => $args{enable_scheduled_pcs});
        enable_disable_scheduled_pcs(node => $args{node_dst}, enable => $args{enable_scheduled_pcs});
        enable_disable_scheduled_pcs(node => $node_src_ha, enable => $args{enable_scheduled_pcs});
        enable_disable_scheduled_pcs(node => $node_dst_ha, enable => $args{enable_scheduled_pcs});
    }
}


# checker for Flex groups
# checker has to be run for each constituent
# XXX integrate it into the other checker methods
sub sm_checker_fg {

    my %args = validate_with(
        params => \@_,
        spec => {
            sm_obj   => {type => ARRAYREF},
            timeout  => {type => SCALAR, default  => 600},
            snapshot => {type => SCALAR, optional => 1},
        },
    );
    my $log = log_global(1);

    my $sm_obj = @{ $args{sm_obj} }[0];
    $log->trace('Got sm obj '. Dumper($sm_obj));
    my $snapshot         = $args{snapshot};
    my $timeout          = $args{timeout};
    my $source_path      = $sm_obj->{source_path};
    my $destination_path = $sm_obj->{destination_path};
    $log->log('relationship destination path='.$destination_path);
    my ($vserver_src,$vol_src) = ($source_path =~ /^(.*):(.*)$/);
    my ($vserver_dst,$vol_dst) = ($destination_path =~ /^(.*):(.*)$/);
    $log->log('relationship vol dst ='.$vol_dst.' vserver dst='.$vserver_dst);
    my $node_src_obj = $sm_obj->{source_ci};
    my $node_dst_obj = $sm_obj->{command_interface};

    # find the FlexGroup constituents
    my @src_fg_constituents = NACL::STask::Volume->get_flexgroup_constituents(
        command_interface => $node_src_obj,
        vserver           => $vserver_src,
        volume            => $vol_src,
    );
    my @dst_fg_constituents = NACL::STask::Volume->get_flexgroup_constituents(
        command_interface => $node_dst_obj,
        vserver           => $vserver_dst,
        volume            => $vol_dst,
    );

    # find the newest snapshot on the parent FlexGroup
    my $snapshot_to_check = $snapshot || $sm_obj->get_one_state_attribute('exported-snapshot');
    $log->log('Snapshot that will be used for flexgroup snapmirror checker is '.$snapshot_to_check);
    my @jobs;
    my $i = 0;
    foreach my $constituent_sm_obj (@dst_fg_constituents) {
        $log->log(
            'Validate flexgroup-index matches between source and destination '
          . 'constituent volumes'
        );
        my $src_index = $src_fg_constituents[$i]->get_one_state_attribute('flexgroup-index');
        my $dst_index = $dst_fg_constituents[$i]->get_one_state_attribute('flexgroup-index');
        $log->log(
            'flexgroup-index for source constituent volume '
          . $src_fg_constituents[$i]->volume . ' is ' . $src_index
        );
        $log->log(
            'flexgroup-index for destination constituent volume '
          . $dst_fg_constituents[$i]->volume . ' is ' . $dst_index
        );
        if ( $src_index ne $dst_index ) {
            NATE::BaseException->throw(
                'flexgroup-index doesn\'t match between source and destination '
              . 'constituent volumes'
            );
        } else {
            $log->log(
                'flexgroup-index matches between source and destination '
              . 'constituent volumes'
            );
        }

        NACL::STask::Snapmirror->check(
            command_interface  => $node_dst_obj,
            'destination-path' => $vserver_dst . ':' . $constituent_sm_obj->volume,
            'target-snapshot'  => $snapshot_to_check,
            nacltask_wait      => 0,
            job_component      => \$jobs[ $i++ ]
        );
    }

    foreach my $index(0..(scalar @jobs - 1)) {
        #the below code only waits for the job to finish and doesnt check for the inconsistencies
        #Hence making the change
        #$_->wait_on_job( 'method-timeout' => $args{timeout} );
        NACL::STask::Snapmirror->wait_for_check(
            command_interface    => $node_dst_obj,
            'destination-path'   => $vserver_dst . ':' . $dst_fg_constituents[$index]->volume,
            job_component        => $jobs[$index],
            nacltask_if_mismatch => 'die',
           'method-timeout'      => $timeout
        );
    }

    return;
}

=head2 is_rel_flexgroup

    Check the given snapmirror relationship
    is flexGroup or not

=item Options

(Required) sm_obj - Snapmirror Nacl object

=item Return

    1  if Flexgroup relationship
    0  if not a Flexgroup relationship

=cut

sub is_rel_flexgroup {
    my %args = validate_with(
        params => \@_,
        spec => {
            sm_obj     => {type => HASHREF},
        },
    );
    my $sm_obj          = $args{sm_obj};

    # Non Flexgroup just wait for the attribute
    my $rel_group_type = $sm_obj->get_one_state_attribute('relationship-group-type');

    return ($rel_group_type =~ /flexgroup/i) ? 1 : 0;
}

=head2 fg_wait_on_attributes

This function is a Flexgroups-enabled variant of the B<wait_on_attributes>
method available to snapmirror objects. This variant determines all the
constituent elements of the flexgroup and polls all of them for the desired
attributes. The first consituent that matches all the requested attributes
will cause the function to return. If the timeout is reached without all
attributes matching, an exception will be thrown.

=over

=item Options

=over

=item B<sm_obj>

(Required) A B<NACL::STask::Snapmirror> object, the relationship to poll.

=item B<sm_group>

(Optional) An array reference of B<NACL::STask::Snapmirror> objects that
represent all the constituent elements of the flexgroup that C<sm_obj>
represents. If this is not passed, then B<fg_get_constituents> will be called
with the C<sm_obj> parameter.

=item B<attributes>

(Required) An array reference of hash references that define the attributes
to check.

=item B<timeout>

(Optional) Timeout in seconds. Default is 300.

=item B<polling_interval>

(Optional) The interval to wait (in seconds) between attempts to poll the set
of constituent elements. This value will be passed to B<Tharn::snooze> between
the polling of the last element in the set and the restarting of the polling
with the first element. The default value is 2 seconds.

=item B<show_all_fields>

(Optional) A boolean value that determines whether each polling of the
constituent elements shows the full set of snapmirror fields, or only those
fields being polled. The default is C<0> (false), meaning that only the
fields being polled will be displayed.

=back

=item Exceptions

=over

=item B<NACL::Exceptions::Timeout>

If none of the constituent elements matches the specified values for attributes
before the timeout occurs, this exception will be thrown.

=back

If the specified attributes are found, the function will return void (rather
than throwing an exception).

=cut

sub fg_wait_on_attributes {

    my %args = validate_with(
        params => \@_,
        spec   => {
            sm_obj           => { type => HASHREF },
            sm_group         => { type => ARRAYREF, optional => 1 },
            attributes       => { type => ARRAYREF },
            timeout          => { type => SCALAR, default => 300 },
            polling_interval => { type => SCALAR, default => 2 },
            show_all_fields  => { type => BOOLEAN, default => 1 },
        },
    );
    my $sm_obj           = $args{sm_obj};
    my $ar_attributes    = $args{attributes};
    my $timeout          = $args{timeout};
    my $polling_interval = $args{polling_interval};
    my $show_all_fields  = $args{show_all_fields};
    my $sm_constituents  =
        $args{sm_group} || fg_get_constituents(sm_obj => $sm_obj);

    # FlexGroup relationship
    # Wait on each constituent relationship
    # for attributes with very short poll time and timeout

    my $found    = 0;
    my $end_time = time + $timeout;
    while (! $found) {
        # Wait on each constituent relationship
        # for attributes with very short poll time and timeout
        for my $sm_c (@$sm_constituents) {
            try {
                $sm_c->wait_on_attributes(
                    attributes       => $ar_attributes,
                    polling_interval => $polling_interval,
                    show_all_fields  => $show_all_fields,
                    # This causes wait_on_attributes to only poll once, then
                    # immediately return or throw the exception:
                    'method-timeout' => -1,
                );
                $found = 1;
            } catch NACL::Exceptions::Timeout with {
                $Log->debug(
                    'Attribute not yet found: ' . $sm_c->destination_path
                );
            };

            if (time > $end_time) {
                NACL::Exceptions::Timeout->throw(
                    "Timed out after waiting for $timeout seconds.\n" .
                    Dumper($ar_attributes)
                );
            }
            last if ($found);
        }

        if ($found) {
            $log->debug('Attributes found: ' . Dumper($ar_attributes) );
            last;
        } else {
            Tharn::snooze($polling_interval);
            $log->debug('Iterate again on all constituents');
        }
    } ## end while (! $found)

    return;
}
=head2 fg_wait_on_attributes_all_constituents

This function is one of the Flexgroup-enabled variants of the B<wait_on_attributes>
method available to snapmirror objects. This variant polls one constituent at a time
,waits till the constituent being polled reaches the required attribute value and
then goes on to poll the next constituent.This function returns when all constituents
have reached the required attribute atleast once in the interval this function executes.
Exception is thrown on timeout.


=over

=item Options

=over

=item B<sm_obj>

(Required) A B<NACL::STask::Snapmirror> object, the relationship to poll.

=item B<sm_group>

(Optional) An array reference of B<NACL::STask::Snapmirror> objects that
represent all the constituent elements of the flexgroup that C<sm_obj>
represents. If this is not passed, then B<fg_get_constituents> will be called
with the C<sm_obj> parameter.

=item B<attributes>

(Required) An array reference of hash references that define the attributes
to check.

=item B<timeout>

(Optional) Timeout in seconds. Default is 300.

=item B<polling_interval>

(Optional) The interval to wait (in seconds) between attempts to poll the set
of constituent elements. This value will be passed to B<Tharn::snooze> between
the polling of the last element in the set and the restarting of the polling
with the first element. The default value is 2 seconds.

=item B<show_all_fields>

(Optional) A boolean value that determines whether each polling of the
constituent elements shows the full set of snapmirror fields, or only those
fields being polled. The default is C<0> (false), meaning that only the
fields being polled will be displayed.

=back

=item Exceptions

=over
=item B<NACL::Exceptions::Timeout>

If none of the constituent elements matches the specified values for attributes
before the timeout occurs, this exception will be thrown.

=back

If the specified attributes are found, the function will return void (rather
than throwing an exception).

=item See Also

fg_wait_on_attributes

=cut

sub fg_wait_on_attributes_all_constituents {

    my %args = validate_with(
        params => \@_,
        spec   => {
            sm_obj           => { type => HASHREF },
            sm_group         => { type => ARRAYREF, optional => 1 },
            attributes       => { type => ARRAYREF },
            timeout          => { type => SCALAR, default => 300 },
            polling_interval => { type => SCALAR, default => 2 },
            show_all_fields  => { type => BOOLEAN, default => 1 },
        },
    );
    my $sm_obj           = $args{sm_obj};
    my $ar_attributes    = $args{attributes};
    my $timeout          = $args{timeout};
    my $polling_interval = $args{polling_interval};
    my $show_all_fields  = $args{show_all_fields};
    my $sm_constituents =
      $args{sm_group} || fg_get_constituents( sm_obj => $sm_obj );
    # FlexGroup relationship
    # Wait on each constituent relationship
    # for attributes with very short poll time and timeout
    my $found    = 0;
    my $end_time = time + $timeout;

    # Wait on each constituent relationship
    # for attributes with very short poll time and timeout
    for my $sm_c (@$sm_constituents) {
        $found = 0;
        do {
            try {
                $sm_c->wait_on_attributes(
                    attributes       => $ar_attributes,
                    polling_interval => $polling_interval,
                    show_all_fields  => $show_all_fields,

                    # This causes wait_on_attributes to only poll once, then
                    # immediately return or throw the exception:
                    'method-timeout' => -1,
                );
                $found = 1;
            }
            catch NACL::Exceptions::Timeout with {
                $log->log(
                    'Expected value for the attribute not yet found: '
                      . $sm_c->destination_path );
                Tharn::snooze($polling_interval);
            };
        } while ( !$found && time < $end_time );
    }

    if ($found) {
        $log->log( 'Attributes found: ' . Dumper($ar_attributes) );
    }
    else {
        NACL::Exceptions::Timeout->throw(
                "Expected value for the attribute not found after waiting for :"
              . $timeout
              . " seconds" );
    }
    return;
}

=head2 fg_get_constituents

    This method returns array ref of
    snapmirror object of all constituents
    for given FlexGroup SM Object

    This function mainly used for
    checking attributes like 'snapshot-checkpoint'
    on all constituent relationships for
    FlexGroup snapmirror relationship

=over

=item Options

(Required) sm_obj - FlexGroup Snapmirror Nacl object

=item Return

    Array Ref of all constituent
    snapmirror objects of
    FlexGroup relationship

=back

=cut

sub fg_get_constituents {

    my %args = validate_with(
        params => \@_,
        spec => {
            sm_obj     => {type => HASHREF},
        },
    );
    my $sm_obj          = $args{sm_obj};

    my $node_dst_obj = $sm_obj->{command_interface};
    my $destination_path = $sm_obj->{destination_path};
    my ($vserver_dst,$vol_dst) = ($destination_path =~ /^(.*):(.*)$/);

    # find the FlexGroup constituents
    my @fg_constituents = NACL::STask::Volume->find(
        command_interface => $node_dst_obj,
        filter            => {
            vserver => $vserver_dst,
            volume  => $vol_dst . '__*',
        },
    );

    my $Sm_constituents;
    foreach (@fg_constituents) {
        push @{$Sm_constituents},
            NACL::STask::Snapmirror->find(
                command_interface  => $node_dst_obj,
                filter => {
                    'destination-path' => $vserver_dst . ':' . $_->volume,
                },
            );
    }

    return $Sm_constituents;
}

=head2 fg_snapshot_checkpoint

Fetches the snapshot checkpoint values for all members of a Flexgroup. The
return value is an array reference of the checkpoint values.

=over

=item Options

(Required) C<sm_obj> - A Flexgroup snapmirror object

(Optional) C<sm_group> - If passed, must be an array reference of
B<NACL::STask::Snapmirror> objects. If given, this is used directly rather
than having a call made to B<fg_get_constituents> on the object given by
the C<sm_obj> option. This is faster (if you already have the list of
constituents) as it saves the overhead of fetching the data.

=item Exceptions

The only exception thrown here is if an invalid option is given.

=back

=cut

sub fg_snapshot_checkpoint {
    my %args = validate_with(
        params => \@_,
        spec   => {
            sm_obj   => { type => HASHREF },
            sm_group => { type => ARRAYREF, optional => 1 },
        }
    );

    my $sm       = $args{sm_obj};
    my $sm_group = $args{sm_group};

    if (! $sm_group) {
        $sm_group = fg_get_constituents(sm_obj => $sm);
    }

    my @checkpoints = map { $_->snapshot_checkpoint } @{$sm_group};
    for (@checkpoints) {
        if ($_ eq q{-}) { $_ = 0; }
    }

    return \@checkpoints;
}

=head2 fg_checkpoint_verify

This method takes a Flexgroup snapmirror object and an array reference of
checkpoints previously derived by B<fg_snapshot_checkpoint>, and verifies
that all consituents are at or past the corresponding previous checkpoints.
The return value is C<1> (true) if all are verified, and C<0> if any of them
do not verify.

=over

=item Options

(Required) C<sm_obj> - A Flexgroup snapmirror object

(Optional) C<sm_group> - If passed, must be an array reference of
B<NACL::STask::Snapmirror> objects. If given, this is used directly rather
than having a call made to B<fg_get_constituents> on the object given by
the C<sm_obj> option. This is faster (if you already have the list of
constituents) as it saves the overhead of fetching the data.

=item Exceptions

An exception may be thrown here if an invalid option is given. Also, if
C<sm_group> is passed and has a different number of elements than the
array reference in C<checkpoint> then an exception will be thrown.

=back

=cut

sub fg_checkpoint_verify {
    my %args = validate_with(
        params => \@_,
        spec   => {
            sm_obj     => { type => HASHREF },
            sm_group   => { type => ARRAYREF, optional => 1 },
            checkpoint => { type => ARRAYREF },
        }
    );

    my $sm       = $args{sm_obj};
    my $sm_group = $args{sm_group};
    my $cp       = $args{checkpoint};

    if (! $sm_group) {
        $sm_group = fg_get_constituents(sm_obj => $sm);
    }

    if (@{$cp} != @{$sm_group}) {
        NATE::Exceptions::Argument->throw(
            'Number of elements in "checkpoint" parameter does not match ' .
            'number of constituents in flexgroup'
        );
    }

    my $verified = 1;
    for my $idx (0 .. $#{$sm_group}) {
        if (! $sm_group->[$idx]->checkpoint_verify(checkpoint => $cp->[$idx])) {
            $verified = 0;
            my $message = 'Checkpoint verification failed for constituent ' .
                'relationship: ' . $sm_group->[$idx]->source_path . '->' .
                $sm_group->[$idx]->destination_path;
            $log->comment($message);
        }
    }

    return $verified;
}

# find common snapshots across SRC and DST
# will compare version-uuid and snapshot name
sub common_snapshots {
    my %args = validate_with(
        params => \@_,
        spec => {
            sm_obj => {type => HASHREF},
        },
    );

    my $Sm = $args{sm_obj};
    my ($source_vserver,$source_volume) = ($Sm->source_path =~ /^(.*):(.*)$/);
    my ($dest_vserver,$dest_volume) = ($Sm->destination_path =~ /^(.*):(.*)$/);
    my $dest_ci = $Sm->command_interface;
    my $source_ci = $Sm->source_ci;

    # Start by getting the snapshots on the source end
    my @snapshots = NACL::CS::VolumeSnapshot->fetch(
        command_interface => $source_ci,
        filter            => {
            vserver => $source_vserver,
            volume  => $source_volume,
        },
        requested_fields => [qw(snapshot version-uuid)],
        allow_empty      => 1
    );

    my %uuid_map;
    # Convert the list of snapshots to the hash %uuid_map
    for my $state (@snapshots) {
        $uuid_map{$state->version_uuid} = $state->snapshot;
    }

    # Now get the list of snapshots on the destination end
    @snapshots = NACL::CS::VolumeSnapshot->fetch(
        command_interface => $dest_ci,
        filter            => {
            vserver => $dest_vserver,
            volume  => $dest_volume,
        },
        requested_fields => [qw(snapshot version-uuid)],
        allow_empty      => 1
    );

    my @common;
    # For each record in @snapshots, if the UUID matches one we have in
    # %uuid_map, then that is a common snapshot. Put the name in @common.
    for my $state (@snapshots) {
        if (my $exists = $uuid_map{$state->version_uuid}) {
            push(@common, $exists);
        }
    }

    return @common;
}

=head2 set_wafl_flags

Method to set wafl flags

=item Options

=over

=item nodes

(Required) The list of nodes to set the flags on.

=item flags

(Required) The list of flags to set on the nodes. flag1=>value1,flag2=>value2

=back

=cut

sub set_wafl_flags {
    my %args = validate_with(
        params => \@_,
        spec   => {
            nodes => { type => ARRAYREF },
            flags => { type => SCALAR },
        });

    my @nodes = @{ $args{nodes} };
    my @all_entries  = split/,/, $args{flags};
    my (@flags, @values);
    foreach my $e (@all_entries) {
        my ($f, $v) = split /=>/, $e;
        push (@flags, $f);
        push (@values, $v);
    }
    my $log = log_global();
    $log->enter();

    my $i = 0;
    foreach my $f (@flags) {
        my $val = $values[$i];
        foreach my $n (@nodes) {
            $log->comment('Setting flag '. $f . ' with  value: ' . $val . ' on node: ' . $n->node());
            my $api = $n->get_7m_or_nodescope_apiset();
            $api->set_privilege( 'privilege-level' => 'test' );
            $api->setflag(
                flag  => $f,
                value => $val);
        }
        $i++;
    }

    $log->exit();
    return;
}

=head2 match_vol_attributes

The method will match the the vol attributes between two volumes.

=item Options

=over

=item src_vol

(Required) The src vol object

=item dst_vol

(Required) The dst vol object

=item match_fields

 (Optional) The list of vol showfileds to match between the src and dst vol when mode = vol_show
            Will be ignored if mode = get_vol_config

=item match_all_but

(Optional) The list of attributes that are expected to have different values when mode = get_vol_config.
           Will be ignored if mode = vol_show

=item unequal_keys

(Optional) The list of attributes that user expects to be different and wants to verify when mode = get_vol_config.
           Will be ignored if mode = vol_show

=back

=cut

sub match_vol_attributes {
    my %args = validate_with (
                    params      =>  \@_,
                    spec        => {
                     src_vol    => { type => OBJECT },
                     dst_vol    => { type => OBJECT },
                     mode       => { type => SCALAR, default => 'get_vol_config', regex => qr /^(get_vol_config|vol_show)$/},
                     match_fields  => { type => ARRAYREF|UNDEF, optional => 1 },
                     match_all_but => { type => ARRAYREF|UNDEF, optional => 1 },
                     unequal_keys  => { type => ARRAYREF|UNDEF, optional => 1 },});

    my $svol = $args{src_vol};
    my $dvol = $args{dst_vol};
    my $matchallbut  = $args{match_all_but};
    my $unequal_keys = $args{unequal_keys};
    my $match_fields = $args{match_fields};
    my $log  = log_global();
    my $mode = $args{mode};

    if ($mode =~ /get_vol_config/) {

        $log->comment('Adopting the get_vol_config_attributes mode');
        if (defined $match_fields) {
            $log->comment('match_fields option is only for the vol_show mode.... ignoring');
        }

        $log->comment ('Get filehandle info for src vol: ' . $svol->volume());
        my $src_fh = get_fh ( vol_obj => $svol );

        $log->comment ('Get filehandle info for dst vol: ' . $dvol->volume());
        my $dst_fh = get_fh ( vol_obj => $dvol );

        my $sn   = NACL::C::Node->new( name => $svol->get_local_node_name() );
        my $dn   = NACL::C::Node->new( name => $dvol->get_local_node_name() );
        my @unexpected_results;

        $log->comment('Get volume attributes for src vol: ' . $svol->volume());
        my $svol_attr = runcmd_get_vol_config_attrs (
                                node    => $sn,
                                fsid    => $src_fh->{fsid},
                                snapid  => $src_fh->{snapid});

        $log->comment('Get volume attributes for dst vol: ' . $dvol->volume());
        my $dvol_attr = runcmd_get_vol_config_attrs (
                                node    => $dn,
                                fsid    => $dst_fh->{fsid},
                                snapid  => $dst_fh->{snapid});

        $log->comment('Match the attributes for both the volumes');
        foreach my $k (keys %{$svol_attr}) {
            $log->comment('Working on key: ' . $k);
            if ( $k eq 'fileshim-suffix' ) {
                if (   !defined($svol_attr->{$k})
                    && !defined($dvol_attr->{$k})){
                    $log->comment('Both volumes undefined for ' . $k );
                    next;
                }
            }

            if ($svol_attr->{$k} ne
                $dvol_attr->{$k}) {
                if ( any { $k eq $_ } @{$matchallbut} ) {
                    $log->comment('Exempt key - mismatch allowed:' . $k );
                } else {

                    NATE::BaseException->throw("Key ($k)values do NOT match.");
                }
            }
        }

        if (defined $unequal_keys) {
            $log->comment('Ensure the fields that are expected to be a mis-match are really so');
            foreach my $k (@{$unequal_keys}) {
                $log->comment('...working on key: ' . $k);
                if ($svol_attr->{$k} eq
                    $dvol_attr->{$k}) {
                    NATE::BaseException->throw("Key ($k) expected to mis-match but they didnt");
                } else {
                    $log->comment("Values for key ($k) were different as expected ");
                }
            }
        }
    } else {

        $log->comment('Adopting the vol_show mode');
        if (defined $unequal_keys || defined $matchallbut) {
            $log->comment('unequal_keys and match_all_but options are only for the get_vol_config_attrs mode.... ignoring');
        }

        $log->comment('Will match attributes: ' . Dumper($match_fields));

        $log->comment('Gathering the vol show info');
        my $volinfo1 = get_volume_show (vol_obj => $svol);
        my $volinfo2 = get_volume_show (vol_obj => $dvol);

        $log->comment('Comparing the vol show info');
        foreach my $k (@{$match_fields}) {
            $log->comment('... looking at key : ' . $k);
            NATE::BaseException->throw( "Value for key ($k) does not match. " .
                        $volinfo1->{$k} . ' : ' .  $volinfo2->{$k} )
            if ( $volinfo1->{$k} ne $volinfo2->{$k} );
        }
    }

    $log->exit();
    return;
}

#Given the vol object returns the values for certain volume fields.
sub get_volume_show {
    my %args = validate_with (
            params => \@_,
            spec   => {
                vol_obj => { type => OBJECT }});

    my $v = $args{vol_obj};
    my $log = log_global();
    $log->enter();

    $log->comment('Gathering the vol obj state for : ' . $v->volume());
    my $op = $v->state (
                requested_fields => [
                    'inodefile-public-capacity', 'files',
                    'files-used',                'size',
                    'maxdir-size',               'percent-used',
                    'used',]);

    $log->exit();
    return $op;
}

=head2 get_fh

Method runs "showfh <volpath> and returns the output.

=item Options

=over

=item vol_obj

(Required) The vol object to run the command on

=back

=cut


sub get_fh {
    my %args = validate_with (
            params =>  \@_,
            spec   => {
                vol_obj => { type => OBJECT } });

    my $log = log_global();
    my $v   = $args{vol_obj};
    $log->enter();

    my $n = NACL::C::Node->new(name => $v->get_local_node_name());
    my $api = $n->get_7m_or_nodescope_apiset();

    $log->comment('Run show filehandle command');
    my $op = $api->showfh( filename => '/vol/' . $v->volume() );
    my $response = ${ $op->get_parsed_output() }[0];

    $log->comment('Output obtained is: ' . Dumper($response));
    $log->exit();

    return $response;
}

=head2 runcmd_get_vol_config_attrs

Method runs the get_vol_config_attrs command with the following options

get_vol_config_attrs -f <fsid> -snapid <snapid>
get_vol_config_attrs -f <fsid> -nvfail

=item Options

=over

=item apiset

(Required) The node apiset object to run the command on

=item fsid

(Required) The fsid of the vol

=item snapid

(Required) The snapid of the vol

=item nvfail

(Optional) Set the option if you want to run the command
get_vol_config_attrs -f <fsid> -nvfail

=item exp_err

(Optional) Pass the expected error string that the command will ignore if encountered.

=back

=cut

sub runcmd_get_vol_config_attrs {
    my %args = validate_with (
                params       =>  \@_,
                spec         => {
                    node    => { type => OBJECT, },
                    fsid    => { type => SCALAR },
                    snapid  => { type => SCALAR, optional => 1 },
                    nvfail  => { type => SCALAR, default => 0, optional => 1},
                    exp_err => { type => SCALAR|UNDEF, optional => 1, }
                });

    my $log     = log_global();
    my $fsid    = $args{fsid};
    my $snapid  = $args{snapid};
    my $n       = $args{node};
    my $api     = $n->get_7m_or_nodescope_apiset();
    my $exp_err = $args{exp_err};
    my $error_hit = 0;
    my $response;
    $log->enter();

    $log->comment('Run the get_vol_config_attrs command');
    if (defined $exp_err) {
        $log->comment('...Expect error: ' . $exp_err);
    }
    if (!$args{nvfail}) {
        $log->comment ('.. with fsid: ' . $fsid .
                       ' .. and snapid: ' . $snapid);
        try {
            $response = $api->get_vol_config_attrs (
                            "fsid"      => $fsid,
                            "snapid"    => $snapid);
        } catch NATE::BaseException with {
            my $ex = shift;
            if (!defined $exp_err) {
                $ex->throw(); #re-throw the error if we are not expecting one
            } else {
               if ($ex->text =~ /$exp_err/) {
                    $log->comment('Hit an expected err .. ignoring');
                    $error_hit = 1;
               } else {
                    $ex->throw();
               }
            }
        };

        if (defined $exp_err && !$error_hit)  {
            NATE::BaseException->throw('Should not be here ... command did not fail ' .
                                       'with expected error');
        } elsif(defined $exp_err && $error_hit) {
            $log->comment('Command completed as expected ... returning');
            return;
        }
        my $op = ${$response->get_parsed_output()}[0];
        $log->comment('Response obtained is: ' . Dumper($op));
        $log->exit();
        return $op;

    } else {
        $log->comment('... with the -nvfail option');
        try {
            $response = $api->get_vol_config_attrs (
                                "fsid"      => $fsid,
                                "nvfail"    => 1);
        } catch NATE::BaseException with {
            my $ex = shift;
            if ($ex->text() !~ /Invalid argument/i) {
                $ex->throw();
            } else {
                $log->comment('Hit the exepcted error');
                return;
            }
        };
    }
}

=head2 enable_dedup_compression

    The purpose of this workflow is to enable storage efficiency - dedup and compression, on the specified volume/list of volumes

=over

=item Example

 To enable compression on the constituent FG volumes:

  enable_dedup_compression(vol_objs => \@fg_constituents,
                       compression => '1',
                       );

 To enable dedup and compression on a FlexVol:

  enable_dedup_compression(vol_objs => [ $Vol ],
		       dedup => '1',
		       compression => '1',
		       );

=item Options

=over

=item vol_objs

(Required): Reference to an array of volume objects of type NACL::STask::Volume.

=item dedup

(Optional): Flag to indicate that dedup needs to be enabled. Default: 0

=item compression

(Optional): Flag to indicate that compression needs to be enabled. Default: 0

=item se_timeout

(Optional): Time limit for the SE operation to complete. Default: 30mins

=back

=cut


sub enable_dedup_compression {
    my %args = validate_with (
        params      => \@_,
        allow_extra => 1,
        spec        => {
	    vol_objs => { type => ARRAYREF },
	    dedup   => { type => SCALAR, default => '0' },
	    compression => { type => SCALAR, default => '0' },
	    se_timeout => { type => SCALAR, default => '1800'},
        },);

    my $vol_obj = delete $args{vol_objs};
    my @const_objs = _list_of_vals($vol_obj);
    my $dedup_flag = $args{dedup};
    my $comp_flag = $args{compression};
    my $storage_efficiency_timeout = $args{se_timeout};
    my @subtest_se;

    foreach my $o(@const_objs){
	my %args_func = ('volume'   => $o,
			 'dedup'    => $dedup_flag,
			 'compression' => $comp_flag,
			 se_timeout => $storage_efficiency_timeout,
			);
	push @subtest_se, create_parallel_processes('_turn_on_dedup_compression',\%args_func);
    }

    run_processes_bg('_turn_on_dedup_compression',\@subtest_se);
}

#Private subroutine
sub _turn_on_dedup_compression {
    my %args = validate_with (
        params      => \@_,
        allow_extra => 1,
        spec        => {
	    volume => { type => OBJECT },
	    dedup  => { type => SCALAR },
            compression => { type => SCALAR },
	    se_timeout	=> { type => SCALAR },
        },);

    my $vol = $args{volume};
    my $dedup_flag = $args{dedup};
    my $comp_flag = $args{compression};
    my $storage_efficiency_timeout = $args{se_timeout};

    #--------------------------------------------------------------------------
    $log->step('SUBROUTINE: Enable Dedup and/or Compression on the specified volume.');
    #--------------------------------------------------------------------------
    my $vol_sis = $vol->efficiency_on;
    if($dedup_flag){
        $vol_sis->start(
            'scan-old-data'   => 'true',
            'dedupe'          => 'true',
            force             => 'true',
            nacltask_wait     => 1,
            'method-timeout'  => $storage_efficiency_timeout,
            );
        $log->log('Dedup is completed on volume '.$vol->volume);
    }
    if($comp_flag){
        $vol_sis->modify('background-compression' => 'true',
                         'inline-compression'     => 'true',);
        $vol_sis->start(
            'scan-old-data'  => 'true',
            'compression'    => 'true',
            force            => 'true',
            nacltask_wait    => 1,
            'method-timeout' => $storage_efficiency_timeout,
            );
        $log->log('Compression is completed on volume '.$vol->volume);
    }
    return;
}


=head2 is_shiner

    Check the given snapmirror relationship
    is shiner or not

=item Options

(Required) sm_obj - Snapmirror Nacl object

=item Return

    1  if shiner(ava) relationship
    0  if not a shiner relationship

=cut

sub is_shiner {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            snapmirror => { type => OBJECT },
        }
    );

    my $shiner = 0;
    my $sm = $args{snapmirror};
    my $path = $sm->destination_path();
    if ($path =~ /share\/(.*)$/) {
        $shiner = 1;
    }

    return $shiner;
}


=head2 get_destination_volume

    get destination ontap volume or a ava share of a snapmirror relationship

=item Options

(Required) sm_obj - Snapmirror Nacl object

=item Return

    destination ontap volume or a ava share (scalar)

=cut

sub get_destination_volume {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            snapmirror => { type => OBJECT },
        }
    );

    my $sm = $args{snapmirror};
    my $path = $sm->destination_path();
    my $vol;
    if ($path =~ /share\/(.*)$/) {
        $vol = $1;
    } else {
        $vol = $sm->destination_volume();
    }

    return $vol;
}


=head2 check_io_fence_client

    check if file is IO fenced (after SFR)

=item Options

(Required) mount - Mount Nacl object
(Required) client - Client Nacl object (NFS)
(Required) file - File name to check

=item Return

    1: file is IO fenced
    0: file is not fenced

=cut

sub check_io_fence_client {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            mount  => { type => OBJECT },
            client => { type => OBJECT },
	    file   => { type => SCALAR },
        }
    );

    my $mount  = $args{mount};
    my $client = $args{client};
    my $file   = $args{file};

    my ($fence, $ex);
    my $file_path = $mount->mount_point() . $file;
    try {
	$client->apiset->dd(
	    'input-file'  => $file_path,
	    'output-file' => $file_path . '_dd.out',
	);
    } catch NATE::BaseException with {
	$ex = shift;
	$log->log( $ex->text() );
    };

    if ($ex) {
	$fence = 1;
    } else {
	$fence = 0;
    }

    return $fence;
}


=head2 check_meta_files

    check meta files (after SFR)

=item Options

(Required) vol_obj - volume Nacl object

=item Return

    0: no files found
    1: meta files found

=cut

sub enable_nfs_v4 {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            command_interface    => { type => OBJECT },
            vserver              => { type => SCALAR },
        }
    );


    my $cmd_int = $args{command_interface};
    my $vs      = $args{vserver};
    my $log     = log_global();
    $log->enter();

    $log->comment('Enabling NFS v.4 on vserver: ' . $vs);
    NACL::STask::VserverNfs->modify(
            command_interface           =>    $cmd_int,
            vserver                     =>    $vs,
            'v4.0'                      =>    'true',
            'v4.1'                      =>    'true');

    $log->exit();
    return;
}


sub check_meta_files {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 0,
        spec        => {
            vol_obj => { type => OBJECT },
        }
    );

    my $vol_obj = $args{vol_obj};

    my $vol_name        = $vol_obj->volume();
    my $local_node_name = $vol_obj->get_one_state_attribute('node');
    my $local_node_obj  = NACL::C::Node->new( name => $local_node_name );

    my $response =
      $local_node_obj->get_7m_or_nodescope_apiset()
      ->execute_command( command => "setflag wafl_metadata_visible 1" );

    my $file_obj_1;
    my $file_obj_2;
    try{
	$file_obj_1 = NACL::C::Volume->explore(
	    command_interface => $local_node_obj,
	    format            => 'inode',
	    dump              => 'inum',
	    scope             => "$vol_name./metadir/SnapMirrorControl"
	);

	$file_obj_2 = NACL::C::Volume->explore(
	    command_interface => $local_node_obj,
	    format            => 'inode',
	    dump              => 'inum',
	    scope             => "$vol_name./metadir/granularTmp/xfer_state"
	);
    } catch NACL::APISet::Exceptions::InvalidParamValueException with{
        $log->log( 'Volume explore command dint find the files' );
    };

    if ( !( $file_obj_1 && $file_obj_2 ) ) {
        return 0;
    }
    else {
        return 1;
    }
}

# will check if the volume is residing on a composite aggr
# returns true/false
sub is_vol_on_CA {
    my @args = @_;
    my %args = validate_with(
        params => \@args,
        spec   => {
            vol_obj => { type => OBJECT },
        }
    );

    my $aggr = $args{vol_obj}->aggregate();
    my $aggr_obj = NACL::STask::Aggregate->new(
           command_interface => $args{vol_obj}->command_interface(),
           aggregate         => $aggr,
    );
    my $is_composite = $aggr_obj->get_one_state_attribute('composite');
    $log->log('Volume '.$args{vol_obj}->volume().'is located on aggr '.$aggr.' composite='.$is_composite);

    return $is_composite;
}


# NOTE : this is a node scope throttle so it will throttle all the transfers on the node
sub throttle_transfer_on_node {
    my @args = @_;
    my %args = validate_with(
        params => \@args,
        spec   => {
            node_obj => { type => OBJECT },
            enable => { type => SCALAR, default => 'on', regex => qr/on|off/ },
            throttle_incoming => { type => SCALAR, default => 0 }, # 0 means unlimited
            throttle_outgoing => { type => SCALAR, default => 0 }, # 0 means unlimited
        }
    );

    my $ci = $args{node_obj}->command_interface();
    my $api = $ci->apiset( category => 'Node', interface => 'CLI', set => 'CMode' );
    my $cmd = 'options replication.throttle.enable '.$args{enable};
    my $raw = $api->execute_raw_command( command => $cmd, 'connectrec-timeout' => '300' );
    if ($args{enable} eq 'on') {
        $cmd = 'options replication.throttle.incoming.max_kbs '.$args{throttle_incoming};
        my $raw = $api->execute_raw_command( command => $cmd, 'connectrec-timeout' => '300' );
        $cmd = 'options replication.throttle.outgoing.max_kbs '.$args{throttle_outgoing};
        $raw = $api->execute_raw_command( command => $cmd, 'connectrec-timeout' => '300' );
    } else {
        $log->log('Skipping setting of incoming and outgoing throttle since enable is off');
    }
    return;
}

=head2 compare_afs

    check afs after a transfer

=item Options

(Required) src_handle - NACL::STask::Client::Mount object for the src volume
(Required) dst_handle - NACL::STask::Client::Mount object for the dst volume

=cut

sub compare_afs {
    my @args = @_;
    my %args = validate_with(
        params => \@args,
        spec   => {
            src_handle   => { type => OBJECT },
            dst_handle   => { type => OBJECT },
	    client	 => { type => OBJECT|SCALAR, default => 'localhost' },
            cmd_timeout  => { type => SCALAR, default => '1800' },
        }
    );

    my $log = log_global(1);
    my $sHandle = $args{src_handle};
    my $dHandle = $args{dst_handle};
    my $client_obj;
    if($args{client} eq 'localhost'){	#if no client name is passed.
	#Keeping this for compatibility with existing scripts
	$client_obj = NACL::C::Client->new( name => $args{client} );
    }else{	#if the object is passed
	$client_obj =$args{client};
    }

    #BURT 1039728 - /usr/bin/software/find not working as expected.
=head
    my %manifest = ();
    my $snapshot_helper = FG::DP::SnapshotHelper->new();
    $manifest{'mani_src'} = $snapshot_helper->get_manifest(
        nfs_mount => $sHandle,
        path      => ' '
        );
    $manifest{'mani_dst'} = $snapshot_helper->get_manifest(
        nfs_mount => $dHandle,
        path => ' '
        );
    this( $manifest{'mani_src'} )->should( Be->deeply_equal_to( $manifest{'mani_dst'} ) );
=cut
    my $mount_pt = $sHandle->mount_point();
    my $find_cmd = "find $mount_pt";
    my $find_output = $client_obj->apiset()->execute_command(
        command              => $find_cmd,
        'connectrec-timeout' => $args{cmd_timeout},
        'privilege-level'    => 'root'
    );

    #Segregate the directories, files and links
    my(@src_dirs_files_links, @dst_dirs_files_links);
    my @find_op_arr = split /\n/, $find_output;
    foreach my $path(@find_op_arr){
        if($path =~ /\.snapshot/){
            next;       #ignore
        }else{
            push(@src_dirs_files_links, $path);
        }
    }
    $mount_pt = $dHandle->mount_point();
    $find_cmd = "find $mount_pt";
    $find_output = $client_obj->apiset()->execute_command(
        command              => $find_cmd,
        'connectrec-timeout' => $args{cmd_timeout},
        'privilege-level'    => 'root'
    );

    @find_op_arr = split /\n/, $find_output;
    foreach my $path(@find_op_arr){
        if($path =~ /\.snapshot/){
            next;       #ignore
        }else{
            push(@dst_dirs_files_links, $path);
        }
    }

    if(scalar @src_dirs_files_links != scalar @dst_dirs_files_links){
	    NATE::BaseException->throw('The number of files, directories and links '.
		'between src and dst volumes do not match.');
    }
    $log->log('Consistent filesystem on both src and dst volumes.');
    return;
}

=head2 compare_ri2p_metafile

    Compare the RI2P metafile between source and destination FGs

=item Options

 (Required) sm_obj - Hashref to NACL::STask::Snapmirror objects

=cut

sub compare_ri2p_metafile {
    my %args = validate_with(
        params => \@_,
        spec   => {
            sm_obj              => { type => HASHREF },
            valid_records_exist => { type => SCALAR, optional => 1, default => 0 },
            no_of_valid_records => { type => SCALAR, optional => 1, default => 0 },
        }
    );

    my $log                 = log_global(1);
    my $sm_obj              = $args{sm_obj};
    my $src_node            = $sm_obj->{source_ci};
    my $dst_node            = $sm_obj->{command_interface};
    my $source_path         = $sm_obj->{source_path};
    my $destination_path    = $sm_obj->{destination_path};
    my ($src_vs,$src_FG)    = ($source_path =~ /^(.*):(.*)$/);
    my ($dst_vs,$dst_FG)    = ($destination_path =~ /^(.*):(.*)$/);
    my $valid_records_exist = $args{valid_records_exist};
    my $no_of_valid_records = $args{no_of_valid_records};

    $src_FG = NACL::STask::Volume->new(
        command_interface => $src_node,
        volume            => $src_FG,
        vserver           => $src_vs
    );
    $dst_FG = NACL::STask::Volume->new(
        command_interface => $dst_node,
        volume            => $dst_FG,
        vserver           => $dst_vs
    );

    my @src_constituents = $src_FG->get_constituents;
    my @dst_constituents = $dst_FG->get_constituents;

    my $src_apiset = $src_node->apiset;
    my $dst_apiset = $dst_node->apiset;
    my $count = 0;
    my $valid_records = 0;
    foreach ( 0 .. $#src_constituents ) {
        my $vol_name = $src_constituents[$_]->volume;
        my $src_ri2p_entries_raw = $src_apiset->execute_raw_command(
            command => "volume explore -format bplus -scope $vol_name.501p -list lhs:40",
            timeout => '300'
        );
        $log->log('Volume explore output for src is :' . Dumper($src_ri2p_entries_raw) );
        my @src_ri2p_entries = split(/\n/, $src_ri2p_entries_raw);

        $vol_name = $dst_constituents[$_]->volume;
        my $dst_ri2p_entries_raw = $dst_apiset->execute_raw_command(
            command => "volume explore -format bplus -scope $vol_name.501p -list lhs:40",
            timeout => '300'
        );
        $log->log('Volume explore output for dst is :' . Dumper($dst_ri2p_entries_raw) );
        my @dst_ri2p_entries = split(/\n/, $dst_ri2p_entries_raw);

        if ( $#src_ri2p_entries ne $#dst_ri2p_entries ) {
            NATE::BaseException->throw("Record count doesn't match");
        } else {
            #index 0 and 1 don't have any records, so skipping them
            foreach my $index ( 2 .. $#src_ri2p_entries ) {
                my @single_src_record = split(/\./, $src_ri2p_entries[$index]);
                $log->log("Src record: " . $src_ri2p_entries[$index]);
                my @single_dst_record = split(/\./, $dst_ri2p_entries[$index]);
                $log->log("Dst record: " . $dst_ri2p_entries[$index]);
                my $i;
                foreach $i ( 0 .. $#single_src_record ) {
                    if ( $i == $#single_src_record ) {
                        if ( $single_src_record[$i] == $single_dst_record[$i] ) {
                            if ( $valid_records_exist ) {
                                #valid record found due to ODR
                                $valid_records++;
                            } else {
                                #the last field should not have same value as source as
                                #it should be marked invalid on dst during transfer
                                NATE::BaseException->throw("Record not marked invalid");
                            }
                        } else {
                            #last field has invalid bit set, XOR it with 65536
                            #to get the fgindex. After XORing, the value should
                            #match with src
                            $single_dst_record[$i] ^= 65536;
                        }
                    }
                    NATE::BaseException->throw("Record mismatch found")
                    if  ! ( $single_src_record[$i] == $single_dst_record[$i] );
                }
                #keep track of number of records compared
                $count++;
            }
        }
    }


    NATE::BaseException->throw("Unexpected number of valid records found")
    if ( $valid_records ne $no_of_valid_records );

    $log->log(
        "RI2P metafiles on both src and dst FGs are consistent. "
      . "Total records compared: $count"
    );
    return $valid_records;
}

#######The following subroutine does Snapmirror Initialize, update and followed by Quiesce.
#######This is the basic operation needed before volume expand is done.

sub SM_initialize_update_quiesce {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_src         => { type => ARRAYREF },
            vol_dst         => { type => ARRAYREF },
            dataset_size    => { type => SCALAR, default => '300m' },
            auto_expand     => { type => SCALAR, default => 'true' },
            mirror_type     => { type => SCALAR, default => 'XDP' },
            sm_policy       => { type => SCALAR, default => 'MirrorAllSnapshots' },
            throttle        => { type => SCALAR, default  => 'unlimited' },
            sm_xfer_timeout => { type => SCALAR, default  => 3600 },
            run_update      => { type => SCALAR, default  => 0 },
            io_handle       => { type => ARRAYREF, optional => 1 },
            client          => { type => ARRAYREF | SCALAR },
            do_cleanup      => { type => BOOLEAN, default => 1 },
        },
    );

    my $log = log_global(1);

    my $sVol = @{ $args{vol_src} }[0];
    my $dVol = @{ $args{vol_dst} }[0];
    $log->log( 'VOL SRC =' . Dumper($sVol) );
    $log->log( 'VOL DST =' . Dumper($dVol) );
    my $vserver_src = $sVol->vserver();
    my $vserver_dst = $dVol->vserver();
    $log->log("Vserver SRC = $vserver_src");
    $log->log("Vserver DST = $vserver_dst");

    # get and verify the dataset size
    my $dataset_size = $args{dataset_size};
    my $throttle     = $args{throttle};
    $log->log( 'Dataset Size ' . $dataset_size );

    my $auto_expand = $args{auto_expand};
    my $mirror_type = $args{mirror_type};
    my $sm_policy   = $args{sm_policy};
    my $run_update  = $args{run_update};
    my $Filer_Src
        = NACL::C::Node->new( name => $sVol->get_local_node_name() );
    my $Filer_Dst
        = NACL::C::Node->new( name => $dVol->get_local_node_name() );

    # The client may come as a scalar or a list. If it's a list just use the first one
    my $client = ( ref $args{client} ) ? $args{client}->[0] : $args{client};

    # There are cases where the scalar value might be a comma-separated list:
    if ( $client =~ /,/ ) {
        $client = ( split /,/ => $client )[0];
    }
    $log->log("Using client $client");
    my $Client = NACL::C::Client->new( name => $client );

    #---------------------------------------------------------------
    $log->step('Create IO handle object if not defined');
    #---------------------------------------------------------------
    my $IOHandle = @{ $args{io_handle} }[0];
    if ( !( defined $IOHandle ) ) {
        $IOHandle = NACL::AppBuilder::WFC::IO->new(
            volume => $sVol,
            client => $Client
        );
    } ## end if ( !( defined $IOHandle...))

    #---------------------------------------------------------------
    $log->step(
        'Mount volume ' . $sVol->volume . ' on client ' . $Client->name );
    #---------------------------------------------------------------
    $IOHandle->create_mount( access => 'RW' );


    $IOHandle->data_populate(
        size      => $dataset_size,
        path      => 'file' . time,
        arguments => { dbfile => random_name_generator( prefix => "db_" ) }
    );

    #--------------------------------------------------------------
    $log->step('Initialize the SnapMirror relationship');
    #--------------------------------------------------------------
    my $Sm = NACL::STask::Snapmirror->initialize(
        command_interface        => $Filer_Dst,
        source_ci                => $Filer_Src,
        'destination-path'       => $dVol,
        'source-path'            => $sVol,
#        'is-auto-expand-enabled' => $expand_mode,
        'type'                   => $mirror_type,
        'policy'                 => $sm_policy,
        'throttle'               => $throttle,
        nacltask_wait            => 1
    );

    #TODO: Need to make this a part of initialize call once implicit create
    #support comes in
    $Sm->modify ( 'is-auto-expand-enabled' => $auto_expand );

    if ( $run_update ) {
        #--------------------------------------------------------------
        $log->step( 'Add more data on the source volume: ' . $sVol->volume );
        #--------------------------------------------------------------
        $IOHandle->data_populate(
            size      => $dataset_size,
            path      => 'file' . time,
            arguments => { dbfile => random_name_generator( prefix => "db_" ) }
        );

        #--------------------------------------------------------------
        $log->step('Issue SnapMirror Update');
        #--------------------------------------------------------------
        $Sm->update();
    }

    # Quiesce the relationship if auto-expand is set to false(manual expand)
    if ( $auto_expand =~ /false/ ) {
        #--------------------------------------------------------------
        $log->step(' Issue SnapMirror Quiesce ');
        #--------------------------------------------------------------
        $Sm->quiesce( nacltask_verify => 1 );
    }
    return $Sm;

} ## end sub SM_initialize_update_quiesce

#####This subroutine does the volume expand of the respective volume object being passed with the required number of aggr-list-multiplier count on the given list of aggregates.
##### This subroutine even validates the output based on the regex sent to this subroutine.

sub fg_expand {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_obj => { type => OBJECT, isa => 'NACL::STask::Volume' },
            'aggr-list'            => { type => ARRAYREF, optional => 1 },
            'aggr-list-multiplier' => { type => SCALAR,   default  => 4 },
            expected_result => { type => SCALAR, regex => qr/^(pass|fail)$/ },
            failure_regex => { type => SCALAR, optional => 1 },
        },
    );
    my $log             = log_global(1);
    my $expected_result = $args{expected_result};

    my $vol_obj = $args{vol_obj};
    my $aggr_list
        = ( $args{'aggr-list'} )
        ? $args{'aggr-list'}
        : $vol_obj->get_one_state_attribute('aggr-list');

    my $SnapshotHelper = FG::DP::SnapshotHelper->new();
    try {
        $SnapshotHelper->expand_fg_volume(
            volume               => $vol_obj,
            aggr_list            => $aggr_list,
            aggr_list_multiplier => $args{'aggr-list-multiplier'},
        );
        if ( lc($expected_result) eq 'fail' ) {
            NATE::BaseException->throw(
                      'Un-expected result!!! Expand of volume '
                    . $vol_obj->volume
                    . ' succeeded. Expected it to fail.' );
        } else {
            $log->log(
                'Expand of volume ' . $vol_obj->volume
              . ' succeeded as expected'
            );
        } ## end else [ if ( lc($expected_result...))]
    } ## end try
    catch NATE::BaseException with {
        my $exception = shift;
        my $text      = $exception->text();
        if ( lc($expected_result) eq 'pass' ) {
            $log->error( 'Un-expected result!!! Expand of volume '
                    . $vol_obj->volume
                    . ' failed with '
                    . $text );
            $exception->throw();    # un-expected failure - rethrow
        } else {
            if ( $text !~ /$args{failure_regex}/ ) {
                $log->error( 'Un-expected result!!! Expand of volume '
                        . $vol_obj->volume
                        . ' failed with '
                        . $text );
                $exception->throw();    # un-expected failure - rethrow
            } else {
                $log->log(
                    'Expand of volume ' . $vol_obj->volume
                  . ' failed as expected'
                );
            } ## end else [ if ( $text !~ /$args{failure_regex}/)]
        } ## end else [ if ( lc($expected_result...))]
    };
} ## end sub fg_expand

##This subroutine creates qtrees on a FlexGroup volume. It takes three params:
## 1. FlexGroup Volume Object
## 2. Location of the qtree(s) to be created: root, non-root-local and non-root-remote
##    > root location denotes a qtree being placed on root constituent
##    > non-root-local location denotes a qtree being placed on same node as root
##      constituent but on a different constituent
##    > non-root-remote location denotes a qtree being placed on a node other than
##      root constituent node
## 3. Number of qtree(s) to be created
sub fg_create_qtrees{
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_obj  => { type => OBJECT, isa => 'NACL::STask::Volume' },
            location => { type => SCALAR },
            count    => { type => SCALAR },
        },
    );
    my $log        = log_global(1);
    my $flexgroup  = $args{vol_obj};
    my $location   = $args{location};
    my $num_of_qts = $args{count};
    my $ci         = $flexgroup->command_interface();
    my $vserver    = $flexgroup->vserver();
    my $volume     = $flexgroup->volume();
    $log->step('Identifying target constituent');
    my $root_constituent = $flexgroup->get_flexgroup_constituents(
        filter  => {
            'constituent-role' => 'root',
        },
    );
    my $root_node = $root_constituent->get_one_state_attribute('node');

    my @target_constituent = ($root_constituent);
    if ($location eq 'non-root-local') {
        try {
            @target_constituent = $flexgroup->get_flexgroup_constituents(
                filter => {
                    node               => $root_node,
                   'constituent-role' => '!root',
                },
            );
        } catch NATE::BaseException with {
            my $ex = shift;
            if ( $ex->text =~ /No matching volume/i ) {
                $log->log(
                    "No matching constituent volume found due to ARL or VolMove"
                  . ", skipping qtree creation"
                );
                return;
            }
        };
    } elsif ($location eq 'non-root-remote') {
        try {
            @target_constituent = $flexgroup->get_flexgroup_constituents(
                filter => {
                    node => "!$root_node",
                },
            );
        } catch NATE::BaseException with {
            my $ex = shift;
            if ( $ex->text =~ /No matching volume/i ) {
                $log->log(
                    "No matching constituent volume found due to ARL or VolMove"
                  . ", skipping qtree creation"
                );
                return;
            }
        };
    }
    
    fg_set_placement( node => $ci, action => 'manual');

    my @securities = qw(default  mixed    unix     ntfs);
    my @oplocks    = qw(enable   default  disable  default);
    while($num_of_qts > 0){
        my @security_oplock_arg;
        my $security = $securities[int(rand(4))];
        my $oplock   = $oplocks[int(rand(4))];
        if ( $security ne 'default' ) {
            push @security_oplock_arg, 'security-style' => $security;
        }
        if ( $oplock ne 'default' ) {
            push @security_oplock_arg, 'oplock-mode' => $oplock;
        }
        my $target_fgindex = $target_constituent[int(rand(scalar(@target_constituent)))]
            ->get_one_state_attribute('flexgroup-index');
        my $qtree_name = random_name_generator(
            prefix => $location . '_' . $security . '_' . $oplock,
            size   => 1,
            suffix => "fgindex${target_fgindex}"
        );
        nacl_method_retry(
            code        => sub {
                NACL::STask::Qtree->create(
                    command_interface => $ci,
                    vserver           => $vserver,
                    volume            => $flexgroup->volume(),
                    qtree             => $qtree_name,
                    @security_oplock_arg,
                );
            },
            tries_count => 6,
            sleep_time  => 10,
            exceptions  => 'NATE::BaseException',
        );
        --$num_of_qts;
        $log->log($num_of_qts.' left to be created.');
    }

    fg_set_placement( node => $ci, action => 'auto');
} ## end sub fg_create_qtrees

## This subroutine populates all the qtrees present in a FlexGroup Volume.
## It takes four params:
## 1. FlexGroup volume Object
## 2. WFC::IO object
## 3. Type of qtrees to be populated: root, non-root-local and non-root-remote
## 4. Dataset Size
sub fg_populate_qtrees {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_obj                        => { type => OBJECT,   isa      => 'NACL::STask::Volume' },
            io_handle                      => { type => ARRAYREF, optional => 1 },
            populate_qtree_root            => { type => SCALAR,   optional => 1, default => 1 },
            populate_qtree_non_root_local  => { type => SCALAR,   optional => 1, default => 1 },
            populate_qtree_non_root_remote => { type => SCALAR,   optional => 1, default => 1 },
            dataset_size                   => { type => SCALAR,   default  => '1m' },
        },
    );
    my $log                            = log_global(1);
    my $flexgroup                      = $args{vol_obj};
    my $populate_qtree_root            = $args{populate_qtree_root};
    my $populate_qtree_non_root_local  = $args{populate_qtree_non_root_local};
    my $populate_qtree_non_root_remote = $args{populate_qtree_non_root_remote};
    my $dataset_size                   = $args{dataset_size};
    my $ci                             = $flexgroup->command_interface();
    my $vserver                        = $flexgroup->vserver();
    my $volume                         = $flexgroup->volume();
    my $io_handle                      = @{ $args{io_handle} }[0];

    my @qtrees_to_be_populated;
    if ( $populate_qtree_root ) {
        $log->step('Fetch the root qtrees');
        my @qtree_root = NACL::CS::Qtree->fetch(
            command_interface => $ci,
            filter            => {
                vserver => $vserver,
                volume  => $flexgroup,
                qtree   => 'root*',
            },
        );
        splice @qtrees_to_be_populated, scalar @qtrees_to_be_populated, 0, @qtree_root;
    }

    if ( $populate_qtree_non_root_local ) {
        $log->step('Fetch the non root local qtrees');
        my @qtree_non_root_local = NACL::CS::Qtree->fetch(
            command_interface => $ci,
            filter            => {
                vserver => $vserver,
                volume  => $flexgroup,
                qtree   => 'non-root-local*',
            },
        );
        splice @qtrees_to_be_populated, scalar @qtrees_to_be_populated, 0, @qtree_non_root_local;
    }

    if ( $populate_qtree_non_root_remote ) {
        $log->step('Fetch the non root remote qtrees');
        my @qtree_non_root_remote = NACL::CS::Qtree->fetch(
            command_interface => $ci,
            filter            => {
                vserver => $vserver,
                volume  => $flexgroup,
                qtree   => 'non-root-remote*',
            },
        );
        splice @qtrees_to_be_populated, scalar @qtrees_to_be_populated, 0, @qtree_non_root_remote;
    }

    foreach ( @qtrees_to_be_populated ) {
        if ( $_->security_style !~ /ntfs/i && $io_handle->client()->type =~ /linux/i ) {
            $log->log(
                'Populating qtree ' . $_->qtree() . ' using client '
              . $io_handle->client()->name
            );
            $io_handle->data_populate(
                size => $dataset_size,
                path => $_->qtree() . "/file_" . time(),
            );
        } elsif ( $_->security_style !~ /unix/i && $io_handle->client()->type =~ /win/i ) {
            $log->log(
                 'Populating qtree ' . $_->qtree() . ' using client '
               . $io_handle->client()->name
            );
            $io_handle->data_populate(
                size => $dataset_size,
                path => $_->qtree() . "/file_" . time(),
            );
        } else {
            $log->log(
                'Skipping population of qtree: ' . $_->qtree() . ' as the '
              . 'security style: ' . $_->security_style . ' and client type: '
              . $io_handle->client()->type . ' of client: '
              . $io_handle->client()->name . ' are not compatible!'
            );
        }
    }
} ## end sub fg_populate_qtrees

=head2 volume_is_snapmirror_source

DPO: a new field in volume show to indicate if the src volume is mirrored

=over

=item Options

(Required) node - filer object

(Required) volume - src volume to check the field

=DEPENDENCIES

ZAPI only

=back

=cut

# volume_is_snapmirror_source
sub volume_is_snapmirror_source {
    my %args = validate_with(
        params => \@_,
        spec => {
            node   => { type => OBJECT, optional => 0 },
	    volume => { type => SCALAR, optional => 0 },
        },
    );

    my $node = $args{'node'};
    my $volume = $args{'volume'};

    my $zapi_api = NACL::APISet->new(
       name      => $node->node(),
       category  => 'Node',
       interface => 'ZAPI',
       set       => 'CMode',
    );

    my $raw = $zapi_api->volume_get_iter(
	'desired-attributes' => {
	    'volume-mirror-attributes' => {
		'is-snapmirror-source' => '',
	    },
	},
	'max-records' => 1000,
 	query => {
	    'volume-id-attributes' => {
		'name' => $volume,
		'style-extended' => '*',
	    },
	},
    )->get_raw_output();

    my $protected = 0;
    if($raw =~ /<is-snapmirror-source>true<\/is-snapmirror-source>/i) {
	$protected = 1 ;
    }

    return $protected;

} # volume_is_snapmirror_source


=head2 Wf_protect_api

    protect API workflow including snapmirror lifecycle test

=head3 STEPS

    1. Issue snapmirror protect command(s)
    2. Verify protects complete successfully
    3. Verify SE/dedup enabled on all dest volumes
    4. Verify snapmirror state, policy, type & dest vol names
       - verify init completes successfully if auto-init is on
    5. Start snapmirror lifecycle test based on policy
       - DR lifecycle|Vault lifecycle
    6. Run 'snapmirror check' on all relations
    7. Clean up relationships and dest volumes

=head3 OPTIONS

    nfsclients: ARRAYREF. (Required) NFS clients
    src_vols:   ARRAYREF. (Required) src volumes to protect.
    dst_vs:     SCALAR.   (Required) dst vserver peered to src vs.
    src_node:   SCALAR.   (Optional) src node.
    dst_node:   SCALAR.   (Required) dst node.
    max_vols:   SCALAR.   (Optional) max vols in one protect command (2.5MB chars).
    concurrs:   SCALAR.   (Optional) how many concurrent protect commands.
    api_must:   SCALAR.   (Optional) zapi or cli. default to zapi.
    timeout:    SCALAR.   (Optional) protect command timeout. default to 900s.
    cleanup:    SCALAR.   (Optional) delete sms and volumes at exit. default to 1.
    vol_pat:    SCALAR.   (Optional) vol pattern for existing src volumes.
    policy:     SCALAR.   (Optional) snapmirror policy.
    schedule:   SCALAR.   (Optional) snapmirror schedule.
    auto_init:  SCALAR.   (Optional) auto-initialization 0(false)|1(true)|undef(alternate).
    dst_prefix: SCALAR.   (Optional) dst volume name prefix.
    dst_suffix: SCALAR.   (Optional) dst volume name surfix.
    negative:   SCALAR.   (Optional) negative test flag.
    src_vols_ext: ARRAYREF. (Optional) src volumes to protect.
    concur_lcs: SCALAR.   (Optional) number of concurrent lifecycle tests.

=head3 CONFIG

    ntest CONFIG_FILE=dp/snapmirror/dpo/Config/dpo/FUNC_DPO_Protect_Basic_Workflow.yml \
    lib/NACL/AppBuilder/reIgniter.thpl

=cut

sub Wf_protect_api {
    my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients => { type => ARRAYREF|SCALAR, optional => 0 },
            src_vols   => { type => ARRAYREF|UNDEF, optional => 1 },
            dst_vs     => { type => SCALAR, optional => 0 },
	    src_node   => { type => SCALAR|UNDEF, optional => 1 },
	    dst_node   => { type => SCALAR, optional => 0 },
	    max_vols   => { type => SCALAR, default => '1500' },
	    concurrs   => { type => SCALAR, default => '1' },
	    api_must   => { type => SCALAR, default => 'zapi'},
	    timeout    => { type => SCALAR, default => '900'},
	    cleanup    => { type => SCALAR, default => '1' },
	    vol_pat    => { type => SCALAR, default => '*' },
	    policy     => { type => SCALAR|UNDEF, optional => 1 },
	    schedule   => { type => SCALAR|UNDEF, optional => 1 },
	    auto_init  => { type => SCALAR|UNDEF, optional => 1 },
	    dst_prefix => { type => SCALAR|UNDEF, optional => 1 },
	    dst_suffix => { type => SCALAR|UNDEF, optional => 1 },
	    del_delay  => { type => SCALAR, default => '900' },
	    negative   => { type => SCALAR, default => 'false' },
	    concur_lcs => { type => SCALAR, default => '6' },
            src_vols_ext => { type => ARRAYREF|UNDEF, optional => 1 },
	    exp_dst_num => {type => SCALAR|UNDEF, optional => 1},
	    skip_lifecycle => {type =>SCALAR|UNDEF, optional => 1},
        },
    );

    my $src_vols = $args{src_vols};
    my $dst_vs   = $args{dst_vs};
    my $src_node = $args{src_node};
    my $dst_node = $args{dst_node};
    my $max_vols = $args{max_vols};
    my $timeout  = $args{timeout};
    my $vol_pat  = $args{vol_pat};
    my $negative = $args{negative};

    my $policy = $args{policy};
    my $schedule = $args{schedule};
    my $del_delay = $args{del_delay};
    my $concur_lcs = $args{concur_lcs};
    my $src_vols_ext = $args{src_vols_ext};

    my $auto_init = $args{auto_init};
    my $dst_prefix = $args{dst_prefix};
    my $dst_suffix = $args{dst_suffix};

    my @policy_array = qw(MirrorAndVault MirrorAllSnapshots Unified7year
		    XDPDefault MirrorLatest DPDefault);
    @policy_array = ($policy) if $policy;

    my $nfsclients = $args{nfsclients};
    my @nfsclients = split ',' , $nfsclients;

    my $exp_dst_num = $args{exp_dst_num};

    my $node_src = NACL::C::Node->new(name => $src_node) if $src_node;
    my $node_dst = NACL::C::Node->new(name => $dst_node);

    my (@sms, @subtests, @vols_list);
    my $file_size = '1m';

    # create 10 min schedule on destination cluster with unique identifier
    my $dpo_schedule = '10min_dpo';
    if(!$schedule){
	$log->step('Create 10 minute schedule on destination cluster');
	try {
	    my $sched_dpo = NACL::C::JobScheduleCron->create(
		command_interface => $node_dst,
		'name'            => $dpo_schedule,
		'minute'          => [6, 16, 26, 36, 46, 56],
	    );
	} catch NACL::APISet::Exceptions::CommandFailedException with {
	    my $ex = shift;
	    my $text = $ex->text();
	    if ( $text !~ /duplicate entry/i ) {
		$ex->throw("ERROR: JobScheduleCron failed unexpectedly with $text");
	    } else {
		$log->log("duplicate schedule $dpo_schedule");
	    }
	};
	$schedule = $dpo_schedule;
    }

    # to use existing src side volumes
    if (!$src_vols && $node_src) {
	$Log->log('Use existing src volumes');

	@{$src_vols} = NACL::STask::Volume->find(
	    command_interface => $node_src,
	    filter => {volume => $vol_pat . '*',},
	);
    }

    # merge volume arrays from two different vs
    if($src_vols_ext){
        splice @{$src_vols}, @{$src_vols_ext}, 0, pop @{$src_vols_ext} while @{$src_vols_ext};
    }

    my %srcvols;
    my $auto_init_flag = 0;
    my $idx = 0;
    my $concurrs = $args{concurrs};
    my $concur_counter = 0;
    foreach my $src_vol (@{$src_vols}) {
	my $pvname = $src_vol->volume;
	my $pvserver = $src_vol->vserver();
	my $ppath = "$pvserver:$pvname";
    }
    foreach my $src_vol (@{$src_vols}) {
	$idx ++;
	my $vname = $src_vol->volume;
	my $vserver = $src_vol->vserver();
	my $path = "$vserver:$vname";

	if(!($idx % 2)) { # test two different command formats
	    my $cid =
		NACL::CS::ClusterIdentity->fetch(
			command_interface => $src_vol->command_interface );
	    $path = $cid->name() . "://$vserver/$vname";
	}

	$srcvols{$path}{obj} = $src_vol;
	push @vols_list, $path;

	# reach #max vols per protect command or the end of the vol list
	if (!($idx % $max_vols) || ($idx eq scalar(@{$src_vols}))) {
	    $auto_init_flag = (defined($auto_init)) ? $auto_init : !$auto_init_flag;
	    my @s_vols_local = @vols_list;

	    my $sm_policy = shift @policy_array;
	    push @policy_array, $sm_policy;
	    my %policy = ('policy' => $sm_policy,);
	    my %dst_prefix = ($dst_prefix) ? ('destination-volume-prefix' => $dst_prefix,) : ();
	    my %dst_suffix = ($dst_suffix) ? ('destination-volume-suffix' => $dst_suffix,) : ();

	    my %opts = ('destination-vserver' => $dst_vs,
			'path-list'           => \@s_vols_local,
			'auto-initialize'     => ($auto_init_flag) ? 'true' : 'false',
			'schedule'            => $schedule,
			%policy, %dst_prefix, %dst_suffix);
	    push @subtests, NATE::Process->new(
		codespec => sub{
		    # refresh command interface
		    $node_dst->refresh_command_interface();

		    my ($job_component, %sms);

		    # use ZAPI or CLI for the protect command
		    $Log->step('Issue snapmirror protect command');
		    if ($args{api_must} eq 'zapi') {
			$Log->log('Use ZAPI for protect command');
			# use API set rather than NACL component. BURT 1058744
			my $api_zapi = NACL::APISet->new(
			    name      => $dst_node,
			    category  => 'Node',
			    interface => 'ZAPI',
			    set       => 'CMode',
			);
			try {
			    $api_zapi->set_timeout('zapi-timeout' => $timeout);
			    $api_zapi->snapmirror_protect(%opts);
			} catch NATE::BaseException with {
			    # protect job has a performance issue (BURT 1055982).
			    # Even timeout is set higher, ZAPI will timeout after
			    # 5 mins (BURT 989227).
			    my $ex = shift;
			    if($ex->text =~ /API did not finish on time/i) {
				$Log->log('Skip ZAPI warning. BURT 1005822');
			    } elsif($ex->text =~ /502 Bad Gateway Bytes received/i) {
				$Log->log('Skip ZAPI warning. BURT 989227. Wait for protect jobs done');
				my $w_time = time + $timeout;
				my $err = 1;
				while (time < $w_time) {
				    my @jobs = NACL::C::Job->find(
					'command_interface' => $node_dst,
					'filter'            => { name => '"SnapMirror protect"' },
					'allow_empty'       => 1,
				    );
				    if(!scalar(@jobs)) {
					$err = 0;
					last;
				    }
				    Tharn::snooze(300);
				}
				NATE::BaseException->throw('Protect jobs not finished after ' . $timeout) if $err;
			    } else {
				$ex->throw();
			    }
			};
		    } else {
			$Log->log('Use CLI for protect command');
			try {
			    my $smp_obj = NACL::STask::Snapmirror->protect(
				'command_interface'   => $node_dst,
				'job_component'       => \$job_component,
				'method-timeout'      => $timeout,
				%opts
			    );
			} catch NATE::BaseException with {
			    my $ex = shift;
			    if($negative eq 'true') {
				$Log->log('Failure expected : check logs to confirm');
				goto CLEANUP;
			    } else {
				$ex->throw();
			    }
			};
		    }

		    # wait for CLI job.
		    if ($job_component) {

		        $job_component->wait_on_attribute(
		           'method-timeout'   => $timeout,
		           attribute_to_check => 'state',
		           till_value         => qr/Success|Failure/,
		           );

			my $job_string = $job_component->get_one_state_attribute('completion');
			if($job_string =~ /has failed for 0 volumes/i) {
			    if($negative eq 'true') {
				NATE::BaseException->throw('Unexpected protect completion string');
			    }
			} else {
			    if($negative eq 'true') {
				$Log->log('Failure expected : check logs to confirm');
				goto CLEANUP;
			    } else {
				NATE::BaseException->throw('Unexpected protect completion string');
			    }
			}
		    }

		    # get the list of SMs created by the protect command
		    $Log->step('Fetch all new snapmirrors');
		    foreach my $vol_path (@s_vols_local) {
			my $srcvolname = $srcvols{$vol_path}{obj}->volume();
			my $pfix = ($dst_prefix) ? $dst_prefix : '';
			my $expnum = ($exp_dst_num) ? $exp_dst_num : '';
			my $sfix = ($dst_suffix) ? $dst_suffix : '_dst';
			my $dstvolname = $pfix . '*' . $expnum . $sfix;
			nacl_method_retry( #ZAPI returns rightaway, retry is needed
			    code => sub {
				my @sm = NACL::STask::Snapmirror->find(
					command_interface => $node_dst,
					filter            => {
					    'source-volume' 	 => $srcvolname,
					    'destination-volume' => $dstvolname,
					},
					allow_empty       => 0,
				);
				#get the last relation
				$sm[-1]->{'source_ci'} = $srcvols{$vol_path}{obj}->command_interface;
				$sms{$sm[-1]->{destination_path}}{sm_obj} = $sm[-1];
				$sms{$sm[-1]->{destination_path}}{vol_obj} = $srcvols{$vol_path}{obj};
			    },
			    tries_count => 60,
			    sleep_time => 30,
			    exceptions => 'NATE::BaseException',
			);
		    }

		    my %vols;
		    my $cnt = 0;
		    # verify SE setting and initializations on dst side
		    for my $key (sort keys %sms) {
			my $sm = $sms{$key}{sm_obj};
			my $source_path = $sm->{source_path};
			my ($vserver_src, $vol_src_name) = ($source_path =~ /^(.*):(.*)$/);

			my $destination_path = $sm->{destination_path};
			my ($vserver_dst, $vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);

			$Log->step('Verify dst name prefix|suffix ' . $vol_dst_name);
			if($dst_prefix && $vol_dst_name !~ /$dst_prefix/) {
			    $Log->error($vol_dst_name . ' prefix not present');
			}
			if($dst_suffix && $vol_dst_name !~ /$dst_suffix/) {
			    $Log->error($vol_dst_name . ' suffix not present');
			}

			$Log->step('Verify dedupe on dst volume ' . $vol_dst_name);
			my $state_sis;
			nacl_method_retry(
			    code => sub {
				$state_sis = NACL::CS::VolumeEfficiency->fetch(
				    command_interface => $node_dst,
				    filter            => {
					volume  => $vol_dst_name,
					vserver => $vserver_dst,
				    },
				    allow_empty => 0,
				);
			    },
			    exceptions  => 'NATE::BaseException',
			    tries_count => 3,
			    sleep_time  => 30,
			);
			$Log->error('Dedupe is not enabled on volume ' . $vol_dst_name)
			    if ($state_sis->state !~ /Enabled/i);

			$Log->step('Verify snapmirror status for ' . $destination_path);
			my $state_to_check = ($opts{'auto-initialize'} eq 'true') ?
						'Snapmirrored' : 'Uninitialized';
			$sm->wait_on_attributes(
			    'method-timeout' => $timeout,
			    attributes       => [
				{
				    attribute_to_check => 'state',
				    till_value         => $state_to_check
				},
				{
				    attribute_to_check => 'policy',
				    till_value         => $sm_policy
				},
				{
				    attribute_to_check => 'schedule',
				    till_value         => $schedule
				},
			    ]
			);
		    }

		    # lifecycle test
		    my $skip_lifecycle = 0;

		    if(defined $args{skip_lifecycle}) {
		        $skip_lifecycle = $args{skip_lifecycle};
		    }
		    if(!$skip_lifecycle && $opts{'auto-initialize'} ne 'true') {
			my $i = 0;
			my ($sm_init_map, $sm_update_map, @sm_ops);
			for my $key (sort keys %sms){
			    my $vol_s = $sms{$key}{vol_obj};
			    my $ci = $sms{$key}{vol_obj}->command_interface;
			    $ci->refresh_command_interface();

			    # create 3 snapshots
			    my @ss_array;
			    for my $n (0..2) {
				try {
				    $Log->step('Create more data on the src volume');
				    my $file_name = 'dpo' . time;
				    NACL::STask::VolumeFile->create(
					command_interface => $ci,
					path              => '/vol/' . $vol_s->volume() . '/' . $file_name,
					vserver           => $vol_s->vserver(),
					nacltask_wait     => 1,
					size              => $file_size,
				    );
				} catch NATE::BaseException with {
				    my $ex = shift;
				    if($ex->text =~ /concurrent mkfile threads exceeded/i) {
					$Log->warn('Failed to create file on volume ' . $vol_s->volume());
				    } else {
					$ex->throw();
				    }
				};
				my $ss = 'ss_' . $$ . $n;
				create_snapshot($vol_s, $ss);
				push @ss_array, $ss;
			    }

			    $Log->step('Remove schedule for lifecycle test');
			    $sms{$key}{sm_obj}->modify(schedule => '""');

			    $Log->log('Create the mapping hash for lifecycle test');
			    my $key_sm = 'sm_obj_' . $i;
			    $sm_init_map->{$key_sm} = shift @ss_array;
			    $sm_update_map->{$key_sm} = \@ss_array;

			    push @sm_ops, $sms{$key}{sm_obj};
			    $i++;
			}

			$Log->comment('sm init mapping:  ' . Dumper($sm_init_map) .  "\n" .
				      'sm update mapping: ' . Dumper($sm_update_map) . "\n");

			my $con_sms = $concur_lcs; # limit concurrent threads
			while (scalar(@sm_ops)) {
			    my @sms_lc = splice @sm_ops, 0, $con_sms;
			    if($sm_policy =~ /XDPDefault/i ) {
				$Log->step('Run vault workflows');
				Wf_vault_lifecycle(sm_objs               => \@sms_lc,
						   nfsclients            => $nfsclients,
						   ss_for_initialize_map => [$sm_init_map],
						   ss_for_update_map     => [$sm_update_map],
						   validate              => 'true',
						   rst_flag		 => 'true');
			    } else {
				$Log->step('Run DR workflows');
				Wf_DR_lifecycle   (sm_objs               => \@sms_lc,
						   nfsclients            => $nfsclients,
						   ss_for_initialize_map => [$sm_init_map],
						   ss_for_update_map     => [$sm_update_map],
						   validate              => 'true',);
			    }
			}

			for my $key (sort keys %sms){
			    my $vol_s = $sms{$key}{vol_obj};
			    my $sm_obj = $sms{$key}{sm_obj};

			    $Log->step('Check volume is-snapmirror-source ' . $vol_s->volume());
			    my $ci = $vol_s->command_interface;
			    $ci->refresh_command_interface();

			    if(volume_is_snapmirror_source (node => $ci, volume => $vol_s->volume())) {
				$Log->log($vol_s->volume() . ' is-snapmirror-source is ture.');
			    } else {
				$Log->error($vol_s->volume() . ' is-snapmirror-source is NOT ture.');
			    }

			    $Log->step('Run checker for ' . $sm_obj->{destination_path});
			    try {
				$sm_obj->quiesce(nacltask_wait => 1);
			    } catch NATE::BaseException with {;};
			    $sm_obj->check(nacltask_wait => 1);
			    try {
				$sm_obj->resume(nacltask_wait => 0);
			    } catch NATE::BaseException with {;};
			}
		    }
CLEANUP:
		    if($args{cleanup}){
			$Log->step('Sleep ' . $del_delay . 's');
			Tharn::snooze($del_delay);
			$Log->step('Clean up snapmirrors and volumes');
			my @subvols;
			for my $key (sort keys %sms) {
			    my $ci = $sms{$key}{vol_obj}->command_interface;
			    my $sm = $sms{$key}{sm_obj};
			    push @subvols, NATE::Process->new(
				codespec => sub{
				    $ci->refresh_command_interface();
				    $node_dst->refresh_command_interface();

				    my $source_path = $sm->{source_path};
				    my ($vserver_src, $vol_src_name) = ($source_path =~ /^(.*):(.*)$/);
				    my $destination_path = $sm->{destination_path};
				    my ($vserver_dst, $vol_dst_name) = ($destination_path =~ /^(.*):(.*)$/);

				    $Log->log('Release SM ' . $source_path);
				    my $rel = 0;
				    nacl_method_retry(
					code => sub {
					    $sm->release(nacltask_wait => 1);
					    $rel = 1;
					},
					exceptions  => 'NATE::BaseException',
					tries_count => 5,
					sleep_time  => 60,
				    );

				    $Log->log('Delete SM ' . $destination_path);
				    nacl_method_retry(
					code => sub {
					    $sm->delete(nacltask_wait => 1);
					},
					exceptions  => 'NATE::BaseException',
					tries_count => 10,
					sleep_time  => 60,
				    );

				    if ($rel) {
					$Log->log('Verify volume is-snapmirror-source ' . $vol_src_name);
					if(volume_is_snapmirror_source (node => $ci, volume => $vol_src_name)) {
					    $Log->error($vol_src_name . ' is-snapmirror-source is still ture.');
					} else {
					    $Log->log($vol_src_name . ' is-snapmirror-source has been cleared.');
					}
				    }

				    $Log->log('Delete volume ' . $vol_dst_name);
				    try {
					NACL::STask::Volume->purge(
					    command_interface => $node_dst,
					    vserver 	      => $vserver_dst,
					    volume 	      => $vol_dst_name,
					);
				    } catch NATE::BaseException with {
					$Log->warn('Failed to delete dst volume ' . $vol_dst_name);
				    };
				},
				runid      => 'cleanup_' . $sms{$key}{vol_obj}->volume,
				background => 1,
				onexit     => \&NATE::Process::on_exit_propagate_worst_result,
			    );
			}
			# make it faster with concurrency
			my $vpm = NATE::ProcManager->new(
			    runid     => 'post_test_cleanup_' . $$,
			    proc_info => \@subvols,
			    inherit_params => 1,
			    max_active_children => 8,
			);
			$vpm->start();
			$vpm->wait();
			$vpm->destroy();
		    }
		},
		runid      => sprintf("%03d_protect_workflow_$$", ++ $concur_counter),
		background => 1,
		onexit     => \&NATE::Process::on_exit_propagate_worst_result,
	    );
	    @vols_list = ();
	}
    }

    # test concurrency of protect jobs
    my $pm = NATE::ProcManager->new(
	runid     => 'Wf_protect_api',
	proc_info => \@subtests,
	inherit_params => 1,
	max_active_children => $concurrs,
    );
    $pm->start();
    $pm->wait();
    $pm->destroy();

    return;
}

=head2 _run_wafliron

    checking _run_wafliron corruption

=item Options

 (Required) aggr - : objetcs

=cut

sub _run_wafliron {
    my %args = validate_with(
        params => \@_,
        spec   => {
            aggr => { type => OBJECT, },
            node => { type => OBJECT, },
            ioc => { type => SCALAR, default=> 'true', },
            commit_option => { type => SCALAR, default=> 'true', },
        },
    );

    my $Node = $args{node};
    my $Aggr = $args{aggr};
    my $ioc  = $args{ioc};
    my $commit_option = $args{commit_option};

    my $log = log_global(1);

  #---------------------------------------------------------------------------
    $log->step('Run wafliron');

  #---------------------------------------------------------------------------
    my $wafliron = NACL::MTask::Wafliron->new(
        command_interface => $Node,
        aggregate         => $Aggr->aggregate,
    );

    $wafliron->start(
        nacltask_include_mirrors => 1,
        nacltask_optional_commit => $ioc,
    );

  #---------------------------------------------------------------------------
    $log->step('Wait for completion');

  #---------------------------------------------------------------------------
    $wafliron->wait_for_completion();

  #---------------------------------------------------------------------------
    $log->step('checking for wafliron corruption');

  #---------------------------------------------------------------------------

    if($wafliron->corruption_found() ){
        $log->warn("iron found corruption, skipping commit, run wafliron review to check corruptions") if ($commit_option ne 'true');
        $commit_option eq 'true' ? $wafliron->commit() : $wafliron->reject();
        $Aggr->online();
    } else {
        $Aggr->online();
    }

} ## end sub _run_wafliron

sub start_monitoring_snapmirror_logs {
    my %args = validate_with(
        params => \@_,
        spec   => {
            node 	 => {type => OBJECT},
	    rst_id 	 => {type => SCALAR},
	    log_reader 	 => {type => HASHREF|UNDEF},
	    set_log_file => {type => ARRAYREF, default => ['snapmirror_audit']},
	    set_log_dir  => {type => SCALAR, default => '/mroot/etc/log'},
        },
    );

    my $log_reader = $args{log_reader};
    my $set_log_file = $args{set_log_file};
    my $set_log_dir = $args{set_log_dir};

    my %check;

    if ( $args{rst_id} ) {
        %check = ( check_for_any_presence => [qr/$args{rst_id}/] );
    }

    $log_reader = NACL::MTask::SystemLogDetector->new(
        command_interface => $args{node},
        set_log_file      => $set_log_file,
        set_log_dir       => $set_log_dir,
        %check,
    );

    $log_reader->start();
    return $log_reader;

} ## end sub start_monitoring_snapmirror_logs


=head2 Wf_Auto_L2L_Fanout

    Wf_Auto_L2L_Fanout

=head3 STEPS

Testcase 1: Config <src comp; dst B comp; dst C comp> 1|0 0|1 0|1
Steps:
1. Create Volumes, source A and destinations B and C. Turn ON|OFF compression on source A.
2. Create XDP relationships, A -> B and A -> C. Initialize them.
3. Create files f1 and f2 on source. Create snapshot ss1. Issue update on both relationships.
4. Modify file f2, create snapshot ss2 and Issue updates.
5. Modify file f2, create snapshot ss3 and Issue updates.
6. Issue Delete, Release and Break on both relationships.
7. Create snapshot ss4 on A, ss5 on B and ss6 on C by modifying file f2 on each.
8. Issue resync with type, preserve and source snapshot from C -> B.

=head3 OPTIONS

    nfsclients:  HASHREF. (Required) NFS clients
    vol_a_compr: HASHREF. (Required) source volume(s) hash table including objs, and compression setting.
    vol_b_compr: HASHREF. (Required) destination volume(s) hash table including objs, and compression setting.
    vol_c_compr: HASHREF. (Required) destination volume(s) hash table including objs, and compression setting.

=head3 CONFIG

    ntest CONFIG_FILE=dp/snapmirror/dpo/Config/dpo/FUNC_Auto_L2L_Fanout.yml \
    lib/NACL/AppBuilder/reIgniter.thpl

=cut

sub Wf_Auto_L2L_Fanout {
    my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients  => {type => ARRAYREF|SCALAR, optional => 0},
	    vol_a_compr => {type => HASHREF, optional => 0},
	    vol_b_compr => {type => HASHREF, optional => 0},
	    vol_c_compr => {type => HASHREF, optional => 0},
	    cleanup     => {type => SCALAR, default => 'false'},
	    validate 	=> {type => SCALAR, default => 'false', regex => qr/^(true|false)$/},
	    timeout     => {type => SCALAR, default => '1800'},
        },
    );

    my $vol_a_compr = $args{vol_a_compr};
    my $vol_b_compr = $args{vol_b_compr};
    my $vol_c_compr = $args{vol_c_compr};

    my @subtests;
    while (keys $vol_a_compr->{vol_obj}{objects} &&
	   keys $vol_b_compr->{vol_obj}{objects} &&
	   keys $vol_c_compr->{vol_obj}{objects}){
	my (%vol_a_hash, %vol_b_hash, %vol_c_hash);
	$vol_a_hash{start_compr} = $vol_a_compr->{start_compr};
	$vol_b_hash{start_compr} = $vol_b_compr->{start_compr};
	$vol_c_hash{start_compr} = $vol_c_compr->{start_compr};
	$vol_a_hash{end_compr} = $vol_a_compr->{end_compr};
	$vol_b_hash{end_compr} = $vol_b_compr->{end_compr};
	$vol_c_hash{end_compr} = $vol_c_compr->{end_compr};

	my $temp_c = (keys $vol_c_compr->{vol_obj}{objects})[0];
	$vol_c_hash{vol_obj} = $vol_c_compr->{vol_obj}{objects}{$temp_c};
	delete $vol_c_compr->{vol_obj}{objects}{$temp_c};

	# gather volume relations starting from C
	my $vol_obj_c = $vol_c_hash{vol_obj};
	my $sm_obj_c = NACL::STask::Snapmirror->find(
	    'command_interface' => $vol_obj_c->command_interface,
	    'filter'            => {
		'destination-vserver' => $vol_obj_c->vserver,
		'destination-volume'  => $vol_obj_c->volume,
	    },
	);
	my $src_path = $sm_obj_c->{source_path};
	my ($vserver_src_c, $vol_src_c) = ($src_path =~ /^(.*):(.*)$/);

	# look for B and A
	for my $key_a (keys $vol_a_compr->{vol_obj}{objects}) {
	    my $vol_obj_a = $vol_a_compr->{vol_obj}{objects}{$key_a};
	    if ($vol_obj_a->volume() eq $vol_src_c) {
		$vol_a_hash{vol_obj} = $vol_a_compr->{vol_obj}{objects}{$key_a};
		delete $vol_a_compr->{vol_obj}{objects}{$key_a};

		my $temp_b = (keys $vol_b_compr->{vol_obj}{objects})[0];
		my $temp_b_obj = $vol_b_compr->{vol_obj}{objects}{$temp_b};
		my $vol_c = $vol_obj_c->volume;
		my $sm_obj = NACL::STask::Snapmirror->find(
		    'command_interface' => $temp_b_obj->command_interface,
		    'filter'            => {
			'destination-volume' => "!$vol_c",
			'source-volume'      => $vol_obj_a->volume,
		    },
		);
		my $dst_path = $sm_obj->{destination_path};
		my ($vserver_dst_a, $vol_dst_a) = ($dst_path =~ /^(.*):(.*)$/);
		for my $key_b (keys $vol_b_compr->{vol_obj}{objects}) {
		    my $vol_obj_b = $vol_b_compr->{vol_obj}{objects}{$key_b};
		    if ($vol_obj_b->volume() eq $vol_dst_a) {
			$vol_b_hash{vol_obj} = $vol_b_compr->{vol_obj}{objects}{$key_b};
			delete $vol_b_compr->{vol_obj}{objects}{$key_b};
		    }
		}
	    }
	}

        my %args_func = (
                'vol_a_compr'   => \%vol_a_hash,
                'vol_b_compr'   => \%vol_b_hash,
                'vol_c_compr'   => \%vol_c_hash,
                'nfsclients'    => $args{nfsclients},
		'cleanup'       => $args{cleanup},
		'timeout'       => $args{timeout},
		'validate'      => $args{validate},
                'wf_option'     => 'fanout',
                'stagger_delay' => 30,
	);
        push @subtests , create_parallel_processes('al2l_cascade_fanout',\%args_func);
    }

    run_processes_bg('al2l_cascade_fanout',\@subtests);
}


=head2 Wf_Auto_L2L_Cascade_Resync

    Wf_Auto_L2L_Cascade_Resync

=head3 STEPS

Testcase(s) : Config : 1(0) 0(1) 0(1)
Steps:
1. Create volumes (A, B, C) and enable compression on|off the source.
2. Create and initialize snapmirror relationships (A->B-> C).
3. Create files f1 and f2, create snapshot ss1, issue update.
4. Switch ON|OFF the compression on source. Modify file f2, create snapshot ss2 and issue update.
5. Modify file f2, create snapshot ss3 and issue update.
6. Modify file f2, create snapshot ss4 and issue update.
7. Delete relationship B-> C.
8. Release relationship B -> C.
9. Break relationship B -> C.
10. Issue volume snapshot restore on C for ss3.
11. Issue Resync A -> C with snapshot ss4 and type XDP.

=head3 OPTIONS

    nfsclients:  HASHREF. (Required) NFS clients
    vol_a_compr: HASHREF. (Required) source volume(s) hash table including objs, and compression setting.
    vol_b_compr: HASHREF. (Required) destination volume(s) hash table including objs, and compression setting.
    vol_c_compr: HASHREF. (Required) destination volume(s) hash table including objs, and compression setting.

=head3 CONFIG

    ntest CONFIG_FILE=dp/snapmirror/dpo/Config/dpo/FUNC_Auto_L2L_Cascade_Resync.yml \
    lib/NACL/AppBuilder/reIgniter.thpl

=cut


sub Wf_Auto_L2L_Cascade_Resync {
    my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients  => {type => ARRAYREF|SCALAR, optional => 0},
	    vol_a_compr => {type => HASHREF, optional => 0},
	    vol_b_compr => {type => HASHREF, optional => 0},
	    vol_c_compr => {type => HASHREF, optional => 0},
	    cleanup     => {type => SCALAR, default => 'false'},
	    validate 	=> {type => SCALAR, default => 'false', regex => qr/^(true|false)$/},
	    timeout     => {type => SCALAR, default => '1800'},
        },
    );

    my $vol_a_compr = $args{vol_a_compr};
    my $vol_b_compr = $args{vol_b_compr};
    my $vol_c_compr = $args{vol_c_compr};

    my @subtests;
    while (keys $vol_a_compr->{vol_obj}{objects} &&
	   keys $vol_b_compr->{vol_obj}{objects} &&
	   keys $vol_c_compr->{vol_obj}{objects}){
	my (%vol_a_hash, %vol_b_hash, %vol_c_hash);
	$vol_a_hash{start_compr} = $vol_a_compr->{start_compr};
	$vol_b_hash{start_compr} = $vol_b_compr->{start_compr};
	$vol_c_hash{start_compr} = $vol_c_compr->{start_compr};

	$vol_a_hash{end_compr} = $vol_a_compr->{end_compr};
	$vol_b_hash{end_compr} = $vol_b_compr->{end_compr};
	$vol_c_hash{end_compr} = $vol_c_compr->{end_compr};

	my $temp_c = (keys $vol_c_compr->{vol_obj}{objects})[0];
	$vol_c_hash{vol_obj} = $vol_c_compr->{vol_obj}{objects}{$temp_c};
	delete $vol_c_compr->{vol_obj}{objects}{$temp_c};

	# gather vol relations starting from C
	my $vol_obj_c = $vol_c_hash{vol_obj};
	my $sm_obj_c = NACL::STask::Snapmirror->find(
	    'command_interface' => $vol_obj_c->command_interface,
	    'filter'            => {
		'destination-vserver' => $vol_obj_c->vserver,
		'destination-volume'  => $vol_obj_c->volume,
	    },
	);
	my $src_path = $sm_obj_c->{source_path};
	my ($vserver_src_c, $vol_src_c) = ($src_path =~ /^(.*):(.*)$/);

	# look for A and B
	for my $key_b (keys $vol_b_compr->{vol_obj}{objects}) {
	    my $vol_obj_b = $vol_b_compr->{vol_obj}{objects}{$key_b};
	    if ($vol_obj_b->volume() eq $vol_src_c) {
		$vol_b_hash{vol_obj} = $vol_b_compr->{vol_obj}{objects}{$key_b};
		delete $vol_b_compr->{vol_obj}{objects}{$key_b};
		my $sm_obj = NACL::STask::Snapmirror->find(
		    'command_interface' => $vol_obj_b->command_interface,
		    'filter'            => {
			'destination-vserver' => $vol_obj_b->vserver,
			'destination-volume'  => $vol_obj_b->volume,
		    },
		);
		my $src_path = $sm_obj->{source_path};
		my ($vserver_src_b, $vol_src_b) = ($src_path =~ /^(.*):(.*)$/);
		for my $key_a (keys $vol_a_compr->{vol_obj}{objects}) {
		    my $vol_obj_a = $vol_a_compr->{vol_obj}{objects}{$key_a};
		    if ($vol_obj_a->volume() eq $vol_src_b) {
			$vol_a_hash{vol_obj} = $vol_a_compr->{vol_obj}{objects}{$key_a};
			delete $vol_a_compr->{vol_obj}{objects}{$key_a};
		    }
		}
	    }
	}

        my %args_func = (
                'vol_a_compr'   => \%vol_a_hash,
                'vol_b_compr'   => \%vol_b_hash,
                'vol_c_compr'   => \%vol_c_hash,
                'nfsclients'    => $args{nfsclients},
		'cleanup'       => $args{cleanup},
		'timeout'       => $args{timeout},
		'validate'      => $args{validate},
                'wf_option'     => 'cascade',
                'stagger_delay' => 30,
	);
        push @subtests , create_parallel_processes('al2l_cascade_fanout',\%args_func);
    }

    run_processes_bg('al2l_cascade_fanout',\@subtests);
}


#### helper for fanout and cascade al2l workflows ####
sub al2l_cascade_fanout {
    my %args = validate_with(
        params => \@_,
        spec => {
            nfsclients  => {type => ARRAYREF|SCALAR, optional => 0},
	    vol_a_compr => {type => HASHREF, optional => 0},
	    vol_b_compr => {type => HASHREF, optional => 0},
	    vol_c_compr => {type => HASHREF, optional => 0},
	    wf_option   => {type => SCALAR, default => 'fanout'},
	    cleanup     => {type => SCALAR, default => 'false'},
	    timeout     => {type => SCALAR, default => '1800'},
	    validate 	=> {type => SCALAR, default => 'false', regex => qr/^(true|false)$/},
	    'stagger_delay' => {type => SCALAR, default => '15'},
        },
    );

    my $vol_a_compr = $args{vol_a_compr};
    my $vol_b_compr = $args{vol_b_compr};
    my $vol_c_compr = $args{vol_c_compr};

    my $wf_option   = $args{wf_option};
    my $nfsclients  = $args{nfsclients};
    my @nfsclients  = split ',' , $nfsclients;
    my $client_name = $nfsclients[0];
    my $client = NACL::C::Client->new( name => $client_name );

    _stagger($$ % $args{stagger_delay});
    my $timeout = $args{timeout};

    # set compression on src vols
    my $vol_obj = $vol_a_compr->{vol_obj};
    $vol_obj->command_interface->refresh_command_interface();
    if($vol_a_compr->{start_compr} eq "on") {
	$log->step('Enable vol efficiency on vol ' . $vol_obj->volume);
	my $vol_eff_obj = $vol_obj->efficiency_on(
			nacltask_if_enabled => 'pass');
	$vol_eff_obj->modify(
	    compression      => 'true',
	    policy 	     => 'default',
	    'method-timeout' => $timeout,
	);
    }
    my $node_src = NACL::C::Node->new(name => $vol_obj->get_local_node_name());
    $node_src->refresh_command_interface();

    # set compression on dst and gather sms
    $vol_obj = $vol_b_compr->{vol_obj};
    $vol_obj->command_interface->refresh_command_interface();
    if($vol_b_compr->{start_compr} eq "on") {
	$log->step('Enable vol efficiency on vol ' . $vol_obj->volume);
	my $vol_eff_obj = $vol_obj->efficiency_on(
			nacltask_if_enabled => 'pass');
	$vol_eff_obj->modify(
	    compression      => 'true',
	    policy 	     => 'default',
	    'method-timeout' => $timeout,
	);
    }
    my $sm_obj_1 = NACL::STask::Snapmirror->find(
	'command_interface' => $vol_obj->command_interface,
	'filter'            => {
	    'destination-vserver' => $vol_obj->vserver,
	    'destination-volume'  => $vol_obj->volume,
	},
    );
    $sm_obj_1->{start_compr} = $vol_b_compr->{start_compr};
    $sm_obj_1->{end_compr} = $vol_b_compr->{end_compr};
    my $node_dst_b = NACL::C::Node->new(name => $vol_obj->get_local_node_name());
    $node_dst_b->refresh_command_interface();
    $sm_obj_1->{command_interface} = $node_dst_b;
    $sm_obj_1->modify(schedule => '""');

    $vol_obj = $vol_c_compr->{vol_obj};
    $vol_obj->command_interface->refresh_command_interface();
    if($vol_c_compr->{start_compr} eq "on") {
	$log->step('Enable vol efficiency on vol ' . $vol_obj->volume);
	my $vol_eff_obj = $vol_obj->efficiency_on(
			nacltask_if_enabled => 'pass');
	$vol_eff_obj->modify(
	    compression      => 'true',
	    policy 	     => 'default',
	    'method-timeout' => $timeout,
	);
    }
    my $sm_obj_2 = NACL::STask::Snapmirror->find(
	'command_interface' => $vol_obj->command_interface,
	'filter'            => {
	    'destination-vserver' => $vol_obj->vserver,
	    'destination-volume'  => $vol_obj->volume,
	},
    );
    $sm_obj_2->{start_compr} = $vol_c_compr->{start_compr};
    $sm_obj_2->{end_compr} = $vol_c_compr->{end_compr};
    my $node_dst_c = NACL::C::Node->new(name => $vol_obj->get_local_node_name());
    $node_dst_c->refresh_command_interface();
    $sm_obj_2->{command_interface} = $node_dst_c;
    $sm_obj_2->modify(schedule => '""');

    # populate data & create ss0
    my $ss_prefix = 'snapshot_al2l_';
    my $file_size = '10m';
    $vol_obj = $vol_a_compr->{vol_obj};
    if ($vol_obj->get_one_state_attribute('junction-path') eq "-") {
	$vol_obj->mount( 'junction-path' => "/" . $vol_obj->volume() );
    }
    $log->step("Mount volume ". $vol_obj->volume() . " on client $client_name");
    my $mount  = NACL::STask::Client::Mount->create(
	command_interface   => $client,
	vserver             => $vol_obj->vserver,
	volume              => $vol_obj,
	vers                => '3',
	nacltask_if_exists  => 'purge',
	nacltask_verify     => 1,
    );

    $log->step('Populate data on vol ' . $vol_obj->volume);
    dataset(
	operation => 'populate',
	volume    => $vol_obj,
	client    => $client,
	mount     => $mount,
	file_size => $file_size,
	file_list => [ qw(/file1 /file2) ],
    );
    $vol_a_compr->{vol_obj}->{mount} = $mount;
    $log->step('Create snapshot on vol ' . $vol_obj->volume);
    my $ss = $ss_prefix . '0';
    NACL::STask::VolumeSnapshot->create(
	command_interface    => $vol_obj->command_interface(),
	volume               => $vol_obj->volume(),
	vserver              => $vol_obj->vserver(),
	snapshot             => $ss,
	'snapmirror-label'   => 'weekly',
	'nacltask_if_exists' => 'purge',
	'method-timeout'     => $timeout,
    );

    # update
    my $err_to_find = 'Volume busy';
    for my $sm ($sm_obj_1, $sm_obj_2) {
	my $log_reader;
	my $sm_id = $sm->state()->relationship_id;
	if($args{validate} eq 'true'){
	    $log->step( 'Start monitoring audit logs before update' );
	    $log->log('sm relationship_id is ' . $sm_id);
	    $log_reader = start_monitoring_snapmirror_logs(
		node       => $sm->{command_interface},
		rst_id     => $sm_id ,
		log_reader => $log_reader,
	    );
	}

	$log->step('Update snapmirror');
	update_with_retry(
	    'sm_obj'          => $sm,
	    'source_snapshot' => $ss,
	    'nacltask_wait'   => 1,
	    'timeout'         => $timeout,
	);

	if($args{validate} eq 'true'){
	    $log->step( 'Validate transfer engine' );
	    Validate_snapmirror_logs(
		op            => 'Update',
		val_to_assert => ($sm->{start_compr} eq 'on') ? $val_to_assert_LRE : $val_to_assert_LRSE,
		rst_id        => $sm_id,
		log_reader    => $log_reader,
		timeout       => $timeout,
		err_to_find   => $err_to_find,
	    );
	}
    }

    # change vols compression settings if needed
    for my $compr ($vol_a_compr, $vol_b_compr, $vol_c_compr) {
	my $vol_obj = $compr->{vol_obj};
	if($compr->{end_compr} eq "on") {
	    if($compr->{start_compr} eq "off") {
		$log->step('Enable vol efficiency on vol ' . $vol_obj->volume);
		my $vol_eff_obj = $vol_obj->efficiency_on(nacltask_if_enabled => 'pass');
		$vol_eff_obj->modify(
		    compression      => 'true',
		    policy 	     => 'default',
		    'method-timeout' => $timeout,
		);
		$vol_eff_obj->start(
		    'scan-old-data'  => 'true',
		    'nacltask_wait'  => '1',
		    'method-timeout' => $timeout,
		);
	    }
	} else { # off
	    if($compr->{start_compr} eq "on") {
		$log->step('Disable vol efficiency on vol ' . $vol_obj->volume);
		# Wait for the SE scan to finish
		NACL::STask::VolumeEfficiency->wait_for_status(
		    command_interface    => $vol_obj->command_interface,
		    vserver              => $vol_obj->vserver,
		    volume               => $vol_obj,
		    status               => 'Idle',
		    'method-timeout'     => $timeout,
		);
		# Turn off SE
		nacl_method_retry(
		    code => sub {
			NACL::STask::VolumeEfficiency->modify(
			    command_interface    => $vol_obj->command_interface,
			    vserver              => $vol_obj->vserver,
			    volume               => $vol_obj,
			    compression          => 'false',
			    'inline-compression' => 'false',
			    'method-timeout'     => $timeout,
			);
		    },
		    tries_count => 5,
		    sleep_time => 60,
		    exceptions => 'NATE::BaseException',
		);
	    }
	}
    }

    # modify data on src and do update 3 times
    for my $i (1..3) {
	$ss = $ss_prefix . $i;
	my $vol_obj = $vol_a_compr->{vol_obj};
	$log->step('Modify data on vol ' . $vol_obj->volume);
	dataset(
	    operation => 'modify',
	    volume    => $vol_obj,
	    client    => $client,
	    mount     => $vol_obj->{mount},
	    file_size => $file_size,
	    file_list => [ qw(/file2) ],
	);
	$log->step('Create snapshot on vol ' . $vol_obj->volume);
	NACL::STask::VolumeSnapshot->create(
	    command_interface    => $vol_obj->command_interface(),
	    volume               => $vol_obj->volume(),
	    vserver              => $vol_obj->vserver(),
	    snapshot             => $ss,
	    'snapmirror-label'   => 'weekly',
	    'nacltask_if_exists' => 'purge',
	    'method-timeout'     => $timeout,
	);

	for my $sm ($sm_obj_1, $sm_obj_2) {
	    my $log_reader;
	    my $sm_id = $sm->state()->relationship_id;
	    if($args{validate} eq 'true'){
		$log->step( 'Start monitoring audit logs before update' );
		$log->log('sm relationship_id is ' . $sm_id);
		$log_reader = start_monitoring_snapmirror_logs(
		    node       => $sm->{command_interface},
		    rst_id     => $sm_id ,
		    log_reader => $log_reader,
		);
	    }

	    $log->step('Update snapmirror');
	    update_with_retry(
		'sm_obj'          => $sm,
		'source_snapshot' => $ss,
		'nacltask_wait'   => 1,
		'timeout'         => $timeout,
	    );

	    if($args{validate} eq 'true'){
		$log->step( 'Validate transfer engine' );
		Validate_snapmirror_logs(
		    op            => 'Update',
		    val_to_assert => ($sm->{end_compr} eq 'on') ? $val_to_assert_LRE : $val_to_assert_LRSE,
		    rst_id        => $sm_id,
		    log_reader    => $log_reader,
		    timeout       => $timeout,
		    err_to_find   => $err_to_find,
		);
	    }
	}
    }

    $log->step('Break, delete, and release initial snapmirrors');
    my @sm_array = ($wf_option eq 'fanout') ? ($sm_obj_1, $sm_obj_2) : ($sm_obj_2);
    for my $sm (@sm_array) {
	$sm->break  ('method-timeout' => $timeout, force => 'true');
	$sm->release('method-timeout' => $timeout, force => 'true', 'nacltask_wait' => 1, 'source_ci' => $node_src);
	$sm->delete ('method-timeout' => $timeout, force => 'true', 'nacltask_wait' => 1);
    }

    my %sm_r;
    if($wf_option eq 'fanout'){ # do more updates with new data
	$ss = $ss_prefix . '4';
	my $vol_obj = $vol_a_compr->{vol_obj};
	$log->step('Modify data on vol ' . $vol_obj->volume);
	dataset(
	    operation => 'modify',
	    volume    => $vol_obj,
	    client    => $client,
	    mount     => $vol_obj->{mount},
	    file_size => $file_size,
	    file_list => [ qw(/file2) ],
	);
	$log->step('Create snapshot on vol ' . $vol_obj->volume);
	NACL::STask::VolumeSnapshot->create(
	    command_interface    => $vol_obj->command_interface(),
	    volume               => $vol_obj->volume(),
	    vserver              => $vol_obj->vserver(),
	    snapshot             => $ss,
	    'snapmirror-label'   => 'weekly',
	    'nacltask_if_exists' => 'purge',
	    'method-timeout'     => $timeout,
	);
	$vol_obj->{snapshot} = $ss;

	# dst volumes need to be mounted
	my $i = 5;
	for my $compr ($vol_b_compr, $vol_c_compr) {
	    $ss = $ss_prefix . $i;
	    my $vol_obj = $compr->{vol_obj};
	    if ($vol_obj->get_one_state_attribute('junction-path') eq "-") {
		$vol_obj->mount( 'junction-path' => "/" . $vol_obj->volume() );
	    }
	    $log->step("Mount volume ". $vol_obj->volume() . " on client $client_name");
	    my $mount  = NACL::STask::Client::Mount->create(
		command_interface  => $client,
		vserver            => $vol_obj->vserver,
		volume             => $vol_obj,
		vers               => '3',
		nacltask_if_exists => 'purge',
		nacltask_verify    => 1,
	    );

	    $log->step('Modify data on vol ' . $vol_obj->volume);
	    dataset(
		operation => 'modify',
		volume    => $vol_obj,
		client    => $client,
		mount     => $mount,
		file_size => $file_size,
		file_list => [ qw(/file2) ],
	    );
	    $log->step('Create snapshot on vol ' . $vol_obj->volume);
	    NACL::STask::VolumeSnapshot->create(
		command_interface    => $vol_obj->command_interface(),
		volume               => $vol_obj->volume(),
		vserver              => $vol_obj->vserver(),
		snapshot             => $ss,
		'snapmirror-label'   => 'weekly',
		'nacltask_if_exists' => 'purge',
		'method-timeout'     => $timeout,
	    );
	    $vol_obj->{snapshot} = $ss;
	    $i ++;
	}
	# prepare resync for fanout
	$sm_r{src_path} = $sm_obj_2->{destination_path};
	$sm_r{dst_path} = $sm_obj_1->{destination_path};
	$sm_r{snapshot} = $ss_prefix . '6';
	$sm_r{dci} = $sm_obj_1->{command_interface};
	$sm_r{sci} = $node_dst_c;
	$sm_r{end_compr} = $sm_obj_1->{end_compr};
    } else {
	# prepare resync for cascade
	$sm_r{src_path} = $sm_obj_1->{source_path};
	$sm_r{dst_path} = $sm_obj_2->{destination_path};
	$sm_r{snapshot} = $ss_prefix . '3';
	$sm_r{dci} = $sm_obj_2->{command_interface};
	$sm_r{sci} = $node_src;
	$sm_r{end_compr} = $sm_obj_2->{end_compr};

	# volume snapshot restore
	my $vol_obj = $vol_c_compr->{vol_obj};
	$log->step('Volume snapshot restore on ' . $vol_obj->volume);
	$vol_obj->snapshot_restore(
	    'snapshot'       => $ss_prefix . '2',
	    'nacltask_wait'  => '1',
	    'method-timeout' => $timeout,
	);
    }

    # resync
    $log->step('Resync snapmirror ' . $sm_r{src_path} . ':' . $sm_r{dst_path});
    my %extra_opts = ($wf_option eq 'fanout') ? ('preserve' => 'true',
						 'force'    => 'true') : ();
    if($wf_option ne 'fanout') {
	$sm_obj_1->wait_on_attributes(
	    'method-timeout' => $timeout,
	    attributes       => [
		{
		    attribute_to_check => 'status',
		    till_value         => qr/idle/i,
		},
		{
		    attribute_to_check => 'state',
		    till_value         => qr/Snapmirrored/i,
		}
	    ]
	);
    }

    my $st = time;
    my $op_done = 0;
    my $sm;
    while (time < $st + $timeout) {
	try {
	    $sm = NACL::STask::Snapmirror->resync(
		command_interface  => $sm_r{dci},
		'source-path'      => $sm_r{src_path},
		'destination-path' => $sm_r{dst_path},
		'type'             => 'XDP',
		'source-snapshot'  => $sm_r{snapshot},
		'nacltask_wait'    => 0,
		'method-timeout'   => $timeout,
		%extra_opts,
	    );
	    $op_done = 1;
	    last;
	} catch NATE::BaseException with {
	    my $ex = shift;
            if ($ex->text =~ /operation is in progress|Try again/i) {
                $log->warn($ex->text . ' Retry resync in 60s');
		Tharn::snooze(60);
	    } else {
		$ex->throw();
	    }
	};
    }
    NATE::BaseException->throw('Resync failed after ' . $timeout . ' seconds') if !$op_done;

    my $log_reader;
    my $sm_id = $sm->state()->relationship_id;
    if($args{validate} eq 'true'){
	$log->step( 'Start monitoring audit logs for resync' );
	$log->log( 'SM relationship_id is ' . $sm_id);
	$log_reader = start_monitoring_snapmirror_logs(
	    node       => $sm_r{dci},
	    rst_id     => $sm_id ,
	    log_reader => $log_reader,
	);
    }

    $sm->wait_on_attributes(
	'method-timeout' => $timeout,
	attributes => [
	    {
		attribute_to_check => 'status',
		till_value         => qr/Idle/i,
	    },
	    {
		attribute_to_check => 'state',
		till_value         => qr/Snapmirrored/i,
	    }
	],
    );

    if($args{validate} eq 'true'){
	$log->step( 'Validate transfer engine' );
	Validate_snapmirror_logs(
	    op            => 'Resync',
	    val_to_assert => ($sm_r{end_compr} eq 'on') ? $val_to_assert_LRE : $val_to_assert_LRSE,
	    rst_id        => $sm_id,
	    log_reader    => $log_reader,
	    timeout       => $timeout,
	    err_to_find   => $err_to_find,
	);
    }

    $log->step('Run checker');
    $sm->quiesce('method-timeout' => $timeout, 'nacltask_wait' => 1);
    $sm->check  ('method-timeout' => $timeout,);

    $log->step('Delete and release the snapmirror relationship');
    $sm->release('method-timeout' => $timeout, 'source_ci' => $sm_r{sci}, 'nacltask_wait' => 1);
    $sm->delete ('method-timeout' => $timeout, 'nacltask_wait' => 1);

    if($args{cleanup} ne 'true'){
	$log->step('Rebuild the snapmirrors'); # for repeatable workflow test
	if($wf_option ne 'fanout') {
	    $sm_obj_1->wait_on_attributes(
		'method-timeout' => $timeout,
		attributes       => [
		    {
			attribute_to_check => 'status',
			till_value         => qr/idle/i,
		    },
		]
	    );
	}
	for my $sm ($sm_obj_1, $sm_obj_2) {
	    my $st = time;
	    my $op_done = 0;
	    while (time < $st + $timeout) {
		try {
		    $sm->resync('method-timeout' => $timeout, nacltask_wait => 1, type => 'xdp');
		    $op_done = 1;
		    last;
		} catch NATE::BaseException with {
		    my $ex = shift;
		    if ($ex->text =~ /operation is in progress|Try again/i) {
		        $log->warn($ex->text . ' Retry resync in 60s');
			Tharn::snooze(60);
		    } else {
			$ex->throw();
		    }
		};
	    }
	    NATE::BaseException->throw('Resync failed after ' . $timeout . ' seconds') if !$op_done;
	}
    }
}


sub update_with_retry {
    my %args = validate_with(
        params => \@_,
        spec   => {
            'sm_obj' 	      => {type => OBJECT},
	    'source_snapshot' => {type => OBJECT|SCALAR|UNDEF, optional => 1},
	    'nacltask_wait'   => {type => SCALAR, default => 1},
            'timeout'         => {type => SCALAR, default => '1800'},
        },
    );

    my $sm_obj = $args{sm_obj};
    my $timeout = $args{timeout};
    my $source_snapshot = $args{source_snapshot};
    my $nacltask_wait = $args{nacltask_wait};

    $sm_obj->wait_on_attributes(
        'method-timeout' => $timeout,
        attributes       => [
            {
                attribute_to_check => 'status',
                till_value         => qr/idle/i,
            },
	    {
		attribute_to_check => 'state',
		till_value         => qr/Snapmirrored/i,
	    }
        ]
    );

    my $st = time;
    my $op_done = 0;
    while (time < $st + $timeout) {
	my %extra_opts = ($source_snapshot) ?
	    ('source-snapshot' => $source_snapshot,) : ();
        try {
            $sm_obj->update(
                'nacltask_wait'   => $nacltask_wait,
		%extra_opts
            );
            $op_done = 1;
            last;
        } catch NATE::BaseException with {
            my $ex = shift;
            if ($ex->text =~ /operation is in progress|Try again/i) {
                $log->warn($ex->text . ' Retry update in 60s');
                Tharn::snooze(60);
            } else {
                $ex->throw();
            }
        };
    }

    NATE::BaseException->throw('Update failed after ' . $timeout . ' seconds')
	if !$op_done;

} ## end sub update_with_retry


=head2 Wf_Appcg_forward_transfer

A wrapper to run Appcg_forward_transfer

=over

=item Example

    Appcg_forward_transfer{sm_objs  => /@snapmirror,
                           nfsclients => @nfsclients,
                           src_cg    => $src_cg,
                           dst_cg  => $dst_cg,
                           };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  ARRAYREF of Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.

=back

=back

=item exceptions

    NATE::BaseException

=cut

sub Wf_Appcg_forward_transfer{
    my %args = validate_with(
        params => \@_,
        allow_extra => 1,
        spec   => {
           sm_objs    => { type => ARRAYREF},
           nfsclients => { type => ARRAYREF|SCALAR},
           src_cg     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/} ,
           dst_cg     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
           timeout    => { type => SCALAR, optional => 1, default => 1800},
           validation => { type => SCALAR, optional => 1, default => 0},
        },
    );

    $log->step('Get sm objects');
    my @snapmirror = @{$args{sm_objs}};
    my $count  = scalar(@snapmirror);

    $log->step('Get nfsclients');
    my $nfsclients = $args{nfsclients};
    my @nfsclients = split ',', $nfsclients;

    my $sm_obj;
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};
    my $validation = $args{validation};
    my $client_name;
    my $timeout = $args{timeout};
    my @subtests_snapmirror;

    my @procs_ls_command;
    my @sm_obj = @{$args{sm_objs}};

    foreach my $snapmirror_obj (@sm_obj) {
	my $client = shift @nfsclients;
        my %args_func = (sm_objs => $snapmirror_obj,
                nfsclients => $client,
                src_cg => $src_cg,
                dst_cg => $dst_cg,
                validation => $validation,
                timeout => $timeout);
        push @subtests_snapmirror, create_parallel_processes('Appcg_forward_transfer',\%args_func);
	push @nfsclients, $client;
    }
    run_processes_bg('Appcg_forward_transfer',\@subtests_snapmirror);
} #end of Wf_Appcg_forward_transfer

=head2 Appcg_forward_transfer

Steps
  1) create and initialize a lrsedr relationship using snapshot s0.
  2) configure with dst_cg and do sm update. Check transfer engine.
  3) configure with src_cg and do sm update. Check transfer engine.
  4) sm update again and make sure Auto L2L happens if dst node is in FT.
  5) do sm abort and sm hard abort during transfer, and check transfer engine after restart.
  6) do sm break and sm resync and check for transfer engine.
  7) turn on the additional compression on dst and confirm it is LRE for sm update.
  8) turn off the additional compression on dst and confirm Auto L2L happens if dst node is FT.

=over

=item Example

    Appcg_forward_transfer{sm_objs  => $sm_obj,
                           nfsclients => $nfsclients,
                           src_cg    => $src_cg,
                           dst_cg  => $dst_cg,
    };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.


=back

=cut

sub Appcg_forward_transfer {
    my %args = validate_with(
        params => \@_,
        spec   => {
           sm_objs    => { type => OBJECT},
           nfsclients => { type => ARRAYREF|SCALAR},
           src_cg     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
           dst_cg     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
           timeout    => { type => SCALAR, optional => 1, default => 1800},
           validation => { type => SCALAR, optional => 1, default => 0},
        },
    );

    $log->step('Get sm objects');
    my $sm_obj = $args{sm_objs};

    $log->step('Get nfsclients');
    my $client_name = $args{nfsclients};
    my $client = NACL::C::Client->new(name => $client_name);

    $log->step('Get cg');
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};

    $log->step('Get Validation Value');
    my $validation = $args{validation};

    $log->comment('To add delay during parallel runs');
    _stagger($$ % 60);

    $log->step('Get node and src vol information from the sm objects');
    my $source_path      = $sm_obj->{source_path};
    my $destination_path = $sm_obj->{destination_path};
    my ($src_vserver, $src_vol_name) = ($source_path      =~ /^(.*):(.*)$/);
    my ($dst_vserver, $dst_vol_name) = ($destination_path =~ /^(.*):(.*)$/);

    $log->step('Get Src and Dst Objs from the sm objects');
    my $src_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{source_ci},
        volume            => $src_vol_name,
        vserver           => $src_vserver,
    );
    my $dst_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{command_interface},
        volume            => $dst_vol_name,
        vserver           => $dst_vserver,
    );
    my $src_node = NACL::STask::Node->new(name => $src_vol->get_local_node_name());
    my $dst_node = NACL::STask::Node->new(name => $dst_vol->get_local_node_name());

    $src_node->refresh_command_interface();
    $dst_node->refresh_command_interface();

    my $src_node_name = $src_node->name();
    my $dst_node_name = $dst_node->name();

    my @subtests;
    my $timeout = $args{timeout};

    my $Mount = NACL::STask::Client::Mount->create(
        command_interface  => $client,
        volume             => $src_vol,
        nacltask_if_exists => 'purge',
        nacltask_verify    => 1,
    );
    my $path = $Mount->mount_point();
    my $net_interface_obj = NACL::CS::NetworkInterface->fetch(
            command_interface => $src_node,
            filter            => {
                'home-node'      => $src_node,
                role             => 'node-mgmt',
                'address-family' => 'ipv4',
            },
            requested_fields => ['address'],
        );
    my $node_mgmt_ip = $net_interface_obj->{'address'};

    my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
    my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
    my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

    my $sis_src;
    my $rst_id;
    my $cg_size;
    my $cg;
    my $val_to_assert;
    my $comp_info_list;
    my $src_comp_info;
    my $dst_comp_info;

    $log->step('Get Ontap Version for source_node and dst_node' );
    my $src_ontap_version = get_ontap_version( node => $src_node );
    my $dst_ontap_version = get_ontap_version( node => $dst_node );
    $log->log('Source Cluster Ontap Version: ' . $src_ontap_version );
    $log->log('Dst Cluster Ontap Version: ' . $dst_ontap_version );

    if( $src_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
           $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
        $log->log('Since it is HN release and Adap16k or Adap32k is not supported' );
        $log->log('The test will exist' );
        $log->exit();
        return;
    }

    $log->step('Get sm relationship_id ');
    my $sm_state = $sm_obj->state();
    $rst_id = $sm_state->relationship_id;
    $log->comment('Relationship_id is : '. $rst_id);

    $log->step('Display Vol Efficiency on src vol ' );
    $sis_src = NACL::CS::VolumeEfficiency->fetch(
             command_interface       => $src_node,
             filter                  => {
                 volume              => $src_vol_name,
                 vserver             => $src_vserver,
             },
             requested_fields        => ['state'],
             allow_empty             => 1,
         );
    my $src_se_state = $sis_src->{'state'};
    $log->log('Print SE state: '. $src_se_state );

    if($src_se_state eq 'Enabled'){
        $log->step('Sis undo on src volume since SE is already enabled ');
        sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'         => $timeout,
        );

        $log->step('Create Snapshot ss0');
        $src_vol->snapshot_create(
            snapshot           => 'ss0',
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        $log->step("Update with ss0 to make sure Dest has compression_info 0");
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => 'ss0',
            'nacltask_wait'   => 1,
            'timeout'         => $timeout,
        );
    }

    $log->step('Enable Efficiency on ' . $src_vol_name );
    $sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    
    $log->step('Change the policy to default');
    $sis_src->modify(
        policy               => 'default',
    );

    $log->step("Configure compression type with   $dst_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $dst_cg, sis_src => $sis_src);

    if ( $validation eq '1') {
	$log->step('Show Efficiency on ' . $src_vol_name  );
	appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
				vserver => $src_vserver, ontap_version => $src_ontap_version );
    }

    $log->step("Overwrite data with : $dst_cg dataset" );
    my $count=0;  #count is used for creating logs with diff run_id for each overwrite
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $dst_cg,
                                   path => $path, count => $count );

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data'  => 'true',
        'nacltask_wait'  => '1',
	'method-timeout' => $timeout,
    );

    $log->step('Create Snapshot s1_'. $dst_cg );
    my $s1 = 's1_' . $dst_cg ;
    $src_vol->snapshot_create(
        snapshot           => $s1,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    my $log_reader;
    if ( $validation eq '1') {
	$log->step( 'Start monitoring audit logs for update' );
	$log_reader = start_monitoring_snapmirror_logs(
	    node       => $dst_node,
	    rst_id     => $rst_id ,
	    log_reader => $log_reader,
	);
    }

    $log->step("Update the snapmirror relationship with snapshot $s1");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s1,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
	$log->step( 'Validate transfer engine' );
        if( $dst_ontap_version eq 'HN_LOWER' && (
             $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
	        Validate_snapmirror_logs(
	            op            => 'Update',
	            val_to_assert => $val_to_assert_LRE,
	            rst_id        => $rst_id,
	            log_reader    => $log_reader,
	        );
        }
	else {
	    Validate_snapmirror_logs(
	        op            => 'Update',
	        val_to_assert => $val_to_assert_LRSE,
	        rst_id        => $rst_id,
	        log_reader    => $log_reader,
	    );
       }
    }

    if (  $dst_cg ne $src_cg ) {
        $log->step('Sis undo on src volume');
	sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'         => $timeout,
        );

	$log->step('Enable Efficiency on source volume'  );
	$sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    }

    $log->step("Configure compression type with   $src_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $src_cg, sis_src => $sis_src);

    if ( $validation eq '1') {
	$log->step('Show Efficiency on ' . $src_vol_name  );
	appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
				vserver => $src_vserver, ontap_version => $src_ontap_version );
    }

    $log->step("Overwrite data with : $src_cg dataset" );
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                   path => $path, count => $count );

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data'  => 'true',
        'nacltask_wait'  => '1',
	'method-timeout' => $timeout,
    );

    $log->step('Create Snapshot s2_'. $src_cg );
    my $s2 = 's2_' . $src_cg;
    $src_vol->snapshot_create(
        snapshot           => $s2,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    if ( $validation eq '1') {
	$log->step( 'Start monitoring audit logs for update' );
	$log_reader = start_monitoring_snapmirror_logs(
	    node       => $dst_node,
	    rst_id     => $rst_id ,
	    log_reader => $log_reader,
	);

	$log->step( 'Print Compression Capability for ' .
	    'source afs , dst afs, transfer snapshots after update' );
	$comp_info_list = print_compression_info (
	    src_node           => $src_node,
	    src_vol_name       => $src_vol_name,
	    dst_node           => $dst_node,
	    dst_vol_name       => $dst_vol_name,
	    transfer_snapshot  => $s2,
	    timeout            => $timeout,
	);
	$src_comp_info = $comp_info_list->{'src_afs'};
	$dst_comp_info = $comp_info_list->{'dst_afs'};

        $log->step( 'Expected String for Transfer Engine ' );
        if( $dst_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
            $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
                $val_to_assert = $val_to_assert_LRE ;
                $log->log( '$val_to_assert' . $val_to_assert );
        }
        else {
            $val_to_assert = Validate_appcg_forward_transfer (
                src_comp_info => $src_comp_info,
                dst_comp_info => $dst_comp_info,
                src_ontap_version => $src_ontap_version,
                dst_ontap_version => $dst_ontap_version,
            );
        $log->log( '$val_to_assert' . $val_to_assert );
        }
    }

    $log->step("Update the snapmirror relationship with snapshot $s2");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s2,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );
    }

    $log->log( 'Auto L2L test will continue only Validation is set to 1' );
    if ($validation eq '1') {
        my $lre=0;
        $log->step( 'Check to see LRE has happened' );
        if($val_to_assert eq '(Logical Transfer -).*(Directory Mode)' ){
           $log->log( 'LRE Transfer has happened' );
           $lre=1;
        }

        $log->step("Overwrite data with : $src_cg dataset" );
        $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                       node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                       path => $path, count => $count );

        $log->step('Issue sis start on ');
	$sis_src->start(
	    'scan-old-data'  => 'true',
	    'nacltask_wait'  => '1',
	    'method-timeout' => $timeout,
	);

        $log->step('Create Snapshot s3_'. $src_cg );
        my $s3 = 's3_' . $src_cg;
        $src_vol->snapshot_create(
            snapshot           => $s3,
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        $log->step( 'Throttle the transfer' );
        $sm_obj->modify( throttle => '10', 'method-timeout' => $timeout, );

        $log->step("Update the snapmirror relationship with snapshot $s3");
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => $s3,
            'nacltask_wait'   => 0,
            'timeout'         => $timeout,
        );

        $log->step('Wait 3 minutes for checkpoint to be generated');
        my $ckpt = 0;
        try {
            $sm_obj->wait_on_attributes(
                'method-timeout' => '180',
                attributes => [
                    {
                        attribute_to_check => 'snapshot-checkpoint',
                        till_value         => qr/[1-9]\d*/
                    },
                ]
            );
            $ckpt = 1;
        } catch NATE::BaseException with {
            $log->comment("exit waiting for checkpoint.");
        };

        $log->step( 'SM abort' );
        snapmirror_abort(
            node_src => $src_node,
            node_dst => $dst_node,
            sm_objs  => [$sm_obj],
            validate => ($ckpt) ? 'true' : 'false',
        );

        $log->step( 'SM restart transfer' );
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => $s3,
            'nacltask_wait'   => 0,
            'timeout'         => $timeout,
        );

        $log->step('Wait 3 minutes for checkpoint to be generated');
        try {
            $sm_obj->wait_on_attributes(
                'method-timeout' => '180',
                attributes => [
                    {
                        attribute_to_check => 'snapshot-checkpoint',
                        till_value         => qr/[1-9]\d*/
                    },
                ]
            );
            $ckpt = 1;
        } catch NATE::BaseException with {
            $log->comment("exit waiting for checkpoint.");
        };

        $log->step( 'SM abort -h ' );
        snapmirror_abort_hard(
            node_src => $src_node,
            node_dst => $dst_node,
            sm_objs  => [$sm_obj],
            validate => ($ckpt) ? 'true' : 'false'
        );

        $sm_obj->wait_on_attributes(
           'method-timeout' => $timeout,
            attributes  => [
               {
                  attribute_to_check => 'status',
                  till_value  => 'Idle'
               }
            ]
        );

        $log->step('Modify snapmirror throttle back to unlimited');
        $sm_obj->modify('throttle' => 'unlimited', 'method-timeout' => $timeout,);

        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        if( $lre eq '1' && $dst_ontap_version eq 'FT_HIGHER' && $src_ontap_version eq 'FT_HIGHER'){
            $log->step( 'SM restart transfer, and Auto L2L will happen' );
            update_with_retry(
                'sm_obj'          => $sm_obj,
                'source_snapshot' => $s3,
                'nacltask_wait'   => 1,
                'timeout'         => $timeout,
            );
            $val_to_assert = $val_to_assert_LRSE;
        }
        elsif( $lre eq '1' &&  (($src_ontap_version eq 'HN_LOWER' || $dst_ontap_version eq 'HN_LOWER') &&
            ($src_cg eq 'Adap32k' || $src_cg eq 'Adap16k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k'))){
            $log->step ( 'SM restart transfer, and User cannot do'.
	        ' manual switch since destination did not understands new CG and it will remain as LRE transfer' );
	    $sm_obj->update(
                'source-snapshot' => $s3,
                'method-timeout'  => $timeout,
                 nacltask_wait     => 1,
             );
            $val_to_assert = $val_to_assert_LRE;
        }
        elsif( $lre eq '1' &&  (($src_ontap_version eq 'HN_LOWER' || $dst_ontap_version eq 'HN_LOWER') &&
            ($src_cg ne 'Adap32k' || $src_cg ne 'Adap16k' || $dst_cg ne 'Adap16k' || $dst_cg ne 'Adap32k'))){
            $log->step ( 'SM restart transfer, and User needs to do'.
               ' manual swich to become LRSE since dest node is HN' );
            $sm_obj->update(
                'source-snapshot' => $s3,
                'enable-storage-efficiency' => 'true',
                'method-timeout'  => $timeout,
                 nacltask_wait     => 1,
             );
            $val_to_assert = $val_to_assert_LRSE;
        }
        else {
            $log->step( 'Previous Transfer is LRSE' );
            update_with_retry(
                'sm_obj'          => $sm_obj,
                'source_snapshot' => $s3,
                'nacltask_wait'   => 1,
                'timeout'         => $timeout,
            );
            $val_to_assert = $val_to_assert_LRSE;
        }

        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );

        $log->step('Break the snapmirror relationship');
        snapmirror_break( node_src => $src_node,
			node_dst => $dst_node,
			sm_objs =>  [$sm_obj],
			validate => '1',
		    );

        $log->step("Overwrite data with : $src_cg dataset" );
        $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                       node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                       path => $path, count => $count );

        $log->step('Issue sis start on the volume');
	$sis_src->start(
	    'scan-old-data'  => 'true',
	    'nacltask_wait'  => '1',
	    'method-timeout' => $timeout,
	);

        $log->step('Create Snapshot s4_'. $src_cg .' for update');
        my $s4 = 's4_' . $src_cg;
        $src_vol->snapshot_create(
            snapshot           => $s4,
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        $log->step('Resync the snapmirror relationship');
        snapmirror_resync(node_src     => $src_node,
			node_dst         => $dst_node,
			sm_objs          => [$sm_obj],
			validate         => '1',
			nacltask_wait    => 1
		    );

        if( $lre eq '1' &&  (($src_ontap_version eq 'HN_LOWER' || $dst_ontap_version eq 'HN_LOWER') &&
            ($src_cg ne 'Adap32k' || $src_cg ne 'Adap16k' || $dst_cg ne 'Adap16k' || $dst_cg ne 'Adap32k'))){
            $val_to_assert = $val_to_assert_LRE;
        }else {
            $val_to_assert = $val_to_assert_LRSE;
        }

        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Resync',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );

        $log->step('Enable Efficiency on the dst volume');
        my $sis_dst =  $dst_vol->efficiency_on(nacltask_if_enabled => 'pass');

        $log->step('Modify Efficiency on the dst volume to add the additional compression ');
        $sis_dst->modify(
            compression          => 'true',
            'inline-compression' => 'true',
             schedule            => '-',
            'method-timeout'     => $timeout,
        );

	if ( $validation eq '1') {
	    $log->step('Show Efficiency on ' . $dst_vol_name  );
	    appcg_efficiency_show ( vol_name => $dst_vol_name , node_obj => $dst_node,
				    vserver => $dst_vserver, ontap_version => $dst_ontap_version );
	}

        $log->step("Overwrite data with : $src_cg dataset" );
        $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                       node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                       path => $path, count => $count );

        $log->step('Issue sis start on ');
	$sis_src->start(
	    'scan-old-data'  => 'true',
	    'nacltask_wait'  => '1',
	    'method-timeout' => $timeout,
	);

        $log->step('Create Snapshot s5_'. $src_cg );
        my $s5 = 's5_' . $src_cg;
        $src_vol->snapshot_create(
            snapshot           => $s5,
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

	if  ( $validation eq '1') {
            $log->step( 'Print Compression Capability for ' .
                'source afs , dst afs, transfer snapshots before update ' );
            $comp_info_list = print_compression_info (
                src_node           => $src_node,
                src_vol_name       => $src_vol_name,
                dst_node           => $dst_node,
                dst_vol_name       => $dst_vol_name,
                transfer_snapshot  => $s5,
            );
        }

        my $log_reader;
        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        $log->step("Update the snapmirror relationship with snapshot $s5");
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => $s5,
            'nacltask_wait'   => 1,
            'timeout'         => $timeout,
        );

        if  ( $validation eq '1') {
            $log->step( 'Print Compression Capability for ' .
                'source afs , dst afs, transfer snapshots before update' );
            $comp_info_list = print_compression_info (
                src_node           => $src_node,
                src_vol_name       => $src_vol_name,
                dst_node           => $dst_node,
                dst_vol_name       => $dst_vol_name,
                transfer_snapshot  => $s5,
            );
        }

        $log->step('Expect LRE Transfer due to additional compression on dst');
        $val_to_assert = $val_to_assert_LRE;

        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );

        $log->step('Turn off the inline compression and compression on the dst volume based ');
        # Make sure the scan is done before disabling it
        NACL::STask::VolumeEfficiency->wait_for_status (
            command_interface    => $dst_node,
            vserver              => $dst_vserver,
            volume               => $dst_vol_name,
            status               => 'Idle',
            'method-timeout'     => $timeout,
        );

	nacl_method_retry(
	    code => sub {
		NACL::STask::VolumeEfficiency->modify(
		    command_interface    => $dst_node,
		    vserver              => $dst_vserver,
		    volume               => $dst_vol_name,
		    compression          => 'false',
		    'inline-compression' => 'false',
		    'method-timeout'     => $timeout,
		);
	    },
	    tries_count => 5,
	    sleep_time => 60,
	    exceptions => 'NATE::BaseException',
	);

	if ( $validation eq '1') {
	    $log->step('Show Efficiency on ' . $dst_vol_name  );
	    appcg_efficiency_show ( vol_name => $dst_vol_name , node_obj => $dst_node,
				    vserver => $dst_vserver, ontap_version => $dst_ontap_version );
	}

        $log->step("Overwrite data with : $src_cg dataset" );
        $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                       node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                       path => $path, count => $count );

        $log->step('Issue sis start on ');
	$sis_src->start(
	    'scan-old-data'  => 'true',
	    'nacltask_wait'  => '1',
	    'method-timeout' => $timeout,
	);

        $log->step('Create Snapshot s6_'. $src_cg );
        my $s6 = 's6_' . $src_cg;
        $src_vol->snapshot_create(
            snapshot           => $s6,
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        if  ( $validation eq '1') {
            $log->step( 'Print Compression Capability for ' .
                'source afs , dst afs, transfer snapshots before update' );
            $comp_info_list = print_compression_info (
                src_node           => $src_node,
                src_vol_name       => $src_vol_name,
                dst_node           => $dst_node,
                dst_vol_name       => $dst_vol_name,
                transfer_snapshot  => $s6,
            );
        }

        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        if(  $dst_ontap_version eq 'FT_HIGHER' && $src_ontap_version eq 'FT_HIGHER'){
            $log->step( 'SM update and Auto L2L will happen' );
            update_with_retry(
                'sm_obj'          => $sm_obj,
                'source_snapshot' => $s6,
                'nacltask_wait'   => 1,
                'timeout'         => $timeout,
            );
            $val_to_assert = $val_to_assert_LRSE;
        }
        elsif( $lre eq '1' &&  (($src_ontap_version eq 'HN_LOWER' || $dst_ontap_version eq 'HN_LOWER') &&
            ($src_cg eq 'Adap32k' || $src_cg eq 'Adap16k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k'))){
            $log->step ( 'SM restart transfer, and User cannot do'.
	        ' manual switch since destination did not understands new CG and it will remain as LRE transfer' );
            $sm_obj->update(
                'source-snapshot' => $s6,
                'method-timeout'  => $timeout,
                 nacltask_wait     => 1,
             );
            $val_to_assert = $val_to_assert_LRE;
        }
        elsif( $src_ontap_version eq 'HN_LOWER' || $dst_ontap_version eq 'HN_LOWER'){
            $log->step ( 'SM update  and User needs to do'.
               ' manual swich to become LRSE since src or dest node is HN_LOWER' );
            $sm_obj->update(
                'source-snapshot' => $s6,
                'enable-storage-efficiency' => 'true',
                'method-timeout'  => $timeout,
                 nacltask_wait     => 1,
             );
            $val_to_assert = $val_to_assert_LRSE;
        }

	if  ( $validation eq '1') {
            $log->step( 'Print Compression Capability for ' .
                'source afs , dst afs, transfer snapshots after update' );
            $comp_info_list = print_compression_info (
                src_node           => $src_node,
                src_vol_name       => $src_vol_name,
                dst_node           => $dst_node,
                dst_vol_name       => $dst_vol_name,
                transfer_snapshot  => $s6,
            );
        }

        $log->step( 'Validate transfer engine' );
        $log->log( '$val_to_assert: '. $val_to_assert );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );

        $log->step('SM check');
	Wf_sm_checker(
	    src_node => $src_node,
	    dst_node => $dst_node,
	    sm_objs  => [$sm_obj],
	);
    }

    $log->step('Sis undo on src volume and it will turn off sis too');
    sis_undo_with_retry(
        'src_vol'         => $src_vol,
        'timeout'         => $timeout,
    );

    $log->step('Create Snapshot s0' );
    my $s0 = 's0';
    $src_vol->snapshot_create(
        snapshot           => $s0,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    $log->step("Update the snapmirror relationship with snapshot $s0");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s0,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if  ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots after update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s0,
        );
    }
    $log->exit();
    return;

}  #end of Appcg_forward_transfer

sub configure_appcg {
    my %args = validate_with(
        params => \@_,
        spec   => {
            cg      => {type => SCALAR, optional => 0},
            sis_src => {type => OBJECT, optional => 0},
	    timeout => {type => SCALAR, default => '1800'},
        },
    );
    my $cg = $args{cg};
    my $sis_src = $args{sis_src};
    my $log = log_global();
    $log->enter();

    if ($cg eq 'Sec32k' ){
        $log->step('Modify Efficiency on the src volume based on '. $cg);
        $sis_src->modify(
            compression            => 'true',
            'inline-compression'   => 'true',
            'compression-type'     => 'secondary',
            #'application-io-size'  => 'default',
             schedule              => '-',
            'method-timeout'       => $args{timeout},
        );
    }
    elsif ($cg eq 'Adap8k'){
        $log->step('Modify Efficiency on the src volume based on '. $cg);
        $sis_src->modify(
            compression            => 'true',
            'inline-compression'   => 'true',
            'compression-type'     => 'adaptive',
            #'application-io-size'   => '8K',
            schedule               => '-',
            'method-timeout'       => $args{timeout},
        );
    }
    elsif ($cg eq 'Adap16k' ){
        $log->step('Modify Efficiency on the src volume based on '. $cg);
        $sis_src->modify(
            compression            => 'true',
            'inline-compression'   => 'true',
            'compression-type'     => 'adaptive',
            'application-io-size'  => '16K',
             schedule              => '-',
            'method-timeout'       => $args{timeout},
        );
    }
    elsif ($cg eq  'Adap32k' ){
        $log->step('Modify Efficiency on the src volume based on '. $cg);
        $sis_src->modify(
            compression            => 'true',
            'inline-compression'   => 'true',
            'compression-type'     => 'adaptive',
            'application-io-size'  => '32K',
            schedule               => '-',
            'method-timeout'       => $args{timeout},
        );
    }
    else {
        $log->step('Enable Efficiency on the src volume with '. $cg);
        $sis_src->modify(
            compression            => 'false',
            'inline-compression'   => 'false',
            'compression-type'     => 'none',
#            'application-io-size'   => 'default',
            schedule               => '-',
            'method-timeout'       => $args{timeout},
        );
    }
    return $sis_src;
}#end of configure_appcg

sub appcg_efficiency_show {
    my %args = validate_with(
        params => \@_,
        spec   => {
            vol_name            => {type => SCALAR},
            node_obj            => {type => OBJECT},
            vserver             => {type => SCALAR},
            ontap_version       => {type => SCALAR},
        },
    );
    my $ontap_version = $args{'ontap_version'};
    my $log = log_global();

    if ( $ontap_version eq 'HN_LOWER') {
        $log->log('Show Efficiency on ' . $args{vol_name} );
        NACL::CS::VolumeEfficiency->fetch(
             command_interface       => $args{node_obj},
             filter                  => {
                 volume              => $args{vol_name},
                 vserver             => $args{vserver},
             },
             requested_fields        => ['inline-compression','compression',
                                        'compression-type' ],
         );
    } else {
        $log->log('Show Efficiency on ' . $args{vol_name} );
        NACL::CS::VolumeEfficiency->fetch(
             command_interface       => $args{node_obj},
             filter                  => {
                 volume              => $args{vol_name},
                 vserver             => $args{vserver},
             },
             requested_fields        => ['inline-compression','compression',
                 'compression-type', 'application-io-size' ],
         );
    }
}#end of appcg_efficiency_show

sub get_ontap_version {
    my %args = @_;
    my $node = $args{'node'};
    my $ontap_version;
    my $log = log_global();

    my $version_string = $node->apiset()->get_version_manager()->get_version_attribute( 'attribute' => 'release');
    $version_string =~ s/.*__(\d)[.](\d)[.]\d/$1 $2/;
    $log->comment('version: ' . $1);
    $log->comment('release: ' . $2);
    if( $1 >= 9 && $2 >= 3) { #FT or Higher
        $log->comment($node->name() . ' is in FT or Higher release' );
        $ontap_version ='FT_HIGHER';
    }elsif( $1 >= 9 && $2 <= 2) { #HN
        $log->comment($node->name() . ' is in HN or Lower release' );
        $ontap_version ='HN_LOWER';
    }
    return $ontap_version;
}#get_ontap_version

sub print_compression_info {
    my %args = validate_with(
        params => \@_,
        spec   => {
            src_node           => {type => OBJECT},
            src_vol_name       => {type => SCALAR},
            dst_node           => {type => OBJECT},
            dst_vol_name       => {type => SCALAR},
            transfer_snapshot  => {type => SCALAR},
	    timeout            => {type => SCALAR, default => '1800'},
        },
    );
    my $trn_snap = $args{transfer_snapshot};
    my $log = log_global();
    $log->enter();

    my $result;
    my %comp_info_list;

    my $vol_explore_obj0 = NACL::C::Volume->explore(
        command_interface => $args{src_node},
        format            => 'volume',
        scope             => "$args{src_vol_name}",
	'method-timeout'  => $args{timeout},
    );

    my $vol_explore_obj1 = NACL::C::Volume->explore(
        command_interface => $args{dst_node},
        format            => 'volume',
        scope             => "$args{dst_vol_name}",
	'method-timeout'  => $args{timeout},
    );

    my $vol_explore_obj2 = NACL::C::Volume->explore(
        command_interface => $args{src_node},
        format            => 'volume',
        scope             => "$args{src_vol_name}\~$trn_snap",
	'method-timeout'  => $args{timeout},
    );

    my $vol_explore_obj3 = NACL::C::Volume->explore(
        command_interface => $args{dst_node},
        format            => 'volume',
        scope             => "$args{dst_vol_name}\~$trn_snap",
	'method-timeout'  => $args{timeout},
    );

    $comp_info_list{'src_afs'} = $vol_explore_obj0->{'compression_info'};
    $comp_info_list{'dst_afs'} = $vol_explore_obj1->{'compression_info'};
    $comp_info_list{'src_trn_snap'} = $vol_explore_obj2->{'compression_info'};
    $comp_info_list{'dst_trn_snap'} = $vol_explore_obj3->{'compression_info'};

    $log->log("Compression infor for  source volume AFS is  $comp_info_list{'src_afs'}");
    $log->log("Compression infor for  dest volume AFS is  $comp_info_list{'dst_afs'}");
    $log->log("Compression infor for  source transfer snapshot is $comp_info_list{'src_trn_snap'} ");
    $log->log("Compression infor for  destination transfer snapshot is $comp_info_list{'dst_trn_snap'} ");

    return \%comp_info_list;
} ## end sub _print_compresssion_inf

sub snapshot_list {
    my %args = validate_with(
        params => \@_,
        spec   => {
            src_node           => {type => OBJECT},
            src_vserver        => {type => SCALAR},
            src_vol_name       => {type => SCALAR},
            dst_node           => {type => OBJECT},
            dst_vserver        => {type => SCALAR},
            dst_vol_name       => {type => SCALAR},
        },
    );
    my $src_node = $args{src_node};
    my $src_vserver = $args{src_vserver};
    my $src_vol_name = $args{src_vol_name};
    my $dst_node = $args{dst_node};
    my $dst_vserver = $args{dst_vserver};
    my $dst_vol_name = $args{dst_vol_name};
    my $log = log_global();
    $log->enter();

    $log->step("List snapshots on src and dst volumes");
    my @snapshot_obj = NACL::C::VolumeSnapshot->find(
        command_interface => $src_node,
        filter => { vserver => $src_vserver, volume => $src_vol_name }
    );

    @snapshot_obj = NACL::C::VolumeSnapshot->find(
        command_interface => $dst_node,
        filter => { vserver => $dst_vserver, volume => $dst_vol_name }
    );
} ## end sub snapshot_list


=head2 Wf_Appcg_restore

A wrapper to run Appcg_restore

=over

=item Example

    Wf_Appcg_restore{sm_objs  => /@snapmirror,
                           nfsclients => @nfsclients,
                           src_cg    => $src_cg,
                           dst_cg  => $dst_cg,
                           local_restore  => $local_restore,
                           remote_restore  => $remote_restore,
                           };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  ARRAYREF of Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.

=item local_restore

(Required):  To do local restore only.

=item remote_restore

(Required):  To do remote restore only.

=back

=back

=item exceptions

    NATE::BaseException

=cut

sub Wf_Appcg_restore {
    my %args = validate_with(
        params => \@_,
        allow_extra => 1,
        spec   => {
           sm_objs                    => { type => ARRAYREF},
           nfsclients                 => { type => ARRAYREF|SCALAR},
           src_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/} ,
           dst_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
	   local_restore              => { type => SCALAR},
	   remote_restore             => { type => SCALAR},
           timeout                    => { type => SCALAR, optional => 1, default => 1800},
           validation                 => { type => SCALAR, optional => 1, default => 0},
        },
    );

        $log->step('Get sm objects');
    my @snapmirror = @{$args{sm_objs}};
    my $count  = scalar(@snapmirror);

    $log->step('Get nfsclients');
    my $nfsclients = $args{nfsclients};
    my @nfsclients = split ',', $nfsclients;

    my $sm_obj;
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};
    my $local_restore = $args{local_restore};
    my $remote_restore = $args{remote_restore};
    my $validation = $args{validation};
    my $client_name;
    my $timeout = $args{timeout};
    my @subtests_snapmirror;

    my @procs_ls_command;
    my @sm_obj = @{$args{sm_objs}};

    foreach my $snapmirror_obj (@sm_obj) {
	my $client = shift @nfsclients;
        my %args_func = (sm_objs => $snapmirror_obj,
                nfsclients => $client,
                src_cg => $src_cg,
                dst_cg => $dst_cg,
		local_restore => $local_restore,
		remote_restore => $remote_restore,
                validation => $validation,
                timeout => $timeout);
        push @subtests_snapmirror, create_parallel_processes('Appcg_restore',\%args_func);
	push @nfsclients, $client;
    }
    run_processes_bg('Appcg_restore',\@subtests_snapmirror);
} #end of Wf_Appcg_restore

=head2 Appcg_restore

Steps
  1) create and initialize a lrsedr relationship using snapshot s0.
  2) configure with src_cg and do sm update with s1. Check transfer engine for validation.
  3) configure with dst_cg and do sm update with s2. Check transfer engine for validation.
  4) delete s1 from source if it is remote restore.
  5) do sm restore and check transfer enginer for validation.
  6) do sm check
  7) turn off the compression on src.

=over

=item Example

    Appcg_restore{sm_objs  => $sm_obj,
                  nfsclients => $nfsclients,
                  src_cg    => $src_cg,
                  dst_cg  => $dst_cg,
                  local_restore  => $local_restore,
                  remote_restore  => $remote_restore,
    };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.

=item local_restore

(Required):  To do local restore only.

=item remote_restore

(Required):  To do remote restore only.


=back

=cut

sub Appcg_restore {
    my %args = validate_with(
        params => \@_,
        spec   => {
           sm_objs                    => { type => OBJECT},
           nfsclients                 => { type => ARRAYREF|SCALAR},
           src_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
           dst_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
	   local_restore              => { type => SCALAR},
	   remote_restore             => { type => SCALAR},
           timeout                    => { type => SCALAR, optional => 1, default => 1800},
           validation                 => { type => SCALAR, optional => 1, default => 0},
        },
    );

    my $log = log_global();
    $log->enter();

    $log->step('Get sm objects');
    my $sm_obj = $args{sm_objs};

    $log->step('Get nfsclients');
    my $client_name = $args{nfsclients};
    my $client = NACL::C::Client->new(name => $client_name);

    $log->step('Get cg');
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};

    my $local_restore = $args{local_restore};
    my $remote_restore = $args{remote_restore};

    $log->step('Get Validation Value');
    my $validation = $args{validation};

    $log->comment('To add delay during parallel runs');
    _stagger($$ % 60);

    $log->step('Get node and src vol information from the sm objects');
    my $source_path      = $sm_obj->{source_path};
    my $destination_path = $sm_obj->{destination_path};
    my ($src_vserver, $src_vol_name) = ($source_path      =~ /^(.*):(.*)$/);
    my ($dst_vserver, $dst_vol_name) = ($destination_path =~ /^(.*):(.*)$/);

    $log->step('Get Src and Dst Objs from the sm objects');
    my $src_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{source_ci},
        volume            => $src_vol_name,
        vserver           => $src_vserver,
    );
    my $dst_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{command_interface},
        volume            => $dst_vol_name,
        vserver           => $dst_vserver,
    );
    my $src_node = NACL::STask::Node->new(name => $src_vol->get_local_node_name());
    my $dst_node = NACL::STask::Node->new(name => $dst_vol->get_local_node_name());

    my $src_node_name = $src_node->name();
    my $dst_node_name = $dst_node->name();

    $src_node->refresh_command_interface(); # workaround for burt863951
    $dst_node->refresh_command_interface(); # workaround for burt863951

    my @subtests;
    my $timeout = $args{timeout};

        my $Mount = NACL::STask::Client::Mount->create(
        command_interface  => $client,
        volume             => $src_vol,
        nacltask_if_exists => 'purge',
        nacltask_verify    => 1,
    );
    my $path = $Mount->mount_point();
    my $net_interface_obj = NACL::CS::NetworkInterface->fetch(
            command_interface => $src_node,
            filter            => {
                'home-node'      => $src_node,
                role             => 'node-mgmt',
                'address-family' => 'ipv4',
            },
            requested_fields => ['address'],
        );
    my $node_mgmt_ip = $net_interface_obj->{'address'};

    my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
    my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
    my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

    my $sis_src;
    my $log_reader;
    my $rst_id;
    my $cg_size;
    my $cg;
    my $val_to_assert;
    my $sm_rst;
    my $comp_info_list;
    my $src_comp_info;
    my $dst_comp_info;

    $log->step("Entering TESTCASE : func_ACG_restore ");

    $log->step('Get Ontap Version for source_node and dst_node' );
    my $src_ontap_version = get_ontap_version( node => $src_node );
    my $dst_ontap_version = get_ontap_version( node => $dst_node );
    $log->log('Source Cluster Ontap Version: ' . $src_ontap_version );
    $log->log('Dst Cluster Ontap Version: ' . $dst_ontap_version );

    if( $src_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
           $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
        $log->log('Since it is HN release and Adap16k or Adap32k is not supported' );
        $log->log('The test will exist' );
        $log->log(NATE::Result::Pass->new('FUNC_restore_SE_Comp_7'));
        $log->exit();
        return;
    }

    $log->step('Get sm relationship_id ');
    my $sm_state = $sm_obj->state();
    $rst_id = $sm_state->relationship_id;
    $log->comment('Relationship_id is : '. $rst_id);

    $log->step('Display Vol Efficiency on src vol ' );
    $sis_src = NACL::CS::VolumeEfficiency->fetch(
             command_interface       => $src_node,
             filter                  => {
                 volume              => $src_vol_name,
                 vserver             => $src_vserver,
             },
             requested_fields        => ['state'],
             allow_empty             => 1,
         );
    my $src_se_state = $sis_src->{'state'};
    $log->log('Print SE state: '. $src_se_state );

    if($src_se_state eq 'Enabled'){
        $log->step('Sis undo on src volume since SE is already enabled ');
        sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'         => $timeout,
        );

        $log->step('Create Snapshot ss0');
        $src_vol->snapshot_create(
            snapshot           => 'ss0',
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        $log->step("Update with ss0 to make sure Dest has compression_info 0");
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => 'ss0',
            'nacltask_wait'   => 1,
            'timeout'         => $timeout,
        );
    }

    $log->step('Enable Efficiency on src vol ' );
    $sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    
    $log->step('Change the policy to default');
    $sis_src->modify(
        policy               => 'default',
    );

    $log->step("Configure compression type with  $src_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $src_cg, sis_src => $sis_src);

    $log->step('Show Efficiency on src vol '  );
    appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
                            vserver => $src_vserver, ontap_version => $src_ontap_version );

    $log->step("Overwrite data with : $src_cg dataset" );
    my $count=0;  #count is used for creating logs with diff run_id for each overwrite
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                   path => $path, count => $count );

    $log->step('Quiesce SM Relationship ');
    $sm_obj->quiesce();

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data' => 'true',
        nacltask_wait   => '1'
    );

    $log->step('Resume SM Relationship ');
    $sm_obj->resume();

    $log->step('Create Snapshot s1_'. $src_cg );
    my $s1 = 's1_' . $src_cg ;
    $src_vol->snapshot_create(
        snapshot           => $s1,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots before update' );
        print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s1,
        );
    }

    $log->step("Update the snapmirror relationship with snapshot $s1");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s1,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots after update' );
        print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s1,
        );

	$log->step( 'Validate transfer engine' );
        if( $dst_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
            $src_cg eq 'Adap32k' )){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
	        Validate_snapmirror_logs(
	            op            => 'Update',
	            val_to_assert => $val_to_assert_LRE,
	            rst_id        => $rst_id,
	            log_reader    => $log_reader,
	        );
        }
	else {
	    Validate_snapmirror_logs(
	        op            => 'Update',
	        val_to_assert => $val_to_assert_LRSE,
	        rst_id        => $rst_id,
	        log_reader    => $log_reader,
	    );
       }
    }

    if (  $dst_cg ne $src_cg ) {
        $log->step('Sis undo on src volume');
        sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'          => $timeout,
        );

    $log->step('Enable Efficiency on source volume'  );
    $sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    }

    $log->step("Configure compression type with  $dst_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $dst_cg, sis_src => $sis_src);

    $log->step('Show Efficiency on ' . $src_vol_name  );
    appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
                            vserver => $src_vserver, ontap_version => $src_ontap_version );

    $log->step("Overwrite data with : $dst_cg dataset" );
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $dst_cg,
                                   path => $path, count => $count );

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data' => 'true',
        nacltask_wait   => '1'
    );

    $log->step('Create Snapshot s2_'. $dst_cg );
    my $s2 = 's2_' . $dst_cg;
    $src_vol->snapshot_create(
        snapshot           => $s2,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots before update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s2,
        );
        $src_comp_info = $comp_info_list->{'src_afs'};
        $dst_comp_info = $comp_info_list->{'dst_afs'};

        $log->step( 'Expected String for Transfer Engine ' );
        if( $dst_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
            $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
                $val_to_assert = $val_to_assert_LRE ;
                $log->log( '$val_to_assert' . $val_to_assert );
        }
        else {
            $val_to_assert = Validate_appcg_forward_transfer (
                src_comp_info => $src_comp_info,
                dst_comp_info => $dst_comp_info,
                src_ontap_version => $src_ontap_version,
                dst_ontap_version => $dst_ontap_version,
            );
        $log->log( '$val_to_assert' . $val_to_assert );
        }
    }

    $log->step("Update the snapmirror relationship with snapshot $s2");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s2,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots after update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s2,
        );
        $src_comp_info = $comp_info_list->{'src_afs'};
        $dst_comp_info = $comp_info_list->{'dst_afs'};

        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );
    }

   if ( $remote_restore eq '1'){
       $log->step ('Since it is remote restore, will delete the s1 snapshot from the source ');
       $src_vol->snapshot_delete(
            snapshot           => $s1,
        );

        if( $src_cg eq $dst_cg || $src_cg eq 'Uncompr' || $dst_cg ne 'Uncompr' ){
        $log->step( 'if $src_cg and $dst_cg are the same, or $src_cg is Uncompr '.
            'or $dst_cg is not Uncompr - turn off the compression on the source '.
            'and create a snapshot before restore' );
            # Make sure the scan is done before turn off the compression
            NACL::STask::VolumeEfficiency->wait_for_status (
                command_interface    => $src_node,
                vserver              => $src_vserver,
                volume               => $src_vol_name,
                status               => 'Idle',
                'method-timeout'     => $timeout,
            );

            $sis_src->modify(
                compression            => 'false',
                'inline-compression'   => 'false',
            );

            $log->log('Show Efficiency on src_vol after disabling compression');
            appcg_efficiency_show( vol_name => $src_vol_name , node_obj => $src_node,
                                vserver => $src_vserver, ontap_version => $src_ontap_version);

            $log->log ('Create another snapshot after turn off the compression on the source');
            my $s3 = 's3_sis_off';
            $src_vol->snapshot_create(
                snapshot           => $s3,
                nacltask_if_exists => 'purge',
                nacltask_wait      => 1
            );
        }
   }

   $log->step( 'Print Compression Capability for ' .
       'source afs , dst afs, previous transfer snapshots ' . $s1 );
   $comp_info_list = print_compression_info (
       src_node           => $src_node,
       src_vol_name       => $src_vol_name,
       dst_node           => $dst_node,
       dst_vol_name       => $dst_vol_name,
       transfer_snapshot  => $s1,
   );

   $log->comment('For restore, source volume afs will be dest' .
        ' and the restore snapshot will be source');
   $dst_comp_info = $comp_info_list->{'src_afs'};
   $src_comp_info = $comp_info_list->{'dst_trn_snap'};

   $log->step( 'Expected String for Transfer Engine for restore based on comp_info ' );
   $val_to_assert = Validate_appcg_restore (
                      src_comp_info => $dst_comp_info,
                      dst_comp_info => $src_comp_info,
                      src_ontap_version => $dst_ontap_version,
                      dst_ontap_version => $src_ontap_version,
    	              local_restore     => $local_restore,
    	              remote_restore    => $remote_restore,
                   );
   $log->log( '$val_to_assert' . $val_to_assert );

   if( $dst_ontap_version eq 'HN_LOWER' && ( $dst_cg eq 'Adap16k' ||
           $dst_cg eq 'Adap32k' || $src_cg eq 'Adap16k' || $src_cg eq 'Adap32k')){
       $val_to_assert = $val_to_assert_LRE,
       $log->log( 'Since the previous transfer is LRE, and it will continue as LRE' );
       $log->log( '$val_to_assert ' . $val_to_assert );
   }

   $log->step(
       'Snapmirror restore ' . $src_cg .  ' transfer snapshot to '. $dst_cg . ' AFS ');
   my $job_component;
   my $sm_restore = NACL::STask::Snapmirror->restore(
       command_interface  => $src_node,
       'source-path'      => $dst_vol,
       'destination-path' => $src_vol,
       'source-snapshot'  => $s1,
       'nacltask_wait'    => '0',
       'method-timeout'   => $timeout,
       'job_component'    => \$job_component,
   );

   my $sm_restore_state = $sm_restore->state();
   my $restore_rst_id = $sm_restore_state->relationship_id;
   $log->comment( 'Restore Relationship ID :' . $restore_rst_id );

   if ( $validation eq '1') {
       $log->step( 'Start monitoring audit logs for restore' );
       $log_reader = start_monitoring_snapmirror_logs(
           node       => $src_node,
           rst_id     => $restore_rst_id ,
           log_reader => $log_reader,
       );
   }

   if( $val_to_assert ne $val_to_assert_failure ) {
       $log->step( 'Wait for Restore job to be completed ' );
       $sm_restore->wait_for_restore(
           job_component    => $job_component,
           'method-timeout' => $timeout,
       );

       $log->step( 'Check for RST relationship ' );
       try {
        $sm_restore->wait_on_attributes(
            attributes => [
                {   attribute_to_check => 'status',
                    till_value         => qr/idle/i,
                }
            ],
        );
        } catch NATE::BaseException with {
            my $sm_rst = NACL::CS::Snapmirror->fetch(
                command_interface => $src_node,
                filter            => {
                    'destination-volume'  => $src_vol,
                    'type'                => 'RST',
                },
                allow_empty => 1,
            );
            if($sm_rst) {
                $log->error('RST relationship still exists');
            };
        };
   }
   elsif ($val_to_assert eq $val_to_assert_failure) {
       $log->step( 'Check for RST relationship after restore failed ' );
       Tharn::snooze(30); #Restore to get started.
       try {
        $sm_restore->wait_on_attributes(
            attributes => [
                {   attribute_to_check => 'status',
                    till_value         => qr/idle/i,
                }
            ],
        );
        } catch NATE::BaseException with {
            my $sm_rst = NACL::CS::Snapmirror->fetch(
                command_interface => $src_node,
                filter            => {
                    'destination-volume'  => $src_vol,
                    'type'                => 'RST',
                },
                allow_empty => 0,
            );
            if(!$sm_rst) {
                $log->error('RST relationship still does not exists');
                Tharn::snooze(10);
            };
        };

    }

   if ( $validation eq '1') {
      $log->step( 'Validate transfer engine' );
      Validate_snapmirror_logs(
          op            => 'RestoreTransfer',
          val_to_assert => $val_to_assert,
          rst_id        => $restore_rst_id,
          log_reader    => $log_reader,
      );

      $log->step( 'Print Compression Capability for ' .
          'source afs , dst afs, previous transfer snapshots ' . $s1. 'after restore' );
      $comp_info_list = print_compression_info (
          src_node           => $src_node,
          src_vol_name       => $src_vol_name,
          dst_node           => $dst_node,
          dst_vol_name       => $dst_vol_name,
          transfer_snapshot  => $s1,
      );

      $log->step('SM check');
      $sm_obj->check( 'method-timeout' => $timeout );
   }

   if($val_to_assert eq $val_to_assert_failure) {
   $log->step( 'Cleanup the failed restore' );
   Tharn::snooze(90);
   NACL::STask::Snapmirror->restore(
       command_interface  => $src_node,
       'source-path'      => $dst_vol,
       'destination-path' => $src_vol,
       'nacltask_wait'    => '1',
       'method-timeout'   => $timeout,
       'clean-up-failure' => 'true',
   );
   }

    $log->step('Sis undo on src volume and it will turn off sis too');
    Tharn::snooze(120); #to wait RST relation got deleted.
    sis_undo_with_retry(
        'src_vol'         => $src_vol,
        'timeout'         => $timeout,
    );

   $log->step('Create Snapshot s0' );
    my $s0 = 's0';
    $src_vol->snapshot_create(
        snapshot           => $s0,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    $log->step("Update the snapmirror relationship with snapshot $s0");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s0,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if  ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots after update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s0,
        );
    }

   $log->exit();
   return;
}  #end of Appcg_restore

=head2 Wf_Appcg_sfr

A wrapper to run Appcg_sfr

=over

=item Example

    Appcg_sfr{sm_objs  => /@snapmirror,
                           nfsclients => @nfsclients,
                           src_cg    => $src_cg,
                           dst_cg  => $dst_cg,
                           local_sfr  => $local_sfr,
                           remote_sfr  => $remote_sfr,
                           };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  ARRAYREF of Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.

=item local_sfr

(Required):  To do local sfr only.

=item remote_restore

(Required):  To do remote sfr only.

=back

=back

=item exceptions

    NATE::BaseException

=cut

sub Wf_Appcg_sfr {
    my %args = validate_with(
        params => \@_,
        allow_extra => 1,
        spec   => {
           sm_objs                    => { type => ARRAYREF},
           nfsclients                 => { type => ARRAYREF|SCALAR},
           src_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/} ,
           dst_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
	   local_sfr                  => { type => SCALAR},
	   remote_sfr                 => { type => SCALAR},
           timeout                    => { type => SCALAR, optional => 1, default => 1800},
           validation                 => { type => SCALAR, optional => 1, default => 0},
        },
    );

    $log->step('Get sm objects');
    my @snapmirror = @{$args{sm_objs}};
    my $count  = scalar(@snapmirror);

    $log->step('Get nfsclients');
    my $nfsclients = $args{nfsclients};
    my @nfsclients = split ',', $nfsclients;

    my $sm_obj;
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};
    my $local_sfr = $args{local_sfr};
    my $remote_sfr = $args{remote_sfr};
    my $validation = $args{validation};
    my $client_name;
    my $timeout = $args{timeout};
    my @subtests_snapmirror;

    my @procs_ls_command;
    my @sm_obj = @{$args{sm_objs}};

    foreach my $snapmirror_obj (@sm_obj) {
	my $client = shift @nfsclients;
        my %args_func = (sm_objs => $snapmirror_obj,
                nfsclients => $client,
                src_cg => $src_cg,
                dst_cg => $dst_cg,
		local_sfr => $local_sfr,
		remote_sfr => $remote_sfr,
                validation => $validation,
                timeout => $timeout);
        push @subtests_snapmirror, create_parallel_processes('Appcg_sfr',\%args_func);
	push @nfsclients, $client;
    }
    run_processes_bg('Appcg_sfr',\@subtests_snapmirror);
} #end of Wf_Appcg_sfr

=head2 Appcg_sfr

Steps
  1) create and initialize a lrsedr relationship using snapshot s0.
  2) configure with src_cg and do sm update with s1. Check transfer engine for validation.
  3) configure with dst_cg and do sm update with s2. Check transfer engine for validation.
  4) delete s1 from source if it is remote sfr.
  5) do sm sfr and check transfer enginer for validation.
  6) do sm check
  7) turn off the compression on src.

=over

=item Example

    Appcg_restore{sm_objs  => $sm_obj,
                  nfsclients => $nfsclients,
                  src_cg    => $src_cg,
                  dst_cg  => $dst_cg,
                  local_sfr  => $local_sfr,
                  remote_sfr  => $remote_sfr,
    };

=item Options

=over

=item sm_obj

(Required): Snapmirror object of type NACL::STask::Snapmirror.

=item nfsclients

(Required):  Nfs client.

=item src_cg

(Required):  Compression group on source vol for sm transfer.

=item dst_cg

(Required):  Compression group on dst vol for sm transfer.

=item local_sfr

(Required):  To do local restore only.

=item remote_restore

(Required):  To do remote restore only.


=back

=cut

sub Appcg_sfr {
    my %args = validate_with(
        params => \@_,
        spec   => {
           sm_objs                    => { type => OBJECT},
           nfsclients                 => { type => ARRAYREF|SCALAR},
           src_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
           dst_cg                     => { type => SCALAR, regex => qr/Uncompr|Sec32k|Adap8k|Adap16k|Adap32k/},
	   local_sfr                  => { type => SCALAR},
	   remote_sfr                 => { type => SCALAR},
           timeout                    => { type => SCALAR, optional => 1, default => 1800},
           validation                 => { type => SCALAR, optional => 1, default => 0},
        },
    );

    my $log = log_global();
    $log->enter();

    $log->step('Get sm objects');
    my $sm_obj = $args{sm_objs};

    $log->step('Get nfsclients');
    my $client_name = $args{nfsclients};
    my $client = NACL::C::Client->new(name => $client_name);

    $log->step('Get cg');
    my $src_cg = $args{src_cg};
    my $dst_cg = $args{dst_cg};

    my $local_sfr = $args{local_sfr};
    my $remote_sfr = $args{remote_sfr};

    $log->step('Get Validation Value');
    my $validation = $args{validation};

    $log->comment('To add delay during parallel runs');
    _stagger($$ % 60);

    $log->step('Get node and src vol information from the sm objects');
    my $source_path      = $sm_obj->{source_path};
    my $destination_path = $sm_obj->{destination_path};
    my ($src_vserver, $src_vol_name) = ($source_path      =~ /^(.*):(.*)$/);
    my ($dst_vserver, $dst_vol_name) = ($destination_path =~ /^(.*):(.*)$/);

    $log->step('Get Src and Dst Objs from the sm objects');
    my $src_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{source_ci},
        volume            => $src_vol_name,
        vserver           => $src_vserver,
    );
    my $dst_vol = NACL::STask::Volume->new(
        command_interface => $sm_obj->{command_interface},
        volume            => $dst_vol_name,
        vserver           => $dst_vserver,
    );
    my $src_node = NACL::STask::Node->new(name => $src_vol->get_local_node_name());
    my $dst_node = NACL::STask::Node->new(name => $dst_vol->get_local_node_name());

    my $src_node_name = $src_node->name();
    my $dst_node_name = $dst_node->name();

    $src_node->refresh_command_interface(); # workaround for burt863951
    $dst_node->refresh_command_interface(); # workaround for burt863951

    my @subtests;
    my $timeout = $args{timeout};

        my $Mount = NACL::STask::Client::Mount->create(
        command_interface  => $client,
        volume             => $src_vol,
        nacltask_if_exists => 'purge',
        nacltask_verify    => 1,
    );
    my $path = $Mount->mount_point();
    my $net_interface_obj = NACL::CS::NetworkInterface->fetch(
            command_interface => $src_node,
            filter            => {
                'home-node'      => $src_node,
                role             => 'node-mgmt',
                'address-family' => 'ipv4',
            },
            requested_fields => ['address'],
        );
    my $node_mgmt_ip = $net_interface_obj->{'address'};

    my $val_to_assert_LRE  = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER;
    my $val_to_assert_LRSE = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_WITH_SE;
    my $val_to_assert_failure = $NACL::AppBuilder::DpgWorkflows::StringConstants::LOGICAL_TRANSFER_FAILURE;

    my $sis_src;
    my $log_reader;
    my $rst_id;
    my $cg_size;
    my $cg;
    my $val_to_assert;
    my $sm_rst;
    my $comp_info_list;
    my $src_comp_info;
    my $dst_comp_info;

    $log->step("Entering TESTCASE : func_ACG_sfr ");

    $log->step('Get Ontap Version for source_node and dst_node' );
    my $src_ontap_version = get_ontap_version( node => $src_node );
    my $dst_ontap_version = get_ontap_version( node => $dst_node );
    $log->log('Source Cluster Ontap Version: ' . $src_ontap_version );
    $log->log('Dst Cluster Ontap Version: ' . $dst_ontap_version );

    if( $src_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
           $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
        $log->log('Since it is HN or Lower release and Adap16k or Adap32k is not supported' );
        $log->log('The test will exist' );
        $log->log(NATE::Result::Pass->new('FUNC_restore_SE_Comp_7'));
        $log->exit();
        return;
    }

    $log->step('Get sm relationship_id ');
    my $sm_state = $sm_obj->state();
    $rst_id = $sm_state->relationship_id;
    $log->comment('Relationship_id is : '. $rst_id);

    $log->step('Display Vol Efficiency on src vol ' );
    $sis_src = NACL::CS::VolumeEfficiency->fetch(
             command_interface       => $src_node,
             filter                  => {
                 volume              => $src_vol_name,
                 vserver             => $src_vserver,
             },
             requested_fields        => ['state'],
             allow_empty             => 1,
         );
    my $src_se_state = $sis_src->{'state'};
    $log->log('Print SE state: '. $src_se_state );

    if($src_se_state eq 'Enabled'){
        $log->step('Sis undo on src volume since SE is already enabled ');
        sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'         => $timeout,
        );

        $log->step('Create Snapshot ss0');
        $src_vol->snapshot_create(
            snapshot           => 'ss0',
            nacltask_if_exists => 'purge',
            nacltask_wait      => 1,
            'method-timeout'   => $timeout,
        );

        $log->step("Update with ss0 to make sure Dest has compression_info 0");
        update_with_retry(
            'sm_obj'          => $sm_obj,
            'source_snapshot' => 'ss0',
            'nacltask_wait'   => 1,
            'timeout'         => $timeout,
        );
    }

    $log->step('Enable Efficiency on src vol ' );
    $sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    
    $log->step('Change the policy to default');
    $sis_src->modify(
        policy               => 'default',
    );

    $log->step("Configure compression type with  $src_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $src_cg, sis_src => $sis_src);

    $log->step('Show Efficiency on src vol '  );
    appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
                            vserver => $src_vserver, ontap_version => $src_ontap_version );

    $log->step("Overwrite data with : $src_cg dataset" );
    my $count=0;  #count is used for creating logs with diff run_id for each overwrite
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $src_cg,
                                   path => $path, count => $count );

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data' => 'true',
        nacltask_wait   => '1'
    );

    $log->step('Create Snapshot s1_'. $src_cg );
    my $s1 = 's1_' . $src_cg ;
    $src_vol->snapshot_create(
        snapshot           => $s1,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Start monitoring audit logs for update' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $dst_node,
            rst_id     => $rst_id ,
            log_reader => $log_reader,
        );

	$log->step( 'Print Compression Capability for ' .
             'source afs , dst afs, transfer snapshots before update' );
        print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s1,
        );
    }

    $log->step("Update the snapmirror relationship with snapshot $s1");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s1,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
        'source afs , dst afs, transfer snapshots after update' );
        print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
             transfer_snapshot  => $s1,
        );

	$log->step( 'Validate transfer engine' );
        if( $dst_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
            $src_cg eq 'Adap32k' )){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
	        Validate_snapmirror_logs(
	            op            => 'Update',
	            val_to_assert => $val_to_assert_LRE,
	            rst_id        => $rst_id,
	            log_reader    => $log_reader,
	        );
        }
	else {
	    Validate_snapmirror_logs(
	        op            => 'Update',
	        val_to_assert => $val_to_assert_LRSE,
	        rst_id        => $rst_id,
	        log_reader    => $log_reader,
	    );
       }
    }

    if (  $dst_cg ne $src_cg ) {
        $log->step('Sis undo on src volume');
        sis_undo_with_retry(
            'src_vol'         => $src_vol,
            'timeout'         => $timeout,
        );

        $log->step('Enable Efficiency on source volume'  );
        $sis_src =  $src_vol->efficiency_on(nacltask_if_enabled => 'pass');
    }

    $log->step("Configure compression type with  $dst_cg on src_vol_name" );
    $sis_src = configure_appcg(cg => $dst_cg, sis_src => $sis_src);

    $log->step('Show Efficiency on ' . $src_vol_name  );
    appcg_efficiency_show ( vol_name => $src_vol_name , node_obj => $src_node,
                            vserver => $src_vserver, ontap_version => $src_ontap_version );

    $log->step("Overwrite data with : $dst_cg dataset" );
    $count = appcg_populate_overwrite(src_vol_name => $src_vol_name , client  => $client,
                                   node_mgmt_ip => $node_mgmt_ip, cg => $dst_cg,
                                   path => $path, count => $count );

    $log->step('Issue sis start on ');
    $sis_src->start(
        'scan-old-data' => 'true',
        nacltask_wait   => '1'
    );

    $log->step('Create Snapshot s2_'. $dst_cg );
    my $s2 = 's2_' . $dst_cg;
    $src_vol->snapshot_create(
        snapshot           => $s2,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    $log->step( 'Start monitoring audit logs for update' );
    $log_reader = start_monitoring_snapmirror_logs(
        node       => $dst_node,
        rst_id     => $rst_id ,
        log_reader => $log_reader,
    );

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
        'source afs , dst afs, transfer snapshots before update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s2,
        );
        $src_comp_info = $comp_info_list->{'src_afs'};
        $dst_comp_info = $comp_info_list->{'dst_afs'};

        $log->step( 'Expected String for Transfer Engine ' );
        if( $dst_ontap_version eq 'HN_LOWER' && ( $src_cg eq 'Adap16k' ||
            $src_cg eq 'Adap32k' || $dst_cg eq 'Adap16k' || $dst_cg eq 'Adap32k')){
                $log->log('Since dst cluster is HN release and Adap16k or Adap32k is not understand, and will be LRE' );
                $val_to_assert = $val_to_assert_LRE ;
                $log->log( '$val_to_assert' . $val_to_assert );
        }
        else {
            $val_to_assert = Validate_appcg_forward_transfer (
                src_comp_info => $src_comp_info,
                dst_comp_info => $dst_comp_info,
                src_ontap_version => $src_ontap_version,
                dst_ontap_version => $dst_ontap_version,
            );
        $log->log( '$val_to_assert' . $val_to_assert );
        }
    }

    $log->step("Update the snapmirror relationship with snapshot $s2");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s2,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
        'source afs , dst afs, transfer snapshots after update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s2,
        );
        $src_comp_info = $comp_info_list->{'src_afs'};
        $dst_comp_info = $comp_info_list->{'dst_afs'};

        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'Update',
            val_to_assert => $val_to_assert,
            rst_id        => $rst_id,
            log_reader    => $log_reader,
        );
    }

    if ( $remote_sfr eq '1'){
        $log->step ('Since it is remote sfr, will delete the s1 snapshot from the source ');
        $src_vol->snapshot_delete(
            snapshot           => $s1,
        );
    }

    my @file_list = ('/foo.1,@/foo.2', '/foo.2,@/foo.3', '/foo.3,@/foo.4',
        '/foo.4,@/foo.1', '/foo.5,@/foo.5', '/foo.6,@/foo.6', '/foo.7,@/foo.7',
        '/foo.8,@/foo.8');

    my @file_list1 = ('/foo.2,@/foo.1', '/foo.3,@/foo.2', '/foo.4,@/foo.3',
        '/foo.1,@/foo.4', '/foo.5,@/foo.5', '/foo.6,@/foo.6', '/foo.7,@/foo.7',
        '/foo.8,@/foo.8');

    if ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
        'source afs , dst afs, previous transfer snapshots ' . $s1 . 'before restore');
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s1,
        );

         $log->comment('For sfr, source volume afs will be dest' .
        ' and the restore snapshot will be source');
        $dst_comp_info = $comp_info_list->{'src_afs'};
        $src_comp_info = $comp_info_list->{'dst_trn_snap'};

        $log->step( 'Expected String for Transfer Engine for restore based on comp_info ' );
        $val_to_assert = Validate_appcg_sfr (
                        src_comp_info => $dst_comp_info,
                        dst_comp_info => $src_comp_info,
                        src_ontap_version => $dst_ontap_version,
                        dst_ontap_version => $src_ontap_version,
    	                local_sfr     => $local_sfr,
    	                remote_sfr    => $remote_sfr,
                    );

	$log->log( '$val_to_assert' . $val_to_assert );

        if( $dst_ontap_version eq 'HN_LOWER' && ( $dst_cg eq 'Adap16k' ||
                $dst_cg eq 'Adap32k' || $src_cg eq 'Adap16k' || $src_cg eq 'Adap32k')){
            $val_to_assert = $val_to_assert_LRE,
            $log->log( 'Since previous transfer is LRE, and it will continue as LRE' );
            $log->log( '$val_to_assert ' . $val_to_assert );
        }
    }

    $log->step(
        'Snapmirror restore ' . $src_cg .  ' transfer snapshot to '. $dst_cg . ' AFS ');
    my $job_component;
    my $sm_sfr = NACL::STask::Snapmirror->restore(
        command_interface  => $src_node,
        'source-path'      => $dst_vol,
        'destination-path' => $src_vol,
        'source-snapshot'  => $s1,
        'file-list'        => [@file_list],
        'create-check-snapshot' => 'true',
        'nacltask_wait'    => '0',
        'method-timeout'   => $timeout,
        'job_component'    => \$job_component,
    );

    my $sm_sfr_state = $sm_sfr->state();
    my $sfr_rst_id = $sm_sfr_state->relationship_id;
    $log->comment( 'Restore Relationship ID :' . $sfr_rst_id );

    if  ( $validation eq '1') {
        $log->step( 'Start monitoring audit logs for SFR ' );
        $log_reader = start_monitoring_snapmirror_logs(
            node       => $src_node,
            rst_id     => $sfr_rst_id ,
            log_reader => $log_reader,
        );
    }

    $log->step( 'Wait for Restore job to be completed ' );
    $sm_sfr->wait_for_restore(
        job_component    => $job_component,
        'method-timeout' => $timeout,
    );

    if  ( $validation eq '1') {
        $log->step( 'Validate transfer engine' );
        Validate_snapmirror_logs(
            op            => 'RestoreTransfer',
            val_to_assert => $val_to_assert,
            rst_id        => $sfr_rst_id,
            log_reader    => $log_reader,
        );

        $log->step( 'Print Compression Capability for ' .
          'source afs , dst afs, previous transfer snapshots ' . $s1. 'after sfr' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s1,
        );

        my $chk_ss_obj = NACL::STask::VolumeSnapshot->find(
            command_interface => $src_node,
            filter            => {
                'snapshot' => 'snapmirrorCHK.*.*',
                'volume'   => $src_vol_name,
            },
            allow_empty => 0,
        );

        $log->step( 'Check for RST relationship ' );
        try {
           $sm_sfr->wait_on_attributes(
               attributes => [
                   {   attribute_to_check => 'status',
                       till_value         => qr/idle/i,
                   }
               ],
           );
         } catch NATE::BaseException with {
             my $sm_rst = NACL::CS::Snapmirror->fetch(
                 command_interface => $src_node,
                 filter            => {
                     'destination-volume'  => $src_vol,
                     'type'                => 'RST',
                 },
                 allow_empty => 1,
             );
             if($sm_rst) {
                 $log->error('RST relationship still exists');
                 Tharn::snooze(10);
             };
        };

        $log->step( 'SM Check ' );
        NACL::STask::Snapmirror->check(
            command_interface  => $dst_node,
            'source-path'      => $src_vol,
            'destination-path' => $dst_vol,
            'source-snapshot'  => $chk_ss_obj->snapshot(),
            'target-snapshot'  => $s1,
            'file-list'        => [@file_list1],
            polling_interval   => 60,
            'method-timeout'   => $timeout,
        );

	$log->step("Delete the check created snapshot");
        $chk_ss_obj->delete();

    }

    $log->step('Sis undo on src volume and it will turn off sis too');
    sis_undo_with_retry(
        'src_vol'         => $src_vol,
        'timeout'         => $timeout,
    );

    $log->step('Create Snapshot s0' );
    my $s0 = 's0';
    $src_vol->snapshot_create(
        snapshot           => $s0,
        nacltask_if_exists => 'purge',
        nacltask_wait      => 1,
        'method-timeout'   => $timeout,
    );

    $log->step("Update the snapmirror relationship with snapshot $s0");
    update_with_retry(
        'sm_obj'          => $sm_obj,
        'source_snapshot' => $s0,
        'nacltask_wait'   => 1,
        'timeout'         => $timeout,
    );

    if  ( $validation eq '1') {
        $log->step( 'Print Compression Capability for ' .
            'source afs , dst afs, transfer snapshots after update' );
        $comp_info_list = print_compression_info (
            src_node           => $src_node,
            src_vol_name       => $src_vol_name,
            dst_node           => $dst_node,
            dst_vol_name       => $dst_vol_name,
            transfer_snapshot  => $s0,
        );
    }
    $log->exit();
    return;
}  #end of Appcg_sfr


sub sis_undo_with_retry {
    my %args = validate_with(
        params => \@_,
        spec   => {
            'src_vol' 	      => {type => OBJECT},
            'timeout'         => {type => SCALAR, default => '1800'},
        },
    );

    my $src_vol = $args{src_vol};
    my $src_vol_name = $src_vol->volume();
    my $src_vserver = $src_vol->vserver();
    my $src_node = NACL::STask::Node->new(name => $src_vol->get_local_node_name());
    my $timeout = $args{timeout};

    # Make sure the scan is done before disabling it
    NACL::STask::VolumeEfficiency->wait_for_status (
        command_interface    => $src_node,
        vserver              => $src_vserver,
        volume               => $src_vol_name,
        status               => 'Idle',
        'method-timeout'     => $timeout,
    );

    my $st = time;
    my $op_done = 0;
    while (time < $st + $timeout) {
        try {
            NACL::STask::VolumeEfficiency->undo (
                command_interface    => $src_node,
                vserver              => $src_vserver,
                volume               => $src_vol_name,
                nacltask_wait        => 1,
                'method-timeout'     => $timeout,
            );
            $op_done = 1;
            last;
        } catch NATE::BaseException with {
            my $ex = shift;
            if ($ex->text =~ /Another sis operation is currently active/i) {
                $log->warn($ex->text . ' Retry sis undo in 60s');
                Tharn::snooze(60);
            } else {
                $ex->throw();
            }
        };
    }

    NATE::BaseException->throw('Sis undo failed after ' . $timeout . ' seconds') if !$op_done;

} ## end sub sis_undo_with_retry

=head2 Wf_restore

    It Works for both incremental restore and base line restore

    The steps include:
    1) NFS Mount
    2) Populate Data
    3) Create snapshot (snap1)
    4) Data Modification
    5) Add More Data
    6) Create snapshot (snap2)
    7) Snapmirror update
    8) Data Modification
    9) Create snapshot (snap3)
    10) SnapMirror Update
    11) Add More Data
    12) Delete the snapshot (snap2)
    13) Snapmirror restore using snapshot (snap2)

=over

=item Example

    For Incremental restore
    Wf_restore( node_src => $node_src ,
        node_dst   => $node_dst
        client     => $client,
        sm_objs    => $sm_obj,
    );

    For baseline restore
    Wf_restore( node_src => $node_src ,
        node_dst   => $node_dst
        client     => $client,
        sm_objs    => $sm_obj,
        restore_vol => $restore_vol
    );

=item Options

=over

=item node_src

(Required): Source Node Cluster object

=item node_dst

(Required):  Destination Node Cluster object.

=item client

(Required):  NFS client , to populate data

=item sm_objs

(Required):  Snapmirror relationship object.

=item restore_vol

(Optional):  when needed baseline restore.

=item timeout

(Optional):  timeout value used for snapshot create.  If not set will default to 60 sec.

=item restore_timeout

(Optional):  method-timeout value for snapmirror restore operation.  If not set will default to 1200 sec.

=item no_client_io

(Optional) If set to 1, this will just do a full volume restore without doing any client IO or creating/deleting any snapshots.  Default is undef.

=item max_active_children

(Optional) Number of parallel processes to start at a time.  Default is 16.

=back

=cut

sub Wf_restore {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src        => { type => OBJECT | SCALAR, optional => 0 },
            node_dst        => { type => OBJECT | SCALAR, optional => 0 },
            client          => { type => SCALAR,          optional => 0 },
            sm_objs         => { type => ARRAYREF,        optional => 0 },
            restore_vol     => { type => ARRAYREF,        optional => 1 },
            timeout         => { type => SCALAR,          optional => 1, default => 60 },
            restore_timeout => { type => SCALAR,          optional => 1, default => 1200 },
            no_client_io    => { type => SCALAR,          optional => 1 },
	    max_active_children => {type => SCALAR, optional => 1, default => 16},
        },
    );
    my $sm_obj               = $args{'sm_objs'};
    my $node_src             = $args{'node_src'};
    my $node_dst             = $args{'node_dst'};
    my $client               = $args{'client'};
    my @Sfr_objs             = _list_of_vals($sm_obj);
    my $restore_vol          = $args{'restore_vol'};
    my @volume_restore       = _list_of_vals($restore_vol);
    my $timeout              = $args{'timeout'};
    my $restore_timeout      = $args{'restore_timeout'};
    my $no_client_io         = $args{'no_client_io'};
    my $max_active_children  = $args{'max_active_children'};

    my @subtests_sfrs;
    my $counter               = 0;
    my $length_volume_restore = scalar(@volume_restore);
    foreach my $o (@Sfr_objs) {
        my %args_func = (
            'node_src'            => $node_src,
            'node_dst'            => $node_dst,
            'client'              => $client,
            'sm_obj'              => $o,
            'stagger_delay'       => 10,
            'timeout'             => $timeout,
            'restore_timeout'     => $restore_timeout,
        );
        if ( exists( $args{'restore_vol'} )
            && $length_volume_restore > $counter )
        {
            $args_func{'restore_vol'} = $volume_restore[$counter];
            $counter = $counter + 1;
        } ## end if ( exists( $args{'restore_vol'...}))
        if ( exists( $args{'no_client_io'} ) ) {
            $args_func{'no_client_io'} = $no_client_io;
        }

        push @subtests_sfrs,
            create_parallel_processes( 'sm_inc_or_base_restore' ,
            \%args_func );

    } ## end foreach my $o (@Sfr_objs)

    run_processes_bg( 'sm_inc_or_base_restore', \@subtests_sfrs, $max_active_children);
} ## end sub Wf_incremental_restore

#private method
sub sm_inc_or_base_restore {
    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            node_src        => { type => OBJECT | SCALAR, optional => 0 },
            node_dst        => { type => OBJECT | SCALAR, optional => 0 },
            client          => { type => SCALAR,          optional => 0 },
            sm_obj          => { type => OBJECT,          optional => 0 },
            stagger_delay   => { type => SCALAR,          optional => 1 },
            restore_vol     => { type => OBJECT,          optional => 1 },
            timeout         => { type => SCALAR,          optional => 1 },
            restore_timeout => { type => SCALAR,          optional => 1, default => 1200 },
            no_client_io    => { type => SCALAR,          optional => 1 },
        },
    );

    my $Sm_Obj          = $args{'sm_obj'};
    my $Src_Node        = $args{'node_src'};
    my $Dst_Node        = $args{'node_dst'};
    my $restore_vol     = $args{'restore_vol'};
    my $timeout         = $args{'timeout'};
    my $restore_timeout = $args{'restore_timeout'};
    my $no_client_io    = $args{'no_client_io'};

    _stagger( $args{'stagger_delay'} );

    my ( $Src_Vserver_Name, $src_vol_name )
        = split( /:/, $Sm_Obj->source_path );
    my ( $Dst_Vserver_Name, $dst_vol_name )
        = split( /:/, $Sm_Obj->destination_path );
    my $dst_path = $Sm_Obj->source_path;
    my $src_path = $Sm_Obj->destination_path;

    my $Client = NACL::C::Client->new( client => $args{'client'} );

    if ( ($Src_Node) && ( ref($Src_Node) !~ /Node/i ) ) {
        $Src_Node = NACL::STask::Node->new( name => $Src_Node );
    }

    if ( ($Dst_Node) && ( ref($Dst_Node) !~ /Node/i ) ) {
        $Dst_Node = NACL::STask::Node->new( name => $Dst_Node );
    }

    if ( exists( $args{'restore_vol'} ) ) {
        $dst_path = NACL::C::Volume->find(
            command_interface => $Src_Node,
            filter            => { volume => $restore_vol->volume(), }
        );
        if ($no_client_io == 1) {
            NACL::STask::Snapmirror->restore(
                command_interface  => $Src_Node,
                'source_ci'        => $Src_Node,
                'source-path'      => $src_path,
                'destination-path' => $dst_path,
                nacltask_wait      => 1,
                'method-timeout'   => $restore_timeout,
            );
            return;
        }
    } ## end if ( exists( $args{'restore_vol'...}))

    my $Src_obj = NACL::C::Volume->find(
        command_interface => $Src_Node,
        filter            => {
            volume  => $src_vol_name,
            vserver => $Src_Vserver_Name,
        },
    );

    my $Dst_obj = NACL::C::Volume->find(
        command_interface => $Dst_Node,
        filter            => {
            volume  => $dst_vol_name,
            vserver => $Dst_Vserver_Name,
        },
    );

    my $src_effieciency =  NACL::CS::VolumeEfficiency->fetch(
             command_interface => $Src_Node,
             filter => {volume  => $src_vol_name,
             vserver => $Src_Vserver_Name,},
             allow_empty => 1,
    );

    my $dst_effieciency =  NACL::CS::VolumeEfficiency->fetch(
             command_interface => $Dst_Node,
             filter => {volume => $dst_vol_name,
                        vserver => $Dst_Vserver_Name},
             allow_empty => 1,
    );

    if(defined($src_effieciency)){
        if($src_effieciency->{policy} ne 'default') {
            NACL::STask::VolumeEfficiency->modify (
                                           command_interface => $Src_Node,
                                           vserver => $Src_Vserver_Name,
                                           volume  => $src_vol_name,
                                           policy  => 'default',
            );
        }
        NACL::STask::VolumeEfficiency->off(
                command_interface => $Src_Node,
                vserver           => $Src_Vserver_Name,
                volume            => $src_vol_name,
                'method-timeout'  => 180
        );
    }

    if(defined($dst_effieciency)){
        if($dst_effieciency->{policy} ne 'default') {
            NACL::STask::VolumeEfficiency->modify (
                                           command_interface => $Dst_Node,
                                           vserver => $Dst_Vserver_Name,
                                           volume  => $dst_vol_name,
                                           policy  => 'default',
            );
        }

        NACL::STask::VolumeEfficiency->off(
                command_interface => $Dst_Node,
                vserver           => $Dst_Vserver_Name,
                volume            => $dst_vol_name,
                'method-timeout'  => 180
        );
    }

    # Mount the volume
    $log->step('Mount Src volume');
    my $Mount = NACL::STask::Client::Mount->create(
        command_interface        => $Client,
        server_command_interface => $Src_obj->command_interface,
        vserver                  => $Src_obj->vserver,
        volume                   => $Src_obj,
        exclusive                => 1,
        mountopts                => 'nfsvers=3',
    );

    # Populate data and modify
    $log->step('Data populate');
    my $dataset_info = {};
    dataset(
        operation    => 'populate',
        volume       => $Src_obj,
        client       => $Client,
        mount        => $Mount,
        file_list    => [qw(/f1 /f2 /f3 /f4)],
        file_type    => [qw(file file file file)],
        file_size    => [qw(20m 10m 50m 70m)],
        dataset_info => $dataset_info,
    );

    my @file_list = @{ $dataset_info->{file_list} };

    # find the snapmirror label from the policy and use that for the snapshot
    my $Policy_Obj = NACL::CS::Snapmirror->fetch(
        command_interface => $Dst_Node,
        filter => { 'destination-path' => $Sm_Obj->destination_path, },
        requested_fields => [qw(policy)]
    );
    my $sm_lbl = NACL::CS::SnapmirrorPolicy->fetch(
        command_interface => $Dst_Node,
        filter            => { policy => $Policy_Obj->policy, },
        requested_fields  => [qw(snapmirror-label)]
    );
    my @sm_lbls  = $sm_lbl->snapmirror_label;
    my $sm_label = $sm_lbls[0];

    #  2. create snap1 and replicate to vault dst
    my %opts = ( 'snapmirror-label' => $sm_label);
    if($timeout) {
        $log->comment('Setting method-timeout for snapshot create command to: ' . $timeout);
        $opts{'method-timeout'} = $timeout;
    }
    my $Snapshot1
        = $Src_obj->snapshot_create(snapshot => "snap1_restore_$$", %opts);

    # Create and Modify Streams
    #  3. modify 20%-50% of data in each file (f1-f4)
    #  4. for f1,f2, delete all streams, create 5 streams; for f3, modify streams; for f4 change names of streams
    $dataset_info->{options} = [
        { delete_streams => { file_list => [qw(/f1 /f2)], }, },
        { modify_streams => { file_list => [qw(/f3)], }, },
        { rename_streams => { file_list => [qw(/f4)], }, },
        { create_streams => { file_list => [qw(/f1 /f2)], number => 5, }, },
    ];
    dataset(
        operation    => 'modify',
        volume       => $Src_obj,
        client       => $Client,
        mount        => $Mount,
        file_list    => [qw(/f1 /f2 /f3 /f4)],
        file_type    => [qw(file file file file)],
        file_size    => [qw(30m 20m 60m 90m)],
        dataset_info => $dataset_info,
    );

    $dataset_info->{options} = undef;

   #  5. create another 4 files f5,f6,f7,f8, attach 5-10 streams to each files
    $dataset_info->{options} = [
        {   create_streams =>
                { file_list => [qw(/f5 /f6 /f7 /f8)], number => 10 },
        },
    ];

    dataset(
        operation    => 'populate',
        volume       => $Src_obj,
        client       => $Client,
        mount        => $Mount,
        file_list    => [qw(/f5 /f6 /f7 /f8)],
        file_size    => [qw(20m 10m 30m 40m)],
        dataset_info => $dataset_info,
    );
    $dataset_info->{options} = undef;

    #  6. create snap2 and replicate to vault dst

    my $Snapshot2
        = $Src_obj->snapshot_create(snapshot => "snap2_restore_$$", %opts);

    $Sm_Obj->update(
        nacltask_verify   => 1,
        'source-snapshot' => $Snapshot2->{'snapshot'}
    );

    #  7. modify 20% data for f5; delete f6; delete f7 and create another file f7; rename f8 to f88
    #  8. modify 20% data for f1; delete f2; delete f3 and create another file f3; rename f4 to f44

    $dataset_info->{options} = [
        { modify_files => { file_list => [qw(/f5 /f1)], percent => 20 }, },
        { delete_files => { file_list => [qw(/f6 /f7 /f2 /f3)], }, },
        { rename_files => { file_list => [qw(/f8=>/f88 /f4=>/f44)], }, },
        { create_files => { file_list => [qw(/f7 /f3)], }, },
    ];
    dataset(
        operation    => 'modify',
        volume       => $Src_obj,
        client       => $Client,
        mount        => $Mount,
        file_list    => [qw(/f1 /f2 /f3 /f4 /f5 /f6 /f7 /f8)],
        file_type    => [qw(file file file file file file file file)],
        dataset_info => $dataset_info,
    );

    # Create extra snapshot to be common snapshot for Vault relation.
    my $Snapshot3
        = $Src_obj->snapshot_create(snapshot => "snap3_restore_$$", %opts);
    $Sm_Obj->update(
        nacltask_verify   => 1,
        'source-snapshot' => $Snapshot3->{'snapshot'}
    );

    # Add more data on src volume
    dataset(
        operation    => 'populate',
        volume       => $Src_obj,
        client       => $Client,
        mount        => $Mount,
        file_count   => 1,
        file_size    => '20m',
        dataset_info => $dataset_info,
    );

    delete_snapshot( $Src_obj, $Snapshot2 );

    NACL::STask::Snapmirror->restore(
        command_interface  => $Src_Node,
        'source_ci'        => $Src_Node,
        'source-path'      => $src_path,
        'destination-path' => $dst_path,
        'source-snapshot'  => $Snapshot2,
        nacltask_wait      => 1,
        'method-timeout'   => $restore_timeout,
    );

    if(defined($src_effieciency)){
        NACL::STask::VolumeEfficiency->on(
        command_interface => $Src_Node,
        vserver           => $Src_Vserver_Name,
        volume            => $src_vol_name,
        'method-timeout'  => 180
        );
    }

    if(defined($dst_effieciency)){
        NACL::STask::VolumeEfficiency->on(
        command_interface => $Dst_Node,
        vserver           => $Dst_Vserver_Name,
        volume            => $dst_vol_name,
        'method-timeout'  => 180
        );
    }

} ## end sub sm_inc_or_base_restore


# fg_snapmirror_restore_baseline
# 1. Performs baseline restore : remote or local
# 2. Returns RST relationship object if abort or background is chosen
# 3. Performs snapshot version-uuid validation
# 4. Performs SnapMirror checker
sub fg_snapmirror_restore_baseline {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            sm_obj          => { type => ARRAYREF },
	    rst_dst         => { type => OBJECT, optional => 1 },
            snapshot        => { type => SCALAR, optional => 1 },
            is_remote       => { type => SCALAR, optional => 1, default => 1 },
            background      => { type => SCALAR, optional => 1, default => 0 },
	    create_ckpt     => { type => SCALAR, optional => 1, default => 0 },
	    check_vers_uuid => {type => SCALAR, optional => 1, default => 1 },
            run_check       => { type => SCALAR, optional => 1, default => 1 },
            run_abort       => { type => SCALAR, optional => 1, default => 0 },
            throttle        => { type => SCALAR, optional => 1, default => 10 },
            timeout         => { type => SCALAR, optional => 1, default => 1200 },
        },
    );

    my $Log = log_global();
    $Log->enter();

    my $sm_obj          = @{ $args{sm_obj} }[0];
    my $rst_dst		= $args{'rst_dst'};
    my $is_remote       = $args{'is_remote'};
    my $run_check       = $args{'run_check'};
    my $check_vers_uuid = $args{'check_vers_uuid'};
    my $snapshot        = $args{'snapshot'};
    my $run_abort       = $args{'run_abort'};
    my $background      = $args{'background'};
    my $throttle        = $args{'throttle'};
    my $timeout         = $args{'timeout'};
    my $create_ckpt     = $args{'create_ckpt'};

    my $sVol = $sm_obj->get_source_volume;
    my $dVol = $sm_obj->get_destination_volume;

    my %vol_opts;
    my %rst_opts;
    my %chk_opts;
    if ( $is_remote ) {
        $vol_opts{'command_interface'} = $sm_obj->source_ci;
        $vol_opts{'aggr-list'}         = $sVol->get_one_state_attribute('aggr-list');
    } else {
        $vol_opts{'command_interface'} = $sm_obj->command_interface;
        $vol_opts{'aggr-list'}         = $dVol->get_one_state_attribute('aggr-list');
    }

    if ( $background || $run_abort ) {
        $rst_opts{'throttle'}      = $throttle;
        $rst_opts{'nacltask_wait'} = 0;
    }

    if ( defined $snapshot ) {
        $rst_opts{'source-snapshot'} = $snapshot;
        $chk_opts{'snapshot'}        = $snapshot;
    }

    #---------------------------------------------------------------------------
    $Log->step('Create DP volume for restore destination, if not already created');
    #---------------------------------------------------------------------------
    my @sVol_consts   = $sVol->get_constituents;
    if(!defined($rst_dst)){
        my $aggr_list     = $vol_opts{'aggr-list'};
        my $consts_count  = scalar ( @sVol_consts );
        my @aggr_list_arr = @$aggr_list;
        my $aggr_count    = scalar( @aggr_list_arr );
        my $aggr_list_mu  = $consts_count/$aggr_count;
        $Log->log("aggr list is: " . @$aggr_list );
        $Log->log("constituent count: " . $consts_count );
        $Log->log("aggr-list count: " . $aggr_count );
        $Log->log("aggr-list-multiplier is:" . $aggr_list_mu );
        $rst_dst = NACL::STask::Volume->create(
            %vol_opts,
            size                   => $sVol->get_one_state_attribute('size'),
            'aggr-list-multiplier' => $aggr_list_mu,
            type                   => 'DP',
        );
    }
    $rst_opts{'source-path'}      = $sm_obj->get_one_state_attribute('destination-path');
    $rst_opts{'destination-path'} = $rst_dst->vserver . ':' . $rst_dst->volume;

    #---------------------------------------------------------------------------
    $Log->step('Start SnapMirror baseline restore');
    #---------------------------------------------------------------------------
    my $job;
    my $rst = NACL::STask::Snapmirror->restore(
        command_interface => $sm_obj->command_interface,
        source_ci         => $rst_dst->command_interface,
        job_component     => \$job,
        %rst_opts,
    );

    if ( $background ) {
        $Log->log(
            "Returning to the main script as background option has been chosen"
        );
        #return RST relationship object for the caller to handle
        return $rst;
    }

    if ( $run_abort ) {
	my $ar_attributes;
	if($create_ckpt){
            $ar_attributes = [
                {
                    attribute_to_check => 'snapshot-checkpoint',
                    till_value         => qr/[1-9]\d*/
                },
                {
                    attribute_to_check => 'status',
                    till_value         => qr/Preparing|Transferring|Finalizing/i
                }
            ];
	}else{
	    $ar_attributes = [
                {
		    attribute_to_check => 'status',
                    till_value         => qr/Preparing|Transferring|Finalizing/i
                }
            ];
	}

        fg_wait_on_attributes_all_constituents (
            sm_obj          => $rst,
            attributes      => $ar_attributes,
            timeout         => 300,
            show_all_fields => 1,
        );

        #---------------------------------------------------------------------------
        $Log->step('Running SnapMirror abort on the RST relationship');
        #---------------------------------------------------------------------------
        $rst->abort;
        $Log->log(
            "Returning to the main script as abort option has been chosen"
        );
        #return RST relationship object for the caller to handle
        return $rst;
    }

    #---------------------------------------------------------------------------
    $Log->step('Wait for SnapMirror baseline restore to complete');
    #---------------------------------------------------------------------------
    $rst->wait_for_restore(
        'destination-path' => $rst_opts{'destination-path'},
        job_component      => $job,
        'method-timeout'   => $timeout,
    );

    #---------------------------------------------------------------------------
    $Log->step('Verify there is only one snapshot on the restore destination');
    #---------------------------------------------------------------------------
    my @snapshots = NACL::STask::VolumeSnapshot->find(
        command_interface => $rst_dst->command_interface,
        filter            => {
            volume   => $rst_dst->volume,
            vserver  => $rst_dst->vserver,
            snapshot => '*',
        },
    );

    if ( 0 == $#snapshots ) {
        $Log->log(
            "Only one snapshot found on the restore destination as expected"
        );
    } else {
        NATE::BaseException->throw(
            "More than one snapshot found on the restore destination"
        );
    }

    if ( defined $snapshot && $snapshot ne $snapshots[0]->snapshot ) {
        NATE::BaseException->throw(
            "Snapshot name is not the one specified in the restore call"
        );
    }

    #determine snapshot to run version uuid validation
    my $snap_to_check;
    if ( defined $snapshot ) {
        $snap_to_check = $snapshot;
    } else {
        $snap_to_check = $snapshots[0]->snapshot;
    }

    my @rst_dst_consts = $rst_dst->get_constituents;
    my @dVol_consts    = $dVol->get_constituents;

    #In case where the snapshot is deleted from the src and restored from the
    #dst, the snapshot fetch will fail on the src FG. We need to handle that
    #seperately in the scripts.
    if($check_vers_uuid){
	#-------------------------------------------------------------------
        $Log->step('Check ' . $snap_to_check . '\'s version-uuid');
	#-------------------------------------------------------------------
	foreach( 0 .. $#sVol_consts ) {
	    my $snap1 = NACL::STask::VolumeSnapshot->new(
                command_interface => $rst_dst->command_interface,
                volume            => $rst_dst_consts[$_]->volume,
                vserver           => $rst_dst_consts[$_]->vserver,
                snapshot          => $snap_to_check,
            );
            my $snap2 = NACL::STask::VolumeSnapshot->new(
                command_interface => $sVol->command_interface,
                volume            => $sVol_consts[$_]->volume,
                vserver           => $sVol_consts[$_]->vserver,
                snapshot          => $snap_to_check,
            );
            my $snap3 = NACL::STask::VolumeSnapshot->new(
                command_interface => $dVol->command_interface,
                volume            => $dVol_consts[$_]->volume,
                vserver           => $dVol_consts[$_]->vserver,
                snapshot          => $snap_to_check,
            );

            if ( ( $snap1->get_one_state_attribute('version-uuid')   eq
                   $snap2->get_one_state_attribute('version-uuid') ) &&
                 ( $snap1->get_one_state_attribute('version-uuid')   eq
                   $snap3->get_one_state_attribute('version-uuid') ) ) {
                $Log->log("version-uuid matches");
            } else {
                NATE::BaseException->throw("version-uuid mismatch found!");
	    }
        }
    }

    my $sm;
    my $policy;

    if ( $run_check ) {
        # storing the information to create the relationship again after checker
        $sm = NACL::STask::Snapmirror->new(
            command_interface  => $sm_obj->command_interface,
            source_ci          => $sm_obj->source_ci,
            'source-path'      => $sm_obj->get_one_state_attribute('source-path'),
            'destination-path' => $sm_obj->get_one_state_attribute('destination-path'),
        );
        $policy = $sm_obj->get_one_state_attribute('policy');

        #-----------------------------------------------------------------------
        $Log->step(
            'Clear the existing SnapMirror relationship in order to perform '
          . 'resync between restore source and destination volumes'
        );
        #-----------------------------------------------------------------------
        if($sm_obj->get_one_state_attribute('state') eq 'Snapmirrored'){
            $sm_obj->break;
	}
        $sm_obj->delete;
        $sm_obj->release( 'relationship-info-only' => 'true' );

        delete $rst_opts{'throttle'};
        delete $rst_opts{'nacltask_wait'};
        delete $rst_opts{'source-path'};
        delete $rst_opts{'destination-path'};

        $rst = NACL::STask::Snapmirror->resync(
            command_interface  => $rst_dst->command_interface,
            source_ci          => $dVol->command_interface,
            'source-path'      => $dVol->vserver . ':' . $dVol->volume,
            'destination-path' => $rst_dst->vserver . ':' . $rst_dst->volume,
            policy             => $policy,
            nacltask_wait      => 1,
            %rst_opts
        );

        #-----------------------------------------------------------------------
        $Log->step('Run SnapMirror checker');
        #-----------------------------------------------------------------------
        sm_checker_fg (
            sm_obj => [ $rst ],
            %chk_opts,
        );

        #-----------------------------------------------------------------------
        $Log->step('Restore the original SnapMirror config');
        #-----------------------------------------------------------------------
        $rst->break;
        $rst->delete;
        $rst->release( 'relationship-info-only' => 'true' );
        $sm->resync;
        return $sm;
    }
    return;

}  ## end sub fg_snapmirror_restore_baseline

# fg_snapmirror_restore_incremental
# 1. Performs incremental restore : remote or local
# 2. Returns RST relationship object if abort or background is chosen
# 3. Performs snapshot version-uuid validation
# 4. Performs SnapMirror checker
sub fg_snapmirror_restore_incremental {

    my %args = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            sm_obj          => { type => ARRAYREF },
            rst_dst         => { type => OBJECT },
            snapshot        => { type => SCALAR, optional => 1 },
            background      => { type => SCALAR, optional => 1, default => 0 },
            run_check       => { type => SCALAR, optional => 1, default => 1 },
            run_abort       => { type => SCALAR, optional => 1, default => 0 },
	    create_ckpt     => { type => SCALAR, optional => 1, default => 0 },
	    check_vers_uuid => {type => SCALAR, optional => 1, default => 1 },
            throttle        => { type => SCALAR, optional => 1, default => 10 },
            timeout         => { type => SCALAR, optional => 1, default => 1200 },
        },
    );

    my $Log = log_global();
    $Log->enter();

    my $sm_obj          = @{ $args{sm_obj} }[0];
    my $rst_dst         = $args{'rst_dst'};
    my $run_check       = $args{'run_check'};
    my $check_vers_uuid = $args{'check_vers_uuid'};
    my $snapshot        = $args{'snapshot'};
    my $run_abort       = $args{'run_abort'};
    my $background      = $args{'background'};
    my $throttle        = $args{'throttle'};
    my $timeout         = $args{'timeout'};
    my $create_ckpt     = $args{'create_ckpt'};

    my $sVol = $sm_obj->get_source_volume;
    my $dVol = $sm_obj->get_destination_volume;

    my %rst_opts;
    my %chk_opts;

    if ( $background || $run_abort ) {
        $rst_opts{'throttle'}      = $throttle;
        $rst_opts{'nacltask_wait'} = 0;
    }

    if ( defined $snapshot ) {
        $rst_opts{'source-snapshot'} = $snapshot;
        $chk_opts{'snapshot'}        = $snapshot;
    }

    $rst_opts{'source-path'}      = $sm_obj->get_one_state_attribute('destination-path');
    $rst_opts{'destination-path'} = $rst_dst->vserver . ':' . $rst_dst->volume;

    #---------------------------------------------------------------------------
    $Log->step('Start SnapMirror incremental restore');
    #---------------------------------------------------------------------------
    my $job;
    my $rst = NACL::STask::Snapmirror->restore(
        command_interface => $sm_obj->command_interface,
        source_ci         => $rst_dst->command_interface,
        job_component     => \$job,
        %rst_opts,
    );

    if ( $background ) {
        $Log->log(
            "Returning to the main script as background option has been chosen"
        );
        #return RST relationship object for the caller to handle
        return $rst;
    }

    if ( $run_abort ) {
	my $ar_attributes;
	if ($create_ckpt){
            $ar_attributes = [
                {
                    attribute_to_check => 'snapshot-checkpoint',
                    till_value         => qr/[1-9]\d*/
                },
                {
                    attribute_to_check => 'status',
                    till_value         => qr/Preparing|Transferring|Finalizing/i
                }
            ];
	}else{
	    $ar_attributes = [
		{
                    attribute_to_check => 'status',
                    till_value         => qr/Preparing|Transferring|Finalizing/i
                }
            ];
	}

        fg_wait_on_attributes_all_constituents (
            sm_obj          => $rst,
            attributes      => $ar_attributes,
            timeout         => 300,
            show_all_fields => 1,
        );

        #---------------------------------------------------------------------------
        $Log->step('Running SnapMirror abort on the RST relationship');
        #---------------------------------------------------------------------------
        $rst->abort;
        $Log->log(
            "Returning to the main script as abort option has been chosen"
        );
        #return RST relationship object for the caller to handle
        return $rst;
    }

    #---------------------------------------------------------------------------
    $Log->step('Wait for SnapMirror incremental restore to complete');
    #---------------------------------------------------------------------------
    $rst->wait_for_restore(
        'destination-path' => $rst_opts{'destination-path'},
        job_component      => $job,
        'method-timeout'   => $timeout,
    );

    #determine snapshot to run version uuid validation
    my $snap_to_check;
    if ( defined $snapshot ) {
        $snap_to_check = $snapshot;
    } else {
        $snap_to_check = $sm_obj->get_one_state_attribute('exported-snapshot');
    }

    my @rst_dst_consts = $rst_dst->get_constituents;
    my @sVol_consts    = $sVol->get_constituents;
    my @dVol_consts    = $dVol->get_constituents;

    #In case where the snapshot is deleted from the src and restored from the
    #dst, the snapshot fetch will fail on the src FG. We need to handle that
    #seperately in the scripts.
    if($check_vers_uuid){
        #-------------------------------------------------------------------
        $Log->step('Check ' . $snap_to_check . '\'s version-uuid');
        #-------------------------------------------------------------------
        foreach( 0 .. $#sVol_consts ) {
            my $snap1 = NACL::STask::VolumeSnapshot->new(
                command_interface => $rst_dst->command_interface,
                volume            => $rst_dst_consts[$_]->volume,
                vserver           => $rst_dst_consts[$_]->vserver,
                snapshot          => 'restored*'.$snap_to_check.'*',
            );
            my $snap2 = NACL::STask::VolumeSnapshot->new(
                command_interface => $sVol->command_interface,
                volume            => $sVol_consts[$_]->volume,
                vserver           => $sVol_consts[$_]->vserver,
                snapshot          => $snap_to_check,
            );
            my $snap3 = NACL::STask::VolumeSnapshot->new(
                command_interface => $dVol->command_interface,
                volume            => $dVol_consts[$_]->volume,
                vserver           => $dVol_consts[$_]->vserver,
                snapshot          => $snap_to_check,
            );

            if ( ( $snap1->get_one_state_attribute('version-uuid')   eq
                   $snap2->get_one_state_attribute('version-uuid') ) &&
                 ( $snap1->get_one_state_attribute('version-uuid')   eq
                   $snap3->get_one_state_attribute('version-uuid') ) ) {
                $Log->log("version-uuid matches");
            } else {
                NATE::BaseException->throw("version-uuid mismatch found!");
	    }
        }
    }

    my $sm;
    my $policy;

    if ( $run_check ) {
        # storing the information to create the relationship again after checker
        $sm = NACL::STask::Snapmirror->new(
            command_interface  => $sm_obj->command_interface,
            source_ci          => $sm_obj->source_ci,
            'source-path'      => $sm_obj->get_one_state_attribute('source-path'),
            'destination-path' => $sm_obj->get_one_state_attribute('destination-path'),
        );
        $policy = $sm_obj->get_one_state_attribute('policy');

        #-----------------------------------------------------------------------
        $Log->step(
            'Clear the existing SnapMirror relationship in order to perform '
          . 'resync between restore source and destination volumes'
        );
        #-----------------------------------------------------------------------
        if($sm_obj->get_one_state_attribute('state') eq 'Snapmirrored'){
            $sm_obj->break;
	}
        $sm_obj->delete;
        $sm_obj->release( 'relationship-info-only' => 'true' );

        delete $rst_opts{'throttle'};
        delete $rst_opts{'nacltask_wait'};
        delete $rst_opts{'source-path'};
        delete $rst_opts{'destination-path'};

        $rst = NACL::STask::Snapmirror->resync(
            command_interface  => $rst_dst->command_interface,
            source_ci          => $dVol->command_interface,
            'source-path'      => $dVol->vserver . ':' . $dVol->volume,
            'destination-path' => $rst_dst->vserver . ':' . $rst_dst->volume,
            policy             => $policy,
            nacltask_wait      => 1,
            %rst_opts
        );

        #-----------------------------------------------------------------------
        $Log->step('Run SnapMirror checker');
        #-----------------------------------------------------------------------
        sm_checker_fg (
            sm_obj => [ $rst ],
            %chk_opts,
        );

        #-----------------------------------------------------------------------
        $Log->step('Restore the original SnapMirror config');
        #-----------------------------------------------------------------------
        $rst->break;
        $rst->delete;
        $rst->release( 'relationship-info-only' => 'true' );
        $sm->resync;
        return $sm;
    }
    return;

}  ## end sub fg_snapmirror_restore_incremental

1;
